
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.1
 *  date    : May 1999
 *
 *  class name  : ScrollCanvas
 *  function    : A Virtual Canvas in a Scrollable Panel
 *
 *  modified    : July 1999
 * 
 *  Completely rewritten from scratch to replace the previous ScrollPane
 *  based ScrollCanvas. It now manages two scrollbars by itself instead of
 *  relying on the ScrollPane.
 *  It also implies the ViewCanvas doesn't contain the complete plotfile
 *  anymore as it did in the previous version. The most important
 *  consequence is that a print canvas instruction will now only print the
 *  current viewarea instead of the whole picture.
 * 
 *********************************************************************/

import java.io.*;
import java.awt.*;
import java.awt.event.*;

class ScrollCanvas extends Panel
implements AdjustmentListener
{
   Scrollbar horz,vert;
   int horzsize,vertsize;
   MouseHandler mouse;
   ViewCanvas canvas;
   Scaler scaler;
   
   public ScrollCanvas(JsepView tool,Dimension size) // constructor
   {
      setLayout(new BorderLayout());

      scaler = new Scaler(this);

      scaler.setOriginOffset(10,10); // shift origin somewhat

      canvas = new ViewCanvas(tool,scaler);
      canvas.setSize(size);

      horzsize = size.width;
      vertsize = size.height;
      
      mouse  = new MouseHandler(this); // for zoomfunctions
      canvas.addMouseListener(mouse);
      canvas.addMouseMotionListener(mouse);
      
      addScrollbars();

      add("Center",canvas);
   }
   
   public void addScrollbars()
   {
      horz = new Scrollbar(Scrollbar.HORIZONTAL);
      vert = new Scrollbar(Scrollbar.VERTICAL);
      horz.setMinimum(0);
      horz.setMaximum(horzsize);
      vert.setMinimum(0);
      vert.setMaximum(vertsize);
      setVisibleAmount(1,1);

      add("South",horz);
      add("East",vert);
      horz.addAdjustmentListener(this);
      vert.addAdjustmentListener(this);
   }

   public void setVisibleAmount(double zmfx,double zmfy)
   {
      if(horz == null || vert == null)
	return; // not yet initialized !
      int h = (int)(horzsize/zmfx);
      int v = (int)(vertsize/zmfy);
      horz.setVisibleAmount(h);
      vert.setVisibleAmount(v);

      if(Debug.LEVEL >= 1)
	Debug.print("Visible amount : " + h + "," + v );
   }
   
   public void setScrollPosition(int zoffx, int zoffy)
   {
      if(horz == null || vert == null)
	return; // not yet initialized !
      int h = zoffx * horzsize/scaler.sdim.width;
      int v = zoffy * vertsize/scaler.sdim.height;
      horz.setValue(h);
      vert.setValue(v);

      if(Debug.LEVEL >= 1)
	Debug.print("ScrollPosition : " + h + "," + v );
 }
   
   public void adjustmentValueChanged(AdjustmentEvent ad)
   {
      scaler.setZoomOffset(horz.getValue() * scaler.sdim.width /horzsize,
			   vert.getValue() * scaler.sdim.height/vertsize);
      
      // Note: using sdim instead of canvas.getSize() makes scrolling faster !
      
      canvas.repaint();

      if(Debug.LEVEL >= 1)
	Debug.print("Bars: " + horz.getValue() + "," + vert.getValue());
   }
}
