cdccplcoli
      subroutine plcoli
c ======================================================================
c
c
c        programmer     Eef Hartman
c        version 1.0    date   28-06-94
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    this subroutine sets the machine independent color table
c    for usage by all low-level plotting routines in color.
c    The table is 3xN elements, in order Red, Green and Blue,
c    each element is between 0 (nothing of this color) and 1 (full
c    intensity).
c
c **********************************************************************
c
c                      KEYWORDS
c
c    plot
c    color
c ********************************************************************
c
cimp      implicit none
c
c **********************************************************************
c
c                      COMMON BLOCKS
c
      integer ncolor
      parameter (ncolor = 145)
      integer curcol
      real colors(3,ncolor)
      common /ccolor/ curcol, colors
      save /ccolor/
c
c                 /ccolor/
c  low level common to hold the color table for all color plotting
c  NOT to be used by user programs or routines!
c
c   ncolor     CONSTANT! Number of colors in table
c   curcol     current color of output device
c   colors     RGB values (between 0 and 1) of all colors to be used
c              color "1" is the default color (white for screen,
c              black for paper), rest has to be rendered to give
c              "as closely as possible" the same colors.
c  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      integer idirec, jtest, ioutsd, dcolor
      double precision wid, ale, xfct, yfct, xprec, yprec, xmint, xmaxt
      double precision ymint, ymaxt, leftm, botm, rightm, topm, aleng,
     +                 width
      common /cplaf/ wid, ale, xfct, yfct, xprec, yprec, xmint, xmaxt,
     +               ymint, ymaxt, leftm, botm, rightm, topm, aleng,
     +               width, idirec, jtest, ioutsd, dcolor
      save /cplaf/
c
c                  /cplaf/
c    Communication between the low-level plotting routines.
c
c    wid,ale    -  Width resp. length of the plot in the vertical resp.
c                  horizontal direction in plotting units
c    xfct,yfct  -  Multiplication factors in resp. x and y-direction.
c                  Such that all coordinates will be transformed into the
c                  unit square (0,1) x (0,1)
c    xprec,yprec - Coordinates of the last point that has been plotted
c    xmint,xmaxt,ymint,ymaxt - Window of coordinates to be plotted.
c                              All points with (x,y) within rectangle
c                              (xmint,xmaxt) x (ymint,ymaxt) are plotted
c    leftm,botm,rightm,topm  - Margins in cm around plotting area
c    aleng         Length of plot in cm's
c    width         Width of plot in cm's
c    idirec     -  Direction of plotting used:
c       1          Normal direction
c      -1          Rotated over 90 degrees
c    jtest      -  Test parameter for the window. Possibilities:
c       0          No tests performed
c       1          Test if point are within rectangle.
c    ioutsd     -  Used when JTEST = 1. Meanings are:
c       0          Both new and preceding point are within rectangle
c       1          New point is within, preceding outside
c       2          New is outside, preceding inside
c       3          Both new and preceding point are outside rectangle
c    dcolor     -  Default (current) color to draw lines and text in
c       0          Do not use any color handling
c
c  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
c                    LOCAL PARAMETERS

      integer i

c    *  The standard colour is white  (1)

      colors(1,1) = 1.0
      colors(2,1) = 1.0
      colors(3,1) = 1.0

c    *  The first colour set consists of 21 colours
c       Start at 1 (internally 2)
c       11 colours from blue to red
c       11 colours from red to yellow

      do 210 i=1,10
         colors(1,i+1)=(i-1)/10.0
         colors(2,i+1)=0.0
         colors(3,i+1)=(11-i)/10.0
210   continue
      do 220 i=1,10
         colors(1,11+i)=1.0
         colors(2,11+i)=(i-1)/10.0
         colors(3,11+i)=0.0
220   continue
      colors(1,22)=1.0
      colors(2,22)=1.0
      colors(3,22)=0.0

c    *  The second colour set consists of 41 colours
c       Start at 22 (internally 23)
c       21 colours from blue to red
c       21 colours from red to yellow

      do 230 i=1,20
         colors(1,i+22)=(i-1)/20.0
         colors(2,i+22)=0.0
         colors(3,i+22)=(21-i)/20.0
230   continue
      do 240 i=1,20
         colors(1,42+i)=1.0
         colors(2,42+i)=(i-1)/20.0
         colors(3,42+i)=0.0
240   continue
      colors(1,63)=1.0
      colors(2,63)=1.0
      colors(3,63)=0.0

c    *  The third colour set consists of 41 colours
c       Start at 63 (internally 64)
c       21 colours from green to yellow
c       21 colours from yellow to red

      do 250 i=1,20
         colors(1,i+63)=(i-1)/20.0
         colors(2,i+63)=1.0
         colors(3,i+63)=0.0
250   continue
      do 260 i=1,20
         colors(1,83+i)=1.0
         colors(2,83+i)=(21-i)/20.0
         colors(3,83+i)=0.0
260   continue
      colors(1,104)=1.0
      colors(2,104)=0.0
      colors(3,104)=0.0

c    *  The fourth colour set consists of 41 colours
c       Start at 104 (internally 105)
c       41 colours from white to black

      do 270 i=1,40
         colors(1,i+104)=(41-i)/40.0
         colors(2,i+104)=(41-i)/40.0
         colors(3,i+104)=(41-i)/40.0
270   continue
      colors(1,145)=0.0
      colors(2,145)=0.0
      colors(3,145)=0.0

c
c    * now set curcol to 1 (default color) and dcolor to 0 (no color)
c

      curcol = 1
      dcolor = 0

c
      end
cdc*eor
