      subroutine askvecstvec ( inpsep, tabmx2, table2, tabout, isignvec,
     +                         number, numvec, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-05-2011 Remove warning
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store vector numbers for subroutine askvec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cinout'
      include 'SPcommon/cscalvec'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), isignvec(*), number, numvec
      double precision tabmx2(*), table2(*), tabout(*), rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     isignvec       o    Contains the signs of the two vectors in case of
!                         linear combination
!     number         i    Vector sequence number
!     numvec        i/o   Number of vectors read
!     rinsep         o    Contains real input at output. The storage depends on
!                         the storage of inpsep
!     table2         i    Contains values of items read corresponding to code2
!     tabmx2         i    Table containing the maximum values of the quantities
!                         given in code2
!     tabout         i    Contains the values found at output of intx02
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, istart, iseq1, iseq2
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq1          Sequence number of first input vector
!     iseq2          Sequence number of second input vector
!     istart         Starting address in tabout
!     n              Number of occurences of keyword found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1901   No (input) vector number found in the record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecstvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecstvec'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      n = nint(tabmx2(3))
      istart = nint(table2(3))
      isignvec(1) = 0
      isignvec(2) = 0
      if ( debug ) write(irefwr,1) 'n, istart', n, istart

      if ( n>=1 ) then

!     --- n>=1, at least one vector number found

         iseq1 = nint(tabout(istart))
         if ( debug ) write(irefwr,1) 'iseq1', iseq1
         if ( iseq1<0 ) then

!        --- Sequence number provided with minus sign

            isignvec(1) = -1
            iseq1 = -iseq1

         else

!        --- Sequence number not provided with minus sign

            isignvec(1) = 1

         end if

      else

         if ( inpsep(4)<=27 ) then
            iseq1 = abs(number)
         else
            call errsub ( 1901, 0, 0, 0 )
         end if

      end if

      if ( n>=2 ) then

!     --- n>=2, at least two vectors

         iseq2  = nint(tabout(istart+1))
         if ( debug ) write(irefwr,1) 'iseq2', iseq2
         if ( iseq2<0 ) then

!        --- Sequence number provided with minus sign

            isignvec(2) = -1
            iseq2 = -iseq2

         else

!        --- Sequence number not provided with minus sign

            isignvec(2) = 1

         end if

      else

!     --- Only one vector

         if ( inpsep(4)==107 ) then

!        --- sign(v) found

            iseq2 = 0

         else

!        --- standard case

            iseq2 = iseq1

         end if  ! ( inpsep(4)==107 )

      end if
      if ( debug ) then
         write(irefwr,1) 'iseq1, iseq2', iseq1, iseq2
         call prinin ( isignvec, 2, 'isignvec' )
      end if  ! ( debug )

      inpsep(3) = abs(number)
      inpsep(9) = iseq1
      inpsep(10) = iseq2
      numvec = min ( maxvectors, max(numvec,number) )
      if ( number<=maxvectors ) ivectors(number) = 1
      if ( inpsep(4)==29 .or. inpsep(4)==30 ) rinsep(1) = table2(5)

1000  call erclos ( 'askvecstvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecstvec'

      end if  ! ( debug )

      end
