      subroutine difsepars ( ibuffr, buffer, mmarr1, mmarr2 )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 09-11-2012 Long length ibuffr
!        version  1.0  date 06-09-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the difference between 2 sepran arrays referred to in mmarr1
!     and mmarr2 and store difference in first one
! **********************************************************************
!
!                       KEYWORDS
!
!     compare
!     sepran_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer mmarr1, mmarr2
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     mmarr1         i    Memory managment sequence number of array 1
!     mmarr2         i    Memory managment sequence number of array 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iparr1, iparr2
      integer length
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iparr1         Starting address of array with mm number mmarr1
!     iparr2         Starting address of array with mm number mmarr2
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer inidln

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2679   The length of the two SEPRAN arrays is not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'difsepars' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from difsepars'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call ini060text ( ibuffr, mmarr1, 'array_1' )
      call ini060text ( ibuffr, mmarr2, 'array_2' )
      length = inidln(mmarr1)
      if ( length/=inidln(mmarr2) ) then

!     --- Length of both arrays different

         call errchr ( 'array_1', 1 )
         call errint ( mmarr1, 1 )
         call errchr ( 'array_2', 2 )
         call errint ( mmarr2, 2 )
         call errint ( length, 3 )
         call errint ( inidln(mmarr2), 4 )
         call errsub ( 2679, 4, 0, 2 )
         go to 1000

      end if  ! ( length/=inidln(mmarr2) )

!     --- Compute starting addresses

      iparr1 = inidgttext ( mmarr1, 'array_1' )
      iparr2 = inidgttext ( mmarr2, 'array_2' )

!     --- Compute differences

      call daxpy ( length, -1d0, buffer(iparr1), 1, buffer(iparr2), 1 )

1000  call erclos ( 'difsepars' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End difsepars'

      end if

      end
