      subroutine er0083d ( coor, tang, inodes, nparts, ipoint )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 17-12-1999
!
!   copyright (c) 1999-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute tangential vector along curves in R^3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nparts, ipoint(*)
      double precision coor(3,*), tang(3,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     inodes  i    Array containing the nodal point numbers at the boundary
!     ipoint  i    Array of length nparts+1 containing the starting addresses
!                  in array inodes of the various parts the boundary consists
!                  of.
!                  ipoint(1) = 1, &
!                  number of nodes at part i = ipoint(i+1)-ipoint(i)
!     nparts  i    number of parts the boundary consists of
!     tang    o    In this array the tangential vector is stored
!                  tang(j,i) contains the j th component of the tangential
!                  vector in the i-th integration point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ipart, iend, i, nodprc, nodnxt, inodp
      double precision dx, dy, dz, h

!     dx      length of vector in x-direction
!     dy      length of vector in y-direction
!     dz      length of vector in z-direction
!     h       distance between two points at the boundary
!     i       Counting variable
!     iend    Last position used at a part
!     inodp   Nodal point number
!     ipart   Sequence number of part
!     istart  First position used at a part
!     nodnxt  Next nodal point at boundary
!     nodprc  Preceding nodal point at boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For ipart := 1 (1) nparts
!      For i := 1 (1) ipoint(ipart+1)-ipoint(ipart)
!         inodp := nodal_point sequence number
!         if ( i=first point ) then
!            t_i := x(2) - x(1)
!         else if ( i=last point ) then
!            t_i := x(last) - x(last-1)
!         else
!            t_i := x(i+1) - x(i-1)
!         n_i := vector perpendicular to t_i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      istart = ipoint(1)
      do ipart = 1, nparts
         iend = ipoint(ipart+1)-1
         do i = istart, iend
            if ( i.eq.istart ) then

!           --- First point

               nodprc = inodes(i)
               nodnxt = inodes(i+1)
               inodp   = nodprc
            else if ( i.eq.iend ) then

!           --- Last point

               nodprc = inodp
            else

!           --- Intermediate point

               nodprc = inodp
               inodp   = nodnxt
               nodnxt = inodes(i+1)
            end if
            dx = coor(1,nodnxt) - coor(1,nodprc)
            dy = coor(2,nodnxt) - coor(2,nodprc)
            dz = coor(3,nodnxt) - coor(3,nodprc)
            h = 1d0 / sqrt(dx**2+dy**2+dz**2)
            tang(1,i) = dx*h
            tang(2,i) = dy*h
            tang(3,i) = dz*h
            istart = iend+1
         end do
      end do
      end
