      subroutine er0091 ( filename, vector, ndegfd, kmesh )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.1  date 20-06-2013 New definition file ref numbers
!        version  2.0  date 27-12-2011 New parameter list
!        version  1.0  date 01-01-1998
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read element numbers and values from file and store in array vector
!     Real case.
!     At this moment only ndegfd degrees of freedom per point is permitted
! **********************************************************************
!
!                       KEYWORDS
!
!     read_file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndegfd, kmesh(*)
      double precision vector(ndegfd,*)
      character (len=*) filename

!     filename       i    Name of file
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     ndegfd         i    Number of degrees of freedom per point
!     vector        i/o   Solution array or array of special structure of type 116
!                         in which the values to be read are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnodal, k, inodp, idum, nelem, error
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:,:)

!     error          Return value of allocate or deallocate
!     idum           General dummy variable
!     inodp          Nodal point number
!     iwork          integer work array of length nelem
!     k              Counting variable
!     nelem          Number of elements
!     numnodal       Number of nodal points
!     work           Work array of length ndegfd*nelem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFILE       Read information from an ASCII file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1672   Nodal point number out of range
!    1679   Wrong number of elements found
! **********************************************************************
!
!                       PSEUDO CODE
!
!    set constants
!    Read numbers from file
!    Fill values in vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'er0091' )

!     --- set constants

      nelem = kmesh(9)

      allocate ( iwork(nelem), work(ndegfd,nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem*(ndegfd+1), 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read numbers from file

      numnodal = ndegfd
      call readfile ( filename, .false., numnodal, &
                      idum, iwork, work, 3 )
      if ( ierror/=0 ) go to 1000
      if ( numnodal>nelem ) then

!     --- Too many numbers in file

         call errint ( numnodal, 1 )
         call errint ( nelem, 2 )
         call errchr ( filename, 1)
         call errsub ( 1679, 2, 0, 1)
         go to 1000

      end if

!     --- Fill values in vector

      do k = 1, numnodal
         inodp = iwork(k)
         if ( inodp<=0 .or. inodp>nelem ) then
            call errint ( k, 1 )
            call errint ( inodp, 2 )
            call errint ( nelem, 3 )
            call errsub ( 1672, 3, 0, 0 )
            go to 1000
         end if
         vector(1:ndegfd,inodp) = work(1:ndegfd,k)

      end do

      deallocate ( iwork, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'er0091' )
      end
