      subroutine erfill119 ( ibuffr, buffer, isol, numsol, numsubcurvs, &
                             outvec, numnodes )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array outvec with the information of array ivectr of type 119
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ibuffr(*)
      integer, intent(in) :: numsol, isol(5,numsol), numsubcurvs, &
                             numnodes
      double precision, intent(in) :: buffer(*)
      double precision, intent(out) :: outvec(numnodes,numsol+1)

!     buffer         i    Buffer array
!     ibuffr         i    Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     numnodes       i    Number of nodal points in array outvec
!     numsol         i    Total number of solution vectors in isol
!     numsubcurvs    i    Number of subcurves
!     outvec         o    Contains the output vector
!                         (.,1) the x-coordinates
!                         (.,2..) the values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, isubcv, n
      integer (kind=8) :: ipisol, ipusol
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipisol         Starting address of integer part of isol
!     ipusol         Starting address of real part of isol
!     istart         First position used at a part
!     isubcv         Subcurve sequence number
!     n              number of nodes in subcurve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) :: inigettext, inidbl

!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfill119' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfill119'
         call prinin1 ( isol, numsol, 5, 'isol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over vectors

      do i = 1, numsol

         ipisol = inigettext ( isol(1,i), 'vector_1' )

!        --- Loop over subcurves

         ipusol = ipisol+numsubcurvs
         if ( intlen==2 .and. mod(numsubcurvs,2)/=0 ) ipusol = ipusol+1
         ipusol = inidbl(ipusol)

         istart = 1
         do isubcv = 1, numsubcurvs

            n = ibuffr(ipisol+isubcv-1)
            if ( i==1 ) then

!           --- First vector, copy coordinates

               call dcopy ( n, buffer(ipusol), 1, outvec(istart,1), 1 )

            end if  ! ( isubcv==1 )
            ipusol = ipusol+n  ! raise ipusol

!           --- Copy function values

            call dcopy ( n, buffer(ipusol), 1, outvec(istart,i+1), 1 )
            ipusol = ipusol+n  ! raise ipusol
            istart = istart+n

         end do  ! isubcv = 1, numsubcurvs

      end do  ! i = 1, numsol

1000  call erclos ( 'erfill119' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( outvec, numsol+1, numnodes, 'outvec' )
         write(irefwr,*) 'End erfill119'

      end if  ! ( debug )

      end
