      subroutine erfillnodskmc1 ( nodes, nnodes, kmeshc, inelem, inpelm, &
                                  iwork )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 17-10-2013 Extra debug
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for elements 1 to inelem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), nnodes, kmeshc(*), inelem, inpelm, iwork(*)

!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     iwork         i/o   integer work array of length npoint
!                         iwork(i) = 0, means node i has not yet been filled
!                         iwork(i)>0, means node i has been filled
!                         The value defines the new node number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         The first position must refer to the first element
!                         to be filled
!     nnodes        i/o   Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     inode          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfillnodskmc1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillnodskmc1'
         write(irefwr,1) 'inelem, inpelm', inelem, inpelm
         call prinin1 ( kmeshc, inelem, inpelm, 'kmeshc' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all elements

      do i = 1, inelem*inpelm

         inode = kmeshc(i)

         if ( inode>0 ) then

!        --- Node number positive

            if ( iwork(inode)==0 ) then

!           --- iwork(inode)=0, new node

               iwork(inode)=1
               nnodes = nnodes+1
               nodes(nnodes) = inode

            end if  ! ( iwork(inode)==0 )

         end if  ! ( inode>0 )

      end do  ! i = 1, inelem*inpelm

1000  call erclos ( 'erfillnodskmc1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End erfillnodskmc1'

      end if  ! ( debug )

      end
