      subroutine error3161 ( lenvec, length, name1, name2, name3 )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print error message 3161
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenvec, length
      character (len=*) name1, name2, name3

!     length         i    Length of vector on right-hand side of expression
!     lenvec         i    Length of vector on left-hand side of expression
!     name1          i    Name of first vector on right-hand side
!     name2          i    Name of second vector on right-hand side
!     name3          i    Name of vector on left-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3161   Vectors in the right-hand side have different lengths
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'error3161' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from error3161'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call errint ( lenvec, 1 )
      call errint ( length, 2 )
      call errchr ( name1, 12)
      call errchr ( name2, 2 )
      call errchr ( name3, 3 )
      call errsub ( 3161, 2, 0, 3 )

1000  call erclos ( 'error3161' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End error3161'

      end if  ! ( debug )

      end
