      subroutine ersvnm ( kmesh, kmeshc, solutn, number )
! ====================================================================== &
!
!
!        programmer     Guus Segal
!        version 1.0    date   03-02-1996
!
!
!
!   copyright (c) 1996-1996 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   Fill ninth point of element in vector by interpolation
!   Complex version
! ********************************************************************
!
!                      KEYWORDS
!
!    complex
!    fill_function
!    interpolation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer number, kmesh(*), kmeshc(*)
      double complex solutn(number,*)

!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     number     i    Number of degrees of freedom per point that must be filled
!     solutn    i/o   Array of length number x npoint in which the
!                     solution array is stored in the sequence:
!                     u (x ) = solutn ( i,k )  (i=1(1)number; k=1(1)npoint)
!                      i  k
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, inpelm, inelem, kelmf, kelmg, kelma, nelgrp, ikelmc, &
              ielgrp, ishape, ielem

!     i          Counting variable
!     ielem      Element number
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     inelem     Number of elements in a specific element group
!     inpelm     number of points per element
!     ishape     Shape number of elements
!     kelma      Indicates if KMESH part a has been filled (>0) or not (0)
!                If kelma>0, kelma is the starting address of KMESH part a
!                in KMESH
!     kelmf      Indicates if KMESH part f has been filled (>0) or not (0)
!                If kelmf>0, kelmf is the starting address of KMESH part f
!                in KMESH
!     kelmg      Indicates if KMESH part g has been filled (>0) or not (0)
!                If kelmg>0, kelmg is the starting address of KMESH part g
!                in KMESH
!     nelgrp     Number of element groups
! ********************************************************************
!
!                      SUBROUTINES CALLED
!
! ********************************************************************
!
!                      I/O
!
! ********************************************************************
!
!                      ERROR MESSAGES
!
! ********************************************************************
!
!                      PSEUDO CODE
!
!   For all element groups do
!      If ( element has shape 6 ) then
!         Fill centroid by interpolation
! ====================================================================== &
!
      inpelm = kmesh(4)
      nelgrp = kmesh(5)
      inelem = kmesh(9)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)
      ikelmc = 0
      do 200 ielgrp = 1, nelgrp

!     --- Compute shape number, number of nodes and number of elements
!         of element group IELGRP

         if ( kelma.ne.0 ) inpelm = kmesh(kelma+ielgrp) - &
                                    kmesh(kelma+ielgrp-1)
         if ( nelgrp.gt.1 ) inelem = kmesh(kelmg+ielgrp-1)
         ishape = kmesh(kelmf+ielgrp-1)
         if ( ishape.ne.6 ) then

!        --- Not an element of shape 6

            ikelmc = ikelmc+inelem*inpelm

         else

!        --- element of shape 6

            do 150 ielem = 1, inelem

               do 100 i = 1, number
                  solutn(i,kmeshc(ikelmc+9)) = 0.25d0* &
                     ( solutn(i,kmeshc(ikelmc+2)) + &
                       solutn(i,kmeshc(ikelmc+4)) + &
                       solutn(i,kmeshc(ikelmc+6)) + &
                       solutn(i,kmeshc(ikelmc+8)) )
100            continue
               ikelmc = ikelmc + 9

150         continue

         end if

200   continue

      end
