      function getdimens ( iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with dimension of constant or scalar corresponding to iseq
!
! **********************************************************************
!
!                       KEYWORDS
!
!     array
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer getdimens, iseq

!     getdimens      o    dimension of constant or scalar corresponding to iseq
!     iseq           i    Sequence number of constant or scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqloc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseqloc        Local value of iseq
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getdimens' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getdimens'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqloc = mod(iseq,100000000)
      if ( iseqloc<5000 ) then
         getdimens = 0
      else
         iseqloc = iseqloc-5000
         if ( iseqloc>maxints+maxreals ) then
            iseqloc = iseqloc-maxints-maxreals
            getdimens = dimscals(iseqloc)
         else
            getdimens = dimcons(iseqloc)
         end if  ! ( iseqloc>maxints+maxreals )
      end if  ! ( iseq<5000 )

1000  call erclos ( 'getdimens' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getdimens'

      end if  ! ( debug )

      end
