      function getinusol ( nodp, kprobf, indprf, inunkp ) result(inusol)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of degrees of freedom in node, as
!     well as last position in usol before this node
! **********************************************************************
!
!                       KEYWORDS
!
!     solution_vector
!     position
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nodp, kprobf(*), indprf
      integer, intent(inout) :: inunkp
      integer :: inusol

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inunkp        i/o   Number of degrees of freedom in point
!     inusol         o    Position of unknown in array USOL (-1)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nodp           i    Nodal point sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getinusol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getinusol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprf==0 ) then

!     --- indprf = 0

         inusol = (nodp-1)*inunkp

      else

!     --- indprf>0

         inusol = kprobf(nodp)
         inunkp = kprobf(nodp+1)-inusol

      end if

1000  call erclos ( 'getinusol' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getinusol'

      end if  ! ( debug )

      end function getinusol
