      function getnameindint ( intname, loopname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the integer in the block
!     constants with name intname
!     If the name does not exist getnameindint gets the value 0
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname, loopname
      integer getnameindint

!     getnameindint  o    Sequence number of integer with respect to array
!                         chincons
!     intname        i    Name of integer
!     loopname       i    Name of loop variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1, iloop
      character (len=80) iname1, iname2

!     i              General loop variable
!     iloop          Value of loop variable
!     iname1         Name of integer translated to capitals
!     iname2         Name of integer translated to capitals
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getvar

!     ASKUPC         Translate all letters to upper case
!     GETVAR         Get the value of a scalar in the input block variables
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numints do
!        if ( intname = chincons(i) ) then
!           getnameindint := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      getnameindint = 0
      iloop = nint(getvar(loopname))
      nch1 = index ( intname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(intname)
      iname1 = intname(1:nch1)
      call askupc(iname1)
      do i = 1, numints
         nch = ncharcons(i)
         if ( nch1==nch ) then
            iname2 = chincons(i)(1:nch1)
            call askupc(iname2)
            if ( iname1(1:nch1)==iname2(1:nch1) ) then

!           --- name of integer found

               getnameindint = i+iloop-1
               go to 1000

            end if
         end if
      end do

1000  end
