      function getnameindvar ( varname, loopname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the scalar in the block
!     variables with name varname
!     If the name does not exist getnameindvar is not changed
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) varname, loopname
      integer getnameindvar

!     getnameindvar  o    Sequence number of variable with respect to array
!                         chscalars
!     loopname       i    Name of loop variable
!     varname        i    Name of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1, iloop
      character (len=80) vname1, vname2

!     i              General loop variable
!     iloop          Value of loop variable
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
!     vname1         Name of variable translated to capitals
!     vname2         Name of variable translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getvar

!     ASKUPC         Translate all letters to upper case
!     GETVAR         Get the value of a scalar in the input block variables
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numscals do
!        if ( varname = chscalars(i) ) then
!           getnameindvar := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      getnameindvar = 0
      iloop = nint(getvar(loopname))
      nch1 = index ( varname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(varname)
      vname1 = varname(1:nch1)
      call askupc(vname1)

      do i = 1, numscals

         nch = ncharscals(i)

         if ( nch1==nch ) then
            vname2 = chscalars(i)(1:nch1)
            call askupc(vname2)
            if ( vname1(1:nch1)==vname2(1:nch1) ) then

!           --- name of scalar found

               getnameindvar = i+iloop-1
               exit

            end if
         end if

      end do  ! i = 1, numscals

      end
