      function getnpelmsurf ( isurnr, ivol, kmesh, kmeshn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2000 extension with element groups
!        version  1.0  date 09-12-1995
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute number of nodal points in elements of surface number isurnr
!   Store the result in getnpelmsurf
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     test
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer getnpelmsurf, ivol, kmesh(*), kmeshn(*), isurnr

!     getnpelmsurf     o    Number of points in element
!     isurnr     i    Surface sequence number
!     ivol       i    Indication whether surfaces (0) or volumes (1) or
!                     element groups (-1) are used
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kmeshn     i    KMESH part n, contains nodal point numbers of the surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, kmesht, ielem, inelem, kmesha, kmeshg, &
              ielgrp

!     ielem      Element number
!     ielgrp     element group number
!     inelem     Number of elements in a specific element group
!     inpelm     number of points per element
!     kmesha     Starting address of kmesh part a in kmesh
!     kmeshg     Starting address of kmesh part g in kmesh
!     kmesht     kmesh part t, contains element numbers of elements with
!                highest dimensionality
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!    none
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    none
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ivol==0 .and. kmesh(28)/=0 ) then

!     --- surface elements, inpelm is stored in kmeshn

         inpelm = kmeshn(isurnr)

      else if ( kmesh(15)==0 ) then

!     --- all elements have the same number of nodes

         inpelm = kmesh(4)

      else if ( ivol==-1 ) then

!     --- element group

         kmesha = kmesh(15)
         inpelm = kmesh(kmesha+isurnr) - kmesh(kmesha+isurnr-1)

      else

!     --- More element groups, the first element number is stored in
!         kmesh part t

         kmesht = kmesh(36)
         ielem  = kmesh(kmesht+2*isurnr-2)

!        --- Find element group corresponding to ielem

         inelem = 0
         kmesha = kmesh(15)
         kmeshg = kmesh(21)
         do 100 ielgrp = 1, kmesh(5)
            inelem = inelem + kmesh(kmeshg-1+ielgrp)
            if ( ielem<=inelem ) then
               inpelm = kmesh(kmesha+ielgrp) - kmesh(kmesha+ielgrp-1)
               go to 200
            end if
100      continue

      end if

200   getnpelmsurf = inpelm
      end
