      function getscal ( iscal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with value of scalar
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iscal
      double precision getscal

!     getscal        o    Vlaue of scalar
!     iscal          i    Sequence number of scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETSCALINDEX   Return with adapted index of scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getscal'
         write(irefwr,1) 'iscal', iscal
  1      format ( a, 1x, (10i7) )
  2      format ( a, 1x, (5d18.10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = getscalindex ( iscal )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>0 ) then

!     --- iscal>0, get result

         getscal = scalars(iseq)  ! result

      else if ( iseq==0 ) then

!     --- iscal = 0, time

         getscal = t

      else

!     --- iscal<0, no result

         getscal = 0d0  ! result

      end if  ! ( iseq>10000 )

1000  call erclos ( 'getscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'getscal', getscal
         write(irefwr,*) 'End getscal'

      end if  ! ( debug )

      end
