      function getscalindex ( iscal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 22-03-2014 Allow expressions
!        version  1.2  date 08-12-2013 Extension with integer scalars
!        version  1.1  date 14-04-2013 Allow loop of scalars
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with adapted index of scalar
!     If result is negative it refers to an integer scalar
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_scalar
!     number
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cseqnrf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iscal
      integer getscalindex

!     getscalindex   o    Computed index of scalar
!     iscal          i    Sequence number of scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, iloop
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iloop          Value of loop variable
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3175   Expression found where no expression is expected
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getscalindex' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getscalindex'
         write(irefwr,1) 'iscal, iseqarray', iscal, iseqarray
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iscal>200000000 ) then

!     --- Error expression found

         call errsub ( 3175, 0, 0, 0 )
         go to 1000

      end if  ! ( iscal>200000000 )

      if ( iscal>=100000000 ) then
         iseq = iscal-100000000
      else
         iseq = iscal
      end if  ! ( iscal>=100000000 )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>10000 ) then

!     --- iseq>10000, indexed scalar

         iloop = iseq/10000          ! index is variable
         iseq = mod(iseq,10000)      ! base of scalar
         if ( iloop>0 ) iloop = nint(scalars(iloop))-1  ! value of index
         getscalindex = iseq+iloop  ! result
         if ( debug ) write(irefwr,1) 'iloop, iseq', iloop, iseq

      else if ( iseq>5000 ) then

!     --- iseq>5000, scalar array, take first one and provide with minus sign

         iseq = mod(iseq,5000)     ! base of scalar
         if ( iseqarray>0 .and. iseqarray<=dimscals(iseq) ) then
            iseq = iseq+iseqarray-1
         else
            iseq = -iseq
         end if  ! ( iseqarray>0 .and. iseqarray<=dimscals(iseq) )
         getscalindex = iseq         ! result

      else

!     --- iseq>0, standard case

         getscalindex = iseq       ! result

      end if  ! ( iseq>10000 )

1000  call erclos ( 'getscalindex' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'getscalindex', getscalindex
         write(irefwr,*) 'End getscalindex'

      end if  ! ( debug )

      end
