      function getvecindex ( ivec  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the adapted value of ivec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseqnrf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec
      integer getvecindex

!     getvecindex   o    Computed index of vector
!     ivec          i    Sequence number of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, iloop
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iloop          Index of loop in case of an array of vectors
!     iseq           Sequence number of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getvecindex' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getvecindex'
         write(irefwr,1) 'ivec, iseqarray', ivec, iseqarray
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ivec>=150000000 ) then
         iseq = ivec-150000000
      else
         iseq = ivec
      end if  ! ( ivec>=150000000 )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>10000 ) then

!     --- iseq>10000, indexed vector

         iloop = iseq/10000          ! index is variable
         iseq = mod(iseq,10000)      ! base of vector
         if ( iloop>0 ) iloop = iloop-1  ! value of index
         getvecindex = iseq+iloop  ! result
         if ( debug ) write(irefwr,1) 'iloop, iseq', iloop, iseq

      else if ( iseq>5000 ) then

!     --- iseq>5000, vector array, take first one and provide with minus sign

         iseq = mod(iseq,5000)     ! base of vector
         if ( iseqarray>0 .and. iseqarray<=dimnames(iseq) ) then
            iseq = iseq+iseqarray-1
         else
            iseq = -iseq
         end if  ! ( iseqarray>0 .and. iseqarray<=dimnames(iseq) )
         getvecindex = iseq         ! result

      else

!     --- iseq>0, standard case

         getvecindex = iseq       ! result

      end if  ! ( iseq>10000 )

1000  call erclos ( 'getvecindex' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getvecindex'

      end if  ! ( debug )

      end
