      subroutine ini0152 ( ibuffr, isepin, icall, maxcal, istart, &
                           ipblc2, ipblc3, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 24-03-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute information concerning input read by readal
!
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipblc2, ipblc3
      integer ibuffr(*), isepin, icall, maxcal, istart, isubr

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icall          o    Call sequence number for present subroutine
!     ipblc2         o    Starting address of Block 2 of ISEPIN
!     ipblc3         o    Starting address of Block 3 of ISEPIN
!     isepin         i    Reference with respect to INFOR for array ISEPIN
!                         containing information of all SEPRAN input
!     istart         o    Starting position in subblocks of ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!     maxcal         o    Maximum value of icall
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipsepi
      integer nsubr
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipsepi         Starting address of array ISEPIN
!     nsubr          Number of different subroutines that may store their
!                    information in ISEPIN
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRINISEPIN     Print the contents of array ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini0152' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini0152'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      call ini060text ( ibuffr, -isepin, 'sepin' )
      ipsepi = inigettext ( isepin, 'sepin' )
      if ( debug ) then
         nsubr = ibuffr(ipsepi)
         call prinisepin ( ibuffr(ipsepi), ibuffr(ipsepi+4), &
                           ibuffr(ipsepi+4+4*nsubr) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ipblc2 = ipsepi + ibuffr(ipsepi+1) - 1
      ipblc3 = ipsepi + ibuffr(ipsepi+2) - 1
      icall  = ibuffr(ipblc2+4*(isubr-1)+1)
      maxcal = ibuffr(ipblc2+4*(isubr-1)+2)
      istart = ibuffr(ipblc2+4*(isubr-1)+3)
      if ( debug ) then
         write(irefwr,100) 'isubr, icall, maxcal', isubr, icall, maxcal
         write(irefwr,100) 'istart ', istart
      end if  ! ( debug )

1000  call erclos ( 'ini0152' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini0152'

      end if

      end
