      subroutine ini017 ( ibuffr, isubr, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 24-12-2014 New definition clinsl
!        version  2.1  date 21-11-2012 Long integers
!        version  2.0  date 20-01-1999 Extra parameter ibuffr
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find the maximum sequence number ISEQNR for a specific subroutine ISUBR
!   Warning: This subroutine uses memory managment subroutines
!            Therefore pointers computed before must be recomputed
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/clinslf90'
      include 'SPcommon/carrayf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, ibuffr(*)

!     ibuffr i/o   Integer buffer array in which all large data is stored
!     iseqnr  o    Maximum sequence number with respect to the storage in
!                  array ISEPIN
!     isubr   i    Indicates the type of calling subroutine
!                  Possible values
!                  1:  START
!                  2:  PROBDF
!                  3:  COMMAT
!                  4:  CREATE
!                  5:  PRESDF
!                  6:  FILCOF
!                  7:  CHNCOF
!                  8:  SOLVE
!                  9:  OUTPUT
!                 10:  SEPCOMP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipsepi, ipblc2

!     ipblc2  Starting address of Block 2 of ISEPIN in IBUFFR
!     ipsepi  Starting address of array ISEPIN in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INI060  Reactivate existing array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Activat isepin
!   Compute starting addresses of block2 and block 3 in IBUFFR
!   Compute ICALL and iseqnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call ini060 ( ibuffr, -isepin )
      ipsepi = infor(isepin)
      ipblc2 = ipsepi + ibuffr(ipsepi+1) - 1
      iseqnr = ibuffr(ipblc2+4*(isubr-1)+2)
      end
