      subroutine ini066text ( ibuffr, index, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 28-09-2012 Jump in case ierror#0
!        version  1.1  date 04-06-2007 Fortran 90
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete a permanent array from the SEPRAN memory
!     The array is removed from common block CARRAY
!     If possible IBFREE and IINFOR are decreased
!     ini066text is identical to subroutine INI056 as described in the
!     Programmer's Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cnamtbf90'
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, ibuffr(*)
      character *(*) text

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     index          i    The value of index is the reference number of the
!                         array that must be deleted
!                         Reference number is meant with respect to array INFOR
!                         in common block IINFOR
!                         This means that the positions (i,INDEX) (i=1,2,3) in
!                         INFOR contain all the information about the storage
!                         of the array
!     text           i    String variable containing the name of the array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) last
      integer i, iextra

!     i         Loop variable
!     iextra    Extra space needed at both sides of the array for control
!               integers
!     last      Last position used by array in IBUFFR corresponding to index
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHK1        Check IBUFFR
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     743  array corresponding to index does not exist
!    1819  array corresponding to index has already been deleted
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( idebug>0 ) then
!       Check ibuffr
!    if ( index<0 or index>iinfor ) then
!       error index out of range
!    if ( array in IBUFFR ) then
!       if ( array is last one ) then
!          Decrease ibfree
!    else if ( array not in file 3 ) then
!       warning: array has already been deleted
!    if ( index=iinfor ) then
!       recompute iinfor
!    if ( idebug>1 ) then
!       Set new free space equal to infinity
!    Clear array from infor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini066text' )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call inichk1(ibuffr)
      end if

      if ( index<0 .or. index>iinfor ) then

!     --- index<0 or index>iinfor  array does not exist

         call errint ( index, 1 )
         call errint ( iinfor+1, 2 )
         call errchr ( text, 1 )
         call errsub ( 743, 2, 0, 1 )

      end if
      if ( index==0 ) go to 1000
      if ( ierror/=0 ) go to 1000

      if ( infor(index)>0 ) then

!     --- Array is positioned in IBUFFR, check if IBFREE may be decreased

         last = infor(index) + infor1(1,index) + iextra*2
         if ( ibfree==last .or. ibfree==last+1 ) then

!        --- Recompute ibfree

            ibfree = numarrs+1
            do i = 1, iinfor
               if ( i/=index .and. ibfree<=infor(i) ) &
                    ibfree = infor(i) + infor1(1,i) + iextra*2

            end do
            if ( intlen==2 .and. mod(ibfree,2_8)==0 ) ibfree=ibfree+1

         end if

      else if ( infor(index)==0 ) then

!     --- Warning array has already been deleted

         call errint ( index, 1)
         call errchr ( text, 1 )
         call errwar ( 1819, 1, 0, 1 )
         go to 1000

      end if

      if ( index==iinfor ) then

!     --- Recompute iinfor

         do i = index-1, 1, -1
            if ( infor(i)/=0 ) then
               iinfor = i
               go to 120
            end if
         end do

      end if

!     --- Clear array infor for this array

120   if ( idebug>1 ) then

!     --- idebug>1, set IBUFFR to IINFIN

         if ( infor(index)>0 ) then
            ibuffr(infor(index)-1:infor(index)+infor1(1,index)) = iinfin
         end if

      end if

      infor(index) = 0
      infor1(1:2,index) = 0

      if ( ioutp>2 ) then
         write(irefwr,*) ' Delete array ', namarr(index), &
                         ' with sequence number', index, &
                         ' in subroutine ', &
                           namsub(levsub-1)//':ini066text'
      end if

1000  index = 0
      call erclos ( 'ini066text' )

      end
