      subroutine iniactnmeshh ( ibuffr, buffer, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 11-06-2014 Adaptation for kmesh(48) = 0
!        version  1.3  date 11-09-2013 Remove ini062
!        version  1.2  date 21-11-2012 Long integers
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create nmesh part h
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmd1, ipkmd2, ipnmpc, ipnmpd
      integer kelmaa, lpnmph, melpnd, nedges, npoint, melped, error
      integer, allocatable :: iwork(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     ipkmd1         Starting address of KMESH part d1 in array IBUFFR
!     ipkmd2         Starting address of KMESH part d2 in array IBUFFR
!     ipnmpc         Pointer to nmesh part c
!     ipnmpd         Pointer to nmesh part d
!     iwork          integer work array to store kmeshh temporarily
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     lpnmph         length of array nmeshh
!     melped         maximum number of elements connected to one edges
!     melpnd         maximum number of elements connected to a node
!     nedges         number of edges
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIACTKMESHD   Fill array kmesh part d
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INICOMPMELPND  Compute melpnd, i.e. maximimum number of elements per node
!     INICOPBUF      Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents of
!                    iarray to ibuffr
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRINKMESH      Print array KMESH
!     TOPEDG05       creates the nmesh part h
!     TOPFILLKMESHAA Fill first part of array KMESH part aa
!     TOPFILLNMESHC  Fill array nmesh parts c and d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create kmesh part d if it does not exist
!     create nmeshc, nmeshd if it does not exist
!     Create temporary space for nmeshh
!     Compute maximum number of elements per node
!     Compute starting addresses
!     Fill nmesh part h (nmeshh) with actual data
!     Make nmeshh permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactnmeshh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactnmeshh'
  1      format ( a, 1x, (10i10) )

      end if

      if ( kmesh(48)==0 ) then

!     --- Kmesh part aa has not been filled, call topfillkmeshaa

         if ( debug ) write(irefwr,*) 'fill kmeshaa'
         call topfillkmeshaa ( kmesh )

      end if

      kelmaa = kmesh(48)-1   ! starting address of kmesh part aa

!     --- Array nmeshh does not exist, create
!         First check if nmeshc, nmeshd, kmeshd exist

      if ( kmesh(18)==0 ) then

!     --- kmesh part d does not exist: create

         if ( debug ) write(irefwr,*) 'before iniactkmeshd'
         call iniactkmeshd ( ibuffr, kmesh )

      end if
      if ( kmesh(kelmaa+33)==0 ) then

!     --- nmeshc, nmeshd do not exist: create

         if ( debug ) write(irefwr,*) 'before topfillnmeshc'
         call topfillnmeshc ( ibuffr, buffer, kmesh )

      end if

!     --- Activate old arrays nmeshc, nmeshd, kmeshd

      call ini060text ( ibuffr, kmesh(kelmaa+33), 'nmeshc' )   ! nmeshc
      call ini060text ( ibuffr, kmesh(kelmaa+34), 'nmeshd' )   ! nmeshd
      call ini060text ( ibuffr, kmesh(18), 'kmeshd' )          ! kmeshd

!     --- Create temporary space for nmeshh

      nedges = kmesh(kelmaa+4)   ! number of edges
      melpnd = kmesh(kelmaa+14)  ! maximimum number of elements per node
      if ( debug ) write(irefwr,1) 'melpnd', melpnd
      if ( melpnd==0 ) then

!     --- melpnd = 0, compute

         ipkmd1 = inigettext ( kmesh(18), 'kmeshd' )
         npoint = kmesh(8)
         if ( debug ) write(irefwr,*) 'before inicompmelpnd'
         call inicompmelpnd ( ibuffr(ipkmd1), melpnd, npoint )
         kmesh(kelmaa+14) = melpnd

      end if
      lpnmph = (melpnd+2)*nedges   ! overestimation of length of nmeshh
      if ( debug ) write(irefwr,1) 'melpnd, nedges, lpnmph', &
                                    melpnd, nedges, lpnmph

      allocate ( iwork(lpnmph), stat = error )
      if ( error/=0 ) call eralloc ( error, lpnmph, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      ipnmpc = inigettext ( kmesh(kelmaa+33), 'nmeshc' )   ! nmeshc
      ipnmpd = inigettext ( kmesh(kelmaa+34), 'nmeshd' )   ! nmeshd
      ipkmd1 = inigettext ( kmesh(18), 'kmeshd' )          ! kmeshd1
      ipkmd2 = ipkmd1+kmesh(8)+1          ! kmeshd2

!     --- Fill nmesh part h (nmeshh) with actual data

      if ( debug ) write(irefwr,*) 'before topedg05'
      call topedg05 ( ibuffr(ipnmpd), iwork, nedges, &
                      melped, lpnmph, ibuffr(ipnmpc), &
                      ibuffr(ipkmd1), ibuffr(ipkmd2) )

!     --- Check length of array

      call inichkarlen ( lpnmph, (melpnd+2)*nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Make nmeshh permanent

      call inicopbuf ( ibuffr, kmesh(kelmaa+38), iwork, lpnmph, &
                       'nmeshh' )
      kmesh(kelmaa+15) = melped  ! maximimum number of elements per edge

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug = true, print nmeshc and nmeshd

         call prinkmesh ( ibuffr, buffer, kmesh, 43 )
         write(irefwr,*) 'End iniactnmeshh'

      end if

1000  call erclos ( 'iniactnmeshh' )

      end
