      subroutine iniactnmeshj ( ibuffr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 26-01-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates nmesh part j, for each surface the face numbers (3d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     surface
!     face
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipnmeshj, ipnmeshk, ipkmsn
      integer kelmaa, mmnmeshj, nsurfs, nfaces, lnnmeshj, mmnmeshk
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmsn         Starting address of array KMESH part n
!     ipnmeshj       Starting address of array nmeshj
!     ipnmeshk       Starting address of array nmeshk
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     lnnmeshj       Length of array nmeshj
!     mmnmeshj       memory management sequence number of array nmeshj
!     mmnmeshk       memory management sequence number of array nmeshk
!     nfaces         Number of faces
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext
      integer ininumelsurfs

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     ININUMELSURFS  Return with number of elements in all surfaces
!     TOPNODFACES    Fill array nmeshk with all faces connected to the nodes
!     TOPSURFACES    Find the face numbers of all faces stored in the
!                    given surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactnmeshj' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactnmeshj'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kelmaa = kmesh(48)-1   ! starting address of kmesh part aa
      mmnmeshj = 0           ! memory management number of nmeshj
      nsurfs = kmesh(12)     ! number of surfaces
      nfaces = kmesh(kelmaa+5)  ! number of faces

!     --- Activate kmesh part n surfaces

      call ini060text ( ibuffr, -kmesh(28), 'kmeshn' )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create kmesh  a part nmeshk

      mmnmeshk = kmesh(kelmaa+41)
      if ( mmnmeshk>0 ) then

!     --- Array nmeshk already exists: activate

         call ini060text ( ibuffr, mmnmeshk, 'nmeshk' )

      else

!     --- Array nmeshk does not exist, create

         call topnodfaces ( ibuffr, kmesh )
         mmnmeshk = kmesh(kelmaa+41)

      end if  ! ( mmnmeshk>0 )

!     --- Create space for array nmeshj and make this array permanent

      ipkmsn = inigettext ( kmesh(28), 'kmeshn' )    ! starting address kmeshn
      lnnmeshj = ininumelsurfs ( ibuffr(ipkmsn), nsurfs )
      call ini081 ( ibuffr, mmnmeshj, lnnmeshj, 'nmeshj' )
      kmesh(kelmaa+40) = mmnmeshj

!     --- Compute starting addresses

      ipnmeshj = inigettext ( mmnmeshj, 'nmeshj' )   ! starting address nmeshj
      ipnmeshk = inigettext ( mmnmeshk, 'nmeshk' )   ! starting address nmeshk
      ipkmsn = inigettext ( kmesh(28), 'kmeshn' )    ! starting address kmeshn

!     --- Actually fill nmeshj

      call topsurfaces ( ibuffr(ipkmsn), nsurfs, ibuffr(ipnmeshj), &
                         ibuffr(ipnmeshk), nfaces )

1000  call erclos ( 'iniactnmeshj' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniactnmeshj'

      end if  ! ( debug )

      end
