      subroutine inich0 ( kmesh, kprob, isol, ipkprb )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-12-1992
!
!   copyright (c) 1992-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check if the arrays KMESH, KPROB and ISOL are filled properly
!   compute IPKPRB
! **********************************************************************
!
!                       KEYWORDS
!
!     check
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), isol(*), ipkprb

!     ipkprb  o    Starting address of array KPROB for actual problem number
!     isol    i    Standard SEPRAN array containing information of the solution
!                  array
!     kmesh   i    Standard SEPRAN array containing information of the mesh
!     kprob   i    Standard SEPRAN array, containing information of the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nprob, iprob, k

!     iprob   Actual problem number
!     k       Counting variable
!     nprob   Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      75   Array KMESH has not been filled
!     103   Array ISOL has not been filled
!     699   Problem number incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inich0' )
      if ( kmesh(2).ne.100 ) then
         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
      end if
      if ( kprob(2).ne.101 ) then
         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( isol(2).ne.110 .and. isol(2).ne.115 ) then
         call errint ( isol(2), 1 )
         call errsub ( 103, 1, 0, 0 )
      end if

      iprob = isol(4) / 1000 + 1
      nprob = max ( 1, kprob(40) )
      if ( iprob.lt.1 .or. iprob.gt.nprob ) then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0 )
      end if
      if ( ierror.ne.0 ) go to 1000
      ipkprb = 1
      do 100 k = 2, iprob
         ipkprb = ipkprb + kprob ( 2+ipkprb )
100   continue

1000  call erclos ( 'inich0' )
      if ( ierror.ne.0 ) call instop
      end
