      subroutine inicopbuf ( ibuffr, mmindex, iarray, length, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 17-07-2012 Use incopy
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate or create the array referred to by the memory management
!     index mmindex and copy the contents of iarray to ibuffr
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmindex, iarray(*), length
      character (len=*) namear

!     iarray         i    array to be copied
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     length         i    length of the array in words
!     mmindex        i    Memory management sequence number of array to be
!                         copied
!     namear         i    Name of array
!                         This name is used in the table if the array did not
!                         yet exist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipindex
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipindex        Starting address of array with memory management sequence
!                    number mmindex in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOPY         Copy one integer array into another
!     INI081         Create permanent space in array IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicopbuf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicopbuf'
         write(irefwr,1) 'length', length
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- create array

      call ini081 ( ibuffr, mmindex, length, namear )

!     --- compute starting address

      ipindex = inigettext(mmindex, namear)
      if ( debug ) write(irefwr,1) 'mmindex, ipindex', mmindex, ipindex

!     --- Copy array

      if ( debug ) write(irefwr,1) 'iarray', iarray(1)
      call incopy ( iarray, ibuffr(ipindex), length )
      if ( debug ) write(irefwr,1) 'ibuffr(ipindex)', ibuffr(ipindex)

1000  call erclos ( 'inicopbuf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicopbuf'

      end if  ! ( debug )

      end
