      subroutine inicopinput ( ibuffr, iinput, mmiinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 19-10-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy sepran integer array referenced by mmiinput into iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), iinput(*), mmiinput

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iinput         o    integer input array to be filled
!     mmiinput       i    Memory management sequence number with respect to
!                         iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipiinput
      integer length
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipiinput       Starting address of array iinput
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicopinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicopinput'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length = inilen(mmiinput)

!     --- Guarantee that array IINPUT is placed into IBUFFR

      call ini060text ( ibuffr, -mmiinput, 'iinput' )

!     --- Copy contents of IINPUT

      ipiinput = inigettext ( mmiinput, 'iinput' )

      iinput(1:length) = ibuffr(ipiinput:ipiinput+length-1)

1000  call erclos ( 'inicopinput' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicopinput'

      end if  ! ( debug )

      end
