      subroutine inicurvusp ( ibuffr, buffer, kmesh, userpoint, &
                              maxusp, numusp, ix )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 24-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all user points along ix axis with coordinates smaller than
!     that of userpoint and more than 0
!     order these user points in increasing sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), userpoint(*), maxusp, &
              numusp, ix
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ix             i    Defines direction of axis
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     maxusp         i    Declared length of array subuserp
!     numusp         o    Number of entries in array subuserp
!     userpoint     i/o   Contains the user point numbers along the
!                         axis sorted in ascending order
!                         userpoint(0) must contain userpoint at input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iprinput
      integer mmrinput, ndim
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iprinput       Starting address of array rinput
!     mmrinput       Memory management sequence number with respect to
!                    rinput
!     ndim           Dimension of the space.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INICURVUSP1    Find all user points along ix axis with coordinates
!                    smaller than that of userpoint and more than 0
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicurvusp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicurvusp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find starting position of rinput

      mmrinput = kmesh(35)
      call ini060text ( ibuffr, mmrinput, 'rinput' )
      iprinput = inidgttext ( mmrinput, 'rinput' )
      ndim = kmesh(6)

!     --- Find user points

      call inicurvusp1 ( kmesh, buffer(iprinput+2), userpoint, &
                         maxusp, numusp, ix, ndim )

1000  call erclos ( 'inicurvusp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicurvusp'

      end if  ! ( debug )

      end
