      subroutine inidbg ( ibuffr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 09-09-2013 New keyword swap_on
!        version  2.6  date 20-06-2013 New definition file ref numbers
!        version  2.5  date 21-11-2012 Long integers
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the file sepran.dbg is available
!     If the file exists, read it and initialize debug facilities
! **********************************************************************
!
!                       KEYWORDS
!
!     debug
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cmachnf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)

!     ibuffr    i/o   Integer buffer array in which all large data is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ih
      integer iref, irefsv, ioutsv, ios
      logical check

!     check   Indicates if the file exist (true) or not
!     ih      Help variable
!     ios     IOSTAT variable, returns from reading or writing
!     ioutsv  Help variable to store the input value of ioutp
!     iref    File reference number
!     irefsv  Help variable to store the input value of irefre
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
!    This subroutine reads the file sepran.dbg if available
!    The following statements of this file are recognised:
!    debug_level = i  (i=0,1,2)  Default: 0
!    int_infin   = ...           Default: 2**31-1
!    nbuffr      = ...           Default: nbuffr from program
!                                NBUFFR must not be larger than the one
!                                in the program
!    no_swap
!    swap_on
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      274  character could not be recognised
!      648  string could not be recognised
!     1317  error in opening of file
!     1811  idebug has wrong value
!     1812  No number found after DEBUG_LEVEL = &
!     1813  No number found after INT_INFIN = &
!     1817  No number found after NBUFFR = &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Set all parameters equal to 0
!   Check if the local file sepran.dbg exists
!   If ( exist sepran.dbg ) then
!      read the file and set parameters
!   if ( iinfin = 0 and idebug>1 ) then
!      iinfin = 2**31-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inidbg' )

!     --- Set parameters in common block

      idebug = 0
      iinfin = 0
      nbufls = 0
      nbufw1 = 0
      nbufw2 = 0
      nbufw3 = 0
      ifirws = 0
      narbuf = 0
      ioutsv = ioutp
      irefsv = irefre
      maxerror = 10
      noswap = 1

!     --- check if the local file sepran.dbg exists

      inquire ( file='sepran.dbg', exist=check )

      if ( check ) then

!     --- File exists, read information until end of file has been found

         iref = inisetnewref()
         open ( unit=iref, file='sepran.dbg', iostat=ios, &
            status = 'unknown' )
         if ( ios/=0 ) then
            call errchr ( 'sepran.dbg', 1 )
            call errint ( ios, 1)
            call errsub ( 1317, 1, 0, 1 )
         end if

         irefre = iref
         ityprd = 1
         istop  = 2
         ioutp  = -1

!        --- While not end of record read and interpret next item

100      if ( ityprd/=0 ) then

            call asknxt
            if ( ityprd==0 ) then

!           --- end of file found, ready

               go to 100

            else if ( ityprd==4 ) then

!           --- keyword has been found

               if ( string(1:5)=='DEBUG' ) then

!              --- debug_level found, read level

                  call asknxt
                  if ( ityprd==2 ) then
                     idebug = nint(realrd)
                     if (idebug<0 .or. idebug>2 ) then
                        call errint ( idebug, 1 )
                        call errsub ( 1811, 1, 0, 0 )
                        call errcrd
                     end if
                  else
                     call errsub ( 1812, 0, 0, 0 )
                     call errcrd
                  end if

               else if ( string(1:4)=='INT_' ) then

!              --- int_infin found, read iinfin

                  call asknxt
                  if ( ityprd==2 ) then
                     iinfin = nint(realrd)
                  else
                     call errsub ( 1813, 0, 0, 0 )
                     call errcrd
                  end if

               else if ( string(1:4)=='NBUF' ) then

!              --- nbuffr, read nbuffr

                  call asknxt
                  if ( ityprd==2 ) then
                     ih = nint(realrd)
                     nbuffr = min ( nbuffr, ih )
                  else
                     call errsub ( 1817, 0, 0, 0 )
                     call errcrd
                  end if

               else if ( string(1:)=='NO_SWAP' ) then

!              --- no_swap found, set noswap

                  noswap = 1

               else if ( string(1:)=='SWAP_ON' ) then

!              --- swap_on found, set noswap

                  noswap = 0

               else

                  call errchr ( string(1:9), 1 )
                  call errchr ( 'DEBUG_LEVEL', 2 )
                  call errchr ( 'INT_INFIN', 3 )
                  call errchr ( 'NBUFFR', 4 )
                  call errsub ( 648, 0, 0, 4 )
                  call errcrd

               end if

            else

               call errsub ( 274, 0, 0, 0 )
               call errcrd

            end if

            go to 100

         end if

      end if

      if ( idebug<=1 .or. (idebug>1 .and. iinfin==0) ) then

!     --- set iinfin

         iinfin = 2 ** 15
         if ( machin==7 ) then
            iinfin = 2 ** 23
         else
            iinfin = iinfin ** 2
         end if
         iinfin = 2*(iinfin-1)+1

      end if

      if ( idebug>0 ) then

!     --- debugging, so set parameters

         ibfree = ibfree + intlen
         if ( idebug>1 ) then
            nbufls = nbuffr
            ih = 1
            ih = max(ih,ibfree-intlen)
            ibuffr(ih:nbufls) = iinfin

         end if

      end if
      if ( check ) then
         call inirmref ( iref )
         close(iref)
      end if  ! ( check )

      istop  = 0
      ioutp  = ioutsv
      irefre = irefsv
      nbufmx = nbuffr
      irecnr = 0
      ityprd = 1

      call erclos ( 'inidbg' )
      end
