      function inidnrm2 ( ibuffr, buffer, ivecx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 10-02-2005 Divide by number of entries
!        version  1.0  date 23-09-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute l_2 norm of sepran vector and store in inidnrm2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     norm
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), ivecx(*)
      double precision inidnrm2, buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inidnrm2       o    Computed l_2 norm
!     ivecx          i    Vector x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvecx
      integer itypvc, nusol
      logical debug, sepranvector

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipvecx         Starting address of array vecx
!     itypvc         Type of sepran vector
!     nusol          Total number of unknowns
!     sepranvector   if true the vector is a known seran vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      double precision dnrm2

!     DNRM2          Blas routine to compute the 2-norm of a vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate array
!     Compute starting addresses
!     Perform operation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inidnrm2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inidnrm2'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check array

      itypvc = ivecx(2)
      sepranvector = itypvc==110 .or. itypvc==115 .or. &
                     itypvc==116 .or. itypvc==126 .or. &
                     itypvc==129
      if ( .not.sepranvector ) then

!     --- Not a known sepran vector

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      end if  ! ( .not.sepranvector )

!     --- Activate array

      call ini060text ( ibuffr, ivecx(1), 'vecx' )

!     --- Compute starting addresses

      ipvecx = inidgttext ( ivecx(1), 'vecx' )
      nusol = ivecx(5)

!     --- Perform operation

      inidnrm2 = dnrm2 ( nusol, buffer(ipvecx), 1 ) / nusol

1000  call erclos ( 'inidnrm2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inidnrm2'

      end if

      end
