      subroutine iniedge2nodes ( nmeshl, inode1, inode2, kmeshaa, &
                                 iedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the edge corresponding to nodes inode1 and inode2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshl(*), inode1, inode2, kmeshaa(*), iedge

!     iedge          o    edge sequence number corresponding to nodes
!     inode1         i    First node in edge
!     inode2         i    Second node in edge
!     kmeshaa        i    KMESH part AA
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nfound, commonset(10), nedges, ipstart1, ipstart2
      logical debug

!     commonset      Work array to store the common set of edges belonging
!                    to both nodes
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipstart1       starting address in nmeshl corresponding to inode1
!     ipstart2       starting address in nmeshl corresponding to inode2
!     nedges         number of edges
!     nfound         number of entries in commonset
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICOMMONARRAY Compute common entries of two sets of arrays and
!                    store in new set
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2745   The number of edges corresponding to 2 nodes more than one
!    2746   No edges corresponding to 2 nodes
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniedge2nodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniedge2nodes'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000
      nedges = kmeshaa(4)
      ipstart1 = nmeshl(inode1)
      ipstart2 = nmeshl(inode2)
      call inicommonarray ( nmeshl(ipstart1+1), nmeshl(ipstart1), &
                            nmeshl(ipstart2+1), nmeshl(ipstart2), &
                            nedges, nfound, commonset )

      if ( nfound.gt.1 ) then

!     --- Error, more than 1 edge connected to nodes inode1 and inode2

         call errint ( inode1, 1 )
         call errint ( inode2, 2 )
         call errint ( nfound, 3 )
         call errsub ( 2745, 3, 0, 0 )

      else if ( nfound.eq.0 ) then

!     --- Error, no edge connected to nodes inode1 and inode2

         call errint ( inode1, 1 )
         call errint ( inode2, 2 )
         call errsub ( 2746, 2, 0, 0 )
         iedge = 0
         go to 1000

      end if  ! ( nfound.gt.1 )
      iedge = commonset(1)

1000  call erclos ( 'iniedge2nodes' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniedge2nodes'

      end if

      end
