      function iniedgeshape ( ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 22-05-2014 Extension with shape 22
!        version  1.5  date 10-03-2006 Extension with interface elements
!        version  1.4  date 07-04-2004 Extension for shape 134 (RvL)
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with number of edges in element as function of the shape number
!     If shape does not exists iniedgeshape gets value -1 and an error message
!     is given
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     shape
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iniedgeshape, ishape

!     iniedgeshape   o    Number of edges as function of the shape number
!     ishape         i    Shape number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer dedpel(18)

!     dedpel         number of edges for each element type
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data dedpel / 1, 1, 3, 3, 4, &
                    4, 3, 3, 4, 3, &
                    6, 6,12,12, 6, &
                    6,12, 6 /
! ======================================================================
!
      if ( ishape==-10001 .or. ishape==-9 ) then
         iniedgeshape = 1
      else if ( 1<=ishape .and. ishape<=18 ) then
         iniedgeshape = dedpel(ishape)
      else if ( ishape>=20 .and. ishape<=22 ) then
         iniedgeshape = 9
      else if ( ishape<=0 .or.ishape==109 .or. &
                ishape>=50 .and. ishape<=60 .or. &
                ishape>70 .and. ishape<=80 .or. &
                ishape==134 ) then
         iniedgeshape = -1
      else

!     --- Shape number not implemented

         call eropen ( 'iniedgeshape' )
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         call erclos ( 'iniedgeshape' )
         iniedgeshape = -1

      end if
      end
