      function inimaxnrm ( ibuffr, buffer, ivecx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 03-01-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute max norm of sepran vector and store in inimaxnrm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     norm
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), ivecx(*)
      double precision inimaxnrm, buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inimaxnrm      o    Computed max norm
!     ivecx          i    Vector x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvecx
      integer itypvc, nusol, imax
      logical debug, sepranvector

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     imax           Index of array with largets element
!     ipvecx         Starting address of array vecx
!     itypvc         Type of sepran vector
!     nusol          Total number of unknowns
!     sepranvector   if true the vector is a known seran vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer idamax

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     IDAMAX         Blas subroutine defining the index of the maximum
!                    element in an array
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     Activate array
!     Compute starting addresses
!     Perform operation
! ======================================================================
!
      call eropen ( 'inimaxnrm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inimaxnrm'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check array

      itypvc = ivecx(2)
      sepranvector = itypvc==110 .or. itypvc==115 .or. &
                     itypvc==116 .or. itypvc==126 .or. &
                     itypvc==129
      if ( .not.sepranvector ) then

!     --- Not a known sepran vector

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      end if  ! ( .not.sepranvector )

!     --- Activate array

      call ini060text ( ibuffr, ivecx(1), 'vecx' )

!     --- Compute starting addresses

      ipvecx = inidgttext ( ivecx(1), 'vecx' )
      nusol = ivecx(5)

!     --- Perform operation

      imax = idamax ( nusol, buffer(ipvecx), 1 )
      inimaxnrm = abs(buffer(ipvecx+imax-1))

1000  call erclos ( 'inimaxnrm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inimaxnrm'

      end if

      end
