      subroutine ininodeelgrp1 ( kmeshc, nodes, nnodes, lenkmeshc, &
                                 map, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all nodes of one element group in array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkmeshc, kmeshc(lenkmeshc), nodes(*), nnodes, &
              npoint, map(npoint)

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements in one element group
!     lenkmeshc      i    length of array kmeshc for this group
!     map            o    Mapping array of length npoint to make an
!                         inverse mapping of array nodes
!                         map(i) = 0, means that node i is not in the
!                                     element group
!                         map(i)>0, defines position of node i in array nodes
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers of the nodes in the
!                         element group
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodeelgrp1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodeelgrp1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array map

      map = 0

!     --- Fill array nodes and compute nnodes

      nnodes = 0
      do i = 1, lenkmeshc

         nodp = kmeshc(i)  ! node in element group
         if ( map(nodp)==0 ) then

!        --- New node found

            nnodes = nnodes+1
            map(nodp) = nnodes
            nodes(nnodes) = nodp

         end if  ! ( map(nodp)==0 )

      end do  ! i = 1, lenkmeshc

1000  call erclos ( 'ininodeelgrp1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininodeelgrp1'

      end if  ! ( debug )

      end
