      function ininumnodedge ( ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 22-05-2014 Extension with shape 22
!        version  2.0  date 14-09-2003 Remove iedge from parameter list
!        version  1.0  date 23-12-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with number of nodes per edge in edge of element as function of the
!     shape number and the edge sequence number
!     If shape does not exists ininumnodedge gets value -1 and an error message
!     is given
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     shape
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ininumnodedge, ishape

!     ininumnodedge  o    number of nodes per edge
!     ishape         i    Shape number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnodesedge(22)

!     numnodesedge   Array containing the number of nodes per edge as
!                    function of the shape number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data numnodesedge / 2, 3, 2, 3, 2,    3, 3, 4, 2, 2, &
                          2, 3, 2, 3, 3,    3, 2, 2, 0, 0, &
                          2, 3 /
! ======================================================================
!
      if ( ishape==-10001 ) then
         ininumnodedge = 1
      else if ( ishape==-9 ) then
         ininumnodedge = 2
      else if ( ishape>=1 .and. ishape<=18 .or. &
                ishape>=21 .and. ishape<=22 ) then
         ininumnodedge = numnodesedge(ishape)
      else if ( ishape<=0 ) then
         ininumnodedge = -1
      else

!     --- Shape number not implemented

         call eropen ( 'ininumnodedge' )
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         call erclos ( 'ininumnodedge' )
         ininumnodedge = -1

      end if
      end
