      subroutine iniopenfile ( ifile, irefwrsav, appendmode, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Open serial file and set irefwr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     file
!     open_file
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepcmf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ifile, appendmode
      integer, intent(out) :: irefwrsav
      character (len=*) :: filename

!     appendmode     i    if 0 the file is opened in standard mode
!                         if 1 in append mode
!     filename       i    Name of file
!                         is only used if ifile = 0
!     ifile          i    Sequence number of file name in array texts
!                         if 0, filename is used
!     irefwrsav      o    Help variable to store the value of irefwr
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nchar, length
      character (len=200) namefile
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     length         number of characters in a string
!     namefile       String variable containing name of the file
!     nchar          Number of characters in a file name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniopenfile' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniopenfile'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      irefwrsav = irefwr
      irefwr = inisetnewref()
      if ( ifile>0 ) then
         namefile = texts(ifile)
      else
         namefile = filename
      end if  ! ( ifile>0 )

      length = len(namefile)
      nchar = index(namefile, ' ' )
      if ( nchar==0 ) nchar = length

      if ( appendmode==0 ) then

!     --- Standard mode

         open ( irefwr, file = namefile(1:nchar) )

      else

!     --- Append mode

         open ( irefwr, file = namefile(1:nchar), position = 'append' )

      end if  ! ( appendmode==0 )

1000  call erclos ( 'iniopenfile' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniopenfile'

      end if  ! ( debug )

      end
