      function inistelgkmeshc ( kmesh, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find starting address of element group in kmeshc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
!     start
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inistelgkmeshc, kmesh(*), ielgrp

!     ielgrp         i    element group number
!     inistelgkmeshc o    starting address of element group in kmeshc
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelma, ikelmg, inpelm, iel, inelem
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iel            i'th element group
!     ikelma         Starting address of kmesh part a in kmesh
!     ikelmg         Starting address of kmesh part g in kmesh
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistelgkmeshc' )
      inistelgkmeshc = 1
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistelgkmeshc'
         write(irefwr,1) 'ielgrp', ielgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ikelma = kmesh(15)
      ikelmg = kmesh(21)

      do iel = 1, ielgrp-1
         inpelm = kmesh(ikelma+iel)-kmesh(ikelma+iel-1)
         inelem = kmesh(ikelmg+iel-1)
         inistelgkmeshc = inistelgkmeshc+inpelm*inelem
         if ( debug ) &
            write(irefwr,1) 'iel, inpelm, inelem, inistelgkmeshc', &
                             iel, inpelm, inelem, inistelgkmeshc
      end do  ! iel = 1, ielgrp

1000  call erclos ( 'inistelgkmeshc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inistelgkmeshc'

      end if  ! ( debug )

      end
