      function inistelkmeshc ( kmesh, ielem, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with starting address of element in kmeshc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     start
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inistelkmeshc, kmesh(*), ielem, inpelm

!     ielem          i    Element number
!     inistelkmeshc  o    starting address of element in kmeshc
!     inpelm         o    number of points per element
!     kmesh          i    Standard SEPRAN array containing information of the
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelgrp, inelem, kelma, kelmg, ikelmc, jelem, &
              ielgrp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inelem         Number of elements in a specific element group
!     jelem          do loop variable with respect to the elements
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistelkmeshc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistelkmeshc'
         write(irefwr,1) 'ielem', ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nelgrp = kmesh(5)
      inelem = kmesh(9)
      inpelm = kmesh(4)
      kelma  = kmesh(15)
      kelmg  = kmesh(21)
      ikelmc  = 0
      jelem = 0

      do ielgrp = 1, nelgrp

         if ( nelgrp>1 ) inelem = kmesh(kelmg+ielgrp-1)
         if ( kelma/=0 ) &
            inpelm = kmesh(kelma+ielgrp)-kmesh(kelma+ielgrp-1)
         if ( jelem+inelem>=ielem ) then

!        --- element group found

            ikelmc = ikelmc + (ielem-jelem-1)*inpelm
            exit

         end if
         jelem = jelem + inelem
         ikelmc = ikelmc+inpelm*inelem

      end do

1000  call erclos ( 'inistelkmeshc' )

      inistelkmeshc = ikelmc+1

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ikelmc, inpelm', ikelmc, inpelm
         write(irefwr,*) 'End inistelkmeshc'

      end if  ! ( debug )

      end
