      subroutine inistrnew ( number, kprob, kmesh, itypvc, ivec, iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 04-03-2011 Set iprob
!        version  1.2  date 04-03-2011 Extension with probextend
!        version  1.1  date 05-05-2007 Replace part by subroutine
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!     If this type does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, kprob(*), itypvc, ivec, iprob, kmesh(*)

!     iprob          i    Actual problem number
!     itypvc        i/o   Indication of the type of solution vector
!                         If itypvc = 110 at input then the output vector may be
!                         a vector of type 110 or 115 else of type 115 only
!                         Output:
!                         110: solution vector
!                         115: vector of special structure
!     ivec           o    Sequence number of vector of special structure
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INISTRNEW1     Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PROBEXTEND     Extend problem definition with vectors of special structure
!                    of exactly number degrees of freedom
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( solution vector contains exactly NUMBER unknowns per point ) then
!        itypvc := 110
!     else
!        ivec := 0
!        For i := 1 (1) NDTVEC, while ( ivec=0 ) do
!           If ( vector of special structure i has exactly NUMBER unknowns
!                per point then )
!              ivec := i
!              itypvc := 115
!        If ( ivec=0 ) then
!           Error: no vector found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistrnew' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistrnew'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      call inistrnew1 ( kprob, iprob, itypvc, ivec, number, kmesh )

      if ( debug ) &
         write(irefwr,1) 'itypvc, ivec, iprob', itypvc, ivec, iprob
      if ( itypvc==115 .and. ivec==0 ) then

!     --- No vector found

         call probextend ( kprob, number )
         ivec = kprob(6)
         iprob = 1

      end if  ! ( ivec==0 )

1000  call erclos ( 'inistrnew' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inistrnew'

      end if

      end
