      subroutine inivolsurface ( kmesh, kmeshr, isurnr, volsurfaces, &
                                 numvolms, subsurfaces, iwork, &
                                 volume_surfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-11-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all volumes corresponding to surface number isurnr and store
!     in array volsurfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     surface
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), isurnr, volsurfaces(*), numvolms, kmeshr(*), &
              subsurfaces(*), iwork(*), volume_surfs(*)

!     isurnr         i    Surface sequence number
!     iwork               integer work array of length nsurfs
!                         iwork is filled as follows:
!                         iwork(i) = 0: surface i is not a subsurface of isurnr
!                         iwork(i) = 1: surface i is a subsurface of isurnr
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     numvolms       o    number of volumes in array volsurfaces
!     subsurfaces    i    Array of length nsubsf in which all the subsurfaces
!                         of surface isurnr are stored with positive sign
!     volsurfaces    o    Array of length numvolms containing all volumes
!                         corresponding to surface isurnr
!     volume_surfs   i    Array containing all single surfaces corresponding
!                         to a volume
!                         Pos. 1 to nvolms contain the starting addresses of
!                         part 2.
!                         Part 2 contains the surface numbers for each volume
!                         in the following way:
!                         Pos. 1 number of surfaces followed by
!                         surface numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nsurfs, ivolnr, nvolms, istart, numsurfs, jsurnr, &
              nsubsf
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     istart         First position used at a part
!     ivolnr         Volume sequence number
!     jsurnr         Surface sequence number
!     nsubsf         Number of subsurfaces
!     nsurfs         number of surfaces
!     numsurfs       number of surfaces
!     nvolms         number of volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INISUBSURFACE  Find all subsurfaces of a composite surface and store in
!                    array subsurfaces
!     PRININ         print 1d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store all subsurfaces of surface isurnr in array subsurfaces
!     Clear array iwork
!     Mark array iwork for all surfaces in subsurfaces
!     For all volumes ivolnr do
!        If ( one of the surfaces in subsurfaces corresponds to ivolnr ) then
!           Store volume in array volsurfaces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inivolsurface' )
      debug = .false.
      nsurfs = kmesh(12)
      nvolms = kmesh(13)
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inivolsurface'
         write(irefwr,*) 'isurnr ',isurnr
         write(irefwr,*) 'Surfaces corresponding to volumes'
         call prinnmeshx ( nvolms, volume_surfs )

      end if

!     --- Store all subsurfaces of surface isurnr in array subsurfaces

      call inisubsurface ( kmesh, kmeshr, isurnr, nsubsf, &
                           subsurfaces )
      if ( debug ) call prinin ( subsurfaces, nsubsf, 'subsurfaces' )

!     --- Clear array iwork

      do i = 1, nsurfs
         iwork(i) = 0
      end do

!     --- Mark array iwork for all surfaces in subsurfaces

      do i = 1, nsubsf
         iwork(subsurfaces(i)) = 1
      end do

!     --- For all volumes ivolnr do
!         If ( one of the surfaces in subsurfaces corresponds to ivolnr ) then
!            Store volume in array volsurfaces

      numvolms = 0
      do ivolnr = 1, nvolms

         istart = volume_surfs(ivolnr)
         numsurfs = volume_surfs(istart)
         if ( debug ) &
            write(irefwr,*) 'ivolnr, numsurfs ',ivolnr, numsurfs

!        --- Loop over surfaces corresponding to volume

         do i = 1, numsurfs

            jsurnr = abs(volume_surfs(istart+i))
            if ( debug ) &
              write(irefwr,*) 'i, jsurnr, iwork(jsurnr) ', &
                               i, jsurnr, iwork(jsurnr)
            if ( iwork(jsurnr).gt.0 ) then

!           --- Surface is a subsurface of isurnr, store volume

               numvolms = numvolms+1
               volsurfaces(numvolms) = ivolnr
               go to 100

            end if

         end do  ! i = 1, numsurfs

100   end do  ! ivolnr = 1, nvolms

      call erclos ( 'inivolsurface' )

      end
