      program makef4
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 28-12-2000 Length of records extended to 240
!        version  2.3  date 07-03-1999 increase maxerrr to 3500 to
!                                      enable user-defined error messages
!                                      include common blocks, MR
!        version  2.2  date 08-02-1994 Replace ASK000 by ASKNXT
!
!   copyright (c) 1982-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  this program reads the error messages from the input file
!  and puts them in file 4
! **********************************************************************
!
!                       KEYWORDS
!
!     create_file
!     menu
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxerr, i, inext, ios, ifrom, ito, &
              idum, iref, iref95, iref96, icount
      parameter ( maxerr=3500 )
      integer istart(maxerr)
      logical finished

!     finished     If true the while loop is finished
!     i            General loop variable
!     icount       Counter
!     idum         Dummy parameter (not used in this program)
!     ierror       Preceding error that has been read
!     ifrom        Array ISTART is written from position IFROM to ITO
!     inext        Next record number to which an error must be written
!     ios          Integer giving the value of the i/o status specifier
!     iref         SEPRAN local reference number to which the error file is
!                  coupled
!                  In SEPRAN this is file 4. The actual reference number may be
!                  different (irefer). This number is set by INITMD
!     iref95       The unit containing the input data file
!     iref96       The unit to contain the messages and error-messages
!                  reported by this program
!     istart       In this array the starting position of the error messages
!                  on the file are stored.
!                  istart is stored in the records 2-(MAXERR+NREC4-1)/NREC4+1
!                  of the direct access file
!     ito          Array ISTART is written from position IFROM to ITO
!     maxerr       Maximum number of errors plus one that may be stored in the
!                  error message file
!     nerror       Error number read from the standard the input file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item from the standard input file
!     FINISH  Stop SEPRAN
!     INIFIL  Open files
!     INITMD  Set machine-dependent quantities
!     WRITFL  Write array to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!     Program MAKEF4 requires an input file consisting of records of
!     at most 80 characters (Standard card input)
!     The input should have the following structure:
!     For each error:
!     Error number followed by colon:
!        The error number must start in column 1
!     Followed by
!     Text describing the error message
!        The text must start after column 1 (So from column 2) and may not
!        exceed column 79.
!        The number of lines to be used for the error message is arbitrary
!     Error numbers must be given in the natural sequence, however,
!     gaps between errors are permitted
!     MAKEF4 produces 2 types of output:
!     Output to the standard output file
!        This output contains errors in the input file
!        and furthermore produces a list of gaps between error numbers
!     Output in the direct access output file
!        The direct access output file is a file with record length of 80 bytes
!        The first record contains one number: the largest error number
!        that is stored in this file
!        The next (MAXERR+NREC4-1)/NREC4 records contain the starting record
!        numbers of the error messages on the direct access file consequetively.
!        NREC4 defines the number of integers that can be stored in one record
!        of the error message file.
!        If an error is not available, the ending record number of the
!        preceding error plus one is stored.
!        In this way the number of records needed for error number i is
!        equal to istarti+1)-istart(i)
!        Finally the last records contain the error messages sequentially
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1    First column is not zero, but no number in input record
!    2    Error number smaller than preceding one
!    3    Error number larger than MAXMES:increase MAXMES
!    4    Error in the input file
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Initialize common blocks
!      Open error message file
!      Initialise current error number and currect record number
!      While (input not exhausted)
!         Read record from the input file
!         If (first column non-empty) then
!            find error number
!            Check consistency of error number
!            If (error number has wrong value) then
!               Print error message
!               stop
!            Else if (error number has correct value) then
!               Fill information in array istart
!         else if (first column empty) then
!            write record to error file
!            raise current record number
!      Fill final positions in array ISTART
!      Write array ISTART to records 2-(MAXERR+NREC4-1)/NREC4
!      Write first record to error message file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- SET common blocks (machine-dependent)

      call startsepran
      call initcb
      call initmd
      iref = 4

!     --- Open files (error message file)

      call inifil(-4)

!     --- Open the input file to be read

      iref95 = 95
      open ( iref95, file='error.dat' )
      iref96 = 96
      open ( iref96, file='error.out' )

!     --- Initialize parameters inext and ierror

      inext  = 2 + (maxerr+nrec4-1)/nrec4
      ierror = 0
      istart(1) = inext

!     --- Read errors from the input file until the input is exhausted

200   read ( iref95, 210, iostat = ios ) cards(1:80)
210   format(a80)
      if ( ios==0 ) then

!     --- ios = 0, so record has been read
!         Check if the first column is blank

         if ( cards(1:1)/=' ' ) then

!        --- cards(1:1) # blank, so number has been read
!            Interpret this number by subroutine ASKNXT

            if ( cards(1:1)<'0' .or. cards(1:1)>'9' ) then

!           --- Error 1:  inum=0, hence no number could be found

               write(iref96,220) ierror,(cards(i:i),i=1,80)
220            format(///' *******Error number 1 in program MAKEF4'///, &
                         ' First column of error message is not blank,', &
                         ' but no error number could be found'// &
                         ' Preceding error number read is:', i5/ &
                         ' record read is'//1x,79a1)
               stop

            else

               icount = 1
               finished = .false.
               do while ( .not. finished )
                  icount = icount+1
                  if ( cards(icount:icount)<'0' .or. &
                       cards(icount:icount)>'9' ) &
                     finished  = .true.
               end do
               icount = icount-1
               if ( icount==1 ) then
                  read(cards(1:1),221) nerror
221               format(i1)
               else if ( icount==2 ) then
                  read(cards(1:2),222) nerror
222               format(i2)
               else if ( icount==3 ) then
                  read(cards(1:3),223) nerror
223               format(i3)
               else if ( icount==4 ) then
                  read(cards(1:4),224) nerror
224               format(i4)
               else
                  read(cards(1:5),225) nerror
225               format(i5)
               end if

            end if

            if ( nerror>=maxerr ) then

!           --- Error 3:  nerror>=maxerr

               write(iref96,230) nerror,maxerr,(cards(i:i),i=1,80)
230            format(///' *******Error number 3 in program MAKEF4'///, &
                         ' Error number read is larger than MAXERR'/ &
                         ' Raise MAXERR'// &
                         ' Error number read is:', i5/ &
                         ' MAXERR is:', i5/ &
                         ' record read is'//1x,80a1)
               stop
            else if ( nerror<=ierror ) then

!           --- Error 2:  nerror<=ierror

               write(iref96,240) nerror,ierror,(cards(i:i),i=1,80)
240            format(///' *******Error number 2 in program MAKEF4'///, &
                         ' Error number read is not larger than', &
                         ' preceding error number read'// &
                         ' Error number read is:', i5/ &
                         ' Preceding error number read is:', i5/ &
                         ' record read is'//1x,80a1)
               stop
            end if
            if ( nerror>ierror+1 ) then

!           --- nerror>ierror+1, i.e. Gap found

               write(iref96,250) ierror+1, nerror-1
250            format(' Message numbers not in use:', i6, ' ...', i6 )

!              --- Fill array ISTART

               do 260 i = ierror+1, nerror-1
                  istart(i) = inext
260            continue
            end if
            ierror = nerror
            istart(ierror) = inext
         else

!        --- First column is blank, hence an error record has been found
!            Write error to file

            write ( irefer, rec = inext ) cards(1:80)
            inext = inext + 1
         end if
         go to 200
      else if ( ios<0 ) then

!     --- End of file reached
!         Fill rest of array ISTART and write to file 4

         istart(nerror+1) = inext
         inext = 2
         ifrom = 1
         ito   = nerror+1
         call writfl ( inext, ifrom, ito, idum, iref, istart )

!        --- write first record to error messages file

         istart(1) = nerror
         ifrom = 1
         ito   = 1
         inext = 1
         call writfl ( inext, ifrom, ito, idum, iref, istart )

      else

!     --- Error 4:  Error in input file

         write(iref96,300) ierror,(cards(i:i),i=1,80)
300      format(///' *******Error number 4 in program MAKEF4'///, &
                   ' Error in input file', &
                   ' Preceding error number read is:', i5/ &
                   ' record read is'//1x,79a1)
      end if
      close ( iref96 )
      call finish(0)
      end
