      function intx00ext ( code, ncode, nncode, ierpri, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.11 date 22-10-2015 Adaption for Si = or Vi = &
!        version  2.10 date 11-05-2014 New call to intx00keyw
!        version  2.9  date 15-03-2014 Special case for S = &
!
!   copyright (c) 1990-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     May be used to check whether a text read by ASKNXT
!     and stored in common CSEPCH, belongs to the table CODE
!     If this text belongs to CODE INTX00 returns with the sequence number
!     of this text in CODE, otherwise INTX00 gets the value 0
!     The number of characters that must be compared is stored in the
!     table NCODE
!     If the text does not belong to code it is checked if it is the left-hand
!     side of an expression and if so it marked as vector or scalar
!     if possible
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intx00ext, nncode, ncode(nncode), ierpri, ncodmx(nncode)
      character (len=*) code(nncode)

!     code           i    Table containing the texts to be compared
!                         nncode texts have been stored in code
!     ierpri         i    Indicates if intx00 must give an error message (1) or
!                         not (0)
!     intx00ext      o    Gives index number of the text that matches the code
!                         table
!                         So if string(1:ncode(i)) = code(1:ncode(i)), intx00
!                         gets the value i
!                         If none of the texts match, intx00 gets the value 0
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numchar, lastitemsav, itype(nncode), iposut, nout, &
              icountkey(1)
      double precision tabout(1), table(nncode), tabmax(nncode)
      character (len=80) keyword
      logical debug, extradebug, readnx

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     extradebug     If true extra debugging statements are carried out
!     i              General loop variable
!     icountkey      dummy
!     iposut         dummy
!     itype          dummy
!     keyword        Text containing the string without numbers
!     lastitemsav    Help parameter to save the value of lastitem
!     nout           dummy
!     numchar        Number of characters in keyword
!     readnx         dummy
!     table          dummy
!     tabmax         dummy
!     tabout         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     INTX00KEYW     Check if string read is (main) keyword
!     INTXCHKSCALVEC Check if item at right-hand side is scalar or vector
!                    if so add left-hand side to the list of scalars or vectors
!     PRININ         print 1d integer array
!     PRINSTR        Print an array of strings
!     READREALSCAL   Read integer, real or scalar
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!                    a number
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx00ext' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx00ext'
         write(irefwr,1) 'nncode, lastitem', nncode, lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
         if ( extradebug ) then
            call prinin ( ncode, nncode, 'ncode' )
            call prinstr ( code, nncode, 'code' )
         end if  ! ( extradebug )
         write(irefwr,*) 'string ', string
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- First we have to check whether the keyword contains numbers
!         If so we have to split the keyword into a part without number
!         and a number
!         This is necessary to check if it is a keyword, since keywords do
!         not contain numbers

      lastitemsav = lastitem
      call readsplitkeyw ( numchar, keyword )
      if ( debug ) then
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,*) 'keyword ', keyword(1:numchar)
         write(irefwr,1) 'ierpri, lastitemsav, lastitem', &
                          ierpri, lastitemsav, lastitem
      end if  ! ( debug )

      if ( numchar==1 .and. &
          (keyword(1:1)=='S' .or.keyword(1:1)=='V') .and. &
           lastitem==2 .and. nitems>3 ) then

!     --- This is the situation Si =, or Vi = &
!         We interpret Si and Vi as new items
!         not as scalar or vector
!         Therefore splitting is undone

         if ( typeitems(3)==13 ) then
            numchar = nchar
            keyword(1:numchar) = string(1:nchar)
         end if  ! ( typeitems(3)==13 )

      end if  ! ( numchar==1 .and. ... )

      nout = 1
      itype = 0
      call intx00keyw ( code, ncode, nncode, ncodmx, intx00ext, &
                        numchar, keyword, &
                        itype, nout, tabout, iposut, icountkey, &
                        table, readnx, tabmax )

      if ( debug ) &
         write(irefwr,1) 'intx00ext, ityprd', intx00ext, ityprd

      if ( (intx00ext==82 .and. string(1:nchar)=='S' .or. &
            intx00ext==12 .and. string(1:nchar)=='SCALAR') .or. &
           (intx00ext==68 .and. string(1:nchar)=='V' .or. &
            intx00ext==67 .and. string(1:nchar)=='VECTOR') &
           .and. nitems>1 ) then

!     --- S found, check if it is followed by equals sign

         if ( typeitems(lastitem+1)==13 ) then

!        --- Followed by =, hence new vector, reset intx00ext
!            First check if it is known vector or scalar

            lastitem = 0
            call readrealscal
            if ( debug ) &
               write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

            if ( ityprd==4 ) intx00ext = 0

         end if  ! ( typeitems(lastitem+1)==13 )

      end if  ! ( intx00ext>0 .and. string(1:nchar)='V' .and. nitems>1 )

      if ( debug ) then
         write(irefwr,1) 'intx00ext, ierpri, lastitem, nitems', &
                          intx00ext, ierpri, lastitem, nitems
         write(irefwr,1) 'lastitemsav', lastitemsav
      end if  ! ( debug )

      if ( intx00ext==0 ) then

!     --- intx00ext = 0, hence no keyword found
!         Check if second item is equals sign

          lastitem = lastitemsav
          if ( nitems>1 ) then

!         --- More than one item

             if ( typeitems(lastitemsav+1)==13 ) then

!            --- Equals sign found, check if we are dealing with a scalar
!                or vector
!                If so create one and add to the list of scalars or vectors

                call intxchkscalvec ( code, ncode, nncode, ncodmx, &
                                      intx00ext )

             end if  ! ( typeitems(2)==13 )

          end if  ! ( nitems>1 )

      end if  ! ( intx00ext==0 )

      if ( intx00ext==0 .and. ierpri==1 ) then

!     --- Print error message

         call errchr ( string(1:nchar), 1 )
         do i = 1, nncode
            call errchr ( code(i)(1:ncodmx(i)), 1+i )
         end do
         call errsub ( 648, 0, 0, nncode+1 )
         call errcrd

      end if

1000  call erclos ( 'intx00ext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'intx00ext, lastitem', intx00ext, lastitem
         write(irefwr,*) 'End intx00ext'

      end if

      end
