      subroutine intx00keyw1 ( icode, jcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 20-04-2012 Indexed scalars
!        version  1.2  date 12-12-2011 Extension with x_coor
!        version  1.1  date 12-09-2011 Extension with coor
!
!   copyright (c) 2008-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check information of vector name read
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     check
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/csepmainf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, jcode

!     icode          o    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     jcode          i    Special value of icode corresponding to vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer valmin, valmax, ivalrd
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ivalrd         Integer value read
!     valmax         Maximum value allowed for the integer read
!     valmin         Minimum value allowed for the integer read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx00keyw1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx00keyw1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = jcode
      ivalrd = mod(nint(realrd),10000)
      ivalrd = mod(ivalrd,5000)   ! correction for a series of vectors
      valmax = numvec
      if ( valmax==0 ) valmax = maxvc1
      valmin = 1
      if ( mainsubr==4 ) valmin = 0
      if ( ivalrd>-1000 .and. ivalrd<valmin .or. ivalrd>valmax ) then

!     --- Vector number out of range

         call errint ( ivalrd, 1 )
         call errint ( valmax, 2 )
         call errsub ( 2951, 2, 0, 0 )
         call errcrd

      end if  ! ( ivalrd<=0 .or. ivalrd>numvec )
      if ( debug ) &
         write(irefwr,1) 'icode, ivalrd, lastitem', &
                          icode, ivalrd, lastitem

1000  call erclos ( 'intx00keyw1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx00keyw1'

      end if  ! ( debug )

      end
