      function intx01 ( code, ncode, n, ierpri )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date 08-09-1998 Improved error message
!        version 1.0    date 24-10-1990
!
!
!
!   copyright (c) 1990-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine INTX01 may be used to check whether a text read by ASKNXT
!    and stored in common CSEPCH, belongs to the table CODE
!    If this text belongs to CODE INTX01 returns with the seuqnce number
!    of this text in CODE, otherwise INTX01 gets the value 0
!    The number of characters that must be compared is stored in the
!    integer NCODE
!    The difference with subroutine INTX00, is that all texts are compared
!    with the same number of characters
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer intx01, n, ncode, ierpri
      character *(*) code(n)

!     code     i  Table containing the texts to be compared
!                 n texts have been stored in code
!     ierpri   i  Indicates if an error message must be printed if
!                 the text can not be found (1) or not (0)
!     intx01   o  Gives index number of the text that matches the code table
!                 So if string(1:ncode(i)) = code(1:ncode(i)), intx01
!                 gets the value i
!                 If none of the texts match, intx01 gets the value 0
!     n        i  Number of texts stored in code
!     ncode    i  Number of characters that must be compared
!                 So string(1:ncode(i)) is compared with code(1:ncode(i))
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!    i      General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRCRD    Prints the last record read and underlines the errorneous part
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, n
         if ( string(1:ncode).eq.code(i) (1:ncode) ) then

!        --- Text found, return

            intx01 = i
            return

         end if

      end do

      intx01 = 0
      if ( ierpri.eq.1 ) then

!     --- Print error message

         call eropen ( 'intx01' )
         call errchr ( string(1:ncode), 1 )
         do i = 1, n
            call errchr ( code(i)(1:ncode), 1+i )
         end do
         call errsub ( 648, 0, 0, n+1 )
         call errcrd
         call erclos ( 'intx01' )

      end if

      end
