      subroutine intx0205 ( icountkey, icode, table, iretur, &
                            tabmin, tabmax, code, finished, readnx, &
                            itype, codemn, nmcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 04-04-2015 Allow digits in strings
!        version  1.1  date 23-01-2006 Debug information
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by one or two keywords
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, icountkey(*), itype(*), nmcode(*), iretur
      character (len=*) code(*), codemn(*)
      double precision table(*), tabmin(*), tabmax(*)
      logical finished, readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     icountkey      i    Array that counts the number of times a keyword has
!                         been found
!     iretur        i/o   Return value of subroutine INTX02
!                         Possible values:
!                         -2:  end of file has been reached
!                         -1:  error has been found
!                          0:  normal return
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, imax, imin, iminsv, istart, ispeclsav
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     imax           Maximum position in table
!     imin           Minimum position in table
!     iminsv         Help variable to store the previous value of imin
!     ispeclsav      Help parameter to save the value of ispecl
!     istart         Starting address of information about second keyword in
!                    itype
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
!    1628   Keyword found more than one for the same problem
!    1879   Not a text found after the keyword
!    1914   Not a second keyword found after keyword = &
!    2811   nmcode(i) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0205' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0205'
         write(irefwr,1) 'icode, itype(icode), ityprd', &
                          icode, itype(icode), ityprd
         write(irefwr,1) 'istop', istop
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
         write(irefwr,*) 'readnx ', readnx

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ispeclsav = ispecl
      ispecl = 9
      call asknxt
      ispecl = ispeclsav
      if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( itype(icode)==5 .and. icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else if ( itype(icode)==5 ) then
         icountkey(icode) = icountkey(icode)+1
      end if
      if ( ityprd/=4 ) then

!     --- Not a keyword found after this keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1879, 0, 0, 1 )
         call errcrd

      else

!     --- keyword found, check if it is in the table

         imin = nint(tabmin(icode))
         imax = nint(tabmax(icode))
         if ( debug ) write(irefwr,1) 'imin, imax', imin, imax
         do i = imin, imax
            if ( string(1:nmcode(i))== &
                 codemn(i)(1:nmcode(i)) ) then

!           --- Text found

               table(icode) = i-imin+1

               if ( debug ) then
                  write(irefwr,1) 'i, table', i, i-imin+1
                  write(irefwr,*) 'string ', string(1:nmcode(i))
               end if  ! ( debug )

               if ( itype(icode)==5 ) then
                  go to 1000
               else
                  go to 200
               end if

            end if

         end do

!        --- keyword not recognized

         call errchr ( string(1:nchar), 1 )
         do i = imin, imax
            call errchr ( codemn(i), 2+i-imin )
         end do
         call errsub ( 648, 0, 0, 2+imax-imin )
         call errcrd
         iretur = -1
         finished = .true.

!        --- itype(icode) = 13, check if second keyword should be
!            present

200      iminsv = imin
         istart = itype(icode+1)
         imin = itype(istart+(i-iminsv)*2)
         if ( imin==0 ) go to 1000
         imax = itype(istart+(i-iminsv)*2+1)
         if ( debug ) then
            write(irefwr,1) 'istart, iminsv, i', istart, iminsv, i
            write(irefwr,1) 'imin, imax, ityprd', imin, imax, ityprd
         end if  ! ( debug )

!        --- Read next keyword

         call asknxt
         if ( debug ) write(irefwr,1) 'ityprd', ityprd

         if ( ityprd/=4 ) then

!        --- Not a keyword found after this keyword

            call errchr ( code(icode), 1 )
            iminsv = nint(table(icode))+iminsv-1
            call errchr ( codemn(iminsv), 2 )
            call errsub ( 1914, 0, 0, 2 )
            call errcrd

         else

!        --- keyword found, check if it is in the table

            do i = imin, imax

               if ( nmcode(i)<1 .or. nmcode(i)>1000 ) then

!              --- nmcode(i) out of range

                  call errint ( i, 1 )
                  call errint ( nmcode(i), 2 )
                  call errint ( 1, 3 )
                  call errint ( 1000, 4 )
                  call errsub ( 2811, 4, 0, 0 )
                  go to 1000

               end if

               if ( string(1:nmcode(i))== &
                    codemn(i)(1:nmcode(i)) ) then

!              --- Text found

                  table(icode+1) = i-imin+1
                  if ( debug ) then
                     write(irefwr,1) 'table =', i-imin+1
                     write(irefwr,1) 'i, nmcode(i)', i, nmcode(i)
                     write(irefwr,*) 'string ', string(1:nmcode(i))
                     write(irefwr,*) 'codemn ', codemn(i)(1:nmcode(i))
                  end if  ! ( debug )
                  go to 1000

               end if

            end do

!           --- keyword not recognized

            if ( debug ) write(irefwr,1) 'istop', istop
            if ( istop==2 ) then

!           --- keyword may be the next keyword

               readnx = .false.
               go to 1000

            else

!           --- Error

               call errchr ( string(1:nchar), 1 )
               do i = imin, imax
                  call errchr ( codemn(i), 2+i-imin )
               end do
               call errsub ( 648, 0, 0, 2+imax-imin )
               call errcrd
               iretur = -1
               finished = .true.
               go to 1000

            end if

         end if

      end if  ! ( ityprd/=4 ) then

1000  call erclos ( 'intx0205' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'readnx ', readnx
         write(irefwr,*) 'End intx0205'

      end if

      end
