      subroutine intx0210 ( icode, table, nout, tabout, iretur, &
                            tabmin, tabmax, code, finished, readnx, &
                            itype, iposut, codemn, nmcode, subkey, &
                            vectorfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 10-11-2014 Check tabout
!        version  1.5  date 20-04-2012 Indexed scalars
!        version  1.4  date 17-06-2011 Extra parameter vectorfound
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by sequence number and number or number
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/creadinfof90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, itype(*), nmcode(*), iretur
      character (len=*) code(*), codemn(*), subkey
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical finished, readnx, vectorfound

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     iposut        i/o   Position in array tabout
!     iretur        i/o   Return value of subroutine INTX02
!                         Possible values:
!                         -2:  end of file has been reached
!                         -1:  error has been found
!                          0:  normal return
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     subkey         i    Sub (second) keyword
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
!     vectorfound    i    If true the last item read is a vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivalrd, minval, maxval, i, ih, iseq, iseqmn, iseqmx, &
              itypsv, numkey
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     iseq           Sequence number read after keyword
!     iseqmn         Minimum value allowed of sequence number read after keyword
!     iseqmx         Maximum value allowed of sequence number read after keyword
!     itypsv         help variable to store the old value of ityprd
!     ivalrd         Integer value read
!     maxval         Maximum value allowed for the integer read
!     minval         Minimum value allowed for the integer read
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     READREALSCAL   Read integer, real or scalar
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1461   incorrect keyword found after keyword
!    1468   Array tabout too small
!    1869   No number found after keyword
!    1870   Integer number read out of range
!    1871   Real number read out of range
!    1880   Not a sequence number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
!    1882   Not a number of the correct type found after keyword followed
!           by sequence number
!    1914   Not a second keyword found after keyword = &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0210' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0210'
         write(irefwr,1) 'icode, itype(icode)', icode, itype(icode)
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ityprd==4 ) call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/2', ityprd, lastitem
         write(irefwr,2) 'realrd', realrd
      end if  ! ( debug )

      if ( (itype(icode)==10 .or. itype(icode)==34 ) .and. &
            ityprd/=2 .and. ityprd/=11 ) then

!     --- Not a sequence number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1880, 0, 0, 1 )
         call errcrd
         iretur = -1
         finished = .true.
         go to 1000

      else if ( itype(icode)==30 .and. ityprd/=2 .and. &
                ityprd/=4 .and. ityprd/=11 ) then

!     --- Not a keyword found after the keyword

         call errchr ( code(icode), 1 )
         call errchr ( codemn(itype(icode+1)), 2 )
         call errsub ( 1914, 0, 0, 2 )
         call errcrd
         iretur = -1
         finished = .true.
         go to 1000

      else

         if ( ityprd==4 ) then

!        --- String found, check if it is vector, scalar or constant

            lastitem = lastitembef
            call readrealscal
            if ( ityprd==102 ) ityprd = 2  ! vector found
            vectorfound = .true.

         end if  ! ( ityprd==4 )

         if ( ityprd/=2 .and. ityprd/=3 .and. ityprd/=11 ) then

!        --- Not a number found after the keyword

            call errchr ( code(icode), 1 )
            call errsub ( 1869, 0, 0, 1 )
            call errcrd
            iretur = -1
            finished = .true.
            go to 1000

         end if  ! ( ityprd/=2 .and. ityprd/=3 .and. ityprd/=11 )

      end if  ! (itype(icode)==10 .or. itype(icode)==34 )..

      iseq = 1
      if ( ityprd==2 .or. ityprd==11 ) iseq = nint(realrd)

      if ( debug ) then
         write(irefwr,1) 'itype(icode), iseq', itype(icode), iseq
         write(irefwr,*) 'vectorfound ', vectorfound
         call prinrl ( table(icode), 2, 'table' )
         call prinrl ( tabmin(icode), 2, 'tabmin' )
         call prinrl ( tabmax(icode), 2, 'tabmax' )
      end if  ! ( debug )

      if ( itype(icode)==12 .and. vectorfound ) then

!     --- Use vector number

         i = 0
         iseq = 1
         do while ( i<=nint(tabmax(icode))-nint(tabmin(icode)) )

            if ( table(icode+i)>0d0 ) then

               iseq = i+2
               i = 10000

            else

               i = i+1

            end if  ! ( table(icode+i)>0d0 )
            itypsv = ityprd
            if ( debug ) write(irefwr,1) 'iseq', iseq

         end do ! while ( i<=iseqmax )

      else if ( itype(icode)==12 ) then

!     --- read next item if itype(icode) = 12

         itypsv = ityprd
         call asknxt
         if ( debug ) &
            write(irefwr,1) 'ityprd, lastitem, typeitems(lastitem+1)', &
                             ityprd, lastitem, typeitems(lastitem+1)
         if ( ityprd==4 ) then

!        --- Next item is string, check if it is a scalar, constant or vector
!            Read complete name

            lastitem = lastitembef
            call readrealscal
            if ( ityprd==102 ) ityprd = 2  ! vector found
            if ( ityprd==2  .or. ityprd==3 .or. ityprd==11 ) then

!           --- Vector name found

               itypsv = ityprd

            else

!           --- Not a number found

               iseq = 1
               lastitem = lastitembef

            end if  ! ( ityprd==2 .or. ityprd==3  .or. ityprd==11 )

            if ( debug ) write(irefwr,1) 'itypsv', itypsv

         else

!        --- Next item is not a string, read

            if ( ityprd/=2 .and. ityprd/=3 .and. ityprd/=11 ) then

!           --- Other type of item found, finished

               readnx = .false.
               iseq = 1

            else

!           --- ityprd = 2, 3 or 11

               itypsv = ityprd

            end if  ! ityprd/=2 .and. ityprd/=3 .and. ityprd/=11 )

         end if  ! ( lastitem<nitems .and. typeitems(lastitem+1)==3 )

      end if

!     --- Sequence number found, check if it is in the correct range

      if ( itype(icode)==34 ) then
         iseqmn = 1
      else
         iseqmn = nint(tabmin(icode))
      end if
      iseqmx = nint(tabmax(icode))
      if ( ityprd/=11 .and. &
           ( iseq<iseqmn .or. iseq>iseqmx ) ) then

         call errchr ( code(icode), 1 )
         call errint ( iseq, 1 )
         call errint ( iseqmn, 2 )
         call errint ( iseqmx, 3 )
         call errsub ( 1881, 3, 0, 1 )
         call errcrd
         iseq = iseqmn

      end if

!     --- Read next item

      if ( itype(icode)==10 .or. itype(icode)==34 ) then

         ispecl = 9
         call asknxt

         if ( ityprd==4 ) then

!        --- string found, check if it is a constant

            call readstrconst

         end if  ! ( ityprd==4 )
         itypsv = ityprd

         if ( debug ) write(irefwr,1) 'ityprd', ityprd

      else if ( itype(icode)==30 ) then

         if ( ityprd==2 .or. ityprd==11 ) call asknxt

         if ( ityprd/=4 ) then

!        --- Not a keyword found after the keyword

            call errchr ( code(icode), 1 )
            call errchr ( codemn(itype(icode+1)), 2 )
            call errsub ( 1914, 0, 0, 2 )
            call errcrd
            iretur = -1
            finished = .true.
            go to 1000

         else

            ih = itype(icode+1)
            subkey = codemn(ih)
            numkey = nmcode(ih)
            if ( string(1:numkey)/=subkey(1:numkey) ) then

!           --- incorrect keyword found

               call errchr ( code(icode), 1 )
               call errchr ( string(1:nchar), 2 )
               call errchr ( subkey, 3 )
               call errsub ( 1461, 0, 0, 3 )
               call errcrd
               finished = .true.

            end if
            call asknxt
            itypsv = ityprd
            if ( ityprd/=2 .and. ityprd/=11 ) then

!           --- Not a sequence number found after the keyword

               call errchr ( string(1:numkey), 1 )
               call errsub ( 1880, 0, 0, 1 )
               call errcrd
               iretur = -1
               finished = .true.
               go to 1000

            end if

         end if

      end if

      if ( debug ) then
         write(irefwr,1) 'itypsv, itype(icode)', itypsv, itype(icode)
         write(irefwr,1) 'itype(icode+1)', itype(icode+1)
         write(irefwr,1) 'iseq, iseqmn', iseq, iseqmn
      end if  ! ( debug )

      if ( itypsv==2 .or. itypsv==3 .and. &
           itype(icode+1)==1 .and. itype(icode)/=30 .or. &
           itypsv==11 .and. itype(icode)==12 ) then

         if ( itype(icode)==34 ) then

!        --- itype(icode)=34 special situation

            if ( table(icode)<=0d0 ) then

!           --- First item read, initialize

               table(icode) = iposut+1

               if ( iposut+iseqmx>nout ) then
                  call errint ( nout, 1 )
                  call errsub ( 1468, 1, 0, 0 )
                  go to 1000
               end if  ! ( iposut>nout )

               do i = 1, iseqmx
                  tabout(iposut+i) = -1d30
               end do
               iposut = iposut+iseqmx
            end if

            if ( nint(table(icode))+iseq-1>nout ) then
               call errint ( nout, 1 )
               call errsub ( 1468, 1, 0, 0 )
               go to 1000
            end if  ! ( iposut>nout )

            tabout(nint(table(icode))+iseq-1) = realrd
            infotabout(nint(table(icode))+iseq-1) = ityprd

         else
            table(icode+iseq-iseqmn) = realrd
            infotable(icode+iseq-iseqmn) = ityprd
            if ( debug ) &
               write(irefwr,2) 'table(icode+iseq-iseqmn)', &
                                table(icode+iseq-iseqmn)
         end if

!        --- Check values

         if ( itype(icode)==30 .or. &
              itype(icode+1)==0 ) then

!        --- Integer value read

            minval = nint(tabmin(icode+1))
            maxval = nint(tabmax(icode+1))
            ivalrd   = nint(realrd)
            ivalrd = mod(ivalrd,5000)   ! correction for a series of vectors
            if ( itypsv/=11 .and. &
                (ivalrd<minval .or. ivalrd>maxval) ) then
               call errint ( ivalrd, 1 )
               call errint ( minval, 2 )
               call errint ( maxval, 3 )
               call errchr ( code(icode), 1 )
               call errsub ( 1870, 3, 0, 1 )
               call errcrd
            end if

         else

!        --- Real value read

            if ( realrd<tabmin(icode+1) .or. &
               realrd>tabmax(icode+1) ) then
               call erreal ( realrd, 1 )
               call erreal ( tabmin(icode+1), 2 )
               call erreal ( tabmax(icode+1), 3 )
               call errsub ( 1871, 0, 3, 0 )
               call errcrd
            end if

         end if

      else

         call errchr ( code(icode), 1 )
         call errsub ( 1882, 0, 0, 1 )
         call errcrd

      end if

1000  call erclos ( 'intx0210' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0210'

      end if

      end
