      subroutine intx0229 ( icode, code, itype, table, codemn, &
                            nmcode, tabmin, tabmax, tabout, nout, jump, &
                            iposut, ihelp, finished, readnx, iretur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 10-11-2014 Check tabout
!        version  1.4  date 30-10-2014 Allow text for option 41
!        version  1.3  date 02-11-2009 Extension with option 41
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part of input record for subroutine intx02 for the
!     possibilities itype = 29, 33 and 37
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/creadinfof90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, itype(*), nmcode(*), iretur, &
              nout, jump, iposut, ihelp(*)
      character *(*) code(*), codemn(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical finished, readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     finished       o    Indicates if the loop has been finished (true) or not
!                         (false)
!     icode          i    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     ihelp          o    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     iposut        i/o   Position in array tabout
!     iretur        i/o   Return code. Possible values
!                         -11  At input means that asknxt has already been
!                              called before
!                         -10  At input means that no error message is given if
!                              a single real or integer is found
!                              The corresponding value is stored in tabout, next
!                              position
!                              iretur gets the value -10-number of numbers found
!                         -2   End of file has been reached
!                         -1   An error has been detected
!                          0   Normal end of subroutine
!                         >0   Main keyword number iretur has been found
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     jump           o    if 0 no error has been found, if 1 array tabout
!                         is too small
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nout           i    Number of entries in tabout
!     readnx         i    Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin        i/o   Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inumlc, iposmn, ivalrd, ivalrdbef, num, valmax
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inumlc         Number of numbers read in a record
!     iposmn         Position of main keyword
!     ivalrd         Integer value read
!     ivalrdbef      Previous integer value read
!     num            Number of numbers read between brackets
!     valmax         Maximum value allowed for the integer read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX05         Read a series of integers
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1468   Array tabout too small
!    1868   Text not recognized
!    1870   Integer number read out of range
!    1880   Not a sequence number found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
!     itype(i) =29, special situation of keyword followed by
!                   sequence number possibly followed by another
!                   keyword followed by integers
!                   input:  code(i) keyword
!                           table(i) -iposmain
!                           codemn(iposmain) special sub keyword
!                           tabmin(i) maximum number of numbers
!                                     that may be read
!                           tabmax(i) maximum value of |numbers|
!                   output: table(i) = iposut
!                           tabmax(i) = m (number of integers read)
!                           tabout(iposut to iposut+m-1) = values
!     itype(i) =33, special situation of keyword followed by
!                   sequence number followed by integers
!                   input:  code(i) keyword
!                           tabmin(i) maximum number of numbers
!                                     that may be read
!                           tabmax(i) maximum value of |numbers|
!                   output: table(i) = iposut
!                           tabmax(i) = m (number of integers read)
!                           tabout(iposut to iposut+m-1) = values
!     itype(i) =37, special situation of keyword followed by
!                   sequence number to other sequence number
!                   followed by integers between brackets
!                   The second sequence number may absent as
!                   well as the numbers between brackets
!                   input:  code(i) keyword
!                           tabmin(i) maximum number of numbers
!                                     that may be read
!                           tabmax(i) maximum value of |numbers|
!                   output: table(i) = iposut
!                           tabmax(i) = m (number of integers read)
!                           tabout(iposut to iposut+m-1) = values
!                           The first two always refer to the
!                           sequence numbers
!     itype(i) =41, special situation of keyword followed by
!                   sequence number possibly followed by
!                   another keyword followed by integers
!                   input:  code(i) keyword
!                           table(i) -iposmain
!                           codemn(iposmain) special sub
!                           keyword
!                           tabmin(i) maximum number of
!                           numbers that may be read
!                           tabmax(i) maximum value of
!                                     |numbers|
!                   output: table(i) = iposut
!                           tabmax(i) = m (number of
!                           integers read)
!                           tabout(iposut to iposut+m-1) = &
!                           values
!                           The first two always refer to
!                           the sequence numbers
!     itype(i) =42, special situation of keyword followed by
!                   sequence number or iseq1 to iseq2
!                   input:  code(i) keyword
!                           tabmax(i) maximum value of
!                                     |numbers|
!                   output: table(i) = iposut
!                           tabout(iposut to iposut+1) = &
!                           values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0229' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0229'
         write(irefwr,1) 'icode, itype(icode)', icode, itype(icode)
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      jump = 0

      call asknxt
      if ( debug ) write(irefwr,1) 'ityprd/1', ityprd
      if ( ityprd==2 ) then

!     --- sequence number found

         ivalrd = nint(realrd)
         if ( ivalrd<1 .or. ivalrd>10000 ) then

!        --- Error integer out of range

             call errint ( ivalrd, 1 )
             call errint ( 1, 2 )
             call errint ( 10000, 3 )
             call errchr ( code(icode), 1 )
             call errsub ( 1870, 3, 0, 1 )
             call errcrd

         end if
         if ( debug ) write(irefwr,1) 'ivalrd/1', ivalrd

      else

!     --- Not a sequence number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1880, 0, 0, 1 )
         call errcrd
         iretur = -1
         finished = .true.
         go to 1000

      end if
      if ( itype(icode)==29 .or. itype(icode)==41 ) then
         iposmn = nint(-table(icode))
      else
         iposmn = 0
      end if
      table(icode) = iposut+1
      iposut = iposut + 1
      if ( debug ) write(irefwr,1) 'iposut/1', iposut
      if ( iposut>nout ) then
         jump = 1
         go to 1000
      end if
      tabout(iposut) = ivalrd
      infotabout(iposut) = ityprd
      num = 1
      ihelp(icode) = num

!     --- read next item

      call asknxt
      if ( debug ) write(irefwr,1) 'ityprd/2, nchar', ityprd, nchar

      if ( itype(icode)==37 .or. itype(icode)>=41 ) then

!     --- itype(icode) = 37, 41, 42
!         Is either followed by a series of numbers between brackets
!         or the keyword TO followed by a next sequence number followed
!         by a series of numbers between brackets

         if ( ityprd==4 .and. string(1:nchar)=='TO' ) then

!        --- keyword TO found
!            Read integer following TO

            call asknxt
            if ( debug ) write(irefwr,1) 'ityprd/3', ityprd

            if ( ityprd/=2 .and. ityprd/=11 ) then

!           --- not an integer found

               call errchr ( 'TO', 1 )
               call errsub ( 574, 0, 0, 1 )
               call errcrd
               go to 1000

            else

               ivalrdbef = ivalrd
               ivalrd = nint(realrd)
               if ( ivalrd<ivalrdbef .or. ivalrd>10000 ) then

!              --- Error integer out of range

                   call errint ( ivalrd, 1 )
                   call errint ( ivalrdbef, 2 )
                   call errint ( 10000, 3 )
                   call errchr ( code(icode), 1 )
                   call errsub ( 1870, 3, 0, 1 )
                   call errcrd

               end if
               iposut = iposut + 1
               if ( iposut>nout ) then
                  call errint ( nout, 1 )
                  call errsub ( 1468, 1, 0, 0 )
                  go to 1000
               end if
               tabout(iposut) = ivalrd
               infotabout(iposut) = ityprd

            end if
            if ( debug ) write(irefwr,1) 'ivalrd/2', ivalrd
            readnx = .true.

         else

!        --- not TO found, set tabout(iposut+1) = tabout(iposut)

            if ( iposut+1>nout ) then
               call errint ( nout, 1 )
               call errsub ( 1468, 1, 0, 0 )
               go to 1000
            end if
            tabout(iposut+1) = tabout(iposut)
            infotabout(iposut+1) = infotabout(iposut)
            iposut = iposut + 1
            readnx = .false.
            if ( debug ) write(irefwr,1) 'iposut after TO', iposut

         end if
         num = 2
         ihelp(icode) = num

         if ( itype(icode)/=42 ) then

!        --- Read next item if necessary, i.e. itype(icode)/=42 )

            if ( readnx ) call asknxt
            readnx = .true.

         end if  ! ( itype(icode)/=42 )

      end if  ! ( itype(icode)==37 .or. itype(icode)==41 )

      if ( debug ) write(irefwr,1) 'iposmn, ityprd', iposmn, ityprd
      if ( iposmn>0 .and. ityprd==4 ) then

!     --- keyword found: test

         nchar = nmcode(iposmn)
         if ( string(1:nchar)/=codemn(iposmn)(1:nchar) ) then

            if ( itype(icode)==41 ) then

!           --- other keyword found, return

               readnx = .false.
               go to 1000

            else

!           --- wrong keyword found

               call errchr ( codemn(iposmn)(1:nchar), 1 )
               call errsub ( 1868, 0, 0, 1 )
               call errcrd
               iretur = -1
               finished = .true.

            end if  ! ( itype(icode)==41 )

         end if

      else

         if ( itype(icode)/=42 ) readnx = .false.

      end if  ! ( iposmn>0 .and. ityprd==4 )

!     --- Next read integer numbers until closing bracket or
!         other type of separator is found

      if ( itype(icode)/=42 ) then

!     --- This part is skipped in case of type 42

         inumlc = nint(tabmin(icode))
         valmax = nint(tabmax(icode))

         if ( debug ) then
            write(irefwr,1) 'inumlc, valmax', inumlc, valmax
            write(irefwr,1) 'num, ihelp(icode)', num, ihelp(icode)
         end if  ! ( debug )

         if ( iposut+1>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         call intx05 ( tabout(iposut+1), inumlc, num, readnx, &
                       .false., valmax, code(icode), &
                       infotabout(iposut+ihelp(icode)) )
         if ( debug ) write(irefwr,1) 'num', num

         ihelp(icode) = num+ihelp(icode)
         iposut = iposut+num
         tabmax(icode) = ihelp(icode)

      else

!     --- itype(icode)=42, check parameters

         valmax = nint(tabmax(icode))
         iposut = nint(table(icode))+1

         if ( iposut>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         ivalrd = nint(tabout(iposut))
         ihelp(icode) = 0

         if ( ivalrd>valmax ) then

!        --- Error: number read out of range

            call errint ( ivalrd, 1 )
            call errint ( 1, 2 )
            call errint ( valmax, 3 )
            call errchr ( code(icode), 1 )
            call errsub ( 1870, 3, 0, 1 )

         end if  ! ( ivalrd>valmax )

      end if  ! ( itype(icode)/=42 ) then

      if ( iposut>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
      end if  ! ( iposut>nout )

1000  call erclos ( 'intx0229' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposut, nout', iposut, nout
         write(irefwr,*) 'readnx, finished ', readnx, finished
         call prinrl ( tabout, iposut, 'tabout' )
         write(irefwr,*) 'End intx0229'

      end if  ! ( debug )

      end
