      subroutine intx0235 ( icode, table, nout, tabout, &
                            tabmin, tabmax, code, finished, &
                            iposut, iretur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-11-2014 Check tabout
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a sequence number followed by
!                              a value or value = v, or func = i
!                              and again the same keyword followed  by
!                              a sequence number followed by
!                              a value or value = v, or func = i
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, iretur
      character (len=*) code(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical finished

!     code           i    Table with keywords
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     iposut        i/o   last position used in array tabout
!     iretur        i/o   Return value of subroutine INTX02
!                         Possible values:
!                         -2:  end of file has been reached
!                         -1:  error has been found
!                          0:  normal return
!     nout           i    Number of entries in tabout
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifunc, iseq, iseqmn, iseqmx
      double precision value(1)
      logical debug, seqk1, seqk2

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ifunc          Parameter k after FUNC = &
!     iseq           Sequence number read after keyword
!     iseqmn         Minimum value allowed of sequence number read after keyword
!     iseqmx         Maximum value allowed of sequence number read after keyword
!     seqk1          Indicates if sequence number 1 has been found
!     seqk2          Indicates if sequence number 2 has been found
!     value          Array containing the real or complex value read
!                    (2 positions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFN         Read items of the shape number, VALUE=number or FUNC=i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1880   Not a sequence number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0235' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0235'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      seqk1 =  .false.
      seqk2 =  .false.
      call asknxt
      if ( ityprd/=2 ) then

!     --- Not a sequence number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1880, 0, 0, 1 )
         call errcrd
         iretur = -1
         finished = .true.
         go to 1000

      end if
      iseq = 1
      if ( ityprd==2 ) iseq = nint(realrd)

!     --- Sequence number found, check if it is in the correct range

      iseqmn = nint(tabmin(icode))
      iseqmx = nint(tabmax(icode))
      if ( iseq<iseqmn .or. iseq>iseqmx ) then

         call errchr ( code(icode), 1 )
         call errint ( iseq, 1 )
         call errint ( iseqmn, 2 )
         call errint ( iseqmx, 3 )
         call errsub ( 1881, 3, 0, 1 )
         call errcrd
         iseq = iseqmn

      end if

      if ( iseq==1) then
         seqk1 = .true.
      else if ( iseq==2 ) then
         seqk2 = .true.
      end if

!     --- Read next item

      if( seqk1 .and. .not. seqk2   .or. &
          seqk2 .and. .not. seqk1) then
         iposut = iposut + 1
         table(icode) = iposut

         if ( iposut+3>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         do i = 1, 4
            tabout(iposut-1+i) =0d0
         end do
      end if

      call readfn ( ifunc, value, .false. )
      if ( ifunc>0 .or. ifunc<-100 ) then

!     --- function found

         if ( iposut+(iseq-1)*2>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         tabout(iposut+(iseq-1)*2) = ifunc

      else

!     --- value found

         if ( iposut+(iseq-1)*2+1>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         tabout(iposut+(iseq-1)*2) = -1d0
         tabout(iposut+1+(iseq-1)*2) = value(1)

      end if

      if ( seqk1 .and. seqk2 ) then
         iposut = iposut + 3
      end if

1000  call erclos ( 'intx0235' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0235'

      end if

      end
