      subroutine intx0243 ( icountkey, icode, code, table )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret information read in case itype(icode) = 43
!     Expected is a constant, scalar or vector
!     See subroutine intx02ext2
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, icountkey(*)
      double precision table(*)
      character (len=*) code(*)

!     code           i    Table with keywords
!     icode          i    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     icountkey      i    Array that counts the number of times a keyword has been
!                         found
!     table         i/o   Contains the numbers read for the various keywords
!                         See intx02ext2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1628   Keyword found more than once for the same problem
!    2070   Name found after the percent symbol contains more than 32 characters
!    3132   Name read does not correspond to a constant, scalar or vector.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0243' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0243'
         write(irefwr,1) 'icode, lastitem, ityprd', &
                          icode, lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)

         call prinin ( typeitems, nitems, 'typeitems' )

  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item

      if ( ityprd/=11 ) call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

      if ( ityprd==4 ) then

!     --- Name found, check if it belongs to the set of vector names

         if ( nchar>32 ) then

!        --- If number of characters more than 32, reduce to 32 and give
!            warning

            call errchr ( string(1:nchar), 1 )
            call errint ( nchar, 1 )
            call errwar ( 2070, 1, 0, 1 )
            call errcrd
            nchar = 32

         end if  ! ( nchar>32 )

         lastitem = lastitem-1
         ispecl = 10
         call readrealscal
         ispecl = 0

         if ( debug ) then
            write(irefwr,1) 'ityprd', ityprd
            call prinin ( index_rdarr, 5, 'index_rdarr' )
            write(irefwr,2) 'realrd', realrd
         end if  ! ( debug )

         select case ( ityprd )

         case(12)

!        --- integer found

            table(icode) = index_rdarr(1)+100000000

         case(13)

!        --- real found

            table(icode) = index_rdarr(1)+maxints+100000000

         case(14)

!        --- scalar found

            table(icode) = index_rdarr(1)+maxints+maxreals+100000000

         case(102)

!        --- vector found

            table(icode) = nint(realrd)+10000

         case default

!        --- None of these items found: error

            call errchr ( string(1:nchar), 1 )
            call errsub ( 3132, 0, 0, 1 )
            call errcrd

         end select  ! case ( ityprd )

      end if  ! ( ityprd==4 )

1000  call erclos ( 'intx0243' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0243'

      end if  ! ( debug )

      end
