      subroutine intx02keyw ( code, ncode, nncode, codemn, nmcode, &
                              nmain, iretur, ncodmx, finished, icode, &
                              itype, nout, tabout, iposut, icountkey, &
                              table, readnx, tabmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 11-05-2014 Extra parameters
!        version  1.3  date 20-04-2012 Indexed scalars
!        version  1.2  date 06-02-2012 Extra debug
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string read is (main) keyword
!     If so return with icode >0 or finished true
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, ncode(nncode), nmain, nmcode(*), iretur, &
              ncodmx(nncode), icode, &
              itype(nncode), nout, iposut, icountkey(*)
      double precision, intent(inout) :: tabout(nout), table(nncode), &
                                         tabmax(nncode)
      logical, intent(inout) :: finished, readnx
      character (len=*) code(nncode), codemn(*)

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     finished      i/o   Indicates if a main keyword has been found (true)
!                         or not (false)
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     icountkey     i/o   Array that counts the number of times a keyword has been
!                         found
!     iposut        i/o   Position in array tabout
!     iretur        i/o   Return value of subroutine INTX02
!                         See subroutine INTX02
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nmain          i    Number of entries in codemn
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numchar, ispeclsav
      character (len=80) keyword
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ispeclsav      Help parameter to save the value of ispecl
!     keyword        Text containing the string without numbers
!     numchar        Number of characters in keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTX00KEYW     Check if string read is (main) keyword
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!                    a number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02keyw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02keyw'
         write(irefwr,1) 'lastitem, ityprd, lastitembef', &
                          lastitem, ityprd, lastitembef
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- A keyword has been found, but at this moment we do not know
!         whether this keyword has been read with or without numbers
!         Therefore we read again

      if ( ityprd==100 ) then

!     --- ityprd = 100

         ityprd = 2
         call intx00keyw ( code, ncode, nncode, ncodmx, icode, &
                           numchar, keyword, &
                           itype, nout, tabout, iposut, icountkey, &
                           table, readnx, tabmax )

      else

!     --- ityprd # 100

         lastitem = lastitembef

         ispeclsav = ispecl
         ispecl = 9
         call asknxt
         ispecl = ispeclsav

         if ( debug ) &
            write(irefwr,1) 'lastitem/1, ityprd', lastitem, ityprd

!        --- Next we have to check whether the keyword contains numbers
!            If so we have to split the keyword into a part without number
!            and a number
!            This is necessary to check if it is a keyword, since keywords do
!            not contain numbers

         call readsplitkeyw ( numchar, keyword )
         if ( debug ) then
            write(irefwr,*) 'string ', string(1:nchar)
            write(irefwr,*) 'keyword ', keyword(1:numchar)
            write(irefwr,1) 'lastitem/2, ityprd', lastitem, ityprd
         end if  ! ( debug )

         do i = 1, abs(nmain)

            if ( keyword(1:nmcode(i))==codemn(i)(1:nmcode(i)) ) then

!           --- Main keyword found

               iretur = i
               lastitem = lastitembef+1   ! Only string is used
               if ( nmain<0 ) lastitem = lastitem-1
               finished = .true.
               if ( debug ) write(irefwr,*) 'main keyword'
               go to 1000

            end if  ! ( string(1:nmcode(i))==codemn(i)(1:nmcode(i)) )

         end do  ! i = 1, abs(nmain)

!        --- Check if string is a standard keyword
!            If not and it is a vector name return with ityprd = 2
!            If not and it is a scalar name return with ityprd = 11

         call intx00keyw ( code, ncode, nncode, ncodmx, icode, &
                           numchar, keyword, &
                           itype, nout, tabout, iposut, icountkey, &
                           table, readnx, tabmax )

      end if
      if ( ityprd==4 ) lastitem = lastitembef+1   ! Only string is used
      if ( debug ) write(irefwr,1) 'icode, ityprd', icode, ityprd

1000  call erclos ( 'intx02keyw' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode, iretur, ityprd, index_rdarr(3)', &
                          icode, iretur, ityprd, index_rdarr(3)
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,*) 'End intx02keyw'

      end if  ! ( debug )

      end
