      subroutine intxchkscalvec ( code, ncode, nncode, ncodmx, intx00 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-12-2014 extension with strings
!        version  1.2  date 26-06-2014 extra debug
!        version  1.1  date 15-03-2014 Save also stringim
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if item at right-hand side is scalar or vector
!     if so add left-hand side to the list of scalars or vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     vector_name
!     name
!     add
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intx00, nncode, ncode(nncode),ncodmx(nncode)
      character (len=*) code(nncode)

!     code           i    Table with keywords
!     intx00        i/o   Gives index number of the text that matches the code
!                         table
!                         At input it has the value 0
!                         At output it returns with the value of the scalar or
!                         vector sequence number in the table code provided
!                         - There is a vector or scalar keyword in the table
!                         - The right-hand is an expression
!                         Otherwise the value remains 0
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer scode, vcode, i, ncharsave, lastitemsav, stringcode
      logical debug, vectorfound, scalarfound, notfound, stringfound
      character (len=240) stringsave, stringimsave

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     lastitemsav    Help parameter to save the value of lastitem
!     ncharsave      Help parameter to save nchar at input
!     notfound       If true no expression ate right-hand side is found
!     scalarfound    If true the keyword scalar is in code
!     scode          sequence number of the keyword scalar in code
!     stringcode     sequence number of the string
!     stringfound    If true the keyword is in the list of strings
!     stringimsave   Help parameter to save string at input
!     stringsave     Help parameter to save string in capitals at input
!     vcode          sequence number of the keyword vector in code
!     vectorfound    If true the keyword vector is in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDSCAL        Add scalar name to the list of scalars
!     ADDSTRING      Add string name to the list of strings
!     ADDVEC         Add vector name to the list of vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTXCHKSCAL    Check if right-hand side is an expression containing
!                    only existing scalars and vectors, operators and numbers
!                    and no other strings
!     INTXCHKSTRING  Check if right-hand side corresponds to a string
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxchkscalvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxchkscalvec'
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      vcode = 0
      scode = 0
      stringcode = 0
      stringsave = string(1:nchar)
      stringimsave = stringim(1:nchar)
      ncharsave = nchar
      lastitemsav = lastitem
      do i = 1, nncode

         vectorfound = ncode(i)==1 .and. code(i)=='V' .or. &
                       ncodmx(i)==6 .and. code(i)=='VECTOR'
         vcode = i
         if ( vectorfound ) exit

      end do  ! i = 1, nncode

      do i = 1, nncode

         scalarfound = ncode(i)==1 .and. code(i)=='S' .or. &
                       ncodmx(i)==6 .and. code(i)=='SCALAR'
         scode = i
         if ( scalarfound ) exit

      end do  ! i = 1, nncode

      do i = 1, nncode

         stringfound = ncodmx(i)==6 .and. code(i)=='STRING'
         stringcode = i
         if ( stringfound ) exit

      end do  ! i = 1, nncode

      if ( debug ) &
         write(irefwr,1) 'scode, vcode, stringcode', &
                          scode, vcode, stringcode

      if ( stringcode>0 ) then

!     --- stringcode>0, possibly string = &
!         check right-hand side

         call intxchkstring ( notfound )

         if ( .not. notfound ) then

!        --- string found, add to list of strings

            nchar = ncharsave
            string(1:nchar) = stringsave
            stringim(1:nchar) = stringimsave(1:nchar)
            call addstring
            intx00 = stringcode
            if ( ierror/=0 ) go to 1000

         end if  ! ( .not. notfound )

      end if  ! ( stringcode>0 )

      if ( notfound .and. (scode>0 .or. vcode>0)  ) then

!     --- scode>0, possibly scalar = &
!         check right-hand side

         call intxchkscal ( notfound, vectorfound )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            write(irefwr,*) 'after intxchkscal notfound, vectorfound', &
                             notfound, vectorfound
            write(irefwr,1) 'vcode', vcode
         end if  ! ( debug )

         nchar = ncharsave
         string(1:nchar) = stringsave(1:nchar)
         stringim(1:nchar) = stringimsave(1:nchar)

         if ( notfound ) then

!        --- No expression found

         else if ( vcode>0 .and. vectorfound ) then

!        --- Vector found, add vector to set of vectors

            if ( debug ) write(irefwr,*) 'vector found, addvec'
            intx00 = vcode   ! Hence v = &
            ityprd = 102
            call addvec
            if ( ierror/=0 ) go to 1000

         else if ( .not. vectorfound ) then

!        --- Scalar found, add scalar to set of scalars

            intx00 = scode   ! Hence s = &
            ityprd = 11
            call addscal
            if ( ierror/=0 ) go to 1000

         end if  ! ( .notfound )

      end if  ! ( scode>0 )

      nchar = ncharsave
      string(1:nchar) = stringsave(1:nchar)
      stringim(1:nchar) = stringimsave(1:nchar)
      lastitem = lastitemsav  ! reset lastitem

1000  call erclos ( 'intxchkscalvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,*) 'End intxchkscalvec'

      end if  ! ( debug )

      end
