      subroutine intxkeyword2 ( code, ncode, nncode, ncodmx, icode, &
                                numchar, keyword )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string read is (main) keyword
!     If so return with icode >0
!     Only a limited number of characters are compared
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/cwarnf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, ncode(nncode), ncodmx(nncode), icode, numchar
      character (len=*) code(nncode), keyword

!     code           i    Table with keywords
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     keyword        i    Text containing the string without numbers
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
!     numchar        i    Number of characters in keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1474   Keyword contains too much characters
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxkeyword2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxkeyword2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nncode

         if ( keyword(1:ncode(i))==code(i)(1:ncode(i)) ) then

!        --- Text found

            icode = i
            lastitem = lastitembef+1
            if ( debug ) then
               write(irefwr,1) 'text found, icode:', icode
               write(irefwr,1) 'lastitem, numchar, ncodmx(i)', &
                                lastitem, numchar, ncodmx(i)
            end if  ! ( debug )

            if ( numchar>ncodmx(i) ) then

!           --- Too much characters in keyword, skip this keyword

               icode = 0
               go to 300

            end if  ! ( nchar>ncodmx(i) )

            if ( nwarn<mxwarn .and. numchar>ncodmx(i) ) then

!           --- Too much characters in keyword

               call errint ( nchar, 1 )
               call errint ( ncodmx(i), 2 )
               call errchr ( string(1:nchar), 1 )
               call errchr ( code(i)(1:ncodmx(i)), 2 )
               call errwar ( 1474, 2, 0, 2 )
               call errcrd

            end if  ! ( nchar>ncodmx(i) )
            go to 1000

300      end if  ! ( string(1:ncode(i))==code(i)(1:ncode(i)) )

      end do  ! i = 1, nncode
      lastitem = lastitembef+1

1000  call erclos ( 'intxkeyword2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intxkeyword2'

      end if  ! ( debug )

      end
