      subroutine intxvector ( code, ncode, nncode, ncodmx, icode, &
                              itype, nout, tabout, iposut, icountkey, &
                              table, readnx, tabmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 11-05-2014 Extra parameters
!        version  1.1  date 20-04-2012 Extra debug
!        version  1.0  date 06-02-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Check if one of the keywords is V or Vector
!      In that case it might be a vectorname with number
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nncode, ncode(nncode), ncodmx(nncode), &
                             itype(nncode), nout
      integer, intent(inout) :: icode, iposut, icountkey(*)
      double precision, intent(inout) :: tabout(nout), table(nncode), &
                                         tabmax(nncode)
      logical, intent(inout) :: readnx
      character (len=*) :: code(nncode)

!     code           i    Table with keywords
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     icountkey     i/o   Array that counts the number of times a keyword has been
!                         found
!     iposut        i/o   Position in array tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, jcode, lastitemsav
      logical :: debug, vectorfound

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     jcode          Special value of icode corresponding to vector or scalar
!     lastitemsav    Help parameter to save the value of lastitem
!     vectorfound    If true vector or scalar is a keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTX00KEYW1    Check information of vector name read
!     INTX0238       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 38
!     PRININ         print 1d integer array
!     READNAMEVEC    Read name of vector in common cpost5
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxvector' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxvector'
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d13.7) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if one of the keywords is V or Vector
!         In that case it might be a vectorname with number

      do i = 1, nncode

         vectorfound = ncode(i)==1 .and. code(i)=='V' .or. &
                       ncodmx(i)==6 .and. code(i)=='VECTOR'
         jcode = i
         if ( vectorfound ) exit

      end do  ! i = 1, nncode

      if ( vectorfound ) then

!     --- The keyword vector is present

         if ( debug ) write(irefwr,*) 'vectorfound/1', vectorfound

         lastitemsav = lastitem
         call readnamevec

         if ( debug ) then
            write(irefwr,1) 'jcode, ityprd, lastitem', &
                             jcode, ityprd, lastitem
            if ( ityprd==2 ) write(irefwr,2) 'realrd', realrd
         end if  ! ( debug )

         if ( ityprd==2 ) then

!        --- vector name found

            if ( itype(jcode)==40 ) then

!           --- itype(jcode) = 40, more than one vector allowed

               lastitem = lastitemsav
               call intx0238 ( icountkey, jcode, code, table, tabout, &
                               nout, iposut, tabmax, itype, readnx, &
                               lastitemsav )

            else

               call intx00keyw1 ( icode, jcode )

            end if  ! ( itype(icode)==40 )

         end if  ! ( ityprd==2 )
         if ( debug ) &
            write ( irefwr,1 ) 'icode, jcode, lastitem', &
                                icode, jcode, lastitem

      end if  ! ( vectorfound )

1000  call erclos ( 'intxvector' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'vectorfound ', vectorfound
         write(irefwr,1) 'ityprd, icode, lastitem', &
                          ityprd, icode, lastitem
         write(irefwr,*) 'End intxvector'

      end if  ! ( debug )

      end
