      subroutine manbnd ( iinman, rinman, kmesh, kprob, ivectr, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-04-1999 New body
!        version  1.4  date 18-12-1997 New call to er0081
!        version  1.3  date 05-06-1997 New call to er0081
!        version  1.2  date 24-02-1996 Jump in case of error
!        version  1.1  date 22-05-1995 New call to er0081
!        version  1.0  date 12-09-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute minimum or maximum of a SEPRAN vector along one or more
!   boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     manipulation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinman(*), ivectr(*), kmesh(*), kprob(*), icurvs(*)
      double precision rinman(*)

!     icurvs   i  Integer array containing the curve or surface numbers
!                 that must be printed.
!                 Pos. 1 indicates how array icurvs is filled.
!                 Possible values:
!                  0   only one curve or surface is used. The curve or
!                      surface number is stored in icurvs(2)
!                 -1   The solution is considered along the curves or surfaces
!                      icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                      icrv2 in icurvs(3).
!                 -2   See -1, however, the sequence of the curves is
!                      reversed. In case of curves this means that the
!                      curves are printed from the end point of icrv2 to
!                      the initial point of icrv2, then the end point of
!                      icrv2-1 to the begin point of icrv2-1 and so on.
!                      In case of surfaces ichois=-2 is treated as ichois=-1
!                 n>0  The curve numbers or surface numbers are stored in
!                      icurvs(2) to icurvs(n+1). That sequence is used.
!                 Remark: a negative curve number means that the curve is
!                      considered from end point to initial point, hence in
!                      reversed order. Negative surface numbers are not yet
!                      available.
!     iinman  i   Integer array in which the user indicates which actions
!                 must be performed by subroutine MANBND
!                 The entries of IINMAN must be filled (by the user) as follows
!                 If the user sets a value equal to 0, the default value is used
!                 1:  Number of entries filled by the user
!                 2:  ICHOIS, indicates the type of action
!                     Possibilities:
!                     1:   The minimum and maximum are computed
!                          The minimum is stored in rinman(1), the
!                          maximum in rinman(2)
!                     2:   The minimum and maximum of a function of the
!                          degree of freedom and the co-ordinates are computed
!                          The minimum is stored in rinman(1), the
!                          maximum in rinman(2)
!                          The user written subroutine funbnd is called
!                 3:  IDIM, indicates if curves (idim=2) or surfaces (idim=3)
!                     must be considered.
!                     The curve or surface numbers must be stored in array
!                     ICURVS
!                     Default value: 2
!                 4:  IDEGFD   Depending on the value of IDEGFD, the algebraic
!                              manipulations will be performed with respect to
!                              all degrees of freedom of the vectors (IDEGFD=0), &
!                              with all degrees of freedom, &
!                              or with the IDEGFD th  degree of
!                              freedom only (IDEGFD > 0).
!                     Default value: IDEGFD = 0
!     ivectr  i   Standard SEPRAN array referring to the vector to be
!                 manipulated
!     kmesh   i   Standard SEPRAN array containing information of the mesh
!     kprob   i   Standard SEPRAN array, containing information of the problem
!     rinman  o   Array containing the computed output as described in iinman
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS     Resets old name of previous subroutine of higher level
!     EROPEN     Produces concatenated name of local subroutine
!     MANBNDBF   Actual body of MABND
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'manbnd' )
      call manbndbf ( ibuffr, buffer, iinman, rinman, kmesh, &
                      kprob, ivectr, icurvs )
      call erclos ( 'manbnd' )
      end
