module sepmoduleintmat

   use sepmodulecons

   integer, parameter :: numintmat = 20        ! number of arrays intmat

   type intmatrix          ! vectors of type intmat
      integer :: icounter  ! sequence number
                           ! defines if array has been copied to module
      integer :: iprob     ! Problem number
      integer :: iseqkp    ! Sequence number of kprob
      integer :: jmethod   ! storage method, see sepmodulematstr
      integer :: kamata    ! Length of internal matrix S_ii
                           ! S_ii +S_if+S_fi+S_iout
      integer :: kamataii  ! Length of internal matrix S_ii
      integer :: leng      ! Length of matrix
      integer :: nbound    ! number of prescribed degrees of freedom
      integer :: ninterface ! number of interface unknowns (parallel)
      integer :: np        ! number of pressure degrees of freedom
      integer :: npboun    ! number of periodical degrees of freedom
      integer :: nrusol    ! number of free degrees of freedom
      integer :: nu        ! number of velocity degrees of freedom
      integer :: nusol     ! number of degrees of freedom
      integer :: signmethod ! if 1 the structure is new, if -1 old
      integer :: typecoupl ! 0: neumann-neumann
                           ! 1: dirichlet-dirichlet

      logical :: complexm  ! if true the matrix is complex
      logical :: direct    ! if true a direct solver is used
      logical :: full      ! if true the matrix is a full matrix
      logical :: lapshift  ! if true a laplace shift is performed
      logical :: mumps     ! if true mumps is used
      logical :: nested    ! if true nested dissection is used
      logical :: nosepmat  ! if true no sepran matrix formed
      logical :: petsc     ! if true petsc is used
      logical :: posdev    ! if true the matrix is positive definite
      logical :: spectral  ! if true it refers to the spectral mesh
      logical :: simple    ! if true a simple type method is used
      logical :: symmetric ! if true the matrix is symmetric
      logical :: Gsymmetric ! if true G = D^T
      logical :: Qsymmetric ! if true the Q-matrix matrix is symmetric
      integer, pointer :: colsDG(:)
      integer, pointer :: colsG(:)
      integer, pointer :: colsPM(:)
      integer, pointer :: colsQ(:)
      integer, pointer :: colsfi(:)
      integer, pointer :: colsic(:)
      integer, pointer :: colsif(:)
      integer, pointer :: colsii(:)
      integer, pointer :: colsiiprec(:)
      integer, pointer :: colsip(:)
      integer, pointer :: colsout(:)
      integer, pointer :: colspp(:)
      integer, pointer :: diag(:)
      integer, pointer :: diagC(:)
      integer, pointer :: diagDG(:)
      integer, pointer :: diagprec(:)
      integer, pointer :: diagPM(:)
      integer, pointer :: diagQ(:)
      integer, pointer :: recunknowns(:)
      integer, pointer :: rowsDG(:)
      integer, pointer :: rowsG(:)
      integer, pointer :: rowsPM(:)
      integer, pointer :: rowsQ(:)
      integer, pointer :: rowsfi(:)
      integer, pointer :: rowsic(:)
      integer, pointer :: rowsif(:)
      integer, pointer :: rowsii(:)
      integer, pointer :: rowsiiprec(:)
      integer, pointer :: rowsip(:)
      integer, pointer :: rowsout(:)
      integer, pointer :: rowspp(:)
      integer, pointer :: sendunknowns(:)
      integer, pointer :: storeinfo(:)

   end type intmatrix

   integer, target :: intmat2d(10,maxmesh) ! information of the matrix structure
   integer, target :: massmt(maxprb)       ! information of the mass matrices
   integer, target :: stiffm(maxprb)       ! information of the
                                           ! stiffness matrices
   integer, target :: mattot(maxprb)       ! information of the
                                           ! complete matrices

   integer, pointer :: intmat1d(:)
   integer, pointer :: intmatar(:)
   integer, pointer :: intmat
   integer, pointer :: stiffmat(:)
   integer, pointer :: massmat
   integer, pointer :: totmat

   integer :: iseqim                           ! sequence number of intmat
   integer :: iseqimpre                        ! sequence number of intmat
                                               ! for preconditioner
   integer :: intmatarr(numintmat)             ! indicates if intmt has been
                                               ! filled:
                                               ! 0: intmt(i) is empty
                                               ! >0: intmt(i) has been filled
   integer :: lastintmat                       ! last position used in intmatarr
   integer :: nextfreeintmt                    ! next free position in intmt
   logical :: nested                           ! if true nested dissection
   type (intmatrix), target :: intmt(numintmat) ! refers to set of intmat arrays

!    colsii        contains column numbers of non-zero elements in original
!                  matrix in case of compact storage
!                  column numbers are stored from row to row, per row in
!                  increasing sequence
!                  In case of a symmetrical storage only the column numbers
!                  int the diagonal and upper triangular part are stored
!    colsiiprec    See colsii, but now for the matrix including fill-in
!    diag          Contains the positions of the diagonal elements in colsii
!    diagprec      Contains the positions of the diagonal elements in colsiiprec
!    rowsii        array of size n+1 containing the starting addresses (-1)
!                  of rows in colsii. Hence rowsii(1)=0 and rows(n+1) contains
!                  the size of colsii
!    rowsiiprec    See rowsii but now with respect to colsiiprec
end module sepmoduleintmat
