module sepmodulekmesh

!  Some quantities for the definition of the mesh

   use sepmodulemeshinf
   use sepmodulekmeshp
   use sepmodulekmeshv

   contains

   function sepgetinpelm ( ielgrp )  ! return with number of nodes per group
   integer, intent(in) :: ielgrp
   integer :: sepgetinpelm

      if ( kelma/=0 ) then
         sepgetinpelm = kmesha(ielgrp+1) - kmesha(ielgrp)
      else
         sepgetinpelm = npelm
      end if  ! ( kelma>0 )

   end function sepgetinpelm

   function sepgetinelem ( ielgrp )  ! return with number of elements per group
   integer, intent(in) :: ielgrp
   integer :: sepgetinelem

      if ( kelmg/=0 ) then
         sepgetinelem = kmeshg(ielgrp)
      else
         sepgetinelem = nelem
      end if  ! ( kelmg>0 )

   end function sepgetinelem

   function sepgetshape ( ielgrp )  ! return with shape number of group
   integer, intent(in) :: ielgrp
   integer :: sepgetshape

      if ( kelmf/=0 ) then
         sepgetshape = kmeshf(ielgrp)
      else
         sepgetshape = -1
      end if  ! ( kelmg>0 )

   end function sepgetshape

   ! --- Compute starting in kmesha

   function sepgetkmesha ( ielgrp ) result(istart)
   implicit none
   integer, intent(in) :: ielgrp    ! element group number
   integer :: istart                ! First position in array
   if ( kelma==0 ) then
      istart = (ielgrp-1)*npelm
   else
      istart = kmesha(ielgrp)
   end if  ! ( kelma==0 )

   end function sepgetkmesha

   ! --- Compute starting and ending address of elements in kmeshc

   function sepgetkmeshc ( isurnr, iend ) result(istart)
   implicit none
   integer, intent(in) :: isurnr    ! Surface sequence number
   integer :: istart                ! First position in array
   integer :: iend                  ! Last position used in array

   integer :: ielem       ! Element number
   integer :: ielem1      ! First element number in surface
   integer :: ielem2      ! Last element number in surface
   integer :: ielgrp      ! element group number
   integer :: inpelm      ! number of nodes per element

   ielem1 = kmesht(1,isurnr)
   ielem2 = kmesht(2,isurnr)
   inpelm = npelm

   !  --- Compute starting address of element ielem1 in kmesh
   !      Compute end address of element ielem2 in kmeshc

   if ( nelgrp>1 ) then

   !  --- Only one element group

      istart = inpelm*(ielem1-1)+1
      iend   = inpelm*ielem2

   else

   !  --- More element groups

      istart = 0
      ielem  = 0
      do ielgrp = 1, nelgrp
         inpelm = sepgetinpelm(ielgrp)
         inelem = sepgetinelem(ielgrp)
         if ( ielem+inelem>=ielem1 ) exit
         ielem = ielem+inelem
         istart = istart+inpelm*inelem
      end do
      istart = max(1,istart+(ielem1-ielem-1)*inpelm+1)
      iend = max(0,istart+(ielem2-ielem1+1)*inpelm-1)

   end if
   inelem = ielem2-ielem1+1

   end function sepgetkmeshc

   ! --- Compute starting and ending address of elements in kmeshc

   function sepstkmeshc ( ielem, inpelm ) result(istart)
   implicit none
   integer :: istart      ! First position in array
   integer, intent(in) :: ielem       ! Element number
   integer, intent(out) :: inpelm      ! number of node sin element

   integer :: ielgrp      ! Element group number

   ielgrp = sepelgrpelem(ielem)
   inpelm = Topgrp(ielgrp)%inpelm
   istart = pointkmsc(ielgrp)+(ielem-inelem-1)*inpelm+1

   end function sepstkmeshc

   ! --- Compute number of nodal points in elements of surface number isurnr

   function sepgetnpelmsurf ( isurnr, ivol ) result(jnpelm)
   implicit none
   integer, intent(in) :: isurnr    ! Surface sequence number
   integer :: jnpelm                ! number of nodes per element
   integer, intent(in) :: ivol      ! Indication whether surfaces (0) or
                                    ! volumes (1) are used

   integer :: ielem       ! Element number
   integer :: ielgrp      ! element group number
   integer :: inelem      ! number of elements in element group

   if ( ivol==0 .and. kelmn>0 ) then

!  --- surface elements, jnpelm is stored in kmeshn

      jnpelm = kmeshn(isurnr)

   else if ( kelma==0 ) then

!  --- all elements have the same number of nodes

      jnpelm = npelm

   else

!  --- More element groups, the first element number is stored in
!      kmesh part t

      ielem  = kmesht(1,isurnr)

!     --- Find element group corresponding to ielem

      inelem = 0
      do ielgrp = 1, nelgrp
         inelem = inelem + sepgetinelem ( ielgrp )
         if ( ielem<=inelem ) then
            jnpelm = sepgetinpelm ( ielgrp )
            exit
         end if
      end do  ! ielgrp = 1, nelgrp

   end if  ! ( ivol==0 .and. ipkmsn>0 )

   end function sepgetnpelmsurf

   function sepelgrpelem ( ielem ) result(elgrp)
   implicit none
   integer :: elgrp              ! element group
   integer, intent(in) :: ielem  ! element number

   integer :: ielgrp     ! localelement group number

   elgrp = 1
   inelem = 0

   if ( nelgrp>1 ) then

!  --- nelgrp>0, more element groups

      inelem = 0
      do ielgrp = 1, nelgrp

         inelem = inelem + kmeshg(ielgrp)
         if ( ielem<=inelem ) then
            elgrp = ielgrp
            exit
         end if  ! ( ielem<=inelem )

      end do  ! ielgrp = 1, kmesh(5)

   end if  ! ( nelgrp>1 )

   end function sepelgrpelem

!  --- Compute shift in parts of array kprob due to element group

   function sepelshift ( ielgrp ) result(ishift)
   implicit none
   integer, intent(in) :: ielgrp    ! element group
   integer :: ishift                ! shift in kprob parts wrt ielgrp
   if ( kelma==0 ) then
      ishift=(ielgrp-1)*npelm
   else
      ishift=kmesha(ielgrp)
   end if
   end function sepelshift

!  --- Compute number of nodes in element of surface

   function sepgetinpelsurf ( isurnr, ivol ) result(inpelm)
   implicit none
   integer, intent(in) :: ivol    ! Indication whether surfaces (0) or volumes
                                  ! (1)are used
   integer, intent(in) :: isurnr  ! surface number
   integer :: inpelm              ! number of nodes in surface
   integer :: ielem               ! element number
   integer :: inelem              ! number of elements in group
   integer :: ielgrp              ! sequence number of element group

   if ( ivol==0 .and. kelmn/=0 ) then

!  --- surface elements, inpelm is stored in kmeshn

      inpelm = kmeshn(isurnr)

   else if ( kelma==0 ) then

!  --- all elements have the same number of nodes

      inpelm = npelm

   else

!  --- More element groups, the first element number is stored in
!      kmesh part t

      ielem  = kmesht(1,isurnr)

!     --- Find element group corresponding to ielem

      inelem = 0
      do ielgrp = 1, nelgrp
         inelem = inelem + kmeshg(ielgrp)
         if ( ielem<=inelem ) then
            inpelm = kmesha(1+isurnr) - kmesha(isurnr)
            exit
         end if
      end do  ! ielgrp = 1, nelgrp

   end if

   end function sepgetinpelsurf

!  --- Compute number of nodes in element

   function sepgetinpelem(ielem) result(inpelm)
   implicit none
   integer, intent(in) :: ielem   ! element number
   integer :: inpelm              ! number of nodes in element
   integer :: ielgrp              ! element group number

   ielgrp = sepelgrpelem(ielem)
   inpelm = sepgetinpelm ( ielgrp )
   end function sepgetinpelem

!  --- Compute number of boundary nodes in element

   function sepgetbounelm(ielgrp) result(nbounelm)
   implicit none
   integer, intent(in) :: ielgrp  ! element group number
   integer :: nbounelm            ! number of boundary nodes in element

   if ( kelmb/=0 ) then
      nbounelm = kmeshb(ielgrp)
   else
      nbounelm = sepgetinpelm ( ielgrp )
   end if  ! ( kelmb/=0 )
   end function sepgetbounelm

!  --- Compute number of faces in element

   function sepgetnfapel(ielgrp) result(nfapel)  ! Old inifpel
   implicit none
   integer, intent(in) :: ielgrp  ! element group number
   integer :: nfapel              ! number of faces in element
   integer :: dfapel(18)
   data dfapel / 0, 0, 1, 1, 1, &
                 1, 1, 1, 1, 1, &
                 4, 4, 6, 6, 4, &
                 4, 6, 4/

   ishape = sepgetshape ( ielgrp )
   if ( ishape<=0 .or. ishape>=50 .and. ishape<=60  ) then
      nfapel = 0
   else if ( 1<=ishape .and. ishape<=18 ) then
      nfapel = dfapel(ishape)
   else if ( ishape>=20 .and. ishape<=22 ) then
      nfapel = 5
   else

!  --- Shape number not implemented

      call eropen ( 'sepgetnfapel' )
      call errint ( ishape, 1 )
      call errsub ( 800, 1, 0, 0 )
      call erclos ( 'sepgetnfapel' )
      nfapel = -1

   end if
   end function sepgetnfapel

!  --- Compute number of edges per face of element

   function sepgetnedpfa(ielgrp) result(nedpfa) ! Old topepfa
   implicit none
   integer, intent(in) :: ielgrp  ! element group number
   integer :: nedpfa              ! number of faces in element
   integer :: dedpfa(22)
   data dedpfa / 0, 0, 3, 3, 4,    4, 3, 3, 4, 3, &
                 3, 3, 4, 4, 3,    3, 4, 3, 0, 0, &
                 4, 4 /
   ishape = sepgetshape ( ielgrp )

   if ( ishape<=0 .or. ishape>=50 .and. ishape<=60 ) then

      nedpfa = 0

   else if ( ishape>18 .and. ishape/=21 .and. ishape/=22 ) then

!  --- Shape number not implemented

      call eropen ( 'sepgetnedpfa' )
      call errint ( ishape, 1 )
      call errsub ( 800, 1, 0, 0 )
      call erclos ( 'sepgetnedpfa' )

   else
      nedpfa = dedpfa(ishape)
   end if
   end function sepgetnedpfa

!  --- Compute number of edges as function of element shape

   function sepgetnedgeshape(ishape) result(nedgeshape) ! Old iniedgeshape
   implicit none
   integer, intent(in) :: ishape  ! shape of element
   integer :: nedgeshape          ! number of faces in element
   integer :: dedpel(18)
   data dedpel / 1, 1, 3, 3, 4, &
                 4, 3, 3, 4, 3, &
                 6, 6,12,12, 6, &
                 6,12, 6 /
   if ( ishape==-10001 .or. ishape==-9 ) then
      nedgeshape = 1
   else if ( 1<=ishape .and. ishape<=18 ) then
      nedgeshape = dedpel(ishape)
   else if ( ishape>=20 .and. ishape<=22 ) then
      nedgeshape = 9
   else if ( ishape<=0 .or. ishape>=50 .and. ishape<=60 ) then
      nedgeshape = -1
   else

!  --- Shape number not implemented

      call eropen ( 'sepgetnedgeshape' )
      call errint ( ishape, 1 )
      call errsub ( 800, 1, 0, 0 )
      call erclos ( 'sepgetnedgeshape' )
      nedgeshape = -1

   end if
   end function sepgetnedgeshape

!  --- Compute number of edges per element

   function sepgetnedpel(ielgrp) result(nedpel) ! Old iniepel
   implicit none
   integer, intent(in) :: ielgrp  ! element group number
   integer :: nedpel              ! number of faces in element

   ishape = sepgetshape ( ielgrp )
   nedpel = sepgetnedgeshape ( ishape )
   end function sepgetnedpel

!  --- Compute number of nodes per edge in element group ielgrp

   function sepgetnndped(ielgrp) result(nndped) ! Old ininped
   implicit none
   integer, intent(in) :: ielgrp  ! element group number
   integer :: nndped              ! number of faces in element
   integer :: dnoped(22)
   data dnoped / 2, 3, 2, 3, 2,   3, 3, 4, 2, 2, &
                 2, 3, 2, 3, 3,   3, 2, 2, 0, 0, &
                 2, 3 /

   ishape = sepgetshape ( ielgrp )
   if ( (ishape<=0 .or. ishape>=50 .and. ishape<=60) .and. nspec==0 ) then

      nndped = 2

   else if (nspec==0 .and. ishape<=22 ) then

      nndped = dnoped(ishape)
      if ( nndped==0 ) then

!     --- Shape number not implemented

         call eropen ( 'sepgetnndped' )
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         call erclos ( 'sepgetnndped' )

      end if

   else if ( nspec>0 .and. &
           ( ishape==5 .or. ishape==13) ) then
      nndped = nspec

   else

      call eropen ( 'sepgetnndped' )
      call errint ( ishape, 1 )
      call errint ( nspec, 2 )
      call errsub ( 1692, 2, 0, 0 )
      call erclos ( 'sepgetnndped' )

   end if
   end function sepgetnndped

!  --- Compute number of edges in element as function of the shape number

   function sepgetnedge_elem(ishape) result(nedge_elem) ! Old  ininumnodedge
   implicit none
   integer, intent(in) :: ishape  ! shape number
   integer :: nedge_elem          ! number of faces in element
   integer :: dedpel(18)
   data dedpel / 1, 1, 3, 3, 4, &
                 4, 3, 3, 4, 3, &
                 6, 6,12,12, 6, &
                 6,12, 6 /
   if ( ishape==-10001 .or. ishape==-9 ) then
      nedge_elem = 1
   else if ( 1<=ishape .and. ishape<=18 ) then
      nedge_elem = dedpel(ishape)
   else if ( ishape>=20 .and. ishape<=22 ) then
      nedge_elem = 9
   else if ( ishape<=0 ) then
      nedge_elem = -1
   else

!  --- Shape number not implemented

      call eropen ( 'sepgetnedge_elem' )
      call errint ( ishape, 1 )
      call errsub ( 800, 1, 0, 0 )
      call erclos ( 'sepgetnedge_elem' )
      nedge_elem = -1

   end if

   end function sepgetnedge_elem

!  --- Compute number of nodes in edge as function of the shape number

   function sepgetnumnodedge(ishape) result(numnodedge)
   implicit none
   integer, intent(in) :: ishape  ! shape number
   integer :: numnodedge          ! number of faces in element
   integer :: numnodesedge(22)
   data numnodesedge / 2, 3, 2, 3, 2,    3, 3, 4, 2, 2, &
                       2, 3, 2, 3, 3,    3, 2, 2, 0, 0, &
                       2, 3 /

   if ( ishape==-10001 ) then
      numnodedge = 1
   else if ( ishape==-9 ) then
      numnodedge = 2
   else if ( ishape>=1 .and. ishape<=18 .or. &
             ishape>=21 .and. ishape<=22 ) then
      numnodedge = numnodesedge(ishape)
   else if ( ishape<=0 ) then
      numnodedge = -1
   else

!  --- Shape number not implemented

      call eropen ( 'sepgetnumnodedge' )
      call errint ( ishape, 1 )
      call errsub ( 800, 1, 0, 0 )
      call erclos ( 'sepgetnumnodedge' )
      numnodedge = -1

   end if
   end function sepgetnumnodedge

!  --- List of subroutines
!
!    subroutine seppointkmeshc    ! Fill array pointkmsc
!    subroutine seppointkmnel     ! Fill array pointnel

!  --- List of functions
!
!    inpelm = sepgetinpelm ( ielgrp )  ! number of nodes per group
!    inpelm = sepgetinpelem ( ielem )  ! number of nodes in element
!    inpelm = sepgetinpelsurf ( isurnr, ivol )  ! number of nodes in surface
!             ivol = 0 and kmeshn exists use kmeshn
!             else extract from first element in surface
!    inelem = sepgetinelem ( ielgrp )  ! number of elements per group
!    ishape = sepgetshape ( ielgrp )   ! shape number of group
!    istart = sepgetkmesha ( ielgrp )  ! start address in kmesha
!    istart = sepgetkmeshc ( isurnr, iend ) ! start and ending address
!                                             of elements in kmeshc
!    istart = sepstkmeshc ( ielem, inpelm )  ! start address of ielem in kmeshc
                                       ! Old iniipkc/inistelkmeshc
!    inpelm = sepgetnpelmsurf ( isurnr, ivol ) ! number of nodes in surface
                                               ! (ivol=0) or volume (ivol=1)
!    ielgrp = sepelgrpelem(ielem)      ! element group corresponding to ielem
!                                      ! Old getelgrpelem
!    ishift = sepelshift ( ielgrp )    ! shift for element group
!    nbounelm = sepgetbounelm(ielgrp)  ! number of boundary nodes of element
!    nfapel = sepgetnfapel(ielgrp)     ! number of faces in element
                                       ! Old inifpel
!    nedpfa = sepgetnedpfa(ielgrp)     ! number of edges per face of element
                                       ! Old topepfa
!    nedpel = sepgetnedpel(ielgrp)     ! number of edges per element.
                                       ! Old iniepel
!    nedgeshape = sepgetnedgeshape(ishape)  ! number of edges as function of
                                            ! element shape
                                       ! Old iniedgeshape
!    nndped = sepgetnndped(ielgrp)     !number of nodes per edge in element
                                       ! group ielgrp
                                       ! Old ininped
!    nedge_elem = sepgetnedge_elem(ishape)  ! number of edges in element as
                                            !function of the shape
                                            ! Old  ininumnodedge

end module sepmodulekmesh
