module sepmodulekmeshv

   use sepmodulemeshinf

!  Some quantities for the definition of the mesh

   integer :: icountmesh   ! sequence number of mesh
   integer :: inelem       ! number of elements in an element group
   integer :: iseqkmobst   ! Sequence number of obstacle w.r.t. kmesh
   integer :: iseqobst     ! Sequence number of obstacle
   integer :: ishape       ! Shape number of elements
   integer :: mmielgrp_nodes  ! Memory management sequence number of array
                           ! ielgrp_nodes
   integer :: mmnode_elgrps  ! Memory management sequence number of array
                           ! node_elgrps
   integer :: nbounelm     ! Number of boundary points at an element
   integer :: nedpfa       ! Number of edges per face of element
   integer :: nedges       ! Number of edges
   integer :: nedgeshape   ! Number of edges as function of the shape of an
                           ! element
   integer :: nedpel       ! Number of edges per element
   integer :: nfaces       ! Number of faces
   integer :: nfapel       ! Number of faces per element
   integer :: niedge       ! number of internal (spectral) points in edge
   integer :: ninner       ! Number of internal curves
   integer :: nintprop     ! number of integer properties per element group
   integer :: nisurf       ! number of internal (spectral) points in surface
   integer :: nivolm       ! number of internal (spectral) points in volume
   integer :: nndped       ! number of nodes per edge in element group
   integer :: npointglob   ! Global number of nodes
   integer :: nrealprop    ! number of real properties per element group
                           ! element
   integer :: nspec        ! Number of points at the boundary of
                                    ! a spectral element

   integer, pointer :: blockinfo(:)  ! Array with global block information for
                              !  surface number isurnr
                              !  Contents:
                              !  Pos. 1  m ( number of neighbor blocks)
                              !  Pos. 2*i   block number of i-th neighbor
                              !  Pos. 2*i+1 starting address of i-th neighbor in
                              !             array transinfo
   integer, pointer :: curveusrp(:)  ! user point numbers corresponding to
                                     ! curves
   integer, pointer :: elemobst(:)   ! Array of length nelem containing the
                                     ! following information
                                     ! elemobst(i) = 1: element i is external
                                     ! elemobst(i) = 2: element i intersects
                                     !                  the obstacle
                                     ! elemobst(i) = 3: element i is internal
   integer, pointer :: elemskip(:)   ! Array of length nelem with for each
                                     ! element to which level it belongs
                                     ! 1<=elemskp(i)<=nlevels+1
   integer, pointer :: globnodes(:)  ! contains the global node numbers of
                                     ! the nodes (parallel)
   integer, pointer :: iactobst(:)   ! Array of length numobst containing
!                         information about active obstacles
!                         iactobs(i) = 0, means obstacle not active
!                         iactobs(i)>0, means obstacle active
   integer, pointer :: icrossobst(:) ! information about the intersection of
                                     ! an obstacle with the mesh
   integer, pointer :: icurvsmsh(:)  ! information of the number of nodes in
                                     ! the curves
   integer, pointer :: icurvsobst(:,:) ! information of the active obstacles.
                                     ! The array corresponds to all curves
                                     ! separately
                                     ! icurvsobst(1,i) = 1 means that the
                                     ! corresponding point is active;
                                     ! 0 means not active
   integer, pointer :: icurvsobst2d(:,:) ! 2d version of icurvsobst
   integer, pointer :: ielemobst(:)  ! information about the elements in an
                                     ! obstacle
                                     ! See PG Section 24.2 for a description
   integer, pointer :: ielgrp_nodes(:)  ! contains all nodes corresponding
                                     ! to element groups
   integer, pointer :: iinputkm(:)   ! iinput
   integer, pointer :: intprop(:,:)  ! integer properties
   integer, pointer :: iobgrid2d(:,:)  ! iobgrid in R^2
!                        array of length nx x ny
!                        position i,j contains:
!                         -1:  cell inside obstacle
!                          0:  cell outside obstacle
!                         >0:  cell contains part of the obstacle
!                         The value contains a starting address in iobgrid1
!                         as in iobgrid2d1
   integer, pointer :: iobgrid2d1(:) ! iobgrid1 in R^2
!                         Must be used in connection with iobgrid2d
!                         iobgrid2d contains the starting addresses for each
!                         cell i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
   integer, pointer :: iobgrid3d(:,:,:,:)    ! In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin,
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
   integer, pointer :: iobscurv(:)   ! Array of length ncurvs indicating if
!                     a curve is on an obstacle or not
!                     iobscurv(i) = 0 means curve i not on an obstacle
!                                   1 points of curve i on an obstacle
!                                   2 only begin and end point on obstacle
!                                   3 only begin point on obstacle
!                                   4 only end point on obstacle! information
!                                     of the active obstacles.
   integer, pointer :: iouterbounseq(:) !kmesh part al
!                    Contains outer boundaries corresponding to a selected
!                    set of element groups
   integer, pointer :: ipointobst(:) ! information about the nodes in an
                                     ! obstacle
                                     ! See PG Section 24.2 for a description
   integer, pointer :: ipointobstpart(:) ! part of array ipointobst
   integer, pointer :: kmesha(:)     ! kmesh part a
   integer, pointer :: kmeshac(:)    ! kmesh part ac
   integer, pointer :: kmeshad(:)    ! kmesh part ad
   integer, pointer :: kmeshb(:)     ! kmesh part b
   integer, pointer :: kmeshc(:)     ! kmesh part c
   integer, pointer :: kmeshc2d(:,:) ! kmesh part c (2d)
   integer, pointer :: kmeshcglob(:) ! global kmesh part c (parallel)
   integer, pointer :: kmeshcobst(:,:) ! kmesh part c corresponding to obstacle
   integer, pointer :: kmeshcpoint(:)  ! contains pointers to elements in kmeshc
   integer, pointer :: kmeshcsub(:)  ! temporary array kmeshc
   integer, pointer :: kmeshctmp(:)  ! temporary array kmeshc
   integer, pointer :: kmeshctri(:)  ! kmeshc for triangles only
   integer, pointer :: kmeshd(:)     ! kmesh part d
   integer, pointer :: kmshd1(:)     ! kmesh part d1
   integer, pointer :: kmshd2(:)     ! kmesh part d2
   integer, pointer :: kmeshe(:)     ! kmesh part e
   integer, pointer :: kmshe1(:)     ! kmesh part e1
   integer, pointer :: kmshe2(:)     ! kmesh part e2
   integer, pointer :: kmeshf(:)     ! kmesh part f
   integer, pointer :: kmeshg(:)     ! kmesh part g
   integer, pointer :: kmeshh(:,:)   ! kmesh part h
   integer, pointer :: kmeshj(:)     ! kmesh part j
   integer, pointer :: kmeshk(:)     ! kmesh part k
   integer, pointer :: kmeshl(:)     ! kmesh part l
   integer, pointer :: kmeshlglob(:) ! global kmesh part l (parallel)
   integer, pointer :: kmeshm(:)     ! kmesh part m
   integer, pointer :: kmeshmglob(:) ! global array kmesh part m (parallel)
   integer, pointer :: kmeshn(:)     ! kmesh part n
   integer, pointer :: kmeshnglob(:) ! global kmesh part n (parallel)
   integer, pointer :: kmesho(:)     ! kmesh part o
   integer, pointer :: kmsho1(:)     ! kmesh part o part 1
   integer, pointer :: kmsho2(:)     ! kmesh part o part 2
   integer, pointer :: kmeshpar(:)   ! Array of size 10 in which information of
                      !  the global mesh in case of parallel computing is stored
                      !  Contents:
                      !  1: npoint_global
                      !  2: memory management number of coor_global
                      !  3: memory management number of kmeshl_global
                      !  4: memory management number of kmeshm_global
                      !  5: memory management number of kmeshn_global
                      !  6: memory management number of mapping array
   integer, pointer :: kmeshq(:)     ! kmesh part q
   integer, pointer :: kmeshr(:)     ! kmesh part r
   integer, pointer :: kmeshs(:)     ! kmesh part s
   integer, pointer :: kmeshs2(:,:)  ! kmesh part s part 2
   integer, pointer :: kmesht(:,:)   ! kmesh part t
   integer, pointer :: kmeshu(:)     ! kmesh part u
   integer, pointer :: kmeshv(:)     ! kmesh part v
   integer, pointer :: kmshv1(:)     ! kmesh part v1
   integer, pointer :: kmshv2(:)     ! kmesh part v2
   integer, pointer :: kmeshw(:)     ! kmesh part w
   integer, pointer :: kmeshx(:,:)    ! kmesh part x
   integer, pointer :: kmeshxsur(:)  ! surfaces corresponding to obstacles
   integer, pointer :: kmeshz(:)     ! kmesh part z
   integer, pointer :: lastlevel(:)  ! last level in series of levels
   integer, pointer :: levelstruc(:) ! contains series of levels
   integer, pointer :: mappingpar(:) ! mapping array (parallel)
   integer, pointer :: mappingparcp(:) ! copy of mapping array (parallel)
   integer, pointer :: nextlevel(:)  ! next level in series of levels
   integer, pointer :: nmesh(:)      ! nmesh
   integer, pointer :: nmesha(:)     ! nmesh part a
   integer, pointer :: nmeshb(:)     ! nmesh part a
   integer, pointer :: nmeshc(:)     ! nmesh part a
   integer, pointer :: nmeshd(:)     ! nmesh part a
   integer, pointer :: nmeshe(:)     ! nmesh part a
   integer, pointer :: nmeshf(:)     ! nmesh part a
   integer, pointer :: nmeshg(:)     ! nmesh part a
   integer, pointer :: nmeshh(:)     ! nmesh part a
   integer, pointer :: nmeshi(:)     ! nmesh part a
   integer, pointer :: nmeshj(:)     ! nmesh part a
   integer, pointer :: nmeshk(:)     ! nmesh part a
   integer, pointer :: nmeshl(:)     ! nmesh part a
   integer, pointer :: node_elgrps(:) ! contains all element groups to
                                      ! nodes
   integer, pointer :: obstfaces(:)  ! edges or faces of the boundary of an
                                     ! obstacle
   integer, pointer :: outerboun(:)  ! outer boundary
   integer, pointer :: outercurves(:)  ! outer curves
   integer, pointer :: pointnel(:)  ! length nelgrp+1
                                    ! pointnel(i) contains number of elements
                                    ! in element groups 1 to i-1
   integer, pointer :: pointkmsc(:) ! length nelgrp+1
                                    ! pointkmsc(i) contains start address
                                    ! of element group i in kmeshc (minus 1)
   integer, pointer :: pointlevel(:) ! pointers to levelstruc
   integer, pointer :: set(:)       ! contains node numbers in a set
   integer, pointer :: surfnodes(:) ! Contains the node numbers corresponding
                       !  to the surface
                       !  Pos. 1 ... nsurfs starting addresses to part 2
                       !  Part2: for each surface:
                       !  number of nodes followed by node numbers
   integer, pointer :: transinfo(:)  ! contains information of the transport of
                       ! unknowns between neighboring blocks
                       ! The starting address of each neighbor block is
                       ! stored in array blockinfo
                       ! For each neighbor block the following information
                       ! is stored
                       ! Pos. 1 number of nodes n on common side of both blocks
                       ! Pos. 2 ... n+1    local node numbers in block i
                       ! Pos. n+2 ... 2n+1 local node numbers in neighbor
                       !                   block j
   integer, pointer :: volume_surfs(:)  ! contains all sufaces corresponding
                                        ! to the volumes

   double precision, pointer :: coor(:,:)     ! coordinates
   double precision, pointer :: coorglob(:,:) ! global coordinates (parallel)
   double precision, pointer :: coorobst(:,:) ! coordinates of obstacle
   double precision, pointer :: curvesmsh(:,:) ! coordinates of the nodes in the
                                              ! curves
   double precision, pointer :: curvefacts(:) ! factors t along the curves
   double precision, pointer :: elsize(:,:)   ! 2d version of kmeshp
!                         elsize(1,i) element length of first element of curve i
!                         elsize(2,i) element length of last element of curve i
   double precision, pointer :: kmeshquant(:) ! contains some mesh quantities
                                              ! At this moment
                                              ! Pos 1:2  xmin, xmax
                                              ! Pos 3:4  ymin, ymax
                                              ! Pos 5:6  zmin, zmax
   double precision, pointer :: normal(:,:)   ! Contains the normals in the
!                         nodes of the surfaces in exactly the same
!                         sequence as in array surfnodes
   double precision, pointer :: realprop(:,:) ! real properties
   double precision, pointer :: rcrossobst(:) ! information about the
                                              ! intersection of an obstacle
                                              ! with the mesh
   double precision, pointer :: rinputkm(:)   ! rinput
   double precision, pointer :: rmeshac(:,:)  ! real part of kmeshac
   double precision, pointer :: rmeshx(:,:) ! real part of kmesh part x

   integer, pointer :: kmeshloc   ! local array kmesh
   type (kmeshobstacle), pointer :: kmobst(:)
   type (kmeshhsur), pointer :: kmhsur(:)
   type(intarr), pointer :: mappar(:) ! contains mapping arrays for parallel
                                      ! computing

   type topology_ielgrp

!  --- Defines the substructure of the topology

      integer :: inpelm         ! number of points per element
      integer :: inelem         ! number of elements in group
      integer, pointer :: topol_elg(:,:) ! topology of element group

   end type topology_ielgrp

   type subcurves

!  --- Defines the subcurves

      integer :: nnodes             ! number of points in curve
      logical inner_curve           ! if true it concerns an inner curve

      integer, pointer :: nodes(:)  ! node numbers of curve

   end type subcurves

   type (topology_ielgrp), pointer :: Topgrp(:)
   type (subcurves), pointer :: Curvesub(:)

end module sepmodulekmeshv
