module sepmodulematr1

use sepmodulesol
use sepmodulemat
use sepmodulecmat

type globmat          ! vectors of type matr
   integer :: intmatref  ! Corresponding intmat sequence number in bufarr
   integer :: icounter   ! sequence number
                         ! defines if array has been copied to module
   integer :: iprob      ! Problem number
   integer :: jmethod    ! storage method
   integer :: nrow       ! number of rows in matrix S_ii
   logical :: complexmat ! If true matrix is complex
   logical :: diagonal   ! If true matrix is diagonal
   logical :: decomp     ! If true the decomposition has not been stored
   logical :: inclbc     ! If true include boundary conditions
   logical :: intmul     ! If true internal multiplication is used
   double precision, pointer :: matcont(:)  ! elements of matrix
   double precision, pointer :: matboun(:)  ! elements of boundary matrix
   double precision, pointer :: matinv(:)   ! elements of inverse matrix
                                            ! or preconditioning matrix
   double precision, pointer :: S_ii(:)   ! large matrix
   double precision, pointer :: S_iib(:)  ! second part large matrix
   double precision, pointer :: S_ip(:)   ! matrix corresponding to essential
                                          ! boundary conditions
   double precision, pointer :: S_pi(:)   ! matrix corresponding to essential
                                          ! boundary conditions
   double precision, pointer :: S_pp(:)   ! matrix corresponding to essential
                                          ! boundary conditions
   double precision, pointer :: S_ic(:)   ! matrix corresponding to periodical
                                          ! boundary conditions
   double precision, pointer :: S_if(:)   ! matrix corresponding to interface
                                          ! unknowns
   double precision, pointer :: S_fi(:)   ! matrix corresponding to interface
                                          ! unknowns
   double precision, pointer :: S_iout(:) ! matrix containing parts of equations
   double precision, pointer :: Dlapshift(:) ! diagonal matrix for
                                          ! shifted laplace

!  --- Complex matrices

   double complex, pointer :: zmatcont(:)  ! elements of matrix
   double complex, pointer :: zmatboun(:)  ! elements of boundary matrix
   double complex, pointer :: zmatinv(:)   ! elements of inverse matrix
                                           ! or preconditioning matrix
   double complex, pointer :: Sz_ii(:)   ! large matrix
   double complex, pointer :: Sz_ip(:)   ! matrix corresponding to essential
                                         ! boundary conditions
   double complex, pointer :: Sz_pi(:)   ! matrix corresponding to essential
                                         ! boundary conditions
   double complex, pointer :: Sz_pp(:)   ! matrix corresponding to essential
                                         ! boundary conditions
   double complex, pointer :: Sz_ic(:)   ! matrix corresponding to periodical
                                         ! boundary conditions
   double complex, pointer :: Sz_if(:)   ! matrix corresponding to interface
                                         ! unknowns
   double complex, pointer :: Sz_fi(:)   ! matrix corresponding to interface
                                         ! unknowns
   double complex, pointer :: Sz_iout(:) ! matrix containing parts of equations
!  --- Matrices for simple

   double precision, pointer :: S_Q(:)   ! Q matrix
   double precision, pointer :: S_DIV(:) ! D matrix
   double precision, pointer :: S_G(:)   ! G matrix
   double precision, pointer :: S_C(:)   ! C matrix
   double precision, pointer :: S_PM(:)  ! PM matrix
   double precision, pointer :: S_PMD(:)  ! PMD matrix
   double precision, pointer :: S_R(:)   ! R matrix
   double precision, pointer :: S_UD(:)  ! Diagonal Velocity matrix
   double precision, pointer :: S_Qprime(:) ! Q' matrix

   integer, pointer :: sendinfo(:)        ! array sendinfo as stored in kprob
   type (rmatrix) :: S_i
   type (cmatrix) :: Sc_i
   type (pmatrix) :: S_p
   type (cpmatrix) :: Sc_p

end type globmat

!  --- Define matrix vector multiplication interface for this type

   interface operator (*)
      module procedure matvecii        ! y = A*x (real matrix)
      module procedure cmatvecii       ! y = A*x (complex matrix)
   end interface

!  --- Define solve interface for this type


   interface operator (/)
      module procedure matsolveii   ! A x = y (real matrix)
      module procedure cmatsolveii  ! A x = y (complex matrix)
   end interface

!  --- Define actual matrix vector multiplication

   contains

   function matvecii ( A, x ) result(y)  ! y = A*x
   implicit none
   type (globmat), intent(in) :: A                ! matrix
   double precision, intent(in) :: x(A%nrow)      ! vector
   double precision :: y(A%nrow)                  ! result
   call sepmaver1 ( A, x, y )
   end function matvecii

   function cmatvecii ( A, x ) result(y)  ! y = A*x
   implicit none
   type (globmat), intent(in) :: A              ! matrix
   double complex, intent(in) :: x(A%nrow)      ! vector
   double complex :: y(A%nrow)                  ! result
   if ( A%complexmat ) then
      call sepcmaver1 ( A, x, y )
   else
      call sepmaver2 ( A, x, y )
   end if  ! ( A%complexmat )
   end function cmatvecii

   function matsolveii ( A, y ) result(x)  ! A x = y

   implicit none
   type (globmat), intent(in) :: A     ! matrix
   double precision, intent(in) :: y(A%nrow)      ! right-hand side
   double precision :: x(A%nrow)                  ! result

   call sepsolvemethi ( A, x, y )

   end function matsolveii

   function cmatsolveii ( A, y ) result(x)  ! A x = y

   implicit none
   type (globmat), intent(in) :: A     ! matrix
   double complex, intent(in) :: y(A%nrow)      ! right-hand side
   double complex :: x(A%nrow)                  ! result

   call sepcsolvemethi ( A, x, y )

   end function cmatsolveii

end module sepmodulematr1
