module sepmoduleprobinf

use sepmodulegen
use sepmodulecons

integer, parameter :: lenkprobfp= 80  ! Length of first part of array kprob

type probsub        ! refers to sub problems
   integer :: iseqkm                ! Sequence  number of corresponding mesh
   integer :: icounter  ! sequence number
   integer :: nprob     ! number of problems
   integer, pointer :: iinputkp(:)  ! iinput
   double precision, pointer :: rinputkp(:)  ! rinput
   type (probinfo), pointer :: kprobsub(:)  ! Refers to sub problems
end type probsub

type probinfo           ! vectors of type kprob (per problem)

   integer, pointer :: intarprb(:)   ! local intarprb array
   integer, pointer :: iinputloc(:)  ! local iinput array
   integer, pointer :: kprobloc(:)   ! kprob array
   type(intarr), pointer :: kproba(:)     ! kprob part a
   integer, pointer :: kprobatmp(:)  ! specific kprob part a
   integer, pointer :: kprobb(:)     ! kprob part b
   integer, pointer :: kprobc(:)     ! kprob part c
   integer, pointer :: kprobd(:)     ! kprob part d
   integer, pointer :: kprobe(:,:)   ! kprob part e (part 1)
   integer, pointer :: kprobe2(:)    ! kprob part e (part 2)
   integer, pointer :: kprobf(:)     ! kprob part f
   integer, pointer :: kprobg(:,:)   ! kprob part g
   integer, pointer :: kprobfi(:)    ! kprob part f for input vector
   integer, pointer :: kprobfo(:)    ! kprob part f for output vector
   integer, pointer :: kprobfh(:)    ! either kprobf, kprobfi or kprobfo
   integer, pointer :: kprobh(:)     ! kprob part h
   integer, pointer :: kprobhinv(:)  ! kprob part h (inverse part)
   integer, pointer :: kprobhdd(:)   ! kprob part hdd (adapted for dd coupling)
   integer, pointer :: kprobhddinv(:) ! kprob part hdd (adapted for dd coupling)
   integer, pointer :: kprobi(:)     ! kprob part i
   integer, pointer :: kprobj(:,:)   ! kprob part j
   integer, pointer :: kprobn(:,:)   ! kprob part n
   integer, pointer :: kprobr(:)     ! kprob part r
   integer, pointer :: kprobo(:)     ! kprob part o
   integer, pointer :: kprobp(:,:)   ! kprob part p
   integer, pointer :: kprobs(:)     ! kprob part s
   integer, pointer :: kprobvp1(:,:) ! kprob part v part 1
   integer, pointer :: kprobvp2(:)   ! kprob part v part 2
   integer, pointer :: kprobwp1(:,:) ! kprob part w part 1
   integer, pointer :: kprobwp2(:)   ! kprob part w part 2
   integer, pointer :: kprobx(:)     ! kprob part x
   integer, pointer :: kproby(:)     ! kprob part y
   integer, pointer :: kprobz(:)     ! kprob part u
   integer, pointer :: kprobaa(:)    ! kprob part aa
   integer, pointer :: kprobab(:)    ! kprob part ab
   integer, pointer :: kprobac(:)    ! kprob part ac
   integer, pointer :: kprobad(:)    ! kprob part ac
   integer, pointer :: kprobfict(:,:) ! array of size nfictunk x 2 with the
                                     ! following contents
                                     ! kprobfict(1,i) contains the structural
                                     ! element corresponding to fictitious
                                     ! unknown i
                                     ! kprobfict(2,i) contains the fluid
                                     ! element corresponding to fictitious
                                     ! unknown i
   integer, pointer :: kprobper(:,:) ! Array of size 2 * numpairs containing the
                                     ! pairs of nodes with periodical
                                     ! boundary conditions
   double precision, pointer :: coortr(:,:) ! kprob part n (real part) (rprobn)
   double precision, pointer :: rinputloc(:) ! local array rinput
   double precision, pointer :: rprobs(:,:) ! kprob part s (real part)
   double precision, pointer :: rprobz(:) ! kprob part z (real part)

   integer, pointer :: kprobpar(:)   ! Part of kprob (start address kprob(65))
                                     ! containing extra information concerning
                                     ! parallel arrays
                                     !  Contents:
                                     !   1: nusolglob
                                     !   2: nrusolglob
                                     !   3: -
                                     !   4: kprobh dd ( special for dd parallel)
   integer, pointer :: kprobpz(:)    ! kprob part z
   integer, pointer :: skipelem(:)   ! skipelem: indicates if elements
                                     ! must be skipped (1) or not (0)

   integer, pointer :: blockinfor(:) ! Array containing information for parallel
                                     ! computing
                                     ! See subroutine probparblocks1 for a
                                     ! description
   integer, pointer :: kprobhglob(:) ! Array kprobh with global numbers of free
                                     ! degrees of freedom (length nrusol)
   integer, pointer :: ninterfdeg(:) ! contains number of free degrees of
                                     ! freedom along interface per block
                                     ! (accumulated) ninterfdeg(0) = 0
   integer, pointer :: nrusollocs(:) ! Contains largest degree of freedom per
                                     ! cpu
   integer, pointer :: nnz(:)        ! Contains number of non-zeros per row of
                                     ! the matrix, but only for the block stored
                                     ! on this cpu (petsc)
   integer, pointer :: onnz(:)       ! Contains number of non-zeros per row
                                     ! stored on other cpu's (in sepran sense)
   integer, allocatable :: kprobhglobinv(:) ! Inverse of array kprobhglob
   integer, pointer :: blockunkinfo(:)
   integer, pointer :: innercontr(:)
   integer, pointer :: sendinfo(:)
   integer, pointer :: sendinfopar(:)
   integer, pointer :: innercontru(:)
   integer, pointer :: sendinfou(:)
   integer, pointer :: innercontrp(:)
   integer, pointer :: sendinfop(:)

   type (realarr), pointer :: probzarr(:) ! Contains information of probz
   type (intarr), pointer :: kprobe3(:)   ! Contains subarrays kprobf
                                          ! corresponding to arrays of type ivec
   type (intarr), pointer :: kprobfglobal(:) ! subarrays kprobf and kprobe3

end type probinfo

integer :: iseqkp              ! Sequence number of problem array
integer :: lastprob            ! last position used in kp
integer :: nextfreekp          ! next free position in kp

type (probsub) :: kp(nummesh)  ! refers to set of kprob arrays
integer :: probarr(nummesh)    ! indicates if kp has been filled:
                               ! 0: kp(i) is empty
                               ! >0: kp(i) has been filled
logical :: removesol           ! if true solution arrays corresponding to
                               ! kprob may be removed in seprmprobinfo
logical :: removeintmat        ! if true intmat arrays corresponding to
                               ! kprob may be removed in seprmprobinfo

! Explanation of some arrays
!
!      kprobe          information of vectors of special structure
!      intarprb        Array containing parameters to be transported between
!                      lower subroutines of PROBDF.
!                      Contents:
!                       1:  iinp1   Starting address of first part in iinputkp
!                       2:  iinp2   Starting address of second part in iinputkp
!                       3:  iinp3   Starting address of third part in iinputkp
!                       4:  iinp4   Starting address of fourth part in iinputkp
!                       5:  iinp5   Starting address of fifth part in iinputkp
!                       6:  iinp6   Starting address of sixth part in iinputkp
!                       7:  iinp7   Starting address of seventh part in iinputkp
!                       8:  iinp8   Starting address of part in iinputkp
!                                   corresponding to global_element type numbers
!                       9:  iinp9   Starting address of part in iinputkp
!                                   corresponding to global element numbers
!                      10:  iinp10  Starting address of part in iinputkp
!                                   corresponding to global element renumbering
!                      11:  junkpt  indicates if number of degrees of freedom
!                                   per point is constant (1) or not (0)
!                      12:
!                      14:  nphys   number of physical unknowns per point
!                      15:  iinp11  Starting address of part in iinputkp
!                                   corresponding to fictitious_element
!                                   type numbers
!                      16:  iinp12  Starting address of part in iinputkp
!                                   corresponding to fictitious element numbers
!                      17:  iinp13  Starting address of part in iinputkp
!                                   corresponding to skip elements
!                      18:  iinp14  Starting address of part in iinputkp
!                                   corresponding to level set functions
!                      19:  iinp15  Starting address of part in iinputkp
!                                   corresponding to periodical boundary conds.
!                      20:  iskipelem
!      kprobe          information of vectors of special structure
!                      size: (ndtvec, kprelgrp)
!                      The first index refers to the vector of special structure
!                      The second index contains the number of degrees of
!                      freedom for each node in the standard elements
!                      First all nodes of standard element 1 and so on
!      kprobe2         Indicates if the number of unknowns in vector of
!                      special structure ivec is constant (<0) or variable (>0)
!                      If <0 minus the value contains the number of unknowns
!                      if >0 npoint is stored
!      kprobe3         array of type intarr of length ndtvec
!                      if ( kprobe2(ivec)>0 ) kprobe3(ivec)%arr is a vector of
!                      size npoint+1 containing array kprobf corresponding
!                      to vector of special structure ivec
!      kprobf          is only used if the number of unknowns per point is
!                      not constant
!                      kprobf(1) = 0
!                      The number of unknowns in node i = kprobf(i+1)-kprobf(i)
!      kprobfglob      array of integer arrays containing the global arrays
!                      kprobf and kprobe3
!                      kprobfglob(0) refers to kprobf
!                      kprobfglob(i) refers to kprobe3(i) i = 1 to ndtvec
end module sepmoduleprobinf
