c ********************************************************************
c *
c *   example for the solution of the navier-stokes equations
c *   channel flow in t-configuration  (only one half used)
c *   power-law viscosity model
c *   programmer Guus Segal
c *   date 01-10-86
c *
c ********************************************************************
c *
      program navsto
      implicit double precision (a-h,o-z)
      integer:: kmesh,kprob,iuser(100),intmat
     +          ipress,isol,istrm
      double precision :: user(100)
      iuser(1)=100
      user(1)=100
      kmesh=0
      kprob=0
      intmat=0
      ipress=0
      isol=0
      istrm=0
      iuser(3)=0
      user(3)=0
c
c ********************************************************************
c
c   *  start of program  (standard for each sepran program)
c
c ********************************************************************
c
      print *,'PROGRAM NAVSTOK2'
      istart=1
      irotat=1
      ioutp=0
      itime=0
      call start(istart,irotat,ioutp,itime)
      ichois=0
      call mesh(ichois,iinput,rinput,kmesh)
c
      ichois=0
      call probdf(ichois,kprob,kmesh,iinput)
      jmetod=2
      call commat(jmetod,kmesh,kprob,intmat)
c
c ********************************************************************
c
c   *  fill boundary conditions
c
c ********************************************************************
c
c
c   *   first set all boundary conditions equal to zero
c
      idim=0
      call bvalue(ichois,idim,kmesh,kprob,isol,value,icrv1,icrv2,idegfd,
     v            jstap)
c
c   *   next fill boundary condition at inflow
c
      ichois=0
      idim=2
      jstap=0
      icrv1=6
      icrv2=6
      idegfd=2
      value=-1d0
      call bvalue(ichois,idim,kmesh,kprob,isol,value,icrv1,icrv2,idegfd,
     v            jstap)
c
c ********************************************************************
c
c   *    solve the system of non-linear equations
c
c ********************************************************************
c
      itertp=1
      maxitr=30
      eps=1d-4
      iprint=1
      ichois=-1
      call nonlin(itertp,maxitr,eps,iprint,iwork,work,intmat,ichois,
     v            iuser,user,kmesh,kprob,isol,ielhlp)
c
c*  proceed with post-processing of the solution
c
      call prinrv(isol,kmesh,kprob,4,0,'velocity')
c
c*********************************************************************
c
c*   compute the pressure
c
c*********************************************************************
c
c     iuser,user as required by nonlin (is still present)
c
      icheld=1
      call deriva(ichois,icheld,ix,jdegfd,ivec,ipress,kmesh,kprob,
     v            isol,isol,iuser,user,ielhlp)
      call prinrv(ipress,kmesh,kprob,4,0,' pressure')
c
c*********************************************************************
c
c*   compute the stream function  , set strm(1)=0.0d0
c
c*********************************************************************
      ichels=1
      jpoint=1
      strm1=0.0d0
      call stream(ichels,ivec,istrm,jpoint,strm1,kmesh,kprob,isol)
c******************************************************************
c
c**   plot velocity vectors
c
c******************************************************************
      facvec=0d0
      call plotvc(1,2,isol,isol,kmesh,kprob,20d0,1d0,facvec)
c******************************************************************
c
c*  plot contour lines for the pressure and the stream function
c
c******************************************************************
      ncntln=0
      number=1
      call plotc1(number,kmesh,kprob,ipress,contln,ncntln,
     v         20d0,1d0,0)
      call plotc1(number,kmesh,kprob,istrm,contln,ncntln,
     v         20d0,1d0,0)
      istop=1
      print *,'END PROGRAM NAVSTOK2'
      call finish(istop)
      end
      subroutine filnln(niter,iuser,user,kmesh,kprob,isol,relax,differ)
      implicit double precision (a-h,o-z)
      integer :: iuser(*),kmesh,kprob,isol,iwork(10)
      double precision :: work(10),user(*)
c
c ********************************************************************
c
c   *  fluid physical properties:
c
c      2: density
c      8: viscosity
c      9: power-law constant
c
c   *  other values of work and iwork:
c
c      1: penalty function parameter
c      3: type of liniarization of convective terms
c      4: angular velocity of rotating system
c      5: body force in x-direction
c      6: body force in y-direction
c      7: type of constitutive equation (2 = power-law)
c
c ********************************************************************
c
      relax=1.2d0
      do 100 i=1,9
         iwork(i)=0
100      work(i)=0d0
      iwork(3)=min(2,niter)
      iwork(7)=2
      work(1)=1d-6
      work(2)=1
      work(8)=0.1d0
      work(9)=0.5d0
      iparm=9
      call fil100(1,iuser,user,kprob,iparm,iwork,work)
      end
