      subroutine prcompidir ( ibuffr, buffer, isol, kprob, kmesh, idir )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 06-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute direction of axis (1=x, 2=y)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direction
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5), kprob(*), kmesh(*), idir
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     idir           o    direction of axis (1=x, 2=y)
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsc, ipcoor
      integer ndim, iprob, ipkprb, istart, ielgrp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ielgrp         first element group number with type number # 0
!     ipcoor         starting address of co-ordinates array
!     ipkmsc         Starting address of array KMESH part c
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     iprob          Actual problem number
!     istart         First position used at a part
!     ndim           Dimension of the space.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext
      integer prcompidir1, iniprb, inifirstelgrp, inistelgkmeshc

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIFIRSTELGRP  Find first element group that has a non-zero type number
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INIPRB         Get value of ipkprb as function of iprob
!     INISTELGKMESHC Find starting address of element group in kmeshc
!     PRCOMPIDIR1    Return with direction of element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find first element group with type number>0
!     Find starting address of element in kmeshc
!     Activate arrays
!     Compute starting addresses
!     Find direction
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompidir' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompidir'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      idir = 1            ! default direction
      ndim = kmesh(6)     ! dimension of space
      ielgrp = 1          ! default element group
      if ( ndim==1 ) go to 1000

!     --- Find first element group with type number>0

      iprob = isol(4)/1000+1  ! problem number
      ipkprb = iniprb ( iprob, kprob )+1     ! starting address in kprob
      ielgrp = inifirstelgrp ( kprob(ipkprb) )  ! element group
      if ( debug ) &
         write(irefwr,1) 'iprob, ipkprb, ielgrp', iprob, ipkprb, ielgrp

!     --- Find starting address of element in kmeshc (minus 1)

      istart = inistelgkmeshc ( kmesh, ielgrp )-1
      if ( debug ) write(irefwr,1) 'istart', istart

!     --- Activate arrays

      call ini060text ( ibuffr, kmesh(17), 'kmeshc' )
      call ini060text ( ibuffr, kmesh(23), 'coor' )

!     --- Compute starting addresses

      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
      ipcoor = inidgttext ( kmesh(23), 'coor' )

!     --- Find direction

      idir = prcompidir1 ( buffer(ipcoor), ibuffr(ipkmsc+istart), &
                           ndim )

1000  call erclos ( 'prcompidir' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompidir'

      end if  ! ( debug )

      end
