      subroutine prcooruserp ( ibuffr, buffer, kmesh, iuserp, coorloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the coordinates in a given user point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     user_point
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), iuserp
      double precision coorloc(*), buffer(*)

!     buffer        i/o   Buffer array
!     coorloc        o    Array with the coordinates of iuserp
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iuserp         i    User point sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prnodeuserp

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOORNODE     Get the value of the coordinates in a given node
!     PRNODEUSERP    Return with nodal point number corresponding to user point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get node number corresponding to user point
!     Get coordinates in node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcooruserp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcooruserp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get node number corresponding to user point

      node = prnodeuserp ( ibuffr, kmesh, iuserp )
      if ( debug ) write(irefwr,1) 'node', node
      if ( ierror/=0 ) go to 1000

!     --- Get coordinates in node

      call prcoornode ( ibuffr, buffer, kmesh, node, coorloc )

1000  call erclos ( 'prcooruserp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcooruserp'

      end if  ! ( debug )

      end
