      subroutine prfindcurvnode ( ibuffr, inode, kmesh, numcurves, &
                                  curvesfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 06-04-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node inode is in one or more of the curves
!     and return with corresponding curve numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), inode, kmesh(*), numcurves, curvesfound(*)

!     curvesfound    o    Array with curve numbers in which inode can be found
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inode          i    Node sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     numcurves      o    Number of curves stored in array curvesfound
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsm, ipkmsq
      integer ncurvs
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmsm         Starting address of array KMESH part m
!     ipkmsq         Starting address of array KMESH part q
!     ncurvs         Number of curves in mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRFINDCURVNOD1 Check if node inode is in one or more of the curves
!                    and return with corresponding curve numbers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract constants
!     activate arrays
!     Compute starting addresses
!     Fill curvesfound
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindcurvnode' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindcurvnode'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      ncurvs = kmesh(11)  ! number of curves
      if ( ncurvs==0 ) go to 1000

!     --- activate arrays

      call ini060text ( ibuffr, kmesh(27), 'kmeshm' )
      if ( kmesh(31)>0 ) call ini060text ( ibuffr, kmesh(31), 'kmeshq' )

!     --- Compute starting addresses

      ipkmsm = inigettext ( kmesh(27), 'kmeshm' )
      ipkmsq = 1
      if ( kmesh(31)>0 ) ipkmsq = inigettext ( kmesh(31), 'kmeshq' )

!     --- Fill curvesfound

      call prfindcurvnod1 ( inode, ibuffr(ipkmsm), ncurvs, numcurves, &
                            curvesfound, ibuffr(ipkmsq), kmesh )

1000  call erclos ( 'prfindcurvnode' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindcurvnode'

      end if

      end
