      function prnode ( ibuffr, kmesh, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find node number as defined by inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     level_set
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer prnode, ibuffr(*), kmesh(*), inpsep(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         Positions of interest are
!                         6:   ipoint defines a sequence number
!                                     in case of level set the level_set no
!                         7:   ichoice defines how to find the node
!                              Possible values:
!                              1:  zero_level_set
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     prnode         o    Node number found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoice, ipoint
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ichoice        See inpsep(7)
!     ipoint         See inpsep(6)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer ininodezerlev

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ININODEZERLEV  Return with node at boundary defined by zero levelset
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prnode' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prnode'
         call prinin ( inpsep, 7, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      prnode = 0   ! initialization

!     --- Get parameters

      ichoice = inpsep(7)
      ipoint = inpsep(6)

      if ( debug ) write(irefwr,1) 'ichoice, ipoint', ichoice, ipoint

      select case (ichoice)

      case (20)

!     --- ichoice = 20, zero_level_set

         prnode = ininodezerlev ( ibuffr, kmesh, ipoint )

      case default

!     --- Option not yet implemented

         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )

      end select  ! case (ichoice)

1000  call erclos ( 'prnode' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prnode'

      end if  ! ( debug )

      end
