      subroutine prplotzerolev ( ibuffr, buffer, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 09-06-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot zero level set curve
!     It is assumed that the plot is already active and hence opened
!     The active color is used
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     level_set
!     plot
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipcoor, ipobstfaces, ipnmeshc
      integer mmobstfaces, mmnmeshc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipcoor         starting address of co-ordinates array
!     ipnmeshc       Starting address of array NMESHC
!     ipobstfaces    Starting address of array obstfaces
!     mmnmeshc       memory management sequence number of array nmeshc
!     mmobstfaces    memory management sequence number of array obstfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext, inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIACTMK       General memory management subroutine to activate or create
!                    a permanent array
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRPLOTZEROLEV1 Plot zero level set curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotzerolev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotzerolev'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kmesh(6)/=2 ) go to 1000  ! only plot in case of ndim = 2
      if ( kmesh(64)==0 .or. kmesh(52)==0 ) go to 1000   ! no level_set found

!     --- Activate or create arrays

      call iniactmk ( ibuffr, 5, 9, kmesh, kmesh, mmobstfaces )
      call iniactmk ( ibuffr, 2, 33, kmesh, kmesh, mmnmeshc )
      call ini060text ( ibuffr, kmesh(23), 'coor' )

!     --- Compute starting addresses

      ipcoor = inidgttext( kmesh(23), 'coor' )
      ipobstfaces = inigettext( mmobstfaces, 'obstfaces' )
      ipnmeshc = inigettext( mmnmeshc, 'nmeshc' )

      call prplotzerolev1 ( buffer(ipcoor), ibuffr(ipnmeshc), &
                            ibuffr(ipobstfaces) )

1000  call erclos ( 'prplotzerolev' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotzerolev'

      end if  ! ( debug )

      end
