      subroutine prroot ( x, x1, x2, func1, func2, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find coordinates where function value is equal to 0
!     function value on end points is given
!     It is necessary that a root exists, hence func1 and func2 should be
!     at both sides of 0
! **********************************************************************
!
!                       KEYWORDS
!
!     root
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision x(ndim), x1(ndim), x2(ndim), func1, func2

!     func1          i    Function value at first point
!     func2          i    Function value at second point
!     ndim           i    Dimension of the space.
!     x              i    co-ordinates of root point
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision delta
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     delta          difference f2-f1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prroot' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prroot'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      delta = 1d0/(func2-func1)
      x = delta*(x1*func2-x2*func1)

1000  call erclos ( 'prroot' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prroot'

      end if  ! ( debug )

      end
