      subroutine read74part2 ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read second part of file sepcomp.out
!     The contents are not stored
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmachnf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum, ipost(1:2), numvec, i, ios
      double precision rdum
      character (len=80) textloc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     idum           General dummy variable
!     ios            IOSTAT variable, returns from reading or writing
!     ipost          Integer array containing information required for
!                    seppost
!                    See the Programmers Guide for a description
!     numvec         Number of vectors stored in isol
!     rdum           dummy variable
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read74part2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read74part2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iref74>0 ) then

!     --- formatted

         read(iref74,100, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
100      format(a)
         read ( iref74,*, iostat=ios ) numvec, (ipost(i),i=1,2)
         if ( ios/=0 ) go to 1000

!        --- Read names of arrays

         if ( numvec>0 ) then
            read(iref74,100, iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( iref74,110, iostat=ios ) (textloc,i=1,numvec)
110         format(a32)
            if ( ios/=0 ) go to 1000
         end if

!        --- Read ipost/rpost

         if ( ipost(1)>0 ) then
            read(iref74,100, iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( iref74,*, iostat=ios ) ipost(1:2), &
                 (idum,i=1,ipost(1))
            if ( ios/=0 ) go to 1000
         end if
         if ( ipost(2)>0 ) then
            read(iref74,100,iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( iref74,*, iostat=ios ) (rdum,i=1,ipost(2))
            if ( ios/=0 ) go to 1000
         end if

      else

!     --- unformatted

         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read ( -iref74, iostat=ios ) numvec, (ipost(i),i=1,2)
         if ( ios/=0 ) go to 1000
         if ( numvec>0 ) read(-iref74, iostat=ios) textloc

!        --- Read names of arrays

         do i = 1, numvec
            read ( -iref74,iostat=ios ) textloc
            if ( ios/=0 ) go to 1000
         end do

!        --- Read ipost/rpost

         if ( ipost(1)>0 ) then
            read(-iref74, iostat=ios) textloc
            read ( -iref74, iostat=ios ) ipost(1:2), &
               (idum,i=1,ipost(1))
            if ( ios/=0 ) go to 1000
            if ( ios/=0 ) go to 1000
         end if
         if ( ipost(2)>0 ) then
            read(-iref74, iostat=ios) textloc
            if ( ios/=0 ) go to 1000
            read ( -iref74, iostat=ios ) (rdum,i=1,ipost(2))
            if ( ios/=0 ) go to 1000
         end if

      end if

1000  call erclos ( 'read74part2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read74part2'

      end if  ! ( debug )

      end
