      subroutine readcd
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 29-04-2013 Extra debug
!        version  2.1  date 18-11-2005 Fortran 90
!        version  2.0  date 02-01-2003 Complete revision
!
!   copyright (c) 1990-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine READCD is used to read a record from the standard
!     input file
!     READCD replaces the old subroutine ASKCRD
!     READCD has the following characteristics:
!     Blank records are skipped
!     Records starting with a * in column 1 are skipped
!     Only information stored before the comment sign # is stored in the
!     string cards
!     The input is stored in cards, with trailing blanks excluded
!     The input is moved to the end of cards, hence cards(maxcolum) contains the
!     last non-blank character before the comments
!     The following special commands are recognised and executed immediately
!       set warn on/off
!       set max warn = &
!       set max error = &
!       set output on/off/out  or level=-1/0/1/2/3
!       set time on/off
!       set skip on/off
!       inlcude 'file_name'
!       All letters are translated to upper case, except those between
!       begin and end quotes
!       However, if ispecl=3 at input, no translation takes place
!       Furthermore with respect to a blank card the subroutine reacts as
!       in the case ispecl=1
!       If 0<ispecl<4, the following special actions are carried out:
!       If ispecl=1 at input, the special situation for the
!       program a111 is present
!       In that case the subroutine reacts on a blank card by
!       setting ispecl = 2 and returning to the calling subroutine
!       Furthermore the subroutine checks the presence of the
!       command stop.
!       If stop is found, subroutine instop is called (stop!)
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/csepchnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical exec, debug
      integer ninclude, irefresav, iretur
      save ninclude, irefresav

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     exec           If true input must be read otherwise skipped
!     irefresav      Help parameter to save the original value of irefre
!     iretur         Return value of subroutine
!                    Possible values:
!                    0:  keyword is recognized, read next record
!                    1:  keyword is not recognized, continue
!     ninclude       Level of include files
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     PRININ         print 1d integer array
!     READINCLUDE    Read include record
!     READRECCONTS   Interpret each item stored in cardimage and store
!                    information in common cseprdnew
!     READRECORD     Read a record
!     READSET        Read set record
! **********************************************************************
!
!                       I/O
!
!    Records are read from the input file
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     131   End of file found
!    1331   Error in reading record
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ready := false
!   exec := true
!   while ( not ready ) do
!      cards := ' '; cardim := ' '
!      irecnr := irecnr+1
!      end do := true
!      Read record into cardim
!      If (end of record reached) then
!         if ( ninclude>0 ) then
!            close file
!            decrease ninclude
!            read next record
!         ityprd := 0
!         ready := true
!         if ( istop = 0 ) stop
!      else
!         lastp := last non-blank in cardim
!         last  := last non-blank before comment character #
!         icolum := first non-blank character
!         if ( first character = * ) then
!         else if ( record is blank ) then
!            if ( ispecl=1 or ispecl=3 or ispecl = 7 ) then
!               ispecl := 2
!               ready := true
!         else
!            if ( ispecl = 3 or ispecl = 7 ) then
!               Copy cardim into cards
!            else
!               Copy Uppercase(cardim) into cards
!               Translate tab into space
!            if ( ispecl = 1 and first word = stop ) stop
!            if ( first word = include ) then
!               if ( file name between quotes found ) then
!                  raise ninclude
!                  open new file
!                  read next record
!            if ( first word = set ) then
!               if ( next word = warn ) then
!                  if ( next word = on or off ) then
!                     set nwarn; end do := false
!               else if ( next word = time ) then
!                  if ( next word = on or off ) then
!                     set itime; end do := false
!               else if ( next word = max ) then
!                  read next text and number
!                  set mxwarn or maxerror; end do := false
!               else if ( next word = output) then
!                  if ( next word = on or off or out ) then
!                     set ioutp; end do := false
!                  else if ( next word = level ) then
!                     read level; set ioutp; end do := false
!               else if ( next word = skip ) then
!                  if ( next word = on or off ) then
!                     set exec; end do := false
!            If ( end do and exec ) then
!               Reset all texts between quotes or after FILE = &
!               move information to end of cards
!               print cards if necessary
!               ready := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ninclude /0/
! ======================================================================
!
      call eropen ( 'readcd' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcd'

      end if

      exec = .true.

!     --- Read record

100   call readrecord (iretur )
      if ( iretur>0 ) then

!     --- Error in reading, stop

         if ( istop==3 ) go to 100
         if ( ispecl==6 .or. ispecl==7 ) then

!        --- ispecl = 6 or 7, reading from the environment file

            write(*,111)
            write(*,114) iretur
            write(*,113)
114         format(  ' A read error has occured', &
                     ' The iostat value is',i5 )
            ierror = 1
            go to 1000

         else

            call errint ( irecnr, 1 )
            call errint ( iretur, 2 )
            call errsub ( 1331, 2, 0, 0 )
            go to 1000

         end if

      else if ( iretur<0 ) then

!     --- End of file reached

         if ( istop==3 ) go to 100

         if ( ninclude>0 ) then

!        --- ninclude>0, close file and decrease ninclude

            ninclude = ninclude-1
            close ( unit=irefre )
            if ( ninclude==0 ) then

!           --- ninclude = 0, return to original file

               irefre = irefresav

            else

!           --- ninclude>0, return to previous file

               irefre = irefre-1

            end if
            go to 100

         end if
         ityprd = 0

         if ( ispecl==6 .or. ispecl==7 ) then

!        --- ispecl = 6 or 7, reading from the environment file

            write(*,111)
            write(*,112)
            write(*,113)
111         format(//' The sepran.env file is incorrect')
112         format(  ' end of file has been reached whereas more', &
                     ' input is expected')
113         format(  ' Please check your environment file'// &
                     ' Execution is terminated' )
            ierror = 1
            go to 1000

         else if ( istop==0 ) then

!        --- istop = 0, stop the process

            call errint ( irecnr, 1 )
            call errsub ( 131, 1, 0, 0 )
            go to 1000

         end if

      else

!     --- Standard case: interpret the contents of the record

         call readrecconts

!        --- If first item is string

         if ( typeitems(1)==3 ) then

!        --- First item is string, replace by uppercase and store in string

            firstcol = firstcolitems(1)
            lastcol = lastcolitems(1)
            numrecord = firstcol/maxcolum+1
            if ( numrecord>1 ) then

!           --- There are more than one records

               firstcol = mod(firstcol,maxcolum)
               lastcol = mod(lastcol,maxcolum)

            end if
            string = cardimage(numrecord)(firstcol:lastcol)
            nchar = lastcol-firstcol+1
            call askupc(string(1:nchar))

!           --- Check on presence of special records

            if ( string(1:nchar)=='INCLUDE' ) then

!           --- Include record found, check remaining text, it must be a text
!               between quotes

               call readinclude ( ninclude, irefresav )
               if ( ierror==0 ) go to 100

            end if

            if ( nitems>=3 .and. string(1:nchar)=='SET' ) then

!           --- Set record found, check remaining text

               call readset ( iretur, exec )
               if ( iretur==0 ) go to 100

            end if

         end if

!        --- Read next record if records must be skipped

         if ( .not. exec ) go to 100

      end if
      if ( iretur==0 .and. ioutp>=0 ) then

!     --- write last record

         lastcol = lastcolumn(numrecstored)
         write(irefwr,200) cardimage(numrecstored)(1:lastp)
200      format(1x,a)

      end if

1000  if ( ierror/=0 ) call instop
      lastitem = 0

      if ( debug ) then

!     --- Debug information

         call prinin ( typeitems,nitems,'typeitems')

      end if
      call erclos ( 'readcd' )
      if ( debug ) write(irefwr,*) 'End readcd'

      end
