      subroutine readexpress5 ( iwork, lenwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove superfluous openings and closings brackets
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), lenwork

!     iwork         i/o   integer work array, see subroutine readexpressnew
!     lenwork       i/o   Length of array work
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, levels(lenwork), ilevel, ipositions(2,lenwork), &
              imark(lenwork), icount, istore(lenwork)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     icount         Counter
!     ilevel         Level number of present set of brackets
!     imark          If a set of brackets in ipositions must be removed 1, &
!                    otherwise 0
!     ipositions     Indicates the positions of openings and closings
!                    brackets in iwork
!                    ipositions(1,i) position of i-th open bracket
!                    ipositions(2,i) position of corresponding close bracket
!     istore         Work array for removing brackets
!     levels         Contains addresses of open bracket of present level of
!                    brackets
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexpress5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexpress5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0
      ilevel = 0
      do i = 1, lenwork

         if ( iwork(i)==-3 ) then

!        --- Open bracket found

            icount = icount+1
            ipositions(1,icount) = i
            ilevel = ilevel+1
            levels(ilevel) = icount

         else if ( iwork(i)==-4 ) then

!        --- Close bracket found

            if ( ilevel>0 ) ipositions(2,levels(ilevel)) = i
            ilevel = ilevel-1

         end if  ! ( iwork(i)==-3 )
         if ( ierror/=0 ) go to 1000

      end do  ! i = 1, lenwork
      if ( debug ) call prinin1 ( ipositions, icount, 2, 'ipositions' )

!     --- Mark open and close brackets to be removed

      ilevel = 0
      if ( ipositions(1,1)==1 .and. ipositions(2,1)==lenwork ) then

!     --- open bracket in first position, close bracket in last position
!         superfluous, mark in imark

         ilevel = 1
         imark(1) = 1

      end if  ! ( ipositions(1,1)==1 .and. ipositions(2,1)==lenwork )

      do i = 2, icount

         if ( ipositions(1,i)==ipositions(1,i-1)+1 .and. &
              ipositions(2,i)==ipositions(2,i-1)-1 ) then

!        --- Open bracket follows open bracket, and corresponding closing
!            brackets also follow each other, hence double set
!            Marj set as to be removed

            ilevel = ilevel+1
            imark(ilevel) = i

         end if  ! ( ipositions(1,i)==ipositions(1,i-1)+1 ... )

      end do  ! i = 2, icount
      if ( debug ) call prinin ( imark, ilevel, 'imark' )

!     --- Remove brackets

      if ( ilevel>0 ) then

!     --- ilevel>0, hence there are brackets to be removed
!         Store the positions in istore

         istore = 0
         do i = 1, ilevel
            istore(ipositions(1,imark(i))) = 1
            istore(ipositions(2,imark(i))) = 1
         end do  ! i = 1, imark

!        --- Remove brackets

         icount = 0
         do i = 1, lenwork
            if ( istore(i)==0 ) then
               icount = icount+1
               iwork(icount) = iwork(i)
            end if  ! ( istore(i)==0 )
         end do  ! i = 1, lenwork
         lenwork = icount

      end if  ! (ilevel >0 )

1000  call erclos ( 'readexpress5' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readexpress5'

      end if  ! ( debug )
      icount = ilevel

      end
