      subroutine readname1 ( minusg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-04-2013 Extra debug
!        version  1.2  date 02-04-2008 Extension of cseprdnew
!        version  1.1  date 22-11-2004 Fortran 90
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read the name of a constant behind the $ or % sign and set
!     The name of the constant may contain digits, letters or an underscore
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer minusg

!     minusg         o    Indicates if a minus sign has been found (-1) or
!                         not (1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iitem, ispeclsav
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iitem          Local sequence number of item
!     ispeclsav      Help parameter to save the value of ispecl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READWR         Read keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1607   No name found after $ or %
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readname1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readname1'
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,1) 'ispecl', ispecl
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         call prinin ( typeitems, nitems, 'typeitems' )
         call prinin ( firstcolitems, nitems, 'firstcolitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      firstcol = firstcolitems(lastitem)
      numrecord = firstcol/240+1
      firstcol = mod(firstcol,240)+1

      if ( debug ) &
         write(irefwr,1) 'firstcol, numrecord', firstcol, numrecord

      minusg = 1

      if ( (ispecl<12 .or. ispecl>13) .and. lastitem>1 ) then

!     --- Last item>1, check if previous item on the same record is - &

         iitem = lastitem-1
         icolum = firstcolitems(iitem)
         if ( icolum/240+1==numrecord .and. typeitems(iitem)==11 ) &
            minusg = -1

      end if

      iitem = min(lastitem+1,nitems)

      if ( debug ) &
         write(irefwr,1) 'iitem, lastitem, icolum', &
                          iitem, lastitem, icolum

      if ( lastitem==nitems .or. typeitems(iitem)/=3 ) then

!     --- %-sign not followed by a name

         lastitem = iitem
         call errsub ( 1607, 0, 0, 0 )
         call errcrd
         go to 1000

      end if
      icolum = firstcolitems(iitem)

      if ( debug ) write(irefwr,1) 'icolum', icolum

      if ( icolum>240*numrecord .or. mod(icolum,240)/=firstcol ) &
         then

!     --- Next item on new record or not immediately after previous one

         lastitem = iitem
         call errsub ( 1607, 0, 0, 0 )
         call errcrd
         go to 1000

      end if

!     --- Read word ( consisting of letters and digits )

      lastitem = lastitem+1
      ispeclsav = ispecl
      ispecl = 9
      call readwr
      ispecl = ispeclsav

1000  call erclos ( 'readname1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readname1'

      end if  ! ( debug )

      end
