      subroutine readnm
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 15-03-2014 Debug statements
!        version  2.1  date 02-04-2008 Fortran 90
!        version  2.0  date 01-01-2003 Complete revision
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read one number from the input file
!       This subroutine supposes that the parameter ICOLUM indicates the
!       first position of the number. This first position may be a
!       digit or one of the following signs:  . + - &
!       The number is read free format
!       It is indicated if the number is real or integer
!        a number is defined by:
!         start:  + - . or digit
!         end:    non digit (except e or d after real), or end of record
!         in between digits . e d + and - are permitted
!       After reading the number, icolum points to the next position to be read
!       If the number is incorrect, realrd gets the value 0
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readnm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnm'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      realrd = realitems(lastitem)
      if ( debug ) write(irefwr,2) 'realrd', realrd

      if ( typeitems(lastitem)==1 ) then

!     --- Number is integer

         ityprd = 2

      else

!     --- Number is real

         ityprd = 3

      end if
      if ( (ispecl<12. .or. ispecl>13) .and. lastitem>1 ) then

!     --- present item is 2, check if previous item is a - immediately
!         before the number

         if  ( typeitems(lastitem-1)==11 ) then

!        --- previous item = -, check columns

            if ( firstcolitems(lastitem-1)+1== &
                 firstcolitems(lastitem) ) realrd = -realrd

         end if

      end if

1000  call erclos ( 'readnm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,*) 'End readnm'

      end if  ! ( debug )

      end
