      subroutine readvecrhs ( vectorfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-01-2015 Extension with compute_pressure
!        version  1.2  date 27-11-2014 Extension with a number of options
!        version  1.1  date 10-04-2014 Extension with stream function
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if right-hand side of equals sign corresponds to
!     a known function that produces a vector
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: vectorfound

!     vectorfound   i/o   If true the right-hand side contains a vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode = 10
      integer :: ncode(nncode), ncodmx(nncode), i
      character (len=16) :: code(nncode)
      logical :: debug

!     code           Table with keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncodmx         Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'INNER_PRODUCT', 'DERIVATIVES', 'STREAM_FUNCTION', &
                  'GRADIENT', 'STRESS_VECTOR', 'RIGHT_HAND_SIDE', &
                  'STRESS_TENSOR', 'FLUX', 'COMPUTE_PRESSURE', 'CURL' /
      data ncode  /  5,  5,  6, 4, 8,   5, 8, 4,12, 4 /
      data ncodmx / 13, 11, 15, 8,13,  15,13, 4,16, 4 /

!     code   Table with keywords corresponding to options of COMPUTE_VECTOR
!             Values:
!             1:  INNER_PRODUCT
!             2:  DERIVATIVES
!             3:  STREAM_FUNCTION
!             4:  GRADIENT
!             5:  STRESS_VECTOR
!             6:  RIGHT_HAND_SIDE
!             7:  STRESS_TENSOR
!             8:  FLUX
!             9:  COMPUTE_PRESSURE
!            10:  CURL
! ======================================================================
!
      call eropen ( 'readvecrhs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readvecrhs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( typeitems(lastitem)==3 ) then

!     --- string found, &
!         check if it is one of the known functions

         if ( debug ) then
            write(irefwr,1) 'ityprd, nchar', ityprd, nchar
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         do i = 1, nncode

            if ( nchar>=ncode(i) .and. nchar<=ncodmx(i) ) then

!           --- Number of characters in correct range

               if ( string(1:nchar)==code(i)(1:nchar) ) then

!              --- string matches keyword

                  vectorfound = .true.
                  exit

               end if  ! ( string(1:nchar)==code(i)(1:nchar) )

            end if  ! ( nchar>=ncode(i) .and. nchar<=ncodmx(i) )

         end do  ! i = 1, nncode

      end if  ! ( typeitems(lastitem)==3 )

1000  call erclos ( 'readvecrhs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'vectorfound ', vectorfound
         write(irefwr,*) 'End readvecrhs'

      end if  ! ( debug )

      end
