      subroutine msh478( coor, ia, jpoint, xm, ym, zm, &
                         e1, e2, e3, xc, yc, zc, radius )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 03-09-2010 New release
!        version  1.0  date 30-12-1992
!
!   copyright (c) 1992-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute coordinates centre of circumsphere of tetrahedron and radius
!     of the sphere
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     centre
!     radius
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), e1,e2,e3, xc, yc, zc, &
                       xm,ym,zm, radius
      integer ia, jpoint

!     coor         i  coordinate array
!     e1           i  first component of normal vector
!     e2           i  second component of normal vector
!     e3           i  third component of normal vector
!     ia           i  node of basis triangle
!     jpoint       i  node number of interest
!     radius       o  radius of circumsfere
!     xc           o  x-coordinate centre circumsphere
!     xm           i  x-component of centre of circumcircle of triangle
!     yc           o  y-coordinate centre circumsphere
!     ym           i  y-component of centre of circumcircle of triangle
!     zc           o  z-coordinate centre circumsphere
!     zm           i  z-component of centre of circumcircle of triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision coef, sumn, sumt, xi, xj, yi, yj, zi, zj

!     coef   coefficient
!     sumn   sum of nominater of radius
!     sumt   sum od denominater of radius
!     xi     x-component of point ia
!     xj     x-component of point jpoint
!     yi     y-component of point ia
!     yj     y-component of point jpoint
!     zi     z-component of point ia
!     zj     z-component of point jpoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS deconcatenate name from string of calling subroutines
!     EROPEN concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run along line given by e1,e2,e3 and determine point that has the
!     same distance to both ia and jpoint
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh478' )

!     --- Determine coordinates of point ia

      xi = coor(1,ia)
      yi = coor(2,ia)
      zi = coor(3,ia)

!     --- Determine coordinates of the point jpoint

      xj = coor(1,jpoint)
      yj = coor(2,jpoint)
      zj = coor(3,jpoint)

!     --- Determine the coefficient

      sumt = (xm-xi)*(xm-xi) - (xm-xj)*(xm-xj) + &
             (ym-yi)*(ym-yi) - (ym-yj)*(ym-yj) + &
             (zm-zi)*(zm-zi) - (zm-zj)*(zm-zj)
      sumn = 2*( (xi-xj)*e1 + (yi-yj)*e2 + (zi-zj)*e3 )
      coef = sumt / sumn

!     --- Compute coordinates centre

      xc = xm + coef * e1
      yc = ym + coef * e2
      zc = zm + coef * e3

!     --- Compute radius squared

      radius = (xc-xi)*(xc-xi) + (yc-yi)*(yc-yi) + (zc-zi)*(zc-zi)
      call erclos( 'msh478' )
      end
