      subroutine msh617( coor, jpn, npoint, itri, i1, i2, i3, &
                         xn, yn, zn, dista )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.0  date 14-04-2005 Update code
!        version  1.2  date 19-02-1993 New layout
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine nearest boundary point to proposed new point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itri(*), jpn, npoint, i1, i2, i3
      double precision coor(3,*), xn, yn, zn, dista

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     dista   o    distance new point to boundary point
!     i1      i    node of basis triangle
!     i2      i    node of basis triangle
!     i3      i    node of basis triangle
!     itri    i    array with for each point number of times
!                  that point appears in present boundary
!     jpn     o    node number of nearest point
!     npoint  i    Number of nodal points
!     xn      i    x-coordinate new point
!     yn      i    y-coordinate new point
!     zn      i    z-coordinate new point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision xi, yi, zi, dx, dy, dz, dist

!     dist    distance
!     dx      length of difference in x-direction
!     dy      length of difference in y-direction
!     dz      length of difference in z-direction
!     i       loop variable
!     xi      x-coordinate node i
!     yi      y-coordinate node i
!     zi      z-coordinate node i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      jpn = 0

      dista = dista * dista

      do i = 1 , npoint

!     --- Skip check if i is node of basis triangle

         if ( i/=i1 .and. i/=i2 .and. i/=i3 ) then

!        --- Check distance if node in boundary

            if ( itri(i)/=0 ) then
               xi = coor( 1, i )
               yi = coor( 2, i )
               zi = coor( 3, i )

               dx = xn - xi
               dy = yn - yi
               dz = zn - zi

               dist =  ( dx*dx + dy*dy + dz*dz )

!              --- Check distance

               if ( dist<(1d0 - 100 * epsmac ) * dista ) then
                  jpn   = i
                  dista = dist
               end if
            end if
         end if
      end do

      dista = sqrt ( dista )

      end
