      subroutine msh618( coor  , npoint, kelem , nelem , &
                         kstapl, kstap , itri  , &
                         i1, i2, i3, xnx, yny, znz)
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.0  date 14-04-2005 Update code
!        version  1.2  date 19-02-1993 New layout
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to fill arrays coor and kelem and to adjust arrays
!     kstapl and itri.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(4,*), kstapl(*), itri(*), npoint, nelem, kstap, &
              i1, i2, i3
      double precision coor(3,*), xnx, yny, znz

!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     i1      i    First point of triangle that forms basis
!     i2      i    Second point of triangle that forms basis
!     i3      i    Third point of triangle that forms basis
!     itri   i/o   array to indicate whether point is in boundary or not
!     kelem  i/o   New KMESH part c with respect to the surface elements
!                  All elements have the same number of nodes
!     kstap  i/o   number of elements in kstapl
!     kstapl i/o   present boundary elements
!     nelem  i/o   Number of elements
!     npoint i/o   Number of nodal points
!     xnx     i    x-coordinate of new point
!     yny     i    y-coordinate of new point
!     znz     i    z-coordinate of new point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ib

!     ib      Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill new node in array coor

      npoint = npoint + 1

!     --- Coordinates

      coor( 1, npoint ) = xnx
      coor( 2, npoint ) = yny
      coor( 3, npoint ) = znz

!     --- Fill array kelem

      nelem = nelem + 1

!     --- Node numbers

      kelem( 1, nelem ) = i1
      kelem( 2, nelem ) = i2
      kelem( 3, nelem ) = i3
      kelem( 4, nelem ) = npoint

!     --- Adjust kstapl and itri

      do ib = 1, 3*kstap-3
         kstapl(ib) = kstapl(ib+3)
      end do
      kstap = kstap + 2

!     --- Determine position to start

      ib = 3*kstap-9

!     --- Place new triangles

      kstapl( ib+1 ) = i1
      kstapl( ib+2 ) = i2
      kstapl( ib+3 ) = npoint
      kstapl( ib+4 ) = i2
      kstapl( ib+5 ) = i3
      kstapl( ib+6 ) = npoint
      kstapl( ib+7 ) = i3
      kstapl( ib+8 ) = i1
      kstapl( ib+9 ) = npoint

!     --- Extent array itri for new node

      itri(npoint) = 3

!     --- Adjust array itri for old nodes

      itri(i1)     = itri(i1) + 2
      itri(i2)     = itri(i2) + 2
      itri(i3)     = itri(i3) + 2

      end
