      subroutine msh622( kelem, nelem, istart, ibuur, npoint )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.5  date 29-04-2005 New norms
!        version  1.4  date 02-06-1992 initialize ibuur for 10 * npoint
!                                      positions
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine the neighbours of each node with node
!     number larger than its own nodenumber.
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!     none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kelem(4,nelem), istart(*), ibuur(*), npoint

!     ibuur          o    array with neighbours
!     istart         o    array containing the starting addresses of
!                         the neighbours in ibuur
!     kelem          i    array with elements (linear hexahedrons)
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, mtal, ntal

!     i           loop variable
!     i1          node
!     i2          node
!     i3          node
!     i4          node
!     mtal        helpvariable for number of neighbours
!     ntal        helpvariable for number of neighbours
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH623      determine approximate length of neighbour array
!     MSH624      place neighbour in array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear istart

      do i = 1, npoint
         istart(i) = 0
      end do

!     --- Clear ibuur

      do i = 1, 10 * npoint
          ibuur(i) = 0
      end do

!     --- Run through all elements and fill neighbours
!         in neighbour arrays

      do i = 1, nelem
          i1 = kelem( 1, i )
          i2 = kelem( 2, i )
          i3 = kelem( 3, i )
          i4 = kelem( 4, i )
          call msh623( i1, i2, istart, ibuur, 5 )
          call msh623( i1, i3, istart, ibuur, 5 )
          call msh623( i1, i4, istart, ibuur, 5 )
          call msh623( i2, i3, istart, ibuur, 5 )
          call msh623( i2, i4, istart, ibuur, 5 )
          call msh623( i3, i4, istart, ibuur, 5 )
      end do

!     --- Refill istart with new startpositions

      do i = 1, npoint - 1
         ntal = istart(i)
         if ( ntal<6 ) then
              mtal = 2 * ntal
         else
              mtal = 12 + 2 * ( (ntal - 5) / 2 )
         end if
         istart(i) = mtal
      end do
      mtal      = istart(1)
      istart(1) = 1
      do i = 2, npoint
         ntal      = istart(i)
         istart(i) = istart(i-1) + mtal
         mtal      = ntal
      end do

!     --- Final round, clear array ibuur again

      do i = 1, istart( npoint )
         ibuur(i) = 0
      end do
      do i = 1, nelem
         i1 = kelem( 1, i )
         i2 = kelem( 2, i )
         i3 = kelem( 3, i )
         i4 = kelem( 4, i )
         call msh624( i1, i2, istart, ibuur )
         call msh624( i1, i3, istart, ibuur )
         call msh624( i1, i4, istart, ibuur )
         call msh624( i2, i3, istart, ibuur )
         call msh624( i2, i4, istart, ibuur )
         call msh624( i3, i4, istart, ibuur )
      end do
      end
