      subroutine msh634( cube, ncube, refvol, refval)
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 08-09-2010 New release
!        version  2.0  date 15-04-2005 Update code
!        version  1.3  date 25-02-1997 Minor reorganisation;
!                                      Smaller value refval
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to calculate a minimal reference volume value.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tesselation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncube
      double precision cube(ncube), refvol(*), refval

!     cube      i     array containing the coarsenesses in the
!                     blocks
!     ncube     i     number of blocks
!     refval    o     overall minimal volume value
!     refvol    o     minimal volume value allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision eps, val

!     eps     local accuracy parameter
!     i       loop variable
!     val     local distance value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     None
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
!     --- Set accuracy

      eps    =  100 * epsmac

!     --- Reference value infinity

      refval =  rinfin

!     --- Run through all cubes

      do i = 1 , ncube

!     --- Determine local coarseness

         val = cube(i)

!        --- Local volume value

         refvol(i) = 0.08 * val * val * val
         if ( val>eps) then
            if ( refvol(i)<refval ) refval = refvol(i)
         end if
      end do

!     --- Take small reference value

      do i = 1, ncube
         val = cube(i)
         if ( val<=eps ) refvol(i) = refval
      end do

      end
