      subroutine msh637 ( kstapl, kstap, ide, irand )
! ======================================================================
!
!        programmer    niek praagman
!        version  4.0  date 08-09-2010 New release
!        version  3.0  date 03-09-2009 Adjust for extra internal surfaces
!        version  2.3  date 23-02-2005 New norms
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether array kstapl constitutes
!     a (set of) closed volume(s).
!     Each line of kstapl is checked: all lines should be double.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstap, ide(kstap), irand(2,*), kstapl(3,*)

!     ide      o     indicator whether triangle i belongs to volume
!                    ide(i) = 0 or not ide(i) = 1
!     irand          helparray
!     kstap    i     number of triangles in kstapl
!     kstapl   i     array containing the boundary triangles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, ia, iaan, ib, ic, ichan, id, idrie, ir, &
              isit, ital, j1, jtal, j2
      logical ready

!     debug   if true debug statements are carried out otherwise
!     i       loop variable
!     i1      node number
!     i2      node number
!     i3      node number
!     ia      node number
!     iaan    helpvariable
!     ib      node number
!     ic      node number
!     ichan   change indicator
!     id      node number
!     idrie   number of triangle
!     ir      number of lines in array irand
!     isit    indicator for situation
!     ital    count how many times node number appears in array irand
!     j1      node number
!     j2      node number
!     jtal    helpvariable for counting lines in array irand
!     ready   logical to indicate whether volume is closed or not yet
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     931   Line appears only once in boundary
!     932   More than five chancings in order
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array ide which indicates which triangles constitute the
!     volume
!     Fill boundary array irand with the contents of the first triangle
!     Loop until ready
!     Look for another triangle with first line of irand
!     If triangle found place contents in irand and cancel double ones
!     If irand not empty not ready
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
!     --- Subroutine to determine which subset of array kstapl constitutes
!         a closed volume

      call eropen ( 'msh637' )

!     --- Set debug

      debug = .false.

!     --- Initialize array ide (all triangles in boundary)

      do i = 1 , kstap
          ide(i) = 1
      end do

!     --- Fill lines of first triangle in irand:

      ia = kstapl(1,1)
      ib = kstapl(2,1)
      ic = kstapl(3,1)

!     --- Set triangle visited:

      ide (1) = 0

!     --- Fill irand

      irand(1,1) = ia
      irand(2,1) = ib
      irand(1,2) = ib
      irand(2,2) = ic
      irand(1,3) = ic
      irand(2,3) = ia

!     --- Number of lines and changes:

      ir    = 3
      ichan = 0

!     --- Goto loop

      ready = .false.

150   if ( .not. ready ) then

!     --- Take first line from irand

         ia = irand(1,1)
         ib = irand(2,1)

!        --- Look for "opposite" line ib - ia
!            Initiate:

         idrie = 0
         isit  = 0
         ital  = 0

         do i = 1 , kstap
            if ( ide(i)==1 .and. ital==0 ) then

!           --- New triangle (not yet added):

               i1 = kstapl(1,i)
               i2 = kstapl(2,i)
               i3 = kstapl(3,i)

               if ( ia==i1 .and. ib==i3 ) then
                  isit  = 1
                  idrie = i
                  ital  = ital + 1
               else if ( ia==i2 .and. ib==i1 ) then
                  isit  = 2
                  idrie = i
                  ital  = ital + 1
               else if ( ia==i3 .and. ib==i2 ) then
                  isit  = 3
                  idrie = i
                  ital  = ital + 1
               end if
            end if
         end do

         if ( ital==0 ) then

!        --- Error: line not found!

            write(irefwr,*) 'Error: line',ia,'- ',ib,'not found twice!'
            call errsub ( 931, 0, 0, 0 )
            call instop
         else if ( ital==1 ) then

!        --- Corresponding line found: add new pieces
!            to array irand and check results

            ichan      = 0
            ide(idrie) = 0

            i1 = kstapl(1,idrie)
            i2 = kstapl(2,idrie)
            i3 = kstapl(3,idrie)

            do i=1,ir-1
                irand(1,i) = irand(1,i+1)
                irand(2,i) = irand(2,i+1)
            end do

            if ( isit==1 ) then
               ia = i1
               ib = i2
               ic = i2
               id = i3
            else if ( isit==2 ) then
               ia = i2
               ib = i3
               ic = i3
               id = i1
            else if ( isit==3 ) then
               ia = i1
               ib = i2
               ic = i3
               id = i1
            end if

!           --- Check  whether ia - ib already belongs to irand

            jtal = 0
            iaan = 0

            do i = 1, ir-1
               j1 = irand(1,i)
               j2 = irand(2,i)
               if ( iaan==0 .and. j1==ib .and. j2==ia ) then
                  iaan = 1
               else
                  jtal = jtal + 1
                  irand(1,jtal) = j1
                  irand(2,jtal) = j2
               end if
            end do

            if ( max(2*ir-1,2*jtal)>kstap+10 ) then

!           --- estimate of  too short

               call errint ( max(2*ir-1,2*jtal), 1 )
               call errint ( kstap+10, 2 )
               call errchr ( 'irand', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000
            end if

            if ( iaan==0 ) then
                 jtal = jtal + 1
                 irand(1,jtal) = ia
                 irand(2,jtal) = ib
            end if

            ir   = jtal
            iaan = 0
            jtal = 0

            do i = 1, ir

               j1 = irand(1,i)
               j2 = irand(2,i)

               if ( iaan==0 .and. j1==id .and. j2==ic ) then
                  iaan = 1
               else
                  jtal = jtal + 1
                  irand(1,jtal) = j1
                  irand(2,jtal) = j2
               end if

            end do

            if ( iaan==0 ) then
               jtal = jtal + 1
               irand(1,jtal) = ic
               irand(2,jtal) = id
            end if

            if ( 2*jtal>kstap+10 ) then

!           --- used estimate too short

               call errint ( 2*jtal, 1 )
               call errint ( kstap+10, 2 )
               call errchr ( 'irand', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000
            end if
         else

!        --- Chance order of array irand

            ichan = ichan + 1
            if ( ichan>kstap ) then
               if ( debug ) then

!              --- Write contents array irand:

                  write(irefwr,*) 'Contents boundary array:'
                  write(irefwr,*) 'Length (kstap =) ',kstap
                  do i = 1, kstap
                     write(irefwr,*) i, irand(1,i),irand(2,i)
                  end do
               end if

!              --- Error

               call errint ( ichan, 1 )
               call errsub ( 932, 1, 0, 0 )
               call instop
            end if

!           --- Move tail of array

            do i = 1, ir - 1
                irand(1,i) = irand(1,i+1)
                irand(2,i) = irand(2,i+1)
            end do
            irand(1,ir) = ia
            irand(2,ir) = ib

            jtal = ir

            if ( 2*jtal>kstap+10 ) then

!           --- used estimate too short

               call errint ( 2*jtal, 1 )
               call errint ( kstap+10, 2 )
               call errchr ( 'irand', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000
            end if
         end if

         if ( jtal==0 ) then

!        --- Check all triangles:

            do i=1, kstap
               if ( ide(i)==1 .and. jtal==0 ) then

!              --- Not yet ready (new subvolume):

                  ia = kstapl(1,i)
                  ib = kstapl(2,i)
                  ic = kstapl(3,i)

                  ide (i) = 0

!                 --- Fill irand:

                  irand(1,1) = ia
                  irand(2,1) = ib
                  irand(1,2) = ib
                  irand(2,2) = ic
                  irand(1,3) = ic
                  irand(2,3) = ia

                  jtal  = 3
                  ir    = 3
                  ichan = 0

                    goto 150

               end if

            end do

         end if

         if ( jtal==0 ) ready = .true.

         ir = jtal

         goto 150
      end if

!     --- Check array ide:
!         (All triangles should be visited and hence involved)

      if ( debug ) then
         ir = 0

!        --- Check all:

         do i=1, kstap
              if ( ide(i)==1 ) then
               write(irefwr,*) 'Prob tri ',i,(kstapl(ir,i),ir=1,3)
               ir = 1
            end if
         end do

!        --- No non-empty triangle found

         if ( ir==0 ) then
              write(irefwr,*) 'Boundary is closed'
         else
              write(irefwr,*) 'Problem in msh637:Boundary is not closed'
              write(irefwr,*) 'Stop of execution. Check input'
              write(irefwr,*) 'If ok. Please inform SEPRA'
              call instop
         end if
      end if
1000  call erclos ( 'msh637' )
      end
