      subroutine msh640( a, b, c, ielem, jelem )
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 08-09-2010 New release
!        version  2.0  date 15-04-2005 Update code
!        version  1.0  date 11-01-1989 First release
!
!   copyright (c) 1989-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute all angles of triangle with sides a,b,c of tetrahedron
!     ielem and check against representative value
! **********************************************************************
!
!                       KEYWORDS
!
!     angle
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem, jelem
      double precision a, b, c

!     a              i    length of side of triangle
!     b              i    length of side of triangle
!     c              i    length of side of triangle
!     ielem          i    triangle number
!     jelem          o    if angle too large, jelem gets number of
!                         triangle considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision aa, bb, cc, q, angle1, angle2, angle3

!     aa          distance a squared
!     angle1      cosinevalue of first angle
!     angle2      cosinevalue of second angle
!     angle3      cosinevalue of third angle
!     bb          distance b squared
!     cc          distance c squared
!     q           boarder value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      aa = a*a
      bb = b*b
      cc = c*c

!     --- Compute cosine of each angle in the triangle

      angle1 = ( aa + bb - cc ) / ( 2 * a * b )
      angle2 = ( cc + aa - bb ) / ( 2 * c * a )
      angle3 = ( bb + cc - aa ) / ( 2 * b * c )

      q = -0.8d0

      if ( angle1<q .or. angle2<q .or. angle3<q ) jelem = ielem

      end
