      subroutine msh643( j1,j2,j3, istart, getal, basis, knoop )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.1  date 29-04-2005 new norms
!        version  1.0  date 17-04-1991
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine for triangle j1-j2-j3 its sequencenumber
!     and to place this number in array GETAL and to place a -1 in array
!     KNOOP.
!     If position has already been filled than only the value in array
!     KNOOP is decreased.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer j1, j2, j3, istart(*), knoop(*)
      double precision getal(*), basis

!     basis          i    Value on which the second numbers are based
!     getal         i/o   Array containing the the second numbers
!     istart         i    array containing the startpositions for the
!                         nodes and correspondent triangles
!                         Each triangle is uniquely determined by two
!                         values : 1-The smallest node number
!                                  2-The value j2*basis+j1
!     j1             i    sorted (j1>j2>j3) nodes of triangle to be considered
!     j2             i    see j1
!     j3             i    see j1
!     knoop         i/o   Array to store the number of appearances of each
!                         triangle (should be two times if also the
!                         boundary triangles are counted for
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ist, ien
      double precision val

!     ien            end position to consider for the present triangle
!     ist            Start position to consider for the present triangle
!     val            Val to be considered for this triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     938            Internal error in subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('msh643')

!     --- Determine startposition and value of this triangle

      ist = istart(j3)
      ien = istart(j3 + 1) - 1

!     --- Compute helpvalue

      val = j2 * basis + j1
100   continue

!     --- Check for error

      if ( ist>ien ) then

!     --- Error that should not occur

           call errsub ( 938, 0, 0, 0)
           go to 1000

      end if

      if ( knoop(ist)==0 ) then
           getal(ist) = val
           knoop(ist) = -1
      else if ( abs( getal(ist)-val )<1d-1 ) then
           knoop(ist) = knoop(ist) - 1
      else
           ist = ist + 1
           goto 100
      end if

1000  call erclos('msh643')

      end
