      subroutine msh644( j1, j2, j3, istart, getal, basis, ipoint )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.1  date 29-04-2005 new norms
!        version  1.0  date 17-04-1991
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine the relative number of face j1-j2-j3
!     in order to place a new node in the barycenter
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer j1, j2, j3, istart(*), ipoint
      double precision getal(*), basis

!     basis          i    helpvariable for second numbers
!     getal          i    array of second numbers triangles (see MSH643)
!     ipoint         o    relative sequence number of face
!     istart         i    array with starting positions
!     j1             i    sorted (j1>j2>j3) nodes of triangle to be considered
!     j2             i    see j1
!     j3             i    see j1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ist, ien
      double precision val

!     ien            end position to consider for the present triangle
!     ist            Start position to consider for the present triangle
!     val            Val to be considered for this triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     938   Internal error in subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('msh644')

!     --- Determine startposition and value of this triangle

      ist = istart(j3)
      ien = istart(j3 + 1) - 1

!     --- Value of triangle

      val = j2 * basis + j1
100   continue

!     --- Check

      if ( ist>ien ) then

!     --- Internal error that should not occur

           call errsub ( 938, 0, 0, 0)
           call instop
      end if

      if ( abs( getal(ist)-val )<1d-1 ) then
           ipoint = ist
      else
           ist = ist + 1
           goto 100
      end if

      call erclos('msh644')

      end
