      subroutine msh645( coor, i1, i2, i3, il )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.1  date 29-04-2005 new norms
!        version  1.0  date 17-04-1991
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to create and place the coordinates of the center- &
!     point of triangle i1,i2,i3 on position il (in 3D)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, il
      double precision coor(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points
!     i1             i    node of triangle
!     i2             i    node of triangle
!     i3             i    node of triangle
!     il             i    node of which coordinates have to be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1, x2, x3, y1, y2, y3, z1, z2, z3

!     x1             x-coordinate
!     x2             x-coordinate
!     x3             x-coordinate
!     y1             y-coordinate
!     y2             y-coordinate
!     y3             y-coordinate
!     z1             z-coordinate
!     z2             z-coordinate
!     z3             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      x1 = coor( 3 * i1 - 2 )
      x2 = coor( 3 * i2 - 2 )
      x3 = coor( 3 * i3 - 2 )

      y1 = coor( 3 * i1 - 1 )
      y2 = coor( 3 * i2 - 1 )
      y3 = coor( 3 * i3 - 1 )

      z1 = coor( 3 * i1     )
      z2 = coor( 3 * i2     )
      z3 = coor( 3 * i3     )

      coor( 3 * il - 2 ) = ( x1+x2+x3 ) / 3d0
      coor( 3 * il - 1 ) = ( y1+y2+y3 ) / 3d0
      coor( 3 * il     ) = ( z1+z2+z3 ) / 3d0

      end
