      subroutine msh646( kstapl, kstap, i1, i2, i3 )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.0  date 08-09-2010 New release
!        version  3.0  date 24-02-2005 New norms
!        version  2.0  date 29-04-1998 No update of itri, remove
!                                      itri from parameters (BvR)
!        version  1.0  date 05-01-1993
!
!   copyright (c) 1993-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to add new triangle to array KSTAPL or to cancel the
!     triangle if already present in KSTAPL.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     triangulation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstapl(3,*), kstap, i1, i2, i3

!     i1         i    first node of new triangle
!     i2         i    second node of new triangle
!     i3         i    third node of new triangle
!     kstap     i,o   number of triangles in kstapl
!     kstapl    i,o   array containing the triangles of the actual
!                     boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ib, ifound, j1, j2, j3, ja, jb, jc

!     ib        local loop variable
!     ifound    check variable whether triangle is already in kstapl
!     j1        node number of triangle
!     j2        node number of triangle
!     j3        node number of triangle
!     ja        node number of triangle
!     jb        node number of triangle
!     jc        node number of triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   deconcatenate name from string of calling subroutine
!     EROPEN   concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through kstapl
!       If ifound = 0 then
!          check triangle and set ifound to 1
!       else
!          replace triangle (take shift =-1)
!       end if
!     Check whether new triangle has to be added and if so add
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
      call eropen( 'msh646' )

!     --- Set not found

      ifound = 0

!     --- Run through kstapl

      do ib = 1, kstap

!     --- Find triangle

         j1 = kstapl( 1,ib )
         j2 = kstapl( 2,ib )
         j3 = kstapl( 3,ib )

!        --- Check if not yet found

         if ( ifound==0 ) then
              ja = 1
              if ( j1==i1.or.j1==i2.or.j1==i3 ) ja = 0
              jb = 1
              if ( j2==i1.or.j2==i2.or.j2==i3 ) jb = 0
              jc = 1
              if ( j3==i1.or.j3==i2.or.j3==i3 ) jc = 0

!             --- if found then exclude

              if ( ja+jb+jc==0 ) ifound = 1
          else

!         --- Keep next triangles and move

              kstapl( 1,ib-1 ) = j1
              kstapl( 2,ib-1 ) = j2
              kstapl( 3,ib-1 ) = j3
          end if
      end do

!     --- Check whether ready

      if ( ifound==1 ) then

!     --- Triangle has been removed

         kstap = kstap - 1
      else

!     --- Triangle not encountered, new hence add

         kstap = kstap + 1

         kstapl( 1, kstap ) = i1
         kstapl( 2, kstap ) = i2
         kstapl( 3, kstap ) = i3
      end if

      call erclos( 'msh646' )

      end
