      subroutine msh666 ( jel1, jel2, jel3, jel4, kelem, coor, &
                          i3, i4, ii1, ii2, i1, i2, isuccs )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 09-09-2010 New release
!        version  1.2  date 26-03-2008 Make sure that new choice is better
!        version  1.1  date 31-01-2008 Rearrangements
!        version  1.0  date 15-02-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether an alternative division of four coupled tetrahedrons
!     improves the overall ratio. If so store the new elements on the old
!     positions
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     adjustment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jel1, jel2, jel3, jel4, kelem(4,*), &
              i1, i2, i3, i4, ii1, ii2, isuccs
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     i1             i    point of original "flat" element
!     i2             i    point of original "flat" element
!     i3             i    point of original "flat" element
!     i4             i    point of original "flat" element
!     ii1            i    node number of extra edge extra element
!     ii2            i    node number of extra edge extra element
!     isuccs         o    indicator new elements (==1) or not (==0)
!     jel1           i    element position in array kelem to be refilled
!     jel2           i    element position in array kelem to be refilled
!     jel3           i    element position in array kelem to be refilled
!     jel4           i    element position in array kelem to be refilled
!     kelem         i/o   New KMESH part c with respect to the surface elements
!                         All elements have the same number of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision ratio, rata, ratb, rat1, rat2, rat3, rat4, &
                       vol, vol1, vol2

!     rat1           help ratio for division
!     rat2           help ratio for division
!     rat3           help ratio for division
!     rat4           help ratio for division
!     rata           smallest ratio of "group A"
!     ratb           smallest ratio of "group B"
!     ratio          ratio to be improved
!     vol            helpvariable to store volume
!     vol1           helpvariable to store first alternative volume
!     vol2           helpvariable to store second alternative volume
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSH632         compute volume of tetrahedron
!     MSH661         Compute ratio of inner and outer sphere of tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2959   Error in rata
!    2960   Error in ratb
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check three different divisions
!    Refill the element positions in a straightforward way
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh666' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh666'
      end if
      if ( ierror/=0 ) go to 1000

!     --- Assume positive result

      isuccs = 1

!     --- Compute old ratio

      call msh661( coor, i1, i2, i3, i4, ratio )

!     --- New tetrahedrons: check first best possibility

      call msh661( coor, i3,  i4, ii2, i2, rat1 )
      call msh661( coor, i3, ii2, ii1, i2, rat2 )
      call msh661( coor, i3, ii2,  i4, i1, rat3 )
      call msh661( coor, i3, ii1, ii2, i1, rat4 )

      rata = min( rat1, rat2, rat3, rat4 )

      call msh661( coor, i4, ii1,  i3, i2, rat1 )
      call msh661( coor, i4, ii2, ii1, i2, rat2 )
      call msh661( coor, i4,  i3, ii1, i1, rat3 )
      call msh661( coor, i4, ii1, ii2, i1, rat4 )

      ratb = min( rat1, rat2, rat3, rat4 )

      if ( debug ) then

!     --- Check all volumes
!         Old volume:

         call msh632( coor,  i1,  i2,  i3,  i4, vol )
         vol1 = vol
         call msh632( coor,  i1,  i3,  i2, ii1, vol )
         vol1 = vol + vol1
         call msh632( coor,  i1,  i2,  i4, ii2, vol )
         vol1 = vol + vol1
         call msh632( coor,  i1,  i2, ii2, ii1, vol )
         vol1 = vol + vol1

         call msh632( coor,  i3, ii2,  i4,  i1, vol )
         vol2 = vol
         call msh632( coor,  i3,  i4, ii2,  i2, vol )
         vol2 = vol + vol2
         call msh632( coor,  i3, ii1, ii2,  i1, vol )
         vol2 = vol + vol2
         call msh632( coor,  i3, ii2, ii1,  i2, vol )
         vol2 = vol + vol2

         if ( abs ( vol2 - vol1 )>1d-8 ) then
            call errsub ( 2959, 0, 0, 0 )
            go to 1000
         end if

         call msh632( coor,  i4,  i3, ii1,  i1, vol )
         vol2 = vol
         call msh632( coor,  i4, ii1,  i3,  i2, vol )
         vol2 = vol + vol2
         call msh632( coor,  i4, ii1, ii2,  i1, vol )
         vol2 = vol + vol2
         call msh632( coor,  i4, ii2, ii1,  i2, vol )
         vol2 = vol + vol2

         if ( abs ( vol2 - vol1 )>1D-8 ) then
            call errsub ( 2960, 0, 0, 0 )
            go to 1000
         end if

      end if

      if ( rata>ratio+1.d-8 .and. rata>=ratb ) then

!     --- Selection A is the best, place four
!         new tetrahedrons:

         kelem(1,jel1) = i3
         kelem(2,jel1) = i4
         kelem(3,jel1) = ii2
         kelem(4,jel1) = i2

         kelem(1,jel2) = i3
         kelem(2,jel2) = ii2
         kelem(3,jel2) = ii1
         kelem(4,jel2) = i2

         kelem(1,jel3) = i3
         kelem(2,jel3) = ii2
         kelem(3,jel3) = i4
         kelem(4,jel3) = i1

         kelem(1,jel4) = i3
         kelem(2,jel4) = ii1
         kelem(3,jel4) = ii2
         kelem(4,jel4) = i1

      else if ( ratb>ratio+1d-8 ) then

!     --- Selection B is the best, place four
!         new tetrahedrons:

         kelem(1,jel1) = i4
         kelem(2,jel1) = ii1
         kelem(3,jel1) = i3
         kelem(4,jel1) = i2

         kelem(1,jel2) = i4
         kelem(2,jel2) = ii2
         kelem(3,jel2) = ii1
         kelem(4,jel2) = i2

         kelem(1,jel3) = i4
         kelem(2,jel3) = i3
         kelem(3,jel3) = ii1
         kelem(4,jel3) = i1

         kelem(1,jel4) = i4
         kelem(2,jel4) = ii1
         kelem(3,jel4) = ii2
         kelem(4,jel4) = i1

      else

!     --- Old selection was the best

         isuccs = 0
      end if
1000  call erclos ( 'msh666' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh666'
      end if

      end
