      subroutine msh674( kelem, nelem, jelem, jel1, jel2, jel3, jel4, &
                         iel14, iel23, coor, npoint )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.0  date 27-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Try to improve the overall ratio of a mesh consisting of a set of
!     seven tetrahedrons. Make a comparison for the best ratio's using
!     either alternative bodydiagonals or by adding a new central node.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     best_ratio
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kelem(4,*), jelem, jel1, jel2, jel3, jel4, &
                         iel14, iel23, npoint
      double precision coor(3,*)

!     coor      i/o   array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     kelem     i/o   element array
!     iel14      i    nr of elmnt i1, ii2, ii3, i4
!     iel23      i    nr of elmnt i2, ii1, ii4, i3
!     jel1       i    nr of neighbour elmnt i1, i3, i2, ii1
!     jel2       i    nr of neighbour elmnt i1, i2, i4, ii2
!     jel3       i    nr of neighbour elmnt i1, i4, i3, ii3
!     jel4       i    nr of neighbour elmnt i2, i3, i4, ii4
!     jelem      i    main element considered i1,i2,i3,i4
!     nelem     i/o   number of elements in mesh
!     npoint    i/o   number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, ii1, ii2, ii3, ii4, &
              ia, ib, ic, id, j, nel
      double precision rat1, rat2, ratio, volume, volumo, vol

!     i1             Node number
!     i2             Node number
!     i3             Node number
!     i4             Node number
!     ii1            Node number
!     ii2            Node number
!     ii3            Node number
!     ii4            Node number
!     nel            helpvariable for number of elements
!     rat            helpvariable to store ratio-value
!     ratio          ratio of tet-element considered
!     vol            volume of tet considered
!     volume         total volume of set of tets
!     volumo         helpvariable to store old volume
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH632         compute volume of tetrahedron
!     MSH661         compute ratio of tetrahedron
!     MSH676         Place four-noded-tetrahedron in array kelem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh674' )
      debug = .false.

!     --- Determine the node numbers of main element jelem

      i1 = kelem(1,jelem)
      i2 = kelem(2,jelem)
      i3 = kelem(3,jelem)
      i4 = kelem(4,jelem)

!     --- Compute ratio of this element:

      call msh661( coor, i1, i2, i3, i4, rat1 )

!     --- Find node ii1:

      ia = kelem(1,jel1)
      ib = kelem(2,jel1)
      ic = kelem(3,jel1)
      id = kelem(4,jel1)

!     --- Compare:

      if ( ia/=i1.and.ia/=i2.and.ia/=i3.and.ia/=i4) ii1 = ia
      if ( ib/=i1.and.ib/=i2.and.ib/=i3.and.ib/=i4) ii1 = ib
      if ( ic/=i1.and.ic/=i2.and.ic/=i3.and.ic/=i4) ii1 = ic
      if ( id/=i1.and.id/=i2.and.id/=i3.and.id/=i4) ii1 = id

!     --- Find node ii2:

      ia = kelem(1,jel2)
      ib = kelem(2,jel2)
      ic = kelem(3,jel2)
      id = kelem(4,jel2)
      if ( ia/=i1.and.ia/=i2.and.ia/=i3.and.ia/=i4) ii2 = ia
      if ( ib/=i1.and.ib/=i2.and.ib/=i3.and.ib/=i4) ii2 = ib
      if ( ic/=i1.and.ic/=i2.and.ic/=i3.and.ic/=i4) ii2 = ic
      if ( id/=i1.and.id/=i2.and.id/=i3.and.id/=i4) ii2 = id

!     --- Find node ii3:

      ia = kelem(1,jel3)
      ib = kelem(2,jel3)
      ic = kelem(3,jel3)
      id = kelem(4,jel3)
      if ( ia/=i1.and.ia/=i2.and.ia/=i3.and.ia/=i4) ii3 = ia
      if ( ib/=i1.and.ib/=i2.and.ib/=i3.and.ib/=i4) ii3 = ib
      if ( ic/=i1.and.ic/=i2.and.ic/=i3.and.ic/=i4) ii3 = ic
      if ( id/=i1.and.id/=i2.and.id/=i3.and.id/=i4) ii3 = id

!     --- Find node ii4:

      ia = kelem(1,jel4)
      ib = kelem(2,jel4)
      ic = kelem(3,jel4)
      id = kelem(4,jel4)
      if ( ia/=i1.and.ia/=i2.and.ia/=i3.and.ia/=i4) ii4 = ia
      if ( ib/=i1.and.ib/=i2.and.ib/=i3.and.ib/=i4) ii4 = ib
      if ( ic/=i1.and.ic/=i2.and.ic/=i3.and.ic/=i4) ii4 = ic
      if ( id/=i1.and.id/=i2.and.id/=i3.and.id/=i4) ii4 = id

      ia = kelem(1,iel14)
      ib = kelem(2,iel14)
      ic = kelem(3,iel14)
      id = kelem(4,iel14)
      if ( debug ) write(irefwr,*) 'iel14',ia,ib,ic,id

      ia = kelem(1,iel23)
      ib = kelem(2,iel23)
      ic = kelem(3,iel23)
      id = kelem(4,iel23)
      if ( debug ) write(irefwr,*) 'iel23',ia,ib,ic,id

      if ( debug ) then
         write(irefwr,*) 'msh674: Nodes found'
         write(irefwr,*) ' i1=', i1,' i2=', i2,' i3=', i3,' i4=', i4
         write(irefwr,*) 'ii1=',ii1,'ii2=',ii2,'ii3=',ii3,'ii4=',ii4
      end if

!     --- For checking compute volume present setting:

      volume = 0d0
      call msh632( coor,  i1,  i2,  i3,  i4, vol )
      volume = volume + vol
      call msh632( coor,  i1,  i3,  i2, ii1, vol )
      volume = volume + vol
      call msh632( coor,  i1,  i4,  i3, ii3, vol )
      volume = volume + vol
      call msh632( coor,  i4,  i1, ii2, ii3, vol )
      volume = volume + vol
      call msh632( coor,  i2,  i4,  i1, ii2, vol )
      volume = volume + vol
      call msh632( coor,  i3,  i4,  i2, ii4, vol )
      volume = volume + vol
      call msh632( coor,  i2, ii4, ii1,  i3, vol )
      volume = volume + vol

!     --- Save this value

      volumo = volume
      if ( debug ) write(irefwr,*) 'msh674 Old volume is',volume

      do j=1,3
         coor(j,npoint+1) = &
             (coor(j,i1)+coor(j,i2)+coor(j,i3)+coor(j,i4))/4d0
      end do

!     --- Check volumes and ratio's:

      volume = 0d0
      ratio  = 1d0
      call msh632( coor,  i1,  i2, ii1, npoint+1, vol )
      call msh661( coor,  i1,  i2, ii1, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i2,  i4, ii4, npoint+1, vol )
      call msh661( coor,  i2,  i4, ii4, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i4,  i3, ii4, npoint+1, vol )
      call msh661( coor,  i4,  i3, ii4, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i3,  i1, ii1, npoint+1, vol )
      call msh661( coor,  i3,  i1, ii1, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i3, ii1, ii4, npoint+1, vol )
      call msh661( coor,  i3, ii1, ii4, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i2, ii4, ii1, npoint+1, vol )
      call msh661( coor,  i2, ii4, ii1, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i2,  i1, ii2, npoint+1, vol )
      call msh661( coor,  i2,  i1, ii2, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i4,  i2, ii2, npoint+1, vol )
      call msh661( coor,  i4,  i2, ii2, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i3,  i4, ii3, npoint+1, vol )
      call msh661( coor,  i3,  i4, ii3, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i1,  i3, ii3, npoint+1, vol )
      call msh661( coor,  i1,  i3, ii3, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i1, ii3, ii2, npoint+1, vol )
      call msh661( coor,  i1, ii3, ii2, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol
      call msh632( coor,  i4, ii2, ii3, npoint+1, vol )
      call msh661( coor,  i4, ii2, ii3, npoint+1, rat2 )
      if ( rat2<ratio ) ratio = rat2
      volume = volume + vol

      if ( abs(volume - volumo)>.00001 * (abs(volume)) ) then
           write(irefwr,*) 'Essential internal Error in msh674'
           write(irefwr,*) 'Please inform SEPRA'
           !call instop
           ratio = rat1 - 1d0
      end if
      if ( debug ) then
         write(irefwr,*) 'in msh674 volinhoud',volumo,'and',volume
         write(irefwr,*) 'rat 1=',rat1,'en rat2=',ratio
         if ( ratio>rat1 ) write(irefwr,*) 'New is better'
      end if

!     --- Improvement? =>

      if ( ratio>rat1 ) then
         if ( debug ) write(irefwr,*) 'Better ratio found in msh674'

!        --- Replace the elements by the new ones:

         npoint = npoint + 1
         nel    = nelem
         call msh676 ( kelem, jelem,  i1,  i2, ii1, npoint )
         call msh676 ( kelem, jel1 ,  i2,  i4, ii4, npoint )
         call msh676 ( kelem, jel2 ,  i4,  i3, ii4, npoint )
         call msh676 ( kelem, jel3 ,  i3,  i1, ii1, npoint )
         call msh676 ( kelem, jel4 ,  i3, ii1, ii4, npoint )
         call msh676 ( kelem, iel14,  i2, ii4, ii1, npoint )
         call msh676 ( kelem, iel23,  i2,  i1, ii2, npoint )
         call msh676 ( kelem, nel+1,  i4,  i2, ii2, npoint )
         call msh676 ( kelem, nel+2,  i3,  i4, ii3, npoint )
         call msh676 ( kelem, nel+3,  i1,  i3, ii3, npoint )
         call msh676 ( kelem, nel+4,  i1, ii3, ii2, npoint )
         call msh676 ( kelem, nel+5,  i4, ii2, ii3, npoint )
         nelem = nelem + 5
      end if

      call erclos ( 'msh674' )

      end
