      subroutine msh677 ( coor, i1, i2, i3, surf )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.0  date 09-09-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute "surface" surf of three Euclidian nodes in R3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3
      double precision coor(3,*), surf

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     i1             i    first node of triangle
!     i2             i    second node of triangle
!     i3             i    third node of triangle
!     surf           o    surface of triangle i1-i2-i3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a, b, c, x1, x2, x3, y1, y2, y3, z1, z2, z3

!     a              help coefficient
!     b              help coefficient
!     c              help coefficient
!     x1             x-coordinate node 1
!     x2             x-coordinate node 2
!     x3             x-coordinate node 3
!     y1             y-coordinate node 1
!     y2             y-coordinate node 2
!     y3             y-coordinate node 3
!     z1             z-coordinate node 1
!     z2             z-coordinate node 2
!     z3             z-coordinate node 3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     499   Node number is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set values of coordinates
!
!     Compute components cross-product
!     Compute surface
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh677' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh677'
         if ( i1==0 .or. i2==0 .or. i3==0 ) then

!        --- Internal error one of i1, i2 or i3 = 0

            call errsub ( 499, 0, 0, 0 )
            go to 1000
         end if
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set the coordinates of the three nodes

      surf = 0

      x1 = coor( 1, i1 )
      y1 = coor( 2, i1 )
      z1 = coor( 3, i1 )

      x2 = coor( 1, i2 )
      y2 = coor( 2, i2 )
      z2 = coor( 3, i2 )

      x3 = coor( 1, i3 )
      y3 = coor( 2, i3 )
      z3 = coor( 3, i3 )

      a = (y2-y1)*(z3-z1) - (z2-z1)*(y3-y1)
      b = (z2-z1)*(x3-x1) - (x2-x1)*(z3-z1)
      c = (x2-x1)*(y3-y1) - (y2-y1)*(x3-x1)

      surf = sqrt( a*a + b*b + c*c )

1000  call erclos ( 'msh677' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End debug msh677'
      end if

      end
