      subroutine sepdbufcop ( mmindex, array, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate the real array referred to by the memory management
!     index mmindex and copy the contents of buffer to array
!     The inverse of this subroutine is inidcopbuf
!     Old subroutine inidbufcop
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: mmindex
      double precision, intent(out) :: array(*)
      character (len=*), intent(in) :: namear

!     array          i    array into buffer must be copied
!     mmindex        i    Memory management sequence number of array to be
!                         copied
!     namear         i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) :: ipindex
      integer :: length
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipindex        Starting address of array with memory management sequence
!                    number mmindex in IBUFFR
!     length         length of the array in words
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) :: inidgttext
      integer :: inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdbufcop' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdbufcop'
         write(irefwr,1) 'mmindex', mmindex
         write(irefwr,*) 'namear', namear
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate array

      call ini060text ( ibuffr, mmindex, namear )

!     --- compute starting address

      ipindex = inidgttext ( mmindex, namear )
      length = inidln ( mmindex )

      if ( debug ) write(irefwr,1) 'length, ipindex', length, ipindex

!     --- Copy array

      array(1:length) = buffer(ipindex:ipindex+length-1)

1000  call erclos ( 'sepdbufcop' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepdbufcop'

      end if  ! ( debug )

      end
