      subroutine askprinmesh ( inpsep, istart )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.0  date 22-03-2014 Use local inpsep
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword print mesh
!     and store in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 1    102 to indicate print mesh
!                              2    ichoice in prinkmesh
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncode
      parameter ( nncode=38 )
      integer ncode(nncode), nmcode(nncode), i, ichoice
      character ( len=16 ) code(nncode)
      logical finished

!     code           Table with keywords
!     finished       Indicates if keyword has been found (true) or not
!                    (false)
!     i              Counting variable
!     ichoice        Integer choice parameter for prinkmesh
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nmcode         Table containing the number of characters
!                    in the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
!    2671   No keyword found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'TOPOLOGY', 'NODE_ELEMENTS', 'NODE_NODES', &
                  'NEW_NUMBERING', 'PART_H', 'POIN_CUR_SUR', &
                  'ELEMENT_ELEMENTS', 'LEVELS', 'SUB_CUR_SUR', &
                  'IINPUT_RINPUT', 'PART_T', 'PART_U', 'PART_V', &
                  'PART_W', 'PART_X', 'PART_Z', 'PART_Y', &
                  'COOR', 'EDGE_NODES', 'ELEMENT_EDGES', &
                  'NODE_EDGES', 'VOLUME_SURFS', 'EDGE_ELEMENTS', &
                  'ELEMENT_FACES', 'FACE_EDGES', 'FACE_ELEMENTS', &
                  'FACE_NODES', 'NODE_FACES', 'BLOCKINFO', &
                  'OUTER_CUR_SUR', 'SURF_NODES', 'NORMALS', &
                  'OUTER_BOUN', 'GROUP_NODES', 'GROUP_INFO', &
                  'EDGE_FACES', 'X', 'QUANTITIES' /
      data ncode  /  3,  7,  7,  3,  6,     8,  9,  3,  7,  4, &
                     6,  6,  6,  6,  6,     6,  6,  4,  8, 11, &
                     7, 11,  7, 11,  7,     7,  7,  7,  3,  8, &
                     7,  3,  8,  8,  8,     7,  1,  3 /
      data nmcode /  8, 13, 10, 13,  6,    12, 16,  6, 11, 13, &
                     6,  6,  6,  6,  6,     6,  6,  4, 10, 13, &
                    10, 12, 13, 13, 10,    13, 10, 10,  9, 13, &
                    10,  7, 10, 11, 10,    10,  1, 10 /

!     code    Table with main keywords
!             Values:
!             1:  TOPOLOGY
!             2:  NODE_ELEMENTS
!             3:  NODE_NODES
!             4:  NEW_NUMBERING
!             5:  PART_H
!             6:  POIN_CUR_SUR
!             7:  ELEMENT_ELEMENTS
!             8:  LEVELS
!             9:  SUB_CUR_SUR
!             10: IINPUT_RINPUT
!             11: PART_T
!             12: PART_U
!             13: PART_V
!             14: PART_W
!             15: PART_X
!             16: PART_Z
!             17: PART_Y
!             18: COOR
!             19: EDGE_NODES
!             20: ELEMENT_EDGES
!             21: NODE_EDGES
!             22: VOLUME_SURFS
!             23: EDGE_ELEMENTS
!             24: ELEMENT_FACES
!             25: FACE_EDGES
!             26: FACE_ELEMENTS
!             27: FACE_NODES
!             28: NODE_FACES
!             29: BLOCKINFO
!             30: OUTER_CUR_SUR
!             31: SURF_NODES
!             32: NORMALS
!             33: OUTER_BOUN
!             34: GROUP_NODES
!             35: GROUP_INFO
!             36: EDGE_FACES
!             37: X
!             38: QUANTITIES
! ====================================================================== &
!
      call eropen ( 'askprinmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprinmesh'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ichoice = 0
      if ( nitems>2 ) then

!     --- More than 2 keywords found; the third must be a string
!         Read next item

         call asknxt

         if ( ityprd/=4 ) then

!        --- Error, not a string found after print mesh

            call errchr ( 'Print mesh', 1 )
            call errsub ( 2671, 0, 0, 1 )
            go to 1000

         end if  ! ( ityprd/=4 )

         finished = .false.

         do i = 1, nncode

            if ( string(1:ncode(i))==code(i)(1:ncode(i)) ) then

!           --- keyword found

               ichoice = i
               if ( ichoice>9 ) ichoice = ichoice+20  ! jump in prinkmesh
               if ( ichoice>50 ) ichoice = ichoice+8  ! jump in prinkmesh
               finished = .true.

            end if  ! ( string(1:ncode(i))==code(1:ncode(i)) )

            if ( finished ) exit

         end do  ! i = 1, nncode

         if ( .not. finished ) then

!        --- No keyword found

            call errchr ( string(1:nchar), 1 )
            do i = 1, nncode
               call errchr ( code(i)(1:nmcode(i)), i+1 )
            end do  ! i = 1, nncode
            call errsub ( 648, 0, 0, 1+nncode )
            go to 1000

         end if  ! ( finished )

      end if  ! ( nitems>2 )

      inpsep(0) = 102
      inpsep(1) = ichoice

1000  call erclos ( 'askprinmesh' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ichoice', ichoice
         call prinin ( inpsep(0), 2, 'inpsep' )
         write(irefwr,*) 'End askprinmesh'

      end if  ! ( debug )

      istart = istart+2  ! raise istart

      end
