      subroutine ercomnor2 ( norm, normaldr, isurnr1, isurnr2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 08-10-2007 Extra debug information
!        version  1.1  date 14-07-2006 Debug information
!        version  1.0  date 18-03-2002
!
!   copyright (c) 2002-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Store the computed normals in array norm with the correct sign
!      Use the outer boundary and normals already available
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     mesh
!     normal_vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer normaldr, isurnr1, isurnr2
      double precision norm(3,*)

!     isurnr1        i    Surface sequence number of first surface
!     isurnr2        i    Surface sequence number of second surface
!     norm           o    Array of length ndim * npoint containing the normal
!                         in the nodal points
!     normaldr       i    If 1 the outward directed normal is required, if -1
!                         the inward directed normal
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(nsurfs), nparts, ipart, ipstart, nouter, isurnr, &
              j, idir, nsubsf, nnodes, node, istart
      double precision dir

!     dir            Multiplication factor for normal
!     i              Counting variable
!     idir           If 1 the normal must be taken in positive direction
!                    otherwise in negative direction
!     ipart          Sequence number of part
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length nsurfs
!                    iwork(i) =-1: normal is directed inwards
!                    iwork(i) = 1: surface is either inner surface or normal is
!                                  directed outwards
!     j              Counting variable
!     nnodes         Number of nodes
!     node           Node number
!     nouter         Number of points outside the obstacle
!     nparts         number of parts the boundary consists of
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     PRINRL1NONZERO print 2d real array
!                    Only non-zero rows are printed
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set iwork = 1, (1...nsurfs)
!     Set sign of outer surfaces in iwork
!     Set sign of composite outer surfaces in iwork
!     For isurnr := isurnr1 (1) isurnr2 do
!        dir := normaldr * iwork(isurnr)
!        copy normal * idir into norm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercomnor2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercomnor2'
         write(irefwr,1) 'nsurfs, isurnr1, isurnr2', &
                          nsurfs, isurnr1, isurnr2
         write(irefwr,*) 'surfnodes'
         call prinnmeshx ( nsurfs, surfnodes )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set iwork = 1, (1...nsurfs)

      iwork = 1

!     --- Set sign of outer surfaces in iwork

      nparts = outerboun(1)
      do ipart = 1, nparts

         ipstart = outerboun(ipart+1)
         nouter = outerboun(ipstart)
         do i = 1, nouter
            isurnr = outerboun(ipstart+i)
            if ( isurnr<0 ) iwork(-isurnr) = -1
         end do

      end do  !ipart = 1, nparts
      if ( debug ) call prinin ( iwork, nsurfs, 'iwork' )

!     --- Set sign of composite outer surfaces in iwork

      if ( kelmr>0 ) then

!     --- kmesh part r exists

         do i = 1, nsurfs

            nsubsf = kmeshr(i+1)-kmeshr(i)
            if ( nsubsf>1 ) then

!           --- There are sub surfaces

               idir = 1
               do j = 1, nsubsf
                  isurnr = abs(kmeshr(kmeshr(i)+j))
                  if ( iwork(isurnr)<0 ) idir = -1
               end do
               iwork(i) = idir

            end if

         end do

      end if

      if ( debug ) then

!     --- debugging statements

         call prinin ( iwork, nsurfs, 'iwork: inner/outer surfaces' )
         write(irefwr,1) 'isurnr1, isurnr2',isurnr1, isurnr2

      end if

!     --- For isurnr := isurnr1 (1) isurnr2 do

      do isurnr = isurnr1, isurnr2

!     --- dir := normaldr * iwork(isurnr)
!         copy normal * idir into norm

         dir = normaldr * iwork(isurnr)
         ipstart = surfnodes(isurnr)
         nnodes = surfnodes(ipstart)
         istart = ipstart-nsurfs-isurnr
         do i = 1, nnodes
            node = surfnodes(ipstart+i)
            do j = 1, 3
               norm(j,node) = normal(j,istart+i)*dir
            end do
         end do

      end do  ! isurnr = isurnr1, isurnr2

      if ( debug ) then

!     --- debugging statements

         call prinrl1nonzero ( norm, npoint, 3, 'norm' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercomnor2'

      end if  ! ( debug )
      call erclos ( 'ercomnor2' )

      end
