      subroutine erconc ( solutn, number )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date   20-02-1998 Improved interpolation
!        version 1.0    date   03-02-1996
!
!   copyright (c) 1996-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill ninth point of element in vector by interpolation
!   Real version
! **********************************************************************
!
!                       KEYWORDS
!
!    fill_function
!    interpolation
!    real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number
      double precision solutn(number,*)

!     number     i    Number of degrees of freedom per point that must be filled
!     solutn    i/o   Array of length number x npoint in which the
!                     solution array is stored in the sequence:
!                     u (x ) = solutn ( i,k )  (i=1(1)number; k=1(1)npoint)
!                      i  k
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, ikelmc, ielgrp, ielem

!     i          Counting variable
!     ielem      Element number
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     inpelm     number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For all element groups do
!      If ( element has shape 6 ) then
!         Fill centroid by interpolation
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ikelmc = 0
      do ielgrp = 1, nelgrp

!     --- Compute shape number, number of nodes and number of elements
!         of element group IELGRP

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( ishape/=6 ) then

!        --- Not an element of shape 6

            ikelmc = ikelmc+inelem*inpelm

         else

!        --- element of shape 6

            do ielem = 1, inelem

               do i = 1, number
                  solutn(i,kmeshc(ikelmc+9)) = 0.5d0* &
                     ( solutn(i,kmeshc(ikelmc+2)) + &
                       solutn(i,kmeshc(ikelmc+4)) + &
                       solutn(i,kmeshc(ikelmc+6)) + &
                       solutn(i,kmeshc(ikelmc+8)) ) - 0.25d0 * &
                     ( solutn(i,kmeshc(ikelmc+1)) + &
                       solutn(i,kmeshc(ikelmc+3)) + &
                       solutn(i,kmeshc(ikelmc+5)) + &
                       solutn(i,kmeshc(ikelmc+7)) )
               end do
               ikelmc = ikelmc + 9

            end do

         end if

      end do

      end
