      subroutine erdisttop ( x1, x2, x3, distance )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the distance of the top node (x3) to base of the triangle
!     (x1-x2) by computing the length of the normal onto the base through
!     the top between base and top
! **********************************************************************
!
!                       KEYWORDS
!
!     distance
!     triangle
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(2), x2(2), x3(2), distance

!     distance       o    distance between top and bottom
!     x1             i    coordinates of first basis point
!     x2             i    coordinates of last basis point
!     x3             i    coordinates of  top
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision n(2), xn(2), fact1, fact2

!     fact1          multiplication factor with respect to intersection
!     fact2          multiplication factor with respect to intersection
!     n              normal vector
!     xn             coordinates of new point on normal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERNORMAL2D     Compute normal vector perpendicular on the line through the
!                    points with coordinates (x1,y1) and (x2, y2)
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE2  Computes the intersection of two lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     n = normal on (x1-x2)
!     compute intersection x3 - x3+n, x1 - x2
!     distance := length of x3 to intersection point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erdisttop' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erdisttop'
         write(irefwr,*) 'co-ordinates of triangle ', x1, x2, x3

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- n = normal on (x1-x2)

      call ernormal2d ( x1, x2, n )

!     --- compute intersection x3 - x3+n, x1 - x2

      xn(1) = x3(1)+n(1)
      xn(2) = x3(2)+n(2)
      if ( debug )  write(irefwr,*) 'n, xn ', n, xn
      call mshcrossline2 ( x3, xn, x1, x2, fact1, fact2 )
      if ( debug )  write(irefwr,*) 'fact1, fact2 ', fact1, fact2

!     --- distance := length of x3 to intersection point

      if ( fact1.gt.-1d20 ) then
         distance = abs(fact1)
      else
         distance = rinfin
      end if   ! ( fact1.gt.-1d20 )

1000  call erclos ( 'erdisttop' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erdisttop'

      end if

      end
