      subroutine erfillnodskmc2 ( nodes, nnodes, kmeshc, inelem, inpelm, &
                                  iwork, numnod, ihelp, istart )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for elements 1 to inelem
!     Special case oly a restricted number of nodes per element
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), nnodes, kmeshc(*), inelem, inpelm, iwork(*), &
              numnod, ihelp(*), istart

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     istart         i    First position to be used in kmeshc (minus 1)
!     iwork         i/o   integer work array of length npoint
!                         iwork(i) = 0, means node i has not yet been filled
!                         iwork(i)>0, means node i has been filled
!                         The value defines the new node number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, ielem, irel

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     irel           Relative node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1119   Relative nodal point number in element is out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfillnodskmc2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillnodskmc2'
         write(irefwr,1) 'numnod', numnod
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check relative nodal points

      do i = 1, numnod

         irel = ihelp(i)

         if ( irel<=0 .or. irel>inpelm ) then

!        --- irel out of range

            call errint ( irel, 1 )
            call errint ( inpelm, 2 )
            call errint ( i, 3 )
            call errsub ( 1119, 3, 0, 0 )

         end if

      end do  ! i = 1, numnod
      if ( ierror/=0 ) go to 1000

!     --- Loop over all elements

      do ielem = 1, inelem

         do i = 1, numnod

            inode = kmeshc(istart+ihelp(i))

            if ( inode>0 ) then

!           --- Node number positive

               if ( iwork(inode)==0 ) then

!              --- iwork(inode)=0, hence new node

                  iwork(inode)=1
                  nnodes = nnodes+1
                  nodes(nnodes) = inode

               end if  ! ( iwork(inode)==0 )

            end if  ! ( inode>0 )

         end do  ! i = 1, numnod

         istart = istart+inpelm

      end do  ! ielem = 1,inelem

1000  call erclos ( 'erfillnodskmc2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillnodskmc2'

      end if  ! ( debug )

      end
