      subroutine errcontrol ( intfirst, intlast, intval, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check control integer around an integer work array
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     array
!     array_length
!     integer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intfirst, intlast, intval
      character *(*) name

!     intfirst       i    value of first control integer
!     intlast        i    value of last control integer
!     intval         i    value of original control integer
!     name           i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1509   Leading control integer incorrect
!    1510   Trailing control integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'errcontrol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errcontrol'

      end if
      if ( ierror.ne.0 ) go to 1000
      if ( intfirst.ne.intval ) then

!     --- First control integer incorrect

         call errint ( intfirst, 1 )
         call errint ( intval, 2 )
         call errchr ( name, 1 )
         call errsub ( 1509, 2, 0, 1 )

      end if

      if ( intlast.ne.intval ) then

!     --- last control integer incorrect

         call errint ( intlast, 1 )
         call errint ( intval, 2 )
         call errchr ( name, 1 )
         call errsub ( 1510, 2, 0, 1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errcontrol'

      end if
      call erclos ( 'errcontrol' )

      end
