      function getstartkmeshc ( inpelm, ielem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting address in kmeshc and information of element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, ielem, getstartkmeshc

!     getstartkmeshc o    starting address of element ielem in kmesh part c
!     ielem          i    Element number from which information is required
!     inpelm         o    number of points per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp

!     ielgrp         element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getstartkmeshc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getstartkmeshc'

      end if
      if ( ierror/=0 ) go to 1000

      ielgrp = sepelgrpelem(ielem)
      inpelm = Topgrp(ielgrp)%inpelm
      getstartkmeshc = pointkmsc(ielgrp)+(ielem-inelem-1)*inpelm+1
      ishape = sepgetshape ( ielgrp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getstartkmeshc'

      end if
      call erclos ( 'getstartkmeshc' )

      end
