      subroutine ini018 ( icurvs, ncurvs, mcurvs, m1curvs )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date  06-09-1997 Correction for negative curves
!        version 1.0    date  29-04-1996
!
!
!
!   copyright (c) 1996-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check uniqueness of curve numbers stored in array icurvs
!
! **********************************************************************
!
!                       KEYWORDS
!
!   check
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer icurvs(*), ncurvs, mcurvs, m1curvs

!     icurvs     i    Integer array containing the curve numbers
!     m1curvs    i    Number of curves in first part of icurvs
!     mcurvs     i    Number of curves in icurvs
!     ncurvs     i    Number of curves in mesh
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(ncurvs)

!     i       Counting variable
!     iwork   Integer work array of length ncurvs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1949   Curve number found more than once in a set of curve numbers
!    1978   Curve number more than once in set of curves to be redistributed
!    1979   Curve number is in set of curves to be redistributed, but not in
!           the set of curves that must be adapted
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    For i := 1 (1) ncurvs do
!       if ( iwork(icurvs(i))>0 ) then
!          error
!       iwork(icurvs(i)) := 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Clear array iwork

      do i = 1, ncurvs
         iwork(i) = 0
      end do

!     --- Check

      do i = 1, m1curvs
         if ( iwork(abs(icurvs(i))).gt.0 ) then

!        --- Error: curve number appears twice

            call eropen ( 'ini018' )
            call errint ( abs(icurvs(i)), 1 )
            call errsub ( 1949, 1, 0, 0 )
            call erclos ( 'ini018' )

         end if
         iwork(abs(icurvs(i))) = 1
      end do

      do i = m1curvs+1, mcurvs
         if ( iwork(abs(icurvs(i))).gt.1 ) then

!        --- Error: curve number appears twice

            call eropen ( 'ini018' )
            call errint ( abs(icurvs(i)), 1 )
            call errsub ( 1978, 1, 0, 0 )
            call erclos ( 'ini018' )

         end if
         if ( iwork(abs(icurvs(i))).eq.0 ) then

!        --- Error: curve number not in first part

            call eropen ( 'ini018' )
            call errint ( abs(icurvs(i)), 1 )
            call errsub ( 1979, 1, 0, 0 )
            call erclos ( 'ini018' )

         end if
         iwork(abs(icurvs(i))) = 2
      end do

      end
