      subroutine inicommonarray ( set1, lenset1, set2, lenset2, maxlen, &
                                  nfound, commonset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute common entries of two sets of arrays and store in new set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     common
!     entry
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer set1(*), lenset1, set2(*), lenset2, maxlen, &
              nfound, commonset(*)

!     commonset      o    Set of common entries of set1 and set2
!     lenset1        i    Number of entries in set1
!     lenset2        i    Number of entries in set2
!     maxlen         i    Maximum entry in set1 and set2
!     nfound         o    Number of entries in commonset
!     set1           i    First set containing only positive integers
!     set2           i    Second set containing only positive integers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(maxlen), i, ientry

!     i              Counting variable
!     ientry         Entry in set
!     iwork          integer work array of length maxlen
!                    iwork(i) = 0, if i not in set1
!                    iwork(i) = 1, if i in set1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:maxlen)
!     Set iwork(i) = 1, if i in set1
!     nfound := 0
!     For all entries i of set2 do
!        if ( iwork(i) == 1 ) then
!           nfound := nfound+1;  commonset(nfound) := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicommonarray' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicommonarray'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array iwork(1:maxlen)

      do i = 1, maxlen
         iwork(i) = 0
      end do   ! i = 1, maxlen

!     --- Set iwork(i) = 1, if i in set1

      do i = 1, lenset1
         iwork(set1(i)) = 1
      end do  ! i = 1, lenset1

!     --- nfound := 0

      nfound = 0

!     --- For all entries i of set2 do

      do i = 1, lenset2

         ientry = set2(i)
         if ( iwork(ientry).eq.1 ) then

!        --- iwork(i) == 1
!            nfound := nfound+1;  commonset(nfound) := i

            nfound = nfound+1
            commonset(nfound) = ientry

         end if  ! ( iwork(ientry).eq.1 )

      end do  ! i = 1, lenset2

1000  call erclos ( 'inicommonarray' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicommonarray'

      end if

      end
