      subroutine inimultadd2nbc ( iu1, iu2, a1, a2, iures )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 23-07-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a1 u1 + a2 u2 and store in ures
!     ures may be one of the two vectors
!     The vectors must be sepran solution vectors with the same length
!     The part corresponding to boundary conditions is not filled
!     If the vector is new, it is filled with zeros
! **********************************************************************
!
!                       KEYWORDS
!
!     multiplication
!     linear_combination
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iu1, iu2, iures
      double precision a1, a2

!     a1             i    multiplication constant
!     a2             i    multiplication constant
!     iu1            i    First vector
!     iu2            i    Second vector
!     iures         i/o   Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              General loop variable
!     j              Position in solution vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2887   The length of two input vectors is not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inimultadd2nbc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inimultadd2nbc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get length of arrays

      nusol = ks(iu1)%nusol
      if ( ks(iu2)%nusol/=nusol ) then

!     --- error: arrays do not have the same length

         call errchr ( 'iu1', 1 )
         call errchr ( 'iu2', 2 )
         call errint ( ks(iu1)%nusol, 1 )
         call errint ( ks(iu2)%nusol, 2 )
         call errsub ( 2887, 2, 0, 2 )
         go to 1000

      end if  ! ( ks(iu2)%nusol/=nusol )

!     --- Check iprob

      iprob  = ks(iu1)%iprob      ! problem number
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      nbound = kprobloc(10)   ! number of essential bc's
      npboun = kprobloc(34)   ! number of periodical bc's
      indprh = kprobloc(50)   ! mm number of kprob part h

      if ( iures==0 ) then

!     --- New array

         call  sepcreasolbf1 ( iures, ks(iu1)%typevec, ks(iu1)%ivec, &
                               'iures', ks(iu1)%compl )
         unew => usoltemp
         unew = 0d0

      else

!     --- Old array

         call  sepcreasolbf1 ( iures, ks(iu1)%typevec, ks(iu1)%ivec, &
                               'iures', ks(iu1)%compl )
         unew => ks(iures)%sol

      end if  ! ( iures==0 )

      u1 => ks(iu1)%sol
      u2 => ks(iu2)%sol

      if ( npboun+nbound==0 ) then

!     --- No boundary conditions, so standard case

         unew = a1*u1+a2*u2

      else

!     --- There are boundary conditions, jump over corresponding
!         positions

         kprobh => kprobpart%kprobh
         kprobhinv => kprobpart%kprobhinv
         nrusol = nusol-nbound-npboun
         do i = 1, nrusol
            j = kprobhinv(i)
            unew(j) = a1*u1(j) + a2*u2(j)
         end do  ! i = 1, nrusol

      end if  ! ( npboun+nbound==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inimultadd2nbc'

      end if  ! ( debug )
      call erclos ( 'inimultadd2nbc' )

      end subroutine inimultadd2nbc
