      subroutine inineighbors ( node, npoint, kmeshe, nneigh, setneigh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all neighbors of a node and store in array neighbors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer node, npoint, kmeshe(*), nneigh, setneigh(nneigh)

!     kmeshe         i    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     nneigh        i/o   at output: Number of neighbors of node
!                         at input: declared length of setneigh
!     node           i    Node number from which the neighbors must be found
!     npoint         i    Number of nodal points in the mesh
!     setneigh       o    Set of neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, lenneigh

!     ipstart        Starting address in array kmeshe
!     lenneigh       Declared length of array setneigh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inineighbors' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inineighbors'
         write(irefwr,1) 'node, npoint', node, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipstart = kmeshe(node)
      lenneigh = nneigh
      nneigh = kmeshe(node+1)-ipstart
      ipstart = ipstart+npoint+1

      if ( debug ) write(irefwr,1) 'nneigh, ipstart', nneigh, ipstart

      call inichkarlen ( nneigh, lenneigh, 'setneigh' )
      if ( ierror/=0 ) go to 1000

      setneigh(1:nneigh) = kmeshe(ipstart+1:ipstart+nneigh)

1000  call erclos ( 'inineighbors' )
      if ( debug ) then

!     --- Debug information

         call prinin ( setneigh, nneigh, 'setneigh' )
         write(irefwr,*) 'End inineighbors'

      end if  ! ( debug )

      end
