      function ininodeshape ( ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 22-05-2014 Extension with shape 22
!        version  1.2  date 18-12-2002 Extension of shapes
!        version  1.0  date 25-05-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with number of nodes in element as function of the shape number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_shape
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ininodeshape, ishape

!     ininodeshape   o    Number of nodes in element
!     ishape         i    Shape number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodesshape(22)

!     nodesshape     Array of size 22 containing the number of nodes for
!                    each shape number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1025   shape number has not yet been implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data nodesshape / 1, 2, 3, 6, 4,   9, 7,10, 5, 4, &
                        4,10, 8,27,14,  15, 9, 5, 0, 0, &
                        6,18 /
! ======================================================================
!
      call eropen ( 'ininodeshape' )
      if ( ishape<=0 .or. ishape>18 .and. ishape<=20 .or. &
           ishape>22 ) then

!     --- shape number not yet implemented

          call errint ( ishape, 1 )
          call errchr ( 'ininodeshape', 1 )
          call errsub ( 1025, 1, 0, 1 )
          ininodeshape = 0

      else

!     --- standard case

         ininodeshape = nodesshape(ishape)

      end if
      call erclos ( 'ininodeshape' )
      end
