      subroutine ininormoutprod ( v1, v2, v3)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normalized inner product of vectors v1 and v2
!     and store in v3
! **********************************************************************
!
!                       KEYWORDS
!
!     norm
!     outer_product
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision v1(3), v2(3), v3(3)

!     v1             i    First vector in outer product
!     v2             i    Second vector in outer product
!     v3             i    Result of outer product, with length 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision h

!     h              Help variable to store the iverse of the length
!                    of the vector
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     v3 = v1 x v2
!     v3 = v3/||v3||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininormoutprod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininormoutprod'

      end if
      if ( ierror/=0 ) go to 1000

!     --- v3 = v1 x v2

      call eloutr ( v1, v2, v3 )

!     --- v3 = v3/||v3||

      h = 1d0/sqrt(v3(1)**2+v3(2)**2+v3(3)**2)
      do i = 1, 3
         v3(i) = h*v3(i)
      end do  ! i = 1, 3

1000  call erclos ( 'ininormoutprod' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininormoutprod'

      end if

      end
