      subroutine inirmf ( ifile )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 11-12-2017 New storage of intmat
!        version  1.3  date 24-04-2012 Remove sepcomp.out if parallel
!        version  1.2  date 02-02-2004 Adaptation for /tmp
!
!   copyright (c) 1995-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove file indicated by ifile if it exists
!
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      use sepmodulemach
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifile

!     ifile    i   File reference number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, lastps, icount
      character (len=100)  filename, name, filename1
      logical check

!     check          Indication if a file has been opened (true) or not (false)
!     filename       Name of file
!     filename1      Name of file
!     icount         Counter
!     iref           File reference number
!     lastps         Last non-blank position in namefl
!     name           Name of file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inirmf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inirmf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ifile = 73 (sepcomp.inf), 74 (sepcomp.out)
!         or 10 (meshoutput)

      if ( abs(ifile)==73 ) then

         name = name73
         iref = abs(iref73)
         if ( name(1:8)=='/tmp/SEP' ) then
            lastps = index ( name, ' ' )-1
            name = name(21:lastps)
         end if

      else if ( abs(ifile)==74 ) then

         name = name74
         iref = abs(iref74)

         if ( debug ) then
            write(irefwr,1) 'iref', iref
            write(irefwr,*) 'name', name
            write(irefwr,*) 'parallel', parallel
         end if  ! ( debug )

         if ( name(1:8)=='/tmp/SEP' ) then

!        --- Special case file on /tmp

            lastps = index ( name, ' ' )-1
            name = name(21:lastps)

         else if ( parallel .and. inodenr==1 ) then

!        --- Parallel case, node 1, remove sepcomp.out

            lastps = index ( name, ' ' )-1

            if ( debug ) &
              write(irefwr,1) 'inodenr, lastps', inodenr, lastps

            if ( lastps>8 ) then

!           --- Number of characters sufficient

               if ( name(lastps-7:lastps)== '_par.001' ) then

!              --- File has name xxx_par.001, remove file xxx.out

                  filename1 = name(1:lastps-8)//'.out'
                  inquire ( file=filename1, exist=check )

                  if ( debug ) then
                     write(irefwr,*) 'filename1 ', filename1
                     write(irefwr,*) 'check ', check
                  end if  ! ( debug )

                  if ( check ) then

                    open ( unit = iref, file = filename1, &
                           status = 'unknown' )
                    close ( unit = iref, status = 'delete' )

                  end if  ! ( check )

               end if  ! ( name(lastps:lastps)== '_par.001' )

            end if  ! ( lastps>8 )

         end if  ! ( name(1:8)=='/tmp/SEP' )

      else if ( abs(ifile)==10 ) then

         name = name10
         iref = abs(iref10)
         if ( name(1:8)=='/tmp/SEP' ) then
            lastps = index ( name, ' ' )-1
            name = name(21:lastps)
         end if
         lastps = index ( name, ' ' )-1
         filename = name
         filename(lastps+1:lastps+8) = '_par.000'
         inquire ( file=filename, exist=check )
         icount = 0
100      if ( check ) then

!        --- There are still files meshoutput_par.inirmf to be deleted

            open ( unit = iref, file = filename(1:lastps+8), &
                   status = 'unknown' )
            close ( unit = iref, status = 'delete' )
            icount = icount+1
            write(filename(lastps+1:lastps+8),110) icount
110         format('_par.',i3.3)
            inquire ( file=filename, exist=check )
            go to 100

         end if

      end if

      inquire ( file=name, exist=check )
      if ( check ) then

         open ( unit = iref, file = name, status = 'unknown' )
         close ( unit = iref, status = 'delete' )

      end if  ! ( check )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inirmf'

      end if  ! ( debug )
      call erclos ( 'inirmf' )

      end
