      subroutine initcb
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 11-12-2017 New storage of intmat
!        version  7.61 date 27-03-2015 New definition ctimhis
!        version  7.60 date 24-12-2014 New definition clinsl
!
!   copyright (c) 1982-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      initialize machine independent common blocks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     default
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulesolve
      use sepmodulecpu
      use sepmoduleerr
      use sepmodulepost
      use sepmoduleuser
      use sepmodulemeshinf
      use sepmoduleprobinf
      use sepmodulematr
      use sepmodulemain
      use sepmoduletimhis
      use sepmoduleloop
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleconsts
      use sepmoduleread
      use sepmodulecpack
      use sepmodulemach
      use sepmoduleplot
      use sepmodulecontact
      use sepmoduletexts
      use sepmoduledebug
      use sepmodulefiles
      use sepmoduleinput
      use sepmodulegen
      use sepmodulexpr
      use sepmoduletypes
      use sepmodulestoc
      use sepmodulemushy
      use sepmodulecactl
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INITCNPARM     Fill array cnparm for each type number that is available
!     SEPCLEAR       clear array map
!     SEPFILLCONSTS  Fill some general constpstants in module sepmoduleconsts
!     SEPFILLMODMESH Fill parameters in sepmodulemesh with start values
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- cplot

      xmin = -rinfin
      xmax =  rinfin
      ymin = -rinfin
      ymax =  rinfin
      zmin = -rinfin
      zmax =  rinfin
      jmax=0
      jmark=0
      jkader=0
      jtimes=0

!     --- cplaf

      dcolor = 0

!     --- cplaf1

      alefig = 15d0
      widfig = 15d0
      marlft = 2.5d0
      marund = 2.5d0
      xpleft = 0d0
      ypleft = 0d0
      xminzm = 0d0
      yminzm = 0d0
      xmaxzm = 0d0
      ymaxzm = 0d0
      xminor = 0d0
      yminor = 0d0
      xmaxor = 0d0
      ymaxor = 0d0
      xcentr = 0d0
      ycentr = 0d0
      zfact  = 0d0
      actfac = 0d0
      lzoom = 0
      iplpic = 0
      ilzoom = 0
      maxplots = 0

!     --- cactl

      ielem = 0
      itype = 0
      ielgrp = 0
      inpelm = 0
      icount = 0
      ifirst = 0
      notmat = 0
      notvec = 0
      irelem = 0
      nusolcac = 0
      nelemcac = 0
      npointcac = 0

!     --- cgrens

      kiesx=-1
      kiesy=-1
      isigx=-1
      isigy=-1
      nstepx=10
      nstepy=10
      expx = 0
      expy = 0
      shiftx = 0d0
      shifty = 0d0
      stepx = 0d0
      stepy = 0d0

!     --- cconst

      ierror=0
      nerror=0

!     --- citer

      omega=1d0
      alpha=2d0/3d0
      beta=.8d0
      alam=0d0
      niter=0
      niter1=-1
      niter2=-1
      numiters = 0

!     --- constpsta

      pi = 4d0*atan(1d0)
      aln10 = log(10d0)
      defplo = 15d0

!     --- coutpt

      ifoutp = 0

!     --- cmessc and cmessg

      chars = ' '
      reals = 0d0
      ints  = 0
      levsub = 0

!     --- cwarn

      nwarn = 0
      mxwarn = 10

!     --- cseprd

      nchar = 0
      realrd = 0d0
      ityprd = 1
      icolum = 1
      kcolum = 1
      irecnr = 0
      ispecl = 0
      istop  = 0
      lastp  = 0
      last   = 0
      iexpression = 0
      maxcolum = 240

!     --- cseprdnew

      realitems = 0d0
      firstcolitems = 0
      lastcolitems = 0

      firstcolumn = 0
      lastcolumn = 0

      numrecstored = 0
      nitems = 0
      numrecord = 0
      firstcol = 0
      lastcol = 0
      lastitem = 0

!     --- celp

      iweigh = 0
      ideriv = 0
      irule  = 1
      igausp = 0
      iphi   = 0
      jcart  = 1
      icoor  = 0
      interp = 0
      isecdv = 0

      numints = 0
      numreals = 0
      numscals = 0
      numstrings = 0
      numintscals = 0
      incons = 0
      chincons = ' '
      chrlcons => chincons(maxints+1:maxints+maxreals)
      rlcons = 0d0
      chrlcons = ' '
      scalars = 0d0
      chscalars = ' '
      ncharcons = 0
      ncharscals = 0
      ncharnames = 0
      dimcons = 0
      dimscals = 0
      dimnames = 0
      chstrings = ''

!     --- cpackage

      parallel = .false.
      namevfound = .false.
      numwis = .false.
      mpistart = .false.
      usepetsc = .false.

!     --- ctimen

      itimdep = 0
      maxcalltim = 0
      icalltim = 0
      theta = 1d0
      oldmethod = .false.

!     --- sepmoduleuser

      do i = 1, 100
         nullify ( iuser1(i)%iuser, iuser1(i)%user )
      end do  ! i = 1, 100
      nullify ( iuser, user )
      do i = 1, maxprb
         nullify ( icofius(i)%iuser, icofius(i)%user )
      end do  ! i = 1, maxprb
      iseqcoef = 0
      iprobls = 1
      nitercom = -1

!     --- csepint

      nextxt = 1
      iprogram = 0

!     --- cpost1/2

      ipost = 0
      rpost = 0d0
      rpost(1) = 1.4d0
      numvec = 0
      numfuncs = 0
      do i = 0, maxfuncs
         nullify ( ifuncs(i)%arr )
      end do  ! i = 0, maxfuncs
      infocomponents(1:maxvc1,1) = 1
      infocomponents(1:maxvc1,2) = 0
      lastmapsol = 0
      imapsol = 0
      iinputpst = 0
      iinputpst(101) = 6   ! number of characters in levels
      rinputpst(16) = 0.25d0 ! default text size (height)
      rinputpst(43) = 0.15d0 ! default text size
      rinputpst(44) = 0.15d0 ! default text size levels
      rinputpst(45) = 0.25d0 ! default size numbers in levels

!     --- cpost5

      do i = 1, maxvc1-1
         names(i) = ' '
         names(i) (1:8) = 'vector_v'
         write( names(i) (9:10), 110 ) i-1
110      format(i2.2)
      end do
      textid(1) = ' '
      textid(2) = 'LEVELS'

!     --- The last vector gets name septimeincr
!         It is used to store the actual times for a time history

      names(maxvc1) = 'septimeincr'

      namefuncs = ' '
      namecomponents = ' '

!     --- cfilenames

      numfilenames = 0
      file_names = ' '

!     --- csepmain

      mainsubr = 0
      readconstants = 0

!     --- cellog

      matrix = .true.
      vector = .true.
      oldsol = .true.
      notmas = 1
      massrule = 0

!     --- cdebug_routs

      debug_mshsortel3d = .false.
      debug_mshcrt = .false.
      debug_mshfillkmeshad = .false.
      debug_mshsrf = .false.
      debug_pltrac = .false.
      debug_buildbf = .false.
      debug_prob01crobst = .false.
      debug_probdfbf = .false.
      debug_probflkprobaa = .false.
      debug_prorea = .false.
      debug_prinmtbf = .false.
      debug_compstat01 = .false.
      debug_extfreesubr = .false.
      debug_hollowmesh = .false.
      debug_initfree = .false.
      plotobst = .false.
      printobst = .false.
      plotskipel = .false.
      printskipel = .false.
      debug_mshobsfl = .false.
      debug_mshobstmarkpt = .false.
      debug_probskippoint = .false.
      checksign = .false.
      debug_mshobstmesh = .false.
      debug_erintpol = .false.
      check_hanging_nodes = .false.
      plotedges = .false.
      trace = .false.
      checksubtetrahedrons = .false.
      debug_mshobstpoint = .false.
      write_geomview = .false.

!     --- cdebug_routs

      startsurface = 1
      num_of_rotat = 1


      accuracy_obstacle = 0.3d0

!     --- cseqnr (sepmoduleinput)

      iseqadapbn = 0
      iseqchcoor = 0
      iseqcofs = 0
      iseqcontct = 0
      iseqcr = 0
      iseqinstfr = 0
      iseqintb = 0
      iseqloop = 0
      iseqnlns = 0
      iseqstatfr = 0
      iseqtimd = 0
      iseqsolv = 0
      iseqoutp = 0
      iseqinverse = 0
      iseqinti = 0
      iseqeigval = 0
      iseqderi = 0
      iseqcomp = 0
      iseqcomm = 0
      iseqcapacity = 0
      iseqadapms = 0
      iseqbc = 0
      iseqchcf = 0
      iseqarray = 0

!     --- cparallel

      iacnodes = 1
      inodenr = -1
      SP_ANY = 0
      SPINT = 0
      SPCHAR = 0
      SPDOUBLE = 0

!     --- ccontact

      contact_changed = .true.
      contact_ncontct = 0

!     --- cmacht

      nameout2 = 'sepcomp2.out'
      namediff = 'sepcompdiff.out'

!     --- citerinfo

      niterout = 0
      niteruout = 0
      niterpout = 0
      timesubr = 0d0

!     --- cstoc

      sigma = 0d0
      sigmax = 0d0
      sigmay = 0d0
      mu = 0d0
      mux = 0d0
      muy = 0d0
      rho = 0d0
      rstoc = 0d0

      ixstoc = 1

!     --- sepmoduletimhis

      ivecttimlast = 0
      nstephis = 0
      istephis = 0
      iscaltimlast = 0
      istephiscal = 0
      nstephiscal = 0
      do i = 1, maxvec+3
         vecttim(i)%npoint_hist = 0
      end do  ! i = 1, maxvec+3
      do i = 1, maxcls
         scaltim(i)%nstephiscal = 0
      end do  ! i = 1, maxvec+3

!     --- set iinfin

      iinfin = 2 ** 15
      if ( machin==7 ) then
         iinfin = 2 ** 23
      else
         iinfin = iinfin ** 2
      end if
      iinfin = 2*(iinfin-1)+1

!     --- cnparm

      call initcnparm

!     --- cmushy

      mushy = .false.
      tdelta = 0d0

!     --- cloop

      infloop = 0
      iseqfor = 0
      ifirstlp = 0
      isollp = 0

!     --- cexpression

      realexpress = 0d0
      intexpress = 0
      istexpress = 0
      numexpress = 0
      lastrealexpr = 0

!     --- cmumps

      firstmumps = .true.
      usemumps = .false.
      skipmumps = .false.

!     --- cpetsc

      iseqmat = 1  ! Default value

!     --- module loop

      whileloop = .false.
      seqwhile = 0

      lastmatrix = 0
      lastmesh = 0
      lastprob = 0
      lastintmat = 0
      lastsol = 0
      nextfreesol = 1
      nextfreekm = 1
      nextfreekp = 1
      nextfreemat = 1
      nextfreeintmt = 1
      removesol = .false.
      removeintmat = .false.
      itotit = 0

!     --- module sepmodulemain

      do i = 1, maxmesh
         do j = 1, maxmesh
            call sepclear ( map(i,j) )
         end do  ! j = 1, maxmesh
      end do  ! i = 1, maxmesh

!     --- module sepmodulevecs

      nullify(particinf, partics)

      call sepfillconsts
      call sepfillmodmesh

      end subroutine initcb
