      subroutine initmd
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-1999
!
!   copyright (c) 1999-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    The task of INITMD is to store machine dependent quantities in
!    common blocks
!    Subroutine INITMD must be adpated for each local computer
!    Important quantities to be updated are the quantities:
!    SPHOME and SPRENV
!    The concatenation of these two texts decides where the global
!    environment file is positioned
!    SPRENV gives the name of the local environment file
!    For unix computers as well as MSDOS computers running under FTN77/386
!    these names are set during the installation procedure
!    For other computers the system manager must set the parameters himself
!    This subroutine is machine dependent and must be adapted for
!    each local machine
!      Available versions:
!      Computer     Operating system     Comment    machine
!      IBM                               cibm         1
!      Cyber        NOS/VE               cybr         2
!      Apollo       Aegis/Unix           capo         3
!      HP 9000      HP/UX                chpu cuni    4
!      ....         Unix                 cuni         5
!      VAX          VMS                  cvax         6
!      Harris                            char         7
!      Cray                              cray         8
!      IBM PC 386   MSDOS+FTN77/386      cipc         9
!      Convex       Unix/Concentrix      ccon        10
!      ALLIANT      Unix/Concentrix      cali        13
! **********************************************************************
!
!                       KEYWORDS
!
!     initialization
!     machine_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecomio
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, timing(8)
      character (len=100) name
      integer lenhom
      character (len=5) zone
      logical :: check

!     iref           Reference number for the environment file
!     lenhom         Number of characters in the name of the home directory
!     name           Name of string read for the environment
!     timing         Dummy
!     zone           Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer spgenv

!     DATE_AND_TIME  Standard fortran 90 date and time subroutine
!     READMD         Reads the SEPRAN environment file and sets constants
!     SPGENV         C-subroutine to catch parameters from the environment
!     STMACH         Machine (installation) dependent subroutine that may be
!                    used to perform local actions not standard for the SEPRAN
!                    package
!     STUNIX         UNIX extra starting subroutine to set some names of files
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set the parameter nbuffr

      call date_and_time(date_start,time_start,zone,timing)

!     --- Set date and time
!         --- Read all parameters from the environment file
!         First the temporary reference number for this file is set
!         This reference number is made equal to 73 and is only used in READMD
!         The file is opened and closed in READMD, so the reference number
!         is free afterwards
!         If necessary 73 should be changed in some other number
!         Next the parameters SPHOME and SPRENV are set
!         SPHOME is only used to define the directory where the global
!         environment
!         file is positioned. This name must contain the / or \ extension
!         if this extension is used to identify a subdirectory
!         SPHOME is concatenated with SPRENV to define the name of the
!         global environment file
!         SPRENV gives the name of the local environment file
!         Implicitly it also defines the name of the global environment file
!         The names of SPHOME and SPRENV must be adapted at installation
!         For unix and msdos computers this is done automatically by the
!         installation procedure

      iref = 73
      SPHOME = ' '
      SPRENV = 'sepran.env'

      length = spgenv(1,name)
      if ( length.gt.0 ) then
         sphome(1:length) = name(1:length)
         sphome(length+1:length+8) = '/instal/'
         inquire ( file = sphome(1:length)//'/seprantest/run', exist=check )
         testversion = check
      else
         write(6,1)
  1      format ( ' No environment variable SPHOME found'/ &
                  ' Please make an environment variable SPHOME'/ &
                  ' and give this parameter the name of the SEPRAN', &
                  ' home directory'// &
                  ' For example in c-shell:', &
                  ' put the line'//' setenv SPHOME sepdir '// &
                  ' in your .cshrc file, where sepdir is the name of', &
                  ' the sepran home dir'/ &
                  ' In bourne shell change the .profile:'// &
                  ' SPHOME=sepdir'/ &
                  ' export sepdir'// &
                  ' Login out and login after changing this file'// &
                  ' Execution is terminated' )
         stop
      end if

!     --- Read the environment file

      call readmd ( iref )

!     --- Call a special procedure for local adaption of names due to
!         batch files (DOS) or shell scripts with settings (unix)
!         These subroutines are special for the operating system

      call stunix

!     --- Call machine (installation) dependent subroutine:

      call stmach

      end
