      subroutine msh070 ( kmeshj, npoint, irefc )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   31-12-1994
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   read kmesh part j from the file indicated by irefc
!
! ********************************************************************
!
!                      KEYWORDS
!
!    mesh
!    read
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
!
      implicit none
      integer npoint, irefc, kmeshj(npoint)

!     irefc   i    dataset reference number for reading (>0 formatted, &
!                  <0 unformatted)
!     kmeshj  o    KMESH part j, contains the new numbering of the nodes
!                  kmeshj(1) contains the first node according to the new
!                  numbering; kmeshj(2) the second one and so on
!     npoint  i    Number of nodal points
! ********************************************************************
!
!                    COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, ios, inodp, iwork(npoint)

!     i       Counting variable
!     inodp   Nodal point number
!     ios     IOSTAT variable, returns from reading or writing
!     iwork   integer work array
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     586   renumbered node read out of range
!     599   Renumbered node found more than once
!     765   End of file Reached of file containing mesh output
!     766   Read error in file containing mesh output
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      call eropen ( 'msh070' )
      if ( irefc.ge.0 ) then
         read(irefc,*,iostat=ios) (kmeshj(i), i=1,npoint)
      else
         read(-irefc,iostat=ios) (kmeshj(i), i=1,npoint)
      end if
      if ( ios.gt.0 ) then

!     --- ios > 0, Error in reading file

         call errint ( ios, 1 )
         call errsub ( 766, 1, 0, 0 )
         go to 1000

      else if ( ios.lt.0 ) then

!     --- ios > 0, End of file reached when reading file 10

         call errsub ( 765, 0, 0, 0)
         go to 1000

      end if

      do 100 i = 1, npoint
         iwork(i) = 0
100   continue

      do 110 i = 1, npoint
         inodp = kmeshj(i)
         if ( inodp.le.0 .or. inodp.gt.npoint ) then

!        --- Nodal point number out of range

            call errint ( i, 1 )
            call errint ( inodp, 2 )
            call errint ( npoint, 3 )
            call errsub ( 586, 3, 0, 0 )

         else if ( iwork(inodp).ne.0 ) then

!        --- Nodal point has already been found

            call errint ( i, 1 )
            call errint ( inodp, 2 )
            call errsub ( 599, 2, 0, 0 )

         else

!        --- Nodal point is new

            iwork(inodp) = 1

         end if

110   continue

1000  call erclos ( 'msh070' )
      end
