      subroutine mshsortelm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 21-11-2012 Long integers
!        version  1.3  date 22-08-2012 New calls to mshsortel2d/3d
!        version  1.2  date 12-02-2003 Replace ini060 by ini060text
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh parts intelmpos and realelmpos with information of in
!     which part region a node is positioned
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmeshac

!     lenkmeshac     Length of array kmeshac
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSORTEL2D    Fill array kmesh part ac with information of in which part
!                    of the region an element is positioned (2D)
!     MSHSORTEL3D    Fill array kmesh part ac with information of in which part
!                    of the region an element is positioned (3D)
!     SEP2DRMESHAC   Make a reference of the 2d array rbuffer2d to the pointer
!                    rmeshac
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( ndim=2 ) then
!       perform algorithm for two-dimensional grid
!    else if ( ndim=3 ) then
!       perform algorithm for three-dimensional grid
!    else
!       skip
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsortelm' )
      if ( ndim==1 ) go to 1000

!     --- Reserve work space and space for arrays kmeshac and rmeshac
!         iwork                 3*npoint
!         kmeshac    estimated: npoint+4*nsplit+10000*nsplit
!         rmeshac               4*nsplit reals

      lenkmeshac = npoint+(2+ndim)*nsplit+10000*nsplit

      if ( associated(km(iseqkm)%kmeshac) ) deallocate(km(iseqkm)%kmeshac)
      if ( associated(km(iseqkm)%rmeshac) ) deallocate(km(iseqkm)%rmeshac)
      allocate ( kmeshac(lenkmeshac), km(iseqkm)%rmeshac(2*ndim,nsplit), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenkmeshac+2*ndim*nsplit, 'kmeshac' )
      if ( ierror/=0 ) go to 1000

      rmeshac => km(iseqkm)%rmeshac

      if ( ndim==2 ) then

!     --- ndim = 2

         call mshsortel2d ( lenkmeshac )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call mshsortel3d ( lenkmeshac )

      end if

!     --- Make kmesh part ac permanent

      allocate (km(iseqkm)%kmeshac(lenkmeshac))
      km(iseqkm)%kmesh(51) = lenkmeshac
      km(iseqkm)%kmesh(67) = nsplit
      km(iseqkm)%kmeshac = kmeshac(1:lenkmeshac)

      deallocate ( kmeshac, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshac' )
      kmeshac => km(iseqkm)%kmeshac

1000  call erclos ( 'mshsortelm' )
      end subroutine mshsortelm
