      subroutine prcopsepvec2 ( isol, idegfd, func, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one degree of freedom of sepran vector in user vector
!     Only nodes in array nodes are used
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: idegfd, isol, nnodes
      double precision, intent(out) :: func(nnodes)

!     func           o    Contains the function values
!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     nnodes         i    Number of nodes in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPSEPVEC3   Copy one degree of freedom of sepran vector in user vector
!     PRCOPYVEC2US   Copy part of sepran vector to user vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopsepvec2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopsepvec2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and related arrays

      call sepactsolbf2 ( isol )

!     --- Compute starting addresses

      usol => ks(isol)%sol

      if ( indprfi==0 ) then

!     --- indprfi = 0, constant number of degrees per point

         call prcopyvec2us ( usol, func, nodes, nnodes, nunkpi, &
                             idegfd )

      else

!     --- indprfi>0, variable number of degrees per point

         call prcopsepvec3 ( func, usol, idegfd, nodes, nnodes )

      end if  ! ( indprfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopsepvec2'

      end if  ! ( debug )
      call erclos ( 'prcopsepvec2' )

      end
