      subroutine prcopysepuser ( isol, newvec, npoint, namevec, iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-04-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy a sepran solution vector with one component per point into the
!     iseqlc^th component of a user vector of size npoint x number_of_components
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), npoint, iseqlc
      double precision newvec(npoint,*)
      character (len=*) namevec

!     iseqlc         i    Sequence number in newvec to which solution in isol
!                         must be copied
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         isol correpsponds to a vector with one component
!     namevec        i    name of vector
!     newvec        i/o   array of length npoint x nunkp from which the iseq_th
!                         component is filled
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1059   number of degrees of freedom incorrect
!    2829   Number of degrees of freedom per point not constant for input vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopysepuser' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopysepuser'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call sepactsolbf2 ( isol )
      if ( ierror/=0 ) go to 1000

      if ( indprfi>0 ) then

!     --- Error indprfi>0, has not been implemented

         call errsub ( 2829, 0, 0, 0 )
         go to 1000

      end if  ! ( indprfi==0 )

      if ( nunkpi/=1 ) then

!     --- Error nunkpi#1, has not been implemented

         call errint ( nunkpi, 1 )
         call errint ( 1, 2 )
         call errchr ( namevec, 1 )
         call errsub ( 1059, 2, 0, 1 )
         go to 1000

      end if  ! ( indprfi==0 )

!     --- Compute starting addresses

      usoltemp => ks(isol(1))%sol

!     --- Copy vector

      newvec(:,iseqlc) = usoltemp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopysepuser'

      end if  ! ( debug )
      call erclos ( 'prcopysepuser' )

      end
