      function prderuserp ( iuserp, dirder )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-07-2015 Jump in case of error
!        version  1.0  date 05-11-2009
!
!   copyright (c) 2009-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the derivative of the solution vector in
!     a given user point in direction dirder
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuserp, dirder
      double precision prderuserp

!     dirder         i    indication what type of derivative must be computed
!                         See DERIVA
!     iuserp         i    User point sequence number
!     prderuserp     o    value of the derivative
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, iuser(1), iskpel(1), icurvs(1)
      integer, target :: ioutvc
      double precision user(1)

!                    they are not
!     icurvs         List of curves to be used
!     ioutvc         Local output vector
!     iskpel         Array containing the element groups to be skipped
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. (Dummy)
!     node           Node number
!     user           Real user array to pass user information from
!                    main program to subroutine. (Dummy)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prnodeuserp
      double precision prvalnode

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRNODEUSERP    Return with nodal point number corresponding to user point
!     PRVALNODE      Get the value of the solution vector in a given node
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get node number corresponding to user point
!     Compute derivatives
!     Get value in node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prderuserp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prderuserp'
         write(irefwr,1) 'iuserp, dirder', iuserp, dirder
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get node number corresponding to user point

      node = kmeshl(iuserp)
      if ( debug ) write(irefwr,1) 'node', node
      if ( ierror/=0 ) go to 1000

!     --- Compute derivatives

      iinder = 0         ! clear array
      ioutvc = 0         ! clear output vector
      iinder(1) = 4      ! calling subroutine
      iinder(2) = 1      ! output vector is of special structure
      iinder(4) = 1      ! icheld
      iinder(5) = dirder ! ix
      iinder(6) = 1      ! jdegfd
      iinder(7) = 1      ! ivec
      iinder(10) = 1     ! get from solution vector
      call sepfillsmf1 ( isolut2d, maxvec )
      SMF%ioutvc => ioutvc
      call to0100 ( iskpel, icurvs )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call sepprintv ( ioutvc, 'outvc' )

!     --- Get value in node

      prderuserp = prvalnode ( ioutvc, node )
      if ( ierror/=0 ) go to 1000

!     --- remove output vector

      call seprmsolinfo ( ioutvc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prderuserp'

      end if  ! ( debug )
      call erclos ( 'prderuserp' )

      end
