      subroutine prfindelm3 ( xgauss, igaussp, kmeshclc, iwork, &
                              iwork1, iwork2, isplit, jnpelm, &
                              nvertices, jelem, jelemlow, jelemtop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 25-08-2000 Change of parameter list
!        version  1.0  date 25-03-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find element number in which point with coordinates (x,y,Z) is lying
!   Three-dimensional case
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     element
!     find
!     find_element
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletypes
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xgauss(m,3)
      integer          jnpelm, kmeshclc(jnpelm,*), iwork(*), iwork1(*), &
                       iwork2(*), nvertices, isplit, jelem, &
                       jelemlow, jelemtop, igaussp

!     jelem          o    Element number
!     jelemlow       i    Smallest element number in group
!     jelemtop       i    Largest element number in group
!     igaussp        i    Sequence number of gauss point
!     jnpelm         i    number of points per element
!     isplit         i    Sequence number of discontinuous part
!     iwork               integer work array of length npointlc
!                         In this array all node numbers of a part are stored
!     iwork1              integer work array of length nelem
!                         iwork1(i) = 0, means element i has not yet been used
!                         iwork1(i) = 1, means element i has been used
!                         In this array all element numbers of a part are stored
!     iwork2              integer work array of length nelem
!     kmeshclc       i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     nvertices      i    Number of vertices
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision phi(8), x(8), y(8), z(8), xmin, xmax, ymin, ymax, &
                       zmin, zmax, eps, distx, disty,  distz, xsi0, &
                       eta0, zeta0, xlocmax, xlocmin, &
                       ylocmax, ylocmin, zlocmax, zlocmin, &
                       xcoord, ycoord, zcoord
      integer          i, j, k, nodp, nodp1, nx, ny, indi, indj, indk, &
                       npointlc, nnodes, istartpart2, istart, indl, &
                       indilow, inditop, indjlow, indjtop, l, jcount, &
                       index1(8), ivertx(8), icell, jcell, kcell, &
                       indklow, indktop, nz
      logical          found

!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     distz          Distance in z-direction
!     eps            Local accuracy
!     eta0           Value of eta in previous iteration
!     found          If true already a possibility has been found
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     jcount         Counter
!     index1         Array of length jnpelm containing the point numbers
!                    of the nodal points in the element
!     indi           i-index
!     indilow        lower i-index
!     inditop        upper i-index
!     indj           j-index
!     indjlow        lower j-index
!     indjtop        upper j-index
!     indk           k-index
!     indklow        lower k-index
!     indktop        upper k-index
!     indl           l-index
!     istart         First position used at a part
!     istartpart2    First position used at part 2
!     ivertx         Array containing the position of the vertices in the
!                    elements
!     j              Counting variable
!     jcell          Cell number of cell under consideration
!     k              Counting variable
!     kcell          Cell number of cell under consideration
!     l              General loop variable
!     nnodes         Number of nodes
!     nodp           Nodal point sequence number
!     nodp1          Node sequence number
!     npointlc       Local number of points
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     x              x co-ordinate of point
!     xcoord         x-coordinate of point
!     xlocmax        local maximum x coordinate
!     xlocmin        local minimum x coordinate
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xsi0           Value of xsi in previous iteration
!     y              y co-ordinate of point
!     ycoord         y-coordinate of point
!     ylocmax        local maximum y coordinate
!     ylocmin        local minimum y coordinate
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zcoord         z-coordinate of point
!     zeta0          Value of zeta in previous iteration
!     zlocmax        local maximum z coordinate
!     zlocmin        local minimum z coordinate
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP630INTPOL   Compute the values of the basis functions in a given
!                    point for a linear tetrahedron
!     ELP632INTPOL   Compute the values of the basis functions in a given
!                    point for a tri-linear hexahedron
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2150   no element found containg this point
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork1
!    Compute cell-index (i,j) in which point (x,y) is lying
!    finished := false
!    for k := 0 (1) nx, while ( not finished ) do
!       Find all nodes in cell (i+/-k,j+/-k), store in iwork
!       Find all elements in element groups ielgrp1 to ielgrp2 corresponding
!       to these nodes, store in iwork2
!       For all elements in the group, while ( not finished ) do
!         if ( point in element ) then
!            jelemfound := element number
!            finished := true
!
!    To check if a point is inside an element we distinguish between
!    triangles and quadrilaterals
!
!    Triangles:
!
!    The three vertices define three linear basis functions phi_i(x,y)
!    with:
!    phi_i(x,y) = a_0 + a_1 x + a_2 y
!    The computation of these basis functions is compted in elp620intpol
!    The point (xcoord,ycoord) is in the triangle if
!    0 <=phi_i(xcoord,ycoord) <= 1
!
!    Quadrilaterals:
!
!    The four vertices define four bi-linear basis functions phi_i(x,y)
!    The point xsi, eta defines the position of (xcoord,ycoord) in the
!    reference square
!    This is computed in elp621intpol
!    If xsi and eta in the interval (0,1) then (x,y) is in the quadrilateral
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindelm3' )

!     --- Store coordinates of point in (xcoord,ycoord,zcoord)

      xcoord = xgauss(igaussp,1)
      ycoord = xgauss(igaussp,2)
      zcoord = xgauss(igaussp,3)

!     --- Clear array iwork1

      do i = 1, nelem
         iwork1(i) = 0
      end do

!     --- Compute cell-index (i,j) in which point (x,y) is lying

      xmin = rmeshac(1,isplit)
      xmax = rmeshac(2,isplit)
      ymin = rmeshac(3,isplit)
      ymax = rmeshac(4,isplit)
      zmin = rmeshac(5,isplit)
      zmax = rmeshac(6,isplit)
      eps = 1d-6
      nx = kmeshac((isplit-1)*5+2)
      ny = kmeshac((isplit-1)*5+3)
      nz = kmeshac((isplit-1)*5+4)
      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny
      distz = (zmax-zmin)/nz
      indi = min(nx,int((xcoord-xmin)/distx)+1)
      indj = min(ny,int((ycoord-ymin)/disty)+1)
      indl = min(nz,int((zcoord-zmin)/distz)+1)
      istart = kmeshac((isplit-1)*5+5)
      if ( jnpelm<=8 ) then

!     --- Linear elements

         do k = 1, nvertices
            ivertx(k) = k
         end do

      else if ( nvertices==8 ) then

!     --- Triquadratic hexahedron

         do k = 1, nvertices
            ivertx(k) = ivertx14(k)
         end do

      else

!     --- Quadratic tetrahedron

         do k = 1, nvertices
            ivertx(k) = ivertx12(k)
         end do

      end if

      do k = 0, nx-1

!     --- for k := 0 (1) nx-1, while ( not finished ) do
!         Find all nodes in cell (i+/-k,j+/-k), store in iwork

         indilow = max(1,indi-k)
         inditop = min(nx,indi+k)
         indjlow = max(1,indj-k)
         indjtop = min(ny,indj+k)
         indktop = min(nz,indl+k)
         indklow = max(1,indl-k)
         nnodes = 0
         do icell = indilow, inditop
            do jcell = indjlow, indjtop
               do kcell = indklow, indktop
                  indk = (kcell-1)*nx*ny + (jcell-1)*nx+icell
                  istartpart2 = kmeshac(istart+indk-1)
                  npointlc = kmeshac(istart+indk)-istartpart2
                  do jcount = 1, npointlc
                     iwork(nnodes+jcount) = &
                        kmeshac(istartpart2+jcount-1)
                  end do
                  nnodes = nnodes+npointlc
               end do
            end do
         end do
         inelem = 0

         do i = 1, nnodes

!        --- Find all elements in element groups ielgrp1 to ielgrp2
!            corresponding to these nodes, store in iwork2

            nodp1 = iwork(i)
            do j = kmshd1(nodp1)+1, kmshd1(nodp1+1)
               jelem = kmshd2(j)
               if ( jelem>=jelemlow .and. jelem<=jelemtop .and. &
                    iwork1(jelem)==0 ) then
                  inelem = inelem+1
                  iwork1(jelem) = 1
                  iwork2(inelem) = jelem
               end if
            end do

!           --- Clear array iwork1

            do j = 1, inelem
               iwork1(iwork2(j)) = 0
            end do

!           --- For all elements in the group, while ( not finished ) do
!               if ( point in element ) then
!                  jelemfound := element number
!                  finished := true

            do j = 1, inelem
               jelem = iwork2(j)
               xlocmin = xmax
               xlocmax = xmin
               ylocmin = ymax
               ylocmax = ymin
               zlocmin = zmax
               zlocmax = zmin
               do l = 1, nvertices
                  index1(l) = kmeshclc(ivertx(l),jelem-jelemlow+1)
                  nodp = index1(l)
                  x(l) = coor(1,nodp)
                  y(l) = coor(2,nodp)
                  z(l) = coor(3,nodp)
                  xlocmin = min(xlocmin,x(l))
                  xlocmax = max(xlocmax,x(l))
                  ylocmin = min(ylocmin,y(l))
                  ylocmax = max(ylocmax,y(l))
                  zlocmin = min(zlocmin,z(l))
                  zlocmax = max(zlocmax,z(l))
                  index1(l) = nodp
               end do
               if ( xcoord<xlocmin .or. xcoord>xlocmax .or. &
                    ycoord<ylocmin .or. ycoord>ylocmax .or. &
                    zcoord<zlocmin .or. zcoord>zlocmax ) go to 100
               if ( nvertices==4 ) then

!              --- tetrahedra

                  call elp630intpol ( coor, index1, phi, xcoord, &
                                      ycoord, zcoord )
                  found = phi(1)>-eps .and. phi(1)<=1d0+eps .and. &
                          phi(2)>-eps .and. phi(2)<=1d0+eps .and. &
                          phi(3)>-eps .and. phi(3)<=1d0+eps .and. &
                          phi(4)>-eps .and. phi(4)<=1d0+eps

               else

!              --- hexahedra
!                  Compute factors for trilinear interpolation
!                  x = a0 + a1 xsi + a2 eta + a3 zeta + a4 xsi eta zeta

                  call elp632intpol ( coor, index1, phi, xcoord, ycoord, &
                                      zcoord, xsi0, eta0, zeta0 )
                  found = xsi0>-eps .and. xsi0<=1d0+eps .and. &
                          eta0>-eps .and. eta0<=1d0+eps .and. &
                          zeta0>-eps .and. zeta0<=1d0+eps

               end if
               if ( found ) go to 1000

100         end do
         end do

      end do
      call errint ( jelemlow, 1 )
      call errint ( jelemtop, 2 )
      call erreal ( xcoord, 1 )
      call erreal ( ycoord, 2 )
      call errsub ( 2150, 2, 2, 0 )

1000  call erclos ( 'prfindelm3' )

      end
