      subroutine prgetvalarr ( isol, nnodes, idegfd, uservec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of a sepran solution array into a user
!     array for a set of nodes and a specific degree of freedom
! **********************************************************************
!
!                       KEYWORDS
!
!     get_parameter
!     value
!     variable
!     array
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, idegfd, isol
      double precision uservec(nnodes)

!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         The corresponding array contains the values that must
!                         be copied
!     nnodes         i    Number of nodes in array nodes
!     uservec        o    The values are copied in this array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCOPARR   Copy the contents of array uold into array unew for
!                    all nodes stored in array nodes and the degrees of
!                    freedom indicated in kdeg
!                    uold a sepran array, unew is a user array
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgetvalarr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgetvalarr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and corresponding arrays

      call sepactsolbf2 ( isol )
      if ( ierror/=0 ) go to 1000

!     --- Get starting addresses

      isoltemp => ks(isol(1))%isol
      uold => usoltemp

      call erfillcoparr ( idegfd, uservec, nnodes )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgetvalarr'

      end if  ! ( debug )
      call erclos ( 'prgetvalarr' )

      end
