      subroutine prmakevector1 ( vector1, npoint, nunkp1, vector2, &
                                 nunkp2, vector3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-03-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new vector from two separate vectors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkp1, nunkp2, npoint
      double precision vector1(nunkp1,npoint), vector2(nunkp2,npoint), &
                       vector3(nunkp1+nunkp2,npoint)

!     npoint         i    Number of nodal points
!     nunkp1         i    number of unknowns per point in vector 1
!     nunkp2         i    number of unknowns per point in vector 2
!     vector1        i    First input vector
!     vector2        i    Second input vector
!     vector3        o    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakevector1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakevector1'
         call prinrl1 ( vector1, npoint, nunkp1, 'vector1' )
         call prinrl1 ( vector2, npoint, nunkp2, 'vector2' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      vector3(1:nunkp1,1:npoint) = vector1(1:nunkp1,1:npoint)
      vector3(nunkp1+1:nunkp1+nunkp2,1:npoint) = &
          vector2(1:nunkp2,1:npoint)

1000  call erclos ( 'prmakevector1' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( vector3, npoint, nunkp1+nunkp2, 'vector3' )
         write(irefwr,*) 'End prmakevector1'

      end if  ! ( debug )

      end
