      subroutine prsort ( srtkey, kgrad, npoint )
! ======================================================================
!
!        programmer    Jos van Kan/Onno Hoitinga
!        version  3.1  date 09-06-2001 Return if npoint = 0
!        version  3.0  date 25-01-1995 Complete new subroutine, programmed
!                                      by Onno Hoitinga
!        version  2.1  date 22-11-1990 Replace declarations to prevent errors
!        version  2.0  date 12-05-1989 Complete revision, with extra parameters
!        version  1.0  date 21-06-1987
!
!   copyright (c) 1987-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   This routine gets as input array srtkey of length npoint
!   on output array kgrad contains the indices to sort array srtkey
!   The Heap sort algorithm is used
! **********************************************************************
!
!                       KEYWORDS
!
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kgrad(npoint)
      double precision srtkey(*)

!     kgrad    o    contains permutation index that sorts srtkey
!                   in ascending order on output.
!     npoint   i    number of entries in srtkey & kgrad
!     srtkey   i    sortkey (integer) on array to be sorted.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, l, ir, kgradt
      double precision q

!     i       Counting variable
!     ir      ?
!     j       Counting variable
!     kgradt  ?
!     l       General loop variable
!     q       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!  This routine originates from Numerical Recipes (fortran edition)
!     Cambridge University Press 1989
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( npoint.le.0 ) go to 1000
      do j = 1, npoint
         kgrad(j) = j
      end do
      if ( npoint.eq.1 ) go to 1000
      l = npoint/2+1
      ir = npoint

200     if ( l.gt.1 ) then
          l = l-1
          kgradt = kgrad(l)
          q = srtkey(kgradt)
        else
          kgradt = kgrad(ir)
          q = srtkey(kgradt)
          kgrad(ir) = kgrad(1)
          ir = ir-1
          if ( ir.eq.1 ) then
            kgrad(1) = kgradt
            go to 1000
          end if
        end if
        i = l
        j = l+l
300     if ( j.le.ir ) then
          if ( j.lt.ir ) then
            if ( srtkey(kgrad(j)).lt.srtkey(kgrad(j+1)) ) j = j+1
          end if
          if ( q.lt.srtkey(kgrad(j)) ) then
            kgrad(i) = kgrad(j)
            i = j
            j = j+j
          else
            j = ir+1
          end if
        go to 300
        end if
        kgrad(i) = kgradt
      go to 200

1000  end
