      subroutine prvecfillc ( iseqvec, vecs, numvecrhs, vectors, leng )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all vectors of right-hand side of expression in array vectors
!     It is assumed that the complete vectors must be filled
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvec, vecs(*), numvecrhs, leng
      double complex vectors(leng,numvecrhs)

!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     leng           i    Length of the vectors to be filled
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size leng x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseqv

!     i              Counting variable
!     iseqv          Sequence number of keyword V in array code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfillc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfillc'
         call prinin ( vecs, numvecrhs, 'vecs' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      vectors = 0d0

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)
         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector
!            activate array


!           --- Find starting address

            usoltemp => ks(isolut2d(iseqv))%sol

!           --- Copy vector

            if ( ks(isolut2d(iseqv))%compl==1 ) then

!           --- Complex vector

               call dcopy ( 2*leng, usoltemp, 1, vectors(1,i), 1 )

            else

!           --- Real vector, skip imaginary part

               call dcopy ( leng, usoltemp, 1, vectors(1,i), 2 )

            end if  ! ( ks(isolut2d(iseqv))%compl==1 ) then

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates
!            Copy vector

            call dcopy ( leng, coor, 1, vectors(1,i), 2 )

         else if ( iseqv<maxvc1+1+ndim ) then

!        --- One of the coordinates

            iseqv = iseqv-maxvc1-1

!           --- Copy vector

            call dcopy ( leng, coor(iseqv,1), ndim, vectors(1,i), 2 )

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( vectors, 2*leng, numvecrhs, 'vectors' )
         write(irefwr,*) 'End prvecfillc'

      end if  ! ( debug )
      call erclos ( 'prvecfillc' )

      end subroutine prvecfillc
