      subroutine putnewint ( intname, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-04-2012 Indexed constants
!        version  1.0  date 08-05-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the integer in the block constants with name intname
!     If the name does not exist a new entry is created
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     put
!     integer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname
      integer value

!     intname       i    Name of integer
!     value         i    Value of integer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intseqnm, nch
      character (len=80) iname

!     iname          Name of integer translated to capitals
!     intseqnm       Sequence number of integer with respect to array
!                    chincons
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint

!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETNAMEINT     Return with sequence number of integer in incons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1610   Too much integers read
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chincons
!     If ( name not in chincons ) then
!        create new entry
!     put value in common
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'putnewint' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from putnewint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intseqnm = getnameint ( intname )
      if ( intseqnm<=0 ) then

!     --- Name has not been stored in common
!         Add name

         numints = numints+1         ! raise numints
         if ( numints>maxints ) then

!        --- Too many integers to store

            call errint ( numints, 1 )
            call errint ( maxints, 2 )
            call errsub ( 1610, 2, 0, 0 )
            go to 1000

         end if  ! ( numints>maxints )
         intseqnm = numints          ! set intseqnm
         nch = index ( intname, ' ' ) -1      ! number of characters
         if ( nch==-1 ) nch = len(intname)
         iname = intname(1:nch)
         call askupc(iname)         ! translate into upper case
         chincons(intseqnm) = iname(1:nch)  ! fill name
         ncharcons(intseqnm) = nch   ! number of characters of constant

      end if  ! ( intseqnm<=0 )

!     --- Put value in common

      incons(intseqnm) = value

1000  call erclos ( 'putnewint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End putnewint'

      end if  ! ( debug )

      end
