      subroutine readb2 ( numarr, kmesh )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.4  date 15-11-2012 Long integers
!        version  5.3  date 01-08-2001 new definition kmesh part h
!        version  5.2  date 04-04-2001 Adaptation for empty surfaces
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    readb2 reads array KMESH from backing storage (file 2)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekmesh
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, kmesh

!    kmesh      o    Standard SEPRAN array containing information of the mesh
!    numarr     i    Array number with respect to the index in common cfile2
!                    for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer index, icheck, lenglc, leng, &
              i, j, leng1, leng2, leng3, leng4, &
              leng5, npointlc, istart
      double precision dum(1)
      character *9 text
      integer, allocatable :: iwork(:)

!     dum            Dummy array
!     i              General loop variable
!     icheck         Indication of the last version of writbs writb2) has been
!                    used (<=-1000000) or not (>-1000000)
!     index          Record number with respect to file 2 from which array
!                    must be read
!     istart         Start position in iwork
!     iwork          integer work array to store records read
!     j              General loop variable
!     leng           Help variable to compute the length of some arrays
!     leng1          Length of first part of surface in kmeshh
!     leng2          Length of second part of surface in kmeshh
!     leng3          Length of third part of surface in kmeshh
!     leng4          Length of fourth part of surface in kmeshh
!     leng5          Length of fifth part of surface in kmeshh
!     lenglc         Length of local array
!     npointlc       Local number of points
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     READFL         Read integer array from a direct access SEPRAN file
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     775   The array KMESH read has been written by an old version of WRITBS
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb2' )
      leng = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

      allocate ( iwork(leng), stat = error )
      if ( error/=0 ) call eralloc ( error, leng, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      istart = 1
      call readfl ( index, 1, leng, 1, 2, iwork )
      if ( ierror/=0 ) go to 1000

!     --- Replace all information
!         First KMESH

      call sepcrmeshinfo ( kmesh )
      if ( ierror/=0 ) go to 1000

      leng = 80
      km(iseqkm)%kmesh(1:leng) = iwork(1:leng)
      istart = istart + leng

!     --- Read kmesh parts c, d and e and put in array IBUFFR

      icheck = iwork(istart)
      if ( icheck<-1000003 .or. icheck>-1000000 ) &
         call errwar ( 775, 0, 0, 0 )

      lenglc = iwork(istart+1)
      istart = istart + 2
      allocate(km(iseqkm)%kmeshc(lenglc))
      km(iseqkm)%kmeshc = iwork(istart:istart+lenglc-1)
      istart = istart + lenglc

      lenglc = iwork(istart+1)
      istart = istart + 2
      allocate(km(iseqkm)%kmeshd(lenglc))
      km(iseqkm)%kmeshd = iwork(istart:istart+lenglc-1)
      istart = istart + lenglc

      lenglc = iwork(istart+1)
      istart = istart + 2
      allocate(km(iseqkm)%kmeshe(lenglc))
      km(iseqkm)%kmeshe = iwork(istart:istart+lenglc-1)
      istart = istart + lenglc

      if ( km(iseqkm)%kmesh(22)>10000000 ) then

!     --- Read kmesh part h and put in array IBUFFR

         icheck = iwork(istart)
         lenglc = iwork(istart+1)/ndim
         istart = istart + 2

         allocate(km(iseqkm)%kmeshh(ndim,lenglc))
         do i = 1, ndim
            km(iseqkm)%kmeshh(i,1:lenglc) = &
              iwork(istart+i-1:istart+lenglc*ndim-1:ndim)
         end do  ! i = 1, ndim
         kmeshh = 0

!        --- First 6 positions

         istart = istart + lenglc
         ncurvs = km(iseqkm)%kmesh(11)
         leng = 2*ncurvs
         allocate(km(iseqkm)%icurvsmsh(leng))
         km(iseqkm)%icurvsmsh = iwork(istart:istart+leng-1)
         istart = istart + leng
         leng = icurvsmsh(ncurvs)
         allocate(km(iseqkm)%curvesmsh(km(iseqkm)%kmesh(6),leng))

         call dcopy ( leng, iwork(istart), 1, km(iseqkm)%curvesmsh, 1 )
         istart = istart + leng*intlen
         leng = leng/km(iseqkm)%kmesh(6)
         allocate(km(iseqkm)%curvefacts(leng))
         call dcopy ( leng, iwork(istart), 1, km(iseqkm)%curvefacts, 1 )
         istart = istart + leng*intlen

      else
         km(iseqkm)%kmesh(22) = 0
      end if

!     --- Read kmesh part i - s, iinput and rinput in array IBUFFR

      lenglc = iwork(istart+1)
      leng = lenglc/intlen
      istart = istart + 2
      allocate(km(iseqkm)%coor(ndim,leng/ndim))
      call dcopy ( leng, iwork(istart), 1, km(iseqkm)%coor, 1 )
   !   call sepdcopy ( iwork(istart), km(iseqkm)%coor, leng )
      istart = istart + lenglc

      if ( km(iseqkm)%kmesh(24)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshj(lenglc))
         km(iseqkm)%kmeshj = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(24)/=0 )

      if ( km(iseqkm)%kmesh(25)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshk(lenglc))
         km(iseqkm)%kmeshk = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(25)/=0 )

      if ( km(iseqkm)%kmesh(26)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshl(lenglc))
         km(iseqkm)%kmeshl = iwork(istart:istart+lenglc-1)
         km(iseqkm)%kmesh(26) = lenglc
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(26)/=0 )

      if ( km(iseqkm)%kmesh(27)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshm(lenglc))
         km(iseqkm)%kmeshm = iwork(istart:istart+lenglc-1)
         km(iseqkm)%kmesh(27) = lenglc
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(27)/=0 )

      if ( km(iseqkm)%kmesh(28)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshn(lenglc))
         km(iseqkm)%kmeshn = iwork(istart:istart+lenglc-1)
         km(iseqkm)%kmesh(28) = lenglc
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(28)/=0 )

      if ( km(iseqkm)%kmesh(29)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmesho(lenglc))
         km(iseqkm)%kmesho = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(29)/=0 )

      km(iseqkm)%kmesh(30) = 0
      if ( km(iseqkm)%kmesh(31)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshq(lenglc))
         km(iseqkm)%kmeshq = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(31)/=0 )

      if ( km(iseqkm)%kmesh(32)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshr(lenglc))
         kmeshr => km(iseqkm)%kmeshr
         kmeshr = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(32)/=0 )

      if ( km(iseqkm)%kmesh(33)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%kmeshs(lenglc))
         km(iseqkm)%kmeshs = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(33)/=0 )

      if ( km(iseqkm)%kmesh(34)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2

         allocate(km(iseqkm)%iinputkm(lenglc))
         km(iseqkm)%iinputkm = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc
      end if  ! ( km(iseqkm)%kmesh(34)/=0 )

      if ( km(iseqkm)%kmesh(35)/=0 ) then
         lenglc = iwork(istart+1)
         istart = istart + 2
         leng = lenglc/intlen

         allocate(km(iseqkm)%rinputkm(leng))
         call dcopy ( leng, iwork(istart), 1, km(iseqkm)%rinputkm, 1 )
         istart = istart + lenglc

      end if  ! ( km(iseqkm)%kmesh(35)/=0 )
      if ( ierror/=0 ) go to 1000
      km(iseqkm)%kmesh(36:80) = 0

!     --- Set KMESH part R3 to -1 and give warning if KMESH part r exists

      if ( km(iseqkm)%kmesh(32)/=0 ) then

!     --- KMESH part r 3

         nsurfs = km(iseqkm)%kmesh(12)
         leng = nsurfs+1+kmeshr(1+nsurfs)
         lenglc = size ( kmeshr ) - leng
         kmeshr(leng+1:leng+lenglc) = -1

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      call sepfillkmesh(kmesh)

1000  call erclos ( 'readb2' )
      end subroutine readb2
