      subroutine readb3 ( numarr )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.3  date 15-11-2012 Long integers
!        version  4.2  date 04-04-2001 Corrections for more problems
!        version  4.1  date 27-06-2000 New definitions
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    readb3 reads array  kprob from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleprobinf
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr

!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer index, lenglc, kprbe1, leng, istart, &
              k, lengol, ivec, i
      double precision dum(1)
      character *9 text
      integer, allocatable :: iwork(:)

!     dum            Dummy array
!     i              Counting variable
!     index          Record number with respect to file 2 from which array
!                    must be read
!     istart         Start position in array iwork
!     ivec           Indication of the structure of a vector of special structure
!     iwork          integer work array to store records read
!     k              General loop variable
!     kprbe1         Address in KPROB part e
!     leng           Help variable to compute the length of some arrays
!     lenglc         Length of local array
!     lengol         Help variable to store KPROB(1) as stored by the user
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCOPY         Copy one integer array into another
!     PRININ         print 1d integer array
!     READFL         Read integer array from a direct access SEPRAN file
!     SEPCRPROBINFO  create a new array in kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readb3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      leng = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

      if ( debug ) write(irefwr,1) 'leng, index', leng, index

      allocate ( iwork(leng), stat = error )
      if ( error/=0 ) call eralloc ( error, leng, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      call readfl ( index, 1, leng, 1, 2, iwork )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*)  'after readfl'

!     --- Replace all information
!         First Kprob

      call sepcrprobinfo ( kprob, iwork(3) )
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob
      nprob = kp(iseqkp)%nprob
      lengol = iwork(4)

      allocate ( kp(iseqkp)%iinputkp(lengol) )
      kp(iseqkp)%iinputkp = iwork(6:5+lengol)

      istart = 6+lengol
      if ( iwork(5)>0 ) then

!     --- rinputkp exists

         lengol = iwork(5)/2
         allocate ( kp(iseqkp)%rinputkp(lengol) )
         call dcopy ( lengol, iwork(istart), 1, kp(iseqkp)%rinputkp, 1 )
         istart = istart+2*lengol

      end if  ! ( iwork(5)>0 )
      do iprob = 1, nprob
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         lengol = iwork(istart+2)
         kprobloc(1:lengol) = iwork(istart:istart+lengol-1)
         istart = istart+lengol
         if ( debug ) call prinin ( kprobloc, lengol, 'kprobloc' )
      end do
      if ( debug ) write(irefwr,1) 'istart/1', istart

!     --- Loop over problems

      do iprob = 1,nprob

!     --- Compute important constants

         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         nunkp  = kprobloc(4)
         ndtvec = kprobloc(6)
         kelgrp = kprobloc(39)
         nphys  = kprobloc(33)

         numnat = kprobloc(12)
         nbcons = kprobloc(32)

!        --- Kprob part a

         indpra = kprobloc(14)
         if ( indpra>0 ) then
            allocate ( kprobpart%kproba(nelgrp+numnat-nbcons) )
            kproba => kprobpart%kproba
            do i = 1, nelgrp+numnat-nbcons
               lenglc = iwork(istart+1)
               allocate(kproba(i)%arr(lenglc))
               kprobatmp => kproba(i)%arr
               istart = istart + 2
               kprobatmp = iwork(istart:istart+lenglc-1)
               istart = istart + lenglc
            end do  ! i = 1, nelgrp+numnat-nbcons
         end if  ! ( indpra>0 )

!        --- Kprob part b

         indprb = kprobloc(15)
         if ( indprb>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobb(lenglc) )
            kprobb => kprobpart%kprobb
            istart = istart + 2
            kprobb = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
         end if  ! ( indpra>0 )

!        --- Kprob part c

         lenglc = iwork(istart+1)
         allocate ( kprobpart%kprobc(lenglc) )
         kprobc => kprobpart%kprobc
         istart = istart + 2
         kprobc = iwork(istart:istart+lenglc-1)
         istart = istart + lenglc

!        --- Kprob part d

         indprd = kprobloc(17)
         if ( indprd>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobd(lenglc) )
            kprobd => kprobpart%kprobd
            istart = istart + 2
            kprobd = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
         end if  ! ( indpra>0 )

!        --- Kprob part e

         allocate ( kprobpart%kprobe(kelgrp,ndtvec) )
         allocate ( kprobpart%kprobe2(ndtvec),  kprobpart%kprobe3(ndtvec) )
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
         lenglc = iwork(istart+1)
         istart = istart + 2
         call incopy ( iwork(istart), kprobe, lenglc )
         istart = istart + lenglc
         lenglc = iwork(istart+1)
         istart = istart + 2
         call incopy ( iwork(istart), kprobe2, lenglc )
         istart = istart + lenglc
         if ( ndtvec>0 ) then

!        --- ndtvec>0  kprob part e

            do ivec = 1,ndtvec
               indpre = kprobe2(ivec)
               nullify(kprobpart%kprobe3(ivec)%arr)
               if ( indpre>0 ) then

!              --- indpre>0  hence part in ibuffr or on file 3

                  lenglc = iwork(istart+1)
                  if ( debug ) &
                     write(irefwr,1) 'ivec, istart, lenglc', &
                                      ivec, istart, lenglc
                  istart = istart + 2
                  kprobe2(ivec) = 0
                  write(text,110) ivec
110               format('indpre', i3.3)
                  allocate ( kprobpart%kprobe3(ivec)%arr(lenglc) )
                  if ( ierror/=0 ) go to 1000
                  kprobe3(ivec)%arr = iwork(istart:istart+lenglc-1)
                  istart = istart + lenglc

               end if

            end do

         end if
         if ( debug ) write(irefwr,1) 'istart/2', istart

!        --- Kprob parts f

         indprf = kprobloc(19)
         if ( indprf>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobf(lenglc) )
            kprobf => kprobpart%kprobf
            istart = istart + 2
            kprobf = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
         end if  ! ( indprf>0 )
         if ( debug ) write(irefwr,1) 'istart/3', istart

!        --- Kprob part i

         indpri = kprobloc(22)
         if ( indpri>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobi(lenglc) )
            kprobi => kprobpart%kprobi
            istart = istart + 2
            kprobi = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
         end if  ! ( indpri>0 )
         if ( debug ) write(irefwr,1) 'istart/4', istart

!        --- Kprob part j

         if ( kprobloc(23)>0 ) then
            lenglc = iwork(istart+1)/2
            istart = istart + 2
            allocate ( kprobpart%kprobj(lenglc,2) )
            kprobj => kprobpart%kprobj
            kprobj(:,1) = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
            kprobj(:,2) = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
         end if  ! ( indpra>0 )

!        --- Kprob part n

         if ( kprobloc(27)>0 ) then

            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobn(ndim,lenglc/ndim) )
            kprobn => kprobpart%kprobn
            istart = istart + 2
            call incopy ( iwork(istart), kprobn, lenglc )
            istart = istart + lenglc

            maxtrn = kprobloc(45)
            lenglc = iwork(istart+1)
            allocate ( kprobpart%coortr(maxtrn,lenglc/(maxtrn*intlen)) )
            coortr => kprobpart%coortr
            istart = istart + 2
            call incopy ( iwork(istart), coortr, lenglc )
            istart = istart + lenglc

         end if
         if ( debug ) write(irefwr,1) 'istart/5', istart

!        --- Kprob part o

         if ( kprobloc(28)>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobo(lenglc) )
            kprobo => kprobpart%kprobo
            istart = istart + 2
            kprobo = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
         end if  ! ( indpra>0 )

!        --- Kprob part p

         if ( kprobloc(35)>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobp(npoint,lenglc/npoint) )
            kprobp => kprobpart%kprobp
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobp, lenglc )
            istart = istart + lenglc

         end if
         if ( debug ) write(irefwr,1) 'istart/6', istart

!        --- Kprob part s

         if ( kprobloc(38)>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobs(lenglc) )
            kprobs => kprobpart%kprobs
            istart = istart + 2
            kprobs = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
            lenglc = iwork(istart+1)/(2*intlen)
            allocate ( kprobpart%rprobs(lenglc,2) )
            rprobs => kprobpart%rprobs
            lenglc = lenglc*2*intlen
            istart = istart + 2
            call incopy ( iwork(istart), rprobs, lenglc )
            istart = istart + lenglc
         end if  ! ( kprobloc(38)>0 )

         if ( debug ) write(irefwr,1) 'istart/8', istart

!        --- Kprob part v

         if ( kprobloc(46)>0 .and. iprob==1 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobvp1(nsurfs,lenglc/nsurfs) )
            kprobvp1 => kprobpart%kprobvp1
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobvp1, lenglc )
            istart = istart + lenglc
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobvp2(lenglc) )
            kprobvp2 => kprobpart%kprobvp2
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobvp2, lenglc )
            istart = istart + lenglc

         end if  ! ( kprobloc(46)>0 )
         if ( debug ) write(irefwr,1) 'istart/10', istart

!        --- Kprob part w

         if ( kprobloc(47)>0 .and. iprob==1 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobwp1(nsurfs,lenglc/nsurfs) )
            kprobwp1 => kprobpart%kprobwp1
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobwp1, lenglc )
            istart = istart + lenglc
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobwp2(lenglc) )
            kprobwp2 => kprobpart%kprobwp2
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobwp2, lenglc )
            istart = istart + lenglc

         end if  ! ( kprobloc(47)>0 )
         if ( debug ) write(irefwr,1) 'istart/10', istart

!        --- Kprob part h

         if ( kprobloc(50)>0 .and. iprob==1 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobh(lenglc) )
            kprobh => kprobpart%kprobh
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobh, lenglc )
            istart = istart + lenglc
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kprobhinv(lenglc) )
            kprobhinv => kprobpart%kprobhinv
            istart = istart + 2
            lenglc = lenglc
            call incopy ( iwork(istart), kprobhinv, lenglc )
            istart = istart + lenglc

         end if  ! ( kprobloc(50)>0 )

!        --- Kprob part y

         if ( kprobloc(51)>0 ) then
            lenglc = iwork(istart+1)
            allocate ( kprobpart%kproby(lenglc) )
            kproby => kprobpart%kproby
            istart = istart + 2
            kproby = iwork(istart:istart+lenglc-1)
            istart = istart + lenglc
            if ( kproby(4)>0 ) then
               lenglc = iwork(istart+1)
               allocate ( kprobpart%kprobfict(2,lenglc/2) )
               kprobfict => kprobpart%kprobfict
               istart = istart + 2
               call incopy ( iwork(istart), kprobfict, lenglc )
               istart = istart + lenglc
            end if  ! ( kproby(4)>0 )
         end if  ! ( kprobloc(51)>0 )
         if ( debug ) write(irefwr,1) 'istart/10', istart

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readb3'

      end if  ! ( debug )

      call erclos ( 'readb3' )

      end subroutine readb3
