      subroutine readexprstore ( istore, store, lenistore, lenstore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-04-2014 Correction for scalar = -scalar
!        version  1.0  date 11-12-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the information as stored in istore/store into the commons
!     cexpression and cseprd
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexpr
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenistore, istore(*), lenstore
      double precision store(*)

!     istore         i    Contains information of the expression
!                         See subroutine READEXPR, array iwork
!     lenistore      i    Number of positions used in istore
!     lenstore       i    Number of positions used in store
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READEXPRSTORE1 Store the information as stored in istore/store into the
!                    commons cexpression and cseprd
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2857   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexprstore' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexprstore'
         call prininlarge ( istore, lenistore, 'istore' )
         call prinrl ( store, lenstore, 'store' )
         write(irefwr,1) 'lastrealexpr', lastrealexpr
  1      format ( a, 1x, (10i9) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( lenistore<=1 ) then

!     --- Nothing read

         ityprd = 20
         realrd = 0

      else if ( lenistore==2 ) then

!     --- Only a number or a minus sign found

         if ( istore(2)>=150000000 ) then

!        --- vector found, store information as expression

            call readexprstore1 ( istore, lenistore )

         else if ( istore(2)>=100000000 ) then

!        --- scalar found

             if ( istore(1)==-1 ) then

!            --- minus found, make expression

                call readexprstore1 ( istore, lenistore )

            else

!           --- not minus found

               ityprd = 11
               realrd = istore(2)

            end if  ! ( istore(1)==-1 )

         else if ( istore(2)>1000 ) then

!        --- Number found

            ityprd = 3
            if ( istore(1)==1 ) then

!           --- Starts with + &

               realrd = store(1)

            else

!           --- Starts with - &

               realrd = -store(1)

            end if  ! ( istore(1)==1 )

         else if ( istore(2)==-1 ) then

!        --- - found

            ityprd = 6

         else

!        --- item not recognized

            call errsub ( 2857, 0, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( istore(2)>1000 )

      else

!     --- lenistore>2, hence an expression found
!         Copy into cexpression
!         First array store is copied

         if ( lenstore>0 ) then

!        --- lenstore>0, hence store has been filled

            call inichkarlen ( lastrealexpr+lenstore, lenrealexpr, &
                               'realexpress' )
            if ( ierror/=0 ) go to 1000

            realexpress(lastrealexpr+1:lastrealexpr+lenstore) = &
               store(1:lenstore)

!           --- Next the contents of istore referring to store
!               are changed such that they refer to realexpress

            do i = 1, lenistore

               if ( istore(i)>1000 .and. istore(i)<100000000 ) then

!              --- istore refers to store

                  istore(i) = istore(i)+lastrealexpr

               end if  ! ( istore(i)>1000 .and. istore(i)<100000000 )

            end do  ! i = 1, lenistore
            lastrealexpr = lastrealexpr+lenstore  ! raise lastrealexpr

         end if  ! ( lenstore>0 )

!        --- Finally the contents of istore are copied into intexpress

         call readexprstore1 ( istore, lenistore )

      end if  ! ( lenistore<=1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'realrd', realrd
         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'End readexprstore'

      end if  ! ( debug )
      call erclos ( 'readexprstore' )

      end
