      subroutine readfd ( index, iunder, iupper, iref, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 04-06-2007 Fortran 90
!        version  1.1  date 26-12-1994 New commons cmessc and cmessg
!        version  1.0  date 18-11-1994 Double precision version of readfl
!
!   copyright (c) 1994-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read array from SEPRAN direct access file into core
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direct_access
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleerr
      use sepmodulefiles
      use sepmodulecomio
      use sepmodulexxx
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, iunder, iupper, iref
      double precision array(*)

!     array   o  array from which the input is read depending on the
!                values of iref
!     index      record number from which array has to be read
!     iref       dataset reference number indicating the type of file
!                iref = 2 or 3
!     iunder     first element in array to be read
!     iupper     last element in array to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jref, nrec, n, iho, i, iht, k, ios

!    i       General loop variable
!    iho     lower index of array to be written in present record
!    iht     upper index of array to be written in present record
!    ios     IOSTATus variable
!    jref    reference number
!    k       General loop variable
!    n       number of records to be written
!    nrec    record length of file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     160   Error in reading file
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( iref==2 ) then

!     --- iref = 2

         jref = iref2
         nrec = nrec2/intlen
         irec2 = index+(iupper-iunder)/nrec+1
         ifree2 = max(ifree2,irec2)

      else if ( iref==3 ) then

!     --- iref = 3

         jref = iref3
         nrec = nrec3/intlen
         irec3 = index+(iupper-iunder)/nrec+1
         ifree3 = max(ifree3,irec3)

      end if

      if ( nrec==0 ) then
         write(*,110) iref
110      format ( ///' ************************************'// &
                     ' fatal error in subroutine readfd '/ &
                     ' File with symbolic reference number',i2/ &
                     ' has not been opened, whereas you try to'/ &
                     ' read from this file'// &
                     ' Execution is terminated' // &
                     ' ************************************')
         ierror = 1
         go to 1000
      end if
      n = (iupper-iunder)/nrec+1
      iho = iunder

      if ( ioutp>=2 ) then

!     --- Extra writes to print all record numbers that are used

         call eropen ( 'readfd' )
         write(irefwr,120) namsub(levsub), iref, nrec, index, index+n-1
120      format(' Reading from file in subroutine ',a/ &
                ' File with symbolic ref. number',i2,'  record length', &
                i5, '  records', i4, ' to ', i4 )
         call erclos ( 'readfd' )

      end if

      do 200 i = 1,n
         iht = min(iupper,iho+nrec-1)
         read(jref,rec = index+i-1,iostat = ios) (array(k),k = iho,iht)
         if ( ios/=0 ) then

!        --- Error in reading file

            call eropen ( 'readfd' )
            call errint ( jref, 1 )
            call errint ( index+i-1, 2 )
            call errint ( ios, 3 )
            call errsub ( 160, 3, 0, 0 )
            call erclos ( 'readfd' )

         end if
         iho = iht+1
200   continue

1000  if ( ierror/=0 ) call instop

      end
