      subroutine readfl ( index, iunder, iupper, ichois, iref, iarray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.12 date 19-03-2007 Debug statements
!        version  3.11 date 06-10-2006 Debug information
!        version  3.10 date 23-11-2002 Only print message if ioutp>=4
!
!   copyright (c) 1982-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read iarray from SEPRAN direct access file into core
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direct_access
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleerr
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iarray(*), index, iunder, iupper, ichois, iref

!     iarray    array from which the input is read depending on the
!               values of ichois and iref
!     ichois    choice parameter,  possibilities
!               1 when iref = 1  irecp1 gets the value of index
!                 when iref>1  ireci gets the value of the last record
!                                read after this subroutine plus one
!               2 when iref = 1  irecp2 gets the value of index
!                 when iref>1  ireci gets the value of the last record
!                                read after this subroutine plus one
!     index     record number from which iarray has to be read
!     iref      dataset reference number indicating the type of file
!               iref = 1, 2, 3, 4 or 17
!     iunder    first element in iarray to be read
!     iupper    last element in iarray to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jref, nrec, n, iho, i, iht, k, ios

!     i              General loop variable
!     iho            lower index of array to be written in present record
!     iht            upper index of array to be written in present record
!     ios            IOSTATus variable
!     jref           reference number
!     k              General loop variable
!     n              number of records to be written
!     nrec           record length of file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readfl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readfl'
         write(irefwr,1) 'iref, iunder, iupper, ichois', &
                          iref, iunder, iupper, ichois
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      if ( iref==1 ) then

!     --- iref = 1

         jref = iref1
         nrec = nrec1
         if ( ichois==1 ) then
            irecp1 = index
         else
            irecp2 = index
         end if

      else if ( iref==2 ) then

!     --- iref = 2

         jref = iref2
         nrec = nrec2
         irec2 = index+(iupper-iunder)/nrec2+1
         ifree2 = max(ifree2,irec2)

      else if ( iref==3 ) then

!     --- iref = 3

         jref = iref3
         nrec = nrec3
         irec3 = index+(iupper-iunder)/nrec3+1
         ifree3 = max(ifree3,irec3)

      else if ( iref==4 ) then

!     --- iref = 4

         jref = irefer
         nrec = nrec4

      else

!     --- iref = 17

         jref = iref17
         nrec = nrec4

      end if
      if ( debug ) write(irefwr,1) 'jref, nrec', jref, nrec

      if ( nrec==0 ) then
         write(*,110) iref
110      format ( ///' ************************************'// &
                     ' fatal error in subroutine readfl '/ &
                     ' File with symbolic reference number',i2/ &
                     ' has not been opened, whereas you try to'/ &
                     ' read from this file'// &
                     ' Execution is terminated' // &
                     ' ************************************')
         ierror = 1
         go to 1000
      end if
      n = (iupper-iunder)/nrec+1
      iho = iunder
      if ( debug ) &
         write(irefwr,1) 'iunder, iupper, n', iunder, iupper, n

      if ( ioutp>=4 ) then

!     --- Extra writes to print all record numbers that are used

         write(irefwr,120) namsub(levsub), iref, nrec, index, index+n-1
120      format(' Reading from file in subroutine ',a/ &
                ' File with symbolic ref. number',i2,'  record length', &
                i5, '  records', i4, ' to ', i4 )

      end if

      do i=1,n
         iht=min(iupper,iho+nrec-1)
         read(jref,rec=index+i-1,iostat=ios) (iarray(k),k=iho,iht)
         if ( debug ) then
            write(irefwr,1) 'i, iho, iht, ios, index', &
                             i, iho, iht, ios, index
            call prinin ( iarray(iho), iht-iho+1, 'iarray' )
         end if  ! ( debug )
         if ( ios/=0 ) then

!        --- Error in reading file

            write (irefwr,'(//1x,78(''*'')/)')
            write (irefwr,130) iref,index+i-1,ios
130         format (' *** ','error number 160 in module readfl'/// &
                    ' Error in reading direct access file.'/ &
                    ' This means that the record you want to', &
                    ' read has not yet been filled.'/ &
                    ' A possible cause is that you try to read from a'/ &
                    ' file (for example SEPRANBACK) before you have', &
                    ' written something.'/ &
                    ' The reference number of the file is ',i8/ &
                    ' The record number to be read is ',i8/ &
                    ' The iostat error number is ',i5 )

            if ( levsub>1 ) then
               write(irefwr,300)
300            format(//' TRACEBACK of calling subroutines'// )
               do k = levsub, 1, -1
                  write(irefwr,310) k,namsub(k)
310               format(10x,i2,': ',a)
               end do
            end if

            ierror = 160
            go to 1000

         end if
         iho=iht+1
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readfl'

      end if  ! ( debug )
      call erclos ( 'readfl' )

      end
