      subroutine readintscals
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if name read corresponds to one the integer scalars
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     keyword
!     scalar
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, iseq
      character (len=80) namecheck

!     i              General loop variable
!     iseq           Sequence number
!     namecheck      Help parameter to store a name
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READARRINDEX   Check if the name of a constant vector or varibale is
!                    followed by an index
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readintscals' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readintscals'
         write(irefwr,1) 'lastitem, numintscals', lastitem, numintscals
  1      format ( a, 1x, (10i7) )
  2      format ( a, 1x, (5d18.10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numintscals

         nch = ncharscals(maxcls+i)
         index_rdarr(1:5) = 0

         if ( nchar==nch ) then

            namecheck = chscalars(maxcls+i)(1:nchar)
            call askupc ( namecheck )
            if ( string(1:nchar)==namecheck(1:nchar) ) then

!           --- name of scalar found

               iseq = readarrindex()
               if ( debug ) write(irefwr,1) 'iseq', iseq
               index_rdarr(3) = abs(iseq)

               if ( iseq>0 ) then

!              --- iseq>0, scalar index is integer

                  iseq = iseq-1  ! correction for adding

               else if ( iseq<0 ) then

!              --- iseq>0, scalar index is scalar

                  iseq = 0

               else if ( dimscals(maxcls+i)>1 ) then

!              --- iseq = 0 but indexed scalar, &
!                  hence complete scalar must be used

                  iseq = 5000  ! to indicate complete scalar

               else

!              --- Standard scalar

                  iseq = 0

               end if  ! ( iseq>0 )

               if ( debug ) &
                  write(irefwr,1) 'ispecl, iseq, i', ispecl, iseq, i

               if ( ispecl==10 .or. ispecl==15 .or. ispecl==19 ) then

                  index_rdarr(1) = i+iseq+maxcls
                  ityprd = 14

               else

                  realrd = i+iseq+100000000d0+maxcls
                  if ( index_rdarr(3)>0 ) &
                     realrd = realrd+10000*index_rdarr(3)
                  index_rdarr(1) = i+iseq
                  index_rdarr(3) = 0
                  ityprd = 11

               end if  ! ( ispecl==10 .or. ispecl==15 .or. ispecl==19 )

               if ( debug ) write(irefwr,1) 'scalar, ityprd', i, ityprd
               go to 1000

            end if  ! ( string(1:nchar)==namecheck(1:nchar) )

         end if

      end do  ! i = 1, numintscals

1000  call erclos ( 'readintscals' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'realrd', realrd
         write(irefwr,1) 'lastitem, index_rdarr', &
                          lastitem, index_rdarr(1:5)
         write(irefwr,*) 'End readintscals'

      end if  ! ( debug )

      end
