      subroutine readname
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 18-03-2014 Also check constpstants
!        version  2.4  date 20-04-2012 Indexed vectors
!        version  2.3  date 29-02-2008 Replace part of body by call subroutine
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read the name of a constpstant behind the % sign and set ityprd as well
!   as realrd
!   The name of the constpstant may contain digits, letters or an underscore
!   The separation character is either the space or end of record sign
!   After reading the keyword, icolum points to the next position to be read
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     none
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, n, minusg
      character (len=80) namecheck

!     i              General loop variable
!     minusg         Indicates if a minus sign has been found (-1) or
!                    not (1)
!     n              Number of names that are known
!     namecheck      Help parameter to store a name
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     READNAME1      read the name of a constpstant behind the $ or % sign
!     READNAMEVEC    Read name of vector in common cpost5
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constpstants
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2068   sequence number of the vector read is incorrect
!    2070   Name found after the percent symbol contains more than 32 characters
!    2071   Name found after the percent symbol does not belong to the list of
!           names
!    2116   -% not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ready  := false
!   nchar := 0
!   kcolum := icolum
!   while ( not ready ) do
!      nchar := nchar+1
!      string(nchar) := cards(icolum)
!      icolum := icolum+1
!      if ( icolum>maxcolum ) then
!         ready := true
!      else if ( string(nchar) = space ) then
!         ready := true
!   If ( nchar = 0 ) then
!      error
!   else
!      if ( string in list of names ) then
!         set realrd and ityprd
!      else
!         error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readname' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readname'
         write(irefwr,1) 'ityprd, lastitem, numvec', &
                          ityprd, lastitem, numvec
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read name following %-sign and detect if it is preceded by a - sign

      call readname1 ( minusg )
      if ( debug ) then
         write(irefwr,1) 'ityprd, minusg, nchar, numvec, lastitem/2', &
                          ityprd, minusg, nchar, numvec, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( minusg<0 ) then

!     --- Minus sign found, not yet allowed

         call errchr ( string(1:nchar), 1 )
         call errsub ( 2116, 0, 0, 1 )
         call errcrd

      end if

!     --- name has been found, check to which group it belongs

      if ( nchar>32 ) then
         call errchr ( string(1:nchar), 1 )
         call errint ( nchar, 1 )
         call errwar ( 2070, 1, 0, 1 )
         call errcrd
         nchar = 32
      end if

!     --- First the names in the common cpost5

      call readnamevec
      if ( debug) &
         write(irefwr,1) 'ityprd, numvec, lastitem/3', &
                          ityprd, numvec, lastitem
      if ( ityprd==2 ) go to 1000  ! vector name found

!     --- then the function names in the common cpost5

      do i = 1, numfuncs
         nch = index ( namefuncs(i), ' ' ) -1
         if ( nch<0 ) nch = 32
         if ( nchar==nch ) then
            namecheck = namefuncs(i)(1:nchar)
            call askupc ( namecheck )
            if ( string(1:nchar)==namecheck(1:nchar) ) then

!           --- name of vector of type 117 found

               realrd = i+1000
               ityprd = 10
               if ( debug ) &
                  write(irefwr,1) 'ityprd, lastitem/4', ityprd, lastitem
               go to 1000

            end if
         end if
      end do

      if ( debug ) write(irefwr,1) 'numvec/1', numvec

!         If so return with ityprd/=4

      call readstrscals
      if ( debug) then
         write(irefwr,1) 'ityprd, lastitem/5', ityprd, lastitem
         write(irefwr,1) 'numvec/2', numvec
      end if  ! ( debug)

      if ( ityprd/=4 ) go to 1000

!         If so return with ityprd/=4

      call readstrconst
      if ( debug) then
         write(irefwr,1) 'ityprd, lastitem/6', ityprd, lastitem
         write(irefwr,1) 'numvec/3', numvec
      end if  ! ( debug)

      if ( ityprd/=4 ) go to 1000

!     --- Name has not been found

      if ( ispecl==11 ) then

!     --- ispecl = 11, Vector name must be added to list of vectors

         numvec = numvec+1
         if ( numvec>maxvc1 ) then

!        --- Too much vector_names

            call errint ( numvec, 1 )
            call errint ( maxvc1, 2 )
            call errsub ( 2068, 2, 0, 0 )
            call errcrd
            go to 1000

         end if
         nchar = min(nchar,32)
         names(numvec) =  stringim(1:nchar)
         ncharnames(numvec) = nchar
         if ( iprogram==3 ) then
            realrd = numvec-1
         else
            realrd = numvec
         end if
         ityprd = 2
         go to 1000

      end if

      n = 1
      do i = 1, numvec
         call errchr ( names(i), n+i )
      end do
      n = n+numvec
      do i = 1, numfuncs
         call errchr ( namefuncs(i), n+i )
      end do
      n = n+numfuncs
      do i = 1, numscals
         call errchr ( chscalars(i), n+i )
      end do
      n = n+numscals
      call errchr ( string(1:nchar), 1 )
      call errsub ( 2071, 0, 0, n )
      call errcrd

1000  call erclos ( 'readname' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem/end', ityprd, lastitem
         write(irefwr,*) 'End readname'
         call ertrace

      end if  ! ( debug )

      end
