      subroutine readnumber ( irecord, endcolum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 18-12-2013 Extra debug
!        version  1.1  date 02-04-2008 Fortran 90
!        version  1.0  date 18-12-2002
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read one number from the input file
!       This subroutine supposes that the parameter ICOLUM indicates the
!       first position of the number. This first position may be a
!       digit or one of the following signs:  . + - &
!       The number is read free format
!       It is indicated if the number is real or integer
!        a number is defined by:
!         start:  + - . or digit
!         end:    non digit (except e or d after real), or end of record
!         in between digits . e d + and - are permitted
!       After reading the number, icolum points to the next position to be read
!       If the number is incorrect, realread gets the value 0
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irecord, endcolum

!     endcolum       i    Last column with information in record
!     irecord        i    Record number in cardimage
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, lreal, lexp, lsign
      integer icount, mantis, ios
      double precision realread
      character (len=7) formt1

!     finished       Indicates if the while loop has finished (true) or not
!                    (false)
!     formt1         Character string to store the format necessary for
!                    reading of the number with an internal read
!     icount         Counts the number of positions necessary for the number
!     ios            IOSTAT variable, returns from reading or writing
!     lexp           Indicates if the number contains an exponent (true)
!                    or not (false)
!     lreal          Indicates if the number is real (true) or integer (false)
!     lsign          Indicates if a sign has been read in the exponent (true)
!                    or not (false)
!     mantis         Number of characters of the mantisse plus one
!     realread       Real that has been read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1332   Number does not satisfy standard requirements for numbers
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Raise nitems
!     Fill firstcolitems
!     Detect number and raise icolum;
!       lreal := false
!       lexp  := false
!       finished := false
!       icount:= 0
!       kcolum:= icolum
!       if ( cards(icolum) = + or - ) then
!          icount := 1
!          string(1) := cards(icolum)
!          icolum := icolum+1
!       while ( not finished ) do
!          icount := icount+1
!          string(icount) := cards(icolum)
!          icolum := icolum+1
!          if ( icolum>endcolum ) then
!             finished := true
!             icount := icount+1
!          else if ( lreal and string(icount) = '.' ) then
!             finished := true
!          else if ( string(icount) = '.' ) then
!             lreal := true
!          else if ( string(icount) = 'D', 'd', 'E' or 'e' ) then
!             finished := true
!             lexp := true
!          else if ( string(icount) not a digit ) then
!             finished := true
!             icolum := icolum - 1
!       mantis = icount
!       if ( lexp ) then
!          lsign := false
!          finished := false
!          while ( not finished ) do
!             if ( icolum>endcolum ) then
!                finished := true
!                icount := icount+1
!             else
!                icount := icount+1
!                string(icount) := cards(icolum)
!                icolum := icolum+1
!                else if ( lsign and string(icount) = '+' or '-' ) then
!                      finished := true
!                else if ( string(icount) = '+', '-' or digit ) then
!                   lsign := true
!                else (other character found) then
!                   finished := true
!                   icolum := icolum - 1
!          if ( string(icount-1) # digit ) then
!             icolum = icolum - icount + mantis
!             icount = mantis
!             lexp = false
!          if ( lexp or lreal ) then
!             ityprd := 3
!          else
!             ityprd := 2
!          translate number with internal read and store in realread
!     Fill itypeitems, lastcolitems
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readnumber' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnumber'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Raise nitems

      nitems = nitems+1

      if ( debug ) write(irefwr,1) 'nitems', nitems

!     --- Fill firstcolitems

      firstcolitems(nitems) = icolum+maxcolum*(irecord-1)

!     --- Initialize some parameters

      lreal  = .false.
      lexp   = .false.
      finished  = .false.
      icount = 0
      kcolum = icolum
      if ( index('+-',cards(icolum:icolum))>0 ) then

!     --- sign found in first position

         icount = icount + 1
         string(icount:icount) = cards(icolum:icolum)
         icolum = icolum + 1

      end if

100   if ( .not. finished ) then

!     --- First while loop for mantisse

         if ( icolum>endcolum ) then

!        --- End of record reached
!            Raise icount, since only the positions 1 until icount-1 contain
!            the number to be interpreted

            finished = .true.
            icount = icount + 1

         else

            icount = icount + 1
            string(icount:icount) = cards(icolum:icolum)
            icolum = icolum + 1
            if ( lreal .and. string(icount:icount)=='.' ) then

!           --- Second decimal point found
!               This decimal point can not be part of the number, so reading is
!               stopped

               finished = .true.

            else if ( string(icount:icount)=='.' ) then

!           --- Decimal point found, so number is real

               lreal = .true.

            else if ( index('DEde',string(icount:icount))>0 ) then

!           --- Letter E, D, e or d found, possibly floating point number

               lexp  = .true.
               finished = .true.

            else if ( string(icount:icount)<'0' .or. &
                      string(icount:icount)>'9' ) then

!           --- Next character is not a digit, end of number

               finished = .true.
               icolum = icolum - 1

            end if
            go to 100

         end if

      end if

!     --- End of first while loop

      mantis = icount
      if ( lexp ) then

!     --- Exponent sign found, read rest of number and check if number
!         contains an exponent

         lsign = .false.
         finished = .false.

200      if ( .not. finished ) then

!        --- Second while loop for exponent

            if ( icolum>endcolum ) then

!           --- End of record reached
!               Raise icount, since only the positions 1 until icount-1 contain
!               the number to be interpreted

               finished = .true.
               icount = icount + 1

            else
               icount = icount + 1
               string(icount:icount) = cards(icolum:icolum)
               icolum = icolum + 1
               if ( lsign .and. index('+-',string(icount:icount)) &
                    >0 ) then

!              --- + or -, found not immediately after the exponent sign
!                  This sign can not be part of the number

                  finished = .true.

               else if ( string(icount:icount)>='0' .and. &
                         string(icount:icount)<='9' .or. &
                         index('+-',string(icount:icount))>0 ) then

!              --- Digit or + or - found, set lreal

                  lreal = .true.

               else

!              --- Other character found, i.e. end of number

                  finished = .true.
                  icolum = icolum - 1

               end if
               go to 200

            end if

         end if

!        --- End of second while loop
!            Check if exponent realy exists

         if ( string(icount-1:icount-1)<'0' .or. &
              string(icount-1:icount-1)>'9' ) then

!        --- Last position of number is not a digit, so exponent does not
!            exist
!            Reset parameters

            icolum = icolum - icount + mantis
            icount = mantis
            lexp = .false.

         end if

      end if

      if ( lexp .or. lreal ) then

!     --- Number is real

         ityprd = 3

      else

!     --- Number is integer

         ityprd = 2

      end if

!     --- Translate the number using internal write

      formt1(1:2) = '(g'
      if ( icount<=10 ) then
         write ( formt1(3:3), fmt='(i1)' ) icount - 1
         formt1(4:7) = '.0 )'
      else
         write ( formt1(3:4), fmt='(i2)' ) icount - 1
         formt1(5:7) = '.0)'
      end if

      read ( string(1:icount-1), fmt=formt1, iostat = ios ) realread

      if ( debug ) then
         write(irefwr,*) 'string ', string(1:icount-1)
         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,2) 'realread', realread
      end if  ! ( debug )

      if ( ios/=0 ) then

!     --- ios # 0, error during reading

         call errint ( ios, 1 )
         call errchr ( string(1:icount-1), 1 )
         call errsub ( 1332, 1, 0, 1 )
         call errcrd
         realread = 0

      end if

!     --- Fill itypeitems, lastcolitems and realitems

      typeitems(nitems) = ityprd-1
      lastcolitems(nitems) = icolum-1+maxcolum*(irecord-1)
      realitems(nitems) = realread

1000  call erclos ( 'readnumber' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'typeitems, lastcolitems', &
                          typeitems(nitems), lastcolitems(nitems)
         write(irefwr,2) 'realitems', realitems(nitems)
         write(irefwr,*) 'End readnumber'

      end if  ! ( debug )

      end
