      subroutine readrealscal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 19-04-2014 Use readstrnames
!        version  1.3  date 08-12-2013 Extension with integer scalars
!        version  1.2  date 29-04-2013 Allow typerd 12 or 13
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read integer, real or scalar
!     If present return with correct value of ityprd
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ispeclsav

!     ispeclsav      Help parameter to save the value of ispecl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READSTRNAMES   Check if string belongs to the list of constants
!                    variables or vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readrealscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readrealscal'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item, include digits

      ispeclsav = ispecl
      ispecl = 9
      call asknxt
      ispecl = ispeclsav
      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,*) string(1:nchar)
      end if  ! ( debug )

      if ( ityprd==4 ) then

!     --- String found, check if it is a constant, scalar or vector

         call readstrnames

      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'End readrealscal'

      end if  ! ( debug )
      call erclos ( 'readrealscal' )

      end
