      subroutine readseqscal ( table, iinput, defvalue, code, itype, &
                               icode, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 28-12-2014 New parameter list
!        version  1.0  date 25-04-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the sequence number read in iinput
!     If no sequence number is found, use default
!     Also an integer or real may be present
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision table(*)
      integer iinput, defvalue, itype, icode, ncodmx(*)
      character (len=*) code(*)

!     code           i    Table with keywords (contains the names)
!     defvalue       i    Default value
!     icode          i    Sequence number of variable in table and tabmin
!     iinput         o    entry of integer input array to be filled with
!                         integer sequence number
!     itype          i    indicates if the output expected is a scalar (3)
!                         or not (0)
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqscal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqscal'
         write(irefwr,1) 'defvalue', defvalue
         write(irefwr,*) 'name ', code(icode)(1:ncodmx(icode))
  1      format ( a, 1x, (10i9) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( table(icode)>0d0 ) then

!     --- scalarname is given

         iinput = nint(table(icode))

      else

!     --- scalarname is not given, use default

         if ( defvalue>0 ) then
            iinput = defvalue  ! default value
         else if ( defvalue==0 ) then
            iinput = defvalue  ! default value
            go to 1000         ! jump
         end if  ! ( iscalnm>0 )

         if ( dimscals(iinput)>1 ) iinput = iinput+5000

      end if  ! ( value(1)>0d0 )

!     --- In case of type = 3, mark as scalar

      if ( itype==3 .and. iinput<maxints+maxreals ) &
          iinput = iinput+maxints+maxreals

      if ( debug ) write(irefwr,1) 'iinput', iinput

1000  call erclos ( 'readseqscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput', iinput
         write(irefwr,*) 'End readseqscal'

      end if

      end
