      subroutine readset ( iretur, exec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-02-2006 replace set output out by none
!        version  1.0  date 18-12-2002
!
!   copyright (c) 2002-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if a set record is recognized
!     If so act as necessary and return with iretur = 0 else return with
!     iretur = 1
!     Set records that are recognized are
!       set warn on/off
!       set max warn = &
!       set max error = &
!       set output on/off/none  or level=-1/0/1/2/3
!       set time on/off
!       set skip on/off
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     interpret
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iretur
      logical exec

!     exec          i/o   If true the input is read, otherwise it is skipped
!     iretur         o    Return value of subroutine
!                         Possible values:
!                         0:  keyword is recognized, read next record
!                         1:  keyword is not recognized, continue
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readset' )

!     --- First word is SET, read next item

      iretur = 1
      if ( typeitems(2)/=3 ) go to 1000
      if ( numrecstored>1 ) go to 1000

!     --- Second item is string, replace by uppercase and store in string

      firstcol = firstcolitems(2)
      lastcol = lastcolitems(2)
      string = cardimage(numrecord)(firstcol:lastcol)
      nchar = lastcol-firstcol+1
      call askupc(string(1:nchar))
      if ( string(1:4)=='WARN' ) then

!     --- Second keyword is warn, check on third one (on/off)

         if ( typeitems(3)/=3 ) go to 1000
         firstcol = firstcolitems(3)
         lastcol = lastcolitems(3)
         string = cardimage(numrecord)(firstcol:lastcol)
         nchar = lastcol-firstcol+1
         call askupc(string(1:nchar))
         if ( string(1:nchar)=='ON' ) then

!        --- Set warn on

            nwarn = 0
            iretur = 0

         else if ( string(1:3)=='OFF' ) then

!        --- Set warn off

            nwarn = 11
            iretur = 0

         end if

      else if ( string(1:nchar)=='MAX' ) then

!     --- Second keyword is max, check on third one (warn/error)

         if ( nitems<4 .or. typeitems(3)/=3 ) go to 1000
         firstcol = firstcolitems(3)
         lastcol = lastcolitems(3)
         string = cardimage(numrecord)(firstcol:lastcol)
         nchar = lastcol-firstcol+1
         call askupc(string(1:nchar))
         if ( string(1:4)=='WARN' ) then

!        --- max warn = ...
!            find number

            if ( typeitems(nitems)/=1 ) go to 1000
            mxwarn = nint(realitems(nitems))
            iretur = 0

         else if ( string(1:5)=='ERROR' ) then

!        --- max error = ...

            if ( typeitems(nitems)/=1 ) go to 1000
            maxerror = nint(realitems(nitems))
            iretur = 0

         end if

      else if ( string(1:6)=='OUTPUT' ) then

!     --- Second keyword is output, check on third one (on/off/out/level)

         if ( typeitems(3)/=3 ) go to 1000
         firstcol = firstcolitems(3)
         lastcol = lastcolitems(3)
         string = cardimage(numrecord)(firstcol:lastcol)
         nchar = lastcol-firstcol+1
         call askupc(string(1:nchar))
         if ( string(1:2)=='ON' ) then

!        --- set output on

            ioutp = 1
            iretur = 0

         else if ( string(1:3)=='OFF' ) then

!        --- set output off

            ioutp = 0
            iretur = 0

         else if ( string(1:3)=='OUT' ) then

!        --- set output out

            ioutp = -1
            iretur = 0

         else if ( string(1:4)=='NONE' ) then

!        --- set output none

            ioutp = -1
            iretur = 0

         else if ( string(1:5)=='LEVEL' ) then

!        --- set output level = -1/0/1/2/3

            if ( typeitems(nitems)/=1 ) go to 1000
            ioutp = nint(realitems(nitems))
            iretur = 0

         end if

      else if ( string(1:4)=='TIME' ) then

!     --- Second keyword is time, check on third one (on/off)

         if ( typeitems(3)/=3 ) go to 1000
         firstcol = firstcolitems(3)
         lastcol = lastcolitems(3)
         string = cardimage(numrecord)(firstcol:lastcol)
         nchar = lastcol-firstcol+1
         call askupc(string(1:nchar))
         if ( string(1:nchar)=='ON' ) then

!        --- set time on

            itime = 1
            iretur = 0

         else if ( string(1:3)=='OFF' ) then

!        --- set time off

            itime = 0
            iretur = 0

         end if

      else if ( string(1:4)=='SKIP' ) then

!     --- Second keyword is skip, check on third one (on/off)

         if ( typeitems(3)/=3 ) go to 1000
         firstcol = firstcolitems(3)
         lastcol = lastcolitems(3)
         string = cardimage(numrecord)(firstcol:lastcol)
         nchar = lastcol-firstcol+1
         call askupc(string(1:nchar))
         if ( string(1:nchar)=='ON' ) then

!        --- set skip on

            exec = .false.
            iretur = 0

         else if ( string(1:3)=='OFF' ) then

!        --- set skip off

            exec = .true.
            iretur = 0

         end if

      end if  ! checking second keyword

1000  call erclos ( 'readset' )

      end
