      subroutine readstrconst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 22-12-2013 Extra debug
!        version  1.3  date 29-04-2013 Set index_rdarr
!        version  1.2  date 20-04-2012 Indexed constants
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if name read corresponds to one the integer or real constants
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduleconsts
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, iseq

!     i              General loop variable
!     iseq           Sequence number
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READARRINDEX   Check if the name of a constant vector or varibale is
!                    followed by an index
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readstrconst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstrconst'
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,1) 'lastitem',lastitem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First the integers

      index_rdarr(3) = 0
      do i = 1, numints
         nch = ncharcons(i)
         index_rdarr(1:5) = 0
         if ( nchar==nch ) then
            if ( string(1:nchar)==chincons(i)(1:nchar) ) then

!           --- integer found

               iseq = readarrindex()
               if ( iseq<0 ) then
                  index_rdarr(3) = -iseq
                  index_rdarr(4) = i
                  index_rdarr(5) = 1
               end if  ! ( iseq<0 ) then
               if ( ispecl==10 .or. ispecl==15 ) then

!              --- Special case, return with sequence number

                  if ( iseq==0 .and. dimcons(i)>1 ) then
                     iseq = 5001  ! complete array found
                  else
                     iseq = max(iseq,0)   ! constant found
                  end if  ! ( iseq==0 .and. dimcons(i)>1 )
                  index_rdarr(1) = i+iseq
                  ityprd = 12

               else

!              --- Standard case, return with value

                  iseq = max(iseq,0)
                  realrd = incons(i+iseq)
                  ityprd = 2
                  index_rdarr(1) = i+iseq

               end if
               if ( debug ) &
                  write(irefwr,1) 'ityprd, int i', ityprd, i+iseq
               go to 1000

            end if
         end if  ! ( nchar==nch )

      end do  ! i = 1, numints

!     --- next the reals

      do i = 1, numreals
         nch = ncharcons(i+maxints)
         index_rdarr(1:5) = 0
         if ( debug ) write(irefwr,1) 'i, nch, nchar', i, nch, nchar
         if ( nchar==nch ) then

            if ( string(1:nchar)==chrlcons(i)(1:nchar) ) then

!           --- real found

               iseq = readarrindex()
               if ( debug ) write(irefwr,1) 'iseq', iseq
               if ( iseq<0 ) then
                  index_rdarr(3) = -iseq
                  index_rdarr(4) = i
                  index_rdarr(5) = 2
               end if  ! ( iseq<0 ) then
               if ( ispecl==10 ) then

!              --- Special case, return with sequence number

                 if ( iseq==0 .and. dimcons(i+maxints)>1 ) then
                     iseq = 5001  ! complete array found
                  else
                     iseq = max(iseq,1)   ! constant found
                  end if  ! ( iseq==0 .and. dimcons(i)>1 )
                  index_rdarr(1) = i+iseq-1
                  ityprd = 13

               else

!              --- Standard case, return with value

                  iseq = max(iseq,1)
                  realrd = rlcons(i+iseq-1)
                  index_rdarr(1) = i+iseq-1+maxints
                  ityprd = 3

               end if

               if ( debug ) &
                  write(irefwr,1) 'ityprd, real i, index_rdarr(3)', &
                                   ityprd, i+iseq-1, index_rdarr(3)
               go to 1000

            end if
         end if

      end do  ! i = 1, numreals

      if ( nchar==2 .and. string(1:2)=='PI' ) then

!     --- Number pi found

         realrd = pi
         ityprd = 3

      end if  ! ( nchar==2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem, index_rdarr', &
                          lastitem, index_rdarr(1:5)
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End readstrconst'

      end if  ! ( debug )
      call erclos ( 'readstrconst' )

      end
