      subroutine readwr
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 26-06-2014 Debug
!        version  2.2  date 06-02-2012 Change in comment
!        version  2.1  date 12-03-2008 Extension with ispecl = 19
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read a KEYWORD
!     a keyword is defined by a number of letters not segregated by any
!     other character
!     If ispecl = 9, the keyword may also contain digits and underscores
!     After reading the keyword, icolum points to the next position to be read
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     none
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: finished
      integer :: iitem

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     iitem          Local sequence number of item
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ispecl=9 ) then
!        Read sequence of keyword followed by numbers and texts if contiguous
!     else ( ispecl#9 )
!        store keyword in string
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readwr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readwr'
         write(irefwr,1) 'ispecl', ispecl
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      stringim = ' '
      string = ' '
      lastitembef = lastitem-1
      if ( mod(ispecl,10)/=9 ) then

!     --- ispecl#9: store keyword in string
!         Get first column and last column and store text in cardim

         firstcol = firstcolitems(lastitem)
         numrecord = firstcol/240+1
         firstcol = mod(firstcol,240)
         lastcol = mod(lastcolitems(lastitem),240)
         nchar = lastcol-firstcol+1
         stringim(1:nchar) = cardimage(numrecord)(firstcol:lastcol)

!        --- store text in card in uppercase form

         string(1:nchar) = stringim(1:nchar)
         call askupc(string(1:nchar))

      else

!     --- ispecl=9 or 19
!         Get first column and last column and store text in cardim

         firstcol = firstcolitems(lastitem)
         numrecord = firstcol/240+1
         firstcol = mod(firstcol,240)
         lastcol = mod(lastcolitems(lastitem),240)

!        --- Read sequence of keyword followed by numbers and keywords if
!            contiguous

         finished = .false.
         do while ( .not. finished .and. lastitem<nitems )
            iitem = lastitem+1
            if ( typeitems(iitem)>=1 .and. typeitems(iitem)<=3 ) &
               then

!           --- Text or number found, check if this immediately follows the
!               last column

               icolum = firstcolitems(iitem)
               if ( icolum>240*numrecord ) then

!              --- Next item starts at new record

                  finished = .true.

               else if ( mod(icolum,240)/=lastcol+1 ) then

!              --- Next item does not immediately follow precious one

                  finished = .true.

               else if ( typeitems(iitem)==3 ) then

!              --- Next item is a text immediately following the previous one

                  lastitem = iitem
                  lastcol = mod(lastcolitems(lastitem),240)

               else

!              --- Next item is a number immediately following the previous one
!                  it may not contain a plus or a minus sign or a .

                  kcolum = mod(lastcolitems(iitem),240)
                  if ( index('.+-', &
                       cardimage(numrecord)(lastcol+1:kcolum))>0 ) &
                     then

!                 --- . + or - found, number does not belong to name

                     finished = .true.

                  else

!                 --- Only a number found, hence it belongs to the string

                     lastitem = iitem
                     lastcol = mod(lastcolitems(lastitem),240)

                  end if

               end if

            else

!           --- Not a string or number found

               finished = .true.

            end if

         end do   ! while ( .not. finished .and. lastitem<nitems )

         nchar = lastcol-firstcol+1
         stringim(1:nchar) = cardimage(numrecord)(firstcol:lastcol)

!        --- store text in card in uppercase form

         string(1:nchar) = stringim(1:nchar)
         call askupc(string(1:nchar))

      end if

      ityprd = 4

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,*) 'End readwr'

      end if  ! ( debug )
      call erclos ( 'readwr' )

      end
