      subroutine sepactsolbf1 ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate solution array or array of special structure
!     Also activate corresponding sub arrays
!     Old subroutine sepactsolbuf1
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol

!     isol    i  The standard SEPRAN solution array or array of special
!                structure of type 115 from which the starting positions
!                must be computed and the relevant information found.
!                Other types of arrays are not allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, itypvc, ivec, iproblc, leng, iseqkpsav

!     i              Loop variable
!     iproblc        Local value of iprob
!     itypvc         Type number of solution vector (110=solution vector,
!                    115=vector of special structure)
!     ivec           Indication of the structure of a vector of special
!                    structure
!     leng           Help variable to compute the length of some arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     744   Fatal error: Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (IPROB), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set indprflc and read KPROB part f in IBUFFR
!        if ( KPROB part h has been filled ) then
!           Set indprhlc and read KPROB part h in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set indprplc and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set indprflc and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 ) then
!        Compute nphyslc
!     else if ( ISOL is of type 127 ) then
!        Compute nphyslc
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactsolbf1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepactsolbf1'
         call ertrace
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkpsav = iseqkp

      indprfi = 0
      indprpi = 0
      itypvc = ks(isol)%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 .and. &
           itypvc/=127 .and. itypvc/=801 ) then

!     --- Array isol has not been filled

         call errint ( itypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end if
      if ( debug ) write(irefwr,1) 'itypvc', itypvc

!     --- Activate isol

      if ( ierror/=0 ) go to 1000

!     --- Compute IPROB and offset for array KPROB

      iproblc = iprob
      iprob  = ks(isol)%iprob
      iseqkp = ks(isol)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indprpi = 0
      indprhi = 0

      if ( debug ) then
         write(irefwr,1) 'iprob, iseqkp', iprob, iseqkp
         call prinin ( kprobloc, 10, 'kprobloc' )
      end if  ! ( debug )

      select case (itypvc)

      case ( 110 )

!     --- ISOL is of type 110:  solution vector

         nphysi = max ( kprobloc(4), kprobloc(33) )
         indprfi = kprobloc(19)
         indprpi = kprobloc(35)
         nunkpi = kprobloc(4)
         nusoli = kprobloc(5)
         nboundi = kprobloc(10)
         npbouni = kprobloc(34)
         if ( indprpi>0 ) kprobpi => kprobpart%kprobp
         if ( indprfi>0 ) kprobfi => kprobpart%kprobf

      case ( 115 )

!     --- ISOL is of typefals 115:  array of special structure

         ivec   = ks(isol)%ivec
         ndtvec = kprobloc(6)
         indpre = kprobloc(18)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3

         if  ( debug ) write(irefwr,1) 'ivec', ivec
         nphysi  = kprobe2(ivec)
         if ( debug ) write(irefwr,1) 'nphysi', nphysi
         if ( nphysi<0 ) then

!        --- nphysi<0, constant number of degrees of freedom per point

            nphysi = -nphysi
            nusoli = npoint*nphysi

         else

!        --- nphysi>0, number of degrees of freedom per point not constant

            indprfi = nphysi
            nphysi = maxval(kprobe(:,ivec))
            kprobfi => kprobe3(ivec)%arr

         end if
         nunkpi = nphysi

      case ( 116, 126, 127, 129, 801 )

!     --- ISOL is of type 116:  array of special structure defined per element

         nphysi = ks(isol)%nphys
         nunkpi = nphysi
         nusoli = nunkpi*nelem

      case default

!     --- Wrong type of array

         call errint ( itypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end select  ! case (itypvc)

      if ( indprfi>0 ) nusoli = kprobfi(size(kprobfi))

      if ( iproblc>0 .and. iproblc<maxprob ) iprob  = iproblc
      iseqkp = iseqkpsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'indprfi, indprpi, nphysi, nunkpi', &
                          indprfi, indprpi, nphysi, nunkpi
         write(irefwr,*) 'End sepactsolbf1'

      end if  ! ( debug )
      call erclos ( 'sepactsolbf1' )

      end subroutine sepactsolbf1
