      subroutine sepcopcurvcoor ( icurnr, curvescp, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates of curve icurnr from array curves into array curvescp
!     Double points are skipped
!     The curve is assumed to be a continuous curve without gaps
!     Old subroutine ercopcurvcoor
! **********************************************************************
!
!                       KEYWORDS
!
!     compound_curve
!     copy
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, nnodes
      double precision :: curvescp(ndim,*)

!     curvescp       o    Contains the coordinates of icurnr where double points
!                         have been deleted
!                         The coordinates are stored from position 1
!     icurnr         i    Curve sequence number
!     nnodes         o    Number of nodes in the curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubcv, istart, i, k, kcurnr, jcurnr, ic, numnod

!     i              Counting variable
!     ic             Relative subcurve number
!     istart         First position used at a part
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     k              Counting variable
!     kcurnr         abs(jcurnr)
!     nsubcv         Number of subcurves of a curve
!     numnod         Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find number of subcurves
!     If ( curve is a single curve )
!        Find starting address of curve in curves and number of nodes
!        Copy coordinates
!     else ( curve is compound curve)
!        For all subcurves do
!          Find starting address of curve in curves and number of nodes
!          Copy coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopcurvcoor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopcurvcoor'

      end if

!     --- Find number of subcurves

      nsubcv = 0
      if ( kelmq>0 ) nsubcv = kmeshq(icurnr+1) - kmeshq(icurnr)
      if ( debug ) write(irefwr,*) 'icurnr, nsubcv ',icurnr, nsubcv

      if ( nsubcv<=1 ) then

!     --- Curve is a single curve
!         Find starting address in curves and number of nodes

         if ( icurnr==1 ) then

!        --- icurnr = 1

            istart = 0
            nnodes = icurvsmsh(icurnr)

         else

!        --- icurnr > 1

            istart = icurvsmsh(icurnr-1)
            nnodes = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)

         end if
         do i = 1, nnodes
            do k = 1, ndim
               curvescp(k,i) = curvesmsh(k,istart+i)
            end do
         end do

      else

!     --- Curve is a compound curve
!         Loop over subcurves

         nnodes = 0
         do ic = 1, nsubcv
            jcurnr = kmeshq( kmeshq(icurnr)+ic )
            kcurnr = abs(jcurnr)

            if ( kcurnr==1 ) then

!           --- kcurnr = 1

               istart = 0
               numnod = icurvsmsh(kcurnr)

            else

!           --- icurnr > 1

               istart = icurvsmsh(kcurnr-1)
               numnod = icurvsmsh(kcurnr)-icurvsmsh(kcurnr-1)

            end if
            if ( debug ) &
               write(irefwr,*) 'ic, jcurnr, istart, numnod, nnodes ', &
                                ic, jcurnr, istart, numnod, nnodes

            if ( jcurnr>0 ) then

!           --- Positive direction

               do i = 1, numnod
                  do k = 1, ndim
                     curvescp(k,i+nnodes) = curvesmsh(k,istart+i)
                  end do
               end do

            else

!           --- Negative direction

               do i = 1, numnod
                  do k = 1, ndim
                     curvescp(k,i+nnodes) = curvesmsh(k,istart+1+numnod-i)
                  end do
               end do

            end if

!           --- Raise nnodes

            if ( ic/=nsubcv ) then

!           --- Not the last curve
!               Since the compound curve is contiguous the last point is
!               identical to the first one of the next curve

               nnodes = nnodes+numnod-1

            else

!           --- Last curve

               nnodes = nnodes+numnod

            end if

         end do

      end if

      call erclos ( 'sepcopcurvcoor' )
      end
