      subroutine sepcreakmhsur
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-09-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     allocate kmhsur
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreakmhsur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreakmhsur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( associated(km(iseqkm)%kmhsur) ) deallocate(km(iseqkm)%kmhsur)
      allocate ( km(iseqkm)%kmhsur(nsurfs+nvolms), stat = error )
      if ( error/=0 ) call eralloc ( error, nsurfs+nvolms, 'kmhsur' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nsurfs+nvolms
         nullify(km(iseqkm)%kmhsur(i)%kmeshcsur)
         nullify(km(iseqkm)%kmhsur(i)%kbndptsur)
         nullify(km(iseqkm)%kmhsur(i)%maparsur)
         nullify(km(iseqkm)%kmhsur(i)%coorsur)
         nullify(km(iseqkm)%kmhsur(i)%renumsur)
         km(iseqkm)%kmeshh(:,i+1) = 0
         km(iseqkm)%kmhsur(i)%filled = .false.
      end do  ! i = 1, nsurfs+nvolms
      kmhsur => km(iseqkm)%kmhsur

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcreakmhsur'

      end if  ! ( debug )
      call erclos ( 'sepcreakmhsur' )

      end subroutine sepcreakmhsur
