      subroutine sepcreasolbf1 ( isol, itypvc, ivec, namear, icompl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate or create solution array or array of special structure
!     Also activate corresponding sub arrays
!     Old subroutine sepcreasolbuf
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, itypvc, ivec, icompl
      character (len=*) namear

!     icompl         i    0 = real, 1 = complex
!     isol           o    sequence number of solution array
!     itypvc         i    Type number of solution vector (110=solution vector,
!                         115=vector of special structure)
!     ivec           i    Indication of the structure of a vector of special
!                         structure
!     namear         i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ktypvc
      logical new

!     i              Counting variable
!     ktypvc         abs(itypvc )
!     new            If true the vector is supposed to be new
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     SEPCRSOLINFO   Create a new vector in ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     744   Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (IPROB), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set indprflc and read KPROB part f in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set indprplc and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set indprflc and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 ) then
!        compute length and nphyslc
!     else if ( ISOL is of type 126 ) then
!        compute length and nphyslc
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreasolbf1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreasolbf1'
         call ertrace
         write(irefwr,1) 'itypvc, ivec, iprob, icompl, isol, kprob', &
                          itypvc, ivec, iprob, icompl, isol, kprob
         write(irefwr,1) 'iseqkp', iseqkp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm = kp(iseqkp)%iseqkm
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      ktypvc = abs(itypvc)
      indprfo = 0
      indprho = 0
      indprpo = 0

      select case (ktypvc)

      case ( 110 )

!     --- ISOL is of type 110:  solution vector

         nphyso = max ( kprobloc(4), kprobloc(33) )
         indprfo = kprobloc(19)
         indprho = kprobloc(50)
         indprpo = kprobloc(35)
         nusolo = kprobloc(5)
         ivec = 0
         if ( indprpo>0 ) kprobpo => kprobpart%kprobp
         if ( indprfo>0 ) kprobfo => kprobpart%kprobf
         if ( indprho>0 ) then
            kprobho => kprobpart%kprobh
            kprobhinvo => kprobpart%kprobhinv
         end if  ! ( indprho>0 )

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'nphyso, indprfo, indprpo', &
                             nphyso, indprfo, indprpo

         end if

         length = kprobloc(5)

         if ( debug ) write(irefwr,1) 'length', length

      case ( 115 )

!     --- ISOL is of type 115:  array of special structure

         ndtvec = kprobloc(6)
         indpre = kprobloc(18)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
         nphyso  = kprobe2(ivec)

         if ( debug ) write(irefwr,1) 'nphyso', nphyso
         if ( nphyso<0 ) then

!        --- nphyso<0, constant number of degrees of freedom per point

            nphyso = -nphyso
            length = nphyso * km(iseqkm)%kmesh(8)

         else

!        --- nphyso>0, number of degrees of freedom per point not constant

            indprfo = nphyso
            nphyso = maxval(kprobe(:,ivec))

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'nphyso, indprfo', nphyso, indprfo

            end if

            if ( associated(kprobe3(ivec)%arr) ) then
               kprobfo => kprobe3(ivec)%arr
               length = kprobfo(km(iseqkm)%kmesh(8)+1)-kprobfo(1)
            else
               indprfo = 0
               length = nphyso * km(iseqkm)%kmesh(8)
            end if  ! ( associated(kprobe3(ivec)%arr) )
            if ( debug ) &
               write(irefwr,1) 'indprfo, length', indprfo, length

         end if

      case ( 116 )

!     --- ktypvc = 116, array of special structure defined per element

         nphyso = ivec
         length = nphyso * km(iseqkm)%kmesh(9)

      case ( 126 )

!     --- ktypvc = 126, array of special structure defined per node per element

         nphyso = ivec
         length = nphyso * size(km(iseqkm)%kmeshc)

      case ( 127 )

!     --- ktypvc = 127, array defined per edge

         nphyso = ivec
         length = nphyso * km(iseqkm)%nmesh(4)

      case ( 129 )

!     --- ktypvc = 129, array of special structure defined per element
!         per integration point

         length = nphyso
         nphyso = ivec

      case ( 801 )

!     --- ktypvc = 801, capacity array

         length = nphyso * (ivec*(ivec-1))/2

      case default

!     --- Wrong type of array

         call errint ( ktypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end select  ! case (ktypvc)

!     --- Put ISOL into IBUFFR

      new = isol<1 .or. isol>lastsol
      if ( .not. new ) new = solarr(isol)==0
      if ( .not. new ) new = ktypvc/=ks(isol)%typevec
      if ( .not. new ) new = ks(isol)%nusol/=length
      if ( .not. new ) new = ks(isol)%compl/=icompl
      length  = length * (icompl+1)

      if ( debug ) then
         write(irefwr,*) 'new ', new
         write(irefwr,1) 'length, icompl', length, icompl
      end if  ! ( debug )
      if ( new ) then

!     --- New solution array

         call sepcrsolinfo ( isol, length, 0 )
         ks(isol)%sol = 0d0
         ks(isol)%jcart = 0
         ks(isol)%n = ivec

      end if
      ks(isol)%iseqkp = iseqkp
      ks(isol)%typevec = ktypvc
      ks(isol)%ivec = ivec
      ks(isol)%n = ivec
      ks(isol)%nphys = nphyso
      ks(isol)%nusol = length/(icompl+1)
      ks(isol)%iprob = iprob
      ks(isol)%compl = icompl
      ks(isol)%changed = .true.
      nunkpo = nphyso

      usoltemp => ks(isol)%sol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isol, length', isol, length
         write(irefwr,*) 'End sepcreasolbf1'

      end if
      call erclos ( 'sepcreasolbf1' )
      end subroutine sepcreasolbf1
