      subroutine sepcreastruc1 ( itypvc, ivec, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!     Old subroutine inistrnew1
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypvc, ivec, number

!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!                         If 0, no vector found
!     ivec           o    Sequence number of vector of special structure
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jprob, iproblc

!     i              Counting variable
!     jprob          Problem number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPSTRELGRP    Check if vector of special structure has a constant number
!                    of unknowns per point for all non-empty elements
!     SEPSTRSLELGRP  Check if solution vector has a constant number of number
!                    unknowns per point for all non-empty elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreastruc1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreastruc1'
         write(irefwr,1) 'itypvc, number', itypvc, number
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

      nunkplc  = kprobloc(4)
      indpra = kprobloc(14)
      ivec = 0

      if ( debug ) write(irefwr,1) 'nunkplc, indpra', nunkplc, indpra

      if ( itypvc==110 .and. nunkplc==number ) then

!     --- nunkplc = number, vector is of type solution vector

         if ( indpra==0 ) then

!        --- Number of unknowns per point is constant

            itypvc = 110
            go to 1000

         else

!        --- Number of unknowns per point is not constant
!            Check if solution vector is a candidate

            call sepstrslelgrp ( number, itypvc )

!           --- if itypvc = 110 the solution vector is found
!               in this case or in case of an error: jump

            if ( ierror/=0 .or. itypvc==110 ) go to 1000

         end if  ! ( indpra==0 )

      end if  ! ( itypvc==110 .and. nunkplc==number )

!     --- vector is not of type solution vector
!         Check if there is a vector of special structure with NUMBER
!         unknowns per point

      ivec = 0
      ndtvec = kprobloc(6)
      indpre = kprobloc(18)
      kelgrp = kprobloc(39)
      kprobe => kprobpart%kprobe
      kprobe2 => kprobpart%kprobe2
      itypvc = 115

      if ( debug ) &
         write(irefwr,1) 'ndtvec, indpre, kelgrp', &
                          ndtvec, indpre, kelgrp

      do i = 1, ndtvec

         nunkplc = -kprobe2(i)

         if ( nunkplc==number ) then

!        --- Vector of special structure found

            itypvc = 115
            ivec = i
            exit

         end if

      end do  ! i = 1, ndtvec

      if ( debug ) write(irefwr,1) 'ivec/1', ivec

      if ( ivec==0 ) then

!     --- No vector with a constant number of degrees of freedom has been found
!         Check if there is a vector with a number degrees of freedom

         itypvc = 115

         do i = 1, ndtvec

!        --- Check if vector of type i is the one with number degrees of freedom
!            per point

            call sepstrelgrp ( number, ivec, i )
            if ( ivec>0 ) then

!           --- ivec>0, vector found

               go to 1000

            end if  ! ( ivec>0 )

         end do  ! i = 1, ndtvec

      end if  ! ( ivec==0 )

      if ( ivec==0 ) then

!     --- No vector found, check if there is a vector for a different problem

         jprob = 1
         iproblc = iprob
         do while ( jprob<=nprob .and. jprob/=iprob )

            iprob = jprob
            kprobpart => kp(iseqkp)%kprobsub(iprob)
            kprobloc => kprobpart%kprobloc
            ndtvec = kprobloc(6)

            do i = 1, ndtvec

!           --- Check if vector of type i is the one with number degrees of
!               freedom per point

               call sepstrelgrp ( number, ivec, i )
               if ( ivec>0 ) then

!              --- ivec>0, vector found

                  iprob = jprob
                  go to 1000

               end if  ! ( ivec>0 )

            end do  ! i = 1, ndtvec

            jprob = jprob+1  ! raise jprob

         end do  ! while ( jprob<=nprob .and. jprob/=iprob )
         iprob = iproblc

      end if  ! ( ivec==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itypvc, ivec', itypvc, ivec
         write(irefwr,*) 'End sepcreastruc1'

      end if  ! ( debug )
      call erclos ( 'sepcreastruc1' )

      end
