      subroutine sepelalloccf ( nextra, nmin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate coeffs and related arrays for element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     allocate
!     element
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nextra, nmin

!     nextra         i    The second dimension of coefs is E%ncoeff+netxra
!     nmin           i    E%ncoeff is reset to E%ncoeff-nmin, before calling
!                         sepelfillccof
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELFILLCOF   Fill all real coefficients in E%coeffs
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelalloccf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelalloccf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( E%coeffs(B%m,E%ncoef+nextra), E%ind(E%ncoef), E%ist(E%ncoef), &
                 E%mst(E%ncoef), E%ivecnums(E%ncoef), E%ich(E%ncoef), &
                 E%coeffsnode(B%n,E%ncoef), E%coeffcp(B%m,E%ncoef), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, B%m*E%ncoef, 'coeffs' )
      if ( ierror/=0 ) go to 1000

      E%ich = 0  ! clear ich
      E%ind = 0  ! clear ind
      E%coeffs = 0d0

      if ( debug ) write(irefwr,1) 'E%ipos/1', E%ipos

      E%ncoef = E%ncoef-nmin
      call sepelfillcof ( 1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelalloccf'

      end if  ! ( debug )
      call erclos ( 'sepelalloccf' )

      end subroutine sepelalloccf
