      subroutine sepfillkmesh ( kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmodulekmesh
!        version  1.0  date 14-12-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters in module sepmodulekmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, target, intent(inout) :: kmesh

!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: newcall

!     newcall        If true it concerns a new situation, i.e. the array
!                    from which the module is filled has changed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLKMESHM  Make relation between kmeshm and Curvesub
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmesh'
         call ertrace
         write(irefwr,1) 'kmesh, icountmesh', kmesh, icountmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm = kmesh

      if ( debug ) &
         write(irefwr,1) 'iseqkm, icounter', iseqkm, km(iseqkm)%icounter
      if ( km(iseqkm)%icounter >= icountmesh ) go to 1000

      if ( associated(Curvesub) ) deallocate (Curvesub )
      if ( associated(pointkmsc) ) deallocate ( pointkmsc )
      allocate ( Curvesub(km(iseqkm)%kmesh(11)), &
                 pointkmsc(km(iseqkm)%kmesh(5)+1) )

      if ( debug ) write(irefwr,*) 'before seprelkmesh'
      call seprelkmesh1

      icountmesh = icountmesh+1
      km(iseqkm)%icounter = icountmesh

      if ( kelmm>0 ) call sepfillkmeshm

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndim', ndim
         write(irefwr,*) 'End sepfillkmesh'

      end if  ! ( debug )
      call erclos ( 'sepfillkmesh' )

      end subroutine sepfillkmesh
