      subroutine sepfillkmeshc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make relation between kmeshc and Topgrp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, inpelm

!     i              Element group number
!     inpelm         number of points per element
!     istart         Start address in kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLKMESHC2D  Make relation between kmeshc and Topgr for one element
!                    group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshc'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmeshc => km(iseqkm)%kmeshc
      inpelm = km(iseqkm)%kmesh(4)
      inelem = km(iseqkm)%kmesh(9)

      if ( debug ) write(irefwr,1) 'size(kmeshc)', size(kmeshc)


      istart = 1
      do i = 1, nelgrp
         if ( km(iseqkm)%kmesh(15)/=0 ) &
            inpelm = km(iseqkm)%kmesha(i+1) - km(iseqkm)%kmesha(i)
         if ( km(iseqkm)%kmesh(21)>0 ) inelem = km(iseqkm)%kmeshg(i)
         Topgrp(i)%inpelm = inpelm
         Topgrp(i)%inelem = inelem

         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, inelem, istart', &
                             i, inpelm, inelem, istart

         if ( Topgrp(i)%inelem>0 ) &
            call sepfillkmeshc2d ( kmeshc(istart), i )
         istart = istart+Topgrp(i)%inpelm*Topgrp(i)%inelem
      end do  ! i = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmeshc'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshc' )

      end subroutine sepfillkmeshc
