      subroutine sepfillkmesho
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part o with neighbor elements of elements
!     Old subroutine topfillkmesho
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     topology
!     mesh
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lengo, lardim
      logical linear

!     lardim         largest dimension of elements used
!     lengo          length of kmesh part o
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHO1 Compute length of KMESH part O2, fill KMESH part O1
!     SEPFILLKMESHO2 Fill KMESH part O2
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create kmesh part d if necessay
!     Reserve nelgrp temporary positions for first part kmesh o
!     Fill first part kmesh o and compute length of second part
!     if ( length of second part>0 ) then
!        Create space for array
!        Reserve work space
!        Compute starting addresses
!        Actually fill kmesh part o
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmesho' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmesho'

      end if

!     --- Reserve nelgrp temporary positions for first part kmesh o

      if ( associated(kmsho1) ) nullify(kmsho1)
      allocate ( kmsho1(nelgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp, 'kmsho1' )
      if ( ierror/=0 ) go to 1000

!     --- Fill first part kmesh o and compute length of second part

      call sepfillkmesho1 ( lengo, lardim, linear )
      if ( lengo>0 ) then

!     --- lengo>0   possible to fill kmesh part o
!         Create space for array

         kelmo = lengo
         allocate ( km(iseqkm)%kmesho(kelmo) )

         km(iseqkm)%kmsho1 => km(iseqkm)%kmesho(1:nelgrp)
         km(iseqkm)%kmsho2 => km(iseqkm)%kmesho(nelgrp+1:lengo)
         km(iseqkm)%kmsho1 = kmsho1

         deallocate ( kmsho1, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmsho1' )

         kmesho => km(iseqkm)%kmesho
         kmsho1 => km(iseqkm)%kmsho1
         kmsho2 => km(iseqkm)%kmsho2


         length = nelem
         if ( lardim==2 ) then
            if ( nelgrp>1 ) length = length+nelem
            if ( kelma/=0 ) length = length+nelem+1
         end if

!        --- Actually fill kmesh part o

         call sepfillkmesho2 ( lengo, lardim )

      else

!      --- Part o is empty

         if ( associated(km(iseqkm)%kmesho) ) deallocate(km(iseqkm)%kmesho)
         deallocate ( kmsho1, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmsho1' )
         nullify(kmesho, kmsho2)
         kelmo = 0

      end if

      if ( debug ) call sepprinkmesh ( 7 )

1000  if ( debug ) write(irefwr,*) 'End sepfillkmesho'
      call erclos ( 'sepfillkmesho' )

      end
