      subroutine sepfillnmeshi
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmeshi with the face numbers of the faces connected to
!     the edges
!     Old subroutine topfaceedge
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lennmeshi

!     lennmeshi      (Estimated) length of array kmeshi
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!                    of iarray to ibuffr
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHI1 Fill array nmeshi with the face numbers of the faces
!                    connected to the edges
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nullify ( nmeshi )
      if ( associated(km(iseqkm)%nmeshi) ) deallocate(km(iseqkm)%nmeshi)

      lennmeshi = nedges*25
      if ( nmesh(31)==0 ) call sepfillnmesha

      allocate ( nmeshi(lennmeshi), stat = error )
      if ( error/=0 ) call eralloc ( error, lennmeshi, 'nmeshi' )
      if ( ierror/=0 ) go to 1000

      call sepfillnmeshi1 ( lennmeshi )
      if ( ierror/=0 ) go to 1000

!     --- make array nmeshi permanent

      allocate ( km(iseqkm)%nmeshi(lennmeshi) )
      km(iseqkm)%nmeshi = nmeshi(1:lennmeshi)
      nmesh(39) = lennmeshi

      deallocate ( nmeshi, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshi' )

      nmeshi => km(iseqkm)%nmeshi

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnmeshi'

      end if  ! ( debug )
      call erclos ( 'sepfillnmeshi' )

      end
