      subroutine sepfillnmeshj
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates nmesh part j, for each surface the face numbers (3d)
!     Old subroutine iniactnmeshj
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     surface
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lnnmeshj

!     lnnmeshj       Length of array nmeshj
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer ininumelsurfs

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ININUMELSURFS  Return with number of elements in all surfaces
!     SEPFILLNMESHK  Fill array nmesh part k
!     TOPSURFACES    Find the face numbers of all faces stored in the
!                    given surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshj' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshj'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nullify ( nmeshj )
      if ( associated(km(iseqkm)%nmeshj) ) deallocate(km(iseqkm)%nmeshj)

!     --- Activate or create kmesh  a part nmeshk

      if ( nmesh(41)==0 ) call sepfillnmeshk

!     --- Create space for array nmeshj and make this array permanent

      lnnmeshj = ininumelsurfs ( kmeshn, nsurfs )
      allocate ( km(iseqkm)%nmeshj(lnnmeshj) )
      nmesh(40) = lnnmeshj

      nmeshj => km(iseqkm)%nmeshj

!     --- Actually fill nmeshj

      call topsurfaces

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnmeshj'

      end if  ! ( debug )
      call erclos ( 'sepfillnmeshj' )

      end
