      subroutine sepfillnodobst ( inpsep, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for obstacle problems (curves only)
!     Old subroutine prfillnodobst
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     obstacle
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst
      integer, allocatable :: iwork(:)

!                    they are not
!     iobst          Obstacle sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLNODOBS1 Fill array nodes for obstacle problems (curves only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodobst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0
      if ( kelmh==0 .or. numobst==0 ) go to 1000  ! no obstacle found

!     --- Get parameters

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- Get parameters from inpsep

      iobst = inpsep(1)

!     --- actual call

      call sepfillnodobs1 ( iobst, icurvsobst2d, nodes, nnodes, iwork )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'sepfillnodobst' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodobst'

      end if  ! ( debug )

      end subroutine sepfillnodobst
