      subroutine sepfillnodsrf1 ( isrf1, isrf2, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of surfaces
!     Old subroutine prfillnodsurf1
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isrf1, isrf2, nodes(*), nnodes

!     isrf1          i    First surface number
!     isrf2          i    Last surface number
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurfaces, i
      integer, allocatable :: surfaces(:)

!                    they are not
!     i              Counting variable
!     nsurfaces      Number of surfaces in array surfaces
!     surfaces       Contains the surface numbers to be used for filling nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPSURFSNOD1   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     SEPSURFSNOD2   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodsrf1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodsrf1'
         write(irefwr,1) 'isrf1, isrf2', isrf1, isrf2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isrf1<=0 .or. isrf1>nsurfs ) then
         call errint ( isrf1, 1 )
         call errint ( nsurfs, 2 )
         call errsub ( 155, 2, 0, 0 )
         isrf1 = 1
      end if

      if ( isrf2<isrf1 .or. isrf2>nsurfs ) then
         call errint ( isrf2, 1 )
         call errint ( isrf1, 2 )
         call errint ( nsurfs, 3 )
         call errsub ( 156, 3, 0, 0 )
         isrf2 = isrf1
      end if

      nsurfaces = isrf2-isrf1+1

      allocate ( surfaces(nsurfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfaces, 'surfaces' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nsurfaces
         surfaces(i) = isrf1+i-1
      end do  ! i = 1, nsurfaces

      if ( debug ) call prinin ( surfaces, nsurfaces, 'surfaces' )

      nnodes = 0

      if ( kelmn>0 ) then

!     --- kmeshn has been filled

         call sepsurfsnod1 ( nsurfaces, surfaces, nodes, nnodes )

      else

!     --- kmeshn has not been filled

         call sepsurfsnod2 ( nsurfaces, surfaces, nodes, nnodes )

      end if  ! ( kelmn>0 )

      deallocate ( surfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'surfaces' )

1000  call erclos ( 'sepfillnodsrf1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodsrf1'

      end if  ! ( debug )

      end subroutine sepfillnodsrf1
