      subroutine sepfillnodsrf3 ( nodes, nnodes, numnod, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for all outer surfaces
!     Old subroutine prfillnodsurf3
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), nnodes, numnod, ihelp(*)

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsurfaces, ipart, istart, isurnr, j, nboun, nparts
      integer, allocatable :: surfaces(:)

!                    they are not
!     ipart          Sequence number of part
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     nboun          Number of boundary points at an element
!     nparts         number of parts the boundary consists of
!     nsurfaces      Number of surfaces in array surfaces
!     surfaces       Contains the surface numbers to be used for filling nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSURFSNOD3   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has been filled
!                    Only a restricted number of nodes per element is used
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodsrf3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodsrf3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nsurfaces = 0

      nparts = outerboun(1)
      if ( debug ) write(irefwr,1) 'nparts', nparts

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         nboun = outerboun(istart)
         nsurfaces = nsurfaces+nboun

      end do  ! ipart = 1, nparts

      allocate ( surfaces(nsurfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfaces, 'surfaces' )
      if ( ierror/=0 ) go to 1000

      nsurfaces = 0
      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         nboun = outerboun(istart)
         do j = 1, nboun
            isurnr = abs(outerboun(istart+j))
            nsurfaces = nsurfaces+1
            surfaces(nsurfaces) = isurnr
         end do  ! j = 1, nboun

      end do  ! ipart = 1, nparts

      if ( debug ) call prinin ( surfaces, nsurfaces, 'surfaces' )

      call sepsurfsnod3 ( nsurfaces, surfaces, nodes, nnodes, &
                          numnod, ihelp )

      deallocate ( surfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'surfaces' )

1000  call erclos ( 'sepfillnodsrf3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodsrf3'

      end if  ! ( debug )

      end subroutine sepfillnodsrf3
