      subroutine sepfillnodsrf5 ( nsurfaces, surfaces, nodes, nnodes, numnod, &
                                  ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of surfaces
!     Only a restricted number of nodes per element is used
!     Old subroutine prfillnodsurf5
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfaces, surfaces(nsurfaces), nodes(*), nnodes, &
               numnod, ihelp(*)

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     nsurfaces      i    Number of surfaces in array surfaces
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
!     surfaces       i    Contains the surface numbers to be used for filling
!                         nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPSURFSNOD3   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has been filled
!                    Only a restricted number of nodes per element is used
!     SEPSURFSNOD4   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has not been filled
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodsrf5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodsrf5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0

      if ( kelmn>0 ) then

!     --- kmeshn has been filled

         call sepsurfsnod3 ( nsurfaces, surfaces, nodes, nnodes, &
                             numnod, ihelp )

      else

!     --- kmeshn has not been filled

         call sepsurfsnod4 ( nsurfaces, surfaces, nodes, nnodes,&
                             numnod, ihelp )

      end if  ! ( kelmn>0 )

1000  call erclos ( 'sepfillnodsrf5' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodsrf5'

      end if  ! ( debug )

      end subroutine sepfillnodsrf5
