      subroutine sepfindivec ( number, itypvc, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 04-03-2011 Extension with probextend
!        version  1.2  date 12-12-2005 Fortran 90
!        version  1.1  date 25-05-1996 (itypvc set at input)
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!     If this type does not exist an error message is given
!     iprob must have a value
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: number
      integer, intent(out) :: itypvc, ivec

!     itypvc        i/o   Indication of the type of solution vector
!                         If itypvc = 110 at input then the output vector may be
!                         a vector of type 110 or 115 else of type 115 only
!                         Output:
!                         110: solution vector
!                         115: vector of special structure
!     ivec           o    Sequence number of vector of special structure
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPROBEXTEND  Extend problem definition with vectors of special structure
!                    of exactly number degrees of freedom
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( solution vector contains exactly NUMBER unknowns per point ) then
!        itypvc := 110
!     else
!        ivec := 0
!        For i := 1 (1) NDTVEC, while ( ivec=0 ) do
!           If ( vector of special structure i has exactly NUMBER unknowns
!                per point then )
!              ivec := i
!              itypvc := 115
!        If ( ivec=0 ) then
!           Error: no vector found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfindivec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfindivec'

      end if
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      nunkp  = kprobloc(4)
      if ( itypvc==110 .and. kprobloc(14)==0 .and. nunkp==number ) then

!     --- nunkp = number, vector is of type solution vector

         itypvc = 110

      else

!     --- vector is not of type solution vector
!         Check if there is a vector of special structure with NUMBER
!         unknowns per point

         ivec = 0
         ndtvec = kprobloc(6)
         indpre = kprobloc(18)
         kelgrp = kprobloc(39)

         if ( indpre>0 ) then

!        --- indpre > 0, hence kprobe exists

            kprobe => kprobpart%kprobe
            kprobe2 => kprobpart%kprobe2
            do i = 1, ndtvec
               nunkp = - kprobe2(i)
               if ( nunkp==number ) then

!              --- Vector of special structure found

                  itypvc = 115
                  ivec = i
                  exit

               end if

            end do  ! i = 1, ndtvec

         end if  ! ( indpre>0 )

         if ( ivec==0 ) then

!        --- No vector found

            call sepprobextend ( number )
            ivec = kprobloc(6)
            itypvc = 115

         end if  ! ( ivec==0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfindivec'

      end if
      call erclos ( 'sepfindivec' )

      end
