      function sepgetstring ( stringname, substrings, numsubstr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number of stringname in substrings
!     If stringname is not part of substrings return with 0
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepgetstring
      integer, intent(in) :: numsubstr
      character (len=*), intent(in) :: stringname
      character (len=*), intent(in) :: substrings(numsubstr)

!     numsubstr      i    Number of strings in substrings
!     sepgetstring   o    sequence number of stringname in substrings
!                         If 0 stringname is not part of substrings
!     stringname     i    Name of string
!     substrings     i    Set of strings in which stringname must be found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nch, lenstring
      character (len=132) :: vname1, vname2


!     i              Counting variable
!     lenstring      Length of string
!     nch            Number of characters in a string
!     vname1         Name of variable translated to capitals
!     vname2         Name of variable translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetstring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetstring'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepgetstring = 0

      lenstring = index ( stringname, ' ' ) -1
      if ( lenstring==-1 ) lenstring = len(stringname)
      vname1 = stringname(1:lenstring)
      call askupc(vname1)

!     --- Check if stringname is part of the set substrings

      do i = 1, numsubstr

         nch = index ( substrings(i), ' ' ) -1
         if ( nch<0 ) nch = len(substrings(1))
         if ( lenstring==nch ) then

            vname2 = substrings(i)(1:lenstring)
            call askupc(vname2)
            if ( vname1(1:lenstring)==vname2(1:lenstring) ) then

!           --- name of string found

               sepgetstring = i
               go to 1000

            end if
         end if

      end do  ! i = 1, numsubstr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgetstring'

      end if  ! ( debug )
      call erclos ( 'sepgetstring' )

      end function sepgetstring
