      function sepgetvar ( iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with value from module cons defined by sequence number iseq
!
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseq
      double precision :: sepgetvar

!     iseq           i    Sequence number
!     sepgetvar      o    Value of variable or constant as function of iseq
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetvar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetvar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseq<=maxints ) then

!     --- iseq<=maxints, hence integer

         sepgetvar = incons(iseq)

      else if ( iseq<maxints+maxreals ) then

!     --- iseq<=maxints+maxreals, hence real

         sepgetvar = rlcons(iseq-maxints)

      else

!     --- iseq>maxints+maxreals, hence scalar

         sepgetvar = scalars(iseq-maxints-maxreals)

      end if  ! ( iseq<=maxints )

1000  call erclos ( 'sepgetvar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgetvar'

      end if  ! ( debug )

      end function sepgetvar
