      subroutine sepreadkeyinit ( ihelp, ispecs, icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters and perform some checks for sepreadsetkeyw
!     Old subroutine intx02init
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ihelp(lnc), ispecs, icount

!     icount         o    Counter, number of keywords with itype = 8 or 9
!     ihelp          o    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     ispecs         o    Help variable to save the value of ispecl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadkeyinit' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadkeyinit'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iposut = 0
      kposut = 0
      nseqkeyw = 0
      ispecs = ispecl

      ihelp(1:nncode) = 0
      icountkey(1:nncode) = 0

      if ( nncode>lnc ) then

!     --- lnc too small

         call errint ( nncode, 1 )
         call errint ( lnc, 2 )
         call errchr ( 'infotable', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      infotable(1:nncode) = 0
      infotabout(1:nout) = 0
      iseqkeywitem(1:nncode) = 0

      if ( ierror/=0 ) go to 1000

!     --- Initialize array pointar

      icount = 0
      do i = 1, nncode

         if ( itype(i)>=8 .and. itype(i)<=9 ) then

!        --- itype(i) = 8 or 9, hence this keyword may be present
!            more than once, raise icount and store in pointar

            icount = icount + 1
            pointar(i) = icount

         else

!        --- itype(i) # 6 or 8, hence this keyword is used only once
!            set pointar to 0

            pointar(i) = 0

         end if  ! ( itype(i)>=8 .and. itype(i)<=9 )

      end do  ! i = 1, n
      if ( debug ) then
         write(irefwr,1) 'icount', icount
         call prinin ( pointar, nncode, 'pointar' )
      end if  ! ( debug )

!     --- Check array length

      call inichkarlen ( icount, maxmultiple, 'numberkeyw' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize array numberkeyw

      numberkeyw(1:icount) = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadkeyinit'

      end if  ! ( debug )
      call erclos ( 'sepreadkeyinit' )

      end subroutine sepreadkeyinit
