      subroutine sepreadseqnr ( iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read sequence number iseqnr from record
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iseqnr

!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadseqnr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadseqnr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      code(1) = 'SEQUENCE_NUMBER'
      ncode(1) = 3
      itype(1) = 0
      tabmin(1) = 1d0
      ncodmx(1) = 15

      table(1) = iseqnr
      tabmax(1) = 10d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      iseqnr = nint(table(1))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadseqnr'

      end if  ! ( debug )
      call erclos ( 'sepreadseqnr' )

      end subroutine sepreadseqnr
