      subroutine sepreadtype08 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword = number repeatedly
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0208
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: finished

!     finished       i    Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     READREALCONS   Check if last string read corresponds to constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1869   No number found after keyword
!    1871   Real number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype08' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype08'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = pointar(icode)

      call asknxt

!     --- in case of a string, check if it is a constant

      if ( ityprd==4 ) call readrealcons

      if ( debug ) &
         write(irefwr,1) 'ityprd, realrd', ityprd, realrd

      if ( ityprd<=1 .or. ityprd>=4 ) then

!     --- Not a real number found

         call errchr ( code(icode), 1 )
         call errsub ( 1869, 0, 0, 1 )
         call errcrd
         finished = .true.

      else

!     --- integer or real found, store

         if ( realrd<tabmin(icode) .or. &
              realrd>tabmax(icode) ) then
            call erreal ( realrd, 1 )
            call erreal ( tabmin(icode), 2 )
            call erreal ( tabmax(icode), 3 )
            call errsub ( 1871, 0, 3, 0 )
            call errcrd
         end if
         numberkeyw(iseq) = numberkeyw(iseq)+1
         valuekeyw(numberkeyw(iseq),iseq) = realrd
         readnx = .true.

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype08'

      end if
      call erclos ( 'sepreadtype08' )

      end subroutine sepreadtype08
