      subroutine sepreadtype09 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword = number repeatedly
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0209
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical finished

!     finished       i    Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivalmax, iseq

!     iseq           Sequence number
!     ivalmax        Maximum integer value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREALCONS   Check if last string read corresponds to constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype09' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype09'
         write(irefwr,1) 'minusg, lastitem, ityprd, lastitembef', &
                          minusg, lastitem, ityprd, lastitembef
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = pointar(icode)

      if ( debug ) write(irefwr,1) 'icode, iseq', icode, iseq

      call asknxt
      if ( ityprd==4 ) call readrealcons
      if ( debug ) &
         write(irefwr,1) 'ityprd, realrd', ityprd, nint(realrd)

      if ( ityprd/=2 ) then

!     --- Not an integer number found

         call errchr ( code(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         finished = .true.

      else

!     --- integer found, store

         ivalrd = nint(realrd)
         ivalmax = nint(tabmax(icode))
         if ( ivalrd<-ivalmax .or. &
              ivalrd>ivalmax ) then
            call errint ( ivalrd, 1 )
            call errint ( -ivalmax, 2 )
            call errint ( ivalmax, 3 )
            call errchr ( code(icode), 1 )
            call errsub ( 1870, 3, 0, 1 )
            call errcrd
         end if
         numberkeyw(iseq) = numberkeyw(iseq)+1
         valuekeyw(numberkeyw(iseq),iseq) = minusg*realrd
         readnx = .true.

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,*) 'End sepreadtype09'

      end if
      call erclos ( 'sepreadtype09' )

      end subroutine sepreadtype09
