      subroutine sepreadtype11 ( ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of positive integers
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0211
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ihelp(*)

!     ihelp          i    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inumlc, num, maxval

!     inumlc         Number of numbers read in a record
!     maxval         Maximum value allowed for the integer read
!     num            Number of numbers read between brackets
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPREADSERINT  Read a series of integers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype11'
         write(irefwr,1) 'icode, iposut', icode, iposut
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      num = 0
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      table(icode) = iposut+1

!     --- Next read integer numbers until closing bracket or
!         other type of separator is found

      inumlc = min(nint(tabmin(icode)),nout-iposut)
      maxval = nint(tabmax(icode))

      if ( iposut+1>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      call sepreadserint ( tabout(iposut+1), inumlc, num, &
                           itype(icode)==11, maxval, code(icode), iposut+1 )
      ihelp(icode) = num
      iposut = iposut+num

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2 ) 'table(icode)', table(icode)
         call prinrl ( tabout(iposut+1-num), num, 'tabout' )
         write(irefwr,*) 'readnx', readnx
         write(irefwr,1) 'iposut', iposut
         write(irefwr,*) 'End sepreadtype11'

      end if
      call erclos ( 'sepreadtype11' )

      end subroutine sepreadtype11
