      subroutine sepreadtype89 ( icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     There are keywords of type 8 and 9
!     Fill information read into table, tabmax and tabout
!     Old subroutine intx02type89
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: icount

!     icount         i    Counter, number of keywords with itype = 8 or 9
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, k, i

!     i              Counting variable
!     iseq           Sequence number
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype89' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype89'
         call prinin ( numberkeyw, icount, 'numberkeyw' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nncode

         if ( pointar(i)>0 ) then

!        --- Sequence number found

            iseq = pointar(i)
            if ( numberkeyw(iseq)>0 ) then

!           --- keyword appears more than once

               tabmax(i) =  numberkeyw(iseq)  ! number of numbers read
               table(i) = iposut+1  ! refers to starting position in tabout
               if ( iposut+numberkeyw(iseq)>nout ) then

!              --- Error: tabout too small

                  call errint ( nout, 1 )
                  call errsub ( 1468, 1, 0, 0 )
                  go to 1000

               end if  ! ( iposut+numberkeyw(iseq)>nout )

!              --- Copy information to tabout

               do k = 1, numberkeyw(iseq)
                  tabout(iposut+k) = valuekeyw(k,iseq)
               end do  ! k = 1, numberkeyw(iseq)
               iposut = iposut+numberkeyw(iseq)

            end if  ! ( numberkeyw(iseq)>0 )

         end if  ! ( pointar(i)>0 )

      end do  ! i = 1, nncode

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype89'

      end if  ! ( debug )
      call erclos ( 'sepreadtype89' )

      end subroutine sepreadtype89
