      subroutine seprmintmt ( intmatlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-12-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove array with sequence number intmatlc from intmt
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: intmatlc

!     intmatlc     i/o   Defines matrix structure by integer sequence number
!                        At output its value is 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseqimsav
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Loop variable
!     iseqimsav      Saves value of iseqim
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPRMMATINFO   Remove set of matrix arrays corresponding to iseqim
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmintmt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmintmt'
         call ertrace
         write(irefwr,1) 'intmatlc, lastintmat, iseqim', &
                          intmatlc, lastintmat, iseqim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqimsav = iseqim

      if ( intmatarr(intmatlc)==0 ) then

!     --- Array has already been removed

         intmatlc = 0
         go to 1000

      end if  ! ( intmatarr(intmatlc)==0 )
!
      if ( intmatlc==lastintmat ) then

!     --- intmatlc = lastintmat, decrease lastintmat

         lastintmat = lastintmat-1

         do i = lastintmat, 1, -1
            if ( intmatarr(lastintmat)==0 ) then
               lastintmat = lastintmat-1
            else
               exit
            end if  ! ( intmatarr(lastintmat)==0 )
         end do  ! i = lastintmat, 1, -1

      end if  ! ( intmatlc==lastintmat )
      intmatarr(intmatlc) = 0
      nextfreeintmt = min(nextfreeintmt,intmatlc)
      IM => intmt(intmatlc)

      if ( associated(IM%rowsii) ) deallocate(IM%rowsii)
      if ( associated(IM%rowsiiprec) ) deallocate(IM%rowsiiprec)
      if ( associated(IM%colsiiprec) ) deallocate(IM%colsiiprec)
      if ( associated(IM%colsii) ) deallocate(IM%colsii)
      if ( associated(IM%rowsip) ) deallocate(IM%rowsip)
      if ( associated(IM%colsip) ) deallocate(IM%colsip)
      if ( associated(IM%rowspp) ) deallocate(IM%rowspp)
      if ( associated(IM%colspp) ) deallocate(IM%colspp)
      if ( associated(IM%rowsic) ) deallocate(IM%rowsic)
      if ( associated(IM%colsic) ) deallocate(IM%colsic)
      if ( associated(IM%rowsif) ) deallocate(IM%rowsif)
      if ( associated(IM%colsif) ) deallocate(IM%colsif)
      if ( associated(IM%rowsfi) ) deallocate(IM%rowsfi)
      if ( associated(IM%colsfi) ) deallocate(IM%colsfi)
      if ( associated(IM%rowsout) ) deallocate(IM%rowsout)
      if ( associated(IM%colsout) ) deallocate(IM%colsout)
      if ( associated(IM%storeinfo) ) deallocate(IM%storeinfo)
      if ( associated(IM%sendunknowns) ) deallocate(IM%sendunknowns)
      if ( associated(IM%recunknowns) ) deallocate(IM%recunknowns)
      if ( .not. IM%direct .and. associated(IM%diag) ) deallocate(IM%diag)
      if ( associated(IM%diagprec) ) deallocate(IM%diagprec)
      if ( associated(IM%rowsQ) ) deallocate(IM%rowsQ)
      if ( associated(IM%colsQ) ) deallocate(IM%colsQ)
      if ( associated(IM%rowsG) ) deallocate(IM%rowsG)
      if ( associated(IM%colsG) ) deallocate(IM%colsG)
      if ( associated(IM%rowsDG) ) deallocate(IM%rowsDG)
      if ( associated(IM%colsDG) ) deallocate(IM%colsDG)
      if ( associated(IM%rowsPM) ) deallocate(IM%rowsPM)
      if ( associated(IM%colsPM) ) deallocate(IM%colsPM)
      if ( associated(IM%diagQ) ) deallocate(IM%diagQ)
      if ( associated(IM%diagPM) ) deallocate(IM%diagPM)
      if ( associated(IM%diagDG) ) deallocate(IM%diagDG)

      iseqim = intmatlc
      call seprmmatinfo  ! remove matrix arrays corresponding to iseqim

      intmatlc = 0
      iseqim = iseqimsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'intmatlc, lastintmat, iseqim', &
                          intmatlc, lastintmat, iseqim
         write(irefwr,*) 'End seprmintmt'

      end if  ! ( debug )
      call erclos ( 'seprmintmt' )

      end subroutine seprmintmt
