      subroutine seprmmeshinfo ( kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove mesh array with sequence number kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: kmesh

!     kmesh        i/o   Defines mesh by integer sequence number
!                        If it already exists the old mesh info is removed
!                        New meshinfo is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPRMMESHINFO1 remove subarrays of kmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmmeshinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmmeshinfo'
         write(irefwr,1) 'kmesh, lastmesh', kmesh, lastmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'kmesh', kmesh
      if ( kmesh==lastmesh ) then

!     --- kmesh = lastmesh, decrease lastmesh

         lastmesh = lastmesh-1

         do i = lastmesh, 1, -1
            if ( mesharr(lastmesh)==0 ) then
               lastmesh = lastmesh-1
            else
               exit
            end if  ! ( mesharr(lastmesh)==0 )
         end do  ! i = lastmesh, 1, -1

      end if  ! ( kmesh==lastmesh )
      mesharr(kmesh) = 0
      iseqkm = kmesh
      nextfreekm = min(nextfreekm,iseqkm)

      call seprmmeshinfo1

      deallocate ( km(kmesh)%kmesh )

      kmesh = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'kmesh, lastmesh', kmesh, lastmesh
         write(irefwr,*) 'End seprmmeshinfo'

      end if  ! ( debug )
      call erclos ( 'seprmmeshinfo' )

      end subroutine seprmmeshinfo
