      subroutine seprmmeshinfo1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove array with sequence number iseq from kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulemeshinf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
     integer :: i

!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHDELKMESHAD  Delete kmesh part ad
!     MSHDELKMESHH   Delete kmesh part h and subarrays
!     SEPRMNMESH     Remove subarrays of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmmeshinfo1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmmeshinfo1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( associated(km(iseqkm)%blockinfo) ) then
         deallocate ( km(iseqkm)%blockinfo, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'blockinfo' )
         nullify(km(iseqkm)%blockinfo)
      end if  ! ( associated(km(iseqkm)%blockinfo) )

      if ( associated(km(iseqkm)%curveusrp) ) then
         deallocate ( km(iseqkm)%curveusrp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'curveusrp' )
         nullify(km(iseqkm)%curveusrp)
      end if  ! ( associated(km(iseqkm)%curveusrp) )

      if ( associated(km(iseqkm)%globnodes) ) then
         deallocate ( km(iseqkm)%globnodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'globnodes' )
         nullify(km(iseqkm)%globnodes)
      end if  ! ( associated(km(iseqkm)%globnodes) )

      if ( associated(km(iseqkm)%ielgrp_nodes) ) then
         deallocate ( km(iseqkm)%ielgrp_nodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ielgrp_nodes' )
         nullify(km(iseqkm)%ielgrp_nodes)
      end if  ! ( associated(km(iseqkm)%ielgrp_nodes) )

      if ( associated(km(iseqkm)%iinputkm) ) then
         deallocate ( km(iseqkm)%iinputkm, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iinputkm' )
         nullify(km(iseqkm)%iinputkm)
      end if  ! ( associated(km(iseqkm)%iinputkm) )

      if ( associated(km(iseqkm)%intprop) ) then
         deallocate ( km(iseqkm)%intprop, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'intprop' )
         nullify(km(iseqkm)%intprop)
      end if  ! ( associated(km(iseqkm)%intprop) )

      if ( associated(km(iseqkm)%iobscurv) ) then
         deallocate ( km(iseqkm)%iobscurv, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iobscurv' )
         nullify(km(iseqkm)%iobscurv)
      end if  ! ( associated(km(iseqkm)%iobscurv) )

      if ( associated(km(iseqkm)%iouterbounseq) ) then
         deallocate ( km(iseqkm)%iouterbounseq, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iouterbounseq' )
         nullify(km(iseqkm)%iouterbounseq)
      end if  ! ( associated(km(iseqkm)%iouterbounseq) )

      if ( km(iseqkm)%kmesh(15)>0 ) then
         deallocate ( km(iseqkm)%kmesha, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmesha' )
         nullify(km(iseqkm)%kmesha)
      end if  ! ( associated(km(iseqkm)%kmesha) )

      if ( km(iseqkm)%kmesh(16)>0 ) then
         deallocate ( km(iseqkm)%kmeshb, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshb' )
         nullify(km(iseqkm)%kmeshb)
      end if  ! ( associated(km(iseqkm)%kmeshb) )

      if ( km(iseqkm)%kmesh(17)>0 ) then
         deallocate ( km(iseqkm)%kmeshc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshc' )
         nullify(km(iseqkm)%kmeshc)
      end if  ! ( associated(km(iseqkm)%kmeshc) )

      if ( associated(km(iseqkm)%kmeshcglob) ) then
         deallocate ( km(iseqkm)%kmeshcglob, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshcglob' )
         nullify(km(iseqkm)%kmeshcglob)
      end if  ! ( associated(km(iseqkm)%kmeshcglob) )

      if ( associated(km(iseqkm)%kmeshcobst) ) then
         deallocate ( km(iseqkm)%kmeshcobst, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshcobst' )
         nullify(km(iseqkm)%kmeshcobst)
      end if  ! ( associated(km(iseqkm)%kmeshcobst) )

      if ( associated(km(iseqkm)%kmeshcpoint) ) then
         deallocate ( km(iseqkm)%kmeshcpoint, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshcpoint' )
         nullify(km(iseqkm)%kmeshcpoint)
      end if  ! ( associated(km(iseqkm)%kmeshcpoint) )

      if ( associated(km(iseqkm)%kmeshd) ) then
         deallocate ( km(iseqkm)%kmeshd, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshd' )
         nullify(km(iseqkm)%kmeshd)
      end if  ! ( associated(km(iseqkm)%kmeshd) )

      if ( associated(km(iseqkm)%kmeshe) ) then
         deallocate ( km(iseqkm)%kmeshe, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshe' )
         nullify(km(iseqkm)%kmeshe)
      end if  ! ( associated(km(iseqkm)%kmeshe) )

      if ( km(iseqkm)%kmesh(20)>0 ) then
         deallocate ( km(iseqkm)%kmeshf, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshf' )
         nullify(km(iseqkm)%kmeshf)
      end if  ! ( km(iseqkm)%kmesh(20)>0 )

      if ( km(iseqkm)%kmesh(21)>0 ) then
         deallocate ( km(iseqkm)%kmeshg, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshg' )
         nullify(km(iseqkm)%kmeshg)
      end if  ! ( km(iseqkm)%kmesh(21)>0 )

      if ( km(iseqkm)%kmesh(22)>0 ) call mshdelkmeshh

      if ( associated(km(iseqkm)%kmeshj) ) then
         deallocate ( km(iseqkm)%kmeshj, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshj' )
         nullify(km(iseqkm)%kmeshj)
      end if  ! ( associated(km(iseqkm)%kmeshj) )

      if ( km(iseqkm)%kmesh(25)>0 ) then
         deallocate ( km(iseqkm)%kmeshk, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshk' )
         nullify(km(iseqkm)%kmeshk)
      end if  ! ( km(iseqkm)%kmesh(25)>0 )

      if ( km(iseqkm)%kmesh(26)>0 ) then
         deallocate ( km(iseqkm)%kmeshl, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshl' )
         nullify(km(iseqkm)%kmeshl)
      end if  ! ( km(iseqkm)%kmesh(26)>0 )

      if ( associated(km(iseqkm)%kmeshlglob) ) then
         deallocate ( km(iseqkm)%kmeshlglob, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshlglob' )
         nullify(km(iseqkm)%kmeshlglob)
      end if  ! ( associated(km(iseqkm)%kmeshlglob) )

      if ( km(iseqkm)%kmesh(27)>0 ) then
         deallocate ( km(iseqkm)%kmeshm, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshm' )
         nullify(km(iseqkm)%kmeshm)
      end if  ! ( km(iseqkm)%kmesh(27)>0 )

      if ( associated(km(iseqkm)%kmeshmglob) ) then
         deallocate ( km(iseqkm)%kmeshmglob, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshmglob' )
         nullify(km(iseqkm)%kmeshmglob)
      end if  ! ( associated(km(iseqkm)%kmeshmglob) )

      if ( km(iseqkm)%kmesh(28)>0 ) then
         deallocate ( km(iseqkm)%kmeshn, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshn' )
         nullify(km(iseqkm)%kmeshn)
      end if  ! ( km(iseqkm)%kmesh(28)>0 )

      if ( associated(km(iseqkm)%kmeshnglob) ) then
         deallocate ( km(iseqkm)%kmeshnglob, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshnglob' )
         nullify(km(iseqkm)%kmeshnglob)
      end if  ! ( associated(km(iseqkm)%kmeshnglob) )

      if ( km(iseqkm)%kmesh(29)>0 ) then
         deallocate ( km(iseqkm)%kmesho, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmesho' )
         nullify(km(iseqkm)%kmesho)
      end if  ! ( km(iseqkm)%kmesh(29)>0 )

      if (km(iseqkm)%kmesh(65)>0 ) then
         deallocate ( km(iseqkm)%kmeshpar, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshpar' )
         nullify(km(iseqkm)%kmeshpar)
      end if  ! ( associated(km(iseqkm)%kmeshpar) )

      if ( km(iseqkm)%kmesh(31)>0 ) then
         deallocate ( km(iseqkm)%kmeshq, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshq' )
         nullify(km(iseqkm)%kmeshq)
      end if  ! ( km(iseqkm)%kmesh(31)>0 )

      if ( km(iseqkm)%kmesh(32)>0 ) then
         deallocate ( km(iseqkm)%kmeshr, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshr' )
         nullify(km(iseqkm)%kmeshr)
      end if  ! ( km(iseqkm)%kmesh(32)>0 )

      if ( km(iseqkm)%kmesh(33)>0 ) then
         deallocate ( km(iseqkm)%kmeshs, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshs' )
         nullify(km(iseqkm)%kmeshs)
      end if  ! ( km(iseqkm)%kmesh(33)>0 )

      if ( km(iseqkm)%kmesh(36)>0 ) then
         deallocate ( km(iseqkm)%kmesht, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmesht1d' )
         nullify(km(iseqkm)%kmesht)
      end if  ! ( km(iseqkm)%kmesh(36)>0 )

      if ( associated(km(iseqkm)%kmeshu) ) then
         deallocate ( km(iseqkm)%kmeshu, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshu' )
         nullify(km(iseqkm)%kmeshu)
      end if  ! ( associated(km(iseqkm)%kmeshu) )

      if ( associated(km(iseqkm)%kmeshv) ) then
         deallocate ( km(iseqkm)%kmeshv, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshv' )
         nullify(km(iseqkm)%kmeshv)
      end if  ! ( associated(km(iseqkm)%kmeshv) )

      if ( associated(km(iseqkm)%kmeshw) ) then
         deallocate ( km(iseqkm)%kmeshw, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshw' )
         nullify(km(iseqkm)%kmeshw)
      end if  ! ( associated(km(iseqkm)%kmeshw) )

      if ( km(iseqkm)%kmesh(47)>0 ) then
         if ( associated(km(iseqkm)%kmobst) ) then
            do i = 1, km(iseqkm)%kmesh(42)
               if ( associated(km(iseqkm)%kmobst(i)%iobgrid2d ) ) &
                  deallocate ( km(iseqkm)%kmobst(i)%iobgrid2d )
               if ( associated(km(iseqkm)%kmobst(i)%iobgrid2d1 ) ) &
                  deallocate ( km(iseqkm)%kmobst(i)%iobgrid2d1 )
               if ( associated(km(iseqkm)%kmobst(i)%iobgrid3d ) ) &
                  deallocate ( km(iseqkm)%kmobst(i)%iobgrid3d )
            end do  ! i = 1, numobst
            deallocate ( km(iseqkm)%kmobst )
         end if  ! ( associated(km(iseqkm)%kmobst) )
         deallocate ( km(iseqkm)%kmeshx, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshx' )
         nullify(km(iseqkm)%kmeshx)
      end if  ! ( associated(km(iseqkm)%kmeshx) )

      if ( associated(km(iseqkm)%kmeshxsur) ) then
         deallocate ( km(iseqkm)%kmeshxsur, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshxsur' )
         nullify(km(iseqkm)%kmeshxsur)
      end if  ! ( associated(km(iseqkm)%kmeshxsur) )

      if ( associated(km(iseqkm)%kmeshz) ) then
         deallocate ( km(iseqkm)%kmeshz, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshz' )
         nullify(km(iseqkm)%kmeshz)
      end if  ! ( associated(km(iseqkm)%kmeshz) )

      if ( associated(km(iseqkm)%mappingpar) ) then
         deallocate ( km(iseqkm)%mappingpar, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'mappingpar' )
         nullify(km(iseqkm)%mappingpar)
      end if  ! ( associated(km(iseqkm)%mappingpar) )

      if ( associated(km(iseqkm)%mappingparcp) ) then
         deallocate ( km(iseqkm)%mappingparcp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'mappingparcp' )
         nullify(km(iseqkm)%mappingparcp)
      end if  ! ( associated(km(iseqkm)%mappingparcp) )

      if ( associated(km(iseqkm)%mappar) ) then
         do i = 0, iacnodes
            if ( associated(km(iseqkm)%mappar(i)%arr) ) then
               deallocate ( km(iseqkm)%mappar(i)%arr, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'mappar(i)%arr' )
               nullify(km(iseqkm)%mappar(i)%arr)
            end if  ! ( associated(km(iseqkm)%mappar(i)%arr) )
         end do  ! i = 0, iacnodes
         deallocate ( km(iseqkm)%mappar, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'mappar' )
         nullify(km(iseqkm)%mappar)
      end if  ! ( associated(km(iseqkm)%mappingparcp) )

      if ( associated(km(iseqkm)%nmesh) ) then
         deallocate ( km(iseqkm)%nmesh, stat = error )
         call seprmnmesh
      end if  ! ( associated(km(iseqkm)%nmesh) )

      if ( associated(km(iseqkm)%node_elgrps) ) then
         deallocate ( km(iseqkm)%node_elgrps, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'node_elgrps' )
         nullify(km(iseqkm)%node_elgrps)
      end if  ! ( associated(km(iseqkm)%node_elgrps) )

      if ( associated(km(iseqkm)%outerboun) ) then
         deallocate ( km(iseqkm)%outerboun, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'outerboun' )
         nullify(km(iseqkm)%outerboun)
      end if  ! ( associated(km(iseqkm)%outerboun) )

      if ( associated(km(iseqkm)%outercurves) ) then
         deallocate ( km(iseqkm)%outercurves, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'outercurves' )
         nullify(km(iseqkm)%outercurves)
      end if  ! ( associated(km(iseqkm)%outercurves) )

      if ( associated(km(iseqkm)%pointnel) ) then
         deallocate ( km(iseqkm)%pointnel, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'pointnel' )
         nullify(km(iseqkm)%pointnel)
      end if  ! ( associated(km(iseqkm)%pointnel) )

      if ( associated(km(iseqkm)%pointkmsc) ) then
         deallocate ( km(iseqkm)%pointkmsc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'pointkmsc' )
         nullify(km(iseqkm)%pointkmsc)
      end if  ! ( associated(km(iseqkm)%pointkmsc) )

      if ( associated(km(iseqkm)%surfnodes) ) then
         deallocate ( km(iseqkm)%surfnodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'surfnodes' )
         nullify(km(iseqkm)%surfnodes)
      end if  ! ( associated(km(iseqkm)%surfnodes) )

      if ( associated(km(iseqkm)%transinfo) ) then
         deallocate ( km(iseqkm)%transinfo, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'transinfo' )
         nullify(km(iseqkm)%transinfo)
      end if  ! ( associated(km(iseqkm)%transinfo) )

      if ( associated(km(iseqkm)%volume_surfs) ) then
         deallocate ( km(iseqkm)%volume_surfs, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'volume_surfs' )
         nullify(km(iseqkm)%volume_surfs)
      end if  ! ( associated(km(iseqkm)%volume_surfs) )

      if ( km(iseqkm)%kmesh(23)>0 ) then
         deallocate ( km(iseqkm)%coor, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coor' )
         nullify(km(iseqkm)%coor)
      end if  ! ( associated(km(iseqkm)%coor) )

      if ( associated(km(iseqkm)%coorglob) ) then
         deallocate ( km(iseqkm)%coorglob, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coorglob' )
         nullify(km(iseqkm)%coorglob)
      end if  ! ( associated(km(iseqkm)%coorglob) )

      if ( associated(km(iseqkm)%coorobst) ) then
         deallocate ( km(iseqkm)%coorobst, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coorobst' )
         nullify(km(iseqkm)%coorobst)
      end if  ! ( associated(km(iseqkm)%coorobst) )

      if ( associated(km(iseqkm)%elsize) ) then
         deallocate ( km(iseqkm)%elsize, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'elsize' )
         nullify(km(iseqkm)%elsize)
         nullify(km(iseqkm)%elsize)
      end if  ! ( associated(km(iseqkm)%elsize) )

      if ( associated(km(iseqkm)%kmeshquant) ) then
         deallocate ( km(iseqkm)%kmeshquant, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshquant' )
         nullify(km(iseqkm)%kmeshquant)
      end if  ! ( associated(km(iseqkm)%kmeshquant) )

      if ( associated(km(iseqkm)%normal) ) then
         deallocate ( km(iseqkm)%normal, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'normal' )
         nullify(km(iseqkm)%normal)
      end if  ! ( associated(km(iseqkm)%normal) )

      if ( associated(km(iseqkm)%realprop) ) then
         deallocate ( km(iseqkm)%realprop, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'realprop' )
         nullify(km(iseqkm)%realprop)
      end if  ! ( associated(km(iseqkm)%realprop) )

      if ( associated(km(iseqkm)%rinputkm) ) then
         deallocate ( km(iseqkm)%rinputkm, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rinputkm' )
         nullify(km(iseqkm)%rinputkm)
      end if  ! ( associated(km(iseqkm)%rinputkm) )

      if ( associated(km(iseqkm)%rmeshac) ) then
         deallocate ( km(iseqkm)%rmeshac, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rmeshac' )
         nullify(km(iseqkm)%rmeshac)
      end if  ! ( associated(km(iseqkm)%rmeshac) )

      if ( km(iseqkm)%kmesh(68)>0 ) then
         deallocate ( km(iseqkm)%rmeshx, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rmeshx' )
         nullify(km(iseqkm)%rmeshx)
      end if  ! ( associated(km(iseqkm)%rmeshx) )

      if ( associated(km(iseqkm)%pointlevel) ) then
         deallocate ( km(iseqkm)%pointlevel, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'pointlevel' )
         nullify(km(iseqkm)%pointlevel)
      end if  ! ( associated(km(iseqkm)%pointlevel) )

      if ( associated(km(iseqkm)%levelstruc) ) then
         deallocate ( km(iseqkm)%levelstruc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'levelstruc' )
         nullify(km(iseqkm)%levelstruc)
      end if  ! ( associated(km(iseqkm)%levelstruc) )

      if ( associated(km(iseqkm)%rennested) ) then
         deallocate ( km(iseqkm)%rennested, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rennested' )
         nullify(km(iseqkm)%rennested)
      end if  ! ( associated(km(iseqkm)%rennested) )

!     --- Remove kmesh part ad

      if ( km(iseqkm)%kmesh(52)>0 ) call mshdelkmeshad

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprmmeshinfo1'

      end if  ! ( debug )
      call erclos ( 'seprmmeshinfo1' )

      end subroutine seprmmeshinfo1
