      subroutine seprmsolsinfo
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove set of solution arrays corresponding to iseqkp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     remove
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih

!     i              Loop variable
!     ih             Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPRMSOLINFO   Remove array with sequence number ivectlc from ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmsolsinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmsolsinfo'
         call prinin ( solarr, lastsol, 'solarr' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, lastsol
         if ( solarr(i)>0 ) then
            ih = i
            if ( debug ) &
               write(irefwr,1) 'i, iseqkp, ks(i)%iseqkp', &
                                i, iseqkp, ks(i)%iseqkp
            if ( ks(i)%iseqkp==iseqkp ) call seprmsolinfo ( ih )
         end if  ! ( solarr(i)>0 )
      end do  ! i = 1, lastsol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprmsolsinfo'

      end if  ! ( debug )
      call erclos ( 'seprmsolsinfo' )

      end subroutine seprmsolsinfo
