      subroutine sepsurfsnod ( nsurfaces, nodes, nnodes, isurfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the surfaces stored in array surfaces
!     in array nodes
!     The number of nodes in nodes is stored in nnodes
!     At this moment the surfaces must be part of kmeshn
!     Old subroutine ersurfsnod
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfaces, nodes(*), nnodes, isurfaces(*)

!     isurfaces      i    Array containing the surfaces from which the nodes
!                         must be found
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSURFSNOD1   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!     SEPSURFSNOD2   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    nnodes := 0
!    For all surfaces isurnr in isurfaces do
!       inelem := number of elements in isurnr
!       inpelm := number of nodes in elements in isurnr
!       istart := starting address of surface in kmeshc
!       For all nodes inode in surface do
!          if ( iwork(inode)=0 ) then
!             iwork(inode):=1
!             nnodes := nnodes+1
!             nodes(nnodes) := inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsurfsnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsurfsnod'
         call prinin ( isurfaces, nsurfaces, 'isurfaces' )

      end if
      if ( ierror/=0 ) go to 1000

      nnodes = 0

      if ( kelmn>0 ) then

!     --- kmeshn has been filled

         call sepsurfsnod1 ( nsurfaces, isurfaces, nodes, nnodes )

      else

!     --- kmeshn has not been filled

         call sepsurfsnod2 ( nsurfaces, isurfaces, nodes, nnodes )

      end if  ! ( kelmn>0 )

1000  call erclos ( 'sepsurfsnod' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsurfsnod'

      end if

      end subroutine sepsurfsnod
