      subroutine sepsurfsnod1 ( nsurfaces, isurfaces, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the surfaces stored in array surfaces
!     in array nodes
!     It is assumed that kmesh part n has been filled
!     Old subroutine ersurfsnod1
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfaces, isurfaces(nsurfaces), nodes(*), nnodes

!     isurfaces      i    Array containing the surfaces from which the nodes
!                         must be found
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsurnr, istart, isurnr, inpelm
      integer, allocatable :: iwork(:)

!     inpelm         number of points per element
!     istart         First position of a surface in kmeshn minus 1
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
!     jsurnr         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODSKMC1 Fill array nodes for elements 1 to inelem
!     EROPEN         Produces concatenated name of local subroutine
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsurfsnod1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsurfsnod1'
         call prinin ( isurfaces, nsurfaces, 'isurfaces' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0

!     --- Loop over surfaces

      do jsurnr = 1, nsurfaces

         isurnr = isurfaces(jsurnr)
         call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                           isurnr )

!        --- For all nodes inode in surface fill nodes

         call erfillnodskmc1 ( nodes, nnodes, kmeshn(istart+1), inelem, &
                               inpelm, iwork )

      end do  ! jsurnr = 1, nsurfaces

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'sepsurfsnod1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsurfsnod1'

      end if  ! ( debug )

      end subroutine sepsurfsnod1
