      subroutine to0017 ( indprfloc, nunkploc, ivectr, jcount )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.8  date 12-03-1999 Extension with type 129
!        version  2.7  date 12-12-1998 Correction for type 126
!        version  2.6  date 06-05-1998 New definition kelgrp
!
!   copyright (c) 1984-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   determine number of degrees of freedom jcount in an element
!   of ielgrp for existing ivectr (ivctr(2)=110,115,116,126,129)
!   (for some vector jcount=icount in common /cactl/)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprfloc, nunkploc, ivectr, jcount

!     indprfloc     i    indication if kprob part f has been filled (>0) or not (0)
!     ivectr        i    standard sepran array denoting usol or vector of special
!                        structure defined in nodal points
!     jcount        o    Number of quantities to be stored in element vector
!     nunkploc      i    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmaprob, ivec

!     i              Counting variable
!     ivec           Sequence number of vector of special structure
!     kelmaprob      Indicates if KMESH part a has been filled (>0) or not (0)
!                    If kelmaprob>0, kelmaprob is the starting address of
!                    kprob part a in kprob
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     200   First input vector is of illegal type
!     206   Input vector with illegal sequence number ivec
!     208   Solution vector (type=110) with zero degrees of freedom
!     209   Vector of special structure (type=115) with zero degrees of freedom
!     210   Vector defined per element (type=116) with zero degrees of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- test if vector exists (itypvc=110,115,116,126,129 allowed)

      call eropen ( 'to0017' )
      jcount=0
      select case ( ks(ivectr)%typevec )

         case ( :109, 111:114, 117:125, 127, 128, 130:135, 137: )

!        --- error : illegal type of vector (unequal to 110,115,116,126,129)

            call errint ( ks(ivectr)%typevec, 1 )
            call errsub ( 200, 1, 0, 0)

         case ( 110 )

!        --- vector is of type usol , itypvc = 110

            if ( indprfloc==0 ) then

!           --- indprfloc = 0

               jcount=nunkploc*inpelm

            else

!           --- number of degrees of freedom not constant

               if ( kelma==0 ) then

!              --- kelma = 0

                  kelmaprob=(ielgrp-1)*inpelm

               else

!              --- kelma>0

                  kelmaprob=kmesha(ielgrp)

               end if
               kelmaprob=kelmaprob+kprobloc(14)-1
               do i = 1, inpelm
                  jcount=jcount+kprobloc(kelmaprob+i)
               end do

            end if
            if ( jcount<0 ) then
               call errint ( irelem, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 208, 2, 0, 0 )
            end if

         case ( 115 )

!        --- vector of special structure , itypvc = 115 , ivectr(4)=ivec

            ivec=ks(ivectr)%ivec
            if ( ivec<=0 .or. ivec>kprobloc(6) ) then

!           --- error : vector with sequence number ivec not defined

               call errint ( ivec, 1 )
               call errint ( kprobloc(6), 2 )
               call errsub ( 206, 2, 0, 0 )
               go to 1000

            end if
            if ( indprfloc==0 ) then

!           --- error : number of degrees of freedom = 0

               jcount=nunkploc*inpelm

            else

!           --- number of degrees of freedom not constant

               if ( kelma==0 ) then

!              --- kelma = 0

                  kelmaprob=(ielgrp-1)*inpelm

               else

!              --- kelma>0

                  kelmaprob=kmesha(ielgrp)

               end if
        !       kelmaprob=kelmaprob+(ivec-1)*kprobloc(39)+kprobloc(18)-1
               do i=1,inpelm
                  jcount=jcount+kprobe(i,ivec)
               end do

            end if

            if ( jcount<0 ) then
               call errint ( ivec, 1 )
               call errint ( irelem, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 209, 3, 0, 0 )
            end if

         case ( 116, 136 )

!        --- vector of special structure is defined per element , itypvc = 116

            jcount=nunkploc
            if ( jcount==0 ) then
               call errint ( ielgrp, 1 )
               call errsub ( 210, 1, 0, 0 )
            end if

         case ( 126 )

!        --- vector of special structure is defined per element per node,
!            itypvc = 126

            jcount=nunkploc*inpelm
            if ( jcount==0 ) then
               call errint ( ielgrp, 1 )
               call errsub ( 210, 1, 0, 0 )
            end if

         case ( 129 )

!        --- vector of special structure is defined per element per integration
!            point, itypvc = 129

            jcount=nunkploc*m
            if ( jcount==0 ) then
               call errint ( ielgrp, 1 )
               call errsub ( 210, 1, 0, 0 )
            end if

      end select  ! case ( ks(ivectr)%typevec )

1000  call erclos ( 'to0017' )
      end
