      subroutine todivvec ( yvec, xvec1, xvec2, nusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-09-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Divide 2 vectors components wise, i.e. y_i = x1_i / x2_i
!     y_i may be equal to one of the other vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     subdivision
!     subdivision
!     vector
!     inner_product
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol
      double precision yvec(*), xvec1(*), xvec2(*)

!     nusol          i    Total number of unknowns
!     xvec1          i    Input vector
!     xvec2          i    Input vector
!     yvec           o    result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'todivvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from todivvec'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      do i = 1, nusol
         yvec(i) = xvec1(i)/xvec2(i)
      end do  ! i = 1, nusol

1000  call erclos ( 'todivvec' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End todivvec'

      end if

      end
