      subroutine topqu6 ( number, idegfd, usol, solutn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-1992
!
!   copyright (c) 1992-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the solution array usol (which may also be an array of special
!     structure) into the new organised solution array solutn
!     The new array has length number x npoint.
!     Array solutn contains the unknowns ordered corresponding to the nodes
!     Depending on the value of ichois, points are computed by interpolation
!     or not
!     USOL is a complex array, however, SOLUTN is real. With the parameter
!     idegfd it is indicated which component is used
!     TOPQU6 is similar to TOPQU4 (real variant) and TOPQU5 (complex SOLUTN)
! **********************************************************************
!
!                       KEYWORDS
!
!     extract_unknown
!     interpolation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, idegfd
      double complex  usol(*)
      double precision solutn(number,npoint)

!     idegfd  i   The first degree of freedom to be used is number idegfd in
!                 each node
!                 Since the solution vector is complex, idegfd refers to
!                 either the real or the imaginary part of the vector in
!                 the following way:
!                 if mod(idegfd,2)=1, idegfd refers to the real part of
!                 unknown (idegfd+1)/2 else
!                 if mod(idegfd,2)=0, idegfd refers to the imaginary part of
!                 unknown idegfd/2
!     number  i   Number of degrees of freedom per point that must be copied
!     solutn  o   Array of length number x npoint in which the reordered
!                 solution array is stored in the sequence:
!                 u (x ) = solutn ( i,k )  (i=1(1)number; k=1(1)npoint)
!                  i  k
!     usol    i   Real solution array of length nusol in which the solution
!                 to be copied is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inusol, inunkp, jnusol, ideg
      logical compl

!    compl    Indicates if the imaginary part (true) or the real part of
!             the vector is required (false)
!    i        Loop variable
!    ideg     Value of idegfd after correction for complex
!    inunkp   Number of unknowns in a node
!    inusol   Position in array USOL.
!             If zero, no solution is available
!    j        Loop variable
!    jnusol   Renumbered position in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array solutn

      do j = 1, number
         do i = 1, npoint
            solutn ( j,i ) = 0d0
         end do
      end do

!     --- Copy usol into solutn

      compl = mod(idegfd,2)==0
      ideg  = (idegfd+1)/2

      if ( indprpi>0 ) then

!     --- indprpi > 0, use array kprob part p

         if ( compl ) then

!        --- imaginary part

            do j = 1,number
               do i = 1, npoint
                  inusol = kprobpi(i,ideg+j-1)
                  if (inusol>0) solutn(j,i) = dimag(usol(inusol))
               end do
            end do

         else

!        --- real part

            do j = 1,number
               do i = 1, npoint
                  inusol = kprobpi(i,ideg+j-1)
                  if (inusol>0) solutn(j,i) = dble(usol(inusol))
               end do
            end do

         end if

      else if ( indprfi>0 ) then

!     --- indprfi > 0, use array kprob part f  ( not constant number of degrees
!                                               of freedom )

         if ( compl ) then

!        --- imaginary part

            do i = 1, npoint
               inusol = kprobfi(i)
               inunkp = kprobfi(i+1) - inusol
               do j = 1, min ( number, inunkp-ideg+1)
                  jnusol = inusol+j+ideg-1
                  solutn(j,i) = dimag(usol(jnusol))
               end do
            end do

         else

!        --- real part

            do i = 1, npoint
               inusol = kprobfi(i)
               inunkp = kprobfi(i+1) - inusol
               do j = 1, min ( number, inunkp-ideg+1)
                  jnusol = inusol+j+ideg-1
                  solutn(j,i) = dble(usol(jnusol))
               end do
            end do

         end if

      else

!     --- indprfi = 0 and indprpi = 0  Constant number of degrees of freedom

         if ( compl ) then

!        --- imaginary part

            do j = 1, min (number,nphysi-ideg+1)
               do i = 1, npoint
                  inusol = (i-1)*nphysi+j+ideg-1
                  solutn(j,i) = dimag(usol(inusol))
               end do
            end do

         else

!        --- real part

            do j = 1, min (number,nphysi-ideg+1)
               do i = 1, npoint
                  inusol = (i-1)*nphysi+j+ideg-1
                  solutn(j,i) = dble(usol(inusol))
               end do
            end do

         end if

      end if

      end
