      subroutine toreordr ( kprobh, rhsd, nusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-10-2010 remove work
!        version  1.0  date 01-01-1998
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Map renumbered array of solution type back to the usual sequence, i.e.
!   numbered per nodal point
!   This is the sequence that corresponds to the matrices
!   Real version
! **********************************************************************
!
!                       KEYWORDS
!
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol, kprobh(nusol)
      double precision rhsd(nusol)

!     kprobh     i    KPROB part h: contains the new numbering of the unknowns
!     nusol      i    Total number of unknowns
!     rhsd      i/o   Right-hand-side vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: work(:)

!     work           Work array of length nusol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toreordr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toreordr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nusol, 'work' )
      if ( ierror/=0 ) go to 1000

      work = rhsd(kprobh(1:nusol))
      rhsd = work

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'toreordr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toreordr'

      end if  ! ( debug )

      end
