      subroutine writb4 ( inum, numarr, namevc, isol, iwork )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.1  date 21-11-2012 Long integers
!        version  5.0  date 03-04-1999 Extra parameters ibuffr, buffer
!        version  4.1  date 18-09-1995 JvdZ: improve iprob
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    writes array of isol type to backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulecomio
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character * (*) namevc
      integer inum, numarr, isol, iwork(*)

!     inum       i    Choice parameter at the input of subroutine WRITBS
!                     When INUM<=0, ISOL is written on the next free positions
!                     of file 2. When INUM = -1, the subroutine prints a text, &
!                     as described in the Programmer's Guide.
!                     When INUM>0, ISOL is written on positions of a preceding
!                     written array, with array number INUM.
!     isol       i    Standard SEPRAN array containing information of the
!                     solution
!     iwork           Work array of length NREC2 which is used as a buffer
!     namevc     i    Name of the array to be written to file 2.
!                     Character variable, is used for printing only
!     numarr     o    At output NUMARR gets the sequence number of ISOL in
!                     file 2.
!                     NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(2), leng, nusol, lensol, k, itemp(8)
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     ihelp          integer work array to store two positions from infor into
!                    the backing storage in order to have the same files for
!                    old and new versions of the backing storage subroutines
!     itemp          work array to store some information to be sent
!     k              Counting variable
!     leng           Help variable to compute the length of some arrays
!     lensol         Length of array usol
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPGATHER      Compute global vector in case of parallel computations by
!                    combining parts of the vectors from the nodes
!     WRIT00         writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writb4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from writb4'
         write(irefwr,*) 'parallel', parallel
         write(irefwr,1) 'inodenr, inum', inodenr, inum
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get solution array

      if ( parallel ) then

!     --- parallel case, get global solution

         call sepgather ( isol )
         if ( inodenr>1 ) go to 1000
         usoltemp => ks(isol)%solglob

      else

!     --- serial case

         usoltemp => ks(isol)%sol

      end if  ! ( parallel )

      if ( debug ) write(irefwr,1) 'size vector', size(usoltemp)
      lensol = size(usoltemp)*2
      leng = 10 + lensol
      nusol  = ks(isol)%nusol
      ihelp(1) = 1
      ihelp(2) = lensol

!     --- Write part of array to backing storage

      itemp(1) = ks(isol)%compl
      itemp(2) = ks(isol)%typevec
      itemp(3) = ks(isol)%ivec
      itemp(4) = ks(isol)%jcart
      itemp(5) = ks(isol)%nphys
      itemp(6) = ks(isol)%nusol
      itemp(7) = ks(isol)%iprob
      itemp(8) = ks(isol)%iseqkp
      call writ00 ( inum, numarr, namevc, itemp, leng, 1, 8, iwork )
      if ( ks(isol)%typevec<0 ) ks(isol)%typevec = -ks(isol)%typevec

      call writ00 ( inum, numarr, namevc, ihelp, leng, 2, 2, iwork )

      call writ00 ( inum, numarr, namevc, usoltemp, leng, 3, &
                    ihelp(2), iwork )
      if ( debug ) write(irefwr,1) 'leng', leng

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End writb4'

      end if  ! ( debug )
      call erclos ( 'writb4' )

      end subroutine writb4
