      subroutine askcos ( number, inpsep, numscl, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.9  date 05-06-2014 Extension with volume
!        version  4.8  date 03-03-2014 Replace inner_product by dot_product
!        version  4.7  date 19-12-2013 Set scal_max scalar
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword compute i and store in inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), number, numscl, istart

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 20+nelgrp
!                         Array INPSEP is filled as follows:
!                          1  ISEQNR       Sequence number corresponding to
!                                          input.  If iseqnr = -1, &
!                                          the next one will be used
!                          2  IPROB        Problem sequence number
!                                          If iprob = -1, the next one will be
!                                          used
!                          3  IVECTOR      Vector sequence number
!                                          (output vector)
!                                          If ivector = -1, the same value as
!                                          iprob will be used
!                          4: ICHOIS (See prmanv)
!                          5: IDEGFD (See manvec)
!                          6: IPOINT (See manvec)
!                          7: ISPEC  (See manvec)
!                          8: ICHNRM (See manvec)
!                          9: Vector sequence number first input vector
!                         10: Vector sequence number second input vector
!                         11: Scalar sequence number first scalar
!                         12: Scalar sequence number second scalar
!                         13: Sequence number first position in RINSEP
!                         14: iskip  defines if element groups must be skipped
!                             (>0) or not (0)
!                             If >0 (21) positions iskip to iskip+nelgrp-1 are
!                             used to indicate if an element group is skipped
!                             or not
!                             0: do not skip
!                             1: skip
!     istart        i/o   Next free position in inpsep
!     number         i    Scalar sequence number
!     numscl        i/o   Number of scalars read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=29, nparm=20
      integer ncode2(nncode2), ncodmx2(nncode2)
      integer i, norm1, norm2, iseq1, iseq2, ipoint, ipos, k
      character (len=19) code2(nncode2)
      logical found

!     code2          Table with keywords
!     found          Indicates if one of the possibilities has been found (true)
!                    or not (false)
!     i              Counting variable
!     ipoint         Nodal point number
!     ipos           Position in array tabout
!     iseq1          Sequence number of first scalar
!     iseq2          Sequence number of second scalar
!     k              Counting variable
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     norm1          Indicates if norm has been read (>0) or not (-1)
!                    In the first case it also defines the type of norm
!     norm2          Indicates if norm_diffenence has been read (>0) or not (-1)
!                    In the first case it also defines the type of norm
!     nparm          Number of parameters in inpsep before information
!                    concerning the element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     askcos requires input from the standard input file
!     The following input is recognised:
!     STRUCTURE    (has been read already be readal)
!         start of input
!      the next records are interpreted in the sequence they are read
!      the program is carried out in the same sequence
!       COMPUTE_SCALAR i, options (all in one line)
!           where options may have the following contents
!               NORM = j             (compute norm of vector)
!                  j defines the type of norm
!                  Possible values:
!                  1:  sum over absolute values of components
!                  2:  2-norm
!                  3:  max-norm
!                  4:  see 1, however, divided by the number of unknowns
!                  5:  see 2, however, divided by the number of unknowns
!               NORM_DIF = j         (compute norm of difference of 2 vectors)
!                  For a definition of j see NORM
!               VECTOR1 = s        Sequence number of first vector
!               VECTOR2 = s        Sequence number of second vector
!               AVERAGE            (Compute average of first vector)
!               DEGREE_OF_FREEDOM i  (Restrict to degree of freedom i, &
!                                     if i = 0 (default) all degrees of freedom
!                                     are used, if i=-1, all degrees of
!                                     freedom except the boundary conditions
!               ELEMENT_GROUPS = (s1, s2, ...)           (Default all)
!               SKIP_ELEMENT_GROUPS = (s1, s2, ... )     (Default none)
!               DOT_PRODUCT
!               MIN_MAX,  [SCAL_MAX = i1],  [COOR_MIN = i2],  [COOR_MAX = i3], &
!                         [ABS_VALUE]
!               EXTRACT_VALUE, [NODE = i], [USER_POINT = i]
!               MEAN_AREA
!               MIN_AREA
!               VOLUME
!               POINT_NUMBER ( ZERO_LEVELSET i )
!               MEAN_VALUE
!               STANDARD_DEVIATION
!       This program only interprets the part options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1888   None of the possibilities: NORM, NORM_DIF and AVERAGE has been found
!    1889   From the possibilities: NORM, NORM_DIF and AVERAGE at least two are
!           found in the record
!    1890   Not VECTOR 1 found in the record
!    1891   Not VECTOR 2 found in the record
!    1900   Both element groups and skip element groups given
!    2177   No NODE number given, whereas EXTRACT_VALUE is required.
!    3086   Not zero_level_set found following point_numbers
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'NORM_DIF', 'NORM', 'VECTOR', 'VECTOR', &
                  'AVERAGE', 'DEGREE_OF_FREEDOM', 'DOT_PRODUCT', &
                  'MIN_MAX', 'SCAL_MAX', 'EXTRACT_VALUE', 'COOR_MIN', &
                  'COOR_MAX', 'ABS_VALUE', 'NODE', 'USER_POINT', &
                  'ELEMENT_GROUPS', 'SKIP_ELEMENT_GROUPS', 'ONE_NORM', &
                  'TWO_NORM', 'INF_NORM', 'MEAN_ONE_NORM', &
                  'MEAN_TWO_NORM', 'MEAN_AREA', 'MIN_AREA', &
                  'POINT_NUMBER', 'ZERO_LEVEL_SET', 'MEAN_VALUE', &
                  'STANDARD_DEVIATION', 'VOLUME' /
      data ncode2 / 8, 4, 3, 3, 4,   3, 5, 5, 7, 5, &
                   8, 8, 5, 4, 7,   4, 4, 4, 4, 4, &
                   8, 8, 8, 6, 5,   4, 8, 8, 3 /
      data ncodmx2 / 8, 4, 6, 6, 7,  17,11, 7, 8,13, &
                    8, 8, 9, 4,10,  14,19, 8, 8, 8, &
                   13,13, 9, 8,12,  14,10,18, 6 /

!     code2   Table with keywords corresponding to options of COMPUTE_SCALAR
!             Values:
!             1:  NORM_DIFFERENCE
!             2:  NORM
!             3:  VECTOR (corresponds to VECTOR 1)
!             4:  VECTOR (corresponds to VECTOR 2)
!             5:  AVERAGE
!             6:  DEGREE_OF_FREEDOM
!             7:  DOT_PRODUCT
!             8:  MIN_MAX
!             9:  SCAL_MAX
!            10:  EXTRACT_VALUE
!            11:  COOR_MIN
!            12:  COOR_MAX
!            13:  ABS_VALUE
!            14:  NODE
!            15:  USER_POINT
!            16:  ELEMENT_GROUPS
!            17:  SKIP_ELEMENT_GROUPS
!            18:  ONE_NORM
!            19:  TWO_NORM
!            20:  INF_NORM
!            21:  MEAN_ONE_NORM
!            22:  MEAN_TWO_NORM
!            23:  MEAN_AREA
!            24:  MIN_AREA
!            25:  POINT_NUMBER
!            26:  ZERO_LEVEL_SET
!            27:  MEAN_VALUE
!            28:  STANDARD_DEVIATION
!            29:  VOLUME
! ======================================================================
!
      call eropen ( 'askcos' )

      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from askcos'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      itype = 0
      table = -1d0
      tabmin = 1d0
      tabmax = 0d0

      tabmax(1) = 5d0 ! NORM_DIFFERENCE

      tabmax(2) = 5d0 ! NORM

      itype(3) = 12  ! VECTOR
      tabmax(3) = 2d0
      tabmin(4) = -1003d0
      tabmax(4) = 1d3

      itype(5) = 2   ! AVERAGE

      tabmin(6) = -1d0 ! DEGREE_OF_FREEDOM
      tabmax(6) = 1d2
      table(6) = 0d0

      itype(7:8) = 2 ! INNER_PRODUCT, MIN_MAX

      itype(9) = 39  ! SCALAR
      tabmax(9) = 1d3 ! SCAL_MAX

      itype(10) = 2  ! EXTRACT_VALUE

      itype(11:12) = 39  ! SCALAR
      tabmax(11:12) = 1d3 ! COOR_MIN, COOR_MAX

      itype(13) = 2  ! ABS_VALUE

      tabmax(14) = npoint  ! NODE

      tabmax(15) = nuspnt ! USER_POINT

      itype(16:17) = 11  ! ELEMENT_GROUPS, SKIP_ELEMENT_GROUPS
      tabmin(16:17) = nelgrp
      tabmax(16:17) = nelgrp

      itype(18:22) = 2   ! NORMs
      itype(23:24) = 2   ! mean_area; min_area

      itype(25) = 2      ! POINT_NUMBER
      tabmax(26) = 1d0    ! maximum value levels
      itype(27:29) = 2   ! mean value, standard_deviation, volume

      inpsep(12) = 0             ! scal_max
      nmain = 0
      call sepreadsetkeyw
      if ( debug ) then
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nncode2, 'table' )
      end if  ! ( debug )

      if ( iretur==-10001 ) go to 1000! special case, it might be an expression

      inpsep(1:nparm+nelgrp) = 0

      norm1 = nint(table(1))
      norm2 = nint(table(2))
      iseq1 = nint(table(3))
      iseq2 = nint(table(4))
      ipoint = nint(table(14))
      if ( debug ) write(irefwr,1) 'iseq1, iseq2', iseq1, iseq2
      if ( table(15)>0 ) then

!     --- User point found

         if ( table(14)>0 ) then

!        --- error keyword NODE already used

             call errkeywexcl ( code2, ncodmx2, 14, 15 )

         else

!        --- User point

            ipoint = nint(table(15))
            inpsep(7) = 1

         end if

      end if
      found = .false.

      if ( norm1/=-1 ) then

!     --- NORM_DIF found

         inpsep(4) = 5
         found = .true.

      end if

      if ( norm2/=-1 ) then

!     --- NORM found

         inpsep(4) = 5
         if ( found ) call errsub ( 1889, 0, 0, 0 )
         found = .true.

      end if

      if ( nint(table(5))==1 ) then

!     --- AVERAGE found

         inpsep(4) = 1
         if ( found ) call errsub ( 1889, 0, 0, 0 )
         found = .true.

      end if

      if ( nint(table(7))==1 ) then

!     --- INNER_PRODUCT found

         inpsep(4) = 4
         if ( found ) call errsub ( 1889, 0, 0, 0 )
         if ( iseq2==-1 ) call errsub ( 1891, 0, 0, 0 )
         found = .true.

      end if

      if ( nint(table(8))==1 ) then

!     --- MIN_MAX found
!         Check if coor_min and/or coor_max are given
!         Check if abs_value is given
!         If one of these last possibilities is given only one degree
!         of freedom is allowed, hence table(6) gets the default 1

         iseq2 = 0
         inpsep(4) = 3
         if ( table(11)>0d0 .or. table(12)>0d0 ) inpsep(7) = 1
         if ( table(13)>0d0) inpsep(7) = 2
         if ( inpsep(7)>0 .and. table(6)<=0d0 ) table(6) = 1d0
         if ( table(9)>0d0 ) then

!        --- scal_max given, store scalar number in inpsep(12)

            inpsep(12) = mod(nint(table(9)),100000000)
            if ( inpsep(12)<=maxcls ) iscalars(inpsep(12)) = 1

         end if
         if ( table(11)>0d0 ) then

!        --- coor_min given, store scalar number in inpsep(13)

            inpsep(13) = mod(nint(table(11)),100000000)
            if ( inpsep(13)<=maxcls ) iscalars(inpsep(13)) = 1

         end if
         if ( table(12)>0d0 ) then

!        --- coor_max given, store scalar number in inpsep(10)

            iseq2 = mod(nint(table(12)),100000000)

         end if
         numscl = max(numscl,inpsep(12),inpsep(9)+ndim-1, &
                      iseq2+ndim-1)
         if ( found ) call errsub ( 1889, 0, 0, 0 )
         found = .true.

      end if

      if ( nint(table(10))==1 ) then

!     --- EXTRACT_VALUE found

         inpsep(4) = 2
         if ( ipoint==0 ) call errsub ( 2177, 0, 0, 0 )

         if ( found ) call errsub ( 1889, 0, 0, 0 )
         found = .true.

      end if

      if ( table(23)>0d0 ) then

!     --- mean_area found

         inpsep(4) = 109
         found = .true.

      end if  ! ( table(23)>0d0 ) then

      if ( table(24)>0d0 ) then

!     --- min_area found

         inpsep(4) = 110
         found = .true.

      end if  ! ( table(23)>0d0 ) then

!     --- Loop over norms

      do i = 1, 5

         if ( nint(table(17+i))==1 ) then

!        --- NORM found

            inpsep(4) = 5
            if ( iseq2>0 ) then
               norm1 = i
            else
               norm2 = i
            end if  ! ( iseq2>0 )
            if ( found ) call errsub ( 1889, 0, 0, 0 )
            found = .true.

         end if  ! ( nint(table(17+i))==1 )

      end do  ! i = 1, 5

      if ( table(25)>0d0 ) then

!     --- Point_number found

         inpsep(4) = 111  ! point number
         found = .true.

      end if  ! ( table(25)>0d0 )

      if ( table(27)>0d0 ) then

!     --- Mean_value found

         inpsep(4) = 112  ! Mean_value
         found = .true.

      end if  ! ( table(27)>0d0 )

      if ( table(28)>0d0 ) then

!     --- Standard_deviation found

         inpsep(4) = 113  ! Standard_deviation
         found = .true.

      end if  ! ( table(28)>0d0 )

      if ( table(29)>0d0 ) then

!     --- min_area found

         inpsep(4) = 117  ! volume
         found = .true.

      end if  ! ( table(29)>0d0 ) then

      if ( table(26)>0d0 ) then

!     --- Zero_level_set found, set number and type

         inpsep(7) = 20  ! ispec refers to zero level set
         ipoint = nint(table(26))  ! ipoint gives number of level set

      else if ( table(25)>0d0 ) then

!     --- Error no other options implemented

         call errsub ( 3086, 0, 0, 0 )

      end if  ! ( table(26)>0d0 )

      if ( .not. found ) call errsub ( 1888, 0, 0, 0 )
      if ( iseq1==-1 .and. inpsep(4)<109 ) call errsub ( 1890, 0, 0, 0 )
      if ( norm1>0 .and. iseq2==-1 ) call errsub ( 1891, 0, 0, 0 )
      inpsep(5) = nint(table(6))
      inpsep(6) = ipoint
      if ( norm1/=-1 ) inpsep(7) = 1
      inpsep(8) = max(norm1,norm2)
      inpsep(9) = iseq1
      inpsep(10) = iseq2
      inpsep(11) = number
      numscl = min ( maxcls, max(numscl,number) )
      if ( number<=maxcls ) iscalars(number) = 1
      if ( iseq1>0 .and. iseq1<=maxcls ) iscalars(iseq1) = 1
      if ( iseq2>0 .and. iseq2<=maxcls ) iscalars(iseq2) = 1

!     --- information about skip element groups

      if ( table(16)>0d0 .or. table(17)>0d0 ) then

!     --- Element groups must be skipped

         inpsep(14) = nparm+1
         ipos  = nint(table(16))
         if ( table(16)>0d0 ) then

!        --- element groups = given

            do k = 1, nelgrp
               inpsep(nparm+k) = 1
            end do
            do k = 1, nint(tabmax(16))
               inpsep(nparm+nint(tabout(k+ipos-1))) = 0
            end do

         else

!        --- skip element groups = given

            do k = 1, nelgrp
               inpsep(nparm+k) = 0
            end do
            ipos = nint(table(17))
            do k = 1, nint(tabmax(17))
               inpsep(nparm+nint(tabout(k+ipos-1))) = 1
            end do

         end if
         if ( table(16)>0d0 .and. table(17)>0d0 ) &
              call errsub ( 1900, 0, 0, 0 )

      end if

!     --- Fill last item of inpsep and raise istart

      istart = istart + 21
      if ( inpsep(14)>0 ) then
         istart = istart+nelgrp      ! iskip (nelgrp elements)
      end if   ! ( inpsep(14)>0 )

1000  if ( debug ) then

!     --- debug information

         if ( iretur/=-10001 ) then
            call prininlarge ( inpsep, nparm+nelgrp, 'inpsep' )
         else
            call prinin ( inpsep, nparm+nelgrp, 'inpsep' )
         end if  ! ( iretur/=-10001 )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End askcos'

      end if
      call erclos ( 'askcos' )

      end
