      subroutine askprinsurf ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-03-2014 Use local inpsep
!        version  1.1  date 29-04-2013 Change icode
!        version  1.0  date 15-01-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the keywords following print surface and store information
!     in inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     surface
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 1    106 to indicate print surface
!                              2    itypcur
!                                   1:  curves
!                                   2:  surfaces
!                              3    number of surfaces
!                              4 ...  surface numbers
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numsurfs, itypcur

!     itypcur        indicates type of item to be printed
!                    1:  curves
!                    2:  surfaces
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     PRININ         print 1d integer array
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askprinsurf' )
      debug = .false. .and. ioutp>=0
      numsurfs = 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprinsurf'
         write(irefwr,1) 'istart, lastitem', istart, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lastitem = lastitem-1

      nncode = 2
      code(1) = 'CURVES'
      code(2) = 'SURFACES'
      ncode(1) = 4
      ncode(2) = 7
      ncodmx(1) = 6
      ncodmx(2) = 8
      codesub(1) = 'C'
      codesub(2) = 'S'
      ncodesub(1:2) = 1

      itype(1:2) = 18
      table(1) = -1d0  ! reference to codesub
      table(2) = -2d0  ! reference to codesub
      tabmin(1:2) = nout
      tabmax(1) = ncurvs
      tabmax(2) = nsurfs
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

!     --- check if both curves and surfaces are used (not allowed)

      call errkeywexclus ( table, code, ncodmx, 1, 2 )
      if ( ierror/=0 ) go to 1000

      if ( table(1)>0d0 ) then
         itypcur = 1  ! curves
      else
         itypcur = 2  ! surfaces
      end if  ! ( table(1)>0d0 )

      numsurfs = nint(tabmax(itypcur))
      iposut = nint(table(itypcur))

      inpsep(0) = 82          ! reference to call in tosepcomsub
      inpsep(1) = itypcur
      inpsep(2) = numsurfs
      inpsep(3:2+numsurfs) =  nint(tabout(iposut:iposut-1+numsurfs))

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), 3+numsurfs, 'inpsep' )
         write(irefwr,*) 'End askprinsurf'

      end if  ! ( debug )
      call erclos ( 'askprinsurf' )
      istart = istart+3+numsurfs

      end subroutine askprinsurf
