      subroutine asksca12 ( inpsep, leninpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-12-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the part following scalar i = ... in case
!     a constant or an array has been found
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), leninpsep

!     inpsep         o    The integer input read is stored in array INPSEP
!                         Array INPSEP is filled as described in asksca
!     leninpsep      o    Length of this part of array inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifound, maxindex

!     ifound         Number of items found after the keywords MIN or MAX
!     maxindex       Upper bound of index for an array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1613   The index number read is smaller than 1 or larger than 1000
!    1614   Character could not be interpreted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'asksca12' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from asksca12'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      inpsep(2) = ityprd-7
      inpsep(3) = index_rdarr(1)
      if ( debug ) write(irefwr,100) 'inpsep(2,3)', inpsep(2), inpsep(3)
      ispecl = 5
      call asknxt
      if ( debug ) write(irefwr,100) 'ityprd', ityprd
      if ( ityprd==8 ) then

!     --- opening bracket found, read next item

         ispecl = 10
         call asknxt
         if ( debug ) write(irefwr,100) 'ityprd/2', ityprd
         if ( ityprd==2 .or. ityprd==12 ) then

!        --- integer found, check if this value is allowed

            if ( ityprd==2 ) then
               ifound = nint(realrd)
            else
               ifound = incons(index_rdarr(1))
            end if

            if ( inpsep(2)==5 ) then
               maxindex = maxints
            else if ( inpsep(2)==6 ) then
               maxindex = maxreals
            else if ( inpsep(2)==7 ) then
               maxindex = maxcls
            end if
            if ( ifound<1 .or. ifound+inpsep(3)>maxindex ) then

!           --- ifound out of range

               call errint ( ifound, 1 )
               call errint ( 1, 2 )
               call errint ( maxindex, 3 )
               call errsub ( 1613, 3, 0, 0 )
               call errcrd
               inpsep(4) = 0

            else

               inpsep(4) = ifound
               leninpsep = 4

            end if

         else if ( ityprd==14 ) then

!        --- index found of type variable

            inpsep(4) = -1
            inpsep(5) = index_rdarr(1)
            leninpsep = 5

         else

!        --- character could not be interpreted

            call errchr ( string(1:nchar), 1 )
            call errsub ( 1614, 0, 0, 1 )
            call errcrd

         end if

      else

!     --- no opening bracket found, set index

         inpsep(4) = 0
         leninpsep = 4

      end if
      ispecl = 0

1000  call erclos ( 'asksca12' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, leninpsep, 'inpsep' )
         write(irefwr,*) 'End asksca12'

      end if

      end
