      subroutine askvec02 ( number, inpsep, istrin, rinsep, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 12-02-2015 Extra parameter finished
!        version  3.0  date 17-01-2015 New parameter list
!        version  2.2  date 26-06-2014 Extension with keyword ivec
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read expression following vector =
!     Store information in common cinformation
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, inpsep(*), istrin
      double precision :: rinsep(*)
      logical, intent(out) :: finished

!     finished       o    Indicates if the reading has finished (true) or not
!                         (false)
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec
!                         inpsep(7) contains ivec
!     istrin        i/o   Starting address of next positions in rinsep
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     rinsep        i/o   Contains real input at output. The storage depends on
!                         the storage of inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lastitemsav, ityprdsav, iseqvec, lastitemsav1, numfun
      logical :: closefound, openfound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     iseqvec        Vector sequence number
!     ityprdsav      Help parameter to save the value of ityprd
!     lastitemsav    Help parameter to save the value of lastitem
!     lastitemsav1   Help parameter to save the value of lastitem
!     numfun         Sequence number of function
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ASKVECMAKEVEC  Check if the situation
!                    vector = (vector expression, vector expression) is found
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READEXPRESS    Interpret part of input as number or expression
!     READNAMEVEC    Read name of vector in common cpost5
!     READREALSCAL   Read integer, real or scalar
!     READSTRFUNC    Check if name read corresponds to one of the known
!                    functions
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvec02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvec02'
         write(irefwr,1) 'lastitem, number', lastitem, number
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      closefound = .false.
      openfound = .false.
      lastitemsav = lastitem
      finished = .false.

      if ( typeitems(lastitem+1)==13 ) lastitem = lastitem+1

      if ( debug ) write(irefwr,1) 'lastitem', lastitem, typeitems(lastitem+1)

      if ( typeitems(lastitem+1)==5 .and. lastitem<nitems+3 ) then

!     --- Starts with a (, check if a vector consisting of two components is
!         meant

         if ( debug ) write(irefwr,*) 'check (v,v)'
         if ( typeitems(lastitem+2)==3 ) then

!        --- Next parameter is a string, so that is possible
!            Check if it is a vector

            lastitem = lastitem+1
            call readnamevec

            call readexpress ( openfound, closefound, .true., .false., &
                               .true., vecfound )

            if ( debug ) then
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
               write(irefwr,2) 'realrd', realrd
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000

            if ( vecfound .and. typeitems(lastitem+1)==26 ) then

!           --- Vector followed by , found
!               Check if this is vector = ( vector expresson, vector expresson )

               call askvecmakevec ( inpsep, finished )
               if ( finished ) go to 1000
               ityprd = 20               ! nothing done
               lastitem = lastitemsav    ! reset lastitem
               go to 1000

            end if  ! ( typeitems(lastitem+1)==26 )

         end if  ! ( typeitems(lastitem+2)==3 )

      else if ( typeitems(lastitem+1)==3 .and. nitems>lastitem+1 ) then

!     --- String found, check if this string is followed by an open bracket

         if ( debug ) write(irefwr,*) 'check string ('
         if ( typeitems(lastitem+2)==5 ) then

!        --- String followed by open bracket found, check if string corresponds
!            to function

            call asknxt
            call readstrfunc ( numfun, string, nchar )

            if ( numfun==0 ) then
               lastitem = lastitemsav    ! reset lastitemsav
               go to 1000
            end if  ! ( numfun==0 )

         end if  ! ( typeitems(lastitem+2)==5 )

      end if  ! ( typeitems(lastitem+1)==5 )

      lastitem = lastitemsav    ! reset lastitem

      if ( debug ) then
         write(irefwr,*) 'before readexpress, openfound ', openfound
         write(irefwr,1) 'lastitem', lastitem
      end if  ! ( debug )

      call readexpress ( openfound, closefound, .true., .false., &
                         .true., vecfound )

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'vecfound', vecfound
      end if  ! ( debug )

      if ( vecfound .and. ityprd==19 .and. &
           typeitems(lastitemsav+1)==13 .and. &  ! =
           typeitems(lastitemsav+2)==5 .and. &   ! (
           typeitems(lastitem)==26 ) then        ! ,

!     --- vector = ( vector expresson, .. found
!         Check if this is the vector = ( vector expresson, vector expresson )

         if ( debug ) write(irefwr,*) '(v,v)'
         call askvecmakevec ( inpsep, finished )
         if ( finished ) go to 1000

      end if  ! ( vecfound .and. ityprd==19 .and. ..

      ityprdsav = ityprd

      inpsep(3) = abs(number)    ! sequence number of vector
      iseqvec = mod(inpsep(3),5000)  ! correction for multivectors
      if ( iseqvec<maxvec ) ivectors(iseqvec) = 1   ! mark vector as created

      if ( debug ) &
          write(irefwr,1) 'ityprd, iseqvec, lastitem, nitems', &
                           ityprd, iseqvec, lastitem, nitems

      if ( ityprd==3 ) then

!     --- Real found, check next item

         if ( nitems>lastitem ) then

!        --- There is something to read

            lastitemsav1 = lastitem
            call readrealscal

         end if  ! ( nitems>lastitem )

         if ( debug ) &
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

         if ( ityprd==102 ) then

!        --- vector found following the real, hence real * vector

            ityprd = 3
            lastitem = lastitemsav    ! reset lastitem
            go to 1000

         else

!        --- Not a vector found following the real, hence real found

            if ( ityprd==4 ) then

!           --- Text found, check if it is ndegfd, with or without number

               lastitem = lastitemsav1
               call asknxt
               if ( debug ) &
                  write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
               if ( string(1:3)=='DEG' ) then

!              --- DEGFD found, read number

                  call asknxt
                  if ( debug ) write(irefwr,1) 'ityprd', ityprd
                  if ( ityprd==2 ) inpsep(5) = nint(realrd)

               end if  ! ( string(1:4)=='DEGFD' )

            end if  ! ( ityprd==4 )
            if ( nitems>lastitem ) lastitem = lastitemsav1
            inpsep(4) = 116            ! vector is constant
            inpsep(7) = 0              ! ivec
            inpsep(13) = istrin
            rinsep(istrin) = realrd
            istrin = istrin+1


         end if  ! ( ityprd==102 )

      else if ( ityprd==19 ) then

!     --- Expression found

         inpsep(4) = 115            ! vector is expression
         inpsep(8) = nint(realrd)   ! sequence number of expression

      else if ( ityprd==20 ) then

!     --- No expression found, reset lastitem

         lastitem = lastitemsav
         go to 1000

      end if  ! ( ityprd==19 )

      if ( debug ) &
         write(irefwr,1) 'lastitem, nitems, ityprd', &
                          lastitem, nitems, ityprd

      if ( lastitem<nitems ) then

!     --- Not all information read, get rest of information

         ispecl = 4
         iretur = 0
         nmain = 0
         if ( debug ) write(irefwr,*) 'before sepreadsetkeyw'
         call sepreadsetkeyw
         ispecl = 0
         if ( ierror/=0 ) go to 1000

         if ( debug ) then

!        --- Debug information

            call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )
            write(irefwr,1) 'iretur', iretur
            call prinrl ( table, nncode, 'table' )
            call prinrl ( tabmax, nncode, 'tabmax' )
            call prinrl ( tabout, 4, 'tabout' )

         end if

         inpsep(2) = nint(table(18)) ! problem number
         inpsep(7) = nint(table(19)) ! type of vector

         if ( inpsep(4)==116 .and.table(45)>0d0 ) &
            inpsep(7) = nint(table(45))            ! ivec

         ityprd = ityprdsav

      end if  ! ( lastitem<nitems )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         call prinin ( inpsep, 10, 'inpsep' )
         write(irefwr,*) 'End askvec02'

      end if  ! ( debug )
      call erclos ( 'askvec02' )

      end
