      subroutine askvecderiv ( inpsep, istart, &
                               nprob, number, code2, ncodmx2, nelgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 13-01-2015 Extra parameter nelgrp
!        version  2.0  date 12-10-2012 Extra parameters code2, ncodmx2
!        version  1.2  date 10-08-2011 Extension with ix
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret input for derivatives following vi = derivatives
!     Warning:  If this subroutine is changed also change READSCDERIV !!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart, nprob, number, ncodmx2(*), nelgrp
      character (len=*) code2(*)

!     code2          i    Table with keywords
!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 10
!                         Array INPSEP is filled as follows:
!                         0: Type of function to be called
!                             8:  derivatives
!                         1-13+m: see readscderiv
!     istart        i/o   Next free position in inpsep
!     ncodmx2        i    Table containing the maximal number of  characters in
!                         the keyword table code2
!     nelgrp         i    Number of element groups
!     nprob          i    Number of various problems
!     number         i    Vector sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, iscal, m, maxtable, i, ipos, itypeboun, &
              istartnum

!     i              Counting variable
!     ipos           Position in array
!     iscal          Sequence number of scalar
!     istartnum      Starting address of a series of numbers in INPSEP
!     itypeboun      See inpsep(12)
!     ivec           Sequence number of vector
!     m              number of values stored in tabout for keyword
!     maxtable       Maximum value found in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READSCSKIPEL   Interpret information concerning skip element groups
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1506   Both VECTOR and SCALAR given in DERIVATIVES
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecderiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecderiv'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istartnum = 15             ! numbers are stored from this position

      inpsep(1:istartnum-1) = 0

      iposut = nint(table(3))
      m = nint(tabmax(3))
      ivec = -1
      if ( m>0 ) ivec = nint(tabout(iposut))   ! input vector

      iposut = nint(table(8))
      m = nint(tabmax(8))
      iscal = -1
      if ( m>0 ) iscal = nint(tabout(iposut))   ! scalar

      if ( ivec==-1 ) then

!     --- Vector not given

         if ( iscal==-1 ) then
            ivec = nprob+1
         else
            ivec = -2
         end if

      else

!     --- Vector has been given, so scalar is not allowed

         if ( iscal>0 ) then
            call errsub ( 1506, 0, 0, 0 )
            call errcrd
         else
            iscal = 0
         end if
      end if

      maxtable = nint(max(table(3), table(31)))

      inpsep(0) = 8   ! icode for derivatives
      inpsep(1) = nint(table(32))   ! seq_coef
      inpsep(2) = nint(table(18))   ! problem
      inpsep(3) = number            ! output vector
      inpsep(4) = nint(table(33))   ! seq_deriv
      if ( inpsep(4)<0 .and. maxtable>0 ) inpsep(4) = -10  ! Use derivatives
      inpsep(5) = iscal             ! scalar
      inpsep(6) = max(1,nint(table(31)))   ! icheld
      inpsep(7) = ivec              ! input vector
      inpsep(8) = 0                 ! type of output vector
      inpsep(9) = nint(table(36))   ! ix
      if ( number>0 .and. number<=maxvec ) ivectors(number) = 1
      inpsep(10) = nint(table(38))-1     ! type output vector

!     --- Check on mutually exclusive keywords

      do i = 39, 40  ! points, surfaces

         if ( table(27)>0d0 .and. table(i)>0d0 ) &
            call errkeywexcl ( code2, ncodmx2, 27, i ) ! curves
         if ( table(30)>0d0 .and. table(i)>0d0 ) &
            call errkeywexcl ( code2, ncodmx2, 30, i ) ! zero level set

      end do  ! i = 39, 40

      if ( table(30)>0d0 .and. table(27)>0d0 ) &
         call errkeywexcl ( code2, ncodmx2, 30, 27 )  ! curves, zero level set
      if ( table(39)>0d0 .and. table(40)>0d0 ) &
         call errkeywexcl ( code2, ncodmx2, 39, 40 )  ! points, surfaces

      itypeboun = 0
      m = 0

      if ( table(27)>0d0 ) then

!     --- curve found

         itypeboun = 2
         m = nint(tabmax(27))
         ipos = nint(table(27))
         inpsep(13) = istartnum
         inpsep(istartnum:istartnum+m-1) = nint(tabout(ipos:ipos+m-1))
         istartnum = istartnum+m

      else if ( table(30)>0d0 ) then

!     --- zero level set found

         itypeboun = 20
         m = 1
         inpsep(13) = istartnum
         inpsep(istartnum) = nint(table(30))
         istartnum = istartnum+1

      else

         do i = 39, 40  ! points, surfaces

            if ( table(i)>0d0 ) then

!           --- P or S found

               if ( i==39 ) then
                  itypeboun = 1
               else
                  itypeboun = 3
               end if  ! ( i==39 )
               m = nint(tabmax(i))
               ipos = nint(table(i))
               inpsep(13) = istartnum
               inpsep(istartnum:istartnum+m-1) = nint(tabout(ipos:ipos+m-1))
               istartnum = istartnum+m

            end if  ! ( table(i)>0d0 )

         end do  ! i = 12, 14

      end if  ! ( table(27)>0d0 )

      inpsep(11) = itypeboun
      inpsep(12) = m

!     --- information about skip elements

      call readscskipel ( table(22), tabout, inpsep, istartnum, &
                          14, tabmax(22), nelgrp )

      if ( debug ) write(irefwr,1) 'itypeboun', itypeboun

!     --- raise istart

      if ( inpsep(4)/=-10 ) then
         istart = istart + 6
      else
         istart = istart + istartnum  ! Change also readscderiv and askvecgrad
      end if  ! ( inpsep(4)/=-10 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep(0), 15, 'inpsep' )
         write(irefwr,*) 'End askvecderiv'

      end if  ! ( debug )
      call erclos ( 'askvecderiv' )

      end
