      subroutine askvecmakevec ( inpsep, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the situation vector = (vector expression, vector expression)
!     is found
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: inpsep(*)
      logical, intent(out) :: finished

!     finished       o    Indicates if the situation has been found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lastitemsav, ityrdsav
      double precision :: realrdsav
      logical ::  openfound, closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     ityrdsav       Help parameter to save the value of ityrd
!     lastitemsav    Help parameter to save the value of lastitem
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     realrdsav      Help parameter to save the value of realrd
!     vecfound       If true a vector has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READEXPRESS    Interpret part of input as number or expression
!     READNAME       Read name following %-sign
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecmakevec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecmakevec'
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lastitemsav = lastitem  ! save value of lastitem
      realrdsav = realrd      ! save value of realrd
      ityrdsav = ityprd       ! save value of ityprd

  !    lastitem = lastitem+1   ! next position following

      if ( debug ) write(irefwr,1) 'lastitem', lastitem

!     --- Check if next item is either a vector or a vector expression

      if ( typeitems(lastitem)==3 .and. typeitems(lastitem+2)==6 .and. &
           typeitems(lastitem+3)==26 .and. nitems==lastitem+3  ) then

!     --- next item could be a vector

         call readname

         if ( ityprd==2 ) then

!        --- Vector found

            finished = .true.
            inpsep(4) = 106             ! vector consisting of two vectors
            inpsep(9) = nint(realrdsav) ! first vector
            if ( inpsep(9)<=-1000 ) inpsep(9) = maxvc1-inpsep(9)-1000
            inpsep(10) = nint(realrd)   ! second vector
            if ( inpsep(10)<=-1000 ) inpsep(10) = maxvc1-inpsep(10)-1000
            lastitem = nitems

         end if  ! ( typeitems(lastitem+1)==26 )

      else

!     --- Next item might be a vector expression

         openfound = .false.
         closefound = .false.
         call readexpress ( openfound, closefound, .true., .false., &
                            .true., vecfound )

         if ( vecfound .and. (ityprd==102 .or. ityprd==19) ) then

            finished = .true.
            inpsep(4) = 106             ! vector consisting of two vectors
            inpsep(9) = nint(realrdsav) ! first vector
            inpsep(10) = nint(realrd)   ! second vector
            if ( inpsep(9)<=-1000 ) inpsep(9) = maxvc1-inpsep(9)-1000
            if ( inpsep(10)<=-1000 ) inpsep(10) = maxvc1-inpsep(10)-1000
            lastitem = nitems

         else

!        --- No vector found

            ityprd = ityrdsav
            lastitem = lastitemsav
            realrd = realrdsav

         end if  ! ( vecfound .and. (ityprd==2 .or. ityprd==19) )

      end if  ! ( typeitems(lastitem)==3 .and. typeitems(lastitem+1)==6 ... )

1000  call erclos ( 'askvecmakevec' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 10, 'inpsep' )
         write(irefwr,*) 'End askvecmakevec'

      end if  ! ( debug )

      end subroutine askvecmakevec
