      subroutine erbearing
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.1  date 26-08-2011 Simplification of subroutine
!        version  1.0  date 04-09-2006
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for bearings
!     The input is stored in the arrays iinput and rinput
!     Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     mass_conservation
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, iseqbearing, nrinp
      parameter ( niinp=10, nrinp=1 )
      integer iinput(niinp)
      double precision rinput(nrinp)

!                    they are not
!     iinput         integer input array
!     iseqbearing  Sequence number of navier stokes input block
!     niinp          Number of entries in iinput
!     nrinp          Number of entries in rinput
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     EROPEN         Produces concatenated name of local subroutine
!     READBEARING    Read the input for bearing
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READBEARING
!    The first record must have the following shape:
!    BEARING [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqbearing /0/
! ======================================================================
!
      call eropen ( 'erbearing' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erbearing'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinput and rinput

      iseqbearing = iseqbearing + 1

      call readbearing ( iinput, rinput, 2, iseqbearing, niinp, nrinp )

!     --- Make input arrays permanent

      call ercopyinput ( 30, iseqbearing, 'BEARING', iinput, &
                         rinput, iinput, niinp, nrinp, 0 )

1000  call erclos ( 'erbearing' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erbearing'

      end if  ! ( debug )

      end
