      subroutine ercomm ( last )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 26-08-2016 use sepmoduleinput
!        version  4.3  date 22-11-2012 Long integers
!        version  4.2  date 08-10-2007 Extension with decoupled unknowns
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the matrix structure from the standard input file
!    Store the result in module sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: last

!     last    i    If true this is the last call of the subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lniincommat=25
      integer ipos, i, nincom, maxcall
      integer, pointer :: iincom(:)
      double precision :: rinput
      logical :: filled

!     filled         Indicates whether an array has been filled (true) or not
!     i              General loop variable
!     iincom         Reference number with respect to array INFOR for
!                    array JMETOD
!     ipos           Position of input arrays in sepmoduleinput
!     lniincommat    Declared length of array iincommat
!     maxcall        Maximum value of icall in sepmoduleinp
!     nincom         Length of array IINCOM
!     rinput         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     EROPEN         Produces concatenated name of local subroutine
!     READ00         Reads input for subroutine COMMAT from standard input
!                    file.
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ03
!
!    The first record must have the following shape:
!
!    MATRIX, [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input array
!   if ( last ) then
!      if (sequence number 1 has not been filled) then
!         Fill sequence number 1 with defaults
!   else ( not last)
!      Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercomm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercomm'
         write(irefwr,*) 'last', last

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in array iincom

      nincom = nprob*lniincommat
      allocate ( iincom(nincom), stat = error )
      if ( error/=0 ) call eralloc ( error, nincom, 'iincom' )
      if ( ierror/=0 ) go to 1000

      if ( last ) then

!     --- last is true, check if first sequence number has been filled

         call sepgetinpfinf ( 3, 1, ipos, maxcall )
         if ( .not. in(ipos)%used ) then

!        --- IINCOMMT has not yet been filled, fill with defaults

            allocate ( in(ipos)%iinput(nincom), stat = error )
            if ( error/=0 ) call eralloc ( error, nincom, 'iincom' )
            if ( ierror/=0 ) go to 1000

            iincom => in(ipos)%iinput

            in(ipos)%used = .true.
            filled = .true.
            iseqcomm = 1
            iincom = 0
            do i = 1, nprob
               iincom((i-1)*lniincommat+2) = 2
               iincom((i-1)*lniincommat+5) = i
            end do

         else

!        --- sequence number 1 has already been filled

            filled = .false.

         end if

      else

         filled = .true.
         iseqcomm = iseqcomm + 1

         call read00 ( iincom, nprob, iseqcomm, 1, lniincommat )

         call ercopyinput ( 3, iseqcomm, 'MATRIX', iincom, &
                            rinput, iincom, nprob*lniincommat, 0, 0 )

      end if

      deallocate ( iincom )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercomm'

      end if  ! ( debug )
      call erclos ( 'ercomm' )

      end subroutine ercomm
