      subroutine ercomp ( manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 26-08-2016 use sepmoduleinput
!        version  4.2  date 26-08-2011 Simplification of subroutine
!        version  4.1  date 06-07-2011 Use allocate
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the main program SEPCOMP from the standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer manag(*)

!     manag          i    Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         See subroutine READSP for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, nrinp
      integer, allocatable :: iincom(:)
      double precision, allocatable :: rincom(:)

!                    they are not
!     iincom         Local integer input array
!     niinp          Length of integer input array
!     nrinp          Length of real input array
!     rincom         Local double precision input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READSC         Read the input for the structure of the main program
!                    sepcomp
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READCM
!    The first record must have the following shape:
!    STRUCTURE
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercomp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercomp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      niinp = 20000
      nrinp = 1000

      allocate ( iincom(niinp), rincom(nrinp), stat = error )
      if ( error/=0 ) call eralloc ( error, niinp+nrinp, 'iincom' )
      if ( ierror/=0 ) go to 1000
      iseqcomp = 1

      call readsc ( iincom, rincom, niinp, nrinp, nprob, manag )
      if ( ierror/=0 ) go to 1000

!     --- Store input arrays as permanent arrays

      call ercopyinput ( 10, iseqcomp, 'STRUCTURE', iincom, &
                         rincom, iincom, niinp, nrinp, 0 )

      deallocate ( iincom, rincom, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iincom' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercomp'

      end if  ! ( debug )
      call erclos ( 'ercomp' )

      end
