      subroutine ercompvector2 ( ncurvslc, icurvslc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-05-2010 Fortran 90
!        version  1.0  date 11-08-2004
!
!   copyright (c) 2004-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store inner product of normal and tensor in all nodes of a set of
!     curves into solution vector defined per node
! **********************************************************************
!
!                       KEYWORDS
!
!     stress
!     2d
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvslc, icurvslc(ncurvslc)

!     icurvslc       i    array containing the curve numbers to be treated
!     ncurvslc       i    Number of curves in icurvs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k1, k2, k, nodp, istar3, icurnr, jcurnr
      double precision nx, ny

!     i              Counting variable
!     icurnr         Curve sequence number (positive)
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     jcurnr         Curve sequence number with sign
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     nx             number of points in x direction
!     ny             number of points in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     255   nodal point number in curve has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all curves icurnr in icurvslc do
!        Find nodes and normals and values
!        store inner product
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercompvector2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercompvector2'
         write(irefwr,*) 'ncurvslc ', ncurvslc

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      istar3 = kmeshm(2)-1

!     --- For all curves icurnr in icurvslc do

      do i = 1, ncurvslc

         jcurnr = icurvslc(i)
         icurnr = abs(jcurnr)
         k1 = kmeshm(4+icurnr)
         k2 = kmeshm(5+icurnr)-1
         if ( debug ) write(irefwr,*) 'jcurnr, k1, k2 ', jcurnr, k1, k2

!        --- Find nodes and normals and values
!            store inner product

         do k = k1, k2

            nx = normal(1,k)
            ny = normal(2,k)
            if ( jcurnr<0 ) then

!           --- Curve number negative, use negative normal

               nx = -nx
               ny = -ny

            end if  ! ( icurnr<0 )
            nodp = kmeshm(istar3+k)
            if ( debug ) write(irefwr,*) 'nodp, nx, ny ', nodp, nx, ny
            if ( nodp<=0 ) then

!           --- Not permitted

               call errint ( icurnr, 1 )
               call errint ( nodp, 2 )
               call errint ( 1000000, 3 )
               call errwar ( 255, 3, 0, 0 )

            else

!           --- standard

              if ( debug ) call prinrl ( stress(1,nodp), 6, 'stress' )
              normalstress(1,nodp) = nx*stress(1,nodp)+ny*stress(4,nodp)
              normalstress(2,nodp) = nx*stress(4,nodp)+ny*stress(2,nodp)

            end if  ! ( nodp<=0 )

         end do  ! k = k1, k2

      end do  ! i = 1, ncurvslc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercompvector2'

      end if
      call erclos ( 'ercompvector2' )

      end
