      subroutine ercrea ( icode, last )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 26-08-2016 use sepmoduleinput
!        version  3.9  date 22-11-2012 Long integers
!        version  3.8  date 26-08-2011 Simplification of subroutine
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning CREATE VECTOR or boundary conditions from the
!    standard input file Store the result in IBUFFR and references in isepi2/3
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     essential
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode
      logical last

!     icode          i    Return code of main keyword in READAL
!                         Possible values:
!                         2   CREATE found
!                         3   ESSENTIAL found
!     last           i    If true this is the last call of the subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnr, isubr, lenrin, leninp, nstart, &
              iinput, ipos, maxcall
      integer, allocatable :: istart(:), inpcre(:), inforinpcre(:), &
                              iincrea1(:)
      double precision, allocatable :: rincre(:)
      character (len=29) namesubr

!                    they are not
!     iincrea1       Array iinput for subroutine ercrea1
!     iinput         reference number with respect to infor for array IINCOM
!     inforinpcre    Array of length of rincre that contains a 0 if the
!                    number stored in rincre is a real and a reference
!                    to scalars if it is a scalar
!     inpcre         The integer input read is stored in array INPCRE from
!                    position 1. Hence in the call INPCRE(ISTART), with
!                    ISTART the starting position must be used.
!                    INPCRE will be filled in correspondence with array
!                    INPCRE for subroutine CREATV
!     ipos           Position of input arrays in sepmoduleinput
!     iseqnr         Sequence number with respect to the storage in
!                    sepmoduleinput
!     istart         Array containing the starting addresses of the various
!                    vectors in INPCRE
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    3:  ERCREA in the case of create vector
!                    4:  ERCREA in the case of essential boundary conditions
!     leninp         Computed length of integer input array
!     lenrin         Computed length of array user
!     maxcall        Maximum number of coefficient inputs read
!     namesubr       Name of subroutine
!     nstart         Number of positions in array ISTART
!     rincre         Double precision array containing the values of the
!                    constant if KFUN<0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERCREA1        Fill array iinput with default values if no boundary
!                    conditions are given
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREADCR       Reads the standard input file for subroutine CREATE
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    This is done by subroutine READCR
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercrea' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercrea'
         write(irefwr,*) 'last ', last
         write(irefwr,1) 'icode', icode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( last ) then

!     --- last = true, fill inpcre with defaults

         leninp = nprob*5+2
         call sepgetinpfinf ( 3, 1, ipos, maxcall )
         if ( in(ipos)%used ) then

!        --- iinput>0, information of commat has been found

            call ercrea1 ( in(ipos)%iinput, nprob )

         else

!        --- iinput = 0, no information of commat
!            Create a default array iinput filled with 2

            allocate ( iincrea1(9*nprob), stat=error )
            if ( error/=0 ) call eralloc ( error, 9*nprob,  'iincrea1' )
            if ( ierror/=0 ) go to 1000

            iincrea1 = 2
            call ercrea1 ( iincrea1, nprob )

            deallocate ( iincrea1, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'iincrea1' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( iinput>0 )

      else

!     --- last = false, standard situation

         if ( icode<0 ) then
            isubr = -icode
            iseqnr = 1
         else if ( icode==2 ) then
            isubr = 3
         else
            isubr = 4
         end if

         if ( debug ) &
            write(irefwr,1) 'isubr, icode, iseqnr', isubr, icode, iseqnr

!        --- Read input and store in arrays INPCRE and RINCRE

         if ( isubr==3 ) then
            iseqcr = iseqcr+1
            iseqnr = iseqcr
         else if ( isubr==4 ) then
            iseqbc = iseqbc+1
            iseqnr = iseqbc
         end if

         if ( debug ) write(irefwr,1) 'isubr, iseqnr', isubr, iseqnr

         lenrin = 1000
         leninp = 10000
         nstart = 100

         allocate ( istart(nstart), inpcre(leninp), inforinpcre(lenrin), &
                    rincre(lenrin), stat=error )
         if ( error/=0 ) call eralloc ( error, nstart+leninp+2*lenrin, &
                  'istart' )
         if ( ierror/=0 ) go to 1000

!        --- Actually filling of arrays

         call erreadcr ( isubr, iseqnr, istart, inpcre, inforinpcre, &
                         rincre, leninp, lenrin, nstart )

         if ( isubr==3 ) then
            iseqcr = iseqnr
            namesubr = 'CREATE'
         else if ( isubr==4 ) then
            iseqbc = iseqnr
            namesubr = 'ESSENTIAL BOUNDARY CONDITIONS'
         end if

!        --- Fill arrays INPCRE and RINCRE
!            The length is given by leninp and lenrin

         if ( isubr<=2 ) then
            isubr = -isubr-3
         else
            isubr = isubr+1
         end if  ! ( isubr<=2 )
         call ercopyinput ( isubr, iseqnr, namesubr, inpcre, &
                            rincre, inforinpcre, leninp, lenrin, lenrin )
         deallocate ( istart, inpcre, inforinpcre, rincre, &
                      stat=error )
         if ( error/=0 ) call erdealloc ( error, 'istart' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqnr', iseqnr
         write(irefwr,*) 'End ercrea'

      end if  ! ( debug )
      call erclos ( 'ercrea' )

      end
