      subroutine ercrea1 ( iinput, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.0  date 26-08-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array inpcre with default values if no boundary conditions
!     are given
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), nprob

!     iinput         o    integer input array containg information for commat
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos, i, jmetod, i1, inpcre(nprob*5+2)
      double precision :: rinput(1)

!                    they are not
!     i              Counting variable
!     i1             Counter
!     inpcre         The integer input read is stored in array INPCRE from
!                    position 1. Hence in the call INPCRE(ISTART), with
!                    ISTART the starting position must be used.
!                    INPCRE will be filled in correspondence with array
!                    INPCRE for subroutine CREATV
!     ipos           Position of input arrays in sepmoduleinput
!     jmetod         Indicates type of storage scheme for the matrix
!                    See Programmers Guide for a description
!     rinput         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercrea1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercrea1'
         write(irefwr,1) 'nprob', nprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpcre(1) = -2
      inpcre(2) = nprob

      i1 = 2
      do i = 1, nprob

         inpcre(1+i1) = 0
         inpcre(2+i1) = i
         inpcre(3+i1) = 1
         jmetod = mod(iinput((i-1)*9+2),100)
         jmetod = mod(jmetod,30)
         if ( debug ) write(irefwr,1) 'i, jmetod', i, jmetod
         if ( jmetod<=2 .or. jmetod==5 .or. jmetod==6 .or. &
              jmetod==9 .or. jmetod==11 .or. jmetod==20 .or. &
              jmetod==21 ) then

!        --- Real case

            inpcre(4+i1) = 0

         else

!        --- Complex case

            inpcre(4+i1) = 1

         end if
         inpcre(5+i1) = -1
         i1 = i1+5

      end do

!     --- Copy input in sepmoduleinput

      call ercopyinput ( 5, 1, 'ercrea1', inpcre, rinput, inpcre, &
                         2+nprob*5, 0, 0 )

1000  call erclos ( 'ercrea1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpcre, 2+nprob*5, 'inpcre' )
         write(irefwr,*) 'End ercrea1'

      end if  ! ( debug )

      end
