      subroutine erderi
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.7  date 10-10-2012 Extension of nparm
!        version  2.6  date 26-08-2011 Simplification of subroutine
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the derivatives from the standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iproblc, nparm
      integer, allocatable :: iinder(:)
      double precision :: dummy
      parameter ( nparm=19)
      save iproblc

!     dummy          Dummy parameter
!     iinder         User input array for subroutine DERIV
!     iproblc        Actual problem number
!     nparm          Number of parameters for a specific element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READDR         Read the input for the subroutine DERIVS
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ03
!    The first record must have the following shape:
!    DERIVATIVES [,PROBLEM = j], [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iproblc /0/
! ======================================================================
!
      call eropen ( 'erderi' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinder

      allocate ( iinder(nelgrp+nparm+100), stat=error )
      if ( error/=0 ) call eralloc ( error, nelgrp+nparm+100, 'iinder' )
      if ( ierror/=0 ) go to 1000

      iseqderi = iseqderi + 1

      call readdr ( iinder, 2, iseqderi, iproblc, nprob )

!     --- Make input arrays permanent

      call ercopyinput ( 11, iseqderi, 'DERIV', iinder, &
                         dummy, iinder, nparm+nelgrp+100, 0, 0 )

      deallocate ( iinder, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'iinder' )

1000  call erclos ( 'erderi' )
      end
