      subroutine erfilcrv ( vector, nodes, n, kdeg, iardeg, ucopy )
! ======================================================================
!
!        programmer    Guus Segal
!        version 2.0   date   20-02-1998 Remove indprh and kprobh
!        version 1.0   date   01-11-1997
!
!
!
!   copyright (c) 1997-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array vector in the nodal points stored in array NODES, &
!     using the information stored in uoldcp as well as subroutine FUNCVECT
!     Real case
!     The vector is only filled for the degrees of freedom required
! **********************************************************************
!
!                       KEYWORDS
!
!   create
!   real
!   vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nodes(n), kdeg, iardeg(*)
      double precision vector(*), ucopy(n,*)

!    iardeg  i   If KDEG < 0, the positions 1 to -KDEG of IARDEG contain
!                the degrees of freedom to be filled
!    kdeg    i   Indication which degrees of freedom must be filled
!                Possibilities:
!                0:    All degrees of freedom
!                >0:   Only the KDEGth degree of freedom
!                <0:   The -KDEG degrees of freedom stored in array IARDEG
!                      must be filled
!                the unkowns per point
!    n       i   Number of nodes stored in array NODES
!    nodes   i   Contains the node numbers that must be filled
!    ucopy   i   Array containing the values that must be stored in vector
!    vector i/o  Array of length NUSOL to be (partly) filled by this subroutine
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, kdegun, kdegup, jdeg, k, inusol, inunkp

!    i         General loop variable
!    inunkp    Number of unknowns in a point
!    inusol    Positions of degree of freedom in array IVECTR
!    jdeg      Actual degree of freedom
!    k         General loop variable
!    kdegun    Under boundary for loop over degrees of freedom
!    kdegup    Upper boundary for loop over degrees of freedom
!    node      Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       ERROR MESSAGES
!
! 1131  Degree of freedom required is larger than number of unknowns per point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute kdegun and kdegup

      if ( kdeg.eq.0 ) then

!     --- kdeg  0, all degrees of freedom

         kdegun = 1
         kdegup = nphyso

      else if ( kdeg.gt.0 ) then

!     --- kdeg > 0, only degree of freedom kdeg

         kdegun = kdeg
         kdegup = kdeg
         if ( kdeg.gt.nphyso ) then
            call eropen ( 'erfilcrv' )
            call errint ( kdeg, 1 )
            call errint ( nphyso, 2 )
            call errsub ( 1131, 2, 0, 0 )
            call erclos ( 'erfilcrv' )
         end if

      else

!     --- kdeg < 0, only degrees of freedom stored in iardeg

         kdegun = 1
         kdegup = -kdeg

      end if

!     --- Loop over degrees of freedom

      do k = 1, kdegup-kdegun+1

!     --- Compute jdeg

         if ( kdeg.lt.0 ) then

!        --- kdeg < 0, find degree of freedom from IARDEG

            jdeg = iardeg(k+kdegun-1)

         else

!        --- kdeg >= 0, degree of freedom is equal to k

            jdeg = k+kdegun-1

         end if

!        --- Distinguish between the various cases

         if ( indprpo.gt.0 ) then

!        --- indprpo>0, degrees of freedom are stored in kprobpo
!             Loop over nodes

            do i = 1, n
               node   = nodes(i)
               inusol = kprobpo(node,jdeg)
               if ( inusol.gt.0 ) vector(inusol) = ucopy(i,k)
            end do

         else if ( indprfo.gt.0 ) then

!        --- indprfo>0, Non-constant number of degrees of freedom per point
!            Loop over nodes

            do i = 1, n
               node   = nodes(i)
               inusol = kprobfo(node)
               inunkp = kprobfo(node+1) - inusol
               if ( inunkp.ge.jdeg ) then

!              --- inunkp >= jdeg, fill degree of freedom

                  inusol = inusol + jdeg
                  vector(inusol) = ucopy(i,k)
               end if
            end do

         else if ( jdeg.le.nphyso ) then

!        --- Constant number of degrees of freedom per point
!            Loop over nodes

            do i = 1, n
               node   = nodes(i)
               inusol = (node-1) * nphyso + jdeg
               vector(inusol) = ucopy(i,k)
            end do

         end if

      end do

      end
