      subroutine erfillcf251
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for boundary elements of type 251
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iposinsav

!     i              Counting variable
!     iposinsav      Parameter to save old value of ipos
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF3  Check if keyword is found in list of constants or scalars
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The default input is:
!     coef 6 = x_load
!     coef 7 = y_load
!     coef 8 = z_load
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf251' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf251'
         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         call prinin ( iincof, iposin, 'iincof' )
         call prinin ( imark, 9, 'imark' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, ndim

!     --- Coefficient 6 (x_load)

         icoef = 5+i
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codestressbn(i)(1:ncodestressbn(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 1, ndim

      icoef = 9   ! THICKNESS
      i = 4
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         iposinsav = iposin
         call erfilldefcof1 ( codestressbn(i)(1:ncodestressbn(i)) )

         if ( iposinsav==iposin .and. iposin_h>0 ) then

!        --- Thickness not found, but read by 250

            iincof(iposin+1) = iincof(iposin_h+1)
            iincof(iposin+2) = icoef
            iincof(iposin+3) = iincof(iposin_h+3)
            iposin = iposin + 3

        end if  ! ( iposinsav==iposin .and. iposin_h>0 )

      end if  ! ( imark(icoef)==0 )

      icoef = 2  ! STRESS_STRAIN_RELATION
      i = 5

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof4 ( codestressbn(i)(1:ncodestressbn(i)) )

      end if  ! ( imark(icoef)==0 )

      icoef = 1  ! ILOAD
      i = 6

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof3 ( codestressbn(i)(1:ncodestressbn(i)) )

      end if  ! ( imark(icoef)==0 )

      icoef = 3  ! INTEGRATION_RULE
      i = 7

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof3 ( codestressbn(i)(1:ncodestressbn(i)) )

      end if  ! ( imark(icoef)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End erfillcf251'

      end if  ! ( debug )
      call erclos ( 'erfillcf251' )

      end subroutine erfillcf251
