      subroutine erfillcf325
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-01-2015  Correction for ndim=1
!        version  1.0  date 15-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 325
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulestrings
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF2  Check if keyword is found in list of vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF3  Check if keyword is found in list of constants or scalars
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf325' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf325'
         write(irefwr,1) 'ndim', ndim
         call prinin ( imark, 20, 'imark' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do icoef = 1, 2

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof4 ( codereynolds(icoef)(1:ncodereynolds(icoef)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 1, 2

      do icoef = 3, 4

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof3 ( codereynolds(icoef)(1:ncodereynolds(icoef)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 3, 4

      do i = 5, 9

         icoef = i+1

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codereynolds(i)(1:ncodereynolds(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 5, 9

      icoef = 11  ! VELOCITY
      i = 10

      if ( ndim>1 .and. imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof2 ( codereynolds(i)(1:ncodereynolds(i)) )

      else if ( ndim==1 .and. imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codereynolds(i)(1:ncodereynolds(i)) )

      end if  ! ( imark(icoef)==0 )

      do i = 11, 10+ndim

         icoef = i  ! U_VELOCITY, V_VELOCITY

         if ( imark(icoef)==0 .and. imark(11)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codereynolds(i)(1:ncodereynolds(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 11, 12

      do i = 13, 14

         icoef = i+1  ! DENSITY, RT

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codereynolds(i)(1:ncodereynolds(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 13, 14

      icoef = 16  ! FORCE
      i = 15

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof2 ( codereynolds(i)(1:ncodereynolds(i)) )

      end if  ! ( imark(icoef)==0 )

      do i = 16, 17

         icoef = i  !  X_FORCE, Y_FORCE

         if ( imark(icoef)==0 .and. imark(15)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codereynolds(i)(1:ncodereynolds(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 16, 17

      icoef = 19  ! UPPER_FACE_VELOCITY
      i = 18

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof2 ( codereynolds(i)(1:ncodereynolds(i)) )

      end if  ! ( imark(icoef)==0 )

      do i = 19, 20

         icoef = i  !  X_UPPER_FACE_VELOCITY, Y_UPPER_FACE_VELOCITY

         if ( imark(icoef)==0 .and. imark(15)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codereynolds(i)(1:ncodereynolds(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 19, 20

      iincof(iposin+1) = -2        ! end of information

1000  call erclos ( 'erfillcf325' )
      if ( debug ) then

!     --- Debug information

         call prinin ( imark, 20, 'imark' )
         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )

         write(irefwr,*) 'End erfillcf325'

      end if  ! ( debug )

      end subroutine erfillcf325
