      subroutine erfillcf702
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 702 (Diffusion)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFDIF    Fill array iincof/rincof for diffusion part of elements
!                    of type 800
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLCFSRHOCP Fill array iincof/rincof for source and rho cp part
!                    of elements
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     The default input is:
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     coeff6-11: diffusion input
!     coef12 = source
!     coef13 = density
!     coef14 = heat_capacity
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf702' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf702'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Coefficient 6-11 (Diffusion)

      if ( imark(6)==0 ) then

!     --- Diffusion has not been filled yet

         call erfillcfdif

      end if  ! ( iposin==iposinsav )

!     --- Coefficient 12-14 (source, density and heat_capacity)

      call erfillcfsrhocp ( 12 )

!     --- integration rule and coordinate system

      call erfillcfrule

      iincof(iposin+1) = -2        ! end of information

1000  call erclos ( 'erfillcf702' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         write(irefwr,*) 'End erfillcf702'

      end if  ! ( debug )

      end subroutine erfillcf702
