      subroutine erfillcf803
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 803
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulestrings
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf803' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf803'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- integration rule and coordinate system

      call erfillcfrule

      icoef = 5  ! LINEARIZATION

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

        call erfilldefcof4 ( codegen(10) )

      end if  ! ( imark(icoef)==0 )

!     --- Coefficient 15 (beta)

      icoef = 15
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codegen(7) )  ! zero_order_coef

      end if  ! ( imark(icoef)==0 )

!     --- Coefficient 16 (source)

      icoef = 16
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codegen(1) )

      end if  ! ( imark(icoef)==0 )

      iincof(iposin+1) = -2        ! end of information

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End erfillcf803'

      end if  ! ( debug )
      call erclos ( 'erfillcf803' )

      end subroutine erfillcf803
