      subroutine erintb
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.5  date 26-08-2011 Simplification of subroutine
!        version  2.4  date 11-01-2002 New call to readbi
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the BOUNDARY_INTEGRAL from the standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm
      parameter ( nparm=6 )
      integer iinint(nparm+100)
      double precision :: dummy

!     dummy          Dummy parameter
!     iinint         Integer input array
!     nparm          Number of entries in iinint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     EROPEN         Produces concatenated name of local subroutine
!     READBI         Read the input for the subroutine INTBND
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READBI
!    The first record must have the following shape:
!    BOUNDARY_INTEGRAL [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erintb' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinint

      iseqintb = iseqintb + 1

      call readbi ( iinint, 2, iseqintb, iinint(1+nparm) )

!     --- Make input arrays permanent

      if ( iinint(4)>2 ) then

!     --- Contact found, curves are not used

         length = 0

      else

!     --- Curves or surfaces found

          length = iinint(6)

      end if  ! ( iinint(4)>2 )

      call ercopyinput ( 14, iseqintb, 'BOUNDARY_INTEGRAL', iinint, &
                         dummy, iinint, nparm+length, 0, 0 )

1000  call erclos ( 'erintb' )
      end
