      subroutine ernavstokes
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.1  date 26-08-2011 Simplification of subroutine
!        version  1.0  date 30-11-2005
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for Navier-Stokes
!     The input is stored in the arrays iinput and rinput
!     Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     navier_stokes_equation
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, iseqnavstokes, nrinp, niinploc, nrinploc
      parameter ( niinp=10, nrinp=1 )
      integer iinput(niinp)
      double precision rinput(nrinp)

!                    they are not
!     iinput         integer input array
!     iseqnavstokes  Sequence number of navier stokes input block
!     niinp          Length of integer input array
!     niinploc       Contains local value of niinp (may be changed)
!     nrinp          Length of real input array
!     nrinploc       Contains local value of nrinp (may be changed)
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     EROPEN         Produces concatenated name of local subroutine
!     READNAVSTOKES  Read the input for navier-stokes
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READNAVSTOKES
!    The first record must have the following shape:
!    NAVIER_STOKES [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqnavstokes /0/
! ======================================================================
!
      call eropen ( 'ernavstokes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ernavstokes'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinput and rinput

      iseqnavstokes = iseqnavstokes + 1

      niinploc = niinp
      nrinploc = nrinp
      call readnavstokes ( iinput, rinput, 2, &
                           iseqnavstokes, niinploc, nrinploc )

!     --- Make input arrays permanent

      call ercopyinput ( 30, iseqnavstokes, 'navier-stokes', iinput, &
                         rinput, iinput, niinploc, nrinploc, 0 )

1000  call erclos ( 'ernavstokes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ernavstokes'

      end if

      end
