      subroutine erouts ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 06-02-2012 Allow indexed vectors
!        version  2.7  date 30-08-2006 New definition of cpost5
!        version  2.6  date 17-11-2001 Default value of iseqnr
!
!   copyright (c) 1993-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine OUTSOL for writing to sepcomp.out
!    Fill array IINOUT with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      use sepmodulemnsub
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep  i    The integer input read is stored in array INPSEP
!                  1  ISEQNR     Sequence number corresponding to input
!                                If iseqnr = -1, the next one will be used
!                  2  IPROB      Problem sequence number
!                                Not used
!                  3  IVECTOR    Vector sequence number (first vector)
!                                If ivector = -1, the same value as iprob
!                                will be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ivec, iseqnr, ivectr, iter, isubr, &
              initer(10), i
      double precision tprev
      logical filusr

!     filusr    Indication if arrays IUSER and USER have been filled
!               explictly in the call (true) or that they must be found from
!               IBUFFR (false)
!     i         General loop variable
!     initer    Iteration number as function of sequence number
!     iseq      Sequence number read
!     iseqnr    Sequence number with respect to the storage in array ISEPIN
!     isubr     Indicates the type of calling subroutine
!               Possible values
!               1:  OUTPUT
!               2:  OUTTIM
!               3:  OUTSOLiuser(1),
!               4:  OUTPOS
!     iter      Iteration number
!     ivec      Sequence number of vector of read
!     ivectr    Sequence number of first vector
!     tprev     Previous time used in this subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PROUTP         Performs the actual write actions for OUTPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivectr /0/, initer/10*0/, tprev/-1d20/
! ======================================================================
!
      call eropen ( 'erouts' )

      if ( tprev<t-epsmac .or. tprev>t+sqreps*tprev ) then

!     --- New time step set initer

         do i = 1, 10
            initer(i) = 0
         end do
         tprev = t

      end if
      iseqnr = 1
      ivectr = 1
      iseq   = inpsep(1)
      ivec   = inivectr(inpsep(3))
      if ( iseq/=-1 ) iseqnr = iseq
      if ( ivec/=-1 ) ivectr = ivec
      initer(iseqnr) = initer(iseqnr) + 1
      iter = initer(iseqnr)
      isubr = 3
      filusr = .false.
      call proutp ( isubr, t, 1, iseqnr, iter, filusr, ivectr )
      numvecmn = max(numvecmn,ivectr)

      call erclos ( 'erouts' )

      end subroutine erouts
