      subroutine erscal ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.5  date 05-06-2014 Extension for print coor
!        version  5.4  date 04-05-2014 Extra parameter istart
!        version  5.3  date 16-04-2014 New call to printvectors
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write scalar variable or vector for subroutine TO0094
!     Or alternatively print text
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduletexts
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         0  ICODE      If 7 then print scalar else print vector
!                         1-19:  See subroutine askpri (1:19)
!                         20 nread   (number of curves, surfaces etc)
!                            If nread>0 followed by nread curves etc.
!                         If iregion#0, 6 positions in rinsep are used
!                         If iequidistant#0, 9 positions in rinsep are used
!                         If icode=7:
!                         Pos. 1 contains the number of scalars
!                         Position 10 to 4+3*numscalars contain 3 positions per
!                         scalar:
!                          1: type (1=integer, 2=real, 3=scalar)
!                          2: position in incons rlcons or scalars
!                          3: if>0, index of scalar used as array index
!                         If iregion#0, this implies
!                         xmin, xmax, ymin, ymax, zmin, zmax
!                         from position rinsep(istrin)
!     rinsep         i    Array containing the real input for this subroutine
!                         If iregion#0, this implies
!                         xmin, xmax, ymin, ymax, zmin, zmax
!                         from position rinsep(istrin)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnr, itext, lentxt, icode, nscals, irefwrsav
      character (len=120) textloc

!     icode          Return code with respect to subroutine INTX01
!                    Indicates position in array code or code1 (INPSEP(1))
!     irefwrsav      Help variable to store the value of irefwr
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     itext          Text sequence number
!     lentxt         length of text parameter, with trailing blanks deleted
!     nscals         Number of scalars
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSCALTXT      Fill string textloc with name of scalar depending on iseqnr
!     GETSCALINDEX   Return with adapted index of scalar
!     INICLOSEFILE   Close file and reset irefwr
!     INIOPENFILE    Open serial file and set irefwr
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINTSCALARS   Write scalar variable for subroutine TO0094
!     PRINTVECTORS   Write vector for subroutine TO0094
!     SEPPRINKMESH   Print array KMESH
!     SEPPRINKPROB   Print array KPROB
!     SEPTEXTVEC     Compute textloc and lentxt as function of iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erscal' )
      debug = .false.

      icode = inpsep(0)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erscal'
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'icode', inpsep(0)
         if ( icode==15 ) then
            call prinin ( inpsep(1), 5, 'inpsep' )
         else
            call prininlarge ( inpsep(1), 22, 'inpsep' )
         end if  ! ( icode==15 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Skip printing if itimloop>0 and ioutact#1
!         except for icode=7

      if ( itimloop/=0 .and. ioutact/=1 .and. icode/=7 )  go to 1000

!     --- Open file if necessary

      if ( icode<=15 .and. icode/=9 .and. inpsep(4)>0 ) &
         call iniopenfile ( inpsep(4), irefwrsav, inpsep(5), &
                            textloc )

      if ( icode<15 ) then

!     --- print text or scalar

         iseqnr = inpsep(1)
         itext  = inpsep(2)
         lentxt = inpsep(3)

      else if ( icode==54 ) then

         if ( itimloop>=0 ) then

!        --- time loop active, so print time

            if ( tend>1d-4 ) then

!           --- tend>1d-4, fixed format

               write(irefwr,200) t
200            format(' Time = ', f12.6 )
            else

!           --- tend<1d-4, exponential format

               write(irefwr,210) t
210            format(' Time = ', 1e12.4 )

            end if  ! ( tend>1d-4 )

         end if
         goto 1000

      else

         itext  = inpsep(1)
         lentxt = inpsep(2)

      end if

      if ( debug ) &
         write(irefwr,1) 'icode, iseqnr, itext', icode, iseqnr, itext

      if ( itext>0 ) then

         textloc = texts(itext)

      else if ( icode<15 ) then

         if ( icode==7 ) then

!        --- scalars

            if ( iseqnr==1 ) then

               iseqnr = getscalindex ( inpsep(11) )
               if ( debug ) write(irefwr,1) 'iseqnr', iseqnr
               if ( iseqnr>0 ) then

!              --- Standard case, Use name of variable

                  call erscaltxt ( iseqnr, textloc, lentxt )

               else

!              --- Complete array, use generic name

                  iseqnr = -iseqnr  ! make positive
                  textloc = chscalars(iseqnr)
                  lentxt = index(textloc,'(')-1

               end if  ! ( iseqnr<5000 )
               if ( ierror/=0 ) go to 1000

            else

               lentxt = 0

            end if  ! ( iseqnr==1 )

         else if ( iseqnr>0 .and. iseqnr<5000 ) then

!        --- Vector names

            call septextvec ( iseqnr, textloc, lentxt )

         end if  ! ( icode==7 )

      end if

      if ( icode==7 ) then

!     --- icode = 7, print scalars

         if ( itext==-1001 ) then

!        --- print kprob

            call sepprinkprob ( inpsep(3) )

         else if ( itext==-1002 ) then

!        --- print kmesh

            call sepprinkmesh ( inpsep(3) )

         else

!        --- print scalars

            call printscalars ( inpsep(1), lentxt, nscals, textloc )

         end if  ! ( itext==-1001 )

      else if ( icode==9 ) then

!     --- icode = 9, print vectors

         call printvectors ( inpsep(1), rinsep, lentxt, textloc )

      else if ( itext>0 ) then

         write(irefwr,300) textloc(1:lentxt)
300      format(1x,a)

      end if

!     --- Close file if opened

      if ( icode<=15 .and. icode/=9 .and. inpsep(4)>0 ) &
         call iniclosefile ( irefwrsav )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erscal'

      end if
      call erclos ( 'erscal' )

      if ( icode==15 ) then
         istartmn = istartmn + 6
      else if ( icode==7 ) then
         istartmn = istartmn + 10 + inpsep(1)*3
      else if ( icode==54 ) then
         istartmn = istartmn + 1
      else
         istartmn = istartmn + 23 + inpsep(20)
      end if

      end
