      subroutine getconstar ( realname, realar, ncomponents )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-09-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the values of a set of reals in the block constants with name realname
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) realname
      integer ncomponents
      double precision realar(ncomponents)

!     ncomponents    i    Number of components
!     realar         o    Contains the values read
!     realname       i    Name of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, i

!     i              Counting variable
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnamereal, getnameint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2156   real name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chrlcons
!     If ( name in chrlcons ) then
!        getconstar := value
!     else
!        Find position of name in chincons
!        If ( name in chincons ) then
!           getconstar := value
!        else
!           error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iseq = getnamereal ( realname )
      if ( iseq>0 ) then
         do i = 1, ncomponents
            realar(i) = rlcons(iseq+i-1)
         end do  ! i = 1, ncomponents
      else
         iseq = getnameint ( realname )
         if ( iseq>0 ) then
            do i = 1, ncomponents
               realar(i) = incons(iseq+i-1)
            end do  ! i = 1, ncomponents
         else
            do i = 1, ncomponents
               realar(i) = 0d0
            end do  ! i = 1, ncomponents
            call eropen ( 'getconstar' )
            call errchr ( realname, 1 )
            call errwar ( 2156, 0, 0, 1 )
            call erclos ( 'getconstar' )
         end if
      end if
      end
