      function getstring ( stringname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-02-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the value of the string in the block constants with name stringname
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) getstring, stringname

!     getstring      o    Value of string
!     stringname     i    Name of string
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer sepgetstring

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     SEPGETSTRING   Get the sequence number corresponding to the string in
!                    the block constants with name stringname
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2156   real name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iseq = sepgetstring ( stringname, chstrings, numstrings )
      if ( iseq>0 ) then
         getstring = texts(instring(iseq))
      else
         getstring = ''
         call eropen ( 'getstring' )
         call errchr ( stringname, 1 )
         call errwar ( 2156, 0, 0, 1 )
         call erclos ( 'getstring' )
      end if

      end
