      subroutine incv03 ( ibinas )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.3    date 21-01-1999 Include commons
!        version 1.2    date 16-11-1994 Adapt format
!        version 1.1    date 16-11-1993 Adapt format
!        version 1.0    date 23-01-1993
!
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    program incv03 is an interactive program to make a conversion
!    of the SEPRAN sepplot.*** files from ASCII to binary or vice-versa
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     transformation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduleread
      use sepmodulemach
      implicit none
      integer ibinas

!     ibinas  i    Indicates if a file must be translated from binary to ASCII
!                  or vice-versa
!                  Possible values:
!                  0:  Translate from binary to ASCII
!                  1:  Translate from ASCII to binary
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefsv, lastps, lastnw, maxpol, maxchr, ios, ichois, j, &
              ntext, npolyg, iref, icount
      parameter ( maxpol = 100, maxchr = 100 )
      real x, y, angle, height, polyg(2*maxpol)
      character text*(maxchr)
      character * 50 namesv, namefl, namenw
      logical ready, endlp, check

!     angle   angle of rotation
!     check   Indicates if the file exist (true) or not
!     endlp   Indicates if end of loop has been reached (true) or not (false)
!     height  Height of characters
!     ichois  Choice parameter
!     icount  Counting variable
!     ios     IOSTAT variable, returns from reading or writing
!     iref    File reference number
!     irefsv  Help variable to save a reference number
!     j       Counting variable
!     lastnw  Last non-blank position in namenw
!     lastps  Last non-blank position in namefl
!     maxchr  Maximum number of characters in text
!     maxpol  Maximum number of points in array polyg
!     namefl  Name of default SEPRAN neutral plotting file
!     namenw  Name of new SEPRAN neutral plotting file
!     namesv  Save of previous name
!     npolyg  Number of points in a polygon
!     ntext   Number of characters in a text
!     polyg   Array containing the co-ordinates of the point in the polygon
!             in the following way:
!             polyg(1,i) contains the x-coordinate of the first point
!             polyg(2,i) contains the y-coordinate of the first point
!     ready   Indicates if the loop has been finished (true) or not (false)
!     text    String variable containing some text
!     x       x co-ordinate of point
!     y       y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item from the standard input file
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     ERTEXT  Produce menu
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     184   Error in opening of plot file
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incv03' )

!     --- Save old name of sepplot.*** and ask for actual name

      namesv = namepl
      irefsv = irefpl
      call errchr ( namepl, 1 )
      call ertext ( 40+ibinas, 0, 0, 1 )

!     --- Read answer

      ityprd = 1
      ispecl = 3
      call asknxt
      if ( ityprd.eq.5 ) namepl = string(1:nchar)
      ispecl = 0

!     --- Open old file

      if ( ibinas.eq.0 ) then

!     --- ibinas = 0, binary file

         irefpl = -abs(irefpl)

      else

!     --- ibinas = 1, ASCII file

         irefpl = abs(irefpl)

      end if

!     --- Check if file namepl.001 exists

      ierror = -1
      iref = abs(irefpl)
      lastps = index ( namepl, ' ' ) + 3
      namefl(1:lastps) = namepl(1:lastps-4)//'.001'
      if ( ibinas.eq.0 ) then
         open ( unit=iref, status='old', iostat=ios, form='unformatted', &
                file=namefl(1:lastps) )
      else
         open ( unit=iref, status='old', iostat=ios, form='formatted', &
                file=namefl(1:lastps) )
      end if
      if ( ios.ne.0 ) then
         call errint ( ios, 1 )
         call errchr ( namefl(1:lastps), 1 )
         call errsub ( 184, 1, 0, 1 )
      end if
      if ( ierror.gt.0 ) then
         irefpl = irefsv
         namepl = namesv
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000
      end if

!     --- ask for name of new file

100   namenw = 'sepplnew'
      call errchr ( namenw, 1 )
      call ertext ( 41-ibinas, 0, 0, 1 )

!     --- Read answer

      ityprd = 1
      ispecl = 3
      call asknxt
      if ( ityprd.eq.5 ) namenw = string(1:nchar)
      lastnw = index ( namenw, ' ' ) + 3
      ispecl = 0
      if ( namenw.eq.namepl ) then

!     --- New and old name are equal

         write(*,*) 'New and old plot files have the same name'
         write(*,*) 'These names must be different'
         go to 100

      end if

!     --- Loop over the plot files

      ready = .false.
      icount = 1

200   if ( .not. ready ) then

!     --- Open new plot file

         iref = abs(irefpl)+1
         write ( namenw(lastnw-3:lastnw),210 ) icount
210      format ( '.',i3.3 )
         if ( ibinas.eq.0 ) then
            open ( unit=iref, iostat=ios, form='formatted', &
                   file=namenw(1:lastnw) )
         else
            open ( unit=iref, iostat=ios, form='unformatted', &
                   file=namenw(1:lastnw) )
         end if
         if ( ios.ne.0 ) then
            call errint ( ios, 1 )
            call errchr ( namenw(1:lastnw), 1 )
            call errsub ( 184, 1, 0, 1 )
         end if

!        --- Read and write plot file until end of file is reached

         endlp = .false.
         iref = abs(irefpl)
300      if ( .not. endlp ) then

!        --- Read next record

            if ( ibinas.eq.0 ) then
               read (iref,iostat=ios) ichois,x,y,angle,height,ntext
            else
               read (iref,*,iostat=ios) ichois,x,y,angle,height,ntext
            end if
            if ( ios.ne.0 ) then
               endlp = .true.
            else
               if ( ibinas.eq.0 ) then
                  write (iref+1,310,iostat=ios) ichois,x,y,angle,height, &
                     ntext
310               format (i1,4(1x,f8.3),1x,i3)
               else
                  write (iref+1,iostat=ios) ichois,x,y,angle,height, &
                     ntext
               end if
            end if
            if ( ichois.eq.6 ) then

!           --- ichois = 6, plot text

               if ( ntext.gt.100 ) ntext = min(mod(ntext,100), maxchr)
               if ( ibinas.eq.0 ) then
                  read (iref,iostat=ios) text(1:ntext)
                  write (iref+1,320,iostat=ios) text(1:ntext)
               else
                  read (iref,320,iostat=ios) text(1:ntext)
                  write (iref+1,iostat=ios) text(1:ntext)
320               format(a)
               end if

            else if ( ichois.eq.7 ) then

!           --- ichois = 7, fill polygon with colour

               ntext = min(ntext,maxpol)
               npolyg = 2*ntext
               if ( ibinas.eq.0 ) then
                  read (iref,iostat=ios) (polyg(j),j=1,npolyg)
                  write (iref+1,330,iostat=ios) (polyg(j),j=1,npolyg)
330               format (10f8.3)
               else
                  read (iref,*,iostat=ios) (polyg(j),j=1,npolyg)
                  write (iref+1,iostat=ios) (polyg(j),j=1,npolyg)
               end if
            end if
            go to 300

         end if

!        --- Check if next file exists

         icount = icount + 1
         write ( namefl(lastps-3:lastps),210 ) icount
         inquire ( file=namefl(1:lastps), exist=check )
         if ( check ) then

!        --- file exists, open

            if ( ibinas.eq.0 ) then

               open ( unit=iref, status='old', iostat=ios, &
                      form='unformatted', file=namefl(1:lastps) )
            else
               open ( unit=iref, status='old', iostat=ios, &
                      form='formatted', file=namefl(1:lastps) )
            end if
            if ( ios.ne.0 ) then
               call errint ( ios, 1 )
               call errchr ( namefl(1:lastps), 1 )
               call errsub ( 184, 1, 0, 1 )
               irefpl = irefsv
               namepl = namesv
               call ertext ( 1, 0, 0, 0 )
               read(*,*)
               go to 1000
            end if

         else

!        --- File does not exist, ready

            ready = .true.

         end if
         go to 200

      end if

1000  call erclos ( 'incv03' )
      end
