      subroutine ini016 ( nooutput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.9  date 18-11-2013 extension of inpsol
!        version  3.8  date 25-11-2012 Long integers
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill default structure in ISEPIN in case the structure of the program
!   has not explicitly been given by the user
!   At this moment the default is linear problem except when non_linear
!   equations has been read, in which case non-linear problem is the default
!   or time-dependent when time integration has been found
! **********************************************************************
!
!                       KEYWORDS
!
!     program_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical nooutput

!     nooutput       i    If true no output is written to the file sepcomp.out
!                         by the time integration subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipinsp, niinp, ipos, lenlinsol, i
      double precision rinput
      integer, pointer :: iinput(:)

!     i              Counting variable
!     iinput         integer input array
!     ipinsp         Starting address in array iinput
!     ipos           Position of input arrays in sepmoduleinput
!     lenlinsol      Number of entries in array iinsol
!     niinp          Length of integer input array
!     rinput         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   -
! **********************************************************************
!
!                       PSEUDO CODE
!
!   if ( iinputarrs(10)%maxcall = 0 ) then
!      Create space for array iinspc
!      Fill array iinspc with default values depending on the input read
!      Fill arrays isepi2 and isepi3 for structure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini016' )
      iseqkp = kprob
      if ( iinputarrs(10)%maxcall==0 ) then

!     --- Structure has not yet been filled
!         Create temporary array iinput

         nprob = kp(iseqkp)%nprob
         niinp = 100

         allocate ( iinput(niinp), stat = error )
         if ( error/=0 ) call eralloc ( error, niinp, 'iinput' )
         if ( ierror/=0 ) go to 1000

!        --- Fill array iinspc with defaults
!            numvec

         ipinsp = 1
         iinput(ipinsp) = nprob

         if ( iinputarrs(15)%maxcall/=0 ) then

!        --- create

            iinput(ipinsp+1) = 4

         else

!        --- presbc

            iinput(ipinsp+1) = 1

         end if

         iinput(ipinsp+2) = -1
         iinput(ipinsp+3) = -1
         iinput(ipinsp+4) = -1
         ipinsp = ipinsp + 4

!        --- linprb or nlnprb

         if ( iinputarrs(12)%maxcall==0 .and. iinputarrs(15)%maxcall==0 ) then

!        --- nlnprb nor timprb found, hence linear problem

            iinput(ipinsp+1) = 2
            iinput(ipinsp+2) = -1
            iinput(ipinsp+3) = -1
            iinput(ipinsp+4) = -1
            iinput(ipinsp+5) = -1
            lenlinsol = 15
            do i = 6, lenlinsol      ! This number must be enlarged in case
                                     ! the same happens in readsclinsol
               iinput(ipinsp+i) = 0

            end do  ! i = 6, lenlinsol
            if ( iinputarrs(8)%maxcall/=0 ) then

!           --- input for the linear solver has been found
!               Check if defect correction should be applied

               ipos = iinputarrs(8)%istart
               if ( associated(in(ipos)%iinput) ) then

!              --- input found

                  if ( in(ipos)%iinput(20)/=0 ) iinput(ipinsp+6) = 1

               end if

            end if
            ipinsp = ipinsp + lenlinsol

         else if ( iinputarrs(15)%maxcall==0 ) then

!        --- nlnprb found, hence non-linear problem

            iinput(ipinsp+1) = 3
            iinput(ipinsp+2) = -1
            iinput(ipinsp+3) = -1
            iinput(ipinsp+4) = -1
            iinput(ipinsp+5) = 0
            ipinsp = ipinsp + 5

         else

!        --- timprb found, hence time-dependent problem

            iinput(ipinsp+1) = 19
            iinput(ipinsp+2) = -1
            iinput(ipinsp+3) = -1
            iinput(ipinsp+4) = -1
            ipinsp = ipinsp + 4

         end if

         if ( iinputarrs(15)%maxcall==0 .or. nooutput ) then

!        --- outsol

            iinput(ipinsp+1) = 5
            iinput(ipinsp+2) = 1
            iinput(ipinsp+3) = 1
            iinput(ipinsp+4) = 1
            ipinsp = ipinsp + 4

         end if

!        --- end

         iinput(ipinsp+1) = -1

!        --- Fill arrays isepi2 and isepi3

         iinputarrs(10)%maxcall = 1

!        --- Check length of array inputstru

         niinp = ipinsp+1
         call inichkarlen ( niinp, 100, 'inputstru' )
         if ( ierror/=0 ) go to 1000

!        --- Store input arrays as permanent arrays

         call ercopyinput ( 10, 1, 'STRUCTURE', iinput, &
                            rinput, iinput, niinp, 0, 0 )

         deallocate ( iinput, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iinput' )

      end if

1000  call erclos ( 'ini016' )

      end
