      subroutine inichkcurvs ( icurves, numcurves, kmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if all curves in icurves are contiguous
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     check
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurves(*), numcurves, kmeshm(*)

!     icurves        i    array containing a series of curve numbers
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     numcurves      i    Number of curves in array icurves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i, icurnr, jcurnr, istar3

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2078   Curves are not contiguous
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find last point of first curve
!     For all other curves do
!        Find first point in curve
!        Check if first point and previous last point are equal
!        Find last point in curve
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inichkcurvs' )

!     --- Find last point of first curve

      istar3 = kmeshm(2) - 1
      icurnr = icurves(1)
      jcurnr = abs(icurnr)
      if ( icurnr.gt.0 ) then
         i2 = kmeshm(istar3+kmeshm(5+jcurnr)-1)
      else
         i2 = kmeshm(istar3+kmeshm(4+jcurnr))
      end if

!     --- For all other curves do

      do i = 2, numcurves
         icurnr = icurves(i)
         jcurnr = abs(icurnr)

!        --- Find first point in curve

         if ( icurnr.gt.0 ) then
            i1 = kmeshm(istar3+kmeshm(4+jcurnr))
         else
            i1 = kmeshm(istar3+kmeshm(5+jcurnr)-1)
         end if

!        --- Check if first point and previous last point are equal

         if ( i1.ne.i2 ) then

!        --- Error first point not equal to previous last point

            call errint ( icurves(i), 1 )
            call errint ( i1, 2 )
            call errint ( icurves(i-1), 3 )
            call errint ( i2, 4 )
            call errsub ( 2078, 4, 0, 0 )
            go to 1000

         end if

!        --- Find last point in curve

         if ( icurnr.gt.0 ) then
            i2 = kmeshm(istar3+kmeshm(5+jcurnr)-1)
         else
            i2 = kmeshm(istar3+kmeshm(4+jcurnr))
         end if
      end do
1000  call erclos ( 'inichkcurvs' )
      end
