      subroutine insp03
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 15-11-2012 Long integers
!        version  2.7  date 01-09-2006 Extension of cpost1
!        version  2.6  date 11-08-2004 Extension with normal stress vector
!
!   copyright (c) 1990-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Translates the compute statements into information with respect
!   to iinputpst, rinputpst etc.
!   help subroutine for subroutine SEPPS1
! **********************************************************************
!
!                       KEYWORDS
!
!     stream_function
!     compute
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, imesh, istrf, i, j, indsol

!     i              Counting variable
!     imesh          Actual mesh and problem number
!     indsol         Sequence number of vector in isol
!     istrf          Starting position of the actual arrayin ISOL to be
!                    processed for stream function
!     ivec           Sequence number of array to be processed
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     533   no free meshes available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Reset and delete old arrays, if necessary
!     if ( ispec(istrf,1)#0 ) then
!        Delete all corresponding arrays
!        if ( ispec(istrf,3)>1 ) then
!           Delete mesh and corresponding prob array
!     if ( iinputpst(1) = 203 ) then
!         fill information for subroutine STREAM
!     else if ( iinputpst(1) = 201 ) then
!         fill information for subroutine INTP2D or INTP3D
!     else if ( iinputpst(1)>=204 ) then
!         fill information for computation of mach number, total enthalpy
!         or stagnation pressure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'insp03' )

!     --- Reset and delete old arrays, if necessary

      ivec  = iinputpst(2)
      istrf = iinputpst(16)
      if ( ispec(istrf,1)/=0 ) then

!     --- Array has been used before
!         Delete all corresponding arrays
!         Loop over all possible time-steps

         do i = 1, ntimvc

            indsol = imapsol(istrf,i)
            if ( indsol>0 ) then

               if ( isolut2d(indsol)>0 ) then

!              --- Array isol has been filled; delete

                  call seprmsolinfo ( isolut2d(indsol) )

               end if

            end if

         end do

         if ( ispec(istrf,3)>1 ) then

!        --- Array corresponds to a new mesh
!            Delete mesh and corresponding prob array

            imesh = ispec(istrf,3)
            if ( kmesh2d(imesh)>0 .and. kmesh2d(imesh)<=lastmesh ) &
               call seprmmeshinfo ( kmesh2d(imesh) )
            ispec(istrf,3) = 1
            indmsh(imesh) = 0

         end if

      end if
      do i = 1, ntimvc
         indsol = imapsol(istrf,i)
         if ( indsol>0 ) then
            if ( isolut2d(indsol)/=0 ) then

!           --- Array isol did already exist, delete it

               call seprmsolinfo ( isolut2d(indsol) )

            end if
         end if
      end do

      if ( iinputpst(1)==203 ) then

!     --- iinputpst(1) = 203 Subroutine STREAM
!         In this case ispec(.,4) must contain the type of output vector

         jpoint = iinputpst(17)
         if ( jpoint >0 ) then
            psiphi = rinputpst(6)
         end if
         ichels=1
         ispec(istrf,1) = 1
         ispec(istrf,2) = ivec
         ispec(istrf,4) = iinputpst(75)
         imesh = ispec(istrf,3)
         call sepfillkmesh ( kmesh2d(imesh) )

         if ( iinputpst(32)>=1 ) then

            if ( associated(iskiparr) ) deallocate ( iskiparr )
            allocate ( iskiparr(nelgrp+5) )
            iskiparr(1) = nelgrp+5
            iskiparr(2:5) = 0
            iskiparr(6:) = 1
            do i = 1, iinputpst(32)
               if ( igroup(i)>0 .and. igroup(i)<= nelgrp ) &
                    iskiparr(5+igroup(i)) = 0
            end do
            ispec(istrf,1) = 1
         end if

      else if ( iinputpst(1)==201 ) then

!     --- iinputpst(1) = 201 Subroutine INTP2D or INTP3D

         imesh  = ispec(ivec,3 )
         call sepfillkmesh ( kmesh2d(imesh) )
         if ( ndim==2 ) then

!        --- 2D mesh, use subroutine INTP2D

            ichint = 1
            iskip  = 0
            numint(istrf) = max ( 1, iinputpst(8) )
            do i = 1,3
               valint(i,istrf) = rinputpst(6+i)
            end do
            ispec(istrf,1) = 2
            ispec(istrf,2) = ivec
            ispec(istrf,3) = imesh

         else

!        --- 3D mesh use subroutine INTP3D

            iinputpst(1) = 202
            numint(istrf) = max ( 1, iinputpst(46) ) + 100 * iinputpst(8)
            do i = 1, 4
               valint(i,istrf) = rinputpst(16+i)
            end do
            valint(5,istrf) = rinputpst(7)
            valint(6,istrf) = rinputpst(8)
            valint(7,istrf) = rinputpst(35)
            do i = 8, 13
               valint(i,istrf) = rinputpst(28+i)
            end do
            ispec(istrf,1) = 3
            ispec(istrf,2) = ivec
            ispec(istrf,4) = iinputpst(99)+4

!           --- Search for a free entry in the mesh arrays

            do i = 2, nummesh

               if ( indmsh(i)==0 ) then

!              --- indmsh(i) = 0, i.e. i is a free entry

                  ispec(istrf,3) = i
                  indmsh(i) = 1
                  go to 230

               end if

            end do

!           --- If no meshes are available anymore the program comes at
!               this place

            call errint ( ivec, 1 )
            call errint ( nummesh, 2 )
            call errsub ( 533, 2, 0, 0 )

230      end if

      else if ( iinputpst(1)==209 ) then

!     --- iinputpst(1) = 209
!         Interpolate vector using erintpol

         ispec(istrf,1) = 209
         ispec(istrf,2) = ivec
         ispec(istrf,4) = iinputpst(93)

      else if ( iinputpst(1)==210 ) then

!     --- iinputpst(1) = 210
!         Compute normal stress vector along curves

         if ( ispec(istrf,1)==10 ) then

!        --- Array has been filled before, delete array icurvs

 !           if ( ispec(istrf,4)>0 ) &
 !              call seprmibuf ( ispec(istrf,4), 'icurvs' )

         end if  ! ( ispec(istrf,1)==10 )

         ispec(istrf,1) = 10
         ispec(istrf,2) = ivec
         ncurvs = icurvs(1)            ! number of curves

         ispec(istrf,4) = iinputpst(67)  ! idim

      else if ( iinputpst(1)==225 ) then

!     --- iinputpst(1) = 225
!         Compute vector using manvec

         ispec(istrf,1) = 225
         ispec(istrf,2) = ivec
         ispec(istrf,4) = iinputpst(86)

      else if ( iinputpst(1)>=204 ) then

!     --- iinputpst(1) = 204, 205, 206, 207 or 208
!         Compute mach number, total enthalpy, stagnation pressure, scaled
!         pressure or entropy

         ispec(istrf,1) = iinputpst(1)-200
         ispec(istrf,2) = ivec
         ispec(istrf,4) = iinputpst(3)

      end if

      call erclos ( 'insp03' )
      end subroutine insp03
