      subroutine mshchcor ( iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.5  date 25-11-2012 Long integers
!        version  1.4  date 24-08-2010 New call to mshchacr
!
!   copyright (c) 1998-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change coordinates in array coor by (repeated) calls to FUNCCOOR
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     change
!     change_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr

!     iseqnr     i    Sequence number of input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idummy, iinpdef(5), i, leniin, maxcal, ipos
      integer, pointer :: iinput(:)

!     i              Counting variable
!     idummy         Dummy parameter
!     iinpdef        Default array iinput for MSHCHACR
!     iinput         integer input array
!     ipos           Position of input arrays in sepmoduleinput
!     leniin         Computed length of array IINEIG
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPMSHCHACR    Change coordinates in array coor by calls to FUNCCOOR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iinpdef / 4, 1, 3*0 /
! ======================================================================
!
      call eropen ( 'mshchcor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshchcor'
100      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read input array from ibuffr

      call sepgetinpfinf ( 22, iseqnr, ipos, maxcal )
      if ( debug) write(irefwr,100) 'iinput ', iinput
      if ( maxcal>0 ) then

!     --- Copy input to iinput and adapt if necessary

         leniin = size( in(ipos)%iinput)
         allocate ( iinput(leniin), stat = error )
         if ( error/=0 ) call eralloc ( error, leniin, 'iinput' )
         if ( ierror/=0 ) go to 1000

         iinput(1:leniin) = in(ipos)%iinput(1:leniin)

!        --- Adapt variables

         call inichinpsol ( iinput, leniin )
         if ( debug ) call prinin ( iinput, leniin, 'iinput' )

      end if
      isurnr = 0
      if ( maxcal>0 ) then

!     --- Input has been read from the input file

         call sepmshchacr ( iinput, 5 )

         deallocate ( iinput, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iinput' )

      else

!     --- Default input

         call sepmshchacr ( iinpdef, 5 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshchcor'

      end if
      call erclos ( 'mshchcor' )

      end
