      subroutine mshvelo ( iseqmsv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 25-11-2012 Long integers
!        version  2.1  date 09-12-2005 Call to sepcreastruc
!        version  2.0  date 04-04-1999 Extra parameters ibuffr/buffer
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the mesh velocity
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     copy
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulevecs
      use sepmodulemain
      use sepmodulemesh
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqmsv

!     iseqmsv    i    Sequence number of mesh velocity vector with respect to
!                     isolut2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ivec, iseqmv

!     iseqmv     Part of iseqmsv referring to the sequence number of the vector
!     itypvc     Indication of the type of solution vector:
!                110: solution vector
!                115, 116: vectors of special structure
!     ivec       Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEP2DVEC       Associate 2d array usoltemp2d to 1d vector
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvelo' )
      itypvc = 110
      ivec = iseqmsv/1000
      iseqmv = mod(iseqmsv,1000)
      call sepcreastruc ( ndim, itypvc, ivec )
      call sepcreasolbf1 ( isolut2d(iseqmv), itypvc, ivec, 'mesh_velo', 0 )
      meshvel => ks(isolut2d(iseqmv))%sol
      if ( ichanmesh/=0 ) then

!     --- mmcoor = 0, set vector to 0

         meshvel = 0d0

      else

!     --- Standard situation

         call sep2dvec ( meshvel, ndim, npoint )
         meshvel2d => usoltemp2d
         meshvel2d = (coor-coorold)/tstep

      end if
      call erclos ( 'mshvelo' )

      end subroutine mshvelo
