      subroutine pradapintbn2d ( isol, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 07-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt the concentration in end points of interface
!     by making it equal to neighbor values
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     boundary
!     interface
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscadpintbn
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqconc, ilev, numendpoints, iseq
      integer, allocatable :: istore(:,:)

!     ilev           Sequence number of level being considered
!     iseq           Dummy
!     iseqconc       Sequence number of array concentration in isol
!     istore         Work array to store end points of interface
!                    istore(1,i) contains i-th end point
!                    istore(2,i) contains its neighbor
!     numendpoints   Number of end points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRADAPINTBN2D1 Find end nodes of interface
!     PRADAPINTBN2D2 Adapt concentration
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pradapintbn2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pradapintbn2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqconc = inpsep(1)  ! sequence number concentration
      ilev = inpsep(2)      ! sequence number level set

!     --- Define work space

      allocate ( istore(2,npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'istore' )
      if ( ierror/=0 ) go to 1000

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( kelmad==0 ) call sepfillkmeshad(iseq)
      if ( kmeshad(9)==0 ) call sepfillobfaces
      if ( nmesh(33)==0 ) call sepfillnmeshc

!     --- Find end nodes of interface (if any)

      call pradapintbn2d1 ( numendpoints, istore, ilev )

      if ( numendpoints>0 ) then

!     --- Number of end points>0, adapt concentration

         usoltemp => ks(isol(iseqconc))%sol
         call pradapintbn2d2 ( numendpoints, istore, usoltemp )

      end if  ! ( numendpoints>0 )

      deallocate ( istore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pradapintbn2d'

      end if  ! ( debug )
      call erclos ( 'pradapintbn2d' )

      end
