      subroutine pradapintboun ( isol, inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt the concentration in end points of interface
!     by making it equal to neighbor values
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     boundary
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5,*), inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscadpintbn
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     istart        i/o   Next position in inpsep to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRADAPINTBN2D  Adapt the concentration in end points of interface
!                    by making it equal to neighbor values
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pradapintboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pradapintboun'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( ndim )

      case(2)

!     --- ndim = 2

         call pradapintbn2d ( isol, inpsep )

      case default

!     --- ndim not yet implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )

      end select  ! case ( ndim )

      istart = istart+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pradapintboun'

      end if  ! ( debug )
      call erclos ( 'pradapintboun' )

      end
