      subroutine prareas ( areas )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-07-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute areas of all elements of highest dimension
!     All other areas are made equal to zero
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision areas(*)

!     areas          o    Contains the areas of the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, shapemin, ikelmc, istart, ielgrp

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     istart         Next position to be used in kmeshc
!     shapemin       Lowest shape number of elements of highest dimension
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSUBVOL1     Compute volume of elements in submesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prareas' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prareas'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute minimum shape number depending on dimension

      if ( ndim==1 ) then
         shapemin = 1
      else if ( ndim==2 ) then
         shapemin = 3
      else
         shapemin = 11
      end if  ! ( ndim==1 )

!     --- Clear areas

      areas(1:inelem) = 0d0

!     --- Loop over element groups

      ikelmc = 1
      istart = 1

      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)   ! shape number of element

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         if ( ishape>=shapemin ) then

!        --- shape belongs to highest dimension

            call mshsubvol1 ( kmeshc(ikelmc), inelem, ishape, inpelm, &
                              ielgrp, coor, ndim, areas(istart), npoint )

         end if  ! ( ishape>=shapemin )

         ikelmc = ikelmc + inelem*inpelm
         istart = istart+inelem

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prareas'

      end if  ! ( debug )
      call erclos ( 'prareas' )

      end
