      subroutine prbnin ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.4  date 27-05-2014 Extension with irule
!        version  3.3  date 29-04-2014 Extension with degfd
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call subroutine INTBND to compute boundary integrals
!     Fill array IINTBN with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      use sepmoduletime
      use sepmodulecons
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine raedscbounint form poistion 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ivectr, iseqin, idummy(1), &
              iscal1, iscal2, iscal3, nch, inpvol(7), iinvol, maxcal, ipos
      integer, allocatable :: inpcop(:)
      double precision outval(3), dummy(1)

!     dummy          Double precision dummy parameter
!     idummy         Dummy parameter
!     iinvol         Infor entry of array INPVOL as stored in IBUFFR
!     inpcop         Copy of integer input array extended with defaults
!     inpvol         Integer array in which the user indicates what
!                    computations must be performed by subroutine intbndbf
!     ipos           Position of input arrays in sepmoduleinput
!     iscal1         Sequence number of first scalar
!     iscal2         Sequence number of second scalar
!     iscal3         Sequence number of third scalar
!     iseq           Sequence number of vector in VECOLD/ISLOLD
!     iseqin         Sequence number with respect to the storage in array ISEPIN
!                    for boundary integrals
!     ivectr         Sequence number of first vector
!     maxcal         Maximum number of coefficient inputs read
!     nch            Number of characters in a string
!     outval         Contains the values of the integrals computed:
!                    1:  integral over boundary
!                    2:  second component of integral over boundary
!                    3:  third component of integral over boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPINTBND      compute an integral along a boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqin /0/
! ======================================================================
!
      call eropen ( 'prbnin' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbnin'
         write(irefwr,1) 'istartmn', istartmn
         call prinin ( inpsep, 9+inpsep(7), 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqin = iseqin + 1
      iseq   = inpsep(1)
      if ( iseq/=-1 ) iseqin = iseq
      ivectr = inpsep(2)
      iscal1 = inpsep(3)
      iscal2 = inpsep(4)
      iscal3 = inpsep(5)

      iinvol = 0

      if ( ioutp>=0 .and. itimloop<=1 ) then
         nch = index(chscalars(iscal1),' ')-1
         if ( nch<=0 ) nch = len(chscalars(iscal1))
         write(irefwr,120) chscalars(iscal1)(1:nch), iscal1
120      format ( ' Compute scalar ', a, &
                  ' with sequence number ', i3, &
                  ' as boundary integral' )
      end if

!     --- Check if input for integral can be found

      if ( inpsep(6)<=0 ) &
         call sepgetinpfinf ( 14, iseqin, ipos, maxcal )

      if ( maxcal<=0 .or. inpsep(6)>0 ) then

!     --- No input found, fill array inpvol

         iseqin = -1  ! indicate that inpvol is filled
         inpvol(1) = 7           ! number of entries in inpvol
         inpvol(2) = inpsep(8)   ! type of integral
         inpvol(3) = 0           ! Function
         inpvol(4) = inpsep(10)  ! irule
         inpvol(5) = inpsep(6)   ! idim
         inpvol(6) = inpsep(9)   ! number
         inpvol(7) = inpsep(7)   ! number of curves

      end if  ! ( iinvol<=0 )

      length = max(15,14+ inpsep(7))
      allocate ( inpcop(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'inpcop' )
      if ( ierror/=0 ) go to 1000

      length = 14 + inpsep(7)
      inpcop(1:length) = inpsep(1:length)
      call inichinpsol ( inpcop, length )  ! replace variables by their values

      outval(1) = 0d0
      outval(2) = 0d0
      outval(3) = 0d0
      idummy(1) = 0

      call sepintbnd ( inpvol, dummy, &
                       isolut2d(ivectr), inpcop(15), outval, iseqin )

      scalars(iscal1) = outval(1)
      numscals = max(numscals,iscal1)
      if ( abs(outval(2))>1d-50 ) then

!     --- outval has been filled

         if ( ioutp>=0 .and. itimloop<=1 ) then
            nch = index(chscalars(iscal2),' ')-1
            if ( nch<=0 ) nch = len(chscalars(iscal2))
            write(irefwr,130) chscalars(iscal2)(1:nch), iscal2
130         format ( ' Compute scalar ', a, &
                     ' with sequence number ', i3, &
                     ' as second component of boundary integral' )
         end if
         scalars(iscal2) = outval(2)
         numscals = max(numscals,iscal2)

      end if
      if ( abs(outval(3))>1d-50 ) then

!     --- outval has been filled

         if ( ioutp>=0 .and. itimloop<=1 ) then
            nch = index(chscalars(iscal3),' ')-1
            if ( nch<=0 ) nch = len(chscalars(iscal3))
            write(irefwr,140) chscalars(iscal3)(1:nch), iscal3
140         format ( ' Compute scalar ', a, &
                     ' with sequence number ', i3, &
                     ' as third component of boundary integral' )
         end if
         scalars(iscal3) = outval(3)
         numscals = max(numscals,iscal3)

      end if
      istartmn = istartmn + 15 + inpsep(7)

      deallocate ( inpcop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpcop' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End prbnin'

      end if  ! ( debug )
      call erclos ( 'prbnin' )

      end
