      subroutine prcallcontct
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-12-2017 New storage of intmat1d
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.2  date 24-12-2014 New definition clinsl
!
!   copyright (c) 2003-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call contact (prconct) for all input parts with respect to contact
!     So all contact input parts are effective, even if the user does not want
!     this
!     This is especially meant for refine
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
!     refine
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpsep(1), maxcal, icall, ipos

!                    they are not
!     icall          Call sequence number for present subroutine
!     inpsep         The integer input read is stored in array INPSEP
!     ipos           Position of input arrays in sepmoduleinput
!     maxcal         Maximum value of icall
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCONTCT       Compute new contact
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Return if no input blocks are active
!     For all input blocks correponding to contact do
!        call prcontct
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcallcontct' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcallcontct'

      end if

!     --- Get information for contact

      call sepgetinpfinf ( 20, 1, ipos, maxcal )

!     --- If no input for contact has been read: return

      if ( maxcal<=0 ) go to 1000

!     --- For all input blocks corresponding to contact do
!         all prcontct

      do icall = 1, maxcal
         call sepgetinpfinf ( 20, icall, ipos, maxcal )
         inpsep(1) = icall
         if ( ipos>0 ) call prcontct ( inpsep )
      end do  ! icall = 1, maxcal

1000  if ( debug ) write(irefwr,*) 'End prcallcontct'
      call erclos ( 'prcallcontct' )

      end
