      subroutine prcontct ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 15-12-2017 New storage of intmat
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.8  date 25-05-2015 Mew call to prinintmat
!
!   copyright (c) 1997-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine TOCONCT to compute contact points
!    Fill information in INPCONCT
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)

!     inpsep  i    The integer input read is stored in array INPSEP
!                  1  ISEQNR     Sequence number with respect to the input
!                                of contact
!                                If iseqnr = -1, the next one will be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ninpcp
      parameter ( ninpcp=25)
      integer iseq, iseqnr, inpconct(ninpcp), &
              idummy(1), lninconct, i, maxcal, ipos, iseqprob
      logical changed

!     changed        If true the contact points have been changed
!     i              General loop parameter
!     idummy         Dummy parameter
!     inpconct       In this array it is indicated how the contact surface
!                    must be computed, See subroutine toconct
!     ipos           Position of input arrays in sepmoduleinput
!     iseq           First position in array INPSEP
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     iseqprob       Sequence number of problem input
!     lninconct      Length of array inpconct as read by readal
!     maxcal         Maximum number of coefficient inputs read
!     ninpcp         Length of array inpconct
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININTMAT     Print array INTMAT
!     PROBDFEXT      Reads and stores problem definition
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPPRINKMESH   Print array KMESH
!     TOCONCT        Compute whether surfaces or curves make contact and if so
!                    compute the points and elements in the contact surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Get input for toconct from ibuffr and store in inpconct
!    Compute contact points and elements
!    If ( the contact points have been changed ) then
!       Recompute KPROB and INTMAT
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcontct' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcontct'

      end if

!     --- Get input for toconct from ibuffr and store in inpconct

      iseqnr = 1
      iseq   = inpsep(1)
      if ( iseq/=-1 ) iseqnr = iseq
      do i = 1, ninpcp
         inpconct(i) = 0
      end do
      call sepgetinpfinf ( 20, iseqnr, ipos, maxcal )
      if ( maxcal>0 ) then

         lninconct = size( in(ipos)%iinput )
         inpconct(1:lninconct) = in(ipos)%iinput

!        --- Adapt variables

         call inichinpsol ( inpconct, lninconct )

      end if

!     --- Compute contact points and elements

      inpconct(1) = iseq
      inpconct(5) = numvecmn
      call toconct ( inpconct, changed )
      if ( debug ) write(irefwr,*) 'changed ', changed
      if ( changed ) then

!     --- Contact points have been changed
!         Recompute KPROB and INTMAT

         iseqprob = 1
         call probdfext ( 3, iseqprob )
         call incomgen ( iseq, intmat1d )
         if ( debug ) then

!        --- Debug information

            call sepprinkmesh ( 1 )
            intmat => intmat1d(1)
            call prinintmat ( 0 )

         end if

      end if

      if ( debug ) write(irefwr,*) 'End prcontct'
      call erclos ( 'prcontct' )

      end
