      subroutine prcopsol ( meshorig, meshobst, map1d, i, all )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate a series of solution vectors from one mesh to the other
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     level_set
!     copy
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst, i
      logical all
      type (mapping) :: map1d

!     all            i    If true all vectors must be used
!     i              i    Sequence number of array
!     map1d         i/o   contains information of the mapping
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum(1), itypvc

!     idum           General dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINTPOLMS    Interpolate solution from one mesh to another
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1055   Array in solution vector has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isolut3d(i,meshorig)>0 .and. isolut3d(i,meshorig)<=lastsol ) then
         if ( solarr(isolut3d(i,meshorig))>0 ) then
            itypvc = ks(isolut3d(i,meshorig))%typevec
            if ( itypvc==110 .or. itypvc==115 ) then

               call sepintpolms ( meshorig, meshobst, iinmap, &
                                  isolut3d(i,meshorig), &
                                  isolut3d(i,meshobst), map1d, idum )
               iinmap(4) = meshorig
               iinmap(5) = meshobst
            end if  ! (itypvc==110 .or. itypvc==115)
         end if  ! ( solarr(isolut3d(i,meshorig)) )

      else if ( .not. all ) then

!      --- error vector does not exist

         call errint ( i, 1 )
         call errint ( ks(isolut3d(i,meshorig))%typevec, 2 )
         call errchr ( names(i), 1 )
         call errsub ( 1055, 2, 0, 1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopsol'

      end if  ! ( debug )
      call erclos ( 'prcopsol' )

      end
