      subroutine prcopy2d2 ( npoint, vector1, ndim1, vector2, ndim2, &
                             ifirst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 23-01-2015 New type of loop
!        version  1.1  date 12-12-2011 Extra debug
!        version  1.0  date 07-00-2005
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy two-dimensional array of size ndim1 x npoint into two-dimensional
!    array of size ndim2 x npoint.
!    ndim2 is supposed to be smaller than ndim1
!    Copying is done as follows:
!    vector2(1...ndim2,.) := vector1(ifirst..ifirst+ndim2-1,.)
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ndim1, ndim2, ifirst
      double precision vector1(ndim1,npoint), vector2(ndim2,npoint)

!     ifirst         i    First entry of vector to which must be copied
!     ndim1          i    First dimension of vector to be copied
!     ndim2          i    First dimension of vector to which must be copied
!     npoint         i    Number of nodal points in vectors
!     vector1        i    Vector to be copied
!     vector2        o    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j

!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopy2d2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopy2d2'
         write(irefwr,1) 'ndim1, ndim2, ifirst', ndim1, ndim2, ifirst
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      do j = 1, min(ndim2,ndim1-ifirst+1)
         vector2(j,1:npoint) = vector1(j+ifirst-1,1:npoint)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopy2d2'

      end if
      call erclos ( 'prcopy2d2' )

      end
