      subroutine prcopy2d2c ( npoint, vector1, ndim1, vector2, ndim2, &
                              ifirst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-00-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy two-dimensional array of size ndim1 x npoint into two-dimensional
!    array of size ndim2 x npoint.
!    ndim2 is supposed to be smaller than ndim1
!    Copying is done as follows:
!    vector2(1...ndim2,.) := vector1(ifirst..ifirst+ndim2-1,.)
!    Complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ndim1, ndim2, ifirst
      double complex vector1(ndim1,npoint), vector2(ndim2,npoint)

!     ifirst         i    First entry of vector to which must be copied
!     ndim1          i    First dimension of vector to be copied
!     ndim2          i    First dimension of vector to which must be copied
!     npoint         i    Number of nodal points in vectors
!     vector1        i    Vector to be copied
!     vector2        o    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopy2d2c' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopy2d2c'

      end if
      if ( ierror/=0 ) go to 1000
      do j = 1, min(ndim2,ndim1-ifirst+1)
         do i = 1, npoint
            vector2(j,i) = vector1(j+ifirst-1,i)
         end do
      end do

1000  call erclos ( 'prcopy2d2c' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopy2d2c'

      end if

      end
