      subroutine prcopychseq ( array, npoint, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-04-2003 Remove array work
!        version  1.0  date 29-12-2000
!
!   copyright (c) 2000-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Interchange sequence of parameters in two-dimensional array by considering
!    it as a one-dimensional array
!    Hence array array of size ndim x npoint is rearranged into an array of
!    size npoint x ndim
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     interchange
!     two_dimensional_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision array(*)
      integer npoint, ndim

!     array         i/o   Two-dimensional array (stored as one-dimensional one)
!                         of which the components must be interchanged
!     ndim           i    First dimension of the input vector, last dimension
!                         of the output vector
!     npoint         i    Last dimension of the input vector, first dimension
!                         of the output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision work(ndim*npoint)

!     i              Counting variable
!     work           Work array of length ndim x npoint in which the
!                    interchanged vector is stored temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Copy array into work and change sequence

      do i = 1, ndim
         call dcopy ( npoint, array(i), ndim, work((i-1)*npoint+1), 1 )
      end do

!     --- Copy work back into array

      call dcopy ( npoint*ndim, work, 1, array, 1 )

      end
