      subroutine prcopyvecs ( iold, inew, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 24-01-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy the contents of vector iold into vector inew with structure ivec
!    In those nodes where inew has less unknowns than iold entries of iold
!    are skipped
!    In those nodes where inew has more unknowns than iold the corresponding
!    entries are set equal to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iold, inew, ivec

!     inew          i/o   Standard SEPRAN array containing information of the
!                         new vector
!     iold           i    Standard SEPRAN array containing information of the
!                         vector to be copied
!     ivec           i    Sequence number of vector of special structure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOPYVECS1    Copy the contents of vector uold into vector unew
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      13   array iold has not been filled.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check type of input array
!     Activate array iold and corresponding arrays
!     Create or activate array inew and corresponding arrays
!     Compute pointers
!     Clear array unew
!     Perform the copying
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyvecs' )

!     --- Check type of input array

      itypvc = ks(iold)%typevec
      if ( itypvc/=110 .and. itypvc/=115 ) then
         call errint ( itypvc, 1 )
         call errsub ( 13, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Activate array iold and corresponding arrays

      call sepactsolbf2 ( iold )
      if ( ierror/=0 ) go to 1000

!     --- Create or activate array inew and corresponding arrays

      if ( ivec==0 ) then

!     --- ivec = 0, solution vector

         itypvc = 110

      else

!     --- ivec>0, vector of special structure

         itypvc = 115

      end if
      iprob = ks(iold)%iprob
      call sepcreasolbf1 ( inew, itypvc, ivec, namesol(iold), ks(iold)%compl )
      usolt => usoltemp
      if ( ierror/=0 ) go to 1000

!     --- Compute pointers

      usoltemp => ks(iold)%sol

!     --- Clear array unew

      usolt = 0d0

!     --- Perform the copying

      call prcopyvecs1 ( usoltemp, usolt )

1000  call erclos ( 'prcopyvecs' )
      end subroutine prcopyvecs
