      subroutine prcorrinterp ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-03-2012 extra parameter numvec
!        version  1.1  date 29-02-2012 Extension with 3d
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct interpolation in case nodes of the mesh crossed
!     the old object
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     level_set
!     node
!     intersection
!     1d
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemnsub
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    Integer input array
!                         See subroutine readsclevel for the contents
!                         The position are shifted one down
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readsclevel for the contents
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mesh1, ndim

!     mesh1          Sequence number of basis mesh
!     ndim           Dimension of space
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRCORRINTERP1D Correct interpolation in case nodes of the mesh crossed
!                    the old object (1d)
!     PRCORRINTERP2D Correct interpolation in case nodes of the mesh crossed
!                    the old object (2d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcorrinterp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcorrinterp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      mesh1 = abs(meshseqs(inirepscal(inpsep(1))))
      iseqkm = mesh1
      ndim = km(iseqkm)%kmesh(6)  ! dimension of space

      select case ( ndim )
      case (1)

!     --- ndim = 1

         call prcorrinterp1d ( inpsep, rinsep )

      case (2, 3)

!     --- ndim = 2, 3

         call prcorrinterp2d ( inpsep, rinsep )

      end select  ! case ( ndim )

      istartmn = istartmn+9  ! raise istartmn

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcorrinterp'

      end if  ! ( debug )
      call erclos ( 'prcorrinterp' )

      end
