      subroutine prcreatekmesh ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a part of kmesh
!     Ols subroutine iniactm0
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readsckmesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical exist
      integer :: iseq

!     exist          If true, an array already exists
!     iseq           Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFILLKMAD2   Fill array kmesh ad  part 2: ielemobst
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLKMESHO  Fill kmesh part o
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHE  Fill array nmeshe
!     SEPFILLNMESHF  Fill array nmeshf
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFILLNMESHI  Fill array nmesh part i
!     SEPFILLNMESHJ  Fill array nmesh part j
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
!     SEPFILLOUTERB  Fill outer boundary of mesh
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcreatekmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcreatekmesh'
         write(irefwr,1) 'jchoice', inpsep(1)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( inpsep(1) )

      case(1)

!     --- iseqrec() = 1, Fill kmesh AB part elgrps
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(2)

!     --- iseqrec() = 2, Fill kmesh AA part nmeshe

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(35)==0 ) call sepfillnmeshe

      case(3)

!     --- iseqrec() = 3, Fill kmesh part d

         if ( kelmd==0 ) call sepfillkmeshd

      case(4)

!     --- iseqrec() = 4, Fill kmesh AA parts nmeshc and nmeshd

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(33)==0 ) call sepfillnmeshc

      case(5)

!     --- iseqrec() = 5, Fill kmesh AA part nmeshh

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(38)==0 ) call sepfillnmeshh

      case(6)

!     --- iseqrec() = 6, Fill kmesh AA part nmeshl

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(42)==0 ) call sepfillnmeshl

      case(7)

!     --- iseqrec() = 7, Fill kmesh AA part nmesha, b

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(31)==0 ) call sepfillnmesha

      case(8)

!     --- iseqrec() = 8, Fill kmesh AA part nmeshf

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(36)==0 ) call sepfillnmeshf

      case(9)

!     --- iseqrec() = 9, Fill kmesh AA part nmeshg

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(37)==0 ) call sepfillnmeshg

      case(10)

!     --- iseqrec() = 10, Fill kmesh AB part elscurvs
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(11)

!     --- iseqrec() = 11, Fill kmesh AB part elssurfs
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(12)

!     --- iseqrec() = 12, Fill kmesh AB part infvolms
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(13)

!     --- iseqrec() = 13, Fill kmesh AB part infsurfs
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(14)

!     --- iseqrec() = 14, Fill kmesh AB part infcurvs
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(15)

!     --- iseqrec() = 15, Fill kmesh AB part infuspnt
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(16)

!     --- iseqrec() = 16, Fill kmesh AB part elsuspnt
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(20)

!     --- iseqrec() = 20, Fill kmesh part e

         if ( kelme==0 ) call sepfillkmeshe

      case(21)

!     --- iseqrec() = 21, Fill kmesh part ad pos. 1

         if ( kelmad==0 ) call sepfillkmeshad(iseq)
         exist = .false.
         if (kmeshad(5)==kmeshad(6) ) exist = .true.
         if ( .not. exist ) call sepflpointobst

      case(22)

!     --- iseqrec() = 22, Fill kmesh part ad pos. 2

         if ( kelmad==0 ) call sepfillkmeshad(iseq)
         exist = .false.
         if ( kmeshad(2)>0 .and.  kmeshad(5)==kmeshad(7) ) exist = .true.
         if ( .not. exist ) call sepfillkmad2

      case(23)

!     --- iseqrec() = 23, Fill kmesh part ad pos. 3,4

         if ( kelmad==0 ) call sepfillkmeshad(iseq)
         exist = .false.
         if ( kmeshad(3)>0 .and.  kmeshad(5)==kmeshad(8) ) exist = .true.

         if ( .not. exist ) call sepfillkmad3

      case(24)

!     --- iseqrec() = 24, Fill kmesh part ae
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(25)

!     --- iseqrec() = 25, Fill kmesh part af
!         not yet implemented, see iniacmt0

         call errint ( inpsep(1), 1 )
         call errsub ( 2917, 1, 0, 0 )

      case(26)

!     --- iseqrec() = 26, Fill kmesh part ad pos. 9

         if ( kelmad==0 ) call sepfillkmeshad(iseq)
         call sepfillobfaces

      case(27)

!     --- iseqrec() = 27, Fill nmeshk

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(41)==0 ) call sepfillnmeshk

      case(28)

!     --- iseqrec() = 28, Fill kmesh part o

         if ( kelmo==0 ) call sepfillkmesho

      case(29)

!     --- iseqrec() = 29, Fill kmesh part ah

        if ( mmouter==0 ) call sepfillouterb

      case(30)

!     --- iseqrec() = 30, Fill kmesh aa part nmeshj

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(40)==0 ) call sepfillnmeshj

      case(31)

!     --- iseqrec() = 31, Fill nmeshi

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(39)==0 ) call sepfillnmeshi

      end select  ! case ( inpsep(1) )
      istartmn = istartmn+2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcreatekmesh'

      end if  ! ( debug )
      call erclos ( 'prcreatekmesh' )

      end
