      subroutine prcreaterhs ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create right-hand side vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekprob
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine askvecrhs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqcf, isubr, ictime, &
                 ivectr, iinbldlc(10)

!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 )
!     iinbldlc       User input array for subroutine BUILD
!     iseqcf         Sequence number corresponding to the coefficients
!     isubr          Defines type of calling subroutine
!     ivectr         standard sepran array denoting USOL or vector of
!                    special structure defined in nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILD       Build matrix and right-hand side
!     SEPFILLCOEFFS  Fill coefficients at the end of the buffer array
!                    with starting addresses ipiusr and ipuser
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcreaterhs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcreaterhs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ivectr = inpsep(1)
      iprob = max(1,inpsep(2))
      iseqcf = inpsep(3)

      ictime = 0      ! otherwise seqcoef is not used properly

!     --- Fill coefficients in IUSER and USER

      call sepfillcoeffs ( 3, iseqcf, ictime )
      if ( ierror/=0 ) go to 1000

!     --- Fill array iinbldlc

      iinbldlc = 0
      iinbldlc(1) = 10        ! number of entries filled
      iinbldlc(2) = 2         ! only the right-hand side is filled
      iinbldlc(3) = 1         ! no effect of boundary conditions
      iinbldlc(9) = iprob     ! problem number
      iinbldlc(10) = numvecmn ! numold

!     --- Compute right-hand side

      call sepfillsmf ( iinbldlc, isolut2d(ivectr), &
                        isolut2d, isolut2d )
      call sepbuild

      numvecmn = max(numvecmn, ivectr)
      istartmn = istartmn+4   ! raise istartmn

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcreaterhs'

      end if  ! ( debug )
      call erclos ( 'prcreaterhs' )

      end subroutine prcreaterhs
