      subroutine prcreatestring ( inpsep, istart, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-12-2013 Extension with integer scalars
!        version  1.0  date 01-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create string from other strings
!
! **********************************************************************
!
!                       KEYWORDS
!
!     keyword
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscreadmesh for a description
!     istart        i/o   next position in inpsep to be used
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, nparts, itype, iseqvar, lenname, lenstring, &
              ival, i
      double precision value
      character (len=100) strtext
      logical integerfound

!     i              Counting variable
!     integerfound   If true an integer has been found
!     iseq           Sequence number of output string in texts or strings
!     iseqvar        Sequence number of input string (see inpsep)
!     itype          Type of input string
!     ival           Integer value
!     lenname        Length of string
!     lenstring      Length of output string
!     nparts         number of parts the string consists of
!     strtext        String to store a text for conversion
!     value          Real value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETSCAL        Return with value of scalar
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcreatestring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcreatestring'
         write(irefwr,1) 'iseq, nparts', inpsep(2:3)
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nparts = inpsep(3)
      if ( nparts<=0 .or. nparts>=1000 ) then

!     --- Error: value of nparts out of range

         call errint ( nparts, 1 )
         call errint ( 1, 2 )
         call errint ( 1000, 3 )
         call errchr ( 'nparts', 1 )
         call errsub ( 1378, 3, 0, 1 )
         go to 1000

      end if  ! ( nparts<=0 .or. nparts>=1000 )

      iseq = inpsep(2)
      if ( iseq<1 .or. iseq>numstrings ) then

!     --- Error: value of nparts out of range

         call errint ( iseq, 1 )
         call errint ( 1, 2 )
         call errint ( numstrings, 3 )
         call errchr ( 'iseq', 1 )
         call errsub ( 1378, 3, 0, 1 )
         go to 1000

      end if  ! (iseq<1 .or. iseq>numstrings )

      iseq = instring(iseq)  ! sequence number of output in array texts
      texts(iseq) = ' '      ! clear texts(iseq)

      if ( debug ) call prinin ( inpsep, 3+2*nparts, 'inpsep' )

      lenstring = 0
      do i = 1, nparts

         itype = inpsep(2+2*i)
         iseqvar = inpsep(3+2*i)
         integerfound = .false.

         select case ( itype )

         case(1, 5)

!        --- 1:  string from the list of strings
!            5:  string between quotes

            if ( itype==1 ) iseqvar = instring(iseqvar)
            lenname = len_trim(texts(iseqvar))
            texts(iseq)(lenstring+1:lenstring+lenname) = &
                texts(iseqvar)(1:lenname)

         case(2, 6)

!        --- 2: integer constant
!            6: integer

            if ( itype==2 ) then
               ival = incons(iseqvar)
            else
               ival = iseqvar
            end if  ! ( itype==2 )
            integerfound = .true.

         case(3, 4, 7)

!        --- 3: real constant
!            4: scalar
!            7: real

            if ( itype==3 ) then
               value = rlcons(iseqvar)
            else if ( itype==4 ) then
               value =  getscal ( iseqvar )
            else
               value = rinsep(iseqvar)
            end if  ! ( itype==4 )

            integerfound = .false.
            if ( abs(value)<1d5 ) then

!           --- value < 1d5, check if it is an integer

               ival = nint(value)
               if ( ival==0 ) then
                  if ( abs(value)<=0d0 ) integerfound = .true.
               else if ( abs((ival-value)/ival)<1d-10 ) then
                  integerfound = .true.
               end if  ! ( ival==0 )

            end if
            if ( .not. integerfound ) then

!           --- it is a real

               write(strtext,110) value
110            format(ss,1pe13.5)
               strtext = adjustl(strtext)
               lenname = len_trim(strtext)
               texts(iseq)(lenstring+1:lenstring+lenname) = &
                  strtext(1:lenname)

            end if  ! ( .not. integerfound )

         case(8)

!        --- sequence number of vector name

            lenname = len_trim(names(iseqvar))
            texts(iseq)(lenstring+1:lenstring+lenname) = &
                names(iseqvar)(1:lenname)

         end select  ! case ( itype )

         if ( integerfound ) then

!        --- integer found

            write(strtext,200) ival
200         format(i14)
            strtext = adjustl(strtext)
            lenname = len_trim(strtext)
            texts(iseq)(lenstring+1:lenstring+lenname) = &
               strtext(1:lenname)

         end if  ! ( integerfound )

         lenstring = lenstring+lenname

      end do  ! i = 1, nparts
      istart = istart+3+2*nparts

1000  call erclos ( 'prcreatestring' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'string ', texts(iseq)(1:lenstring)
         write(irefwr,*) 'End prcreatestring'

      end if  ! ( debug )

      end
