      subroutine prderv ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.7  date 13-01-2015 Change of istart
!        version  4.6  date 24-12-2014 New definition clinsl
!        version  4.5  date 12-10-2012 Change of istart
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine DERIVS to compute derivatives
!    Fill array IINDER with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    See subroutine readscderiv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ivec, iseqcf, ivectr, iseqdr, iproblc, iprobr, &
              idummy(2), iread(2), iscalr, nch
      double precision output(1)

!     idummy         Help array for DERIVS
!                    Pos. 1:  iscalr
!                    Pos. 2:  numvecmn
!     iproblc        Actual problem number
!     iprobr         Problem number read
!     iread          Integer input array for subroutine DERIVS
!                    iread(1) contains iseqdr
!                    iread(2) contains iseqcf
!     iscalr         Sequence number of scalar
!     iseqcf         Sequence number with respect to the storage in array ISEPIN
!                    for coefficients
!     iseqdr         Sequence number with respect to the storage in array ISEPIN
!                    for derivatives
!     iseqlc           Sequence number of vector in VECOLD/ISLOLD
!     ivec           Sequence number of vector of read
!     ivectr         Sequence number of first vector
!     nch            Number of characters in a string
!     output         Dummy parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     DERIVATIVES    Compute derivatives
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     SEPDERIVS      Compute derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqcf /0/, ivectr /0/, iseqdr /0/, iproblc /0/
! ======================================================================
!
      call eropen ( 'prderv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prderv'
         write(irefwr,1) 'istartmn', istartmn
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iseqdr = iseqdr + 1
      iseqlc   = inpsep(4)
      if ( iseqlc>0 ) iseqdr = iseqlc

      iseqkp = kprob
      nprob = kp(iseqkp)%nprob
      iproblc  = min ( iproblc+1, nprob )
      iseqcf = inpsep(1)
      iprobr = inpsep(2)
      if ( iprobr/=-1 ) iproblc = iprobr
      ivectr = min ( nprob+1, maxvc1 )
      ivec   = inivectr(inpsep(3))
      if ( ivec>0 ) ivectr = ivec

      if ( debug ) &
         write(irefwr,1) 'iseqlc, iseqkp, iseqcf, iprobr', &
                          iseqlc, iseqkp, iseqcf, iprobr

      if ( iseqlc/=-10 ) then

!     --- Standard case, use sequence number

         iscalr = inpsep(5)
         iread(1) = iseqdr
         iread(2) = iseqcf
         if ( debug ) then
            write(irefwr,1) 'iproblc, ivec, iseqdr, iseqcf, iscalr', &
                             iproblc, ivec, iseqdr, iseqcf, iscalr
         end if  ! ( debug )

         if ( iseqcf/=0 .and. ioutp>=0 .and. itimloop<=1 .and. &
              nitercom<=1 .or. debug ) then
            write ( irefwr, 110 ) iseqcf
110         format ( ' Fill coefficients with sequence number ', i3 )

         end if

         idummy(1) = iscalr
         idummy(2) = numvecmn

         if ( debug .or. &
              ioutp>=0 .and. itimloop<=1 .and. nitercom<=1 ) then

            if ( iscalr>0 ) then

               nch = index(chscalars(iscalr),' ')-1
               if ( nch<=0 ) nch = len(chscalars(iscalr))
               write ( irefwr, 120 ) chscalars(iscalr)(1:nch), iscalr
120            format ( ' Fill scalar ', a, &
                        ' with sequence number ', i3, &
                        ' as boundary integral' )
            else

               nch = index(names(ivectr),' ')-1
               if ( nch<=0 ) nch = len(names(ivectr))
               write ( irefwr, 130 ) names(ivectr)(1:nch), ivectr
130            format ( ' Create vector ', a, &
                        ' with sequence number ', i3, &
                        ' as derivative quantity' )

            end if  ! ( iscalr>0 )

         end if  ! ( debug .or. ... )

         call sepderivs ( idummy, isolut2d(ivectr), isolut2d, iread, &
                          output, iproblc, idummy )
         istartmn = istartmn + 6

      else

!     --- New case, all input in inpsep

         if ( debug .or. &
              ioutp>=0 .and. itimloop<=1 .and. nitercom<=1 ) then

            nch = index(names(ivectr),' ')-1
            if ( nch<=0 ) nch = len(names(ivectr))
            write ( irefwr, 130 ) names(ivectr)(1:nch), ivectr

         end if  ! ( debug .or. ... )

         iscalr = 0  ! temporary

         call derivatives ( ivectr, inpsep )

      end if  ! ( iseqlc/=-10 )

      if ( iscalr>0 ) then

!     --- Scalar must be filled

         scalars(iscalr) = output(1)
         numscals = max(numscals,iscalr)

      else

!     --- Vector must be filled

         numvecmn = max(numvecmn,ivectr)

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End prderv'

      end if
      call erclos ( 'prderv' )

      end
