      function prevalboolend ( bwork, ibwork, ncount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate final boolean expression
!     No brackets are present
! **********************************************************************
!
!                       KEYWORDS
!
!     boolean_expression
!     boolean
!     evaluate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncount, ibwork(ncount)
      logical prevalboolend, bwork(ncount)

!     bwork         i/o   Array to store boolean values corresponding to ibwork
!     ibwork        i/o   In this array the boolean operators are stored
!                         See subroutine prevalboolexpr
!     ncount        i/o   Number of entries in bwork
!     prevalboolend  o    Result of operations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINBOOL       Print an array of booleans
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := ncount (-1) 2 do
!        if ( ibwork(i)=7 ) then
!           bwork(i-1) := bwork(i-1) and bwork(i)
!           shift bwork and ibwork
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=8 ) then
!           bwork(i-1) := bwork(i) or bwork(i-1)
!           ncount := ncount-1
!           shift bwork and ibwork
!
!     result := bwork(1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalboolend' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalboolend'
         call prinin ( ibwork, ncount, 'ibwork' )
         call prinbool ( bwork, ncount, 'bwork' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For i := 1(1)ncount do
!            remove all not expressions

100   do i = 1, ncount

         if ( ibwork(i)==9 ) then

!        --- and found
!            bwork(i-1) := bwork(i-1) and bwork(i)
!            shift bwork and ibwork

            bwork(i-1) = .not. bwork(i)
            ncount = ncount-1
            do j = i, ncount
               bwork(j) = bwork(j+1)
               ibwork(j) = ibwork(j+1)
            end do
            bwork(ncount+1) = .false.
            ibwork(ncount+1) = 0
            go to 100

         end if  ! ( ibwork(i)==9 )

      end do  ! i = 1, ncount

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'After evaluation of not'
         call prinin ( ibwork, ncount, 'ibwork' )
         call prinbool ( bwork, ncount, 'bwork' )

      end if  ! ( debug )

!     --- For i := 1(1)ncount do
!            remove all and expressions

150   do i = 1, ncount

         if ( ibwork(i)==7 ) then

!        --- and found
!            bwork(i-1) := bwork(i-1) and bwork(i)
!            shift bwork and ibwork

            bwork(i-1) = bwork(i-1) .and. bwork(i)
            ncount = ncount-1
            do j = i, ncount
               bwork(j) = bwork(j+1)
               ibwork(j) = ibwork(j+1)
            end do
            bwork(ncount+1) = .false.
            ibwork(ncount+1) = 0
            go to 150

         end if  ! ( ibwork(i)==9 )

      end do  ! i = 1, ncount

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'After evaluation of and'
         call prinin ( ibwork, ncount, 'ibwork' )
         call prinbool ( bwork, ncount, 'bwork' )

      end if  ! ( debug )

!     --- Store final result

      prevalboolend = bwork(1)
      do i = 2, ncount
         prevalboolend = prevalboolend .or. bwork(i)
      end do  ! i = 2, ncount

1000  call erclos ( 'prevalboolend' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'prevalboolend ', prevalboolend
         write(irefwr,*) 'End prevalboolend'

      end if  ! ( debug )

      end
