      function prevalboolint ( int1, int2, boolop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate the expression int1 boolean_operator int2
!     Return with the result
! **********************************************************************
!
!                       KEYWORDS
!
!     boolean_expression
!     boolean
!     evaluate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer boolop, int1, int2
      logical prevalboolint

!     boolop         i    Value of boolean operator
!                         1: operator is <
!                         2: operator is <= &
!                         3: operator is >
!                         4: operator is >= &
!                         5: operator is /= &
!                         6: operator is == &
!     int1           i    Integer before the operation
!     int2           i    Integer after the operation
!     prevalboolint  o    Result of operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2939   Boolean operator has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalboolint' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalboolint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( boolop )

         case(1)

!        --- boolop = 1, i.e. int1<int2

            prevalboolint = int1<int2

         case(2)

!        --- boolop = 2, i.e. int1<=int2

            prevalboolint = int1<=int2

         case(3)

!        --- boolop = 3, i.e. int1>int2

            prevalboolint = int1>int2

         case(4)

!        --- boolop = 4, i.e. int1>=int2

            prevalboolint = int1>=int2

         case(5)

!        --- boolop = 5, i.e. int1/=int2

            prevalboolint = int1/=int2

         case(6)

!        --- boolop = 1, i.e. int1==int2

            prevalboolint = int1==int2

         case default

!        --- error: boolop has wrong value

            call errint ( boolop, 1 )
            call errsub ( 2939, 1, 0, 0 )

      end select  ! case ( boolop )

1000  call erclos ( 'prevalboolint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prevalboolint'

      end if  ! ( debug )

      end
