      subroutine prexecwhile ( inpsep, istart, rinsep, ioutpsav )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 18-12-2013 Start inpsep from 0
!        version  2.1  date 14-11-2013 New call to prevalboolexpr
!        version  2.0  date 21-12-2011 Extra parameter kmesh
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Execute while statement in tosepcomsub
!     Return with correct value of istart
! **********************************************************************
!
!                       KEYWORDS
!
!     compute_all
!     while_statement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleloop
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart, ioutpsav
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents
!                         1:   83 to indicate that this is an if statement
!                         2:   if >0 parameter k in call to userbool
!                              if <0, -length of boolean expression in
!                              array inpsep (no use of userbool)
!                         3:   Next address in inpsep if result of
!                              expression is false
!                         Rest of positions, informaion of boolean expression
!                         as described in readboolexpr
!     ioutpsav       i    Help parameter to store the initial value of ioutp
!     istart         i    First position used in inpsep
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical userbool, prevalboolexpr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALBOOLEXPR Evaluate boolean expression and return with result
!     USERBOOL       User subroutine corresponding to boolean_expr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prexecwhile' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prexecwhile'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpsep(1)<0 ) then

!     --- inpsep(1)<0, hence a boolean expression given
!         evaluate this expression

         if ( debug ) write(irefwr,*) 'evaluate boolean expression'
         if ( prevalboolexpr ( inpsep(3), -inpsep(1), rinsep, &
                               'while statement' ) ) then

!        --- While must be executed, go to next step

            istart = istart-inpsep(1)+3
            ioutp = -1

         else

!        --- If must not be executed, skip loop

            istart = inpsep(2)
            ioutp = ioutpsav

         end if  ! ( prevalboolexpr ( inpsep(3), istart, rinsep, ... )

      else

!     --- inpsep(1)>0, hence use userbool

         if ( userbool ( inpsep(1) ) ) then

!        --- If must be executed, go to next step

            istart = istart + 3
            ioutp = -1

         else

!        --- If must not be executed, skip loop

            istart = inpsep(2)
            ioutp = ioutpsav

         end if

      end if  ! ( inpsep(1)<0 ) then

      whileloop = .true.
      seqwhile = seqwhile+1

1000  call erclos ( 'prexecwhile' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prexecwhile'

      end if  ! ( debug )

      end
