      subroutine prfillfunc ( iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays funcx and funcy  with x-coodinates and function
!     values for a series of sepran vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulevecs
      use sepmoduleplotfn
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqvecs(numvecs)

!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isolut
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPSEPVEC2   Copy one degree of freedom of sepran vector in user vector
!     PRFILLFUNCX    Fill array funcx with arc length along set of nodes
!     SEPCOPSEPVEC   Copy one degree of freedom of sepran vector in user vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillfunc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillfunc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==1 ) then

!     --- ndim = 1, complete coor

         funcx(1:nnodes) = coor(1,1:nnodes)
         do i = 1, numvecs
            call sepcopsepvec ( isolut2d(iseqvecs(i)), &
                                idegfd, funcy(1,i) )
         end do  ! i = 1, numvecs

      else

!     --- ndim>1
!         First fill relative arc length in funcx

         call prfillfuncx ( nodes, nnodes, funcx, coor, ndim )

!        --- next fill the function values

         do i = 1, numvecs
            call prcopsepvec2 ( isolut2d(iseqvecs(i)), idegfd, &
                                funcy(1,i), nnodes )
         end do  ! i = 1, numvecs

      end if  ! ( ndim==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillfunc'

      end if  ! ( debug )
      call erclos ( 'prfillfunc' )

      end
