      subroutine prfillfuncin ( icross, numsubcurvs, nnodes, numveclc, &
                                funcx, funcy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays funcx and funcy with values that are stored in
!     array icross of type 119
!     After that remove the contents of icross from the buffer
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     fill
!     intersection
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numveclc, numsubcurvs, nnodes, &
                             icross(5,numveclc)
      double precision, intent(out) :: funcx(nnodes), &
                                       funcy(nnodes,numveclc)

!     funcx          o    User array containing the "x"-coordinates
!     funcy          o    user array containing the function values
!                         for the numveclc components
!     icross         i    Integer SEPRAN array of type 119 containing
!                         the information of the intersections
!     nnodes         i    Number of nodes
!     numsubcurvs    i    Number of subcurves
!                         At this moment only 1 subcurve is allowed
!     numveclc         i    Number of vectors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ishift, i

!     i              Counting variable
!     ishift         Shift in array corresponding to icross to get
!                    the function value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillfuncin' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillfuncin'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address and shift

      usoltemp => ks(icross(1,1))%sol

!     --- Fill array funcx

      funcx = usoltemp(1:nnodes)

!     --- Fill array funcy

      do i = 1, numveclc

!     --- Copy function values

         funcy(:,i) = usoltemp(nnodes+1:)

!        --- Remove arrays icross

         call seprmsolinfo ( icross(1,i) )

         if ( i<numveclc ) then
            usoltemp => ks(icross(1,i+1))%sol
         end if  ! ( i<numveclc )

      end do  ! i = 1, numveclc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillfuncin'

      end if  ! ( debug )
      call erclos ( 'prfillfuncin' )

      end subroutine prfillfuncin
