      subroutine prfilltextint ( textaxis, inpsep, ivectr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array textaxis with text to be plotted along the axis
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(4), ivectr
      character (len=*) :: textaxis(2)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotint
!     ivectr         i    sequence number of vector in isol
!     textaxis       o    Contains the text to be plotted along the axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itextx, itexty

!     itextx         Position of x-text in text array in common csepcm
!     itexty         Position of y-text in text array in common csepcm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilltextint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilltextint'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itextx = inpsep(1)  ! start of textaxis(1)
      ntextx = inpsep(2)  ! number of characters of textaxis(1)
      itexty = inpsep(3)  ! start of textaxis(2)
      ntexty = inpsep(4)  ! number of characters of textaxis(2)

      if ( itextx>0 ) then

!     --- textaxis(1) found

         textaxis(1) = texts(itextx)

      else

!     --- textaxis(1) not found, use default value

         textaxis(1)  = 's'
         ntextx = 1

      end if  ! ( itextx>0 )
      if ( debug ) write(irefwr,*) 'textaxis(1) ', textaxis(1)(1:ntextx)

      if ( itexty>0 ) then

!     --- textaxis(2) found

         textaxis(2) = texts(itexty)

      else

!     --- textaxis(2) not found, use default value (name of vector)

         textaxis(2)  = names(ivectr)
         ntexty = index(names(ivectr),' ')-1
         if ( ntexty<=0 ) ntexty = len(names(ivectr))

      end if  ! ( itextx>0 )

1000  call erclos ( 'prfilltextint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ntextx, ntexty', ntextx, ntexty
         write(irefwr,*) 'textx ', textaxis(1)
         write(irefwr,*) 'texty ', textaxis(2)
         write(irefwr,*) 'End prfilltextint'

      end if  ! ( debug )

      end subroutine prfilltextint
