      subroutine prfilluserdef ( iuser, inpsep, nelgrp, inpvol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-12-2014 Extension with COORDINATE_SYSTEM
!        version  1.0  date 02-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iuser and user with default values for integration
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), inpsep(*), nelgrp, inpvol(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscintegral
!     inpvol        i/o   input array for subroutine INTEGSBF
!     iuser          o    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, istart, ivectr

!     ielgrp         element group number
!     istart         First position used at a part
!     ivectr         standard sepran array denoting USOL or vector of
!                    special structure defined in nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilluserdef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilluserdef'
         write(irefwr,1) 'nelgrp', nelgrp
         call prinin ( inpsep,13, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivectr = inpsep(4)

      if ( inpsep(8)==0 ) then

!     --- icheli not given

         if ( ivectr==0 ) then

!        --- No vector given, icheli = 1

            inpvol(2) = 1

         else

!        --- Vector given, icheli = 2

            inpvol(2) = 2

         end if  ! ( ivectr==0 )

      end if  ! ( inpsep(8)==0 )

!     --- Fill default values

      istart = 6+nelgrp
      iuser(2:5) = 0
      do ielgrp = 1, nelgrp
         iuser(5+ielgrp) = istart
         iuser(istart) = inpsep(12)
         iuser(istart+1) = inpsep(13)
         iuser(istart+2) = 0
         iuser(istart+3) = 0
         istart = istart+4
      end do  ! ielgrp = 1, nelgrp

      iuser(1) = istart-1
      iuser(3:4) = istart-1
      iuser(2) = nelgrp

1000  call erclos ( 'prfilluserdef' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iuser, istart-1, 'iuser' )
         write(irefwr,*) 'End prfilluserdef'

      end if  ! ( debug )

      end
