      subroutine printerpolate ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.3  date 30-12-2013 Extra debug
!        version  1.2  date 14-04-2013 Loop in case of series
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate solution from one mesh to the other
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine readscinterpol
!     istart        i/o   At input first entry to be used in inpsep
!                         At output next entry to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecseq, meshin, meshout, nch1, numarray, iseqarrsav, &
              iseqarrmin, iseqarrmax, iseqkm1

!                    they are not
!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqkm1        Sequence number with respect to km of first mesh
!     ivecseq        Sequence number of vector to be considered with respect
!                    to the solution array (isol or isolut3d)
!     meshin         Sequence number of first mesh to be used
!     meshout        Sequence number of last mesh to be used
!     nch1           Number of characters in name
!     numarray       Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINMAP        Print the contents of the mapping array map
!     PRINTERPOLATE1 Interpolate solution from one mesh to the other
!     PRINVERSEMAP   Fill reverse mapping array in case mapping array is given
!     SEPCOPYVC      Copy one SEPRAN vector into another one
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerpolate' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerpolate'
         call ertrace
         call prinin ( inpsep, 3, 'inpsep' )
         call prinin ( meshseqs, 3, 'meshseqs' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         call inireducevec ( abs(inpsep(1)), ivecseq, numarray )
         iseqarrmin = 1
         iseqarrmax = numarray

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray

      end if  ! ( iseqarray==0 )

      if ( debug ) &
         write(irefwr,1) 'iseqarrmin, iseqarrmax',iseqarrmin, iseqarrmax

      meshin  = abs(meshseqs(inirepscal(inpsep(2))))  ! input mesh
      meshout = abs(meshseqs(inirepscal(inpsep(3))))  ! output mesh
      iseqkm1 = kmesh2d(meshin)

      if ( debug .or. ioutp>=0 ) then

!     --- debug information or write information

         nch1  = index(names(ivecseq),' ')-1
         if ( nch1<=0 ) nch1 = len(names(ivecseq))
         write(irefwr,100) names(ivecseq)(1:nch1), meshin, meshout
100      format ( ' Interpolate vector ', a, ' from mesh ', i1, &
                  ' to mesh ', i1 )

      end if  ! ( debug .or. ioutp>=0 )

      if ( debug ) then

!     --- debug information

         write(irefwr,1) 'meshin, meshout, ivecseq', &
                          meshin, meshout, ivecseq
         write(irefwr,1) 'km(iseqkm1)%kmesh(42)', km(iseqkm1)%kmesh(42)
         call prinin ( iinmap, 5, 'iinmap' )

      end if  ! ( debug )

      if ( map(meshin,meshout)%equal ) then

!     --- map(2,.,.) = -130, i.e. both meshes are equal
!         Copy arrays instead of interpolating

         if ( debug ) write(irefwr,*) 'meshes are identical'
         do iseqarray = iseqarrmin, iseqarrmax

            call sepcopyvc ( isolut3d(ivecseq,meshin), &
                             isolut3d(ivecseq,meshout) )
            ivecseq = ivecseq+1
         end do  ! iseqarray = iseqarrmin, iseqarrmax

         go to 900

      else if ( km(iseqkm1)%kmesh(42)>0 .and. &
                .not. associated(map(meshin,meshout)%imap) .and. &
                associated(map(meshout,meshin)%imap) ) then

!     --- inverse mapping exists, new mapping does not, create new mapping
!         by using the already existing inverse mapping

         if ( debug ) &
            call prinmap ( map(meshout,meshin), &
                           kmesh2d(meshin) )
         call prinversemap ( map(meshout,meshin), &
                             map(meshin,meshout), kmesh2d(meshin), &
                             kmesh2d(meshout) )
         iinmap(2) = 2   ! array map has now been filled
         iinmap(4) = meshin
         iinmap(5) = meshout

      else if ( iinmap(2)==2 ) then

!     --- Check if meshin and meshout correspond to previous map
!         If not it must be filled again

         if ( iinmap(4)/=meshin .or. iinmap(5)/=meshout ) iinmap(2) = 1
         if ( .not. associated(map(meshin,meshout)%rmap) ) iinmap(2) = 1

      end if  ! ( map(meshin,meshout)==0 )

      do iseqarray = iseqarrmin, iseqarrmax

         call printerpolate1 ( meshin, meshout, ivecseq )
         ivecseq = ivecseq+1

      end do  ! iseqarray = iseqarrmin, iseqarrmax

      iinmap(2) = 2
      iinmap(4) = meshin
      iinmap(5) = meshout

900   istart = istart+4  ! raise istart

      iseqarray = iseqarrsav  ! reset iseqarray

1000  if ( debug ) then

!     --- Debug information

   !      call prinin ( kprob2d(meshout), 40, 'kprob2d(meshout)' )
         ivecseq = ivecseq-1
         call prinin ( isolut3d(ivecseq,meshin), 5, &
                       'isolut3d(ivecseq,meshin)' )
         call prinin ( isolut3d(ivecseq,meshout), 5, &
                       'isolut3d(ivecseq,meshout)' )
         write(irefwr,*) 'End printerpolate'

      end if  ! ( debug )
      call erclos ( 'printerpolate' )

      end subroutine printerpolate
