      subroutine printersect ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Prepare input for intersection subroutine
!     Store result in array scalars
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     curve
!     line
!     2d
!     fill_scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents:
!                         1:  intlinedef  Type of storage of line
!                         2:  iscal       Sequence number of scalar
!                         3:  icurnr      Curve sequence number
!                         4:  istrin      Starting address in RINSEP
!     rinsep         i    Array containing the real input for this subroutine
!                         Contents:
!                         1: x_origin
!                         2: y_origin
!                         3: angle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intlinedef, iscal, icurnr, istrin
      double precision coors(3)

!     coors          Array containing the coordinates and distance
!                    of the intersection from origin
!                    they are not
!     icurnr         Curve sequence number
!     intlinedef     Defines the way the line is given in reallinedef
!     iscal          Sequence number of scalar
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHINTCURVELIN intersect curve icurnr with a straight line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printersect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printersect'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Get constants from inpsep

      intlinedef = inpsep(1)
      iscal  = inpsep(2)
      icurnr = inpsep(3)
      istrin = inpsep(4)

!     --- Compute intersection, store in coors

      call mshintcurvelin ( icurnr, intlinedef, rinsep(istrin), coors )

!     --- Store intersection in scalars and adapt numscals

      scalars(iscal) = coors(1)
      numscals = max( numscals, iscal )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printersect'

      end if
      call erclos ( 'printersect' )

      end subroutine printersect
