      subroutine prlineint ( inpsep, istart, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of line with 2d mesh and compute the
!     corresponding line integral
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     vector
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecons
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      integer, intent(inout) :: istart
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscintersint
!     istart        i/o   Next position in array inpsep
!                         istart is raised in this subroutine
!     rinsep         i    The real input read is stored in array RINSEP
!                         See subroutine readscintersint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivectr, i, iscal
      double precision :: result, values(3)

!                    they are not
!     i              Loop variable
!     iscal          Sequence number of scalar
!     ivectr         standard sepran array denoting USOL or vector of
!                    special structure defined in nodal points
!     result         result of the computation
!     values         Double precision or complex array in which the real
!                    or complex information concerning the line is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivectr
      double precision :: getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INIVECTR       Return with sequence number of vector
!     PRININ         print 1d integer array
!     TOINTERSECINT  Compute intersection of line with 2d mesh and compute the
!                    corresponding line integral
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlineint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlineint'
         call prinin ( inpsep, 4, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      ivectr = inivectr(inpsep(1))   ! sequence number of vector in isolut
      iscal = inpsep(3)              ! position in scalar array
      do i = 1, 3
         values(i) = getparm ( inpsep, rinsep, i+4, 'values' )
      end do  ! i = 1, 3

      isolut => isolut2d(ivectr)
      call tointersecint ( inpsep, values, result, rinsep )

      scalars(iscal) = result

!     --- Raise istart

      istart = istart+11

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlineint'

      end if  ! ( debug )
      call erclos ( 'prlineint' )

      end
