      subroutine prlins ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 01-09-2016 New parameter list
!        version  4.0  date 01-03-2014 Extra parameter stiffm
!        version  3.5  date 06-02-2014 Use max(0,numvecmn)
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call subroutine LINPRB to solve a linear problem
!     Fill array IINSOL with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine READSCLINSOL
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpsol(20), ivectr

!                    they are not
!     inpsol         Input array defined by the user
!                    With this array the user may indicate which actions
!                    he wants to be performed by LINPRB
!                    See subroutine LINPRBBF
!     ivectr         Sequence number of first vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPLINBUILDSOL Build system of equations and solves linear system of
!                    equations
!     SEPLINSINP     Fill array inpsol with information for building the matrix
!                    as well as for the linear solver
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('prlins')
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlins'
         write(irefwr,1) 'numvecmn', numvecmn
         call prinin ( inpsep, 13, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array inpsol with information for the linear solver

      call seplinsinp ( inpsol, inpsep )

!     --- if inpsep(14)>0, transform boundary conditions from Cartesian
!         to local coordinates

      if ( inpsep(14)>0 ) call seploctrn ( 1, isolut )

!     --- Actual routine

      iprob = inpsol(5)
      ivectr = inpsol(6)

      call seplinbuildsol ( inpsol )
      numvecmn = max(numvecmn,ivectr,inpsol(10), inpsol(11),inpsol(18))

      istartmn = istartmn+15    ! raise istartmn

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsol, 19, 'inpsol' )
         write(irefwr,*) 'End prlins'

      end if  ! ( debug )
      call erclos('prlins')

      end
