      subroutine prmanvsub ( ivectr, ivec2, ivec3, inpsep, rinvec, &
                             ichoice, iscal1, iscal2, iinvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 04-09-2016 Use sepmanmap
!        version  1.5  date 12-02-2015 New call to prmakevector
!        version  1.3  date 17-02-2014 Adapt numvecmn
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Vector/scalar manipulation subroutine
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, ivec2, ivec3, inpsep(*), &
              ichoice, iscal1, iscal2, iinvec(*)
      double precision rinvec(*)

!     ichoice        i    Integer choice parameter, see prmanv
!     iinvec         i    User input array for subroutine MANVEC
!                         For a description see subroutine MANVECBF
!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine ASKVEC
!     iscal1         i    Sequence number of first scalar
!     iscal2         i    Sequence number of second scalar
!     ivec2          i    Sequence number of second vector
!     ivec3          i    Sequence number of third vector
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     rinvec         i    Real user input array for subroutine MANVEC
!                         The contents depend on iinvec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: prnode
      double precision :: prmeanarea, prminarea, prmeanval, prstdev

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSTRESSVEC    Compute the normal stress vector along a set of curves and
!                    store in a solution vector
!     PRMAKESIGNVEC  Create new vector V by either taking the sign of the input
!                    vector or multiplication of the sign of input vector 1
!                    by input vector 2
!     PRMAKEVECTOR   Make new vector out of two old vectors
!     PRMEANAREA     Compute the mean area of all elements
!     PRMEANVAL      Find mean value of vector as defined by inpsep
!     PRMINAREA      Compute the minimum area of all elements
!     PRNODE         Find node number as defined by inpsep
!     PRREACNODE     Replace the integrated reaction force by the nodal force
!     PRREACNODES    Replace the integrated reaction force by the nodal force
!     PRSETARRCONS   Set the values of an existing vector equal to a constant
!     PRSTDEV        Find standard deviation of vector as defined by inpsep
!     PRTRANSFVECT   Transform vector to other coordinate system
!     PRTRANSNORM    Compute the normal vector along a set of curves and store
!                    in a solution vector
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     sepcreasolbf1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPMANMAP      Map vector of one structure onto a vector of another
!                    structure
!     SEPMANVECS     Vector/scalar manipulation subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmanvsub' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmanvsub'
         write(irefwr,1) 'ichoice', ichoice
         write(irefwr,1) 'ivectr, ivec2', ivectr, ivec2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Some special options not standard manvec

      select case ( ichoice )

      case(51)

!     --- Map vector to other structure

         call sepmanmap ( inpsep, isolut2d(ivectr), isolut2d(ivec2), &
                          iinvec )

      case(101)

!     --- Compute mesh velocity

         coornew1d => ks(isolut2d(ivec2))%sol
         coorold1d => ks(isolut2d(ivec3))%sol
         call sepcreasolbf1 ( isolut2d(ivectr), &
                              ks(isolut2d(ivec2))%typevec, &
                              ks(isolut2d(ivec2))%ivec, &
                              'mesh_veloc', 0 )
         meshvel => usoltemp
         meshvel = (coornew1d-coorold1d)/tstep
         numvecmn = max(numvecmn,ivectr)

      case(102)

!     --- Make vector constant

         call prsetarrcons ( isolut2d(ivectr), rinvec(1), inpsep )
         numvecmn = max(numvecmn,ivectr)

      case(103)

!     --- Transform vector from integrated to nodal

         if ( inpsep(7)==0 ) then

!        --- ispec = 0, all outer boundaries

            call prreacnode ( isolut2d, ivec2, ivectr )

         else

!        --- ispec = 1, zero_level_set i

            if ( ivectr/=ivec2 ) & ! Copy ivec2 into ivectr
               call  sepcopyvc ( isolut2d(ivec2), isolut2d(ivectr) )
            call prreacnodes ( isolut2d, ivectr )
            numvecmn = max(numvecmn,ivectr)

         end if  ! ( inpsep(5)==0 )

      case(104)

!     --- Transform vector along boundary from Cartesian to normal
!         and tangential components along curves

         call prtransnorm ( inpsep )

      case(105)

!     --- Transform stress tensor along boundary from Cartesian to vector
!         using the normal components along curves

         call erstressvec ( inpsep )

      case(106)

!     --- Make new vector out of two old vectors
!         The components of the first vector are the first ones, &
!         the components of the second vector are the last

         call prmakevector ( isolut2d, iinvec, numvecmn, ivectr )
         numvecmn = max(numvecmn,ivectr)

      case(107, 108)

!     --- Either V1 = sign(V2) or V1 = sign(V2) * V3
!                V1 = V2 * V3  or V1 = V2 / V3

         call prmakesignvec ( inpsep, numvecmn )
         numvecmn = max(numvecmn,ivectr)

      case(109,117)

!     --- Scalar is mean area

         scalars(iscal1) = prmeanarea ( ichoice )

      case(110)

!     --- Scalar is min area

         scalars(iscal1) = prminarea()

      case(111)

!     --- Point_number

         scalars(iscal1) = prnode ( inpsep )

      case(112)

!     --- Mean_value

         scalars(iscal1) = prmeanval ( iinvec )

      case(113)

!     --- Standard deviation

         scalars(iscal1) = prstdev ( iinvec )

      case(114)

!     --- Transform vector to other coordinate system

         iinvec(13) = ivectr
         call prtransfvect ( iinvec )

      case default

!     --- Standard case, call sepmanvec

         call sepmanvecs ( ivectr, ivec2, ivec3, inpsep, rinvec, &
                           ichoice, iscal1, iscal2, iinvec )

      end select  ! case ( ichoice )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmanvsub'

      end if  ! ( debug )
      call erclos ( 'prmanvsub' )

      end subroutine prmanvsub
