      subroutine prmoveobstac1 ( iobst, uvel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-01-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Move obstacle over a distance u
!    The positions 1 to ndim*2 of rmeshx are adapted
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     displacement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst
      double precision uvel(ndim)

!     iobst          i    Obstacle sequence number
!     uvel           i    Contains the displacement vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, icurve, nsubcv, j, istart, nnodes, k

!     i              Counting variable
!     icurnr         Curve sequence number
!     icurve         Curve sequence number
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     nnodes         Number of nodes
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Update rmeshx
!     icurnr := curve number of obstacle
!     Change coordinates of curve
!     If curve is a composite curve, also change coordinates of subcurves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Update rmeshx

      rmeshx(1,iobst) = rmeshx(1,iobst)+uvel(1)
      rmeshx(2,iobst) = rmeshx(2,iobst)+uvel(1)
      rmeshx(3,iobst) = rmeshx(3,iobst)+uvel(2)
      rmeshx(4,iobst) = rmeshx(4,iobst)+uvel(2)
      if ( ndim==3 ) then
         rmeshx(5,iobst) = rmeshx(5,iobst)+uvel(3)
         rmeshx(6,iobst) = rmeshx(6,iobst)+uvel(3)
      end if

!     --- icurnr := curve number of obstacle

      icurnr = kmeshx(2,iobst)

!     --- Change coordinates of curve

      if ( icurnr==1 ) then
         istart = 1
         nnodes = icurvsmsh(1)
      else
         istart = icurvsmsh(icurnr-1)+1
         nnodes = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)
      end if
      do i = 0, nnodes-1
         do j = 1, ndim
            curvesmsh(j,istart+i) = curvesmsh(j,istart+i)+uvel(j)
         end do
      end do

!     --- If curve is a composite curve, also change coordinates of subcurves

      if ( kelmq/=0 ) then

!     --- kelmq > 0, there are  composite curves

         nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         if ( nsubcv>0 ) then

!        --- Curve contains subcurves

            do k = 1, nsubcv
               icurve = abs(kmeshq(kmeshq(icurnr)+k))
               if ( icurve==1 ) then
                  istart = 1
                  nnodes = icurvsmsh(1)
               else
                  istart = icurvsmsh(icurve-1)+1
                  nnodes = icurvsmsh(icurve)-icurvsmsh(icurve-1)
               end if
               do i = 0, nnodes-1
                  do j = 1, ndim
                     curvesmsh(j,istart+i) = curvesmsh(j,istart+i)+uvel(j)
                  end do
               end do
            end do

         end if

      end if

      end
