      subroutine prnewprob ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.2  date 25-11-2012 Long integers
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call subroutine PROBDFEXT to define a new problem definition
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleinput
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: inpsep(*), istart

!     inpsep         i    Sequence number of input for PROBDF
!     istart        i/o   Starting address in array INPVEC
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichprb, iseqprob, ipos, maxcal
      double precision rinput(5)

!     ichprb         Choice parameter for subroutine PROBDF
!     ipos           Position of input arrays in sepmoduleinput
!     iseqprob       Sequence number of problem input
!     maxcal         Maximum number of coefficient inputs read
!     rinput         Real input array for PROBDFEXT
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     PROBDFEXT      Reads and stores problem definition
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get memory management sequence numbers of input
!     Put arrays at the back of the buffer array
!     Call probdfext
!     adapt intmat
!     Clear end of buffer
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prnewprob' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prnewprob'
         write(irefwr,1) 'inpsep(1)', inpsep(1)
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ichprb = 1

!     --- Get memory management sequence numbers of input

      call sepgetinpfinf ( 27, inpsep(1), ipos, maxcal )
      if ( ierror/=0 ) go to 1000
      iinputtmp => in(ipos)%iinput(6:)
      nprob = in(ipos)%iinput(5)

      if ( associated(re(ipos)%rinput) ) then
         rinputtmp => re(ipos)%rinput(6:)
      else
         allocate(rinputtmp(1))
      end if  ! ( associated(re(ipos)%rinput) )

      removesol = .false.
      call probdfext ( ichprb, iseqprob )
      removesol = .true.
      if ( ierror/=0 ) go to 1000

!     --- adapt intmat

      if ( inpsep(2)==0 ) call incomgen ( -1, intmat1d )

      istart = istart+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prnewprob'

      end if
      call erclos ( 'prnewprob' )

      end
