      subroutine prplotfn1 ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot function along curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     plot_function
!     driving_subroutine
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulevecs
      use sepmoduleplotfn
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotfn
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, numcurves

!                    they are not
!     i              Counting variable
!     numcurves      Number of curves in array inpsep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFILLFUNC     Fill arrays funcx and funcy  with x-coodinates and function
!                    values for a series of sepran vectors
!     PRFILLTEXT     Fill array loctext with the text to be used along axis
!     PRININ         print 1d integer array
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3188   Set of curves consists of more than 1 subparts
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotfn1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotfn1'
         call prinin ( inpsep, 15, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store functions in funcx, funcy

      if ( ndim==1 ) then

!     --- One dimensional case, use all nodes

         nnodes = npoint
         numcurves = 1
         istartcur = 1

      else

!     --- More dimensional case, use nodes in curves

         allocate ( nodes(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
         if ( ierror/=0 ) go to 1000

         numcurves = inpsep(4)
         istartcur = inpsep(5)
         call sepfillnodes ( 2, numcurves, inpsep(istartcur), nodes, &
                             nnodes, ipoint, nparts )

         if ( nparts>1 ) then

!        --- More than one subparts

            do i = 1, numcurves
               call errint ( inpsep(istartcur+i-1), i )
            end do  ! i = 1, numcurves
            call errsub ( 3188, numcurves, 0, 0 )
            go to 1000

         end if  ! ( nparts>1 )

      end if  ! ( ndim==1 )

!     --- Allocate arrays funcx and funcy

      allocate ( funcx(nnodes), funcy(nnodes,numvecs), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nnodes*(numvecs+1), 'funcx' )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays funcx and funcy

      call prfillfunc ( inpsep(istartvecs) )

      if ( ndim>1 ) then

!     --- Deallocate array nodes

         deallocate ( nodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nodes' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( ndim>1 )

!     --- Fill text along axis

      call prfilltext ( textaxis, kmesh, inpsep(6), inpsep(istartcur), &
                        numcurves, numvecs, inpsep(istartvecs) )

1000  call erclos ( 'prplotfn1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotfn1'

      end if  ! ( debug )

      end
