      subroutine prplotmesh ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 15-11-2013 Extension with curve plot info
!        version  1.4  date 21-11-2012 Extra debug
!        version  1.3  date 12-04-2011 Extra options
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the mesh as stored in kmesh
!     Use information stored in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotmesh for a description
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscplotmesh for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iplots(21), i, iregion, jmaxsave
      double precision rplots(8)

!                    they are not
!     i              Counting variable
!     iplots         Integer input array for plot subroutine
!     iregion        indicates if the complete region must be plotted
!                    (0) or that the plotting is restricted to a sub region
!                    defined by the parameters xmin, xmax, ymin, ymax, zmin
!                    zmax in the common block CPLOT.
!                    Possible values:
!                    0:  The complete region is plotted
!                    1:  The region is restricted to xmin<=x<=xmax, &
!                        ymin<=y<=ymax, zmin<=z<=zmax
!     jmaxsave       Help variable to save the value of jmax
!     rplots         Real input array for plot subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPPLOTMS      plot mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iplots
!     Fill array rplots
!     Fill parameters corresponding to region
!     Plot mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotmesh'
         call prinin ( inpsep, 7, 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill array iplots

      iplots = 0
      iplots(1) = 21

!     --- Clear array rplots

      rplots = 0d0

      iregion = inpsep(1)

      if ( debug ) write(irefwr,1) 'iregion', iregion

!     --- Fill array rplots

      rplots(1) = 15d0                          ! plotfm
      if ( inpsep(2)==0 ) then
         rplots(2) = 1d0                        ! yfact
      else
         rplots(2) = rinsep(iregion)
      end if

!     --- Fill parameters corresponding to region

      if ( iregion>0 ) then

!     --- iregion>0, xmin, ..., zmax are given

         jmaxsave = jmax
         jmax = 1
         xmin = rinsep(iregion)
         xmax = rinsep(iregion+1)
         ymin = rinsep(iregion+2)
         ymax = rinsep(iregion+3)

         if ( debug ) &
            write(irefwr,2) 'xmin, xmax, ymin, ymax', &
                             xmin, xmax, ymin, ymax

         if ( ndim>2 ) then
            zmin = rinsep(iregion+4)
            zmax = rinsep(iregion+5)
         end if  ! ( ndim>2 )

      end if  ! ( iregion>0 )

      if ( inpsep(3)>0 ) then

!     --- inpsep(3)>0, hence eye-point given

         do i = 0, 2
            rplots(3+i) = rinsep(inpsep(3)+i)
         end do  ! i = 0, 2

      end if  ! ( inpsep(3)>0 )

      iplots(14) = inpsep(4)   ! orientation (3d)
      iplots(21) = inpsep(8)   ! plotboun

      iplots(2:4) = inpsep(5:7)  ! mark nodes/elements

      if ( debug ) then
         call prinin ( iplots, 21, 'iplots' )
         call prinrl ( rplots, 8, 'rplots' )
      end if  ! ( debug )

!     --- Plot mesh

      call sepplotms ( iplots, rplots, iskip )
      if ( iregion==1 ) jmax = jmaxsave

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotmesh'

      end if
      call erclos ( 'prplotmesh' )

      end subroutine prplotmesh
