      function prstdevval20 ( usol, nodes, nnodes, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-09-2012 Allow nnodes = 0
!        version  1.0  date 16-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find standard deviation of vector
!     nunkp degrees of freedom, selection of nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     standard_deviation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), nunkp
      double precision usol(nunkp,*), prstdevval20

!     nnodes         i    Number of nodes
!     nodes          i    Contains the node numbers that must be used
!     nunkp          i    Maximum number of unknowns in a point
!     prstdevval20   o    standard deviation
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart
      double precision meanval
      double precision, allocatable :: diff(:)

!     diff           Difference vector
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     istart         First position used at a part
!     meanval        mean value of usol in nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstdevval20' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstdevval20'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nnodes==0 ) then

!     --- No nodes

         prstdevval20 = 0d0
         go to 1000

      end if  ! ( nnodes==0 )

!     --- All degrees of freedom

      allocate ( diff(nunkp*nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, nnodes, 'diff' )
      if ( ierror/=0 ) go to 1000
      meanval = sum(usol(1:nunkp,nodes(1:nnodes)))/nnodes*nunkp

      istart = 0
      do i = 1, nnodes
         diff(istart+1:istart+nunkp) = usol(1:nunkp,nodes(i))-meanval
         istart = istart+nunkp
      end do  ! i = 1, nnodes

      prstdevval20 = sqrt(dot_product(diff,diff)/nnodes)

      deallocate ( diff, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'diff' )

1000  call erclos ( 'prstdevval20' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstdevval20'

      end if  ! ( debug )

      end
