      subroutine prstoccompvel1 ( numnodes, iseqdiff, iseqvel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 23-12-2011 replace mmnodes by nodes
!        version  1.0  date 16-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute diff/velocity and store in velocity at free interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     free_surface
!     stochastic
!     investment
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, iseqdiff, iseqvel

!     iseqdiff       i    sequence number of difference vector in isolut2d
!     iseqvel        i    sequence number of velocity in isolut2d
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: kprobf0(:)

!     kprobf0        kprob part f corresponding to iseqdiff
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSTOCCOMPVEL2 Compute diff/velocity and store in velocity at free
!                    interface
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstoccompvel1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstoccompvel1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      call sepactsolbf2 ( isolut2d(iseqdiff) )
      kprobf0 => kprobfi
      call sepactsolbf2 ( isolut2d(iseqvel) )

!     --- compute starting addresses

      velocity => ks(isolut2d(iseqvel))%sol
      usoltemp => ks(isolut2d(iseqdiff))%sol

!     --- Actual call

      call prstoccompvel2 ( nodes, numnodes, kprobf0, &
                            kprobfi, velocity, usoltemp, indprfi )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstoccompvel1'

      end if  ! ( debug )
      call erclos ( 'prstoccompvel1' )

      end subroutine prstoccompvel1
