      subroutine prstoccompvel2 ( nodes, numnodes, kprobfw0, kprobfw1, &
                                  velocity, diff, indprfw1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute diff/velocity and store in velocity at free interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     free_surface
!     stochastic
!     investment
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), numnodes, kprobfw0(*), kprobfw1(*), &
              indprfw1
      double precision velocity(*), diff(*)

!     diff           i    Contains the difference vector
!     indprfw1       i    Indicates if the number of unknowns in vel is constant
!                         0 or not (1)
!     kprobfw0       i    kprobf corresponding to diff
!     kprobfw1       i    kprobf corresponding to velocity
!     nodes          i    Contains the node numbers on the free surface
!     numnodes       i    Number of nodal points in array nodes
!     velocity      i/o   velocity vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, iposw0, iposw1, node
      double precision di, uvel

!     di             difference in one point
!     i              Counting variable
!     iposw0         Position of node in array diff
!     iposw1         Position of node in array velocity
!     k              Counting variable
!     node           Node number
!     uvel           Component of the velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstoccompvel2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstoccompvel2'
         write(irefwr,1) 'indprfw1', indprfw1
         call prinin ( nodes, numnodes, 'nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numnodes

         node = nodes(i)                ! node number
         iposw0 = kprobfw0(node)+1      ! position in w0 and diff vector
         di = diff(iposw0)
         if ( indprfw1>0 ) then

!        --- Non-constant number of degrees of freedom in w1

            iposw1 = kprobfw1(node)     ! position in w1 if indprfw1>0

         else

!        --- Constant number of degrees of freedom in w1

            iposw1 = node*2-2           ! position in w1 if indprfw1=0

         end if  ! ( indprfw1>0 )

         do k = 1, 2

!        --- x_k component

            iposw1 = iposw1+1
            uvel = velocity(iposw1)

            if ( debug ) then
               write(irefwr,1) 'k, iposw1', k, iposw1
               write(irefwr,2) 'uvel, di', uvel, di
            end if  ! ( debug )

            if ( abs(uvel)<=1d-20 ) then
               velocity(iposw1) = -di
            else
               velocity(iposw1) = -di/uvel
            end if  ! ( abs(uvel<=1d-20 )

         end do  ! k = 1, 2

      end do  ! i = 1, numnodes

1000  call erclos ( 'prstoccompvel2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstoccompvel2'

      end if  ! ( debug )

      end
