      subroutine prtimehis3 ( ipoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 16-04-2014 Allow more degrees of freedom
!        version  1.1  date 14-10-2012 Extra debug
!        version  1.0  date 17-05-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one time step in time history array for one specific
!     solution array
!     Mark that istephis must have been raised before the call of this
!     subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduletimhis
      use sepmoduletime
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipoints(npoint_hist)

!     ipoints        i    Contains the nodal points where the time history
!                         must be stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, inunkp

!     i              Counting variable
!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in a node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehis3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehis3'
         call prinin ( ipoints, npoint_hist, 'ipoints' )
         write(irefwr,1) 'indprfi, nphysi', indprfi, nphysi
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ipoints(1)==0 ) go to 1000  ! jump in case no nodes

      if ( istephis>1 ) then

!     --- istephis>1, check if t is equal to previous value
!         if so overwrite previous time step

         if ( abs(rvectim(1,istephis)-t)<=epsmac*t ) &
            istephis = istephis-1

      end if  ! ( istephis>1 )

!     --- Store time step

      rvectim(1,istephis) = t

!     --- Store solution

      if ( indprfi==0 ) then

!     --- indprfi = 0, hence constant number of degrees of freedom

         do i = 1, npoint_hist
            inode = ipoints(i)
            rvectim(2+(i-1)*nphysi:1+i*nphysi,istephis) = &
               vectin((inode-1)*nphysi+1:inode*nphysi)
         end do  ! i = 1, npoint_hist

      else

!     --- indprfi>0, hence variable number of degrees of freedom
!         if inunkp< nphysi fill rest with zeros

         do i = 1, npoint_hist
            inode = ipoints(i)
            inunkp = kprobfi(inode+1)-kprobfi(inode)
            rvectim(2+(i-1)*nphysi:1+(i-1)*nphysi+inunkp,istephis) = &
               vectin(kprobfi(inode)+1:kprobfi(inode)+inunkp)
            if ( inunkp<nphysi ) &
               rvectim(2+(i-1)*nphysi+inunkp:1+i*nphysi,istephis) = 0d0
         end do  ! i = 1, npoint_hist

      end if  ! ( indprfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'rvectim', rvectim(1:npoint_hist+1,1)
         write(irefwr,*) 'End prtimehis3'

      end if  ! ( debug )
      call erclos ( 'prtimehis3' )

      end
