      subroutine pruser ( inpsep )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.2  date 16-09-2011 Correction inpcop(1)
!        version  2.1  date 04-06-2008 Debug information
!        version  2.0  date 08-06-2005 Extra parameter intmat
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Prepare output for userout or userouts subroutine.
!    Scalars are filled in as double precision numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         1  ICODE      If 7 then print scalar else print vector
!                         2  ISEQNR     Sequence number of scalar or vector
!                         3  itext      Text sequence number
!                         4  nchar      Number of characters
!                         If icode=15, position 2 is deleted
!                         If icode=9 The following extra postions are used:
!                         5  istrin (Starting address in rinsep)
!                         6  ichoice (inppri(2), see prinvc)
!                         7  idim    (inppri(3), see prinvc)
!                         8  iquant  (inppri(4), see prinvc)
!                         9  idegfd  (inppri(5), see prinvc)
!                         10 iseq    (inppri(6), see prinvc)
!                         11 iregion (inppri(7), see prinvc)
!                         12 iheader (inppri(8), see prinvc)
!                         13 nread   (number of curves, surfaces etc)
!                            If nread>0 followed by nread curves etc.
!                         If iregion#0, 6 positions in rinsep are used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpcop(1003)
      double precision scals(maxcls)

!                    they are not
!     i              Counting variable
!     inpcop         Copy of integer inpsep array, with positin 3 removed
!     scals          Array filled with the values of requested scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     USEROUT        User output subroutine
!     USEROUTS       User output subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check length of array inpcop
!     Copy inpsep into inpcop, skip position 3
!     if ( inpsep(3)=0 ) then
!        call userout
!     else ( inpsep(3)>0 )
!        call userouts
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pruser' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pruser'
         call prinin ( inpsep, 3+inpsep(3)+inpsep(2), 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check length of array inpcop

      call inichkarlen ( inpsep(3)+inpsep(2), 1000, 'inpcop' )
      if ( ierror/=0 ) go to 1000

!     --- Copy inpsep into inpcop, skip position 3

      inpcop(1) = max(inpsep(1),1)
      inpcop(2) = inpsep(2)
      do i = 1, inpsep(3)+inpsep(2)
         inpcop(i+2) = inpsep(3+i)
      end do
      if ( inpsep(3)==0 ) then

!     --- no scalars given, so call USEROUT

         call userout ( isolut2d, inpcop )

      else

!     --- extra scalars are given, call USEROUTS

         do i = 1, inpsep(3)
            scals(i) = scalars(inpsep(3+inpsep(2)+i))
         end do

         if ( debug ) then
            call prinin ( inpcop, 2+inpsep(3)+inpsep(2), 'inpcop' )
            call prinrl ( scals, inpsep(3), 'scals' )
         end if  ! ( debug )

         call userouts ( isolut2d, inpcop, scals )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pruser'

      end if
      call erclos ( 'pruser' )

      istartmn = istartmn + 4 + inpsep(2) + inpsep(3)

      end subroutine pruser
