      subroutine read02 ( compl, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.2  date 16-02-2015 Small change debug
!        version 10.1  date 13-12-2014 Extension with default coefficients
!        version 10.0  date 30-10-2014 Extra parameter kmesh
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reads the input for subsequent calls of subroutines FIL103, FIL104,
!     FIL153 and FIL154.
!     Help subroutine for subroutine FILCOF or CHNCOF
!     The input is filled in the arrays IINPUT and RINPUT
!     The input must be enclosed between COEFFICIENTS and END
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulenparm
      use sepmoduleread1
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr
      logical compl

!     compl          o    indication whether the arithmetic is real (false)
!                         or complex (true)
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  FILCOF
!                         2:  CHNCOF
!                         3:  ERCOFS in case of coefficients
!                         4:  ERCOFS in case of change coefficients
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=7
      integer :: ncode2(nncode2), ncodmx2(nncode2)
      integer :: num, ngroup, itypel, icompl, ideg, kcode, &
                 ival, ifunc, ifirstgroup, ilastgroup, iposinbef
      character (len=6) code2(nncode2)
      logical finished, elgrpfound, filldefaults

!     code2          Table with keywords
!     elgrpfound     If true elgrp i = ... has been found
!     filldefaults   If true defaults must be filled
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     ideg           Degree of freedom read
!     ifirstgroup    First value of ielgrp in set of groups
!     ifunc          Choice parameter with respect to function calls
!     ilastgroup     Last value of ielgrp in set of groups
!     iposinbef      Previous value of iposin
!     itypel         Parameter itype in iincof
!     ival           Reference to position in rincof
!     kcode          Return code corresponding to sepsubkeyw
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodmx2        See ncodmx, now for code2
!     ngroup         number of element groups including boundary groups etc
!     nncode2        Number of entries in code2
!     num            Number of numbers in iincof
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCOFS     Fill arrays iincof/rincof with default values for standard
!                    elements
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRININLARGE    print 1d integer array
!     PRINRL         Print 1d real vector
!     READ02CHECK    Fill default coefficients for element groups for which no
!                    input is given
!     READ02COEF     Read information following coef =
!     READ02ICOEF    Read information following icoef =
!     READ02NPARM    Read information following ielgrp
!     READ02TYPES    Read coefficients depending on type numbers
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!    Input is read from the standard input file
!    The following input is required:
!    Per group:
!    ELGRP i to k (NPARM=n) or BNGRP i (NPARM=n) or GLGRP i (NPARM=n)
!     At most NPARM statements of one of the following shapes
!       coef j = c
!       coef j = (value=c)
!       coef j = (func=k)
!      icoef j = l
!       coef j = old_solution_j1 [degree_of_freedom d] [coef=c]
!       coef j = old_solution_j1 [degree_of_freedom d] [coef=(value=c)]
!       coef j = old_solution_j1 [degree_of_freedom d] [coef=(func=k)]
!       coef j = coef j2
!       coef j = (points, iref=r)
!       coef j = (elements, iref=r)
!       coef j = (sol_func= k)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     499   Internal SEPRAN error
!     626   Wrong character found
!     646   Coefficient number read out of range
!    1723   No number found after COEF
!    2862   No input for coefficients
!    3208   Not elgrp i found, whereas there are boundary groups
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'END', 'ELGRP', 'BNDGRP', 'COEFF', 'ICOEFF', 'NPARM', &
                   'GLGRP' /
      data ncode2  / 3, 3, 2, 3, 4, 4, 3 /
      data ncodmx2 / 3, 5, 6, 5, 6, 5, 5 /

!     code2   Table with main keywords
!             Values:
!             1:  END
!             2:  ELGRP
!             3:  BNGRP
!             4:  COEF
!             5:  ICOEF
!             6:  NPARM
!             7:  GLGRP
! ======================================================================
!
      call eropen ( 'read02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02'
         write(irefwr,1) 'nelgrp, numnat, nglgrp', &
                          nelgrp, numnat, nglgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobc => kprobpart%kprobc

      elgrpfound = .false.
      ngroup = nelgrp+numnat+nglgrp

!     --- Read next record ( elgrp i ( nparm = j) )

      iposin = 3+ngroup
      iposrn = 0
      itypel = 0
      maxprm = 50
      if ( compl ) then
         icompl = 1
      else
         icompl = 0
      end if
      finished = .false.
      if ( debug ) write(irefwr,1) 'iposin/1', iposin

      ifirstgroup = 0       ! initialization of element groups
      ilastgroup  = 0       ! initialization of element groups
      iposinbef = iposin
      typenr = kprobc(1)
      if ( typenr>0 ) then
         nparm = nparmarr(typenr)
      else
         nparm = 0
      end if  ! ( typenr>0 )
      if ( debug ) write(irefwr,1) 'typenr, nparm', typenr, nparm

      imark = 0
      filldefaults = .false.

      do while ( .not. finished )

!     --- Read record

         ityprd = 1
         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd/1', ityprd
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )
         if ( ityprd/=4 ) then

!        --- error: keyword read is not a main keyword

            call errsub ( 626, 0, 0, 0 )
            call errcrd
            if ( ityprd==0 ) then
               finished = .true.
            else
               go to 1000
            end if

         end if  ! ( ityprd/=4 )

!        --- Interpret keyword

         call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
         ierpri = 0
         kcode = sepsubkeyw()
         if ( debug ) write(irefwr,1) 'kcode', kcode
         if ( ierror/=0 ) go to 1000

!        --- Check type of record

         select case ( kcode )

            case(1)

!           --- END record found   return

               ielgrp = ifirstgroup

               if ( debug ) then
                  write(irefwr,1) 'ielgrp, iposinbef', ielgrp, iposinbef
                  write(irefwr,*) 'filldefaults ', filldefaults
               end if  ! ( debug )

               if ( filldefaults ) then

!              --- Fill default values for coefficients

                  typenr = kprobc(ielgrp)
                  call erfillcofs

               end if  ! ( filldefaults )

               num = iposin-iposinbef
               if ( debug ) then
                  write(irefwr,1) 'num, ifirstgroup, ilastgroup', &
                                   num, ifirstgroup, ilastgroup
                  write(irefwr,1) 'iposin, iposinbef, iposrn', &
                                   iposin, iposinbef, iposrn
                  write(irefwr,1) 'nparm', iincof(3+ifirstgroup)
                  call prinin ( iincof(iposinbef+1), num, 'iincof' )
               end if  ! ( debug )

               do ielgrp = ifirstgroup+1, ilastgroup

!              --- ilastgroup>ifirstgroup, hence information for several groups
!                  Copy information both for iincof and rincof

                  iincof(3+ielgrp) = iincof(3+ifirstgroup)  ! nparm

!                 --- fill -1 and new elgrp in iincof


                  iincof(iposin+1) = -1
                  iincof(iposin+2) = ielgrp

!                 --- Skip first element, elgroup number

                  iincof(iposin+3:iposin+num+2) = &
                     iincof(iposinbef+1:iposinbef+num)
                  iposin = iposin+num+2

               end do  ! ielgrp = ifirstgroup+1, ilastgroup
               if ( debug ) then
                  write(irefwr,1) 'iposin, iposinbef', &
                                   iposin, iposinbef
               end if  ! ( debug )
               finished = .true.

            case ( 2, 3, 6, 7 )

!           --- Text ELG, BNG or GLG found element group
!               Possibly only NPARM found, i.e. ielgrp = 1 assumed
!               Read element group number as well as nparm

               elgrpfound = .true.  ! element group has been found
               ielgrp = ifirstgroup

               if ( debug ) then
                  write(irefwr,1) 'ielgrp', ielgrp
                  write(irefwr,*) 'filldefaults ', filldefaults
               end if  ! ( debug )

               if ( filldefaults ) then

!              --- Fill default values for coefficients

                  typenr = kprobc(ielgrp)
                  call erfillcofs

               end if  ! ( filldefaults )

               num = iposin-iposinbef
               if ( debug ) then
                  write(irefwr,1) 'num, ifirstgroup, ilastgroup', &
                                   num, ifirstgroup, ilastgroup
                  write(irefwr,1) 'iposin, iposinbef', iposin, iposinbef
                  call prinin ( iincof(iposinbef+1), num, 'iincof' )
               end if  ! ( debug )
               do ielgrp = ifirstgroup+1, ilastgroup

!              --- ilastgroup>ifirstgroup, hence information for several groups
!                  Copy information both for iincof and rincof

                  iincof(3+ielgrp) = iincof(3+ifirstgroup)  ! nparm

!                 --- fill -1 and new elgrp in iincof

                  iincof(iposin+1) = -1
                  iincof(iposin+2) = ielgrp

!                 --- Skip first element, elgroup number

                  iincof(iposin+3:iposin+num+2) = &
                     iincof(iposinbef+1:iposinbef+num)
                  iposin = iposin+num+2
               end do  ! ielgrp = ifirstgroup+1, ilastgroup

               iposinbef = iposin+1  ! save value of iposin
               if ( ilastgroup>0 ) iposinbef = iposin+2

               call read02nparm ( isubr, ifirstgroup, ilastgroup )

               if ( debug ) then
                  write(irefwr,1) 'ifirstgroup, ilastgroup', &
                                   ifirstgroup, ilastgroup
                  call prinin ( iincof, iposin, 'iincof/1' )
               end if  ! ( debug )

               imark(1:nparm) = 0

            case ( 4)

!           --- Text COE found;  coef i = ( .... )
!               Read first number

               if ( .not. elgrpfound ) then

!              --- Not elgrp found, fill first part of iincof

                  iincof(4) = nparm
                  iincof(iposin+1) = 1
                  iposin = iposin + 1
                  elgrpfound = .true.
                  ifirstgroup = 1
                  ilastgroup = nelgrp

               end if  ! ( .not. elgrpfound )

               call asknxt
               if ( ityprd/=2 .and. ityprd/=3 ) then

!              --- No number found after COEF

                  call errsub ( 1723, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if
               icoef = nint(realrd)
               iincof(iposin+2) = icoef
               if ( icoef<=0 .or. icoef>nparm ) then
                  call errint ( icoef, 1 )
                  call errint ( nparm, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 646, 3, 0, 0 )
                  call errcrd
               end if
               kcolum = icolum
               call read02coef ( compl, icompl, ideg, ifunc, ival, itypel )

            case(5)

!           --- Letter i found;  icoef i = ....

               if ( .not. elgrpfound ) then

!              --- Not elgrp found, fill first part of iincof

                  iincof(4) = nparm
                  iincof(iposin+1) = 1
                  iposin = iposin + 1
                  elgrpfound = .true.
                  ifirstgroup = 1
                  ilastgroup = nelgrp

               end if  ! ( .not. elgrpfound )

               call read02icoef ( itypel )

            case default

!           --- Main keyword not recognized
!               Might be one of the standard coefficients depending on
!               the type of equation

               if ( debug ) write(irefwr,*) 'elgrpfound', elgrpfound
               if ( .not. elgrpfound ) then

!              --- Not elgrp i = found, check if there are boundary groups

                  if ( numnat>0 ) then

!                 --- Error there are boundary groups

                     call errsub ( 3208, 0, 0, 0 )
                     go to 1000

                  end if  ! ( numnat>0 )
                  ielgrp = 1
                  ilastgroup = nelgrp
                  ifirstgroup = 1
                  iposinbef = iposin+1  ! save value of iposin
                  nparm =  nparmarr(kprobc(ielgrp))

                  iincof(3+ielgrp) = nparm
                  iincof(iposin+1) = ielgrp
                  iposin = iposin + 1
                  maxprm = max(nparm, maxprm)
                  elgrpfound = .true.

               end if  ! ( .not. elgrpfound )

               ielgrp = ifirstgroup
               call read02types ( code, nncode )
               filldefaults = .true.

         end select  ! case ( kcode )

         if ( .not. filldefaults .and. kcode>3 .and. kcode<6 .and. &
              (itypel<=5 .or. itypel>=9) ) then

            iincof(iposin+1) = itypel
            iposin = iposin + 3

         else if ( .not. filldefaults .and. kcode>3 .and. kcode<7 .and. &
                   itypel==6 ) then

            iincof(iposin+4) = ideg
            if ( ival>0 ) then
               iincof(iposin+5) = ival
               itypel = 7
            else if( ifunc>0 ) then
               iincof(iposin+5) = ifunc
               itypel = 8
            end if
            iincof(iposin+1) = itypel
            iposin = iposin + 4
            if ( itypel/=6 ) iposin = iposin + 1

         end if

         if ( iposin>nicof ) then
            call errsub ( 499, 0, 0, 0 )
            go to 1000
         end if
         if ( iposrn>nrcof ) then
            call errsub ( 499, 0, 0, 0 )
            go to 1000
         end if

      end do  ! while ( .not. finished )

      if ( isubr==1 .or. isubr==3 ) then

!     --- Check if all types have been filled, if not fill with defaults

         call read02check ( ngroup )

      end if  ! ( isubr==1 )

      if ( iposin==3+ngroup ) then

!     --- No input found for subroutine read02

         call errwar ( 2862, 0, 0, 0 )

      end if  ! ( iposin==3+nelgrp+numnat+nglgrp )

      iposin = iposin + 1
      if ( iposin>nicof ) then
         call errsub ( 499, 0, 0, 0 )
      end if
      iincof(iposin) = -2
      iincof(1) = maxprm
      iincof(2) = icompl
      iincof(3) = iprob
      nrcof = iposrn
      nicof = iposin

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin/end', iposin
         call prininlarge ( iincof, nicof, 'iincof' )
         if ( nrcof>0 ) call prinrl ( rincof, nrcof, 'rincof' )
         write(irefwr,*) 'End read02'

      end if  ! ( debug )
      call erclos ( 'read02' )

      end
