      subroutine read02_250
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-02-2015 Extension with E_X ...
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 250
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numcomponents, iforce, icoefforce, i

!     i              General loop variable
!     icoefforce     Coefficient sequence number w.r.t. force vector
!     iforce         Sequence number of FORCE in code
!     numcomponents  Number of components of the vector/tensor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02_FORCE   Read coefficients for force vector part
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLV    Fill information concerning coefficient into iincof/rincof
!                    Special case of a vector of coefficients
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 250
!                1: ELASTIC_MODULUS
!                2: POISSONS_RATIO
!                3: THICKNESS
!                4: STRESS_STRAIN_RELATION
!                5: INTEGRATION_RULE
!                6: FORCE
!                7: X_FORCE
!                8: Y_FORCE
!                9: Z_FORCE
!               10: INITIAL_STRAIN
!               11: XX_INITIAL_STRAIN
!               12: XY_INITIAL_STRAIN
!               13: XZ_INITIAL_STRAIN
!               14: YY_INITIAL_STRAIN
!               15: YZ_INITIAL_STRAIN
!               16: ZZ_INITIAL_STRAIN
!               17: INITIAL_STRESS
!               18: XX_INITIAL_STRESS
!               19: XY_INITIAL_STRESS
!               20: XZ_INITIAL_STRESS
!               21: YY_INITIAL_STRESS
!               22: YZ_INITIAL_STRESS
!               23: ZZ_INITIAL_STRESS
!               24: DENSITY
!               25: E_X
!               26: E_X
!               27: NU_X
!               28: NU_Y
!               29: G_XY
!               30: DMAT_11
!               31: DMAT_21
!               32: DMAT_31
!               33: DMAT_22
!               34: DMAT_23
!               35: DMAT_33
!               36: DMAT_34
!               37: DMAT_44
!     code1      contains subkeywords
!                1: PLANE_STRESS', &
!                2: PLANE_STRAIN
!                3: AXI_SYMMTRIC_STRESS
!                4: ORTHOTROPIC_PLANE_STRESS
!                5: ORTHOTROPIC_PLANE_STRAIN
!                6: STRATIFIED_AXI_SYMMTRIC_STRESS
!                7: USER_DEFINED_ELASTICITY_CART
!                8: USER_DEFINED_ELASTICITY_AXI
! ======================================================================
!
      call eropen ( 'read02_250' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_250'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codestress, nncodestress, nscodestress, &
                         ncodestress )
      call sepcopycodesub ( codestressmod, nncodestressmod, nscodestressmod )

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read
      iposin_h = 0

      tabmin(1:3) = 1d-20        ! ELASTIC_MODULUS, POISSONS_RATIO, THICKNESS
      itype(4) = 5               ! STRESS_STRAIN_RELATION
      tabmin(4) = 1d0
      tabmax(4) = tabmin(1)+7d0

      itype(5) = 0               ! INTEGRATION_RULE
      tabmax(5) = 3d0
      tabmin(24) = 1d-20         ! DENSITY

      nmain = 0
      iretur = -11  ! keyword has been read

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4
      icoef = 0

      if ( table(1)>0d0 ) then

!     --- ELASTIC_MODULUS found

         icoef = 6
         call read02fillc

      else

!     --- Check for elastic coefficients

         do i = 25, 29

            if ( table(i)>0d0 ) then

               icoef = i-19
               call read02fillc
               go to 1000

            end if  ! ( table(i)>0d0 )

         end do  ! i = 25, 29

         do i = 30, 37

            if ( table(i)>0d0 ) then

               icoef = i-24
               call read02fillc
               go to 1000

            end if  ! ( table(i)>0d0 )

         end do  ! i = 30, 37

      end if  ! ( table(1)>0d0 )

      if ( table(2)>0d0 ) then

!     --- POISSONS_RATIO found

         icoef = 7
         call read02fillc

      else if ( table(3)>0d0 ) then

!     --- THICKNESS found

         icoef = 27
         iposin_h = iposin
         call read02fillc

      else if ( table(4)>0d0 ) then

!     --- STRESS_STRAIN_RELATION found

         icoef = 2
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(4))-1
         iposin = iposin+3

      else if ( table(5)>0d0 ) then

!     --- INTEGRATION_RULE found

         icoef = 3
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(5))
         iposin = iposin+3

      end if  ! ( table(5)>0d0 )
      if ( icoef>0 ) go to 1000

!     --- Force vector

      iforce = 6
      icoefforce = 28
      call read02_force ( iforce, icoefforce )
      if ( icoef>0 ) go to 1000

      if ( table(10)>0d0 ) then

!     --- INITIAL_STRAIN found

         icoef = 31
         if ( ndim==2 ) then
            numcomponents = 3
         else
            numcomponents = 6
         end if  ! ( ndim==2 )
         call read02fillv ( numcomponents )
         imark(icoef) = 0
         go to 1000

      else

!     --- INITIAL_STRAIN not found, check each component

        if ( table(11)>0d0 ) then

!       --- XX_INITIAL_STRAIN found

           icoef = 31
           call read02fillc

        else if ( table(12)>0d0 ) then

!       --- XY_INITIAL_STRAIN found

           icoef = 32
           call read02fillc

        else if ( table(13)>0d0 ) then

!       --- XZ_INITIAL_STRAIN found

           icoef = 33
           call read02fillc

        else if ( table(14)>0d0 ) then

!       --- YY_INITIAL_STRAIN found

           icoef = 34
           call read02fillc

        else if ( table(15)>0d0 ) then

!       --- YZ_INITIAL_STRAIN found

           icoef = 35
           call read02fillc

        else if ( table(16)>0d0 ) then

!       --- ZZ_INITIAL_STRAIN found

           icoef = 36
           call read02fillc

        end if  ! ( table(16)>0d0 )

      end if  ! ( table(10)>0d0 )
      if ( icoef>0 ) go to 1000

      if ( table(17)>0d0 ) then

!     --- INITIAL_STRESS found

         icoef = 37
         if ( ndim==2 ) then
            numcomponents = 3
         else
            numcomponents = 6
         end if  ! ( ndim==2 )
         call read02fillv ( numcomponents )
         imark(icoef) = 0

      else

!     --- INITIAL_STRESS not found, check each component

        if ( table(18)>0d0 ) then

!       --- XX_INITIAL_STRESS found

           icoef = 37
           call read02fillc

        else if ( table(19)>0d0 ) then

!       --- XY_INITIAL_STRESS found

           icoef = 38
           call read02fillc

        else if ( table(20)>0d0 ) then

!       --- XZ_INITIAL_STRESS found

           icoef = 39
           call read02fillc

        else if ( table(21)>0d0 ) then

!       --- YY_INITIAL_STRESS found

           icoef = 40
           call read02fillc

        else if ( table(22)>0d0 ) then

!       --- YZ_INITIAL_STRESS found

           icoef = 41
           call read02fillc

        else if ( table(23)>0d0 ) then

!       --- ZZ_INITIAL_STRESS found

           icoef = 42
           call read02fillc

        end if  ! ( table(23)>0d0 )

      end if  ! ( table(17)>0d0 )
      if ( icoef>0 ) go to 1000

      if ( table(24)>0d0 ) then

!     --- DENSITY found

         icoef = 43
         call read02fillc

      end if  ! ( table(24)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_250'

      end if  ! ( debug )
      call erclos ( 'read02_250' )

      end subroutine read02_250
