      subroutine read02_801
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 801
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ihint, ihcoor

!     i              Counting variable
!     ihcoor         Start address of coordinate_system in code
!     ihint          Start address of integration_rule in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02_801' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_801'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=5

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      itype(1) = 0
      tabmin(1) = 0d0
      tabmax(1) = 6d0
      nmain = 0
      iretur = -11  ! keyword has been read
      ityprd = 4    ! indicates that the first item is a string

      code(1:3) = codenatbnd(1:3)
      ncode(1:3) = nscodenatbnd(1:3)
      ncodmx(1:3) = ncodenatbnd(1:3)
      code(4:5) = codegen(2:3)
      ncode(4:5) = nscodegen(2:3)
      ncodmx(4:5) = ncodegen(2:3)

      call sepcopycodesub ( codecsys, nncodecsys, nscodecsys )

      ihint = 4   ! INTEGRATION_RULE
      itype(ihint) = 0
      tabmin(ihint) = 0d0
      tabmax(ihint) = 4d0

      ihcoor = ihint+1   ! COORDINATE_SYSTEM
      itype(ihcoor) = 5
      tabmin(ihcoor) = 1d0
      tabmax(ihcoor) = tabmin(ihcoor)+nncodecsys-1

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      if ( table(1)>0d0 ) then

!     --- TYPENATBND found

         icoef = 1
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(1))-1
         iposin = iposin+3
         go to 1000

      end if  ! ( table(1)>0d0 )

      do i = 4, 5

         if ( table(i)>0d0 ) then

!        --- INTEGRATION_RULE, COORDINATE_SYSTEM found

            icoef = i-1
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))-1
            iposin = iposin+3
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 2, 3

      if ( table(2)>0d0 ) then

!     --- DIFF_SIGMA found

         icoef = 6
         call read02fillc
         go to 1000

      end if  ! ( table(2)>0d0 )

      if ( table(3)>0d0 ) then

!     --- DIFF_FLUX found

         icoef = 7
         call read02fillc
         go to 1000

      end if  ! ( table(3)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_801'

      end if  ! ( debug )
      call erclos ( 'read02_801' )

      end subroutine read02_801
