      subroutine read02_816
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 816
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodel=2
      integer :: ncodel(nncodel), ncodmxl(nncodel)
      character (len=13) :: codel (nncodel)

!     codel          Table with keywords
!     ncodel         Table containing the number of significant characters
!                    in the keyword table code
!     ncodmxl        Table containing the maximal number of  characters in
!                    the keyword table code
!     nncodel        Length of array codel
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codel / 'TEMPERATURE', 'CONCENTRATION' /
      data ncodel  /  4, 4 /
      data ncodmxl / 11,13 /

!     code       contains keywords recognized for coefficients type 251
!                1: temperature
!                2: concentration
! ======================================================================
!
      call eropen ( 'read02_816' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_816'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=2

      code(1:nncode) = codel
      ncode(1:nncode) = ncodel
      ncodmx(1:nncode) = ncodmxl

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4
      icoef = 0

      if ( table(1)>0d0 ) then

!     --- temperature found

         icoef = 6
         call read02fillc

      else if ( table(2)>0d0 ) then

!     --- concentration found

         icoef = 7
         call read02fillc

      end if  ! ( table(2)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_816'

      end if  ! ( debug )
      call erclos ( 'read02_816' )

      end subroutine read02_816
