      subroutine read02_912
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 912
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 251
!                1: integration_rule
!                2: coordinate_system
!                3: seq_normal
!                4: mass_flux
!                5: penalty_flux
!     code1      contains subkeywords
!                 1: CARTESIAN
!                 2: AXI_SYMMETRIC
! ======================================================================
!
      call eropen ( 'read02_912' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_912'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=5

      code(1:2) = codenavstoks(2:3)
      ncode(1:2) = nscodenavstoks(2:3)
      ncodmx(1:2) = ncodenavstoks(2:3)
      code(3) = 'MASS_FLUX'
      ncode(3) = 4
      ncodmx(3) = 5
      code(4) = 'PENALTY_FLUX'
      ncode(4) = 9
      ncodmx(4) = 12

      call sepcopycodesub ( codecsys, 2, nscodecsys )

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read

      itype(1) = 0   ! INTEGRATION_RULE
      tabmin(1) = 0d0
      tabmax(1) = 2d0

      itype(2) = 5   ! COORDINATE_SYSTEM
      tabmin(2) = 1d0
      tabmax(2) = tabmin(2)+1d0

      itype(3) = 0   ! SEQ_NORMAL
      tabmin(3) = 1d0
      tabmax(3) = 2d0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4
      icoef = 0

      do i = 1, 3, 2

         if ( table(i)>0d0 ) then

!        --- INTEGRATION_RULE, SEQ_NORMAL  found

            icoef = i+2
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))
            iposin = iposin+3
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 1, 3, 2

      if ( table(2)>0d0 ) then

!     --- COORDINATE_SYSTEM  found

         icoef = 4
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(2))-1
         iposin = iposin+3
         go to 1000

      end if  ! ( table(2)>0d0 )

      if ( table(4)>0d0 ) then

!     --- mass_flux found

         icoef = 6
         call read02fillc

      else if ( table(5)>0d0 ) then

!     --- penalty_flux found

         icoef = 7
         call read02fillc

      end if  ! ( table(7)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_912'

      end if  ! ( debug )
      call erclos ( 'read02_912' )

      end subroutine read02_912
