      subroutine read02_force ( iforce, icoefforce )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients for force vector part
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecoeffs
      use sepmoduleread1
       implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icoefforce, iforce

!     icoefforce     i    Coefficient sequence number w.r.t. force vector
!     iforce         i    Sequence number of FORCE in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLV    Fill information concerning coefficient into iincof/rincof
!                    Special case of a vector of coefficients
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02_force' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_force'
         write(irefwr,1) 'iforce, icoefforce', iforce, icoefforce
         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         write(irefwr,2) 'table(iforce)', table(iforce)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table(iforce)>-1d19 ) then

!     --- FORCE found

         icoef = icoefforce
         call read02fillv ( ndim )
         imark(icoef) = 0

      else

!     --- FORCE not found, check each component

        if ( table(iforce+1)>-1d19 ) then

!       --- X_FORCE found

           icoef = icoefforce
           call read02fillc

        else if ( ndim>1 .and. table(iforce+2)>-1d19 ) then

!       --- Y_FORCE found

           icoef = icoefforce+1
           call read02fillc

        else if ( ndim>2 .and. table(iforce+3)>-1d19 ) then

!       --- Z_FORCE found

           icoef = icoefforce+2
           call read02fillc

        end if  ! ( ndim>2 .and. table(iforce+3)>-1d19 )

      end if  ! ( table(iforce)>-1d19 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icoef', icoef
         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         write(irefwr,*) 'End read02_force'

      end if  ! ( debug )
      call erclos ( 'read02_force' )

      end subroutine read02_force
