      subroutine read02check ( ngroup )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill default coefficients for element groups for which no input
!     is given
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulenparm
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ngroup

!     ngroup         i    number of element groups including boundary groups etc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLTYPES    Fill coefficients depending on type numbers
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     499   Internal SEPRAN error
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02check' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02check'
         write(irefwr,1) 'ngroup, iposin, iposrn', &
                          ngroup, iposin, iposrn
         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all element groups

      do ielgrp = 1, ngroup

         if ( iincof(3+ielgrp)==0 ) then

!        --- element group has no input yet

            typenr = kprobc(ielgrp)

            if ( debug ) &
               write(irefwr,1) 'ielgrp, typenr', ielgrp, typenr

            if ( typenr>99 ) then

!           --- type number>99, fill default values

               call erfilltypes ( ngroup )

               if ( iposin>nicof ) then
                  call errsub ( 499, 0, 0, 0 )
                  go to 1000
               end if
               if ( iposrn>nrcof ) then
                  call errsub ( 499, 0, 0, 0 )
                  go to 1000
               end if

            end if  ! ( typenr>99 )

         end if  ! ( iincof(3+ielgrp)==0 )

      end do  ! ielgrp = 1, ngroup

1000  call erclos ( 'read02check' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,*) 'End read02check'

      end if  ! ( debug )

      end subroutine read02check
