      subroutine read02fillc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read and fill information concerning coefficient into iincof/rincof
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ideg, ifunc, ival, itypel

!                    they are not
!     ideg           Degree of freedom read
!     ifunc          Choice parameter with respect to function calls
!     itypel         Parameter itype in iincof
!     ival           Reference to position in rincof
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READ02COEF     Read information following coef = &
!     READ02FILL     Fill information concerning coefficient into iincof
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02fillc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02fillc'
         call prinin ( iincof, iposin, 'iincof' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call read02coef ( .false., 1, ideg, ifunc, ival, itypel )
      call read02fill ( itypel, ideg, ifunc, ival )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End read02fillc'

      end if  ! ( debug )
      call erclos ( 'read02fillc' )

      end subroutine read02fillc
