      subroutine read02icoef ( itypel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-01-2015 New call to readreal
!        version  1.1  date 08-12-2013 Extension with integer scalars
!        version  1.0  date 01-05-2008
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following icoef = &
!     Store this information
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulenparm
      use sepmoduleread
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypel

!     itypel         o    Parameter itype in iincof
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ispeclsav, lastitemsav
      logical :: openfound

!                    they are not
!     ispeclsav      Help parameter to save the value of ispecl
!     lastitemsav    Help parameter to save the value of lastitem
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READNAMEVEC    Read name of vector in common cpost5
!     READREAL       Read a real or real expression from the input file
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     118   text found after scalar, which is not permitted
!     646   Coefficient number read out of range
!     647   Item found following icoef . = is not recognized.
!    2948   Not an integer sequence number found after icoef
!    2949   The text found after icoef . = is not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02icoef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02icoef'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next two numbers

      itypel = 1
      call asknxt
      if ( debug ) write(irefwr,1) 'case 5: ityprd', ityprd
      if ( ityprd==2 ) then

!     --- Coefficient number found

         icoef = nint(realrd)
         if ( icoef<=0 .or. icoef>nparm ) then

!        --- Error: coefficient out of range

            call errint ( icoef, 1 )
            call errint ( nparm, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 646, 3, 0, 0 )
            call errcrd

         end if

      else

!     --- Error: Not a coefficient number found

         call errsub ( 2948, 0, 0, 0 )
         call errcrd

      end if  ! ( ityprd==2 )

!     --- Read next item

      ispeclsav = ispecl
      lastitemsav = lastitem
      ispecl = 9
      call asknxt
      ispecl = ispeclsav
      if ( debug ) then
         write(irefwr,1) 'ityprd/2', ityprd
         if ( ityprd==4 ) then
            write(irefwr,*) 'string ', string(1:nchar)
         else
            write(irefwr,1) 'realrd', realrd
         end if  ! ( ityprd==4 )
      end if  ! ( debug )

      if ( ityprd==4 ) then

         if ( string(1:5)=='VALUE' ) then

!        --- value found, read next item

            ispeclsav = ispecl
            ispecl = 9
            call asknxt
            ispecl = ispeclsav

         end if  ! ( string(1:5)/='VALUE' )

      end if  ! ( ityprd==4 )

      if ( ityprd==4 ) then

!     --- text found, must be value
!         First we check if it is a scalar

         call readstrscals
         if ( ityprd==11 ) then

!        --- Scalar found, jump

            if ( lastitem<nitems ) then

!           --- Text found after scalar (not permitted)

               call errchr ( string(1:nchar), 1 )
               call errsub ( 118, 0, 0, 1 )
               call errcrd
               go to 1000

            end if  ! ( lastitem<nitems )
            go to 100

         end if  ! ( ityprd==11 )

!        --- Next check if it is a vector name

         call readnamevec

         if ( ityprd==2 ) then

!        --- Vector name found, jump

            if ( lastitem<nitems ) then

!           --- Text found after scalar (not permitted)

               call errchr ( string(1:nchar), 1 )
               call errsub ( 118, 0, 0, 1 )
               call errcrd
               go to 1000

            end if  ! ( lastitem<nitems )
            go to 100

         end if  ! ( ityprd==2 )

!        --- Next check if it is a constant

         ispeclsav = lastitem
         lastitem = lastitemsav
         openfound = .false.
         call readreal ( openfound, .false. )
         if ( debug ) then
            write(irefwr,1) 'ityprd/3', ityprd
            if ( ityprd==2 ) then
               write(irefwr,1) 'icoef', nint(realrd)
            else if ( ityprd==3 ) then
               write(irefwr,2) 'icoef', realrd
            end if  ! ( ityprd==2 )
         end if  ! ( debug )

         if ( ityprd==4 ) then

!        --- ityprd = 4, hence still a text
!            No constant or vector name

            lastitem = ispeclsav
            if ( string(1:5)/='VALUE' ) then

!           --- Not value found

                call errint ( icoef, 1 )
                call errchr ( string(1:nchar), 1 )
                call errsub ( 2949, 1, 0, 1 )
                call errcrd

            end if  ! ( string(1:3)/='VAL' )
            call asknxt

         end if  ! ( ityprd==4 )

      end if  ! ( ityprd==4 )

100   if ( ityprd==2 .or. ityprd==3 .or. ityprd==11 ) then

!     --- Value found

         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(realrd)

      else

!     --- No value found

          call errint ( icoef, 1 )
          call errsub ( 647, 1, 0, 0 )
          call errcrd

      end if  ! ( ityprd==2 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02icoef'

      end if  ! ( debug )
      call erclos ( 'read02icoef' )

      end
