      subroutine read02nparm ( isubr, ifirstgroup, ilastgroup )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 09-03-2013 Correction length code
!        version  2.2  date 13-11-2010 error 1488 --> errkeywexcl
!        version  2.1  date 07-12-2009 Extra parameter kprobc
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following ielgrp
!     Store this information
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulenparm
      use sepmodulekprob
      use sepmoduleread1
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT(1) = OUTPUT PARAMETERS
!
      integer isubr, ifirstgroup, ilastgroup

!     ifirstgroup    i    First value of ielgrp in set of groups
!     ilastgroup     i    Last value of ielgrp in set of groups
!     isubr          i    Indicates the type of calling subroutine
!                         See subroutine READ02
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iglgrp, ibngrp

!     ibngrp         Actual boundary element group number
!     iglgrp         global element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02nparm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02nparm'
         write(irefwr,1) 'iposin', iposin
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode = 4
      code(1) = 'ELGRP'
      code(2) = 'NPARM'
      code(3) = 'BNGRP'
      code(4) = 'GLGRP'
      ncode(1:4) = 3
      ncodmx(1:4) = 5
      ncodmx(2) = 6
      itype(1:4) = 42
      itype(2) = 0
      tabmin(1:4) = 1d0

!     --- Initialize table (-1)

      table(1:nncode) = -1d0
      tabmax(1) = nelgrp  ! maximum number of element groups
      tabmax(2) = 1000d0  ! maximum number of parameters
      tabmax(3) = numnat  ! maximum number of boundary element groups
      tabmax(4) = nglgrp  ! maximum number of global element groups

!     --- Interpret rest of record

      iretur = -11  ! asknxt has been called before
      nmain = 0     ! no main keywords
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinrl ( table, 4, 'table' )

      ielgrp = 1  ! Default element group number
      if ( table(1)>0d0 ) then

!     --- Element group found

         iposut = nint(table(1))
         ifirstgroup = nint(tabout(iposut))
         ilastgroup = nint(tabout(iposut+1))
         ielgrp = ifirstgroup
         if ( debug ) &
            write(irefwr,1) 'ifirstgroup, ilastgroup', &
                             ifirstgroup, ilastgroup

!        --- Give error message if bngrp or glgrp is found

         if ( table(3)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 3 )
         if ( table(4)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 4 )

      end if  ! ( table(1)>0d0 )

      if ( table(3)>0d0 ) then

!     --- Boundary group found

         iposut = nint(table(3))
         ibngrp = nint(tabout(iposut))
         ielgrp = ibngrp+nelgrp
         ifirstgroup = ielgrp
         ilastgroup = ielgrp+nint(tabout(iposut+1))-ibngrp

!        --- Give error message if glgrp is found

         if ( table(4)>0d0 ) call errkeywexcl ( code, ncodmx, 3, 4 )

      end if  ! ( table(3)>0d0 )

      if ( ifirstgroup>0 .and. kprobc(max(1,ifirstgroup))>0 ) then
         nparm = nparmarr(kprobc(ifirstgroup))
      else
         nparm = 0
      end if  ! ( ifirstgroup>0 )
      if ( table(2)>0d0 ) nparm = max(nparm,nint(table(2)))
      if ( (isubr==2 .or. isubr==4) .and. nparm==0 ) nparm = maxprm
      maxprm = max(maxprm,nparm)

      if ( table(4)>0d0 ) then

!     --- Global group found

         iposut = nint(table(4))
         iglgrp = nint(tabout(iposut))
         ielgrp = iglgrp+nelgrp+numnat
         ifirstgroup = ielgrp
         ilastgroup = ielgrp+nint(tabout(iposut+1))-iglgrp

      end if  ! ( table(4)>0d0 )

!     --- Put nparm into iincof

      iincof(3+ielgrp) = nparm
      if ( iposin/=3+nelgrp+numnat+nglgrp ) then

!     --- iposin # 3, hence next element group found

         iposin = iposin + 1
         iincof(iposin) = -1

      end if
      iincof(iposin+1) = ielgrp
      iposin = iposin + 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ielgrp, ifirstgroup, ilastgroup, nparm', &
                          ielgrp, ifirstgroup, ilastgroup, nparm
         write(irefwr,1) 'iposin', iposin
         write(irefwr,*) 'End read02nparm'

      end if  ! ( debug )
      call erclos ( 'read02nparm' )

      end subroutine read02nparm
