      subroutine read03direct ( isubeq, inpsol, rinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all information corresponding to the sub-keyword DIRECT_SOLVER
!     corresponding to the main keyword SOLVE
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     solve
!     mumps
!     direct_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubeq, inpsol(*)
      double precision rinsol(*)

!     inpsol        i/o   Contains integer information concerning the solution
!                         process.
!                         For a description see subroutine READ03
!     isubeq         i    Sequence number of subequation
!     rinsol        i/o   Array containing the real user input if iread=-1
!                         For a description see subroutine READ03
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=10
      integer ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2)
      double precision tabmin2(nncode2), tabmax2(nncode2)
      character (len=25) code2(nncode2)

!     code2          Table with keywords
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     tabmax2        Contains maximum values allowed in table
!     tabmin2        Contains minimum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'PROFILE', 'COMPACT', 'DROPTOLER', &
                  'NUM_PIVOT_ROWS', 'NUM_ITER_REF', 'ACCURACY', &
                  'PRINT_LEVEL', 'LU_STORAGE', 'MUMPS', 'RENUM_MUMPS' /
      data ncode2  / 3, 3, 4, 6, 6,   3, 3, 5, 5, 5 /
      data ncodmx2 / 7, 7,12,14,12,   8,11,12, 5,11 /
      data itype2  / 2, 2, 1, 0, 0,   1, 0, 2, 2, 0 /
      data tabmin2 / 0d0, 0d0, 0d0, 0d0, 0d0,   0d0, 0d0, 2d0, 0d0, 0d0 /
      data tabmax2 / 0d0, 0d0, 1d0, 1d2, 1d4,   1d3, 6d0, 1d2, 0d0, 7d0 /

!     code2   Table with the keywords with the options of direct_solver
!             Values:
!             1:  profile
!             2:  compact
!             3:  drop_tolerance
!             4:  num_pivot_rows
!             5:  num_iter_ref
!             6:  accuracy
!             7:  print_level
!             8:  lu_storage
!             9:  mumps
!            10:  renum_mumps
! ======================================================================
!
      call eropen ( 'read03direct' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read03direct'
         write(irefwr,1) 'isubeq, ninpsol1', isubeq, ninpsol1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmin2, tabmax2 )
      table(1:nncode) = 0d0
      table(3) = -1d0
      table(4) = 3d0
      table(6) = 1d-3
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, 10, 'table' )
      if ( iretur==-1 ) go to 1000

!     --- Check if keywords are mutually exclusive

      call errkeywexclus ( table, code, ncodmx, 1, 2 )
      call errkeywexclus ( table, code, ncodmx, 1, 9 )
      call errkeywexclus ( table, code, ncodmx, 2, 9 )

      if ( table(2)>0d0 ) inpsol(ninpsol1*isubeq+3) = -1
      inpsol(ninpsol1*isubeq+8) = nint(table(4))
      if ( table(9)>0d0 ) then
         inpsol(ninpsol1*isubeq+3) = 19  ! mumps

         if ( table(10)>=0d0 ) then
            inpsol(ninpsol1*isubeq+8) = nint(table(10))  ! type of renumbering
            if ( inpsol(ninpsol1*isubeq+8)==1 ) &
               inpsol(ninpsol1*isubeq+8) = 7       ! value 1 not available
         else
            inpsol(ninpsol1*isubeq+8) = 7    ! default
         end if  ! ( table(10)>=0d0 )

      end if  ! ( table(9)>0d0 )

      if ( table(3)>=0d0 ) then
         rinsol(nrinsol1*isubeq+2) = table(3)
      else
         rinsol(nrinsol1*isubeq+2) = 1d-12
      end if
      inpsol(ninpsol1*isubeq+5) = nint(table(5))
      rinsol(nrinsol1*isubeq+1) = table(6)
      inpsol(ninpsol1*isubeq+6) = nint(table(7))
      if ( table(8)>0d0 ) then
          inpsol(ninpsol1*isubeq+4) = nint(table(8))
      else
          inpsol(ninpsol1*isubeq+4) = 15
      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsol, 10, 'inpsol' )
         write(irefwr,*) 'End read03direct'

      end if  ! ( debug )
      call erclos ( 'read03direct' )

      end subroutine read03direct
