      subroutine read74 ( timelc, niter, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 04-03-2011 New call to readoutp
!        version  2.5  date 09-06-2010 Correction itimdep
!        version  2.4  date 28-11-2006 Replace part of body by subroutines
!
!   copyright (c) 1999-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read all information from the files sepcomp.inf and sepcomp.out
!    Store in-core
!    Remark:  If the storage of sepcomp.inf or sepcomp.out is changed,
!             also subroutine incv02 must be changed!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulekprob
      use sepmoduletime
      use sepmodulemain
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: niter, manag(*)
      double precision timelc(*)

!     manag          i    Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         See subroutine readsp
!     niter          o    Iteration sequence number
!     timelc         o    Actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ltime, i, ios, icount, iseqlc, iter, numvecsav, &
              indsol, ltimefirst, ltimelast, ndtvecold
      logical endfil
      double precision told

!     endfil         Indicates if end of file has been reached (true ) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     indsol         Sequence number of vector in isol
!     ios            IOSTAT variable, returns from reading or writing
!     iseqlc         Sequence number
!     iter           Iteration sequence number
!     ltime          Integer time sequence number
!     ltimefirst     First time step for reading solution vectors
!     ltimelast      Last time step for reading solution vectors
!     ndtvecold      Value of ndtvec at start of subroutine
!     numvecsav      Help variable to store the value of numvec
!     told           previous time read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     READ74SUB      Read names of vectors from the sepcomp.out file
!     READ74SUB1     Read first part of information of solution array from
!                    sepcomp.out
!     READOUTP       read real/complex sepran solution array from the
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
!    The files sepcomp..out is read
!    For a description of the file see the Programmers Guide
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     769   End of file Reached of file sepcomp.out
!     770   Read error in file sepcomp.out
!    1455   niter>1 and ntime>1
!    1485   Sequence number read out of range
!    1486   Iteration number or time step read out of range
!    2409   Too many solution errors must be stored
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize irefar
!     Read first record of sepcomp.inf (ivers, itype)
!     If ( ivers<2 ) then
!        Read and test numvec
!        Read irefar
!        Set ntime
!        If ( time-dependent ) then
!           Read time array
!        Read solution arrays
!     else ( ivers=2 ) then
!        niter := 0
!        ntime := 0
!        numvec := 0
!        icount := 0
!        time(1) := 0
!        end_of_file := false
!        while ( not end_of file ) do
!           Read next record in sepcomp.inf (iseq,iter,t)
!           icount := icount+1
!           If ( end of file ) then
!              end_of_file := true
!           else
!              numvec := max(numvec,iseq)
!              if ( iter>1 ) then
!                 niter := max(niter,iter)
!              if ( ntime=0 and t#0 ) then
!                 if ( icount=1 ) then
!                    ntime := 1
!                 else
!                    ntime := 2
!                 time(ntime) = t
!            ltime=max(iter,ntime,1)
!            Read solution in isol(iseq,ltime)
!         if ( niter>1 and ntime>1 ) then
!            error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read74' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read74'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set irefar equal to zero

      numvecsav = numvec
      irefar = 0
      irefar(0) = 1
      niter = 0
      told = -1d0
      ntime = 0
      icount = 0
      time(1) = 0d0
      endfil = .false.
      ios = 0
      do i = 0, 1000
         irefar(i) = i+1
      end do
      ltimefirst = manag(9)
      ltimelast = manag(10)
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc

      ndtvecold = kprobloc(6)

!     --- Read names of vectors

      call read74sub
      if ( debug ) write(irefwr,*) 'after read74sub'

      numvec = 0

!     --- while loop end file

300   if ( .not. endfil ) then

!     --- Read next record in sepcomp.out

         icount = icount + 1
         call read74sub1 ( iseqlc, iter, t, ios )

         if ( ios<0 ) then

!        --- end of file found

            endfil = .true.

         else if ( ios>0 ) then

!        --- error in reading of file

            go to 500

         else

!        --- Correct record found

            if ( iseqlc<0 .or. iseqlc>maxvc1 ) then

!           --- iseqlc too large

               call errint ( iseqlc, 1 )
               call errint ( maxvc1, 2 )
               call errsub ( 1485, 2, 0, 0 )
               iseqlc = 0

            end if
            numvec = max(numvec,iseqlc+1)
            if ( debug ) write(irefwr,1) 'iter, numvec', iter, numvec
            if ( iter>1 ) then

!           --- iter>1

               niter = max(niter,iter)

            end if
            if ( abs(t-told)>sqreps*abs(t) ) then

!           --- First time a different time level has been found

               if ( icount<=ltimefirst ) then

!              --- First record

                  ntime = 1

               else

!              --- Not first record

                  ntime = max(2,ntime+1)

               end if
               if ( ntime>ltimelast ) then

!              --- ntime too large, skip reading

                  call errint ( ntime, 1 )
                  call errint ( ltimelast, 2 )
                  call errwar ( 1486, 2, 0, 0 )
                  ntime = ltimelast
                  go to 1000

               end if
               timelc(ntime) = t
               told = t

            end if

            ltime = max(iter,ntime,1)
            if ( ltime>ltimelast ) then

!           --- ltime too large, skip reading

               call errint ( ltime, 1 )
               call errint ( ltimelast, 2 )
               call errwar ( 1486, 2, 0, 0 )
               ltime = ltimelast
               go to 1000

            end if

!           --- Read solution

            indsol = imapsol(iseqlc+1,ltime)
            if ( indsol==0 ) then

!           --- Create new entry in isol

               lastmapsol = lastmapsol+1
               imapsol(iseqlc+1,ltime) = lastmapsol
               indsol = lastmapsol
               if ( indsol>maxsoluts ) then

!              --- indsol too large

                  call errint ( maxsoluts, 1 )
                  call errwar ( 2409, 1, 0, 0 )
                  ntime = ntime-1
                  ltime = ltime-1
                  lastmapsol = lastmapsol-1
                  go to 1000

               end if

            end if
            if ( debug ) write(irefwr,*) 'before readoutp'
            call readoutp ( isolut2d(indsol), ndtvecold )
            if ( debug ) call prinin ( isolut2d(indsol),5, 'isolut2d')
            if ( ierror/=0 ) then
               ierror = 0
               nerror = 0
               do i = 1, 5
                  isolut2d(indsol) = 0
               end do
               ntime = ntime-1
            end if

         end if

         go to 300

      end if

      go to 1000

!     --- Error messages if ios # 0

500   if ( ios>0 ) then

!     --- ios>0, Error found in reading of file 74

         call errsub ( 770, 0, 0, 0 )

      else

!     --- ios<0, End of file found in reading of file 74

         call errsub ( 769, 0, 0, 0 )

      end if
1000  if ( niter==0 ) niter = 1

      if ( niter>1 .and. ntime>1 ) then

!     --- Both niter>1 and ntime>1 not allowed

         call errint ( niter, 1 )
         call errint ( ntime, 2 )
         call errsub ( 1455, 2, 0, 0 )

      end if
      if ( ntime>1 ) itimdep = 1
      numvec = max(numvec, numvecsav)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read74'

      end if  ! ( debug )
      call erclos ( 'read74' )

      end subroutine read74
