      subroutine readaddvec
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 07-01-2015 Check if vector exists
!        version  1.2  date 27-02-2014 Extra error messages
!        version  1.1  date 18-02-2014 Remove lastitemsav
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reread name of new vector and add to list of vectors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ispeclsav

!     ispeclsav      Help parameter to save the value of ispecl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDVEC         Add vector name to the list of vectors
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     READNAMEVEC    Read name of vector in common cpost5
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constpstants
!     READSTRSCALS   Check if name read corresponds to one of the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3173   Vector name is already in use as constpstant
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readaddvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readaddvec'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check if the name is already a vector

      call readnamevec
      if ( ityprd==2 ) then

!     --- vector already exists

         numvec = nint(realrd)
         go to 1000

      end if  ! ( ityprd==2 )

!     --- First check if the name is not already in use

      call readstrconst
      if ( ityprd/=4 ) then

!     --- Error vector has name of constpstant

         call errchr ( string(1:nchar), 1 )
         call errsub ( 3173, 0, 0, 1 )
         go to 1000

      end if  ! ( ityprd/=4 )

      call readstrscals
      if ( ityprd/=4 ) then

!     --- Error vector has name of constpstant

         call errchr ( string(1:nchar), 1 )
         call errsub ( 3173, 0, 0, 1 )
         go to 1000

      end if  ! ( ityprd/=4 )

      ispeclsav = ispecl      ! save ispecl
      lastitem = 1            ! reset lastitem
      ispecl = 9              ! include digits to name
      call asknxt             ! reread name
      ispecl = ispeclsav      ! reset ispecl

      if ( debug ) write(irefwr,1) 'lastitem', lastitem

      call inichkarlen ( numvec+1, maxvc1, 'names' )  ! check array length
      if ( ierror/=0 ) go to 1000

      call addvec  ! add vector to list of vectors

1000  call erclos ( 'readaddvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, numvec', ityprd, numvec
         write(irefwr,*) 'End readaddvec'

      end if  ! ( debug )

      end
