      subroutine readal ( iseqprob, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 20-08-2016 use sepmoduleinput
!        version  6.9  date 24-12-2014 New definition clinsl
!        version  6.8  date 30-10-2014 New call to read02
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all SEPRAN input after the input for PROBDF and store in IBUFFR
!     The sequence of the input is arbitrary
!     Reading is stopped if end of file has been found or the text
!     END_OF_SEPRAN_INPUT
!     Warning:  if readal is modified, also modify FILLINP !!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleinput
      use sepmoduleread1
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqprob, manag(*)

!     iseqprob       i    Sequence number of problem input
!     manag          i    Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         See subroutine READSP for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=33
!       if changed, change fillinp
      character (len=37) codelc(nncodelc)
      integer :: ncodelc(nncodelc), ncodmxlc(nncodelc), i
      logical finished, bouncn, nooutput

!     bouncn         If true boundary conditions have been given explicitly
!     codelc         Table with keywords
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     ncodelc        Table containing the number of significant characters in
!                    the keyword table codelc
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc
!     nncodelc       Number of entries in codelc
!     nooutput       If true no output is written to the file sepcomp.out
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERADAPBN       Read input concerning the adaptation of the boundaries
!     ERADAPMS       Read input concerning the adaptation of the mesh
!     ERBEARING      Read the input for bearings
!     ERCAPACITY     Read the input for the subroutine to compute the capacity
!     ERCHCOOR       Read input concerning the changing of coordinates
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOFS         Read input concerning the coefficients
!     ERCOMM         Read input concerning the matrix structure
!     ERCOMP         Read input concerning the main program SEPCOMP
!     ERCONTCT       Read input concerning contact
!     ERCREA         Read input concerning CREATE VECTOR or boundary conditions
!     ERDERI         Read input concerning DERIVATIVES
!     EREIGVAL       Read input concerning eigenvalues
!     ERENTHALPY     Read the input for the subroutine prenthalpy
!     ERINSTFR       Read input concerning the instationary free boundaries
!     ERINTB         Read input concerning BOUNDARY_INTEGRALS
!     ERINTI         Read input concerning INTEGRALS
!     ERINVERSE      Read the input for the subroutine compinverse
!     ERLOOP         Read input concerning the loop
!     ERNAVSTOKES    Read the input for Navier-Stokes
!     ERNLNS         Read input concerning non-linear equations
!     EROPEN         Produces concatenated name of local subroutine
!     EROUTP         Read input concerning the module output
!     ERPRESSCOR     Read input concerning pressure correction
!     ERPROB         Read input with respect to the problem definition
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERREFINE       Read the input for refinement
!     ERRSUB         Error messages
!     ERSOLV         Read input concerning the linear solver
!     ERSTATFR       Read input concerning the stationary free boundaries
!     ERTIMD         Read card input for time-dependent solver for READAL
!     INI016         Fill default structure in ISEPIN
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPROBSTART   Fill problem input read first into isepi2/3
! **********************************************************************
!
!                       I/O
!
!     This subroutine recognizes the following input records:
!
!     MATRIX
!     CREATE
!     ESSENTIAL
!     COEFFICIENTS
!     CHANGE COEFFICIENTS
!     CHANGE_COORDINATES
!     SOLVE
!     NONLINEAR_EQUATIONS
!     END_OF_SEPRAN_INPUT
!     COMPLEX
!     OUTPUT
!     STRUCTURE
!     DERIVATIVES
!     INTEGRALS
!     BOUNDARY_INTEGRAL
!     EIGENVALUES
!     TIME_INTEGRATION
!     ADAPT_BOUNDARY
!     ADAPT_MESH
!     STATIONARY_FREE_BOUNDARY
!     INSTATIONARY_FREE_BOUNDARY
!     CONTACT
!     CAPACITY
!     LOOP_INPUT
!     INVERSE_PROBLEM
!     PROBLEM
!     ENTHALPY_INTEGRATION
!     REFINE
!     PRESSURE_CORRECTION
!     NAVIER_STOKES
!     LEVEL_SET
!     OBSTACLE_MESH
!     BEARING
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
!    1857   The next text found is not a keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Reserve temporary space to store arrays ISEPIN and ISEPRL
!   Initialize array ISEPIN
!   finished := false
!   While ( not finished ) do
!      Read next keyword
!      if ( end of file has been found ) then
!         finished := true
!      else if ( keyword = MATRIX ) then
!         read input for subroutine COMMAT until END is found
!      else if ( keyword = CREATE ) then
!         read input for subroutine CREATE until END is found
!      else if ( keyword = ESSENTIAL ) then
!         read input for subroutine PRESDF until END is found
!      else if ( keyword = COEFFICIENTS or COMPLEX ) then
!         read input for subroutine FILCOF until END is found
!      else if ( keyword = CHANGE COEFFICIENTS ) then
!         read input for subroutine CHNCOF until END is found
!      else if ( keyword = SOLVE ) then
!         read input for subroutine LINSOL until END is found
!      else if ( keyword = NONLINEAR ) then
!         read input for subroutine NLNPRB until END is found
!      else if ( keyword = TIME_INTEGRATION ) then
!         read input for subroutine TIMPRB until END is found
!      else if ( keyword = OUTPUT ) then
!         read input for subroutine OUTPUT until END is found
!      else if ( keyword = CHANGE_COORDINATES ) then
!         read input for subroutine MSHCHCOR until END is found
!      else if ( keyword = END_OF_SEPRAN_INPUT ) then
!         finished := true
!      else if ( keyword = STRUCTURE ) then
!         read input for main program until END is found
!      else if ( keyword = DERIVATIVES ) then
!         read input for subroutine DERIV until END is found
!      else if ( keyword = INTEGRALS ) then
!         read input for subroutine INTEGR until END is found
!      else if ( keyword = BOUNDARY_INTEGRALS ) then
!         read input for subroutine INTBND until END is found
!      else if ( keyword = EIGENVALUES ) then
!         read input for subroutine EIGVAL until END is found
!      else if ( keyword = ADAPT_BOUNDARY) then
!         read input for subroutine ADAPBOUN until END is found
!      else if ( keyword = ADAPT_MESH) then
!         read input for subroutine ADAPMESH until END is found
!      else if ( keyword = STATIONARY_FREE_BOUNDARY ) then
!         read input for subroutine STATFREE until END is found
!      else if ( keyword = CONTACT ) then
!         read input for subroutine CONTACT until END is found
!      else if ( keyword = LOOP_INPUT ) then
!         read input for subroutine LOOP until END is found
!      else if ( keyword = CAPACITY ) then
!         read input for subroutine CAPACITY until END is found
!      else if ( keyword =  INVERSE_PROBLEM) then
!         read input for subroutine INVERSE_PROBLEM until END is found
!      else if ( keyword =  PROBLEM) then
!         read input for subroutine PROBDF until END is found
!      else if ( keyword = ENTHALPY_INTEGRATION ) then
!         read input for subroutine ENTH_INTEG until END is found
!      else if ( keyword =  REFINE) then
!         read input for subroutine REFINE until END is found
!
!      else
!         error message
!   Fill structure in ISEPIN in case no structure is given
!   Make array ISEPIN permanent
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'MATRIX', 'CREATE', 'ESSENTIAL', 'COEFFICIENTS', &
                  'CHANGE_COORDINATES', 'SOLVE', 'NONLINEAR_EQUATIONS', &
                  'END_OF_SEPRAN_INPUT', 'COMPLEX', 'OUTPUT', &
                  'STRUCTURE', 'DERIVATIVES', 'INTEGRALS', &
                  'BOUNDARY_INTEGRAL', 'TIME_INTEGRATION', &
                  'ADAPT_BOUNDARY', 'ADAPT_MESH', &
                  'STATIONARY_FREE_BOUNDARY', &
                  'INSTATIONARY_FREE_BOUNDARY', 'CONTACT', &
                  'LOOP_INPUT', 'CHANGE', 'EIGENVALUES', 'CAPACITY', &
                  'INVERSE_PROBLEM', 'ENTHALPY_INTEGRATION', 'PROBLEM', &
                  'REFINE', 'PRESSURE_CORRECTION', 'NAVIER_STOKES', &
                  'LEVEL_SET', 'OBSTACLE_MESH', 'BEARING' /
      data ncodelc / 4, 5, 5,  4,10,   4, 3,  7, 4, 4, &
                   5, 5, 5, 11, 4,  10,10,  5, 7, 4, &
                   4, 6, 5,  5, 5,   6, 6,  4,12, 6, &
                   9, 8, 4 /
      data ncodmxlc/ 6, 6, 9, 12,18,   5,20, 19, 7, 6, &
                   9,11, 9, 17,16,  14,10, 25,27, 7, &
                  10, 6,11,  8,15,  20, 7,  6,19,13, &
                   9,13, 7 /

!     codelc    Table with main keywords
!             Possible values:
!             1:  MATRIX
!             2:  CREATE
!             3:  ESSENTIAL
!             4:  COEFFICIENTS
!             5:  CHANGE_COORDINATES
!             6:  SOLVE
!             7:  NONLINEAR_EQUATIONS
!             8:  END_OF_SEPRAN_INPUT
!             9:  COMPLEX
!            10:  OUTPUT
!            11:  STRUCTURE
!            12:  DERIVATIVES
!            13:  INTEGRALS
!            14:  BOUNDARY_INTEGRAL
!            15:  TIME_INTEGRATION
!            16:  ADAPT_BOUNDARY
!            17:  ADAPT_MESH
!            18:  STATIONARY_FREE_BOUNDARY
!            19:  INSTATIONARY_FREE_BOUNDARY
!            20:  CONTACT
!            21:  LOOP_INPUT
!            22:  CHANGE COEFFICIENTS
!            23:  EIGENVALUES
!            24:  CAPACITY
!            25:  INVERSE_PROBLEM
!            26:  ENTHALPY_INTEGRATION
!            27:  PROBLEM
!            28:  REFINE
!            29:  PRESSURE_CORRECTION
!            30:  NAVIER_STOKES
!            31:  LEVEL_SET
!            32:  OBSTACLE_MESH
!            33:  BEARING
! ======================================================================
!
      call eropen ( 'readal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readal'
  1      format ( a, 1x, (10i12) )

      end if
      if ( ierror/=0 ) go to 1000

      call sepfillkmesh ( kmesh )

!     --- Set some general constants

      bouncn = .false.
      nooutput = .false.

!     --- Store the first input read for problem into the block 27
!         Only if manag(11)>=0

      if ( manag(11)>=0 ) call sepprobstart ( iseqprob )
      if ( ierror/=0 ) go to 1000

      finished = .false.

!     --- While loop finished

10    do while ( .not. finished )

!     --- Read next keyword

         if ( manag(12)>0 ) then

!        --- manag(12)>0, the keyword structure has been found

            icode = manag(12)
            manag(12) = 0

         else

!        --- Standard case

            istop  = 1
            ityprd = 1
            call asknxt

            if ( ityprd==0 ) then

!           --- end of file has been found

               finished = .true.
               go to 10

            else if ( ityprd/=4 ) then

!           --- Error: not a keyword found

               call errsub ( 1857, 0, 0, 0 )
               call errcrd
               finished = .true.
               go to 10

            end if

!           --- Keyword found, interpret

            ierpri = 0
            nncodemain = nncodelc
            codemain(1:nncodemain) = codelc
            ncodemain(1:nncodemain) = ncodelc
            ncodmxmain(1:nncodemain) = ncodmxlc
            icode = sepmainkeyw()

         end if  ! ( manag(12)==1 )

         if ( debug ) write(irefwr,1) 'icode', icode

         select case ( icode )

            case(1)

!           --- icode = 1, keyword MATRIX found
!               read input for subroutine COMMAT until END is found

               call ercomm ( .false. )

            case(2)

!           --- icode = 2, keyword CREATE found
!               read input for subroutine CREATE until END is found

               call ercrea ( icode, .false. )

            case(3)

!           --- icode = 3, keyword ESSENTIAL found
!               read input for subroutine PRESDF until END is found

               call ercrea ( icode, .false. )
               bouncn = .true.

            case(4, 9, 22)

!           --- icode = 4, 9 or 22, keyword COEFFICIENTS or COMPLEX or
!               CHANGE (COEFFICIENTS) found
!               read input for subroutine FILCOF until END is found

               call ercofs ( icode )

            case(5)

!           --- icode = 5, keyword CHANGE_COORDINATES found
!               read input for subroutine MSHCHCOR until END is found

               call erchcoor

            case(6)

!           --- icode = 6, keyword SOLVE found
!               read input for subroutine LINSOL until END is found

               call ersolv

            case(7)

!           --- icode = 7, keyword NONLINEAR found
!               read input for subroutine NLNPRB until END is found

               call ernlns

            case(8)

!           --- icode = 8, keyword END_OF_SEPRAN_INPUT found

               finished = .true.

            case(10)

!           --- icode = 10, keyword OUTPUT found

               call eroutp

            case(11)

!           --- icode = 11, keyword STRUCTURE found

               call ercomp ( manag )

            case(12)

!           --- icode = 12, keyword DERIVATIVES found
!               read input for subroutine DERIV until END is found

               call erderi

            case(13)

!           --- icode = 13, keyword INTEGRALS found
!               read input for subroutine INTEGR until END is found

               call erinti

            case(14)

!           --- icode = 14, keyword BOUNDARY_INTEGRAL found
!               read input for subroutine INTBND until END is found

               call erintb

            case(15)

!           --- icode = 15, keyword TIME_INTEGRATION found
!               read input for subroutine TIMPRB until END is found

               call ertimd ( nooutput )

            case(16)

!           --- icode = 16, keyword ADAPT_BOUNDARY found
!               read input for subroutine ADAPBOUN until END is found

               call eradapbn

            case(17)

!           --- icode = 17, keyword ADAPT_MESH found
!               read input for subroutine ADAPMESH until END is found

               call eradapms

            case(18)

!           --- icode = 18, keyword STATIONARY_FREE_BOUNDARY found
!               read input for subroutine statfree until END is found

               call erstatfr

            case(19)

!           --- icode = 19, keyword INSTATIONARY_FREE_BOUNDARY found
!               read input for subroutine instfree until END is found

               call erinstfr

            case(20)

!           --- icode = 20, keyword CONTACT found
!               read input for subroutine contact until END is found

               call ercontct

            case(21)

!           --- icode = 21, keyword LOOP_INPUT found
!               read input for subroutine toloop until END is found

               call erloop

            case(23)

!           --- icode = 23, keyword EIGENVALUES found
!               read input for subroutine eigval until END is found

               call ereigval

            case(24)

!           --- icode = 24, keyword CAPACITY found
!               read input for subroutine compcapcity until END is found

               call ercapacity

            case(25)

!           --- icode = 25, keyword INVERSE_PROBLEM found
!               read input for subroutine compcapacity until END is found

               call erinverse

            case(26)

!           --- icode = 26, keyword ENTHALPY_INTEGRATION found
!               read input for subroutine enthalpyint until END is found

               call erenthalpy

            case(27)

!           --- icode = 27, keyword PROBLEM found
!               read input for subroutine PROBDF until END is found
!               Do not store anything in ISEPIN, but extend kprob

               call erprob ( iseqprob )

            case(28)

!           --- icode = 28, keyword REFINE found
!               read input for subroutine refinement

               call errefine

            case(29)

!           --- icode = 29, keyword PRESSURE_CORRECTION found
!               read input for subroutine pressure correction

               call erpresscor

            case(30)

!           --- icode = 30, keyword NAVIER-STOKES found
!               read input for subroutine navstokes

               call ernavstokes

            case(33)

!           --- icode = 33, keyword BEARING found
!               read input for subroutine bearing

               call erbearing

            case default

!           --- Error: word not recognized as main keyword

               call errchr ( string(1:nchar), 1 )
               do i = 1, nncodelc
                  call errchr ( codelc(i), i+1 )
               end do  ! i = 1, nncodelc
               call errsub ( 648, 0, 0, nncodelc+1 )
               call errcrd
               go to 1000

         end select  ! case ( icode )
         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finished )

!     --- Fill structure of program in ISEPIN in case no structure is given

      call ini016 ( nooutput )
      if ( ierror/=0 ) go to 1000

!     --- Create default input for boundary conditions for nprob solution
!         vectors in case no input is found

      if ( manag(11)==0 .and. .not. bouncn ) &
         call ercrea ( 5, .true. )

1000  if ( debug) then

!     --- Debug information

         write(irefwr,*) 'End readal'

      end if
      call erclos ( 'readal' )

      end
