      subroutine readboolcheck ( inpsep, operators, icount, jcount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-12-2013 Completely rewritten
!        version  1.0  date 08-02-2008
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check contents of array inpsep for boolean expression
!     Reorder these contents
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     boolean
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), icount, jcount
      character (len=*) operators(*)

!     icount         i    Length of array inpsep at input
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readboolexpr
!     jcount         o    Length of array inpsep at output, i.e.
!                         after cleaning
!     operators      i    Character array containing the
!                         logical operators recognized
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numreals, i, iopen
      logical firstreal

!     firstreal      If true the first real in a relation must still be read
!     i              Counting variable
!     iopen          Contains the number of open openings brackets
!                    Hence not yet closed
!     numreals       Number of reals or real expressions found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRININLARGE    Print 1d integer vector (large numbers)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2927   operator not preceded by expression
!    2932   Open bracket ( in boolean expression preceded by close bracket )
!    2933   Open bracket ( in boolean expression preceded by number, &
!           expression or operator
!    2934   Closing bracket ) in boolean expression preceded by number, &
!           expression or operator
!    2935   Expression or real found that is preceded by either a real or
!           expression, or boolean
!    2936   "Second" expression or real found not precded by boolean operator
!    2937   More opening than closing brackets found in boolean expression
!    2938   More closing than opening brackets found in boolean expression
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readboolcheck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readboolcheck'
         call prininlarge ( inpsep, icount, 'inpsep' )
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount = 2
      numreals = 0
      firstreal = .true.
      iopen = 0

      if ( inpsep(jcount)>1000 ) then

!     --- Boolean expression starts with number

         firstreal = .false.

      else if ( inpsep(jcount)==11 ) then

!     --- First item is (

         iopen = 1

      end if  ! ( inpsep(jcount)>1000 )

      do i = 3, icount

         if ( debug ) write(irefwr,1) 'i, inpsep(i)', i, inpsep(i)
         if ( inpsep(i)>=1 .and.  inpsep(i)<=6 ) then

!        --- Boolean operator found that must preceded by number

            if ( i>3 .and. &
                 inpsep(i-1)>=0 .and. inpsep(i-1)<=1000 ) then

!           --- Operator not preceded by number

               call errchr ( operators(inpsep(i)), 1 )
               call errsub ( 2927, 0, 0, 1 )

            end if  ! ( inpsep(i)>=1 .and.  inpsep(i)<=6 )
            jcount = jcount+1
            inpsep(jcount) = inpsep(i)  ! copy

            if ( debug ) &
               write(irefwr,1) 'boolean operator: jcount', &
                                jcount, inpsep(jcount)

         else  if ( inpsep(i)>=7 .and.  inpsep(i)<=9 ) then

!        --- Boolean operator found that must preceded by boolean

            jcount = jcount+1
            inpsep(jcount) = inpsep(i)  ! copy
            if ( debug ) &
               write(irefwr,1) 'boolean operator/2: jcount', &
                                jcount, inpsep(jcount)

         else  if ( inpsep(i)==11 ) then

!        --- ( found, may not be preceded by number )

            if ( inpsep(i-1)==12 ) then

!           --- ( preceded by )

               call errsub ( 2932, 0, 0, 0 )

            else if ( i>3 .and. inpsep(i-1)/=11 .and. &
                      ( inpsep(i-1)<0 .or. inpsep(i-1)>1000 ) ) then

!           --- ( preceded by operator or expression or real

               call errsub ( 2933, 0, 0, 0 )

            else

!           --- store information

               iopen = iopen+1
               jcount = jcount+1
               inpsep(jcount) = inpsep(i)  ! copy

            end if  ! ( inpsep(i-1)==12 )
            if ( debug ) &
               write(irefwr,1) '(: jcount', &
                                jcount, inpsep(jcount)

         else  if ( inpsep(i)==12 ) then

!        --- ) found, may not be preceded by operator or number

            if ( inpsep(i-1)==11 ) then

!           --- ) preceded by (, skip both

               jcount = jcount-1

            else if ( inpsep(i-1)>0 .and. inpsep(i-1)<9 ) then

!           --- ) preceded by operator, error

               call errsub ( 2934, 0, 0, 0 )

            else

!           --- store information

               iopen = iopen-1
               jcount = jcount+1
               inpsep(jcount) = inpsep(i)  ! copy

            end if  ! ( inpsep(i-1)==11 )
            if ( debug ) &
               write(irefwr,1) '): jcount', &
                                jcount, inpsep(jcount)

         else

!        --- inpsep(i)<0 or>1000, hence expression or real found
!            This must be either the first item in a boolean relation
!            or the second one with a boolean result

            if ( debug ) then
               write(irefwr,1) 'real or expression: i', i
               write(irefwr,*) 'firstreal ', firstreal
            end if  ! ( debug )

            if ( firstreal ) then

!           --- This must be the first real, so it must be preceded
!               by nothing or a boolean operator and or or

               if ( i>3 .and. &
                    ( inpsep(i-1)>0 .and. inpsep(i-1)<7 .or. &
                      inpsep(i-1)<0 .or. inpsep(i-1)>=1000 ) ) then

!              --- Error wrong operator before first real or expression

                  call errsub ( 2935, 0, 0, 0 )

               else

!              --- Store information

                  jcount = jcount+1
                  inpsep(jcount) = inpsep(i)
                  firstreal = .false.

               end if  ! ( i>3 .and. .. )
               if ( debug ) &
                  write(irefwr,1) 'first real: jcount', &
                                   jcount, inpsep(jcount)

            else

!           --- This must be the second real, so it must be preceded
!               by boolean operator

               if ( inpsep(i-1)>=1 .and. inpsep(i-1)<=6 ) then

!              --- Store information

                  jcount = jcount+1
                  inpsep(jcount) = inpsep(i)

                  firstreal = .true.
                  numreals = numreals+1

               else

!              --- Error second real or expression not preceded by operator

                  call errsub ( 2936, 0, 0, 0 )

               end if
               if ( debug ) &
                  write(irefwr,1) 'second real: jcount', &
                                   jcount, inpsep(jcount)

            end if  ! ( firstreal )

         end if  ! ( inpsep(i)>=1 .and.  inpsep(i)<=6 )

      end do  ! i = 3, icount

      if ( iopen>0 ) then

!     --- Too many open brackets

         call errsub ( 2937, 0, 0, 0 )

      else if ( iopen<0 ) then

!     --- Too many closing brackets

         call errsub ( 2938, 0, 0, 0 )

      end if  ! ( iopen>0 )

1000  call erclos ( 'readboolcheck' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount, jcount', icount, jcount
         call prininlarge ( inpsep, jcount, 'inpsep' )
         write(irefwr,*) 'End readboolcheck'

      end if  ! ( debug )

      end
