      subroutine readc0curv ( iwork, table, tabout, tabmin, tabmax, &
                              code )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret all curve information read by readc0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*)
      double precision table(*), tabout(*), tabmin(*), tabmax(*)
      character (len=*) code(*)

!     code           i    Table with keywords
!     iwork         i/o   integer work array, see readc0
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut, m

!     iposut         Position in array tabout
!     m              number of curves read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     891   In one record more than one of the combinations:
!           POINTS, CURVES, SURFACES, NODES, ELEMENTS or GROUP is found
!    1045   Too much points found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readc0curv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0curv'
         call prinrl ( table, 32, 'table' )
         call prinrl ( tabmin, 32, 'tabmin' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iwork(2)/=0 ) then
         call errsub ( 891, 0, 0, 0 )
         call errcrd
      end if

      iwork(2) = 22  ! series of curves

      if ( table(11)>0d0 ) then

!     --- table(11) >0, standard case

         iwork(3) = nint(tabmin(11))
         if ( table(12)>0 ) iwork(3) = nint(table(12))
         iposut = nint(table(11))
         m = nint(tabmax(11))
         iwork(4) = m
         if ( debug ) call prinrl ( tabout(iposut), m, 'tabout' )
         if ( m>20 ) then
            call errchr ( code(10), 1 )
            call errint ( m, 1 )
            call errint ( 2, 2 )
            call errsub ( 1045, 2, 0, 1 )
            call errcrd
         end if
         iwork(5:4+m) = nint(tabout(iposut:iposut+m-1))

      else  ! ( table(32)>0d0 )

!     --- table(32) >0, special case: outer curves

         iwork(2) = 2
         iwork(3) = -1
         iwork(4) = -1
         iwork(5) = -1

      end if  ! ( table(11)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'm', m
         call prinin ( iwork, 5, 'iwork' )
         write(irefwr,*) 'End readc0curv'

      end if  ! ( debug )
      call erclos ( 'readc0curv' )

      end subroutine readc0curv
