      subroutine readdebugparms
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 09-12-2004 Extra parameter
!        version  1.6  date 21-04-2004 Extra parameter
!        version  1.5  date 29-12-2002 Extension common
!
!   copyright (c) 2001-2004  "I>=ieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "I>=ieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read the debug parameters from the input file until an end is found
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     debug
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelog=35, nncodeint=3, &
                            nncode1=nncodelog+nncodeint+1
      integer ncode1(nncode1), ncodmx1(nncode1)
      character (len=20) code1(nncode1)
      logical finished

!     code1          Table with keywords
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     nncodeint      Number of other parameters in code1
!     nncodelog      Number of debug parameters in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error mess>=
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!      The following records are recognized
!
!      check_hanging_nodes
!      checkobstconsistency
!      checksubtetrahedrons
!      debug_buildbf
!      debug_compstat01
!      debug_erintpol
!      debug_extfreesubr
!      debug_hollowmesh
!      debug_initfree
!      debug_mshcrt
!      debug_mshfillkmeshad
!      debug_mshobsfl
!      debug_mshobstmarkpt
!      debug_mshobstmesh
!      debug_mshobstpoint
!      debug_mshsortel3d
!      debug_mshsrf
!      debug_mshvlm
!      debug_pltrac
!      debug_prinmtbf
!      debug_prob01crobst
!      debug_probdfbf
!      debug_probflkprobaa
!      debug_probskippoint
!      debug_prorea
!      plote>=
!      plotobstacles
!      plotskipelements
!      printobstacles
!      printskipelements
!      trace
!
!      start_surface
!      number_of_rotations
!
!      write>=mview
!      write_iinput_mesh
!      end
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    2551   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'DEBUG_MSHSORTEL3D', 'DEBUG_MSHCRT', &
                  'DEBUG_MSHFILLKMESHAD', 'DEBUG_MSHSRF', &
                  'DEBUG_PLTRAC', 'DEBUG_BUILDBF', &
                  'DEBUG_PROB01CROBST', 'DEBUG_PROBDFBF', &
                  'DEBUG_PROBFLKPROBAA', &
                  'DEBUG_PROREA', 'DEBUG_PRINMTBF', 'DEBUG_COMPSTAT01', &
                  'DEBUG_EXTFREESUBR', 'DEBUG_HOLLOWMESH', &
                  'DEBUG_INITFREE', 'PLOTOBSTACLES', 'PRINTOBSTACLES', &
                  'PLOTSKIPELEMENTS', 'PRINTSKIPELEMENTS', &
                  'DEBUG_MSHOBSFL', 'DEBUG_MSHOBSTMARKPT', &
                  'DEBUG_PROBSKIPPOINT', 'CHECK_SIGN_JACOBIAN', &
                  'DEBUG_MSHOBSTMESH', 'DEBUG_ERINTPOL', &
                  'CHECK_HANGING_NODES', 'PLOTEDGES', 'TRACE', &
                  'CHECKSUBTETRAHEDRONS', 'DEBUG_MSHOBSTPOINT', &
                  'DEBUG_MSHVLM', 'WRITE_GEOMVIEW', 'WRITE_IINPUT_MESH', &
                  'CHECKOBSTCONSISTENCY', 'WRITE_MATLAB_MAT', &
                  'START_SURFACE', 'NUMBER_OF_ROTATIONS', &
                  'FORMAT_MATLAB', &
                  'END' /
      data ncode1 / 17, 12, 20, 12, 12,   13, 18, 14, 19, 12, &
                   14, &
                   16, 17, 16, 14,  8,    9, 10, 11, 14, 19, &
                   19, 19, 17, 14, 19,    9,  5, 20, 14, 14, &
                   18, 17, 13, 10, 13,   19, 10,  3 /
      data ncodmx1 / 17, 12, 20, 12, 12,   13, 18, 14, 19, 12, &
                    14, &
                    16, 17, 16, 14, 13,   14, 16, 17, 14, 19, &
                    19, 19, 17, 14, 19,    9,  5, 20, 14, 14, &
                    18, 17, 20, 16, 13,   19, 13,  3 /

!     code    Table with keywords (first all logicals)
!             Contents
!             1:  DEBUG_MSHSORTEL3D
!             2:  DEBUG_MSHCRT
!             3:  DEBUG_MSHFILLKMESHAD
!             4:  DEBUG_MSHSRF
!             5:  DEBUG_PLTRAC
!             6:  DEBUG_BUILDBF
!             7:  DEBUG_PROB01CROBST
!             8:  DEBUG_PROBDFBF
!             9:  DEBUG_PROBFLKPROBAA
!            10:  DEBUG_PROREA
!            11:  DEBUG_PRINMTBF
!            12:  DEBUG_COMPSTAT01
!            13:  DEBUG_EXTFREESUBR
!            14:  DEBUG_HOLLOWMESH
!            15:  DEBUG_INITFREE
!            16:  PLOTOBSTACLES
!            17:  PRINTOBSTACLES
!            18:  PLOTSKIPELEMENTS
!            19:  PRINTSKIPELEMENTS
!            20:  DEBUG_MSHOBSFL
!            21:  DEBUG_MSHOBSTMARKPT
!            22:  DEBUG_PROBSKIPPOINT
!            23:  CHECK_SIGN_JACOBIAN
!            24:  DEBUG_MSHOBSTMESH
!            25:  DEBUG_ERINTPOL
!            26:  CHECK_HANGING_NODES
!            27:  PLOTEDGES
!            28:  TRACE
!            29:  CHECKSUBTETRAHEDRONS
!            30:  DEBUG_MSHOBSTPOINT
!            31:  DEBUG_MSHVLM
!            32:  WRITE_GEOMVIEW
!            33:  WRITE_IINPUT_MESH
!            34:  CHECKOBSTCONSISTENCY
!            35:  WRITE_MATLAB_MAT
!            36:  START_SURFACE
!            37:  NUMBER_OF_ROTATIONS
!            38:  FORMAT_MATLAB
!            39:  END
! ======================================================================
!
      call eropen ( 'readdebugparms' )
      finished = .false.

!     --- Copy code1 into code

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      do while ( .not. finished )
         ityprd = 1
         ispecl = 9
         call asknxt
         if ( ityprd/=4 ) then

!        --- Not a string found, stop

            finished  = .true.
            call errsub ( 2551, 0, 0, 0 )
            call errcrd

         else

!        --- String found, interpret

            iretur = sepsubkeyw()
            if ( iretur<=0 .or. iretur>=nncode ) then

!           --- incorrect value or end found

               finished  = .true.

            else if ( iretur<=nncodelog ) then

!           --- logical found

               select case ( iretur )
               case(1)
                  debug_mshsortel3d = .true.
               case(2)
                  debug_mshcrt = .true.
               case(3)
                  debug_mshfillkmeshad = .true.
               case(4)
                  debug_mshsrf = .true.
               case(5)
                  debug_pltrac = .true.
               case(6)
                  debug_buildbf = .true.
               case(7)
                  debug_prob01crobst = .true.
               case(8)
                  debug_probdfbf = .true.
               case(9)
                  debug_probflkprobaa = .true.
               case(10)
                  debug_prorea = .true.
               case(11)
                  debug_prinmtbf = .true.
               case(12)
                  debug_compstat01 = .true.
               case(13)
                  debug_extfreesubr = .true.
               case(14)
                  debug_hollowmesh = .true.
               case(15)
                  debug_initfree = .true.
               case(16)
                  plotobst = .true.
               case(17)
                  printobst = .true.
               case(18)
                  plotskipel = .true.
               case(19)
                  printskipel = .true.
               case(20)
                  debug_mshobsfl = .true.
               case(21)
                  debug_mshobstmarkpt = .true.
               case(22)
                  debug_probskippoint = .true.
               case(23)
                  checksign  = .true.
               case(24)
                  debug_mshobstmesh = .true.
               case(25)
                   debug_erintpol = .true.
              case(26)
                  check_hanging_nodes = .true.
               case(27)
                  plotedges = .true.
               case(28)
                  trace = .true.
               case(29)
                  checksubtetrahedrons = .true.
               case(30)
                  debug_mshobstpoint = .true.
               case(31)
                  debug_mshvlm = .true.
               case(32)
                  write_geomview = .true.
               case(33)
                  write_iinput_mesh = .true.
               case(34)
                  checkobstconsistency = .true.
               case(35)
                  write_matlab_mat = .true.
               end select  ! case ( iretur )

            else

!           --- integer found, read value

               call asknxt
               if ( ityprd==2 ) then

!              --- integer found, store in intdebugarray

                  select case (iretur-nncodelog)
                  case(1)
                     startsurface = nint(realrd)
                  case(2)
                     num_of_rotat = nint(realrd)
                  case(3)
                     format_matlab = nint(realrd)
                  end select  ! case (iretur-nncodelog)

               else

!              --- Not an integer number found

                  call errchr ( code(iretur), 1 )
                  call errsub ( 574, 0, 0, 1 )
                  call errcrd

               end if

            end if

         end if

      end do
      ityprd = 1
      ispecl = 0
      call erclos ( 'readdebugparms' )
      end
