      subroutine readdr ( iinder, isubr, iseqnr, iprob, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 10-10-2012 Extension with points and surfaces
!        version  3.0  date 26-08-2011 Remove tabout from parameter list
!        version  2.3  date 13-11-2010 error 1488 --> errkeywexcl
!
!   copyright (c) 1993-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the subroutine DERIVS
!     The input is stored in array IINDER
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinder(*), isubr, iseqnr, iprob, nprob

!    iinder  o   The integer input read is stored in array IINDER
!                Array IINDER is filled as follows:
!                1  ICHOIS, indicates the type of output vector
!                   Possibilities:
!                  -1:   No output vector required
!                        In this case ITYPE_BOUN must be>0
!                   0:   Output vector is of the type of solution vector
!                   1:   Output vector is a vector of special structure
!                        (type 115)
!                   2:   Output vector is a vector of special structure
!                        (type 116), i.e. defined per element
!                        No averaging takes place.
!                   3:   The output vector is a vector of special structure
!                        with sequence number  ivec.  The output vector is
!                        made by adding quantities in common nodal points, &
!                        hence DERIV acts as if a right-hand-side vector is
!                        built.
!                   4:   The input vector is supposed to be a vector
!                        corresponding to quadratic elements, of which only
!                        the vertices are filled.
!                        With this value of ICHOIS the output vector consists
!                        of a vector filled in all points, where the midpoints
!                        are created by linear interpolation.
!                        In this case it is necessary that there is at least
!                        a solution vector or a vector of special structure
!                        defined such that one unknown is available in all
!                        nodes.
!                   5:   Output vector is a vector of special structure
!                        (type 126), i.e. defined per element
!                        Each quantity is stored for each node per element
!                        No averaging takes place.
!                   6:   Output vector is a vector of special structure
!                        (type 129), i.e. defined per integration point per
!                        element
!                        No averaging takes place.
!                   7:   The reaction force must be computed, hence the result
!                        is an output vector of the type of the solution
!                        vector created by S u
!                   11:  Compute integral by trapezoid rule of
!                        "grad u" . n at curve
!                        grad u is defined by icheld, it may be for example
!                        A grad u
!                   Default value: 0
!                2  ICOMPL, indicates if the output vector is real (0) or
!                   complex (1)
!                   Default value: 0
!                3  ICHELD, choice parameter for the element subroutines
!                   Default value: 1
!                4  IX,  choice parameter for the element subroutines
!                   Default value: 1
!                5  JDEGFD,  choice parameter for the element subroutines
!                   Default value: 1
!                6  IVEC,   Sequence number for the type of vector of special
!                           structure, when ICHOIS = 1, or
!                           number of degrees of freedom to be computed per
!                           element when ICHOIS = 2
!                   Default value: 1
!                7  NUMVEC, Number of vectors that are stored in array INPVEC
!                           Compare with subroutine BUILD
!                           If NUMVEC=0, it is supposed that one vector should
!                           be used
!                   Default value: nprob
!                8  CLEARVECTOR, This parameter indicates whether the output
!                           vector must be cleared before adding the new
!                           results (0), or that the results are added to the
!                           previous vector
!                   Default value: 0
!                9  SEQ_INPUTVEC1 Indicates the sequence number of the first
!                           vector in array INPVEC
!                   Default value: 1
!               10  SEQ_INPUTVEC2 Indicates the sequence number of the second
!                           vector in array INPVEC
!                   Default value: 1
!               11  ALLELEMENTS  Indication if all standard element
!                           sequence numbers are used (0) or not (>0)
!                           In that case the positions ALLELEMENTS to
!                           ALLELEMENTS+NELGRP-1
!                           (NELGRP is number of element groups)
!                           are considered as an array IELHLP to be
!                           filled by the user in the following way:
!                           In IELHLP(IELGRP)  ( 1<=IELGRP<=NELGRP)
!                           the user must store, whether elements with
!                           standard element sequence number IELGRP
!                           are added to the large matrix and large vector or
!                           not.
!                           When IELHLP(IELGRP) = 0 the elements with standard
!                           element sequence number IELGRP are skipped, when
!                           IELHLP(IELGRP) = 1 these elements are added.
!                   Default value: 0
!               12: ITYPE_BOUN  Indicates the type of boundary integral to
!                           be computed. Possible values:
!                   0:   No boundary integral is computed
!                   2:   / u.n ds, where u is the vector just computed
!                        Hence the derivative must be a vector
!                        In this case ICHOIS must be -1
!                   Other values are not yet available
!                   Default value: 0
!               13: IRULE_BOUN  Indicates the type of integration rule to be
!                        applied. Possible values:
!                   1:   trapezoid rule,  cartesian co-ordinates
!                   Other values are not yet available
!                   Default value: 0
!               14: INUMCURVES Is only used if iinder(13)>0 or iinder(2)=7
!                           It indicates the number of
!                           curves to be used, See als array ICURVS
!                   Default value: 0
!                The positions 15-14+nelgrp are reserved for array IELHLP
!                The positions 15+nelgrp-114+nelgrp are reserved for array
!                ICURVS
!               15: ITYPEBOUN Is only used if iinder(13)>0 or iinder(2)=7
!                   The following values are available
!                   1:   the boundary consists of points
!                   2:   the boundary consists of curves
!                   3:   the boundary consists of surfaces
!                   Default value: 2
!               16-19: Not yet used
!    iprob  i/o  Problem number
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine READDR is called
!                Possible values:
!                1:  DERIVS,  read record with DERIVATIVES
!                2:  ERDERI,  do not read record with DERIVATIVES
!    nprob   i   Number of problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=18, nncod2=11, nparm=19
      integer ncode1(nncode1), ncode2(nncod2), ncodmx1(nncode1)
      integer k, m, ipos, maxtype, itypeboun, i
      character (len=19) code1(nncode1)
      character (len=26) code2(nncod2)

!     code1          Table with keywords corresponding to options
!     code2          Table with the main keyword as well as the keywords
!                    following TYPE_OUTPUT
!     i              Loop variable
!     ipos           Position in array tabout
!     itypeboun      See iinder(15)
!     k              Counting variable
!     m              Number of curves read
!     maxtype        Last position of subkeywords for type_output in code2
!     ncode1         Table containing the number of significant characters in
!                    the keyword table code1
!     ncode2         See ncode1, now for table code2
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in table code2
!     nncode1        Number of entries in table code1
!     nparm          Number of parameters in iinder
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADPROBSEQ Read sequence number iseqnr and problem number iprob
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readdr requires input from the standard input file
!     The following input is recognised:
!     DERIVATIVES [, SEQUENCE_NUMBER = s][, PROBLEM = p] (mandatory):
!        start of input
!      optional records:
!        TYPE_OUTPUT = t, options (at the same line)
!        where t may take one of the following values:
!           t = SOLUTION                            (default)
!               SPECIAL
!               ELEMENTWISE
!               INTERPOLATE
!               ELEMENT_NODES
!               ELEMENT_INTEGRATION_POINTS
!               REACTION_FORCE
!        and options may be chosen from the following list:
!           COMPLEX                                (default: real)
!           REAL                                   (default: real)
!           IVEC = s                               (default: 1)
!        ICHELD = s                               (Default 1 )
!        ELEMENT_GROUPS = (s1, s2, ...)           (Default all)
!        SKIP_ELEMENT_GROUPS = (s1, s2, ... )     (Default none)
!        IX = s                                   (Default 1)
!        DEGREE_OF_FREEDOM = d                    (Default 1)
!        SEQ_INPUT_VECTOR [k] = i                 (Default 1)
!           k = 1 or 2
!        NUMVEC = n                               (Default 1)
!        CLEAR                                    (Default clear)
!        NO_CLEAR                                 (Default clear)
!        BOUNDARY_INTEGRAL (C1, C4, ... )         (Default no boundary integral)
!        CURVES (C1, C4, ... )                    (Default no curves)
!        POINTS (P1, P4, ... )
!        SURFACES (S1, S4, ... )
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to SOLVE
!    1900   Both element groups and skip element groups given
!    2056   Curves found, but not output vector is of type reaction force
!    2057   output vector is of type reaction force, but not curves found
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (derivatives)
!    Read rest of first record (sequence number, problem number )
!    Read the rest of the records by intx02 unitl END is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'TYPE_OUTPUT', 'ICHELD', 'ELEMENT_GROUPS', &
                  'SKIP_ELEMENT_GROUPS', 'IX', &
                  'DEGREE_OF_FREEDOM', 'CLEAR', 'NO_CLEAR', 'NUMVEC', &
                  'COMPLEX', 'REAL', 'IVEC', 'SEQ_INPUT_VECTOR', &
                  'SEQ_INPUT_VECTOR', 'BOUNDARY_INTEGRAL', 'POINTS', &
                  'CURVES', 'SURFACES' /
      data ncode1 / 7, 4, 4, 4, 2,    3, 3, 4, 4, 4, &
                   4, 3, 7, 7,10,    4, 4, 4 /
      data ncodmx1/11, 6,14,19, 2,   17, 5, 8, 6, 7, &
                   4, 4,16,16,17,    6, 6, 8 /
      data code2 / 'SOLUTION', 'SPECIAL', 'ELEMENTWISE', &
                   'RIGHT_HAND_SIDE', 'INTERPOLATE', 'ELEMENT_NODES', &
                   'ELEMENT_INTEGRATION_POINTS', 'REACTION_FORCE', &
                   'P', 'C', 'S' /
      data ncode2 / 3, 4, 4, 4,  5, 11, 11, 5, 1, &
                    1, 1 /

!     code1   Table with keywords corresponding to options
!             Values:
!             1:  TYPE_OUTPUT
!             2:  ICHELD
!             3:  ELEMENT_GROUPS
!             4:  SKIP_ELEMENT_GROUPS
!             5:  IX
!             6:  DEGREE_OF_FREEDOM
!             7:  CLEAR
!             8:  NO_CLEAR
!             9:  NUMVEC
!            10:  COMPLEX
!            11:  REAL
!            12:  IVEC
!            13:  SEQ_INPUT_VECTOR
!            14:  SEQ_INPUT_VECTOR
!            15:  BOUNDARY_INTEGRAL
!            16:  POINTS
!            17:  CURVES
!            18:  SURFACES
!     code2   Table with the main keyword as well as the keywords
!             following TYPE_OUTPUT
!             Values:
!             1: SOLUTION
!             2: SPECIAL
!             3: ELEMENTWISE
!             4: RIGHT_HAND_SIDE
!             5: INTERPOLATE
!             6: ELEMENT_NODES
!             7: ELEMENT_INTEGRATION_POINTS
!             8: REACTION_FORCE
!             9: P
!            10: C
!            11: S
! ======================================================================
!
      call eropen ( 'readdr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readdr'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize iinder

      iinder(1:nparm+nelgrp+100) = 0
      iinder(3:6) = 1
      iinder(7) = nprob
      iinder(9) = 1

      if ( isubr==1 ) then

!     --- Call by DERIVS, read first record  (DERIVATIVES)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'DERIVATIVES', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:5)/='DERIV' ) then
            call errchr ( 'DERIVATIVES', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadprobseq ( iprob, iseqnr, nprob ) ! read iseqnr and iprob
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 ) ! copy code1 in code
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend                  ! Fill END as main keyword

!     --- Initialize itype, table tabmin and tabmax

      table(1:nncode) = -1d0
      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 0d0

      maxtype = 8
      itype(1) = 5          ! TYPE_OUTPUT
      tabmin(1) = 1d0
      tabmax(1) = maxtype

      tabmax(2) = 1d3       ! ICHELD

      itype(3:4) = 11       ! ELEMENT_GROUPS / SKIP_ELEMENT_GROUPS
      tabmin(3:4) = nelgrp
      tabmax(3:4) = nelgrp

      tabmax(5:6) = 1d1     ! IX / DEGREE_OF_FREEDOM

      itype(7:8) = 2       ! CLEAR / NO_CLEAR
      tabmin(7:8) = 0d0

      tabmax(9) = 1d2       ! NUMVEC

      itype(10:11) = 2        ! COMPLEX / REAL
      tabmin(10:11) = 0d0

      tabmax(12) = 1d2       ! IVEC

      itype(13) = 12         ! SEQ_INPUT_VECTOR
      tabmax(13) = 2d0       ! iseq
      tabmax(14) = 1d2       ! vector

      itype(15:18) = 18      ! BOUNDARY_INTEGRAL POINTS/CURVES/SURFACES
      table(15) = -maxtype-2
      do i = 1, 3
         table(15+i) = -maxtype-i
      end do  ! i = 1, 3
      tabmin(15) = ncurvs
      tabmin(16) = nuspnt
      tabmin(17) = ncurvs
      tabmin(18) = nsurfs
      tabmax(15:18) = tabmin(15:18)
      istop = 2
      iretur = 0
      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncode, 'table' )

      istop = 0
      if ( table(1)>0d0 )  iinder(1) = nint(table(1))-1
      if ( table(10)>0d0 ) iinder(2) = 1
      if ( table(2)>0d0 )  iinder(3) = nint(table(2))
      if ( table(5)>0d0 )  iinder(4) = nint(table(5))
      if ( table(6)>0d0 )  iinder(5) = nint(table(6))
      if ( table(12)>0d0 ) iinder(6) = nint(table(12))
      if ( table(9)>0d0 )  iinder(7) = nint(table(9))
      if ( table(8)>0d0 )  iinder(8) = 1
      if ( table(13)>0d0 )  iinder(9) = nint(table(13))
      iinder(10) = iinder(9)
      if ( table(14)>0d0 )  iinder(10) = nint(table(14))

      if ( table(3)>0d0 .or. table(4)>0d0 ) then

!     --- Element groups must be skipped

         iinder(11) = nparm+2
         ipos  = nint(table(3))
         if ( table(3)>0d0 ) then

!        --- element groups = given

            do k = 1, nelgrp
               iinder(nparm+k) = 0
            end do
            do k = 1, nint(tabmax(3))
               iinder(nparm+nint(tabout(k+ipos-1))) = 1
            end do

         else

!        --- skip element groups = given

            iinder(nparm+1:nparm+nelgrp) = 1
            ipos = nint(table(4))
            do k = 1, nint(tabmax(4))
               iinder(nparm+nint(tabout(k+ipos-1))) = 0
            end do

         end if
         if ( table(3)>0d0 .and. table(4)>0d0 ) &
              call errsub ( 1900, 0, 0, 0 )

      else

         iinder(11) = 0

      end if

!     --- Check on mutually exclusive keywords

      do i = 17, 18

         if ( table(16)>0d0 .and. table(i)>0d0 ) &
            call errkeywexcl ( code, ncodmx, 16, i ) ! points

      end do  ! i = 17, 18

      if ( table(17)>0d0 .and. table(18)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 17, 18 )  ! curves, surfaces

      itypeboun = 0
      m = 0

      if ( table(15)>0d0 ) then

!     --- boundary integral found

         itypeboun = 2
         m = nint(tabmax(15))
         ipos = nint(table(15))

      end if  ! ( table(15)>0d0 )

      do i = 16, 18  ! points, curves, surfaces

         if ( table(15)>0d0 .and. table(i)>0d0 ) &
            call errkeywexcl ( code, ncodmx, 15, i ) ! boundary integral

         if ( table(i)>0d0 ) then

!        --- P, C or S found

            itypeboun = i-15
            m = nint(tabmax(i))
            ipos = nint(table(i))

         end if  ! ( table(i)>0d0 )

      end do  ! i = 16, 18

      if ( debug ) write(irefwr,1) 'itypeboun', itypeboun

      if ( itypeboun>0 ) then

!     --- boundary integral or curves found

         iinder(15) = itypeboun
         if ( nint(table(15))>0 ) then
            iinder(1)  = -1
            iinder(12) =  2
            iinder(13) =  1
         end if
         iinder(14) =  m

!        --- Store curves in iinder

         do k = 1, m
            iinder(nparm+nelgrp+k) = nint(tabout(ipos+k-1))
         end do

      end if
      if ( nint(table(15))==0 .and. itypeboun>0 ) then

!     --- Curves found, only allowed in combination with reaction force

         if ( iinder(1)/=7 ) then

!        --- Not reaction force found

            call errchr ( code(15+itypeboun), 1 )
            call errsub ( 2056, 0, 0, 1 )

         end if

      else if ( iinder(1)==7 .and. itypeboun==0 ) then

!     --- output vector is type reaction force, but not curves found

         call errsub ( 2057, 0, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinder, 20, 'iinder' )
         write(irefwr,*) 'End readdr'

      end if  ! ( debug )
      call erclos ( 'readdr' )

      end subroutine readdr
