      subroutine readexprclos ( levels, nlevel, icount, iwork, work, &
                                text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2013 Extra parameter text
!        version  1.1  date 20-02-2008 Correction for extra brackets
!        version  1.0  date 28-12-2005
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret last part of an expression, possibly closed by brackets
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer levels(*), nlevel, icount, iwork(*)
      double precision work(*)
      character (len=*) text

!     icount        i/o   Counter
!                         Defines present position in iwork
!     iwork         i/o   integer work array
!                         For a description see subroutine readexpression
!     levels        i/o   Array defining the starting addresses of each level
!                         Each time an opening bracket ( is found a new level is
!                         created, each time a closing bracket is found the
!                         level is decreased by one
!     nlevel        i/o   Number of levels stored in levels
!     text           i    String variable containing some text for error message
!     work          i/o   Work array
!                         work contains the numbers that must be used to apply
!                         the operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ncount, i
      double precision result

!     i              Counting variable
!     istart         First position used at a part
!     ncount         Counter
!     result         Real to store the result of an expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READEXPR       Read subexpression not containing any brackets and
!                    translate into a real or integer number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexprclos' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexprclos'
         write(irefwr,1) 'icount', icount
         call prinin ( levels, nlevel, 'levels' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      istart = levels(nlevel)
      ncount = icount-istart

      if ( debug ) then
         write(irefwr,*) 'istart, ncount ', istart, ncount
         call prinin ( iwork, istart+ncount-1, 'iwork' )
         call prinrl ( work, istart+ncount-1, 'work')
      end if

      if ( ncount>1 ) then

!     --- ncount>1, evaluate expression

         call readexpr ( work(istart), iwork(istart), ncount, &
                         result, text )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            write(irefwr,*) 'result ', result
            call prinin ( iwork, istart+ncount-1, 'iwork' )
            call prinrl ( work, istart+ncount-1, 'work')
            write(irefwr,*) 'istart, ncount ', istart, ncount
         end if

!        --- Clear end of iwork and work, &
!            Decrease icount and nlevel
!            Store result in work

         do i = 0, ncount-1
            iwork(istart+i) = 0
            work(istart+i) = 0d0
         end do
         icount = max(istart,2)
         work(icount-1) = result
         if ( icount==2 .and. iwork(1)==0 ) iwork(1) = 1

      else if ( ncount==1 .and. icount>2 )  then

!     --- ncount = 1

         if ( iwork(icount-1)==1 ) then

!        --- iwork(icount-1) = 1, hence addition
!            decrease icount and update work

            work(icount-2) = work(icount-1)
            iwork(icount-1) = 0
            icount = icount-1

         end if  ! ( iwork(icount-1)==1 )

      end if  ! ( ncount>1 )
      nlevel = nlevel-1

1000  call erclos ( 'readexprclos' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount, nlevel ', icount, nlevel
         call prinin ( levels, nlevel, 'levels' )
         call prinin ( iwork, icount-1, 'iwork' )
         call prinrl ( work, icount-1, 'work')
         write(irefwr,*) 'End readexprclos'

      end if

      end
