      subroutine readnlglob ( iinsol, rinsol, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-01-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read global options for non-linear equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinsol(*), isubr
      double precision :: rinsol(*)

!     iinsol        i/o   Contains information concerning the non-linear solver
!                         See readnl
!     isubr          i    Indicates the type of calling subroutine
!                         See readnl
!     rinsol        i/o   Contains information concerning the non-linear solver
!                         See readnl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     READNLFILL     Fill arrays itype2, tabmn2, tabmx2 for subroutine readnl
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readnlglob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnlglob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill code and codesub and related arrays

      call readnlfill

      table(1:) = -1d0
      table(1) = 20d0
      table(2) = 2d0
      table(3) = 10d0
      table(4) = 1d-3
      table(5:6) = 1d0
      if ( isubr==1 ) then
         table(7) = 0d0
      else
         table(7) = 1d0
      end if
      table(8) = 1d0
      table(9) = 0d0
      table(10:11) = 1d0
      table(14:16) = 0d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( iretur==-1 ) go to 1000
      iinsol(2) = nint(table(1))
      iinsol(3) = nint(table(3))
      iinsol(4) = max(1,nint(table(8))-1)  ! at_error
      iinsol(5) = nint(table(2))
      iinsol(6) = nint(table(6))-1
      iinsol(7) = nint(table(7))
      iinsol(9) = nint(table(16))   ! call_func
      iinsol(11) = nint(table(9))
      iinsol(13) = nint(table(10))-1
      iinsol(14) = nint(table(11))-1
      iinsol(17) = nint(table(14))
      iinsol(18) = nint(table(15))
      if ( table(4)>0d0 ) rinsol(1) = table(4)
      if ( table(5)>0d0 ) rinsol(2) = table(5)
      if ( iinsol(7)==0 .and. isubr/=1 ) iinsol(7) = 1
      if ( table(12)>=0d0 .or. table(13)>=0d0 ) then

!     --- residual accuracy required

         if ( table(12)>=0d0 .and. table(13)>=0d0 ) &
            call errkeywexcl ( code, ncodmx, 12, 13 )

         if ( table(12)>=0d0 ) then

!        --- absolute residual criterion

            rinsol(3) = table(12)
            iinsol(6) = iinsol(6) + 100

         else

!        --- relative residual criterion

            rinsol(3) = table(13)
            iinsol(6) = iinsol(6) + 200

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readnlglob'

      end if  ! ( debug )
      call erclos ( 'readnlglob' )

      end subroutine readnlglob
