      subroutine readprobiinp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 07-07-2006 iniget --> seppointer
!        version  1.0  date 21-06-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information concerning the input for probdf from file
!     sepcomp.out and call subroutine probdf
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     problem_definition
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekprob
      use sepmodulemain
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ios, leniinput, lenrinput, ivers, &
              isubr, ichois, ihelp(5), iseqprob
      character (len=80) text

!     i              Counting variable
!     ichois         Choice parameter for subroutine PROBDF
!                    See PROBDF
!     ihelp          Work array to store the first 5 positions of iinput
!     ios            IOSTAT variable, returns from reading or writing
!     iseqprob       Sequence number of problem input
!     isubr          dummy
!     ivers          Version sequence number
!     leniinput      Length of array iinput
!     lenrinput      Length of array rinput
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PROBDFEXT      Reads and stores problem definition
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     767   end of file found before end is expected
!     768   Read error
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read input from file
!     Call PROBDF
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readprobiinp' )

      if ( iref74>0 ) then

!     --- new version, formatted

         read(iref74,100) text
100      format(a)
         read(iref74,*) ivers, isubr, leniinput
         read(iref74,*, iostat=ios) lenrinput
         read(iref74,100, iostat=ios) text
         read(iref74,*, iostat=ios) ihelp

      else

!     --- new version, unformatted

         read(-iref74) text
         read(-iref74) ivers, isubr, leniinput
         read(-iref74, iostat=ios) lenrinput
         read(-iref74, iostat=ios) text
         read(-iref74, iostat=ios) ihelp

      end if
      if ( ios/=0 ) go to 500

      nprob = max(1,ihelp(5))

      if ( lenrinput>0 ) then

!     --- lenrinput>0, so rinput is used

         allocate (rinputtmp(lenrinput) )

      else

         allocate (rinputtmp(1) )

      end if  ! ( lenrinput>0 )

      allocate ( iinputtmp(leniinput) )

!     --- Create arrays iinput and rinput in ibuffr, buffer

      if ( iref74>0 ) then

!     --- new version, formatted

         read ( iref74,*, iostat=ios ) iinputtmp
         if ( lenrinput>0 ) then
            read(iref74,100) text
            read ( iref74,*, iostat=ios ) rinputtmp
         end if

      else

!     --- new version, unformatted

         read ( -iref74, iostat=ios ) iinputtmp
         if ( lenrinput>0 ) then
            read(-iref74) text
            read ( -iref74, iostat=ios ) rinputtmp
         end if

      end if
      if ( ios/=0 ) go to 500

      ichois = 5
      iseqprob = 1
      call probdfext ( ichois, iseqprob )
      go to 1000

!     --- Error messages if ios # 0

500   if ( ios>0 ) then

!     --- ios>0, Error found in reading of file 73

         call errsub ( 768, 0, 0, 0 )

      else

!     --- ios<0, End of file found in reading of file 73

         call errsub ( 767, 0, 0, 0 )

      end if
      deallocate ( iinputtmp )
      deallocate ( rinputtmp )

1000  call erclos ( 'readprobiinp' )
      end subroutine readprobiinp
