      subroutine readscintscmsh ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-11-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keywords
!     INTERSECT_MESH
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          2: MESH_OUT Sequence number of mesh to be created
!                          3: iposplane Position of plane input in rinsep
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Next free position in rinsep
!     rinsep        i/o   The information of the plane is stored in
!                         rinsep from position istrin
!                         Pos. 1-4   a .. d (define plane)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istartprev

!     istartprev     Value of istart at beginning of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       INTERSECT_MESH, options (all in one line)
!           where options may have the following contents
!           MESH_OUT (Default meshnr+1)
!           PLANE (ax+by+cz=d)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3119   input and output mesh have different dimensions
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscintscmsh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscintscmsh'
         write(irefwr,1) 'istart, istrin', istart, istrin
         istartprev = istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode  ! code

      nncode = 2
      code(1) = 'MESH_OUT'
      code(2) = 'PLANE'
      ncode(1:2) =  4
      ncodmx(1) = 8
      ncodmx(2) = 5
      table(1:nncode) = -1d0
      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 5d0

      itype(2) = 21    ! plane

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) then
         call prinrl ( table, 2, 'table' )
         call prinrl ( tabout, 4, 'tabout' )
      end if  ! ( debug )

      inpsep(istart+1) = nint(table(1))  !  MESH_OUT
      iposut = nint(table(2))  !  plane

      if ( iposut<0 ) then

!     --- Not plane found after INTERSECT_MESH

         call errchr ( code(2)(1:ncode(2)), 1 )
         call errchr ( 'INTERSECT_MESH', 2 )
         call errsub ( 3119, 0, 0, 1 )

      else

!     --- Plane found

         inpsep(istart+2) = istrin
         rinsep(istrin:istrin+3) = tabout(iposut:iposut+3)
         istrin = istrin+4

      end if  ! ( iposut<0 )


!     --- raise istart

      istart = istart+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, istrin/end', istart, istrin
         call prinin ( inpsep(istartprev), istart-istartprev, 'inpsep' )
         call prinrl ( rinsep(istrin-4), 4, 'rinsep' )
         write(irefwr,*) 'End readscintscmsh'

      end if  ! ( debug )
      call erclos ( 'readscintscmsh' )

      end subroutine readscintscmsh
