      subroutine readsckmesh ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the information following create_kmesh and store in inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         0:  icode (111)
!                         1:  jchoice for iniactm0
!     istart        i/o   Starting address in array INPSEP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!     create_kmesh options
!     The following options are available
!     nmesha (b-l)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2555   Not a text found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsckmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsckmesh'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt  ! read next item
      if ( debug ) then
         write(irefwr,1) 'ityprd, nchar', ityprd, nchar
         write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( ityprd==4 ) then

!     --- text found, interprete

         if ( string(1:nchar)=='NMESHE' ) then
            inpsep(1) = 2
         else if ( string(1:nchar)=='NMESHC' ) then
            inpsep(1) = 4
         else if ( string(1:nchar)=='NMESHD' ) then
            inpsep(1) = 4
         else if ( string(1:nchar)=='NMESHH' ) then
            inpsep(1) = 5
         else if ( string(1:nchar)=='NMESHL' ) then
            inpsep(1) = 6
         else if ( string(1:nchar)=='NMESHA' ) then
            inpsep(1) = 7
         else if ( string(1:nchar)=='NMESHB' ) then
            inpsep(1) = 7
         else if ( string(1:nchar)=='NMESHF' ) then
            inpsep(1) = 8
         else if ( string(1:nchar)=='NMESHG' ) then
            inpsep(1) = 9
         end if  ! ( string(1:nchar)=='NMESHE' )

      else

!     --- No string found after create_kmesh

         call errchr ( 'CREATE_KMESH', 1 )
         call errsub ( 2555, 0, 0, 1 )

      end if  ! ( ityprd==4 )

      inpsep(0) = 111    ! icode
      istart = istart+2  ! raise istart

1000  call erclos ( 'readsckmesh' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readsckmesh'

      end if  ! ( debug )

      end
