      subroutine readscplot3d ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-07-2012 Use subroutine intxstoretext
!        version  1.0  date 11-03-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the 3D plot with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     3d
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-5 are used:
!                         1:  ICODE
!                         2:  IVECTOR
!                         3:  IDEGFD
!                         4:  TEXT, sequence number in text array
!                         5:  NTEXT, number of characters in text
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=3
      integer ncode1(nncode1), ncodmx1(nncode1)
      character (len=20) code1(nncode1)

!     code1          Table with keywords
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       PLOT_3D V1, options
!           where options may have the following contents
!           degfd=i
!           text = '..'
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'DEGFD', 'TEXT', 'V' /
      data ncode1  /  3, 4, 1 /
      data ncodmx1 /  5, 4, 1 /
! ======================================================================
!
      call eropen ( 'readscplot3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplot3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      itype(1:nncode) = 0
      itype(2) = 6   ! text
      tabmin(1:nncode) = 0d0
      tabmin(1) = 1d0  ! degfd
      tabmin(3) = 1d0  ! V
      tabmax(1:nncode) = 0d0
      tabmax(1) = 1d2  ! degfd
      tabmax(3) = 1d2  ! V
      table(1:nncode) = 0d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncode, 'table' )

      if ( table(1)>0d0 ) then
         inpsep(istart+2) = nint(table(1))   ! degfd
      else
         inpsep(istart+2) = 1                ! default degfd
      end if

!     --- text

      call intxstoretext ( inpsep(istart+3), table(2), tabmax(2), &
                           strout )

      inpsep(istart+1) = nint(table(3))  ! sequence number of vector

      istart = istart+5

1000  call erclos ( 'readscplot3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscplot3d'

      end if  ! ( debug )

      end
