      subroutine readscplotzerl ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-11-2013 Extension with curve plot info
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword PLOT_ZERO_LEVELSET
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-3 are used:
!                          0:  70  (plot_zero_levelset)
!                          1:  sequence number of level set
!                          2:  PLOTBOUN
!                              0,1:   only the outer boundaries are plotted
!                              2:     only the inner boundaries are plotted
!                              3:     all boundaries are plotted
!                              4:     no boundaries are plotted
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=4
      integer ncode2(nncode2), ncodmx2(nncode2), ilev
      character (len=18) code2(nncode2)

!     code2          Table with keywords
!     ilev           Sequence number of level being considered
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2         Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READPLOTBOUN   Read information on how boundary curves must be plotted
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following keywords are recognized
!           plot_zero_levelset i
!           inner_curves
!           outer_curves
!           no_curves
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'PLOT_ZERO_LEVELSET', &
                  'INNER_CURVES', 'OUTER_CURVES', 'NO_CURVES' /
      data ncode2  /  4, 3, 3, 4 /
      data ncodmx2 / 18,12,12, 9 /

!     code2   Table with sub keywords corresponding to keyword plot_zero_levelset
!             Values:
!             1:   PLOT_ZERO_LEVELSET
!             2:   INNER_CURVES
!             3:   OUTER_CURVES
!             4:   NO_CURVES
! ======================================================================
!
      call eropen ( 'readscplotzerl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplotzerl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      lastitem = lastitem-1  ! reread plot_zero_levelset
      itype(1) = 3  ! integer or no integer
      itype(2:4) = 2  ! keyword present or not
      table(1:nncode) = -1d0
      nmain = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 1d0
      errorcode = 0
      iretur = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

!     --- sequence number of level set

      if ( table(1)>0 ) then
         ilev = nint(table(1))
      else
         ilev = 1
      end if  ! ( table(1)>0 )

      inpsep(0) = 70  ! print zero level set
      inpsep(1) = ilev  ! sequence number of level set

!     --- Fill plotboun

      call readplotboun ( inpsep(2), code2(2), ncodmx2(2), table(2) )

1000  call erclos ( 'readscplotzerl' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(istart), 3, 'inpsep' )
         write(irefwr,*) 'End readscplotzerl'

      end if  ! ( debug )

      istart = istart+3  ! raise istart

      end
