      subroutine readscpresmesh ( istart, inpsep, icode, name_keyw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 26-10-2011 Allow scalars in input
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-11-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   PRESENT_MESH
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), icode
      character (len=*) name_keyw

!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  Sequence number of mesh
!     istart        i/o   Next free position in inpsep
!     name_keyw      i    Name of keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
!       PRESENT_MESH = 2 (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscpresmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscpresmesh'
         write(irefwr,1) 'icode', icode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call readrealscal
      inpsep(istart) = icode
      if ( debug ) write(irefwr,1) 'ityprd', ityprd
      if ( ityprd/=2 .and. ityprd/=11 ) then

!     --- Not an integer number found after present mesh

         call errchr ( name_keyw, 1 )
         call errsub ( 574, 0, 0, 1 )
         inpsep(istart+1) = 1
      else
         inpsep(istart+1) = nint(realrd)
      end if
      istart = istart+2

1000  call erclos ( 'readscpresmesh' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscpresmesh'

      end if  ! ( debug )

      end
