      subroutine readscreadmesh ( istart, inpsep )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  1.1  date 30-05-2012 Extension with keyword problem
!        version  1.0  date 12-12-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the keyword
!     READ_MESH
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  MESNNR Mesh number to store the mesh
!                         3:  INAME Sequence number of name in text array
!                                   0 = default name
!                         4:  Type of problem definition
!                              1:  none
!                              2:  default
!                              3:  old
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
      integer :: imeshnr

!     imeshnr        Mesh sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       READ_MESH j, file = 'file_name', problem = xxx
!           j (Mesh number)
!           FILE = 'file_name' (Default meshoutput)
!           problem = xxx:
!              none
!              default
!              old
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscreadmesh' )

      inpsep(istart) = icode

      nncode =2
      code(1) = 'FILE'
      code(2) = 'PROBLEM'
      ncodmx(1) = 4
      ncodmx(2) = 7
      ncode(1) = 4
      ncode(2) = 3
      itype(1) = 6
      itype(2) = 5
      tabmin(1:2) = 1d0
      tabmax(1) = 1d2
      tabmax(2) = 3d0
      codesub(1) = 'NONE'
      codesub(2) = 'DEFAULT'
      codesub(3) = 'OLD'
      ncodesub(1:3) = 3
      table(1:nncode) = -1d0
      table(2) = 3d0   ! problem = old

!     --- read mesh number

      imeshnr = 1
      call asknxt
      if ( ityprd/=2 ) then

!     --- item read is not an integer

         call errchr ( code(1), 1 )
         call errsub ( 574, 0, 0, 1 )

      else

         imeshnr = nint(realrd)

         if ( imeshnr<1 .or. imeshnr>maxmesh ) then

!        --- integer out of range

            call errint ( imeshnr, 1 )
            call errint ( 1, 2 )
            call errint ( maxmesh, 3 )
            call errchr ( 'READ_MESH', 1 )
            call errsub ( 1881, 3, 0, 1 )
            call errcrd
         end if

      end if
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      inpsep(istart+1) = imeshnr

      if ( table(1)>100000000d0 ) then

!     --- table(1) > 100000000, hence filename is in strings
!         Store position in text array into inpsep(istart+2)

         inpsep(istart+2) = instring(nint(table(1))-100000000)

      else if ( table(1)>0 ) then

!     --- table(1) > 0, hence filename is found
!         Store in array texts and reference in inpsep(istart+2)

         inpsep(istart+2) = nextxt
         texts(nextxt) = strout(1)
         nextxt = nextxt+1
      else
         inpsep(istart+2) = 0
      end if
      inpsep(istart+3) = nint(table(2))
      istart = istart+4

1000  call erclos ( 'readscreadmesh' )

      end subroutine readscreadmesh
