      subroutine readscsoltim1 ( inpsep, rinsep, istrin, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input following solve_time_dependent_problem in the
!     structure block
!     No sequence number found
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nprob
      integer, intent(inout) :: inpsep(*), istrin
      double precision, intent(inout) :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         inpsep(1) gets value -10
!                         inpsep(2) gets the value of istrin at input
!                         The next 31 positions contain the input read as
!                         described in pr1000
!                         This is followed by the array inforintim
!                         See readtm1
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     nprob          i    Number of various problems
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leniintim, numcop, lengreal, nhist

!     lengreal       Length of real input array in readtm1
!     leniintim      Length of first part of iintim in readtm1
!     nhist          Number of points where a time history must be stored
!     numcop         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READTM1        Read first part of array iintim referring to general items
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscsoltim1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscsoltim1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      leniintim = 31
      inpsep(1) = -10
      inpsep(2) = istrin
      call readtm1 ( 2, inpsep(3), rinsep(istrin), &
                     1, ndim, numcop, length, istrin )

      nhist = inpsep(26)
      lengreal = nhist*ndim+40
      inpsep(3) = leniintim

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 2, 'inpsep/1' )
         call prinin ( inpsep(3), leniintim, 'inpsep/2' )
         call prinin ( inpsep(3+leniintim), lengreal, 'inpsep/3' )
         call prinrl ( rinsep(istrin), lengreal, 'rinsep')
         write(irefwr,*) 'End readscsoltim1'

      end if  ! ( debug )
      call erclos ( 'readscsoltim1' )
      istrin = istrin+lengreal

      end subroutine readscsoltim1
