      subroutine readsctime ( istart, inpsep, instat, timefnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.2  date 22-11-2004 Fortran 90
!        version  1.1  date 12-12-2002 Use V or VECTOR
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   TIME_INTEGRATION
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)
      logical instat, timefnd

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  time_method sequence number
!                         3:  vector sequence number
!     instat        i/o   If true an instationary free boundary loop is active
!     istart        i/o   Next free position in inpsep
!     timefnd       i/o   If true, time integration has been found in loop
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=3
      integer ncode1(nncod1), ncod1m(nncod1)
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1936   Keyword TIME_INTEGRATION found, and no instationary loop active
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQUENCE', 'VECTOR', 'V' /
      data ncod1m /15, 6, 1 /
      data ncode1 / 6, 6, 1 /

!     code1   Table with sub keywords corresponding to some keywords
!             Values:
!             1:  SEQUENCE
!             2:  VECTOR
!             3:  V
! ======================================================================
!
      call eropen ( 'readsctime' )

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:nncode) = 0
      table(1) = 1d0
      table(2:3) =-1d0
      tabmin(1:nncode) = 1d0
      tabmax(1) = 25d0
      tabmax(2:3) = dble(maxvec)
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      if ( table(3)>0d0 .and. table(2)>0d0 ) &
         call errkeywexcl ( code1, ncod1m, 3, 2 )

      table(2) = max(1d0,table(3),table(2))
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = nint(table(2))
      istart = istart + 3
      if ( .not. instat ) then

!     --- keyword time_integration found, whereas no instationary
!         loop is active

         call errsub ( 1936, 0, 0, 0 )

      else
         timefnd = .true.
      end if

1000  call erclos ( 'readsctime' )

      end subroutine readsctime
