      subroutine readscwritfile ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 19-04-2014 Complete revision
!        version  1.0  date 23-02-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part following the keyword WRITE_TO_FILE
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     print
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 0    94 to indicate write_to_file
!                              1    position of file name in array texts
!                              2    appendmode
!                                   0: not
!                                   1: append
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following options are allowed:
!       WRITE_TO_FILE, 'name_of_file', append
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscwritfile' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscwritfile'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lastitem = 0  ! reset reading
      nncode = 2
      code(1) = 'WRITE_TO_FILE'
      code(2) = 'APPEND'
      ncode(1:2) = 3
      ncodmx(1) = 13
      ncodmx(2) = 6
      table(1:nncode) = 0d0

      itype(1) = 6     ! file name
      itype(2) = 2     ! append
      iretur = 0
      nmain = 0

      call sepreadsetkeyw

!     --- file

      call intxstoretext ( inpsep(1), table(1), tabmax(1), strout )
      inpsep(2) = nint(table(2))  ! append mode

      inpsep(0) = 94   ! WRITE_TO_FILE

!     --- Raise istart

      istart = istart + 3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscwritfile'

      end if  ! ( debug )
      call erclos ( 'readscwritfile' )

      end subroutine readscwritfile
