      subroutine readseqstring ( table, code, iinput, icode, &
                                 defvalue, ncodmx, subtract )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the integer variable read in iinput
!     If no sequence number is found, check if the variable is known
!     as string
!     If so fill the string sequence number otherwise the default value
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: table(*)
      integer, intent(in) :: icode, defvalue, ncodmx(*), subtract
      integer, intent(out) :: iinput
      character (len=*) :: code(*)

!     code           i    Table with keywords (contains the names)
!     defvalue       i    Default value
!     icode          i    Sequence number of variable in table and tabmin
!     iinput         o    Entry of integer input array referring to variable
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     subtract       i    This value must be sutracted from the value
!                         in table if table(icode)>0
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!                    they are not
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepgetstring

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPGETSTRING   Get the sequence number corresponding to the string in
!                    the block constants with name stringname
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqstring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqstring'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table(icode)>0 ) then

!     --- string found

         iinput = nint(table(icode))-subtract

      else

!     --- string not found, check if it is an existing string

         iseq = sepgetstring ( code(icode)(1:ncodmx(icode)), chstrings, &
                               numstrings )

         if ( debug ) write(irefwr,1) 'iseq', iseq

         if ( iseq>0 ) then

!        --- Name found, store sequence number

            iinput = iseq+100100000  ! mark as string

         else

!        --- Name not found, use default

            iinput = defvalue

         end if  ! ( iseq>0 )

      end if  ! ( table(icode)>0 )

1000  call erclos ( 'readseqstring' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readseqstring'

      end if  ! ( debug )

      end subroutine readseqstring
