      subroutine readseqvector ( value, name, iinput, defvalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 14-04-2013 Allow loop of vectors
!        version  2.0  date 07-03-2012 Extra parameter defvalue
!        version  1.0  date 28-06-2004
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the sequence number read in iinput
!     If no sequence number is found, check if the vector name is known
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision value
      integer iinput, defvalue
      character (len=*) name

!     defvalue       i    Default value
!     iinput         o    entry of integer input array to be filled with
!                         integer sequence number
!     name           i    Name of array
!     value          i    real value read containing the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecnm

!     ivecnm         vector sequencer number with respect to isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRGETNAME      Get sequence number of vector with specific name
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3133   No vector with name name found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqvector' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqvector'
         write(irefwr,1) 'defvalue', defvalue
         write(irefwr,2) 'value', value
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( value>0d0 ) then

!     --- vectorname is given

         iinput = mod(nint(value),5000)

      else

!     --- vectorname is not given, test if there is a vector with name
!         name

         ivecnm = 0
         call prgetname ( name, ivecnm )

         if ( ivecnm>0 ) then

            iinput = ivecnm  ! sequence number of vector

         else if ( defvalue>0 ) then

            iinput = defvalue  ! default value

         else if ( defvalue==0 ) then

            iinput = defvalue  ! default value
            go to 1000

         else

!        --- defvalue<0: No vector with name name found

            call errchr ( name, 1 )
            call errsub ( 3133, 0, 0, 1 )
            go to 1000

         end if  ! ( ivecnm>0 )

      end if  ! ( value(1)>0d0 )

      if ( debug ) write(irefwr,1) 'iinput', iinput

      if ( dimnames(iinput)>1 ) iinput = iinput+5000

1000  call erclos ( 'readseqvector' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput', iinput
         write(irefwr,*) 'End readseqvector'

      end if

      end
