      subroutine readservar ( table1, iinsep, rinsep, numitems, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store numitems reals or variables in iinsep
!     The information is stored in tabout
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numitems
      integer, intent(inout) :: istrin, iinsep(0:*)
      double precision, intent(in) :: table1
      double precision, intent(inout) :: rinsep(*)

!     iinsep        i/o   The integer input read is stored in arrayiinsep
!     istrin        i/o   Starting address in array RINSEP
!     numitems       i    Number of items stored in tabout
!     rinsep         o    Real input array. The storage depends on iinsep
!     table1         i    Contains the number read for the keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iseq

!     i              General loop variable
!     iseq           Sequence number of scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readservar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readservar'
         write(irefwr,1) 'numitems', numitems
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposut = nint(table1)
      if ( debug ) write(irefwr,1) 'iposut', iposut

      do i = 0, numitems-1
         if ( infotabout(iposut+i)==11 ) then
            iseq = nint(tabout(iposut+i))
            iinsep(i) = iseq+maxints+maxreals
         else
            iinsep(i) = -istrin
            rinsep(istrin) = tabout(iposut+i)
            istrin = istrin+1
         end if
      end do

1000  call erclos ( 'readservar' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinsep(0), numitems, 'iinsep' )
         call prinrl ( rinsep, istrin, 'rinsep' )
         write(irefwr,*) 'End readservar'

      end if  ! ( debug )

      end subroutine readservar
