      subroutine readsp ( manag, lenmanaglc, nextrecord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.4  date 25-04-2014 Extra debug
!        version  7.3  date 08-12-2013 Extension with compcons and princons
!        version  7.2  date 30-05-2012 Allow skipping of problem
!
!   copyright (c) 1989-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help subroutine for subroutine SEPSTR
!     This subroutine reads the input for the records of part 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleplot
      use sepmoduleread1
      use sepmodulemach
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenmanaglc, manag(lenmanaglc), nextrecord

!     lenmanaglc       i    Length of array manag
!     manag         i/o   Manager array that sets the parameters in subroutine
!                         SEPSTR
!                         MANAG must be filled as follows:
!                          1: Parameter ISTART for subroutine START
!                          2: Parameter IROTAT for subroutine START
!                          3: Parameter IOUTP  for subroutine START
!                          4: Parameter ITIME  for subroutine START
!                          5: Parameter IRENUM for subroutine MESHRD
!                             if irenum = 1000 call by sepmesh without
!                             suppressing all output, -1000 with suppressing all
!                             output
!                             if irenum = 10001 call by sepdiffout
!                          6: Parameter ICHOIS for subroutine PROBDF
!                          7: Indication if the file sepcomp.out must
!                             be filled (1) or not (0)
!                          8: ndim in case of MESHnD as first record
!                          9: itimefirst (first integer time step that must be
!                             read)
!                         10: itimelast (last integer time step that must be
!                             read)
!                         Default value for manag(5) must be filled at input, &
!                         this value may be changed if the keyword start is
!                         found
!                         11: Input of program
!                            -1:  skip reading problem definition
!                             0:  standard
!                             1:  read sepcomp.out (sepcomp only)
!                         12: Defines type of keyword found
!                             0: anything
!                            11: structure
!                         13-20:  Not yet in use
!     nextrecord     o    Defines type of next record read
!                         Possible values:
!                         0: none
!                         1: PROBLEM
!                         2: POSTPROCESSING
!                         3: MESH
!                         4: READ MESH
!                         5: EXTRA_MESH_INPUT
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=22, nnmcod=16, nncod2=10
      character (len=17) codelc(nncodelc), codemn2(nnmcod)
      character (len=17) code2(nncod2)
      logical ::  frecor, meshfn, sepdiffout
      integer :: ncodelc(nncodelc), nmcodelc(nnmcod), jsepbc, &
              jrefs, irenum, ichprb, i, ndim, &
              ncodmxlc(nncodelc), ncode2(nncod2), ncodmx2(nncod2), iposren

!     code2          Table with keywords
!     codelc         Table with keywords
!     codemn2        Table with main keywords
!     frecor         If true the first record is of the type PROBLEM, POST
!                    or MESH
!     i              Counting variable
!     ichprb         Choice parameter for subroutine PROBDF
!     iposren        Position of renumber keyword in array code
!     irenum         Type of renumbering  Possible values
!                    0  No renumbering
!                    1  Cuthill-McKee  optimal profile
!                    2  Sloan  optimal profile
!                    3  Best  optimal profile
!                    4  Cuthill-McKee  optimal band width
!                    5  Sloan  optimal band width
!                    6  Best  optimal band width
!                    7  Cuthill-McKee  always
!                    8  Sloan always
!                    9  Default depending on mesh
!     jrefs          Indication of the structure of the files sepcomp.out
!                    and sepcomp.inf
!                    Possible values:
!                    -1   Files are used; the defaults from sepran.env
!                        are used
!                    0   Files are not used
!                    1   files are formatted
!                    2   Bulk files are unformatted
!     jsepbc         Choice parameter indicating if file 2 must be opened
!                    or not
!                    Possible values:
!                    0   File 2 is not used
!                    1   File 2 is new
!                    -1   File 2 is old
!     meshfn         If true the mesh record has been found
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodelc        Table containing the number of significant characters
!                    in the keyword table codelc
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc
!     ndim           Dimension of the space.
!     nmcodelc         Table containing the number of significant characters
!                    in the main keyword table codemn2, including those of
!                    the extra texts
!     nncod2         Number of entries in code2
!     nncodelc       Number of entries in codelc
!     nnmcod         Number of entries in codemn2
!     sepdiffout     if true the subroutine has been called by program
!                    sepdiffout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     COMPCONS       User subroutine to compute constants
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINCONS       Print all constants, variables and vector names
!     PRININ         print 1d integer array
!     READCO         Read constants
!     READDEBUGPARMS Read debug parameters
!     READGENCONSTS  Read general constants
!     READSPSTART    Read part of input defined by keyword START
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!   This subroutine reads the input from the standard input file
!   Records to be read are of the shape
!       CONSTANTS
!         See subroutine readco
!       END
!       START
!          database = not/new/old
!          rotate
!          norotate
!          renumber = cuthill  profile/band/always
!          renumber = sloan    profile/band/always
!          renumber = best     profile/band
!          renumber = not
!          renumber = levels   centroid
!          renumber = nested_dissection
!          sepcomp = not/formatted/unformatted
!          name_back = 'name_backing_storage_file'
!          name_plot = 'name_plot_files'
!          name_sepinf = 'name_sepcomp.inf_file'
!          name_sepout = 'name_sepcomp.out'
!          name_sepcomp_in = 'name_sepcomp.out_input_file'
!          name_firstfile = 'name_sepcomp.out1'
!          name_secondfile = 'name_sepcomp.out2'
!          name_difffile = 'name_sepdiff.out'
!          name_mesh = 'name_mesh_file'
!          maxplots = m
!          itimefirst = i
!          itimelast = i
!          cpu_time
!          wall_clock_time
!          input_sepcomp_out
!          no_write_sep
!       END
!       PROBLEM or POSTPROCESSING
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!     550   array length too short
!    1853   The first record read after END is not PROBLEM or POSTPROCESSING
!    2232   constants block not closed
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set defaults
!     Read first record
!     If ( first record = PROBLEM or POST PROCESSING ) then
!        ichprb = 102
!     else if ( first record = START ) then
!        Read and interpret rest of the records until END is found
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'DATABASE', 'ROTATE', 'NOROTATE', 'SEPCOMP', &
                  'NAME_BACK', 'NAME_PLOT', 'NAME_SEPINF', &
                  'NAME_SEPOUT', 'NAME_MESH', &
                  'NAME_FIRSTFILE', 'NAME_SECONDFILE', &
                  'NAME_DIFFFILE', 'MAXPLOTS', &
                  'ITIMEFIRST', 'ITIMELAST', 'CPU_TIME', &
                  'WALL_CLOCK_TIME', 'INPUT_SEPCOMP_OUT', &
                  'NAME_SEPCOMP_IN', 'NO_WRITE_SEP', &
                  'RENUMBER', ' ' /
      data codemn2/ 'OLD', 'NOT', 'NEW', 'NOT', 'FORMATTED', &
                   'UNFORMATTED', 'NOT', 'CUTHILL', 'SLOAN', 'BEST', &
                   'LEVELS', 'NESTED_DISSECTION', &
                   'PROFILE', 'BAND', 'ALWAYS', 'CENTROID' /
      data code2/ 'CONSTANTS', 'PROBLEM', 'POSTPROCESSING', 'MESH', &
                  'READ', 'START', 'DEBUG_PARAMETERS', &
                  'GENERAL_CONSTANTS', 'EXTRA_MESH_INPUT', 'STRUCTURE' /
      data ncodelc  / 4, 3, 5, 4, 7,    7, 9, 9, 7, 8, &
                      8, 8, 5, 8, 8,    3, 4,12,15, 8, &
                      3, 0 /
      data ncodmxlc / 8, 6, 8, 7, 9,    9,11,11, 9,14, &
                     15,13, 8,10, 9,    8,15,17,15,12, &
                      8, 0 /
      data ncode2 /   4, 4, 4, 4, 4,    4, 9, 9, 8, 5 /
      data ncodmx2 /  9, 7,14, 4, 4,    5,16,17,16, 9 /
      data nmcodelc / 3, 3, 3, 3, 4,    6, 3, 3, 3, 4, &
                      3, 4, 4, 4, 3,    3 /

!    codelc    Code table containing the keywords that are recognized
!              Contents:
!              1:  database
!              2:  rotate
!              3:  norotate
!              4:  sepcomp
!              5:  name_back
!              6:  name_plot
!              7:  name_sepinf
!              8:  name_sepout
!              9:  name_mesh
!             10:  name_firstfile
!             11:  name_secondfile
!             12:  name_difffile
!             13:  maxplots
!             14:  itimefirst
!             15:  itimelast
!             16:  cpu_time
!             17:  wall_clock_time
!             18:  input_sepcomp_out
!             19:  name_sepcomp_in
!             20:  no_write_sep
!             21:  renumber
!             22:  -
!    codemn2    Code table containing all main keywords as well as the
!              subkeywords
!              Contents:
!              1:  old
!              2:  not
!              3:  new
!              4:  not
!              5:  formatted
!              6:  unformatted
!              7:  not
!              8:  cuthill
!              9:  sloan
!             10:  best
!             11:  levels
!             12:  nested_dissection
!             13:  profile
!             14:  band
!             15:  always
!             16:  centroid
!    code2     Code table containing all main keywords that may be read as start
!              Contents:
!              1:  CONSTANTS
!              2:  PROBLEM
!              3:  POSTPROCESSING
!              4:  MESH
!              5:  READ
!              6:  START
!              7:  DEBUG_PARAMETERS
!              8:  GENERAL_CONSTANTS
!              9:  EXTRA_MESH_INPUT
!             10:  STRUCTURE
! ======================================================================
!
      call eropen ( 'readsp' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsp'
         write(irefwr,1) 'istop', istop
         call prinin ( manag, 5, 'manag' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      itype(1) = 5
      itype(2:3) = 2
      itype(4) = 5
      itype(5:12) = 6
      itype(13:15) = 0
      itype(16:18) = 2
      itype(19) = 6
      itype(20) = 2
      itype(21) = 13
      itype(22) = 0  ! refers to next position; is updated in source
      itype(23:29) = 13
      itype(24) = 15
      itype(26) = 15
      itype(28) = 15
      itype(30) = 14
      itype(31:32) = 16

      tabmin(1) = 1d0
      tabmin(2:22) = 0d0
      tabmin(4) = 4d0
      tabmin(13:15) = 1d0
      tabmin(21) = 7d0

      tabmax(1) = 3d0
      tabmax(2:22) = 0d0
      tabmax(4) = 6d0
      tabmax(13) = 1d4
      tabmax(14) = 1d5
      tabmax(15) = 1d6
      tabmax(21) = 12d0

      readconstants = 0  ! no constants read yet

!     --- Set defaults

      if ( lenmanaglc<20 ) then

!     --- lenmanaglc too small

         call errint ( 20, 1 )
         call errint ( lenmanaglc, 2 )
         call errchr ( 'manag', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      jsepbc = 0
      irotat = 0
      ioutp  = 0
      iposren = 21                 ! position of keyword renumber
      itype(iposren+1) = iposren+2   ! starting position
      sepdiffout = .false.
      if ( manag(5)==10001 ) then

!     --- Special case for sepdiffout

         itime  = 0
         ioutp = -1
         irotat = 1
         manag(5) = -1
         sepdiffout = .true.
         mxwarn = 0

      else if ( manag(5)==1000 ) then

         itime  = 0
         irotat = 1
         manag(5) = 0

      else if ( manag(5)==-1000 ) then
         itime  = 0
         ioutp = -1
         irotat = 1
         manag(5) = 0
      else if ( manag(5)<-1000 ) then
         irotat = 1
         manag(5) = manag(5)+1000
      else
         itime  = 0
      end if
      jrefs  = -1
      irenum = manag(5)
      ndim = -1
      ichprb = 2
      manag(7) = 0
      manag(9) = 1
      manag(10) = ntimvc   ! maximum number of time steps that may be stored
                           ! See comcons2
      manag(11:20) = 0
      nextrecord = 0

!     --- Initialize names of scalars

      do i = 1, maxcls-1
         if ( chscalars(i)==' ' ) write(chscalars(i),150) i
150      format ( 'Scalar_',i3.3 )
      end do

      frecor = .false.
      meshfn = .false.

!     --- Read first record and check it

      if ( debug ) write(irefwr,1) 'before asknxt/1'
      ityprd = 1
      call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

!     --- Check string

200   nncodemain = nncod2
      codemain(1:nncod2) = code2
      ncodemain(1:nncod2) = ncode2
      ncodmxmain(1:nncod2) = ncodmx2
      ierpri = 1
      iretur = sepmainkeyw()
      if ( debug ) write(irefwr,1) 'iretur, istop', iretur, istop

      select case (iretur)

      case(:-1)

!     --- iretur<0, jump

         go to 1000

      case(1)

!     --- CONSTANTS found

         call readco
         readconstants = 1
         ityprd = 1
         istop = 1
         call asknxt
         if ( ityprd==0 ) then

!        --- Constants block not closed

            call errint ( irecnr, 1 )
            call errwar ( 2232, 1, 0, 0 )
            go to 1000

         end if
         istop = 0
         go to 200

      case(2:3)

!     --- PROBLEM or POST PROCESSING found, all defaults are used

         ichprb = 102
         frecor = .true.
         nextrecord = iretur-1

      case(4)

!     --- MESH found, all defaults are used

         frecor = .true.
         meshfn = .true.
         nextrecord = 3

      case(5)

!     --- READ MESH found, all defaults are used

         frecor = .true.
         meshfn = .true.
         ndim = 0
         nextrecord = 4

      case(6)

!     --- START found, read next records until END is found


         call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )
         call sepcopycodesub ( codemn2, nnmcod, nmcodelc )
         call sepsetmainend
         call readspstart ( manag, iposren, irenum, jrefs, jsepbc )

      case(7)

!     --- DEBUG_PARAMETERS found

         call readdebugparms
         ityprd = 1
         istop = 1
         call asknxt
         if ( ityprd==0 ) then

!        --- Constants block not closed

            call errint ( irecnr, 1 )
            call errsub ( 2232, 1, 0, 0 )
            go to 1000

         end if
         istop = 0
         go to 200

      case(8)

!     --- GENERAL_CONSTANTS found

         call readgenconsts
         ityprd = 1
         istop = 1
         call asknxt
         if ( ityprd==0 ) then

!        --- Constants block not closed

            call errint ( irecnr, 1 )
            call errsub ( 2232, 1, 0, 0 )
            go to 1000

         end if
         istop = 0
         go to 200

      case(9)

!     --- EXTRA_MESH_INPUT found, all defaults are used

         frecor = .true.
         ndim = 0
         nextrecord = 5

      case(10)

!     --- STRUCTURE found, i.e. no problem definition yet

         sepdiffout = .true.  ! just to skip
         manag(11) = -1       ! prevents reading of problem definition
         manag(12) = 11       ! keyword structure found
         ichprb = 202         ! default problem is used

      end select  ! case (iretur)

      if ( readconstants==1 ) then

!     --- constants found, call user subroutine to compute constants

         call compcons

!        --- Print a list of all constants, variables, reals and vector names

         call princons

      end if  ! ( readconstants==1 )

!     --- Store default values into array manag

      manag(1) = abs(jsepbc) + 20000
      if ( jsepbc<0 ) manag(1) = -manag(1)
      if ( jrefs>0 .or. jrefs==-1 .and. manag(11)==0 ) then

!     --- jrefs>0 or jrefs = -1, file sepcomp.out is used

         manag(7) = 1
         if ( jrefs==2 ) then

!        --- jrefs = 2, file sepcomp.out is unformatted

            iref74 = - abs ( iref74 )

         else if ( jrefs==1 ) then

!        --- jrefs = 1, file sepcomp.out is formatted

            iref74 = abs ( iref74 )

         end if

      end if
      if ( sepdiffout ) go to 1000
      if ( .not. frecor .and. manag(11)==0 ) then

!     --- PROBLEM nor POSTPROCESSING has been read

         ityprd = 1
         call asknxt

!        --- Check string

         if ( ityprd==4 .and. ( string(1:4)=='PROB' .or. &
              string(1:4)=='POST' ) ) then

!        --- PROBLEM or POST PROCESSING found

            ichprb = 102

         else if ( ityprd==4 .and. string(1:4)=='MESH' ) then

!        --- MESH found, all defaults are used

            meshfn = .true.

         else if ( ityprd==4 .and. string(1:4)=='READ' ) then

!        --- READ MESH found, all defaults are used

            meshfn = .true.
            ndim = 0

         else

             call errsub ( 1853, 0, 0, 0 )

         end if

      end if
      if ( meshfn .and. ndim/=0 ) then

!     --- read ndim

         call asknxt
         ndim = nint(realrd)
         if ( ndim<=0 .or. ndim>3 ) then
            call errint ( ndim, 1 )
            call errsub ( 62, 1, 0, 0 )
            call errcrd
         end if

      end if

1000  manag(2) = irotat
      manag(3) = ioutp
      manag(4) = itime
      manag(5) = irenum
      manag(6) = ichprb
      manag(8) = ndim

      if ( debug ) then

!     --- Debug information

         call prinin ( manag, 20, 'manag' )
         write(irefwr,*) 'End readsp'

      end if

      call erclos ( 'readsp' )

      end subroutine readsp
