      subroutine readstouter ( nelgrp, inpsep, ireadseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the keyword
!   STORE_OUTER_BOUNDARY
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelgrp, inpsep(*), ireadseq(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         1:  iseq: Sequence number
!                         2:  numgroups:  Number of element groups
!                         3-numgroups+2: element group numbers
!     ireadseq      i/o   Array of size nelgrp indicating if sequence numbers
!                         are free (0) or already in use (>0)
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, i, numgroups

!     i              Counting variable
!     iseq           Sequence number
!     numgroups      Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     store_outer_boundary iseq, elgrps = (i1, i2, ... )
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1378   Parameter has wrong value
!    2227   Only one of a combination of keywords found
!    2491   Sequence number found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readstouter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstouter'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- read sequence number

      iseq = 0
      call asknxt
      if ( ityprd.ne.2 ) then

!     --- item read is not an integer

         call errchr ( 'STORE_OUTER_BOUNDARY', 1 )
         call errsub ( 574, 0, 0, 1 )

      else

         iseq = nint(realrd)
         if ( iseq.lt.1 .or. iseq.gt.nelgrp ) then

!        --- error iseq out of range

            call errint ( iseq, 1 )
            call errint ( 1, 2 )
            call errint ( nelgrp, 3 )
            call errchr ( 'iseq', 1 )
            call errsub ( 1378, 3, 0, 1 )
            call errcrd

         else if ( ireadseq(iseq).ne.0 ) then

!        --- error sequence number used before

            call errint ( iseq, 1 )
            call errchr ( 'iseq', 1 )
            call errsub ( 2491, 1, 0, 1 )
            call errcrd

         else

!        --- New sequence number, fill ireadseq

            ireadseq(iseq) = 1

         end if  ! ( iseq.lt.1 .or. iseq.gt.nelgrp )

      end if
      inpsep(1) = iseq

!     --- Read element group numbers

      if ( iseq.eq.0 ) go to 1000

      iretur = 0
      nmain = 0
      code(1) = 'ELGRPS'
      ncode(1) = 5
      ncodmx(1) = 6
      itype(1) = 11
      tabmin(1) = nelgrp
      tabmax(1) = nelgrp
      table(1) = -1d0

      call sepreadsetkeyw
      iposut = nint(table(1))

      if ( iposut.gt.0 ) then

!     --- iposut > 0, store input in inpsep

         numgroups = nint(tabmax(1))
         inpsep(2) = numgroups
         do i = 1, numgroups
            inpsep(2+i) = nint(tabout(iposut+i-1))
         end do  ! i = 1, numgroups

      else

!     --- Error no element groups read

         call errchr ( 'STORE_OUTER_BOUNDARY', 1 )
         call errchr ( code(1)(1:ncodmx(1)), 2 )
         call errsub ( 2227, 0, 0, 2 )
         go to 1000

      end if  ! ( iposut.gt.0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readstouter'

      end if
      call erclos ( 'readstouter' )

      end subroutine readstouter
