      subroutine readstrnames
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string belongs to the list of constants variables or vectors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     vector
!     constant
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READNAMEVEC    Read name of vector in common cpost5
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
!     READSTRINGS    Check if name read corresponds to one of the strings
!     READSTRSCALS   Check if name read corresponds to one of the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readstrnames' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstrnames'
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call readstrconst  ! constant
      if ( debug ) write(irefwr,1) 'ityprd/1', ityprd
      if ( ityprd>=2 .and. ityprd<=3 .or. &
           ityprd>=12 .and. ityprd<=13 ) go to 1000  ! integer or real found

      call readstrscals  ! scalar
      if ( debug ) write(irefwr,1) 'ityprd/2', ityprd
      if ( ityprd==11 .or. ityprd==14 ) go to 1000  ! scalar found

      call readstrings  ! string
      if ( debug ) write(irefwr,1) 'ityprd/3', ityprd
      if ( ityprd==15 .or. ityprd==16 ) go to 1000  ! string found

      call readnamevec    ! name of vector
      if ( debug ) write(irefwr,1) 'ityprd/4', ityprd
      if ( ityprd==2 ) ityprd=102  ! indicates that vector has been found

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readstrnames'

      end if  ! ( debug )
      call erclos ( 'readstrnames' )

      end
