      subroutine readtmtend ( ndt, iintim, rintim, leniintim, istrin, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information concerning tend in iintim/rintim
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqnr
      integer, intent(out) :: ndt
      integer, intent(inout) :: leniintim
      integer, intent(inout) :: istrin, iintim(*)
      double precision, intent(inout) :: rintim(*)

!     iintim        i/o   Contains the input for time integration
!                         See subroutine readtm1
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     istrin        i/o   Starting address in array RINSEP
!     leniintim     i/o   Computed length of array iintim
!     ndt            o    Number of different time steps
!     rintim        i/o   Contains real input for time integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number of scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getnamevar, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     GETNAMEVAR     Return with sequence number of variable in scalars
!     PRINRL         Print 1d real vector
!     READSERVAR     Store numitems reals or variables in iinput
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1591   No end times found
!    2262   No end times found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readtmtend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readtmtend'
         write(irefwr,1) 'leniintim', leniintim
         call prinrl ( table, 35, 'table' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndt = 1
      if ( table(3)>0d0 ) ndt = nint(tabmax(3))
      leniintim = leniintim+1
      iintim(18) = ndt

      if ( debug ) then
         write(irefwr,1) 'ndt', ndt
         write(irefwr,2) 'table(3), table(28), table(35)', &
                          table(3), table(28), table(35)
      end if  ! ( debug )

      if ( table(3)>0d0 ) then

!     --- tend given

         call readservar ( table(3), iintim(leniintim), rintim, min(10,ndt), &
                           istrin )

      else if ( table(28)<=0d0 .and. table(35)<=0d0 ) then

!     --- Not keep_t

         iseq = getnamevar ( 'tend' )   ! check for variable
         if ( iseq>0 ) then
            iseq = iseq+maxints+maxreals+100000000  ! scalar found
         else
            iseq = getnamereal ( 'tend' )   ! check for real
            if ( iseq>0 ) iseq = iseq+maxints+100000000  ! real found
         end if  ! ( iseq>0 )

         if ( debug ) write(irefwr,1) 'iseq', iseq
         if ( iseq>0 ) then

!        --- iseq>0, variable name found

            iintim(leniintim) = iseq

         else

!        --- iseq=0, Use default values

            if ( debug ) then
               write(irefwr,1) 'iseqnr', iseqnr
               write(irefwr,2) 'table(5), table(9), table(10)', &
                                table(5), table(9), table(10)
            end if  ! ( debug )
            if ( table(9)>0d0 .or. table(10)>0d0 ) then
               rintim(istrin) = 1d3
            else if ( table(5)>0d0 ) then
               rintim(istrin) = 10d0 * tabout(nint(table(5)))
            else if ( iseqnr==1 ) then
               rintim(istrin) = 1d0
            else
               iintim(51) = 0
               go to 1000 ! skip
            end if
            iintim(leniintim) = -istrin
            istrin = istrin+1
            if ( iseqnr==1 ) then
               call erreal ( rintim(istrin), 1 )
               call errwar ( 1591, 0, 1, 0 )
            else
               call errwar ( 2262, 0, 0, 0 )
            end if

         end if  ! ( iseq>0 )

      end if  ! ( table(3)>0d0 )
      iintim(51) = leniintim
      leniintim = leniintim+ndt-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndt, leniintim', ndt, leniintim
         write(irefwr,*) 'End readtmtend'

      end if  ! ( debug )
      call erclos ( 'readtmtend' )

      end subroutine readtmtend
