      subroutine sepcomsub ( inpsep, rinsep  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 18-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutines to0094 and tofreesb
!     Old subroutine tosepcomsub
! **********************************************************************
!
!                       KEYWORDS
!
!     compute_all
!     driving_subroutine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulecomp
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleloop
      use sepmoduletime
      use sepmodulecons
      use sepmodulecpack
      use sepmodulemach
      use sepmoduleplot
      use sepmoduletexts
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                  Array INPSEP is filled as follows:
!                  Pos. 1  NUMVECTORS, i.e. number of vectors to be created
!                          Next positions information of the subroutines to be
!                          called with the corresponding information
!                          The subroutines are stored in the sequence as they
!                          are called, which means that the input of this
!                          subroutine is essential for the course of the
!                          program
!                          For each subroutine at least the following
!                          information is stored:
!                  Pos. 1  Subroutine sequence number
!                          See isubrmn in sepmodulemnsub

!                  Pos. 2  ISEQNR     Sequence number corresponding to input
!                                     If iseqnr = -1, the next one will be used
!                  Pos. 3  IPROB      Problem sequence number
!                                     If iprob = -1, the next one will be used
!                  Pos. 4  IVECTOR    Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!                          The next positions depend on the subroutine itself:
!                   --- In case of MANVEC the positions 5 - 20+nelgrp are
!                          used:, see subroutine PRMANV
!                   --- In case of PRINT_SCALAR  position 2
!                        refers to the number of scalars  (numscalars)
!                        position 3 refers to the position in the text array
!                        In this case 0, means no text
!                        Position 4 indicates the number of characters in the
!                        text
!                        Position 5 to 4+3*numscalars contain 3 positions per
!                        scalar:
!                         1: type (1=integer, 2=real, 3=scalar)
!                         2: position in incons rlcons or scalars
!                         3: if>0, index of scalar used as array index
!                   --- In case of PRINT_VECTOR position 2
!                        refers to the vector sequence number and
!                        position 3 refers to the position in the text array
!                        In this case 0, means no text
!                        Position 4 indicates the number of characters in the
!                        text
!                       Also the positions 5-13 and possibly more
!                       are used as well as positions in rinsep in the following
!                       way:
!                        5:  ISTRIN (Starting address in rinsep)
!                        6:  ICHOICE (inppri(2), see prinvc)
!                        7:  IDIM    (inppri(3), see prinvc)
!                        8:  IQUANT  (inppri(4), see prinvc)
!                        9:  IDEGFD  (inppri(5), see prinvc)
!                        10: ISEQ    (inppri(6), see prinvc)
!                        11: IREGION (inppri(7), see prinvc)
!                        12: IHEADER (inppri(8), see prinvc)
!                        13: ICOOR   (inppri(9), see prinvc)
!                        14: IFORMAT (inppri(10), see prinvc)
!                        15: NREAD   (number of curves, surfaces etc)
!                            If NREAD>0 followed by NREAD curves etc.
!                        If IREGION#0, 6 positions in rinsep are used
!                   --- In case of NONLINEAR_SOLVER 5 positions are used:
!                         2  ISEQNR  Sequence number corresponding to input
!                                    If iseqnr = -1, the next one will be used
!                         3  IPROB   Problem sequence number
!                                    If iprob = -1, the next one will be used
!                         4  IVECTOR Vector sequence number (first vector)
!                                    If ivector = -1, the same value as iprob
!                                    will be used
!                         5: IVEC_REACT indicates if reaction forces must be
!                            computed (>0) or not (0)
!                            If >0, IVEC_REACT defines the sequence number
!                            of the vector in which the reaction forces
!                            must be stored
!                   --- In case of LINEAR_SOLVER 8 positions are used:
!                         2: contains the sequence number for the coefficients
!                         3  IPROB   Problem sequence number
!                                    If iprob = -1, the next one will be used
!                         4  IVECTOR Vector sequence number (first vector)
!                                    If ivector = -1, the same value as iprob
!                                    will be used
!                         5: contains the sequence number for the solver
!                         6: indicates if defect correction should be used:
!                            0: no defect correction
!                            1: defect correction
!                         7: indicates if quadratic elements should be
!                            treated as a cluster of linear subelements
!                            0: no subelements
!                            1: subelements
!                         8: IVEC_REACT indicates if reaction forces must be
!                            computed (>0) or not (0)
!                            If >0, IVEC_REACT defines the sequence number
!                            of the vector in which the reaction forces
!                            must be stored
!                         9: IVEC_FEM is only used in case of a spectral mesh
!                            It indicates if also the fem solution vector
!                            must be stored besides the sem solution vector
!                            if>0, IVEC_FEM defines the sequence number
!                            of the fem solution vector
!                        10: Indicates if a FEM preconditioning must be applied
!                            in case of a spectral mesh
!                            Possible values:
!                            0: no fem preconditioning
!                            1: fem preconditioning
!                   --- In case of DERIV the positions 2 - 5 are used:
!                        2:  Sequence number coefficients
!                        3:  IPROB
!                        4:  IVECTOR Sequence number output vector
!                        5:  ISEQNR Sequence number derivatives
!                   --- In case of INTEGR the positions 2 - 7 are used:
!                        2:  Sequence number coefficients
!                        3:  ISEQNR Sequence number integrals
!                        4:  IVECTOR  Sequence number input vector
!                        5:  ISCALAR1 Sequence number scalar number of integral
!                        6:  Scalar number of min integral per element
!                        7:  Scalar number of max integral per element
!                   --- In case of INTBND the positions 2 - 6 are used:
!                        2:  ISEQNR Sequence number boundary integrals
!                        3:  IVECTOR  Sequence number input vector
!                        4:  ISCALAR1 Sequence number first scalar of integral
!                        5:  ISCALAR2 Sequence number second scalar of integral
!                        6:  ISCALAR3 Sequence number third scalar of integral
!                   --- In case of SCALAR i = value the positions 2-3 are used:
!                        2:  Sequence number scalar
!                        3:  Type of constpstruction
!                            1:  value
!                            2:  func
!                            3:  min
!                            4:  max
!                        4:  Position of value in rinsep (>0) or
!                            parameter IFUNC in FUNCSCAL ( IFUNC, SCALARS)
!                            In case of min/max: number of parameters
!                        5...:  In case of min/max: information of parameters
!                            >0: value in rinsep; <0 -Scalar sequence number
!                   --- In case of COMMAT position 2 is used:
!                        2:  Sequence number COMMAT input
!                   --- In case of COPY VECTOR the positions 2-3 are used:
!                        2:  Sequence number vector1
!                        3:  Sequence number vector2
!                        4:  Degree of freedom vector 1
!                        5:  Degree of freedom vector 2
!                   --- In case of PRINT_TEXT the positions 2-3 are used:
!                        2:  Position in text array
!                        3:  number of characters
!                   --- In case of WRITE_VECTOR or READ_VECTOR the positions
!                        2-4 are used:
!                        2:  array sequence number
!                        3:  sequence number in file 2
!                        4:  save administration (1) or not yet (0)
!                   --- In case of USEROUT at least positions 2 and 3 are used:
!                        2:  Sequence number USEROUT
!                        3:  nextra is number of extra integer positions
!                            If this number is larger than 0, nextra
!                            positions are used after position 2
!                   --- In case of WHILE or if the positions 2-3 are used:
!                        2:  sequence number k of boolean_expr(k)
!                        3:  address in inpsep just after the while loop
!                   --- In case of END_WHILE or END_IF only position 2 is used:
!                        2:  starting address of while loop
!                   --- In case of START_(IN)STAT_FREE_BOUN or START_(TIME)_LOOP
!                       the positions 2-3 are used:
!                        2:  sequence number k
!                        3:  address in inpsep just after the loop
!                   --- In case of END_(IN)STAT_FREE_BOUN or END_(TIME)_LOOP
!                       only position 2 is used:
!                        2:  starting address of loop + 3
!                   --- In case of CHANGE_COEFFICIENTS the 4 positions have
!                       the meaning
!                        2:  coefficient sequence number
!                        3:  change coefficient sequence number
!                        4:  iteration sequence number
!                        5:  problem sequence number (if 0, the last one)
!                   --- In case of TIME_INTEGRATION 3 positions are used:
!                        2:  time_method sequence number
!                        3:  vector sequence number
!                   --- In case of INPUT_VECTOR the positions 2-5 are used:
!                        2:  Sequence number of output vector
!                        3:  Number of unknowns per point
!                        4:  Position of file name in text array
!                        5:  number of characters of file name
!                   --- In case of CLOSE_PLOT or OPEN_PLOT
!                       only position 1 is used
!                   --- In case of PLOT_BOUNDARY positions 2-3 are used:
!                        2:  IREGION
!                        3:  Position of yfact
!                        If IREGION#0, 4 positions in rinsep are used
!                   --- In case of PLOT_MESH position 2 is used:
!                        2:  IREGION
!                        3:  Position of yfact
!                        If IREGION#0, 4 positions in rinsep are used
!                   --- In case of MAKE_OBSTACLE_MESH position 2-3 are used:
!                        2:  MESH_ORIG Sequence number of mesh to be adapted
!                        3:  MESH_OBST Sequence number of adapted mesh
!                   --- In case of REMOVE_OBSTACLE_MESH position 2-3 are used:
!                        2:  MESH_ORIG Sequence number of mesh to be adapted
!                        3:  MESH_OBST Sequence number of adapted mesh
!                   --- In case of REFINE_MESH position 2-5 are used:
!                        2:  N (Number of refinements)
!                        3:  MESH_IN Sequence number of mesh to be refined
!                        4:  MESH_OUT Sequence number of refined mesh
!                        5:  ISEQ Sequence number of refine input
!                   --- In case of PRESENT_MESH position 2 is used:
!                        2:  Sequence number of mesh
!                   --- In case of INTERCHANGE_MESH position 2-3 ARE used:
!                        2:  New sequence number of mesh 1
!                        3:  New sequence number of mesh 2
!                   --- In case of INTERPOLATE positions 2-4 are used:
!                        2:  Sequence number V1 (Defines also MESH_IN)
!                        3:  Sequence number V2
!                        4:  MESH_OUT
!                   --- In case of OUTPUT positions 2-4 are used:
!                        2:  ISEQNR
!                        3:  INAME Sequence number of name in text array
!                                  0 = default name
!                        4:  IVECTOR
!                   --- In case of FOR_LOOP positions 2-6 are used:
!                        2:  Lower bound loop
!                        3:  Upper bound loop
!                        4:  Step in loop
!                        5:  Sequence number of variable
!                        6:  address in inpsep just after the for loop
!                   --- In case of END_FOR_LOOP position 2 is used:
!                        2:  starting address in for loop
!                   --- In case of PLOT_VECTOR or PLOT_TENSOR positions 2-7
!                       are used:
!                        2:  IVECTOR
!                        3:  IREGION
!                        If IREGION#0, 4 positions in rinsep are used
!                        4:  IDEGFD1
!                        5:  IDEGFD2
!                        6:  IYFACT
!                        If IYFACT#0, 1 extra position in rinsep is used
!                        7:  IFACTOR
!                        If IFACTOR#0, 1 extra position in rinsep is used
!                         In case of plot tensor position 8 is also used:
!                        8:  IDEGFD3
!                   --- In case of PLOT_CONTOUR or PLOT_COLOURED_LEVELS
!                       positions 2-8 are used:
!                        2:  IVECTOR
!                        3:  IREGION
!                        If IREGION#0, 4 positions in rinsep are used
!                        4:  IDEGFD
!                        5:  NLEVEL
!                        6:  MINLEVEL
!                        If MINLEVEL#0, 1 extra position in rinsep is used
!                        7:  MAXLEVEL
!                        If MAXLEVEL#0, 1 extra position in rinsep is used
!                        8:  IYFACT
!                        If IYFACT#0, 1 extra position in rinsep is used
!                   --- In case of CREATE_FORCE_VECTOR 5 positions are used:
!                         2: contains the sequence number for the coefficients
!                         3  IPROB   Problem sequence number
!                                    If iprob = -1, the next one will be used
!                         4  IVECTOR Vector sequence number (first vector)
!                                    If ivector = -1, the same value as iprob
!                                    will be used
!                         5: indicates if quadratic elements should be
!                            treated as a cluster of linear subelements
!                            0: no subelements
!                            1: subelements
!                   --- In case of CHANGE_COORDINATES 2 positions are used:
!                         2: contains the sequence number for the input
!                   --- In case of EIGENVALUES 5 positions are used:
!                         2: contains the sequence number for the input
!                         3: IVECTOR
!                         4: ISCALAR
!                         5: IPROB
!                         6: NEIGV
!                   --- In case of DEFORM_MESH
!                         2: Sequence number of the vector to add
!                         3: SCALE
!                   --- In case of COMPUTE_PRINCIPAL_STRESSES 5 positions are
!                       used:
!                         2: iseqstress
!                         3: iseqeigval
!                         4: iseqeigvec
!                         5: ichoice
!                   --- In case of MOVE_OBSTACLE 12 positions are used:
!                         2: iobst
!                         3: Sequence number first position in RINSEP
!                         4: if>0 reference to scalar w.r.t. u_veloc
!                         5: if>0 reference to scalar w.r.t. v_veloc
!                         6: if>0 reference to scalar w.r.t. w_veloc
!                         7: if>0 reference to scalar w.r.t. phi_veloc
!                         8: Rotation plane 1: (xy) 2: (yz) 3: (xz)
!                            RINSEP(1-3) contain the velocity vector
!                            RINSEP(4) contains the rotation velocity
!                         9: Indicates if positions 4 to 6 refer to a
!                            velocity (0) or displacement (1)
!                        10: if>0 reference to scalar w.r.t. x_orig
!                        11: if>0 reference to scalar w.r.t. y_orig
!                        12: if>0 reference to scalar w.r.t. z_orig
!                   --- In case of READ_MESH 3 positions are used:
!                         2: meshnr
!                         3: sequence number for name in text array
!                   --- In case of SET_TIME positions 2 is used:
!                        2:  Position of value in rinsep (>0)
!                   Warning: the contents of inpsep and rinsol are yet not
!                           defined in such a way that they are upwards
!                           compatible. In the future the description may be
!                           changed. Please do not take the present description
!                           for granted
!     rinsep        i/o   Array containing the real input for this subroutine
!                         See subroutine TO0094 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rplots(3)
      integer iplots(10), i, iname, iref, jmaxsave, &
              k, lenname, maxcalltimsav, leninpsep, iaddress
      character (len=100) namesav

!     i              Counting variable
!     iaddress       Starting address of array
!     iname          Sequence number of name in text array
!     iplots         Integer input array for plot subroutine
!     iref           File reference number
!     jmaxsave       Help variable to save the value of jmax
!     k              Counting variable
!     leninpsep      Length of part of array inpsep
!     lenname        Number of significant characters in a text
!     maxcalltimsav  Help parameter to save the value of maxcalltim
!     namesav        Help variable to save a name
!     rplots         Real input array for plot subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCRET         Create vector
!     ERLINC         Help subroutine to call CHNCOF, which changes coefficients
!                    in IUSER and USER
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     ERLOOPFOR      Perform all actions necessary for the start of a for loop
!     EROPEN         Produces concatenated name of local subroutine
!     EROUTS         Write solution to file "sepcomp.out"
!     ERPRES         Read essential boundary conditions
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSCAL         Print scalar
!     INICLOSEFILE   Close file and reset irefwr
!     INIFIL         Open files
!     INIOPENFILE    Open serial file and set irefwr
!     INIREPSCAL     Corrects integer value in case of scalars
!     MSHCHCOR       Change coordinates in array coor by calls to FUNCCOOR
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTTIME       Plot the actual time in a plot depending on the parameters
!                    itimdep and t in common ctimen
!     PLOTTIMEHIS    Plot time history for one vector
!     PLOTZEROLEV    Plot the edges (2d) or faces (3d) of the zero level set
!     PRADAPINTBOUN  Adapt the concentration in end points of interface
!                    by making it equal to neighbor values
!     PRBEARING      Perform the complete iteration for one time step
!                    of the mass conservation scheme of Kumar
!     PRBNIN         Compute boundary integrals
!     PRBREAK        Handle the break option depending on the type of loop
!     PRBUBBLE       Compute bubbles defined by all regions satisfying function
!                    value larger than given threshold
!     PRCHANPROB     Change problem number of solution vector
!     PRCOMM         New call to COMMAT
!     PRCOMPCAP      calls the actual capacity subroutine prcompcapacity
!     PRCOMPENTHALPY Compute the enthalpy from the given temperature
!     PRCOMPINVER    calls the actual inverse subroutine prcompinv
!     PRCOMPMESH     Check if two meshes are equal by comparing the coordinates
!                    and the topology
!     PRCOMPPSI      Compute pseudo stream function psi
!     PRCONTCT       Compute new contact
!     PRCOPY         Copy one SEPRAN vector into another one
!     PRCOPYCOOR     Copy coordinates into a vector
!     PRCOPYMESH     Copy mesh to different mesh number
!     PRCORRINTERP   Correct interpolation in case nodes of the mesh crossed
!                    the old object
!     PRCREALEVVECT  Create level set vector from given obstacle
!     PRCREATEKMESH  Create a part of kmesh
!     PRCREATERHS    Create right-hand side vector
!     PRCREATESTRING Create string from other strings
!     PRDERV         Compute derivatives
!     PREIGVAL       calls the actual eigenvalue subroutine EIGENVBF
!     PRENTHREFINE   Solve the time dependent temperature equation
!                    with  freezing front by a enthalpy method and local
!                    refinement
!     PREXECWHILE    Execute while statement in sepcomsub
!     PRFIL2         Read solutions from and write to backing storage
!     PRFORCEV       Fill the force vector with a right-hand side vector
!     PRINCIPSTR     Computes the principle stresses from a stress tensor
!     PRINCONSTA     Print value of constpstant from block constants
!     PRININ         print 1d integer array
!     PRINTCURSUR    Print a set of curves or surfaces
!     PRINTERPOLATE  Interpolate solution from one mesh to the other
!     PRINTERSECT    Prepare input for intersection subroutine
!     PRINTG         Compute integrals
!     PRINTIMEHIS    Print time history for one vector!
!     PRINTSECTMESH  Intersect 3d mesh by plane and store the result as 2d mesh
!     PRINTZEROLEV   Print the edges (2d) or faces (3d) of the zero level set
!     PRLINEINT      Compute intersection of line with 2d mesh and compute the
!                    corresponding line integral
!     PRLINS         Build system of equations and solves linear system of
!                    equations
!     PRMAKEDIST     Make the level set function phi a distance function
!     PRMANV         Vector manipulation subroutine
!     PRMATSTRUC     Compute the structure of the large matrix
!     PRMOVEBOUN     Compute displacement vector for boundary due to change of
!                    coordinates
!     PRMOVEOBSTAC   Move obstacle over a distance for one time step
!     PRMSHLEVMS     Create a new mesh from a fixed mesh with a zero levelset
!                    crossing the mesh
!     PRMSHOBSTMS    Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh
!     PRNEWPROB      Call subroutine PROBDFEXT to define a new problem
!                    definition
!     PRNLNS         Solves non-linear system of equations
!     PRPARTPNLNCONC One time step for the integration of a particle
!                    is carried out
!     PRPLOT3D       Make a 3d plot of a 2d scalar function
!     PRPLOTCN       Plot contours
!     PRPLOTCOL      Driving subroutine for colored level plot
!     PRPLOTFN       Plot function along curve
!     PRPLOTLN       Plot function along a straight intersection of a 2d mesh
!     PRPLOTMESH     plot mesh
!     PRPLOTVC       Plot vectors
!     PRPRINKMESH    Print the contents of parts of array kmesh
!     PRPRINTINTS    Compute intersection of line with 2d mesh
!                    and print the results
!     PRREADMESH     Read mesh input from file
!     PRREFINE       Refine an existing mesh, compute the corresponding problem
!                    definition and create the corresponding matrix structure
!     PRRMLEVMESH    Delete the levelset mesh created for a zero levelset
!                    crossing the mesh
!     PRRMOBSTMESH   Remove obstacle mesh created from obstacle mesh
!     PRSCLR         Fill scalar corresponding to the input part SCALAR i = ...
!     PRSETTIME      Set time in main program sepcomp or sepfree
!     PRSTOC         Solve stochastic investment problem
!     PRSTREAM       Compute stream function
!     PRTIMEHIS      Information concerning the storage of time history is
!                    stored in common block ctimhis
!     PRTIMEHISSCAL  Fill time and value for all scalars in time history at the
!                    start of the time loop
!     PRTIMEHISSCAL2 Fill time and value for all scalars in time history during
!                    the time loop
!     PRUSER         Prepare output for userout or userouts subroutine
!     READSOLUTS     Read all solution arrays from sepcomp.in for one time step
!     SEPCOMPPRESS   Compute pressure in case of Navier-Stokes and store in isol
!     SEPDEFMSHNEW   Deform mesh for solid computation
!     SEPELSE        Execute else statement in sepcomsub
!     SEPENDIF       Execute end_if statement in sepcomsub
!     SEPENTHALPYMN  Solve the time dependent temperature equation
!                    with  freezing front by a enthalpy method
!     SEPEXECIF      Execute if statement in sepcomsub
!     SEPEXTEND      Copy mesh corresponding to axi-symmetric problem onto
!                    complete mesh defined on both sides of the symmetry axis
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPMAKEREL     Make relation between local and global arrays
!     SEPMESHWR      fill the file meshoutput with the mesh
!     SEPNAVSTOKES   Solve the time dependent Navier-Stokes equations
!     SEPPLOTCU      Plot curves and user points
!     SEPPRESSCORR   Perform one or more time steps with the pressure correction
!                    method to solve the Navier-Stokes equations
!     SEPPRIGLOB     Print global vector in parallel case
!     SEPREADVEC     Read vector from ASCII file
!     SEPRENUMBER    renumbering of mesh
!     SEPTIMD        Solves time-dependent system of equations
!     SEPTIMEFREE    Actual body of subroutine TIMEFREE
!     TOLOOP         Perform test in loop
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     433   Open plot found, whereas the plot has already been opened
!    1472   Plot file has been opened and closed without making any plot.
!    1473   The plot file has not been opened  before, whereas jtimes = 10.
!    1928   free boundaries not allowed in SEPCOMP
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcomsub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcomsub'
         write(irefwr,1) 'isubrmn, istartmn, numvecmn, numvec, meshnr', &
                          isubrmn, istartmn, numvecmn, numvec, meshnr
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      call sepmakerel  ! make relation between global and local arrays
      call sepfillkmesh(kmesh)

      select case ( isubrmn )

         case(1)

!        --- isubrmn = 1, call to presbc
!                       Fill essential boundary conditions

            if ( debug ) write(irefwr,*) 'essential boundary conditions'
            call erpres ( isolut2d, inpsep(istartmn+1), maxvc1, &
                          numvecmn, istartmn, rinsep )

         case(2)

!        --- isubrmn = 2, call to linprb
!                       Build and solve system of linear equations
!                       istartmn is raised by the subroutine itself

            if ( debug ) write(irefwr,*) 'system of linear equations'
            call prlins ( inpsep(istartmn+1) )
            if ( debug ) write(irefwr,*) 'after prlins'

         case(3)

!        --- isubrmn = 3, call to nonlns
!                       Build and solve system of non-linear equations

            if ( debug ) write(irefwr,*)'system of non-linear equations'
            call prnlns ( inpsep(istartmn+1), rinsep )

         case(4,108)

!        --- isubrmn = 4, call to creatn
!                       Create vector

            if ( debug ) write(irefwr,*) 'Create vector'
            call ercret ( inpsep(istartmn), rinsep )

         case(5)

!        --- isubrmn = 5, call to outsol
!                       Write result to file "sepcomp.out"

            if ( debug ) write(irefwr,*) 'call to outsol'
            if ( .not. numwis .and. (itimloop==0 .or. ioutact==1) ) then
               numvec = max(numvec,numvecmn)
               call erouts ( inpsep(istartmn+1) )
            end if
            istartmn = istartmn + 4

         case(6)

!        --- isubrmn = 6, call to manvec
!                       Manipulate vectors

            if ( debug ) then

!           --- debug information

               write(irefwr,*) 'Manipulate vectors'
               write(irefwr,*) 'Before prmanv'
               call prinin ( inpsep(istartmn), 21+nelgrp, &
                             'inpsep from istartmn' )
            end if
            call prmanv ( inpsep(istartmn+1), rinsep )

         case(7,9,15,54)

!        --- isubrmn = 7, print scalar or isubrmn = 9, print vector
!            or isubrmn = 15, print text or isubrmn = 54, print time

            if ( debug ) write(irefwr,*) 'print scalar/vector'
            call erscal ( inpsep(istartmn), rinsep )

         case(8)

!        --- isubrmn = 8, call to derivs
!                       Compute derivatives

            if ( debug ) write(irefwr,*) 'Compute derivatives'
            call prderv ( inpsep(istartmn+1) )

         case(10)

!        --- icode = 10, print constant

            if ( debug ) write(irefwr,*) 'Print constant'
            call princonsta ( inpsep(istartmn+1), istartmn )

         case(11)

!        --- isubrmn = 11, call to integr
!                        Compute integral

            if ( debug ) write(irefwr,*) 'Compute integral'
            call printg ( inpsep(istartmn) )

         case(12)

!        --- isubrmn = 12, scalar i = value/func

            if ( debug ) write(irefwr,*) 'scalar i = value'
            leninpsep = inpsep(istartmn+1)
            call prsclr ( inpsep(istartmn+1), rinsep )
            istartmn = istartmn + leninpsep+2

         case(13)

!        --- isubrmn = 13, call to COMMAT, i.e. change structure of matrix

            if ( debug ) write(irefwr,*) 'change structure of matrix'
            call prcomm ( inpsep(istartmn+1) )
            istartmn = istartmn + 3

         case(14)

!        --- isubrmn = 14, call to PRCOPYCOOR, i.e. copy coordinates in vector

            if ( debug ) write(irefwr,*) 'copy coordinates'
            call prcopycoor ( inpsep(istartmn+1), numvecmn, istartmn )

         case(16)

!        --- isubrmn = 16, call to intbnd
!                        Compute boundary integral

            if ( debug ) write(irefwr,*) 'boundary integral'
            call prbnin ( inpsep(istartmn+1) )

         case(17,18)

!        --- isubrmn = 17 or 18, call to PRFIL2
!                              Read or write array to file 2

            if ( debug ) write(irefwr,*) 'array to file 2'
            call prfil2 ( inpsep(istartmn) )
            istartmn = istartmn + 4

         case(19)

!        --- isubrmn = 19, call to PRTIMD
!                        Solve time-dependent problem

            if ( debug ) write(irefwr,*) 'time-dependent problem'
            maxcalltimsav = maxcalltim   ! save value of maxcalltim
            maxcalltim = 0   ! set maxcalltim to 0 to indicate new time problem
            call septimd ( inpsep(istartmn+1), rinsep )
            maxcalltim = maxcalltimsav   ! reset maxcalltim

         case(20)

!        --- isubrmn = 20, call to USEROUT
!                        User output subroutine

            if ( debug ) write(irefwr,*) 'User output subroutine'
            call pruser ( inpsep(istartmn+1) )

         case(21)

!        --- isubrmn = 21, Start while loop

            if ( debug ) write(irefwr,*) 'Start while loop'
            call prexecwhile ( inpsep(istartmn+1), istartmn, rinsep, &
                               ioutpsav )

         case(22)

!        --- isubrmn = 22, End while loop

            if ( debug ) write(irefwr,*) 'End while loop'
            istartmn = inpsep(istartmn+1)
            ioutp = ioutpsav
            seqwhile = seqwhile-1
            if ( seqwhile<=0 ) whileloop = .false.

         case(23,24,26,27)

!        --- Not implemented

            call errsub ( 1928, 0, 0, 0 )

         case(25)

!        --- isubrmn = 25, call subroutines ERLINC and ERLINS

            call erlins ( 3, inpsep(istartmn+4), inpsep(istartmn+1) )
            call erlinc ( 3, inpsep(istartmn+4), inpsep(istartmn+2) )
            istartmn = istartmn + 5
            nitercom = 1

         case(28)

!        --- isubrmn = 28, call subroutine TIMEFRBF to perform a time step

            if ( debug ) write(irefwr,*) 'perform a time step'
            complrun = .false.
            if ( icalltimstr==1 ) maxcalltim = maxcalltim+1
            if ( seqwhile+iseqfor<1 ) icalltim = icalltim+1
            call septimefree ( timefinished, inpsep(istartmn+1), &
                               rinsep )

         case(29)

!        --- isubrmn = 29, call subroutine READVEC

            if ( debug ) write(irefwr,*) 'subroutine READVEC'
            call sepreadvec ( isolut2d(inpsep(istartmn+1)), &
                              inpsep(istartmn+2), &
                              texts(inpsep(istartmn+3))(1:inpsep(istartmn+4)) )
            numvecmn = max(numvecmn,inpsep(istartmn+1))
            istartmn = istartmn + 5

         case(30)

!        --- OPEN PLOT found
!            Check value of jtimes, and set this value to 1

            if ( debug ) write(irefwr,*) 'OPEN PLOT'
            if ( jtimes/=10 ) then

!           --- Error 433:  Plot file has already been opened before

               call errwar ( 433, 0, 0, 0 )
            else
               jtimes = 11
            end if
            istartmn = istartmn+1

         case(31)

!        --- CLOSE PLOT found
!            Check value of jtimes, and set this value to 3

            if ( debug ) write(irefwr,*) 'CLOSE PLOT'
            if ( jtimes==11 ) then

!           --- Error 1472:  Plot file is opened and closed, without
!                           making any plot

               call errwar ( 1472, 0, 0, 0 )
               jtimes = 10

            else if ( jtimes==10 ) then

!           --- Error 1473:  Plot file has not been opened

               call errwar ( 1473, 0, 0, 0 )

            else

!           --- Close Plot dataset

               jtimes = 13
               call plottime
               call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
               jtimes = 10

            end if
            istartmn = istartmn+1

         case(32)

!        --- PLOT BOUNDARY found

            if ( debug ) write(irefwr,*) 'PLOT BOUNDARY'
            iplots(1) = 2
            do i = 2, 10
               iplots(i) = 0
            end do
            iplots(3) = 1
            rplots(1) = 15d0                          ! plotfm
            if ( inpsep(istartmn+2)==0 ) then
               rplots(2) = 1d0                        ! yfact
            else
               rplots(2) = rinsep(inpsep(istartmn+2))
            end if
            if ( inpsep(istartmn+1)>0 ) then
               jmaxsave = jmax
               jmax = 1
               xmin = rinsep(inpsep(istartmn+1))
               xmax = rinsep(inpsep(istartmn+1)+1)
               ymin = rinsep(inpsep(istartmn+1)+2)
               ymax = rinsep(inpsep(istartmn+1)+3)
            end if
            textx = 'x'
            texty = 'y'
            call sepplotcu ( iplots, rplots )
            if ( inpsep(istartmn+1)==1 ) jmax = jmaxsave
            istartmn = istartmn+3

         case(33)

!        --- REFINE_MESH found

            if ( debug ) write(irefwr,*) 'REFINE_MESH'
            call prrefine ( inpsep(istartmn+1) )

         case(34)

!        --- PRESENT_MESH found

            if ( debug ) write(irefwr,*) 'PRESENT_MESH'
            meshnr = abs(meshseqs(inirepscal(inpsep(istartmn+1))))
            kmesh =>kmesh2d(meshnr)
            call sepfillkmesh ( kmesh )

            if ( ioutp>=0 ) write(irefwr,695) meshnr
695         format ( ' Present mesh number is ', i1 )
            istartmn = istartmn+2

         case(35)

!        --- WRITE_MESH found

            if ( debug ) write(irefwr,*) 'WRITE_MESH'
            iname = inpsep(istartmn+1)
            namesav = name10
            if ( iname>0 ) then

!           --- iname>0, set name of meshoutput file

               name10 = texts(iname)(1:50)
            end if
            call inifil(10)
            lenname = index(name10,' ')-1
            if ( lenname==-1 ) lenname = len(name10)
            if ( ioutp>=0 ) write(irefwr,705) meshnr, &
                 name10(1:lenname)
705         format ( ' Write mesh ', i1, ' to file ', a )
            call sepmeshwr ( iref10 )
            name10 = namesav
            iref = abs(iref10)
            close ( iref )
            istartmn = istartmn+2

         case(36)

!        --- INTERPOLATE found

            if ( debug ) write(irefwr,*) 'INTERPOLATE'
            call printerpolate ( istartmn, inpsep(istartmn+1) )

         case(37)

!        --- isubrmn = 37, Start for loop

            if ( debug ) write(irefwr,*) 'Start for loop'
            call erloopfor ( istartmn, inpsep )
            ioutp = -1

         case(38)

!        --- isubrmn = 38, End for loop

            if ( debug ) write(irefwr,*) 'End for loop'
            istartmn = inpsep(istartmn+1)
            ioutp = ioutpsav

         case(39,57)

!        --- PLOT VECTOR or TENSOR found

            if ( debug ) write(irefwr,*) 'PLOT VECTOR or TENSOR'
            if ( itimloop==0 .or. ioutact==1 ) then

!           --- Only at certain time steps

               call prplotvc ( inpsep(istartmn), rinsep )
            end if
            istartmn = istartmn+11

         case(40)

!        --- PLOT CONTOUR or COLOURED LEVELS found

            if ( debug ) write(irefwr,*) 'PLOT CONTOUR'
            if ( itimloop==0 .or. ioutact==1 ) then

!           --- Only at certain time steps

               call prplotcn ( inpsep(istartmn+1), rinsep )
            end if
            istartmn = istartmn+14

         case(41)

!        --- CONTACT found

            if ( debug ) write(irefwr,*) 'CONTACT'
            call prcontct ( inpsep(istartmn+1) )
            istartmn = istartmn+4

         case(42)

!        --- INTERCHANGE_MESH found

            if ( debug ) write(irefwr,*) 'INTERCHANGE_MESH'
            k = meshseqs(inirepscal(inpsep(istartmn+1)))
            meshseqs(inirepscal(inpsep(istartmn+1))) = &
               meshseqs(inirepscal(inpsep(istartmn+2)))
            meshseqs(inirepscal(inpsep(istartmn+2))) = k
            kmesh => kmesh2d(meshseqs(inirepscal(inpsep(istartmn+1))))
            call sepfillkmesh(kmesh)
            istartmn = istartmn+3

         case(43)

!        --- isubrmn = 43, call to build
!                       Build right-hand side vector

            if ( debug ) write(irefwr,*) 'Build right-hand side vector'
            call prforcev ( inpsep(istartmn+1) )

         case(44)

!        --- isubrmn = 44, start loop

            if ( debug ) write(irefwr,*) 'start loop'
            iseqlp = iseqlp+1
            ireadlp(iseqlp) = inpsep(istartmn+1)
            niterlp(iseqlp) = 0
            call toloop ( niterlp(iseqlp), loopfinished, &
                          ireadlp(iseqlp), iseqlp )
            ioutp = -1
            istartmn = istartmn + 3

         case(45)

!        --- isubrmn = 45, call subroutine TOLOOP and check on convergence

            if ( debug ) write(irefwr,*) 'TOLOOP'
            call toloop ( niterlp(iseqlp), loopfinished, &
                          ireadlp(iseqlp), iseqlp )
            if ( loopfinished ) then
               istartmn = istartmn+2
               niterlp(iseqlp) = 1
               iseqlp = iseqlp-1
               loopfinished = .false.
               ioutp = ioutpsav
            else
               istartmn = inpsep(istartmn+1)
            end if

         case(46)

!        --- isubrmn = 46, start time loop

            if ( debug ) write(irefwr,*) 'start time loop'
            if ( .not. timefinished ) then

!           --- Not yet finished, go to next step

               istartmn = istartmn + 3
               if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=0 ) &
                  write(irefwr,811)
811            format ( /'Start time loop'/)
               icalltimstr = icalltimstr+1
               icalltim = 0
               itimdep = 1
               if ( itimloop==0 ) then

!              --- Start of time loop

                  itimloop = 1
                  ioutact = 1

!                 --- Fill time history of scalars

                  call prtimehisscal

               end if  ! ( itimloop==1 )

            else

!           --- End of loop

               istartmn = inpsep(istartmn+2)
               itimloop = 0
               itimdep = 0
               icalltimstr = 0
               timefinished = .false.
               maxcalltim = 0
               icalltim = 0
               if ( ioutp>=0 ) write(irefwr,812)
812            format ( /'End time loop'/)
            end if

         case(47)

!        --- isubrmn = 47, End time loop

            if ( debug ) write(irefwr,*) 'End time loop'
            istartmn = inpsep(istartmn+1)-3

!           --- Store all scalars in a time history

            call prtimehisscal2

         case(48)

!        --- isubrmn = 48, call to mshchcor
!                        Change coordinates

            if ( debug ) write(irefwr,*) 'Change coordinates'
            call mshchcor ( inpsep(istartmn+1) )
            iinmap(2) = 1
            istartmn = istartmn + 2

         case(49)

!        --- isubrmn = 49, Compute eigenvalues

            if ( debug ) write(irefwr,*) 'Compute eigenvalues'
            call preigval ( inpsep(istartmn+1) )

         case(50)

!        --- isubrmn = 50, Change problem

            if ( debug ) write(irefwr,*) 'Change problem'
            call prchanprob ( inpsep(istartmn+1) )

         case(51)

!        --- isubrmn = 51, Compute capacity

            if ( debug ) write(irefwr,*) 'Compute capacity'
            call prcompcap ( inpsep(istartmn+1) )

         case(52)

!        --- isubrmn = 52, Solve inverse problem

            if ( debug ) write(irefwr,*) 'Solve inverse problem'
            call prcompinver ( inpsep(istartmn+1) )

         case(53)

!        --- isubrmn = 53, Deform mesh

            if ( debug ) write(irefwr,*) 'Deform mesh'
            call sepdefmshnew ( isolut2d(inpsep(istartmn+1)), &
                                rinsep(inpsep(istartmn+2)) )
            istartmn = istartmn + 3

         case(55)

!        --- isubrmn = 55, COMPUTE_PRINCIPLE_STRESSES

            if ( debug ) write(irefwr,*) 'COMPUTE_PRINCIPLE_STRESSES'
            call principstr ( isolut2d, inpsep(istartmn+1) )
            numvecmn = max(numvecmn, inpsep(istartmn+3), inpsep(istartmn+4))
            istartmn = istartmn + 5

         case(56)

!        --- PLOT COLOURED LEVELS found

            if ( debug ) write(irefwr,*) 'PLOT COLOURED LEVELS'
            if ( itimloop==0 .or. ioutact==1 ) then

!           --- Only at certain time steps

               call prplotcol ( inpsep(istartmn+1), rinsep )

            end if
            istartmn = istartmn+14

         case(58)

!        --- isubrmn = 58, MOVE_OBSTACLE

            if ( debug ) write(irefwr,*) 'MOVE_OBSTACLE'
            call prmoveobstac ( inpsep(istartmn+1), rinsep )
            istartmn = istartmn + 12

         case(59)

!        --- isubrmn = 59, MAKE_OBSTACLE_MESH

            if ( debug ) write(irefwr,*) 'MAKE_OBSTACLE_MESH'
            call prmshobstms ( inpsep(istartmn+1) )
            istartmn = istartmn + 4

         case(60)

!        --- isubrmn = 60, REMOVE_OBSTACLE_MESH

            if ( debug ) write(irefwr,*) 'REMOVE_OBSTACLE_MESH'
            call prrmobstmesh ( inpsep(istartmn+1) )

         case(61,64)

!        --- Jump

         case(62)

!        --- PLOT MESH found

            if ( debug ) write(irefwr,*) 'PLOT MESH'
            if ( itimloop==0 .or. ioutact==1 ) then

!           --- Only at certain time steps

               call prplotmesh ( inpsep(istartmn+1), rinsep )
            end if  ! ( itimloop==0 .or. ioutact==1 )

            istartmn = istartmn+9  ! must be outside prplotmesh because of if

         case(63)

!        --- STREAM_FUNCTION found

            if ( debug ) write(irefwr,*) 'COMPUTE STREAM_FUNCTION'
            call prstream ( inpsep(istartmn+1), rinsep )

         case(65)

!        --- isubrmn = 65, READ_MESH

            if ( debug ) write(irefwr,*) 'READ_MESH'
            call prreadmesh ( inpsep(istartmn+1) )

         case(66)

!        --- isubrmn = 66, SET_TIME

            if ( debug ) write(irefwr,*) 'SET_TIME'
            call prsettime ( inpsep(istartmn+1), rinsep )
            istartmn = istartmn + 2

         case(67)

!        --- isubrmn = 67, MAKE_LEVELSET_MESH

            if ( debug ) write(irefwr,*) 'MAKE_LEVELSET_MESH'
            call prmshlevms ( inpsep(istartmn+1), rinsep )

         case(68)

!        --- isubrmn = 68, REMOVE_LEVELSET_MESH

            if ( debug ) write(irefwr,*) 'REMOVE_LEVELSET_MESH'
            call prrmlevmesh ( inpsep(istartmn+1), istartmn )

         case(69)

!        --- isubrmn = 69, PRINT ZERO_LEVELSET

            if ( debug ) write(irefwr,*) 'PRINT ZERO_LEVELSET'
            call printzerolev ( inpsep(istartmn+1), istartmn )

         case(70)

!        --- isubrmn = 70, PLOT_ZERO_LEVELSET

            if ( debug ) write(irefwr,*) 'PLOT_ZERO_LEVELSET'
            call plotzerolev ( inpsep(istartmn+1), istartmn )

         case(71)

!        --- isubrmn = 71, ENTHALPY_INTEGRATION

            complrun = .false.
            if ( icalltimstr==1 ) maxcalltim = maxcalltim+1
            icalltim = icalltim+1
            if ( debug ) write(irefwr,*) 'ENTHALPY_INTEGRATION'
            call sepenthalpymn  ( timefinished, inpsep(istartmn+1) )
            istartmn = istartmn + 2

         case(72)

!        --- isubrmn = 72, COMPUTE_ENTHALPY

            if ( debug ) write(irefwr,*) 'COMPUTE_ENTHALPY'
            call prcompenthalpy ( inpsep(istartmn+1) )

         case(73)

!        --- isubrmn = 73, INTERSECTION

            if ( debug ) write(irefwr,*) 'INTERSECTION'
            call printersect ( inpsep(istartmn+1), rinsep )
            istartmn = istartmn + 5

         case(74)

!        --- isubrmn = 74, TIME_HISTORY

            if ( debug ) write(irefwr,*) 'TIME_HISTORY'
            call prtimehis ( inpsep(istartmn+1), rinsep, istartmn )

         case(75)

!        --- isubrmn = 75, New problem description

            if ( debug ) write(irefwr,*) 'NEW_PROBLEM_DESCRIPTION'
            call prnewprob ( inpsep(istartmn+1), istartmn )

         case(76)

!        --- isubrmn = 76, REF_ENTHALPY_INTEGRATION

            complrun = .false.
            if ( icalltimstr==1 ) maxcalltim = maxcalltim+1
            icalltim = icalltim+1
            if ( debug ) write(irefwr,*) 'REF_ENTHALPY_INTEGRATION'
            call prenthrefine ( inpsep(istartmn+1) )
            istartmn = istartmn + 3

         case(77)

!        --- isubrmn = 77, PRESSURE_CORRECTION

            complrun = .false.
            if ( icalltimstr==1 ) maxcalltim = maxcalltim+1
            icalltim = icalltim+1
            if ( debug ) write(irefwr,*) 'PRESSURE_CORRECTION'
            call seppresscorr ( timefinished,  inpsep(istartmn+1) )
            istartmn = istartmn + 2

         case(78)

!        --- isubrmn = 78, NAVIER_STOKES

            complrun = .false.
            if ( icalltimstr==1 ) maxcalltim = maxcalltim+1
            icalltim = icalltim+1
            if ( debug ) write(irefwr,*) 'NAVIER_STOKES'
            call sepnavstokes ( inpsep(istartmn+1) )

         case(79)

!        --- isubrmn = 79, BEARING

            complrun = .false.
            if ( icalltimstr==1 ) maxcalltim = maxcalltim+1
            icalltim = icalltim+1
            if ( debug ) write(irefwr,*) 'BEARING'
            call prbearing ( inpsep(istartmn+1) )

         case(80)

!        --- isubrmn = 80, Read solutions

            call readsoluts ( inpsep(istartmn+1) )

         case(81)

!        --- isubrmn = 81, Plot function
!                        istartmn is raised in subroutine itself

            call prplotfn ( inpsep(istartmn+1), rinsep )

         case(82)

!        --- icode = 82, PRINT CURVES/SURFACES found

            call printcursur ( inpsep(istartmn+1), istartmn )

         case(83, 115)

!        --- isubrmn = 83, 115 Start if or else_if statement

            if ( debug ) write(irefwr,*) 'Start if statement'
            call sepexecif ( inpsep(istartmn+1), rinsep, isubrmn )

         case(84)

!        --- isubrmn = 84, End if statement

            if ( debug ) write(irefwr,*) 'End if statement'
            call sependif

         case(85)

!        --- isubrmn = 85, call to COPYVC, i.e. copy vectors

            if ( debug ) write(irefwr,*) 'copy vectors'
            call prcopy ( inpsep(istartmn+1) )

         case(86)

!        --- isubrmn = 86, call to print time_history

            if ( debug ) write(irefwr,*) 'print time_history'
            call printimehis ( inpsep(istartmn+1) )

         case(87)

!        --- isubrmn = 87, call to mshcompdispl, i.e. move boundary

            call prmoveboun ( inpsep(istartmn+1) )

         case(88)

!        --- isubrmn = 88, call to , i.e. compute psi

            call prcomppsi ( inpsep(istartmn+1) )

         case(89)

!        --- isubrmn = 89, call to prstoc, i.e. stochastic_investment

            call prstoc ( inpsep(istartmn+1), rinsep )

         case(90)

!        --- isubrmn = 90, call to prcopymesh, i.e. copy mesh

            call prcopymesh ( inpsep(istartmn+1) )

         case(91)

!        --- isubrmn = 91, call to prbubble, i.e. compute bubble

            call prbubble ( inpsep(istartmn+1), rinsep )

         case(92)

!        --- isubrmn = 92, call to plot time_history

            if ( debug ) write(irefwr,*) 'plot time_history'
            call plottimehis ( inpsep(istartmn+1), rinsep )

         case(93)

!        --- isubrmn = 93, matrix_structure

            if ( debug ) write(irefwr,*) 'matrix_structure'
            call prmatstruc ( inpsep(istartmn+1) )
         case(94)

!        --- icode = 94, WRITE_TO_FILE found

            if ( debug ) write(irefwr,*) 'write to file'
            call iniopenfile ( inpsep(istartmn+1), irefwrsave, &
                               inpsep(istartmn+2), ' ' )
            istartmn = istartmn+3

         case(95)

!        --- icode = 95, CLOSE_FILE found, raise istartmn

            if ( debug ) write(irefwr,*) 'close file'
            call iniclosefile ( irefwrsave )
            istartmn = istartmn+1

         case(96)

!        --- icode = 96, PLOT_3D found, raise istartmn

            if ( debug ) write(irefwr,*) 'plot_3D'
            call prplot3d ( inpsep(istartmn+1), istartmn )

         case(97)

!        --- icode = 97, PLOT_INTERSECTION found, raise istartmn

            if ( debug ) write(irefwr,*) 'PLOT_INTERSECTION'
            call prplotln ( inpsep(istartmn+1), rinsep )

         case(98)

!        --- icode = 98, LEVELSET_MESH_VELOCITY found, raise istartmn

            if ( debug ) write(irefwr,*) 'LEVELSET_MESH_VELOCITY'
            call prcorrinterp ( inpsep(istartmn+1), rinsep )

         case(99)

!        --- icode = 99, ADAPT_INTERFACE_BOUNDARY found, raise istartmn

            if ( debug ) write(irefwr,*) 'ADAPT_INTERFACE_BOUNDARY'
            call pradapintboun ( isolut2d, inpsep(istartmn+1), istartmn )

         case(100)

!        --- icode = 100, MAKE_DISTANCE_FUNCTION found, raise istartmn

            if ( debug ) write(irefwr,*) 'MAKE_DISTANCE_FUNCTION'
            call prmakedist ( inpsep(istartmn+1), rinsep )

         case(101)

!        --- icode = 101, COMPARE_MESH found, raise istartmn

            if ( debug ) write(irefwr,*) 'COMPARE MESH'
            call prcompmesh ( inpsep(istartmn+1), istartmn )

         case(102)

!        --- icode = 102, PRINT MESH found, raise istartmn

            if ( debug ) write(irefwr,*) 'PRINT MESH'
            call prprinkmesh ( inpsep(istartmn+1), istartmn )

         case(103)

!        --- icode = 103, BREAK found, jump

            if ( debug ) write(irefwr,*) 'BREAK'
            iaddress = max(1,iseqlp)
            call prbreak ( istartmn, inpsep, ioutpsav, &
                           iseqlp, niterlp(iaddress), timefinished, &
                           icalltimstr, loopfinished )

         case(104)

!        --- icode = 104, CREATE_LEVELSET_VECTOR found

            if ( debug ) write(irefwr,*) 'CREATE_LEVELSET_VECTOR'
            call prcrealevvect ( inpsep(istartmn+1), rinsep )

         case(105)

!        --- icode = 105, INTERSECT_MESH found

            if ( debug ) write(irefwr,*) 'INTERSECT_MESH'
            call printsectmesh ( inpsep(istartmn+1), rinsep )

         case(106)

!        --- icode = 106, PART_NONLINEAR_CONCENTRATION found

            if ( debug ) write(irefwr,*) 'PART_NONLINEAR_CONCENTRATION'
            call prpartpnlnconc ( timefinished, inpsep(istartmn), rinsep )

         case(107)

!        --- icode = 107, STRING = found

            if ( debug ) write(irefwr,*) 'STRING = '
            call prcreatestring ( inpsep(istartmn), istartmn, rinsep )

         case(109)

!        --- icode = 109, print_intersection found

            if ( debug ) write(irefwr,*) 'PRINT_INTERSECTION'
            call prprintints ( inpsep(istartmn+1), rinsep )

         case(110)

!        --- icode = 110, intersection_integral found

            if ( debug ) write(irefwr,*) 'INTERSECTION_INTEGRAL'
            call prlineint ( inpsep(istartmn+1), istartmn, rinsep )

         case(111)

!        --- icode = 111, create_kmesh found

            if ( debug ) write(irefwr,*) 'CREATE_KMESH'
            call prcreatekmesh ( inpsep(istartmn+1) )

         case(112)

!        --- icode = 112, create right-hand side

            if ( debug ) write(irefwr,*) 'CREATE RIGHT_HAND_SIDE'
            call prcreaterhs ( inpsep(istartmn+1) )

         case(113)

!        --- icode = 113, compute pressure

            if ( debug ) write(irefwr,*) 'COMPUTE PRESSURE'
            call sepcomppress ( inpsep(istartmn+1) )

         case(114)

!        --- icode = 114, else

            if ( debug ) write(irefwr,*) 'ELSE'
            call sepelse ( inpsep(istartmn+1) )

         case(116)

!        --- icode = 116, extend_axi_symmetric_mesh

            if ( debug ) write(irefwr,*) 'EXTEND_AXI_SYMMETRIC_MESH'
            call sepextend ( inpsep(istartmn+1) )

         case(117)

!        --- icode = 117, printglob

            if ( debug ) write(irefwr,*) 'PRINTGLOB'
            call seppriglob ( inpsep(istartmn), rinsep )

         case(118)

!        --- icode = 118, renumber

            if ( debug ) write(irefwr,*) 'RENUMBER'
            call seprenumber ( inpsep(istartmn) )

         case default

!        --- internal error, wrong value of isubrmn

            call errint ( isubrmn, 1 )
            call errsub ( 2917, 1, 0, 0 )
            go to 1000

      end select  ! case ( isubrmn)

1000  if ( debug ) then
         write(irefwr,1) 'istartmn, ierror', istartmn, ierror
         write(irefwr,*) 'End sepcomsub'
      end if  ! ( debug )
      call erclos ( 'sepcomsub' )

      end subroutine sepcomsub
