      subroutine sepfilldefcof
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill default coefficients in iuser/user
!     Old subroutine erfilldefcof
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulenparm
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icompl, isubr, lenius, lenusr, ngroup
      logical :: found, compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     found          If true already a possibility has been found
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  CREATE
!     lenusr         Computed length of array USER
!     ngroup         number of element groups including boundary groups etc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLCOFS     Fill arrays iincof/rincof with default values for standard
!                    elements
!     EROPEN         Produces concatenated name of local subroutine
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     SEPCOPIBUF     Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents
!                    of iarray to ibuffr
!     SEPCOPRBUF     Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents
!                    of array to buffer
!     SEPFILLIUSER   Fill information of coefficients as stored in arrays
!                    iincofarr and rincof into iuser/user
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilldefcof' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilldefcof'
         write(irefwr,1) 'iprob', iprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobc => kprobpart%kprobc

!     --- get constants

      ngroup = nelgrp+numnat+nglgrp  ! number of global groups
      nrcof = ngroup * 100           ! size of rincof
      nicof = 4 * nrcof              ! size of iincof

      if ( debug ) &
         write(irefwr,1) 'ndim, nelgrp, numnat, nglgrp', &
                          ndim, nelgrp, numnat, nglgrp

!     --- allocate temporary arrays

      allocate ( iincof(nicof), rincof(nrcof), stat = error )
      if ( error/=0 ) call eralloc ( error, nicof+nrcof, 'iincof' )
      if ( ierror/=0 ) go to 1000

      found = .false.  ! nothing done yet
      iposin = 3+ngroup  ! last address in iincof that is used
      iposrn = 0         ! last address in rincof that is used
      maxprm = 50
      icompl = 0

      iincof = 0
      rincof = 0d0

      if ( debug ) write(irefwr,1) 'iposin, iposrn', iposin, iposrn

!     --- Loop over element groups

      do ielgrp = 1, ngroup

         typenr = kprobc(ielgrp)

         if ( typenr>99 ) then

!        --- typenr>99, hence standard element, fill iuser, user

            if ( found ) then

!           --- already some part has been filled, close with -1

               iposin = iposin+1
               iincof(iposin) = -1

            end if  ! ( found )

            found = .true.  ! some part is filled
            nparm = nparmarr(typenr)  ! number of parameters
            iincof(3+ielgrp) = nparm
            maxprm = max(maxprm, nparm)
            iincof(iposin+1) = ielgrp
            iposin = iposin + 1

            imark(1:nparm) =  0

            if ( debug ) &
               write(irefwr,1) 'ielgrp, typenr, nparm', &
                                ielgrp, typenr, nparm

!           --- Fill coefficients depending on type number

            iposin_h = 0
            call erfillcofs

         end if  ! ( typenr>99 )
         if ( debug ) write(irefwr,1) 'iposin, iposrn', iposin, iposrn

      end do  ! ielgrp = 1, nelgrp+numnat

      if ( found ) then

!     --- found is true, hence iincof has been filled

         iincof(1) = maxprm
         iincof(2) = icompl
         iincof(3) = iprob
         iposin = iposin+1
         iincof(iposin) = -2

         if ( debug ) then
            write(irefwr,1) 'maxprm, icompl, iprob', &
                             maxprm, icompl, iprob
            call prininlarge ( iincof, iposin, 'iincof' )
            call prinrl ( rincof, iposrn, 'rincof' )
         end if  ! ( debug )

!        --- allocate iuser/user

         allocate ( iuser(5+101*ngroup), user(5+50*ngroup), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, 10+151*ngroup, 'iuser' )
         if ( ierror/=0 ) go to 1000

!        --- Fill first 5 positions of IUSER and USER

         iuser(2:5) = 0
         user(2:5) = 0d0

!        --- Initialize first part of IUSER

         iuser(6:5+ngroup) = 0
         iuser(1) = 5 + 101 * ngroup
         user(1) = 5 + 50 * ngroup

!        --- Fill arrays iuser/user from iincof and rincof

         isubr = 1
         compl = .false.
         call sepfilliuser ( isubr, compl )

!        --- Make arrays IUSER and USER permanent

         lenius = iuser(4)
         lenusr = nint(user(4))

         if ( associated(iuser1(iprob)%iuser) ) &
            deallocate ( iuser1(iprob)%iuser, iuser1(iprob)%user )
         allocate ( iuser1(iprob)%iuser(lenius), iuser1(iprob)%user(lenusr) )
         iuser1(iprob)%iuser = iuser(1:lenius)
         iuser1(iprob)%user = user(1:lenusr)

         if ( debug ) then
            call prininlarge ( iuser, lenius, 'iuser' )
            call prinrl ( user, lenusr, 'user' )
         end if  ! ( debug )

!        --- deallocate temporary arrays

         deallocate ( iuser, user, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iuser' )

      end if  ! ( found )

!     --- deallocate temporary arrays

      deallocate ( iincof, rincof, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iincof' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfilldefcof'

      end if  ! ( debug )
      call erclos ( 'sepfilldefcof' )

      end subroutine sepfilldefcof
