      function sepfilldefstr ( namestring, defval, substrings, numsubstr, &
                               numsign, maxnumlet )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give sepfilldefstr either the default value defval or the value of
!     the variable namestring if it exists
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepfilldefstr
      integer, intent(in) :: defval, numsubstr, numsign(numsubstr), &
                             maxnumlet(numsubstr)
      character (len=*), intent(in) :: namestring
      character (len=*), intent(in) :: substrings(numsubstr)

!     defval         i    Default value
!     maxnumlet      i    Maximum number of characters in sub-keywords
!     namestring     i    Name of string
!     numsign        i    Number of siginificant characters in sub-keywords
!     numsubstr      i    Number of sub-keywords
!     sepfilldefstr  o    Value of integer at output
!     substrings     i    Set of sub-keywords
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=132) :: stringname
      integer :: iseq, i

!     i              Counting variable
!     iseq           Sequence number
!     stringname     Name of string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepgetstring, sepgetsubstr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     SEPGETSTRING   Get the sequence number corresponding to the string in
!                    the block constants with name stringname
!     SEPGETSUBSTR   Get sequence number of stringname in set of substrings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3212   String has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilldefstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilldefstr'
         write(irefwr,*) 'namestring ', namestring
         write(irefwr,1) 'defval', defval
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = sepgetstring ( namestring, chstrings, numstrings )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>0 ) then

!     --- iseq > 0, hence it concerns an existing string
!         Find sequence number in substrings

         stringname = texts(instring(iseq))

         if ( debug ) write(irefwr,*) 'stringname ', stringname

         sepfilldefstr = sepgetsubstr ( stringname, substrings, numsubstr, &
                                        numsign, maxnumlet )
         if ( sepfilldefstr==0 ) then

!        --- stringname has incorrect value

            call errchr ( namestring, 1 )
            call errchr ( stringname, 2 )
            do i = 1, numsubstr
               call errchr ( substrings(i)(1:numsign(i)), 2+i )
            end do  ! i = 1, numsubstr
            call errsub ( 3212, 0, 0, numsubstr+2 )

         end if  ! ( sepfilldefstr==0 )

      else

!     --- iseq = 0, use default

         sepfilldefstr = defval

      end if  ! ( iseq>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'sepfilldefstr', sepfilldefstr
         write(irefwr,*) 'End sepfilldefstr'

      end if  ! ( debug )
      call erclos ( 'sepfilldefstr' )

      end function sepfilldefstr
