      subroutine sepfilliuser ( isubr, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information of coefficients as stored in arrays iincof
!     and rincof into iuser/user
!     Old subroutine erfilliuser
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulecoeffs
      use sepmodulenparm
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr
      logical, intent(inout) :: compl

!     compl         i/o   indication whether the arithmetic is real (false)
!                         or complex (true)
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  CREATE
!     iuser         i/o   Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     user          i/o   Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ngroup, nwork, &
                 ibngrp, icount, ielgrp, jcount, jparm, iend
      integer, allocatable :: iwork(:,:), nlparm(:)
      double precision, allocatable :: work(:)

!                    they are not
!     ibngrp         Actual boundary element group number
!     icount         Counter for iincof
!     ielgrp         element group number
!     iend           Last position used at a part
!     iwork          integer work array
!                    See subroutine erexpc
!     jcount         Counter for rincof
!     jparm          Number of parameters in an element group
!     ngroup         Total number of element groups
!     nlparm         Array with number of parameters for element groups
!     nwork          Maximal number of coefficients
!     work           Work array
!                    See subroutine erexpc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCHCF         Translates the contents of IUSER/USER back into IWORK/WORK
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EREXPC         Expands array IINCOF and RINCOF into IINPUT and WORK
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLCOFBNGC Fill IUSER and USER for boundary element group
!                    (complex case)
!     SEPFILLCOFBNGR Fill IUSER and USER for boundary element group (real case)
!     SEPFILLCOFELGC Fill IUSER and USER for element group (complex case)
!     SEPFILLCOFELGR Fill IUSER and USER for element group (real case)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1724   Array IUSER has not been filled correctly
!    1725   Array USER has not been filled correctly
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilliuser' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilliuser'
         write(irefwr,1) 'isubr', isubr
         call prinin ( iincof, 10, 'iincof' )
         call prinrl ( rincof, 1, 'rincof' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ngroup = nelgrp+numnat+nglgrp

      if ( debug ) &
         write(irefwr,1) 'nelgrp, numnat, ngroup', nelgrp, numnat, ngroup

      maxprm = iincof(1)
      nwork  = ngroup * maxprm

      allocate ( iwork(4,nwork), work(2*nwork), nlparm(ngroup), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 6*nwork+ngroup, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize IWORK, WORK and NPARM

      if ( isubr==1 .or. isubr==3 ) then

!     --- isubr = 1, 3 Clear arrays and copy NPARM from IINCOF

         work = 0d0
         iwork = 0
         nlparm(1:ngroup) = iincof(4:ngroup+3)

      else if ( isubr==2 .or. isubr==4 ) then

!     --- isubr = 2, 4 Fill from arrays IUSER and USER
!         Check if arrays IUSER and USER have been filled

         if ( iuser(3)<6+ngroup .or. &
              iuser(3)>iuser(1) .or. &
              iuser(4)>iuser(3) .or. &
              iuser(5)<0 .or. iuser(5)>1 .or. &
              iuser(2)/=ngroup ) &
            call errsub ( 1724, 0, 0, 0)
         if ( user(3)>user(1) .or. user(4)>user(3) ) &
            call errsub ( 1725, 0, 0, 0 )
         if ( ierror/=0 ) go to 1000

         compl = iuser(5)==1
         iend  = iuser(4)
         call erchcf ( iwork, work, nwork, nlparm, ngroup, compl, iend )

      end if

!     --- Expand arrays IINCOFARR and RINCOF into IWORK, WORK and NPARM

      if ( debug ) write(irefwr,1) 'ngroup before erexpc', ngroup
      call erexpc ( iwork, work, nwork, nlparm, ngroup, compl )
      if ( debug ) then

!     --- Debug information

         call prinin ( nlparm, ngroup, 'nlparm' )
         call prinin1 ( iwork, nwork, 4, 'iwork' )
         call prinrl ( work, nwork, 'work' )

      end if  ! ( debug )

!     --- Next  call subroutines FIL103, FIL104, FIL153 and FIL154

      user(2) = 0d0
      user(5) = 0d0
      if ( compl ) then
         iuser(5) = 1
      else
         iuser(5) = 0
      end if
      icount = 1
      jcount = 1
      do ielgrp = 1,nelgrp
         jparm = nlparm(ielgrp)
         if ( jparm>0 ) then
            if ( compl ) then
               call sepfillcofelgc ( ielgrp, jparm, &
                                     iwork(1,icount), work(jcount) )
               icount = icount + jparm
               jcount = jcount + 2*jparm
            else
               call sepfillcofelgr ( ielgrp, jparm, &
                                     iwork(1,icount), work(jcount) )
               icount = icount + jparm
               jcount = jcount + jparm
            end if
         else
            iuser(2) = ielgrp
            if ( ielgrp==1 ) then
               iuser(4) = 5+ngroup
               user(4) = 5d0
            end if
            iuser(5+ielgrp) = iuser(4)+1
         end if
      end do

      do ibngrp = 1,numnat+nglgrp
         jparm = nlparm(ibngrp+nelgrp)
         if ( jparm>0 ) then
            if ( compl ) then
               call sepfillcofbngc ( ibngrp, jparm, &
                                     iwork(1,icount), work(jcount) )
               icount = icount + jparm
               jcount = jcount + 2*jparm
            else
               call sepfillcofbngr ( ibngrp, jparm, &
                                     iwork(1,icount), work(jcount) )
               icount = icount + jparm
               jcount = jcount + jparm
            end if
         else
            iuser(2) = ibngrp+nelgrp
            iuser(5+ibngrp+nelgrp) = iuser(4)+1
         end if
      end do

!     --- deallocate temporary arrays

      deallocate ( iwork, work, nlparm, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iuser, iuser(4), 'iuser' )
         call prinrl ( user, 10, 'user' )
         write(irefwr,*) 'End sepfilliuser'

      end if  ! ( debug )
      call erclos ( 'sepfilliuser' )

      end subroutine sepfilliuser
