      subroutine sepfilltypes ( ngroup )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients depending on type numbers
!     Old subroutine erfilltypes
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulenparm
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ngroup

!     ngroup         i    number of element groups including boundary groups etc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCOFS     Fill arrays iincof/rincof with default values for standard
!                    elements
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilltypes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilltypes'
         write(irefwr,1) 'ielgrp, iposin, iposrn', &
                          ielgrp, iposin, iposrn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobc => kprobpart%kprobc

      typenr = kprobc(ielgrp)
      nparm =  nparmarr(typenr)

      if ( iposin>3+ngroup ) then

!     --- incof has been partly filled
!          Put -1 to close previous input

         iincof(iposin+1) = -1
         iposin = iposin + 1

      end if  ! ( iposin>3+ngroup )

      iincof(3+ielgrp) = nparm
      iincof(iposin+1) = ielgrp
      iposin = iposin + 1
      maxprm = max(nparm, maxprm)

      imark(1:nparm) = 0

      if ( debug ) write(irefwr,1) 'typenr', typenr

      iposin_h = 0
      call erfillcofs

1000  call erclos ( 'sepfilltypes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ielgrp, iposin, iposrn', &
                          ielgrp, iposin, iposrn
         write(irefwr,*) 'End sepfilltypes'

      end if  ! ( debug )

      end subroutine sepfilltypes
