      subroutine sepfindcurvsym
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all curves that are positioned on the symmetry axis
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     symmetry
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesymm
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nnodes, icount, i, j
      integer, pointer :: nodes(:)

!     i              Counting variable
!     icount         Counter
!     j              Counting variable
!     nnodes         Number of nodes in curve
!     nodes          Contains the node numbers of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3218   not all nodes of curve on th symmetry axis
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfindcurvsym' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfindcurvsym'
         call prinin ( iworksymm, npoint, 'iworksymm' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iworkcur(ncurvs), listsymmc(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iworkcur' )
      if ( ierror/=0 ) go to 1000

      iworkcur = 0
      numcurvsymm = 0

      do i = 1, ncurvs

         if ( .not. Curvesub(i)%inner_curve ) then

!        --- Not an internal curve

            nnodes = Curvesub(i)%nnodes
            nodes => Curvesub(i)%nodes

            icount = 0
            do j = 1, nnodes
               if ( iworksymm(nodes(j))>0 ) icount = icount+1
            end do  ! j = 1, nnodes

            if ( debug ) then
               write(irefwr,1) 'icount', icount
               call prinin ( nodes, nnodes, 'nodes' )
            end if  ! ( debug )

            if ( icount==nnodes ) then

!           --- Curve on symmetry axis, mark curve

               iworkcur(i) = 1
               numcurvsymm = numcurvsymm+1
               listsymmc(numcurvsymm) = i

            else if ( icount==2 ) then

               if ( iworksymm(nodes(1))==0 .or. &
                    iworksymm(nodes(nnodes))==0 ) then

!              --- Error, not the situation of begin and end point on
!                         symmetry axis

                  call errint ( i, 1 )
                  call errsub ( 3218, 1, 0, 0 )

               end if  ! ( iworksymm(nodes(1))==0 .or. ... )

            else if ( icount>1 ) then

!           --- More than one point on symmetry axis

               call errint ( i, 1 )
               call errsub ( 3218, 1, 0, 0 )

            end if  ! ( icount==nnodes )

         end if  ! ( .not. Curvesub(i)%inner_curve )

      end do  ! i = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         call prinin ( listsymmc, numcurvsymm, 'listsymmc' )
         write(irefwr,*) 'End sepfindcurvsym'

      end if  ! ( debug )
      call erclos ( 'sepfindcurvsym' )

      end subroutine sepfindcurvsym
