      subroutine sepmshmain
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-01-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Subroutine sepmshmain reads an input file containing information of the
!   mesh, and generates a SEPRAN mesh
!   The output is written to the file with reference number iref10
!   and name name10
!   sepmshmain is called by program sepmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     mesh
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulemain
      use sepmodulecpack
      use sepmodulemach
      use sepmoduleplot
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinput(3), manag(lenmanag), &
              kmeshloc(maxmesh), jstart, icount, &
              iref, lastps
      double precision rinput(1)
      character (len=100)  filename
      logical check

!     check          Indicates if the file exist (true) or not
!     filename       Name of file
!     icount         Counter
!     iinput         integer input array
!     iref           File reference number
!     jstart         Absolute value of first parameter in the call of START
!     kmeshloc       Subdivision of array KMESH
!     lastps         Last non-blank position in name10
!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTR
!                    See subroutine readsp
!     rinput         Dummy parameter for subroutine mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINISH         Stop SEPRAN
!     INIFIL         Open files
!     INIRMF         Remove file if it exists
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     MSH068         write information of the mesh to the file indicated by iref
!     MSHPARBF       Actual body of MSHPAR
!     SEPMESHSUB     General mesh generator
!     SEPSTART       Standard SEPRAN starting subroutine
!     STARTSEP       General start routine for sepran
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
! **********************************************************************
!
!                       I/O
!
!    The I/O is defined by subroutine mesh
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize array KMESH
!     Start SEPRAN
!     Open the error message file
!     Create the mesh by subroutine mesh
!     Write the mesh to the meshoutput file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.

!     --- Initialize array KMESH

      kmesh => kmesh2d(1)
      kmesh = 0

!     --- Start the SEPRAN program

      call startsepran
      call startsep ( 0, manag )

!     --- Standard SEPRAN start subroutine

      jstart = abs(manag(1))
      if ( mod(jstart/100,10)==0 ) jstart = jstart + 100
      if ( manag(1)<0 ) then
         manag(1) = -jstart
      else
         manag(1) = jstart
      end if
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )
      cputiming = .true.
      mainsubr = 1

      call eropen ( 'SEPMESH' )
      call eropen ( 'sepmshmain' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmshmain'

      end if  ! ( debug )

!     --- Remove the file meshoutput

      call inirmf ( 10 )

!     --- Remove files meshoutput_par.xxx if they exist

      iref = inisetnewref()
      lastps = index ( name10, ' ' )-1
      filename = name10
      filename(lastps+1:lastps+8) = '_par.000'
      icount = 0

!     --- Loop over all files meshoutput_par.xxx

100   inquire ( file=filename, exist=check )
      if ( check ) then

!     --- File exists, remove it

         open ( unit = iref, file = filename, status = 'unknown' )
         close ( unit = iref, status = 'delete' )

!        --- Raise icount and construct new filename

         icount = icount+1
         write(filename(lastps+1:lastps+8),110) icount
110      format('_par.',i3.3)
         go to 100

      end if

      call inirmref ( iref )

!     --- Call subroutine mesh, i.e. read input and create mesh
!         ichois = 100, suppresses the call of subroutine TOPOL
!         jkader = -1, suppresses the plot frame

      if ( debug ) write(irefwr,*) 'before mesh'
      jkader = -1
      call sepmeshsub ( 50+manag(8), iinput, rinput )
      if ( ierror/=0 ) go to 1000
      if ( iparallel>0 ) then

!     --- Parallel computing, write submeshes to the files meshoutput_par.xxx

         iinput(1) = mod(iparallel,100)
         iinput(2) = iparallel/100
         iinput(3) = iplot
         call mshparbf ( iinput, kmeshloc )

      end if
      print *,' mesh generation completed'
      call inifil ( 10 )
      call msh068 ( iref10 )

1000  call erclos ( 'sepmshmain' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmshmain'

      end if  ! ( debug )

      call erclos ( 'SEPMESH' )
      call finish (0)
      end subroutine sepmshmain
