      subroutine sepps2output ( imesh, iactmesh, iuser, user, mmcoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 16-06-2010 extra debug
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 26-06-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Output per timestep
!     in case of postprocessing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     post_processing
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iactmesh, imesh, iuser(*), mmcoor(*)
      double precision user(*)

!     iactmesh      i/o   Actual mesh sequence number
!     imesh          i    Actual mesh and problem number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     mmcoor         i    Memory management sequence number with respect to
!                         coor
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPPS2SUB      Perform the actual postprocessing for one time step
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2766   interpolation for a time-dependent problem to another mesh
!           has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2output' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2output'
         write(irefwr,1) 'ntime, iactmesh', ntime, iactmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ntime==1 ) then

!     --- ntime = 1, no time stepping, use iactmesh

         if ( iinputpst(1)==120 .and. iinputpst(80)>0 ) then

!        --- iinputpst(1)=120 function plot
!            iinputpst(106) = 1, all in one picture

            iinputpst(22) = 1
            iinputpst(23) = 1

         end if  ! ( iinputpst(1)==120 .and. iinputpst(80)>0 ) then

         itimst = iactmesh  ! set time to actual mesh. in order
                            ! to get correct solution

!                           --- Call actual subroutine

         call sepps2sub ( mmcoor, itimst, iactmesh, imesh, &
                          iinputpst, lastmapsol, imapsol, jtimst, &
                          iuser, user, ispec )

      else

!     --- ntime>1, time stepping
!         check iactmesh ( must be 1)

         if ( iactmesh/=1 ) then

!        --- error: iactmesh#1, whereas time-dependent solution

            call errsub ( 2766, 0, 0, 0 )
            go to 1000

         end if  ! ( iactmesh/=1 )

         if ( debug ) write(irefwr,1) 'iltime', iltime(1:3)
         do itimst = iltime(1), iltime(2), iltime(3)

            t = time(itimst)
            write ( irefwr, '(a,d12.4 )' ) &
                            ' Actual time ', t
            if ( iinputpst(1)==120 .and. iinputpst(80)>0 ) then

!           --- iinputpst(1)=120 function plot
!               iinputpst(106) = 1, all in one picture

               iinputpst(22) = (itimst-iltime(1))/iltime(3)+1
               iinputpst(23) = (iltime(2)-iltime(1))/iltime(3)+1

            end if  ! ( iinputpst(1)==120 .and. iinputpst(80)>0 ) then

!           --- Call actual subroutine

            call sepps2sub ( mmcoor, itimst, &
                             iactmesh, imesh, iinputpst, lastmapsol, &
                             imapsol, jtimst, iuser, user, ispec )

         end do  ! itimst = iltime(1), iltime(2), iltime(3)

      end if  ! ( ntime==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2output'

      end if  ! ( debug )
      call erclos ( 'sepps2output' )

      end
