      subroutine sepps2sub1 ( itimst, lastmapsol, imapsol, &
                              jtimst, ispec, ivec, skip )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 26-06-2006
!
!   copyright (c) 2006-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute solution corresponding to compute statement
!     in case of postprocessing
!     This may be for example an intersection with a plane
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     post_processing
!     interpolation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itimst, ispec(maxvc1,4), ivec, &
              lastmapsol, imapsol(maxvc1,ntimvc), jtimst
      logical skip

!     imapsol        i    Array of size maxvc1 x ntimvc defining a mapping from
!                         a two-dimensional storage to a one-dimensional storage
!                         with respect to array isol
!                         imapsol(i,j) refers to the i-th vector at time level j
!                         It contains an integer reference number k to isol(.,k)
!                         if this array is used and otherwise a 0
!     ispec          i    Array of length maxvc1 x 3 which indicates for each
!                         array if it corresponds to a function to be computed
!                         or not, from which array it has been computed, and to
!                         which problem resp. mesh it belongs
!                         See common cpost1
!     itimst         i    Integer time level
!     ivec           i    Sequence number of vector of special structure
!     jtimst         i    Integer time level
!     lastmapsol     i    Last entry in isol that has been used
!     skip          i/o   If true the solution vector does not exist
!                         and no actions may be carried out for this vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indsol, lastmapbef

!     indsol         Sequence number of vector in isol
!     lastmapbef     Help variable to store the last value of lastmapsol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INSP01         Compute quantities given by a compute statement
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     671   The vector from which output is required does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2sub1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2sub1'
         write(irefwr,1) 'ivec, itimst', ivec, itimst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indsol = imapsol(ivec,itimst)
      lastmapbef = lastmapsol
      if ( debug ) write(irefwr,1) 'indsol', indsol
      if ( indsol==0 ) then

!     --- Create new entry in isolut2d

         lastmapsol = lastmapsol+1
         imapsol(ivec,itimst) = lastmapsol
         indsol = lastmapsol

      end if
      if ( debug ) &
          write(irefwr,1) 'indsol, isolut2d(indsol)/1', &
                           indsol, isolut2d(indsol)

      if ( isolut2d(indsol)==0 ) then

!     --- isol(ivec,itimst) = 0, so array isol has not been
!         filled.  Check if array must be computed

         call insp01

      end if
      if ( debug ) write(irefwr,1) 'isol(indsol)/2', isolut2d(indsol)
      if ( isolut2d(indsol)==0 ) then

!     --- Array has not been filled

         if ( lastmapsol>lastmapbef ) then

!        --- A new array has been tried to create, reset

            lastmapsol = lastmapsol-1
            imapsol(ivec,itimst) = 0

         end if
         call errint ( ivec-1, 1 )
         call errint ( ispec(ivec,1), 2 )
         call errchr ( names(ivec), 1 )
         call errwar ( 671, 2, 0, 1 )
         jtimes = jtimst
         skip = .true.
         go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2sub1'

      end if  ! ( debug )
      call erclos ( 'sepps2sub1' )

      end
