      subroutine sepreadcoftyps ( code, nncode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients depending on type numbers
!     Old subroutine read02types
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nncode
      character (len=*) :: code(nncode)

!     code           i    Table with keywords
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READ02_220     Read coefficients in case of type number 220
!     READ02_250     Read coefficients in case of type number 250
!     READ02_251     Read coefficients in case of type number 251
!     READ02_304     Read coefficients in case of type number 304
!     READ02_325     Read coefficients in case of type number 325
!     READ02_420     Read coefficients in case of type number 420
!     READ02_700     Read coefficients in case of type number 700
!     READ02_701     Read coefficients in case of type number 701
!     READ02_702     Read coefficients in case of type number 702
!     READ02_703     Read coefficients in case of type number 703
!     READ02_705     Read coefficients in case of type number 705
!     READ02_801     Read coefficients in case of type number 801
!     READ02_803     Read coefficients in case of type number 803
!     READ02_814     Read coefficients in case of type number 814
!     READ02_816     Read coefficients in case of type number 816
!     READ02_900     Read coefficients in case of type number 900
!     READ02_912     Read coefficients in case of type number 912
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadcoftyps' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadcoftyps'
         write(irefwr,1) 'ielgrp, iposin, iposrn', &
                          ielgrp, iposin, iposrn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoef = 0

      select case (typenr)

         case(220)

            call read02_220

         case(250)

            call read02_250

         case(251)

            call read02_251

         case(304)

            call read02_304

         case(325)

            call read02_325

         case(420)

            call read02_420

         case(700)

            call read02_700

         case(701)

            call read02_701

         case(702)

            call read02_702

         case(703)

            call read02_703

         case(705, 800)

            call read02_705

         case(801)

            call read02_801

         case(802)

            call read02_802

         case(803)

            call read02_803

         case(814)

            call read02_814

         case(816)

            call read02_816

         case(900:903)

            call read02_900

         case(912)

            call read02_912

         case default

!        --- For this type number no names of coefficients are known

            call errchr ( string(1:nchar), 1 )
            do i = 1, nncode
               call errchr ( code(i), 1+i )
            end do  ! i = 1, nncode
            call errsub ( 648, 0, 0, nncode+1 )
            call errcrd

      end select  ! case (typenr)

      if ( icoef>0 ) call sepreadcofmark  ! fill and check imark

1000  call erclos ( 'sepreadcoftyps' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         call prininlarge ( iincof, iposin, 'iincof' )
         if ( iposrn>0 ) &
            call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,*) 'End sepreadcoftyps'

      end if  ! ( debug )

      end subroutine sepreadcoftyps
