      subroutine seprelkmesh1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make the relation between kmesh and the pointers in sepmodulekmeshp
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPPOINTKMESHC Fill array pointkmsc with starting addresses of element
!                    groups in kmeshc (minus 1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprelkmesh1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprelkmesh1'
         call ertrace
         write(irefwr,*) 'parallel ', parallel
         write(irefwr,1) 'icountmesh', icountmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'iseqkm', iseqkm

      npelm  => km(iseqkm)%kmesh(4)
      nelgrp => km(iseqkm)%kmesh(5)
      ndim   => km(iseqkm)%kmesh(6)
      nlevel => km(iseqkm)%kmesh(7)
      npoint => km(iseqkm)%kmesh(8)
      nelem  => km(iseqkm)%kmesh(9)
      nuspnt => km(iseqkm)%kmesh(10)
      ncurvs => km(iseqkm)%kmesh(11)
      nsurfs => km(iseqkm)%kmesh(12)
      nvolms => km(iseqkm)%kmesh(13)
      kelgrp => km(iseqkm)%kmesh(14)
      kelma  => km(iseqkm)%kmesh(15)
      kelmb  => km(iseqkm)%kmesh(16)
      kelmc  => km(iseqkm)%kmesh(17)
      kelmd  => km(iseqkm)%kmesh(18)
      kelme  => km(iseqkm)%kmesh(19)
      kelmf  => km(iseqkm)%kmesh(20)
      kelmg  => km(iseqkm)%kmesh(21)
      kelmh  => km(iseqkm)%kmesh(22)
      kelmi  => km(iseqkm)%kmesh(23)
      kelmj  => km(iseqkm)%kmesh(24)
      kelmk  => km(iseqkm)%kmesh(25)
      kelml  => km(iseqkm)%kmesh(26)
      kelmm  => km(iseqkm)%kmesh(27)
      kelmn  => km(iseqkm)%kmesh(28)
      kelmo  => km(iseqkm)%kmesh(29)
      kelmp  => km(iseqkm)%kmesh(30)
      kelmq  => km(iseqkm)%kmesh(31)
      kelmr  => km(iseqkm)%kmesh(32)
      kelms  => km(iseqkm)%kmesh(33)
      kmiinput => km(iseqkm)%kmesh(34)
      kmrinput => km(iseqkm)%kmesh(35)
      kelmt  => km(iseqkm)%kmesh(36)
      kelmz  => km(iseqkm)%kmesh(37)
      kelmu  => km(iseqkm)%kmesh(38)
      meshadapt => km(iseqkm)%kmesh(39)
      nextpnt => km(iseqkm)%kmesh(41)
      numobst => km(iseqkm)%kmesh(42)
      ichangeobst => km(iseqkm)%kmesh(43)
      nsplit => km(iseqkm)%kmesh(44)
      kelmv  => km(iseqkm)%kmesh(45)
      kelmw  => km(iseqkm)%kmesh(46)
      kelmx  => km(iseqkm)%kmesh(47)
      kelmaa => km(iseqkm)%kmesh(48)
      kelmac => km(iseqkm)%kmesh(51)
      kelmad => km(iseqkm)%kmesh(52)
      kelmae => km(iseqkm)%kmesh(55)
      kelmaf => km(iseqkm)%kmesh(56)
      kelmag => km(iseqkm)%kmesh(57)
      mmouter => km(iseqkm)%kmesh(58)
      mmsurfnodes => km(iseqkm)%kmesh(60)
      mmnormal => km(iseqkm)%kmesh(61)
      kelmak => km(iseqkm)%kmesh(62)
      kelmal => km(iseqkm)%kmesh(63)
      nlevelsets => km(iseqkm)%kmesh(64)
      ipstkmeshpar => km(iseqkm)%kmesh(65)
      kelman => km(iseqkm)%kmesh(66)
      kelmxrl => km(iseqkm)%kmesh(68)

      kmesha => km(iseqkm)%kmesha
      kmeshb => km(iseqkm)%kmeshb
      kmeshf => km(iseqkm)%kmeshf
      kmeshg => km(iseqkm)%kmeshg
      kmeshu => km(iseqkm)%kmeshu
      kmesht => km(iseqkm)%kmesht

      coor => km(iseqkm)%coor
      kmeshc => km(iseqkm)%kmeshc
      kmeshd => km(iseqkm)%kmeshd
      if ( associated(km(iseqkm)%kmeshd) ) then
         kmshd1 => kmeshd(1:npoint+1)
         kmshd2 => kmeshd(npoint+2:)
      end if  ! ( associated(km(iseqkm)%kmeshd) )
      kmeshe => km(iseqkm)%kmeshe
      if ( associated(km(iseqkm)%kmeshe) ) then
         kmshe1 => kmeshe(1:npoint+1)
         kmshe2 => kmeshe(npoint+2:)
      end if  ! ( associated(km(iseqkm)%kmeshe) )
      kmeshh => km(iseqkm)%kmeshh
      curvesmsh => km(iseqkm)%curvesmsh
      icurvsmsh => km(iseqkm)%icurvsmsh
      icurvsobst2d => km(iseqkm)%icurvsobst2d
      iactobst => km(iseqkm)%iactobst
      iobscurv => km(iseqkm)%iobscurv
      curvefacts => km(iseqkm)%curvefacts
      kmeshj => km(iseqkm)%kmeshj
      kmeshk => km(iseqkm)%kmeshk
      kmeshl => km(iseqkm)%kmeshl
      kmeshm => km(iseqkm)%kmeshm
      kmeshn => km(iseqkm)%kmeshn
      kmesho => km(iseqkm)%kmesho
      kmsho1 => km(iseqkm)%kmsho1
      kmsho2 => km(iseqkm)%kmsho2
      elsize => km(iseqkm)%elsize
      elsize => km(iseqkm)%elsize
      kmeshq => km(iseqkm)%kmeshq
      kmeshr => km(iseqkm)%kmeshr
      kmeshs => km(iseqkm)%kmeshs
      kmeshs2 => km(iseqkm)%kmeshs2
      iinputkm => km(iseqkm)%iinputkm
      rinputkm => km(iseqkm)%rinputkm
      kmeshv => km(iseqkm)%kmeshv
      kmshv1 => km(iseqkm)%kmshv1
      kmshv2 => km(iseqkm)%kmshv2
      kmeshx => km(iseqkm)%kmeshx
      kmeshxsur => km(iseqkm)%kmeshxsur
      if ( kelmx/=0 ) then
         iseqkmobst = kmeshx(3+ndim,1)
         if ( iseqkmobst>0 ) kmobst => km(iseqkmobst)%kmobst
      end if  ! ( kelmx/=0 )
      kmeshz => km(iseqkm)%kmeshz
      surfnodes => km(iseqkm)%surfnodes
      normal => km(iseqkm)%normal
      rmeshx => km(iseqkm)%rmeshx
      intprop => km(iseqkm)%intprop
      realprop => km(iseqkm)%realprop
      ielgrp_nodes => km(iseqkm)%ielgrp_nodes
      node_elgrps => km(iseqkm)%node_elgrps
      transinfo => km(iseqkm)%transinfo
      blockinfo => km(iseqkm)%blockinfo
      outerboun => km(iseqkm)%outerboun
      curveusrp => km(iseqkm)%curveusrp
      iouterbounseq => km(iseqkm)%iouterbounseq
      kmeshquant => km(iseqkm)%kmeshquant
      if ( kelmaa==0 ) then
         nullify(nmesh)
      else
         nmesh => km(iseqkm)%nmesh
         nmesha => km(iseqkm)%nmesha
         nmeshb => km(iseqkm)%nmeshb
         nmeshc => km(iseqkm)%nmeshc
         nmeshd => km(iseqkm)%nmeshd
         nmeshe => km(iseqkm)%nmeshe
         nmeshf => km(iseqkm)%nmeshf
         nmeshg => km(iseqkm)%nmeshg
         nmeshh => km(iseqkm)%nmeshh
         nmeshi => km(iseqkm)%nmeshi
         nmeshj => km(iseqkm)%nmeshj
         nmeshk => km(iseqkm)%nmeshk
         nmeshl => km(iseqkm)%nmeshl
         nedges = km(iseqkm)%nmesh(4)
         nfaces = km(iseqkm)%nmesh(5)
      end if  ! ( kelmaa==0 )
      kmeshac => km(iseqkm)%kmeshac
      rmeshac => km(iseqkm)%rmeshac
      kmeshad => km(iseqkm)%kmeshad
      ipointobst => km(iseqkm)%ipointobst
      ielemobst => km(iseqkm)%ielemobst
      icrossobst => km(iseqkm)%icrossobst
      rcrossobst => km(iseqkm)%rcrossobst
      obstfaces => km(iseqkm)%obstfaces
      volume_surfs => km(iseqkm)%volume_surfs
      coorglob => km(iseqkm)%coorglob
      kmeshlglob => km(iseqkm)%kmeshlglob
      kmeshmglob => km(iseqkm)%kmeshmglob
      kmeshnglob => km(iseqkm)%kmeshnglob
      mappingpar => km(iseqkm)%mappingpar
      kmeshcglob => km(iseqkm)%kmeshcglob
      globnodes => km(iseqkm)%globnodes
      mappingparcp => km(iseqkm)%mappingparcp

      if ( .not. associated(kmeshpar) ) then

!     --- serial case

         coorglob => coor
         kmeshlglob => kmeshl
         kmeshmglob => kmeshm
         kmeshnglob => kmeshn
         npointglob = npoint

      else

!     --- parallel case

         kmeshpar => km(iseqkm)%kmeshpar
         coorglob => km(iseqkm)%coorglob
         kmeshlglob => km(iseqkm)%kmeshlglob
         kmeshmglob => km(iseqkm)%kmeshmglob
         kmeshnglob => km(iseqkm)%kmeshnglob
         npointglob = kmeshpar(1)

      end if  ! ( .not. parallel )

      if ( associated(km(iseqkm)%kmeshc) ) then
         call sepfillkmeshc   ! Make relation between kmeshc and Topgrp
         call seppointkmeshc  ! Fill array pointkmsc
      end if  ! ( associated(kmeshc) )

      if ( .not. associated(km(iseqkm)%kmeshw) .and. nelgrp>0 ) then

         allocate ( km(iseqkm)%kmeshw(4) )
         km(iseqkm)%kmeshw(3:4) = 0
         km(iseqkm)%kmeshw(1:2) = 1

      end if  ! ( .not. associated(km(iseqkm)%kmeshw) .and. nelgrp>0 )
      if ( .not. associated(km(iseqkm)%intprop) .and. nelgrp>0 ) then

         allocate ( km(iseqkm)%intprop(1,nelgrp) )
         km(iseqkm)%intprop = 0
         nintprop = 1


      end if  ! ( .not. associated(km(iseqkm)%intprop) .and. nelgrp>0 )
      if ( .not. associated(km(iseqkm)%realprop) .and. nelgrp>0 ) then

         allocate ( km(iseqkm)%realprop(1,nelgrp) )
         km(iseqkm)%realprop = 0
         nrealprop = 1

      end if  ! ( .not. associated(km(iseqkm)%realprop) .and. nelgrp>0 )

      kmeshw => km(iseqkm)%kmeshw
      intprop => km(iseqkm)%intprop
      realprop => km(iseqkm)%realprop

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndim', ndim

         write(irefwr,*) 'End seprelkmesh1'

      end if  ! ( debug )
      call erclos ( 'seprelkmesh1' )

      end subroutine seprelkmesh1
