      subroutine sepst1sub1 ( posint, meshonly, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.6  date 25-11-2012 Long integers
!        version  1.5  date 17-05-2012 Extension for parallel
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the first part of subroutine sepst1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletexts
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: manag(lenmanag)
      logical posint, meshonly

!     manag          o    Manager array that sets the parameters in subroutine
!                         SEPSTL
!                         See subroutine readsp
!     meshonly       o    If true only the mesh is plotted
!     posint         o    true if seppost is called interactively
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iref, ihelp(10), lenname,ih, nextrecord
      character (len=100) :: filename

!     filename       Name of file
!     i              General counting variable
!     ihelp          Work array to store input from the input file
!     iref           Help variable to store a reference number
!     lenname        Length of name
!     nextrecord     Defines last record read by readsp
!                    Possible values:
!                    0: none
!                    1: PROBLEM
!                    2: POSTPROCESSING
!                    3: MESH
!                    4: READ MESH
!                    5: EXTRA_MESH_INPUT
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     INITMX         Initialize parameters xmin, xmax, etc and view point
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     PRGATHER       Performs the actual task of joining sepcomp_par.xxx files
!     READPROBIINP   Read information concerning the input for probdfbf from
!                    file sepcomp.out and call subroutine probdfbf
!     READSP         Reads information for subroutine SEPSTL
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFEMESH      creates a linear element mesh based on the nodes
!                    of a spectral element mesh
!     SEPLOGO        Print copyright statement of SEPRAN
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPSTART       Standard SEPRAN starting subroutine
! **********************************************************************
!
!                       I/O
!
!    The files meshoutput, sepcomp.inf and sepcomp.out and if it exists
!    sepcomp.freq are read
!    For a description of meshoutput: see the Programmers Guide Section 3.3
!    For a description of the file sepcomp.inf, see subroutine PROUTP
!    For a description of the file sepcomp.out, see subroutine PRINSL
!    For a description of the file sepcomp.freq, see subroutine FVWRITE
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill common blocks
!     Open files
!     Read array manag
!     Start SEPRAN
!     Open files with mesh and solution
!     Read mesh
!     Read problem definition from sepcomp.inf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepst1sub1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Start the program

      call initcb

      cputiming = .true.
      iprogram = 3

      call initmd

!     --- Open read and write files

      if ( posint ) then
         irefre = 5
         irefwr = 6
      else
         call inifil(5)
         call inifil(6)
      end if

!     --- Open standard elements file and error message file

      call inifil(1)
      call inifil(4)

      call seplogo      ! print copyright statement

!     --- Read array manag (defaults)

      manag(5) = -1
      if ( posint ) then
         manag(1) = 20000
         manag(2) = 1
         manag(3) = -1
         manag(4) = 0
         manag(6) = 102
         manag(7) = 1
         manag(8) = 1
         manag(9) = 1
         manag(10) = ntimvc   ! maximum number of time steps that may be stored
                              ! See comcons2
      else
         call readsp ( manag, lenmanag, nextrecord )
      end if

!     --- Standard SEPRAN start subroutine

      if ( mod(manag(1),100)==0 ) manag(1) = manag(1) + 100
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )
      mainsubr = 4
      call eropen ( 'sepst1' )
      call eropen ( 'sepst1sub1' )
      if ( debug ) write(irefwr,*) 'after startbf'

!     --- Open files meshoutput, sepcomp.inf and sepcomp.out and read mesh

      inquire ( file=name74, exist=meshonly )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'meshonly ', meshonly

      if ( .not. meshonly ) then

!     --- File sepcomp.out does not exist, &
!         check if files sepcomp_par.xxx exist
!         In that case we have a parallel session and create a new file
!         sepcomp.out from the files sepcomp_par.xxx

         lenname = index ( name74, ' ' )-1
         filename = name74(1:lenname-4)//'_par.001'
         inquire ( file=filename, exist=meshonly )

         if ( debug ) then
            write(irefwr,*) 'filename ', filename
            write(irefwr,*) 'meshonly ', meshonly
         end if  ! ( debug )

         if ( meshonly ) then

!        --- Parallel files exist, create sepcomp.out

            call prgather

!           --- Reopen files 1 and 4

            call inifil(1)
            call inifil(4)

!           --- New calls to eropen

            call eropen ( 'sepst1' )
            call eropen ( 'sepst1sub1' )

!           --- Redefine name74 and meshoutput; it has been changed by prgather

            name74 = filename(1:lenname-4)//'.out'
            lenname = index ( name10, ' ' )-1
            name10 = name10(1:lenname-8)
            if ( debug ) then
               write(irefwr,*) 'name74 ', name74
               write(irefwr,*) 'name10 ', name10
            end if  ! ( debug )

         end if  ! ( meshonly )

      end if  ! ( meshonly ) then

      meshonly = .not. meshonly

!     --- Open files

      call inifil ( -10 )

      kmesh => kmesh2d(1)

      if ( kmesh<=0 .or. kmesh>lastmesh ) kmesh = 0
      iseqkm = kmesh
      if ( iseqkm==0 ) call sepcrmeshinfo ( kmesh )
      call seprelkmesh1

      call msh071 ( manag(5), iref10, ihelp )
      iref = abs(iref10)
      close ( unit = iref )
      if ( debug ) write(irefwr,*) 'after msh071'
      if ( ierror/=0 ) go to 1000

!     --- Compute xmin, xmax etc.

      call initmx ( ndim, npoint, coor )
      if ( debug ) write(irefwr,*) 'after initmx'
      if ( ierror/=0 ) go to 1000

!     --- Check if spectral mesh has been found
!         If so call femesh

      if ( kelmu/=0 ) then

!     --- Spectral mesh: fill fem mesh in kmesh2d(.,2)

         iref = 22
         call sepfemesh ( iref, 1 )
         close ( unit = iref )

!        --- interchange kmesh2d(1) and kmesh2d(2)

         ih = kmesh2d(1)
         kmesh2d(1) = kmesh2d(2)
         kmesh2d(2) = ih

      end if
      if ( meshonly ) go to 1000
      if ( debug ) write(irefwr,*) 'after femeshbf'
      if ( ierror/=0 ) go to 1000

!     --- Read problem definition

      call inifil ( -74 )
      if ( debug ) write(irefwr,*) 'before readprobiinp'
      call readprobiinp
      if ( debug ) write(irefwr,*) 'after readprobiinp'

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepst1sub1'

      end if
      call erclos ( 'sepst1sub1' )

      end subroutine sepst1sub1
