      subroutine to0089check ( ictime, iseqkmesh, iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 28-03-2015 Correction ictime=6
!        version  2.1  date 24-12-2014 New definition clinsl
!        version  2.0  date 01-03-2014 Extra parameter stiffmatr
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some check with respect to subroutine to0089
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     right_hand_side
!     solve
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ictime, iseqkmesh, iinsol(*)

!     ictime         i    Indication if information must be read from standard
!                         input file ( 0 ) or that information read before is
!                         used ( 1 )
!                         For an extended description see linprbnew
!     iinsol         i    Input array defined by the user, see to0089
!     iseqkmesh     i/o   Sequence number of mesh in kmesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprecfem, iprob, lenkmesh

!     iprecfem       Indicates if a FEM preconditioning must be applied
!                    in case of a spectral mesh
!                    Possible values:
!                    0: no FEM preconditioning
!                    1: FEM preconditioning
!     iprob          Actual problem number
!     lenkmesh       Length of array kmesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TO0089SEMTST   Check arrays kmesh and intmat in case of spectral
!                    element method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     813   ICTIME out of range
!    1130   ICTIME = 1, whereas TO0089 has not been called before
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089check' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089check'

      end if
      if ( ierror/=0 ) go to 1000
      if ( ictime<0 .or. ictime>7 ) then
         call errint ( ictime, 1 )
         call errsub ( 813, 1, 0, 0 )
      end if

      iprob = max(1,iinsol(5))

      if ( ( ictime==1 .or. ictime>=3 ) .and. &
             .not. associated(iuser1(iprob)%iuser) .and. &
             ictime/=6 ) call errsub ( 1130, 0, 0, 0 )

      iprecfem = iinsol(12)
      if ( iprecfem>0 ) then

!     --- Finite element preconditioning required, check kmesh

         lenkmesh = kmesh
         iseqkmesh = 2
         call to0089semtst ( kmesh, iseqkmesh, iprecfem )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089check'

      end if
      call erclos ( 'to0089check' )

      end
