      subroutine to0089sem ( ivectr, iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 23-02-2016 New call to prsolveinput
!        version  1.2  date 30-07-2014 Use common comcons1
!        version  1.1  date 09-11-2008 Extension of iinsol, rinsol1
!
!   copyright (c) 2003-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute SEM solution by FEM preconditioning
!     reset intmat(2)
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesmf
      use sepmodulecons
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, iread

!     iread          i    Defines how the input must be read
!                         Possible values:
!                         -1:  No input is required
!                          0:  All SEPRAN input has been read by
!                              subroutine SEPSTN until
!                              END_OF_SEPRAN_INPUT or
!                              end of file has been found
!                              This input is used
!                          1:  The input is read as described for
!                              EIGENVAL
!     ivectr        i/o   standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinbldlc(4), iinsol(2)
      double precision rinsol1(1)

!     iinbldlc       User input array for subroutine BUILD
!     iinsol         Integer input array for subroutine PRPCGRAD
!     rinsol1        Real input array for subroutine PRPCGRAD
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRPCGRAD       finite-element preconditioned conjugate-gradient solver
!     SEPBUILD       assemble a system of equations
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPSOLVEINPUT  Fill arrays inpsol and rinsol
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2482   iread has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089sem' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089sem'

      end if
      if ( ierror/=0 ) go to 1000

      iinbldlc(1) = 4
      iinbldlc(2) = 2
      iinbldlc(3) = 1
      iinbldlc(4) = 0
      call sepfillsmf ( iinbldlc, irhsd, isolut2d(ivectr), isolut2d )
      call sepbuild
      if ( ierror/=0 ) go to 1000

!     --- Fill input for linear solver into inpsol and rinsol

      if ( iread<=0 ) then

!     --- Only iread>0, is permitted

         call errint ( iread, 1 )
         call errsub ( 2482, 1, 0, 0 )
         go to 1000

      end if
      call sepsolveinput ( iread, iinsol, rinsol1 )

!     --- Copy specific input for pcgrad into iinsol and rinsol1

      iinsol(1) = inpsol(38,1)
      iinsol(2) = inpsol(37,1)
      rinsol1(1) = rinsol(8,1)
      call prpcgrad ( isolut2d(ivectr), isolut2d(ivectr), iinsol, &
                      rinsol1 )
      intmt(iseqim)%spectral = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089sem'

      end if
      call erclos ( 'to0089sem' )

      end subroutine to0089sem
