      subroutine to0091 ( iinsol, rinsol1, iseqreaclc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.20 date 11-12-2017 New storage of intmat
!        version  3.19 date 23-02-2016 Replace solve1 by seplinsol
!        version  3.18 date 07-01-2015 Extension with erfilldefcof
!
!   copyright (c) 1993-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of non-linear equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     matrix
!     non_linear
!     right_hand_side
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulesol
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulesmf
      use sepmoduleuser
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulesolve
      use sepmodulekmesh
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*), iseqreaclc
      double precision rinsol1(*)

!     iinsol         i    Input array in which all integer user information with
!                         respect to this subroutine is stored
!                         See subroutine readnl
!     iseqreaclc     i    Sequence number of reaction vector in array ISOL
!                         If <0 also the right-hand side contribution is added
!     rinsol1         i    Contains the real input for the solver
!                         At this moment he input is defined as follows
!                         1:  Global accuracy eps
!                         2:  Global relaxation parameter omega
!                         3:  Accuracy with respect to residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm
      parameter ( nparm=10 )
      integer iwork(nparm), isolol(nparm), &
              iter, ieqnt, ncopeq, maxitr, minitr, jprint, iretur, &
              icrit, iseqgl, iseqsl, iseqcf, isubr, i, &
              numchn, isequt, icall, istart, maxcall, &
              lenius, lenusr, iinvec(6), &
              ivectr, istchn, itnext, minchk, maxvc0, &
              inpsol1(ninpsol1), ireads, inpsep(3), &
              iplot, ivecseq, ivectot, methodnl, &
              iresid, icritres, icrittmp, &
              isolit(2), iseqrc, iuseusr, icopyold, limiter, &
              ipinsl, idummy(1), isubdivide, icalluser, iinput(3), jpos, &
              iposep, iposom, iposepr
      integer, target :: matr(2)
      logical finished, diver, first, residu, inputfound
      double precision eps, rinvec(2), diffsl(nparm,7), &
                       rnpsol(nrinsol1), epsres, &
                       minval(nparm), maxval(nparm)
      character (len=25) text(1)
      type (userinfo) :: iusrlc(nparm)
      save matr

!     diffsl         In this array information about the convergence of the
!                    iteration process is stored for each coupled equation
!                    DIFFSL is filled as follows:
!                    diffsl(i,1)   maximum difference between present and
!                                  preceding iteration, i.e. || u(n) - u(n-1) ||
!                                  or  || u(n) - u(n-1) || / || u(n) ||
!                    diffsl(i,2)   value of diffsl(i,1) in the preceding
!                                  iteration, i.e. || u(n-1) - u(n-2) ||
!                    diffsl(i,3)   value of diffsl(i,2) in the preceding
!                                  iteration, i.e. || u(n-2) - u(n-3) ||
!                    diffsl(i,4)   required accuracy
!                    diffsl(i,5)   ||residual||, i.e. || A(n) u(n-1) - f(n)||
!                    diffsl(i,6)   ||residual|| in the preceding iteration
!                    diffsl(i,7)   required residual accuracy
!                    The parameter i indicates the sequence number of the
!                    equation
!     diver          Indicates if the iteration process diverges (true)
!                    or not (false)
!     eps            Local accuracy
!     epsres         Local accuracy of residual
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     first          Indicates if it is the first iteration (true) or not
!     i              Counting variable
!     icall          Call sequence number for present subroutine
!     icalluser      Indicates if the user subroutine finnonlin1 must be
!                    called (>0) or not (0)
!     icopyold       See iinsol(16)
!     icrit          Type of criterion to be used for the termination
!                    See input array iinsol
!     icritres       Type of criterion to be used for the termination with
!                    respect to the residual.  See input array iinsol
!     icrittmp       Help variable to store a value of icrit temporarily
!     idummy         Dummy parameter
!     ieqnt          Equation sequence number
!     iinput         integer input array with respect to subroutine funcnonlin1
!                    iinput is filled as follows:
!                    1:   ieqnt (equation sequence number)
!                    2:   iter (iteration sequence number)
!                    3:   icalluser (value following call_func in input)
!     iinvec         User input array for subroutine MANVEC
!     inpsep         Array with parameters for subroutine EROUTS
!                    This  array is only used for isequt>0
!                    Subroutine EROUTS writes the solution at each iteration
!                    to the file sepcomp.out
!     inpsol1         Array with parameters for subroutine SOLVEL
!                    This  array is only used for iseqsl = 0
!                    Subroutine SOLVEL fills inpsol1 at iteration 1 and uses it
!                    for all other iterations
!     inputfound     If true an input blokc non_linear has been found
!     ipinsl         Starting address of array IINSOL in array IBUFFR
!     iplot          Indicates if the solution must be plotted during the
!                    iteration process (0) or not (1)
!     iposep         Position of eps in array RINSOL
!     iposepr        Position of epsres in array RINSOL
!     iposom         Position of omega in array RINSOL
!     ireads         Parameter iread for subroutine solvel
!     iresid         Standard SEPRAN array to store information of the residual
!     iretur         Indicates if the subroutine must stop if
!                    no convergence has been found (0) or that
!                    control must be given back to the main program (1)
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqgl         Global sequence number
!     iseqrc         Local value of iseqreac
!     iseqsl         Sequence number corresponding to the linear solver
!     isequt         Sequence number corresponding to the output subroutine
!     isolit         Corresponds to the iteration vector. In the iteration
!                    vector the solution of one iteration for one equation
!                    is stored
!     isolol         Corresponds to the solution of the preceding iteration
!     istart         First position used at a part
!     istchn         Starting address of information of calls to CHNCOF in array
!                    IINSOL
!     isubdivide     indicates if quadratic elements must be
!                    treated as a conglomerate of linear elements
!                    (1) or as quadratic elements itself (0)
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  NLNPRB
!     iter           Iteration sequence number
!     itnext         Sequence of next iteration where the coefficients
!                    must be changed
!     iuseusr        indicates if the arrays IUSER and USER from the parameter
!                    list are used (1) or if the parameter ISEQFILCOF is used
!                    (0)
!     iusrlc         contains references to iuser(iprob)
!     ivecseq        Sequence number of the total solution vector with respect
!                    to array isol
!     ivectot        Sequence number of the total solution vector with respect
!                    to array isol
!     ivectr         Sequence number of the solution vector with respect to
!                    array isol
!     iwork          In this array the next relative position in the part
!                    change coefficients of array IINSOL is stored
!     jpos           Position of input arrays in sepmoduleinput
!     jprint         Defines the amount of output produced
!                    Possible values:
!                    -1:  All output information is suppressed, &
!                         even the message of no-convergence
!                     0:  No information about the iteration
!                         process is printed except in the case
!                         of an error
!                     1:  The number of iterations performed is
!                         printed
!                     2:  Some extra information about the iteration
!                         process is printed for each iteration
!                     3:  See 2, in this case also the ratio of two
!                         succeeding iterations is printed
!                     100:  Extended printing, contains all previous options and
!                         prints also the solution in each iteration
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
!     limiter        Sequence number of keyword found after keyword limiter
!     matr           Standard SEPRAN array, containing information of the large
!                    matrix
!     maxcall        Maximum value of icall
!     maxitr         Maximum number of iterations
!     maxval         Maximum value
!     maxvc0         Maximum number of vectors stored in ISOL
!     methodnl       Type of nonlinear iteration process:
!                    0: Standard method
!                    1: Newton
!                    2: incremental newton
!     minchk         Minimum number of iterations before divergence is checked
!     minitr         Minimum number of iterations
!     minval         Minimum value
!     ncopeq         Number of coupled equations
!     nparm          Maximal number of coupled equations
!     numchn         Number of calls of change coefficients
!     residu         If true the residual must be computed
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     rnpsol         Array with parameters for subroutine SOLVEL
!                    This  array is only used for iseqsl = 0
!                    Subroutine SOLVEL fills rnpsol at iteration 1 and uses it
!                    for all other iterations
!     text           text with respect to writing or plotting
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CREATV01       Create a SEPRAN real or complex array (vector)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINC         Help subroutine to call CHNCOF, which changes coefficients
!                    in IUSER and USER
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     EROUTS         Write solution to file "sepcomp.out"
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     FUNCNONLIN1    User subroutine to perform actions during the iteration of
!                    a non-linear iteration process
!     PLOTSL         Plot solution vector
!     PRININ         print 1d integer array
!     SEPBUILD       Build matrix and right-hand side
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLDEFCOF  Fill default coefficients in iuser/user
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPLINSOL      Body of linear solver
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPMANVEC      Vector manipulation subroutine
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMMATRIX    Remove array with sequence number iseq from realm
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLVEINPUT  Fill arrays inpsol and rinsol
!     TOCONVERGCHECK Check if an iteration process has converged or diverged
!     TODIFFNORMS    Fill array diffsl with information of norms of differences
!                    between succesive iterations in a non-linear
!                    iteration process
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1470   IPLOT>0 only implemented for NDIM = 2
!    1658   Start vector has not been filled
!    1659   No information found for coupled equation
!    1856   The i-th solution vector has not been initialized in isol
!    1911   Too many coupled equations
!    1946   icrit>1 only for newton or incremental newton
! **********************************************************************
!
!                       PSEUDO CODE
!
!   finished := false
!   get parameters from array IINSOL
!   iter  := 0
!   While ( not finished ) do
!      Copy solution vectors in sol_old
!      For ieqtn := 1 (1) ncopeq do
!         if ( iter=0 ) then
!            Transform isol(ieqtn) back if necessary
!         Get local parameters if necessary
!         if ( coefficients must be filled ) then
!            call filcof
!         else if ( coefficients must be changed ) then
!            call chncof
!         Transform iteration vector if necessary
!         Build system of equations
!         Compute residual if required
!         solve system of equations
!         Compute norm(s) of difference
!         Print information of iteration process if required
!         Plot solution if required
!         iter := iter + 1
!         Test divergence or convergence of iteration process
!         if ( convergence ) then
!            finished := true
!         else if ( divergence ) then
!            error message
!            finished := true
!         else if ( iter>=maxitr ) then
!            error message
!            finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data matr /2*0/
! ======================================================================
!
      call eropen ( 'to0091' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0091'
         call prinin ( iinsol, 26, 'iinsol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill modules (at this moment only kmesh)

      call sepfillkmesh ( kmesh )

      stiffmat => matr

!     --- get parameters from array IINSOL

      finished  = .false.
      residu = .false.
      first  = .true.
      iter  = 0
      ncopeq = iinsol(1)
      maxitr = iinsol(2)
      jprint = iinsol(3)
      iretur = iinsol(4)
      minitr = iinsol(5)
      iseqgl = iinsol(7)
      ivectr = iinsol(8)-1
      icalluser = iinsol(9)
      maxvc0 = iinsol(10)
      if ( maxvc0==1 ) maxvc0 = 0
      isequt = iinsol(11)
      iplot  = iinsol(12)
      methodnl = iinsol(13)
      ivectot = iinsol(14)
      iuseusr = iinsol(15)
      icopyold = iinsol(16)
      iseqcoef = 0
      omega = 0d0
      isubdivide = 0
      do i = 1, ncopeq
         nullify ( iusrlc(i)%iuser, iusrlc(i)%user )
      end do  ! i = 1, ncopeq

      text(1)   = 'solution at iteration '
      if ( iplot>0 .and. ndim/=2 ) then

!     --- iplot>0 has only been implemented for ndim = 2

         call errint ( iplot, 1 )
         call errint ( ndim, 2 )
         call errwar ( 1470, 2, 0, 0 )
         iplot = 0

      end if
      minchk = max(5,minitr)
      isubr = 3
      diver = .false.
      if ( ncopeq>nparm ) then
         call errint ( ncopeq, 1 )
         call errint ( nparm , 2 )
         call errsub ( 1911, 2, 0, 0 )
      end if

!     --- Check NCOPEQ arrays ISOL and initialize arrays iwork and iusrlc

      diffsl = 0d0
      iwork(1:ncopeq) = 1
      isolol(1:ncopeq) = 0
      do i = 1, ncopeq
         if ( ks(isolut2d(i+ivectr))%typevec/=110 ) then
            call errint ( i+ivectr, 1 )
            call errint ( ncopeq, 2 )
            call errint ( i+ivectr, 2 )
            call errint ( i+ivectr, 3 )
            call errint ( i+ivectr, 4 )
            call errint ( ks(isolut2d(i+ivectr))%typevec, 5 )
            call errsub ( 1856, 5, 0, 0 )
         end if
      end do
      if ( ierror/=0 ) go to 1000

      isolit = 0
      iresid = 0

!     --- limit solutions if necessary

      do i = 1, ncopeq
         limiter = iinsol(17)
         ipinsl = iinsol(25+i)
         if ( ipinsl>0 ) then

!        --- There is information per subcurve

            if ( iinsol(ipinsl+13)>0 ) limiter = iinsol(ipinsl+13)
            if ( iinsol(ipinsl+10)>0 ) methodnl = iinsol(ipinsl+10)
            if ( methodnl==2 ) then
               ivecseq = ivectot
            else
               ivecseq = ivectr
            end if

         end if
         if ( limiter==1 ) then

!        --- limiter = 1; limit solution

            iinvec(1) = 2
            iinvec(2) = 3
            call sepmanvec ( iinvec, rinvec, isolut2d(i+ivecseq), &
                             isolut2d, isolut2d )
            minval(i) = rinvec(1)
            maxval(i) = rinvec(2)
         end if

      end do

!     --- While loop not finished

      do while ( .not. finished )

!     --- Loop over the equations

         do ieqnt = 1, ncopeq

            if ( icopyold>0 ) then

!           --- icopyold>0, this means that the old solution must be copied
!               into isol from position icopyold

               call sepcopyvc ( isolut2d(ivectr+ieqnt), &
                                isolut2d(icopyold-1+ieqnt) )

            end if

!           --- Define global parameters

            eps    = rinsol1(1)
            omega  = rinsol1(2)
            epsres = rinsol1(3)
            icrit  = mod(iinsol(6),100)
            icritres = iinsol(6)/100

            if ( debug ) &
               write(irefwr,1) 'icrit, icritres', icrit, icritres

!           --- Get local parameters if necessary

            ipinsl = iinsol(25+ieqnt)
            iseqcf = 0
            iseqsl = iseqgl
            iprob = ks(isolut2d(ivectr+ieqnt))%iprob
            numchn = 0
            if ( ks(isolut2d(ivectr+ieqnt))%typevec/=110 ) then

!           --- No initial vector has been filled

               call errint ( ieqnt, 1 )
               call errint ( ivectr+ieqnt, 2 )
               call errint ( ncopeq, 3 )
               call errsub ( 1658, 3, 0, 0 )
               go to 1000

            end if

            if ( debug ) write(irefwr,1) 'ipinsl', ipinsl
            if ( ipinsl>0 ) then

!           --- ipinsl>0, information of the equation has been given

               if ( iinsol(ipinsl)>=0 ) then
                  icrittmp = iinsol(ipinsl)
                  if ( icrittmp>90 ) then
                     icritres = (icrittmp+2)/100
                     icrittmp = icrittmp-icritres*100
                     if ( icrittmp>-1 ) icrit = icrittmp
                  else
                     icrit = icrittmp
                  end if
               end if
               if ( debug ) then
                  write(irefwr,1) 'ipinsl, icrit', ipinsl, icrit
                  call prinin ( iinsol(ipinsl), 13, 'iinsol/extra' )
               end if  ! ( debug )

               iposep = iinsol(ipinsl+1)
               iposom = iinsol(ipinsl+2)
               iseqsl = iinsol(ipinsl+3)
               iseqcf = iinsol(ipinsl+4)
               if ( iinsol(ipinsl+5)>0 ) ivectr = iinsol(ipinsl+5)-1
               numchn = iinsol(ipinsl+6)
               istchn = iinsol(ipinsl+7)
               iposepr = iinsol(ipinsl+12)
               isubdivide = iinsol(18)
               if ( iinsol(ipinsl+14)==1 ) isubdivide = 1
               if ( iposep>0 ) eps = rinsol1(iposep)
               if ( iposom>0 ) omega = rinsol1(iposom)
               if ( iposepr>0 ) epsres = rinsol1(iposepr)
               if ( iinsol(ipinsl+10)>0 ) &
                   methodnl = iinsol(ipinsl+10)
               if ( iinsol(ipinsl+11)>0 ) &
                   ivectot = iinsol(ipinsl+11)
               if ( iseqsl<=0 ) iseqsl = iseqgl

            else if ( ncopeq>1 ) then

!           --- Warning no information for the equation has been given

               call errint ( ieqnt, 1 )
               call errint ( ncopeq, 2 )
               call errwar ( 1659, 2, 0, 0 )

            end if
            if ( methodnl==2 ) then
               ivecseq = ivectot
            else
               ivecseq = ivectr
            end if
            residu = icritres>0 .or. icrit>5

            if ( debug ) write(irefwr,1) 'iseqcf', iseqcf

            if ( iseqcf==0 ) then

!           --- No coefficient given, check if coefficients have been readnl

               call sepgetinpfinf ( 6, 1, jpos, maxcall )
               inputfound = maxcall>0
               if ( inputfound ) inputfound = in(jpos)%used
               if ( inputfound ) iseqcf = 1

            end if  ! ( iseqcf==0 )
            if ( debug ) &
               write(irefwr,1) 'iseqcf, methodnl, icrit', &
                                iseqcf, methodnl, icrit

            if ( methodnl==0 .and. icrit>1 ) then

!           --- Special criteria only possible in combination with
!               Newton

               if ( icrit==2 ) then
                  call errchr ( 'rhs_absolute', 1 )
                  call errchr ( 'absolute', 2 )
                  icrit = 0
               else if ( icrit==3 ) then
                  call errchr ( 'rhs_relative', 1 )
                  call errchr ( 'relative', 2 )
                  icrit = 1
               else if ( icrit==4 ) then
                  call errchr ( 'energy_absolute', 1 )
                  call errchr ( 'absolute', 2 )
                  icrit = 0
               else if ( icrit<6 ) then
                  call errchr ( 'energy_relative', 1 )
                  call errchr ( 'relative', 2 )
                  icrit = 1
               end if
               call errwar ( 1946, 0, 0, 2 )
            end if
            diffsl(ieqnt,4) = eps
            diffsl(ieqnt,7) = epsres
            if ( icrit>5 )diffsl(ieqnt,7) = eps

            if ( iter==0 .and. iseqcf>0 ) then

!           --- First iteration, fill coefficients

               if ( iseqcf/=iseqcoef .or. iseqcf==0 ) then

!              --- New coefficient

                  iuser1(iprob)%iuser => iusrlc(ieqnt)%iuser
                  iuser1(iprob)%user => iusrlc(ieqnt)%user
                  call erlins ( isubr, iprob, iseqcf )
               end if
               iusrlc(ieqnt)%iuser => iuser1(iprob)%iuser
               iusrlc(ieqnt)%user => iuser1(iprob)%user

            else if ( iter==0 .and. iseqcf==0 ) then

!           --- First iteration, fill coefficients from structure block

               call sepfilldefcof
               iusrlc(ieqnt)%iuser => iuser1(iprob)%iuser
               iusrlc(ieqnt)%user => iuser1(iprob)%user

            else if ( iter>0 .and. numchn>0 ) then

!           --- Next iteration, check if coefficients must be changed

               itnext = iinsol(istchn+2*iwork(ieqnt)-1)
               if ( iter+1==itnext ) then

!              --- coefficients must be changed

                  iseqcf = iinsol(istchn+2*iwork(ieqnt))
                  if ( associated(iusrlc(ieqnt)%iuser) ) then
                     iuser1(iprob)%iuser => iusrlc(ieqnt)%iuser
                     iuser1(iprob)%user => iusrlc(ieqnt)%user
                  end if  ! ( associated(iusrlc(ieqnt)%iuser) )
                  call erlinc ( isubr, iprob, iseqcf )
                  iusrlc(ieqnt)%iuser => iuser1(iprob)%iuser
                  iusrlc(ieqnt)%user => iuser1(iprob)%user
                  if ( iwork(ieqnt)<numchn ) iwork(ieqnt) = &
                       iwork(ieqnt) + 1

               end if

            end if
            if ( ierror/=0 ) go to 1000

!           --- Copy solution vector in sol_iter

            if ( iter==0 .or. omega<epsmac .or. &
                 abs(omega-1d0)<epsmac ) then

!           --- either first iteration or omega = 0 or 1

               if ( iter==0 ) &
                   call seploctrn ( 2, &
                                   isolut2d(ivectr+ieqnt) )
               if ( methodnl==0 .or. methodnl==2 .and. iter==0 ) &
                  then

!              --- Either standard method or first iteration of incremental
!                  Newton method. Put boundary conditions into isolit

                  call sepcopyvc ( isolut2d(ivectr+ieqnt), isolit )

               else

!              --- Newton method or subsequent iteration of incremental newton
!                  Set boundary conditions equal to zero

                  iinvec(1) = 0
                  call creatv01 ( isolit, iinvec, rinvec, isolut2d )

               end if

            else

!           --- create combination between previous iteration and
!               last iteration and store in isolit as well as in isolut2d

               iinvec(1) = 2
               iinvec(2) = 27
               rinvec(1) = omega
               rinvec(2) = 1d0-omega
               if ( omega<epsmac .or. abs(omega-1d0)<epsmac ) then
                  call sepcopyvc ( isolut2d(ivectr+ieqnt), &
                                  isolit )
               else
                  call sepmanvec ( iinvec, rinvec, isolut2d(ivectr+ieqnt), &
                                   isolol(ieqnt), isolit )
                  call sepcopyvc ( isolit, isolut2d(ivectr+ieqnt) )
               end if

            end if
            if ( ierror/=0 ) go to 1000

!           --- Build system of equations
!               Copy arrays IUSER and USER to end of IBUFFR.  In this way BUILD
!               cannot overwrite the positions used by these arrays

            if ( associated(iusrlc(ieqnt)%iuser) .and. iuseusr==0 ) then

!           --- iuseusr = 0, use filcof iuser/user

               iuser1(iprob)%iuser => iusrlc(ieqnt)%iuser
               iuser1(iprob)%user => iusrlc(ieqnt)%user
               iuser => iuser1(iprob)%iuser
               user => iuser1(iprob)%user

            else if ( iuseusr==0 ) then

  !             allocate ( iuser(1), user(1), stat = error )
  !             if ( error/=0 ) call eralloc ( error, 2, 'iuser' )
  !             if ( ierror/=0 ) go to 1000

  !             iuser(1) = 1
  !             user(1) = 1d0

            end if
!           --- Copy new iterand into solold

            if ( ierror/=0 ) go to 1000
            if ( methodnl==0 ) &
               call sepcopyvc ( isolit, isolol(ieqnt) )

!           --- Transform isolit back in case of local transform

            call seploctrn ( 1, isolit )

            iinbld = 0
            iinbld(1) = 15
            iinbld(2) = 1
            iinbld(5) = 1
            if ( iseqreaclc>0 ) iinbld(8) = 1
            iinbld(9) = iprob
            iinbld(10) = maxvc0
            iinbld(12) = 1
            iinbld(13) = ieqnt+ivecseq
            iinbld(15) = isubdivide
            if ( methodnl==0 .or. methodnl==2 .and. iter==0 ) then

!           --- Either standard method or first iteration of incremental
!               Newton method. Put boundary conditions into isolit

               iinbld(3) = 0

            else

!           --- Newton method or subsequent iteration of incremental newton
!               Set boundary conditions equal to zero

               iinbld(3) = 1

            end if
            if ( debug ) then
               call prinin ( iinbld, 15, 'iinbld' )
               write(irefwr,1) 'iuseusr', iuseusr
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000

            isubcall = 5

            intmat => intmat1d(iprob)
            call sepfillsmf ( iinbld, irhsd, isolit, isolut2d )
            call sepbuild

            if ( ierror/=0 ) go to 1000

            if ( residu .and. iter>0 ) then

!           --- Compute part of the residual (Au), using the old solution

               call sepmaver ( matr, isolol(ieqnt), iresid, 4 )

!              --- Store norm of residual into diffsl

               diffsl(ieqnt,6) = diffsl(ieqnt,5)
               iinvec(1) = 6
               iinvec(2) = 5
               iinvec(3) = 0
               iinvec(4) = 1
               iinvec(5) = 1
               iinvec(6) = 3
               call sepmanvec ( iinvec, rinvec, iresid, irhsd, iresid )
               diffsl(ieqnt,5) = rinvec(1)

            end if

!           --- solve system of equations (the solution is transformed)

            if ( iseqsl==0 ) then

!           --- iseqsl = 0, no input given before

               if ( first ) then
                  ireads = -2
                  first = .false.
               else
                  ireads = -1
               end if

            else

!           --- iseqsl>0, input given before

               ireads = iseqsl

            end if
            inpsol1(1) = ninpsol1
            rnpsol(1) = dble(nrinsol1)
            if ( iter>0 .and. ireads>0 .and. ireads<1000 ) &
               ireads = ireads+1000
            if ( iseqreaclc/=0 ) then

!           --- Reaction forces are used,
!               in this case it is necessary to store the reaction force
!               in isolit(.,2)

               iseqrc = 2
            else
               iseqrc = 0
            end if
            ireadinpsol = ireads
            iseqsol = 1
            iseqreac = iseqrc
            iseqintmat = iprob
            keepmat = 0
            iseqsolut = 1
            iseqmt = matr(1)
            call sepsolveinput ( ireadinpsol, inpsol1, rnpsol )
            call seplinsolkprob ( irhsd )
            call seplinsol ( matr, isolit, irhsd )
            if ( ierror/=0 ) go to 1000
            iseqrc = abs(iseqreaclc)
            if ( iseqreaclc/=0 ) &
               call sepcopyvc ( isolit(2), isolut2d(iseqrc) )
            if ( ierror/=0 ) go to 1000

!           --- Put isolit into isolut2d

            if ( methodnl==0 ) then

!           --- Standard situation (copy)

               call sepcopyvc ( isolit(1), isolut2d(ieqnt+ivectr) )

            else

!           --- Newton type of equation: add correction to solution

               iinvec(1) = 2
               iinvec(2) = 27
               rinvec(1) = 1d0
               rinvec(2) = 1d0
               call sepmanvec ( iinvec, rinvec, &
                                isolut2d(ieqnt+ivecseq), isolit, &
                                isolut2d(ieqnt+ivecseq) )

            end if
            write(text(1)(23:25),230) iter + 1
230         format(i3)
            isolut => isolut2d(ieqnt+ivecseq)
            if ( jprint>=100 ) call sepprintv ( isolut, text(1) )
            if ( iplot==1 ) call plotsl ( text )
            if ( ierror/=0 ) go to 1000

!           --- Compute norm(s) of difference

            call todiffnorms ( methodnl, icrit, isolut2d, &
                               isolol, ieqnt, ivectr, &
                               isolit, irhsd, diffsl, nparm, icritres, &
                               ivecseq )
            if ( ierror/=0 ) go to 1000

            if ( icalluser>=1 ) then

!           --- Compute quantities in user defined subroutine

               iinput(1) = ieqnt
               iinput(2) = iter
               iinput(3) = icalluser
               call funcnonlin1 ( iinput, isolut2d, kmesh, kprob )

            end if  ! ( icalluser==1 ) then

  !          if ( .not. associated(iusrlc(ieqnt)%iuser) .and. iuseusr==0 ) then

  !             deallocate ( iuser, user, stat = error )
  !             if ( error/=0 ) call erdealloc ( error, 'iuser' )
  !             if ( ierror/=0 ) go to 1000

  !          end if  ! ( .not. associated(iusrlc(ieqnt)%iuser) .and. iuseusr==0 )

         end do  ! ieqnt = 1, ncopeq

!        --- Print information of iteration process if required

         iter = iter + 1

!        --- Test divergence or convergence of iteration process
!            Print information of the convergence depending on jprint
!            Copy diffsl(.,2) into diffsl(.,3) and diffsl(.,1) into diffsl(.,2)

         call toconvergcheck ( iter, minitr, finished, diffsl, &
                               nparm, ncopeq, residu, minchk, diver, &
                               jprint, iretur, maxitr, icrit )
         if ( ierror/=0 ) go to 1000

         if ( isequt>0 ) then

!        --- isequt>0, i.e. Output must be written to the file sepcomp.out
!                        in each iteration

            numvecmn = max(1,maxvc0)
            inpsep(1) = isequt
            inpsep(2) = 0
            inpsep(3) = ivecseq+1
            call erouts ( inpsep )

         end if

      end do  ! while ( .not. finished )

!     --- limit solutions if necessary

      do i = 1, ncopeq
         limiter = iinsol(17)
         ipinsl = iinsol(25+i)
         if ( ipinsl>0 ) then

!        --- There is information per subcurve

            if ( iinsol(ipinsl+13)>0 ) limiter = iinsol(ipinsl+13)
            if ( iinsol(ipinsl+10)>0 ) methodnl = iinsol(ipinsl+10)-1
            if ( methodnl==2 ) then
               ivecseq = ivectot
            else
               ivecseq = ivectr
            end if

         end if
         if ( limiter==1 ) then

!        --- limiter = 1; limit solution

            iinvec(1) = 2
            iinvec(2) = 42
            rinvec(1) = minval(i)
            rinvec(2) = maxval(i)
            call sepmanvec ( iinvec, rinvec, isolut2d(i+ivecseq), &
                             isolut2d, isolut2d )
         end if

      end do

!     --- Clear superfluous arrays

      call seprmmatrix ( matr(1) )
      if ( matr(2)>0 ) call seprmmatrix ( matr(2) )
      call seprmsolinfo ( irhsd(1) )
      if ( irhsd(2)>0 ) call seprmsolinfo ( irhsd(2) )
      call seprmsolinfo ( isolit(1) )
      if ( iseqreaclc/=0 ) call seprmsolinfo ( isolit(2) )
      do i = 1, ncopeq
         if ( isolol(i)/=0 ) &
            call seprmsolinfo ( isolol(i) )
         nullify ( iusrlc(i)%iuser, iusrlc(i)%user )
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0091'

      end if  ! ( debug )
      call erclos ( 'to0091' )

      end subroutine to0091
