      subroutine to0094 ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.55 date 13-12-2014 Remove change coefficients
!        version  3.54 date 17-08-2013 Set nitercom
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutine SEPCOM, with exception of SEPRAN start and SEPRAN
!     reading of input
! **********************************************************************
!
!                       KEYWORDS
!
!     compute_all
!     driving_subroutine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulecomp
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision rinsep(*)

!     inpsep  i    The integer input read is stored in array INPSEP
!                  See subroutine sepcomsub for a description
!     rinsep  i    Real input array (Depends on inpsep)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOMSUB      Actual body of TO0094 and TOFREESB
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0094' )

      debug = .false.

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from to0094'
         call prinin ( inpsep, 10, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if

      istartmn = 2
      isubrmn  = 0
      iseqlp = 0
      ioutpsav = ioutp

!     --- Clear arrays

      do k = 1, maxmesh
         meshseqs(k) = k
      end do  ! k = 1, maxmesh

      numvecmn = 0
      nitercom = 0
      niterlp = 0
      jtimes = 10
      jkader = -1
      meshnr = meshseqs(1)
      timefinished = .false.
      icalltimstr = 0
      iinmap(1) = 2
      iinmap(2) = 1

      iseqif = 0
      checkif = .false.

100   if ( isubrmn>=0 ) then

!     --- isubrmn>=0, take next subroutine

         isubrmn  = inpsep(istartmn)

         if ( debug ) write(irefwr,1) 'istartmn, isubrmn', istartmn, isubrmn

         if ( isubrmn==64 .or. ierror/=0 ) go to 1000

         if ( isubrmn==-1 ) go to 100

!        --- Call main subroutine for sepcomp

         if ( isubrmn==64 ) then

!        --- isubrmn = 64, stop

            go to 1000

         end if

         if ( debug ) write(irefwr,*) 'before sepcomsub'
         call sepcomsub ( inpsep, rinsep )
         if ( debug ) write(irefwr,*) 'after sepcomsub'
         if ( ierror/=0 ) go to 1000
         go to 100

      end if

1000  call erclos ( 'to0094' )
      if ( debug ) write(irefwr,*) 'end to0094'
      end
