      subroutine chksbm ( kmeshclc, nselem, jshape, jnpelm, isurnr, &
                          coor2d, nldim, icheck, npointlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.4  date 22-01-2014 Change format
!        version  3.3  date 22-01-2007 Compute ratio of inner and outer circles
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check volume of submesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nselem, jnpelm, kmeshclc(jnpelm,nselem), jshape, isurnr, &
              nldim, icheck, npointlc
      double precision coor2d(nldim,npointlc)

!     coor2d         i    contains the co-ordinates of the nodal points
!     icheck         i    Level of check of elements
!                         0:  no special checks (default)
!                         1:  check volume of elements
!                         2:  check volume of elements and print minimum and
!                             maximum value
!                             per surface/volume
!     isurnr         i    Surface sequence number
!     jnpelm         i    Number of nodes in an element
!     jshape         i    Shape number of elements
!     kmeshclc       i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     nldim          i    Dimension of space
!     npointlc       i    Local number of points
!     nselem         i    number of surface elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision valmin, valmax, value

!     valmax         Maximum value of computed integral
!     valmin         Minimum value of computed integral
!     value          Computed integral
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHECKTETS   Compute the minimal ratio between radius of inner sphere
!                    and outer sphere of all tets in a volume
!     MSHSUBVOL      Compute volume of submesh
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chksbm' )

!     --- Compute volume of submesh

      call mshsubvol ( kmeshclc, nselem, jshape, jnpelm, isurnr, &
                       coor2d, nldim, valmin, valmax, value, npointlc )
      if ( ierror/=0 ) go to 1000

      if ( icheck>=2 ) then

!     --- write minimum and maximum value of integral

         if ( isubr==2 ) then
            write ( irefwr, 110 ) isurnr, value, isurnr, valmin, isurnr, &
                                  valmax
110         format ( ' Volume of volume ', i6, ' is ', ss, 1pe15.6e2 /&
                     ' Volume of smallest element in volume ', i6, &
                     ' is ', ss, 1pe15.6e2 /&
                     ' Volume of largest  element in volume ', i6, &
                     ' is ', ss, 1pe15.6e2 )

!           --- Compute ratio of radius of inner sphere and outer sphere
!               of all elements (test only)

            call mshchecktets ( jshape, inpelm, kmeshclc, nselem, &
                                isurnr, coor2d )

         else

            write ( irefwr, 120 ) isurnr, value, isurnr, valmin, isurnr, &
                                  valmax
120         format ( ' Volume of surface', i6, ' is ', ss, 1pe15.6e2 /&
                     ' Volume of smallest element in surface ', i6, &
                     ' is ', ss, 1pe15.6e2 /&
                     ' Volume of largest  element in surface ', i6, &
                     ' is ', ss, 1pe15.6e2 )

         end if

      end if
1000  call erclos ( 'chksbm' )

      end
