      subroutine msh015 ( iinput, inpold, ipoint, nsurfs, leng1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.17 date 30-11-2014 Change isurf 23 in 99
!        version  5.16 date 20-10-2008 Allow isurf = 24
!        version  5.15 date 01-07-2008 Extension with internal curves
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array iinput with information of surfaces by rearranging the
!     information in inpold
!     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!     If this subroutine is changed also subroutine MSH049 must be changed
!     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), inpold(*), ipoint(*), nsurfs, leng1

!     iinput         i    new array iinput
!     inpold         i    old array iinput
!     ipoint         i    pointer array, see mshrd2
!     leng1          i    length of first part of array iinput corresponding to
!                         the curves
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, isurf, j, jinp,  n, nrow

!     i              Counting variable
!     iinp           Starting address in array inpold
!     isurf          Type of surface
!     j              Loop variable
!     jinp           Starting address in array iinput
!     n              Number of integers information in iinput for surface
!     nrow           Number of subsurfaces in case isurf=11 or 12
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     379   Incorrect type of surface generator found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh015' )

      jinp = leng1

      do i = 1, nsurfs

         iinp = ipoint(i)
         if ( iinp<=0 ) then

!        --- surface i has not been defined

             iinput(jinp+1) = 0
             jinp = jinp + 1

         else

            isurf = inpold(iinp)
            if ( isurf==24 ) isurf = 7    ! compound surface
            if ( isurf<=0 .or. isurf>24 .and. isurf/=99 ) then

!           --- Type of surface out of range

               call errint ( isurf, 1 )
               call errint ( i, 2 )
               call errsub ( 379, 2, 0, 0)

            end if  ! ( isurf<=0 .or. isurf>99 )

            n = 3+mod(inpold(iinp+2),10000)
            if ( isurf==1 .or. isurf==5 ) then
               n = n+2
            else if ( isurf==4 ) then
               n = n+2
            else if ( isurf==18 ) then
               n = n+3
            else if ( isurf==20 ) then
               n = n+6
            else if ( isurf==13 .or. isurf==15 .or. isurf==17 ) then
               n = n+4
            else if ( isurf== 8 .or. isurf== 9 .or. &
                      isurf==14 .or. isurf==16 .or. &
                      isurf==19 .or. isurf==21 ) then
               n = n+1
            else if ( isurf==11 .or. isurf==12 ) then
               nrow  =  inpold(iinp+2)
               do j  =  1,nrow
                  n  =  n + inpold(iinp+2+j)
               end do
            else if ( isurf==22 .or. isurf==99 ) then

!           --- isurf = 22, 99, framesurf or triangle with extra points
!               extra space for internal points and internal curves

               n = n+inpold(iinp+n)+2+inpold(iinp+n+1)

            end if
            iinp = iinp-1

            do j = 1, n
               iinput(jinp+j) = inpold(iinp+j)
            end do
            jinp = jinp+n

         end if

      end do

      iinput(4) = nsurfs

      call erclos ( 'msh015' )

      end
