      subroutine msh037 ( iinput, nlines, nsurel, nvolel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.4  date 02-09-2013 Correction ishape
!        version  5.3  date 28-07-1999 Layout
!        version  5.2  date 21-09-1992 Extra error message
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill array iinput with information of elements when no
!      card with mesh.... is read
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nlines, nsurel, nvolel

!     iinput        i/o   integer input array
!                         iinput must be called from position 6
!                         the information of the mesh.... card is filled
!     nlines        i/o   number of parts where line elements are created
!     nsurel        i/o   number of parts where surface elements are defined
!     nvolel        i/o   number of parts where volume elements are defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurvs, nsurfs, nvolms, iinp, icurnr, icode, ishape, ish, &
              isurnr, ivolm, ndim, ncurvsact, nsurfsact

!     icode          Return code with respect to subroutine INTX01
!                    Indicates position in array code or code1
!     icurnr         curve sequence number
!     iinp           last position used in array iinput
!     ish            Shape number last used
!     ishape         Shape number of elements
!     isurnr         surface sequence number
!     ivolm          Type of volume with respect to mesh generator
!     ncurvs         Number of curves in mesh
!     ncurvsact      Actual number of curves to be used
!     ndim           Dimension of the space.
!     nsurfs         Number of surfaces in mesh
!     nsurfsact      Actual number of surfaces to be used
!     nvolms         number of volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSH046         Raise iinp as function of icurnr
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     MSH050         Raise iinp as function of ivolm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     301   no record with meshlines found
!     302   no record with meshsurfs found
!     303   no record with meshvolume found
!    1818   Surfaces of surfaces are created in a two-dimensional mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh037' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh037'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim   = iinput(1)
      ncurvs = iinput(3)
      nsurfs = iinput(4)
      nvolms = iinput(5)
      iinp = 16
      ish = -1
      do icurnr = 1, ncurvs
         icode = iinput(iinp)
         if ( icode>0 ) then
            ishape = iinput(iinp+1)
            if ( ish==-1) ish = ishape
            if ( ishape/=ish ) ish = 0
            ncurvsact = icurnr
         end if
         call msh046(iinput, iinp, icode)
      end do
      if ( nsurfs==0 .and. nvolms==0 ) then
         if ( ish==0 ) call errsub ( 301, 0, 0, 0 )
         nlines = 1
         iinput(iinp) = ish
         iinput(iinp+1) = 1
         iinput(iinp+2) = 1
         iinput(iinp+3) = ncurvsact
         go to 1000
      end if
      ish = -1
      do isurnr = 1, nsurfs
         icode = iinput(iinp)
         if ( ndim==2 .and. icode==11 ) call errsub ( 1818, 0, 0, 0)
         if ( icode>0 ) then
            ishape = iinput(iinp+1)
            if ( ish==-1 .and. ishape/=0 ) ish = ishape

            if ( debug ) &
               write(irefwr,1) 'isurnr, icode, ish, ishape', &
                                isurnr, icode, ish, ishape

            if ( ishape/=ish .and. ishape/=0 ) ish = 0
            nsurfsact = isurnr
         end if
         call msh049(iinput, iinp, icode)
      end do
      if ( nvolms==0 ) then
         if ( ish==0 ) call errsub ( 302, 0, 0, 0 )
         nsurel = 1
         iinput(iinp) = ish
         iinput(iinp+1) = 1
         iinput(iinp+2) = 1
         iinput(iinp+3) = nsurfsact
         go to 1000
      end if
      do ivolm = 1, nvolms
         icode = iinput(iinp)
         ishape = iinput(iinp+1)
         if ( ivolm==1) ish = ishape
         if ( ishape/=ish ) ish = 0
         call msh050(iinput, iinp, icode)
      end do
      if ( ish==0 ) call errsub ( 303, 0, 0, 0 )
      nvolel = 1
      iinput(iinp) = ish
      iinput(iinp+1) = 1
      iinput(iinp+2) = 1
      iinput(iinp+3) = nvolms
1000  call erclos ( 'msh037' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh037'

      end if  ! ( debug )

      end
