      subroutine msh049 ( iinput, iinp, isurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.19 date 30-11-2014 Change isurf 23 in 99
!        version  2.18 date 20-10-2008 Allow isurf = 24
!        version  2.17 date 01-07-2008 Extension with internal curves
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     raise iinp as function of isurf
!     is used to find information in array iinput
!    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!    If this subroutine is changed also subroutine MSH015 must be changed
!    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, isurf

!    iinp   i/o At input last position of preceding surface
!               At output last position of present surface in iinput
!    iinput  i  standard input array for subroutine mesh
!               it is supposed that the first five positions have been
!               deleted
!    isurf   i  type of surface to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, n

!     i      General loop variable
!     k      General loop variable
!     n      General sum variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate subroutine name from calling string
!     EROPEN   Concatenate name of subroutine to calling string
!     ERRINT   Place integer in integerarrary error message
!     ERRSUB   Submit error message
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     125   Wrong surface generator sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh049' )

      if ( isurf<0 .or. isurf>24 .and. isurf/=99 ) then

         call errint ( isurf, 1 )
         call errint ( 23, 2 )
         call errsub ( 125, 2, 0, 0 )

      end if

      if ( isurf==0 ) then
         iinp = iinp + 1
         go to 300
      end if

      n = mod(iinput(iinp+2),10000)
      iinp = iinp+3+n

      select case ( isurf )

         case ( 2, 3, 6, 7, 10, 24 )

!        --- isurf = 2, 3, 6, 7, 10: jump

         case ( 1, 4, 5 )

!        --- isurf = 1, 4, 5

           iinp = iinp+2

         case ( 8, 9, 14, 16, 19, 21 )

!        --- isurf = 8, 9, 14, 16, 19, 21

           iinp = iinp+1

         case ( 11, 12 )

!        --- isurf = 11 or 12

           k = 0
           do i = 1, n
              k = k + iinput(iinp+i-n-1)
           end do
           iinp = iinp + k

         case ( 18 )

!        --- isurf = 18

            iinp = iinp + 3

         case ( 13, 15, 17 )

!        --- isurf = 13, 15, 17

            iinp = iinp + 4

         case ( 20 )

!        --- isurf = 20

            iinp = iinp + 6

         case ( 22, 99 )

!        --- isurf = 22, 99: framesurf or triangle with internal points

            iinp = iinp + 2 + iinput(iinp) + iinput(iinp+1)

      end select  ! case (isurf)

300   call erclos ( 'msh049' )

      end
