      subroutine msh055 ( holes, kmeshsp2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 02-09-2010 Remove intarmsh
!        version  6.1  date 20-10-2008 Allow isurf = 24
!        version  6.0  date 02-02-2004 Complete new body
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Sort outer boundary of surfaces of surfaces in order to get a
!    complete continuous outer boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kmeshsp2(8,*)
      logical holes

!     holes         i/o   If true, a hole in the surface has been found
!     kmeshsp2      i/o   Second part of kmeshs, referring to surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurfc, iprec

!     iprec          Surface number of surface to be copied etc.
!     isurfc         Type of the surface to be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH055SURF12   Sort outer boundary of surfaces of surfaces in order to
!                    get a complete continuous outer boundary
!                    (isurf = 12, pipe surfaces)
!     MSH055SURF7    Sort outer boundary of surfaces of surfaces in order to
!                    get a complete continuous outer boundary
!                    (isurf = 7 or 11)
!     MSH055SURFC10  Sort outer boundary of surfaces of surfaces in order to
!                    get a complete continuous outer boundary
!                    (isurfc = 10,12, i.e. copied pipe surfaces)
!     MSH055SURFC7   Check if outer boundary of surface to be copied from
!                    surface of surfaces has the same structure as the surface
!                    to be copied
!     MSH055SURFS    Check if the outer boundary of surface is closed
!                    surfaces of surfaces have already been checked
!     PR0008         Print information of curves
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055' )
      debug = .false.
      if ( debug ) then

!     --- debug = true, perform debugging

         write(irefwr,*) 'Debug information in msh055'
         call pr0008 ( kmeshm, ncurvs )
  1      format ( a, 1x, (10i6) )

      end if

      do isurnr = 1, nsurfs
         isurf = kmeshsp2(1,isurnr)
         if ( isurf== 8 .or. isurf== 9 .or. &
              isurf==14 .or. isurf==16 ) then
            iprec  = kmeshsp2(4,isurnr)
            isurfc = kmeshsp2(1,iprec)
         else
            isurfc = isurf
            iprec = isurnr
         end if
         if ( isurf==0 ) go to 400

         if ( debug ) then

!        --- debug = true, perform debugging

            write(irefwr,1) 'isurnr, isurf, iprec', isurnr, isurf, iprec

         end if

         if ( isurf==7 .or. isurf==11 .or. isurf==24 ) then

!        --- surface of surfaces

            call msh055surf7 ( ncurvs, isurnr, kmeshr, kmeshs, &
                               kmeshsp2, iprec, isurf, holes, kmeshm )

         else if ( isurf==12 ) then

!        --- isurf = 12  surface of pipe surfaces, Check boundaries

            call  msh055surf12 ( kmeshsp2, isurnr, kmeshm, iprec, &
                                 kmeshs )

         else if ( isurfc==10 .or. isurfc==12 ) then

!        --- isurfc = 10  pipe surface, Fill part of kmeshs

            call msh055surfc10 ( kmeshsp2, isurnr, kmeshm, iprec, &
                                 kmeshs )

         else if ( isurfc==7 .or. isurfc==11 ) then

!        --- isurfc = 7, 11  original surface is surface of surface
!            check boundary

            call msh055surfc7 ( kmeshsp2, isurnr, kmeshm, iprec, &
                                kmeshs )

         else if ( isurfc<=5 .or. isurf==8 .or. isurf==9 .or. &
                   isurf==14 .or. isurf==16 .or. isurf==17 .or. &
                   isurfc==19) then

!        --- isurfc = 1, 2, 3, 4 or 5, i.e. surface is of type RECTANGLE, &
!            GENERAL, TRIANGLE, QUADRILATERAL or USER
!            Check if the outer boundary is closed

            call msh055surfs ( kmeshsp2, isurnr, kmeshm, kmeshs, &
                               isurfc )

         end if

400   end do

      if ( debug ) then

!     --- debug = true, perform debugging

         write(irefwr,*) 'Curves corresponding to surfaces'
         call pr0012 ( nsurfs, kmeshs, kmeshs )
         call prinin1 ( kmeshsp2(1,1), nsurfs, 8, &
                        'Rest of information per surface' )

      end if

      call erclos ( 'msh055' )

      end
