      subroutine msh055surfc10 ( kmeshsp2, isurnr, kmeshm, iprec, &
                                 kmeshs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Sort outer boundary of surfaces of surfaces in order to get a
!    complete continuous outer boundary (isurfc = 10,12, i.e. copied
!    pipe surfaces)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshsp2(8,*), isurnr, kmeshm(*), iprec, kmeshs(*)

!     iprec          i    Surface number of surface to be copied etc.
!     isurnr         i    Surface sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2      i/o   Second part of kmeshs, referring to surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishape, k, nh, j, numsurfs, kcur, firstpnt, lastpnt, &
              kfirst, klast, istart, istar3

!     firstpnt       First point of a curve
!     ishape         Shape number of elements
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kcur           curve sequence number
!     kfirst         First user point of a curve
!     klast          Last user point of a curve
!     lastpnt        last point of a curve
!     nh             number of nodes along a subcurve
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1224   The boundary of the pipe surface is not correct
!    1225   The boundary of the pipe surface is not correct
!    1226   The boundary of the pipe surface is not correct
!    1227   The boundary of the pipe surface is not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055surfc10' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh055surfc10'

      end if
      if ( ierror.ne.0 ) go to 1000

      istart = kmeshs(isurnr)
      istar3 = kmeshm(2)-1

      ishape = kmeshsp2(2,iprec)
      do k = 1, 3
         nh = 0
         numsurfs = kmeshsp2(k+2,iprec)
         do j = 1, numsurfs
            kcur = abs( kmeshs(istart+j) )
            if ( k.ne.2 ) nh = nh+kmeshm(kcur+5)-kmeshm(kcur+4)-1
         end do
         istart = istart + numsurfs
         if ( ishape.ne.3 .and. ishape.ne.5 ) then
            nh = nh/2
         end if
         if ( k.eq.1 ) then
            kmeshsp2(7,iprec) = nh
         else if ( k.eq.3 ) then
            kmeshsp2(8,iprec) = nh
         end if
      end do
      istart = kmeshs(isurnr)
      kcur = kmeshs(istart+1)

!     --- Find first user point of first curve (kcur)

      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         firstpnt = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

      else

!     --- kcur < 0, negative direction

         firstpnt = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

      end if
      kcur = kmeshs(istart+kmeshsp2(3,iprec))

!     --- Find last user point of first curve (kcur)

      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         lastpnt = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )

      else

!     --- kcur < 0, negative direction

         lastpnt = abs ( kmeshm(istar3+kmeshm(4-kcur)) )

      end if

!     --- Check if these user points coincide with the first user points
!         of the curves 3 and 4

      numsurfs = kmeshsp2(3,iprec)+kmeshsp2(4,iprec)
      kcur = kmeshs(istart+numsurfs+1)
      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         kfirst = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

      else

!     --- kcur < 0, negative direction

         kfirst = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

      end if
      if ( kfirst.ne.firstpnt ) then

!     --- First point of bottom curve different from first point of
!         first generating curve

         call errint ( isurnr, 1 )
         call errint ( kcur, 2 )
         call errint ( kfirst, 3 )
         call errint ( firstpnt, 4 )
         call errwar ( 1224, 4, 0, 0 )
      end if
      if ( kmeshsp2(4,iprec).gt.0 ) &
         numsurfs = numsurfs + kmeshsp2(5,iprec)
      kcur = kmeshs(istart+numsurfs+1)
      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         kfirst = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

      else

!     --- kcur < 0, negative direction

         kfirst = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

      end if
      if ( kfirst.ne.lastpnt ) then

!     --- Last point of bottom curve different from first point of
!         second generating curve

         call errint ( isurnr, 1 )
         call errint ( kcur, 2 )
         call errint ( kfirst, 3 )
         call errint ( lastpnt, 4 )
         call errwar ( 1225, 4, 0, 0 )
      end if
      kcur = kmeshs(istart+kmeshsp2(3,iprec)+1)

!     --- Find first user point of second curve (kcur)

      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         firstpnt = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

      else

!     --- kcur < 0, negative direction

         firstpnt = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

      end if
      kcur = kmeshs(istart+kmeshsp2(3,iprec)+kmeshsp2(4,iprec))

!     --- Find last user point of second curve (kcur)

      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         lastpnt = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )

      else

!     --- kcur < 0, negative direction

         lastpnt = abs ( kmeshm(istar3+kmeshm(4-kcur)) )

      end if

!     --- Check if these user points coincide with the last user points
!         of the curves 3 and 4

      numsurfs = &
         kmeshsp2(3,iprec)+kmeshsp2(4,iprec)+kmeshsp2(5,iprec)
      kcur = kmeshs(istart+numsurfs)
      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         klast = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )
      else

!     --- kcur < 0, negative direction

         klast = abs ( kmeshm(istar3+kmeshm(4-kcur)) )
      end if
      if ( klast.ne.firstpnt ) then

!     --- First point of top curve different from last point of
!         first generating curve

         call errint ( isurnr, 1 )
         call errint ( kcur, 2 )
         call errint ( klast, 3 )
         call errint ( firstpnt, 4 )
         call errwar ( 1226, 4, 0, 0 )

      end if
      kcur = kmeshs(istart+numsurfs+kmeshsp2(6,iprec))
      if ( kcur.gt.0 ) then

!     --- kcur > 0, positive direction

         klast = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )

      else

!     --- kcur < 0, negative direction

         klast = abs ( kmeshm(istar3+kmeshm(4-kcur)) )

      end if
      if ( klast.ne.lastpnt ) then

!     --- Last point of top curve different from last point of
!         second generating curve

         call errint ( isurnr, 1 )
         call errint ( kcur, 2 )
         call errint ( klast, 3 )
         call errint ( lastpnt, 4 )
         call errwar ( 1227, 4, 0, 0 )
      end if

1000  call erclos ( 'msh055surfc10' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh055surfc10'

      end if

      end
