      subroutine msh063gen ( iinput, iinp, ndefsf, &
                             kmeshscopy, lstart, maxncursur, &
                             istart, isurfc, rpin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 08-10-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmeshs part curves for single surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, ndefsf, &
              kmeshscopy(*), lstart, maxncursur, &
              istart, isurfc
      double precision rpin(4,*)

!     iinp           i    last position used in array iinput
!     iinput         i    integer input array containing information of the
!                         surfaces
!     istart         i    Position in kmeshscopy
!     isurfc         i    Type of the surface to be copied
!     kmeshscopy     i    Work array of the same length as kmeshs part 2
!                         containing a copy of kmeshs part 2 in the sequence
!                         of iseqsurfs
!     lstart         i    Position in kmeshs
!     maxncursur     i    Estimate of the length of kmeshscopy
!     ndefsf         i    Number of curves in surface
!     rpin           i    Array containing the thicknesses and factors
!                         of certain curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, jcurve, jstart, kcurve, j, scurv, tcurv, n

!     j              Counting variable
!     jcurve         Curve number
!     jstart         starting position of curve i in kmeshm
!     k              Counting variable
!     kcurve         abs(jcurve)
!     n
!     scurv          temporary curve number
!     tcurv          |scurv|
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh063gen' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh063gen'
         write(irefwr,1) 'istart, lstart, isurfc', istart, lstart, isurfc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isurfc==10 ) lstart = lstart + 2
      n = 0

      do k = 1, ndefsf

         jcurve = iinput(iinp+2+k)
         kcurve = abs(jcurve)
         if ( kelmq/=0 ) then
            jstart = kmeshq(kcurve)
            n = kmeshq(kcurve+1) - jstart
         end if
         if ( debug ) write(irefwr,1) 'n, istart', n, istart

         if ( n==0 ) then

!        --- n = 0  only one curve

            istart = istart + 1
            if ( istart>maxncursur ) then

!           --- Estimation of length of kmeshscopy too small

               call errint ( istart+n, 1 )
               call errint ( maxncursur, 2 )
               call errchr ( 'kmeshscopy', 1 )
               call errsub ( 550, 2 , 0, 1 )
               go to 1000

            end if
            kmeshscopy(istart) = jcurve

         else

!        --- n>0  curve consists of subcurves

            if ( istart+n>maxncursur ) then

!           --- Estimation of length of kmeshscopy too small

               call errint ( istart+n, 1 )
               call errint ( maxncursur, 2 )
               call errchr ( 'kmeshscopy', 1 )
               call errsub ( 550, 2 , 0, 1 )
               go to 1000

            end if
            if ( jcurve>0 ) then

!           --- jcurve>0  copy forwards

               do j = 1, n
                  kmeshscopy(istart+j) = kmeshq(jstart+j)
                  if ( irnp4>0 ) then
                     scurv = kmeshq(jstart+j)
                     tcurv = abs(scurv)
                     if ( abs(rpin(2,tcurv))<=0d0 )then
                        rpin(2,tcurv) = 1d0
                     end if
                     if ( abs(rpin(4,tcurv))<=0d0 )then
                        rpin(4,tcurv) = 1d0
                     end if
                     if ( scurv>=0 .and. &
                          abs(rpin(4,kcurve))>0d0 ) then

!                    --- positive curve

                        rpin(3,tcurv) = rpin(3,kcurve)
                        rpin(4,tcurv) = rpin(4,kcurve)

                     else if ( abs(rpin(4,kcurve))>0d0 ) then

!                    --- negative curve

                        rpin(1,tcurv) = rpin(3,kcurve)
                        rpin(2,tcurv) = rpin(4,kcurve)

                     end if

                  end if

               end do

            else

!           --- jcurve<0  copy backwards

               do j = 1, n
                  kmeshscopy(istart+j) = - kmeshq(jstart+n-j+1)
                  if ( irnp4>0 ) then
                     scurv = kmeshscopy(istart+j)
                     tcurv = abs(scurv)
                     if ( abs(rpin(2,tcurv))<=0d0 ) then
                        rpin(2,tcurv) = 1d0
                     end if
                     if ( abs(rpin(4,tcurv))<=0d0 ) then
                        rpin(4,tcurv) = 1d0
                     end if
                     if ( scurv>=0 .and. &
                          abs(rpin(2,kcurve))>0d0 ) then

!                    --- positive curve

                        rpin(3,tcurv) = rpin(1,kcurve)
                        rpin(4,tcurv) = rpin(2,kcurve)

                     else if ( abs(rpin(2,kcurve))>0d0)  then

!                    --- negative curve

                        rpin(1,tcurv) = rpin(1,kcurve)
                        rpin(2,tcurv) = rpin(2,kcurve)
                     end if

                  end if
               end do

            end if
            istart = istart + n

         end if
         if ( isurf==10 .or. isurfc==10 ) then

!        --- isurf = 10  pipe surface

            kmeshs(lstart+2+k) = max(1,n)
            if ( ndefsf==3 .and. k==3 ) then

!           --- surface consists of 3 curves, copy third curve
!               into fourth one

               if ( istart+max(n,1)>maxncursur ) then

!              --- Estimation of length of kmeshscopy too small

                  call errint ( istart+max(n,1), 1 )
                  call errint ( maxncursur, 2 )
                  call errchr ( 'kmeshscopy', 1 )
                  call errsub ( 550, 2 , 0, 1 )
                  go to 1000

               end if
               if ( n==0 ) then
                  kmeshscopy(istart+1) = jcurve
                  istart = istart + 1
               else
                  do j = 1, n
                     kmeshscopy(istart+j) = kmeshscopy(istart+j-n)
                  end do
                  istart = istart + n
              end if
              kmeshs(lstart+6) = kmeshs(lstart+5)
           end if
        end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh063gen'

      end if  ! ( debug )
      call erclos ( 'msh063gen' )

      end
