      subroutine msh064 ( iinput, iposi, maxholes, irefplax )
! ======================================================================
!
!        programmer    Guus Segal
!        version 11.1  date 30-11-2014 Change isurf 23 in 99
!        version 11.0  date 20-06-2013 Extra parameter irefplax
!        version 10.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for mshsrf
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi, maxholes, irefplax

!     iinput         i    integer input array
!     iposi         i/o   next free position of iinput
!     irefplax       o    File reference number for plaxis
!     maxholes       i    Maximum number of holes allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgeg(8,4), nbc, ndefsf, iinp, meshtp, nrow, j, isurfc, &
              i, jcurv, k, iprec, leng, iposk, npoinl, neleml, &
              npelmred, numcurvs, istart
      logical trian

!     i              General loop variable
!     ielgeg         Array containing the values of nboune, mtrian, inpelm and
!                    nboun for the various element shapes
!     iinp           Position in array IINPUT
!     iposk          Last position in array IINPUT before the information of the
!                    present surface
!     iprec          Surface number of surface to be copied etc.
!     istart         First position used at a part
!     isurfc         Type of the surface to be copied
!     j              General loop variable
!     jcurv          Absolute value of curve number
!     k              General loop variable
!     leng           Help variable to compute the length of some arrays
!     meshtp         Defines the type of elements to be created
!     nbc            Number of points along a side
!     ndefsf         Number of curves in surface
!     neleml         Estimated number of elements by user (parsurf)
!     npelmred       Reduced number of points per element
!     npoinl         Estimated number of points by user (parsurf)
!     nrow           Number of rows of surfaces in ordened surface of surfaces
!     numcurvs       Number of curves
!     trian          Indication whether triangles (true) or quadrilaterals
!                    are created (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHMAPSURIN    extract mapping information form iinput
!     MSHNUMNODCRV   Compute number of nodes along a curve
!     MSHPLAXREAD    Read number of nodes and triangles that defines the frame
!                    for the submesh generator framesurf
!     MSHZ05         Estimates upper bounds of user arrays and scratch arrays, &
!                    can be called by user, it is not called from the pack
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      35   Incorrect number of nodal points along the boundary
!     129   Incorrect curve number found
!     140   Incorrect shape number found
!     379   Incorrect type of surface generator found
!     526   The curves c1 and c2 of PIPE SURFACE do not have the same
!           number of nodal points.
!     632   The fourth generating curve of a PIPE SURFACE does not have
!           the same number of nodes as the third generating curve.
!     912   Mesh generator COONS3D or QUADRILATERAL requires exactly 4 curves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ielgeg / 1, 2, 1, 2, 2,  3, 1, 1, &
                    2, 2, 1, 1, 2,  2, 1, 2, &
                    3, 6, 4, 9, 7, 10, 5, 4, &
                    3, 6, 4, 8, 6,  9, 4, 3 /
! ======================================================================
!
      call eropen ( 'msh064' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh064'
         write(irefwr,1) 'iposi', iposi
         call prinin ( iinput(iposi), 20, 'iinput_iinp' )
  1      format ( a, 1x, (10i6) )

      end if

      nmes = 0
      mmes = 0
      nbndpt = 0
      nelemlc  = 0
      npointlc = 0
      lnrenu = 0
      lnwrk2 = 0
      leniwk = 0
      mapping3d = 0
      icentre = 0
      nlayers = 0
      nextrapnts = 0   ! number of extra points
      nextracrvs = 0   ! number of extra curves
      lenextracrvs = 0 ! extra number of points, due to extra curves
      nz = 0
      icirc = 0
      maxloops = 0
      nnodes = 0
      ntria = 0

      iposk = iposi-1

      isurf = iinput(iposk+1)
      if ( debug ) write(irefwr,1) 'isurnr, isurf', isurnr, isurf

      if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
           isurf==16 .or. isurf==21 ) then
         iprec = kmeshs(nsurfs+1+(isurnr-1)*8+4)
         isurfc = kmeshs(nsurfs+1+(iprec-1)*8+1)
         if ( debug ) write(irefwr,1) 'iprec, isurfc', iprec, isurfc
      else
         isurfc = isurf
      end if

      if ( isurf==24 ) isurf = 7  ! surface of surfaces

      if ( isurf==0 ) then

!     --- empty surface found

         iposi  = iposi + 1
         go to 900

      else if ( isurf==20 ) then

!     --- sphere, must be skipped

         iposi  = iposi + 10
         go to 900

      else if ( isurf<0 .or. isurf==6 .or. isurf>24 .and. isurf/=99 ) then

!     --- Error 379:  Type of surface incorrect

         call errint ( isurf, 1 )
         call errint ( isurnr, 2 )
         call errsub ( 379, 2, 0, 0)

      end if

      ishape = iinput(iposk+2)
      if ( debug ) write(irefwr,1) 'ishape', ishape
      if ( ishape<=2 .or. ishape>10 ) then
         call errint ( ishape, 1 )
         call errint ( isurnr, 2 )
         call errsub ( 140, 2, 0, 0)
      end if
      ndefsf = mod(iinput(iposk+3),10000)
      idiag = iinput(iposk+3)/10000
      iinp = 3+ndefsf
      meshtp = ishape-2

!     --- compute nboune, mtrian

      nboune = ielgeg(meshtp,1)
      mtrian = ielgeg(meshtp,2)
      inpelm = ielgeg(meshtp,3)
      nboun  = ielgeg(meshtp,4)

      if ( isurnr==1 ) nbnmax = 0

      if ( isurf==7 ) then

!     --- isurf = 7, 24   surfaces of surfaces

         iposi = iposi + iinp
         go to 900

      else if ( isurf==11 .or. isurf==12 ) then

!     --- isurf = 11  surfaces of surfaces

         nrow = ndefsf
         ndefsf = 0
         do j = 1, nrow
            ndefsf = ndefsf + iinput(iposk+3+j)
         end do
         iposi = iposi + 3 + nrow + ndefsf
         go to 900

      end if

!     --- Compute nbndpt

      istart = kmeshs(isurnr)
      numcurvs = kmeshs(isurnr+1) - istart
      if ( debug ) write(irefwr,*) 'isurnr, numcurvs ', isurnr, numcurvs

      nbndpt = 1
      do i = 1, numcurvs
         icurnr = kmeshs(istart+i)
         jcurv = abs(icurnr)
         if ( debug ) write(irefwr,*) 'i, icurnr ', i, icurnr
         if ( jcurv==0 .or. jcurv>ncurvs ) then
            call errint ( icurnr, 1 )
            call errint ( ncurvs, 2 )
            call errint ( isurnr, 3 )
            call errsub ( 129, 3, 0, 0)
         end if
         if ( jcurv==1 ) nbc = icurvsmsh(1)
         if ( jcurv>1 ) nbc = icurvsmsh(jcurv)-icurvsmsh(jcurv-1)
         nbndpt = nbndpt+nbc-1
         if ( debug ) write(irefwr,*) ' i, icurnr, nbndpt ', &
                                        i, icurnr, nbndpt
      end do
      if ( debug ) write(irefwr,*) 'nbndpt, ndefsf ',nbndpt, ndefsf

      if ( ierror/=0 ) go to 1000
      nbnmax = max ( nbnmax, nbndpt )
      leng = nboun*2

!     --- compute number of boundary points

      select case ( isurf )

         case ( 6, 7, 11, 12 )

!        --- isurf = 6,7,11,12,24 skip

         case ( 1, 17 )

!        --- isurf = 1, 17     subroutine mshre1

            nmes = iinput(iposk+iinp+1)
            mmes = iinput(iposk+iinp+2)
            if ( nmes==0 .and. mmes==0 ) then

!           --- n and m must be computed from the first and second curve

               jcurv = abs(iinput(iposk+4))
               call mshnumnodcrv ( jcurv, nbc )
               nmes = (nbc-1)/nboune

               jcurv = abs(iinput(iposk+5))
               call mshnumnodcrv ( jcurv, nbc )
               mmes = (nbc-1)/nboune

               iinput(iposk+iinp+1) = nmes
               iinput(iposk+iinp+2) = mmes
               kmeshs(nsurfs+1+8*(isurnr-1)+3) = nmes
               kmeshs(nsurfs+1+8*(isurnr-1)+4) = mmes

            end if
            iposi = iposi+iinp+2
            npointlc = (nmes*nboune+1)*(mmes*nboune+1)
            nelemlc = nmes*mmes*mtrian
            nbcrd = (nmes+mmes)*2*nboune+1
            if ( inpelm==7 .or. inpelm==5 .or. ishape==10 ) &
               npointlc = npointlc+nelemlc
            if ( nbcrd/=nbndpt ) then
               call errint ( isurnr, 1 )
               call errint ( nbndpt-1, 2 )
               call errint ( nbcrd-1, 3 )
               call errchr ( 'rectangle', 1 )
               call errsub ( 35, 3, 0, 1 )
            end if
            if ( isurf==1 ) then
               ismoot = 0
            else
               ismoot = iinput(iposk+iinp+3)
               iposi = iposi + 2
            end if

         case ( 2, 3, 18, 19, 22, 99 )

!        --- isurf = 2, 3 or 18, 19, 22, 99
!            subroutine mshg, mshoce, mshisp, mshplaxis

            iposi = iposi+iinp
            k = (nbndpt-1)/4
            if ( 4*nbndpt/=k) k = k+1
            nelemlc = k*(k+1)*3/2*mtrian+10000
            if ( isurf==3 ) nelemlc = nelemlc*2
            if ( inpelm>4 .and. ishape<=8 ) nelemlc = nelemlc/4
            nelemlc = nelemlc+1000
            k = 4*k+4
            npointlc = k*k/10+10000
            if ( isurf==3 .or. isurf==22 ) npointlc = npointlc*2
            if ( inpelm==7 .or. inpelm==9 .or. inpelm==10 .or. &
                 ishape==10 ) npointlc = npointlc+nelemlc
            lnrenu = npointlc
            if ( isurf==22 ) then

!           --- isurf = 22, framesurf
!               estimate extra space
!               First open triangle file and read number of nodes and triangles

               call mshplaxread ( isurnr, nnodes, ntria, irefplax )
               lnwrk2 = 3*nnodes
               leniwk = 3*ntria+3*npointlc
               nbndpt = nbndpt + maxholes

            else if ( isurf==3 .or. isurf==19 .or. isurf==18 .or. &
                      isurf==99 ) then

!           --- isurf = 3, 99 or 18, 19;  mshoce or mshisp or paver
!               extra space for boundary points
!               (see also end of msh011)

               if ( isurf==3 .or. isurf==19 .or. isurf==99 ) then
                  nbndpt = nbndpt + maxholes
               end if
               lnwrk2 = max(lnwrk2,2 * npointlc)
               if ( isurf==19 ) then
                  maxloops = max(nbndpt / 3, 40)
               end if
               if ( isurf==18 ) then

!              --- mshisp, extract information about mapping

                  call mshmapsurin ( iinput, iinp, iposk )
                  iposi = iposi+2

!                 --- Fill the number of nodes along the three "boundaries"

                  jcurv = abs(iinput(iposk+4))
                  call mshnumnodcrv ( jcurv, nmes )
                  jcurv = abs(iinput(iposk+5))
                  call mshnumnodcrv ( jcurv, mmes )
                  jcurv = abs(iinput(iposk+6))
                  call mshnumnodcrv ( jcurv, nz )

               else if ( isurf==19 ) then

!              --- extract nlayers

                  nlayers = iinput(iposk+iinp+1)
                  iposi = iposi+1

               end if

            else

!           --- isurf = 2 --- general

               leniwk = max(16*npointlc+20, leng)
               lnwrk2 = max(5*npointlc, lnwrk2)

            end if

            if ( isurf==22 .or. isurf==99 ) then

!           --- isurf = 22, 99, framesurf or triangle
!                                with extra points and curves

               nextrapnts = iinput(iposk+iinp+1)
               nextracrvs = iinput(iposk+iinp+2)
               ncoarstart = iposk+iinp+3  ! starting address of extra
                                             ! points in iinput
               ncurvstart = ncoarstart+nextrapnts  ! starting address
                                             ! of extra curves in iinput
               if ( debug ) then
                  write(irefwr,1) 'nextrapnts, nextracrvs', &
                                   nextrapnts, nextracrvs
                  if ( nextrapnts>0 ) &
                     call prinin ( iinput(ncoarstart), nextrapnts, &
                                   'extra points' )
                  if ( nextracrvs>0 ) &
                     call prinin ( iinput(ncurvstart), nextracrvs, &
                                   'extra curves' )
               end if  ! ( debug )

               iposi = iposi+2+nextrapnts+nextracrvs

            end if  ! ( isurf==22 .or. isurf==99 )

         case ( 4, 13, 15 )

!        --- isurf = 4, 13 or 15 subroutine meshr1 or mshzde
!            Surface has exactly 4 sides

            if ( isurf==15 ) then

!           --- Estimates for npointlc and nelemlc may have been given by user
!               (not yet interesting, for later use already implemented)

               npoinl = iinput(iposk+iinp+1)
               neleml = iinput(iposk+iinp+2)
               iinp   = iinp + 2

            end if

            iblend = iinput(iposk+iinp+1)
            icurvt = iinput(iposk+iinp+2)
            if ( isurf==13 ) then

!           --- isurf = 13, coons3d, detect mapping

               call mshmapsurin ( iinput, iinp+2, iposk )
               iposi = iposi+2

            end if
            iposi = iposi+iinp+2
            if ( ndefsf/=4 ) then
               call errint ( ndefsf, 1 )
               call errsub ( 912, 1, 0, 0)
            end if

!           --- Compute the parameters n1, n2, n3 and n4 as defined by Zdenek
!               Vlasek and store in nx, ny, nz and nc resp.
!               First generating curve (n1)

            icurnr = abs(iinput(iposk+4))
            call mshnumnodcrv ( icurnr, nbc )

            nx = ( nbc-1 ) /nboune

!           --- Second generating curve (n2)

            icurnr = abs(iinput(iposk+5))
            call mshnumnodcrv ( icurnr, nbc )

            ny = ( nbc-1 ) /nboune

!           --- Third generating curve (n3)

            icurnr = abs(iinput(iposk+6))
            call mshnumnodcrv ( icurnr, nbc )

            nz = ( nbc-1 ) /nboune

!           --- Fourth generating curve (n4)

            icurnr = abs(iinput(iposk+7))
            call mshnumnodcrv ( icurnr, nbc )

            nc = ( nbc-1 ) /nboune

!           --- Estimate npointlc, nelemlc and length work arrays

            trian = mtrian==2
            npelmred = inpelm
            if ( ishape>8 ) npelmred = npelmred-1
            call mshz05 ( trian, nx,ny,nz,nc, inpelm, nboune+1, iblend, &
                          nelemlc, npointlc )
            if ( ishape>8 ) npointlc = npointlc+nelemlc
            if ( isurf/=4 .and. nx==nz .and. ny==nc ) &
               lenwrk = lenwrk+nbndpt+29+16

            if ( isurf==15 ) then

!           --- Compare with estimates given by user

               if ( npoinl>npointlc ) npointlc = npoinl
               if ( neleml>nelemlc  ) nelemlc  = neleml

            end if

            if ( isurf==4 ) then

!           --- isurf = 4, MSHZDE

               lnwrk2 = max(lnwrk2,npointlc*3)

            else

!           --- isurf = 13 or 15, COONS or PARSURF

               lnwrk2 = max(lnwrk2,max(lenwrk, nx, ny, nz, nc ))

            end if

         case ( 5 )

!        --- isurf = 5   meshus

            nelemlc = iinput(iposk+iinp+1)
            npointlc = iinput(iposk+iinp+2)
            iposi = iposi+iinp+2

         case ( 8, 9, 14, 16, 21 )

!        --- isurf = 8, 9, 14, 16 21  translate, rotate, similar, reflect, copy

            iprec = iinput(iposk+iinp+1)
            npointlc = abs(kmeshh(1,iprec+1))
            nelemlc = kmeshh(2,iprec+1)
            leniwk = nelemlc*inpelm
            iposi = iposi+iinp+1
            lnrenu = npomxs + nelmax
            if ( isurfc==7 .or. isurfc==24 ) then

!           --- Previous surface is of type surface of surfaces
!               add number of holes in outer boundary, stored in kmesh part s2

               nbndpt = nbndpt + kmeshs(nsurfs+1+(iprec-1)*8+6)

            else if ( isurfc==10 ) then

!           --- Previous surface is of type surface of pipe surfaces

               nbndpt = nbndpt + 3

            else if ( isurfc==3 .or. isurfc==19 .or. isurfc==22 .or. &
                      isurfc==99 ) then

!           --- Previous surface is of type triangle or paver

               nbndpt = nbndpt + maxholes

            end if

!           --- isurf = 10     pipe surface  |  c1, c2   closed curves of size m
!                                            |  c3    generating curve of size n
!                                            |  c4      (optional) second
!                                            |        generating curve of size n

         case ( 10 )

            icurnr = abs( iinput(iposk+4) )
            call mshnumnodcrv ( icurnr, nbc )
            mmes = nbc
            if ( debug ) write(irefwr,*) 'icurnr, nbc ', icurnr, nbc
            icurnr = abs( iinput(iposk+5) )
            call mshnumnodcrv ( icurnr, nbc )
            if ( debug ) write(irefwr,*) 'icurnr, nbc ', icurnr, nbc

            if ( nbc/=mmes ) then
               call errint ( isurnr, 1 )
               call errint ( icurnr, 2 )
               call errint ( nbc  , 3 )
               call errint ( mmes , 4 )
               call errsub ( 526, 4, 0, 0 )
            end if
            icurnr = abs( iinput(iposk+6) )
            call mshnumnodcrv ( icurnr, nbc )
            if ( debug ) write(irefwr,*) 'icurnr, nbc ', icurnr, nbc

            nmes = nbc

            if ( ndefsf==4 ) then

!           --- ndefsf = 4, curve c4 has been defined, i.e. second generating
!               curve

               icurnr = abs( iinput(iposk+7) )
               call mshnumnodcrv ( icurnr, nbc )
               if ( debug ) write(irefwr,*) 'icurnr, nbc ', icurnr, nbc

               if ( nbc/=nmes ) then
                  call errint ( isurnr, 1 )
                  call errint ( icurnr, 2 )
                  call errint ( nbc  , 3 )
                  call errint ( nmes , 4 )
                  call errsub ( 632, 4, 0, 0 )
               end if
            end if
            nbndpt = nbndpt+3
            npointlc = nmes*mmes
            icirc = 0
            nmes = (nmes-1)/nboune
            mmes = (mmes-1)/nboune
            nelemlc = nmes*mmes*mtrian
            if ( inpelm==7 .or. inpelm==9 .or. ishape>8 ) &
                 npointlc = npointlc+nelemlc
            iposi = iposi+iinp

            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'isurnr, n, m, npointlc, nbndpt', &
                                isurnr, nmes, mmes, npointlc, nbndpt

            end if

      end select  ! case ( isurf )

900   lnkbnd = nbndpt

      if ( isurf==18 ) lnkbnd = nbndpt+5
      if ( iinp13/=0 ) leniwk = leniwk+2*npointlc

      if ( nextracrvs>0 ) then

!     --- nextracurvs>0, hence there are extra curves
!         Compute the number of nodes in these curves and store
!         in lenextracrvs

         istart = ncurvstart
         do i = 0, nextracrvs-1

            icurnr = abs(iinput(istart+i))
            if ( debug ) write(irefwr,1) 'i, icurnr', i, icurnr
            if ( icurnr==1 ) then
               nbc = icurvsmsh(1)
            else
               nbc = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)
            end if  ! ( icurnr==1 )
            lenextracrvs = lenextracrvs+nbc

         end do  ! i = 1, nextracrvs

      end if  ! ( nextracrvs>0 )

!     --- Compute length of array bcord and increase length of kbndpt

      lnbcor = nbndpt+lenextracrvs
      ibcrds = ndim
      if ( isurf==19 ) ibcrds = ibcrds+2
      lnkbnd = lnkbnd+lenextracrvs+nextrapnts

      if ( ndim==3) lnwrk2 = max(lnwrk2, 3*nbndpt, npointlc*ndim)
      nbnmax = max(nbnmax, nbndpt+lenextracrvs+nextrapnts)+1

!     --- Check, and eventually adjust, startpositions

      if ( mod(lnwrk2,2)==1 ) lnwrk2 = lnwrk2 + 1

      lmes = nz

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'isurnr, nbndpt ', isurnr, nbndpt
         write(irefwr,*) 'end msh064'

      end if
      call erclos ( 'msh064' )

      end subroutine msh064
