      subroutine msh067 ( iwork, npointlc, iellist, iwork1 )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date  23-08-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill array iellist with the element numbers corresponding to the
!   nodes stored in iwork
! **********************************************************************
!
!                       KEYWORDS
!
!    element
!    fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointlc, iwork(*), iellist(*), &
              iwork1(*)

!     iellist    o    Array of length npointlc-1 containing a
!                     list of elements that are adjacent to the boundary
!                     element iellist(i) contains both nodal points
!                     iwork(i) and iwork(i+1)
!     iwork      i    Contains the nodal point numbers along the boundary
!     iwork1          integer work array of length nelem
!     npointlc     i    Number of nodal points along boundary
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, inodp, nneigh, j
      logical found

!     found      If true already an element has been found
!     i          Counting variable
!     ielem      Element number
!     inodp      Nodal point number
!     j          Counting variable
!     nneigh     Number of neighbours of node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1642   Two nodal points belong to more than one element
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For i := 1 (1) npointlc-1 do
!       Compute intersection of elements corresponding to
!       iwork(i) and iwork(i+1)
!       If ( intersection contains more than one element ) then
!          error
!       Store element number in iellist(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, nelem
         iwork1(i) = 0
      end do
      do i = 1, npointlc-1

         inodp = iwork(i)
         nneigh = kmshd1(inodp+1)-kmshd1(inodp)
         do j = 1, nneigh
            iwork1(kmshd2(kmshd1(inodp)+j)) = 1
         end do
         found = .false.
         inodp = iwork(i+1)
         nneigh = kmshd1(inodp+1)-kmshd1(inodp)
         do j = 1, nneigh
            ielem = kmshd2(kmshd1(inodp)+j)
            if ( iwork1(ielem)==1 ) then
               if ( found ) then
                  call eropen ( 'msh067' )
                  call errint ( iwork(i), 1 )
                  call errint ( iwork(i+1), 2 )
                  call errsub ( 1642, 2, 0, 0 )
                  call erclos ( 'msh067' )
               else
                  iellist(i) = ielem
                  found = .true.
               end if
            end if
         end do
         inodp = iwork(i)
         nneigh = kmshd1(inodp+1)-kmshd1(inodp)
         do j = 1, nneigh
            iwork1(kmshd2(kmshd1(inodp)+j)) = 0
         end do

      end do

      end
