      subroutine msh072 ( coorol, coornw, inpada, rinada, n, icurvs, &
                          curves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2009 Replace body by subroutines
!        version  1.1  date 22-09-1997 Call to eropen/erclos
!        version  1.0  date 23-08-1996
!
!   copyright (c) 1996-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct end points of free boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), n, icurvs(*)
      double precision coorol(2,*), coornw(2,*), rinada(*), curves(2,*)

!     coornw    i/o   Array containing the new co-ordinates along the boundary
!     coorol     i    Array containing the old co-ordinates along the boundary
!     curves     i    array containing the coordinates of the curves
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     inpada     i    User input array for subroutine ADAPBOUN
!     n          i    number of points along the boundary
!     rinada     i    User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iangl1, iangl2, icurnr

!     iangl1     Indicates how the starting point must be adapted, see INPADA
!     iangl2     Indicates how the end point must be adapted, see INPADA
!     icurnr     Curve sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH073         Correct end points of free boundary
!     MSHCORREND     Correct end points of free boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract parameters from inpada and rinada
!    Adapt end points as function of iexcl1/2
!    Adapt end points if angles have been described
!    if ( iangl1/2<0 ) then
!       project end point onto prescribed curve
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh072' )

!     --- Extract parameters from inpada and rinada

      iangl1 = inpada(13)
      iangl2 = inpada(14)

!     --- Adapt end points as function of iexcl1/2

      call mshcorrend ( coorol, coornw, inpada, rinada, n )

!     --- Project end points onto prescribed curves if required

      if ( iangl1<0 ) then

!     --- iangle1<0, point must ly along curve |iangl1|

         icurnr = abs(iangl1)
         call msh073 ( coorol, coornw, icurnr, 1, icurvs, &
                       curves )

      end if
      if ( iangl2<0 ) then

!     --- iangle2<0, point must ly along curve |iangl2|

         icurnr = abs(iangl2)
         call msh073 ( coorol, coornw, icurnr, n, icurvs, &
                       curves )

      end if

      call erclos ( 'msh072' )

      end
