      subroutine msh075 ( iinput, lengc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 02-09-2010 Remove intarmsh
!        version  3.0  date 13-03-2006 Extra parameters
!        version  2.0  date 14-10-2002 Extra parameter kmeshq
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine checks the input for interface elements. It computes the
!     required length of these elements with respect to KMESH part c
!     Furthermore inodp is raised if necessary
!     msh075 is closely related to MSH076, if one of them is changed, the other
!     must be changed also
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengc

!     iinput         i    integer input array containing information of the
!                         interface elements
!     lengc         i/o   computed length of kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurvs, iinp, isurfs

!     icurvs         Number of parts where curve interface elements are given
!     iinp           last position used in array iinput
!     isurfs         Number of parts where surface interface elements
!                    are given
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH075CURVS    checks the input for interface elements.
!                    computes the required length of these elements with
!                    respect to KMESH part c (curves)
!     MSH075SURFS    checks the input for interface elements.
!                    computes the required length of these elements with
!                    respect to KMESH part c (surfaces)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     321   The number of curves connected is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh075' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh075'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      icurvs = iinput(2)
      isurfs = iinput(3)
      if ( debug ) &
         write(irefwr,1) 'icurvs, isurfs, inodp, iselem', &
                          icurvs, isurfs, inodp, iselem
      if ( icurvs<0 .or. icurvs>ncurvs ) then
         call errint(icurvs,1)
         call errint(ncurvs,2)
         call errsub ( 321, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      iinp = 4

      if ( icurvs>0 ) then

!     --- icurvs>0, there are curve interface elements

         call msh075curvs ( iinput, iinp, icurvs, inodp, lengc, iselem )

      end if  ! ( icurvs>0 )

      if ( isurfs>0 ) then

!     --- isurfs>0, there are surface interface elements

         call msh075surfs ( isurfs, iinput, iinp, inodp, lengc, iselem )

      end if  ! ( icurvs>0 )

      npoint = inodp-1
      nelem = iselem-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inodp, iselem', inodp, iselem
         write(irefwr,*) 'End msh075'

      end if
      call erclos ( 'msh075' )

      end
