      subroutine msh090 ( curves, ih, ndim  ,xend  , yend,  zend, ja )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.1    date   21-11-1993 Make curves two-dimensional
!        version 1.0    date   26-02-1991
!
!
!
!   copyright (c) 1991-1993  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Subroutine to check whether the points with coordinates
!     (xend,yend,zend) and (curves (ih+1,2,3)) are different (ja=1)
!     or not (ja=0)
!
! ***********************************************************************
!
!                      KEYWORDS
!
!    curve
!    mesh
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer ih, ndim, ja
      double precision curves(ndim,*), xend, yend, zend

!     curves i  array with at least coordinates of two points
!     ih     i  startposition of coordinates point considered in curves
!     ja    i,o at input parameter to indicate whether the curve con- &
!               sidered starts with pointposition ih (ja=1) or end with
!               point position ih (ja=-1)
!               at output parameter to indicate whether the points are
!               different (ja=1) or not (ja=0)
!     ndim   i  dimension
!     xend
!     yend   i  point to check with
!     zend
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      double precision dif, dx, dy, dz, ref

!     dif      difference variable
!     dx       difference in x-coordinates
!     dy       difference in y-coordinates
!     dz       difference in z-coordinates
!     ref      reference distance
! ***********************************************************************
!
!     I / O
!
!     None
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     trivial
! ======================================================================
!
!     --- Check for new point ( make reference value first)

      if ( ja.eq.1 ) then

!     --- second point is forward

         dx = curves(1,ih) - curves(1,ih+1)
         dy = curves(2,ih) - curves(2,ih+1)
         dz = 0
         if ( ndim.eq.3 ) dz = curves(3,ih) - curves(3,ih+1)

      else if ( ja.eq.-1 ) then

!     --- second point is backward

         dx = curves(1,ih) - curves(1,ih-1)
         dy = curves(2,ih) - curves(2,ih-1)
         dz = 0
         if ( ndim.eq.3 ) dz = curves(3,ih) - curves(3,ih-1)

      end if

!     --- Reference length

      ref = sqrt(dx*dx + dy*dy + dz*dz)

      dx = curves(1,ih) - xend
      dy = curves(2,ih) - yend
      if ( ndim.eq.3 ) dz = curves(3,ih) - zend

      dif = sqrt(dx*dx + dy*dy + dz*dz )

      ja = 0

      if ( dif.gt.1d-3*ref ) ja = 1

      end
