      subroutine msh466( coorh, npunth, kelemh, funkh, nelemh, &
                         coor, ndim, npoint, vector )

! =======================================================================
!
!         programmer   Niek Praagman
!
!         version 1.2  date  19-04-1994 small adjustments
!         version 1.1  date  17-03-1994 new call of pltr01
!         version 1.0  date  30-03-1992
!
!
!
!   copyright (c) 1992-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     MSH466 interpolates for all (npoint) points of array coor the
!     values as given in arrays funkh, kelemh and coorh
!     If a point is outside the domain extrapolation is used and a
!     warning is given if the distance from point to domain is (too)
!     large
!
! ***********************************************************************
!
!     KEYWORDS
!
!     interpolation
!
! ********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS

      implicit none
      integer npunth, nelemh, ndim, npoint, kelemh(3,nelemh)
      double precision coor( ndim, npoint ), coorh( ndim, npunth), &
                       funkh( npunth ), vector( npoint )

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     coorh   i    coordinate array of TESSELATION mesh
!     funkh   i    array with function values in all nodes of kelemh
!     kelemh  i    element array of TESSELATION mesh
!     ndim    i    Dimension of the space.
!     nelemh  i    number of elements (triangles) in array kelemh
!     npoint  i    Number of nodal points
!     npunth  i    number of points in coorh
!     vector  o    array of special structure of type 115 with all
!                  interpolated values
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer i, nel, inod(3), iresul, j
      double precision det, fact, ski, eta, xp, yp, x(3), y(3)

!     det     area value of triangle
!     eta     local variable in triangle
!     fact    factor to be used to determine extrapolated value
!     i       Counting variable
!     inod    node values
!     iresul  indicator whether point belongs to triangle or not
!     j       Counting variable
!     nel     temporary number of elements
!     ski     local variable in triangle
!     x       array with x-coordinates
!     xp      x-coordinates of point p
!     y       array with y-coordinates
!     yp      y-coordinates of point p
!
! ***********************************************************************
!
!     I/O
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     MSH467  Extrapolate values
!     PLTR01  Check whether point belongs to triangle
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all points
!
!       Run through all elements and determine element for point xp,yp
!
!       If element found then interpolate else extrapolate
!
!       Fill value in vector array
!
! =======================================================================

      call eropen( 'msh466' )

!     Run through all points of coor and find element of kelemh

      do 200 i = 1 , npoint

         xp = coor( 1, i )
         yp = coor( 2, i )

         do 110 nel = 1 , nelemh

            do 100 j = 1, 3
               inod(j) = kelemh( j, nel )
               x(j) = coorh( 1, inod(j) )
               y(j) = coorh( 2, inod(j) )
100         continue

!           Check whether point is inside triangle

            call pltr01( xp, yp, iresul, x, y, ski, eta, det )

            if ( iresul.eq.1 ) goto 150

110      continue

         iresul = 0

150      continue

         if ( iresul.eq.0 ) then

!           No triangle found extrapolation

            call msh467 (xp, yp, coorh, npunth, inod(1), inod(2), fact)

            vector(i) =  funkh(inod(1)) + &
                         fact * ( funkh(inod(2)) - funkh(inod(1)) )

         else if ( iresul.eq.1 ) then

!           Triangle found , interpolate the value

            vector(i) = ( 1d0 - ski - eta ) * funkh( inod(1) ) + &
                                ski         * funkh( inod(2) ) + &
                                      eta   * funkh( inod(3) )

         end if

200   continue

      call erclos( 'msh466' )

      end
