      subroutine msh467( xp, yp, coorh, npunth, i1, i2, fact )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 19-04-1994  small adjustments
!        version  1.0  date 20-03-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the two nearest points with respect to point p.
!     p is the "outside" point for which extrapolation is needed.
!     The value fact is used to compute the extrapolated value
!     by the formula ps = p1 + fact * ( p2 - p1 )
! **********************************************************************
!
!                       KEYWORDS
!
!     extrapolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npunth, i1, i2
      double precision xp, yp, coorh( 2, npunth), fact

!     coorh   i    coordinate array of TESSELATION mesh
!     fact    o    factor to be used for the extrapolation
!     i1      o    node number of the first point that is nearest to p
!     i2      o    node number of the second point that is nearest to p
!     npunth  i    number of points in coorh
!     xp      i    x-coordinate of point p
!     yp      i    y-coordinate of point p
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwarnlc
      double precision dist, distx, disty, dist1, dist2, x1, x2, y1, y2, &
                       distp, xi, yi

!     dist    local distance
!     dist1   local distance
!     dist2   local distance
!     distp   local distance
!     distx   local distance
!     disty   local distance
!     i       Counting variable
!     iwarnlc   indicator for warning
!     x1      x coordinate of point 1
!     x2      x coordinate of point 2
!     xi      x coordinate of point i
!     y1      y coordinate of point 1
!     y2      y coordinate of point 2
!     yi      y coordinate of point i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRWAR  Submit warning
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     943   Distance is too large for extrapolation : strange results
!           can be expected
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all points and find the two points that are nearest
!     to point P. Determine the nearest point of the line through these
!     two points and characterize this point by the parameter fact
!     nearest point = i1 + fact * ( i2 - i1 )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh467' )

      dist1 = rinfin
      dist2 = rinfin
      i1    = 0
      i2    = 0

!     --- Run through all points of coorh and find the two nearest points

      do 200 i = 1 , npunth

          xi = coorh( 1, i )
          yi = coorh( 2, i )

          distx = xi - xp
          disty = yi - yp

          dist  = distx * distx + disty * disty

!         --- If dist is smaller than dist 1 then shift everything

          if ( dist.lt.dist1 ) then

             dist2 = dist1
             i2    = i1
             dist1 = dist
             i1    = i

          else if ( dist.lt.dist2 ) then

             dist2 = dist
             i2    = i

          end if

200   continue

!     --- Determine factor

      x1 = coorh( 1, i1 )
      y1 = coorh( 2, i1 )

      x2 = coorh( 1, i2 )
      y2 = coorh( 2, i2 )

      distx = x2 - x1
      disty = y2 - y1

      dist  = distx * distx + disty * disty

      fact  = ( y1 - y2 )*( y1 - yp ) + ( x1 - x2 )*( x1 - xp )

      fact  = fact / dist

      iwarnlc = 0

      if ( fact.gt.1.5d0 ) then

         iwarnlc = 1

      else if ( fact.lt.-0.5d0 ) then

         iwarnlc = 1

      else

!     --- Compute distance from point p to line

         dist1 = xp - ( x1 + fact * ( x2 - x1 ) )
         dist2 = yp - ( y1 + fact * ( y2 - y1 ) )

         distp = dist1*dist1 + dist2*dist2

         if ( distp.gt.dist ) iwarnlc = 1

      end if

!     --- Give warning if iwarnlc = 1

      if ( iwarnlc.eq.1 ) call errwar ( 943, 0, 0, 0 )

      call erclos( 'msh467' )

      end
