      subroutine msh468( nbndpt, coor  , npoint, kelem, nelem, kstapl, &
                         klaar , klasse )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  7.4  date 27-04-1994  introduction of classes
!        version  7.3  date 19-04-1992  small adjustments
!        version  7.2  date 13-04-1994  adapt comments: JvdZ
!        version  7.1  date 01-05-1992  disturb factor increased
!        version  7.0  date 01-04-1992  new version of TESSA for SEPRAN
!        version  6.0  date 26-07-1991  update
!        version  5.1  date 17-06-1991  general revision by Arno van Duin
!        version  5.0  date 02-01-1989
!
!   copyright (c) 1987-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Construct a triangular grid if only the nodes in a domain are given.
!     Use is made of a special TESSELATION property.
!     It is assumed that the first NBNDPT points form the boundary of
!     the domain and that they are given counterclockwise.
!     In this version use is made of the advancing boundary technique and
!     of subdivisions for the points.
! **********************************************************************
!
!                       KEYWORDS
!
!     tesselation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, kelem(3,nelem), klaar(npoint), &
              kstapl(2*npoint), nbndpt, klasse(2,npoint)
      double precision coor(2,npoint)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     kelem   o    array containing the nodes of the triangles.
!     klaar   o    helparray for detecting that all work is done
!     klasse  i    helparray with for each point its subclass number
!     kstapl  i    helparray for actual boundary of region, &
!                  length 2 * npoint
!     nbndpt  i    the first nbndpt nodes are the boundary points.
!     nelem   o    number of triangles created.
!                  should be : nelem=2*(npoint-1)-nbndpt. (if all points
!                  are really inside the given region)
!     npoint  i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ia, ib, ieen, ii, ip, itwe, j, jap, jj, k, ks, &
              kstap, ls, m, mm, nelm, klasx1, klasx2, &
              klasy1, klasy2, nout
      double precision afij, afik, afkj, alva, beta, dela, det, dr, eps, &
              epsa, ex1, ex2, gama, r, totaal, xp, yp, xmin

!     afij      local distance i to j
!     afik      local distance i to k
!     afkj      local distance k to j
!     alva      helpcoefficient
!     beta      helpcoefficient
!     dela      local distance
!     det       surface of triangle considered
!     dr        local distance
!     eps       accuracy
!     epsa      local accuracy
!     ex1       local sum variable
!     ex2       local sum variable
!     gama      local x-distance
!     i         loop variable
!     ia        first node of line segment
!     ib        second node of line segment
!     ieen      helpvariable for adjustment of local boundary array
!     ii        helpvariable for klaar array
!     ip        loop variable
!     itwe      helpvariable for adjustment of local boundary array
!     j         loop variable
!     jap       indicator for inside and visibility checks
!     jj        helpvariable for klaar array
!     k         loop variable
!     klasx1    lower boundary in x-class
!     klasx2    upper boundary in x-class
!     klasy1    lower boundary in y-class
!     klasy2    upper boundary in y-class
!     ks        loop variable
!     kstap     number of line segments in actual boundary
!     ls        helpvariable to adjust array kstapl
!     m         nearest nodal point
!     mm        helpvariable for klaar array
!     nelm      number of triangles
!     nout      number of nodes outside region indicated
!     r         radius of smallest innercircle
!     totaal    local distance parameter
!     xmin      smallest x-coordinate
!     xp        x-coordinate
!     yp        y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Place double precision real in error array
!     ERRINT  Place integer in error array
!     ERRSUB  Error messages
!     ERRWAR  Submit warning
!     INSTOP  Stop the program
!     MSH469  Determine whether a line is permitted, i.e. whether a
!             proposed line does not cross the boundary
!     MSH474  Check whether point is inside region indicated by boun- &
!             dary segments
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      942    Not the correct number of elements found; most probably
!             boundary points are not given counterclockwise or there
!             are points outside the boundary
!      970    Node is outside region as indicated by boundary
!             elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear boundary array
!     Set array klaar for indication whether points in boundary, inside
!     or outside
!     Set boundary points in kstapl and for all these points a -1 in
!     array klaar
!     Check for all other points whether they are inside the given boundary
!     While boundary not empty do
!        Determine boundary segment to be considered
!        Determine the appropriate klasse numbers
!        Create triangle for this segment by checking for all permitted
!        points in the domain which point creates a circumcircle for
!        the triangle with smallest radius, i.e. for which the distance
!        to the midpoint of the segment is smallest.
!        If no point found give message since this situation should never
!        occur
!        New point m found
!        Place new element in array kelem
!        Adjust boundary array
!        Adjust array klaar
!        Return to begin
!     endwhile
!     Check number of elements found and give message if not correct
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh468' )

!     --- Set accuracy

      eps = 10 * epsmac

!     --- Clear boundary array kstapl

      do 10 i = 1, 2 * npoint
         kstapl(i) = 0
10    continue

!     --- Set array klaar

      do 20 i = 1, npoint
         klaar(i) = 1
20    continue

      nelem = 0

!     --- Set boundary points in kstapl and for all these points -1 in klaar (-1
!         in klaar means that these points are in the boundary )

      xmin = rinfin

      do 40 i = 1, nbndpt
         kstapl(2*i-1) = i
         kstapl(2*i)   = i + 1
         klaar(i)      = -1
         if ( coor(1,i).lt.xmin ) xmin = coor(1,i)
40    continue

      kstapl(2*nbndpt) = 1

      kstap = nbndpt

!     --- Check for all other points that they are inside the given boundary

      nout = 0

      do 45 i = nbndpt + 1 , npoint

!     --- Check for point i

          xp = coor( 1, i )
          yp = coor( 2, i )

          call msh474( xp, yp, xmin, coor, kstapl, kstap, jap )

          if ( jap.eq.0 ) then

!         --- Point i with coordinates ( xp,yp ) is outside the region
!             given by the boundary points ! Execution is not stopped !

             call errint(  i, 1 )
             call erreal( xp, 1 )
             call erreal( yp, 2 )
             call errwar( 970, 1, 2, 0 )

             klaar(i) = 0
             nout     = nout + 1

          end if

45    continue

!     --- First compute number of elements expected, adapted for nout

      nelm = 2 * ( npoint - nout - 1 ) - nbndpt

50    if ( kstap.ne.0 ) then

!     --- Determine boundary points to be considered

         i = kstapl(1)
         j = kstapl(2)

!        --- Determine the appropriate klasse numbers

         klasx1 = min( klasse(1,i), klasse(1,j) ) - 1
         klasx2 = max( klasse(1,i), klasse(1,j) ) + 1
         klasy1 = min( klasse(2,i), klasse(2,j) ) - 1
         klasy2 = max( klasse(2,i), klasse(2,j) ) + 1

!        --- Create triangle for line i-j

         m  = 0
         r  = 1.d30

         do 70 k = 1, npoint

!        --- Check whether point is interesting

             if ( klasse(1,k).lt.klasx1 ) goto 70
             if ( klasse(1,k).gt.klasx2 ) goto 70
             if ( klasse(2,k).lt.klasy1 ) goto 70
             if ( klasse(2,k).gt.klasy2 ) goto 70

             if ( klaar(k).ne.0 .and. k.ne.i .and. k.ne.j ) then

!            --- Suppose point is visible both from i and j

                alva = coor(1,i) - coor(1,j)
                beta = coor(2,i) - coor(2,j)
                gama = coor(1,i) - coor(1,k)
                dela = coor(2,i) - coor(2,k)

!               --- Counterclockwise ?

                det = alva * dela - beta * gama

                if ( det.gt.eps ) then

!               --- Point is counterclockwise

                   epsa   = ( ( coor(1,i) - coor(1,k) ) * &
                              ( coor(1,k) - coor(1,j) ) + &
                              ( coor(2,i) - coor(2,k) ) * &
                              ( coor(2,k) - coor(2,j) ) ) / 2d0
                   afij   = alva * alva + beta * beta
                   dr     = abs ( sqrt ( afij ) * epsa / det )
                   afik   = gama * gama + dela * dela
                   ex1    = alva - gama
                   ex2    = beta - dela
                   afkj   = ex1 * ex1 + ex2 * ex2
                   totaal = afij - afik - afkj
                   if ( totaal.gt.0d0 ) dr = -dr

                else

!               --- Point is not counterclockwise, skip

                   dr = 1d30

                end if

                if ( dr.lt.r ) then

!               --- Point is candidate, check visibility
!                   (jap = 1 means not visible !)

                   call msh469( j, k, coor, kstapl, kstap, jap )

                   if ( jap.eq.0 ) then

!                  --- Check also second line

                      call msh469( i, k, coor, kstapl, kstap, jap )

                   end if

                   if ( jap.eq.0 ) then

!                  --- Both lines j-k and i-k are allowed

                      m  = k
                      r  = dr

                   end if

                end if

             end if

70       continue

!        --- New point m found

         if ( m.eq.0 ) then

            call errint( nbndpt, 1 )
            call errsub ( 942, 1, 0, 0 )
            call instop

         end if

!        --- Place new element in array kelem

         nelem = nelem + 1

         if ( nelem.gt.nelm ) then

            call errint( nbndpt, 1 )
            call errsub ( 942, 1, 0, 0 )
            call instop

         end if

         kelem( 1,nelem ) = i
         kelem( 2,nelem ) = j
         kelem( 3,nelem ) = m

!        --- Adjust array kstapl

         ieen = 1
         itwe = 1
         ls   = 0

         do 80 ks = 2, kstap

            ia = kstapl(2*ks-1)
            ib = kstapl(2*ks)

            if ( ia.eq.m .and. ib.eq.i ) then
               ieen = 0
            else if ( ia.eq.j .and. ib.eq.m ) then
               itwe = 0
            else

               ls = ls + 1
               kstapl(2*ls-1) = ia
               kstapl(2*ls)   = ib

            end if

80       continue

         if ( ieen.eq.1 ) then

            ls = ls + 1
            kstapl(2*ls-1) = i
            kstapl(2*ls)   = m

         end if

         if ( itwe.eq.1 ) then

            ls = ls + 1
            kstapl(2*ls-1) = m
            kstapl(2*ls)   = j

         end if

         kstap = ls

!        --- Adjust array klaar

         ii = 0
         jj = 0
         mm = 0

         do 90 ip = 1, ls

            ia = kstapl(2*ip-1)
            ib = kstapl(2*ip  )

            if ( i.eq.ia .or. i.eq.ib ) ii = -1
            if ( j.eq.ia .or. j.eq.ib ) jj = -1
            if ( m.eq.ia .or. m.eq.ib ) mm = -1

90       continue

         klaar(i) = ii
         klaar(j) = jj
         klaar(m) = mm

         go to 50

      end if

!     --- Check length KELEM

      if ( nelm.ne.nelem ) then

         call errint( nbndpt, 1 )
         call errsub ( 942, 1, 0, 0 )
         call instop

      end if

      call erclos( 'msh468' )

      end
