      subroutine msh474( xp, yp, xmini, coor, kbound, nbound, ja )

! =======================================================================
!
!
!         programmer   niek praagman
!
!         version 1.0  date  27-04-1994 see MSHO07
!
!
!
!   copyright (c) 1994-1994  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Subroutine to determine whether point (xp,yp) belongs to
!     the area given by kbound.
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     2d
!
! ***********************************************************************
!
!     INPUT / OUTPUT  PARAMETERS
!
      implicit none

      double precision xp, yp, xmini, coor(2,*)
      integer kbound(*), nbound, ja

!     coor       i  coordinate array
!
!     ja         o  output parameter
!                     if point belongs to area ( ja = 1 )
!                     else ( ja = 0 )
!
!     kbound     i  boundary element array
!
!     nbound     i  number of boundary elements
!
!     xmini      i  smallest x-coordinate of all nodes
!
!     xp         i  x-coordinate of point considered
!
!     yp         i  y-coordinate of point considered
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      double precision xleft, yleft, xmin, ymin, ymax, x1, y1, x2, y2
      integer i, isnij, ih, i1, i2

!
!     i         loop variable
!
!     i1        first node boundary segment
!
!     i2        second node boundary segment
!
!     ih        help indicator for crossing point
!
!     isnij     number of crossings with boundary segments
!
!     x1        x-coordinate of node i1
!
!     x2        x-coordinate of node i2
!
!     xleft     reference x-coordinate (far away)
!
!     xmin      min value of x-values of boundary line
!
!     y1        y-coordinate of node i1
!
!     y2        y-coordinate of node i2
!
!     yleft     reference y-coordinate
!
!     ymax      max value of y-values of boundary line
!
!     ymin      min value of y-values of boundary line
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     MSH470  Check whether two line segments have a common point
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Suppose that a line is drawn from point (xp,yp) to reference
!     point (xleft,yleft). Count all crossings of this line with boundary
!     lines (i1,i2) from array KBOUND. If the number of crossings is odd
!     than point (xp,yp) belongs to area given by KBOUND.
!
! =======================================================================

      xleft = xmini - 10d0
      yleft = yp    + 1d-3

      isnij = 0
      ja    = 0

      do 100 i = 1, nbound

          i1 = kbound(2*i-1)
          i2 = kbound(2*i  )

          x1 = coor( 1,i1 )
          y1 = coor( 2,i1 )
          x2 = coor( 1,i2 )
          y2 = coor( 2,i2 )

          xmin = min(x1,x2)
          ymin = min(y1,y2)
          ymax = max(y1,y2)

          if ( xp.lt.xmin ) goto 100
          if ( yp.lt.ymin ) goto 100
          if ( yp.gt.ymax ) goto 100

!         Potential point

          call msh470( x1, y1, x2, y2, xleft, yleft, xp, yp, ih )

          if ( ih.eq.1 ) goto 100

!         Real intersection point

          isnij = isnij + 1

100   continue

      if ( (isnij - (isnij/2) * 2).ne.0 ) ja = 1

      end
