      subroutine mshc05(coor,npoint,line,naant,grof,nelm,ndim,incv, &
                        ribbe,stotal)
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 14-04-1996 Update by Sander de Snoo
!        version  1.0  date 24-01-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    determine number of elements to be created along polygon- &
!    track of NAANT points with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*),grof(*),ribbe,stotal
      integer npoint,line(*),nelm,ndim,incv,naant

!    coor   i  coordinate array
!    grof   i  array containing the coarseness of the coor points
!    incv   i  curve number
!    line   i  array containing the node number of the defining points
!              sequentially
!    naant  i  number of defining points
!    ndim   i  dimension of problem
!    nelm   o  number of elements to be created on curve
!    npoint i  number of points in coor
!    ribbe  i  coarseness unit
!    stotal o  total length of all segments together
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision sleng,x1,x2,y1,y2,z1,z2,nlocal,ntotal,edge1,edge2
      integer n,i,j1,j2

!     edge1      coarseness at begin piece
!     edge2      coarseness at end piece
!     i          Counting variable
!     j1         Lower index number along second direction of a face
!     j2         Upper index number along second direction of a face
!     n          number of line pieces
!     nlocal     (real) number of elements along local piece
!     ntotal     number of elements along spline
!     sleng      length of line pieces
!     x1         x-coordinate
!     x2         x-coordinate
!     y1         y-coordinate
!     y2         y-coordinate
!     z1         z-coordinate
!     z2         z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     127   user point number out of range
!     470   initial and end point of line have the same co-ordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshc05' )

      stotal =  0
      ntotal =  0
      n      =  naant - 1

      do 100 i = 1 , n

          j1 = line( i     )
          j2 = line( i + 1 )

!         --- Check whether node numbers exist

          if ( j1.le.0 .or. j1.gt.npoint ) then
             call errint ( j1, 1 )
             call errint ( npoint, 2 )
             call errsub ( 127, 2, 0, 0 )
          end if
          if ( j2.le.0 .or. j1.gt.npoint ) then
             call errint ( j2, 1 )
             call errint ( npoint, 2 )
             call errsub ( 127, 2, 0, 0 )
          end if

          x1 =  coor(ndim*(j1-1)+1)
          x2 =  coor(ndim*(j2-1)+1)

          if ( ndim.ne.1 ) then
               y1 = coor(ndim*(j1-1)+2)
               y2 = coor(ndim*(j2-1)+2)
          if ( ndim.eq.3 ) then
               z1 = coor(ndim*(j1-1)+3)
               z2 = coor(ndim*(j2-1)+3)
          end if

          end if

!         --- calculate local length sleng

          if ( ndim.eq.1 ) then

             sleng = abs( x2 - x1 )
             if ( sleng.le.epsmac * abs(x1+x2) ) then
                call errint ( incv, 1 )
                call erreal ( sleng, 1 )
                call errsub ( 470, 1, 1, 0 )
             end if

          else if ( ndim.eq.2 ) then

               sleng = sqrt((x2-x1)**2+(y2-y1)**2)
               if ( sleng**2.le.epsmac*((abs(x1)+abs(x2))**2+ &
                    (abs(y1)+abs(y2))**2) ) then
                call errint ( incv, 1 )
                call erreal ( sleng, 1 )
                call errsub ( 470, 1, 1, 0 )
             end if

          else if ( ndim.eq.3 ) then

               sleng = sqrt((x2-x1)**2+(y2-y1)**2+(z2-z1)**2)
               if ( sleng**2.le.epsmac*((abs(x1)+abs(x2))**2+ &
                    (abs(y1)+abs(y2))+(abs(z1)+abs(z2))**2)) then
                call errint ( incv, 1 )
                call erreal ( sleng, 1 )
                call errsub ( 470, 1, 1, 0 )
             end if

          end if

!         --- coarseness at begin/end segment

          edge1 = grof(j1)*ribbe
          edge2 = grof(j2)*ribbe

!         --- compute (real) number of elements on segment

          if (abs(edge1/edge2-1d0).ge.epsmac) then
             nlocal = sleng/(edge1-edge2)*log(edge1/edge2)
          else
             nlocal = 2d0*sleng/(edge1+edge2)
          end if

          stotal = stotal + sleng

          ntotal = ntotal + nlocal

100   continue

!     --- number of elements on spline should be integer and at least 1

      nelm = max(1,nint(ntotal))

      call erclos ( 'mshc05' )

      end
