      subroutine mshchacr ( iinput, coorlc, kmeshclc1, icurvs, icall )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 23-11-2012 Extra debug
!        version  3.0  date 24-08-2010 Remove parameters iwork/nodes
!        version  2.2  date 31-01-2006 Debug statements
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change coordinates in array coorlc by (repeated) calls to FUNCCOOR
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), kmeshclc1(*), icurvs(*), icall
      double precision coorlc(*)

!     coorlc   i/o   array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     icall      i    Defines which coordinates must be adapted:
!                     icall = 1, user points
!                     icall = 2, curves
!                     icall = 3, surfaces
!                     icall = 4, volumes
!                     icall = 5, all
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     iinput     i    integer input array
!     kmeshclc1  i    kmesh part c, contains the nodal points corresponding
!                     to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, nchancoor, ichanfunc, itypepos, ifirst, &
              ilast, istar3, i1, i2, istart, numnodes, iend
      integer, allocatable :: iwork(:), nodes(:)

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     ichanfunc      Choice parameter for subroutine FUNCCOOR
!     iend           Last position used at a part
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     itypepos       Type of position parameter
!                    0: all points
!                    1: user points
!                    2: curves
!                    3: surfaces
!                    4: volumes
!     iwork          integer work array of length npoint
!     j              Counting variable
!     k              Counting variable
!     nchancoor      number of data records for change coordinates
!     nodes          Work array of length npoint
!                    Contains the node numbers that must be filled
!     numnodes       number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCCOOR       User written subroutine to adapt the coordinates
!     MSHCHACR1      Overestimate npoint for subroutine mshchacr
!     PRININ         print 1d integer array
!     SEPMSHEXTKT    Compute starting and ending address of elements in kmeshc
!                    by using the information stored in kmesh and kmesht
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nchangecoor = iinput(1)
!     For i := 1 (1) nchangecoor do
!        ichanfunc := iinput(4*i-2)
!        itypepos  := iinput(4*i-1)
!        ifirst    := iinput(4*i)
!        ilast     := iinput(4*i+1)
!        Compute number of nodes and store in iwork
!        call funccoor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchacr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshchacr'
         write(irefwr,1) 'icall', icall
         call prinin ( iinput, 4*iinput(1)+1, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      nchancoor = iinput(1)
      if ( debug ) write(irefwr,1) 'nchancoor', nchancoor

      numnodes = 0

!     --- Loop over all cases

      do i = 1, nchancoor

         ichanfunc = iinput(4*i-2)
         itypepos  = iinput(4*i-1)
         ifirst    = iinput(4*i)
         ilast     = iinput(4*i+1)
         if ( debug ) &
            write(irefwr,1) 'ichanfunc, itypepos, ifirst, ilast', &
                             ichanfunc, itypepos, ifirst, ilast

!        --- Compute number of nodes

         call mshchacr1 ( numnodes, icall, itypepos, icurvs, &
                          ifirst, ilast )

         allocate ( iwork(numnodes), nodes(numnodes), stat = error )
         if ( error/=0 ) then

!        --- error#0, so not enough space

            call errint ( 2*numnodes, 1 )
            call errint ( error, 2 )
            call errchr ( 'iwork', 1 )
            call errsub ( 2910, 2, 0, 1 )
            go to 1000

         end if  ! ( error/= 0 )

!        --- Recompute number of nodes and store in iwork

         if ( icall==5 .and. mod(itypepos,100)==0 ) then

!        --- itypepos = 0, all points

            numnodes = npoint
            do j = 1, numnodes
               nodes(j) = j
            end do

         else if ( icall==5 .and. itypepos==1 ) then

!        --- itypepos = 1, user points from ifirst to ilast

            numnodes = 0
            do j = ifirst, ilast
               inodp = kmeshl(j)
               if ( inodp>0 ) then

!              --- User point found

                  numnodes = numnodes+1
                  nodes(numnodes) = inodp

               end if

            end do

         else if ( icall==5 .and. itypepos==2 ) then

!        --- itypepos = 2, curves from ifirst to ilast

            numnodes = npoint
            do j = 1, numnodes
               iwork(j) = 0
            end do
            numnodes = 0
            istar3 = kmeshm(2)-1
            do j = ifirst, ilast
               i1 = kmeshm(4+j)
               i2 = kmeshm(5+j)-1
               do k = i1, i2
                  inodp = kmeshm(istar3+k)
                  if ( inodp>0 ) then

!                 --- Nodal point found

                     if ( iwork(inodp)==0 ) then

!                    --- Nodal point has not yet been used

                        iwork(inodp) = 1
                        numnodes = numnodes+1
                        nodes(numnodes) = inodp

                     end if

                  end if

               end do

            end do

         else if ( icall==5 .and. itypepos==3 ) then

!        --- itypepos = 3, surfaces from ifirst to ilast

            numnodes = npoint
            do j = 1, numnodes
               iwork(j) = 0
            end do
            numnodes = 0
            if ( kelmn>0 ) then

!           --- kmesh part n exists

               istart = 2*nsurfs
               do j = 1, ifirst-1
                  istart = istart + kmeshn(j)*kmeshn(j+nsurfs)
               end do
               do j = ifirst, ilast
                  inpelm = kmeshn(j)
                  inelem = kmeshn(j+nsurfs)
                  do k = 1, inpelm*inelem
                     inodp = kmeshn(istart+k)
                     if ( inodp>0 ) then

!                    --- Nodal point found

                        if ( iwork(inodp)==0 ) then

!                       --- Nodal point has not yet been used

                           iwork(inodp) = 1
                           numnodes = numnodes+1
                           nodes(numnodes) = inodp

                        end if

                     end if

                  end do
                  istart = istart+inpelm*inelem

               end do

            else

!           --- Use array kmesh part t

               do j = ifirst, ilast

!              --- Compute starting address of element ielem1 in kmeshclc1
!                  Compute end address of element ielem2 in kmeshclc1

                  call sepmshextkt ( j, istart, iend )

                  do k = istart, iend
                     inodp = kmeshclc1(k)
                     if ( inodp>0 ) then

!                    --- Nodal point found

                        if ( iwork(inodp)==0 ) then

!                       --- Nodal point has not yet been used

                           iwork(inodp) = 1
                           numnodes = numnodes+1
                           nodes(numnodes) = inodp

                        end if

                     end if

                  end do

               end do

            end if

         else if ( icall==5 .and. itypepos==4 ) then

!        --- itypepos = 4, volumes from ifirst to ilast
!            Use array kmesh part t

            numnodes = npoint
            do j = 1, numnodes
               iwork(j) = 0
            end do
            numnodes = 0
            do j = ifirst, ilast

!           --- Compute starting address of element ielem1 in kmeshclc1
!               Compute end address of element ielem2 in kmeshclc1

               call sepmshextkt ( j, istart, iend )

               do k = istart, iend
                  inodp = kmeshclc1(k)
                  if ( inodp>0 ) then

!                 --- Nodal point found

                     if ( iwork(inodp)==0 ) then

!                    --- Nodal point has not yet been used

                        iwork(inodp) = 1
                        numnodes = numnodes+1
                        nodes(numnodes) = inodp

                     end if

                  end if

               end do

            end do

         else if ( icall==1 .and. itypepos==101 ) then

!        --- itypepos = 101, user points from ifirst to ilast

            numnodes = 0
            do j = ifirst, ilast
               numnodes = numnodes+1
               nodes(numnodes) = j
            end do

         else if ( icall==2 .and. itypepos==102 ) then

!        --- itypepos = 102, curves from ifirst to ilast

            numnodes = 0
            if ( ifirst==1 ) then
               istart = 1
            else
               istart = icurvs(ifirst-1)+1
            end if
            do j = istart, icurvs(ilast)
               numnodes = numnodes+1
               nodes(numnodes) = j
            end do

         else if ( icall==3 .and. itypepos==103 .or. &
                   icall==4 .and. itypepos==104 ) then

!        --- itypepos = 103, surfaces from ifirst to ilast or
!                       104, volumes from ifirst to ilast

            numnodes = npoint
            do j = 1, numnodes
               iwork(j) = 0
            end do
            numnodes = 0
            do j = ifirst, ilast
               if ( j==isurnr ) then
                  do k = 1, inpelm*inelem

                     inodp = kmeshclc1(k)
                     if ( inodp>0 ) then

!                    --- Nodal point found

                        if ( iwork(inodp)==0 ) then

!                       --- Nodal point has not yet been used

                           iwork(inodp) = 1
                           numnodes = numnodes+1
                           nodes(numnodes) = inodp

                        end if

                     end if

                  end do

               end if

            end do

         end if

         if ( debug ) call prinin ( nodes, numnodes, 'nodes' )

!        --- call funccoor

         if ( numnodes>0 ) &
            call funccoor ( ichanfunc, ndim, coorlc, nodes, numnodes )

         deallocate ( iwork, nodes, stat = error )
         if ( error/= 0 ) then

!        --- error#0, problem with deallocate

            call errint ( error, 1 )
            call errchr ( 'iwork', 1 )
            call errsub ( 2911, 1, 0, 1 )

         end if  ! ( error/= 0 )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshchacr'

      end if
      call erclos ( 'mshchacr' )

      end
