      subroutine mshcheckelms
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 18-06-2014 Jump in case of error
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 20-05-2005 Fortran 90
!
!   copyright (c) 2001-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if all edges in the two-dimensional mesh correspond to 2 elements
!    Edges may only be connected to one element if both nodes are on the
!    boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     element
!     node
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHECKELMS1  Check if two-dimensional mesh contains hanging nodes
!     MSHCHECKELMS2  Check if three-dimensional mesh contains hanging nodes
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHF  Fill array nmeshf
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHH  Fill array nmesh part h
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     compute starting addresses
!     Perform testing
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckelms' )
      if ( ierror/=0 ) go to 1000
      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( ndim==2 ) then

!     --- ndim = 2
!         activate arrays

         if ( nmesh(33)==0 ) call sepfillnmeshc
         if ( nmesh(38)==0 ) call sepfillnmeshh
         if ( ierror/=0 ) go to 1000

!        --- Perform testing

         call mshcheckelms1

      else

!     --- ndim = 3
!         activate arrays

         if ( nmesh(36)==0 ) call sepfillnmeshf
         if ( nmesh(37)==0 ) call sepfillnmeshg
         if ( ierror/=0 ) go to 1000

!        --- Perform testing

         call mshcheckelms2

      end if
1000  call erclos ( 'mshcheckelms' )
      end
