      subroutine mshcpcom ( iwork, usework )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 24-11-2004 Fortran 90
!        version  1.2  date 21-03-2003 Correction for compound curve 1
!        version  1.1  date 21-09-2002 New storage compound curves
!
!   copyright (c) 1986-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array curves in case of compound curves by copying from
!     the single curves it consists of
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     compound_curve
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*)
      logical usework

!     iwork      i    integer work array of length ncurvs containing
!                     information if a curve must be used for copying
!                     iwork(icurve)>0 or not iwork(icurve)=0
!     usework    i    Indicates if array iwork is used (true) or not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, n, jcurve, icount, istart, icurve, idirec, icurnr, &
              nsubcv
      logical loop

!     i              Counting variable
!     icount         number of unknowns per element
!     icurnr         Curve sequence number
!     icurve         Type of curve
!     idirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     istart         First position used at a part
!     j              Counting variable
!     jcurve         Curve number
!     loop           If true the loop must be carried out
!     n              number of points at the curve
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcpcom' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcpcom'

      end if

!     --- Loop over curves

      loop = .true.
      icount = 1
      do icurnr = 1, ncurvs
         nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         if ( debug ) write(irefwr,*) 'icurnr, nsubcv ', icurnr, nsubcv
         if ( nsubcv>0 ) then

!        --- Curve contains subcurves

            if ( icurnr>1 ) icount = icurvsmsh(icurnr-1)+1
            do j = 1, nsubcv
               jcurve = kmeshq(kmeshq(icurnr)+j)
               icurve = abs(jcurve)
               if ( icurve==1 ) then
                  n = icurvsmsh(icurve)
                  if ( jcurve>0 ) then
                     istart = 1
                     idirec = 1
                  else
                     istart = icurvsmsh(icurve)
                     idirec =-1
                  end if
               else
                  n = icurvsmsh(icurve)-icurvsmsh(icurve-1)
                  if ( jcurve>0 ) then
                     istart = icurvsmsh(icurve-1)+1
                     idirec = 1
                  else
                     istart = icurvsmsh(icurve)
                     idirec =-1
                  end if
               end if
               if ( usework ) loop = iwork(icurve)>0
               if ( debug ) then
                  write(irefwr,*) 'j, jcurve, n ', j, jcurve, n
                  write(irefwr,*) 'istart, idirec, icount ', &
                                   istart, idirec, icount
               end if

               if ( loop ) then

!              --- Copy co-ordinates

                  do i = 1, n
                     curvesmsh(1,icount+i-1) = &
                        curvesmsh(1,istart+idirec*(i-1))
                     curvesmsh(2,icount+i-1) = &
                        curvesmsh(2,istart+idirec*(i-1))
                  end do

               end if
               icount = icount+n

            end do

         end if

      end do

      if ( debug ) write(irefwr,*) 'End mshcpcom'
      call erclos ( 'mshcpcom' )

      end
