      subroutine mshd08 ( coor, npt, npti, iwork, ibuurp, rim )
! ======================================================================
!
!
!        programmer     Niek Praagman
!        version 1.2    date   20-03-1993 Correction due to Mark Noot
!        version 1.1    date   03-05-1990 new programming norms
!
!
!   copyright (c) 1982-1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!    perform a repositioning of all new created points
!
! ********************************************************************
!
!                      KEYWORDS
!
!   mesh
!   surface
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      double precision coor(*),rim
      integer npt,npti,iwork(*),ibuurp(*)

!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ibuurp  i    array with node numbers of neighbour points sequentially
!     iwork        help array with startadresses neighbours
!     npt     i    total number of points in the mesh
!     npti    i    number of boundary points + 1
!     rim     i    length of smallest edge along the boundary
! **********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      double precision afstnd, xz, yz, xh, yh, af
      integer i, jr, ikn, jstart, jeind, jkn, nbuur

!     af      ?
!     afstnd  ?
!     i       Counting variable
!     ikn     ?
!     jeind   ?
!     jkn     ?
!     jr      ?
!     jstart  ?
!     nbuur   ?
!     xh      ?
!     xz      ?
!     yh      ?
!     yz      ?
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- At most fifty steps of repositioning

      do 200 jr = 1 , 50

          afstnd = 0d0

!         --- Run through all internal points

          do 100 ikn = npti , npt

!         --- ikn is local node number

              jstart  =  0
              if (ikn.ne.1) jstart = iwork(ikn-1)

              jstart  =  jstart + 1
              jeind   =  iwork(ikn)

              if ( jstart.gt.jeind ) goto 100

              xz = 0d0
              yz = 0d0

              do 150 i = jstart , jeind

                  jkn  = ibuurp(i)
                  xz   = xz + coor(2*jkn-1)
                  yz   = yz + coor(2*jkn)

150           continue

              nbuur  =  jeind - jstart + 1

              xz = xz / nbuur
              yz = yz / nbuur

              xh = coor( 2 * ikn - 1 )
              yh = coor( 2 * ikn     )

              af = sqrt( (xh-xz)*(xh-xz) + (yh-yz)*(yh-yz) )

              if ( af.gt.afstnd ) afstnd = af

              coor(2*ikn-1) = xz
              coor(2*ikn  ) = yz

100       continue

          if ( jr.gt.2 .and. afstnd.lt.0.01d0 * rim ) return

200   continue

      end
