      subroutine mshexint ( usol, number )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   04-10-1996
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Interpolate solution in extra points created by mshcrtsp
!   This subroutine is meant for reading own data
! **********************************************************************
!
!                       KEYWORDS
!
!    mesh
!    interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number
      double precision usol(number,*)

!     number     i    Indicates the number of degrees of freedom stored
!                     per point
!     usol      i/o   Solution vector
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, npointlc, inodp, nneigh, istart, ineigh
      double precision sum(100), w, dist

!     dist       Distance between two points
!     i          Counting variable
!     ineigh     neighbour
!     inodp      Nodal point number
!     istart     First position used at a part
!     j          Counting variable
!     k          Counting variable
!     nneigh     Number of neighbours of node
!     npointlc   Number of nodal points
!     sum        Help parameter to compute a sum
!     w          sum of weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all extra points do
!        Find all "real" neighbours
!        Interpolate solution from "real" neighbours
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      npointlc = npoint-nextpnt

!     --- Loop over extra points

      do i = 1, nextpnt
         inodp = npointlc+i
         nneigh = kmshe1(inodp+1)-kmshe1(inodp)
         istart = kmshe1(inodp)
         do k = 1, number
            sum(k) = 0d0
         end do
         w = 0d0
         do j = 1, nneigh
            ineigh = kmshe2(istart+j)
            dist = 0d0
            do k = 1, ndim
               dist = dist + (coor(k,inodp)-coor(k,ineigh))**2
            end do
            dist = sqrt(1d0/dist)
            w = w + dist
            do k = 1, number
               sum(k) = sum(k) + usol(k,ineigh)*dist
            end do
         end do
         if ( nneigh.gt.0 ) then
            do k = 1, number
               usol(k,inodp) = sum(k)/w
            end do
         end if
      end do
      end
