      subroutine mshfilcr ( icurvs, curves, kmeshm ,coor, ncurvs, ndim )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   20-06-1996
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill arrays icurvs and curves from existing arrays kmeshm and coor
!
! **********************************************************************
!
!                       KEYWORDS
!
!   mesh
!   fill
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer  icurvs(*), ndim, ncurvs, kmeshm(*)
      double precision curves(ndim,*), coor(ndim,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     curves     o    array containing the coordinates of the curves
!     icurvs     o    array containing the number of points in the curves
!                     accumulated
!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     ncurvs     i    Number of curves in mesh
!     ndim       i    Dimension of the space.
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, istar3, ipoint

!     i          Counting variable
!     ipoint     Nodal point sequence number
!     istar3     Starting address of KMESHM part 3 in KMESH part M
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Copy array kmeshm into icurvs
!    For all points in kmeshm do
!       If ( point number = 0 ) then
!          Put zero in array curves
!       else
!          Copy co-ordinates in array curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Copy array kmeshm into icurvs

      do i = 1, ncurvs
         icurvs(i) = kmeshm(5+i)-1
      end do

!     --- For all points in kmeshm do

      istar3 = kmeshm(2)-1
      do i = 1, icurvs(ncurvs)
         ipoint = kmeshm(istar3+i)
         if ( ipoint.eq.0 ) then

!        --- Put zero in array curves

            do k = 1, ndim
               curves(k,i) = 0d0
            end do

         else

!        --- Copy co-ordinates in array curves

            do k = 1, ndim
               curves(k,i) = coor(k,ipoint)
            end do

         end if

      end do

      end
