      subroutine mshg24( ski, eta, zta, x1, x2, x3, xn )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 06-12-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute coordinates of internal point ski,eta,zta in case of a
!     spectral hexahedronal element for Gauss-Legendre-Lobatto points
!     while along the boundary the cornerpoints and the midpoints have
!     been given and also the centroid of faces and volume is known.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision ski, eta, zta, x1(27), x2(27), x3(27), xn(3)

!     eta       i     second coordinate local system
!     ski       i     first coordinate local system
!     x1        i     array with first coordinate of 27 basis points
!     x2        i     array with second coordinate of 27 basis points
!     x3        i     array with third coordinate of 27 basis points
!     xn        o     coordinates of transformed point (ski,eta,zta)
!     zta       i     third coordinate local system
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision phi(27)

!     i          loop variable
!     phi        array to store values of the basis functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from string
!     EROPEN  Concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all basis functions and determine the value for
!     point (ski,eta,zta)
!     Compute the coordinates of point (ski,eta,zta)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshg24' )

!     --- Run through all basis functions and determine value

      phi(1) = 0.125d0 * ski * (ski-1d0) * eta * (eta-1d0) * &
                         zta * (zta-1d0)
      phi(2) = 0.250d0 * (1d0-ski)*(1d0+ski)*eta*(eta-1d0) * &
                         zta * (zta-1d0)
      phi(3) = 0.125d0 * ski * (ski+1d0) * eta * (eta-1d0) * &
                         zta * (zta-1d0)
      phi(4) = 0.250d0 * ski * (ski-1d0) * (1d0-eta)*(1d0+eta) * &
                         zta * (zta-1d0)
      phi(5) = 0.500d0 * (1d0-ski)*(1d0+ski)*(1d0-eta)*(1d0+eta) * &
                         zta * (zta-1d0)
      phi(6) = 0.250d0 * ski * (ski+1d0)*(1d0-eta)*(1d0+eta) * &
                         zta * (zta-1d0)
      phi(7) = 0.125d0 * ski * (ski-1d0) * eta * (eta+1d0) * &
                         zta * (zta-1d0)
      phi(8) = 0.250d0 * (1d0-ski)*(1d0+ski)*eta*(1d0+eta) * &
                         zta * (zta-1d0)
      phi(9) = 0.125d0 * ski * (ski+1d0) * eta * (eta+1d0) * &
                         zta * (zta-1d0)
      phi(10) = 0.250d0 * ski * (ski-1d0) * eta * (eta-1d0) * &
                         (1d0-zta) * (1d0+zta)
      phi(11) = 0.500d0 * (1d0-ski)*(1d0+ski)*eta*(eta-1d0) * &
                         (1d0-zta) * (1d0+zta)
      phi(12) = 0.250d0 * ski * (ski+1d0) * eta * (eta-1d0) * &
                         (1d0-zta) * (1d0+zta)
      phi(13) = 0.500d0 * ski * (ski-1d0) * (1d0-eta)*(1d0+eta) * &
                         (1d0-zta) * (1d0+zta)
      phi(14) = 0.500d0 * ski * (ski+1d0)*(1d0-eta)*(1d0+eta) * &
                         (1d0-zta) * (1d0+zta)
      phi(15) = 0.250d0 * ski * (ski-1d0) * eta * (eta+1d0) * &
                         (1d0-zta) * (1d0+zta)
      phi(16) = 0.500d0 * (1d0-ski)*(1d0+ski)*eta*(1d0+eta) * &
                         (1d0-zta) * (1d0+zta)
      phi(17) = 0.250d0 * ski * (ski+1d0) * eta * (eta+1d0) * &
                         (1d0-zta) * (1d0+zta)
      phi(18) = 0.125d0 * ski * (ski-1d0) * eta * (eta-1d0) * &
                          zta * (zta+1d0)
      phi(19) = 0.250d0 * (1d0-ski)*(1d0+ski)*eta*(eta-1d0) * &
                          zta * (zta+1d0)
      phi(20) = 0.125d0 * ski * (ski+1d0) * eta * (eta-1d0) * &
                          zta * (zta+1d0)
      phi(21) = 0.250d0 * ski * (ski-1d0) * (1d0-eta)*(1d0+eta) * &
                          zta * (zta+1d0)
      phi(22) = 0.500d0 * (1d0-ski)*(1d0+ski)*(1d0-eta)*(1d0+eta) * &
                          zta * (zta+1d0)
      phi(23) = 0.250d0 * ski * (ski+1d0)*(1d0-eta)*(1d0+eta) * &
                          zta * (zta+1d0)
      phi(24) = 0.125d0 * ski * (ski-1d0) * eta * (eta+1d0) * &
                          zta * (zta+1d0)
      phi(25) = 0.250d0 * (1d0-ski)*(1d0+ski)*eta*(1d0+eta) * &
                          zta * (zta+1d0)
      phi(26) = 0.125d0 * ski * (ski+1d0) * eta * (eta+1d0) * &
                          zta * (zta+1d0)
      phi(27) =           (1d0-ski)*(1d0+ski) * (1d0-eta)*(1d0+eta) * &
                          (1d0-zta)*(1d0+zta)

!     --- Compute the coordinates of the point (ski,eta) in the original
!         setting

      xn(1) = 0d0
      xn(2) = 0d0
      xn(3) = 0d0

      do 100 i = 1 , 27

               xn(1) = xn(1) + x1(i) * phi(i)
               xn(2) = xn(2) + x2(i) * phi(i)
               xn(3) = xn(3) + x3(i) * phi(i)

100   continue

      call erclos ( 'mshg24' )

      end
