      subroutine mshg25( kelem, coor, nelem )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 06-12-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check submitted hexahedrons before starting with spectral method
!     (Ordering of 8 nodes is classic)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(*), nelem
      double precision coor(3,*)

!     coor     i    coordinate array
!     kelem    i    element array (spectral elements)
!     nelem    i    number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer in(8), iel, ip, jpl

!     iel       loop variable
!     in        node array
!     ip        loop variable
!     jpl       pointer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    unconcatenate name from calling sequence
!     EROPEN    concatenate name to calling sequence
!     INSTOP    stop execution of SEPRAN program
!     MSH319    check value of volume of tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the eight cornernodes of each subelement
!     Check for six tetrahedrons that the volumes are positive
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'mshg25' )

      do 200 iel = 1 , nelem

         jpl = 8 * ( iel - 1 )

         do 100 ip = 1 , 8

             in( ip ) = kelem( jpl + ip )

100      continue

!        --- Check all elements first

         call msh319( in(1), in(6), in(2), in(7), coor )
         call msh319( in(1), in(2), in(3), in(7), coor )
         call msh319( in(1), in(3), in(4), in(7), coor )
         call msh319( in(1), in(4), in(8), in(7), coor )
         call msh319( in(1), in(8), in(5), in(7), coor )
         call msh319( in(1), in(5), in(6), in(7), coor )

200   continue

      if ( nerror.gt.0 ) call instop

      call erclos( 'mshg25' )

      end
