      subroutine mshmapsurin ( iinput, iinp, iposk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 21-08-2006 Extension with mapping3d = 2
!        version  1.0  date 07-07-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Extract parameters for mapping to 3d surface from array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     map
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, iposk

!     iinp           i    last position used in array iinput
!     iinput         i    integer input array
!     iposk          i    Last position in array IINPUT before the information
!                         of the present surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurve, j, jinp, ndefsf

!     i              Counting variable
!     icurve         Type of curve
!     j              Counting variable
!     jinp           last position used in array iinput with respect to
!                    curves
!     ndefsf         Number of curves in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH046         Raise iinp as function of icurve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2082   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get mapping parameters from iinput
!     Check mapping
!     If ( mapping3d=1 ) then
!        if ( icentre=0 ) then
!           find centre of arc in curves surrounding surface
!        Check icentre
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmapsurin' )

!     --- Get mapping parameters from iinput

      mapping3d = iinput(iposk+iinp+1)
      icentre = iinput(iposk+iinp+2)
      ndefsf = mod(iinput(iposk+3),10000)

!     --- Check mapping3d

      if ( mapping3d<0 .or. mapping3d>2 ) then

!     --- mapping3d out of range

         call errint ( isurnr, 1 )
         call errint ( mapping3d, 2 )
         call errint ( 0, 3 )
         call errint ( 1, 4 )
         call errchr ( 'mapping3d', 1 )
         call errsub ( 2082, 4, 0, 1 )

      end if
      if ( mapping3d==1 .or. mapping3d==2 ) then

!     --- mapping3d = 1, map onto sphere

         if ( icentre==0 ) then

!        --- icentre has not been filled yet, check if a centre can be found
!            by considering all surrounding curves

            ndefsf = mod(iinput(iposk+3),10000)
            do i = 1, ndefsf
               icurnr = abs(iinput(iposk+3+i))
               jinp = iinp1
               do j = 1, icurnr
                  icurve = iinput(jinp)
                  if ( icurve==3 .or. icurve==6 ) then

!                 --- arc or carc found, detect centre

                     icentre = iinput(jinp+6)
                     go to 1000

                  else

!                 --- Raise jinp

                     call msh046 ( iinput, jinp, icurve )

                  end if

               end do

            end do

         end if

         if ( icentre<=0 .or. icentre>nuspnt ) then

!        --- icentre out of range

            call errint ( isurnr, 1 )
            call errint ( icentre, 2 )
            call errint ( 1, 3 )
            call errint ( nuspnt, 4 )
            call errchr ( 'icentre', 1 )
            call errsub ( 2082, 4, 0, 1 )

         end if

      end if

1000  call erclos ( 'mshmapsurin' )

      end
