      subroutine msho77( nx, ny, kmeshm )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.0    date   27-06-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part m in case of a structured mesh in 2D where nx and
!     ny are the number of points in the both coordinatedirections
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nx, ny, kmeshm(*)

!     kmeshm  o    part M of array KMESH see SEPRAN PG
!     nx      i    number of nodal points in x-direction
!     ny      i    number of nodal points in y-direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nbound

!     i       loop variable
!     nbound  total number of boundary nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill all positions as indicated in the PG
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill the array positions as indicated in the Programmers Guide

      nbound = 2 * ( nx + ny ) - 4

      kmeshm(  1 ) = 0
      kmeshm(  2 ) = 7
      kmeshm(  3 ) = 7 + nbound
      kmeshm(  4 ) = 0
      kmeshm(  5 ) = 1
      kmeshm(  6 ) = 1 + nbound

!     --- Run along the "lower" x-axis

      do 100 i = 1 , nx - 1

          kmeshm( 6 + i ) = i

100   continue

!     --- Run along the "right" y-axis

      do 200 i = 1 , ny - 1

          kmeshm( 5 + nx + i ) = i * nx

200   continue

!     --- Run along the "upper" x-axis

      do 300 i = 1 , nx - 1

          kmeshm( 4 + nx + ny + i ) = nx * ny + 1 - i

300   continue

!     --- Run along the "left" y-axis

      do 400 i = 1 , ny - 1

          kmeshm( 3 + 2 * nx + ny + i ) = ( ny - i ) * nx + 1

400   continue

!     --- All boundary nodenumbers have been placed

      end
