      subroutine mshobsfledge1 ( iobgrid, nx, ny, i1, j1, numcells, &
                                 iedge, ipiwork, iwork, leniwork, &
                                 jpiwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put information of cell i1, j1 in iobgrid, numcells and iwork
!     Only if not present in these arrays
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), i1, j1, numcells(2,*), &
              iedge, ipiwork, leniwork, iwork(2,leniwork), jpiwork

!     i1             i    Index of point with respect to rectangular grid
!     iedge          i    curve part sequence number
!     iobgrid       i/o   Is filled with the number of curve parts that
!                         intersect cell i,j
!     ipiwork       i/o   Starting address in array iwork
!     iwork         i/o   Contains the cell numbers (i,j) in the sequence
!                         of the curve parts
!                         Must be used in connection with numcells
!     j1             i    Index of point with respect to rectangular grid
!     jpiwork        i    Prior value of ipiwork in loop
!     leniwork       i    Declared length of array iwork
!     numcells      i/o   numcells(1,k) contains the number of cells curvepart
!                         k intersects
!                         numcells(1,k) contains the starting address of
!                         the cell numbers in iwork
!     nx             i    number of points of grid in x direction
!     ny             i    number of points of grid in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfledge1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfledge1'
         write(irefwr,1) 'i1, j1', i1, j1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check if i1, j1 is already stored in iwork

      do i = jpiwork, ipiwork
         if ( iwork(1,i)==i1 .and. iwork(2,i)==j1 ) go to 1000 ! skip
      end do  ! i = jpiwork, ipiwork

!     --- Fill number of edges in iobgrid and number of cells in numcells

      iobgrid(i1,j1) = iobgrid(i1,j1)+1 ! number of edges
      numcells(1,iedge) = numcells(1,iedge)+1
      ipiwork = ipiwork+1

!     --- Check array length

      call inichkarlen ( ipiwork, leniwork, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill cell numbers in iwork

      if ( numcells(2,iedge)==0 ) numcells(2,iedge) = ipiwork
      iwork(1,ipiwork) = i1
      iwork(2,ipiwork) = j1

1000  call erclos ( 'mshobsfledge1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'boundary cell:', i1, j1
         write(irefwr,*) 'End mshobsfledge1'

      end if  ! ( debug )

      end
