      subroutine mshobsflperm ( iobgrid, iobgrid1, &
                                nx, ny, leniobgrid1, iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store array iobgrid as permanent array in ibuffr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     memory_management
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, leniobgrid1, iobgrid(0:nx-1,0:ny-1), &
              iobgrid1(leniobgrid1), iobst

!     iobgrid        i    position i,j contains:
!                         -1:  cell inside obstacle
!                          0:  cell outside obstacle
!                         >0:  cell contains part of the obstacle
!                         The value contains a starting address in iobgrid1
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     iobst          i    Obstacle sequence number
!     leniobgrid1    i    Length of array iobgrid1
!     nx             i    number of points in x direction of grid
!     ny             i    number of points in y direction of grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsflperm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsflperm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( km(iseqkm)%kmobst(iobst)%iobgrid2d(0:nx-1,0:ny-1), &
                 km(iseqkm)%kmobst(iobst)%iobgrid2d1(leniobgrid1) )
      kmeshx(5,iobst) = iseqkm
      iobgrid2d => km(iseqkm)%kmobst(iobst)%iobgrid2d
      iobgrid2d1 => km(iseqkm)%kmobst(iobst)%iobgrid2d1
      kmobst => km(iseqkm)%kmobst
      iobgrid2d = iobgrid
      iobgrid2d1 = iobgrid1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsflperm'

      end if  ! ( debug )
      call erclos ( 'mshobsflperm' )

      end
