      subroutine mshobsin31 ( nsubsf, iwork, found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 02-11-2012 Adaptation for spheres
!        version  3.0  date 27-11-2003 Extra parameter found
!        version  2.1  date 27-09-2002 Extension for quadratic elements
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of nodes, elements and number of nodes per element of
!    composite surface isurnr
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsubsf, iwork(inpelm,*)
      logical found

!     found          i    ?
!     iwork          i    ?
!     nsubsf         o    Number of subsurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isubsf, jstart, jsurnr, sumelm
      logical found1

!     found1         ?
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     jstart         starting position
!     jsurnr         Surface sequence number
!     sumelm         ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIDLN         Compute length of array in BUFFER
!     MSHFINDSURF    Refill information in kmesh part x concerning obstacles in
!                    case the obstacle surface has been deformed
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute number of subsurfaces nsubsf
!     if ( nsubsf>1 ) then
!        Check shape number of subsurfaces
!        Compute inpelm
!     else ( nsubsf=1 )
!        Check shape number of surface
!        Compute inpelm
!     Compute nelemlc and npointlc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsin31' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsin31'
         write(irefwr,*) 'found', found
         write(irefwr,1) 'isurnr', isurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      found1 = found

      istart = nsurfs+1 + (isurnr-1)*8
      isurf = kmeshs(istart+1)

      if ( debug ) write(irefwr,1) 'istart, isurf', istart, isurf

      if ( isurf==20 ) isurnr = kmeshs(istart+3)

      if ( debug ) then
         write(irefwr,1) 'isurnr', isurnr
         write(irefwr,1) 'ndim, nsurfs', ndim, nsurfs
      end if  ! ( debug )

!     --- Compute number of subsurfaces

      sumelm=0
      nsubsf = 0
      if ( kelmr>0 ) nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
      if ( debug ) write(irefwr,1) 'nsubsf', nsubsf

      if ( nsubsf>1 ) then

!     --- The surface consists of subsurfaces

         istart = kmeshr(isurnr)
         nelemlc = 0
         do isubsf = 1, nsubsf
            jsurnr = abs(kmeshr(istart+isubsf))
            jstart = nsurfs+1+(jsurnr-1)*8
            ishape = kmeshs(jstart+2)

            if ( ishape==3 ) then

!           --- shape 3, linear triangles

               inpelm = 3

            else if ( ishape==6 ) then

!           --- shape 6, biquadratic quadrilaterals

               inpelm = 9

            else

!           --- shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )

            end if

            found=found1
            call mshfindsurf( jsurnr, sumelm, iwork(1,sumelm+1), found )

         end do

      else

!     --- isurnr=8
!         No subsurfaces

         jstart = nsurfs+1+(isurnr-1)*8
         ishape = kmeshs(jstart+2)
         if ( ishape==3 ) then

!        --- shape 3, linear triangles

            inpelm = 3

         else if ( ishape==6 ) then

!        --- shape 6, linear triangles

             inpelm = 9

         else

!        --- shape number not available

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

         end if

         call mshfindsurf ( isurnr, sumelm, iwork, found )

      end if

      if (found) then

         nelemlc  = sumelm
         npointlc = npoint

      else

!     --- if isurnr is not stored in kmeshn

         npointlc = abs(kmeshh(1,isurnr+1))
         nelemlc = kmeshh(2,isurnr+1)

      end if

1000  call erclos ( 'mshobsin31' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsin31'

      end if  ! ( debug )

      end
