      subroutine mshobsin32 ( iobst, found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-11-2003 Extra parameter found
!        version  1.1  date 27-09-2002 Adaptation for open obstacles
!        version  1.0  date 11-06-2001
!
!   copyright (c) 2001-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in kmesh part x concerning one obstacle
!     3d case, obstacle must be a closed surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst
      logical found

!     found          i    Logical which is true if the obstacle surface is
!                         also part of a volume
!     iobst          i    Obstacle sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision xmin, xmax, ymin, ymax, zmin, zmax, dx, dy, dz, &
                       x, y, z, unit, deltax
      logical closed

!     closed         If true the boundary is closed
!     deltax         maximum unit length of element
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     i              Counting variable
!     j              Counting variable
!     unit           unit length
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTCHSURF  Check if surface of obstacle is closed and if all edges
!                    contain exactly 2 obstacle surface elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Compute xmin, xmax, ymin, ymax, zmin, zmax
!      Check if obstacle is closed or open
!      unit := minimum length in elsize
!      dx := xmax-xmin; dy := ymax-ymin; dz := zmax-zmin;
!      xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!      ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!      zmax := zmax+0.005*dz  zmin := zmin-0.005*dz
!      nx := max(5,4*(xmax-xmin)/unit+1)
!      ny := max(5,4*(ymax-ymin)/unit+1)
!      nz := max(5,4*(zmax-zmin)/unit+1)
!      dx := (xmax-xmin)/nx;  dy := (ymax-ymin)/ny;  dz := (zmax-zmin)/nz
!      Fill information in arrays kmeshx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsin32' )

!     --- unit := minimum length in elsize

      unit = rinfin
      deltax= -rinfin
      do i = 1, ncurvs
         x = elsize(1,i)
         if ( x>0d0 ) unit = min(unit,x)
         if ( x>0d0 ) deltax = max(unit,x)
         x = elsize(2,i)
         if ( x>0d0 ) unit = min(unit,x)
         if ( x>0d0 ) deltax = max(unit,x)
      end do

!     --- Compute xmin, xmax, ymin, ymax, zmin, zmax

      xmin = rinfin
      xmax =-rinfin
      ymin = rinfin
      ymax =-rinfin
      zmin = rinfin
      zmax =-rinfin

      if (found) then

!     --- obstacle surface is part of a volume
!         hence kmeshcobst has been filled using kmeshn

         do j=1, nelemlc
            do i=1, inpelm
               x    = coorobst(1,kmeshcobst(i,j))
               y    = coorobst(2,kmeshcobst(i,j))
               z    = coorobst(3,kmeshcobst(i,j))
               xmin = min(xmin,x)
               xmax = max(xmax,x)
               ymin = min(ymin,y)
               ymax = max(ymax,y)
               zmin = min(zmin,z)
               zmax = max(zmax,z)
            end do
         end do

      else

!     --- obstacle surface is not part of a volume
!         hence kmeshcobst has been filled using kmeshh

         do i = 1, npointlc
            x    = coorobst(1,i)
            y    = coorobst(2,i)
            z    = coorobst(3,i)
            xmin = min(xmin,x)
            xmax = max(xmax,x)
            ymin = min(ymin,y)
            ymax = max(ymax,y)
            zmin = min(zmin,z)
            zmax = max(zmax,z)
         end do

      end if

!     --- dx := xmax-xmin; dy := ymax-ymin

      if (abs(xmax-xmin)<1d-10) then
         deltax=deltax*0.5d0
         xmin=xmin-deltax
         xmax=xmax+deltax
      end if

      if (abs(ymax-ymin)<1d-10) then
         deltax=deltax*0.5d0
         ymin=ymin-deltax
         ymax=ymax+deltax
      end if

      if (abs(zmax-zmin)<1d-10) then
         deltax=deltax*0.5d0
         zmin=zmin-deltax
         zmax=zmax+deltax
      end if

!     --- dx := xmax-xmin; dy := ymax-ymin; dz := zmax-zmin

      dx = xmax-xmin
      dy = ymax-ymin
      dz = zmax-zmin

!     --- Check if obstacle is closed

      call mshobstchsurf ( kmeshcobst, nelemlc, npointlc, inpelm, closed )
      if ( ierror/=0 ) go to 1000

!     --- xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!         ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!         zmax := zmax+0.005*dz  zmin := zmin-0.005*dz
!         nx := max(5,4*(xmax-xmin)/unit+1)
!         ny := max(5,4*(ymax-ymin)/unit+1)
!         nz := max(5,4*(zmax-zmin)/unit+1)
!         dx := (xmax-xmin)/nx; dy := (ymax-ymin)/ny; dz := (zmax-zmin)/nz

      xmax = xmax+0.005d0*dx
      xmin = xmin-0.005d0*dx
      ymax = ymax+0.005d0*dy
      ymin = ymin-0.005d0*dy
      zmax = zmax+0.005d0*dz
      zmin = zmin-0.005d0*dz
      dx   = xmax-xmin
      dy   = ymax-ymin
      dz   = zmax-zmin
      nx   = min(max(5,int(1.1d0*dx/unit+1d0)),100)
      ny   = min(max(5,int(1.1d0*dy/unit+1d0)),100)
      nz   = min(max(5,int(1.1d0*dz/unit+1d0)),100)
      dx   = dx/nx
      dy   = dy/ny
      dz   = dz/nz

!     --- Fill information in arrays kmeshx

      rmeshx(1,iobst) = xmin
      rmeshx(2,iobst) = xmax
      rmeshx(3,iobst) = ymin
      rmeshx(4,iobst) = ymax
      rmeshx(5,iobst) = zmin
      rmeshx(6,iobst) = zmax
      rmeshx(7,iobst) = dx
      rmeshx(8,iobst) = dy
      rmeshx(9,iobst) = dz
      if ( closed ) then
         kmeshx(1,iobst) = 2
      else
         kmeshx(1,iobst) = -2
      end if
      kmeshx(2,iobst) = isurnr
      kmeshx(3,iobst) = nx
      kmeshx(4,iobst) = ny
      kmeshx(5,iobst) = nz
      kmeshx(6,iobst) = 0
      kmeshx(7,iobst) = 0
      kmeshx(10,iobst) = inpelm
      kmeshx(11,iobst) = nelemlc
      kmeshx(12,iobst) = npointlc

!     --- The filling of kmeshxa has not yet been implemented

1000  call erclos ( 'mshobsin32' )
      end
