      subroutine mshobsin3d ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 15-11-2012 Long integers
!        version  3.1  date 02-11-2012 Change in debug
!        version  3.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in kmesh part x concerning obstacles
!     3d case, obstacle must be a closed surface
!     Remark: this subroutine uses mm routines
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    integer input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, lenin, leng, nsubsf, idum(1), i
      logical new, found

!     found          Logical which is true if the obstacle surface is
!                    also part of a volume
!     i              General loop variable
!     idum           General dummy variable
!     iobst          Obstacle sequence number
!     leng           Help variable to compute the length of some arrays
!     lenin          Number of integers in one row of kmeshx
!     new            indication whether the mesh is new (true) or not
!                    (false)
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMKTRIANG    Make a triangulation in case surface is of ishape 6
!     MSHOBSIN31     Compute number of nodes, elements and number of nodes per
!                    element of composite surface isurnr
!     MSHOBSIN32     Fill information in kmesh part x concerning one obstacle,3D
!     MSHRDCOMPSF    Read information of subsurfaces in general coordinates
!                    and topology array of composite surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear arrays kmeshx and rmeshx
!   For all obstacles do
!      Compute inpelm, nelemlc and npointlc
!      Store coordinates of nodes
!      Store elements
!      Compute xmin, xmax, ymin, ymax, zmin, zmax etc
!      Fill information in arrays kmeshx and rmeshx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsin3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsin3d'
  1      format ( a, 1x, (10i6) )

      end if

!     --- initializations

      new = .not. associated(km(iseqkm)%kmeshx)
      if ( debug ) &
         write(irefwr,1) 'iinp14, ndim, nsurfs, numobst', &
                          iinp14, ndim, nsurfs, numobst

!     --- create arrays kmeshx and rmeshx

      lenin = 12
      leng = 9*numobst
      if ( debug ) write(irefwr,1) 'leng', leng

      if ( new ) &
         allocate ( km(iseqkm)%kmeshx(12,numobst), &
                    km(iseqkm)%rmeshx(9,numobst), &
                    km(iseqkm)%kmobst(numobst) )
      kmeshx => km(iseqkm)%kmeshx
      rmeshx => km(iseqkm)%rmeshx

!     --- Clear arrays kmeshx and rmeshx

      if ( new ) then

!     --- new mesh

         kmeshx = 0
         rmeshx = 0d0

      end if

!     --- Loop over obstacles

      do iobst = 1, numobst

!     --- Compute inpelm, nelemlc and npointlc

         isurnr = iinput(iinp14+2*iobst)

         found = .false.
         inpelm = 1
         call mshobsin31 ( nsubsf, idum, found )

         if ( debug ) then
            write(irefwr,1) 'iobst, isurnr, inpelm, nelemlc, npointlc', &
                             iobst, isurnr, inpelm, nelemlc, npointlc
            write(irefwr,1) 'ishape, nsubsf', ishape, nsubsf
            write(irefwr,*) 'found ', found
         end if

!        --- Store coordinates of nodes
!            Store elements
!            First reserve space

         kmobst => km(iseqkm)%kmobst
         if ( ishape==3 ) then
            allocate ( kmobst(iobst)%kmeshcobst(3,nelemlc) )
         else if ( ishape==6 ) then
            allocate ( kmobst(iobst)%kmeshcobst(3,nelemlc*8) )
         end if
         kmeshx(8,iobst) = 3*nelemlc
         kmeshcobst => kmobst(iobst)%kmeshcobst

         if ( .not.found ) then
            allocate ( kmobst(iobst)%coorobst(3,npointlc) )
            kmeshx(9,iobst) = npointlc*ndim
         end if
         coorobst => kmobst(iobst)%coorobst

         if ( nsubsf>1 ) then

!        --- The obstacle surface consists of subsurfaces

            if ( found ) then

!           --- The obstacle surface is part of a volume
!               hence, kmeshn can be used to fill kmeshc_obst

               leng = nelemlc*inpelm
               nullify(kmeshclc)
               allocate ( kmeshclc(inpelm*nelemlc), stat = error )
               if ( error/=0 ) call eralloc ( error, leng, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

!              --- fill array kmeshc_obst

               call mshobsin31 ( nsubsf, kmeshclc, found )

               if ( ishape==6 ) then

!              --- surface consists of quads -> make a triangulation

                  call mshmktriang ( kmeshcobst, kmeshclc, nelemlc, inpelm )

               else

!              --- ishape = 3

                  do i = 1, 3
                     kmeshcobst(i,1:nelemlc) = kmeshclc(i:nelemlc*3:3)
                  end do  ! i = 1, 3

               end if

               deallocate ( kmeshclc, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

               kmeshx(9,iobst) = -npoint*ndim
               coorobst => coor

            else

!           --- The obstacle surface is not part of a volume
!               hence, kmeshh is used to fill kmeshc_obst
!               Reserve work space:

               allocate ( irenum(npointlc+nelemlc), stat = error )
               if ( error/=0 ) call eralloc ( error, npointlc, 'irenum' )
               if ( ierror/=0 ) go to 1000

               nullify (kmeshclc)
               allocate ( kmeshclc(inpelm*nelemlc), stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, nelemlc*inpelm, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

               call mshrdcompsf ( coorobst, irenum, isurnr, nsubsf )

               deallocate ( irenum, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'irenum' )
               if ( ierror/=0 ) go to 1000

               if ( ishape==6 ) then

!              --- surface consists of quads -> make a triangulation

                  call mshmktriang ( kmeshcobst, kmeshclc, &
                                     nelemlc, inpelm )

               else

!              --- ishape = 3

                  do i = 1, 3
                     kmeshcobst(i,1:nelemlc) = kmeshclc(i:nelemlc*3:3)
                  end do  ! i = 1, 3

               end if

               deallocate ( kmeshclc, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

            end if  ! ( found )

         else

!        --- The surface does not contains subsurfaces

            if ( found ) then

!           --- The obstacle surface is part of a volume
!               hence, kmeshn can be used to fill kmeshc_obst

               nullify (kmeshclc)
               allocate ( kmeshclc(inpelm*nelemlc), stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, nelemlc*inpelm, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

!              --- fill array kmeshc_obst

               call mshobsin31 ( nsubsf, kmeshclc, found )

               if ( ishape==6 ) then

!              --- surface consists of quads -> make a triangulation

                  call mshmktriang ( kmeshcobst, kmeshclc, &
                                     nelemlc, inpelm )

               else

!              --- ishape = 3

                  do i = 1, 3
                     kmeshcobst(i,1:nelemlc) = kmeshclc(i:nelemlc*3:3)
                  end do  ! i = 1, 3

               end if

               deallocate ( kmeshclc, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

               kmeshx(9,iobst) = -npoint*ndim
               coorobst => coor

            else

!           --- The obstacle surface is not part of a volume
!               hence, kmeshh is used to fill kmeshc_obst

!               Read information from kmesh part h

!              --- Copy coordinates

               call dcopy ( ndim*npointlc, kmhsur(isurnr)%coorsur, &
                            1, coorobst, 1 )

!              --- Copy topology

               nullify (kmeshclc)
               allocate ( kmeshclc(inpelm*nelemlc), stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, nelemlc*inpelm, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

               leng = inpelm*nelemlc
               kmeshclc(1:leng) = kmhsur(isurnr)%kmeshcsur(1:leng)

               if ( ishape==6 ) then

!              --- surface consists of quads -> make a triangulation

                  call mshmktriang ( kmeshcobst, kmeshclc, nelemlc, inpelm )

               else

!              --- ishape = 3

                  do i = 1, 3
                     kmeshcobst(i,1:nelemlc) = kmeshclc(i:nelemlc*3:3)
                  end do  ! i = 1, 3

               end if

               deallocate ( kmeshclc, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'kmeshclc' )
               if ( ierror/=0 ) go to 1000

            end if  ! ( found )

         end if  ! ( nsubsf>1 )

!        --- Compute xmin, xmax, ymin, ymax, zmin, zmax etc
!            Fill information in arrays kmeshx and rmeshx

         call mshobsin32 ( iobst, found )

      end do  ! iobst = 1, numobst

1000  if ( debug ) write(irefwr,*) 'End of mshobsin3d'
      call erclos ( 'mshobsin3d' )

      end subroutine mshobsin3d
