      subroutine mshobstac2 ( iobst, nspecialpnts, &
                              nskip, specialpoints, isequence, icurvobs, &
                              newcurvesstrt, nextracurves, &
                              newcurves, istnewcurves, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find position and subcurve number on obstacle for all special points
!   Subdivide subcurves on obstacle
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, nspecialpnts, nskip, &
              specialpoints(7,*), isequence(*), icurvobs(2,*), &
              newcurvesstrt, nextracurves, newcurves(*), iwork(*), &
              istnewcurves(*)

!     icurvobs       i    Array indicating for each point at the boundary
!                         whether it is positioned at an obstacle (>0) or not
!                         (0)
!                         icurvobs must have been filled before
!                         If icurvobs(k) = j (j>0), j denotes the sequence
!                         number of the obstacle
!     iobst          i    Obstacle sequence number
!     isequence      i    ?
!     istnewcurves   i    Array of length ncurvs containing the starting
!                         addresses of all curves in array NEWCURVES
!                         if istnewcurves(i)=0, no information of the curve is
!                         stored, hence the curve is not changed
!                         if istnewcurves(i)>0, the value is the starting
!                         address of curve i
!     iwork          i    integer work array
!     newcurves      i    Contains information about the new curves to be
!                         created by splitting old curves
!                         The starting addresses of the information per curve is
!                         stored in array istnewcurves
!                         For each curve that is stored in newcurves nparts+1
!                         positions are used, where nparts is the number of
!                         parts the curve is subdivided into
!                         The first position contains nparts, the following
!                         nparts positions the relative starting addresses
!                         of the parts in the curve
!     newcurvesstrt  i    Defines the next free position in newcurves
!     nextracurves   i    Number of extra curves to be created
!     nskip          i    Number of points that must be skipped
!     nspecialpnts   i    Number of special points
!     specialpoints  i    This array of size 3 x nspecialpnts contains
!                         information about the special points on the curve
!                         specialpoints(1,i) contains the position in curves
!                                           of special point i
!                         specialpoints(2,i) contains first curve number in the
!                                           free surface connected to point i
!                         specialpoints(3,i) contains second curve number in the
!                                           free surface connected to point i
!                                           If 0, there is only one curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nparts, istartsubcv, isub, isubcv, inext, ifirst, &
              ilast, nsubcv, icurnr, indobst, indspecialpnt, inpncv, &
              iseq, istartobs, jsubcv, istart
      double precision eps, x, y, x0, y0, x1, y1, alpha
      logical obstacle, repeat

!     alpha
!     eps            Local accuracy
!     i              Counting variable
!     icurnr         Curve sequence number
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     indobst        ?
!     indspecialpnt  ?
!     inext          Next point in curve
!     inpncv         Number of points in curve
!     iseq           Sequence number
!     istart         First position used at a part
!     istartobs      Starting address in array icurvs of obstacle curve
!     istartsubcv    ?
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isubcv         Subcurve sequence number with direction
!     jsubcv         |isubcv|
!     nparts         number of parts the boundary consists of
!     nsubcv         Number of subcurves of a curve
!     obstacle       if true an obstacle has been found
!     repeat         If true the process has not yet been finished
!     x              x co-ordinate of point
!     x0             ?
!     x1             x-coordinate of first point
!     y              y co-ordinate of point
!     y0             ?
!     y1             y-coordinate of first point in curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2105   Free surface intersects first element of obstacle
!    2106   Obstacle curve number < 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for i := 1 (1) nspecialpnts-nskip do
!        Find relative position of special point i in obstacle
!        Locate exact position of point in obstacle
!        Store both in istore and rstore
!     nsubcv := number of subcurves of obstacle curve
!     if ( nsubcv=1 ) then
!        Store information in newcurves, istnewcurves and nextracurves
!        if ( point near a point on the obstacle ) then
!           split subcurve into 2 curves
!           adapt coordinates of point on obstacle in array curves
!        else
!           split subcurve into 2 curves
!           make a reordening of the 2 subcurves and adapt array curves
!     else ( nsubcv>1 )
!        for i := 1 (1) nspecialpnts-nskip do
!        Find corresponding subcurve number
!        if ( point near a point on the obstacle ) then
!           split subcurve into 2 curves
!           adapt coordinates of point on obstacle in array curves
!        else
!           split subcurve into 2 curves
!           make a reordening of the 2 subcurves and adapt array curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstac2' )
      icurnr = kmeshx(2,iobst)
      if ( icurnr<=0 ) then

!     --- icurnr < 0

         call errint ( icurnr,1 )
         call errsub ( 2106, 1, 0, 0 )

      end if
      if ( icurnr==1 ) then

!     --- icurnr = 1, first curve

         istartobs = 0

      else

!     --- icurnr > 1

         istartobs = icurvsmsh(icurnr-1)

      end if
      inpncv = icurvsmsh(icurnr)-istartobs
      eps = sqreps* &
               sqrt((curvesmsh(1,istartobs+2)-curvesmsh(1,istartobs+1))**2+ &
                    (curvesmsh(2,istartobs+2)-curvesmsh(2,istartobs+1))**2)

!     --- for i := 1 (1) nspecialpnts do

      obstacle = .true.
      do i = 1, nspecialpnts-nskip

!     --- Find relative position of special point i in obstacle

         iseq = isequence(i)

!        --- Compute x

         indspecialpnt = specialpoints(1,iseq)
         indobst = icurvobs(2,indspecialpnt)
         if ( ishape==2 .and. mod(indobst,2)==0 ) indobst=indobst-1
         x = curvesmsh(1,indspecialpnt)
         y = curvesmsh(2,indspecialpnt)
         repeat = .true.
100      x0 = curvesmsh(1,istartobs+indobst)
         y0 = curvesmsh(2,istartobs+indobst)
         x1 = curvesmsh(1,istartobs+indobst+ishape)
         y1 = curvesmsh(2,istartobs+indobst+ishape)

!        --- Compute alpha
!            work(i) := alpha

         if ( abs(x1-x0)>abs(y1-y0) ) then
            alpha = (x-x0)/(x1-x0)
         else
            alpha = (y-y0)/(y1-y0)
         end if
         if ( alpha<-eps ) then

!        --- point in previous part

            if ( indobst>1 .and. repeat ) then

!           --- indobst > 1, decrease

               indobst = indobst-ishape
               repeat = .false.
               go to 100

            end if

         else if ( alpha>1d0+eps ) then

!        --- point in next part

            if ( indobst<inpncv .and. repeat ) then

!           --- indobst > 1, increase

               indobst = indobst+ishape
               repeat = .false.
               go to 100

            end if

         end if
         if ( alpha<0.25d0 ) then

!        --- alpha < 0.25d0, move first node indobst to cross section

            curvesmsh(1,istartobs+indobst) = x
            curvesmsh(2,istartobs+indobst) = y

         else if ( alpha>0.75d0 ) then

!        --- alpha < 0.25d0, move first node indobst+1 to cross section

            indobst = indobst+ishape
            curvesmsh(1,istartobs+indobst) = x
            curvesmsh(2,istartobs+indobst) = y

         else

!        --- alpha in middle of element
!            if first node of obstacle than use end node

            if ( obstacle ) then

!           --- start of obstacle

               indobst = indobst+ishape
               curvesmsh(1,istartobs+indobst) = x
               curvesmsh(2,istartobs+indobst) = y
               if ( specialpoints(3,iseq)/=0 ) obstacle = .false.

            else

!           --- end of obstacle, use start point unless already used

               if ( iwork(i-1)==indobst ) indobst = indobst+ishape
               curvesmsh(1,istartobs+indobst) = x
               curvesmsh(2,istartobs+indobst) = y
               obstacle = .true.

            end if

         end if
         if ( ishape==2 ) then

!        --- quadratic elements, adapt mid points

            if ( indobst>1 ) then
               curvesmsh(1,istartobs+indobst-1) = 0.5d0* &
                 (curvesmsh(1,istartobs+indobst-2)+x)
               curvesmsh(2,istartobs+indobst-1) = 0.5d0* &
                 (curvesmsh(2,istartobs+indobst-2)+y)
            end if
            if ( indobst<inpncv ) then
               curvesmsh(1,istartobs+indobst+1) = 0.5d0* &
                 (curvesmsh(1,istartobs+indobst+2)+x)
               curvesmsh(2,istartobs+indobst+1) = 0.5d0* &
                 (curvesmsh(2,istartobs+indobst+2)+y)
            end if
         end if
         iwork(i) = indobst

      end do

      if ( iwork(1)==1 ) then

!     --- First element of obstacle contains special point
!         Not yet implemented

         call errint ( iobst, 1 )
         call erreal ( alpha, 1 )
         call errchr ( 'first', 1 )
         call errsub ( 2105, 1, 1, 1 )

      end if
      if ( iwork(nspecialpnts-nskip)==inpncv ) then

!     --- Last element of obstacle contains special point
!         Not yet implemented

         call errint ( iobst, 1 )
         call erreal ( alpha, 1 )
         call errchr ( 'last', 1 )
         call errsub ( 2105, 1, 1, 1 )

      end if
      if ( ierror/=0 ) go to 1000
      if ( kelmq==0 ) then

!     --- Curve does not contain subcurves
!         Fill information in newcurves

         istnewcurves(icurnr) = newcurvesstrt
         nparts = nspecialpnts-nskip+1
         newcurves(newcurvesstrt) = nparts
         newcurves(newcurvesstrt+1) = 1
         newcurves(newcurvesstrt+2) = 0
         do i = 1, nspecialpnts-nskip
            indobst = iwork(i)
            newcurves(newcurvesstrt+2*i+1) = indobst
            newcurves(newcurvesstrt+2*i+2) = isequence(i)
         end do
         newcurvesstrt = newcurvesstrt+nparts*2+1
         nextracurves = nextracurves+nparts

      else

!     --- Curve contains subcurves
!         Detect the number of points in each subcurve
!         Fill information in newcurves

         istartsubcv = kmeshq(icurnr)
         inext = 1
         nsubcv = kmeshq(icurnr+1)-istartsubcv
         ifirst = 1
         do isub = 1, nsubcv
            isubcv = kmeshq(istartsubcv+isub)
            jsubcv = abs(isubcv)
            if ( jsubcv==1 ) then
               inpncv = icurvsmsh(jsubcv)
            else
               inpncv = icurvsmsh(jsubcv)-icurvsmsh(jsubcv-1)
            end if
            ilast = ifirst+inpncv-1
            if ( iwork(inext)<=ilast ) then

!           --- Curve contains at least one special point

               if ( isubcv>0 ) then
                  istnewcurves(isubcv) = newcurvesstrt
               else
                  istnewcurves(-isubcv) = -newcurvesstrt
               end if
               nparts = 0
               if ( iwork(inext)==ifirst ) then
                  newcurves(newcurvesstrt+1) = 1
                  newcurves(newcurvesstrt+2) = isequence(inext)
                  nparts = 1
               else
                  newcurves(newcurvesstrt+1) = 1
                  newcurves(newcurvesstrt+2) = 0
                  newcurves(newcurvesstrt+3) = iwork(inext)-ifirst+1
                  newcurves(newcurvesstrt+4) = isequence(inext)
                  nparts = 2
               end if
200            inext = inext+1
               if ( inext>nspecialpnts-nskip ) then

!              --- Last point has been treated, finish

                  if ( isubcv>0 ) then
                     istnewcurves(isubcv) = newcurvesstrt
                  else
                     istnewcurves(-isubcv) = -newcurvesstrt
                  end if
                  newcurves(newcurvesstrt) = nparts
                  newcurvesstrt = newcurvesstrt+nparts*2+1
                  nextracurves = nextracurves+nparts
                  go to 300

               end if
               if ( iwork(inext)<=ilast ) then

!              --- Curve contains more special points

                  nparts = nparts+1
                  newcurves(newcurvesstrt+2*nparts-1) = &
                     iwork(inext)-ifirst+1
                  newcurves(newcurvesstrt+2*nparts) = &
                     isequence(inext)
                  if ( iwork(inext)<ilast ) then
                     go to 200
                  else
                     if ( isubcv>0 ) then
                        istnewcurves(isubcv) = newcurvesstrt
                     else
                        istnewcurves(-isubcv) = -newcurvesstrt
                     end if
                     newcurves(newcurvesstrt) = nparts
                     newcurvesstrt = newcurvesstrt+nparts*2+1
                     nextracurves = nextracurves+nparts
                  end if

               else

!              --- curve does not end in special point

                  if ( isubcv>0 ) then
                     istnewcurves(isubcv) = newcurvesstrt
                  else
                     istnewcurves(-isubcv) = -newcurvesstrt
                  end if
                  newcurves(newcurvesstrt) = nparts
                  newcurvesstrt = newcurvesstrt+nparts*2+1
                  nextracurves = nextracurves+nparts

               end if

            end if
            ifirst = ilast

         end do

!        --- Copy curves from composite curve to subcurves

300      do isub = 1, nsubcv
            isubcv = kmeshq(istartsubcv+isub)
            jsubcv = abs(isubcv)
            if ( jsubcv==1 ) then
               inpncv = icurvsmsh(jsubcv)
               istart = 0
            else
               istart = icurvsmsh(jsubcv-1)
               inpncv = icurvsmsh(jsubcv)-istart
            end if
            if ( isubcv>0 ) then
               do i = 1, inpncv
                  curvesmsh(1,istart+i) = curvesmsh(1,istartobs+i)
                  curvesmsh(2,istart+i) = curvesmsh(2,istartobs+i)
               end do
            else
               do i = 1, inpncv
                  curvesmsh(1,istart+inpncv-i+1) = curvesmsh(1,istartobs+i)
                  curvesmsh(2,istart+inpncv-i+1) = curvesmsh(2,istartobs+i)
               end do
            end if
            istartobs = istartobs+inpncv-1

         end do

      end if

1000  call erclos ( 'mshobstac2' )
      end
