      subroutine mshobstac4 ( isequence, istart, iend, specialpoints, &
                              icurvobs, curves, icurnr, work, iwork, &
                              icurvs  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Sort part of array istore with equal values contents
!   The sorting is done by comparing the coordinates
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isequence(*), istart, iend, specialpoints(6,*), &
              icurvobs(2,*), iwork(*), icurvs(*), icurnr
      double precision curves(2,*), work(*)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, i, indspecialpnt, istartobs
      double precision x, x0, x1, y, y0, y1, alpha
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The positions istart to iend of isequence correspond to equal values
!    of istore
!    The coordinates of the 2 end points of the interval (x0 and x1) are
!    given by curvesnw(.,indspecialpnt) and curvesnw(.,indspecialpnt+1), with
!    indspecialpnt = istartobs+icurvobs(2,specialpoints(1,i))
!
!    The points between are given by:
!
!    x = x0 + alpha ( x1-x0 )
!
!    The value of x is given curves(.,specialpoints(1,i))
!    The points must be sorted for increasing alpha values
!
!    Compute x0 and x1
!    For i := istart(1) iend do
!       Compute x
!       Compute alpha
!       work(i) := alpha
!    Sort array alpha in increasing sequence
!    Adapt array isequence
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute x0 and x1

      iseq = isequence(istart)
      indspecialpnt = specialpoints(1,iseq)
      indspecialpnt = icurvobs(2,indspecialpnt)
      if ( icurnr.eq.1 ) then

!     --- icurnr = 1, first curve

         istartobs = 0

      else

!     --- icurnr > 1

         istartobs = icurvs(icurnr-1)

      end if
      x0 = curves(1,istartobs+indspecialpnt)
      y0 = curves(2,istartobs+indspecialpnt)
      x1 = curves(1,istartobs+indspecialpnt+1)
      y1 = curves(2,istartobs+indspecialpnt+1)

!     --- For i := istart(1) iend do

      do i = istart, iend
         iseq = isequence(i)

!        --- Compute x

         indspecialpnt = specialpoints(1,iseq)
         x = curves(1,indspecialpnt)
         y = curves(2,indspecialpnt)

!        --- Compute alpha
!            work(i) := alpha

         if ( abs(x1-x0).gt.abs(y1-y0) ) then
            alpha = (x-x0)/(x1-x0)
         else
            alpha = (y-y0)/(y1-y0)
         end if
         work(i-istart+1) = alpha

      end do

!     --- Sort array alpha in increasing sequence

      call prsort ( work, iwork, iend-istart+1 )

!     --- Adapt array isequence

      do i = istart, iend
         work(i-istart+1) = isequence(i)
      end do
      do i = istart, iend
         isequence(i) = nint(work(iwork(i-istart+1)))
      end do
      end
