      subroutine mshobstms10 (  kmeshcnew, kmeshnnew, &
                               ipointobstnew, ielemobstnew, &
                               kmeshmnew, coornew, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 19-09-2013 New parameter list
!        version  6.0  date 22-11-2011 Complete new subroutine
!        version  5.0  date 20-09-2006 Extra parameter ipkmsmnew
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check the control integers of the work arrays defined in mshobstms
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     check
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(0:*), kmeshnnew(0:*), iinput(*), &
              ipointobstnew(0:*), ielemobstnew(0:*), &
              kmeshmnew(0:*)
      double precision coornew(0:*)

!     coornew        i    Array containing the new coordinates of the nodes
!     ielemobstnew   i    New array ielemobst
!     iinput         i    integer input array
!                         See subroutine mshobstms23d
!     ipointobstnew  i    New array ipointobst
!     kmeshcnew      i    New kmesh part c
!     kmeshmnew      i    New array kmeshm
!     kmeshnnew      i    New array kmeshn
!     lencoornew     i    Estimated length of array coornew
!     lenielemobst   i    Computed length of array ielemobst
!     lenipointobst  i    length of array ipointobst
!     lenkmscnew     i    Length of array kmshcnew
!     lenkmsmnew     i    Length of array kmshmnew
!     lenkmsnnew     i    Length of array kmshnnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmscnew, lenkmsnnew, &
              lenkmsmnew, lencoornew, &
              lenipointobst, lenielemobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRLEADING     Check leading control integer in case of temporary array
!     ERRLEADINGD    Check leading control integer in case of real array
!     ERRTRAILING    Check trailing control integer in case of temporary array
!     ERRTRAILINGD   Check trailing control integer in case of real array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms10' )

      lenkmsmnew = iinput(7)
      lenkmsnnew = iinput(8)
      lenkmscnew = iinput(12)
      lencoornew = iinput(14)
      lenipointobst = iinput(15)
      lenielemobst = iinput(17)

      call errleading  ( kmeshcnew(0), 10002, 'kmscnew' )
      call errtrailing ( kmeshcnew(lenkmscnew+1), &
                         10002, 'kmscnew' )

      call errleading  ( kmeshnnew(0), 10003, 'kmsnnew' )
      call errtrailing ( kmeshnnew(lenkmsnnew+1), 10003, 'kmsnnew' )

      call errleadingd  ( coornew(0), 10006, 'coornew' )
      call errtrailingd ( coornew(lencoornew+1), 10006, 'coornew' )

      call errleading  ( ipointobstnew(0), 10008, 'ipointobstnew' )
      call errtrailing ( ipointobstnew(lenipointobst+1), 10008, &
                         'ipointobstnew' )

      call errleading  ( ielemobstnew(0), 10009, 'ielemobstnew' )
      call errtrailing ( ielemobstnew(lenielemobst+1), 10009, &
                         'ielemobstnew' )

      call errleading  ( kmeshmnew(0), 10011, 'kmsmnew' )
      call errtrailing ( kmeshmnew(lenkmsmnew+1), 10011, 'kmsmnew' )

      call erclos ( 'mshobstms10' )
      end
