      subroutine mshpartcirc ( iinput, iwork1, iinp, rinput, curves, &
                               ishape, icurnr, icurvs, inpncv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a circle into 4 parts, each approximately 90 degrees
!     The circle itself already contains elements
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iwork1(*), iinp, ishape, inpncv, icurnr, &
              icurvs(*)
      double precision rinput(3,*), curves(3,*)

!     curves        i/o   array containing the coordinates of the curves
!     icurnr         i    Curve sequence number
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     inpncv         i    Number of points in curve
!     ishape         i    Shape number of elements
!     iwork1        i/o   integer work array
!     rinput        i/o   Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision prodbefore, xc(3), tang1(3), tang2(3), produc, &
                       eps
      integer i, j, ivert(5), icentre, icurnrorig, icount, iinplocal, &
              ifirst, istartcircle, istartpart, iuserp, nodp, k

!     eps            Local accuracy
!     i              Counting variable
!     icentre        User point number of centroid
!     icount         number of unknowns per element
!     icurnrorig     Curve number of originating curve
!     ifirst         First point for a part
!     iinplocal      Starting address of iinput for local curve
!     istartcircle   Starting address of circle in curves
!     istartpart     Starting address of part of circle in curves
!     iuserp         User point sequence number
!     ivert          Contains the relative position of the 4 vertices in the
!                    circle
!     j              Counting variable
!     k              Counting variable
!     nodp           Nodal point sequence number
!     prodbefore     Previous value of inner product
!     produc         Inner product between two tangential vectors
!     tang1          First tangential vector
!     tang2          Second tangential vector
!     xc             array with the coordinates of the centre point of the
!                    element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2090   coordinates of user point incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Find 4 points on circle that subdivide the circle approximately into
!    4 equal parts
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpartcirc' )

!     --- Find 4 points on circle that subdivide the circle approximately into
!         4 equal parts, store 4 numbers in ivert
!         Start with some initializations

      icount = 1
      prodbefore = 0d0
      iinplocal = iwork1(icurnr)-1
      icurnrorig = iinput(iinplocal+4)
      icentre = iinput(iinplocal+8)
      ifirst = iinput(iinplocal+6)
      if ( icurnrorig.gt.1 ) then
         inpncv = icurvs(icurnrorig)-icurvs(icurnrorig-1)
         istartcircle = icurvs(icurnrorig-1)
      else
         inpncv = icurvs(icurnrorig)
         istartcircle = 0
      end if
      istartpart = icurvs(icurnr-1)

!     --- Store the coordinates of the centre in xc and create first tangential
!         vector

      do i = 1, 3
         xc(i) = rinput(i,icentre)
         tang1(i) = rinput(i,ifirst)-xc(i)
      end do
      ivert(1) = 1
      do j = 2, inpncv, ishape
         do i = 1, 3
            tang2(i) = curves(i,j+istartcircle)-xc(i)
         end do
         produc = tang1(1)*tang2(1)+tang1(2)*tang2(2)+tang1(3)*tang2(3)
         if ( icount.eq.1 ) then

!        --- First quarter

            if ( produc.le.0d0 ) then

!           --- negative inner product has been reached, hence quarter found

               if ( abs(prodbefore).le.abs(produc) ) then
                  ivert(2) = j-1
               else
                  ivert(2) = j
               end if
               icount = 2

            end if

         else if ( icount.eq.2 ) then

!        --- Second quarter

            if ( produc.ge.prodbefore ) then

!           --- local maximum has been reached, hence quarter found

               ivert(3) = j-1
               icount = 3

            end if

         else

!        --- Third quarter

            if ( produc.ge.0d0 ) then

!           --- positive inner product has been reached, hence quarter found

               if ( abs(prodbefore).le.abs(produc) ) then
                  ivert(4) = j-1
               else
                  ivert(4) = j
               end if
               go to 100

            end if

         end if
         prodbefore = produc

      end do

!     --- 4 vertices have been found, now rinput and icurvs can be filled

100   ivert(5) = inpncv
      do i = 1, 3
         iuserp = iinput(iinplocal+6+8*i)
         nodp = ivert(i+1)
         do j = 1, 3
            xc(j) = curves(j,istartcircle+nodp)
         end do
         eps = sqreps*max(abs(xc(1)),abs(xc(2)),abs(xc(3)))
         if ( abs(rinput(1,iuserp)).le.epsmac .and. &
              abs(rinput(3,iuserp)).le.epsmac .and. &
              abs(rinput(3,iuserp)).le.epsmac ) then

!        --- user point has not yet been filled

            do j = 1, 3
               rinput(j,iuserp) = xc(j)
            end do

         else if ( abs(rinput(1,iuserp)-xc(1)).gt.eps .or. &
                   abs(rinput(2,iuserp)-xc(2)).gt.eps .or. &
                   abs(rinput(3,iuserp)-xc(3)).gt.eps ) then

!        --- user point has been filled, coordinates incorrect

            call errint ( iuserp, 1 )
            call errint ( i+1, 2 )
            call errint ( icurnr, 3 )
            call erreal ( xc(1), 1 )
            call erreal ( xc(2), 2 )
            call erreal ( xc(3), 3 )
            call erreal ( rinput(1,iuserp), 4 )
            call erreal ( rinput(2,iuserp), 5 )
            call erreal ( rinput(3,iuserp), 6 )
            call errsub ( 2090, 3, 6, 0 )

         end if

      end do

!     --- Fill array icurvs correctly

      do i = 1, 3
         icurvs(icurnr+i-1) = icurvs(icurnr+i-2)+ivert(i+1)-ivert(i)+1
      end do

!     --- Copy coordinates of circle to new positions in curves

      do i = 1, 4
         do k = ivert(i), ivert(i+1)
            do j = 1, 3
               curves(j,istartpart+i-1+k) = curves(j,istartcircle+k)
            end do
         end do
      end do

!     --- Fill nelm for 4 curves

      do i = 1, 4
         iinput(iinplocal+3+(i-1)*8) = ivert(i+1)-ivert(i)
      end do

      iinp = iinp+4
      call erclos ( 'mshpartcirc' )
      end
