      subroutine mshpsr ( curv1, curv2, curv3, kbndpt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  7.2  date 29-05-2014 Extension of timebefore
!        version  7.1  date 22-05-2014 Debug statements
!        version  7.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Generate elements on pipesurface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*)
      double precision curv1(*), curv2(*), curv3(*)

!     curv1
!     curv2
!     curv3
!     kbndpt     i    array containing the nodal point numbers of the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idiag_orig, interpol
      double precision :: timebefore(2)

!     idiag_orig     Defines diagonal direction:
!                    0: standard (counter clockwise)
!                    1: clockwise
!                    2: alternating
!     interpol       Defines the type of interpolation
!                    0: default
!                    1: straight
!                    2: hor_straight
!                    3: vert_straight
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHD7          Places coorindates of centroids of 7-point triangles
!                    in array coor
!     MSHP00
!     MSHP04
!     MSHP05
!     MSHP06
!     MSHTRIANCENTR  Create centre point in 6-point triangles
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpsr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpsr'
         write(irefwr,1) 'ishape, inpelm', ishape, inpelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Check if elements are linear or quadratic

      if ( inpelm==3 .or. inpelm==4 ) then
         linear=1
      else
         linear=2
      end if
      idiag_orig = mod(idiag,10)
      interpol = idiag/10

!     --- Check curv arrays

      call mshp00 ( curv1, curv2, curv3, curv3(ndim*(nmes*linear+1)+1), &
                    linear, mmes, nmes )

!     --- Fill array coor

      call mshp04 ( curv1, curv2, curv3, curv3(ndim*(nmes*linear+1)+1), &
                    coor2d, linear, mmes, nmes, npointlc, interpol )

!     --- Fill elements in kmeshclc

      call mshp05 ( kmeshclc, mmes, nmes, inpelm, nelemlc, ishape, idiag_orig )

      if ( ishape==7 ) then

!     --- ishape = 7, Create centroid
!         extend array coor2d with elements of centroids

         call mshtriancentr ( kmeshclc, npointlc, nelemlc, inpelm )
         call mshd7 ( kmeshclc, coor2d, nelemlc, 3 )

      end if

!     --- Fill boundary points in kbndpt

      call mshp06 ( kbndpt, linear, mmes, nmes )

      if(itime==1) call printtime('pipe surface')

1000  call erclos ( 'mshpsr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpsr'

      end if  ! ( debug )

      end
