      subroutine mshr04 ( ichangecr, nchancoor, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 28-03-2013 Extra test
!        version  1.5  date 30-12-2004 Set defaults
!        version  1.4  date 18-03-2000 Allow the keyword immediately
!
!   copyright (c) 1997-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the information following the keyword change_coordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichangecr(4,*), nchancoor, iinput(*)

!     ichangecr  o    integer work array of length 40 to store the information
!                     concerning change_coordinates
!                     For each data record 4 positions are required, at most 10
!                     different cases are allowed
!                     Contents:
!                     (1,i):  ifunc_chan
!                     (2,i):  itype
!                     (3,i):  ifirst
!                     (4,i):  ilast
!     iinput     i    integer input array
!     nchancoor  o    number of data records for change coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=4, nncode2=8
      integer itype2(nncode2), ncode2(nncode2), ncodmx2(nncode2), nfound, &
              nscods(ncodsb), i, ncurvs, nsurfs, nuspnt, nvolms
      double precision tabmn2(nncode2), tabmx2(nncode2)
      character * 11 codesb(ncodsb), code2(nncode2)
      logical found, dubbel

!     code2      Table with keywords
!     codesb     Table with subkeywords
!     dubbel     If true exclusive keywords have been found
!     found      If true already a possibility has been found
!     i          Counting variable
!     itype2      Table containing information about the numbers behind
!                the keywords in the table code2
!     ncode2      Table containing the number of significant characters in
!                the keyword table code2
!     ncodmx2     Table containing the number of maximum characters in
!                the keyword table code2
!     ncodsb     Number of entries in codesb
!     ncurvs     Number of curves in mesh
!     nfound     Number of numbers found after points, curves etc
!     nncode2     Number of entries in code2
!     nscods     Number of sub keywords
!     nsurfs     Number of surfaces in mesh
!     nuspnt     Number of user points
!     nvolms     number of volumes
!     tabmn2     Contains minimum values allowed in table
!     tabmx2     Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following type of input is permitted:
!     position_part,  function_description, when_part
!        with
!     position_part may have one of the following shapes:
!        all
!        point pi to pj
!        curve ci to cj
!        surface si to sj
!        volume vi to vj
!     function_description may have one of the following shapes:
!        func_chan = i
!     when_part may have one of the following shapes:
!        immediately
!        at_end
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1036   change_coordinates found more than once
!    1037   Too many data records in CHANGE_COORDINATES block
!    1038   Last point, curve, surface or volume less than first one
!    1039   At least two of the keywords ALL, CURVE, POINT, SURFACE or
!           VOLUME have been used.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'ALL', 'POINTS', 'CURVES', 'SURFACES', 'VOLUMES', &
                  'FUNC_CHAN', 'IMMEDIATELY', 'AT_END' /
      data ncode2 / 3, 3, 3, 3, 3, 3, 5, 5 /
      data ncodmx2/ 3, 6, 6, 8, 7, 9,11, 6 /
      data itype2 / 2,18,18,18,18, 0, 2, 2 /
      data tabmn2 / 0d0, 2d0, 2d0, 2d0, 2d0,  1d0, 0d0, 0d0 /
      data tabmx2 / 0d0, 0d0, 0d0, 0d0, 0d0,999d0, 0d0, 0d0 /
      data codesb / 'P', 'C', 'S', 'V' /
      data nscods /  1, 1, 1, 1 /

!     code2    Table containing the keywords that are recognized
!              Contents
!              1:  ALL
!              2:  POINT
!              3:  CURVE
!              4:  SURFACE
!              5:  VOLUME
!              6:  FUNC_CHAN
!              7:  IMMEDIATELY
!              8:  AT_END
!     codesb   Table of local subkeywords
!              1: P
!              2: C
!              3: S
!              4: V
! ======================================================================
!
      call eropen ( 'mshr04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr04'
         write(irefwr,1) 'nmain', nmain
         write(irefwr,*) 'codemain(1) ', codemain(1)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nchancoor>0 ) then

!     --- change_coordinates has been found more than once

         call errsub ( 1036, 0, 0, 0 )
         call errcrd
         go to 1000

      end if

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )
      call sepcopycodesub ( codesb, ncodsb, nscods )

      nchancoor = 0
      nuspnt = iinput(2)
      ncurvs = iinput(3)
      nsurfs = iinput(4)
      nvolms = iinput(5)
      ichangecr(1,1) = 1
      ichangecr(2,1) = 0
      ichangecr(3,1) = 0
      ichangecr(4,1) = 0
      ityprd = 1

!     --- Read next record until main keyword is found
!         In order to prevent conflicts with the main keywords SURFACES and
!         VOLUMES, positions 1 and 2 of ncodemain are set to large values
!         This is only done in the case of a call by mshrea
!         After the call of intx02 they are reset

100   table(1) = 0d0
      table(2) = -1
      table(3) = -2
      table(4) = -3
      table(5) = -4
      table(6) = 1d0
      table(7) = 0d0
      table(8) = 0d0
      tabmax(2) = nuspnt
      tabmax(3) = ncurvs
      tabmax(4) = nsurfs
      tabmax(5) = nvolms
      if ( nmain>1 ) then
         ncodemain(1) = 12
         ncodemain(2) = 12
      end if
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      icode = iretur

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         write(irefwr,1) 'iretur', iretur
      end if  ! ( debug )
      if ( nmain>1 ) then
         ncodemain(1) = 4
         ncodemain(2) = 4
      end if

      if ( iretur>0 ) go to 1000
      nchancoor = nchancoor + 1
      if ( nchancoor>10 ) go to 100

      found = .false.
      dubbel = .false.
      ichangecr(1,nchancoor) = 1
      ichangecr(2,nchancoor) = 0
      ichangecr(3,nchancoor) = 0
      ichangecr(4,nchancoor) = 0
      if ( table(1)>0 ) then

!     --- all found

         found = .true.

      end if
      if ( table(2)>0 ) then

!     --- points found

         if ( found ) dubbel = .true.
         found = .true.
         ichangecr(2,nchancoor) = 1
         iposut = nint(table(2))
         nfound = nint(tabmax(2))

      end if
      if ( table(3)>0 ) then

!     --- curve found

         if ( found ) dubbel = .true.
         found = .true.
         ichangecr(2,nchancoor) = 2
         iposut = nint(table(3))
         nfound = nint(tabmax(3))

      end if
      if ( table(4)>0 ) then

!     --- surface found

         if ( found ) dubbel = .true.
         found = .true.
         ichangecr(2,nchancoor) = 3
         nfound = nint(tabmax(4))
         iposut = nint(table(4))

      end if
      if ( table(5)>0 ) then

!     --- volume found

         if ( found ) dubbel = .true.
         found = .true.
         ichangecr(2,nchancoor) = 4
         iposut = nint(table(5))
         nfound = nint(tabmax(5))

      end if
      if ( table(6)>0 ) then

!     --- FUNC_CHAN found

         ichangecr(1,nchancoor) = nint(table(6))

      end if
      if ( table(7)>0 ) then

!     --- IMMEDIATELY found

         ichangecr(2,nchancoor) = ichangecr(2,nchancoor)+100

      end if
      if ( table(2)>0 .or. table(3)>0 .or. table(4)>0 .or. &
           table(5)>0 ) then

!     --- points, curves, surfaces or volumes found

         ichangecr(3,nchancoor) = nint(tabout(iposut))
         if ( nfound.eq.1 ) then

!        --- Only one number found

            ichangecr(4,nchancoor) = ichangecr(3,nchancoor)

         else

!        --- two numbers found

            ichangecr(4,nchancoor) = nint(tabout(iposut+1))

            if ( ichangecr(4,nchancoor).lt.ichangecr(3,nchancoor) ) then

               call errint ( ichangecr(4,nchancoor), 1 )
               call errint ( ichangecr(3,nchancoor), 2 )
               call errsub ( 1038, 2, 0, 0 )
               call errcrd
               ichangecr(4,nchancoor) = nint(tabout(iposut+1))

            end if

         end if

      end if
      if ( dubbel ) then

!     --- exclusive keywords used

         call errsub ( 1039, 0, 0, 0 )
         call errcrd

      end if
      go to 100

1000  if ( nchancoor>10 ) then

!     --- nchancoor > 10

         call errint ( nchancoor, 1 )
         call errint ( 10, 2 )
         call errsub ( 1037, 2, 0, 0 )

      end if
      nchancoor = max(nchancoor,1)

     if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshr04'

      end if  ! ( debug )

      call erclos ( 'mshr04' )

      end
