      subroutine mshr05 ( iobstacle, numobst,  iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 28-03-2013 Extra test
!        version  1.1  date 11-06-2001 Extension with surface obstacles
!        version  1.0  date 18-10-1997
!
!   copyright (c) 1997-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the information following the keyword obstacles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobstacle(2,*), numobst, iinput(*)

!     iinput     i    integer input array
!     iobstacle  o    integer work array of length 20 to store the information
!                     concerning the obstacles
!                     Contents
!                     (1,i) Type of obstacle (1=curve, rest not available)
!                     (2,i) Curve number
!     numobst    o    number of obstacles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=2
      integer  :: nscods(ncodsb), i, ncurvs, m, iseq, ndim, nsurfs
      character * 11 codesb(ncodsb)

!     codesb         Table with subkeywords
!     i              Counting variable
!     iseq           Sequence number of obstacle
!     m              Number of numbers read
!     ncodsb         Number of entries in codesb
!     ncurvs         Number of curves in mesh
!     ndim           Dimension of the space.
!     nscods         Number of sub keywords
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following type of input is permitted:
!       cobs i = cj
!       sobs i = sj
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1046   No curve number found after the obstacle
!    1047   Obstacle is empty
!    1048   Obstacles found more than once
!    1881   The sequence number found after the keyword is incorrect
!    2246   Dimension not correct for type of obstacle
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codesb / 'C', 'S' /
      data nscods /  1, 1 /

!     codesb   Table of local subkeywords
!              1: C
!              2: S
! ======================================================================
!
      call eropen ( 'mshr05' )

      if ( numobst>0 ) then

!     --- obstacles has been found more than once

         call errsub ( 1048, 0, 0, 0 )
         call errcrd
         go to 1000

      end if
      iretur = icode

      numobst = 0
      iobstacle(:,1:10) = 0
      ndim   = iinput(1)
      ncurvs = iinput(3)
      nsurfs = iinput(4)
      ityprd = 1

!     --- Copy extra information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )
      codesub(1:ncodsb) = codesb
      nncode = 2
      code(1) = 'COBS'
      code(2) = 'SOBS'
      ncode(1:2) = 3
      ncodmx(1:2) = 4
      itype(1:2) = 31

!     --- Read next record until main keyword is found

100   table(1) = -1
      tabmin(1) = 1d0
      tabmax(1) = ncurvs
      table(2) = -2
      tabmin(2) = 1d0
      tabmax(2) = nsurfs
      nmain = nncodemain
      call sepreadsetkeyw

      if ( iretur>0 ) go to 900

      if ( table(1)>0 ) then

!     --- curve obstacle found

         if ( ndim/=2 ) then

!        --- ndim # 2

            call errchr ( 'curve', 1 )
            call errint ( 2, 1 )
            call errint ( ndim, 2 )
            call errsub ( 2246, 2, 0, 1 )

         end if
         iposut = nint(table(1))
         iseq = nint(tabmin(1))
         if ( iseq<1 .or. iseq>10 ) then

!        --- Sequence number out of range

            call errint ( iseq, 1 )
            call errint ( 1, 2 )
            call errint ( 10, 3 )
            call errchr ( code(1), 1 )
            call errsub ( 1881, 3, 0, 1 )
            go to 100

         end if
         m = nint(tabmax(1))
         if ( m==0 ) then

!        --- No curves found

            call errsub ( 1046, 0, 0, 0 )
            go to 100

         end if
         iobstacle(1,iseq) = 1
         iobstacle(2,iseq) = nint(tabout(iposut))
         numobst = max(iseq,numobst)

      end if

      if ( table(2)>0 ) then

!     --- surface obstacle found

         if ( ndim/=3 ) then

!        --- ndim # 3

            call errchr ( 'surface', 1 )
            call errint ( 3, 1 )
            call errint ( ndim, 2 )
            call errsub ( 2246, 2, 0, 1 )

         end if
         iposut = nint(table(2))
         iseq = nint(tabmin(2))
         if ( iseq<1 .or. iseq>10 ) then

!        --- Sequence number out of range

            call errint ( iseq, 1 )
            call errint ( 1, 2 )
            call errint ( 10, 3 )
            call errchr ( code(2), 1 )
            call errsub ( 1881, 3, 0, 1 )
            go to 100

         end if
         m = nint(tabmax(2))
         if ( m==0 ) then

!        --- No surfaces found

            call errsub ( 1046, 0, 0, 0 )
            go to 100

         end if
         iobstacle(1,iseq) = 2
         iobstacle(2,iseq) = nint(tabout(iposut))
         numobst = max(iseq,numobst)

      end if
      go to 100

900   do i = 1, numobst

         if ( iobstacle(1,i)==0 ) then

!        --- obstacle not present

            call errint ( i, 1 )
            call errwar ( 1047, 1, 0, 0 )

         end if

      end do

1000  call erclos ( 'mshr05' )

      icode = iretur

      end
