      subroutine mshr12 ( bcord, coor, ivert1, ivert3, ivert4, &
                          numnp1, numnp2, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 27-06-2013 Debug statements
!        version  4.1  date 22-08-1994 Extension with inpelm = 5
!        version  4.0  date 27-11-1992 Removal of ivert2
!
!   copyright (c) 1982-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill coordinates of boundary in array coor
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(2,*), coor(2,*)
      integer ivert1, ivert3, ivert4, numnp1, numnp2, inpelm

!     bcord   i    array containing the coordinates of the boundaries
!     coor    o    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     inpelm  i    number of points per element
!     ivert1  i    |  positions of 4 vertices of the mesh
!     ivert3  i    |  in array kbndpt (local array)
!     ivert4  i    |
!     numnp1  i    number of nodal points along side 1 minus one
!     numnp2  i    number of nodal points along side 2 minus one
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i2, ic1, iextra, ih, ih1, ih2, ihk1, ihk3, ihk4, &
              j, j2, m, n, n1, num1, num2

!     i2             Last point of curve in KMESH part m
!     ic1            ?
!     iextra         Extra number of nodes because of centroid points in a row
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ih2            Help variable to store a constant temporarily
!     ihk1           ?
!     ihk3           ?
!     ihk4           ?
!     j              Counting variable
!     j2             ?
!     m              number of points in 2-direction
!     n              number of points in 1-direction
!     n1             ?
!     num1           ?
!     num2           ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr12'
         write(irefwr,1) 'ivert1, ivert3, ivert4', &
                          ivert1, ivert3, ivert4
         write(irefwr,1) 'numnp1, numnp2, inpelm', &
                          numnp1, numnp2, inpelm
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if(inpelm<10) then

!     --- inpelm<10

        ihk1=(ivert1-1)*2
        ihk3=(ivert3-1)*2
        ihk4=(ivert4-1)*2
        num1=2*numnp1
        num2=2*numnp2
        n1=numnp1+1
        if ( inpelm==5 ) then
           iextra = numnp1
        else if ( inpelm==7 ) then
           iextra = numnp1/2
        else
           iextra = 0
        end if
        if ( debug ) then
           write(irefwr,1) 'ihk1, ihk3, ihk4', ihk1, ihk3, ihk4
           write(irefwr,1) 'num1, num2, n1, iextra', &
                            num1, num2, n1, iextra
        end if  ! ( debug )

!       --- side i   ( i = 0 )

        coor(:,1:n1) = bcord(:,1:n1)
        if ( debug ) call prinrl1 ( coor, n1, 2, 'coor-1' )

!       --- side ii  except first point  ( j = numnp1 )

        ih2 = n1
        ih1 = n1
        n1 = n1+iextra
        do i2 = 1, numnp2-1
           coor(:,ih1+i2*n1) = bcord(:,ih2+i2)
        end do  ! i2 = 2, numnp2
        if ( debug ) &
           call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor-2' )

!       --- side iii  except first point  ( i = numnp2 )

        ih = n1*numnp2+1
        do j2 = 1, numnp1
           coor(:,ih+j2)=bcord(:,ivert3-j2)
        end do  ! j2 = 1, numnp1-1
        if ( debug ) &
           call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor-3' )

!       --- side iv  except first point  ( j = 0 )

        do i2 = 1, numnp2
           coor(:,i2*n1+1)=bcord(:,ivert4-i2)
        end do  ! i2 = 1, numnp2-1
        if ( debug ) &
           call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor-4' )
      else

!     --- inpelm = 10

        n=numnp1/3
        m=numnp2/3
        ic1=numnp1+1

!       --- side i   ( i = 0 )

        do j2 = 1, ic1
           coor(:,j2)=bcord(:,j2)
        end do  ! j2 = 1, ic1

!       --- side ii  except first point  ( j = numnp1 )

        ih1 = ic1
        ih2 = ivert1
        do j=1,m
           ih1=ih1+2*n+1
           ih2=ih2+1
           coor(:,ih1)=bcord(:,ih2)
           ih1=ih1+4*n+1
           ih2=ih2+1
           coor(:,ih1)=bcord(:,ih2)
           ih1=ih1+3*n+1
           ih2=ih2+1
           coor(:,ih1)=bcord(:,ih2)
        end do  ! j=1,m

!       --- side iii  except first point  ( i = numnp2 )

        do j=1,numnp1
           ih1=ih1-1
           ih2=ih2+1
           coor(:,ih1)=bcord(:,ih2)
        end do  ! j=1,numnp1

!       --- side iv  except first point  ( j = 0 )

        do j=1,m
           ih1=ih1-(2*n+1)
           ih2=ih2+1
           coor(:,ih1)=bcord(:,ih2)
           ih1=ih1-(4*n+1)
           ih2=ih2+1
           coor(:,ih1)=bcord(:,ih2)
           if(j/=m) then

!          --- j # m

              ih1=ih1-(3*n+1)
              ih2=ih2+1
              coor(:,ih1)=bcord(:,ih2)
           end if
        end do  ! j=1,m
      end if
1000  call erclos ( 'mshr12' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor' )
         write(irefwr,*) 'End mshr12'
      end if  ! ( debug )
      end
