      subroutine mshr13 ( bcord, tauar, coor, ivert1, ivert2, ivert3, &
                          ivert4, iboune, numnp1, numnp2, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 27-06-2013 Also fill mid points
!        version  4.0  date 15-09-2010 Remove work arrays
!        version  3.1  date 22-08-1994 Extension with inpelm = 5
!
!   copyright (c) 1982-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill coordinates of vertices of elements for internal elements
!     Also the mid points of sides are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(2,*), coor(2,*), tauar(*)
      integer ivert1, ivert2, ivert3, ivert4, iboune, numnp1, numnp2, &
              inpelm

!     bcord   i    array containing the coordinates of the boundaries
!     coor    o    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     iboune  i    ?
!     inpelm  i    number of points per element
!     ivert1  i    ?
!     ivert2  i    ?
!     ivert3  i    ?
!     ivert4  i    ?
!     numnp1  i    ?
!     numnp2  i    ?
!     tauar   i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iextra, j, n1, n1i, n1ij, num1, num2
      double precision a(2), sigma, tau, w1, w2, w3, w4, wcor, wdif, &
                       alpha(2,4), beta(2,4)

!     a
!     alpha          Contains coordinates of vertices
!     beta           Contains coordinates of nodes on boundary
!     i              Counting variable
!     iextra         Extra number of nodes because of centroid points in a row
!     j              Counting variable
!     n1             ?
!     n1i            ?
!     n1ij           ?
!     num1           ?
!     num2           ?
!     sigma
!     tau            ?
!     w1
!     w2
!     w3             ?
!     w4             ?
!     wcor           ?
!     wdif           ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr13' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr13'
         write(irefwr,1) 'ivert1, ivert2, ivert3, ivert4', &
                          ivert1, ivert2, ivert3, ivert4
         write(irefwr,1) 'iboune, numnp1, numnp2, inpelm', &
                          iboune, numnp1, numnp2, inpelm

  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n1=numnp1+1
      if ( inpelm==5 ) then
         iextra = numnp1
      else if ( inpelm==7 ) then
         iextra = numnp1/2
      else
         iextra = 0
      end if
      n1=n1+iextra

!     --- fill array alpha

      alpha(:,1)=bcord(:,1)
      alpha(:,2)=bcord(:,ivert1)
      alpha(:,3)=bcord(:,ivert3)
      alpha(:,4)=bcord(:,ivert2)
      a(1)=0.25d0*sum(alpha(1,:))
      a(2)=0.25d0*sum(alpha(2,:))

      if ( debug ) then
         call prinrl1 ( alpha, 4, 2, 'alpha' )
         call prinrl ( a, 2, 'a' )
      end if  ! ( debug )

      num1=numnp1-1
      num2=numnp2-1

!     --- do i = iboune, num2, iboune

      do i = 1, num2

         n1i=n1*i+1
         w2=tauar(ivert1+i)
         w4=tauar(ivert4-i)
         wdif=w2-w4
         beta(:,2)=bcord(:,i+ivert1)
         beta(:,4)=bcord(:,ivert4-i)

!        --- do j=iboune,num1,iboune

         do j = 1, num1
            n1ij=n1i+j
            w1=tauar(j+1)
            w3=tauar(ivert3-j)
            wcor=w1-w3
            beta(:,1)=bcord(:,j+1)
            beta(:,3)=bcord(:,ivert3-j)

            sigma=(w1*wdif+w4)/(1d0+wcor*wdif)
            tau=sigma*w3+(1d0-sigma)*w1
            wcor=wcor*wdif

            if ( debug ) then
               write(irefwr,1) 'i, j, n1ij', i, j, n1ij
               write(irefwr,2) 'wcor, tau, sigma', wcor, tau, sigma
               write(irefwr,2) 'w1, w2, w3, w4', w1, w2, w3, w4
               call prinrl1 ( beta, 4, 2, 'beta' )
            end if  ! ( debug )

               coor(:,n1ij)=(1d0-tau)*beta(:,4)+tau*beta(:,2) &
                        +(1d0-sigma)*beta(:,1)+sigma*beta(:,3) &
                        -(1d0-w4)*(1d0-w1)*alpha(:,1)-w1*(1d0-w2) &
                        *alpha(:,2)-(1d0-w3)*w4*alpha(:,3) &
                        -w3*w2*alpha(:,4)-wcor*a(:)
         end do  ! j=iboune,num1,iboune
      end do  ! i=iboune,num2,iboune

1000  call erclos ( 'mshr13' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor' )
         write(irefwr,*) 'End mshr13'

      end if  ! ( debug )

      end
