      subroutine mshr16 ( kmeshc, numnp1, numnp2, inpelm, coor, jdiag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 15-09-2010 Remove work arrays
!        version  5.1  date 13-05-2010 Fortran 90
!        version  5.0  date 24-07-2003 Remove phi from parameter list
!        version  4.3  date 22-02-1999 random diagonal of triangle (BvR)
!
!   copyright (c) 1982-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill elements in array kmesh part c, in the case of triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     rectangle
!     structured
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), numnp1, numnp2, inpelm, jdiag
      double precision coor(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     jdiag          i    indication whether the direction of the diagonal in a
!                         quadrilateral must be computed (jdiag=0) or not
!                         (jdiag=1)
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     numnp1         i    number of nodal points along side 1 minus one
!     numnp2         i    number of nodal points along side 2 minus one
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iel(4), ih, i, ih1, ih2, ikelmc, in2, j, k, m, n, n1, &
              idirec, itel1(14), itel2(14)
      double precision randnr, x(6), y(6)
      logical even

!     even           Gives direction of diagonal
!                    True from 1 to 3, False from 2 to 4 in rectangle (1,2,3,4)
!     i              Counting variable
!     idirec         Gives direction of diagonal
!                    1 even, 0 odd, 2 no specific direction
!     iel            Contains relative node numbers for the quadrilaterals
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ih2            Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     in2            Help counter
!     itel1          Contains shift of relative node numbers
!     itel2          Contains shift of relative node numbers
!     j              Counting variable
!     k              Counting variable
!     m              number of elements in the "second" direction
!     n              number of elements in the "first" direction
!     n1             Number of elements along the  south  side
!                    of the current quadrilateral
!     randnr         Value of a random number
!     x              work array to store the x-coordinates of a quad
!     y              work array to store the y-coordinates of a quad
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: prandm

!     MSHR17         detect direction of diagonal for triangular elements
!     PRANDM         returns random number between [0,1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      even=.true.
      ikelmc=0
      n1=numnp1+1

      if ( testversion ) then
         randnr = prandm(numnp1**2+numnp2)
      else
         call random_number ( randnr )
      end if  ! ( testversion )
      if ( inpelm==3 ) then

!     --- inpelm = 3,  linear elements

         iel(1)=0
         iel(2)=1
         iel(3)=1+n1
         iel(4)=n1
         itel1(1)=1
         itel1(2)=1+n1
         itel1(3)=0
         itel1(4)=n1
         itel1(5)=0
         itel1(6)=n1+1
         itel2(1)=0
         itel2(2)=1
         itel2(3)=n1
         itel2(4)=n1+1
         itel2(5)=n1
         itel2(6)=1
         do i=1,numnp2
            in2=(i-1)*n1
            do j=1,numnp1
               do k=1,4
                  ih=2*(in2+j+iel(k))
                  x(k+1)=coor(ih-1)
                  y(k+1)=coor(ih)
               end do
               if ( jdiag==0 ) then
                  call mshr17 ( x, y, idirec )
                  even = idirec==1
               else if ( jdiag==2 ) then
                  if ( testversion ) then
                     randnr = prandm(0)
                  else
                     call random_number ( randnr )
                  end if  ! ( testversion )
                  even=.false.
                  if ( randnr<0.5d0 ) even=.true.
               end if
               if ( even ) then

!              --- even situation

                  do k = 1, 3
                     kmeshc(ikelmc+k)=in2+j+itel1(k+3)
                  end do
                  do k = 4, 6
                     kmeshc(ikelmc+k)=in2+j+itel1(k-3)
                  end do

               else

!              --- odd situation

                  do k = 1, 3
                     kmeshc(ikelmc+k)=in2+j+itel2(k+3)
                  end do
                  do k = 4, 6
                     kmeshc(ikelmc+k)=in2+j+itel2(k-3)
                  end do

               end if
               ikelmc=ikelmc+6

            end do

         end do

      else if ( inpelm==6 .or. inpelm==7 ) then

!     --- inpelm = 6,or 7   quadratic elements

         if ( inpelm==7 ) n1=n1+numnp1/2
         iel(1)=0
         iel(2)=2
         iel(3)=2+2*n1
         iel(4)=2*n1
         itel1(1)=2
         itel1(2)=2+n1
         itel1(3)=2*n1+2
         itel1(4)=n1+1
         itel1(5)=0
         itel1(6)=1
         itel1(inpelm+1)=2*n1
         itel1(inpelm+2)=n1
         itel1(inpelm+3)=0
         itel1(inpelm+4)=n1+1
         itel1(inpelm+5)=2*n1+2
         itel1(inpelm+6)=2*n1+1
         itel2(1)=0
         itel2(2)=1
         itel2(3)=2
         itel2(4)=n1+1
         itel2(5)=2*n1
         itel2(6)=n1
         itel2(inpelm+1)=2*n1+2
         itel2(inpelm+2)=2*n1+1
         itel2(inpelm+3)=2*n1
         itel2(inpelm+4)=n1+1
         itel2(inpelm+5)=2
         itel2(inpelm+6)=n1+2
         if ( inpelm==7 ) then
            itel1(7)=numnp1+1
            itel1(14)=n1+numnp1+1
            itel2(7)=numnp1+1
            itel2(14)=n1+numnp1+1
         end if
         do i=1,numnp2,2
            in2=(i-1)*n1
            do j=1,numnp1,2
               do k=1,4
                  ih=2*(in2+j+iel(k))
                  x(k+1)=coor(ih-1)
                  y(k+1)=coor(ih)
               end do
               if ( jdiag==0 ) then
                  call mshr17 ( x, y, idirec )
                  even = idirec==1
               else if ( jdiag==2 ) then
                  if ( testversion ) then
                     randnr = prandm(0)
                  else
                     call random_number ( randnr )
                  end if  ! ( testversion )
                  even=.false.
                  if ( randnr<0.5d0 ) even=.true.
               end if
               if ( even ) then

!              --- even situation

                  do k = 1, inpelm
                     kmeshc(ikelmc+k)=in2+j+itel1(k+inpelm)
                  end do
                  do k = inpelm+1, 2*inpelm
                     kmeshc(ikelmc+k)=in2+j+itel1(k-inpelm)
                  end do

               else

!              --- odd situation

                  do k = 1, inpelm
                     kmeshc(ikelmc+k)=in2+j+itel2(k+inpelm)
                  end do
                  do k = inpelm+1, 2*inpelm
                     kmeshc(ikelmc+k)=in2+j+itel2(k-inpelm)
                  end do
               end if
               if ( inpelm==7 ) then
                  kmeshc(ikelmc+7)  = kmeshc(ikelmc+7)-j/2
                  kmeshc(ikelmc+14) = kmeshc(ikelmc+14)-j/2
               end if
               ikelmc=ikelmc+2*inpelm
               end do
         end do

      else if ( inpelm==10 ) then

!     --- inpelm = 10  special elements

         n=numnp1/3
         m=numnp2/3
         iel(1)=0
         iel(2)=3
         iel(3)=6+n*9
         iel(4)=3+n*9
         do i = 1, m
            do j = 1, n
               ih=1+(i-1)*(9*n+3)
               ih1=ih+(j-1)*3
               ih2=ih+(j-1)*2
               do k=1,4
                  ih=ih1+iel(k)
                  x(k+1)=coor(2*ih-1)
                  y(k+1)=coor(2*ih)
               end do
               if ( jdiag==0 ) then
                  call mshr17 ( x, y, idirec )
                  even = idirec==1
               else if ( jdiag==2 ) then
                  if ( testversion ) then
                     randnr = prandm(0)
                  else
                     call random_number ( randnr )
                  end if  ! ( testversion )
                  even=.false.
                  if ( randnr<0.5d0 ) even=.true.
               end if
               if ( even ) then

!              --- even situation

                  kmeshc(ikelmc+1)=3+ih1
                  kmeshc(ikelmc+2)=3+n*3+ih2
                  kmeshc(ikelmc+3)=7*n+4+ih2
                  kmeshc(ikelmc+4)=9*n+6+ih1
                  kmeshc(ikelmc+5)=7*n+3+ih2
                  kmeshc(ikelmc+6)=3*n+2+ih2
                  kmeshc(ikelmc+7)=ih1
                  kmeshc(ikelmc+8)=1+ih1
                  kmeshc(ikelmc+9)=2+ih1
                  kmeshc(ikelmc+10)=5*n+3+ih2
                  kmeshc(ikelmc+11)=9*n+3+ih1
                  kmeshc(ikelmc+12)=7*n+2+ih2
                  kmeshc(ikelmc+13)=3*n+1+ih2
                  kmeshc(ikelmc+14)=ih1
                  kmeshc(ikelmc+15)=3*n+2+ih2
                  kmeshc(ikelmc+16)=7*n+3+ih2
                  kmeshc(ikelmc+17)=9*n+6+ih1
                  kmeshc(ikelmc+18)=9*n+5+ih1
                  kmeshc(ikelmc+19)=9*n+4+ih1
                  kmeshc(ikelmc+20)=5*n+2+ih2

               else

!              --- odd situation

                  kmeshc(ikelmc+1)=ih1
                  kmeshc(ikelmc+2)=1+ih1
                  kmeshc(ikelmc+3)=2+ih1
                  kmeshc(ikelmc+4)=3+ih1
                  kmeshc(ikelmc+5)=3*n+2+ih2
                  kmeshc(ikelmc+6)=7*n+3+ih2
                  kmeshc(ikelmc+7)=9*n+3+ih1
                  kmeshc(ikelmc+8)=7*n+2+ih2
                  kmeshc(ikelmc+9)=3*n+1+ih2
                  kmeshc(ikelmc+10)=5*n+2+ih2
                  kmeshc(ikelmc+11)=9*n+6+ih1
                  kmeshc(ikelmc+12)=9*n+5+ih1
                  kmeshc(ikelmc+13)=9*n+4+ih1
                  kmeshc(ikelmc+14)=9*n+3+ih1
                  kmeshc(ikelmc+15)=7*n+3+ih2
                  kmeshc(ikelmc+16)=3*n+2+ih2
                  kmeshc(ikelmc+17)=3+ih1
                  kmeshc(ikelmc+18)=3*n+3+ih2
                  kmeshc(ikelmc+19)=7*n+4+ih2
                  kmeshc(ikelmc+20)=5*n+3+ih2
               end if
               ikelmc=ikelmc+2*inpelm

            end do

         end do

      end if

      end
