      subroutine mshrd1reflect ( iinput, iinp, ndim, nuspnt, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information for reflect curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nuspnt, icurnr
      integer, intent(inout) :: iinput(*), iinp
      integer, intent(out) :: ndim

!     icurnr         i    Curve sequence number
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
!     ndim           o    Dimension of the space.
!     nuspnt         i    Number of user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inum, i

!     i              Counting variable
!     inum           Number of numbers read in a record
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHR00         Read information after the part Ci = LINE ( .. )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     498   Curve number of curve to be translated or rotated too large
!     966   Reflection axis or plane not specified
!     967   Not enough user points specified for reflection
!    2945   Wrong string found following reflect
!    2946   No integer number following reflect c
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1reflect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1reflect'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(1300) )

!     --- Reflect
!         Read number of curve to be reflected

      call asknxt
      if ( ityprd==4 ) then

!     --- string found, check if it is a C

         if ( nchar>1 .or. string(1:1)/='C' ) then

!        --- error not C found

            call errint ( icurnr, 1 )
            call errchr ( string(1:nchar), 1 )
            call errsub ( 2945, 1, 0, 1 )

         end if  ! ( nchar>1 .or. string(1:1)/='C' )
         call asknxt  ! read next item

      end if  ! ( ityprd==4 )

      if ( ityprd/=2 ) then

!     --- Not an integer found following reflect c

         call errint ( icurnr, 1 )
         call errsub ( 2946, 1, 0, 0 )

      end if  ! ( ityprd/=2 )

!     --- Place number in iinput

      iinput(iinp+1) = 1
      iinput(iinp+4) = nint(realrd)

!     --- Read whether reflection on axis or on rplane

      call mshr00 ( nuspnt )

      inum = nint(work(1))

      iinput(iinp+3) = inum

      do i = 1, inum
         iinput(iinp+4+i) = nint(work(24+i))
      end do

      if ( work(11)<0.9d0 .or. work(11)>2.1d0 ) then

           call errint( icurnr, 1 )
           call errsub ( 966, 1, 0, 0 )
           call errcrd

      end if

      ndim = nint(work(11)) + 1

      if ( inum<ndim+2 ) then

           call errint( icurnr, 1 )
           call errsub ( 967, 1, 0, 0 )
           call errcrd

      end if

      if ( iinput(iinp+4)<=0 ) then
         call errint ( iinput(iinp+4), 1 )
         call errint ( icurnr, 2 )
         call errsub ( 498, 2, 0, 0 )
         call errcrd
      end if

      iinp = iinp+5+inum
      deallocate ( work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd1reflect'

      end if  ! ( debug )
      call erclos ( 'mshrd1reflect' )

      end subroutine mshrd1reflect
