      subroutine mshrd1type ( iinput, iinp, icurnr, icurve, jcoars )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Detect type of curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: icurnr, jcoars, iinp
      integer,intent(out) :: icurve
      integer,intent(inout) :: iinput(*)

!     icurnr         i    Curve sequence number
!     icurve         o    Type of curve
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
!     jcoars         i    If 1 coarseness is given, if 0 not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1336   Ci = is not followed by a text
!    1616   No coarseness given in relation with carc, cline, cparam or cspline
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1type' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1type'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt
      if ( ityprd/=4 ) then

!     --- Not a text found after Ci = &

         call errsub ( 1336, 0, 0, 0 )
         call errcrd
         go to 1000

      end if

      ierpri = 1
      icurve = sepsubkeyw()
      if ( jcoars==0 .and. (icurve==5 .or. icurve==6 .or. &
           icurve==9 .or. icurve==12 .or. icurve==19 .or. &
           icurve==21 .or. icurve==22) ) then
         call errsub ( 1616, 0, 0, 0 )
      end if
      if ( debug ) write(irefwr,1) 'icurnr, icurve', icurnr, icurve

      if ( icurve>0 ) then

!     --- icurve>0

         if ( icurve>=18 ) icurve = icurve+2  ! Curve numbers 18 and 19
                                              ! correspond to circle and sphere

         iinput(iinp) = icurve
         iinput(iinp+2) = 0
         iinput(iinp+3) = 0

      end if  ! ( icurve>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd1type'

      end if  ! ( debug )
      call erclos ( 'mshrd1type' )

      end subroutine mshrd1type
