      subroutine mshsp3( coor, i, j, k, dx )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.0  date 24-03-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine the vector dx(3) which is perpen- &
!     dicular to the plane of the points i-j-k.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), dx(3)
      integer i,j,k

!     coor       i  coordinate array
!     dx         o  direction vector
!     i          i  first node in plane
!     j          i  second node in plane
!     k          i  third node in plane
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eleng, xi, xj, xk, yi, yj, yk, zi, zj, zk

!     eleng   euclidian length of vector
!     xi      x-coordinate of point i
!     xj      x-coordinate of point j
!     xk      x-coordinate of point k
!     yi      y-coordinate of point i
!     yj      y-coordinate of point j
!     yk      y-coordinate of point k
!     zi      z-coordinate of point i
!     zj      z-coordinate of point j
!     zk      z-coordinate of point k
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   deconcatenate name of string of calling subroutines
!     EROPEN   concatenate name to string of calling subroutines
!     ERRINT   place integer in error array
!     ERRSUB   submit error message
!     INSTOP   stop execution of SEPRAN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     965   Reflection surface not correct specified
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'mshsp3' )

!     --- Determine coordinates of the three nodes

      xi = coor(1,i)
      yi = coor(2,i)
      zi = coor(3,i)

      xj = coor(1,j)
      yj = coor(2,j)
      zj = coor(3,j)

      xk = coor(1,k)
      yk = coor(2,k)
      zk = coor(3,k)

!     --- Determine the "inward pointing normal"

      dx(1) = (yj-yi)*(zk-zi) - (yk-yi)*(zj-zi)
      dx(2) = (zj-zi)*(xk-xi) - (zk-zi)*(xj-xi)
      dx(3) = (xj-xi)*(yk-yi) - (xk-xi)*(yj-yi)

!     --- Normalize

      eleng = sqrt( dx(1)*dx(1) + dx(2)*dx(2) + dx(3)*dx(3) )

      if ( eleng.lt.100 * epsmac ) then

!     --- Error : reflection plane not correct defined

         call errint( i, 1 )
         call errint( j, 2 )
         call errint( k, 3 )

         call errsub ( 965, i, j, k )
         call instop

      end if

      dx(1) = dx(1) / eleng
      dx(2) = dx(2) / eleng
      dx(3) = dx(3) / eleng

      call erclos( 'mshsp3' )

      end
