      subroutine mshspc ( coorh, kmeshcnew, lcoor, lenkmeshc, &
                          kmeshmnew, lenkmeshmnew, iwork, klakh, lvlak, &
                          ibrp, input, iinput, rinput, lncurves )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 07-11-2009 Remove some parameters
!        version  4.2  date 25-02-2004 Layout
!        version  4.1  date 15-10-2003 Extra debug statements
!
!   copyright (c) 1995-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the neighbor information and make a spectral mesh from
!     an existing SEPRAN mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(ncurvs), klakh(*), &
              lcoor, lenkmeshc, lenkmeshmnew, lvlak, ibrp, input(5), &
              iinput(*), lncurves, kmeshcnew(lenkmeshc), &
              kmeshmnew(lenkmeshmnew)
      double precision coorh(ndim,lcoor/ndim), rinput(*)

!     coorh          o    new coordinates array (KMESH part i)
!     ibrp           i    max number of neighbors allowed
!     iinput         i    integer input array
!                         part for curves of original SEPRAN input array
!     input          i    array in which is described where and how many points
!                         have to be placed
!     iwork               help array for information on filled curves
!     klakh          i    KMESH part N after refinement
!     kmeshcnew      o    KMESH part C after refinement
!     kmeshmnew      i    KMESH part M after refinement
!     lcoor         i/o   (i) declared length coorh, (o) used length
!     lenkmeshc     i/o   (i) declared length kmeshmnew, (o) used length
!     lenkmeshmnew  i/o   (i) declared length kmeshcnew, (o) used length
!     lncurves       i    Length of array curves
!     lvlak         i/o   (i) declared length klakh, (o) used length
!     rinput         i    Real input array
!                         array with the coordinates of the user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, js, ih, ikmeshc, iklemh, ipnt, ipos, &
              jstart, jstend, kcoor, lenibrpnt, &
              kvlak, npt, nelhel, npelem, npnold, &
              ntal, nisrf, nsubd, newplm, kelold
      double precision factor(0:25), tauarr(1000)
      integer, allocatable :: istart(:), ibrpnt(:)
      double precision, allocatable :: curves(:)

!     curves         help array to store the coordinates of the new
!                    computed nodes temporarily
!     factor         array to store the distribution of points along
!                    a line
!     i              loop variable
!     ibrpnt         help array to store nodenumbers neighbors
!     ielgrp         elementgroup number
!     ih             helpvariable to store node number
!     iklemh         pointer in array kmeshcnew
!     ikmeshc        pointer in array kmeshc
!     ipnt           node number
!     ipos           position indicator in surface array
!     istart         help array for start addresses neighbors
!     js             loop variable
!     jstart         start pointer neighbors
!     jstend         end pointer neighbors
!     kcoor          length of new coordinates array
!     kelold         old number of augmented points in elements
!     kvlak          length of new surfaces array
!     lenibrpnt      length of array ibrpnt
!     nelhel         number of elements
!     newplm         new number of nodes per element
!     nisrf          number of internal (spectral) points in surface
!     npelem         number of points in element
!     npnold         old number of points
!     npt            node number
!     nsubd          type of subdivision to be used
!                    0  :  equidistant division
!                    1  :  Gauss-Legendre division
!     ntal           number of elements in surface subgroup
!     tauarr         help array to be used for the division of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELLGLP         compute the Legendre Gauss-Lobatto points
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH401         Run through group of elements to determine neighbors
!     MSH444         Place all triangles in neighbor array
!     MSH462         Determine new nodes
!     MSH463         Place internal points on line in accordance with given
!                    distribution
!     MSH464         Place internal points in a surface/volume in accordance
!                    with given distribution
!     MSHG25         Check all (submitted) hexahedrons
!     MSHSPCKMESHM   Fill new array kmesh part m and also fill coordinates
!                    on the curves
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     SEPPRINKMESHC  print kmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!     940   Not enough positions declared for helparray
!     972   Wrong value for meshtype found
!    1018   Maximum number of points at side exceeded
! **********************************************************************
!
!                       PSEUDO CODE
!
!     PSEUDO-CODE
!     Determine some important parameters
!     Clear array istart (array for the starting addresses)
!     Clear array ibrpnt (array for the neighbors)
!     Run through all elements to fill neighbor array
!     Rearrange arrays istart and ibrpnt
!     Reserve neighbor point places
!     If necessary place extra neighbors
!     Set number of points for copying later
!     Place new nodal-point numbers on reserved places, make distinction
!     between edge, surface and volume
!     Save new number of nodes
!     Copy old coordinates to new array
!     Fill new curves array (kmeshmnew), run through all user curves and
!     adjust the nodal points if necessary
!     Refill kmeshc ( and place the correct coordinates of quadrilaterals
!     and hexahedrons with respect to the  spectral points)
!     Refill kmesh
!     If necessary : Fill new kmesh part n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspc'
         write(irefwr,1) 'size(kmeshc)', size(kmeshc)
         call sepprinkmeshc
         call prinrl1 ( coor, npoint, ndim, 'coor' )
  1      format ( a, 1x, (10i6) )

      end if

!     --- Start checking array input

      niedge = input(1)
      nisurf = input(2)
      nivolm = input(3)
      nsubd  = input(4)

!     --- Check whether array factor is long enough

      if ( niedge>24 ) then

!     --- Error, array not large enough

         call errint ( niedge, 1 )
         call errint ( 24, 2 )
         call errsub ( 1018, 2, 0, 0 )
         go to 1000

      end if

      if ( nsubd==0 ) then

!     --- Place points equidistant, fill array factor

         do js = 1, niedge
            factor(js) = -1. + 2 * js / (1. + niedge)
         end do

      else if ( nsubd==1 ) then

!     --- Use Gauss-Legendre weighting (-1,1), fill array factor

         call ellglp ( factor(0), niedge+2 )

      end if

      lenibrpnt = (npoint+1)*ibrp
      allocate ( istart(npoint+1), curvefacts(lncurves/ndim), &
                 curves(lncurves), ibrpnt(lenibrpnt), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint+lncurves+lenibrpnt, 'istart' )
      if ( ierror/=0 ) go to 1000

!     --- Set variables to be used for the surfaces

      iklemh = 1
      ikmeshc = 1

!     --- Run through group of elements, check each side and make reservation
!         if not yet done

      istart = 0
      ibrpnt = 0

      kmeshcnew(1) = 0

      inpelm = npelm

      if ( debug ) write(irefwr,1) 'kelma', kelma
      if ( kelma==0 ) then

!     --- Only one type of standard element

         ishape = kmeshf(1)

!        --- Check that element is linear

         if ( ishape/= 1 .and. ishape/=50 .and. &
              ishape/= 3 .and. ishape/= 5 .and. &
              ishape/=11 .and. ishape/=13 ) then

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if

!        --- Check elements submitted if ishape = 13

         if ( ishape==13 ) call mshg25( kmeshc, coor, nelem )

!        --- Store (if wanted) all lines, faces and volumes in array ibrpnt

         if ( debug ) write(irefwr,*)  'before msh401/1'
         call msh401( npoint, kmeshc, nelem, istart, ibrp, ibrpnt, &
                      kmeshcnew, ishape, inpelm, niedge, nisurf, nivolm )

!        --- Special treatment in case of triangular meshes

         if ( ishape==3 .or. ishape==11 ) then

            if ( nisurf==-1 ) then
               nisrf = 1
            else
               nisrf = nisurf
            end if

            if ( ndim>2 .and. nisrf>0 ) &
                 call msh444( kmeshc, nelem, istart, ibrp, ibrpnt, &
                              kmeshcnew, ishape, npoint )

         end if

      else

!     --- Several standard elements
!         Startaddress in kmeshc

         ikmeshc = 1
         inpelm = npelm

         do ielgrp = 1, nelgrp

            if ( kelma/=0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
            ishape = kmeshf(ielgrp)

!           --- Number of elements of this type

            npelem = kmeshg(ielgrp)

            if ( ishape>0 ) then

               if ( ishape/= 1 .and. ishape/=50 .and. &
                    ishape/= 3 .and. ishape/= 5 .and. &
                    ishape/=11 .and. ishape/=13 ) then

!              --- error, wrong value for ishape

                  call errint( ishape, 1 )
                  call errsub ( 972, 1, 0, 0 )
                  go to 1000

               end if

!              --- Check elements submitted if ishape = 13

               if ( ishape==13 ) &
                    call mshg25( kmeshc(ikmeshc), coor, npelem )

!              --- Store (if wanted) all lines, all faces and all volume
!                  elements in ibrpnt

               if ( debug ) write(irefwr,*)  'before msh401/2'
               call msh401 ( npoint, kmeshc(ikmeshc), npelem, istart, &
                             ibrp, ibrpnt, kmeshcnew, ishape, inpelm, &
                             niedge, nisurf, nivolm )

!              --- Special treatment in case of triangular meshes

               if ( ishape==3 .or. ishape==11 ) then

                  if ( nisurf==-1 ) then
                       nisrf = 1
                  else
                       nisrf = nisurf
                  end if

                  if ( ndim>2 .and. nisrf>0 ) &
                       call msh444( kmeshc(ikmeshc), nelem, istart, &
                                    ibrp, ibrpnt, kmeshcnew, ishape, &
                                    npoint )

               end if

            end if

!           --- Increase ikmeshc

            ikmeshc = ikmeshc + inpelm * npelem

         end do

      end if

      npnold = npoint

!     --- All neighbors have been placed, rearrange arrays istart and
!         ibrpnt. Determine node numbers of new neighbors and place
!         new node numbers in array ibrpnt

      if ( debug ) write(irefwr,*)  'before msh462'
      call msh462( ibrp, ibrpnt, istart, npoint, kmeshcnew, &
                   niedge, nisurf, nivolm )

!     --- Save new number of nodes

      kcoor = ndim * npoint

      if (  kcoor>lcoor ) then

         call errint ( kcoor, 1 )
         call errint ( lcoor, 2 )
         call errsub ( 940, 2, 0, 0 )
         go to 1000

      end if

!     --- Fill coordinates of all points in coorh
!         Start copying the points already known

      coorh(:,1:npnold) = coor(:,1:npnold)

!     --- Run through all curves and place the coordinates
!         (in case of arcs etc only a first approximation)

      do i = 1, npnold

         jstart =  istart(i  )
         jstend =  istart(i+1) - 2

         do ipnt = jstart, jstend, 2

            ih  = ibrpnt(ipnt  )
            npt = ibrpnt(ipnt+1)

            if ( abs(ih)<npnold ) then

!           --- Points are on a line : ( niedge points )

               call msh463 ( i, ih, npt, coorh, niedge, ndim, factor )

            end if

!           --- If ih>npnold then
!               Points are in a surface : ( nisurf points )
!               or in a volume : ( nivolm points )
!               These points will be placed later

         end do

      end do
      if ( debug ) write(irefwr,*)  'after msh463'

      if ( lenkmeshmnew>0 ) then

!     --- Fill kmeshmnew (new kmesh part m)

         call mshspckmeshm ( kmeshmnew, iwork, istart, ibrpnt, npnold, &
                             input, iinput, rinput, coorh, &
                             factor, tauarr, curves, &
                             lenkmeshmnew, lenibrpnt, lcoor, lncurves )

      end if
      if ( debug ) write(irefwr,*)  'after mshspckmeshm'

!     --- Refill kmeshc and compute the coordinates of the surface- and
!         volume points, using a quadratic isoparametric transformation

      if ( nelgrp==1 ) then

!     --- only one group of elements

         ishape = kmeshf(1)

         if ( ishape==1 ) then

            newplm = 2 + niedge

         else if ( ishape==3 ) then

            if ( nisurf==-1 ) then

               if ( niedge==0 ) then

                  newplm = 3

               else

                  newplm = 3 + 3 * niedge + 1

               end if

            else

               newplm = 3 + 3 * niedge + nisurf

            end if

         else if ( ishape==5 ) then

            if ( nisurf==-1 ) then

               newplm = (2 + niedge) * (2 + niedge)

            else

               newplm = 4 + 4 * niedge + nisurf

            end if

         else if ( ishape==11 ) then

            if ( nisurf==-1 .and. nivolm==-1 ) then

               if ( niedge==0 ) then

                  newplm = 4

               else

                  newplm = 4 + 6 * niedge + 5

               end if

            else

               newplm = 4 + 6 * niedge + 4 * nisurf + nivolm

            end if

         else if ( ishape==13 ) then

            if ( nisurf==-1 .and. nivolm==-1 ) then

               newplm = 8 + 12 * niedge + 6 * niedge * niedge + &
                        niedge * niedge * niedge

            else

               newplm = 8 + 12 * niedge + 6 * nisurf + nivolm

            end if

         end if

         if ( debug ) write(irefwr,*)  'before msh464/1'
         call msh464 ( kmeshc, nelem, ibrpnt, istart, kmeshcnew, &
                       ishape, coorh, npnold, ndim, 1, &
                       factor, niedge, nisurf, nivolm, npoint )

         iklemh    = nelem * newplm + 1
         npelm = newplm
         kelgrp = newplm

      else

!     --- Several groups

         nelhel = nelgrp

!        --- Start address in kmeshc

         ikmeshc = 1
         iklemh = 1

         kelgrp = 0
         kelold = 0
         inpelm = npelm

         do ielgrp = 1, nelhel

            if ( kelma/=0 ) inpelm = kmesha(ielgrp+1)-kelold
            ishape = kmeshf(ielgrp)
            if ( ishape==1 ) then

               newplm = 2 + niedge

            else if ( ishape==3 ) then

               if ( nisurf==-1 ) then

                  if ( niedge==0 ) then

                     newplm = 3

                  else

                     newplm = 3 + 3 * niedge + 1

                  end if

               else

                  newplm = 3 + 3 * niedge + nisurf

               end if

            else if ( ishape==5 ) then

               if ( nisurf==-1 ) then

                  newplm = (2 + niedge) * (2 + niedge)

               else

                  newplm = 4 + 4 * niedge + nisurf

               end if

            else if ( ishape==11 ) then

               if ( nisurf==-1 .and. nivolm==-1 ) then

                  if ( niedge==0 ) then

                     newplm = 4

                  else

                     newplm = 4 + 6 * niedge + 5

                  end if

               else

                  newplm = 4 + 6 * niedge + 4 * nisurf + nivolm

               end if

            else if ( ishape==13 ) then

               if ( nisurf==-1 .and. nivolm==-1 ) then

                  if ( niedge==0 ) then

                     newplm = 8

                  else

                     newplm = 8 + 12 * niedge + 6 * niedge * niedge + &
                                  niedge * niedge * niedge

                  end if

               else

                  newplm = 8 + 12 * niedge + 6 * nisurf + nivolm

               end if

            else if ( ishape==50 ) then

               newplm = 2*(2 + niedge)

            else if ( ishape<0 ) then

               newplm = inpelm

            end if

!           --- number of elements of this type

            npelem = kmeshg(ielgrp)
            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'ielgrp, inpelm, ishape, npelem, npnold', &
                                ielgrp, inpelm, ishape, npelem, npnold
               write(irefwr,*) 'ndim, niedge, nisurf, nivolm, npoint ', &
                                ndim, niedge, nisurf, nivolm, npoint
               write(irefwr,*) 'ikmeshc, iklemh ', &
                                ikmeshc, iklemh
               call prinin1 ( kmeshc(ikmeshc), npelem, inpelm, &
                              'kmescold' )

            end if

            if ( debug ) write(irefwr,*)  'before msh464/2'
            if ( npelem>0 ) &
               call msh464 ( kmeshc(ikmeshc), npelem, ibrpnt, istart, &
                             kmeshcnew(iklemh), ishape, coorh, npnold, &
                             ndim, 1, factor, niedge, nisurf, nivolm, &
                             npoint )

!           --- Increase ikmeshc and iklemh

            ikmeshc = ikmeshc + inpelm * npelem
            iklemh = iklemh + newplm * npelem

            if ( kelma==0 ) then

               npelm = newplm

            else

               kelold = kmesha(1+ielgrp)

               kmesha(1+ielgrp) = newplm + kmesha(ielgrp)

               if ( newplm>npelm ) npelm = newplm

            end if

            kelgrp = kelgrp + newplm

         end do

      end if

!     --- Save new number of nodes (may be changed in case of tetrahedrons)

      kcoor = ndim * npoint

      if ( kcoor>lcoor ) then

         call errint ( kcoor, 1 )
         call errint ( lcoor, 2 )
         call errsub ( 940, 2, 0, 0 )
         go to 1000

      end if

      lcoor  = kcoor
      lenkmeshc = iklemh-1

!     --- fill (if existent) KMESH part n

      if ( nvolms>0 ) then

         ipos  = 2 * nsurfs + 1
         kvlak = ipos

         do isurf = 1, nsurfs

!        --- Determine type of surfaceelements and number of elements

            inpelm = kmeshn(isurf         )
            ntal   = kmeshn(isurf + nsurfs)

            if ( inpelm==3 ) then

               ishape = 3

               if ( nisurf==-1 ) then

                  newplm = 3 + 3 * niedge + 1

               else

                  newplm = 3 + 3 * niedge + nisurf

               end if

            else if ( inpelm==4 ) then

               ishape = 5

               if ( nisurf==-1 ) then

                  newplm = (2+niedge) * (2+niedge)

               else

                  newplm = 4 + 4 * niedge + nisurf

               end if

            end if

            klakh(isurf         )  =     newplm
            klakh(isurf + nsurfs)  =     ntal

!           --- Independent of the original surface generator the new points
!               are determined

            if ( debug ) write(irefwr,*)  'before msh464/3'
            call msh464 ( kmeshn(ipos), ntal, ibrpnt, istart, &
                          klakh(kvlak), ishape, coorh, npnold, &
                          ndim, 0, factor, niedge, nisurf, &
                          nivolm, npoint )

            ipos  = ipos  + ntal * inpelm
            kvlak = kvlak + ntal * newplm

         end do

         kvlak = kvlak - 1

         if ( kvlak>lvlak ) then

            call errint ( kvlak, 1 )
            call errint ( lvlak, 2 )
            call errsub ( 940, 2, 0, 0 )
            go to 1000

         end if

         lvlak = kvlak

      end if

      if ( debug ) write(irefwr,*)  'before deallocate'
      deallocate ( istart, curves, curvefacts, ibrpnt, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istart' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspc'

      end if  ! ( debug )
      call erclos ( 'mshspc' )

      end subroutine mshspc
