      subroutine mshsrf ( iinput, rinput, iposi, jnew, &
                          lengc, holeinfo, maxholes, kmeshscp, irefplax )
! ======================================================================
!
!        programmer    Guus Segal
!        version 24.3  date 30-11-2014 Change isurf 23 in 99
!        version 24.2  date 22-01-2014 New call to mshsurfgen
!        version 24.1  date 02-09-2013 New call to msharr
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create the surfaces and write information to file 3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleplot
      use sepmoduledebug
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*), iposi, lengc, maxholes, &
              holeinfo(2,0:*), kmeshscp(*), irefplax
      double precision :: rinput(*)

!     holeinfo       o    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                               first curve of part i provided with a sign
!                               If the sign is positive the boundary is
!                               created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                               first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    integer input array see subroutine mesh
!     iposi         i/o   position in array iinput
!     irefplax       i    File reference number for plaxis
!     jnew           i    indication whether a new mesh must be created (true)
!                         or not
!     kmeshscp      i/o   Copy of kmeshs part 3
!                         The curves may be reversed so that all boundary
!                         parts have curves pointing in the same direction
!     lengc          o    length of kmesh part c
!     maxholes       i    Maximum number of holes found in a boundary of a
!                         surface
!     rinput         i    real input array, part with surface parameters
!                            umin, umax, vmin, vmax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprec, nsubsf, lenkmsc, nbndptsav, &
              lenkbndpt, npoh, nelh, idum(1), &
              ndimlc, npelmsv, irinpu, iscolr, ichnge, nholes, lenkmeshscp, &
              boundary(2,ncurvs+10), numcurvboun
      double precision plotfm, yfact, jac, trans(22)
      double precision, allocatable :: bcord3d(:,:)
      logical check, jnewsv, doublecurvs
      character (len=12) charac

!     bcord3d        Contains the coordinates of the 3d boundary
!     boundary       Array of size 2 x numcurvboun containing
!                    information of the boundary of the surface
!                    (1,i) contains the i-th curve number (including sign)
!                    (2,i) contains the starting address in kbndpt
!     charac         Help variable to store the text to be plotted
!     check          Logical variable
!     doublecurvs    If true the boundary contains double curves
!     ichnge         indicator how many elements were changed
!     idum           Dummy array
!     iprec          Surface number of surface to be copied etc.
!     irinpu         starting address in RINPUT local 3D surface parameters
!                    (u,v)
!     iscolr         Help variable to store the old value of dcolor
!     jac            jacobian of element
!     jnewsv         Help variable to save the value of jenw
!     lenkbndpt      Length of array kbndpt
!     lenkmeshscp    Length of kmeshs parts 1 + 2
!     lenkmsc        Length of array kmeshc
!     nbndptsav      Help parameter to store old value of nbndpt
!     ndimlc         Local dimension of space
!     nelh           Estimate of the number of elements
!     nholes         Number of holes in the boundary of a surface
!     npelmsv        Help variable to store the value of inpelm
!     npoh           Estimate of the number of points
!     nsubsf         Number of subsurfaces
!     numcurvboun    Number of curves in the boundary
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a value
!                    smaller than 1 cm, then the length is chosen equal to
!                    the default length stored in common block consta
!     trans          Contains information of the tranformation 3D-->2D
!                    Pos. 1-3:  t1
!                    Pos. 4-6:  t2
!                    Pos. 7-9:  t3
!                    Pos.10-18:  trans (inverse of transformation)
!                    Pos.19-21:  bcorut (coordinates of first point)
!     yfact          Multiplication factor in width direction of plot
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHKSBM         Check volume of submesh
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH2DQ         plot one surface (2D)
!     MSH3DQ         plot one surface in 3-D (no hidden lines)
!     MSHARR         map two-dimensional array coor into three-dimensional array
!                    coor3d
!     MSHCHACR       Change coordinates in array coor by calls to FUNCCOOR
!     MSHCHECKTRIA   Compute the minimal ratio between radius of inner circle
!                    and outer circle of all triangles in a surface
!     MSHCORTRI      Check for all triangles with 3 vertices on the boundary if
!                    there are neighbouring elements connected with the
!                    same property
!     MSHFILLNUMEXTC Fill array numnodextcurvs with number of nodes in all
!                    extra curves
!     MSHORI2D       Make the orientation of the elements counter clockwise
!     MSHR20         Fill array coor for centroid points in 5-point elements
!     MSHR22         Remove centroid points in 5-point elements in array kmeshc
!     MSHR23         Add centroid points in 5-point elements in array kmeshc
!     MSHRDCOMPSF    Read information of subsurfaces in general coordinates
!                    and topology array of composite surface
!     MSHSRFBOUN     Fill arrays bcord with coordinates of the boundary of a
!                    surface
!                    Fill array boundary with information of the boundary curves
!     MSHSRFCOPY     Check if kmeshs part 3 in surface to be copied is equal to
!                    kmeshcp.
!                    If not change direction in kmeshcp for present surface
!     MSHSRFHOLES    Reverse the direction of boundaries in kmeshscp in case
!                    there are holes and the orientation of the boundary part
!                    is different from the first one
!     MSHSURFGEN     Create subsurfaces
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLSCAL         Plot scales
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     PRINSURF       Print surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     379   Incorrect type of surface generator found
!    1330   The boundary of the surface, which is to be copied from another
!           surface consists of a different number of points as the originating
!           one
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('mshsrf')
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsrf'
         write(irefwr,1) 'isurf, inpelm, ndim', isurf, inpelm, ndim
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute local array bcord

      lengc  = 0
      jnewsv =  jnew
      if ( isurf==0 .or. isurf==20 ) go to 1000

      if ( isurf<0 .or. isurf==6 .or. isurf>24 .and. isurf/=99 ) then

!     --- Error 379:  Type of surface incorrect

         call errint ( isurf, 1 )
         call errint ( isurnr, 2 )
         call errsub ( 379, 2, 0, 0)
         go to 1000

      end if

      npelmsv = inpelm
      lenkmeshscp = 9*nsurfs+1
      irinpu = irnp2 + 3 * ncurvs-1

      allocate ( bcorold(ibcrds,lnbcor), bcord3d(3,nbndpt), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 3*nbndpt+ibcrds*lnbcor, 'bcorold' )
      if ( ierror/=0 ) go to 1000

!     --- Create boundary and store in bcord
!         If necessary make a mapping from 3d to 2d

      call mshsrfboun ( rinput, holeinfo, &
                        maxholes, nholes, boundary, &
                        numcurvboun, doublecurvs, trans, &
                        iprec, iinput, iposi, bcord3d )
      if ( ierror/=0 ) go to 1000

      nbndptsav = nbndpt

      if ( nextracrvs>0 ) then

!     --- nextracurvs>0, fill array numnodextcurvs with number of nodes
!         of each extra curve (not accumulated )
!         Array numnodextcurvs is stored in kbndptlc from position lnkbnd+1

         call mshfillnumextc ( iinput, icurvsmsh, kbndptlc(lnkbnd+1) )

      end if  ! ( nextracrvs>0 )

      check = .not. jnew

      if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
           isurf==16 .or. isurf==21 ) then

!     --- New mesh

         jnew = .true.
         if ( kelmr==0) then

!        --- kelmr = 0, no subsurfaces

            check = .true.

         else

!        --- New mesh with isurf = 8, 9, 14, 16, 21
!            There are surfaces of surfaces
!            Check if the surface to be copied is stand alone or not

            nsubsf = kmeshr(iprec+1) - kmeshr(iprec)
            if ( nsubsf==0 ) check = .true.

         end if
         if ( debug ) &
            write(irefwr,1) 'isurnr, iprec, ncurvs', &
                             isurnr, iprec, ncurvs
         if ( isurf/=21 ) &
            call mshsrfcopy ( isurnr, iprec, kmeshs, kmeshscp, ncurvs, &
                              lenkmeshscp )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'check ', check

      if ( check ) then

!     --- jnew = false old mesh
!              or
!         isurf = 8,9, 14 or 16 copy preceding mesh

         if(.not. jnew) iprec = isurnr
         npoh = npointlc
         nelh = nelemlc
         npointlc = kmeshh(1,iprec+1)
         nelemlc = kmeshh(2,iprec+1)
         lenkmsc = inpelm*nelemlc

         call dcopy ( npointlc*ndim, kmhsur(iprec)%coorsur, 1, coor2d, 1 )
         kmeshclc(1:lenkmsc) = kmhsur(iprec)%kmeshcsur

         if ( .not. jnew ) then

!        --- jnew = false: old mesh
!            for 5-point quadrilaterals and the mesh generators MSHOCE, MSHG
!            and pipe surface
!            the centroid is eliminated from the old mesh and npointlc reduced

            if ( ishape==9 .and. &
                 ( isurf==2 .or. isurf==3 .or. isurf==10 .or. &
                   isurf==99 ) .or. &
                 ishape==10 .and. (isurf<=4 .or. isurf==10 .or. &
                 isurf==13 .or. isurf==18 ) ) &
               call mshr22

         end if
         if ( jnew .and. isurf/=21 ) then

!        --- jnew = true  hence isurf = 8, 9, 14 or 16 read kbndptlc
!            in case of isurf = 21 (copy surface) we do not have a boundary

            lenkbndpt = kmeshh(4,iprec+1)+1
            lenkbndpt = lenkbndpt-kmhsur(iprec)%kbndptsur(lenkbndpt)-1
            if ( jnewsv .and. lenkbndpt/=nbndpt ) then

!           --- Error:  old KBNDPT is larger than new one

               call errint ( isurnr, 1 )
               call errint ( iprec, 2 )
               call errint ( nbndpt, 3 )
               call errint ( lenkbndpt, 4 )
               call errsub ( 1330, 4, 0, 0 )
               go to 1000
            end if

            kbndptlc(1:lenkbndpt) = kmhsur(iprec)%kbndptsur(1:lenkbndpt)

         else if ( isurf==2 ) then

!        --- jnew = false   hence isurf = 2  read irenum

            irenum(1:npointlc) = kmhsur(iprec)%renumsur(1:npointlc)

         end if

      else if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
                isurf==16 ) then

!     --- isurf = 8,9,14 or 16, jnew = true and preceding mesh
!                               consists of submeshes
!         Copy preceding mesh from subsurfaces
!         Consider subsurfaces of iprec and copy into kmeshclc and coor2d
!         Store local numbering in irenum

         lenkbndpt = kmeshh(4,iprec+1)+1
         kbndptlc(1:nbndpt) = kmhsur(iprec)%kbndptsur(1:nbndpt)
         call mshrdcompsf ( coor2d, irenum, iprec, nsubsf )
         if ( debug ) then
            call prinin1 ( kmeshclc, nelemlc, inpelm, 'topology' )
            call prinrl1 ( coor2d, npointlc, ndim, 'coor2d' )
         end if  ! ( debug )

      end if

!     --- surface generation

      call mshsurfgen ( jnew, npoh, nelh, trans, check, rinput, &
                        iposi, iinput, irinpu, &
                        holeinfo, nholes, doublecurvs, &
                        boundary, numcurvboun, irefplax, bcord3d )
      if ( ierror/=0 .or. isurf==20 ) go to 1000

      deallocate ( bcorold, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'bcorold' )

      if ( jnew .and. ( isurf==2 .or. isurf==3 ) &
           .and. (inpelm==3 .or. inpelm==6 .or. inpelm==7) ) then

!     --- In case of triangular elements, switch diagonals of "quadrilaterals"
!         at boundary if necessary

         call mshcortri ( npointlc, nelemlc, inpelm, kmeshclc, nbndpt, &
                          kbndptlc, coor2d, nholes, holeinfo )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Place extra points if necessary

      inpelm = npelmsv
      if ( ishape>=9 ) then

!     --- Special actions for 5-point quadrilaterals and 4-point triangles
!         For the mesh generators mshg and mshoce the extra point is added
!         npointlc is increased

         if ( isurf==2 .or. isurf==3 .or. isurf==10 .or. &
              ishape==10 .and. (isurf<=4 .or. isurf==10 .or. &
              isurf==13 .or. isurf==18 ) ) &
            call mshr23 ( kmeshclc, npointlc, nelemlc, inpelm )

!        --- The co-ordinates of the centroid are filled

         ndimlc = ndim
         if ( isurf>=1 .and. isurf<=4 ) ndimlc = 2
         call mshr20 ( kmeshclc, coor2d, nelemlc, inpelm, ndimlc )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 ) then

!     --- ndim = 2, check if all elements are defined counter clockwise
!                   if not rotate

         call mshori2d ( kmeshclc, inpelm, nelemlc, coor2d, ishape, ichnge, &
                         jac )

      end if
      if ( ierror/=0 ) go to 1000

      plotfm = rinput(1)
      yfact = rinput(2)
      if ( abs(plotfm)<=0.1d0 ) plotfm = defplo
      if ( abs(yfact)<=0d0 ) yfact = 1d0
      iscolr = dcolor
      if ( ndim==2 ) then

!     --- ndim = 2

         if ( debug ) then

!        --- debug = true print surface

            call prinsurf ( isurnr, npointlc, nelemlc, ndim, inpelm, &
                            coor2d, kmeshclc, 'surfaces' )
            call prinin ( kbndptlc, nbndpt, 'kbndptlc' )

         end if

!        --- plot surface if necessary

         if ( debug .or. jnew .and. iplot/=0 .and. isurnr>numsub ) then

!        --- Plot submesh

            call msh2dq ( bcord2d, coor2d, kmeshclc, plotfm, &
                          yfact, 0, 0 )

         end if

         if ( icheck>=2 ) then

!        --- Check ratios of inner and outer circles for triangles

            call mshchecktria ( ishape, inpelm, kmeshclc, nelemlc, &
                                isurnr, coor2d )

         end if  ! ( icheck>=2 )

         if ( debug .or. icheck>0 ) &
            call chksbm ( kmeshclc, nelemlc, ishape, inpelm, isurnr, coor2d, &
                          ndim, icheck, npointlc )

      else

!     --- ndim = 3
!         plot surface if necessary

         if ( isurf>=1 .and. isurf<=4 .or. &
              (isurf==17 .or. isurf==19) .or. isurf==99 ) then

!        --- isurf  = 1, 2, 3 or 4   map co-ordinates in 3d

            if ( icheck>=2 ) then

!           --- Check ratios of inner and outer circles for triangles

               call mshchecktria ( ishape, inpelm, kmeshclc, nelemlc, &
                                   isurnr, coor2d )

            end if  ! ( icheck>=2 )

            if ( checksign .and. icheck>0 ) &
               call chksbm ( kmeshclc, nelemlc, ishape, inpelm, isurnr, &
                             coor2d, 2, 1, npointlc )
            call msharr ( coor2d, trans, trans(4), coor3d, npointlc, &
                          bcord3d, kbndptlc, nbndpt )

            if ( iinp13/=0 ) then

!           --- iinp13 # 0   change coordinates of surface

               call mshchacr ( iinput(iinp13), coor3d, &
                               kmeshclc, idum, 3 )

            end if

            if ( debug ) then

!           --- debug = true print surface

               call prinsurf ( isurnr, npointlc, nelemlc, ndim, inpelm, &
                               coor3d, kmeshclc, 'surfaces' )
               if ( nbndpt>0 ) call prinin ( kbndptlc, nbndpt, 'kbndptlc' )

            end if

            if ( debug .or. jnew .and. iplot/=0 .and. &
                 isurnr>numsub ) then

!           --- Plot surface

               call msh3dq ( isurnr, coor3d, kmeshclc, npointlc, plotfm, &
                             yfact, inpelm, nelemlc )

            end if

            if ( debug .or. icheck>0 ) &
               call chksbm ( kmeshclc, nelemlc, ishape, inpelm, isurnr, &
                             coor3d, ndim, icheck, npointlc )

         else

!        --- other situations   plot and write to file 3

            if ( iinp13/=0 ) then

!           --- iinp13 # 0   change coordinates of surface

               call mshchacr ( iinput(iinp13), coor2d, &
                               kmeshclc, idum, 3 )

            end if

            if ( debug ) then

!           --- debug = true print surface

               call prinsurf ( isurnr, npointlc, nelemlc, ndim, inpelm, &
                               coor2d, kmeshclc, 'surfaces' )
               if ( nbndpt>0 ) call prinin ( kbndptlc, nbndpt, 'kbndptlc' )

            end if

            if ( jnew .and. iplot/=0 .and. isurnr>numsub ) &
               call msh3dq ( isurnr, coor2d, kmeshclc, npointlc, plotfm, &
                             yfact, inpelm, nelemlc )

            if ( icheck>0 ) &
               call chksbm ( kmeshclc, nelemlc, ishape, inpelm, isurnr, &
                             coor2d, ndim, icheck, npointlc )

         end if

      end if

      deallocate ( bcord3d, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'bcord3d' )
      if ( ierror/=0 ) go to 1000

      if ( debug .or. jnew .and. jtimes==2 .and. isurnr>numsub &
           .and. iplot/=0 ) then

!     --- jtimes = 2, plot the text "submesh ..." and close the plot

         call plscal
         charac(1:8) = 'SURFACE '
         write ( charac(9:12), 210 ) isurnr
210      format(i4)
         call plafp7 ( ale*0.3d0, 0.01d0, charac, 0d0, &
                       0.5d0, 12 )
         jtimes = 3
         call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
         jtimes = 1

      end if
      dcolor = iscolr
      if ( ierror/=0 ) go to 1000

      lengc = nelemlc*inpelm

      if ( nholes>0 ) then

!     --- nholes>0, check if some curves must be reversed

         call mshsrfholes ( nholes, kmeshs, kmeshscp, holeinfo, &
                            lenkmeshscp, ncurvs, isurnr )

      end if
      nbndpt = nbndptsav

1000  if ( debug .and. isurf>0 ) then

!     --- Debug information

         write(irefwr,1) 'isurnr', isurnr
         call prinin1 ( kmeshclc, nelemlc, inpelm, 'kmeshclc' )
         call prinrl1 ( coor2d, npointlc, ndim, 'coor2d' )
         call prinin ( kbndptlc, nbndpt, 'kbndptlc' )

         write(irefwr,*) 'End mshsrf'

      end if
      jnew = jnewsv
      call erclos('mshsrf')

      end subroutine mshsrf
