      subroutine mshz25 ( new, trian, npelm, npedge, &
                          mpoint, melem, mrec, mstk, &
                          npoint, nelem, kmeshc2d, &
                          iStk, iQ, iR, iS, iT, iU, Coor, Ref, &
                          nbcord, icurve, Tau, iblend )
! ======================================================================
!
!       programmer      Zdenek Vlasek
!       version 2.0     date 31-07-1990  n1..n4 are now local
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990-1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Submesh generator for 2-dim quadrilateral region, &
!   for triangular elements with 3, 6, 7 or 10 nodes.
!   User gives all boundary nodes, type of element and the number of
!   elements to be created along each side of the quadrilateral.
!   It is not suitable for too curved quadrilateral regions
!   or for very irregular distribution of boundary nodes, &
!   in which cases some general mesh generator should be used.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new, trian
      integer npelm, npedge, &
              mpoint, melem, mrec, mstk, &
              npoint, nelem, kmeshc2d ( npelm, melem ), &
              istk ( mstk, mrec ), &
              iq ( mrec ), ir ( mrec ), is ( mrec ), it ( mrec ), &
              iu ( mrec ), nbcord, icurve, iblend
      double precision coor ( 2, mpoint ), ref ( 2, mpoint ), &
                       tau ( 2, nbcord )

!     coor      i/o   array of length 2 x mpoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!                     Physical coordinates of all meshpoints
!                     ( the points number 1..npoint must be initialized
!                     when mshz21 is called, at that time npoint=nbcord )
!     iblend     i    User's choice:
!                     0  many  l o c a l  mappings are used:
!                        boundary nodes of any current quadrilateral or
!                        any quadrilateral element are mapped onto its
!                        reference rectangle   e q u i d i s t a n t l y, &
!                        Coon's blending formula is used for interpolation
!                        in the interior
!                        ( not suitable for a nonuniform distribution of
!                          boundary points )
!                     1  only one - g l o b a l - mapping is used:  user's
!                        boundary nodes are mapped onto the reference
!                        rectangle "Tau"   n o n e q u i d i s t a n t l y, &
!                        proportionaly to the length of arc, &
!                        and at the same time they are mapped onto the
!                        reference rectangle "Ref" equidistantly, then
!                        Coon's formula is used to map "Ref" into the
!                        physical "xy" plane, but the coefficients are
!                        pre-transformed this way:
!                        for given position [a,b] in "Ref" we find the
!                        four projections onto the boundary in "Ref"
!                        [a,    0], [a,    1], [0,    b], [1,    b]
!                        and then the four corresponding points
!                        [tau0, 0], [tau1, 1], [0, sig0], [1, sig1]
!                        on the boundary in "Tau", &
!                        ( it may happen tau0<>tau1, sig0<>sig1 ), now
!                        in "Tau" we connect the opposite points by
!                        straight lines and the coordinates [ aa, bb ]
!                        of their intersection point in "Tau"
!                        are used in Coon's formula instead of the
!                        "Ref"-coordinates [a,b]
!                        ( this approach is used in other Sepran parts )
!                     2  as iblend=0 but an "easy" topology is used
!                        for quadrilateral elements
!                        ( not suitable for long strips with big n1-n3 ), &
!                        for triangular elements the value iblend=2
!                                is useless, it works like iblend=0
!                     3  as iblend=1 but an easy topology is used
!                        for quadrilateral elements
!                        ( not suitable for long strips with big n1-n3 , &
!                        for triangular elements the value iblend=3
!                                is useless, it works like iblend=1
!     icurve     i    User's choice:
!                     0   all new element edges must be  s t r a i g h t
!                        ( it may cause problems when boundary is curved )
!                     1  all new element edges which contain no boundary
!                        point must be straight
!                        ( it may cause problems when boundary is curved ), &
!                        the edges connected to boundary may be curved
!                     2  all new element edges may be curved, &
!                        in accordance with the mapping chosen by iblend
!     iq         i    Integer array of length mrec.  The record must be
!                     i n i t i a l i z e d   by calling subroutine
!                     to describe boundary of a quadrilateral region, &
!                     the region will be subdivided here.
!                     Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!                     On return, the record iQ may contain a garbage.
!                     The items of the record are:
!                     iQ(1)    actual length of the record
!                     iQ(2)    number of elements along south
!                     iQ(3)    number of elements along east
!                     iQ(4)    number of elements along north
!                     iQ(5)    number of elements along west
!                     iQ(6..9)    point into iQ, they tell where in iQ
!                                 is the global nodenumber
!                                 of the  f i r s t  point of the s/e/n/west
!                     iQ(10..13)  pointers into iQ, they tell where in iQ
!                                 is the global nodenumber
!                                 of the  l a s t  point of the s/e/n/west
!                                 thus, &
!                                 if iQ(6) < iQ(10) then
!                                    the south nodenumbers occupy part
!                                    iQ ( iQ(6)..iQ(10) )
!                                    and are ordered  n o r m a l l y
!                                 else if iQ(6) > iQ(10) then
!                                    the south nodenumbers occupy part
!                                    iQ ( iQ(10)..iQ(6) )
!                                    and are  r e v e r s e  ordered
!                     iQ(14)   +1  when quadrilateral is positive
!                                      ( anticlockwise ) oriented
!                              -1  when it is negative oriented
!                     iQ(15.. )  global nodenumbers of the boundary
!                                points of the quadrilateral
!                                they start from one of the corners
!                                ( not necessery from the southwest )
!                                they are ordered in the positive or
!                                in the negative direction ( see iQ(14) )
!                                the last is the same as the first:
!                                iQ(iQ(1)) = iQ(15)
!     ir              Integer scratch arrays, each of length mrec, used to
!                     pass the sub-quadrilaterals to called subroutines, &
!                     the structure is as at iQ
!     is              Integer scratch arrays, each of length mrec, used to
!                     pass the sub-quadrilaterals to called subroutines, &
!                     the structure is as at iQ
!     istk            Integer scratch array (mstk,mrec).
!                     The stack which can keep upto mstk records, &
!                     the current number of records is nstk.
!                     Each record represents one quadrilateral boundary
!                     and is built like the iQ(mrec), see mshz21, &
!                     the records wait in the stack for processing
!     it              Integer scratch arrays, each of length mrec, used to
!                     pass the sub-quadrilaterals to called subroutines, &
!                     the structure is as at iQ
!     iu              Integer scratch arrays, each of length mrec, used to
!                     pass the sub-quadrilaterals to called subroutines, &
!                     the structure is as at iQ
!     kmeshc2d      o    Integer array (npelm,melem). Global nodenumbers for
!                     all elements created.
!     melem      i    Upper bound for nelem, see mshzde
!     mpoint     i    Upper bound for npoint, see mshzde
!     mrec       i    Upper bound for length of iQ
!     mstk       i    Upper bound for number of records on the stack
!     nbcord     i    Number of user's boundary points, see mshzde
!     nelem      o    Number of elements created
!     new        i    indication whether the mesh is new (true) or not (false)
!     npedge     i    Number of points per one element edge, both ends
!                     included
!     npelm      i    Number of points per element
!     npoint    i/o   Current number of all meshpoints.
!                     It must be initialized on input, see mshzde
!     ref        i    Real array (2,mpoint).  Reference coordinates of all
!                     meshpoints, the inner-element points are not needed.
!                     The coordinates of points in the reference
!                     quadrilateral [0,1] x [0,1], &
!                     in the same order as in Coor(2,mpoint), &
!                     the boundary points are  e q u i d i s t a n t
!                     on each of the four sides, &
!                     -- it is used only in the case  iblend=1 or 3
!                        and then it must be initialized on input
!     tau        i    Real array (2,nbcord). Reference nonequidistant
!                     coordinates of user's boundary points, see mshz21.
!                     The coordinates of  b o u n d a r y  points
!                     in the reference quadrilateral [0,1] x [0,1], &
!                     in the same order as in bCord(2,nbcord), &
!                     The points are  n o n - e q u i d i s t a n t
!                     on each of the four sides, in accordance with
!                     the relative length of the arc meassured in the
!                     physical plane, &
!                     -- it is used only in the case  iblend=1 or 3
!                        and then it must be filled on input
!     trian      i    Indication whether triangles (true) or quadrilaterals
!                     are created (false)
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nstk, n1, n2, n3, n4

!     n1         Number of elements along the  south  side
!                of the current quadrilateral
!     n2         Number of elements along the  east  side
!                of the current quadrilateral
!     n3         Number of elements along the  north side
!                of the current quadrilateral
!     n4         Number of elements along the  west side
!                of the current quadrilateral
!     nstk       The current number of items on the stack
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHZ12    decomposition of one quadrilateral with irregular but
!               elementary values n1,n2,n3,n4,  into three or more parts, &
!               each of them is processed directly, without stack
!     MSHZ18    To change the inner pointers in the record like iQ, &
!               due to a choice of new first vertex, &
!               the orientation is not influenced
!     MSHZ19    To pop one record iQ from the stack, for the processing
!     MSHZ22    To divide a quadrilateral region regularily, &
!               into quadrilaterals, applicable when n2=n4 and n1=n3, &
!               into triangles,      applicable when n2=n4
!     MSHZ23    To change the inner pointers in the record like iQ, &
!               due to the change of the orientation, change iQ(14), &
!               the first vertex remains the same
!     MSHZ24    to split a quadrilateral region into two quadrilateral
!               parts, both parts are put on stack, &
!               the cut leads from south to north, &
!               this subroutine is the only source or  r e c u r s i o n
!               ( but no recursive call is used )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   see mshzde
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

!     --- initialize the stack and  nelem

      nstk = 0
      nelem = 0

!     --- stack loop ( repeat ... until stack_is_empty ), by "goto 10"

   10 continue

         if ( nstk>0 ) then

!        --- get one record from the stack

            call mshz19 ( mstk, mrec, nstk, iQ, iStk )

         else

!        --- iQ is the input parameter, from mshzde

         end if

         n1 = iQ(2)
         n2 = iQ(3)
         n3 = iQ(4)
         n4 = iQ(5)

!        --- if possible, divide regularily

         if ( n1==n3 .and. n2.eq.n4 ) then

            call mshz22 ( new, trian, mrec, iQ, iR, iU, &
               npelm, npedge, &
               mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
               nbcord, icurve, Tau, iblend )

         else

!        --- some irregular division needed

!             ensure  n1<=n3, to reduce branching

            if ( n1>n3 ) then
               call mshz18 ( iQ, 3, n1, n2, n3, n4 )
            end if

!           --- ensure  n4<=n2, to reduce branching

            if ( n4>n2 ) then
               call mshz18 ( iQ, 2, n1, n2, n3, n4 )
               call mshz23 ( iQ, n1, n2, n3, n4 )
            end if

!           --- apply a recursive splitting algorithm

!                ensure  n2/n4 <= n3/n1
!                if the ratios are equal, then ensure  n1>=n4, &
!                                to reduce branching
!                  ( other inequalities then in mshz21 )

            if ( n2 * n1>n3 * n4 .or. &
               ( n2 * n1==n3 * n4 ) .and. ( n1<n4 ) ) then
               call mshz23 ( iQ, n1, n2, n3, n4 )
            end if

!           --- remember !!!   1 <= n2/n4 <= n3/n1  !!!

            if ( n2==n4 ) then

!           --- horizontal layers, it is job for  mshz22

               call mshz22 ( new, trian, mrec, iQ, iR, iS, &
                  npelm, npedge, &
                  mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                  nbcord, icurve, Tau, iblend )

!              --- remaining cases: 1 < n2/n4 <= n3/n1

            else if ( n1==1 .and. n4.eq.1 ) then

!           --- n1=n4=1, 1 < n2 <= n3, &
!               divide  iQ  into three parts for mshz22

               call mshz12 ( new, trian, mrec, iQ, iR, iS, iT, iU, &
                  npelm, npedge, &
                  mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                  nbcord, icurve, Tau, iblend )

            else

               if ( n1<n4 ) then

!              --- horizontal cut, leading from west to east

                  call mshz23 ( iQ, n1, n2, n3, n4 )

!                 --- now the cut should lead from south to north

               end if

!              --- vertical cut, from south to north

               call mshz24 ( trian, mrec, iQ, iR, &
                  npedge, mstk, nstk, iStk, &
                  mpoint, npoint, Coor, Ref, nbcord, Tau, iblend )

            end if

         end if

!        --- no case was left !!!  use the stack

      if ( nstk>0 ) goto 10

      end
