      subroutine sepmsh077 ( irefc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write kmesh parts v and w to file irefc
!     Old subroutine msh077
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc

!     irefc      i    dataset reference number for reading (>0 formatted, &
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmsv
      character (len=80) text

!     lenkmsv        Length of array KMESH part v
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmsh077' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmsh077'
         write(irefwr,1) 'kelmv/w', kelmv, kelmw
         write(irefwr,1) 'irefc', irefc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenkmsv = size(kmeshv)
      if ( kelmv>0 ) then

!     --- Kmesh part v has been filled

         if ( irefc>=0 ) then

!        --- Write kmeshv (formatted) to backing storage

            write(irefc,110) '//Kmeshv'
            write(irefc,100) lenkmsv
            if ( lenkmsv>0 ) write(irefc,100) kmeshv
100         format(10i8)
110         format(a)

         else
            text = '//Kmeshv'
            write(-irefc) text
            write(-irefc) lenkmsv
            if ( lenkmsv>0 ) write(-irefc) kmeshv

         end if

      end if
      if ( kelmw>0 ) then

         if ( debug ) then
            write(irefwr,1) 'nelgrp, nintprop, nrealprop', &
                             nelgrp, nintprop, nrealprop
            if ( nrealprop>0 ) &
              call prinrl1 ( realprop, nelgrp, nrealprop, 'realprop' )
         end if  ! ( debug )
         if ( irefc>=0 ) then

!        --- Write kmeshw (formatted) to backing storage

            write(irefc,110) '//Kmeshw'
            write(irefc,100) nintprop, nrealprop
            if ( nintprop>0 ) write(irefc,100) intprop
            if ( nrealprop>0 ) write(irefc,120) realprop
120         format(1x, 3e19.10)

         else
            text = '//Kmeshw'
            write(-irefc) text
            write(-irefc) nintprop, nrealprop
            if ( nintprop>0 ) write(-irefc) intprop
            if ( nrealprop>0 ) write(-irefc) realprop

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmsh077'

      end if  ! ( debug )
      call erclos ( 'sepmsh077' )

      end
