      subroutine msh316 ( bcord, nx, ny, nz, ivolnr )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 06-03-1999 Extra parameter
!        version  1.2  date 28-02-1999 Include commons
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 07-01-1985
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  check whether the input by bcord of msh315 is correct
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(*)
      integer nx, ny, nz, ivolnr

!     bcord      i    array containing the coordinates of the boundaries
!     ivolnr     i    volume sequence number
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, dist, ds, eps
      integer i1, i2, i4, i5, ist1, ist2, ist3, kvlak1, kvlak2, &
              kvlak3

!     afst
!     dist       Distance between two points
!     ds         Minimum distance between two points along a curve
!     eps        Local accuracy
!     i1
!     i2
!     i4
!     i5
!     ist1
!     ist2
!     ist3
!     kvlak1
!     kvlak2
!     kvlak3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     MSH304    determine distance of nodal points n and m
!     MSH307    compute sum of distances ist1-ist2 and ist1-ist3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     299   Surfaces have not been created in the correct sequence
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh316' )

!     --- helpvariables

      kvlak1=nx*ny+(nx-1)*(ny-1)/2
      kvlak2=nx*nz+(nx-1)*(nz-1)/2
      kvlak3=ny*nz+(ny-1)*(nz-1)/2

!     --- determine value for eps

      i1=1
      i2=nx
      i4=kvlak1-nx+1
      i5=kvlak1+kvlak2-nx+1
      call msh304(i1,i2,bcord,dist)
      afst=dist/nx
      call msh304(i1,i4,bcord,dist)
      afst=dist/ny+afst
      call msh304(i1,i5,bcord,dist)
      afst=dist/nz+afst

      eps=sqreps*afst*100

!     --- point 1

      ist1=0
      ist2=3*kvlak1
      ist3=3*(kvlak1+2*kvlak2+kvlak3)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+1, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( kvlak1+2*kvlak2+kvlak3+1, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 2

      ist1=3*(nx-1)
      ist2=3*(kvlak1+nx-1)
      ist3=3*(kvlak1+kvlak2)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 2, 2 )
         call errint ( nx, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+nx, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( kvlak1+kvlak2+1, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 3

      ist1=3*(kvlak1-1)
      ist2=3*(kvlak1+kvlak2+ny-1)
      ist3=3*(kvlak1+kvlak2+kvlak3+nx-1)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 3, 2 )
         call errint ( kvlak1, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+kvlak2+ny, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( kvlak1+kvlak2+kvlak3+nx, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 4

      ist1=3*(kvlak1-nx)
      ist2=3*(kvlak1+kvlak2+kvlak3)
      ist3=3*(kvlak1+2*kvlak2+kvlak3+ny-1)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 4, 2 )
         call errint ( kvlak1-nx+1, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+kvlak2+kvlak3+1, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( kvlak1+2*kvlak2+kvlak3+ny, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 5

      ist1=3*(kvlak1+kvlak2-nx)
      ist2=3*(kvlak1+2*kvlak2+2*kvlak3-ny)
      ist3=3*(kvlak1+2*kvlak2+2*kvlak3)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 5, 2 )
         call errint ( kvlak1+kvlak2-nx+1, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+2*kvlak2+2*kvlak3-ny+1, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( kvlak1+2*kvlak2+2*kvlak3+1, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 6

      ist1=3*(kvlak1+kvlak2-1)
      ist2=3*(kvlak1+kvlak2+kvlak3-ny)
      ist3=3*(kvlak1+2*kvlak2+2*kvlak3+nx-1)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 6, 2 )
         call errint ( kvlak1+kvlak2, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+kvlak2+kvlak3-ny+1, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( kvlak1+2*kvlak2+2*kvlak3+nx, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 7

      ist1=3*(kvlak1+kvlak2+kvlak3-1)
      ist2=3*(kvlak1+2*kvlak2+kvlak3-1)
      ist3=3*(2*kvlak1+2*kvlak2+2*kvlak3-1)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 7, 2 )
         call errint ( kvlak1+kvlak2+kvlak3, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+2*kvlak2+kvlak3, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( 2*kvlak1+2*kvlak2+2*kvlak3, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- point 8

      ist1=3*(kvlak1+2*kvlak2+kvlak3-nx)
      ist2=3*(kvlak1+2*kvlak2+2*kvlak3-1)
      ist3=3*(2*kvlak1+2*kvlak2+2*kvlak3-nx)
      call msh307(bcord,ist1,ist2,ist3,ds)
      if (ds.gt.eps) then
         call errint ( ivolnr, 1 )
         call errint ( 5, 2 )
         call errint ( kvlak1+2*kvlak2+kvlak3-nx+1, 3 )
         call erreal ( bcord(ist1+1), 1 )
         call erreal ( bcord(ist1+2), 2 )
         call erreal ( bcord(ist1+3), 3 )
         call errint ( kvlak1+2*kvlak2+2*kvlak3, 4 )
         call erreal ( bcord(ist2+1), 4 )
         call erreal ( bcord(ist2+2), 5 )
         call erreal ( bcord(ist2+3), 6 )
         call errint ( 2*kvlak1+2*kvlak2+2*kvlak3-nx+1, 5 )
         call erreal ( bcord(ist3+1), 7 )
         call erreal ( bcord(ist3+2), 8 )
         call erreal ( bcord(ist3+3), 9 )
         call errsub ( 299, 5, 9, 0 )
      end if

!     --- end of test

      call erclos ( 'msh316' )

      end
