      subroutine msh356 ( kmeshc, nelem, npoint, inpelm, iwork, &
                          ibuurp, lengnb, linear, kbndbt, klp, nlp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!        version  1.0  date 15-05-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array iwork which contains the number of neighbour
!    points accumulated
!    and fill array ibuurp which contains the nodal point
!    numbers of the neighbours in accordance with iwork
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, kbndbt(*), iwork(*), ibuurp(*), &
              inpelm, kmeshc(*), lengnb, linear, klp(2,*), nlp

!     ibuurp     o    array with the neighbour nodal points
!     inpelm     i    number of points per element
!     iwork      o    array containing pointers for array ibuurp
!     kbndbt     i    array with node numbers of boundary points
!     klp        i    linear (=1) or quadratic (=2) case
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     lengnb     i    declared length array ibuurp
!     linear     i
!     nelem      i    Number of elements
!     nlp        i    ?
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,j,ih,ij,isum,i1,i2,i3,i4,ilp

!     i          Counting variable
!     i1
!     i2
!     i3
!     i4
!     ih         Help variable to store a constant temporarily
!     ij
!     ilp
!     isum       Integer variable to compute a sum
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put string in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     MSHD07    fill nodal point number ij in right position of array ibuurp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array iwork

      do i = 1 , npoint

          iwork(i) = 0

      end do

!     --- Fill iwork(ij) with the number of elements point ij is situated in

      ilp = inpelm

      if ( inpelm.eq.9 ) ilp = ilp - 1

      do i = 1 , nelem

          ih = inpelm * (i-1)

          do j = 1 , ilp, linear

             ij = kmeshc(ih+j)
             iwork(ij) = iwork(ij) + 1

          end do

      end do

!     --- All boundary points have one neighbour extra

      do ilp = 1, nlp
         do i = klp(1,ilp) , klp(2,ilp) , linear
            ij = kbndbt(i)
            iwork(ij) = iwork(ij) + 1
         end do
      end do

!     --- Rearrange and accumulate

      isum = 0

      do i = 1 , npoint

          iwork(i) = iwork(i) + isum

          isum     = iwork(i)

      end do

!     --- isum positions needed for the neighbour points

      if ( isum.gt.lengnb ) then

!     --- Array ibuurp too small

         call eropen ( 'msh356' )
         call errint ( isum, 1 )
         call errint ( lengnb, 2 )
         call errchr ( 'buurp', 1 )
         call errsub ( 550, 2, 0, 1 )
         call erclos ( 'msh356' )
         go to 1000

      end if

!     --- Clear array ibuurp

      do i = 1 , isum

          ibuurp(i) = 0

      end do

!     --- Fill array ibuurp

      if ( inpelm.eq.3 .or. inpelm.eq.6 ) then

           do i = 1 , nelem

               ih = inpelm * (i-1) + 1
               i1 = kmeshc(ih         )
               i2 = kmeshc(ih+  linear)
               i3 = kmeshc(ih+2*linear)

               call mshd07(ibuurp,iwork,i1,i2)
               call mshd07(ibuurp,iwork,i1,i3)
               call mshd07(ibuurp,iwork,i2,i1)
               call mshd07(ibuurp,iwork,i2,i3)
               call mshd07(ibuurp,iwork,i3,i1)
               call mshd07(ibuurp,iwork,i3,i2)

           end do

      else if ( inpelm.eq.4 .or. inpelm.eq.9 ) then

           do i = 1 , nelem

               ih = inpelm * (i-1) + 1
               i1 = kmeshc(ih         )
               i2 = kmeshc(ih+  linear)
               i3 = kmeshc(ih+2*linear)
               i4 = kmeshc(ih+3*linear)

               call mshd07(ibuurp,iwork,i1,i2)
               call mshd07(ibuurp,iwork,i1,i4)
               call mshd07(ibuurp,iwork,i2,i1)
               call mshd07(ibuurp,iwork,i2,i3)
               call mshd07(ibuurp,iwork,i3,i2)
               call mshd07(ibuurp,iwork,i3,i4)
               call mshd07(ibuurp,iwork,i4,i3)
               call mshd07(ibuurp,iwork,i4,i1)

           end do

      end if

1000  end
