      subroutine msh443 ( kelem , nelem , kelemh, npoint, ibrpnt, &
                          istart)
! =======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.1    date   27-04-1995 New basis for surface values
!        version 2.0    date   14-02-1995 Redesign, use ibrpnt and
!                                        istart)
!        version 1.0    date   27-03-1992
!
!
!
!   copyright (c) 1992 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Place new point in barycentre of triangle, store coordinates
!     in array coorh and create new elements in array kelemh
!
! ***********************************************************************
!
!     KEYWORDS
!
!     2d
!     create_point
!     mesh
!
! ***********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS
!
      implicit none

      integer kelem(*), ibrpnt(*), istart(*), nelem, npoint, kelemh(*)

!     ibrpnt  i   neighbour array
!     istart  i   array with starting positions in ibrpnt
!     kelem   i   element array
!     kelemh  o   new element array
!     nelem   i   number of elements
!     npoint  i   old number of nodes
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, ia, ib, ic, ih, nelhel
!
!     i        loop variable
!     i1       node
!     i2       node
!     i3       node
!     i4       node
!     i5       node
!     i6       node
!     i7       node
!     ia       help to store lowest node number
!     ib       help to store middle node number
!     ic       help to store largest node number
!     ih       help variable for node number
!     nelhel   local element number
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     MSH403   Determine node number in middle of line segment
!     MSH405   Place element in element array
!     MSH461   Order the node numbers of the triangle
!
! ***********************************************************************
!
!     I / O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all elements of MESHTP = 3
!       Check node numbers in middle of sides
!       Create new elements of MESHTP = 5
!
! =======================================================================

!     Check each element; place nodenumbers of new elements in array
!     kelemh

      do 100 i = 1 , nelem

          i1 = kelem(3*i-2)
          i2 = kelem(3*i-1)
          i3 = kelem(3*i  )

!         Compute middle node numbers

          call msh403( npoint, i1, i2, ibrpnt, istart, i4 )
          call msh403( npoint, i2, i3, ibrpnt, istart, i5 )
          call msh403( npoint, i3, i1, ibrpnt, istart, i6 )

!         Sort the node numbers

          call msh461 ( i1, i2, i3, ia, ib, ic )

          ih = - ( ib + npoint * ic )

          call msh403( npoint, ia, ih, ibrpnt, istart, i7 )

!         Fill new elements in kelemh

          nelhel = 3 * i - 2
          call msh405( kelemh, nelhel, i1, i4, i7, i6 )
          nelhel = nelhel + 1
          call msh405( kelemh, nelhel, i4, i2, i5, i7 )
          nelhel = nelhel + 1
          call msh405( kelemh, nelhel, i7, i5, i3, i6 )

100   continue

      end
