      subroutine msh459 ( ibrp, ibrpnt, istart, kelemh, i1, i2, i3, &
                          npoint )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.2    date   18-04-1997 Extension with eropen/erclos
!        version 1.1    date   27-04-1995 New basis for triangle
!        version 1.0    date   13-02-1995
!
!
!
!   copyright (c) 1995-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Place nodes of triangle in neighbour array for determination
!     of barycenter of triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     mesh_generation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ibrp, ibrpnt(*), istart(*), kelemh(*), i1, i2, i3, &
              npoint

!     i1      i   node of triangle to be considered
!     i2      i   node of triangle to be considered
!     i3      i   node of triangle to be considered
!     ibrp    i   helpvariable for number of expected neighbours
!     ibrpnt  i   array to store neighbours
!     istart  i   array for starting positions
!     kelemh  i   helparray for too many neighbours, i.e. more than
!                 ibrp neighbours
!     npoint  i   number of nodes in original mesh
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ia, ib, ic, ih

!     ia      smallest node number
!     ib      middle node number
!     ic      largest node number
!     ih      help node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Resets old name of previous subroutine of higher level
!     EROPEN   Produces concatenated name of this subroutine
!     MSH402   Place characteristics of line in arrays ibrpnt and istart
!     MSH461   Order three integers from small to large
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the smallest and largest node number
!
!     Determine helpnodenumber
!
!     Place characteristics in neighbour arrays
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh459' )

!     --- Order the node numbers

      call msh461 ( i1, i2, i3, ia, ib, ic )

!     --- Place in neighbour array

      ih = - ( ib + npoint * ic )

      call msh402 ( npoint, ibrp, ibrpnt, istart, ia, ih, kelemh )

      call erclos ( 'msh459' )

      end
