      subroutine msh462( ibrp  , ibrpnt, istart, npoint, kelemh, &
                         niedge, nisurf, nivolm )
! =======================================================================
!
!
!         programmer   niek praagman
!         version 1.1  date  27-04-1995 possibility to create different
!                                      groups of new points; introduction
!                                      of basis for triangle, quadrilateral
!                                      and volume elements )
!         version 1.0  date  13-02-1995
!
!
!
!   copyright (c) 1995 - 1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Fill the nodal point numbers in the right positions of array ibrpnt
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     neighbour
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer npoint, ibrpnt(*), ibrp, istart(*), kelemh(*), niedge, &
              nisurf, nivolm

!     ibrp        i    number of places available in IBRPNT for each
!                      point
!     ibrpnt     i,o   array containing sequentially the neighbour- &
!                      points
!     istart      i    array of startposition for each nodal point
!                      in ibrpnt
!     kelemh      o    array to store extra neighbours if a point has
!                      more than ibrp neighbours
!     niedge      i    number of nodes to be placed on lines
!
!     nisurf      i    number of nodes to be placed in surface element
!
!     nivolm      i    number of nodes to be placed in volume element
!
!     npoint      i    number of nodes before operation
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, ii, itotal, ibrlen, nummer, iextra, jstart, jstend, &
              nbr, npt, ia, ib, npnold

!     i         loop variable
!     ia        first node of segment
!     ib        second node of segment
!     ibrlen    declared length of ibrpnt array
!     iextra    indicator whether there are extra points (=1) or not (=0)
!     ii        loopvariable
!     itotal    total number of points computed
!     jstart    startposition in ibrpnt
!     jstend    endposition in ibrpnt
!     nbr       local variable to store the number of neighbours for
!               actual point considered
!     npnold    number of points before refining
!     npt       indicator for actual number of positions needed
!     nummer    all extra points on the lines
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name from string of calling subroutines
!     EROPEN  Concatenate name to string of calling subroutines
!     ERRINT  Place integer in error array
!     ERRSUB  Produce error message
!     INSTOP  Stop SEPRAN execution
!     MSH402  Place line in neighbour array
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
!     1274    No place left for neighbour
!     1275    Not enough positions in neighbourarray
!
! ***********************************************************************
!
!     METHOD
!
!     Skip all empty places in neighbour array
!
!     Check whether enough places have been declared
!
!     Make shift in neighbour array
!
!     Place new nodal-point numbers
!     ( If line then niedge, if surface then nisurf and
!       if volume then nivolm )
!
! =======================================================================

      call eropen( 'msh462' )

!     Make shift in array for all neighbour points

      nummer = 0

!     First skip all empty places

      do 150 i = 1 , npoint

          jstart = ibrp * (i-1) + 1
          jstend = ibrp *  i

          do 140 ii = jstart, jstend

              if ( ibrpnt(ii).ne.0 ) then

!                Place neighbour point

                 nummer         = nummer + 1
                 ibrpnt(nummer) = ibrpnt(ii)

              end if

140       continue

150   continue

      itotal = nummer

!     Number of positions needed is 2 * nummer and number of
!     positions declared is

      ibrlen = ibrp * ( npoint + 1 )

      if ( ibrlen.lt.2 * nummer ) then

!          Error, not enough positions

           call errint ( 2*nummer, 1 )
           call errint ( ibrlen  , 2 )
           call errsub ( 1275, 2, 0, 0 )
           call instop

      end if

!     Make shift

      npt    = ibrlen
      iextra = 0

      do 160 i = npoint, 1, -1

!          Determine number of neighbours

           nbr = istart(i)

           if ( nbr.gt.ibrp ) then

!               There are extra points

                iextra = 1

                do 155 ib = 1 , nbr-ibrp

                    ibrpnt(npt) = 0
                    npt = npt - 1

155             continue

                itotal = itotal + nbr - ibrp
                nbr = ibrp

           end if

           do 158 ib = 1 , nbr

                ibrpnt( npt ) = ibrpnt( nummer )
                npt = npt - 1
                nummer = nummer - 1

158        continue

           if ( npt.lt.nummer ) then

!               Error, too few positions

                call errsub ( 1274, 0, 0, 0 )
                call instop

           end if

160   continue

      if ( ibrlen.lt.2 * itotal ) then

!          Error, not enough positions declared

           call errint ( 2*itotal, 1 )
           call errint ( ibrlen  , 2 )
           call errsub ( 1275, 2, 0, 0 )
           call instop

      end if

!     Finally shift all points

      do 170 i = 1 , itotal

          ibrpnt(i) = ibrpnt( i+npt )

170   continue

      nummer = itotal
      jstend = 0

      do 200 i = 1, npoint

          jstart = jstend + 1
          jstend = jstend + istart(i)

          istart(i) = jstart

200   continue

      itotal           =          0
      istart(npoint+1) = jstend + 1

      if ( kelemh(1).ne.0 ) then

           if ( iextra.eq.0 ) then

              call errsub ( 1274, 0, 0, 0 )
              call instop

           end if

!          Place extra neighbours (more than ibrp/2)

           itotal = kelemh(1)

           do 210 i = 1 , itotal

               ia = -kelemh( 2 * i     )
               ib =  kelemh( 2 * i + 1 )

               call msh402( npoint, ibrp, ibrpnt, istart, ia, ib, &
                            kelemh )

210        continue

      end if

!     Check declared length of array ibrpnt

      if ( 2 * nummer.gt.ibrlen ) then

         call errint ( 2*nummer, 1 )
         call errint ( ibrlen  , 2 )
         call errsub ( 1275, 2, 0, 0 )
         call instop

      end if

!     Set number of points for copiing later

      npnold = npoint

      do 250 i = 2, npoint+1

          istart(i) = 2 * istart(i)-1

250   continue

!     Place new nodal-point numbers
!     ( If line then niedge, if surface then nisurf and
!       if volume then nivolm )

      do 300 i = nummer, 1, -1

          ibrpnt(2*i-1) = ibrpnt(i)
          ibrpnt(2*i  ) = 0

300   continue

      do 310 i = 1 , nummer

          ibrpnt(2*i) = npoint + 1

          if ( abs(ibrpnt(2*i-1)).lt.npnold ) then

!            Line : skip niedge points

             npoint = npoint + niedge

          else if ( abs(ibrpnt(2*i-1)) .lt. &
                    npnold * ( npnold + 1 ) ) then

             if ( ibrpnt(2*i-1).gt.0 ) then

!               Surface (Quadrilateral) : skip nisurf points

                if ( nisurf.eq.-1 ) then

!                  Skip niedge * niedge points

                   npoint = npoint + niedge * niedge

                else

                   npoint = npoint + nisurf

                end if

             else

!               Surface (Triangle) : skip nisurf points

                if ( nisurf.eq.-1 ) then

!                  Skip 1 point

                   npoint = npoint + 1

                else

                   npoint = npoint + nisurf

                end if

             end if

          else

!            Volume

             if ( nivolm.eq.-1 ) then

!               Volume : skip niedge * niedge * niedge points

                npoint = npoint + niedge * niedge * niedge

             else

                npoint = npoint + nivolm

             end if

          end if

310   continue

      call erclos( 'msh462' )

      end
