      subroutine mshchkstapl ( kstapl, lenkstapl, text, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-11-2013 Extra parameter isurnr
!        version  1.0  date 10-02-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check the contents of array kstapl
!     It is checked if a node that is on the left-hand side is also the
!     right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkstapl, kstapl(2,lenkstapl), isurnr
      character (len=*) text

!     isurnr         i    Surface sequence number
!     kstapl         i    Array containing the actual boundary build by
!                         pairs of nodes
!     lenkstapl      i    Number of pairs in kstapl
!     text           i    String variable containing some text to be
!                         printed as part of the debugging
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, iseq1(lenkstapl), iseq2(lenkstapl), &
              iwork(lenkstapl), ihelp(lenkstapl)
      logical found

!     found          If true an error has been found
!     i              Counting variable
!     ihelp          integer work array to be used for the sorting
!     iseq1          Array containing the nodes of the left-hand side
!                    of the pairs after sorting
!     iseq2          Array containing the node after sortings of the right-hand
!                    side of the pairs
!     iwork          integer work array to store the nodes of either the
!                    left-hand side or the right-hand side of the pairs
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3152   Error in boundary of surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchkstapl' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshchkstapl'
         call prinin1 ( kstapl, lenkstapl, 2, 'kstapl' )

      end if
      if ( lenkstapl==0 ) go to 1000

!     --- Check array by sorting each row of pairs

      iwork = kstapl(1,:)

      call chsort ( iwork, ihelp, lenkstapl )
      iseq1 = iwork(ihelp)

      iwork = kstapl(2,:)
      call chsort ( iwork, ihelp, lenkstapl )
      iseq2 = iwork(ihelp)

      found = .false.
      do i = 1, lenkstapl
         if ( iseq1(i)/=iseq2(i) ) then

!        --- first point found in second column

            found = .true.
            j = i
            go to 100

         end if

      end do

100   if ( found ) then

!     --- Both rows do not contain the same node numbers

         call errchr ( text, 1 )
         call errint ( isurnr, 1 )
         call errint ( j, 2 )
         call errsub ( 3152, 2, 0, 1 )
         do i = 1, lenkstapl
            write(irefwr,110) i, iseq1(i), iseq2(i)
110         format ( i5, ':  ', 2i6 )
         end do
         call prinin1 ( kstapl, lenkstapl, 2, 'kstapl' )
         go to 1000

      end if

1000  call erclos ( 'mshchkstapl' )
      if ( debug ) write(irefwr,*) 'End mshchkstapl'

      end
