      subroutine mshcopsel ( kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Copy a selected set of parts of kmesh1 to array kmesh2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     mesh
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2

!     kmesh1         i    Standard SEPRAN array containing information of mesh 1
!     kmesh2        i/o   Contains copy of old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqkm1, iseqkm2

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopsel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopsel'

      end if

      iseqkm1 = kmesh1
      iseqkm2 = kmesh2

!     --- kmesh d and e

      if ( km(iseqkm1)%kmesh(18)>0) then
         if ( associated(km(iseqkm2)%kmeshd) ) deallocate(km(iseqkm2)%kmeshd)
         length = size(km(iseqkm1)%kmeshd)
         allocate(km(iseqkm2)%kmeshd(length))
         km(iseqkm2)%kmeshd = km(iseqkm1)%kmeshd
         km(iseqkm2)%kmshd1 => km(iseqkm1)%kmeshd(1:npoint+1)
         km(iseqkm2)%kmshd2 => km(iseqkm1)%kmeshd(npoint+2:length)
      end if  ! ( km(iseqkm1)%kmesh(18)>0)

      if ( km(iseqkm1)%kmesh(19)>0) then
         if ( associated(km(iseqkm2)%kmeshe) ) deallocate(km(iseqkm2)%kmeshe)
         length = size(km(iseqkm1)%kmeshe)
         allocate(km(iseqkm2)%kmeshe(length))
         km(iseqkm2)%kmeshe = km(iseqkm1)%kmeshe
         km(iseqkm2)%kmshe1 => km(iseqkm1)%kmeshe(1:npoint+1)
         km(iseqkm2)%kmshe2 => km(iseqkm1)%kmeshe(npoint+2:length)
      end if  ! ( km(iseqkm1)%kmesh(19)>0)

!     --- kmesh j, k

      if ( km(iseqkm1)%kmesh(24)>0) then
         if ( associated(km(iseqkm2)%kmeshj) ) deallocate(km(iseqkm2)%kmeshj)
         length = size(km(iseqkm1)%kmeshj)
         allocate(km(iseqkm2)%kmeshj(length))
         km(iseqkm2)%kmeshj = km(iseqkm1)%kmeshj
      end if  ! ( km(iseqkm1)%kmesh(24)>0)

      if ( km(iseqkm1)%kmesh(25)>0) then
         if ( associated(km(iseqkm2)%kmeshk) ) deallocate(km(iseqkm2)%kmeshk)
         length = size(km(iseqkm1)%kmeshk)
         allocate(km(iseqkm2)%kmeshk(length))
         km(iseqkm2)%kmeshk = km(iseqkm1)%kmeshk
      end if  ! ( km(iseqkm1)%kmesh(25)>0)

      if ( debug ) write(irefwr,*) 'End mshcopsel'
      call erclos ( 'mshcopsel' )

      end
