      subroutine mshcreatelgrp ( map, ielgrp, kmeshcloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create mesh corresponding to one element group
!     The nodes are renumbered to a local renumbering
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element_group
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer map(*), ielgrp, kmeshcloc(*)

!     ielgrp         i    element group number
!     kmeshcloc      o    local kmesh part c, contains the nodal points
!                         corresponding to the elements in the element group
!     map            i    Mapping array of length npoint to map global node
!                         numbers to local node numbers
!                         map(i) = 0, means that node i is not in the
!                                     element group
!                         map(i)>0, defines position of node i in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, lenkmeshc, ikelmc

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     lenkmeshc      length of array kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIMAPINTARR   Map integer array onto new numbering using numbering
!                    stored in irenum
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcreatelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcreatelgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address of element group in kmeshc

      ikelmc = 1
      do i = 1, ielgrp-1

         inpelm = Topgrp(i)%inpelm
         inelem = Topgrp(i)%inelem
         ikelmc = ikelmc+inelem*inpelm

      end do  ! i = 1, ielgrp-1

!     --- Compute length of kmeshc for ielgrp

      inelem = Topgrp(ielgrp)%inelem
      inpelm = Topgrp(ielgrp)%inpelm

      lenkmeshc = inpelm*inelem

!     --- Copy part of mesh corresponding to element group into array kmeshc

      kmeshcloc(1:lenkmeshc) = kmeshc(ikelmc:ikelmc+lenkmeshc-1)

!     --- renumber nodes

      call inimapintarr ( kmeshcloc, map, lenkmeshc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcreatelgrp'

      end if  ! ( debug )
      call erclos ( 'mshcreatelgrp' )

      end
