      subroutine mshcurves3 ( iwork, iworkj, numpoints, jcurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if new curve stored in istore is not already stored in kmeshm
!     If new curve already in kmeshm, set numpoints = 0
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), iworkj(*), numpoints, jcurnr

!     iwork               integer work array of length npoint
!                         if iwork(i) > 0, point i is in curve
!     iworkj         i    integer work array created by mshcurves1
!                         if iworkj(i)  > 0, point i is in surface jsurnr and
!                         in surface isurnr
!     jcurnr         o    curve sequence number of new curve
!     numpoints     i/o   Number of points on new curve
!                         If the curve already existed, numpoints is set to 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, inode, nnodes, istar3, icurnr, j

!     i              Counting variable
!     icount         Counter
!     icurnr         Curve sequence number
!     inode          Node sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     j              Counting variable
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear iwork
!     For all curves icurnr stored in kmeshm do
!        Make iwork(i) = 1 for for all nodes i in icurnr
!        if ( all nodes in curve are stored in istore and
!             all nodes in istore are stored in iwork ) then
!           numpoints := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves3'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork

      do i = 1, npoint
         iwork(i) = 0
      end do  ! i = 1, npoint

!     --- For all curves icurnr stored in kmeshm do

      do icurnr = 1, ncurvs

!     --- Make iwork(i) = 1 for for all nodes i in icurnr

         istar3 = kmeshm(2)-1
         nnodes = kmeshm(5+icurnr)-kmeshm(4+icurnr)

         if ( kmeshm(istar3+kmeshm(4+icurnr))== &
              kmeshm(istar3+kmeshm(5+icurnr)-1) ) nnodes = nnodes-1
         if ( debug ) write(irefwr,*) 'icurnr, nnodes ', icurnr, nnodes
         icount = 0

         do i = 1, nnodes

            inode = kmeshm(istar3+i+kmeshm(4+icurnr)-1)
            if ( inode>0 ) then

               if ( iwork(inode)==0 .and. iworkj(inode)/=0 ) then

!              --- New node on curve found

                  icount = icount+1
                  iwork(inode) = 1

               else if ( iwork(inode)==0 ) then

!              --- Both curves have different nodes
!                  Clear iwork and jump

                  do j = 1, i-1
                     inode = kmeshm(istar3+j+kmeshm(4+icurnr))
                     if ( inode>0 ) then
                        iwork(inode) = 0
                     end if
                  end do  ! j = 1, i-1
                  go to 100

               end if  ! ( iwork(i)==0 )

            end if  ! ( inode>0 )

         end do  ! i = 1, nnodes
         if ( debug ) &
            write(irefwr,*) 'icount, numpoints ', icount, numpoints

         if ( icount==numpoints ) then

!        --- all nodes in curve are stored in istore and
!            all nodes in istore are stored in iwork ) then
!               numpoints := 0

            numpoints = 0
            jcurnr = icurnr
            go to 1000

         end if  ! ( nnodes==numpoints )

100   end do  ! icurnr = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurves3'

      end if
      call erclos ( 'mshcurves3' )

      end
