      subroutine mshcurves4 ( ncurvsnew, kmeshmtmp, ipoint, lenkmeshmtmp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 27-11-2012 Long integers
!        version  1.1  date 02-02-2005 Extra debug statements
!        version  1.0  date 08-03-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Merge old array kmeshm with new part and create a new array kmeshm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvsnew, kmeshmtmp(*), ipoint(*), lenkmeshmtmp

!     ipoint         i    Array of length ncurvsnew containing the starting
!                         address of the next curve in kmeshmtmp
!                         So in ipoint(1) the number of nodes for the first new
!                         curve in kmeshmtmp plus one is stored etc.
!     kmeshmtmp      i    Temporary array kmeshm in which the nodes of the new
!                         curves are stored
!     lenkmeshmtmp   i    Length of array kmeshmtmp
!     ncurvsnew      i    Number of new curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: kmeshmold(:)

!     kmeshmold      Old array kmeshm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCURVES41    Merge old array kmeshm with new part and create a new
!                    array kmeshm
!     SEPPRINKMESHM  Print the contents of array kmeshm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves4'
         call sepprinkmeshm

      end if
      if ( ierror/=0 ) go to 1000

!     --- The old array kmeshm is copied to the back of ibuffr
!         The new array kmeshm overwrites the old one

      if ( kelmm/=0 ) then

!     --- kmeshm old exists

         length = size ( kmeshm )

         allocate ( kmeshmold(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshmold' )
         if ( ierror/=0 ) go to 1000

         kmeshmold(1:length) = kmeshm

      else

!     --- kmeshm old does not exist

         length = 5

         allocate ( kmeshmold(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshmold' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( kelmn/=0 )


      length = 2*ncurvsnew+lenkmeshmtmp+length

!     --- Reserve space for new array kmeshm

      if ( km(iseqkm)%kmesh(27)>0 ) deallocate(km(iseqkm)%kmeshm)
      allocate(km(iseqkm)%kmeshm(length))
      kmeshm => km(iseqkm)%kmeshm
      kmeshmglob => km(iseqkm)%kmeshm
      km(iseqkm)%kmesh(27) = length
      if ( ierror/=0 ) go to 1000

!     --- Copy arrays

      call mshcurves41 ( kmeshmold, kmeshmtmp, ncurvsnew, ipoint )

      deallocate ( kmeshmold, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshmold' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'New array kmeshm'
         call sepprinkmeshm
         write(irefwr,*) 'End mshcurves4'

      end if
      call erclos ( 'mshcurves4' )

      end subroutine mshcurves4
