      subroutine mshdefcurvs ( icurvs, curves, ndim, ncurvs, kmeshm, &
                               coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-12-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates from array coor into array curves in case a
!     change of coordinates has been carried out
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     coordinate
!     mesh
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), ndim, ncurvs, kmeshm(*)
      double precision coor(ndim,*), curves(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curves        i/o   array containing the coordinates of the curves
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     ncurvs         i    Number of curves in mesh
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, numnodes, istar3, istart, i, j, inode, jstart

!     i              Counting variable
!     icurnr         Curve sequence number
!     inode          Node sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     j              Counting variable
!     jstart         starting position of curve icurnr in kmeshm
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all curves icurnr do
!        if ( number of nodes in icurvs = number of nodes in kmeshm ) then
!           for all nodes along curve do
!              if ( node number > 0 ) then
!                 Copy coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdefcurvs' )
      istar3 = kmeshm(2)-1

!     --- For all curves icurnr do

      do icurnr = 1, ncurvs

         if ( icurnr.eq.1 ) then

!        --- First curve number

            numnodes = icurvs(icurnr)
            istart = 0

         else

!        --- Next curve number

            numnodes = icurvs(icurnr)-icurvs(icurnr-1)
            istart = icurvs(icurnr-1)

         end if  ! ( icurnr.eq.1 )
         if ( numnodes.eq.kmeshm(5+icurnr)-kmeshm(4+icurnr) ) then

!        --- number of nodes in icurvs = number of nodes in kmeshm
!            for all nodes along curve do

            jstart = kmeshm(4+icurnr)+istar3-1
            do i = 1, numnodes
               inode = kmeshm(jstart+i)
               if ( inode.gt.0 ) then

!              --- node number > 0
!                  Copy coordinates

                  do j = 1, ndim
                     curves(j,istart+i) = coor(j,inode)
                  end do

               end if  ! ( inode.gt.0 )

            end do  ! i = 1, numnodes

         end if  ! ( numnodes.eq.kmeshm(5+icurnr)-kmeshm(4+icurnr) )

      end do  ! icurnr = 1, ncurvs

      call erclos ( 'mshdefcurvs' )

      end
