      subroutine mshdircrvsrf ( firstpnt, idirec, isurnr, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-05-2002 New call to pr0008
!        version  1.0  date 29-03-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the direction of the outer boundary of a surface by
!     checking the direction of the first two nodes on the boundary
!     compared to the element it is lying in
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     boundary
!     direction
!     mesh
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer firstpnt, idirec, isurnr, icurnr

!     firstpnt       i    First point of a curve
!     icurnr         i    Curve sequence number
!     idirec        i/o   Defines in which direction the curves must be
!                         considered
!                         idirec = 1, from low to high, idirec = -1: reversed
!                         order
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, istart, idummy(1)
      double precision dummy(1)
      logical filled

!     dummy          Dummy parameter
!     filled         If true the surface has been filled
!     i              Counting variable
!     idummy         Dummy parameter
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHDIRSURF     Compute the direction of the outer boundary of a surface by
!                    checking the direction of the first two nodes on the
!                    boundary
!     MSHFIRPNTSCRV  Find first and next point of a curve using the information
!                    stored in kmeshm
!     PR0008         Print information of curves
!     PR0009         Print information of surfaces
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdircrvsrf' )
      debug = .false.

      idirec = 1

      if ( debug ) then

!     --- Debug statements

         call pr0008 ( kmeshm, ncurvs )

      end if

!     --- Find first two nodes of curve icurnr

      call mshfirpntscrv ( icurnr, kmeshm, firstpnt, nextpnt )

      if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'icurnr, firstpnt, nextpnt ', &
                          icurnr, firstpnt, nextpnt
         write(irefwr,*) 'kelmn ',kelmn

      end if

      if ( kelmn>0 ) then

!     --- kelmn>0, kmeshn exists
!         Compute starting address of surface in kmeshn

         if ( debug ) then

!        --- Debug statements

            call pr0009 ( 0, kmeshn, dummy, idummy, 0, ndim, nsurfs )

         end if

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)
         istart = 2*nsurfs+1
         do i = 1, isurnr-1
            istart = istart+kmeshn(i)*kmeshn(nsurfs+i)
         end do

!        --- Check if surface has been filled

         filled = .true.
         do i = 0, inpelm-1
            if ( kmeshn(istart+i)<=0 ) filled = .false.
         end do

!        --- Compute direction

         if ( filled ) &
            call mshdirsurf ( kmeshn(istart), inpelm, inelem, &
                              firstpnt, nextpnt, idirec, isurnr, &
                              icurnr )

      else

!     --- kelmn=0, kmeshn does not exist, use kmesh part t
!         Compute starting address of surface in kmeshc

         call sepextkmesht ( isurnr, isurnr, istart, inpelm )

!        --- Check if surface has been filled

         filled = inpelm>0
         do i = 0, inpelm-1
            if ( kmeshc(istart+i)<=0 ) filled = .false.
         end do

!        --- Compute direction

         if ( filled ) &
            call mshdirsurf ( kmeshc(istart), inpelm, inelem, &
                              firstpnt, nextpnt, idirec, isurnr, &
                              icurnr )

      end if  ! kelmn
      call erclos ( 'mshdircrvsrf' )
      end
