      subroutine mshfillkmeshm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 06-03-2015 Extra Debug
!        version  1.2  date 27-11-2012 Long integers
!        version  1.1  date 28-02-2005 seppointer --> seppointer
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshm for a two dimensional mesh where only kmeshc is given
!     i.e. the topology is known
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     2d
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmeshm3, i
      integer, allocatable :: kmsmtmp(:)

!     i              Counting variable
!     kmsmtmp        Temporary array kmeshm
!     lenkmeshm3     Length of kmeshm part 3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPRMNMESH     Remove subarrays of nmesh
!     TOPLM7         Fill boundary of a 2D mesh in kmeshm, where the information
!                    has been stored in nmeshc and nmeshh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create the edges of the mesh
!     Compute the elements corresponding to the edges
!     Store all edges that are connected to one element
!     Sort these edges to get a continuous boundary
!     Delete superfluous arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshm'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create the edges of the mesh
!         Compute the elements corresponding to the edges

      if ( debug ) write(irefwr,*) 'before nmeshc'

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( debug ) write(irefwr,*) 'before nmeshh'
      if ( nmesh(38)==0 ) call sepfillnmeshh
      if ( debug ) write(irefwr,*) 'after nmeshh'
      if ( ierror/=0 ) go to 1000

!     --- Store all edges that are connected to one element
!         Sort these edges to get a continuous boundary

      allocate ( kmsmtmp(3*npoint+5), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*npoint+5, 'kmsmtmp' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'before toplm7'
      call toplm7 ( kmsmtmp(npoint+6), npoint, lenkmeshm3, &
                    nedges, nmeshh, nmeshc, ncurvs, &
                    kmsmtmp(5), kmsmtmp )
      if ( debug ) write(irefwr,*) 'after toplm7'
      km(iseqkm)%kmesh(11) = ncurvs

!     --- Make array kmeshm permanent

      allocate ( km(iseqkm)%kmeshm(ncurvs+5+lenkmeshm3) )
      kmeshm => km(iseqkm)%kmeshm
      kmeshm = kmsmtmp(1:ncurvs+5+lenkmeshm3)

!     --- Delete superfluous arrays

      if ( kelmd>0 ) then
         deallocate ( km(iseqkm)%kmeshd )
         nullify(kmeshd)
         km(iseqkm)%kmesh(18) = 0
      end if  ! ( kelmd>0 )

      call seprmnmesh
      kelmaa = 0
      km(iseqkm)%kmesh(48) = 0

      deallocate ( kmsmtmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmsmtmp' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillkmeshm'

      end if
      call erclos ( 'mshfillkmeshm' )

      end
