      subroutine mshfindsurf ( jsurnr, sumelm, iwork, found )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 17-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    This routine is used to check if the obstacle surface is also
!    part of a volume. If so, the routine is used again to fill
!    information in kmesh part x concerning obstacles from kmeshn.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     deformation
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jsurnr, sumelm, iwork(inpelm, *)
      logical found

!     found         i/o   Logical that will be set to true if the
!                         obstacle surface is also part of a volume
!     iwork          i    Work array to store kmeshc_obst
!     jsurnr         i    Surface sequence number
!     sumelm         i    Variable used to store the total number of
!                         elements in a surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nelmsurf, jstart, ielem, inode, npelmsurf

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     j              Counting variable
!     jstart         Starting point
!     nelmsurf       Number of elements in a surface
!     npelmsurf      Number of nodal points in a surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Get starting adress in kmeshn part 3 for surface 'jsurnr'
!       loop over elements in surface
!           loop over nodal point in element
!               if (found = true)
!                  this routine has already been called before
!                  and will now be used to fill kmeshc_obst in iwork
!               else
!                  if (kmeshn part 3 is filled for 'jsurnr')
!                     found is set to true
!                  end
!                  leave this file
!               end
!           end
!       end
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfindsurf' )

!     --- loop over surfaces

      jstart=0

      do j=1,jsurnr-1

!     --- number of elements and points in surface

         npelmsurf=kmeshn(j)
         nelmsurf=kmeshn(nsurfs+j)
         jstart=jstart+npelmsurf*nelmsurf

      end do

!     --- number of elements in surface 'jsurnr'

      nelmsurf=kmeshn(nsurfs+jsurnr)

!     --- loop over elements in surface 'jsurnr'

      do ielem=1,nelmsurf

!     --- loop over nodal points in element

         do i=1,inpelm

            inode=kmeshn(2*nsurfs+jstart+inpelm*(ielem-1)+i)

            if (found) then

!           --- the obstacle surface is part of a volume
!               iwork is filled using kmeshn

               iwork(i,ielem)=inode

            else

!           --- Check if the obstacle surface is part of a volume

               if (inode.gt.0) then

!              --- part 3 of kmeshn is filled and therefore
!                  can be used to fill kmeshc_obst

                  found=.true.

               end if

!              --- leave loop

               goto 100

            end if

         end do

      end do

!     --- update the total number of elements in the obstacle surface

100   sumelm=sumelm+nelmsurf

      call erclos ( 'mshfindsurf' )

      end
