      subroutine mshfl0 ( kbndlc, kbndpt, icurvs, ipoint, npoloc, &
                          nelloc, iboun, iwork, iuserpnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 02-09-2010 Remove intarmsh
!        version  4.7  date 08-04-2009 Correction internal curves and
!                                      double curves (ws)
!        version  4.6  date 27-10-2008 Extra parameter nextracrvs
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array KBNDPT for one surface  ( isurf = 7:  not ordened )
!    Help subroutine for subroutine MSHFIL
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), icurvs(*), ipoint(*), iboun(*), &
              kbndlc(*), iwork(*), npoloc, nelloc, &
              iuserpnt(*)

!     iboun      i    Array of length nboun containing the curve numbers of the
!                     boundary of the local surface
!     icurvs    i/o   Array of length ncurvs containing information of the
!                     curves with respect to kbndpt
!                     icurvs is filled as follows:
!                     icurvs(i) = 0    Curve i is not present
!                     icurvs(i)>0      Curve i has not been used before
!                                      The value gives the starting position in
!                                      array kbndpt
!                     icurvs(i)<0      Curve i has been used before
!                                      Minus the value gives the starting
!                                      position in array kbndpt
!                                      In the case of a double curve in one
!                                      submesh the second one is made equal to
!                                      icurvs(i)-large
!     ipoint     i    Help array of length npoloc, to store the new nubering of
!                     the nodes
!     iuserpnt  i/o   Integer work array of length nuspnt
!                     In this array the local node number of the user points
!                     are stored once they are found
!                     Initially the array is set equal to 0 in mshfil
!     iwork      i    Work array of length NCURVS.
!                     IWORK is filled as follows:
!                     If iwork(i) = 0, then curve i does not belong to the
!                     surface
!                     If iwork(i)#0, then curve iwork(i) belongs to the surface
!                     The sign of iwork(i) indicates the direction of storage in
!                     KBNDPT
!     kbndlc     i    Local array KBNDPT corresponding to surface
!     kbndpt    i/o   Array containing the boundary points
!                     of the composite surface;curves are counted for only once
!     nelloc     i    Number of elements in kmsclc
!     npoloc     i    Number of nodes in coorlc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurve, ifirst, inodlp, inodls, inodst, isign, istar3, &
              istart, j, jcurve, jnodlp, jsign, k1, k2, large, large2, &
              n, super, ncontiguous
      logical double, contiguous
      parameter ( large = 100000, large2 = large/2, super=100000000 )

!     contiguous     If true two succeeding curves have the same number
!     double         indicates if a double curve in the submesh has been found
!                    (true) or not (false)
!     i              Counting variable
!     icurve         Curve number
!     ifirst         First curve (surface) number read
!     inodlp         Nodal point number
!     inodls         Last user point number of a curve
!     inodst         First user point number of a curve
!     isign          Indicates the direction of the curve
!                    1: forward direction, -1: reversed direction
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         Starting address
!     j              Counting variable
!     jcurve         abs(icurve)
!     jnodlp         Help variable to store a nodal point number
!     jsign          Gives sign of curve in composite surface as stored in
!                    KBNDPT
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     large          A large integer number that must be larger than the number
!                    of points at the boundary
!     large2         large/2
!     n              Parameter for mesh generation
!     ncontiguous    number of contiguous curves already treated
!     super          Very large number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFL5         Fills arrays KBNDPT for inner curves
!     PRININ         print 1d integer array
!     PRININLARGE    print 1d integer array in large format
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfl0' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfl0'
         call prinin ( kbndlc, lendebug, 'kbndlc' )
         call prinin ( icurvs, ncurvs, 'icurvs' )
         call prinin ( iboun, nboun, 'iboun' )
         call prinin ( iwork, ncurvs, 'iwork' )
         call prinin ( iuserpnt, nuspnt, 'iuserpnt' )
         write(irefwr,1) 'nextracrvs', nextracrvs
  1      format ( a, 1x, (10i6) )
  3      format ( a, 3x, (5i10) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,1) 'nbndpt, npointlc, nelemlc, nboun', &
                          nbndpt, npointlc, nelemlc, nboun

!     --- Fill array ipoint
!         First clear ipoint

      ipoint(1:npoloc) = 0

!     --- Check if boundary curves have already been filled or not
!         If filled, store points in ipoint

      istar3 = kmeshm(2)-1
      ifirst = 1
      inodls = 0
      double = .false.
      contiguous = .false.  ! two succeeding curves are not the same
      ncontiguous = 0       ! counter for number of contiguous curves

      do i = 1, nboun

         icurve = iboun(i)
         jcurve = abs(icurve)

         if ( i>1 ) then

            if ( .not. contiguous ) then

!           --- previous curve was not a double curve

               contiguous = icurve==-iboun(i-1)
               if ( contiguous ) ncontiguous = 1  ! first contiguous curve

            else

!           --- previous set of curves were part of a double curve

               if ( i>1+2*ncontiguous ) then

                  if ( icurve==-iboun(i-1-2*ncontiguous) ) then

!                 --- curve is also part of this double curve

                     ncontiguous = ncontiguous+1

                  else

!                 --- Curve is not part of this double curve

                     contiguous = .false.

                  end if  ! ( icurve==-iboun(i-1-ncontiguous) ) then

               else

                  contiguous = .false.

               end if ! ( i>1+2*ncontiguous )

            end if  ! ( .not. contiguous )

         end if  ! ( i>1 )
         if ( debug ) &
            write(irefwr,*) 'contiguous, ncontiguous', &
                             contiguous, ncontiguous

         k1 = kmeshm(jcurve+4)
         k2 = kmeshm(jcurve+5)-1
         n = k2-k1+1

         if ( debug ) then
            write(irefwr,1) 'i, icurve, k1, k2, n, ifirst', &
                             i, icurve, k1, k2, n, ifirst
            write(irefwr,*) 'contiguous', contiguous
         end if  ! ( debug )

         if ( icurve>0 ) then

!        --- icurve>0   Use curve forwards

            if ( icurvs(jcurve)>-large2 ) then
               istart = abs(icurvs(jcurve))
            else
               istart = abs(icurvs(jcurve)+large)
            end if
            isign = 1
            inodst = kmeshm(istar3+k1)
            if ( inodst==inodls .and. i<=nboun-nextracrvs ) &
               ifirst = ifirst - 1      ! not for internal curves
            inodls = kmeshm(istar3+k2)

         else

!        --- icurve<0   Use curve backwards

            if ( icurvs(jcurve)>-large2 ) then
               istart = abs(icurvs(jcurve)) + n-1
            else
               istart = abs(icurvs(jcurve)+large) + n-1
            end if
            isign = -1
            inodst = kmeshm(istar3+k2)
            if ( inodst==inodls .and. i<=nboun-nextracrvs ) &
               ifirst = ifirst - 1      ! not for internal curves
            inodls = kmeshm(istar3+k1)

         end if
         if ( iwork(jcurve)>=0 ) then

!        --- Curve in positive sense in KBNDPT
!            iwork(jcurv) =0 , implies internal curve!

            jsign = 1

         else

!        --- Curve in negative sense in KBNDPT

            jsign = -1
            ifirst = ifirst + n - 1

         end if
         if ( debug ) &
            write(irefwr,1) 'jsign, ifirst, n, jcurve, icurvs(jcurve)', &
                             jsign, ifirst, n, jcurve, icurvs(jcurve)

         if ( icurvs(jcurve)<0 ) then

!        --- icurvs(jcurve)<0, hence curve has already been filled

            do j = 1, n

               inodlp = kbndlc((j-1)*jsign+ifirst)
               jnodlp = kbndpt(istart+(j-1)*isign)
               if ( debug ) &
                  write(irefwr,3) 'j, inodlp, jnodlp, ipoint(inodlp)', &
                                   j, inodlp, jnodlp, ipoint(inodlp)

               if ( jnodlp>0 ) then

!              --- Curve has been filled before

                  ipoint(inodlp) = jnodlp

               else

!              --- Double curve that has not been filled before
!                  The first and last point are skipped, &
!                  since they are user points
!                  Otherwise we might end up with a negative entry in kbndpt

                  if ( .not. contiguous .or. (j>1 .and. j<n) ) then

!                 --- In case of two succeeding curves are the same
!                     the first node is skipped
!                     Fill negative number if node has not been marked before
!                     If point refers to itself, reset to 0

                     if ( ipoint(inodlp)==0 ) &
                        ipoint(inodlp) = -(istart+(j-1)*isign)

                  end if  ! ( .not. contiguous .or. j>1 )
                  double = .true.

               end if
               if ( debug ) &
                  write(irefwr,3) 'ipoint(inodlp)', ipoint(inodlp)

            end do

         else

!        --- Curve has not been filled, but perhaps user points

            do j = 1, n

               if  ( isign*jsign==1 ) then
                  jnodlp = kmeshm(istar3+j-1+k1)
               else
                  jnodlp = kmeshm(istar3-j+1+k2)
               end if
               if ( jnodlp<0 ) then

!              --- user point found

                  inodlp = kbndlc((j-1)*jsign+ifirst)
                  ipoint(inodlp) = iuserpnt(-jnodlp)

               else

!              --- other point found, mark node as found
!                  This to avoid problems with double nodes in kbndlc, &
!                  which are already identified in the submesh generator

                  inodlp = kbndlc((j-1)*jsign+ifirst)
                  if ( ipoint(inodlp)==0 ) ipoint(inodlp) = super

               end if  ! ( jnodlp<0 )
               if ( debug ) &
                  write(irefwr,3) 'j, inodlp, jnodlp, ipoint(inodlp)', &
                                   j, inodlp, jnodlp, ipoint(inodlp)

            end do

         end if
         if ( jsign==-1 ) ifirst = ifirst-n+1
         ifirst = ifirst + n

!        --- Mark curve used

         if ( icurvs(jcurve)>-large2 ) &
            icurvs(jcurve) = icurvs(jcurve)-large

      end do
      if ( debug ) call prininlarge ( ipoint, npoloc, 'ipoint/1' )

!     --- Reset icurvs

      do i = 1, nboun
         jcurve = abs(iboun(i))
         if ( icurvs(jcurve)<-large2 ) &
              icurvs(jcurve) = icurvs(jcurve)+large
      end do
      if ( debug ) call prinin ( icurvs, ncurvs, 'icurvs/2' )

!     --- Fill rest of ipoint
!         Raise npointlc

      do i = 1, npoloc
         if ( ipoint(i)==0 .or. ipoint(i)==super ) then
            npointlc = npointlc + 1
            ipoint(i) = npointlc
         end if
      end do
      if ( debug ) call prininlarge ( ipoint, npoloc, 'ipoint/3' )

!     --- Fill elements in array ipoint

      do i = 1, nelloc
         ipoint(npoloc+i) = i+nelemlc
      end do
      nelemlc = nelemlc + nelloc

!     --- Finally fill array kbndpt

      ifirst = 0

      call mshfl5 ( iboun, kmeshm, icurvs, nboun, kbndpt, kbndlc, &
                    ipoint, iwork, ifirst, nextracrvs )
      if ( debug ) then
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
         write(irefwr,*) 'double ', double
      end if  ! ( debug )

      if ( double ) then

!     --- Double curves have been found in the submesh
!         Correct ipoint and kbndpt

         do i = 1, nbndpt

            if ( kbndpt(i)<0 ) kbndpt(i) = kbndpt(-kbndpt(i))
            if ( kbndpt(i)<0 ) then

!           --- Apparantly kbndpt refers to itself
!               Raise npointlc and fill in kbndpt

               npointlc = npointlc + 1
               kbndpt(i) = npointlc

            end if  ! ( kbndpt(i)<0 ) then

         end do
         do i = 1, npoloc
            if ( ipoint(i)<0 ) ipoint(i) = kbndpt(-ipoint(i))
         end do

      end if
      if ( debug ) then
         call prinin ( ipoint, npoloc, 'ipoint/4' )
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
      end if  ! ( debug )

!     --- Fill user points

      ifirst = 1
      inodls = 0
      do i = 1, nboun
         icurve = iboun(i)
         jcurve = abs(icurve)
         k1 = kmeshm(jcurve+4)
         k2 = kmeshm(jcurve+5)-1
         n = k2-k1+1
         if ( iwork(jcurve)>0 ) then

!        --- Curve in positive sense in KBNDPT

            jsign = 1

         else

!        --- Curve in negative sense in KBNDPT

            jsign = -1
            ifirst = ifirst + n - 1

         end if
         if ( icurve>0 ) then

!        --- icurve>0   Use curve forwards

            isign = 1
            inodst = kmeshm(istar3+k1)
            if ( inodst==inodls .and. i<=nboun-nextracrvs ) then
               ifirst = ifirst - 1
            end if
            inodls = kmeshm(istar3+k2)

         else

!        --- icurve<0   Use curve backwards

            isign = -1
            inodst = kmeshm(istar3+k2)
            if ( inodst==inodls .and. i<=nboun-nextracrvs ) then
               ifirst = ifirst - 1
            end if
            inodls = kmeshm(istar3+k1)

         end if

         do j = 1, n

            if  ( isign*jsign==1 ) then
               jnodlp = kmeshm(istar3+j-1+k1)
            else
               jnodlp = kmeshm(istar3-j+1+k2)
            end if
            if ( jnodlp<0 ) then

!           --- user point found

               inodlp = kbndlc((j-1)*jsign+ifirst)
               iuserpnt(-jnodlp) = ipoint(inodlp)

            end if

         end do
         if ( jsign==-1 ) ifirst = ifirst-n+1
         ifirst = ifirst + n

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( ipoint, npoloc, 'ipoint' )
         write(irefwr,*) 'End mshfl0'

      end if
      call erclos ( 'mshfl0' )

      end
