      subroutine mshfl4 ( kbndlc, kbndpt, icurvs, ipoint, nloc, mloc, &
                          ntot, iboun, nrel, npoloc, idirec, &
                          isubsf, irow, nrow, icolum, ncolum, &
                          iwork, neltot, nptot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 09-03-2013 Correction negative curve number
!        version  5.0  date 02-09-2010 Remove intarmsh
!        version  4.2  date 27-10-2008 New call to mshfl5
!
!   copyright (c) 1988-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill arrays KBNDPT and ipoint for one surface
!    isurf = 12:  ordened surface of pipe surfaces
!    Help subroutine for subroutine MSHFIL
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), icurvs(*), ipoint(*), iboun(*), &
              kbndlc(*), nloc, mloc, ntot, nrel, npoloc, &
              idirec, isubsf, irow, nrow, icolum, ncolum, iwork(*), &
              neltot, nptot

!     iboun      i    Array of length nboun containing the curve numbers of the
!                     boundary of the local surface
!     icolum     i    Actual column number
!     icurvs    i/o   Array of length ncurvs containing information of the
!                     curves with respect to kbndpt
!                     icurvs is filled as follows:
!                     icurvs(i) = 0    Curve i is not present
!                     icurvs(i)>0    Curve i has not been used before
!                                      The value gives the starting position in
!                                      array kbndpt
!                     icurvs(i)<0    Curve i has been used before
!                                      Minus the value gives the starting
!                                      position in array kbndpt
!     idirec     i    Gives direction of subsurfaces
!                     idirec = 1:  normal direction  ("left to right")
!                     idirec = 2:  reversed direction ("right to left")
!     ipoint     i    Help array of length npoloc, to store the new nubering of
!                     the nodes
!     irow       i    Actual row number
!     isubsf     i    Surface number of local surface
!     iwork      i    Work array of length NCURVS.
!                     IWORK is filled as follows:
!                     If iwork(i) = 0, then curve i does not belong to the
!                     surface
!                     If iwork(i)#0, then curve iwork(i) belongs to the surface
!                     The sign of iwork(i) indicates the direction of storage in
!                     KBNDPT
!     kbndlc     i    Local array KBNDPT corresponding to surface
!     kbndpt    i/o   Array containing the boundary points
!                     of the composite surface;curves are counted for only once
!     mloc       i    Number of nodes in "vertical direction of local surface"
!     ncolum     i    Number of columns
!     neltot     i    Total number of elements in composite surface
!     nloc       i    Number of nodes in "horizontal direction of local surface"
!     npoloc     i    Number of nodes in local surface
!     nptot      i    Total number of nodes in composite surface, &
!                     centroids excluded
!     nrel       i    Relative nodal point number minus one of at most left
!                     nodal point at the bottom of local surface
!     nrow       i    Number of rows
!     ntot       i    Number of nodes in "horizontal direction" of row of
!                     local surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ich1, icurve, if1, ifirst, ileft, iright, &
              iunder, iupper, j, jcurve, m, n, n1, nb, nb1, numc, &
              nloc1, nr, nrel1, ns, nt, ntot1
      save iunder, iupper, ileft, iright

!     i              Counting variable
!     ich1
!     icurve         Type of curve
!     if1
!     ifirst         First curve (surface) number read
!     ileft
!     iright
!     iunder         Lower bound of array
!     iupper         Upper bound of array
!     j              Counting variable
!     jcurve         Curve number
!     m              Parameter for mesh generation
!     n              Parameter for mesh generation
!     n1             Number of elements along the  south  side
!                    of the current quadrilateral
!     nb
!     nb1
!     nloc1
!     nr
!     nrel1
!     ns             Starting address of information of specific surface in
!                    array KMESHS
!     nt
!     ntot1          Number of elements in "horizontal" direction of row of
!                    surfaces
!     numc           number of points along the curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFL5         Fill array kbndpt
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfl4' )
      debug = .false.
      if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'Debug information from mshfl4'
         write(irefwr,1) 'ncurvs, nsurfs, npointlc, nelemlc', &
                          ncurvs, nsurfs, npointlc, nelemlc
         write(irefwr,1) 'isurnr, ishape', isurnr, ishape
  1      format ( a, 1x, (10i6) )

      end if

!     --- First compute some constants depending on the type of element

      if ( ishape==3 .or. ishape==5 ) then

!     --- ishape = 3, 5:  Linear elements

         n = nloc+1
         m = mloc+1
         nr = nrel
         nt = ntot + 1

      else

!     --- ishape = 4, 6, 7:  Quadratic elements

         n = nloc*2+1
         m = mloc*2+1
         nr = 2*nrel
         nt = 2*ntot + 1

      end if

      if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'nloc, mloc, nrel, ntot ', &
                         nloc, mloc, nrel, ntot
         write(irefwr,*) 'n, m, nr, nt ', n, m, nr, nt

      end if

      if ( ishape==3 .or. ishape==4 .or. ishape==7 ) then

!     --- ishape = 3, 4 or 7  triangular elements

         nrel1 = 2 * nrel
         nloc1 = 2 * nloc
         ntot1 = 2 * ntot

      else

!     --- ishape = 5 or 6  quadrilateral elements

         nrel1 = nrel
         nloc1 = nloc
         ntot1 = ntot

      end if
      if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'nrel1, nloc1, ntot ', nrel1, nloc1, ntot
         write(irefwr,1) 'neltot, nptot', neltot, nptot
         call prinin ( icurvs, ncurvs, 'icurvs input' )
         call prinin ( kmeshs(nsurfs+2+(isurnr-1)*8), 8, 'kmeshs_part' )

      end if

!     --- Some constants specific for pipe surfaces
!         Constants for global surface

      if ( irow==1 .and. icolum==1 ) then

!     --- irow = 1 and icolum = 1

         ns = nsurfs + 2 + (isurnr-1)*8

!        --- Under boundary

         iunder = 0
         numc = kmeshs(ns+2)
         ich1 = kmeshs(isurnr)
         icurve = abs ( kmeshs(ich1+1) )
         icurvs(icurve) = - icurvs(icurve)
         ich1 = kmeshs(isurnr) + numc

!        --- Upper boundary

         numc = kmeshs(ns+3)
         icurve = abs ( kmeshs(1+ich1) )
         iupper = abs(icurvs(icurve)) - 1
         icurvs(icurve) = - icurvs(icurve)
         ich1 = ich1 + numc

         if ( debug ) &
            write(irefwr,1) 'numc, ich1, icurve, iupper', &
                             numc, ich1, icurve, iupper

!        --- First generating curve

         numc = kmeshs(ns+4)
         icurve = abs ( kmeshs(1+ich1) )
         ileft  = icurvs(icurve) - 1
         icurvs(icurve) = - icurvs(icurve)
         ich1 = ich1 + numc

!        --- Second generating curve

         icurve = abs ( kmeshs(1+ich1) )
         if ( icurvs(icurve)>=0 ) then
            iright = icurvs(icurve) - 1
         else
            iright = ileft
            do i = 1, numc
               icurve = abs ( kmeshs(i+ich1) )
               iright = iright + kmeshm(icurve+5) - kmeshm(icurve+4)-1
            end do
            iright = iright + 1
         end if
         do i = 1, ncurvs
            if ( icurvs(i)==0 ) icurvs(i) = -10000
         end do

      end if

      if ( debug ) then

!     --- Debug statements

         write(irefwr,1) 'iunder, iupper, ileft, iright ', &
                          iunder, iupper, ileft, iright
         call prinin ( icurvs, ncurvs, 'icurvs intermediate' )

      end if

!     --- Constants for local surface

      ns = nsurfs + 2 + (isubsf-1)*8

!     --- Fill array ipoint

      if ( idirec>0 ) then

!     --- idirec = 1   normal direction

         do j = 1, m
            do i = 1, n
               ipoint( (j-1)*n + i ) = npointlc + nr + (j-1)*nt + i
            end do
         end do

      else

!     --- idirec = -1   reversed direction

         do j = 1, m
            do i = 1, n
               ipoint( (j-1)*n + i ) = npointlc+nr+(j-1)*nt-i+n+1
            end do
         end do
      end if

      if ( ishape==7 ) then

!     --- ishape = 7, fill centroids

         if ( idirec>0 ) then

!        --- idirec = 1   normal direction

            do j = 1, mloc
               do i = 1, nloc1
                  ipoint(npoloc+1-((j-1)*nloc1+i)) = nptot+neltot+1- &
                          ((j-1)*ntot1+nrel1+i+nelemlc)
               end do
            end do

         else

!        --- idirec = -1   reversed direction

            do j = 1, mloc
               do i = 1, nloc1
                  ipoint(npoloc+1-((j-1)*nloc1+i)) = nptot+neltot+1- &
                         ((j-1)*ntot1+nrel1+nloc1+1-i+nelemlc)
               end do
            end do
         end if

      end if

!     --- Fill elements in array ipoint

      if ( idirec>0 ) then

!     --- idirec = 1   normal direction

         do j = 1, mloc
            do i = 1, nloc1
               ipoint(npoloc+(j-1)*nloc1+i) = (j-1)*ntot1+nrel1+i+nelemlc
            end do
         end do

      else

!     --- idirec = -1   reversed direction

         do j = 1, mloc
            do i = 1, nloc1
               ipoint(npoloc+(j-1)*nloc1+i) = (j-1)*ntot1+ &
                                              nrel1+nloc1+1-i+nelemlc
            end do
         end do

      end if
      if ( debug ) then

!     --- Debug statements

         call prinin1 ( ipoint, n, m, 'ipoint-nodes' )
         if ( ishape==7 ) then

!        --- ishape = 7, also centroids

            call prinin ( ipoint(n*m+1), npoloc-n*m, &
                          'ipoint-centroid nodes' )
         end if
         call prinin1 ( ipoint(npoloc+1), nloc1, mloc, &
                        'ipoint-elements' )

      end if

!     --- Finally fill array kbndpt

      ifirst = 0

!     --- Under boundary

      nb = kmeshs(ns+2)
      if ( irow==1 ) then

!     --- irow = 1  Fill under boundary

         if ( idirec==1 ) then

!        --- idirec = 1,  Copy forwards

            do i = 1, nb
               jcurve = abs ( iboun(i) )
               n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
               do j = 1, n1
                  kbndpt(iunder+j) = ipoint(kbndlc(j+ifirst))
               end do
               iunder = iunder + n1 - 1
               ifirst = ifirst + n1 - 1
            end do
            ifirst = ifirst + 1

         else

!        --- idirec = -1,  Copy backwards

            if1 = ifirst + n - 1
            do i = nb, 1, -1
               jcurve = abs ( iboun(i) )
               n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
               if1 = if1 - n1 + 1
               do j = 1, n1
                  kbndpt(iunder+j) = ipoint(kbndlc(if1+n1+1-j))
               end do
               iunder = iunder + n1 - 1
            end do
            ifirst = ifirst + n

         end if

      else

!     --- irow # 1  Under boundary must be skipped

         ifirst = ifirst + n

      end if

      if ( debug ) write(irefwr,*) 'iupper', iupper

!     --- Upper boundary

      nb1 = kmeshs(ns+3)
      if ( irow==nrow ) then

!     --- irow = nrow  Fill upper boundary

         if ( idirec==1 ) then

!        --- idirec = 1,  Copy forwards

            do i = 1, nb1
               jcurve = abs ( iboun(i+nb) )
               n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
               do j = 1, n1
                  kbndpt(iupper+j) = ipoint(kbndlc(j+ifirst))
               end do
               iupper = iupper + n1 - 1
               ifirst = ifirst + n1 - 1
            end do
            ifirst = ifirst + 1

         else

!        --- idirec = -1,  Copy backwards

            if1 = ifirst + n - 1
            do i = nb1, 1, -1
               jcurve = abs ( iboun(i+nb) )
               n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
               if1 = if1 - n1 + 1
               do j = 1, n1
                  kbndpt(iupper+j) = ipoint(kbndlc(if1+n1+1-j))
               end do
               iupper = iupper + n1 - 1
            end do
            ifirst = ifirst + n

         end if

      else

!     --- irow # nrow  Upper boundary is internal

         call mshfl5 ( iboun(nb+1), kmeshm, icurvs, nb1, kbndpt, kbndlc, &
                       ipoint, iwork, ifirst, 0 )
      end if

!     --- First generating curve

      nb = nb + nb1
      nb1 = kmeshs(ns+4)
      if ( icolum==1 .and. idirec==1 ) then

!     --- idirec = 1,  normal direction
!         icolum = 1   First generating curve (globally)

         do i = 1, nb1
            jcurve = abs ( iboun(i+nb) )
            n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
            do j = 1, n1
               kbndpt(ileft+j) = ipoint(kbndlc(j+ifirst))
            end do
            ileft  = ileft  + n1 - 1
            ifirst = ifirst + n1 - 1
         end do
         ifirst = ifirst + 1

      else if ( idirec==-1 .and. icolum==ncolum ) then

!     --- idirec = -1,  reversed direction
!         icolum = ncolum

         do i = 1, nb1
            jcurve = abs ( iboun(i+nb) )
            n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
            do j = 1, n1
               kbndpt(iright+j) = ipoint(kbndlc(j+ifirst))
            end do
            iright = iright + n1 - 1
            ifirst = ifirst + n1 - 1
         end do
         ifirst = ifirst + 1
      else

!     --- internal curve

         call mshfl5 ( iboun(nb+1), kmeshm, icurvs, nb1, kbndpt, kbndlc, &
                       ipoint, iwork, ifirst, 0 )
      end if

!     --- Second generating curve

      nb = nb + nb1
      nb1 = kmeshs(ns+5)
      if ( idirec==1 .and. icolum==ncolum ) then

!     --- idirec = 1,  normal direction
!         icolum = ncolum

         do i = 1, nb1
            jcurve = abs ( iboun(i+nb) )
            n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
            do j = 1, n1
               kbndpt(iright+j) = ipoint(kbndlc(j+ifirst))
            end do
            iright = iright + n1 - 1
            ifirst = ifirst + n1 - 1
         end do

      else if ( idirec==-1 .and. icolum==1 ) then

!     --- idirec = -1,  reversed direction
!         icolum = 1  First generating curve if available (globally)

         do i = 1, nb1
            jcurve = abs ( iboun(i+nb) )
            n1 = kmeshm(jcurve+5) - kmeshm(jcurve+4)
            do j = 1, n1
               kbndpt(ileft+j) = ipoint(kbndlc(j+ifirst))
            end do
            ileft  = ileft  + n1 - 1
            ifirst = ifirst + n1 - 1
         end do

      else if ( icolum<ncolum .or. idirec==-1 ) then

!     --- internal curve

         call mshfl5 ( iboun(nb+1), kmeshm, icurvs, nb1, kbndpt, kbndlc, &
                       ipoint, iwork, ifirst, 0 )
      end if
      if ( debug ) then

!     --- Debug statements

         call prinin ( icurvs, ncurvs, 'icurvs output' )
         ifirst = max(iunder, iupper, ileft, iright)
         call prinin ( kbndpt, ifirst, 'kbndpt' )

      end if
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfl4'

      end if  ! ( debug )
      call erclos ( 'mshfl4' )

      end
