      subroutine mshfl5 ( iboun, kmeshm, icurvs, nboun, kbndpt, kbndlc, &
                          ipoint, iwork, ifirst, nextracrvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 27-10-2008 Extra parameter nextracrvs
!        version  2.3  date 11-02-2004 Debug statements
!        version  2.2  date 24-07-1998 Layout
!        version  2.1  date 28-08-1993 Correction for double curves in one
!                                      surface
!        version  2.0  date 02-03-1991 Complete revision because of closed
!                                      boundaries
!        version  1.0  date 14-01-1988
!
!   copyright (c) 1988-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array KBNDPT for one surface
!    Internal curve
!    Help subroutine for subroutines MSHFL3 and MSHFL4
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), icurvs(*), ipoint(*), iboun(*), kmeshm(*), &
              kbndlc(*), iwork(*), ifirst, nextracrvs

!     iboun       i   Array of length nboun containing the curve numbers of the
!                     boundary of the local surface
!     icurvs     i/o  Array of length ncurvs containing information of the
!                     curves with respect to kbndpt
!                     icurvs is filled as follows:
!                     icurvs(i) = 0    Curve i is not present
!                     icurvs(i)>0      Curve i has not been used before
!                                      The value gives the starting position in
!                                      array kbndpt
!                     icurvs(i)<0      Curve i has been used before
!                                      Minus the value gives the starting
!                                      position in array kbndpt
!     ifirst     i/o  Last position used in kbndlc
!     ipoint      i   Help array of length npoloc, to store the new nubering of
!                     the nodes
!     iwork       i   Array of length ncurvs with information about the curves
!                     corresponding to the composite surface
!                     If iwork(i) = 0, then curve i does not belong to the
!                     surface
!                     If iwork(i)#0, then curve iwork(i) belongs to the surface
!                     The sign of iwork(i) indicates the direction of storage in
!                     KBNDPT
!     kbndlc      i   Local array KBNDPT corresponding to surface
!     kbndpt     i/o  Array containing the boundary points
!                     of the composite surface (curves are counted
!                     for only once)
!     kmeshm      i   Array containing the number of nodes in each curve
!                     accumulated (KMESH part m)
!     nboun           Number of boundary curves of local surface
!     nextracrvs  i   Number of extra (internal) curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, last, i, nboun, icurve, jcurve, k1, k2, n, &
              istart, isign, inodst, inodls, jsign, j, inodp

!     i              Counting variable
!     icurve         Curve number
!     inodls         Last user point number of a curve
!     inodp          Nodal point number
!     inodst         First user point number of a curve
!     isign          Indicates the direction of the curve
!                    1: forward direction, -1: reversed direction
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         Starting address
!     j              Counting variable
!     jcurve         abs(icurve)
!     jsign          Gives sign of curve in composite surface as stored in
!                    KBNDPT
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     last           Last user point number of preceding curve
!     n              Number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfl5' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfl5'

      end if
      if ( ierror/=0 ) go to 1000
      istar3 = kmeshm(2)-1
      last = 0
      do i = 1, nboun
         icurve = iboun(i)
         jcurve = abs(icurve)
         k1 = kmeshm(jcurve+4)
         k2 = kmeshm(jcurve+5)-1
         n = k2-k1+1
         if ( debug ) write(irefwr,*) 'i, icurve, k1, k2, n, last ', &
             i, icurve, k1, k2, n, last
         if ( icurve>0 ) then

!        --- icurve>0   Use curve forwards

            istart = icurvs(jcurve)
            isign = 1
            inodst = kmeshm(istar3+k1)
            inodls = kmeshm(istar3+k2)

         else

!        --- icurve<0   Use curve backwards

            istart = icurvs(jcurve) + n - 1
            isign = -1
            inodst = kmeshm(istar3+k2)
            inodls = kmeshm(istar3+k1)

         end if
         if ( last==inodst .and. i<=nboun-nextracrvs ) &
               ifirst = ifirst - 1      ! not for internal curves
         if ( debug ) write(irefwr,*) 'last, inodst, ifirst ', &
            last, inodst, ifirst
         if ( icurvs(jcurve)>0 ) then

!        --- icurvs(jcurve)>0, Fill curve in kbndpt

            if ( iwork(jcurve)>0 ) then

!           --- Curve in positive sense in KBNDPT

               jsign = 1

            else

!           --- Curve in negative sense in KBNDPT

               jsign = -1
               ifirst = ifirst + n + 1

            end if

            do j = 1, n
               inodp = kbndlc(j*jsign+ifirst)
               if ( inodp>0 ) kbndpt(istart+(j-1)*isign) = &
                    ipoint(inodp)
            end do
            icurvs(jcurve) = -icurvs(jcurve)
            if ( jsign==-1 ) ifirst = ifirst-n-1
            if ( debug ) write(irefwr,*) 'ifirst ', ifirst

         end if
         ifirst = ifirst + n
         last = inodls

      end do

1000  call erclos ( 'mshfl5' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfl5'

      end if

      end
