      subroutine mshmapkbnlin ( kbndbtold, map, kbndptnew, npointold, &
                                npointnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Map array kbndbtold onto kbndptnew using the information stored in array
!      map
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndbtold(*), map(*), kbndptnew(*), npointold, npointnew

!     kbndbtold      i    Old array kbndpt containing the node numbers of the
!                         7-point triangles
!     kbndptnew      o    New array kbndpt containing the node numbers of the
!                         3-point triangles
!     map            i    Mapping array containing the new node numbers of the
!                         vertices
!     npointnew      i    New number of nodes in the boundary
!     npointold      i    Old number of nodes in the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node

!     i              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2341   Internal error: node non-positive
!    2342   Internal error: map of node non-positive
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmapkbnlin' )
      npointnew = 0
      do i = 1, npointold
         node = kbndbtold(i)
         if ( node.le.0 ) then

!        --- error node <= 0

            call errint ( i, 1 )
            call errint ( node, 2 )
            call errsub ( 2341, 2, 0, 0 )
            go to 1000

         end if
         if ( map(node).gt.0 ) then

!        --- Mapping possible

            npointnew = npointnew+1
            kbndptnew(npointnew) = map(node)

         else if ( map(node).lt.0 ) then

!        --- error map(node) <= 0

            call errint ( node, 1 )
            call errint ( i, 2 )
            call errint ( map(node), 3 )
            call errsub ( 2342, 3, 0, 0 )
            go to 1000

         end if

      end do
1000  call erclos ( 'mshmapkbnlin' )
      end
