      subroutine mshmetis1 ( kmeshcpnts, ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill pointers to elements in kmeshcpnts and adapt kmeshc
!     by decreasing by 1
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     pointer
!     parallel
!     metis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: kmeshcpnts(*), ikelmc

!     ikelmc         o    position in kmesh part c
!                         At output: the length of kmeshc
!     kmeshcpnts     o    Array of size nelem+1
!                         Contains pointers to elements in kmeshc
!                         kmeshcpnts(1) = 0 and kmeshcpnts(nelem+1) contains
!                         length of kmeshc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, inpelm, ielem, iielem, i

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmetis1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmetis1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over element groups

      ikelmc = 0 ! start with 0 (c-style)
      ielem = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

!        --- Fill kmeshcpnts

         do iielem = 1, inelem

            ielem = ielem+1
            kmeshcpnts(ielem) = ikelmc
            ikelmc = ikelmc+inpelm

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp
      kmeshcpnts(ielem+1) = ikelmc

!     --- decrease kmeshc by 1 (c-style)

      kmeshc(1:ikelmc) = kmeshc(1:ikelmc)-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmetis1'

      end if  ! ( debug )
      call erclos ( 'mshmetis1' )

      end subroutine mshmetis1
