      subroutine mshmktriang ( kmeshc, iwork, nelem, inpelm )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 11-06-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Make a triangulation over an element surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), nelem, iwork(*), inpelm

!     inpelm         i    number of points per element
!     iwork          i    integer work array
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inod, itriang, ielem, idx1(8,3)

!     idx1           Array for renumbering
!     ielem          Element number
!     inod           Counting variable
!     itriang        Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
       data idx1 /2,2,3,3,9,9,4,4, 1,8,2,9,8,7,9,6, 8,9,9,4,7,6,6,5/
! ======================================================================
!
      call eropen ( 'mshmktriang' )

!     --- loop over elements

      do ielem=1,nelem

!     --- surface of brick is subdivided into 8 triangles

         do itriang=1,8

!        --- loop over nodes of separate triangles

            do inod=1,3

!           --- Adapt kmeshc of obstacle

                kmeshc(inod,(ielem-1)*8+itriang) = &
                   iwork((ielem-1)*inpelm+idx1(itriang,inod))

            end do

         end do

      end do

!     --- number of elements increases by a factor 8

      nelem=8*nelem
      inpelm=3

      call erclos ( 'mshmktriang' )

      end
