      subroutine msho54( bcord, tau, u, x, y, z, n )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.1  date 22-11-2012 Change of criterion
!        version  2.0  date 09-03-1993 new norms
!        version  1.0  date 10-05-1990
!
!   copyright (c) 1990-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to compute (x,y,z) interpolation for value u along line
!     given by a 1d array TAU
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(3,*), tau(*), u, x, y, z
      integer n

!     bcord   i     3-D coordinate array
!     n       i     number of points on line
!     tau     i     1-D coordinate array
!     u       i     arclength along line
!     x       o     x-coordinate along line for arclength u
!     y       o     y-coordinate along line for arclength u
!     z       o     z-coordinate along line for arclength u
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision ule, s, t, xp(2), yp(2), zp(2)
      integer i

!     i              loop variable
!     s              arclength
!     t              ratio parameter
!     ule            arclength
!     xp             global x-coordinates start and end point
!     yp             global y
!     zp             global z
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all points of line as given by coor
!     While total arclength not enough
!      Determine arclength for each new point
!      Determine new coordinates in 3d
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- determine coordinates along line for arclength u

      ule = 0d0
      do 100 i = 2 , n
          s    = ule
          ule  = tau(i)
          if ( ule>u ) then

!         --- interpolate

               if ( u<1d-5) then
                    t = 0
               else
                    t = ( u - s ) / ( ule - s )
               end if
               xp(1) = bcord(1,i-1)
               yp(1) = bcord(2,i-1)
               zp(1) = bcord(3,i-1)
               xp(2) = bcord(1,i)
               yp(2) = bcord(2,i)
               zp(2) = bcord(3,i)
               x  = xp(1) + t * ( xp(2) - xp(1) )
               y  = yp(1) + t * ( yp(2) - yp(1) )
               z  = zp(1) + t * ( zp(2) - zp(1) )

!              --- Ready, escape

               goto 200
          end if
100   continue
200   continue
      end
