      subroutine mshobstboun3d1 ( iwork, numbounelms, ibounelems )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 29-11-2004 Correction for more obstacles
!        version  3.0  date 26-05-2004 Extension for more obstacles
!        version  2.0  date 07-01-2004 Extra parameter outerboun
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements that have all nodes on the obstacle or on the
!     obstacle and an outer boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     3d
!     surface
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), numbounelms, ibounelems(*)

!     ibounelems     o    Array of length 3+numbounelms (<nelem) containing
!                         the element numbers of the elements that have all
!                         points on the boundary or on the obstacle
!                         Pos. 1 contains the number of elements that have all
!                                points on the obstacle
!                         Pos. 2 contains the number of elements that have all
!                                points on the boundary
!                         Pos. 3 contains the number of elements that have some
!                                points on the boundary and the others on the
!                                obstacle
!                         The other positions contain the element numbers in the
!                         sequence: first all elements corresponding to pos. 1
!                         then to pos 2 and finally to pos 3
!     iwork          o    integer work array of length npoint
!                         iwork(i) is filled as follows (see pseudo code)
!                          0 for all points outside the obstacle and
!                            the boundaries
!                         -1 for all points inside the obstacle
!                          i for all points on the boundary of the obstacle i
!                         1001 for all points on the outer surfaces
!                              but not on an obstacle
!                         1002 for all points on the outer surfaces
!                              and on an obstacle
!     numbounelms    o    Number of elements that have all points on the
!                         boundary of the obstacle or a set of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, numpntobst, numinnobst, numbndobst, &
              ipart, j, inode, ielem, ikelmc, istartn, &
              ielgrp, iielem, numobstelms, numbnobelms, nparts, nouter, &
              list(1), iobst, numinonobst
      logical same
      integer :: iwork1(3,nelem)

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inode          Node sequence number
!     iobst          Obstacle sequence number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     istartn        First position of a surface in kmeshn
!     iwork1         integer work array of length 3*nelem
!                    iwork1(1,.) contains the elements that have all points
!                    on the obstacle
!                    iwork1(2,.) contains the elements that have all points
!                    on the boundary
!                    iwork1(3,.) contains the elements that have some
!                    points on the boundary and the others on the obstacle
!     j              Counting variable
!     list           dummy
!     nouter         Number of outer boundaries in a part
!     nparts         number of parts the outer boundary consists of
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numbnobelms    number of elements that have some points on the boundary
!                    and the others on the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numobstelms    number of elements that have all points on the obstacle
!     numpntobst     Number of nodal points inside the obstacle
!     same           If true all nodes belong to the same group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0009         Print information of surfaces
!     PRINIBOUNELS   Print the contents of array ibounelems
!     PRININ         print 1d integer array
!     PRINIOUTERCS   Print array ioutercurvs or outerboun, &
!                    depending on the dimension
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iwork in the following way:
!        iwork(i) = 0 for all points outside the obstacle and the boundaries
!        iwork(i) =-1 for all points inside the obstacle
!        iwork(i) = i for all points on the boundary of the obstacle i
!        iwork(i) = 1001 for all points on the outer surfs but not on an
!                        obstacle
!        iwork(i) = 1002 for all points on the outer surfs and on an obstacle
!     numbounelms := 0
!     numobstelms := 0
!     numbnobelms := 0
!     For all elements ielem do
!        If ( all nodes in element have iwork(i) > 0 ) then
!           if ( all nodes in element have iwork(i) = 1 ) then
!              numobstelms := numobstelms+1
!              iwork1(1,numobstelms) := ielem
!           else if ( all nodes in element have iwork(i) = 2 ) then
!              numbounelms := numbounelms+1
!              iwork1(2,numbounelms) := ielem
!           else ( all nodes in element have iwork(i) = 1 or 2 ) then
!              numbnobelms := numbnobelms+1
!              iwork1(3,numbnobelms) := ielem
!     Copy contents of iwork1 to ibounelems
!     numbounelms := numbounelms+numobstelms+numbnobelms
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun3d1' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstboun3d1'
         call prinioutercs
         call pr0009 ( 0, kmeshn, coor, list, 0, 3, nsurfs )
         call sepprinipobst ( ipointobst )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill array iwork in the following way:
!         iwork(i) = 0 for all points outside the obstacle and the boundaries
!         iwork(i) =-1 for all points inside the obstacle
!         iwork(i) = i for all points on the boundary of the obstacle i
!         iwork(i) = 1001 for all points on the outer surfaces but not on an
!                         obstacle
!         iwork(i) = 1002 for all points on the outer surfaces and on an
!                         obstacle

!     --- Clear iwork

      do i = 1, npoint
         iwork(i) = 0
      end do

!     --- For all obstacles do

      do iobst = 1, numobst

!     --- Fill points inside the obstacle

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+3
         do i = 1, numpntobst+numinnobst+numinonobst
            iwork(ipointobst(istart+i)) = -1
         end do

!        --- Fill points on the boundary of the obstacle

         istart = istart+numpntobst+numinnobst
         do i = 1, numbndobst+numinnobst
            iwork(ipointobst(istart+i)) = iobst
         end do

      end do  ! iobst = 1, numobst

!     --- Fill points on surfaces

      nparts = outerboun(1)   ! number of parts the outer surface consists of

!     --- Loop over all parts

      do ipart = 1, nparts

         istart = outerboun(1+ipart)   ! starting address part ipart
         nouter = outerboun(istart)    ! number of surfaces in this part

!        --- Loop over surfaces in this part

         do j = 1, nouter

            isurnr = abs(outerboun(istart+j))  ! surface number

            istartn = 2*nsurfs    ! starting address of surface
            do i = 1, isurnr-1
               istartn = istartn+kmeshn(i)*kmeshn(nsurfs+i)
            end do
            inpelm = kmeshn(isurnr)
            inelem = kmeshn(nsurfs+isurnr)
            do i = 1, inpelm*inelem
               inode = kmeshn(istartn+i)
               if ( inode>0 ) then

!              --- Node on surface found
!                  Raise iwork if it is not an inner node of the obstacle

                  if ( iwork(inode)==0 ) then

!                 --- Node not on obstacle

                     iwork(inode) = 1001

                  else if ( iwork(inode)>0 .and. &
                            iwork(inode)<=1000 ) then

!                 --- Node on obstacle and boundary

                     iwork(inode) = iwork(inode)+1001  ! add 1001 to iobst

                  end if  ! ( iwork(inode)==0 )

               end if  ! ( inode>0 )

            end do  ! i = 1, inpelm*inelem

         end do  ! j = 1, nouter

      end do  ! ipart = 1, nparts

!     --- numbounelms := 0
!         numobstelms := 0
!         numbnobelms := 0

      numbounelms = 0
      numobstelms = 0
      numbnobelms = 0

!     --- For all elements ielem do

      ielem = 0
      ikelmc = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         if ( ishape<11 ) then

!        --- element is not three-dimensional: skip

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem

         else

            do iielem = 1, inelem

               ielem = ielem+1

!              --- If ( all nodes in element have iwork(i) > 0 ) then
!                  if ( all nodes in element have iwork(i) = iobst(<1001) ) then
!                     numobstelms := numobstelms+1
!                     iwork1(1,numobstelms) := ielem
!                  else if ( all nodes in element have iwork(i) = 1001 ) then
!                     numbounelms := numbounelms+1
!                     iwork1(2,numbounelms) := ielem
!                  else ( all nodes in element have same iwork(i) ) then
!                     numbnobelms := numbnobelms+1
!                     iwork1(3,numbnobelms) := ielem

               inode = kmeshc(ikelmc+1)   ! first node in element
               if ( iwork(inode)<=0 ) go to 100  ! not all nodes on
                                                   ! boundaries, jump
               iobst = iwork(inode)       ! type of first node
               same = .true.
               do i = 2, inpelm
                  inode = kmeshc(ikelmc+i)
                  if ( iwork(inode)<=0 ) go to 100  ! see node 1
                  if ( iwork(inode)/=iobst ) same = .false.

               end do  ! i = 2, inpelm

               if ( .not. same ) then

!              --- some nodes on boundary of obstacle, others on boundary

                  numbnobelms = numbnobelms+1
                  iwork1(3,numbnobelms) = ielem
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Nodes of element ', ielem, &
                                     ' on boundary and obstacle'
                     call prinin ( kmeshc(ikelmc+1), inpelm, 'element' )

                  end if

               else if ( same .and. iwork(inode)==1001 ) then

!              --- all nodes on boundary

                  numbounelms = numbounelms+1
                  iwork1(2,numbounelms) = ielem
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Nodes of element ', ielem, &
                                     ' on boundary'
                     call prinin ( kmeshc(ikelmc+1), inpelm, 'element' )

                  end if
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Nodes of element ', ielem, &
                                     ' on boundary of obstacle'
                     call prinin ( kmeshc(ikelmc+1), inpelm, 'element' )

                  end if

               else if ( same .and. iwork(inode)<=1000 ) then

!              --- all nodes on boundary of obstacle iobst

                  numobstelms = numobstelms+1
                  iwork1(1,numobstelms) = ielem
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Nodes of element ', ielem, &
                                     ' on boundary of obstacle ', iobst
                     call prinin ( kmeshc(ikelmc+1), inpelm, 'element' )

                  end if

               end if

100            ikelmc = ikelmc+inpelm

            end do

         end if

      end do

!     --- Copy contents of iwork1 to ibounelems

      ibounelems(1) = numobstelms
      ibounelems(2) = numbounelms
      ibounelems(3) = numbnobelms
      istart = 3
      do i = 1, numobstelms
         ibounelems(istart+i) = iwork1(1,i)
      end do
      istart = istart+numobstelms
      do i = 1, numbounelms
         ibounelems(istart+i) = iwork1(2,i)
      end do
      istart = istart+numbounelms
      do i = 1, numbnobelms
         ibounelems(istart+i) = iwork1(3,i)
      end do

!     --- numbounelms := numbounelms+numobstelms+numbnobelms

      numbounelms = numbounelms+numobstelms+numbnobelms

1000  if ( debug ) then

!     --- Debug information

         call prinibounels ( ibounelems )
         write(irefwr,*) 'End mshobstboun3d1'

      end if
      call erclos ( 'mshobstboun3d1' )

      end
