      subroutine mshobstms2d04 ( trianglenode, trianglenodenew, &
                                 edgenode )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  3.3  date 19-05-2005 Reset edgenode
!        version  3.2  date 07-04-2004 Extension of edgenode (RvL)
!        version  3.1  date 28-12-2001 Adaptation comments
!        version  3.0  date 19-10-2001 Remove parameters
!        version  2.0  date 03-07-2001 Complete revision
!        version  1.0  date 07-03-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into two triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(3), trianglenodenew(6), edgenode(3)

!     edgenode        i   Array with nodes on the crossed edge
!                         edgenode(1) first node on crossed edge
!                         edgenode(2) last node on crossed edge
!                         edgenode(3) new node on crossed edge
!     trianglenode    i   Array of nodal point numbers of the corners of
!                         the original triangle
!     trianglenodenew o   Array of nodal point numbers of the corners of
!                         the new triangles.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer raise(3), i, inode, iseq
      logical counterclock

!     counterclock   If true the direction of the nodes is counter clockwise
!     i              Counting variable
!     inode          Node sequence number of the node not on the crossed edge
!     iseq           Sequence number of node inode in kmeshc
!     raise          Raise-operator
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     inode := third node in triangle
!     triangle1 := ( inode,newpoint,inode1)
!     triangle2 := ( inode,newpoint,inode2)
!     Change direction if necessary to get counterclockwise ordering
!
!     We have the following situations
!
!   edgenode(2)                       edgenode(1)
!     |\                                |\
!     | \                               | \
!     |  \                              |  \
!     |   \                             |   \
!     |    \                            |    \
!     |     \ edgenode(3)               |     \ edgenode(3)
!     |    / \                          |    / \
!     |  /    \                         |  /    \
!     |/       \                        |/       \
!     ---------- ---------- &
!   inode       edgenode(1)           inode     edgenode(2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise / 2, 3, 1 /
! ======================================================================
!
!     --- inode := third node in triangle

      do i = 1, 3
         if ( trianglenode(i)/=edgenode(1) .and. &
              trianglenode(i)/=edgenode(2) ) then

!        --- Third node found

            inode = trianglenode(i)
            iseq = i
            go to 100
         end if
      end do
100   continue

      if ( trianglenode(raise(iseq))==edgenode(1) ) then

!     --- inode to edgenode(1) is counterclockwise

         counterclock = .true.

      else

!     --- inode to edgenode(1) is clockwise

         counterclock = .false.

      end if

!     --- triangle1 := ( inode,edgenode(1),edgenode(3))

      if ( counterclock ) then

!     --- counterclockwise

         trianglenodenew(1) = inode
         trianglenodenew(2) = edgenode(1)
         trianglenodenew(3) = edgenode(3)

      else

!     --- clockwise

         trianglenodenew(1) = inode
         trianglenodenew(2) = edgenode(3)
         trianglenodenew(3) = edgenode(1)

      end if

!     --- triangle2 := ( inode,edgenode(3),edgenode(2))

      if ( counterclock ) then

!     --- counterclockwise

         trianglenodenew(4) = inode
         trianglenodenew(5) = edgenode(3)
         trianglenodenew(6) = edgenode(2)

      else

!     --- clockwise

         trianglenodenew(4) = inode
         trianglenodenew(5) = edgenode(2)
         trianglenodenew(6) = edgenode(3)

      end if

      end
