      subroutine mshobstms2d05 ( trianglenode, trianglenodenw, &
                                 edgenode, coornew, nedgenode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 19-05-2005 Extra parameter nedegenode
!        version  2.2  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.1  date 28-12-2001 Adaptation comments
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into three triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(3), trianglenodenw(9), nedgenode, &
              edgenode(nedgenode,2)
      double precision coornew(2,*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgenode       i    Array with nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     nedgenode      i    Length of first index of edgenode
!     trianglenode   i    Array of nodal point numbers of the corners of
!                         the original triangle
!     trianglenodenw o    Array of nodal point numbers of the corners of
!                         the new triangles.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer raise(4), i, node1, node2, node3, node4, node5, iseq
      double precision l2norm, tang1(2), tang2(2), cosangle1, cosangle2

!     cosangle1      cosine of angle1
!     cosangle2      cosine of angle2
!     i              Counting variable
!     iseq           Sequence number
!     l2norm         L2-norm of vector
!     node1          Node of face
!     node2          Node of face
!     node3          Node of face
!     node4          Node of face
!     node5          Node of face
!     raise          Raise-operator
!     tang1          First tangential vector
!     tang2          Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     node1 := common node of edgenode(1,1), edgenode(2,1), edgenode(1,2)
!              and edgenode(2,2)
!     iseq := sequence number of node1 in trianglenode
!     node2 := next node in trianglenode
!     node3 := last node in trianglenode
!     if ( node2 in ( edgenode(1,1),edgenode(2,1) ) then
!        node4 := edge(3,1)
!        node5 := edge(3,2)
!     else ( node2 in ( edgenode(1,2),edgenode(2,2) )
!        node4 := edge(3,2)
!        node5 := edge(3,1)
!     triangle1 := (node1,node4,node5)
!     if ( angle(node5-node4,node4-node2)>angle(node3-node5,node5-node4)) then
!        triangle2 := (node4,node2,node3)
!        triangle3 := (node3,node5,node4)
!     else ( angle(node5-node4,node4-node2)<angle(node3-node5,node5-node4) )
!        triangle2 := (node5,node4,node2)
!        triangle3 := (node2,node3,node5)
!
!      1                      1
!      |\                     |\
!      | \                    | \                 5                     5
!      |  \5                  |  \ 5            /\                    /\
!      |  /\                  |  /            /   \                 / / \
!      |/   \                 |/          4  |\    \            4  |     \
!   4  |     \             4  |              |   \  \              | /    \
!      |      \  -->                         |       \     or      |       \
!      |       \                             |      \ \            |/       \
!      |        \                            ---------- ---------- &
!      ---------- 2          3          2          3
!      2          3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise / 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshobstms2d05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms2d05'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- node1 := common node of inode1, inode2, inode3, inode4

      if ( edgenode(1,1)==edgenode(1,2) .or. &
           edgenode(1,1)==edgenode(2,2) ) then
         node1 = edgenode(1,1)
      else
         node1 = edgenode(2,1)
      end if

!     --- iseq := sequence number of common node node1 in triangle node

      do i = 1, 3
         if ( trianglenode(i)==node1 ) then

!        --- Node found

            iseq = i
            go to 100
         end if
      end do
 100  continue

!     --- node2 := next node in trianglenode
!         node3 := last node in trianglenode

      node2 = trianglenode(raise(iseq))
      node3 = trianglenode(raise(iseq+1))

      if ( node2==edgenode(1,1) .or. node2==edgenode(2,1) ) then

!     --- node2  = edgenode(1,1) or edgenode(2,1))
!         node4 := edgenode(3,1)
!         node5 := edgenode(3,2)

         node4 = edgenode(3,1)
         node5 = edgenode(3,2)

      else

!     --- node2 in (edgenode(1,2),edgenode(2,2))
!         node4 := edgenode(3,2)
!         node5 := edgenode(3,1)

         node4 = edgenode(3,2)
         node5 = edgenode(3,1)

      end if

!     --- triangle1 := (node1,node4,node5)

      trianglenodenw(1) = node1
      trianglenodenw(2) = node4
      trianglenodenw(3) = node5

!     --- Compute cosine of two angles

      tang1(1) = coornew(1,node4)-coornew(1,node5)
      tang1(2) = coornew(2,node4)-coornew(2,node5)
      l2norm = 1d0/sqrt(tang1(1)**2+tang1(2)**2)
      tang1(1) = l2norm*tang1(1)
      tang1(2) = l2norm*tang1(2)
      tang2(1) = coornew(1,node3)-coornew(1,node5)
      tang2(2) = coornew(2,node3)-coornew(2,node5)
      l2norm = 1d0/sqrt(tang2(1)**2+tang2(2)**2)
      tang2(1) = l2norm*tang2(1)
      tang2(2) = l2norm*tang2(2)
      cosangle1 = tang1(1)*tang2(1)+tang1(2)*tang2(2)

      tang2(1) = coornew(1,node2)-coornew(1,node4)
      tang2(2) = coornew(2,node2)-coornew(2,node4)
      l2norm = 1d0/sqrt(tang2(1)**2+tang2(2)**2)
      tang2(1) = l2norm*tang2(1)
      tang2(2) = l2norm*tang2(2)
      cosangle2 = -tang1(1)*tang2(1)-tang1(2)*tang2(2)

      if ( cosangle1>=cosangle2 ) then

!     --- angle(node5-node4,node4-node2)>angle(node3-node5,node5-node4)
!         triangle2 := (node4,node2,node3)

         trianglenodenw(4) = node4
         trianglenodenw(5) = node2
         trianglenodenw(6) = node3

!        --- triangle3 := (node3,node5,node4)

         trianglenodenw(7) = node3
         trianglenodenw(8) = node5
         trianglenodenw(9) = node4

      else

!     --- angle(node5-node4,node4-node2)<angle(node3-node5,node5-node4)
!         triangle2 := (node5,node4,node2)

         trianglenodenw(4) = node5
         trianglenodenw(5) = node4
         trianglenodenw(6) = node2

!        --- triangle3 := (node2,node3,node5)

         trianglenodenw(7) = node2
         trianglenodenw(8) = node3
         trianglenodenw(9) = node5

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms2d05'

      end if  ! ( debug )
      call erclos ( 'mshobstms2d05' )

      end subroutine mshobstms2d05
