      subroutine mshobstms2d06 ( trianglenode, trianglenodenw, &
                                 edgenode, coornew, nedgenode )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  4.0  date 19-05-2005 Extra parameter nedegenode
!        version  3.2  date 25-10-2004 Debug statements
!        version  3.1  date 07-04-2004 Extension of edgenode (RvL)
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into four triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(3), trianglenodenw(12), nedgenode, &
              edgenode(nedgenode,3)
      double precision coornew(2,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     edgenode       i    Array with nodes on the crossed edges
!                         edgenode(1,i): first node on the i-th crossed edge
!                         edgenode(2,i): last node on the i-th crossed edge
!                         edgenode(3,i): new node on the i-th crossed edge
!     nedgenode      i    Length of first index of edgenode
!     trianglenode   i    Array of nodal point numbers of the corners of
!                         the original triangle
!     trianglenodenw o    Array of nodal point numbers of the corners of
!                         the new triangles.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, gt90, temp, case
      integer local(3), edgenodel(3), raise(4)
      double precision tempdp, sqvec1, sqvec2, sqvec4, sqvec5, &
                       sqcos2, sqcos4, sqcos5
      double precision vec(2,5), dot(5)

!     case        two ways to subdivide (rotated) triangle
!     dot         i-th inner product
!     edgenodel   The global node number of the i-th local edgenode
!     gt90        true if one of the angles a1, a2, a3 is larger than 90
!     i           Counting variable
!     j           Counting variable
!     local       the local position of edgenode (i)
!     raise       Raise-operator
!     sqcos2      square of cosine of angle 2
!     sqcos4      square of cosine of angle 4
!     sqcos5      square of cosine of angle 5
!     sqvec1      inner product of vec1 with itself
!     sqvec2      inner product of vec2 with itself
!     sqvec4      inner product of vec4 with itself
!     sqvec5      inner product of vec5 with itself
!     temp        temporary integer
!     tempdp      temporary double precission
!     vec         the x and y coordinae of vector i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find the local position of the edgenodes
!     If the angles a1 and a2 and a3<90 subdivide as follows
!
!
!                        t3
!     case 1             /\
!     a1,a2,a3<90     /  \
!                      /    \
!                     /      \
!                e3  /_______ \ e2
!                   /\a3    a2/\
!                  /  \      /  \
!                 /    \    /    \
!                /      \a1/      \
!               /        \/        \
!              ---------------------- &
!             t1         e1          t2
!
!
!     If not (a1 and a2 and a3<90) rotate the ai>90 to a2 position
!
!     If a2<a4 or a2<a5
!        subdivide as below (the same as a1,a2,a3<90)
!
!                           t3                               t3
!        case 1             /\                               /\
!        a2>90 and       /  \                             /  \
!        (a2<a4 or   e3 /_   \                        e3 /_   \
!         a2<a5 )      /a4\__ \                         /| \__ \
!                       /      \_\ e2                    / |    \_\ e2
!                      /       a2/\          -->        /   |     /\
!                     /         /  \                   /    |    /  \
!                    /         /    \                 /     |   /    \
!                   /         /      \               /       | /      \
!                  /       a5/        \             /        |/        \
!                 ---------------------- ---------------------- &
!                t1         e1          t2         t1         e1          t2
!
!     else
!
!
!                           t3                               t3
!        case 2             /\                               /\
!        a2>90 and       /  \                             /  \
!        (a2>a4 and     /_   \                        e3 /_   \
!         a2>a5 )      /a4\__ \                         /  \__ \
!                   e3  /      \_\ e2                    /      \_\ e2
!                      /       a2|\       -->           /       _/|\
!                     /          | \                   /     __/  | \
!                    /           |  \                 /   __/     |  \
!                   /            |   \               / __/        |   \
!                  /           a5|    \             /_/           |    \
!                 ---------------------- ---------------------- &
!                t1              e1     t2        t1              e1     t2
!      end if
!
!      where ti = i-th trianglenode     }
!            ei = i-th edgenode         }    after rotation
!            ai = i-th angle            }
!
!      Mark the new elements as in or outside obstacle
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise / 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshobstms2d06' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms2d06'
         call prinin ( trianglenode, 3, 'trianglenode' )
         call prinin ( edgenode(1,1), 3, 'edgenodes_1' )
         call prinin ( edgenode(1,2), 3, 'edgenodes_2' )
         call prinin ( edgenode(1,3), 3, 'edgenodes_3' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find the local position of the edgenodes

      do i = 1, 3
         if ( edgenode(1,1)==trianglenode(i) )then
            if( edgenode(2,1)==trianglenode(raise(i)) ) then
               local(1) = i
            else
               local(1) = raise(i+1)
            end if
            if(edgenode(1,2)==trianglenode(local(1)).or. &
               edgenode(2,2)==trianglenode(local(1))    ) then
               local(2) = raise(local(1)+1)
               local(3) = raise(local(1))
            else
               local(3) = raise(local(1)+1)
               local(2) = raise(local(1))
            end if
            go to 10
         end if
      end do
 10   continue

      if ( local(1)<1 .or. local(1)>3 ) stop
      if ( local(2)<1 .or. local(2)>3 ) stop
      if ( local(3)<1 .or. local(3)>3 ) stop
      do i = 1, 3
         edgenodel(local(i)) = edgenode(3,i)
      end do

!     --- calculate the vectors representing the edges of
!         triangle (edgenodel(i),i=1,3). This is done to examine
!         the angles of the triangle.

      do i = 1, 3
         do j = 1, 2
           vec(j,i) =  coornew(j,edgenodel(raise(i))) &
                      -coornew(j,edgenodel(i))
        end do
      end do

!     --- check the angles of this triangle (edgenodel(i),i=1,3))
!         Only one of these angles can be>90 degrees.
!         The inner product of its nabouring edges is then positive.
!         This angle  will be rotated to edgenode position 2

      dot(1) = vec(1,3)*vec(1,1) + vec(2,3)*vec(2,1)
      dot(2) = vec(1,1)*vec(1,2) + vec(2,1)*vec(2,2)
      dot(3) = vec(1,2)*vec(1,3) + vec(2,2)*vec(2,3)
      if ( dot(2)>0.d00 ) then

!     --- angle 2>90, no rotation required

         gt90 = 2

      else if ( dot(3)>0.d00 ) then

!     --- angle 3>90, rotate

         gt90 = 3
         temp = trianglenode(2)
         trianglenode(2) = trianglenode(3)
         trianglenode(3) = trianglenode(1)
         trianglenode(1) = temp
         temp = edgenodel(2)
         edgenodel(2) = edgenodel(3)
         edgenodel(3) = edgenodel(1)
         edgenodel(1) = temp
         tempdp  = dot(2)
         dot(2) = dot(3)
         dot(3) = dot(1)
         dot(1) = tempdp
         do j =1,2
            tempdp = vec(j,2)
            vec(j,2) = vec(j,3)
            vec(j,3) = vec(j,1)
            vec(j,1) = tempdp
         end do

      else if ( dot(1)>0.d00 ) then

!     --- angle 1>90, rotate

         gt90 = 1
         temp = trianglenode(2)
         trianglenode(2) = trianglenode(1)
         trianglenode(1) = trianglenode(3)
         trianglenode(3) = temp
         temp = edgenodel(2)
         edgenodel(2) = edgenodel(1)
         edgenodel(1) = edgenodel(3)
         edgenodel(3) = temp
         tempdp  = dot(2)
         dot(2) = dot(1)
         dot(1) = dot(3)
         dot(3) = tempdp
         do j =1,2
            tempdp = vec(j,2)
            vec(j,2) = vec(j,1)
            vec(j,1) = vec(j,3)
            vec(j,3) = tempdp
         end do

      else

!     --- no angle>90

         gt90 = 0

      end if

      if( gt90==0 ) then

!     --- case 1

         case = 1

      else

         sqvec1 = -dot(1)-dot(2)
         sqvec2 = -dot(2)-dot(3)
         sqcos2 = (dot(2)*dot(2))/(sqvec1*sqvec2)
         do j = 1, 2
            vec(j,4) = coornew(j,trianglenode(1)) &
                      -coornew(j,edgenodel(3))
            vec(j,5) = coornew(j,edgenodel(1)) &
                      -coornew(j,trianglenode(1))
         end do
         dot(4) = vec(1,4)*vec(1,2) + vec(2,4)*vec(2,2)
         if( dot(4)>0.d00 ) then
            sqvec4 = vec(1,4)*vec(1,4) + vec(2,4)*vec(2,4)
            sqcos4 = (dot(4)*dot(4))/(sqvec2*sqvec4)
            if( sqcos4>sqcos2 ) then

!           --- case 1

               case = 1
               go to 20
            end if
         end if
         dot(5) = vec(1,5)*vec(1,1) + vec(2,5)*vec(2,1)
         if( dot(5)>0.d00 ) then
            sqvec5 = vec(1,5)*vec(1,5) + vec(2,5)*vec(2,5)
            sqcos5 = (dot(5)*dot(5))/(sqvec1*sqvec5)
            if( sqcos5>sqcos2 ) then

!           --- case 1

               case = 1
            else

!           --- no case 1 encountered, so case 2

               case = 2
            end if
         end if
      end if
 20   continue

      trianglenodenw(1) = trianglenode(1)
      trianglenodenw(2) = edgenodel(1)
      if ( case==1 ) then
         trianglenodenw(3) = edgenodel(3)
         trianglenodenw(4) = edgenodel(1)
      else
         trianglenodenw(3) = edgenodel(2)
         trianglenodenw(4) = trianglenode(1)
      end if
      trianglenodenw(5) = edgenodel(2)
      trianglenodenw(6) = edgenodel(3)
      trianglenodenw(7) = trianglenode(2)
      trianglenodenw(8) = edgenodel(2)
      trianglenodenw(9) = edgenodel(1)
      trianglenodenw(10) = trianglenode(3)
      trianglenodenw(11) = edgenodel(3)
      trianglenodenw(12) = edgenodel(2)

1000  call erclos ( 'mshobstms2d06' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms2d06'

      end if

      end
