      subroutine mshobstms3d04 ( tetnode, tetnodenew, edgenode, &
                                 isort, nelemnew )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  2.2  date 19-05-2005 Reset of edgenode
!        version  2.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 03-09-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide tetrahedron with only one crossed edge into 2 tetrahedrons.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4), tetnodenew(8), edgenode(3,3), isort(4), &
              nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     nelemnew       o    Number of new elements
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, trans(4), transtemp, &
               tetnodet(4), edgenodet(6)
      logical oddpairintchan

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! --- Transform tetrahedron {tetnode(i),i=1,4} to {tetnodet(i),i=1,4}
!     such that
!     tetnodet(1) = edgenode(1,1) and tetnodet(2) = edgenode(1,2).
! --- Split tetrahedron into
!        tetrahedron1 = { tetnodet(1), edgenode(3,1), &
!                       tetnodet(3), tetnodet(4) }
!     and
!        tetrahedron2 = { edgenode(3,1), tetnodet(2), &
!                       tetnodet(3), tetnodet(4) }
!
!
!     We have the following situation
!
!
!                 tetnodet(4)
!
!                    /|\
!                   / | \ \
!                  /  |  \  \
!                 /   |   \   \  tetnodet(3)
!                /    |    \ _/|
!               /     |.  __X  |
!              /  .   | _/   \ |
!             /_______|/______\|
!
!    tetnodet(1)   edgenode(3,1)   tetnodet(2)
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms3d04' )

!     --- Define transformation to move edgenode(3,1) to edge 1
!         This will be done by moving
!         1)  tetnode(isort(4)) to position 1
!         2)  tetnode(isort(3)) to position 2

!         --- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move tetnode(isort(4)) to position 1

         if( isort(4)/=1 ) then
            transtemp = trans(1)
            trans(1) = isort(4)
            trans(isort(4)) = transtemp
            oddpairintchan = .not.oddpairintchan

         end if

!        --- move tetnode(isort(3)) to position 2

         if( isort(3)/=trans(2) ) then
            transtemp = trans(2)
            trans(2) = isort(3)

!           --- note that at this stage trans is still its own inverse

            trans(trans(isort(3))) = transtemp
            oddpairintchan = .not.oddpairintchan

         end if

!        --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(3)
         trans(3) = trans(4)
         trans(4) = transtemp
      end if

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Transform the intersection points to their new positions

      edgenodet(1) = edgenode(3,1)

!     --- Subdivide tetrahedron into two tetrahedrons

      tetnodenew(1) = tetnodet(1)
      tetnodenew(2) = edgenodet(1)
      tetnodenew(3) = tetnodet(3)
      tetnodenew(4) = tetnodet(4)
      tetnodenew(5) = edgenodet(1)
      tetnodenew(6) = tetnodet(2)
      tetnodenew(7) = tetnodet(3)
      tetnodenew(8) = tetnodet(4)

      nelemnew = 2

      call erclos ( 'mshobstms3d04' )

      end
