      subroutine mshparelsort ( ifirst, ilast, iprocessor, &
                                nprocessors, elements, numelms, &
                                nelemproc, elementsact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-03-2015 New parameter list
!        version  1.0  date 27-11-2003
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elements with the elements corresponding to one block
!     and also the corresponding array points with the point numbers
!     In this case the elements are subdivided such that layers of elements
!     arise with approximately the same number of elements per layer
!     Elements are not sorted
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifirst, ilast, iprocessor, nprocessors, &
              elements(*), numelms, nelemproc, elementsact(*)

!     elements       o    Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     elementsact   i/o   Array of size nelem indicating which elements are
!                         already part of a submesh (>0) or not yet (=0)
!     ifirst        i/o   First node in next block
!     ilast         i/o   Last node in previous block
!     iprocessor     i    Processor sequence number
!     nelemproc      i    Number of nodes per processor (approximately)
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     numelms        o    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem

!     i              Counting variable
!     ielem          Element number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     clear array iwork(1:npoint)
!     ilast := ilast+nelemproc
!     if ( iprocessor = nprocessors ) then
!        ilast := nelem
!     numelms := ilast-ifirst+1
!     For i := 1 (1) numelms do
!        ielem := i+ifirst-1
!        elements(i) := ielem
!        for all nodes inode in element ielem do
!           iwork(inode) := 1
!     ifirst := ilast+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelsort' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelsort'
         write(irefwr,*) 'nprocessors, npoint, nelemproc ', &
                          nprocessors, npoint, nelemproc

      end if

!     --- ilast := ilast+nelemproc
!         if ( iprocessor = nprocessors ) then
!            ilast := nelem

      ilast = ilast+nelemproc
      if ( iprocessor==nprocessors ) ilast = nelem

!     --- numelms := ilast-ifirst+1
!         For i := 1 (1) numelms do

      numelms = ilast-ifirst+1
      do i = 1, numelms

!     --- ielem := i+ifirst-1
!         elements(i) := ielem

         ielem = i+ifirst-1
         elements(i) = ielem
         elementsact(ielem) = iprocessor

      end do  ! i = 1, numelms

!     --- ifirst := ilast+1

      ifirst = ilast+1

      if ( debug ) then
         call prinin ( elements, numelms, 'elements' )
         write(irefwr,*) 'End mshparelsort'
      end if
      call erclos ( 'mshparelsort' )

      end
