      subroutine mshparinv ( mapar, lenloc, maparinv, lenglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-03-2015 Debug
!        version  1.0  date 04-07-2001
!
!   copyright (c) 2001-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the inverse mapping (i.e. mapping from global to local mesh)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mapping_array
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenloc, lenglob, mapar(lenloc), maparinv(lenglob)

!     lenglob        i    Length of array with global nodes, hence
!                         length of maparinv
!     lenloc         i    Length of array with local nodes, hence
!                         length of mapar
!     mapar          i    mapping array: contains the mapping from local to
!                         global numbering of the nodal points
!     maparinv       o    Inverse of array mapar, etended by zero's where
!                         mapar does not exist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear maparinv
!     Fill maparinv
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparinv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparinv'
         call prinin ( mapar, lenloc, 'mapar' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear maparinv

      maparinv(1:lenglob) = 0

!     --- Fill maparinv

      do i = 1, lenloc
         maparinv(mapar(i)) = i
      end do

1000  call erclos ( 'mshparinv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparinv'

      end if  ! ( debug )

      end subroutine mshparinv
