      subroutine mshparkmeshc ( kmeshcloc, lenkmeshc, &
                                nelemloc, elements, pointmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-02-2004 New parameter list
!        version  1.0  date 12-02-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill local arrays kmeshc and kmeshg using global arays kmeshc and kmeshg
!     and the arrays elements and points
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcloc(*), lenkmeshc, &
              nelemloc, elements(*), pointmap(*)

!     elements       i    Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     kmeshcloc      o    Local array kmeshc containing the node numbers of one
!                         element
!     lenkmeshc      o    length of array kmeshc
!     nelemloc       i    Number of elements in local mesh
!     pointmap       i    Array of size npoint containing the local node
!                         numbers of the nodes in the global mesh
!                         if node i is not present in local mesh pointmap(i) = 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifirst, ilast, j, numelms, ielem, inpelm, ielgrp, &
              iielem, ikelmc
      integer, allocatable :: elemmap(:)

!     elemmap        Array of size nelem containing the local element
!                    numbers of the elements in the global mesh
!                    if element i is not present in local mesh
!                    elemmap(i) = 0
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     ifirst         First global element number of element group
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     ilast          Last global element number of element group
!     inpelm         number of points per element
!     j              Counting variable
!     numelms        Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array elemmap
!     Fill array elemmap from elements
!     Fill array kmeshgloc from kmeshgglob and elemmap
!     Fill array kmeshcloc from kmeshc and elemmap and pointmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparkmeshc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparkmeshc'

      end if

!     --- Clear array elemmap

      allocate ( elemmap(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'elemmap' )
      if ( ierror/=0 ) go to 1000

      elemmap = 0

!     --- Fill array elemmap from elements

      do i = 1, nelemloc
         elemmap(elements(i)) = i
      end do

      if ( debug ) then

!     --- Debug information

         call prinin ( elements, nelemloc, 'elements' )
         call prinin ( elemmap, nelem, 'elemmap' )

      end if

!     --- Fill array kmeshgloc from kmeshgglob and elemmap

      if ( nelgrp>1 ) then

!     --- nelgrp > 1, more than one element group, kmeshg is used
!         The number of elements per group for the global mesh is stored
!         in kmesh from position kelmg
!         Only those elements that satisfy elemmap(i) > 0 correspond to the
!         local mesh

         if ( km(iseqkm)%kmesh(21)>0 ) deallocate(km(iseqkm)%kmeshg)
         allocate ( km(iseqkm)%kmeshg(nelgrp) )
         km(iseqkm)%kmesh(21) = nelgrp
         if ( debug ) write(irefwr,*) 'nelgrp ', nelgrp
         ifirst = 1                                 ! first element in group
         do i = 0, nelgrp-1
            ilast = ifirst+kmeshg(i+1)-1     ! last element in group
            numelms = 0                             ! local number of elements
            if ( debug ) write(irefwr,*) 'i, ifirst, ilast ', &
               i, ifirst, ilast
            do j = ifirst, ilast
               if ( elemmap(j)>0 ) numelms = numelms+1 ! active element
            end do
            km(iseqkm)%kmeshg(1+i) = numelms            ! Fill local kmeshg
            ifirst = ilast+1
         end do
         if ( debug ) call prinin ( km(iseqkm)%kmeshg, nelgrp, 'kmeshg' )

      end if

!     --- Fill array kmeshcloc from kmeshc and elemmap and pointmap

      ielem = 0
      ikelmc = 0
      lenkmeshc = 0
      inelem = nelem
      do ielgrp = 1, nelgrp        ! loop over element groups

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         do iielem = 1, inelem     ! loop over elements in group

            ielem = ielem+1
            if ( elemmap(ielem)>0 ) then

!           --- element belongs to local mesh, copy information

               do i = 1, inpelm     ! loop over nodes in element
                  kmeshcloc(lenkmeshc+i) = &
                     pointmap(kmeshc(ikelmc+i))
               end do  ! i = 1, inpelm
               lenkmeshc = lenkmeshc+inpelm

            end if  ! ( elemmap(ielem)>0 )
            ikelmc = ikelmc+inpelm

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

      deallocate ( elemmap, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'elemmap' )

1000  if ( debug ) write(irefwr,*) 'End mshparkmeshc'
      call erclos ( 'mshparkmeshc' )

      end
