      subroutine mshparm ( kmeshlocm, maparinv, lengthm )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  2.1  date 19-02-2003 Correction for empty curves
!        version  2.0  date 16-05-2001
!        version  1.0  date 21-07-2000
!
!   copyright (c) 2000-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make local array kmeshm from global array kmeshm using array maparinv
!     This is used for subdivision of region in subregions in case of
!     parallel computing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
!     curve
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlocm(*), maparinv(*), lengthm

!     kmeshlocm      o    Local KMESH part m
!     lengthm        i    Length of array KMESHM
!     maparinv       i    Contains mapping from global numbering to local
!                         numbering
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, istar3, ivalue, npointlc

!     i              Counting variable
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     ivalue         Help parameter to store an integer
!     j              Counting variable
!     npointlc       Local number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- copy kmeshm to kmeshlocm

      do i = 1, lengthm
         kmeshlocm(i) = kmeshm(i)
      end do

!     --- determine start position part 3 containing nodal point
!         numbers of the curves

      istar3 = kmeshm(2)

!     --- loop over global curves

      do j = 1, ncurvs

!     --- determine number of points on curve

         npointlc = kmeshm(5 + j) - kmeshm(4 + j)

         do i = 1, npointlc
            ivalue = kmeshm(istar3+i-1)
            if ( ivalue>0 ) then
               kmeshlocm(istar3+i-1) = maparinv(ivalue)
            else
               kmeshlocm(istar3+i-1) = ivalue
            end if
         end do

         istar3 = istar3 + npointlc
      end do
      end
