      subroutine mshparncorr1 ( kmeshn, inpelm, nodesurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear one node in element in kmesh in the case that surface
!     elements do not belong to the set of outer faces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshn(inpelm), nodesurf(*)

!     inpelm         i    number of points per element
!     kmeshn         i    KMESH part n, contains nodal point numbers of one
!                         element in a surface
!     nodesurf      i/o   Array of length npoint containing the number
!                         of times a node in a surface appears
!                         If a node is cleared the number is decreased
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, maxnum, maxindex, inode, numappear, nodestore

!     i              Counting variable
!     inode          Node sequence number
!     maxindex       Index of node in element with maximum number of
!                    appearances in surface
!     maxnum         Maximum row number for part of decomposition
!     nodestore      Node corresponding to maxindex
!     numappear      Number of appearances of node in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3073   Internal error, node appears only once in outer surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxnum = 0
      maxindex = 0
      do i = 1, inpelm

         inode = kmeshn(i)
         if ( inode>0 ) then

!        --- inode>0, find node with maximum number of appearances

            numappear = nodesurf(inode)
            if ( numappear>maxnum ) then

!           --- This node has more appearances

               maxnum = numappear
               maxindex = i
               nodestore = inode

            end if  ! ( numappear>maxnum )

         else

!        --- inode<=0, skip element

            go to 1000

         end if  ! ( inode>0 )

      end do  ! i = 1, inpelm

      if ( maxnum>1 ) then

!     --- maxnum>1, hence node in element may be cleared

         kmeshn(maxindex) = 0
         nodesurf(nodestore) = nodesurf(nodestore)-1

      else

!     --- maxnum = 1, error

         call errint ( nodestore, 1 )
         call errsub ( 3073, 1, 0, 0 )

      end if  ! ( maxnum>1 )

1000  call erclos ( 'mshparncorr1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparncorr1'

      end if  ! ( debug )

      end
