      subroutine mshparncorr3 ( list, istore, numfacesurf, inpelm, &
                                nmeshg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a sorted list of nodes in outer faces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numfacesurf, inpelm, nmeshg(*), &
              istore(inpelm,numfacesurf), list(numfacesurf)

!     inpelm         i    number of points per element
!     istore         o    Contains the part of the set of faces to be considered
!     list           i    List of element numbers in kmeshncopy with respect to
!                         kmeshn
!     nmeshg         i    nmesh part g, contains node numbers of faces
!     numfacesurf    i    Number of faces in istore to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iface, ihelp(inpelm), istart, iloc
      integer, allocatable :: ipointer(:), istore1(:,:), iseq(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     iface          Face sequence number
!     ihelp          integer work array to store node numbers of element
!     iloc           Sequence number of local node in element
!     ipointer       work array for sorting algorithm
!                    Contains sequence of sorted set
!     iseq           work array for sorting algorithm
!                    Contains sequence of sorted set
!     istart         First position used at a part
!     istore1        Help array to store sorted array istore
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPARNCORR4   Sort nodes in istore in increasing sequence
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( ipointer(numfacesurf), istore1(inpelm,numfacesurf), &
                 iseq(numfacesurf), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, numfacesurf*(inpelm+2), 'ipointer' )

!     --- Store nodes of faces in array istore
!         Sort in increasing sequence per face

      do i = 1, numfacesurf
         iface = list(i)
         istart = nmeshg(iface)
         ihelp(1:inpelm) = nmeshg(istart+1:istart+inpelm)
         call chsort ( ihelp, ipointer, inpelm )
         istore(1:inpelm,i) = ihelp(ipointer(1:inpelm))
      end do  ! i = 1, numfacesurf

      if ( debug ) &
         call prinin1 ( istore, numfacesurf, inpelm, 'istore' )

!     --- Sort nodes in istore in increasing sequence

      iloc = 1
      call mshparncorr4 ( istore, istore1, iloc, &
                          numfacesurf, inpelm, iseq, list )

      deallocate ( ipointer, istore1, iseq, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipointer' )

1000  call erclos ( 'mshparncorr3' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( istore, numfacesurf, inpelm, 'istore' )
         write(irefwr,*) 'End mshparncorr3'

      end if  ! ( debug )

      end
