      function mshparneighbl1 ( iwork, points, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if at least one of the nodes in array points is also a node of
!     of the block blocknr
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     neighbour
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), points(*), nnodes
      logical mshparneighbl1

!     iwork          i    integer work array of length npoint
!                         iwork(i) = 0, means that node i is not in blocknr
!                         iwork(i) > 0, means that node i is in blocknr
!     mshparneighbl1 o    Returns with value true if there are nodes in array
!                         points with iwork(i) > 0
!     nnodes         i    Number of nodes in array points
!     points         i    Array of size nnodes containing the node
!                         numbers corresponding to the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     mshparneighbl1 := false
!     i := 0
!     do while ( not mshparneighbl1 and i < nnodes )
!       i := i+1
!       if ( iwork(points(i)) > 0 ) then
!          mshparneighbl1 := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparneighbl1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparneighbl1'

      end if

!     --- mshparneighbl1 := false

      mshparneighbl1 = .false.

!     --- i := 0

      i = 0

!     --- do while ( not mshparneighbl1 and i < nnodes )

      do while ( .not. mshparneighbl1 .and. i.lt.nnodes )

!     --- i := i+1
!         if ( iwork(points(i)) > 0 ) then
!            mshparneighbl1 := true

         i = i+1
         if ( iwork(points(i)).gt.0 ) mshparneighbl1 = .true.

      end do  ! while ( .not. mshparneighbl1 .and. i.lt.nnodes )

      call erclos ( 'mshparneighbl1' )
      if ( debug ) write(irefwr,*) 'End mshparneighbl1'

      end
