      subroutine mshpav1close ( rgridinfo, nx, ny, grid, nowbnd, &
                                maxbounds, nloop, gridinfo, crd, layer, &
                                lengridinfo, gridfilled, lengridfilled, &
                                numgridfilled, allmarked, markboun, &
                                lnnowbnd, bndcoornew, maxlen, nel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-06-2004 Put thickness in crd(4,.)
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all nodes on the boundary of which the distance to the
!     opposite boundary is less than 4h, with h the average distance on
!     both boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, maxbounds, nloop, nowbnd(maxbounds,*), &
              grid(0:nx,0:ny), gridinfo(*), layer, lengridinfo, &
              gridfilled(2,*), lengridfilled, numgridfilled, lnnowbnd, &
              markboun(lnnowbnd,nloop,*), nel
      double precision crd(4,*), rgridinfo(4), &
                       bndcoornew(4,lnnowbnd,*), maxlen
      logical allmarked

!     allmarked      o    If true all nodes on the actual boundary are marked
!                         and hence there are no points left to perform the
!                         paver algorithm
!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     grid           i    Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if > 0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridfilled     i    Work array of size 2 x lengridfilled, indicating which
!                         cells in array grid have value larger than zero
!                         gridfilled(1,i) contains ix and
!                         gridfilled(2,i) contains iy for a cell with
!                         grid(ix,iy) > 0
!     gridinfo       i    In this array the boundary points corresponding to a
!                         specific grid cell in grid are stored
!                         istart = grid(ix,iy)
!                         In gridinfo(istart) the number of boundary points of
!                         cell (ix,iy) are stored (if > 0). This is followed
!                         by 2 positions for each boundary point
!                         The pair consists of the loop sequence number in the
!                         boundary where loop refers to a closed loop and
!                         a sequence number in the loop
!     layer          i    Present layer sequence number
!     lengridfilled  i    Contains the declared length of array gridfilled
!     lengridinfo    i    Contains the declared length of array gridinfo
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     markboun      i/o   Three dimensional array of size lnnowbnd x nloop x 3
!                         with the following contents
!                         markboun(i,j,k) refers to relative node i along the
!                         j-th boundary in nowbnd
!                         k has the following meaning.
!                         if Pos. 1 = 0 then the node is free otherwise (1)
!                         it is clcoe to another node.
!                         In that case position 2 contains the boundary number
!                         of that other node and position 3 the relative
!                         position  of the node along that boundary
!     maxbounds      i    Max number of loops in nowbnd
!     maxlen         i    Maximum amount of contiguous space in IBUFFR after a
!                         starting address
!     nel            i    number of elements in the topology
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numgridfilled  i    Contains the number of gridpoints filled in array
!                         gridfilled
!     nx             i    number of cells in x direction in array grid
!     ny             i    number of cells in y direction in array grid
!     rgridinfo      i    Array of size 4 containing the following information
!                         about array grid
!                         1:  xmin
!                         2:  ymin
!                         3:  dx
!                         4:  dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibndnr, ind, numnodes, jbndnr, jnd, inode
      logical found

!     found          If true an intersection has been found
!     ibndnr         i'th boundary in nowbnd
!     ind            Relativeequence number of node alon boundary
!     inode          Node sequence number
!     jbndnr         jth boundary in nowbnd
!     jnd            jth node in jbndnr
!     numnodes       Number of nodal points in boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVGRID     Cover region by rectangular grid and store in each cell of
!                    the grid which boundary nodes are present
!     MSHPAVMNDIST   Compute intersection of normal starting at a node on the
!                    boundary with edges of the boundary
!                    If intersection at distance less than 3h return with edge
!                    and loop number
!     PRINMARKBOUN   Print the contents of array markboun
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     allmarked := true
!     Cover present boundary with rectangular grid and store boundary node
!     numbers in grid arrays
!     For both boundaries do
!        For all nodes in the boundary do
!           find node at same bounday with distance in normal direction < 3h
!           if ( distance in normal direction < 3h ) then
!              Fill array markboun for node
!           else
!              allmarked := false
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav1close' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav1close'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- allmarked := true

      allmarked = .true.

!     --- Cover present boundary with rectangular grid and store boundary node
!         numbers in grid arrays

      call mshpavgrid ( rgridinfo, nx, ny, grid, nowbnd, maxbounds, &
                        nloop, gridinfo, crd, layer, lengridinfo, &
                        gridfilled, lengridfilled, numgridfilled )
      if ( ierror.ne.0 ) go to 1000

!     --- For both boundaries do

      do ibndnr = 1, nloop

!     --- For all nodes in the boundary do

         numnodes = nowbnd(ibndnr,1)
         if ( ibndnr.eq.1 ) then
            jbndnr = 2
         else
            jbndnr = 1
         end if

         do ind = 1, numnodes
            if ( markboun(ind,ibndnr,1).eq.0 .and. &
                 bndcoornew(3,ind,ibndnr).gt.0d0 ) then

!           --- Node on boundary not yet coupled to opposite boundary
!               find node at opposite bounday with shortest distance

                inode = abs(nowbnd(ibndnr,ind+1))    ! node in ind
                call mshpavmndist ( ibndnr, ind, nowbnd, maxbounds, crd, &
                                    nx, ny, grid, gridinfo, rgridinfo, &
                                    layer, maxlen, found, &
                                    bndcoornew(1,1,ibndnr), &
                                    jnd, jbndnr, nel )
                if ( ierror.ne.0 ) go to 1000
                if ( found ) then

!               --- intersection found, fill array markboun for nodes
!                   set thickness equal to 0

                   markboun(ind,ibndnr,1) = 1
                   markboun(ind,ibndnr,2) = jbndnr
                   markboun(ind,ibndnr,3) = jnd
                   if ( abs(crd(3,inode)).gt.0d0 ) then

!                  --- thickness > 0, put new thickness in crd(4,.)

                      crd(4,inode) = abs(crd(3,inode))*crd(4,inode)

                   end if  ! ( crd(3,inode).gt.0d0 )
                   crd(3,inode) = 0d0

                else

!               --- nodes at distance more than 4h
!                   allmarked := false

                   allmarked = .false.

                end if  ! ( found )

            end if  ! ( markbound(ind,ibndnr,1).eq.0 )

         end do  ! ind = 1, numnodes

      end do  ! ibndnr = 1, nloop

1000  call erclos ( 'mshpav1close' )

      if ( debug ) then

!     --- Debug information

         call  prinmarkboun ( nowbnd, maxbounds, markboun, nloop, &
                              lnnowbnd )
         write(irefwr,*) 'End mshpav1close'

      end if

      end
