      subroutine mshpav1trian ( inode1, inode2, inode3, nel, kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create 1 triangle from the three nodes inode1-3 which form the
!     closed boundary in that sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode1, inode2, inode3, nel, kmeshc(3,*)

!     inode1         i    First node in boundary
!     inode2         i    Second node in boundary
!     inode3         i    Third node in boundary
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in kmeshc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav1trian' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav1trian'

      end if
      if ( inode1.ne.inode2 .and. inode1.ne.inode3 .and. &
           inode2.ne.inode3 ) then

!     --- three nodes are different, create triangle

         nel = nel+1
         kmeshc(1,nel) = inode1
         kmeshc(2,nel) = inode2
         kmeshc(3,nel) = inode3

      end if

      call erclos ( 'mshpav1trian' )
      if ( debug ) write(irefwr,*) 'End mshpav1trian'

      end
