      subroutine mshpavmndist ( ibndnr, ind, nowbnd, maxbounds, crd, nx, &
                                ny, grid, gridinfo, rgridinfo, layer, &
                                maxlen, found, bndcoornew, jind, &
                                jbndnr, nel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of normal starting at a node on the boundary with
!     edges of the boundary
!     If intersection at distance less than 3h return with edge and loop
!     number
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     boundary
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibndnr, ind, maxbounds, nowbnd(maxbounds,*), layer, nel, &
              nx, ny, grid(0:nx,0:ny), gridinfo(*), jind, jbndnr, &
              i
      double precision crd(4,*), rgridinfo(*), maxlen, bndcoornew(4,*)
      logical found

!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     found          o    If true already a possibility has been found
!     grid           i    Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if > 0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridinfo       i    In this array the boundary points corresponding to a
!                         specific grid cell in grid are stored
!                         istart = grid(ix,iy)
!                         In gridinfo(istart) the number of boundary points of
!                         cell (ix,iy) are stored (if > 0). This is followed
!                         by 2 positions for each boundary point
!                         The pair consists of the loop sequence number in the
!                         boundary where loop refers to a closed loop and
!                         a sequence number in the loop
!     ibndnr         i    i'th boundary in nowbnd
!     ind            i    Sequence number of node in ibndnr
!     jbndnr         o    jth boundary in nowbnd, defines boundary that is
!                         at distance less than 3h of ind
!     jind           o    Sequence number of node in jbndnr that is
!                         at distance less than 3h of ind
!     layer          i    Present layer sequence number
!     maxbounds      i    Max number of loops in nowbnd
!     maxlen         i    Maximum length of edges in the surface
!     nel            i    number of elements in the topology
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     nx             i    number of cells in x direction
!     ny             i    number of cells in y direction
!     rgridinfo      i    Array of size 4 containing the following information
!                         about array grid
!                         1:  xmin
!                         2:  ymin
!                         3:  dx
!                         4:  dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, inodeprev, inodenext, idx, idy, ix, iy, numnodes
      double precision xmingrid, ymingrid, dx, dy, len, ds, n(2), h, &
                       nwcr(2),  nxcr(2), t1(2), &
                       t2(2), norm, n1(2), n2(2), norm1, norm2
      logical finished

!     ds             Lenght of edge (squared)
!     dx             x-distance of cells
!     dy             y-distance of cells
!     finished       If true a node close to ind has been found
!     h              width in node
!     idx            max search length in x-dir
!     idy            max search length in y-dir
!     inode          Node sequence number
!     inodenext      Next node along boundary
!     inodeprev      Previous node along boundary
!     ix             x index of cell containing inode
!     iy             y index of cell containing inode
!     len            length of edge
!     n              normal in node
!     n1             Normal on edge
!     n2             Normal on edge
!     norm           length of n
!     norm1          length of n1
!     norm2          length of n2
!     numnodes       Number of nodal points in array nodes
!     nwcr           coordinates of indnr
!     nxcr           coordinates of i+1th node in jbndnr
!     t1             First tangential vector
!     t2             Second tangential vector
!     xmingrid       xmin for array grid
!     ymingrid       ymin for array grid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVMNDIST3  Find edge in boundary that is close to given node
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters
!     inode := node number corresponding to ind and ibndnr
!     nwcr := coordinates of inode
!     h := width in inode
!     ix, iy := indices of node in array grid
!     len := min(4*maxlen, max(nx*dx,ny*dy)/2)
!     nxcr := coordinates of normal through nwcr at distance len
!     Compute idx and idy, i.e. max range for loops to check
!     For all cells around node within range do
!        Find loop jloop and index jind of nodes in cell
!        Compute intersection of corresponding edge with line nwcr, nxcr
!        If ( there is an intersection at distance less than 3h ) then
!           found := true
!           store jloop and jind
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmndist' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmndist'
         write(irefwr,1) 'layer, ibndnr, ind, nel', &
                          layer, ibndnr, ind, nel
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

!     --- Extract parameters
!         len := min(4*maxlen, max(nx*dx,ny*dy)/2)

      xmingrid = rgridinfo(1)    ! xmin for grid
      ymingrid = rgridinfo(2)    ! ymin for grid
      dx = rgridinfo(3)          ! x-distance of cells
      dy = rgridinfo(4)          ! y-distance of cells
      len = min ( 4d0*maxlen, 0.2d0*max(nx*dx,ny*dy) ) !maximum search length

!     --- inode := node number corresponding to ind and ibndnr
!         nwcr := coordinates of inode
!         h := width in inode
!         ix, iy := indices of node in array grid
!         nxcr := coordinates of normal through nwcr at distance len
!         Compute idx and idy, i.e. max range for loops to check

      inode = abs(nowbnd(ibndnr,ind+1))    ! node in ind
      nwcr(1) = crd(1,inode)               ! x_inode
      nwcr(2) = crd(2,inode)               ! y_inode
      h = abs(crd(3,inode))                ! width in inode
      ix = nint((nwcr(1)-xmingrid)/dx)     ! x index of cell containing inode
      iy = nint((nwcr(2)-ymingrid)/dy)     ! y index of cell containing inode
      idx = nint(len/dx)+1                 ! max search length in x-dir
      idy = nint(len/dy)+1                 ! max search length in y-dir
      finished = .false.
      if ( debug ) then
         write(irefwr,1) 'inode', inode
         write(irefwr,2) 'h, nwcr', nwcr
         write(irefwr,2) 'bndcoornew', bndcoornew(1:2,ind)
      end if

      numnodes = nowbnd(ibndnr,1)
      if ( ind.eq.1 ) then

!     --- first node on boundary

         inodeprev = abs(nowbnd(ibndnr,numnodes+1))

      else

!     --- not first node on boundary

         inodeprev = abs(nowbnd(ibndnr,ind))

      end if
      if ( ind.eq.numnodes ) then

!     --- last node on boundary

         inodenext = abs(nowbnd(ibndnr,2))

      else

!     --- not last node on boundary

         inodenext = abs(nowbnd(ibndnr,ind+2))

      end if

      n(1) = bndcoornew(1,ind)-nwcr(1)
      n(2) = bndcoornew(2,ind)-nwcr(2)
      ds = n(1)**2+n(2)**2
      if ( ds.le.0d0 ) then

!     --- Thickness in node is equal to 0
!         Store both normals in n1 and n2

         t1(1) = crd(1,inode)-crd(1,inodeprev)
         t1(2) = crd(2,inode)-crd(2,inodeprev)
         t2(1) = crd(1,inodenext)-crd(1,inode)
         t2(2) = crd(2,inodenext)-crd(2,inode)
         n1(1) = -t1(2)
         n1(2) =  t1(1)
         n2(1) = -t2(2)
         n2(2) =  t2(1)
         norm1 = 1d0/(sqrt(n1(1)**2+n1(2)**2))
         norm2 = 1d0/(sqrt(n2(1)**2+n2(2)**2))
         do i = 1, 2
            n1(i) = n1(i) * norm1
            t1(i) = t1(i) * norm1
            n2(i) = n2(i) * norm2
            t2(i) = t2(i) * norm2
         end do
         if ( debug ) write(irefwr,1) 'n1, n2', n1, n2

      end if  ! ( ds.le.0d0 )

      if ( ds.le.0d0 ) then

!     --- Thickness in node is equal to 0
!         In this case we have to compute the normal and set ds and the "new"
!         point
!         The normal is defined as the average norm in the ind-th node

         do i = 1, 2
            n(i) = n1(i) + n2(i)
         end do
         norm = 1d0/sqrt(n(1)**2+n(2)**2)
         n(1) = n(1)*norm
         n(2) = n(2)*norm
         ds = maxlen*2d0
         nxcr(1) = nwcr(1)+ds*n(1)
         nxcr(2) = nwcr(2)+ds*n(2)

      else  ! ( ds.gt.0d0 )

!     --- Standard case, the normal exists

         ds = 1d0/sqrt(ds)
         n(1) = n(1)*ds
         n(2) = n(2)*ds
         ds = len
         nxcr(1) = nwcr(1)+ds*n(1)
         nxcr(2) = nwcr(2)+ds*n(2)

      end if  ! ( ds.le.0d0 )

      if ( debug ) then
         write(irefwr,1) 'ind, ibndnr, inode',ind, ibndnr, inode
         write(irefwr,2) 'nwcr, nxcr', nwcr, nxcr
         write(irefwr,1) 'ix, iy', ix, iy
         write(irefwr,2) 'h, ds', h, ds
      end if

!     --- Find edge corresponding to node

      call mshpavmndist3 ( idx, idy, ix, iy, nx, ny, grid, &
                           gridinfo, finished, maxbounds, &
                           nowbnd, inode, layer, crd, nwcr, nxcr, &
                           h, len, ind, ibndnr, jbndnr, &
                           jind, n, maxlen )

      found = finished

      call erclos ( 'mshpavmndist' )

      if ( debug ) then

!     --- Debug information

         if ( finished ) then
            write(irefwr,*) 'found ',found
            write(irefwr,1) 'jind', jind
         end if  ! ( finished )
         write(irefwr,*) 'End mshpavmndist'

      end if

      end
