      subroutine mshpavmndist1 ( crd, jnode1, jnode2, nwcr, nxcr, h, ds, &
                                 found, factor, fact2, layer, &
                                 intersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of edge jnode1-jnode2 with line nwcr, nxcr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     boundary
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jnode1, jnode2, layer
      double precision  crd(4,*), nwcr(2), nxcr(2), h, ds, factor, fact2
      logical found, intersect

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ds             i    Maximum search length
!     fact2          o    if an intersection is found it is positioned
!                         at x_jnode1 + fact2 + (x_jnode2-x_jnode1)
!     factor         o    if an intersection is found it is positioned
!                         at nwcr + factor + (nwcr-nxcr)
!     found         i/o   If true an intersection has been found at distance
!                         less than 3h
!     h              i    width of layer in node
!     intersect      o    If true an intersection has been found at distance
!                         less than 3h
!     jnode1         i    first node of edge
!     jnode2         i    Last node in edge
!     layer          i    Present layer sequence number
!     nwcr           i    coordinates of point from which the normal starts
!     nxcr           i    coordinates of line from nwcr in the direction of
!                         the normal and with length ds, hence nxcr = nwcr+ds n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision hj, fact1, x1(2), x2(2), hi

!     fact1          multiplication factor, see mshcrossline1
!     hi             height of the mesh in node indnr
!     hj             height of the mesh in node jndnr
!     x1             coordinates of jnode1
!     x2             coordinates of jnode2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE1  Find the intersection of the lines x1-x2 and x3-x4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store coordinates of edge in x1, x2
!     Compute intersectio of the line nwcr-nxcr and x1-x2
!     if ( both lines intersect ) then
!        check if distance to nwcr is less than 4h
!        if so set intersect equal to true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmndist1' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmndist1'

      end if
      intersect = .false.

!     --- Store coordinates of edge in x1, x2

      x1(1) = crd(1,jnode1)     ! x-coordinate of jnode1
      x1(2) = crd(2,jnode1)     ! y-coordinate of jnode1
      x2(1) = crd(1,jnode2)     ! x-coordinate of jnode2
      x2(2) = crd(2,jnode2)     ! y-coordinate of jnode2

!     --- Compute intersectio of the line nwcr-nxcr and x1-x2

      call mshcrossline1 ( nwcr, nxcr, x1, x2, &
                           fact1, fact2, sqreps )

      if ( debug ) then
         write(irefwr,*)  'jnode1, jnode2 ', jnode1, jnode2
         write(irefwr,*)  'nwcr, nxcr ', nwcr, nxcr
         write(irefwr,*)  'x1, x2 ', x1, x2
         write(irefwr,*)  'fact1, fact2 ', fact1, fact2
      end if

      if ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps ) then

!     --- The lines  and x1-x2 intersect
!         Check if this intersection is at distance less than 4h

         hj = max ( abs(crd(3,jnode1)), abs(crd(3,jnode2)) )
         if ( hj.le.0d0 ) hj = h
         hi = h
         if ( hi.le.0d0 ) hi = hj
         if ( fact1.le.2d0*(hi+hj)/ds ) then

!        --- there is an intersection at distance less than
!            4h (actually 2*(hi+hj))
!            found := true
!            store jloop and jind
!            In the case that fact1 < 1.5h, we set intersect is true, &
!            since then the normal is so close to the edge that we
!            have to create a connection

            found = .true.
            factor = fact1
            if ( fact1.le.2d0*h/ds ) intersect = .true.
            if ( debug ) then

!           --- debug information

               write(irefwr,*) 'hi, hj, ds, fact1 ', hi, hj, ds, fact1
               write(irefwr,*) 'h/ds, 2d0*(hi+hj)/ds ', &
                                h/ds, 2d0*(hi+hj)/ds
               write(irefwr,*) 'intersect ', intersect

            end if

         end if  ! ( fact2.le.2d0*(h+hj)/ds )

      end if  ! ( fact2.ge.-sqreps .and. fact2.le.1d0+sqreps )

      call erclos ( 'mshpavmndist1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmndist1'

      end if

      end
