      subroutine mshpavmninter4 ( numnodes, bndcoornew, nodenrs, &
                                  bndcoorold, layer, ibndnr, nowbnd, &
                                  maxbounds, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Smooth final new boundary
!     Check if no intersections in the boundary are present
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodenrs(*), layer, ibndnr, maxbounds, &
              nowbnd(maxbounds,*), imark(*)
      double precision bndcoornew(4,*), bndcoorold(2,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     ibndnr         i    i'th boundary in nowbnd
!     imark         i/o   Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     layer          i    Present layer sequence number
!     maxbounds      i    Max number of loops in nowbnd
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    Number of nodal points in array nodenrs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iprev, icount

!     i              Counting variable
!     icount         Counter
!     iprev          Previous node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVBNDCHECK Check if old and new boundary do not intersect and also if
!                    lines from old to new boundary do not introduce
!                    intersections
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check if lines from old boundary to new boundary do not intersect
!     each other nor the old boundary
!     Check if new boundary does not intersect old boundary
!     if nodes on new boundary are close to each other then
!        Compute mean value of these coordinates
!        if ( replacing these two nodes by 1 does not cause intersections ) then
!           identify entries in nodenrs and adapt bndcoornew
!     Smooth new boundary provided no ill shaped elements occur
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter4' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter4'
         call prinin ( nodenrs, numnodes, 'nodenrs/start' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Check if there are more than one new nodes on boundary

      icount = 0
      iprev = 0
      do i = 1, numnodes

         if ( nodenrs(i).ne.iprev ) then

!        --- New node found

            icount = icount+1
            iprev = nodenrs(i)

         end if  ! ( nodenrs(i).ne.iprev )

      end do  ! i = 1, numnodes

      if ( icount.gt.1 ) then

!     --- Check if old boundary and new boundary do not intersect
!         each other

         call mshpavbndcheck ( numnodes, bndcoornew, nodenrs, &
                               bndcoorold, layer, ibndnr, nowbnd, &
                               maxbounds, imark )

      end if  ! ( icount.gt.1 )
      if ( ierror.ne.0 ) go to 1000

1000  call erclos ( 'mshpavmninter4' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs/end' )
         write(irefwr,*) 'End mshpavmninter4'

      end if

      end
