      subroutine mshpavmnsmoot2 ( crd, newbnd, numnodes, layer, imark, &
                                  kmeshc, nelprev, nel, deleted, &
                                  ibndnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Try to remove all edges on the new boundary that have length less than
!     0.5 of both its neighbours, provided no strange elements are created
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     smoothing
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer newbnd(*), numnodes, layer, imark(*), kmeshc(3,*), &
              nelprev, nel, ibndnr
      double precision crd(4,*)
      logical deleted

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     deleted       i/o   If true an element has been deleted
!     ibndnr         i    i'th boundary in nowbnd
!     imark          i    array of size ncrd indicating if a node is
!                         on new boundary or old boundary
!                         0: not on boundary or on outer boundary
!                         -1: on old boundary
!                         >0: on new boundary
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     nel            i    number of elements in the topology
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     newbnd         i    temp array containing the boundary nodes for one loop
!     numnodes       i    Number of nodal points in array newbnd
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, i, nelstart, inode1, inode2, ielem, j, iseq
      double precision x1, x2, y1, y2, edgelength(0:numnodes+1), ds, &
                       dsnext, dsprev
      logical remove

!     ds             Length of edge
!     dsnext         Length of next edge
!     dsprev         Length of previous edge
!     edgelength     Contains the length of the edges along the present
!                    boundary (squared)
!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     iseq           Sequence number
!     j              Counting variable
!     nelstart       Number of elements in kmeshc at the start of this
!                    subroutine
!     remove         if true element is removed
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     y1             y-coordinate of first point
!     y2             y-coordinate of last point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTREAL    Check control integer around a real work array
!     MSHPAVFINDEL   Find element in set of elements that contains both nodes
!                    inode1 and inode2
!     MSHPAVMNSMOOT0 Check if element may be removed without problem
!     MSHPAVMNSMOOT3 Remove double entries from array newbnd
!     MSHPAVMNSMOOT5 Check if edge is candidate for removing by checking the
!                    angle between edges at other sides
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store length of all edges of new boundary in array edgelength
!     Check if there are any edges with length less than both its neighbours
!     If so remove these edges if possible
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnsmoot2' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnsmoot2'
         write(irefwr,*) 'ibndnr, layer ', ibndnr, layer
         call prinin ( newbnd(2), newbnd(1), 'newbnd' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Set control integers

      edgelength(0) = 1001d0
      edgelength(numnodes+1) = 1001d0

      nelstart = nelprev+1

!     --- Store length of all edges of new boundary in array edgelength

      inode = newbnd(2)
      x1 = crd(1,inode)
      y1 = crd(2,inode)
      do i = 1, numnodes-1
         inode = newbnd(i+2)
         x2 = crd(1,inode)
         y2 = crd(2,inode)
         edgelength(i) = (x2-x1)**2+(y2-y1)**2
         x1 = x2
         y1 = y2
      end do
      inode = newbnd(2)
      x2 = crd(1,inode)
      y2 = crd(2,inode)
      edgelength(numnodes) = (x2-x1)**2+(y2-y1)**2
      if ( debug ) call prinrl ( edgelength(1), numnodes, 'edgelength' )

!     --- Check if there are any edges with length less than both its neighbours

      dsprev = edgelength(numnodes)
      ds = edgelength(1)
      do i = 2, numnodes
         dsnext = edgelength(i)
         remove = ds.gt.0d0 .and. ds.lt.0.25d0*dsprev .and. &
                  ds.lt.0.25d0*dsnext
         if ( remove ) then

!        --- Edge is short enough
!            However the edge may only be removed if the angle between
!            the edges at both sides is not too far from pi
!            Otherwise we may end up with a boundary that intersects the
!            opposite boundary

            call mshpavmnsmoot5 ( i, remove, newbnd, crd, layer )

         end if  ! ( remove )
         if ( remove ) then

!        --- Candidate for removing found
!            Find element corresponding to this edge

            inode1 = newbnd(i)
            inode2 = newbnd(i+1)
            ielem = 0
            if ( imark(inode1).gt.0 .and. imark(inode2).gt.0 ) then

!           --- Both nodes on new boundary

               call mshpavfindel ( inode1, inode2, kmeshc, nelstart, &
                                   ielem, nel )
               if ( debug ) &
                  write(irefwr,*) 'i, inode1, inode2 ', i, inode1,inode2

!              --- Find sequence number of third node

               do j = 1, 3
                  if ( kmeshc(j,ielem).ne.inode1 .and. &
                       kmeshc(j,ielem).ne.inode2 ) iseq = j
               end do

               if ( debug ) &
                  write(irefwr,*) 'iseq, ielem ', iseq, ielem
               call mshpavmnsmoot0 ( crd, layer, ibndnr, nel, &
                                     kmeshc, newbnd, nelprev, &
                                     deleted, imark, ielem, iseq )

            end if  ! ( imark(inode1).gt.0 .and. imark(inode2).gt.0 )

         end if  ! ( remove )
         dsprev = ds
         ds = dsnext

      end do  ! i = 2, numnodes

      dsnext = edgelength(1)
      remove = ds.gt.0d0 .and. ds.lt.0.25d0*dsprev .and. &
               ds.lt.0.25d0*dsnext
      if ( remove ) &
         call mshpavmnsmoot5 ( 1, remove, newbnd, crd, layer )

      if ( remove ) then

!     --- Find element corresponding to this edge

         inode1 = newbnd(numnodes+1)
         inode2 = newbnd(2)
         ielem = 0
         if ( imark(inode1).gt.0 .and. imark(inode2).gt.0 ) then

!        --- Nodes on new boundary

            call mshpavfindel ( inode1, inode2, kmeshc, nelstart, &
                                ielem, nel )
            if ( debug ) &
               write(irefwr,*) 'inode1, inode2 ', inode1, inode2

!           --- Find sequence number of third node

            do j = 1, 3
               if ( kmeshc(j,ielem).ne.inode1 .and. &
                    kmeshc(j,ielem).ne.inode2 ) iseq = j
            end do
            if ( debug ) write(irefwr,*) 'iseq, ielem ', iseq, ielem

            call mshpavmnsmoot0 ( crd, layer, ibndnr, nel, &
                                  kmeshc, newbnd, nelprev, &
                                  deleted, imark, ielem, iseq )

         end if  ! ( imark(inode1).gt.0 .and. imark(inode2).gt.0 )

      end if  ! ( ds.gt.0d0 .and. ds.lt.0.25d0*dsprev  ... )

!     --- Remove double entries in newbnd

      call mshpavmnsmoot3 ( newbnd )

!     --- Check control integers

      call errcontreal ( edgelength(0), edgelength(numnodes+1), &
                         1001d0, 'edgelength' )

1000  call erclos ( 'mshpavmnsmoot2' )

      if ( debug ) then

!     --- Debug information

         call prinin ( newbnd(2), newbnd(1), 'newbnd' )
         write(irefwr,*) 'End mshpavmnsmoot2'

      end if

      end
