      subroutine mshpavremnode ( numnodes, bndcoornew, nodenrs, &
                                 layer, bndcoorold, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove node from new boundary if allowed
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodenrs(*), layer, ind
      double precision bndcoornew(4,*), bndcoorold(2,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     ind            i    Sequence number of node in ibndnr
!     layer          i    Present layer sequence number
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indnext, indprev
      double precision h, ds, n(2), x(2)

!     ds             distance from old node ind to new line
!     h              Width of layer in node
!     indnext        Node following ind
!     indprev        Node before ind
!     n              normal
!     x              co-ordinates of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ind = 1 ) then
!        indprev := numnodes, indnext := 2
!     else if ( ind = numnodes ) then
!        indprev := numnodes-1, indnext := 1
!     else
!        indprev := ind-1, indnext := ind+1
!     h := average width at node ind
!     ds := distance from old node ind to new line indprev -- indnext
!     if ( h/2 < ds < 2h ) then
!        remove ind: imark(ind) = 0, nodenrs(ind) := nodenrs(indprev)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavremnode' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavremnode'
         write(irefwr,*) 'layer ', layer

      end if
      if ( ierror.ne.0 ) go to 1000
      if ( ind.eq.1 ) then

!     --- ind = 1
!         indprev := numnodes, indnext := 2

         indprev = numnodes
         indnext = 2

      else if ( ind.eq.numnodes ) then

!     --- ind = numnodes
!         indprev := numnodes-1, indnext := 1

         indprev = numnodes-1
         indnext = 1

      else

!     --- standard case
!         indprev := ind-1, indnext := ind+1

         indprev = ind-1
         indnext = ind+1

      end if  ! ( ind.eq.1 )

!     --- h := average width at node ind

      h = sqrt((bndcoornew(1,indprev)-bndcoorold(1,indprev))**2+ &
               (bndcoornew(2,indprev)-bndcoorold(2,indprev))**2)
      ds = sqrt((bndcoornew(1,indnext)-bndcoorold(1,indnext))**2+ &
                (bndcoornew(2,indnext)-bndcoorold(2,indnext))**2)
      h = 0.5d0*(h+ds)

!     --- ds := distance from old node ind to new line indprev -- indnext

      n(1) = -(bndcoornew(2,indnext)-bndcoornew(2,indprev))  ! -t_2
      n(2) = bndcoornew(1,indnext)-bndcoornew(1,indprev)     !  t_1
      ds = 1d0/sqrt(n(1)**2+n(2)**2)
      n(1) = n(1)*ds    ! normal from new boundary inside the region
      n(2) = n(2)*ds
      x(1) = bndcoornew(1,indnext)-bndcoorold(1,ind)
      x(2) = bndcoornew(2,indnext)-bndcoorold(2,ind)
      ds = n(1)*x(1)+n(2)*x(2)
      if ( debug ) write(irefwr,*) 'ind, ds, h ', ind, ds, h

      if ( ds.ge.0.5d0*h .and. ds.le.2d0*h ) then

!     --- h/2 < ds < 2h
!         remove ind: imark(ind) = 0, nodenrs(ind) := nodenrs(indprev)

         nodenrs(ind) = nodenrs(indprev)
         bndcoornew(1,ind) = bndcoornew(1,indprev)
         bndcoornew(2,ind) = bndcoornew(2,indprev)
         if ( debug ) write(irefwr,*) 'remove ', ind

      end if  ! ( ds.ge.0.5d0*h .and. ds.le.2d0*h )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavremnode'

      end if
      call erclos ( 'mshpavremnode' )

      end
