      subroutine mshrdparmesh1 ( irefc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.3  date 01-05-2015 Open files locally
!        version  1.2  date 09-05-2012 New call to mshrdparmsh3
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of global mesh from meshoutput
!     Store information in array kmeshpar
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc

!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nboun, ihelp(10), ivers, elgrp(2,nelgrp), lastps, &
              lenkmeshc, i, iref
      character (len=100) :: name10sav

!     elgrp          Array of size 2 x nelgrp
!                    (1,i) contains number of nodes in element group i
!                    (2,i) contains number of elements of element group i
!     i              Counting variable
!     ihelp          Contains the input written by MSH059 on the second record
!                    after nvolms
!     iref           File reference number
!     ivers          Version sequence number
!     lastps         Last non-blank position in namefl
!     lenkmeshc      Length of global array kmeshc
!     name10sav      Help variable to store value of name10
!     nboun          Number of boundary points at an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIL         Open files
!     MSHRDKMESHLMN  Read kmesh parts l m and n from the file meshoutput
!     MSHRDMESHCOOR  Read coordinates from file meshoutput
!     MSHRDMESHDIM   Read first records from file meshoutput
!     MSHRDPARMSH2   Read information about element groups
!     MSHRDPARMSH3   Read and skip topology array
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdparmesh1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdparmesh1'
         write(irefwr,1) 'irefc, nelgrp', irefc, nelgrp
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Open file meshoutput_xxx

      name10sav = name10       ! save name10
      lastps = index ( name10, ' ' )-1
      name10(lastps-7:lastps) = '        '
      call inifil(10)
      if ( debug ) write(irefwr,*) 'name10, iref10 ', name10, iref10

!     --- First read dimension parameters

      call mshrdmeshdim ( irefc, nboun, ihelp, ivers )

      if ( debug) write(irefwr,1) 'npoint', npoint

      kmeshpar(1) = npoint  ! global number of nodes

      if ( nelgrp>1 ) then

!     --- Read information concerning element groups
!         (number of elements, number of nodes per element)

         call mshrdparmsh2 ( nelgrp, elgrp, irefc )

      else

!     --- Fill elgrp

         elgrp(1,1) = npelm
         elgrp(2,1) = nelem

      end if  ! ( nelgrp>1 )

!     --- Compute length of global array kmeshc

      lenkmeshc = 0
      do i = 1, nelgrp
         lenkmeshc = lenkmeshc+elgrp(1,i)*elgrp(2,i)
      end do  ! i = 1, nelgrp

      if ( debug) call prinin1 ( elgrp, nelgrp, 2, 'elgrp' )

!     --- Reserve space for the global coordinates array
!         mm address in kmeshpar(2)

      if ( associated(km(iseqkm)%coorglob) ) &
         deallocate(km(iseqkm)%coorglob)
      allocate(km(iseqkm)%coorglob(ndim,npoint))
      coorglob => km(iseqkm)%coorglob
      kmeshpar(2) = ndim*npoint
      if ( debug ) &
         write(irefwr,1) 'ndim, npoint, kmeshpar(2)', &
                          ndim, npoint, kmeshpar(2)

!     --- Read coordinates

      call mshrdmeshcoor ( irefc, ndim, npoint, coorglob, ivers )

!     --- Read the topology array

      length = lenkmeshc+nelgrp
      if ( associated(km(iseqkm)%kmeshcglob) ) &
         deallocate(km(iseqkm)%kmeshcglob)
      allocate(km(iseqkm)%kmeshcglob(length))
      kmeshcglob => km(iseqkm)%kmeshcglob
      kmeshpar(7) = length
      kmeshcglob(1:nelgrp) = elgrp(2,1:nelgrp)
      call mshrdparmsh3 ( irefc, elgrp, nelgrp, &
                          kmeshcglob(1+nelgrp) )

!     --- Read global arrays kmeshl/m and n

      call mshrdkmeshlmn ( irefc, ivers, ihelp )

!     --- Close file meshoutput

      iref = abs(iref10)
      close ( unit = iref )
      name10 = name10sav   ! reset name10

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshpar, 6, 'kmeshpar' )
         write(irefwr,*) 'End mshrdparmesh1'

      end if  ! ( debug )
      call erclos ( 'mshrdparmesh1' )

      end
