      subroutine mshrearef ( refine, renum, iinput, irefin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-11-2014 New default for irenum
!        version  1.0  date 28-03-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read and interprete part of input corresponding to refine
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefin
      logical refine, renum

!     iinput        i/o   integer input array
!     irefin         i    Number of times that the mesh should be refined
!     refine        i/o   If true refine has been found
!     renum         i/o   If true renumber has been found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1240   The number of times the mesh should be refined is out of range
!    1241   Incorrect information found after REFINE.
!    1552   keyword found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrearef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrearef'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( refine ) then

!     --- refine has been found more than once

         call errchr ( string(1:nchar), 1 )
         call errwar ( 1552, 0, 0, 1 )

      end if
      refine = .true.

      call asknxt
      if ( ityprd==2 ) then

!     --- integer found after refine. Check this integer

         irefin = nint(realrd)
         if ( irefin<1 .or. irefin>1000 ) then

!        --- irefin out of range

            call errint ( irefin, 1 )
            call errsub ( 1240, 1, 0, 0 )
            call errcrd

         end if

      else if ( ityprd==1 ) then

!     --- End of record found, i.e. no number

         irefin = 1

      else

!     --- Wrong character found after refine

         call errsub ( 1241, 0, 0, 0 )
         call errcrd

      end if
      if ( .not. renum ) iinput(10) = 9

1000  call erclos ( 'mshrearef' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrearef'

      end if  ! ( debug )

      end
