      subroutine mshrefcoor ( nodenumbers, eldimmax, fillmap, nmapout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!        version  1.1  date 27-11-2011 use function getshapeelgrp
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new coodinates in a refine step of the complete mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodenumbers(*), eldimmax, nmapout
      logical :: fillmap

!     eldimmax       i    Maximum dimension of the elements, i.e.
!                         3 for volume elements
!                         2 for surface elements
!                         1 for line elements
!     fillmap        o    If true array imap must be filled
!     nmapout        o    Size of last dimension of rmap
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, ipstart, numnodes, i, inode, inode1, inode2, &
              ielem, ielgrp, iielem, inpelm

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFCOORFC   Fill the coordinates for the "face" points in for all faces
!     MSHREFCOORSF   Fill the coordinates for the "face" points in for all
!                    2d elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( eldimmax = 1 ) then
!        For all element groups do
!           ishape := shape number of element
!           for all elements in group do
!              ielem := element sequence number
!              inpelm := number of nodes in element
!              for i := 1 (1) inpelm-1 do
!                 inode := nodenumbers(iedge)+i-1
!                 inode1 := i-th node in element
!                 inode2 := (i+1)-th node in element
!                 coor(inode) := (coor(inode1)+coor(inode2))/2
!     else ( eldimmax>1 )
!        for iedge := 1 (1) number_of_edges do
!           numnodes := number of nodes along edge iedge
!           for i := 1 (1) numnodes-1 do
!              inode := nodenumbers(iedge)+i-1
!              inode1 := i-th node along edge; inode2 := i+1-th node along edge
!              coor(inode) := (coor(inode1)+coor(inode2))/2
!        if ( eldimmax = 2 ) then
!           Fill nodes corresponding to elements (face nodes)
!        else ( eldimmax = 3 )
!           Fill nodes corresponding to faces
!           Fill nodes corresponding to elements (volume nodes)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoor'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      inpelm = npelm
      inelem = nelem
      ielem = 0

      if ( fillmap ) then

!     --- Fill first part of imap and rmap corresponding to old nodes

        imap(npoint+1:,1) = 0  ! clear first part of imap
        imap(1:npoint,1) = 1       ! nodes in old mesh are not interpolated
        rmap(1:npoint,1) = 1d0     ! standard weight
        do i = 1, npoint
           imap(i,2) = i           ! old node numbers
        end do  ! i = 1, npoint
        nmapout = 2

      end if  ! ( fillmap )

      if ( eldimmax==1 ) then

!     --- eldimmax = 1, hence line elements only
!         For all element groups do

         ipstart = 0
         do ielgrp = 1, nelgrp

!        --- ishape := shape number of element

            ishape = kmeshf(ielgrp)
            if ( kelmg>0 ) inelem = kmeshg(ielgrp)
            if ( kelma>0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
            if ( debug ) &
               write(irefwr,1) 'ielgrp, inelem, inpelm', &
                                ielgrp, inelem, inpelm
            if ( ishape>=1 .and. ishape<=2 ) then

!           --- ishape = 1 or 2 linear or quadratic line element
!               for all elements in group do

               do iielem = 1, inelem

!              --- ielem := element sequence number
!                  nodenumbers(ielem) := first_node

                  ielem = ielem+1
                  do i = 1, inpelm-1

!                 --- inode := nodenumbers(iedge)+i-1
!                     inode1 := i-th node in element
!                     inode2 := (i+1)-th node in element
!                     coor(inode) := (coor(inode1)+coor(inode2))/2

                     inode = nodenumbers(ielem)+i-1
                     inode1 = kmeshc(ipstart+i)
                     inode2 = kmeshc(ipstart+i+1)
                     coor(1:ndim,inode) = 0.5d0* &
                        (coor(1:ndim,inode1)+coor(1:ndim,inode2))

                     if ( fillmap ) then
                        imap(inode,1) = 2
                        imap(inode,2) = inode1
                        imap(inode,3) = inode2
                        rmap(inode,1:2) = 0.5d0
                     end if  ! ( fillmap )

                  end do  ! i = 1, ishape
                  ipstart = ipstart+inpelm

               end do  ! iielem = 1, inelem

            else if ( ishape/=-10001 .and. ishape/=0 ) then

!           --- ishape has other value: not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            else

!           --- ishape must be skipped

               ipstart = ipstart+inpelm*inelem

            end if  ! ( ishape==1 ) then

         end do  ! ielgrp = 1, nelgrp

      else  ! ( eldimmax>1 )

!     --- eldimmax>1, i.e. surface and/or volume elements
!         for iedge := 1 (1) number_of_edges do

         do iedge = 1, nedges

!        --- numnodes := number of nodes along edge iedge

            ipstart = nmeshc(iedge)
            numnodes = nmeshc(ipstart)

!           --- for i := 1 (1) numnodes-1 do

            do i = 1, numnodes-1

!           --- inode := nodenumbers(iedge)+i-1
!               inode1 := i-th node along edge;
!               inode2 := i+1-th node along edge
!               coor(inode) := (coor(inode1)+coor(inode2))/2

               inode = nodenumbers(iedge)+i-1
               inode1 = nmeshc(ipstart+i)
               inode2 = nmeshc(ipstart+i+1)
               coor(1:ndim,inode) = 0.5d0*(coor(1:ndim,inode1)+ &
                                           coor(1:ndim,inode2))

               if ( fillmap ) then
                  imap(inode,1) = 2
                  imap(inode,2) = inode1
                  imap(inode,3) = inode2
                  rmap(inode,1:2) = 0.5d0
               end if  ! ( fillmap )

            end do  ! i = 1, numnodes-1

         end do  ! iedge = 1, nedges
         ielem = nedges

         if ( eldimmax==2 ) then

!        --- eldimmax = 2, surface elements
!            Fill nodes corresponding to elements (face nodes)

            call mshrefcoorsf ( ielem, nodenumbers, fillmap, nmapout )
            if ( ierror/=0 ) go to 1000

         else  ! ( eldimmax = 3 )

!        --- eldimmax = 3
!            Fill nodes corresponding to faces

            call mshrefcoorfc ( ielem, nodenumbers, fillmap, nmapout )
            if ( ierror/=0 ) go to 1000

         end if  ! ( eldimmax==2 )

      end if  ! ( eldimmax==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoor'

      end if
      call erclos ( 'mshrefcoor' )

      end
