      subroutine mshrefcoor6 ( kmeshclc, first_node, fillmap  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in case of bi-quadratic
!     quadrilaterals of shape 6
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(9), first_node
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     first_node    i/o   Node number of next node to be created
!     kmeshclc       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, inode1, inode2, inode3, inode4

!     i              Counting variable
!     inode          Node sequence number
!     inode1         Node sequence number
!     inode2         Node sequence number
!     inode3         Node sequence number
!     inode4         Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The bi-quadratic quadrilateral has 9 nodes according to
!
!            7      6      5
!            x------x------x
!            |             |
!            |             |
!            |      9      |
!         8  x      x      x  4
!            |             |
!            |             |
!            |             |
!            x------x------x
!           1       2      3
!
!
!     The first four face nodes are defined as follows
!     f_1 between 2 and 9
!     f_2 between 4 and 9
!     f_3 between 6 and 9
!     f_4 between 8 and 9
!     The other 4 in the centers of the 4 quadrants:
!     f_5 in  1 - 2 - 9 - 8
!     f_6 in  2 - 3 - 9 - 2
!     f_7 in  4 - 5 - 6 - 9
!     f_8 in  6 - 7 - 8 - 9
!
!     inode2 := 9-th node in element
!     for i := 1 (1) 4 do
!        inode := nodenumbers(ielem)+i-1
!        inode1 := 2*i-th node in element
!        coor(inode) := (coor(inode1)+coor(inode2))/2
!     Fill other 4 nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoor6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoor6'
         call prinin ( kmeshclc, 9, 'kmeshclc' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- for i := 1 (1) 4 do

      inode2 = kmeshclc(9)
      do i = 1, 4

!     --- inode := nodenumbers(iedge)+i-1
!         inode1 := 2*i-th node in element
!         coor(inode) := (coor(inode1)+coor(inode2))/2

         inode = first_node+i-1
         inode1 = kmeshclc(i*2)
         coor(1:ndim,inode) = 0.5d0*(coor(1:ndim,inode1)+ &
                                     coor(1:ndim,inode2))

        if ( fillmap ) then
           imap(inode,1) = 2
           imap(inode,2) = inode1
           imap(inode,3) = inode2
           rmap(inode,1:4) = 0.5d0
        end if  ! ( fillmap )

      end do  ! i = 1, ishape

!     --- Fill other 4 nodes

      inode = inode+1
      inode1 = kmeshclc(1)
      inode2 = kmeshclc(2)
      inode3 = kmeshclc(9)
      inode4 = kmeshclc(8)
      coor(1:ndim,inode) = 0.25d0*(coor(1:ndim,inode1)+ &
         coor(1:ndim,inode2)+coor(1:ndim,inode3)+coor(1:ndim,inode4))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         imap(inode,5) = inode4
         rmap(inode,1:4) = 0.25d0
      end if  ! ( fillmap )

      inode = inode+1
      inode1 = kmeshclc(2)
      inode2 = kmeshclc(3)
      inode3 = kmeshclc(4)
      inode4 = kmeshclc(9)
      coor(1:ndim,inode) = 0.25d0*(coor(1:ndim,inode1)+ &
         coor(1:ndim,inode2)+coor(1:ndim,inode3)+coor(1:ndim,inode4))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         imap(inode,5) = inode4
         rmap(inode,1:4) = 0.25d0
      end if  ! ( fillmap )

      inode = inode+1
      inode1 = kmeshclc(4)
      inode2 = kmeshclc(5)
      inode3 = kmeshclc(6)
      inode4 = kmeshclc(9)
      coor(1:ndim,inode) = 0.25d0*(coor(1:ndim,inode1)+ &
         coor(1:ndim,inode2)+coor(1:ndim,inode3)+coor(1:ndim,inode4))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         imap(inode,5) = inode4
         rmap(inode,1:4) = 0.25d0
      end if  ! ( fillmap )

      inode = inode+1
      inode1 = kmeshclc(6)
      inode2 = kmeshclc(7)
      inode3 = kmeshclc(8)
      inode4 = kmeshclc(9)
      coor(1:ndim,inode) = 0.25d0*(coor(1:ndim,inode1)+ &
         coor(1:ndim,inode2)+coor(1:ndim,inode3)+coor(1:ndim,inode4))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         imap(inode,5) = inode4
         rmap(inode,1:4) = 0.25d0
      end if  ! ( fillmap )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1(coor(1,first_node),8,3,'coor_in_face')
         write(irefwr,*) 'End mshrefcoor6'

      end if
      call erclos ( 'mshrefcoor6' )

      end
