      subroutine mshrefcoor7 ( kmeshclc, first_node, fillmap  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in case of extended quadratic
!     triangle of shape 7
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(7), first_node, nmap
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     first_node    i/o   Node number of next node to be created
!     kmeshclc       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ind(3), inode, inode1, inode2, inode3
      double precision onethird

!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3, starting from 3
!     inode          Node sequence number
!     inode1         Node sequence number
!     inode2         Node sequence number
!     inode3         Node sequence number
!     onethird       1/3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The extended quadratic triangle has 7 nodes according to
!
!                  5
!                  x
!                 / \
!                /   \
!             6 x     x 4
!              /   x   \
!             /     7   \
!            x-----x-----x
!           1      2     3
!
!     The six face nodes are defined as follows
!     f_1 between 6 and 2
!     f_2 between 2 and 4
!     f_3 between 4 and 6
!     The other 3 in the centers of the 3 triangles:
!     f_4 in  1 - 2 - 6
!     f_5 in  2 - 3 - 4
!     f_6 in  4 - 5 - 6
!
!     for i := 1 (1) 3 do
!        inode := nodenumbers(ielem)+i-1
!        inode1 := 2*(i+2)-th node in element
!        inode2 := (2*i)-th node in element
!        coor(inode) := (coor(inode1)+coor(inode2))/2
!     Fill other 3 nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshrefcoor7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoor7'

      end if
      if ( ierror/=0 ) go to 1000
      onethird = 1d0/3d0

!     --- for i := 1 (1) 3 do

      do i = 1, 3

!     --- inode := nodenumbers(ielem)+i-1
!         inode1 := 2*(i+2)-th node in element
!         inode2 := (2*i)-th node in element
!         coor(inode) := (coor(inode1)+coor(inode2))/2

         inode = first_node+i-1
         inode1 = kmeshclc(ind(i)*2)
         inode2 = kmeshclc(i*2)
         coor(1:ndim,inode) = 0.5d0* &
            (coor(1:ndim,inode1)+coor(1:ndim,inode2))

         if ( fillmap ) then
            imap(inode,1) = 2
            imap(inode,2) = inode1
            imap(inode,3) = inode2
            rmap(inode,1:2) = 0.5d0
         end if  ! ( fillmap )

      end do  ! i = 1, ishape

!     --- Fill other 3 nodes

      inode = inode+1
      inode1 = kmeshclc(1)
      inode2 = kmeshclc(2)
      inode3 = kmeshclc(6)
      coor(1:ndim,inode) = onethird*(coor(1:ndim,inode1)+ &
                      coor(1:ndim,inode2)+coor(1:ndim,inode3))

      if ( fillmap ) then
         imap(inode,1) = 3
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         rmap(inode,1:3) = onethird
      end if  ! ( fillmap )

      inode = inode+1
      inode1 = kmeshclc(2)
      inode2 = kmeshclc(3)
      inode3 = kmeshclc(4)
      coor(1:ndim,inode) = onethird*(coor(1:ndim,inode1)+ &
                      coor(1:ndim,inode2)+coor(1:ndim,inode3))

      if ( fillmap ) then
         imap(inode,1) = 3
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         rmap(inode,1:3) = onethird
      end if  ! ( fillmap )

      inode = inode+1
      inode1 = kmeshclc(4)
      inode2 = kmeshclc(5)
      inode3 = kmeshclc(6)
      coor(1:ndim,inode) = onethird*(coor(1:ndim,inode1)+ &
                      coor(1:ndim,inode2)+coor(1:ndim,inode3))

      if ( fillmap ) then
         imap(inode,1) = 3
         imap(inode,2) = inode1
         imap(inode,3) = inode2
         imap(inode,4) = inode3
         rmap(inode,1:3) = onethird
      end if  ! ( fillmap )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoor7'

      end if
      call erclos ( 'mshrefcoor7' )

      end
