      subroutine mshrefkmeshcl9 ( kmeshc, kmeshcnew, inpelm, nelprev, &
                                  nmeshd, nodenumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape -9, i.e. large line element
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(*), kmeshcnew(*), nodenumbers(*), &
              nelprev, nmeshd(*)

!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements (one element)
!     kmeshcnew      o    New kmesh part c (one element)
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iedge, ipstart

!     i              Counting variable
!     iedge          edge sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for i := 1 (1) inpelm-1 do
!        kmeshcnew(2*i-1) := kmeshc(i)
!        iedge := i^th edge number of element
!        kmeshcnew(2*i) := nodenumbers(iedge)
!     kmeshcnew(2*inpelm-1) := kmeshc(inpelm)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshcl9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshcl9'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- for i := 1 (1) inpelm-1 do

      ipstart = nmeshd(nelprev+1)
      do i = 1, inpelm-1

!     --- kmeshcnew(2*i-1) := kmeshc(i)
!         iedge := i^th edge number of element
!         kmeshcnew(2*i) := nodenumbers(iedge)

         kmeshcnew(2*i-1) = kmeshc(i)
         iedge = abs(nmeshd(ipstart+i))
         kmeshcnew(2*i) = nodenumbers(iedge)

      end do  ! i = 1, inpelm-1

!     --- kmeshcnew(2*inpelm-1) := kmeshc(inpelm)

      kmeshcnew(2*inpelm-1) = kmeshc(inpelm)

1000  call erclos ( 'mshrefkmeshcl9' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshcl9'

      end if

      end
