      subroutine mshspckmeshm ( kmeshmnew, iwork, istart, ibrpnt, npnold, &
                                input, iinput, rinput, coorh, &
                                factor, tauarr, curves, &
                                lenkmeshmnew, lenibrpnt, lcoor, lncurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-11-2009 Remove some parameters
!        version  1.0  date 15-10-2002
!
!   copyright (c) 2002-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nodes on kmesh part m (curves) in case of a spectral mesh
!     Fill coordinates on curves in case of a spectral mesh
!     The coordinates of the original curves have already been filled
!     The curvature of the mesh is taken into account
!     Remark: if this subroutine is changed also update mshtrakmeshm and
!             mshspckmeshm !!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     curve
!     coordinate
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(ncurvs), istart(npoint+1), npnold, input(*), &
              iinput(*), lenkmeshmnew, kmeshmnew(lenkmeshmnew), lenibrpnt, &
              ibrpnt(lenibrpnt), lcoor, lncurves
      double precision rinput(*), coorh(lcoor), factor(26), tauarr(1000), &
                       curves(lncurves)

!     coorh         i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curves              help array to store the coordinates on the curve
!     factor         i    array with the distribution of points for one element
!     ibrpnt         i    neighbour array
!     iinput         i    integer input array
!     input          i    ?
!     istart         i    start addresses neighbour array
!     iwork          i    integer work array of length ncurvs
!                         if iwork(i)=0 curve i has been filled
!                         if iwork(i)=1 curve i has not been filled
!     kmeshmnew      o    New array kmeshm
!     lcoor          i    Length of coorh
!     lenibrpnt      i    Length of ibrpnt
!     lenkmeshmnew   i    Length of kmeshmnew
!     lncurves       i    Length of curves
!     npnold         i    old number of nodes in mesh
!     rinput         i    array with the user points
!     tauarr              help array to store the distribution of points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ist, ish, nsubcv, iposcv, ilold, &
              j, icurvn, jcurnr, ipos3, kkrom, i

!     i              Counting variable
!     icurvn         Defines type of curve
!     ilold          old length curve in array kmeshm
!     ipos3          position indicator in curves array
!     iposcv         Position in array work with respect to icurvs
!     ish            help loop variable
!     ist            help start position
!     j              Counting variable
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     kkrom          length of new curves array
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH046         Raise iinp as function of icurve
!     MSH482         Fill new curve array and compute (if necessary) the
!                    coordinates of new internal points on the lines
!     MSHFILLMSUB    Fill kmesh part m for composite curves
!     MSHSPCBOUN     Fill coordinates on curves in case of a spectral mesh
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     940   Not enough positions declared for help array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspckmeshm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspckmeshm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ninner = kmeshm(1)
      kmeshmnew(1) = kmeshm(1)
      kmeshmnew(2) = kmeshm(2)
      kmeshmnew(4) = kmeshm(4)
      kmeshmnew(5) = 1
      niedge = input(1)
      ist = kmeshm(2)
      ish = kmeshmnew(2)
      nsubcv = 0
      if ( debug ) call pr0008 ( kmeshm, ncurvs )

!     --- Check whether coordinates of points on curves have to be
!         adjusted
!         First kmeshm part 2 must be filled

      iposcv = 16
      do icurnr = 1, ncurvs
         if ( kelmq/=0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         if ( nsubcv==0 ) then

!        --- No subcurves, hence single curve
!            Compute number of nodes along curves

            ilold = kmeshm(icurnr+5) - kmeshm(icurnr+4)
            if ( iwork(icurnr)==0 ) &
               ilold = (1 + niedge) * (ilold - 1) + 1
            kmeshmnew(icurnr+5) = ilold
         end if
      end do
      if ( kelmq/=0 ) then

!     --- kelmq/=0, fill also the composite curves

         do icurnr = 1, ncurvs
            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
            if ( nsubcv>0 ) then

!           --- nsubcv>0, there are subcurves
!               Fill by adding the subcurves

               ilold = 0
               do j = 1, nsubcv
                  jcurnr = abs(kmeshq(kmeshq(icurnr)+j))
                  ilold = ilold+kmeshmnew(jcurnr+5)
               end do
               kmeshmnew(icurnr+5) = ilold

            end if ! nsubcv>0

         end do  ! icurnr = 1, ncurvs

      end if  ! kelmq/=0
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmnew(6), ncurvs, &
          'Number of nodes in curves after filling intermediate points')

      end if

!     --- Acumulate kmeshm part 2

      kmeshmnew(5) = 1
      do icurnr = 1, ncurvs
         kmeshmnew(5+icurnr) = &
            kmeshmnew(5+icurnr)+kmeshmnew(4+icurnr)
      end do
      if ( debug ) call prinin ( kmeshmnew(5), ncurvs+1, &
         'kmeshm part 2' )

      do icurnr = 1, ncurvs

         icurvn = iinput(iposcv)
         if ( kelmq/=0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)

         if ( nsubcv==0 ) then

!        --- Single curve

            ilold = kmeshm(icurnr+5) - kmeshm(icurnr+4)
            ist = kmeshm(2)+kmeshm(icurnr+4)-1
            ish = kmeshm(2)+kmeshmnew(icurnr+4)-1
            if ( debug ) write(irefwr,*) 'icurnr, ilold, ist, ish ', &
               icurnr, ilold, ist, ish
            if ( iwork(icurnr)==1 ) then

!           --- icurnr has not been filled, special situation

               do i = 0, ilold-1
                  kmeshmnew(ish+i) = kmeshm(ist+i)
               end do

            else

!           --- iwork(icurnr) = 0 curve has been filled

               call msh482 ( kmeshm, ist, ilold, kmeshmnew, ish, &
                             istart, ibrpnt, npnold, niedge )

            end if

         end if   ! single curve

!        --- Skip positions for this curve

         call msh046( iinput, iposcv, icurvn )

      end do  ! icurnr = 1, ncurvs

!     --- Fill composite curves

      if ( kelmq/=0 ) call mshfillmsub ( ncurvs, kmeshmnew, kmeshq )

!     --- Raise first positions and fill part 4

      kmeshmnew(3) = kmeshmnew(5+ncurvs)+kmeshm(2)-1
      ipos3 = kmeshmnew(3)

      kmeshmnew(ipos3) = 0

      do i = 1, ninner
          kmeshmnew(ipos3+i-1) = kmeshm(kmeshm(3)+i-1)
      end do
      if ( debug ) call pr0008 ( kmeshmnew, ncurvs )
      kkrom = kmeshmnew(3) + ninner

      if ( kkrom>lenkmeshmnew ) then

         call errint ( kkrom, 1 )
         call errint ( lenkmeshmnew, 2 )
         call errsub ( 940, 2, 0, 0 )
         go to 1000

      end if

      lenkmeshmnew = kkrom

!     --- Adapt coordinates of curves

      call mshspcboun ( iinput, input, tauarr, &
                        factor, curves, rinput, coorh, &
                        kmeshmnew, iwork, lncurves/ndim, lenkmeshmnew )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspckmeshm'

      end if  ! ( debug )
      call erclos ( 'mshspckmeshm' )

      end subroutine mshspckmeshm
