      subroutine mshspecreord13 ( kmeshc, inelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder elements of shape 13 in case of spectral elements
!     without fill-in (one element group only)
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     reordering
!     hexahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(8,*), inelem

!     inelem         i    Number of elements in a specific element group
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         One element group only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, i

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Switch nodes 3 and 4 and 7 and 8 for all elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspecreord13' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspecreord13'

      end if
      if ( ierror.ne.0 ) go to 1000
      do i = 1, inelem
         ih = kmeshc(4,i)
         kmeshc(4,i) = kmeshc(3,i)
         kmeshc(3,i) = ih

         ih = kmeshc(8,i)
         kmeshc(8,i) = kmeshc(7,i)
         kmeshc(7,i) = ih
      end do

1000  call erclos ( 'mshspecreord13' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspecreord13'

      end if

      end
