      subroutine mshsurfaces4 ( kmeshn, nsurfs, nsurfsold, surffaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set array ihelp equal to 0 for all surfaces that do not have
!     common faces with other surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     faces
!     mesh
!     compound_surface
!     surfaces
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), nsurfs, nsurfsold, surffaces(*)

!     ihelp         i/o   integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!     iwork          i    integer work array of length nfaces
!                         iwork(iface) contains the number of surfaces
!                         face iface is stored in
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nsurfs         i    number of surfaces
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, jstart, isurnr, inelem, inpelm, iielem, iface
      logical newsurface

!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     jstart         starting position of curve i in kmeshm
!     newsurface     If true the surface is single
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces isurnr do
!        If ( all faces in the surface correspond to one surface only ) then
!           ihelp(isurnr) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces4'

      end if
      if ( ierror.ne.0 ) go to 1000

      istart = 0
      jstart = 2*nsurfs

!     --- For all surfaces do

      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)  ! number of elements in surface
         inpelm = kmeshn(isurnr)         ! number of nodes per element
         newsurface = .true.             ! mark surface as single

!        --- For all faces iface in surface do

         do iielem = 1, inelem

!        --- iface := surffaces(istart+iielem)

            iface = surffaces(istart+iielem)  ! face number of element
            if ( iwork(iface).gt.1 ) then

!           --- Double face found

               if ( newsurface ) then

!              --- newsurface found

                  newsurface = .false.  ! mark surface as double
                  go to 100

               end if  ! ( newsurface )

            end if  ! ( iwork(iface).gt.0 )
            jstart = jstart+inpelm

         end do  ! iielem = 1, inelem

100      if ( newsurface ) ihelp(isurnr) = 0  ! single surface

         istart = istart+inelem

      end do  ! isurnr = 1, nsurfsold

1000  call erclos ( 'mshsurfaces4' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces4'

      end if

      end
