      subroutine mshtransinfo2 ( iwork, npoint, kmshe1, kmshe2, &
                                 npointloc, points )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-03-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute all neigbour points of block blocknr and store in array iwork
!     in the following way:
!        iwork(i) = -1:  point is in block blocknr
!        iwork(i) =  0:  point is not a neighbour of blocknr
!        iwork(i) =  1:  point is a neighbour of blocknr
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     topology
!     neighbour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), npoint, kmshe1(*), kmshe2(*), npointloc, &
              points(*)

!     iwork          o    integer work array of length npoint that is filled
!                         in the following way:
!                         iwork(i) = -1:  point i is in block blocknr
!                         iwork(i) =  0:  point i is not a neighbour of blocknr
!                         iwork(i) =  1:  point i is a neighbour of blocknr
!     kmshe1         i    Contains for each nodal the starting point of the
!                         neighbouring points  in array kmshe2 minus one
!                         kmshe1(1) = 0
!                         number of neighbours of node i is
!                         kmshe1(i+1)-kmshe1(i)
!     kmshe2         i    Contains the neighbours of all the nodes
!     npoint         i    Number of nodal points
!     npointloc      i    Number of points in local mesh
!     points         i    Array of size npointloc containing the node
!                         numbers corresponding to the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, node

!     i              Counting variable
!     j              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Set iwork = -1 for all nodes in iblock
!     Fill iwork for all neighbour points of points in iblock
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtransinfo2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtransinfo2'

      end if

!     --- Clear array iwork

      do i = 1, npoint
         iwork(i) = 0
      end do  !

!     --- Set iwork = -1 for all nodes in iblock

      do i = 1, npointloc
         iwork(points(i)) = -1
      end do

!     --- Fill iwork for all neighbour points of points in iblock

      do i = 1, npoint

         if ( iwork(i).eq.-1 ) then

!        --- iwork(i) = -1, hence node in iblock
!            Loop over all neighbours of point i

            do j = kmshe1(i)+1, kmshe1(i+1)

               node = kmshe2(j)

               if ( iwork(node).eq.0 ) then

!              --- Node is neighbour of point in block i but not in block i
!                  itself, so it is a node we are looking for

                  iwork(node) = 1

               end if  ! ( iwork(inode).eq.0 )

            end do  ! j = kmshe1(i)+1, kmshe1(i+1)

         end if

      end do  ! i = 1, npoint

      call erclos ( 'mshtransinfo2' )
      if ( debug ) write(irefwr,*) 'End mshtransinfo2'

      end
