      subroutine mshwrelgrps ( iinput, rinput, irefmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 01-02-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write parts of arrays iinput and rinput corresponding to element groups
!   in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh
      double precision rinput(*)

!     iinput         i    integer input array
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indent, ilast, i, iinp, ielgrp, intproplc, j
      double precision realproplc
      character *1000 wrtrecord

!     i              Counting variable
!     ielgrp         element group number
!     iinp           last position used in array iinput
!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     intproplc      Integer property
!     j              Counting variable
!     realproplc     Real property
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTRL       Put real number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshwrelgrps' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwrelgrps'

      end if
      indent = 3
      ilast = 0

      if ( nlines>0 ) then

!     --- if ( nlines>0 ) then
!            write record with meshline

         call iniputxt ( 'meshline', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         indent = 6
         iinp = iinp3

!        --- write line elements

         do i = 1, nlines
            ielgrp = iinput(iinp+1)
            call iniputxt ( 'lelm', wrtrecord, ilast )
            call iniputin ( ielgrp, wrtrecord, ilast )
            call iniputxt ( ' = ( shape = ', wrtrecord, ilast )
            call iniputin ( iinput(iinp), wrtrecord, ilast )
            call iniputxt ( ', c', wrtrecord, ilast )
            call iniputin ( iinput(iinp+2), wrtrecord, ilast )
            call iniputxt ( ' to c', wrtrecord, ilast )
            call iniputin ( iinput(iinp+3), wrtrecord, ilast )
            call iniputxt ( ' )', wrtrecord, ilast )

!           --- Integer properties

            do j = 1, nintprop
               call iniputxt ( ', int_property ', wrtrecord, ilast )
               call iniputin ( j, wrtrecord, ilast )
               call iniputxt ( ' = ', wrtrecord, ilast )
               intproplc = iinput(iinp12-1+(ielgrp-1)*nintprop+j)
               call iniputin ( intproplc, wrtrecord, ilast )
            end do

!           --- Real properties

            do j = 1, nrealprop
               call iniputxt ( ', real_property ', wrtrecord, ilast )
               call iniputin ( j, wrtrecord, ilast )
               call iniputxt ( ' = ', wrtrecord, ilast )
               realproplc = rinput(irnp5-1+(ielgrp-1)*nrealprop+j)
               call iniputrl ( realproplc, wrtrecord, ilast )
            end do

            call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
            iinp = iinp+4

         end do
         indent = 3

      end if

      if ( nsurel>0 ) then

!     --- nsurel>0
!           write record with meshsurf

         call iniputxt ( 'meshsurf', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         indent = 6
         iinp = iinp4

!        --- write surface elements

         do i = 1, nsurel
            ielgrp = iinput(iinp+1)
            call iniputxt ( 'selm', wrtrecord, ilast )
            call iniputin ( ielgrp, wrtrecord, ilast )
            call iniputxt ( ' = ( s', wrtrecord, ilast )
            call iniputin ( iinput(iinp+2), wrtrecord, ilast )
            if ( iinput(iinp+2)/=iinput(iinp+3) ) then
               call iniputxt ( ' to s', wrtrecord, ilast )
               call iniputin ( iinput(iinp+3), wrtrecord, ilast )
            end if
            call iniputxt ( ' )', wrtrecord, ilast )

!           --- Integer properties

            do j = 1, nintprop
               call iniputxt ( ', int_property ', wrtrecord, ilast )
               call iniputin ( j, wrtrecord, ilast )
               call iniputxt ( ' = ', wrtrecord, ilast )
               intproplc = iinput(iinp12-1+(ielgrp-1)*nintprop+j)
               call iniputin ( intproplc, wrtrecord, ilast )
            end do

!           --- Real properties

            do j = 1, nrealprop
               call iniputxt ( ', real_property ', wrtrecord, ilast )
               call iniputin ( j, wrtrecord, ilast )
               call iniputxt ( ' = ', wrtrecord, ilast )
               realproplc = rinput(irnp5+(ielgrp-1)*nrealprop+j)
               call iniputrl ( realproplc, wrtrecord, ilast )
            end do
            call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
            iinp = iinp+4

         end do
         indent = 3

      end if

      if ( nvolel>0 ) then

!     --- if ( nvolel>0 ) then
!            write record with meshvolume

         call iniputxt ( 'meshvolume', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         indent = 6
         iinp = iinp6

!        --- write volume elements

         do i = 1, nvolel
            ielgrp = iinput(iinp+1)
            call iniputxt ( 'velm', wrtrecord, ilast )
            call iniputin ( ielgrp, wrtrecord, ilast )
            call iniputxt ( ' = ( v', wrtrecord, ilast )
            call iniputin ( iinput(iinp+2), wrtrecord, ilast )
            if ( iinput(iinp+2)/=iinput(iinp+3) ) then
               call iniputxt ( ' to v', wrtrecord, ilast )
               call iniputin ( iinput(iinp+3), wrtrecord, ilast )
            end if
            call iniputxt ( ' )', wrtrecord, ilast )

!           --- Integer properties

            do j = 1, nintprop
               call iniputxt ( ', int_property ', wrtrecord, ilast )
               call iniputin ( j, wrtrecord, ilast )
               call iniputxt ( ' = ', wrtrecord, ilast )
               intproplc = iinput(iinp12-1+(ielgrp-1)*nintprop+j)
               call iniputin ( intproplc, wrtrecord, ilast )
            end do

!           --- Real properties

            do j = 1, nrealprop
               call iniputxt ( ', real_property ', wrtrecord, ilast )
               call iniputin ( j, wrtrecord, ilast )
               call iniputxt ( ' = ', wrtrecord, ilast )
               realproplc = rinput(irnp5+(ielgrp-1)*nrealprop+j)
               call iniputrl ( realproplc, wrtrecord, ilast )
            end do

            call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
            iinp = iinp+4
         end do
         indent = 3

      end if

      if ( debug ) write(irefwr,*) 'End mshwrelgrps'
      call erclos ( 'mshwrelgrps' )

      end
