      subroutine mshwrtraninfo ( iprocessor, blockinfo, lenblockinfo, &
                                 transportinfo, lentransinfo, points, &
                                 npointloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-03-2007 Adaptation format
!        version  1.1  date 20-03-2003 New definition of meshoutput_par.000
!        version  1.0  date 07-01-2003
!
!   copyright (c) 2003-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write the contents of the arrays blockinfo and sendinfo to the
!     file meshoutput_par.000 for one processor
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iprocessor, blockinfo(*), transportinfo(*), lentransinfo, &
              lenblockinfo, points(*), npointloc

!     blockinfo      i    Array with global block information for surface
!                         number isurnr
!                         Length: 2*m+1
!                         See Programmers Guide Section 24.2 for a description
!     iprocessor     i    Processor sequence number
!     lenblockinfo   i    Length of array blockinfo
!     lentransinfo   i    Length of array transinfo
!     npointloc      i    Number of points in local mesh
!     points         i    Array of size npointloc containing the node
!                         numbers corresponding to the local mesh
!     transportinfo  i    Array containing information of the transport of
!                         unknowns between neighbouring blocks
!                         See Programmers Guide Section 24.2 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numneigh
      character (len=80) text

!     i              Counting variable
!     numneigh       number of neighbours of a block
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     This subroutine writes the information of one block to the file
!     meshoutput_par.000
!
!     Structure of the file
!     Record 1: Text: //number of blocks
!     Record 2: number of blocks
!     For each block:
!        Record 1: Text: //block "block sequence number"
!        Record 2: Text: //number of neighbours
!        Record 3: number of neighbours
!        Record 4: Text: //blockinfo (first length)
!        Record 5: lenblockinfo (length of blockinfo)
!        Record 6 ... : array blockinfo
!        Next record: Text: //transportinfo (first length)
!        Next record: lentransinfo (length of transinfo)
!        Next records:  array transinfo
!        Next record: Text: //global node numbers (first number)
!        Next record: nnodes (number of nodes in block)
!        Next records:  global node numbers in block
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshwrtraninfo' )

      numneigh = blockinfo(1)

      write(text,90) '//block ', iprocessor
 90   format ( a8, i5 )
      if ( iref10>0 ) then

!     --- formatted

         write(iref10,100) text
100      format ( a )
         text = '//number of neighbours'
         write(iref10,100)  text
         write(iref10,110) numneigh
110      format ( 10i9 )

      else

!     --- unformatted

         write(-iref10) text
         text = '//number of neighbours'
         write(-iref10) text
         write(-iref10) numneigh

      end if
      if ( iref10>0 ) then

!     --- formatted

         text = '//blockinfo (first length)'
         write(iref10,100) text
         write(iref10,110) lenblockinfo
         write(iref10,110) (blockinfo(i), i=1,lenblockinfo)
         text = '//transportinfo (first length)'
         write(iref10,100) text
         write(iref10,110) lentransinfo
         write(iref10,110) (transportinfo(i), i=1,lentransinfo)
         text = '//global node numbers (first number)'
         write(iref10,100) text
         write(iref10,110) npointloc
         write(iref10,110) (points(i), i=1,npointloc)

      else

!     --- unformatted

         text = '//blockinfo (first length)'
         write(-iref10) text
         write(-iref10) lenblockinfo
         write(-iref10) (blockinfo(i), i=1,lenblockinfo)
         text = '//transportinfo (first length)'
         write(-iref10) text
         write(-iref10) lentransinfo
         write(-iref10) (transportinfo(i), i=1,lentransinfo)
         text = '//global node numbers (first number)'
         write(-iref10) text
         write(-iref10) npointloc
         write(-iref10) (points(i), i=1,npointloc)

      end if
      call erclos ( 'mshwrtraninfo' )
      end
