      subroutine sepflielemobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ielemobst if it has not been filled yet
!     Old subroutine iniactm0 (22)
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: exist

!     exist          If true array ielemobst has been filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLKMAD2   Fill array kmesh ad  part 2: ielemobst
!     SEPINITKMESHAD Initialize kmesh ad
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflielemobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflielemobst'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      exist = .false.
      if ( debug ) write(irefwr,1) 'kelmad', kelmad
      if ( kelmad>0 ) then

!     --- Array kmesh ad already exists: check if sequence number is
!         correct

         if ( debug ) call prinin ( kmeshad, 10, 'kmeshad' )
         if ( kmeshad(2)>0 .and. kmeshad(5)==kmeshad(7) ) exist = .true.

      else

!     --- kmesh ad does not exist. Create

         call sepinitkmeshad

      end if  ! ( kelmad>0 )

      if ( debug ) write(irefwr,*) 'exist ', exist

      if ( .not. exist ) then

!     --- Array has not been created yet, fill

         call sepfillkmad2

      end if  ! ( .not. exist )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflielemobst'

      end if  ! ( debug )
      call erclos ( 'sepflielemobst' )

      end subroutine sepflielemobst
