      subroutine sepupdobs
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in kmesh part x concerning obstacles
!     Old subroutine mshupdobs
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nx, ny, iobst, n, istart, icurnr, j
      double precision xmin, xmax, ymin, ymax, dx, dy, x, y, &
                       unit, deltax

!     deltax         max unit length of element
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     icurnr         Curve sequence number
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     j              Counting variable
!     n              number of basis functions for each component
!     nx             number of points in x direction
!     ny             number of points in y direction
!     unit           unit length
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y-coordinate read from the SEPRAN neutral plot file
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For all obstacles do
!      Compute xmin, xmax, ymin, ymax
!      Check if obstacle is closed or open
!      unit := minimum length in elsize
!      dx := xmax-xmin; dy := ymax-ymin
!      xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!      ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!      nx := max(5,4*(xmax-xmin)/unit+1)
!      ny := max(5,4*(ymax-ymin)/unit+1)
!      dx := (xmax-xmin)/nx;  dy := (ymax-ymin)/ny
!      Fill information in arrays kmeshx
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepupdobs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepupdobs'

      end if

!     --- unit := minimum length in elsize

      unit = rinfin
      do i = 1, ncurvs
         do j = 1, 2
            x = elsize(j,i)
            if ( x>0d0 ) unit = min(unit,x)
            if ( x>0d0 ) deltax = max(deltax,x)
         end do  ! j = 1, 2
      end do
      if ( debug ) write(irefwr,*) 'numobst, unit, deltax ', &
         numobst, unit, deltax

!     --- For all obstacles

      do iobst = 1, numobst

!     --- Compute xmin, xmax, ymin, ymax

         icurnr = kmeshx(2,iobst)
         if ( icurnr>0 ) then

!        --- icurnr>0, hence existing obstacle

            if ( icurnr==1 ) then

!           --- icurnr = 1, first curve

               n = icurvsmsh(icurnr)
               istart = 0

            else

!           --- icurnr > 1

               n = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)
               istart = icurvsmsh(icurnr-1)

            end if
            if ( debug ) write(irefwr,*) 'icurnr, n ', icurnr, n

            xmin = rinfin
            xmax =-rinfin
            ymin = rinfin
            ymax =-rinfin
            do i = 1, n
               x    = curvesmsh(1,istart+i)
               y    = curvesmsh(2,istart+i)
               xmin = min(xmin,x)
               xmax = max(xmax,x)
               ymin = min(ymin,y)
               ymax = max(ymax,y)
            end do
            if ( debug ) write(irefwr,*) 'xmin, xmax, ymin, ymax ', &
               xmin, xmax, ymin, ymax

!           --- dx := xmax-xmin; dy := ymax-ymin

            if (abs(xmax-xmin)<deltax) then

!           --- xmax = xmin

               deltax=deltax*0.5d0
               xmin=xmin-deltax
               xmax=xmax+deltax

            end if

            if (abs(ymax-ymin)<deltax) then

!           --- ymax = ymin

               deltax=deltax*0.5d0
               ymin=ymin-deltax
               ymax=ymax+deltax

            end if

            dx = xmax-xmin
            dy = ymax-ymin

!           --- xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!               ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!               nx := max(5,4*(xmax-xmin)/unit+1)
!               ny := max(5,4*(ymax-ymin)/unit+1)
!               dx := (xmax-xmin)/nx;  dy := (ymax-ymin)/ny

            xmax = xmax+0.005d0*dx
            xmin = xmin-0.005d0*dx
            ymax = ymax+0.005d0*dy
            ymin = ymin-0.005d0*dy
            dx   = xmax-xmin
            dy   = ymax-ymin
            nx   = min(max(5,int(1.1d0*dx/unit+1d0)),100)
            ny   = min(max(5,int(1.1d0*dy/unit+1d0)),100)
            dx   = dx/nx
            dy   = dy/ny
            if ( debug ) write(irefwr,*) 'xmin, xmax, ymin, ymax ', &
               xmin, xmax, ymin, ymax
            if ( debug ) write(irefwr,*) 'nx, ny, dx, dy ', &
               nx, ny, dx, dy

!           --- Fill information in arrays kmeshx

            rmeshx(1,iobst) = xmin
            rmeshx(2,iobst) = xmax
            rmeshx(3,iobst) = ymin
            rmeshx(4,iobst) = ymax
            rmeshx(5,iobst) = dx
            rmeshx(6,iobst) = dy
            kmeshx(3,iobst) = nx
            kmeshx(4,iobst) = ny

         end if

      end do

!     --- icrossobst and rcrossobst need to be filled again

      iseqobst = iseqobst + 1

      call erclos ( 'sepupdobs' )
      end
