      subroutine meshwrpar1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write information of refined mesh to file meshoutput_par.000
!
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemach
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenname, irefsav, lentransinfo, ipstart, ipmapping, &
              lenblockinfo, npointloc, ipblockinfo, &
              npoint_glob, i, startaddress(iacnodes,2,3), &
              icontext, iblock, iptransinfo
      integer, allocatable :: iwork(:)
      character (len=100) name10sav
      character (len=80) text

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     ipblockinfo    Starting address in array blockinfo
!     ipmapping      Starting adress of array mapping
!     ipstart        Starting address in array iwork
!     iptransinfo    Starting address in array transinfo
!     irefsav        Help parameter to save the value of iref
!     iwork          integer work array to receive informatio from other blocks
!     lenblockinfo   Length of array blockinfo
!     lenname        Length of name
!     lentransinfo   Length of array transinfo
!     name10sav      Help variable to store value of name10
!     npoint_glob    Number of nodes in global mesh
!     npointloc      Number of points in local mesh
!     startaddress   Work array of size iacnodes x 2 x 3 to keep track of
!                    starting addresses in iwork
!                    (iblock,1,1) contains the startaddress of itransinfo
!                                 for block iblock
!                    (iblock,2,1) contains the length of itransinfo
!                                 for block iblock
!                    (iblock,.,2) refers to blockinfo
!                    (iblock,.,3) refers to mapping
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIFIL         Open files
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     MSHWRTRANINFO  Write the contents of the arrays blockinfo and transinfo to
!                    the file meshoutput_par.000
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'meshwrpar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from meshwrpar1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenname = index(name10,' ')-1
      name10sav = name10   ! save name10
      name10(lenname-2:lenname) = '000'  ! create meshoutput_par.000

      if ( debug ) then
         write(irefwr,1) 'lenname', lenname
         write(irefwr,*) 'name10', name10
      end if  ! ( debug )

      irefsav = iref10     ! save iref10
      if ( iref10>0 ) then
         iref10 = 24
      else
         iref10 = -24
      end if
      call inifil(10)      ! open file

      text = '//number of blocks'
      if ( iref10>0 ) then

!     --- formatted

         write(iref10,190) text
190      format ( a )
         write(iref10,200) iacnodes
200      format ( i6 )

      else

!     --- unformatted

         write(-iref10) text
         write(-iref10) iacnodes

      end if

!     --- First we write node number 1

      lentransinfo = size(transinfo)
      lenblockinfo = size(blockinfo)

!     --- write

      call mshwrtraninfo ( inodenr, blockinfo, lenblockinfo, &
                           transinfo, lentransinfo, &
                           mappingparcp, npoint )

!     --- Next we receive information from other blocks and store in iwork
!         First allocate work space

      npoint_glob = kmeshpar(1)  ! global number of points
      allocate ( iwork(npoint_glob*iacnodes), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint_glob*(iacnodes-1)*3, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all other nodes

      ipstart = 1
      do i = 1, iacnodes-1

!     --- Receive array transinfo

         icontext = 31
         length = npoint_glob*iacnodes-ipstart+1
         call inireceivempi ( iwork(ipstart), length, SPINT, iblock, &
                              icontext )
         call inichkarlen ( length, npoint_glob*iacnodes-ipstart+1, &
                            'transinfo' )
         if ( ierror/=0 ) go to 1000
         startaddress(iblock,1,1) = ipstart  ! starting address transinfo
         startaddress(iblock,2,1) = length   ! length transinfo
         ipstart = ipstart+length

      end do  ! i = 1, iacnodes-1

      do i = 1, iacnodes-1

!     --- Receive array blockinfo

         icontext = 32
         length = npoint_glob*iacnodes-ipstart+1
         call inireceivempi ( iwork(ipstart), length, SPINT, iblock, &
                              icontext )
         call inichkarlen ( length, npoint_glob*iacnodes-ipstart+1, &
                            'blockinfo' )
         if ( ierror/=0 ) go to 1000
         startaddress(iblock,1,2) = ipstart  ! starting address blockinfo
         startaddress(iblock,2,2) = length   ! length blockinfo
         ipstart = ipstart+length

      end do  ! i = 1, iacnodes-1

      do i = 1, iacnodes-1

!     --- Receive array mapping

         icontext = 33
         length = npoint_glob*iacnodes-ipstart+1
         call inireceivempi ( iwork(ipstart), length, SPINT, iblock, &
                              icontext )
         call inichkarlen ( length, npoint_glob*iacnodes-ipstart+1, &
                            'mapping' )
         if ( ierror/=0 ) go to 1000
         startaddress(iblock,1,3) = ipstart  ! starting address mapping
         startaddress(iblock,2,3) = length   ! length mapping
         ipstart = ipstart+length

      end do  ! i = 1, iacnodes-1

!     --- Write rest of information

      do iblock = 2, iacnodes

         iptransinfo = startaddress(iblock,1,1)
         ipblockinfo = startaddress(iblock,1,2)
         ipmapping = startaddress(iblock,1,3)
         lentransinfo = startaddress(iblock,2,1)
         lenblockinfo = startaddress(iblock,2,2)
         npointloc = startaddress(iblock,2,3)
         call mshwrtraninfo ( iblock, iwork(ipblockinfo), lenblockinfo, &
                              iwork(iptransinfo), lentransinfo, &
                              iwork(ipmapping), npointloc )
      end do  ! iblock = 2, iacnodes

      close(24)     ! close file
      name10 = name10sav   ! reset name10
      iref10 = irefsav  ! reset iref10

!     --- remove work array

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'meshwrpar1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End meshwrpar1'

      end if  ! ( debug )

      end
