      subroutine msh043case1 ( iinput, ndefvl, n, m, l, nsurfs, kmeshs, &
                               ivolnr, irand, nx, ny, nz, npoint, &
                               ishape, nelem, mtetra, nbcrd, nbndpt, &
                               lenwrk, mtrian )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-03-2006 Extra debug statements
!        version  1.0  date 04-05-2005
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for mshvlm
!     Case 1: subroutine brick
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndefvl, n, m, l, nsurfs, kmeshs(*), &
              ivolnr, irand, nx, ny, nz, npoint, ishape, nelem, mtetra, &
              nbcrd, nbndpt, lenwrk, mtrian

!     iinput         i    integer input array containing the information
!                         of volumes
!     irand          i    number of points on the side of an element - 1
!     ishape         i    Shape number of elements
!     ivolnr         i    Volume sequence number
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     l              o    Number of elements in z-direction
!     lenwrk         o    Length of array iwork
!     m              o    number of elements in y-direction
!     mtetra         i    number of elements in one hexahedron
!                         hexahedron: 1   tetrahedron: 6
!     mtrian         i    indication whether the standard element is a
!                         quadrilateral (1) or a triangle (2)
!     n              o    number of elements in x-direction
!     nbcrd          o    number of nodal points in boundary of submesh
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     ndefvl         i    Number of surfaces in volume
!     nelem          o    Number of elements
!     npoint         o    Number of nodal points
!     nsurfs         i    number of surfaces
!     nx             o    number of points in x direction
!     ny             o    number of points in y direction
!     nz             o    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, iorien, ns, isur, nmaxyz

!     iorien         Defines orientation of the base vectors in 3D plots
!                    1 x-y-z
!                    2 z-x-y
!                    3 y-z-x
!     isur           surface number
!     isurnr         Surface sequence number
!     nmaxyz         maximum of nx, ny and nz
!     ns             Starting address of information of specific surface
!                    in array KMESHS
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2099   Parsurf's in brick not yet fully supported.
!    2148   Type of surface not allowed for brick
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh043case1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh043case1'
         write(irefwr,1) 'nsurfs', nsurfs
         call prinin ( kmeshs, 10, 'kmeshs' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      n = iinput(4+ndefvl)
      m = iinput(5+ndefvl)
      l = iinput(6+ndefvl)
      if ( debug ) write(irefwr,1) 'n, m, l', n, m, l

      if ( n==0 .or. m==0 .or. l==0 ) then

!     --- n,m,l = 0; Compute these parameters from the surfaces S1 and S2

         isurnr = iinput(4)
         iorien = (isurnr-1)/nsurfs
         isurnr = mod(isurnr-1,nsurfs)+1
         if ( debug ) &
            write(irefwr,1) 'isurnr, iorien', isurnr, iorien
         ns = nsurfs+1+(isurnr-1)*8
         if ( debug ) &
            write(irefwr,1) 'ns, kmeshs(ns+1)', ns, kmeshs(ns+1)
         if ( kmeshs(ns+1)/=1  .and. kmeshs(ns+1)/=11 .and. &
              kmeshs(ns+1)/=13 .and. kmeshs(ns+1)/=17 ) then

!        --- surface is copied from surface of type rectangle, coons
!            or ordered surface
!            post a warning if this is a surface generated by PARSURF:

            if ( kmeshs(ns+1)==15 ) then

!           --- kmesh is of type PARSURF

               call errint ( ivolnr, 1 )
               call errint ( isurnr, 2 )
               call errwar ( 2099, 2, 0, 0 )

            end if
            isur = kmeshs(ns+4)
            iorien = (isur-1)/nsurfs
            isur = mod(isur-1,nsurfs)+1
            ns = nsurfs+1+(isur-1)*8

         end if
         if (  kmeshs(ns+1)==11 ) ns = ns+2
         if ( mod(iorien,2)==0 .and. iorien<=3 .or. &
              mod(iorien,2)==1 .and. iorien>3 ) then
            n  = kmeshs(ns+3)
            m  = kmeshs(ns+4)
         else
            n  = kmeshs(ns+4)
            m  = kmeshs(ns+3)
         end if

         isurnr = iinput(5)
         iorien = (isurnr-1)/nsurfs
         isurnr = mod(isurnr-1,nsurfs)+1
         ns = nsurfs+1+(isurnr-1)*8
         call errint ( ivolnr, 2 )
         call errint ( isurnr, 1 )
         if ( kmeshs(ns+1)==2 ) then
            call errchr ( 'general', 1 )
            call errsub ( 2148, 2, 0, 1 )
         else if ( kmeshs(ns+1)==3 ) then
            call errchr ( 'triangle', 1 )
            call errsub ( 2148, 2, 0, 1 )
         else if ( kmeshs(ns+1)==4 ) then
            call errchr ( 'quadrilateral', 1 )
            call errsub ( 2148, 2, 0, 1 )
         else if ( kmeshs(ns+1)==10 ) then
            call errchr ( 'pipesurface', 1 )
            call errsub ( 2148, 2, 0, 1 )
         else if ( kmeshs(ns+1)==15 ) then
            if ( kmeshs(ns+3)/=kmeshs(ns+5) .or. &
                 kmeshs(ns+4)/=kmeshs(ns+6) ) then

!           --- surface created by quadrilateral

               call errchr ( 'PARSURF', 1 )
               call errsub ( 2148, 2, 0, 1 )

            end if

         else if ( kmeshs(ns+1)==18 ) then
            call errchr ( 'isopar', 1 )
            call errsub ( 2148, 2, 0, 1 )
         else if ( kmeshs(ns+1)==19 ) then
            call errchr ( 'paver', 1 )
            call errsub ( 2148, 2, 0, 1 )
         else if ( kmeshs(ns+1)/=1  .and. kmeshs(ns+1)/=11 .and. &
                   kmeshs(ns+1)/=13 .and. kmeshs(ns+1)/=17 ) then

!        --- surface is copied from surface of type rectangle, coons
!            or ordered surface
!            post a warning if this is a surface generated by PARSURF:

            if ( kmeshs(ns+1)==15 ) then
              call errint ( ivolnr, 1 )
              call errint ( isurnr, 2 )
              call errwar ( 2099, 2, 0, 0 )
            end if
            isur = kmeshs(ns+4)
            iorien = (isur-1)/nsurfs
            isur = mod(isur-1,nsurfs)+1
            ns = nsurfs+1+(isur-1)*8

         end if

         if (  kmeshs(ns+1)==11 ) ns = ns+2
         if ( mod(iorien,2)==0 .and. iorien<=3 .or. &
              mod(iorien,2)==1 .and. iorien>3 ) then
            l  = kmeshs(ns+4)
         else
            l  = kmeshs(ns+3)
         end if

      end if
      nx = n*irand+1
      ny = m*irand+1
      nz = l*irand+1
      npoint = nx*ny*nz
      if ( ishape==15 .or. ishape==16 ) &
        npoint = (5*nx*ny-2*nx-2*ny+1)*(nz-1)/2+nx*ny+(nx-1)*(ny-1)/2
      nelem = n*m*l*mtetra
      if ( ishape==16 ) npoint = npoint + nelem
      nbcrd = (nx*ny+nx*nz+ny*nz)*2
      if ( ishape==15 .or. ishape==16 ) nbcrd = nbcrd + &
         (n*m+n*l+l*m)*4
      nbndpt = (n*m+n*l+l*m)*2*mtrian
      nmaxyz = max(nx, ny, nz)
      lenwrk = 4*nmaxyz

1000  call erclos ( 'msh043case1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh043case1'

      end if

      end
