      subroutine msh076surfs ( iinput, iinp, isurfs, ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kmeshc for interface elements
!     surface elements only
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, isurfs, ikelmc

!     iinp          i/o   last position used in array iinput with respect to
!                         interface elements
!     iinput         i    integer input array with respect to the mesh
!     ikelmc        i/o   last position used in kmesh part c
!     isurfs         i    Number of parts where surface interface elements
!                         are given
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr1, isurnr2, istart1, istart2, j, inpelm, &
              ielem, ielgrp

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     inpelm         number of points per element
!     istart1        First position used in kmeshn for surface 1
!     istart2        First position used in kmeshn for surface 2
!     isurnr1        Surface sequence number
!     isurnr2        Surface sequence number
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh076surfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh076surfs'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      if ( debug ) then
         write(irefwr,1) 'nsurfs', nsurfs
         call prinin ( kmeshn, nsurfs, 'kmeshn/1' )
         call prinin ( kmeshn(nsurfs+1), nsurfs, 'kmeshn/2' )
      end if  ! ( debug )

!     --- Loop over isurfs parts

      do i = 1, isurfs

         ielgrp = iinput(iinp)        ! element group
         isurnr1 = iinput(iinp+2)    ! first surface number
         isurnr2 = iinput(iinp+3)    ! second surface number

!        --- Compute starting addresses of surfaces

         istart1 = 2*nsurfs          ! starting address surface 1
         istart2 = istart1           ! starting address surface 2
         do j = 1, isurnr1-1
            istart1 = istart1+kmeshn(j)*kmeshn(j+nsurfs)
         end do  ! j = 1, isurnr1-1
         do j = 1, isurnr2-1
            istart2 = istart2+kmeshn(j)*kmeshn(j+nsurfs)
         end do  ! j = 1, isurnr2-1
         inpelm = kmeshn(isurnr1)
         inelem = kmeshn(isurnr1+nsurfs)
         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'i, ielgrp, isurnr1, isurnr2', &
                             i, ielgrp, isurnr1, isurnr2
            write(irefwr,1) 'istart1, istart2', istart1, istart2
            write(irefwr,1) 'inpelm, inelem', inpelm, inelem

         end if  ! ( debug )

!        --- Raise kmesh part g

         kmeshg(ielgrp) = kmeshg(ielgrp)+inelem

!        --- Fill kmeshc

         do ielem = 1, inelem

            do j = 1, inpelm
               kmeshc(ikelmc+j) = kmeshn(istart1+j)
            end do  ! j = 1, inpelm
            ikelmc = ikelmc+inpelm
            istart1 = istart1+inpelm
            do j = 1, inpelm
               kmeshc(ikelmc+j) = kmeshn(istart2+j)
            end do  ! j = 1, inpelm
            ikelmc = ikelmc+inpelm
            istart2 = istart2+inpelm

            if ( debug ) then
               write(irefwr,1) 'ielem, ikelmc', ielem, ikelmc-2*inpelm
               call prinin ( kmeshc(ikelmc-2*inpelm), 2*inpelm, &
                             'element' )
            end if  ! ( debug )

         end do  ! ielem = 1, inelem

         iinp = iinp+4  ! raise iinp

      end do  ! i = 1, isurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh076surfs'

      end if
      call erclos ( 'msh076surfs' )

      end
