      subroutine mshadapcoorcrv ( kmeshm, coor, ndim, icurnr, curves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates from curve stored in array curves into
!     array coor
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), ndim, icurnr
      double precision coor(ndim,*), curves(ndim,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curves         i    array containing the coordinates of the curves
!     icurnr         i    Curve sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, istar3, numnodes, node

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     node           Node number
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapcoorcrv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapcoorcrv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istar3 = kmeshm(2)-1
      i1 = kmeshm(4+icurnr)
      i2 = kmeshm(5+icurnr)-1
      istar3 = istar3+i1-1
      numnodes = i2-i1
      do i = 1, numnodes
         node = kmeshm(istar3+i)
         coor(1:ndim,node) = curves(1:ndim,i)
      end do  ! i = 1, nnodes

1000  call erclos ( 'mshadapcoorcrv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapcoorcrv'

      end if  ! ( debug )

      end
