      subroutine mshadapmids ( kmeshc, ishape, iwork, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of mid side points of one element such that
!     they are in the centre of the side
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), ishape, iwork(*)
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ishape         i    Shape number of elements
!     iwork          i    integer work array of length npoint
!                         only nodes i with iwork(i)=0 may be moved
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHADAPMIDS4   Adapt coordinates of mid-side points of quadratic
!                    triangle
!     MSHADAPMIDS6   Adapt coordinates of mid-side points of bi-quadratic
!                    quadrilateral
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapmids' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapmids'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      select case ( ishape)

         case(3,5)

!        --- Shape 3, 5, no action

         case(4,7)

!        --- Shape 4,7, quadratic triangle

            call mshadapmids4 ( coor, kmeshc, iwork, ishape )

         case(6)

!        --- Shape 6, biquadratic quadrilateral

            call mshadapmids6 ( coor, kmeshc, iwork )

         case default

!        --- Shape number not implemented (error)

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

      end select  ! case ( ishape)

1000  call erclos ( 'mshadapmids' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapmids'

      end if  ! ( debug )

      end
