      subroutine mshadsurfs ( usol, iwork, n, indprf, kprobf, &
                              nunkp, iadapt, rinada, coorol, &
                              coornw, anorm, inpada, psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates at boundary (3d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iadapt, iwork(*), n, indprf, kprobf(*), &
              nunkp, inpada(*)
      double precision  usol(*), rinada(*), coorol(3,n), coornw(3,n), &
                        anorm(3,n), psi(*)

!     anorm          i    In this array the normal vector is stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector in the i-th integration point
!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     iadapt         i    Indicates how the boundary must be adapted, see INPADA
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inpada         i    User input array for subroutine ADAPBOUN
!     iwork          i    integer work array containing the nodal points of the
!                         curves
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     n              i    number of nodes in boundary
!     nunkp          i    maximal number of degrees of freedom in nodal points
!     psi            i    Contains the "pseudo" stream function at the
!                         boundary
!     rinada         i    User input array for subroutine ADAPBOUN
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imult, ncomp, number
      double precision amult, uloc(n,3)

!     amult          Multiplication factor
!     imult          Parameter imult as described in inpada(11)
!     ncomp          Number of components of the solution vector that are used
!     number         The number-th unknown in each nodal point of the array
!                    is considered.
!     uloc           Array of size n x ncomp in which the required
!                    degrees of freedom along the boundary are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSOLSELECT    Extract ncomp components from array usol in each point
!                    stored in iwork and put these components in array uloc
!     MSHCHSURF01    Compute the co-ordinates of the new 3D boundary in the case
!                    iadapt = 1: x_new = x_old + factor u n
!     MSHCHSURF03    Compute the co-ordinates of the new 3d boundary in the case
!                    iadapt = 3 i.e. x_new is computed by the so-called film
!                    method
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2990   Value of iadapt not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadsurfs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadsurfs'
         write(irefwr,1) 'iadapt, n, nunkp', iadapt, n, nunkp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      imult  = inpada(11)
      number = inpada(6)

      if ( imult==0 ) then
         amult = 1d0
      else if ( imult==1 ) then
         amult = tstep
      else if ( imult==2 ) then
         amult = rinada(1)
      else
         amult = rinada(1) * tstep
      end if
      if ( iadapt==1 ) then

!     --- iadapt = 1

         ncomp = 1

      else

!     --- iadapt>1

         ncomp = 3

      end if

      if ( debug ) then
         write(irefwr,1) 'imult, number, ncomp', imult, number, ncomp
         write(irefwr,2) 'amult', amult
      end if  ! ( debug )

      if ( iadapt==1 .or. iadapt>2 .and. iadapt<=5 ) then

!     --- Fill array uloc if necessary

         call ersolselect ( usol, iwork, n, indprf, kprobf, &
                            number, ncomp, uloc, nunkp )

      end if

!     --- compute new co-ordinates and store in array coornw

      select case (iadapt)

      case (1)

!     --- iadapt = 1, x_new = x_old + u n

         call mshchsurf01 ( coorol, coornw, uloc, anorm, n )

      case (3)

!     --- iadapt = 3, x_new = computed by film method

         call mshchsurf03 ( psi, n, uloc, anorm, rinada, &
                            coornw, coorol, inpada )

      case default

         call errint ( iadapt, 1 )
         call errsub ( 2990, 1, 0, 0 )
         go to 1000

      end select  ! (case)

      if ( debug ) then

!     --- Debug information

         call  prinrl1(coorol, n, 3, 'coorol')
         call  prinrl1(coornw, n, 3, 'coornw')

      end if

1000  call erclos ( 'mshadsurfs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadsurfs'

      end if  ! ( debug )

      end
