      subroutine mshcheckkbn ( kbndpt, kmeshc, npoh, npoint, nelh, &
                               nelem, isurnr, nbndpt, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Perform some checks on the mesh created in meshus
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), npoh, npoint, nelh, nbndpt, &
              nelem, isurnr, inpelm, kmeshc(inpelm,*)

!     inpelm         i    number of points per element
!     isurnr         i    Surface sequence number
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nelem          i    Number of elements
!     nelh           i    Estimate of the number of elements
!     npoh           i    Estimate of the number of points
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     380   estimation of npoint too small, &
!     381   estimation of nelem too small, &
!    2874   kbndpt not correctly filled in meshus
!    2875   kmeshc not correctly filled in meshus
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckkbn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcheckkbn'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check nelem and npoint

      if ( npoh<npoint .or. npoint<1 ) then
         call errint ( isurnr, 1 )
         call errint ( npoint, 2 )
         call errint ( npoh  , 3 )
         call errsub ( 380, 3, 0, 0 )
      end if
      if ( nelh<nelem .or. nelem<1 ) then
         call errint ( isurnr, 1 )
         call errint ( nelem , 2 )
         call errint ( nelh  , 3 )
         call errsub ( 381, 3, 0, 0 )
      end if

!     --- Check kbndpt

      do i = 1, nbndpt

         if ( kbndpt(i)<=0 .or. kbndpt(i)>npoint ) then

!        --- kbndpt not filled correctly

            call errint ( isurnr, 1 )
            call errint ( i, 2 )
            call errint ( kbndpt(i), 3 )
            call errint ( npoint, 4 )
            call errsub ( 2874, 4, 0, 0 )

         end if  ! ( kbndpt(i)<=0 .or. kbndpt(i)>npoint )

      end do  ! i = 1, nbndpt

!     --- Check kmeshc

      do i = 1, nelem

         do j = 1, inpelm

            if ( kmeshc(j,i)<=0 .or. kmeshc(j,i)>npoint ) then

!           --- kbndpt not filled correctly

               call errint ( isurnr, 1 )
               call errint ( j, 2 )
               call errint ( i, 3 )
               call errint ( kmeshc(j,i), 4 )
               call errint ( npoint, 5 )
               call errsub ( 2875, 5, 0, 0 )

            end if  ! ( kmeshc(j,i)<=0 .or. kmeshc(j,i)>npoint )

         end do  ! j = 1, inpelm

      end do  ! i = 1, nbndpt

1000  call erclos ( 'mshcheckkbn' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcheckkbn'

      end if  ! ( debug )

      end
