      subroutine mshcompdispl30 ( coornew, inpada, rinada, nunkp, &
                                  usol, indprf, &
                                  kprobf, marksurf, marknodes, psiglob, &
                                  kprobfpsi, markcurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 23-12-2011 new call to ersurfsnod
!        version  1.1  date 28-12-2010 new call to ersurfsnod
!        version  1.0  date 31-12-2008
!
!   copyright (c) 2008-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute new coordinates of boundary (3d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), nunkp, &
              indprf, kprobf(*), kprobfpsi(*), &
              marksurf(*), marknodes(*),markcurv(*)
      double precision coornew(3,npoint), rinada(*), &
                       usol(*), psiglob(*)

!     coornew        o    Array containing the new coordinates of the nodes
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inpada         i    User input array for subroutine ADAPBOUN
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     kprobfpsi      i    Kprob part f corresponding to stream function psi
!     markcurv       o    Array of size ncurvs that marks all curves that
!                         have been adapted in the moving boundary
!                         markcurv(icurnr) = 0: curve has not been moved
!                         markcurv(icurnr) = 1: curve has been moved
!     marknodes      o    Array of size npoint that marks all nodes that
!                         have been adapted in the moving boundary
!                         marknodes(node) = 0: node has not been moved
!                         marknodes(node) = 1: node has been moved
!     marksurf       o    Array of size nsurfs that marks all surfaces that
!                         have been adapted in the moving boundary
!                         marksurf(icurnr) = 0: surface has not been moved
!                         marksurf(icurnr) = 1: surface has been moved
!     nunkp          i    Maximum number of unknowns in a point
!     psiglob        i    Contains the "pseudo" stream function along the
!                         free boundary
!     rinada         i    User input array for subroutine ADAPBOUN
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nadapt
      parameter ( nadapt=20 )
      integer nsurfsl, i, inodp, n, normaldr, isurnr
      integer, allocatable :: isurfar(:), isurfsl(:), nodes(:)
      double precision , allocatable :: coorbn(:,:), anorm(:,:), &
                                        coorbnnew(:,:), norm(:,:), &
                                        weight(:), psi(:)

!     anorm          In this array the normal vector is stored
!                    anorm(j,i) contains the j th component of the normal
!                    vector in the i-th integration point
!     coorbn         array containing the old co-ordinates of the boundary
!     coorbnnew      array containing the new co-ordinates of the boundary
!     i              Counting variable
!     inodp          Nodal point number
!     isurfar        Array of length nsurfs that contains information of
!                    the surfaces that must be changed. isurfar(i) mus be
!                    filled as follows:
!                    0  surfe i is not yet adapted
!                    1  surfe i has been adapted and is a single surfe
!     isurfsl        Array of length nsurfsl+1 that is filled with all
!                    stand alone surfaces that must be adapted
!                    isurfsl(0) contains the number of surfaces in isurfsl
!     isurnr         Surface sequence number
!     n              number of nodes in boundary
!     nadapt         Length of array inpada
!     nodes          integer work array in which the node numbers of
!                    the free surface are stored
!     norm           Array of length ndim * npoint containing the normal
!                    in the nodal points
!     normaldr       If 1 the outward directed normal is required, if -1
!                    the inward directed normal
!     nsurfsl        Number of stand-alone surfaces that must be adapted
!     psi            Contains the "pseudo" stream function at the
!                    boundary in the sequence of array nodes
!     weight         work array of length npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERMARKSURNOD   Fill array marknodes for all nodes in array nodes
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCOMPDISPL32 recompute co-ordinates of surfaces
!     MSHMARKCURV    Mark all curves in free surface that have been changed
!     MSHSINGLESURF  Copy contents of array isurfs into temporary array isurfsl
!     PRINRL1        Print 2d real vector
!     SEPCOMNOR      Compute the averaged normals in all points on the surfaces
!                    isrf1 to isrf2
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     385   boundary contains less than two points
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     coornew := coor
!     Store nodes of surfaces to be changed in nodes and coordinates in surfaces
!     Compute new coordinates of surfaces
!     Adapt coornew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl30' )

      allocate(isurfar(nsurfs), isurfsl(0:nsurfs), nodes(npoint), &
               norm(3,npoint), weight(npoint), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( 2*nsurfs+npoint, 1 )
         call errint ( error, 2 )
         call errchr ( 'isurfar', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl30'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isurfar = 0

!     --- coornew := coor

      coornew = coor

!     --- Fill single surfaces and check if surfaces are used only once

      call mshsinglesurf ( inpada(nadapt+1), inpada(2), isurfar, &
                           isurfsl(1), nsurfsl, .false. )
      if ( ierror/=0 ) go to 1000

!     --- mark surfaces in array marksurf

      do i = 1, nsurfsl
         isurnr = abs(isurfsl(i))
         marksurf(isurnr) = 1
      end do  ! i = 1, nsurfsl

!     --- Compute ishape

      ishape = kmeshs(nsurfs+1+8*(isurnr-1)+2)

!     --- Store nodes of surfaces to be changed in nodes

      n = npoint
      call sepsurfsnod ( nsurfsl, nodes, n, isurfsl(1) )
      if ( n<2 ) then
         call errint ( n, 1 )
         call errsub ( 385, 1, 0, 0 )
         go to 1000
      end if
      if ( debug ) write(irefwr,1) 'n', n

!     --- mark nodes in array marknodes

      call ermarksurnod ( nodes, n, marknodes, npoint )

      allocate(coorbn(3,n), coorbnnew(3,n), anorm(3,n), psi(n), &
               stat = error)
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 9*n, 1 )
         call errint ( error, 2 )
         call errchr ( 'coorbn', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- compute normal to coorol and store in array anorm
!         First component nx, second one ny

      isurfsl(0) = nsurfsl
      normaldr = 1
      call sepcomnor ( norm, weight, isurfsl(0), normaldr )
      if ( ierror/=0 ) go to 900

!     --- Fill array coorbn with coordinates of boundary

      do i = 1, n
         inodp = nodes(i)
         coorbn(1:3,i) = coor(1:3,inodp)
         anorm(1:3,i) = norm(1:3,inodp)
         psi(i) = psiglob(kprobfpsi(inodp)+1)
      end do
      if ( debug ) then
         call prinrl1 ( coorbn, n, 3, 'coorbn' )
         call prinrl1 ( anorm, n, 3, 'anorm' )
      end if  ! ( debug )

      coorbnnew = coorbn

!     --- Compute new coordinates of surfaces

      call mshcompdispl32 ( inpada, rinada, nunkp, kprobf, indprf, &
                            usol, nodes, coorbnnew, coorbn, anorm, &
                            n, psi )
      if ( ierror/=0 ) go to 900
      if ( debug ) call prinrl1 ( coorbnnew, n, 3, 'coorbnnew' )

!     --- Adapt coornew

      do i = 1, n
         inodp = nodes(i)
         coornew(1:3,inodp) = coorbnnew(1:3,i)
      end do

!     --- Fill array markcurv

      call mshmarkcurv ( markcurv, kmeshs, isurfsl(0), kmeshm, &
                         coornew, coor )

900   deallocate ( coorbn, anorm, coorbnnew, psi, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'coorbn', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  deallocate ( isurfar, isurfsl, nodes, &
                   norm, weight, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'isurfar', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl30'

      end if  ! ( debug )
      call erclos ( 'mshcompdispl30' )

      end
