      subroutine mshcoppart ( kmeshnew, coornew, iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of kmesh to kmeshnew
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm2, kmeshnew
      double precision coornew(2,*)

!     coornew        o    Array containing the new coordinates of the nodes
!     iseqkm2        o    Sequence number with respect to km of second mesh
!     kmeshnew      i/o   New array kmesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseqkm1
      double precision dummy(1)

!     dummy          Dummy array
!     i              Counting variable
!     iseqkm1        Sequence number with respect to km of first mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     MSHCOP1        Copy part of kmesh1 to kmesh2 (a,b,f,g)
!     MSHCOP2        Copy part of kmesh1 to kmesh2 (coor, c,k,l,m,n)
!     MSHCOP3        Copy part of kmesh1 to kmesh2 (q,r,s,iinput,rinput,
!                    kmesht, kmeshquant)
!     SEPCRMESHINFO  Create permanent space for mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcoppart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcoppart'
         call ertrace
         write(irefwr,1) 'iseqkm, lastmesh', iseqkm, lastmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

      call sepcrmeshinfo ( kmeshnew )
      iseqkm2 = kmeshnew
      km(iseqkm2)%kmesh(4:14) = km(iseqkm1)%kmesh(4:14)
      km(iseqkm2)%kmesh(7) = 0
      km(iseqkm2)%kmesh(42) = km(iseqkm1)%kmesh(42)
      km(iseqkm2)%kmesh(44) = km(iseqkm1)%kmesh(44)
      km(iseqkm2)%kmesh(64) = km(iseqkm1)%kmesh(64)

      call mshcop1 ( iseqkm1, iseqkm2 )
      call mshcop2 ( iseqkm1, iseqkm2 )

      if ( kelmp>0 ) then
         if ( associated(km(iseqkm2)%elsize) ) deallocate(km(iseqkm2)%elsize)
         length = size(km(iseqkm1)%elsize,2)
         allocate ( km(iseqkm2)%elsize(2,length) )
         km(iseqkm2)%elsize = km(iseqkm1)%elsize
      end if  ! ( kelmp>0 )

!     --- Copy kmeshq, kmeshr, kmeshs iinput, rinput, kmesht

      call mshcop3 ( iseqkm1, iseqkm2 )

      if ( kelmz>0 ) then

!     --- kmesh part z

         if ( associated(km(iseqkm2)%kmeshz) ) deallocate(km(iseqkm2)%kmeshz)
         allocate ( km(iseqkm2)%kmeshz(ncurvs) )
         km(iseqkm2)%kmeshz = kmeshz

      end if

      if ( kelmu>0 ) then

!     --- kmesh part u

         if ( associated(km(iseqkm2)%kmeshu) ) deallocate(km(iseqkm2)%kmeshu)
         allocate ( km(iseqkm2)%kmeshu(5) )
         km(iseqkm2)%kmeshu = kmeshu

      end if

      if ( kelmw>0 ) then

!     --- kmesh part w

         if ( associated(km(iseqkm2)%kmeshw) ) deallocate(km(iseqkm2)%kmeshw)
         allocate ( km(iseqkm2)%kmeshw(4) )
         km(iseqkm2)%kmeshw = kmeshw

      end if

!     --- Copy coordinates from coor to coornew

      if ( debug ) write(irefwr,1) 'ndim, npoint', ndim, npoint
      call dcopy ( ndim*npoint, km(iseqkm1)%coor, 1, coornew, 1 )

      km(iseqkm2)%kmesh(47) = -km(iseqkm1)%kmesh(47)
      km(iseqkm2)%kmesh(68) = -km(iseqkm1)%kmesh(68)
      km(iseqkm2)%kmeshx => km(iseqkm1)%kmeshx
      km(iseqkm2)%kmeshxsur => km(iseqkm1)%kmeshxsur
      km(iseqkm2)%rmeshx => km(iseqkm1)%rmeshx

      if ( kelmag>0 ) then
         if ( associated(km(iseqkm2)%volume_surfs) ) &
            deallocate(km(iseqkm2)%volume_surfs)
         length = size(km(iseqkm1)%volume_surfs)
         allocate ( km(iseqkm2)%volume_surfs(length) )
         km(iseqkm2)%volume_surfs = volume_surfs
      end if  ! ( kelmag>0 )

      if ( kelmak>0 ) then
         if ( associated(km(iseqkm2)%curveusrp) ) &
            deallocate(km(iseqkm2)%curveusrp)
         length = size(km(iseqkm1)%curveusrp)
         allocate ( km(iseqkm2)%curveusrp(length) )
         km(iseqkm2)%curveusrp = km(iseqkm1)%curveusrp
      end if  ! ( kelmak>0 )

      if ( associated(km(iseqkm1)%icurvsmsh) ) then
         if ( associated(km(iseqkm2)%icurvsmsh) ) &
            deallocate(km(iseqkm2)%icurvsmsh)
         length = size(km(iseqkm1)%icurvsmsh)
         allocate ( km(iseqkm2)%icurvsmsh(length) )
         km(iseqkm2)%icurvsmsh = km(iseqkm1)%icurvsmsh
      end if  ! ( associated(km(iseqkm1)%icurvsmsh) )

      if ( associated(km(iseqkm1)%curvesmsh) ) then
         if ( associated(km(iseqkm2)%curvesmsh) ) &
            deallocate(km(iseqkm2)%curvesmsh)
         length = size(km(iseqkm1)%curvesmsh,2)
         allocate ( km(iseqkm2)%curvesmsh(ndim,length) )
         km(iseqkm2)%curvesmsh = km(iseqkm1)%curvesmsh
      end if  ! ( associated(km(iseqkm1)%curvesmsh) )

      if ( associated(km(iseqkm1)%intprop) ) then
         allocate ( km(iseqkm2)%intprop(size(km(iseqkm1)%intprop,1),nelgrp) )
         km(iseqkm2)%intprop = km(iseqkm1)%intprop
      end if  ! ( associated(km(iseqkm1)%intprop) )
      if ( associated(km(iseqkm1)%realprop) ) then
         allocate ( km(iseqkm2)%realprop(size(km(iseqkm1)%realprop,1),nelgrp) )
         km(iseqkm2)%realprop = km(iseqkm1)%realprop
      end if  ! ( associated(km(iseqkm1)%realprop) )

      iseqkm = iseqkm1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcoppart'

      end if  ! ( debug )
      call erclos ( 'mshcoppart' )

      end
