      subroutine mshcopsol2d ( isol3d, isolt2, iwork, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution from3d mesh onto 2d mesh given the node numbers
!     of the surfaces to which the solution must be copied
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     surface
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol3d, isolt2, iwork(*), npoint

!     isol3d         i    Standard SEPRAN array containing information of the
!                         3d velocity vector stored as solution vector
!     isolt2        i/o   Standard SEPRAN array containing information of the
!                         2d velocity vector stored as solution vector
!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     npoint         i    Number of nodal points in 3d mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: kproblc

!     kproblc        Contains information of the problem definition
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOPSOL2D1   Copy solution from3d mesh onto 2d mesh given the node
!                    numbers of the surfaces to which the solution
!                    must be copied
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOL110  Activate or create arrays of type 110
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopsol2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopsol2d'

      end if
      if ( ierror/=0 ) go to 1000

      kproblc => kprob2d(maxmesh+1)
      iseqkp = kproblc
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc

!     --- Activate arrays
!         First activate solution array and corresponding arrays

      call sepactsolbf2 ( isol3d )
      if ( kprobloc(19)>0 ) &

!     --- Create new array

      call sepcreasol110 ( isolt2, 'isolt2', ks(isol3d)%compl )
      usol => usoltemp
      usoltemp => ks(isol3d)%sol

!     --- Copy solution

      call mshcopsol2d1 ( usol, usoltemp, npoint, iwork )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcopsol2d'

      end if
      call erclos ( 'mshcopsol2d' )

      end subroutine mshcopsol2d
