      subroutine mshcopybouncr ( kbound, nbound, coor, coorsav, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy boundary coordinates into temporary array coorsav
!     By using coorsav instead of coor back vopying is carried out
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbound(*), nbound, inpelm
      double precision coor(3,*), coorsav(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorsav        o    array of length 12*nbound containing the coordinates
!                         of the boundary nodes
!     inpelm         i    number of nodes per boundary element
!     kbound         i    Array of lenth inpelm*nbound containing the node
!                         numbers of the boundary elements
!     nbound         i    Number of essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iwork(4*nbound), k, inode, npointmax

!     i              Counting variable
!     inode          Node sequence number
!     iwork          integer work array of length 4*nbound to store the
!                    nodes that have already been treated (iwork(i) = 1)
!     k              Counting variable
!     npointmax      Largst node number allowed in kbound (used for dimensions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear iwork
!     For all nodes inode in kbound do
!        if ( iwork(inode)=0 ) then
!           Copy coordinates
!           iwork(inode) := 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopybouncr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopybouncr'

      end if
      if ( ierror.ne.0 ) go to 1000
      npointmax = 4*nbound

!     --- Set array iwork to 0

      do i = 1, npointmax
         iwork(i) = 0
      end do  ! i = 1, npointmax

!     --- For all nodes in kbound do
!           Copy coordinates and set iwork is 1 if node is used

      do i = 1, inpelm*nbound
         inode = kbound(i)                           ! node number
         if ( inode.gt.npointmax ) then

!        --- Length of arrays coorsav and iwork too small

            call errint ( inode, 1 )
            call errint ( npointmax, 2 )
            call errchr ( 'coorsav/iwork', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         else if ( iwork(inode).eq.0 ) then

!        --- Node has not been used yet, copy coordinates

            do k = 1, 3
               coorsav(k,inode) = coor(k,inode)
            end do  ! k = 1, 3
            iwork(inode) = 1             ! set iwork for this node

         end if  ! ( inode.gt.3*nbound )

      end do  ! i = 1, 6*nbound

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcopybouncr'

      end if
      call erclos ( 'mshcopybouncr' )

      end
