      subroutine mshcrobstelm3d ( kmeshcnew, fluidnd, ielem, coornew )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 06-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Determine coupling elements and store and store the information
!    in kmeshcnew
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     fluid_structure
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kmeshcnew(*), fluidnd(7,*), ielem
      double precision coornew(*)

!     coornew        i    Array containing the coordinates of the nodes
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     ielem          i    Element number
!     kmeshcnew     i/o   KMESH part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, iobst, inode(7), jnode(9), &
              isurnr, ishapelam, ishape_curve, istart, igauss, &
              inpelmsurf, nelmsurf, inpelm, ngauss
      double precision xi, xgauss(9), philam(3,3), w(3), phi(9), eta
      logical found

!     eta            ?
!     found          If true already a possibility has been found
!     i              Counting variable
!     igauss         Counter for loop over lagrange multipliers
!     inode          Node sequence number
!     inpelm         number of points per element
!     inpelmsurf     number of points per surface element
!     iobst          Obstacle sequence number
!     ishape_curve   Shape of fluid basis functions
!     ishapelam      Shape of lambda basis functions
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     jnode          Array containing the node numbers of an element
!     k              Counting variable
!     nelmsurf       ?
!     ngauss         ?
!     phi            Array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     philam         Shape functions of lagrange multipliers
!     w              array of length m containing the weights for
!                    integration
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!     xi             Local coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMFICTLAM     Compute the integration points and basis functions for the
!                    Lagrangian multipliers
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
!     PRFINDELM2OBST Determine local coordinates of xgauss on curved line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- do loop over obstacles or boundary elements (not always the same)

      call eropen('mshcrobstelm3d')

      debug=.false.
      inpelm = 7

!     --- clear array

      do i=1, inpelm
         inode(i)=0
      end do

      do iobst = 1, numobst

!     --- iobst := obstacle number

         isurnr = kmeshx(2,iobst)

         istart=0
         do j=1,isurnr-1

!        --- number of elements in surface

            inpelmsurf=kmeshn(j)
            nelmsurf=kmeshn(nsurfs+j)
            istart=istart+inpelmsurf*nelmsurf

         end do

         inpelmsurf=kmeshn(isurnr)
         nelmsurf=kmeshn(nsurfs+isurnr)
         if (debug) write(*,*)'number of elements and nodes', &
                               'in surfaces  ', &
                               nelmsurf, inpelmsurf

!        --- find fluidnodes corresponding to element

         do i=1,inpelm
            inode(i)=fluidnd(i,ielem)
            kmeshcnew(i)=inode(i)
            if (debug) write(*,*)'fluid nodes  ',inode(i), &
                                              coornew((inode(i)-1)*ndim+1), &
                                              coornew((inode(i)-1)*ndim+2), &
                                              coornew((inode(i)-1)*ndim+3)
         end do

!        --- ishape of fluid

         ishape_curve=3

!        --- ishape of lagrange multiplier

         ishapelam=6

         call elmfictlam ( coornew, inode, ishape_curve, ndim, xgauss, &
                           philam, w, ishapelam, .false. )

         if (ishapelam.eq.6) ngauss=3

!        --- loop over lambda's

         do igauss=1, ngauss
            found=.false.

!           --- loop over obstacle elements in curve

            do i = 1, nelmsurf
               if (.not.found) then

!              --- find node numbers

                  do j=1,inpelmsurf
                     jnode(j)=kmeshn(2*nsurfs+istart+inpelmsurf*(i-1)+j)
                  end do

!                 --- print the obstacle nodes and the gauss coordinates

                  if (debug) write(*,*)'element', i, igauss
                  if (debug) write(*,*)'structure node ', &
                                        (jnode(k), k=1,inpelmsurf)
                  if (debug) write(*,*)'xgauss',igauss, &
                                        xgauss(igauss), &
                                        xgauss(ngauss+igauss), &
                                        xgauss(2*ngauss+igauss)

!                 --- find obstacle node in element

                  call prfindelm2obst ( coornew, jnode, xgauss, igauss, &
                                        xi, eta, found, phi )

                  if (debug) write(*,*)'local coordinate  ', &
                                                 found, xi, eta

               end if

            end do !end loop over obstacle elements

            if (found) then
               do i=1,inpelmsurf
                  kmeshcnew(i+(igauss-1)*inpelmsurf+inpelm)=jnode(i)
               end do
            else
                write(*,*)'no solid element found for fluid element'
                call instop
            end if

         end do !end loop over lagrange multipliers

         if (debug) then
            write(*,*)'coupling element ', ielem
            write(*,*)'fluid nodes  ', &
            (inode(i),i=1,inpelm)
            write(*,*)'structure nodes (elm1) ', &
            (kmeshcnew(i+inpelm), i=1,inpelmsurf)
            write(*,*)'structure nodes (elm2)   ', &
            (kmeshcnew(i+inpelmsurf+inpelm), i=1,inpelmsurf)
            write(*,*)'structure nodes (elm3)   ', &
            (kmeshcnew(i+2*inpelmsurf+inpelm), i=1,inpelmsurf)
         end if

      end do !end obstacle loop

      call erclos('mshcrobstelm3d')
      end
