      subroutine mshcrossline2 ( x1, x2, x3, x4, fact1, fact2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find the intersection of the lines x1-x2 and x3-x4
!    The result is the parameter fact, where the intersection point is defined
!    as x1 + fact*(x2-x1)
!    If no intersection point is found fact gets the value -infinity
!    Two-dimensions only
!    The difference with mshcrossline is that it is not checked if fact1 between
!    0 and 1
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     intersection
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(2), x2(2), x3(2), x4(2), fact1, fact2

!     fact1          o    Defines the factor for the intersection point
!                         The intersection point on the face is defined by
!                         x1 + fact1 * (x2-x1)
!     fact2          o    Defines the factor for the intersection point
!                         The intersection point on the face is defined by
!                         x3 + fact2 * (x4-x3)
!     x1             i    Contains the coordinates of the point x1
!     x2             i    Contains the coordinates of the point x2
!     x3             i    Contains the coordinates of the point x3
!     x4             i    Contains the coordinates of the point x4
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision det

!     det            determinant of system of equations to be solved
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The line through x1-x2 is defined as
!     x = x1 + fact1 * (x2-x1)
!     The line through x3-x4 is defined as
!     x = x3 + fact2 * (x4-x3)
!     If an intersection is present then
!        x1 + fact * (x2-x1) = x3 + alpha1 * (x4-x3)
!        Hence:
!        [ (x2-x1) (x4-x3) ] [fact -alpha1]^T = x3-x1
!        This system can be solved if det( [ (x2-x1) (x4-x3) ] ) # 0
!        The solution is:
!
!        [fact1 -fact2]^T =  [ (x2-x1) (x4-x3) ]^(-1) (x3-x1)
!
!        or:
!
!        det = (x2-x1)*(y4-y3)-(x4-x3)*(y2-y1)
!        fact1 = ((y4-y3)*(x3-x1)+(x3-x4)*(y3-y1))/det
!        fact2 = -((y1-y2)*(x3-x1)+(x2-x1)*(y3-y1))/det
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      det = (x2(1)-x1(1))*(x4(2)-x3(2))-(x4(1)-x3(1))*(x2(2)-x1(2))
      if ( abs(det).le.1d-50 ) then
         fact1 = -rinfin
         fact2 = -rinfin
      else
         fact1 = ((x4(2)-x3(2))*(x3(1)-x1(1))+ &
                 (x3(1)-x4(1))*(x3(2)-x1(2)))/det
         fact2 = -((x1(2)-x2(2))*(x3(1)-x1(1))+ &
                   (x2(1)-x1(1))*(x3(2)-x1(2)))/det
      end if

      end
