      subroutine mshelemref1d ( nextranodes, elementnode, &
                                elementnodenw, edgenode, &
                                ikelmcnew, ielemnew )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 15-03-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a 1d element if necessary into two subelements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     intersection
!     linear
!     element
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nextranodes, elementnode(*), elementnodenw(*), &
              edgenode(*), ikelmcnew, ielemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     elementnode    i    Array of nodal point numbers of the corners of
!                         the original element.
!     elementnodenw  o    Array of nodal point numbers of the corners of
!                         the new subelements
!     ielemnew      i/o   Sequence number of element in new mesh
!     ikelmcnew     i/o   Starting address of next element in kmeshcnew
!     nextranodes    i    Number of extra nodes in element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshelemref1d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshelemref1d'
         write(irefwr,100) 'nextranodes', nextranodes
100      format ( a, 1x, (10i6) )
         call prinin ( elementnode, 3, 'elementnode' )

      end if
      if ( ierror/=0 ) go to 1000
      if ( nextranodes==0 ) then

!     --- nextranodes=0
!         copy element

         do i = 1, 2
            elementnodenw(i) = elementnode(i)
         end do

      else

!     --- nextranodes=1
!         subdivide element into 2 elements

          elementnodenw(1) = elementnode(1)
          elementnodenw(2) = edgenode(3)
          elementnodenw(3) = edgenode(3)
          elementnodenw(4) = elementnode(2)

      end if

      ikelmcnew = ikelmcnew+(nextranodes+1)*2
      ielemnew = ielemnew+nextranodes

1000  call erclos ( 'mshelemref1d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshelemref1d'

      end if

      end
