      subroutine mshextra
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read extra input for mesh and apply the actions required by this extra
!     input
!     At this moment it concerns  the definition of outer boundaries
!     corresponding to element groups as well as subdivision of elements in
!     a special way
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     outer_boundary
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leninpsep
      integer, allocatable :: inpsep(:)

!     inpsep         The integer input read is stored in array INPSEP
!                    Organization of inpsep:
!                    Pos. 1 number of main keywords (presently 2)
!                    Pos. 2 nseq1: Number of input parts defining
!                           outer boundaries
!                    Pos. 3 istart1: Starting address part outer boundaries
!                    Pos. 4 nseq2: Number of input parts defining
!                           subdivisions
!                    Pos. 5 istart2: Starting address part subdivisions
!                    From istart1
!                    Pos. 1-nseq1: starting addresses for each
!                         outer boundary
!                    From these starting addresses:
!                    - Number of element groups used for this group
!                      followed by element group sequence numbers
!                    From istart2
!                    Pos. 1-nseq2: starting addresses for each
!                         subdivision
!                    From these starting addresses:
!                    - Type of subdivision (1=nodes_on_outer_boundary)
!                    - Sequence number outer boundary
!     leninpsep      Length of array inpsep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMAKEOUTER   Compute outer boundary curves for 2d mesh
!                    and store the curve numbers in kmesh part AL
!     READMESHEXTRA  Read input found following the keyword EXTRA_MESH_INPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read input
!     Define extra outer boundaries and store in kmesh part al
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshextra' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshextra'

      end if
      if ( ierror/=0 ) go to 1000
      leninpsep = 2*nelgrp**2+1+2*nelgrp

!     --- First define sufficient space at the end of ibuffr

      allocate ( inpsep(leninpsep), stat = error )
      if ( error/=0 ) call eralloc ( error, leninpsep, 'inpsep' )
      if ( ierror/=0 ) go to 1000

!     --- Read input

      call readmeshextra ( nelgrp, inpsep, leninpsep )

      if ( inpsep(2)>0 ) then

!     --- iinput(2) > 0, hence there are extra outer boundaries
!         Define extra outer boundaries and store in kmesh part al

         call mshmakeouter ( inpsep )

      end if  ! ( inpsep(2))>0 )

!     --- Release buffer space

      deallocate ( inpsep, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpsep' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshextra'

      end if
      call erclos ( 'mshextra' )

      end
