      subroutine mshfillkmesh2d ( kmeshlc, iwork, nnodes, isurfaces, &
                                  nsurfaces, normallc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh for a temporary 2d mesh given a 3d surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlc, iwork(*), &
              nnodes, isurfaces(*), nsurfaces
      double precision normallc(3)

!     isurfaces      i    Array containing the surfaces from which information
!                         must be found
!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     kmeshlc        o    Standard SEPRAN array containing information of the
!                         2d mesh
!     nnodes         i    Number of nodes
!     normallc       i    normal vector (normal to plane)
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ishapelc, nelemlc, npelmlc
      double precision dummy(1)

!     dummy          Dummy parameter
!     i              Counting variable
!     ishapelc       local shape number
!     nelemlc        Local number of elements
!     npelmlc        Local number of nodes per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFILLCOORKMC Fill array kmeshc and coor for a three-dimensional
!                    surface mesh mapped onto a two-dimesional mesh
!     MSHFILLKMESHM  Fill array kmeshm for a two dimensional mesh where only
!                    kmeshc is given i.e. the topology is known
!     MSHSURFINFO    Get information concerning a number of surfaces stored
!                    in array isurfaces
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array kmeshlc, except first position
!     Fill part of kmeshlc
!     Create space for arrays coor and kmeshc
!     Fill arrays coor and kmeshc
!     Create array kmeshm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmesh2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmesh2d'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- First initialize kmeshlc and kprobloc

      call sepcrmeshinfo ( kmeshlc )

!     --- Fill part of kmeshlc

      km(iseqkm)%kmesh(5) = 1               ! nelgrp
      km(iseqkm)%kmesh(20) = 1              ! kmeshlcf
      km(iseqkm)%kmesh(36) = 1              ! kmeshlct
      km(iseqkm)%kmesh(44) = 0              ! no subparts of mesh

      km(iseqkm)%kmesh(6) = 2

!     --- Find number of elements in surface as well as shape number
!         of elements

      call mshsurfinfo ( nsurfaces, isurfaces, ishapelc, nelemlc, npelmlc )
      if ( debug ) &
         write(irefwr,100) 'nsurfaces, ishapelc, nelemlc, npelmlc', &
                            nsurfaces, ishapelc, nelemlc, npelmlc

!     --- Put information into kmeshlc

      km(iseqkm)%kmesh(4) = npelmlc
      km(iseqkm)%kmesh(8) = nnodes
      km(iseqkm)%kmesh(9) = nelemlc
      allocate ( km(iseqkm)%kmeshf(1), km(iseqkm)%kmesht(1,1), &
                 km(iseqkm)%coor(2,nnodes), km(iseqkm)%kmeshc(npelm*nelem) )
      km(iseqkm)%kmeshf(1) = ishapelc
      km(iseqkm)%kmesht(1,1) = nelemlc

      call sepfillkmesh(kmeshlc)

!     --- Fill arrays coor and kmeshc

      call mshfillcoorkmc ( kmeshc, isurfaces, nsurfaces, &
                            coor, normallc, iwork )

!     --- Create array kmeshm

      call mshfillkmeshm

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillkmesh2d'

      end if
      call erclos ( 'mshfillkmesh2d' )

      end
