      subroutine mshfillshape ( iinput, iseqsurfs, ipointsurf, nsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill shape numbers of surfaces that have not been filled yet
!     It concerns surfaces that are a copy of another surface
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iseqsurfs(*), ipointsurf(*), nsurfs

!     iinput        i/o   integer input array
!                         It concerns the part corresponding to surfaces
!                         only
!                         At output shape numbers 0, are replaced by the
!                         actual shape numbers
!     ipointsurf     i    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     iseqsurfs      i    Contains new sequence of surface numbers
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, iinp5, isurf, ishape

!     i              Counting variable
!     iinp5          first pos. iinput definition surfaces
!     ishape         Shape number of elements
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFILLSHAPE1  Fill shape numbers of a surface that has not been
!                    filled yet
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillshape' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillshape'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nsurfs
         isurnr = iseqsurfs(i)
         iinp5 = ipointsurf(isurnr)
         isurf = iinput(iinp5)
         if ( debug ) &
            write(irefwr,1) 'i, isurnr, isurf', i, isurnr,  isurf
         if ( isurf==0 ) then
            iinp5 = iinp5 + 1
            go to 400
         end if
         ishape = iinput(iinp5+1)

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'iinp5, isurf, ishape', iinp5, isurf, ishape

         end if  ! ( debug )

         if ( ishape==0 ) then

!        --- ishape = 0   either error or ishape must be copied

            call mshfillshape1 ( iinput, ipointsurf, isurf, iinp5, &
                                 isurnr )

         end if

400   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillshape'

      end if  ! ( debug )
      call erclos ( 'mshfillshape' )

      end subroutine mshfillshape
