      subroutine mshfillshape1 ( iinput, ipointsurf, isurf, iinp5, &
                                 isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 28-01-2009 correction shape = 0
!        version  1.1  date 20-10-2008 Allow isurf = 24
!        version  1.0  date 30-03-2007
!
!   copyright (c) 2007-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill shape numbers of a surface that has not been filled yet
!     It concerns surfaces that are a copy of another surface
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ipointsurf(*), isurf, iinp5, isurnr

!     iinp5          i    first pos. iinput definition surface
!     iinput        i/o   integer input array
!                         It concerns the part corresponding to surfaces
!                         only
!                         At output shape numbers 0, are replaced by the
!                         actual shape numbers
!     ipointsurf     i    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     isurf          i    Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishape, ndefsf, iprec, kinp, nrow, nstart, j, jshape

!     iprec          Surface number of surface to be copied etc.
!     ishape         Shape number of elements
!     j              Counting variable
!     jshape         Shape number of elements
!     kinp           Starting address of prior surface in iinput
!     ndefsf         Number of curves in surface
!     nrow           Number of rows of surfaces in ordered surface of
!                    surfaces
!     nstart         Position in iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     137   For this type of surface, a shape number zero is not permitted
!     563   Surface number in surface of surfaces is incorrect
!     564   Not all surfaces in surface of surfaces have the same element shapes
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillshape1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillshape1'
         write(irefwr,1) 'isurf, isurnr', isurf, isurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isurf<7 .or. isurf==10 .or. isurf==13 .or. isurf>24 ) then

!     --- isurf # 8, 9, 11, 14 or 16 : error

         call errint ( isurf, 1 )
         call errint ( isurnr, 2 )
         call errsub ( 137, 2, 0, 0 )

      else if (  isurf== 8 .or. isurf== 9 .or. &
                 isurf==14 .or. isurf==16 .or. isurf==21 ) then

!     --- isurf = 8, 9, 14 or 16 (rotate, translate, reflect
!                   or similar)
!         copy from preceding surface

         ndefsf = mod(iinput(iinp5+2),10000)
         iprec = iinput(iinp5+3+ndefsf)
         kinp = ipointsurf(iprec)
         ishape = iinput(kinp+1)
         if ( debug ) &
            write(irefwr,1) 'ndefsf, iprec, ishape', &
                             ndefsf, iprec, ishape
         iinput(iinp5+1) = ishape

      else if ( isurf==7 .or. isurf==11 .or. isurf==12 .or. &
                isurf==24 ) then

!     --- isurf = 7, 11 or 24  surface of surfaces

         ndefsf = mod(iinput(iinp5+2),10000)
         nstart = iinp5 + 2
         if ( isurf==11 ) then

!        --- isurf = 11   ordened surface

            nrow = ndefsf
            ndefsf = 0
            do j = 1, nrow
               ndefsf = ndefsf + iinput(nstart+j)
            end do
            nstart = nstart + nrow

         end if

!        --- Check surfaces

         ishape = 0
         do j = 1, ndefsf
            iprec = abs ( iinput(nstart+j) )
            if ( iprec<=0 ) then
               call errint ( iprec, 1 )
               call errint ( isurnr, 2 )
               call errsub ( 563, 2, 0, 0 )
            end if
            kinp = ipointsurf(iprec)
            isurf = iinput(kinp)
            if ( debug ) &
               write(irefwr,1) 'j, iprec, ishape, isurf', &
                                j, iprec, ishape, isurf
            if ( ishape==0 .and. isurf/=0 ) then

               ishape = iinput(kinp+1)
               iinput(iinp5+1) = ishape

            else if ( isurf/=0 ) then

               jshape = iinput(kinp+1)
               if ( jshape>0 .and. ishape/=jshape ) then
                  call errint ( isurnr, 1 )
                  call errint ( ishape, 2 )
                  call errint ( j, 3 )
                  call errint ( iprec, 4 )
                  call errint ( jshape, 5 )
                  call errsub ( 564, 5, 0, 0 )
               end if

            end if

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillshape1'

      end if  ! ( debug )
      call erclos ( 'mshfillshape1' )

      end
