      subroutine mshmarkedgpar1 ( istore, istorerec, nstore, icount, &
                                  iedges, nsubdiv, refedges, lenrefedges, &
                                  nodenumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare both sets of nodes on common interface of two blocks
!     and if there are differences mark extra edges
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     refine
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nstore, icount, istore(2,nstore), istorerec(2,icount), &
              iedges(*), nsubdiv, refedges(*), &
              lenrefedges, nodenumbers(*)

!     icount         i    Length of second dimension of array istorerec
!                         This is the number of edges at the interface that
!                         are intersected in the neigbor block
!     iedges        i/o   Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     istore         i    Work array of length (2,nsubdiv) to store the local
!                         node numbers of the edges at the interface that are
!                         intersected
!     istorerec      i    Work array of length (2,nsubdiv) to store the local
!                         node numbers of the edges at the interface that are
!                         intersected in the neighbor block
!     lenrefedges    o    Computed length of array refedges
!     nodenumbers    i    Array containing the local node numbers of the
!                         interface as stored in transinfo
!     nstore         i    number of edges in istore
!     nsubdiv        i    Number of edges that are subdivided, hence number
!                         of extra nodes to be created
!     refedges      i/o   Contains the edge numbers of the edges that
!                         must be subdivided
!                         Length: nsubdiv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iedge, indstore, &
              indstorerec, inode1, inode2
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iedge          edge sequence number
!     indstore       position in array istore
!     indstorerec    position in array istorerec
!     inode1         First node in edge
!     inode2         Second node in edge
!     iwork          integer work array of length nedges to be used by iniedge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     CHSORT2DARR    Sort a 2-d array in increasing sequence
!                    First first entry, next second entry (if first are equal)
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( icount==1 and istorerec(1,1)==0 ) then
!        no action
!     else if ( nstore==1 and istore(1,1)==0 ) then
!        mark all edges stored in istorerec
!     else
!        Sort istore and istorerec
!        Compare both arrays
!        mark edges in istorerec not available in istore
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkedgpar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkedgpar1'
         write(irefwr,1) 'nstore, icount', nstore, icount
         call prinin1 ( istore, nstore, 2, 'istore' )
         call prinin1 ( istorerec, icount, 2, 'istorerec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenrefedges = 0

      allocate ( iwork(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork = 0
      if ( nstore==1 .and. istore(1,1)==0 ) then

!     --- nstore==1 and istore(1,1)==0
!         mark all edges stored in istorerec

         if ( icount<=1 .and. istorerec(1,1)==0 ) then

!        --- No edges to be subdivided

            lenrefedges = 0

         else

!        --- mark edges

            do i = 1, icount

               inode1 = nodenumbers(istorerec(1,i))
               inode2 = nodenumbers(istorerec(2,i))
               iedge = iniedge ( inode1, inode2, nmeshl, iwork )  ! edge number
               nsubdiv = nsubdiv+1            ! raise nsubdiv
               iedges(iedge) = nsubdiv+npoint ! mark edge
               lenrefedges = lenrefedges+1
               refedges(lenrefedges) = iedge
               if ( debug ) &
                  write(irefwr,1) 'iedge, nsubdiv', iedge, nsubdiv

            end do  ! i = 1, icount

         end if  ! ( icount<=1 .and. istorerec(1,1)==0 )

         if ( debug ) write(irefwr,1) 'lenrefedges/1', lenrefedges

      else if ( icount<=1 .and. istorerec(1,1)==0 ) then

!     --- nstorerec==1 and istorerec(1,1)==0
!         no action

         do i = 1, nstore

            inode1 = nodenumbers(istore(1,i))
            inode2 = nodenumbers(istore(2,i))
            iedge = iniedge ( inode1, inode2, nmeshl, iwork )  ! edge number
            lenrefedges = lenrefedges+1
            refedges(lenrefedges) = iedge
            if ( debug ) &
               write(irefwr,1) 'iedge, nsubdiv', iedge, nsubdiv

         end do  ! i = 1, nstore

         if ( debug ) write(irefwr,1) 'lenrefedges/2', lenrefedges

      else

!     --- Sort istore and istorerec

         call chsort2darr ( istore, nstore, 2 )
         call chsort2darr ( istorerec, icount, 2 )

!        --- Compare both arrays
!            mark edges in istorerec not available in istore

         indstore = 1
         indstorerec = 1

         do while ( indstorerec<=icount )

            if ( debug ) then
               write(irefwr,1) 'indstorerec, icount', &
                                indstorerec, icount
               write(irefwr,1) 'istore(1,indstore)', istore(1,indstore)
               write(irefwr,1) 'istorerec(1,indstorerec)', &
                                istorerec(1,indstorerec)
            end if  ! ( debug )

            if ( istore(1,indstore)==istorerec(1,indstorerec) ) then

!           --- First entries are equal, compare second entries

               if ( debug ) then
                  write(irefwr,1) 'istore(2,indstore)', &
                                   istore(2,indstore)
                  write(irefwr,1) 'istorerec(2,indstorerec)', &
                                   istorerec(2,indstorerec)
               end if  ! ( debug )

               if ( istore(2,indstore)==istorerec(2,indstorerec) ) then

!              --- Both entries are equal, skip

                  inode1 = nodenumbers(istore(1,indstore))
                  inode2 = nodenumbers(istore(2,indstore))
                  iedge = iniedge ( inode1, inode2, nmeshl, iwork )
                  lenrefedges = lenrefedges+1
                  refedges(lenrefedges) = iedge
                  indstore = indstore+1
                  indstorerec = indstorerec+1

               else if ( istore(2,indstore)>istorerec(2,indstorerec) ) &
                  then

!              --- Second entry of istore larger than the one in istorerec
!                  New edge found

                  inode1 = nodenumbers(istorerec(1,indstorerec))
                  inode2 = nodenumbers(istorerec(2,indstorerec))
                  iedge = iniedge ( inode1, inode2, nmeshl, iwork )
                  lenrefedges = lenrefedges+1
                  refedges(lenrefedges) = iedge
                  nsubdiv = nsubdiv+1            ! raise nsubdiv
                  iedges(iedge) = nsubdiv+npoint ! mark edge
                  if ( debug ) &
                     write(irefwr,1) 'iedge, nsubdiv', iedge, nsubdiv
                  indstorerec = indstorerec+1

               else

!              --- Second entry of istore smaller than the one in istorerec

                  inode1 = nodenumbers(istore(1,indstore))
                  inode2 = nodenumbers(istore(2,indstore))
                  iedge = iniedge ( inode1, inode2, nmeshl, iwork )
                  lenrefedges = lenrefedges+1
                  refedges(lenrefedges) = iedge
                  indstore = indstore+1

               end if  ! ( istore(2,indstore)==istorerec(2,indstorerec) )

            else if ( istore(1,indstore)>istorerec(1,indstorerec) ) then

!           --- First entry of istore larger than the one in istorerec
!               New edge found

               inode1 = nodenumbers(istorerec(1,indstorerec))
               inode2 = nodenumbers(istorerec(2,indstorerec))
               iedge = iniedge ( inode1, inode2, nmeshl, iwork )
               nsubdiv = nsubdiv+1            ! raise nsubdiv
               iedges(iedge) = nsubdiv+npoint ! mark edge
               lenrefedges = lenrefedges+1
               refedges(lenrefedges) = iedge
               if ( debug ) &
                  write(irefwr,1) 'iedge, nsubdiv', iedge, nsubdiv
               indstorerec = indstorerec+1

            else

!           --- First entry of istore smaller than the one in istorerec

               inode1 = nodenumbers(istore(1,indstore))
               inode2 = nodenumbers(istore(2,indstore))
               iedge = iniedge ( inode1, inode2, nmeshl, iwork )
               lenrefedges = lenrefedges+1
               refedges(lenrefedges) = iedge
               indstore = indstore+1

            end if  ! ( istore(1,indstore)==istorerec(1,indstorerec) )
            if ( debug ) write(irefwr,1) 'lenrefedges/2', lenrefedges

         end do  ! while ( indstorerec<=icount )

      end if  ! ( nstore==1 .and. istore(1,1)==0 ) then

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshmarkedgpar1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( refedges, lenrefedges, 'refedges' )
         write(irefwr,*) 'End mshmarkedgpar1'

      end if  ! ( debug )

      end
