      subroutine mshmprsurel ( iinput, infel, jnew, indsur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.2  date 17-11-2008 Correction in case nbndpt=0
!        version  1.1  date 21-03-2007 Extra debug
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array infel and raise kmeshg for surface elements
!     in case surface elements are connected to an element group
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*), infel(*), indsur

!     iinput         i    integer input array for subroutine mesh
!     indsur        i/o   Position in array infel
!     infel         i/o   array containing some information of how the elements
!                         created are stored on file 3
!     jnew           i    indication whether the mesh is new (true) or old
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, i, ielgrp, ifirst, &
              ilast, jinp, itype, index, &
              isubsf, isursf, nsubsf, kinp, jsurnr, jsursf

!     i              Counting variable
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     index          Help parameter to store a value
!     isubsf         Sequence number of subsurface
!     isursf         Surface sequence number
!     itype          Type number of element
!     jinp           position in array iinput
!     jsurnr         Surface sequence number
!     jsursf         Surface sequence number with sign
!     kinp           position in array iinput
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     MSH018         compute position of surface in array iinput
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmprsurel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmprsurel'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iinp = iinp4

      do i = 1, nsurel

         ielgrp = iinput(iinp+1)
         ifirst = iinput(iinp+2)
         ilast = iinput(iinp+3)
         iinp = iinp+4
         if ( kelma/=0 ) inpelm = kmesha(ielgrp+1) - kmesha(ielgrp)
         inelem = 0
         if ( debug ) &
            write(irefwr,1) 'i, ifirst, ilast', i, ifirst, ilast

         do jsurnr = ifirst, ilast

            isurnr = jsurnr
            if ( debug ) write(irefwr,1) 'jsurnr', jsurnr
            if ( kmeshs(nsurfs+1+8*(isurnr-1)+1)==20 ) then

!           --- Surface is of type sphere, use the surface from which it must
!               be copied

               isurnr = kmeshs(nsurfs+1+8*(isurnr-1)+3)

            else if ( kmeshs(nsurfs+1+8*(isurnr-1)+1)==0 ) then

!           --- Empty surface, skip

               jinp = jinp+1
               go to 150

            end if

            infel(indsur) = isurnr
            indsur = indsur+1

            if ( jnew) then

!           --- jnew = true   new mesh

               call msh018 ( iinput(iinp3), isurnr, 0, jinp )
               itype  = iinput(jinp+iinp3)
               inelem  = kmeshh(2,isurnr+1)

               if ( nvolms==0 ) then

!              --- nvolms = 0, fill array KMESH part t with surfaces

                  kmesht(1,isurnr) = iselem
                  kmesht(2,isurnr) = iselem+inelem-1
                  kmesht(1,jsurnr) = iselem
                  kmesht(2,jsurnr) = iselem+inelem-1
                  if ( debug ) &
                     write(irefwr,1) 'isurnr, jsurnr, iselem, inelem', &
                                      isurnr, jsurnr, iselem, inelem

               end if

               index = kmeshh(4,isurnr+1)

               if ( index<=0 ) then

!              --- index<0, hence surface has not been treated yet

                  if ( itype==7 .or. itype==11 .or. itype==12 ) then

!                 --- itype = 7, 11 or 12  surface of subsurfaces
!                     Perform a loop over the subsurfaces

                     nsubsf = kmeshr(isurnr+1) - kmeshr(isurnr)
                     do isubsf = 1, nsubsf
                        jsursf = kmeshr(kmeshr(isurnr)+isubsf)
                        isursf = abs(jsursf)
                        call msh018 ( iinput(iinp3), isursf, 0, kinp )
                        itype  = iinput(kinp+iinp3)
                        index = kmeshh(4,isursf+1)

                        if ( index<0 ) then

!                       --- index<0, hence surface has not been treated yet

                           index = - index
                           kmeshh(4,isursf+1) = index

                        end if

                     end do

                  else

!                 --- itype # 7, 11 or 12  Standard surface

                     index = - index
                     kmeshh(4,isurnr+1) = index

                  end if

                  if ( nelgrp>1 ) kmeshg(ielgrp) = kmeshg(ielgrp)+inelem
                  iselem = iselem+inelem

               end if

            end if

150      end do ! jsurnr = ifirst, ilast

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmprsurel'

      end if
      call erclos ( 'mshmprsurel' )

      end
