      subroutine mshmvcoorusp1 ( rinput, distance, ndim, userpoint, &
                                 numusp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Move coordinates of user point to new position
!     Only array rinput is adapted
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     user_point
!     change_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, userpoint(*), numusp
      double precision rinput(ndim,*), distance(ndim)

!     distance       i    distance that point has to be moved
!     ndim           i    Dimension of the space.
!     numusp         i    Contains number of nodes between origin and interface
!     rinput        i/o   Real input array for subroutine mesh
!     userpoint      i    Contains the user point numbers along the
!                         axis sorted in ascending order from position 2
!                         userpoint(1) contains the interface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision frac(ndim)

!     frac           Fraction that node must be moved
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmvcoorusp1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmvcoorusp1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rinput(1:ndim,userpoint(1)) = rinput(1:ndim,userpoint(1))+distance

      frac = distance/(numusp+1)  ! fraction to be moved

      do i = 1, numusp
         rinput(1:ndim,userpoint(i+1)) = rinput(1:ndim,userpoint(i+1)) + &
                                         i*frac
      end do  ! i = 1, numusp

1000  call erclos ( 'mshmvcoorusp1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmvcoorusp1'

      end if  ! ( debug )

      end
