      subroutine mshnormalsurf ( kmeshn, nsurfs, isurnr, coor, normal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal onto the first element of surface isurnr
!     The surface must be stored in kmeshn
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmeshn(*), nsurfs, isurnr
      double precision, intent(in) :: coor(3,*)
      double precision, intent(out) :: normal(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     isurnr         i    Surface sequence number
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!     normal         o    Infor entry with respect to the array containing the
!                         normal vectors along the boundaries
!                         This array is only used in case of curves
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inelem, inpelm, istart

!                    they are not
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     PRINRL         Print 1d real vector
!     SEPNORMALSURF  Compute the normal onto the first element of surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find first element of surface
!     t1 := first tangential vector
!     t2 := second tangential vector
!     n := t1 x t2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnormalsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnormalsurf'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address of surface as well as
!         number of nodes

      call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                        isurnr )

!     --- Compute normal

      call  sepnormalsurf ( kmeshn(istart+1), coor, normal, isurnr, inpelm )

1000  call erclos ( 'mshnormalsurf' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( normal, 3, 'normal' )
         write(irefwr,*) 'End mshnormalsurf'

      end if

      end
