      subroutine mshobstms1d02 ( iwork, nelem, numobst, icrossobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear array iwork (1:nelem) and set iwork(i) = starting address in
!     iobstcross if element i intersects obstacle iobst
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, iwork(nelem,2), numobst, icrossobst(*)

!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array)
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iwork          o    integer work array of length nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!                         iwork(ielem,2) contains the corresponding obstacle
!                         sequence number iobst
!     nelem          i    Number of elements
!     numobst        i    number of obstacles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, istarticross, numintersect, istart, ielem

!     i              Counting variable
!     ielem          Element number
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     numintersect   Number of intersections
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ2        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2734   Element is intersected by at least two obstacles
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms1d02' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms1d02'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- For iobst := 1 (1) numobst do

      do iobst = 1, numobst

         istarticross = icrossobst(iobst)        ! starting address of obstacle
         numintersect = icrossobst(istarticross) ! number of intersections
         istart = istarticross                 ! starting address of elements

         if ( debug ) &
            write(irefwr,1) 'iobst, istarticross, numintersect, istart', &
                             iobst, istarticross, numintersect, istart

!        --- For all elements ielem in icrossobst(iobst) do
!               iwork(ielem) := starting address

          do i = 1, numintersect
             ielem = icrossobst(istart+1)        ! element number
             if ( debug ) write(irefwr,1) 'i, ielem', i, ielem
             if ( iwork(ielem,2)/=0 ) then

!            --- Element in more than one obstacle (not permitted)

                call errint ( ielem, 1 )
                call errint ( iwork(ielem,2), 2 )
                call errint ( iobst, 3 )
                call errsub ( 2734, 3, 0, 0 )

             end if  ! ( iwork(ielem,2)/=0 )
             iwork(ielem,1) = istart+1           ! starting address element
             iwork(ielem,2) = icrossobst(istart+2)  ! address in rcrossobst
             istart = istart+2
          end do  ! i = 1, numelms

      end do  ! iobst = 1, numobst

1000  call erclos ( 'mshobstms1d02' )
      if ( debug ) then

!     --- Debug information

         call prinin2 ( iwork, nelem, 2, 'iwork' )
         write(irefwr,*) 'End mshobstms1d02'

      end if  ! ( debug )

      end
