      subroutine mshobstms1d03 ( coornew, kmeshcnew, npointnew, &
                                 iinput, map, ipointobstnew, ielemobstnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make temporary elements corresponding to new 1d mesh permanent
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointnew, &
              kmeshcnew(*), iinput(*), ipointobstnew(*), ielemobstnew(*)
      double precision coornew(1,*)
      type (mapping) :: map

!     coornew        i    Array containing the new coordinates of the nodes
!     ielemobstnew   i    New array ielemobst
!     iinput         i    integer input array
!     ipointobstnew  i    New array ipointobst
!     kmeshcnew      i    New kmesh part c
!     map           i/o   contains information of the mapping
!     npointnew      i    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
!     PRINRL         Print 1d real vector
!     SEPCLEAR       clear array map
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms1d03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms1d03'
         write(irefwr,1) 'npointnew', npointnew
         call prinin ( iinput, 3, 'iinput' )
         call prinin2 ( imap, npointnew, 3, 'imap' )
         call prinrl ( coornew, npointnew, 'coornew' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- coor

      if ( km(iseqkm)%kmesh(23)>0 ) deallocate(km(iseqkm)%coor)
      allocate(km(iseqkm)%coor(1,npointnew))
      coor => km(iseqkm)%coor
      coor = coornew(:,1:npointnew)
      km(iseqkm)%kmesh(23) = npointnew

!     --- kmeshc

      if ( km(iseqkm)%kmesh(17)>0 ) deallocate(km(iseqkm)%kmeshc)
      length = iinput(1)
      allocate(km(iseqkm)%kmeshc(length))
      kmeshc => km(iseqkm)%kmeshc
      kmeshc = kmeshcnew(1:length)
      km(iseqkm)%kmesh(17) = length
      call sepfillkmeshc

!     --- imap, rmap

      if ( associated(map%imap) ) deallocate ( map%imap, map%rmap )
      call sepclear ( map )
      allocate ( map%imap(npointnew,3), map%rmap(npointnew,2) )
      map%imap = imap(1:npointnew,1:3)
      map%rmap = rmap(1:npointnew,1:2)

!     --- ipointobstnew

      if ( associated(km(iseqkm)%ipointobst) ) &
         deallocate(km(iseqkm)%ipointobst)
      length = iinput(2)
      allocate(km(iseqkm)%ipointobst(length) )
      km(iseqkm)%ipointobst = ipointobstnew(1:length)
      km(iseqkm)%kmeshad(1) = length
      ipointobst => km(iseqkm)%ipointobst

!     --- ielemobstnew

      if ( associated(km(iseqkm)%ielemobst) ) &
         deallocate(km(iseqkm)%ielemobst)
      length = iinput(3)
      allocate(km(iseqkm)%ielemobst(length) )
      km(iseqkm)%ielemobst = ielemobstnew(1:length)
      km(iseqkm)%kmeshad(2) = length
      ielemobst => km(iseqkm)%ielemobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms1d03'

      end if  ! ( debug )
      call erclos ( 'mshobstms1d03' )

      end
