      subroutine mshobstms23d ( meshorig, meshobst, maploc, shft, rinputusr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 05-11-2013 Extra parameter rinputusr
!        version  2.0  date 19-09-2013 New body
!        version  1.4  date 05-05-2013 Delete kmesh part ad
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements, &
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     2d and 3d case without shift
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: meshorig, meshobst, shft
      double precision rinputusr(*)
      type (mapping) :: maploc

!     maploc         o    contains information of the mappin
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
!     rinputusr      i    See subroutine mshobstms
!     shft           i    Parameter that determines if nodes must be shifted (1)
!                         or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointnew, lenkmsmnewsv, lenkmsnnewsv, npointlc, &
              lenkmscnew, nelemnew, lenkmsnnew, &
              iinput(20), iseqkm1, iseqkm2, &
              mmnmeshk, lencoornew, &
              lenipointobst, lenkmsmnew, lenielemobst
      double precision volold(3), volumes(3), voltot, voltmp
      integer, pointer :: kmeshcnew(:), kmeshnnew(:), &
                              ipointobstnew(:), kmeshmnew(:), &
                              ielemobstnew(:), imaplc(:,:)
      double precision, pointer :: coornew(:), rmaplc(:,:)
      integer, pointer :: kmeshnew

!     coornew        Array containing the new coordinates of the nodes
!     ielemobstnew   New array ielemobst
!     iinput         integer array to pass information from mshobstms01
!                    Contents
!                     1:  length of new array kmeshc
!                     2:  length of new array ipointobst
!                     3:  length of new array ielemobst
!                     4:  npointnew
!                     5:  shft
!                     6:  linear
!                     7:  lenkmsmnew
!                     8:  lenkmsnnew
!                     9:  numbnelmobst
!                    10:  numcross
!                    11:  nelemnew
!                    12:  lenkmscnew
!                    13:  lenrmap
!                    14:  lencoornew
!                    15:  lenipointobst
!                    16:  lenimap
!                    17:  lenielemobst
!     ipointobstnew  New array ipointobst
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnew       New array kmesh
!     kmeshnnew      New array kmeshn
!     lencoornew     Estimated length of array coornew
!     lenielemobst   Computed length of array ielemobst
!     lenipointobst  length of array ipointobst
!     lenkmscnew     Length of array kmshcnew
!     lenkmsmnew     Length of array kmshmnew
!     lenkmsmnewsv   Help parameter to store original value of lenkmsmnew
!     lenkmsnnew     Length of array kmshnnew
!     lenkmsnnewsv   Help parameter to store original value of kmshnnew
!     mmnmeshk       memory management sequence number of array nmeshk
!     nelemnew       Number of elements in new mesh
!     npointlc       Local number of nodes
!     npointnew      Sequence number of last generated node
!     volold         Old computed value of volume of region
!     voltmp         Temporary computed value of volume of region
!     voltot         Total volume of region
!     volumes        Array of length 3 containing the volume of the
!                    following subparts:
!                    1: all elements completely inside the obstacle
!                    2: all elements partly inside the obstacle
!                    3: all elements outside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     INICOPINT2D    Copy two-dimensional integer array
!     INICOPREAL2D   Copy two-dimensional real array
!     MSHDELKMESHAD  Delete kmesh part ad
!     MSHOBST01      Compute some parameters for subroutine mshobstms
!     MSHOBST2D      Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (2d)
!     MSHOBST3D      Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (3d)
!     MSHOBSTBOUN2D  Check if there are elements that have all nodes on the
!                    obstacle or on the obstacle and an outer boundary
!                    If so create new elements and nodes(2D case)
!     MSHOBSTBOUN3D  Check if there are elements that have all nodes on the
!                    obstacle or on the obstacle and an outer boundary
!                    If so create new elements and nodes (3D case)
!     MSHOBSTMARKBT  Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
!     MSHOBSTMS04    Put control integers around the temporary arrays in
!                    mshobstsms
!     MSHOBSTMS06    Remove some subarrays of kmeshnew that are no longer valid
!                    Recompute topology
!     MSHOBSTMS07    Make new arrays permanent for subroutines mshobstms
!                    and refineloc
!     MSHOBSTMS10    Check the control integers of the work arrays defined
!                    in MSHOBSTMESH
!     MSHOBSTPARM    Estimate number of nodes and elements in new mesh
!     MSHOBSTVOL3D   Compute volume of part of region inside, outside and
!                    near obstacle
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     PRIN3DOBST     Write the coordinates and topology of the obstacle to a
!                    file in openview format
!     PRININ         print 1d integer array
!     PRINMAP1       Actual printing of array imap
!     SEPCOPIBUF     Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents
!                    of iarray to ibuffr
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPPLOTINTERS  plot and print intersection points
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2268   The volume of the original tetrahedron is not equal to the sum of
!           the volumes of the new tetrahedron
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Reserve temporary space
!     Compute starting addresses
!     Create new mesh
!     Make new arrays permanent
!     Compute some extra arrays by topol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms23d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms23d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug .and. ndim==3 ) then

!     --- debug true, compute volumes of parts of mesh, store in array volold
!         and print

         call mshobstvol3d ( volold )
         voltot = volold(1)+volold(2)+volold(3)
         write(irefwr,100) volold, voltot
100      format ( 'Before subdivision:'/ &
                  'Volume of elements inside obstacle  ', d17.8/ &
                  'Volume of elements near obstacle    ', d17.8/ &
                  'Volume of elements outside obstacle ', d17.8/ &
                  'Volume of complete region           ', d17.8 )

      end if  ! ( debug .and. ndim==3 )

      if ( debug ) call sepplotinters

      kmesh => kmesh2d(meshorig)
      iseqkm1 = kmesh
      kmeshnew => kmesh2d(meshobst)
      iseqkm2 = kmeshnew

!     --- Compute some parameters

      call mshobst01 ( iinput )

!     --- Compute length of temporary arrays and estimate npointnew

      if ( debug ) write(irefwr,*) 'before mshobstparm'
      call mshobstparm ( iinput )

      iinput(5) = shft
      npointnew = iinput(4)
      lenkmsmnew = iinput(7)
      lenkmsnnew = iinput(8)
      lenkmscnew = iinput(12)
      lencoornew = iinput(14)
      lenipointobst = iinput(15)
      lenielemobst = iinput(17)

      lenkmsmnewsv = lenkmsmnew
      lenkmsnnewsv = lenkmsnnew

!     --- Reserve space for temporary arrays

      allocate ( coornew(0:lencoornew+1), rmaplc(npointnew,2), &
                 kmeshcnew(0:lenkmscnew+1), kmeshnnew(0:lenkmsnnew+1), &
                 ipointobstnew(0:lenipointobst+1), &
                 ielemobstnew(0:lenielemobst+1), &
                 imaplc(npointnew,3), kmeshmnew(0:lenkmsmnew), &
                 stat = error )
      length = lencoornew+lenkmscnew+lenkmsnnew+npointnew*5+ &
               lenipointobst+lenielemobst+lenkmsmnew+12*2
      if ( error/=0 ) call eralloc ( error, length, 'coornew' )
      if ( ierror/=0 ) go to 1000

!     --- Put control integers around these arrays

      call mshobstms04 ( kmeshcnew, kmeshnnew, &
                         ipointobstnew, ielemobstnew, &
                         kmeshmnew, coornew, iinput )

!     --- Delete kmeshnew part ad if it exists

      iseqkm = iseqkm2
      call mshdelkmeshad
      iseqkm = iseqkm1
      imap => imaplc
      rmap => rmaplc

!     --- Create new mesh

      if ( ndim==2 ) then

!     --- ndim = 2

         call mshobst2d ( kmeshnew, coornew(1), ielemobstnew(1), &
                          ipointobstnew(1), kmeshcnew(1), kmeshmnew(1), &
                          iinput, rinputusr )

      else

!     --- ndim = 3

         call mshobst3d ( kmeshnew, coornew(1), ielemobstnew(1), &
                          ipointobstnew(1), kmeshcnew(1), kmeshmnew(1), &
                          kmeshnnew(1), iinput, rinputusr )

      end if  ! ( ndim==2 )

      if ( ierror/=0 ) go to 1000

!     --- Copy contents of imap and rmap to final positions

      npointlc = iinput(4)
      npointnew = km(iseqkm2)%kmesh(8)
      if ( ierror/=0 ) go to 1000

!     --- Check control integers

      iinput(7) = lenkmsmnewsv
      iinput(8) = lenkmsnnewsv

      call mshobstms10 ( kmeshcnew, kmeshnnew, &
                         ipointobstnew, ielemobstnew, &
                         kmeshmnew, coornew, iinput )
      if ( ierror/=0 ) go to 1000

!     --- Make new arrays permanent

      nelemnew = km(iseqkm2)%kmesh(9)
      call mshobstms07 ( kmeshnew, coornew(1), kmeshcnew(1), &
                         kmeshnnew(1), kmeshmnew(1), iinput )
      if ( ierror/=0 ) go to 1000

!     --- Fill part 4 of ipointsobst of nodes on the boundary of the
!         obstacle which are only in elements completely inside
!         the obstacle

      kmesh => kmeshnew
      call sepfillkmesh ( kmesh )
      call sepfillkmeshd
      call mshobstmarkbt ( ipointobstnew(1), nelemnew, &
                           npointnew, kmshd1, kmshd2, &
                           kmeshcnew(1), iinput(2) )
      if ( ierror/=0 ) go to 1000

!     --- Make arrays permanent and copy to these permanent positions

      if ( associated(km(iseqkm)%ipointobst) ) deallocate(km(iseqkm)%ipointobst)
      allocate(km(iseqkm)%ipointobst(iinput(2)))
      ipointobst => km(iseqkm)%ipointobst
      ipointobst = ipointobstnew(1:iinput(2))
      km(iseqkm)%kmeshad(1) = iinput(2)
      if ( associated(km(iseqkm)%ielemobst) ) deallocate(km(iseqkm)%ielemobst)
      allocate(km(iseqkm)%ielemobst(iinput(3)))
      ielemobst => km(iseqkm)%ielemobst
      ielemobst = ielemobstnew(1:iinput(3))
      km(iseqkm)%kmeshad(2) = iinput(3)
      if ( associated(maploc%imap) ) deallocate ( maploc%imap, maploc%rmap )
      allocate ( maploc%imap(npointnew,3) )
      maploc%imap = imaplc(1:npointnew,1:3)
      allocate(maploc%rmap(npointnew,2))
      maploc%rmap = rmaplc(1:npointnew,1:2)
      maploc%iseqkm1 = iseqkm1
      maploc%iseqkm2 = iseqkm2
      if ( debug ) then
         call sepprinipobst ( ipointobst )
         call sepprinielobst ( ielemobst )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug .and. ndim==3 ) then

!     --- debug true, compute volumes of parts of mesh, store in array volold
!         and print

         call mshobstvol3d ( volumes )
         voltmp = volumes(1)+volumes(2)+volumes(3)
         if ( abs(voltot-voltmp)>1d-13*abs(voltot) ) then

!        --- Error volume after subsdivision incorrect

            call erreal ( voltot, 1 )
            call erreal ( voltmp, 2 )
            call errsub ( 2268, 0, 2, 0 )

         end if
         write(irefwr,200) volumes, voltmp
200      format ( 'After subdivision:'/ &
                  'Volume of elements inside obstacle  ', d17.8/ &
                  'Volume of elements near obstacle    ', d17.8/ &
                  'Volume of elements outside obstacle ', d17.8/ &
                  'Volume of complete region           ', d17.8 )

      end if

!     --- Remove some subarrays of kmeshnew that are no longer valid
!         Recompute topology

      call mshobstms06 ( meshorig, meshobst )
      if ( debug ) write(irefwr,*) 'after mshobstms06'
      kmesh => kmeshnew
      call sepfillkmesh ( kmesh )

      if ( debug .or. plotobst .or. printobst ) then

!     --- debug = true, plot nodes and elements with a color
!         First print arrays

         call plotobstacle ( debug, 0 )

         if ( debug ) then

!        --- debug true, print map

            call prinmap1 ( npointnew )

         end if

      end if

!     --- Check if some elements in the new mesh must be refined in order
!         to prevent the possibility to have elements with all nodes on
!         the obstacle and/or the boundary
!         If this is the case the mesh is refined

      if ( shft==0 ) then

!     --- Nodes must not be shifted

         if ( ndim==2 ) then

!        --- ndim = 2

            call mshobstboun2d ( maploc )

         else

!        --- ndim = 3

            call mshobstboun3d ( maploc, voltot )

         end if

      end if

      if ( write_geomview ) call prin3dobst

!     --- Remove temporary arrays

      deallocate ( kmeshcnew, kmeshnnew, ipointobstnew, &
                   ielemobstnew, imaplc, coornew, rmaplc, &
                   kmeshmnew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coornew' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms23d'

      end if  ! ( debug )
      call erclos ( 'mshobstms23d' )

      end subroutine mshobstms23d
