      subroutine mshobstms3dq6d ( tetnodet, tetnodenew, facenodet, &
                                  centroid,  edgenodet, nelemnew )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 15-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with four crossed edges which form a closed
!     curve.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnodet(15), tetnodenew(15,*), edgenodet(2,6), &
              facenodet(4,4), centroid(*), nelemnew

!     centroid       i    Array containing the centroid numbers of the
!                         subtetrahedrons
!     edgenodet      i    Array of global nodal point numbers of point on edge i
!                         after transformation
!     facenodet      i    Array containing the centroids of the faces of
!                         the subtetrahedrons
!     nelemnew       o    Number of newly created subtetrahedrons
!     tetnodenew     o    New kmesh part c
!                         In this case only the new elements are stored
!     tetnodet       i    Array of global nodal point numbers of the corners of
!                         the transformed original tetrahedron
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielm(5,4)

!     i              Counting variable
!     ielm           ?
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ielm /1, 2, 3, 4, 0, &
                 5, 6, 7, 8, 9, &
                 4, 7, 8, 9, 11, &
                 1, 5, 6, 9, 10/
! ======================================================================
!
      call eropen ( 'mshobstms3dq6d' )

      if ( tetnodet(2).lt.tetnodet(8).and. &
           tetnodet(2).lt.tetnodet(6) ) i=1
      if ( tetnodet(8).lt.tetnodet(2).and. &
           tetnodet(6).lt.tetnodet(2) ) i=2
      if ( tetnodet(8).lt.tetnodet(2).and. &
           tetnodet(2).lt.tetnodet(6) ) i=3
      if ( tetnodet(2).lt.tetnodet(8).and. &
           tetnodet(6).lt.tetnodet(2) ) i=4

      j=1
      if (ielm(j,i).eq.1) then
         tetnodenew(1,j) = tetnodet(1)
         tetnodenew(2,j) = edgenodet(1,1)
         tetnodenew(3,j) = tetnodet(2)
         tetnodenew(4,j) = tetnodet(11)
         tetnodenew(5,j) = tetnodet(6)
         tetnodenew(6,j) = edgenodet(1,3)
         tetnodenew(7,j) = tetnodet(7)
         tetnodenew(8,j) = facenodet(4,2)
         tetnodenew(9,j) = tetnodet(14)
         tetnodenew(10,j) = tetnodet(10)
         tetnodenew(11,j) = facenodet(1,1)
         tetnodenew(12,j) = facenodet(3,2)
         if (i.eq.1) then
            tetnodenew(13,j) = centroid(5)
         else
            tetnodenew(13,j) = centroid(6)
         end if
         tetnodenew(14,j) = facenodet(1,4)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.2) then
         tetnodenew(1,j) = tetnodet(2)
         tetnodenew(2,j) = facenodet(4,1)
         tetnodenew(3,j) = tetnodet(5)
         tetnodenew(4,j) = edgenodet(2,3)
         tetnodenew(5,j) = tetnodet(6)
         tetnodenew(6,j) = tetnodet(11)
         tetnodenew(7,j) = facenodet(4,2)
         tetnodenew(8,j) = tetnodet(9)
         tetnodenew(9,j) = tetnodet(14)
         tetnodenew(10,j) = tetnodet(10)
         tetnodenew(11,j) = facenodet(2,1)
         tetnodenew(12,j) = tetnodet(15)
         tetnodenew(13,j) = facenodet(2,4)
         tetnodenew(14,j) = centroid(5)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.3) then
         tetnodenew(1,j) = tetnodet(2)
         tetnodenew(2,j) = facenodet(4,2)
         tetnodenew(3,j) = tetnodet(10)
         tetnodenew(4,j) = edgenodet(2,5)
         tetnodenew(5,j) = tetnodet(8)
         tetnodenew(6,j) = tetnodet(12)
         tetnodenew(7,j) = facenodet(4,1)
         tetnodenew(8,j) = tetnodet(9)
         tetnodenew(9,j) = tetnodet(13)
         tetnodenew(10,j) = tetnodet(5)
         tetnodenew(11,j) = facenodet(2,2)
         tetnodenew(12,j) = tetnodet(15)
         tetnodenew(13,j) = facenodet(2,3)
         tetnodenew(14,j) = centroid(6)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.4) then
         tetnodenew(1,j) = tetnodet(2)
         tetnodenew(2,j) = edgenodet(2,1)
         tetnodenew(3,j) = tetnodet(3)
         tetnodenew(4,j) = tetnodet(4)
         tetnodenew(5,j) = tetnodet(5)
         tetnodenew(6,j) = facenodet(4,1)
         tetnodenew(7,j) = tetnodet(12)
         tetnodenew(8,j) = edgenodet(1,5)
         tetnodenew(9,j) = tetnodet(13)
         tetnodenew(10,j) = tetnodet(8)
         tetnodenew(11,j) = facenodet(3,1)
         tetnodenew(12,j) = facenodet(1,2)
         tetnodenew(13,j) = facenodet(1,3)
         if (i.eq.1) then
            tetnodenew(14,j) = centroid(6)
         else
            tetnodenew(14,j) = centroid(7)
         end if
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.5) then
         tetnodenew(1,j) = tetnodet(2)
         tetnodenew(2,j) = edgenodet(2,1)
         tetnodenew(3,j) = tetnodet(3)
         tetnodenew(4,j) = facenodet(4,1)
         tetnodenew(5,j) = tetnodet(6)
         tetnodenew(6,j) = tetnodet(11)
         tetnodenew(7,j) = tetnodet(12)
         tetnodenew(8,j) = edgenodet(1,5)
         tetnodenew(9,j) = tetnodet(15)
         tetnodenew(10,j) = tetnodet(8)
         tetnodenew(11,j) = facenodet(2,1)
         tetnodenew(12,j) = facenodet(1,2)
         tetnodenew(13,j) = centroid(7)
         tetnodenew(14,j) = centroid(8)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.6) then
         tetnodenew(1,j) = tetnodet(6)
         tetnodenew(2,j) = facenodet(4,1)
         tetnodenew(3,j) = tetnodet(3)
         tetnodenew(4,j) = tetnodet(4)
         tetnodenew(5,j) = tetnodet(5)
         tetnodenew(6,j) = edgenodet(2,3)
         tetnodenew(7,j) = tetnodet(15)
         tetnodenew(8,j) = edgenodet(1,5)
         tetnodenew(9,j) = tetnodet(13)
         tetnodenew(10,j) = tetnodet(8)
         tetnodenew(11,j) = facenodet(3,1)
         tetnodenew(12,j) = centroid(7)
         tetnodenew(13,j) = facenodet(1,3)
         tetnodenew(14,j) = centroid(9)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.7) then
         tetnodenew(1,j) = tetnodet(1)
         tetnodenew(2,j) = edgenodet(1,1)
         tetnodenew(3,j) = tetnodet(2)
         tetnodenew(4,j) = tetnodet(11)
         tetnodenew(5,j) = tetnodet(6)
         tetnodenew(6,j) = edgenodet(1,3)
         tetnodenew(7,j) = facenodet(4,2)
         tetnodenew(8,j) = tetnodet(12)
         tetnodenew(9,j) = tetnodet(15)
         tetnodenew(10,j) = tetnodet(8)
         tetnodenew(11,j) = facenodet(1,1)
         tetnodenew(12,j) = facenodet(2,2)
         tetnodenew(13,j) = centroid(8)
         tetnodenew(14,j) = centroid(6)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.8) then
         tetnodenew(1,j) = tetnodet(1)
         tetnodenew(2,j) = tetnodet(7)
         tetnodenew(3,j) = tetnodet(10)
         tetnodenew(4,j) = edgenodet(2,5)
         tetnodenew(5,j) = tetnodet(8)
         tetnodenew(6,j) = facenodet(4,2)
         tetnodenew(7,j) = edgenodet(1,3)
         tetnodenew(8,j) = tetnodet(14)
         tetnodenew(9,j) = tetnodet(15)
         tetnodenew(10,j) = tetnodet(6)
         tetnodenew(11,j) = facenodet(3,2)
         tetnodenew(12,j) = facenodet(1,4)
         tetnodenew(13,j) = centroid(10)
         tetnodenew(14,j) = centroid(6)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.9) then
         tetnodenew(1,j) = tetnodet(8)
         tetnodenew(2,j) = tetnodet(15)
         tetnodenew(3,j) = tetnodet(6)
         tetnodenew(4,j) = tetnodet(14)
         tetnodenew(5,j) = tetnodet(10)
         tetnodenew(6,j) = edgenodet(2,5)
         tetnodenew(7,j) = tetnodet(13)
         tetnodenew(8,j) = edgenodet(2,3)
         tetnodenew(9,j) = tetnodet(9)
         tetnodenew(10,j) = tetnodet(5)
         tetnodenew(11,j) = centroid(10)
         tetnodenew(12,j) = centroid(9)
         tetnodenew(13,j) = facenodet(2,4)
         tetnodenew(14,j) = facenodet(2,3)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.10) then
         tetnodenew(1,j) = tetnodet(2)
         tetnodenew(2,j) = tetnodet(11)
         tetnodenew(3,j) = tetnodet(6)
         tetnodenew(4,j) = tetnodet(14)
         tetnodenew(5,j) = tetnodet(10)
         tetnodenew(6,j) = facenodet(4,2)
         tetnodenew(7,j) = tetnodet(12)
         tetnodenew(8,j) = tetnodet(15)
         tetnodenew(9,j) = edgenodet(2,5)
         tetnodenew(10,j) = tetnodet(8)
         tetnodenew(11,j) = centroid(6)
         tetnodenew(12,j) = centroid(8)
         tetnodenew(13,j) = centroid(10)
         tetnodenew(14,j) = facenodet(2,2)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      if (ielm(j,i).eq.11) then
         tetnodenew(1,j) = tetnodet(2)
         tetnodenew(2,j) = facenodet(4,1)
         tetnodenew(3,j) = tetnodet(5)
         tetnodenew(4,j) = edgenodet(2,3)
         tetnodenew(5,j) = tetnodet(6)
         tetnodenew(6,j) = tetnodet(11)
         tetnodenew(7,j) = tetnodet(12)
         tetnodenew(8,j) = tetnodet(13)
         tetnodenew(9,j) = tetnodet(15)
         tetnodenew(10,j) = tetnodet(8)
         tetnodenew(11,j) = facenodet(2,1)
         tetnodenew(12,j) = centroid(7)
         tetnodenew(13,j) = centroid(9)
         tetnodenew(14,j) = centroid(8)
         tetnodenew(15,j) = centroid(j)
         j=j+1
      end if

      nelemnew=j-1

      call erclos ( 'mshobstms3dq6d' )
      end
