      subroutine mshobstms3dq7b ( tetnode, tetnodenew, edgenode, &
                                  ledgenode, nelemnew, centroid, &
                                  facenode, midfacenodes, fluidnd  )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 06-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with four crossed edges which form a closed
!     curve.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     quadratic
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(15),tetnodenew(15,6), edgenode(5,4), centroid(*), &
              ledgenode(2,4), nelemnew, midfacenodes(4), facenode(4,4), &
              fluidnd(*)

!     centroid       i    ?
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     facenode       i    ?
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     midfacenodes   i    ?
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, trans(4), transi(4),transtemp, transposj, &
              tetnodet(15), edgenodet(2,6), &
              lowedgenode, transtot(15,12), itran, idx(6), facetrans(4), &
              facenodet(4,4)
      integer pedgenodet(4,4)
      logical oddpairintchan, transpos2, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     facenodet      ?
!     facetrans      ?
!     i              Counting variable
!     idx            ?
!     itran          ?
!     j              Counting variable
!     lowedgenode    The number of the intersection node with the lowest local
!                    edge node number
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transpos2      transpos2 = .true. if there is an edge node on local edge 2
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
!     transtot       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS3DQ7C Subdivide a tetrahedron with four crossed edges which form
!                    a closed curve
!     MSHOBSTMS3DQ7D Subdivide a tetrahedron with four crossed edges which form
!                    a closed curve
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine intersection point with lowest node index
!
!     Define transformation to move this node to local edge position 1.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     If intersection points at positions 1,3,5,6 transform tetrahedron such
!     that they get into positions 1,2,4,6
!
!     Split tetrahedron into six subtetrahedrons, taking into account the
!     determinant of the transformation.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/

!     trans: transtot
!      1234: 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
!      2314: 3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15
!      3124: 5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15
!      1423: 1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15
!      2143: 3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15
!      4213: 10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15
!      1342: 1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15
!      3412: 5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15
!      4132: 10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15
!      2431: 3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15
!      3241: 5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15
!      4321: 10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15
      data transtot / 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, &
                      3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15, &
                      5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15, &
                      1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15, &
                      3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15, &
                      10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15, &
                      1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15, &
                      5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15, &
                      10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15, &
                      3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15, &
                      5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15, &
                      10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15/
      data idx / 1,3,1,1,3,10 /
! ======================================================================
!
      call eropen ( 'mshobstms3dq7b' )

      debug=.true.

      if (debug) write(*,*)'mshobstms3dq7b'

!     --- Search for the intersection node with the lowest node index

      lowedgenode = 1
      if( edgenode(4,2).lt.edgenode(4,lowedgenode)) lowedgenode = 2
      if( edgenode(4,3).lt.edgenode(4,lowedgenode)) lowedgenode = 3
      if( edgenode(4,4).lt.edgenode(4,lowedgenode)) lowedgenode = 4

!     --- Move the intersection node with the lowest node index
!         to local intersection point 1:
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node ledgenode(1,lowedgenode) to node 1 and
!              node ledgenode(2,lowedgenode) to node 2

!         -- move node ledgenode(1,lowedgenode) to node 1

      if( ledgenode(1,lowedgenode).ne.trans(1) ) then
         transtemp = trans(1)
         trans(1) = ledgenode(1,lowedgenode)
         trans( ledgenode(1,lowedgenode)) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- move node ledgenode(2,lowedgenode) to node 2

      if (ledgenode(2,lowedgenode).ne.trans(2) ) then
         transtemp = trans(2)
         trans(2) = ledgenode(2,lowedgenode)
         trans(trans(ledgenode(2,lowedgenode))) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

      if( oddpairintchan ) then
         transtemp = trans(3)
         trans(3) = trans(4)
         trans(4) = transtemp
      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

      if (trans(4).eq.4) then
         if (trans(1).eq.1) itran=1
         if (trans(1).eq.2) itran=2
         if (trans(1).eq.3) itran=3
      else if (trans(4).eq.3) then
         if (trans(1).eq.1) itran=4
         if (trans(1).eq.2) itran=5
         if (trans(1).eq.4) itran=6
      else if (trans(4).eq.2) then
         if (trans(1).eq.1) itran=7
         if (trans(1).eq.3) itran=8
         if (trans(1).eq.4) itran=9
      else if (trans(4).eq.1) then
         if (trans(1).eq.2) itran=10
         if (trans(1).eq.3) itran=11
         if (trans(1).eq.4) itran=12
      end if

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,15
         tetnodet(i) = tetnode(transtot(i, itran))
         if (debug) write(*,*)'nodes of element', tetnodet(i)
      end do

!     --- Move the intersection points to their transformed positions

      transpos2 = .false.
      do j = 1,4
         transposj = pedgenodet(transi(ledgenode(1,j)), &
                            transi(ledgenode(2,j)))
         if( transposj.eq.2) transpos2 = .true.
         if (edgenode(1,j).eq.tetnodet(idx(transposj))) then
            edgenodet(1,transposj) = edgenode(3,j)
            edgenodet(2,transposj) = edgenode(5,j)
         else
            edgenodet(1,transposj) = edgenode(5,j)
            edgenodet(2,transposj) = edgenode(3,j)
         end if
         if (debug) write(*,*)'edgenodet', transposj, &
                                edgenodet(1,transposj), &
                                edgenodet(2,transposj)
      end do

      do j=1,4
         do i=1,4
            if (midfacenodes(i).eq.tetnodet(10+j)) facetrans(j)=i
         end do
      end do

!     --- Transform faces

      do j=1,4
         do i=1,4
            facenodet(i,j)=facenode(i,facetrans(j))
            if (debug) write(*,*)'facenodet', i, j, facenodet(i,j)
         end do
      end do

      if ( transpos2 ) then
         if (debug) write(*,*)'mshobstms3dq7c'
         call mshobstms3dq7c ( tetnodet, tetnodenew, facenodet, &
                               centroid,  edgenodet )

         fluidnd(1)=tetnodet(7)
         fluidnd(2)=tetnodet(12)
         fluidnd(3)=tetnodet(2)
         fluidnd(4)=tetnodet(15)
         fluidnd(5)=tetnodet(9)
         fluidnd(6)=tetnodet(14)
         fluidnd(7)=centroid(7)

         fluidnd(8)=tetnodet(2)
         fluidnd(9)=tetnodet(11)
         fluidnd(10)=tetnodet(4)
         fluidnd(11)=tetnodet(13)
         fluidnd(12)=tetnodet(9)
         fluidnd(13)=tetnodet(15)
         fluidnd(14)=centroid(10)

      else
         if (debug) write(*,*)'mshobstms3dq7d'
         call mshobstms3dq7d ( tetnodet, tetnodenew, facenodet, &
                               centroid,  edgenodet )

         fluidnd(1)=tetnodet(6)
         fluidnd(2)=tetnodet(11)
         fluidnd(3)=tetnodet(2)
         fluidnd(4)=tetnodet(15)
         fluidnd(5)=tetnodet(9)
         fluidnd(6)=tetnodet(14)
         fluidnd(7)=centroid(7)

         fluidnd(8)=tetnodet(2)
         fluidnd(9)=tetnodet(12)
         fluidnd(10)=tetnodet(8)
         fluidnd(11)=tetnodet(13)
         fluidnd(12)=tetnodet(9)
         fluidnd(13)=tetnodet(15)
         fluidnd(14)=centroid(11)

      end if

      nelemnew = 6

      call erclos ( 'mshobstms3dq7b' )

      end
