      subroutine mshobstparm ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 19-09-2013 New parameter list
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Estimate number of nodes and elements in new mesh
!     Estimate length of some arrays
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput        i/o   integer input array
!                         See subroutine mshobstms23d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcross, npointnew, nelemnew, &
              linear, lenkmscnew, lenkmsmnew, lenkmsnnew, &
              numbnelmobst, lencoornew, &
              lenipointobst, lenielemobst

!     lencoornew     Estimated length of array coornew
!     lenielemobst   Computed length of array ielemobst
!     lenipointobst  length of array ipointobst
!     lenkmscnew     Length of array kmshcnew
!     lenkmsmnew     Length of array kmshmnew
!     lenkmsnnew     Length of array kmshnnew
!     linear         Indicates whether the element is a linear simplex
!                    (1) or not (2)
!     nelemnew       Number of elements in new mesh
!     npointnew      Sequence number of last generated node
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numcross       Number of cross sections
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstparm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstparm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npointnew = iinput(4)
      numbnelmobst = iinput(9)
      numcross = iinput(10)
      numcross=10*numcross  ! overstimation by Raoul

      npointnew = npoint+numcross
      nelemnew  = nelem+ndim**2*numbnelmobst

      lencoornew = npointnew*ndim
      lenipointobst = 5*numobst+npointnew
      lenielemobst = 6*numobst+nelemnew

      if ( npelm<=4 ) then

!     --- Only linear 2d elements

         lenkmscnew = size(kmeshc)+nelemnew*3
         linear = 1

      else if ( npelm<=9 ) then

!     --- Quadratic 2d elements

         lenkmscnew = size(kmeshc)+nelemnew*7
         linear = 2

      else if ( npelm<=11 ) then

!     --- Linear 3d elements

         lenkmscnew = size(kmeshc)+nelemnew*4
         linear = 1

      else

!     --- Quadratic 3d elements

         lenkmscnew = size(kmeshc)+nelemnew*15
         linear = 2

      end if
      if ( debug ) &
         write(irefwr,1) 'nelemnew, lenkmscnew, npointnew', &
                          nelemnew, lenkmscnew, npointnew

      if ( ndim>=2 ) then
         if ( kelmm>0 ) then
            lenkmsmnew = size(kmeshm)*2
         else
            lenkmsmnew = 0
         end if
      end if  ! ( ndim>=2 )

      lenkmsnnew = 0
      if ( ndim==3 ) then
         if ( kelmn>0 )  lenkmsnnew = size(kmeshn)*4
      end if  ! ( ndim==3 )

      iinput(4) = npointnew
      iinput(6) = linear
      iinput(7) = lenkmsmnew
      iinput(8) = lenkmsnnew
      iinput(11) = nelemnew
      iinput(12) = lenkmscnew
      iinput(14) = lencoornew
      iinput(15) = lenipointobst
      iinput(17) = lenielemobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstparm'

      end if  ! ( debug )
      call erclos ( 'mshobstparm' )

      end
