      subroutine mshobstupdcoor ( coor, jnode, ndim )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 25-03-2004
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     position intermediate nodes based on the coordinates of the
!     vertex nodes. Straight triangle/tetrahedron are formed.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  ndim, jnode(15)
      double precision  coor(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!     jnode          i    Array containing the node numbers of an element
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  j

!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstupdcoor' )

!     --- place midside nodes and centroid based on
!         vertex positions

      if (ndim.eq.2) then

         do j = 1, ndim

            coor(j,jnode(2)) = (coor(j,jnode(1))+ &
                                coor(j,jnode(3)))*0.5d0
            coor(j,jnode(4)) = (coor(j,jnode(3))+ &
                                coor(j,jnode(5)))*0.5d0
            coor(j,jnode(6)) = (coor(j,jnode(5))+ &
                                coor(j,jnode(1)))*0.5d0
            coor(j,jnode(7)) = ( coor(j,jnode(1))+ &
                                 coor(j,jnode(3))+ &
                                 coor(j,jnode(5)) )/3d0

         end do

      end if

!     --- extended tetrahedron

      if (ndim.eq.3) then

         do j = 1, ndim

            coor(j,jnode(2))= (coor(j,jnode(1))+ &
                               coor(j,jnode(3)))*0.5d0
            coor(j,jnode(4))= (coor(j,jnode(3))+ &
                               coor(j,jnode(5)))*0.5d0
            coor(j,jnode(6))= (coor(j,jnode(5))+ &
                               coor(j,jnode(1)))*0.5d0
            coor(j,jnode(7))= (coor(j,jnode(1))+ &
                               coor(j,jnode(10)))*0.5d0
            coor(j,jnode(8))= (coor(j,jnode(3))+ &
                               coor(j,jnode(10)))*0.5d0
            coor(j,jnode(9))= (coor(j,jnode(5))+ &
                               coor(j,jnode(10)))*0.5d0
            coor(j,jnode(11))= ( coor(j,jnode(1))+ &
                                 coor(j,jnode(3))+ &
                                 coor(j,jnode(5)) )/3d0
            coor(j,jnode(12))= ( coor(j,jnode(1))+ &
                                 coor(j,jnode(3))+ &
                                 coor(j,jnode(10)) )/3d0
            coor(j,jnode(13))= ( coor(j,jnode(3))+ &
                                 coor(j,jnode(5))+ &
                                 coor(j,jnode(10)) )/3d0
            coor(j,jnode(14))= ( coor(j,jnode(1))+ &
                                 coor(j,jnode(5))+ &
                                 coor(j,jnode(10)) )/3d0
            coor(j,jnode(15))= ( coor(j,jnode(1))+ &
                                 coor(j,jnode(3))+ &
                                 coor(j,jnode(5))+ &
                                 coor(j,jnode(10)) )*2.5d-1

         end do

      end if

      call erclos('mshobstupdcoor')

      end
