      subroutine mshparelems2 ( nprocessors, nsubdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-03-2015 Fortran 90
!        version  1.0  date 04-05-2004
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the region into nprocessors blocks depending on the parameter
!     methodpar
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors, nsubdiv

!     nprocessors    o    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     nsubdiv        i    number of processors ^ (1/3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprocessor, numelms, &
              numnodes, iproci, iprocj, iprock
      integer, allocatable :: elements(:), points(:)

!     elements       Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     iprocessor     Processor sequence number
!     iproci         Loop counter for sub block renumbering in "i"-dir
!     iprocj         Loop counter for sub block renumbering in "j"-dir
!     iprock         Loop counter for sub block renumbering in "k"-dir
!     numelms        Number of elements in array elements
!     numnodes       Number of nodal points in array points
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPAR3DBLOCK  Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points with the
!                    point numbers in case we have to subdivide a 3d block
!                    into identical 3d subblocks
!     MSHPARELEMS3   Store arrays elements and points into ibuffr and store the
!                    corresponding memory management sequence numbers into
!                    elempointers and nodepointers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2654   ndim#3 or nelgrp#1 or npelm#8
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelems2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelems2'

      end if
      if ( ierror/=0 ) go to 1000
      allocate ( elements(nelem), points(npoint) )

      if ( nelgrp>1 .or. npelm/=8 .or. ndim/=3 ) then

!     --- Error: ndim # 3 or npelm # 8 or nelgrp # 1

         call errint ( ndim, 1 )
         call errint ( nelgrp, 2 )
         call errint ( npelm, 3 )
         call errsub ( 2654, 3, 0, 0 )
         go to 1000

      end if  ! ( nelgrp>1 .or. npelm/=8 .or. ndim/=3 )

!     --- Loop over the processors

      iprocessor = 0
      do iprock = 1, nsubdiv

         do iprocj = 1, nsubdiv

            do iproci = 1, nsubdiv

               iprocessor = iprocessor+1

!              --- Fill arrays elements and points

               call mshpar3dblock ( iproci, iprocj, iprock, &
                                    nprocessors, elements, points, &
                                    numelms, numnodes, kmeshc )

!              --- Copy arrays elements and ipoints in ibuffr

               call  mshparelems3 ( elements, iprocessor, numelms, points, &
                                    numnodes )

            end do  ! iproci = 1, nsubdiv

         end do  ! iprocj = 1, nsubdiv

      end do  ! iprock = 1, nsubdiv

      deallocate ( elements, points )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparelems2'

      end if
      call erclos ( 'mshparelems2' )

      end
