      function mshpavnumedges ( inode, markedges, nmeshl, iedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find number of boundary connecting edges in node inode and store
!     in mshpavnumedges
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mshpavnumedges, inode, markedges(*), nmeshl(*), iedge

!     iedge          o    Edge sequence number of boundary connecting edge
!                         in node inode
!     inode          i    Node sequence number
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     mshpavnumedges o    Number of boundary connecting edges corresponding
!                         to inode
!     nmeshl         i    nmesh part l contains edge numbers connected to nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, nnode_edge, i

!     i              Counting variable
!     ipstart        Starting address in array
!     nnode_edge     number of nodes corresponding to edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavnumedges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavnumedges'

      end if
      if ( ierror.ne.0 ) go to 1000
      ipstart = nmeshl(inode)        ! start position of inode in nmeshl
      nnode_edge = nmeshl(ipstart)   ! number of edges connected to inode
      mshpavnumedges = 0
      do i = 1, nnode_edge

         if ( markedges(nmeshl(ipstart+i)).eq.-2 ) then

!        --- Boundary connecting edge found

            mshpavnumedges = mshpavnumedges+1
            iedge = nmeshl(ipstart+i)

         end if  ! ( markedges(nmeshl(ipstart+i)).eq.-2 )

      end do  ! i = 1, nnode_edge
      if ( mshpavnumedges.ne.1 ) iedge = 0

1000  call erclos ( 'mshpavnumedges' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavnumedges'

      end if

      end
