      subroutine mshpavtrianinf ( itrian, iedge1, iedge2, iedge3, h, &
                                  dist, nmeshc, thickness, coor, &
                                  markedges, nmeshd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract information about a triangle
!     for mesh generator paver
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     mesh_generation
!     edge
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itrian, iedge1, iedge2, iedge3, nmeshc(*), &
              markedges(*), nmeshd(*)
      double precision h, dist, thickness(*), coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dist           o    Distance between two points
!     h              o    Average thickness in boundary edge edge1
!     iedge1         o    edge sequence number of boundary edge
!     iedge2         o    edge sequence number of connecting edge
!     iedge3         o    edge sequence number of connecting edge
!     itrian         i    Sequence number of triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     thickness      i    Array containing the thickness in the nodes in exactly
!                         the same sequence as in coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, inode1, inode2, inode3, ipstart
      double precision x1(2), x2(2), x3(2)

!     ih             Help variable to store a constant temporarily
!     inode1         First node in triangle on base
!     inode2         Second node in triangle on base
!     inode3         Third node in triangle (top)
!     ipstart        Starting address in array
!     x1             coordinates of first point of first boundary edge
!     x2             coordinates of last point of first boundary edge
!     x3             coordinates of top
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDISTTOP      Compute the distance of the top node to base of triangle
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     e1 := boundary edge
!     e2, e3 := remaining edges
!     h := mean thickness in e1
!     dist := distance from top to node to e1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavtrianinf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavtrianinf'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- e1 := boundary edge
!         e2, e3 := remaining edges

      ipstart = nmeshd(itrian)
      iedge1 = abs(nmeshd(ipstart+1))
      iedge2 = abs(nmeshd(ipstart+2))
      iedge3 = abs(nmeshd(ipstart+3))
      if ( markedges(iedge1).ne.-1 ) then

!     --- iedge1 is not boundary edge, try next one

         ih = iedge1
         iedge1 = iedge2
         iedge2 = iedge3
         iedge3 = ih

         if ( markedges(iedge1).ne.-1 ) then

!        --- iedge1 is not boundary edge, it must be the next one

            ih = iedge1
            iedge1 = iedge2
            iedge2 = iedge3
            iedge3 = ih

         end if  ! ( markedges(iedge1).ne.-1 )

      end if  ! ( markedges(iedge1).ne.-1 )

!     --- node1, node2 := nodes e1

      ipstart = nmeshc(iedge1)
      inode1  = nmeshc(ipstart+1)
      inode2  = nmeshc(ipstart+2)

!     --- node3 = top node

      ipstart = nmeshc(iedge2)
      inode3  = nmeshc(ipstart+1)
      if ( inode3.eq.inode1 .or. inode3.eq.inode2 ) &
         inode3 = nmeshc(ipstart+2)

!     --- h := mean thickness in e1

      h = 0.5d0*(thickness(inode1)+thickness(inode2))

!     --- dist := distance from top to node to e1

      x1(1) = coor(1,inode1)
      x1(2) = coor(2,inode1)
      x2(1) = coor(1,inode2)
      x2(2) = coor(2,inode2)
      x3(1) = coor(1,inode3)
      x3(2) = coor(2,inode3)
      call erdisttop ( x1, x2, x3, dist )

1000  call erclos ( 'mshpavtrianinf' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavtrianinf'

      end if

      end
