      subroutine mshposjac
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Change sequence of nodes in elements with negative Jacobian
!     After this call all elements have positive Jacobian
! **********************************************************************
!
!                       KEYWORDS
!
!     determinant
!     jacobian
!     element
!     positive
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielem, ikelmc, ielgrp

!     ielem          Element number in loop
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     MSHPOSJAC2     Compute Jacobian of linear tetrahedron
!                    If Jacobian negative, change sequence of nodes
!     MSHPOSJAC3     Compute Jacobian of quadratic tetrahedron
!                    If Jacobian negative, change sequence of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshposjac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshposjac'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ikelmc = 1

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, ishape, inelem', &
                             ielgrp, inpelm, ishape, inelem
         if ( ishape==11 ) then

!        --- Linear tetrahedron

            do ielem = 1, inelem
               if ( debug ) write(irefwr,1) 'ielem', ielem
               call mshposjac2 ( kmeshc(ikelmc), coor )
               ikelmc = ikelmc+inpelm
            end do  ! ielem = 1, inelem

         else if ( ishape==12 ) then

!        --- Quadratic tetrahedron

            do ielem = 1, inelem
               if ( debug ) write(irefwr,1) 'ielem', ielem
               call mshposjac3 ( kmeshc(ikelmc), coor )
               ikelmc = ikelmc+inpelm
            end do  ! ielem = 1, inelem

         else

!        --- Warning, shape number not yet implemented

            call errint ( ishape, 1 )
            call errwar ( 800, 1, 0, 0 )
            ikelmc = ikelmc+inelem*inpelm

         end if

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshposjac'

      end if  ! ( debug )
      call erclos ( 'mshposjac' )

      end subroutine mshposjac
