      subroutine mshrdmeshcoor ( irefc, ndim, npoint, coor, ivers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coordinates from file meshoutput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, ndim, npoint, ivers
      double precision coor(*)

!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     ivers          i    Version sequence number
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, i, ios
      character (len=80) text

!     i              Counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     n              length of array coor (ndim x npoint)
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     195   error in reading of co-ordinate records
!     196   end of file found when reading co-ordinate records
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdmeshcoor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdmeshcoor'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = ndim*npoint

      if ( irefc>=0 ) then

!     --- Formatted case

         if ( ivers>=7 ) read (irefc,100,iostat=ios) text
         if ( debug ) write(irefwr,*) text(1:20)
100      format(a)

         read(irefc, *, iostat = ios) (coor(i), i = 1, n)
         if ( ios<0 ) then
            call errsub ( 196, 0, 0, 0 )
         else if ( ios>0 ) then
            call errint ( ios, 1 )
            call errsub ( 195, 1, 0, 0 )
         end if
         if ( debug ) then
            write(irefwr,110) (coor(i),i=1,ndim)
110         format('Coordinates first point ', 3d12.3 )
            write(irefwr,120) (coor(n-ndim+i),i=1,ndim)
120         format('Coordinates last point ', 3d12.3 )
         end if  ! ( debug )

      else

!     --- Unformatted case

         if ( ivers>=7 ) read (-irefc,iostat=ios) text
         if ( debug ) write(irefwr,*) text(1:10)
         read(-irefc, iostat = ios) (coor(i), i = 1, n)
         if ( ios<0 ) then
            call errsub ( 196, 0, 0, 0 )
         else if ( ios>0 ) then
            call errint ( ios, 1 )
            call errsub ( 195, 1, 0, 0 )
         end if

      end if  ! ( irefc>=0 )

1000  call erclos ( 'mshrdmeshcoor' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrdmeshcoor'

      end if  ! ( debug )

      end
