      subroutine mshrdparmsh2 ( nelgrp, elgrp, irefc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information about element groups
!     Store in array elgrp
!     Only in case of nelgrp>1
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelgrp, elgrp(2,nelgrp), irefc

!     elgrp          o    Array of size 2 x nelgrp
!                         (1,i) contains number of nodes in element group i
!                         (2,i) contains number of elements of element group i
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ios, idum

!     idum           General dummy variable
!     ielgrp         element group number
!     ios            IOSTAT variable, returns from reading or writing
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     199   error in reading of dimension parameters
!     251   end of file found in reading of dimension parameters
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdparmsh2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdparmsh2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do ielgrp = 1, nelgrp

         if ( irefc>=0 ) then
            read ( irefc, *, iostat = ios ) &
                elgrp(1,ielgrp), idum, elgrp(2,ielgrp), idum
         else
            read ( -irefc, iostat=ios ) &
                elgrp(1,ielgrp), idum, elgrp(2,ielgrp), idum
         end if
         if ( ios>0 ) then

!        --- ios>0, Error in reading file

            call errint ( ios, 1 )
            call errsub (199, 1, 0, 0)
            go to 1000

         else if ( ios<0 ) then

!        --- ios>0, End of file reached when reading file 10

            call errsub ( 251, 0, 0, 0)
            go to 1000

         end if

      end do  ! ielgrp = 1, nelgrp

1000  call erclos ( 'mshrdparmsh2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrdparmsh2'

      end if  ! ( debug )

      end
