      subroutine mshrect_grid ( coorrect, nx, ny, ox, oy, dx, dy)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create coordinates for rectangular grid in R^2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     coordinate
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny
      double precision coorrect(2,nx,ny), ox, oy, dx, dy

!     coorrect       o    Coordinates of rectangle
!     dx             i    step size in x-direction
!     dy             i    step size in y-direction
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     ox             i    x-coordinate of origin
!     oy             i    y-coordinate of origin
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision x, y

!     i              Counting variable
!     j              Counting variable
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for i = 0 (1) nx-1 do
!        x := ox+i*dx
!        for j = 0 (1) ny-1 do
!           y := oy+j*dy
!           coorrect(1,i+1,j+1 ) := x;  coorrect(2,i+1,j+1 ) := y
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrect_grid' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrect_grid'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- for i = 0 (1) nx-1 do
!           x := ox+i*dx

      do i = 0, nx-1

         x = ox+i*dx

!        --- for j = 0 (1) ny-1 do
!               y := oy+j*dy
!               coorrect(1,i+1,j+1 ) := x;  coorrect(2,i+1,j+1 ) := y

         do j = 0, ny-1

            y = oy+j*dy
            coorrect(1,i+1,j+1 ) = x
            coorrect(2,i+1,j+1 ) = y

         end do  ! j = 0, ny-1

      end do  ! i = 0, nx-1

1000  call erclos ( 'mshrect_grid' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrect_grid'

      end if

      end
