      subroutine mshrefcopy (refineloc, refine, reflvl, nelem_basis)
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 31-08-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy a refinement array such that all elements marked as intersected
!     (having value equal to reflvl + 1) are 'normalized', that is, these
!     values are replaced by reflvl
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem_basis, refineloc(*), refine(*), reflvl

!     nelem_basis    i    Number of elements of the basis mesh
!     refine         i    Refinement vector
!     refineloc      o    'Normalized' refinement vector
!     reflvl         i    Contains the maximum level of refinement
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcopy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcopy'

      end if
      if ( ierror/=0 ) go to 1000

      do i = 1, nelem_basis
         if (refine(i)==reflvl+1) then
            refineloc(i) = refine(i)-1
         else
            refineloc(i) = refine(i)
         end if
      end do

1000  call erclos ( 'mshrefcopy' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcopy'

      end if

      end
