      subroutine mshrefedge ( nmeshd, ielem, edgenode, nmeshc, iedges, &
                              nedgenode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-04-2005 Added nedgenode (John Brusche)
!        version  1.0  date 03-03-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array edgenode with information concerning edges
!     to be subdivided
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     intersection
!     linear
!     triangle
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshd(*), ielem, edgenode(3,*), nmeshc(*), iedges(*), &
              nedgenode

!     edgenode       o    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     iedges         i    Work array of size nedges
!     ielem          i    Element number
!     nedgenode      o    number of added (internal) nodes for this
!                         level of refinement
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, iedge, j, jpstart

!     iedge          edge sequence number
!     ipstart        Starting address in array
!     j              Counting variable
!     jpstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefedge' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefedge'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      nedgenode = 0
      ipstart = nmeshd(ielem)
      do j = 1, nmeshd(ipstart)
         iedge = abs(nmeshd(ipstart+j))
         if ( debug ) &
            write(irefwr,100) 'j, iedge, iedges(iedge) ', &
                               j, iedge, iedges(iedge)
         if ( iedges(iedge)>0 ) then
            jpstart = nmeshc(iedge)
            nedgenode = nedgenode+1
            edgenode(1,nedgenode) = nmeshc(jpstart+1)
            edgenode(2,nedgenode) = nmeshc(jpstart+2)
            edgenode(3,nedgenode) = iedges(iedge)
            if ( debug ) call prinin ( edgenode(1,nedgenode), 3, &
                                      'edgenode' )
         end if  ! ( iedges(iedge)>0 )
      end do

1000  call erclos ( 'mshrefedge' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefedge'

      end if

      end
