      subroutine mshrefestimate ( nelemnew, npointnew, &
                                  lenkmscnew, linear, idimedges, &
                                  lenkmsnnew, lencoornew, lenkmsmnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 09-03-2013 Set length at minimum of 1
!        version  2.0  date 20-09-2006 Extra parameter lenkmsmnew
!        version  1.1  date 18-10-2005 New estimates
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters to estimate array length in case of
!     local refinement
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelemnew, npointnew, &
              lenkmscnew, linear, idimedges, lenkmsnnew, &
              lencoornew, lenkmsmnew

!     idimedges      o    Length of first dimension of array iedges
!                         (linear:1, quadratic: 3)
!     lencoornew     o    Estimated length of array coornew
!     lenkmscnew     o    Length of array kmshcnew
!     lenkmsmnew     o    Length of array kmshmnew
!     lenkmsnnew     o    Length of array kmshnnew
!     linear         o    Indicates if linear elements are used (1) or
!                         quadratic (2)
!     nelemnew       o    Number of elements in new mesh
!     npointnew      o    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefestimate' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefestimate'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute some parameters

      if ( ndim==1 ) then

!     --- !d case, special

         nelemnew = 2*nelem        ! overestimation of new number of elements

      else

!     --- 2d and 3d

         nelemnew = ndim**2*nelem  ! overestimation of new number of elements

      end if  ! ( ndim==1 )
      npointnew = 2**ndim*npoint         ! overestimation of new number of nodes

      if ( ndim==1 ) then

!     --- Only 1d elements

         lenkmscnew = size(kmeshc)+nelemnew*2

      else if ( npelm<=3 ) then

!     --- Only linear 2d elements

         lenkmscnew = size(kmeshc)+nelemnew*3
         linear = 1
         idimedges = 1

      else if ( ndim==2 .and. npelm<=9 ) then

!     --- Quadratic 2d elements (not yet implemented)

         lenkmscnew = size(kmeshc)+nelemnew*7
         linear = 2
         idimedges = 3

      else if ( npelm<=4 ) then

!     --- Linear 3d elements

         lenkmscnew = size(kmeshc)+nelemnew*4
         linear = 1
         idimedges = 1

      else

!     --- Quadratic 3d elements

         lenkmscnew = size(kmeshc)+nelemnew*15
         linear = 2
         idimedges = 3

      end if

      lenkmsnnew = 1
      lenkmsmnew = 1

      if ( ndim>=2 ) then

!     --- ndim>=2, array kmeshm

         if ( kelmm>0 ) lenkmsmnew = size(kmeshm)*2

      end if  ! ( ndim>=2 )

      if ( ndim==3 ) then

!     --- ndim = 3, array kmeshn

         if ( kelmn>0 ) lenkmsnnew = size(kmeshn)*4

      end if  ! ( ndim==1 )

      lencoornew = npointnew*ndim

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefestimate'

      end if
      call erclos ( 'mshrefestimate' )

      end
