      subroutine mshrefintnodes ( intnodes, edgenode, ielem, nedgenode, &
                                  ireflvl, nodeextrael, refmapold, &
                                  ielemlast, nextranode, enthwork, &
                                  npoint_basis, lnintnodes )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 26-10-2005 extra parameter lnintnodes
!        version  1.1  date 29-09-2005 Added extra check not to add
!                                      nodes to intnodes for which
!                                      0<ereflvl<reflvl holds
!        version  1.0  date 20-04-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array intnodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nedgenode, edgenode(3, nedgenode), ielem, ireflvl, &
              intnodes(*), nodeextrael(0:*), refmapold(2,*), ielemlast, &
              nextranode, enthwork(*), npoint_basis, lnintnodes

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     enthwork      i/o   work array to keep track of added nodes
!     ielem          i    Element number
!     ielemlast     i/o   Element number of latest considered basis
!                         mesh element
!     intnodes      i/o   array of added internal nodes
!     ireflvl        i    current level of refinement
!     lnintnodes     i    Declared length of array intnodes
!     nedgenode      i    number of added (internal) nodes for this
!                         level of refinement
!     nextranode    i/o   number of extra nodes added
!     nodeextrael    i    help array containing the number of extra
!                         nodes for each level of refinement
!     npoint_basis   i    npoint of the basis mesh
!     refmapold      i    mapping array corresponding to the old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipintnodes

!     i              Counting variable
!     ipintnodes     Starting location in array intnodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefintnodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefintnodes'
         write(irefwr,*) 'ireflvl', ireflvl

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute position in array intnodes to write added nodes

      if (debug) write(irefwr,*) 'ielem, refmapold(1,ielem),ielemlast', &
                           ielem, refmapold(1,ielem),ielemlast
      if (debug) write(irefwr,*) 'nodeextrael(1)', nodeextrael(1)
      if (debug) &
         write(irefwr,*) 'refmapold(2,ielem)', refmapold(2,ielem)

      if (refmapold(1,ielem)/=ielemlast) then

!     --- in case we have reached the next element of the basismesh
!         shift pointer to new position in array intnodes

         nextranode = 0
         ielemlast = refmapold(1,ielem)
      else

!     --- we stay in the same basismesh element

         nextranode = nextranode + nodeextrael(1)
      end if
      if (debug) write(irefwr,*) 'nextranode', nextranode

      ipintnodes = intnodes(ielemlast)+nextranode + &
                   nodeextrael(ireflvl-1)
      if (debug) write(irefwr,*) 'ipintnodes', ipintnodes

!     --- copy node numbers to array intnodes

      do i = 0, nedgenode-1
         if (debug) write(irefwr,*) 'enthwork before', &
                               enthwork(edgenode(3,i+1))

!        --- add a node to intnodes only when for the corresponding
!            basismesh element this node has not yet been added

         if (enthwork(edgenode(3,i+1)-npoint_basis)==0 .or. &
             enthwork(edgenode(3,i+1)-npoint_basis) /= &
                                       refmapold(1,ielem)) then

!        --- only add nodes for elements for which 0<ereflvl<reflvl
!            does not hold

            if (refmapold(2,ielem)>=ireflvl .or. ireflvl==1) then
               intnodes(ipintnodes+i) = edgenode(3,i+1)
               enthwork(edgenode(3,i+1)-npoint_basis) = &
                                                   refmapold(1,ielem)
            end if
         else

!        --- correct some indices for not adding a node to intnodes

            nextranode = nextranode - 1
            ipintnodes = ipintnodes - 1
         end if
         if (debug) write(irefwr,*) 'edgenode', edgenode(3,i+1)
         if (debug) write(irefwr,*) 'enthwork after', &
                               enthwork(edgenode(3,i+1)-npoint_basis)
      end do
      call inichkarlen ( ipintnodes, lnintnodes, 'intnodes' )

1000  call erclos ( 'mshrefintnodes' )
      if ( debug ) then

!     --- Debug information

         call prinin(intnodes(ipintnodes), nedgenode, 'intnodes')
         call prinin(intnodes, 750, 'intnodesnew')
         write(irefwr,*) 'End mshrefintnodes'

      end if

      end
