      subroutine mshrefintnodpo ( intnodes, refinenew, nodeextrael, &
                                  nelem_basis, reflvl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the first nelem_basis positions of intnodes with pointers to
!     the correct starting locations in the rest of array intnodes
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intnodes(*), refinenew(*), nodeextrael(0:*), nelem_basis, &
              reflvl

!     intnodes      i/o   array of added internal nodes
!     nelem_basis    i    number of elements of the basis mesh
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     refinenew      i    refinement array for the new mesh
!     reflvl         i    Level of refinement
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart

!     i              Counting variable
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefintnodpo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefintnodpo'

      end if
      if ( ierror/=0 ) go to 1000

      istart = nelem_basis + 1

      if (debug) then
         print *, 'reflvl', reflvl
         call prinin(refinenew, nelem_basis, 'refinenew')
      end if

      do i = 1, nelem_basis
         intnodes(i) = istart
         if (debug) then
             print *, 'refinenew(i), min, nodeextrael', refinenew(i), &
                   min(reflvl,refinenew(i)), &
                   nodeextrael(min(reflvl,refinenew(i)))
         end if
         istart = istart + nodeextrael(min(reflvl,refinenew(i)))
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefintnodpo'

      end if
      call erclos ( 'mshrefintnodpo' )

      end
