      subroutine mshrefpar10 ( kmeshmglobnew, kmeshmglobold, &
                               kmeshmtmp, istartcur, lenkmeshmglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshmglob in case of refinement
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmglobnew(*), kmeshmglobold(*), &
              kmeshmtmp(*), istartcur(*), lenkmeshmglob

!     istartcur      i    starting position of curves in kmeshmtmp
!     kmeshmglobnew  o    Array kmeshmglob corresponding to new mesh
!     kmeshmglobold  i    Array kmeshmglob corresponding to old mesh
!     kmeshmtmp      i    Temporary array kmeshm in which the nodes of the new
!                         curves are stored
!     lenkmeshmglob i/o   Length of array kmeshmglob
!                         At input: declared length
!                         At output: length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, icurnr, nsubcv, lencurve, ipstart, &
              i, jpstart, jcurnr

!     i              Counting variable
!     icurnr         Curve sequence number
!     ipstart        Starting address in array
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     jcurnr         Curve sequence number
!     jpstart        Starting address in array
!     lencurve       Number of nodes in curve
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar10'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some parameters

      kmeshmglobnew(1:4) = kmeshmglobold(1:4)  ! ninner and istart3
      istar3 = kmeshmglobnew(2)  ! position in part 3
      kmeshmglobnew(5) = 1  ! relative start of curve 1

      do icurnr = 1, ncurvs

         if ( kelmq/=0 ) then

!        --- kelmq/=0, there are subcurves

            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)

         else

!        --- kelmq = 0, there are no subcurves

            nsubcv = 0

         end if  ! ( kelmq/=0 )

         if ( nsubcv==0 ) then

!        --- there are no subcurves, hence information must be copied

            ipstart = istartcur(icurnr)
            lencurve = istartcur(icurnr+1)-ipstart
            kmeshmglobnew(istar3:istar3+lencurve-1) = &
               kmeshmtmp(ipstart:ipstart+lencurve-1)
            istar3 = istar3+lencurve

         else

!        --- Curve contains subcurves copy information of each subcurve

            jpstart = kmeshq(icurnr)
            do i = 1, nsubcv   ! loop over subcurves

               jcurnr = kmeshq(jpstart+i)
               ipstart = istartcur(jcurnr)
               lencurve = istartcur(jcurnr+1)-ipstart
               kmeshmglobnew(istar3:istar3+lencurve-1) = &
                  kmeshmtmp(ipstart:ipstart+lencurve-1)
               istar3 = istar3+lencurve

            end do  ! i = 1, nsubcv

         end if  ! ( nsubcv==0 )
         kmeshmglobnew(5+icurnr) = istar3-kmeshmglobnew(2)+1

      end do  ! icurnr = 1, ncurvs

      call inichkarlen ( istar3, lenkmeshmglob, 'kmeshmglobnew' )
      if ( ierror/=0 ) go to 1000

      kmeshmglobnew(3) = istar3  ! istart4
      ninner = kmeshmglobold(1)  ! number of internal curves
      ipstart = kmeshmglobold(3) ! istart4 for old kmeshmglob

      call inichkarlen ( istar3+ninner-1, lenkmeshmglob, &
                         'kmeshmglobnew' )
      if ( ierror/=0 ) go to 1000

      kmeshmglobnew(istar3:istar3+ninner-1) = &
          kmeshmglobold(ipstart:ipstart+ninner-1)

      lenkmeshmglob = istar3+ninner-1  ! computed length

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmglobnew, lenkmeshmglob, 'kmeshmglobnew' )
         write(irefwr,*) 'End mshrefpar10'

      end if  ! ( debug )
      call erclos ( 'mshrefpar10' )

      end
