      subroutine mshrefpar8 ( kmeshmnew, newmapping, &
                              iseqkm2, lenbuf, ibufarray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find single curves that are changed by refinement
!     Store these curves in ibufarray
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     coordinate
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmnew(*), newmapping(*), &
              iseqkm2, lenbuf, ibufarray(*)

!     ibufarray     i/o   Buffer array to store information to be sent or
!                         received
!                         In this case it concerns only the new part which
!                         must be filled
!     iseqkm2        i    Sequence number with respect to km of second mesh
!     kmeshmnew      i    New array kmeshm
!     lenbuf         o    Length of part stored in buffer arrayibufarray
!     newmapping     i    contains global node numbers for refined mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, icurnr, i, nsubcv, lencurvenew, lencurveold

!     i              Counting variable
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     lencurvenew    Number of nodes in new curve
!     lencurveold    Number of nodes in old curve
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar8' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar8'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istar3 = kmeshmnew(2)  ! starting address in kmeshm part 3

      lenbuf = 1

!     --- Loop over all curves

      do icurnr = 1, km(iseqkm2)%kmesh(11)

         lencurvenew = kmeshmnew(5+icurnr)-kmeshmnew(4+icurnr)
         if ( kelmq/=0 ) then

!        --- kelmq/=0, there are subcurves

            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)

         else

!        --- kelmq = 0, there are no subcurves

            nsubcv = 0

         end if  ! ( kelmq/=0 )

         if ( debug ) &
            write(irefwr,1) 'icurnr, nsubcv, lencurvenew', &
                             icurnr, nsubcv, lencurvenew

         if ( nsubcv==0 ) then

!        --- there are no subcurves
!            Check if curve is changed by refinement

            lencurveold = kmeshm(5+icurnr)-kmeshm(4+icurnr)

            if ( debug ) &
               write(irefwr,1) 'lencurveold, istar3', lencurveold, istar3

            if ( lencurvenew>lencurveold ) then

!           --- New curve length larger than old one, hence curve must be
!               stored

               ibufarray(lenbuf+1) = icurnr
               ibufarray(lenbuf+2) = lencurvenew
               ibufarray(lenbuf+3:lenbuf+2+lencurvenew) = &
                  kmeshmnew(istar3:istar3+lencurvenew-1)
               if ( debug ) &
                  call prinin ( kmeshmnew(istar3), lencurvenew, 'kmeshmnew' )

!              --- Replace local node numbers by global node numbers

               do i = 1, lencurvenew
                  if ( ibufarray(lenbuf+2+i)>0 ) &
                     ibufarray(lenbuf+2+i) = &
                     newmapping(ibufarray(lenbuf+2+i))
               end do  ! i = 1, lencurvenew

               if ( debug ) &
                  call prinin ( ibufarray(lenbuf+1), 2+lencurvenew, &
                                'ibufarray' )
               lenbuf = lenbuf+2+lencurvenew

            end if  ! ( lencurvenew>lencurveold )

         end if  ! ( nsubcv==0 )

         istar3 = istar3 + lencurvenew

      end do  ! icurnr = 1, km(iseqkm2)%kmesh(11)

      ibufarray(1) = lenbuf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenbuf', lenbuf
         call prinin ( ibufarray, lenbuf, 'ibufarray' )
         write(irefwr,*) 'End mshrefpar8'

      end if  ! ( debug )
      call erclos ( 'mshrefpar8' )

      end
