      subroutine mshrefupdateln ( nelem, refinenew, nodeextrael, &
                                  lnintnodes, lnparentnodes, &
                                  lnenthwork, reflvl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-10-2005 'fixed' lnintnodes bug (JB)
!        version  1.0  date 21-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the length of some arrays to be used by subroutine
!     mshrefupdate or at least an overestimation
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     enthalpy
!     mesh
!     length
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, refinenew(*), nodeextrael(0:*), lnintnodes, &
              lnparentnodes, lnenthwork, reflvl

!     lnenthwork     o    Maximal length of array enthwork
!     lnintnodes     o    Maximal length of array intnodes
!     lnparentnodes  o    Maximal length of array parentnodes
!     nelem          i    Number of elements on basis mesh
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     refinenew     i/o   Refine array defined on basis mesh of length nelem
!                         If an element is intersected by the freezing front
!                         refinenew(i) is set to reflvl+1, &
!                         if an element is neighbour of such an element it is
!                         set to reflvl, otherwise it is set to 0
!     reflvl         i    Level of refinement
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, reflvlloc

!     ielem          Element number
!     reflvlloc      Local value of level of refinement
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The length of the following arrays must be computed
!     intnodes: length is the number of extra nodes to be created + &
!               the number of elements of the basis mesh (pointers)
!               For each refined element the number of extra nodes depends
!               on the level of refinement and the dimension according
!               to the following table
!
!               level    0     1    2    3
!                 1D     0     1    3    7
!                 2D     0     3   12   42
!                 3D     0     6   31  161
!     parentnodes: (length of intnodes - nelem_basis) * 2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefupdateln' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefupdateln'
         call prinin(refinenew, nelem, 'refinenew')
      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute the length of intnodes

      lnintnodes = 2*nelem

!     --- Loop over the elements

      do ielem = 1, nelem
         reflvlloc = min(refinenew(ielem),reflvl)
         lnintnodes = lnintnodes+nodeextrael(reflvlloc)
      end do  ! ielem = 1, nelem

!     --- Compute the length of parentnodes

      if (lnintnodes>nelem) then
         lnparentnodes = 2*(lnintnodes-nelem)
         lnenthwork = lnintnodes-nelem
      else
         lnparentnodes = 1
         lnenthwork = 1
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefupdateln'

      end if
      call erclos ( 'mshrefupdateln' )

      end
