      subroutine mshshift2d2 ( iwork, markedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-02-2012 use getnpelmelgrp
!        version  1.0  date 01-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all edges that are intersected by the obstacle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(nelem,2), markedge(*)

!     iwork          i    integer work array
!                         See subroutine mshobstms2d
!     markedge      i/o   Array of size nedges
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, istart, ielgrp, iielem, i, iobst, &
              ireledge, icross, iedge, istarticross

!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     iobst          Obstacle sequence number
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d2'
         call prinin1 ( iwork, nelem, 2, 'iwork' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize

      ielem  = 0
      istart = 0

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         inelem = sepgetinelem ( ielgrp )

!        --- loop over elements

         do iielem = 1, inelem

            ielem=ielem+1

            if ( iwork(ielem,1)/=0 ) then

!           --- iwork(ielem)#0, hence element is intersected

               istart = iwork(ielem,1)  ! starting address in icrossobst
               iobst = iwork(ielem,2)
               istarticross = icrossobst(iobst)

!              --- loop over crossed edges

               do i = 1, icrossobst(istart)

                  ireledge = icrossobst(istart+i)
                  icross  = istarticross+1+2*(ireledge-1)
                  iedge = icrossobst(icross)
                  markedge(iedge) = icross

               end do  ! i = 1, icrossobst(istart)

            end if  ! ( iwork(ielem)/=0 )

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2d2'

      end if  ! ( debug )
      call erclos ( 'mshshift2d2' )

      end
