      subroutine mshsubvol1 ( kmeshclc, nselem, jshape, knpelm, isurnr, &
                              coorlc, nldim, areas, npointlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 09-04-2014 Debug statements
!        version  1.0  date 17-07-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute volume of elements in submesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nselem, knpelm, jshape, isurnr, nldim, npointlc
      integer, target :: kmeshclc(knpelm,nselem)
      double precision, target :: coorlc(nldim,npointlc)
      double precision :: areas(nselem)

!     areas      o     Contains the areas of the elements of the submesh
!     coorlc     i     array of length ndim x npoint containing the co-ordinates
!                      of the nodal points with respect to the global numbering
!     isurnr     i     Surface sequence number
!     jshape     i     Shape number of element
!     kmeshclc   i     kmesh part c, contains the nodal points corresponding
!                      to the elements
!     knpelm     i     Number of nodes in an element
!     nldim      i     Dimension of space
!     npointlc   i     Number of nodes
!     nselem     i     Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ndimsav

!     i              Counting variable
!     ndimsav        Help variable to save the value of ndim
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision sepeli705

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPELDEFAULT   Set default values for parameters in module sepmodulebasefn
!     SEPELI705      Element subroutine for second order elliptic equation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsubvol1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsubvol1'
         write(irefwr,1) 'npointlc, nselem, nldim', npointlc, nselem, nldim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndimsav = ndim
      icheli = 7
      numold = 1
      jdegfd = 1
      E%ielgrp = isurnr
      itype  = 800
      E%itype  = 800
      E%first = 0
      inpelm = knpelm
      icount = inpelm
      nelemcac  = nselem
      E%ndim   = nldim
      ndim = nldim
      E%ishape = jshape
      E%npelm = inpelm
      if ( E%ishape<10 ) then
         isubr = 1
      else
         isubr = 2
      end if
      if ( debug ) then
         write(irefwr,1) 'ishape, knpelm', E%ishape, knpelm
         call prinin ( kmeshclc(1,1), knpelm, 'kmeshclc' )
      end if  ! ( debug )
      E%coor => coorlc

      call sepeldefault

!     --- Standard case

      if ( jshape==4 .and. knpelm==12 ) then

!     --- Special case of 12 node triangles

         if ( associated(E%nodes) ) nullify(E%nodes)
         allocate ( E%nodes(6) )
         inpelm = 6
         E%npelm = 6
         do ielem = 1, nelemcac
            E%ielem = ielem
            do i = 1, 6
               E%nodes(i) = kmeshclc(2*i-1,ielem)
            end do
            areas(ielem) = sepeli705()
            E%first = 1
         end do

         deallocate(E%nodes)

      else

!     --- Standard case

         do ielem = 1, nelemcac
            E%nodes => kmeshclc(1:knpelm,ielem)
            E%ielem = ielem
            if ( debug ) call prinin ( E%nodes, knpelm, 'E%nodes' )
            areas(ielem) = sepeli705()
            E%first = 1
         end do

      end if  ! ( ishape==4 .and. knpelm==12 )
      ndim = ndimsav

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( areas, nselem, 'areas' )
         write(irefwr,*) 'End mshsubvol1'

      end if  ! ( debug )
      call erclos ( 'mshsubvol1' )

      end
