      subroutine mshsurfaces10 ( kmeshn, nsurfs, nsurfsold, surffaces, &
                                 iworksurfs, icount, nsurfsnew, &
                                 extsurffaces, jcount, lenextsurffac, &
                                 ipoint, leniworksurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-11-2004 Fortran 90
!        version  1.1  date 04-08-2004 Correction for complex surfaces
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Treat the complicated surfaces containing several groups of subsurfaces
!     that can not be identified in a simple way
!     These surfaces are marked by ihelp(i) = -10
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), nsurfs, nsurfsold, surffaces(*), &
              iworksurfs(*), icount, &
              nsurfsnew, extsurffaces(*), jcount, &
              lenextsurffac, ipoint(*), leniworksurfs

!     extsurffaces   o    Array of length lenextsurffac containing the
!                         face sequence numbers of the new surfaces to be
!                         created
!     icount        i/o   Number of surfaces stored in iworksurfs
!     ihelp         i/o   integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!                         At output ihelp(i) > 0 refers to a starting address
!                         in array iworksurfs containing the subsurfaces of
!                         the surface
!                         ihelp(i) = -10 corresponds to a complicated situation
!     ipoint        i/o   Array of estimated length 10*nsurfsold that is used to
!                         store the end addresses of each new surface in array
!                         extsurffaces
!                         The positions 1 to nsurfs get the value 0
!     iwork         i/o   integer work array of length nfaces
!                         iwork(iface) contains the number of surfaces
!                         face iface is stored in
!                         If a face corresponds to a new subsurface ksurnr in
!                         this subroutine iwork(iface) is set to -ksurnr
!     iwork1         i    integer work array of length nfaces
!                         contains for each face iface with iwork(iface)>1 a
!                         starting address in iwork2
!     iwork2         i    integer work array of length leniwork2
!                         contains for each face iface with iwork(iface)>1 the
!                         sequence numbers of the surfaces iface is lying in
!     iworksurfs    i/o   Array in which the subsurfaces corresponding to a
!                         surface are stored
!     jcount        i/o   Last position used in extsurffaces
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     lenextsurffac  i    Length of array extsurffaces
!     leniworksurfs  i    Length of array iworksurfs
!     nsurfs         i    number of surfaces
!     nsurfsnew     i/o   Last surface sequence number used
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, istart, ifirst, iface, jstart, icompare(nsurfs), &
              kstart, i, isurface, iielem, inelem, numsurfaces, ksurnr, &
              icheck(10*nsurfs)
      logical finished, single, same

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icheck         Work array of length 10*nsurfs that is used to avoid
!                    double surfaces
!                    icheck(i) = 0, means that surface i has not yet been
!                    stored for isurnr, &
!                    icheck(i) = 1, it has been stored
!     icompare       Work array of length nsurfs
!                    icompare(i) = 0 if surface i is not coupled to the first
!                    face and icompare(i) = 1 otherwise
!     iface          Face sequence number
!     ifirst         First new relative element number
!     iielem         Relative element number in element group
!     inelem         Number of elements in a specific element group
!     istart         First position used at a part
!     isurface       surface sequence number
!     isurnr         Surface sequence number
!     jstart         starting position
!     kstart         Start value
!     ksurnr         surface sequence number
!     numsurfaces    Number of surfaces coupled to face
!     same           If true the surfaces coupled ot the faces are the same
!     single         If true the surface is a single surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For isurnr := 1 (1) nsurfsold do
!        if ( ihelp(isurnr) = -10 ) then
!           finished := false
!           while ( not finished ) do
!              ifirst := first element with iwork(iface) # 0
!              if ( ifirst > 0 ) then
!                 Clear array icompare (1:nsurfs)
!                 Set icompare(i) = 1 for all surfaces i coupled to iface
!                 if ( one of the surfaces is single ) then
!                    ksurnr := that surface; newsurface := false
!                 else
!                    nsurfsnew := nsurfsnew+1; ksurnr := nsurfsnew
!                    newsurface := true
!                 icount := icount+1; iworksurfs(icount) := ksurnr
!                 For all elements in surface coupled to same surfaces as
!                   ifirst do
!                    jcount := jcount+1; extsurffaces(jcount) := iface
!                    iwork(iface) := -ksurnr
!                 if ( newsurface ) then
!                    ipoint(nsurfsnew) := jcount
!              else
!                 finished = true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces10' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces10'
         write(irefwr,*) 'icount, jcount, nsurfs, nsurfsold ', &
                          icount, jcount, nsurfs, nsurfsold
         call prinin ( ihelp, nsurfs, 'ihelp' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array icheck

      do i = 1, 10*nsurfs
         icheck(i) = 0
      end do  ! i = 1, 10*nsurfs

!     --- For isurnr := 1 (1) nsurfsold do

      istart = 0   ! position in array surffaces
      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)  ! number of elements in isurnr
         if ( ihelp(isurnr).eq.-10 ) then

!        --- ihelp(isurnr) = -10, i.e. surface has not been treated yet
!            finished := false

            finished = .false.
            ihelp(isurnr) = icount+1
            jstart = 1  ! first relative element number
            ksurnr = 0

!           --- while ( not finished ) do

            do while ( .not. finished )

!           --- ifirst := first element with iwork(iface) # 0

               ifirst = 0
               do iielem = jstart, inelem

!              --- get face corresponding to element

                 iface = surffaces(istart+iielem)
                 if ( iwork(iface).ne.0 .and. iwork(iface).ne.-ksurnr ) &
                    then

!                --- New face found, store ifirst and jump

                    ifirst = iielem
                    go to 100

                 end if  ! ( iwork(iface).ne.0 )

               end do  ! iielem = jstart, inelem

100            jstart = ifirst+1
               if ( ifirst.gt.0 ) then

!              --- ifirst > 0, so there are still faces left in surface
!                  Clear array icompare (1:nsurfs)
!                  Set icompare(i) = 1 for all surfaces i coupled to iface

                  do i = 1, nsurfs
                     icompare(i) = 0
                  end do  ! i = 1, nsurfs
                  numsurfaces = iwork(iface)   ! number of surfaces
                                               ! corresponding to this face
                  if ( debug ) &
                     write(irefwr,*) 'isurnr, ifirst, numsurfaces ', &
                                      isurnr, ifirst, numsurfaces
                  kstart = iwork1(iface)
                  single = .false.
                  if ( numsurfaces.gt.1 ) then

!                 --- More subsurfaces

                     do i = 1, numsurfaces
                        isurface = iwork2(kstart+i-1)
                        icompare(isurface) = 1
                        if ( ihelp(isurface).eq.0 ) then

!                       --- Surface is single

                           ksurnr = isurface
                           single = .true.

                        else if ( iwork(iface).lt.0 ) then

!                       --- Surface is single

                           ksurnr = -iwork(iface)
                           single = .true.

                        end if  ! ( ihelp(isurface).eq.0 )
                        if ( debug ) then
                           write(irefwr,*) 'i, isurface, ksurnr ', &
                                            i, isurface, ksurnr
                           write(irefwr,*) 'iface, iwork(iface) ', &
                                            iface, iwork(iface)
                        end if  ! ( debug )

                     end do  ! i = 1, numsurfaces

                  else if ( numsurfaces.lt.0 ) then

!                 --- Surface is single

                     ksurnr = -numsurfaces
                     single = .true.

                  end if  ! ( numsurfaces.gt.1 )
                  if ( debug ) write(irefwr,*) 'single ', single

                  if ( .not. single ) then

!                 --- None of the surfaces is single, create new surface
!                     nsurfsnew := nsurfsnew+1; ksurnr := nsurfsnew

                     nsurfsnew = nsurfsnew+1
                     if ( nsurfsnew.gt.10*nsurfs ) then

!                    --- array length too small

                        call errint ( nsurfsnew, 1 )
                        call errint ( 10*nsurfs, 2 )
                        call errchr ( 'ipoint', 1 )
                        call errsub ( 550, 2, 0, 1 )
                        go to 1000

                     end if  ! ( nsurfsnew.gt.10*nsurfs )
                     ksurnr = nsurfsnew

                  end if  ! ( single )

                  if ( icheck(ksurnr).eq.0 ) then

!                 --- icount := icount+1; iworksurfs(icount) := ksurnr

                     icount = icount+1
                     if ( icount.gt.leniworksurfs ) then

!                    --- array length too small

                        call errint ( icount, 1 )
                        call errint ( leniworksurfs, 2 )
                        call errchr ( 'iworksurfs', 1 )
                        call errsub ( 550, 2, 0, 1 )
                        go to 1000

                     end if
                     iworksurfs(icount) = ksurnr
                     icheck(ksurnr) = 1

                  end if  !( icheck(ksurnr).eq.0 )

                  if ( debug ) &
                     write(irefwr,*) 'icount, ksurnr ',icount, ksurnr

!                 --- For all elements in surface coupled to same surfaces as
!                       ifirst do

                  do iielem = ifirst, inelem

!                 --- iface := surffaces(istart+iielem)

                     iface = surffaces(istart+iielem)
                     if ( iwork(iface).eq.1 .and. numsurfaces.eq.1 ) &
                        then

!                    --- Only one surface

                        same = .true.

                     else if ( ksurnr.eq.nsurfsnew .and. &
                               iwork(iface).eq.-ksurnr ) then

!                    --- Same surface ksurnr

                        same = .true.

                     else if ( iwork(iface).lt.0 ) then

!                    --- Different surface ksurnr

                        same = .false.

                     else

                        if ( iwork(iface).ne.numsurfaces ) then

!                       --- Number of surfaces different from first face, &
!                           so not all surfaces the same

                           same = .false.

                        else

!                       --- Number of surfaces same as previous faces
!                           Check if it concerns the same surfaces

                           same = .true.
                           kstart = iwork1(iface)
                           do i = 1, numsurfaces

                              if ( icompare(iwork2(kstart+i-1)).ne.1 ) &
                                 then

!                             --- Surface numbers different from first face,
!                                 so not all surfaces the same

                                 same = .false.
                                 go to 200

                              end if  ! ( icompare(iwork2(kstart+i-1)).ne.1 )

                           end do  ! i = 1, numsurfaces

                        end if  ! ( iwork(iface).ne.numsurfaces )

                     end if  ! ( numsurfaces.eq.1 )

200                  if ( same ) then

!                    --- same surfaces; jcount := jcount+1;
!                        extsurffaces(jcount) := iface; iwork(iface) := -ksurnr

                        jcount = jcount+1
                        extsurffaces(jcount) = iface
                        iwork(iface) = -ksurnr

                     end if  ! ( same )

                  end do  ! iielem = ifirst+1, inelem

                  if ( .not. single ) then

!                 --- new surface;  ipoint(nsurfsnew) := jcount

                     if ( jcount.gt.lenextsurffac ) then

!                    --- array length too small

                        call errint ( jcount, 1 )
                        call errint ( lenextsurffac, 2 )
                        call errchr ( 'extsurffaces', 1 )
                        call errsub ( 550, 2, 0, 1 )
                        go to 1000

                     end if
                     ipoint(nsurfsnew) = jcount

                  end if  ! ( .not. single )

               else

!              --- ifirst = 0; finished = true

                  finished = .true.

               end if  ! ( ifirst.gt.0 )

            end do  ! while ( .not. finished )

         end if  ! ( ihelp(isurnr).eq.-10 )

         istart = istart+inelem  ! raise starting position

!        --- Clear array icheck

         do i = 1, icount
            icheck(iworksurfs(i)) = 0
         end do  ! i = 1, icount

      end do  ! isurnr = 1, nsurfsold

1000  call erclos ( 'mshsurfaces10' )

      if ( debug ) then

!     --- Debug information

         call prinin ( ipoint, nsurfsnew, 'ipoint' )
         call prinin ( ihelp, nsurfs, 'ihelp' )
         call prinin ( iworksurfs, icount, 'iworksurfs' )
         write(irefwr,*) 'End mshsurfaces10'

      end if

      end
