      subroutine mshsurfaces14 ( nsurfsold, nsurfsnew, kmeshsold, &
                                 kmeshsnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refill kmesh part s in case new surfaces have been created
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfsold, nsurfsnew, kmeshsold(*), kmeshsnew(*)

!     kmeshsnew      o    New array kmesh part s
!     kmeshsold      i    Old array kmesh part s
!     nsurfsnew      i    Number of surfaces in new array kmesh part s
!     nsurfsold      i    Number of surfaces in old array kmesh part s
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ishape, istartold, istartnew, ishift

!     i              Counting variable
!     ishape         Shape number of elements
!     ishift         Shift of position in new kmeshs compared to old one
!     istartnew      starting address in array kmeshsnew
!     istartold      starting address in array kmeshsold
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2651   Different shape numbers have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy pos. 1-nsurfsold+1 from kmeshsold to kmeshsnew
!     Set kmeshsnew(nsurfsold+2 ... nsurfs+1) equal to kmeshsold(nsurfsold+1)
!     Copy kmeshsold part 2 to kmeshsnew part 2
!     Extend kmeshsnew part 2 with information for new surfaces
!     Copy kmeshsold part 3 to kmeshsnew part 3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces14' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces14'
         write(irefwr,*)' Curves corresponding to old surfaces'
         call pr0012 ( nsurfsold, kmeshsold, kmeshsold )
         istartold = nsurfsold+2
         call prinin1 ( kmeshsold(istartold), nsurfsold, 8, &
                        'Rest of information per surface' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Copy pos. 1-nsurfsold+1 from kmeshsold to kmeshsnew

      ishift = 9*(nsurfsnew-nsurfsold)
      do i = 1, nsurfsold+1
         kmeshsnew(i) = kmeshsold(i)+ishift
      end do  ! i = 1, nsurfsold+1

!     --- Set kmeshsnew(nsurfsold+2 .. nsurfs+1) equal to kmeshsold(nsurfsold+1)

      do i = nsurfsold+2, nsurfsnew+1
         kmeshsnew(i) = kmeshsold(nsurfsold+1)+ishift
      end do  ! i = 1, nsurfsold+1

!     --- Copy kmeshsold part 2 to kmeshsnew part 2

      istartold = nsurfsold+1
      istartnew = nsurfsnew+1
      do i = 1, 8*nsurfsold
         kmeshsnew(istartnew+i) = kmeshsold(istartold+i)
      end do  ! i = 1, 8*nsurfsold

!     --- Extend kmeshsnew part 2 with information for new surfaces

      ishape = kmeshsold(istartold+2)  ! shape number for first surface
      do i = 1, nsurfsold-1

         if ( kmeshsold(istartold+2+8*i).ne.ishape ) then

!        --- Different shape numbers in surfaces

            call errint ( ishape, 1 )
            call errint ( i+1, 2 )
            call errint ( kmeshsold(istartold+2+8*i), 3 )
            call errsub ( 2651, 3, 0, 0 )

         end if  ! ( kmeshsold(istartold+2+8*i).ne.ishape )

      end do
      istartnew = istartnew+8*nsurfsold
      do i = nsurfsold+1, nsurfsnew

         kmeshsnew(istartnew+1) = 2        ! define surface type as general
         kmeshsnew(istartnew+2) = ishape
         do j = 3, 8
            kmeshsnew(istartnew+j) = 0
         end do  ! j = 3, 8
         istartnew = istartnew+8

      end do  ! i = nsurfsold+1, nsurfsnew

!     --- Copy kmeshsold part 3 to kmeshsnew part 3

      do i = kmeshsold(1)+1, kmeshsold(nsurfsold+1)
         kmeshsnew(i+ishift) = kmeshsold(i)
      end do  ! i = kmeshsold(1), kmeshsold(nsurfsold+1)-1

1000  call erclos ( 'mshsurfaces14' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*)' Curves corresponding to new surfaces'
         call pr0012 ( nsurfsnew, kmeshsnew, kmeshsnew )
         istartnew = nsurfsnew+2
         call prinin1 ( kmeshsnew(istartnew), nsurfsnew, 8, &
                        'Rest of information per surface' )
         write(irefwr,*) 'End mshsurfaces14'

      end if

      end
