      subroutine mshsurfaces7 ( kmeshn, nsurfs, nsurfsold, surffaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all surfaces that have faces in common with other surfaces
!     but can be considered as a subset of these surfaces as single
!     by setting ihelp(i) = 0 for these surfaces
!     Set ihelp(i) = 1 if surface i is compound and all faces point to
!     the same subsurfaces (rare)
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), nsurfs, nsurfsold, surffaces(*)

!     ihelp         i/o   integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!                         If surface i is compound but all faces refer to the
!                         same surfaces: ihelp(i) = 1
!     iwork          i    integer work array of length nfaces
!                         iwork(iface) contains the number of surfaces
!                         face iface is stored in
!     iwork1         i    integer work array of length nfaces
!                         contains for each face iface with iwork(iface)>1 a
!                         starting address in iwork2
!     iwork2         i    integer work array of length leniwork2
!                         contains for each face iface with iwork(iface)>1 the
!                         sequence numbers of the surfaces iface is lying in
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nsurfs         i    number of surfaces
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isurnr, inelem, iface, numsurfaces, i, &
              iielem, kstart, icompare(nsurfs), isurface
      logical same, single

!     i              Counting variable
!     icompare       Work array of length nsurfs
!                    icompare(i) = 0 if surface i is not coupled to the first
!                    face and icompare(i) = 1 otherwise
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inelem         Number of elements in a specific element group
!     istart         First position used at a part
!     isurface       surface sequence number
!     isurnr         Surface sequence number
!     kstart         Start value of parameter
!     numsurfaces    Number of surfaces coupled to face
!     same           If true the arrays are the same
!     single         If true the surface is a single surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all double surfaces do
!     if ( Double surface found ) then
!        check if all faces point to the same set of surfaces
!        if ( all faces refer to the same surfaces ) then
!           Check if surface is single and fill ihelp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces7'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- For all double surfaces do

      istart = 0
      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)
         same = .false.

         if ( ihelp(isurnr).eq.-1 ) then

!        --- Double surface found, check if all faces point to the same set of
!            surfaces

            same = .true.
            iface = surffaces(istart+1)  ! face number first element
            numsurfaces = iwork(iface)   ! number of surfaces corresponding to
                                         ! this face
            if ( numsurfaces.le.1 ) then

!           --- First face is single, so not all surfaces the same

               same = .false.
               go to 200

            end if  ! ( numsurfaces.le.1 )

!           --- Set icompare = 1 for all surfaces corresponding to this face

            do i = 1, nsurfs
               icompare(i) = 0
            end do  ! i = 1, nsurfs
            kstart = iwork1(iface)
            do i = 1, numsurfaces
               icompare(iwork2(kstart+i-1)) = 1
            end do  ! i = 1, numsurfaces

!           --- Now check other faces

            do iielem = 2, inelem

!           --- iface := surffaces(istart+iielem)

               iface = surffaces(istart+iielem)
               if ( iwork(iface).ne.numsurfaces ) then

!              --- Number of surfaces different from first face, &
!                  so not all surfaces the same

                  same = .false.
                  go to 200

               else

!              --- Number of surfaces same as previous faces
!                  Check if it concerns the same surfaces

                  kstart = iwork1(iface)
                  do i = 1, numsurfaces

                     if ( icompare(iwork2(kstart+i-1)).ne.1 ) then

!                    --- Surface numbers different from first face, &
!                        so not all surfaces the same

                        same = .false.
                        go to 200

                     end if  ! ( icompare(iwork2(kstart+i-1)).ne.1 )

                  end do  ! i = 1, numsurfaces

               end if  ! ( iwork(iface).ne.numsurfaces )

            end do  ! iielem = 1, inelem

         end if  ! ( ihelp(isurnr).eq.-1 )

200      if ( same ) then

!        --- All faces refer to the same surfaces
!            Check if one of them has been filled or not

            single = .true.
            do i = 1, numsurfaces
               isurface = iwork2(kstart+i-1)
               if ( ihelp(isurface).ne.-1 ) single = .false.
            end do  ! i = 1, numsurfaces

            if ( single ) then

!           --- Surface is single

               ihelp(isurnr) = 0

            else

!           --- Surface is compound surface consisting of numsurfaces-1
!               subsurfaces
!               Set ihelp equal to 1

               ihelp(isurnr) = 1

            end if  ! ( single ) then

         end if  ! ( same )
         istart = istart+inelem

      end do  ! isurnr = 1, nsurfsold

1000  call erclos ( 'mshsurfaces7' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces7'

      end if

      end
