      subroutine mshupdmeshinp1 ( iinput, nvolinsurfs, sumextrasurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 23-07-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of volumes with internal surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     adapt_mesh
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nvolinsurfs, sumextrasurfs

!     iinput        i/o   integer input array
!     nvolinsurfs    o    Number of volumes with internal surfaces
!     sumextrasurfs  o    sum of number of extra surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoice, krenum(10), i, iinp

!     i              Counting variable
!     ichoice        Dummy
!     iinp           position in array iinput
!     krenum         Dummy array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH000         Fill some parameters from array IINPUT
!     MSH050         Raise iinp as function of ivolm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshupdmeshinp1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshupdmeshinp1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nvolinsurfs = 0        ! number of volumes with internal surfaces
      sumextrasurfs = 0      ! sum of number of extra surfaces

      nvolms = iinput(5)     ! number of volumes
      if ( debug ) write(irefwr,1) 'nvolms', nvolms
      if ( nvolms==0 ) go to 1000   ! return if no volumes present

!     --- Find starting position of volumes in iinput

      ichoice = 0          ! dummy parameter, 0 to prevent extra activities

!     --- Find starting positions in iinput

      call msh000 ( iinput, krenum, ichoice )

      if ( debug ) write(irefwr,1) 'nvolms, iinp5', nvolms, iinp5

!     --- Loop over all volumes

      iinp = iinp5
      do i = 1, nvolms

         ivol = iinput(iinp)
         if ( debug ) write(irefwr,1) 'i, ivol', i, ivol
         if ( ivol==10 ) then

!        --- Volume with possibly internal surfaces found

            nextrasrfs = iinput(iinp+6)
            sumextrasurfs = sumextrasurfs+nextrasrfs
            if ( nextrasrfs>0 ) nvolinsurfs = nvolinsurfs+1
            if ( debug ) write(irefwr,1) 'nextrasrfs', nextrasrfs

         end if  ! ( ivol==10 )
         call msh050 ( iinput, iinp, ivol )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nvolinsurfs', nvolinsurfs
         write(irefwr,*) 'End mshupdmeshinp1'

      end if  ! ( debug )
      call erclos ( 'mshupdmeshinp1' )

      end
