      subroutine mshvlmtetracp ( rinput, work3, iinput, rinputcoarse )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 19-07-2008 New parameter list
!        version  1.0  date 05-05-2005
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates of user points from rinput into array work3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     tetrahedron
!     copy
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)
      double precision rinput(3,*), work3(4,*), rinputcoarse(*)

!     iinput         i    integer input array containing the user point numbers
!                         of the extra internal points
!     rinput         i    Real input array, see MESHBODY
!                         This part contains the coordinates of the user points
!     rinputcoarse   i    Real input array
!                         This part contains the coarseness of the user points
!     work3          o    array containing the coordinates as well as the
!                         local coarseness of internal points to be placed
!                         in the surface
!                         Length 3 x nextrapnts
!                         (1,i):  x-coordinate of point i
!                         (2,i):  y-coordinate of point i
!                         (3,i):  z-coordinate of point i
!                         (4,i):  coarseness of point i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iuserp
      double precision unitlength

!     i              Counting variable
!     iuserp         User point sequence number
!     unitlength     Unit length for coarseness
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvlmtetracp' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvlmtetracp'
         call prinin ( iinput, nextrapnts, 'user points' )
         write(irefwr,100) 'jcoars, nuspnt', jcoars, nuspnt
100      format ( a, 1x, (10i6) )
         call prinrl1 ( rinput, nuspnt, 3, &
                        'coordinates of user points' )
         if ( jcoars==1 ) &
            call prinrl ( rinputcoarse(2), nuspnt, 'coarseness' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( jcoars==1 ) unitlength = rinputcoarse(1)

!     --- Fill coordinates of extra points

      do i = 1, nextrapnts
         iuserp = iinput(i)
         work3(1,i) = rinput(1,iuserp)
         work3(2,i) = rinput(2,iuserp)
         work3(3,i) = rinput(3,iuserp)
         if ( jcoars==1 ) then
            work3(4,i) = rinputcoarse(iuserp+1)*unitlength
         else
            work3(4,i) = 0d0
         end if  ! ( jcoars==1 )
      end do  ! i = 1, nextrapnts

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( work3, nextrapnts, 4, &
                        'coordinates of user points' )
         write(irefwr,*) 'End mshvlmtetracp'

      end if
      call erclos ( 'mshvlmtetracp' )

      end
