      subroutine mshwrinterfc ( iinput, wrtrecord, irefmesh, jlast )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write parts of array iinput corresponding to interface elements
!     in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     interface_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh, jlast
      character (len=*) wrtrecord

!     iinput         i    integer input array for subroutine mesh
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
!     jlast          i    iinput(20) refers to new part in iinput
!     wrtrecord     i/o   Contains record to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indent, iinp, ncurvints, nsurfints, i, j, ilast

!     i              Counting variable
!     iinp           last position used in array iinput
!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     j              Counting variable
!     ncurvints      Number of curves with interface elements
!     nsurfints      Number of surfaces with interface elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshwrinterfc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwrinterfc'

      end if
      if ( ierror/=0 ) go to 1000

      ilast = 0
      indent = 3
      call iniputxt ( 'interface_elements', wrtrecord, ilast )
      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
      indent = 6
      iinp = iinput(jlast-1)-5
      ncurvints = iinput(iinp+1)
      nsurfints = iinput(iinp+2)
      iinp = iinp+3

!     --- write interface elements

      do i = 1, ncurvints
         call iniputxt ( 'ielm', wrtrecord, ilast )
         call iniputin ( iinput(iinp), wrtrecord, ilast )
         call iniputxt ( ' = curves ( ', wrtrecord, ilast )
         if ( iinput(iinp+1)>0 ) then
            call iniputxt ( 'shape = ', wrtrecord, ilast )
            call iniputin ( iinput(iinp+1), wrtrecord, ilast )
            call iniputxt ( ', ', wrtrecord, ilast )
         end if
         do j = 2, 3
            if ( iinput(iinp+j)<0 ) &
               call iniputxt ( '-', wrtrecord, ilast )
               call iniputxt ( 'c', wrtrecord, ilast )
               call iniputin ( abs(iinput(iinp+j)), wrtrecord, &
                     ilast )
               call iniputxt ( ', ', wrtrecord, ilast )

         end do
         ilast = ilast-2
         call iniputxt ( ' )', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         iinp = iinp+4

      end do

      do i = 1, nsurfints
         call iniputxt ( 'ielm', wrtrecord, ilast )
         call iniputin ( iinput(iinp), wrtrecord, ilast )
         call iniputxt ( ' = surfaces ( ', wrtrecord, ilast )
         call iniputxt ( 's', wrtrecord, ilast )
         call iniputin ( iinput(iinp+2), wrtrecord, ilast )
         call iniputxt ( ', s', wrtrecord, ilast )
         call iniputin ( iinput(iinp+3), wrtrecord, ilast )
         call iniputxt ( ' )', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         iinp = iinp+4

      end do  ! i = 1, nsurfints

1000  call erclos ( 'mshwrinterfc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshwrinterfc'

      end if

      end
