      subroutine seprdkmeshlmn ( irefc, ivers, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read kmesh parts l m and n from the file meshoutput
!     Old subroutine mshrdkmeshlmn
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, ivers, ihelp(10)

!     ihelp          i    Contains the input written by MSH059 on the second
!                         record after nvolms
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     ivers          i    Version sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios
      character (len=80) text

!     ios            IOSTAT variable, returns from reading or writing
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH023         Read KMESH part l from a file
!     MSH024         Read KMESH part m from a file
!     MSH025         Read KMESH part n from a file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     195   error in reading of co-ordinate records
!     196   end of file found when reading co-ordinate records
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprdkmeshlmn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprdkmeshlmn'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
100   format ( a )

      if ( nuspnt>0 ) then

!     --- nuspnt>0: kmesh part l

         allocate ( km(iseqkm)%kmeshl(nuspnt) )
         kmeshl => km(iseqkm)%kmeshl
         kmeshlglob => kmeshl
         kelml = nuspnt
         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( ios<0 ) then
               call errsub ( 196, 0, 0, 0 )
            else if ( ios>0 ) then
               call errint ( ios, 1 )
               call errsub ( 195, 1, 0, 0 )
            end if
            if ( debug ) write(irefwr,*) text
         end if
         call msh023 ( irefc, npoint, nuspnt, kmeshl )
      end if
      if ( ierror/=0 ) go to 1000

      if ( ncurvs>0 .and. (ihelp(1)==0 .or. nsurfs>0 .or. &
           ivers>=1 .and. ihelp(6)>0) ) then

!     --- ncurvs>0  read KMESH part m

         length = max(10*ncurvs+10000,npoint+10000)
         allocate ( kmeshm(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshm' )
         if ( ierror/=0 ) go to 1000

         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( ios<0 ) then
               call errsub ( 196, 0, 0, 0 )
            else if ( ios>0 ) then
               call errint ( ios, 1 )
               call errsub ( 195, 1, 0, 0 )
            end if
            if ( debug ) write(irefwr,*) text
         end if
         call msh024 ( irefc, npoint, kmeshm, ncurvs, length )
         allocate ( km(iseqkm)%kmeshm(length) )
         km(iseqkm)%kmeshm = kmeshm(1:length)
         kelmm = length

         deallocate ( kmeshm, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshm' )
         if ( ierror/=0 ) go to 1000

         kmeshm => km(iseqkm)%kmeshm
         kmeshmglob => kmeshm

      end if

      if ( nsurfs>0 .and. (ihelp(1)==0 .or. nvolms>0 .or. &
           ivers>=1 .and. ihelp(7)>0) ) then

!     --- nsurfs>0   read KMESH part n

         length = nsurfs*2+nelem*3+25000
         allocate ( kmeshn(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshn' )
         if ( ierror/=0 ) go to 1000

         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( ios<0 ) then
               call errsub ( 196, 0, 0, 0 )
            else if ( ios>0 ) then
               call errint ( ios, 1 )
               call errsub ( 195, 1, 0, 0 )
            end if
            if ( debug ) write(irefwr,*) text
         end if
         call msh025 ( irefc, npoint, kmeshn, nsurfs, length )
         allocate ( km(iseqkm)%kmeshn(length) )
         km(iseqkm)%kmeshn = kmeshn(1:length)
         kmeshnglob => kmeshn

         kelmn = length

         deallocate ( kmeshn, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshn' )
         if ( ierror/=0 ) go to 1000

         kmeshn => km(iseqkm)%kmeshn

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprdkmeshlmn'

      end if  ! ( debug )
      call erclos ( 'seprdkmeshlmn' )

      end
