      subroutine er0005 ( vector, npoint, number, kprobf, indprf, &
                          nunkp, text, layout, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.4  date 09-06-2010 Change of layout
!        version  5.3  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  5.2  date 30-04-1991  New type format
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   print unknown number from array vector
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character text *(*)
      logical compl
      double precision vector(*)
      integer npoint, number, kprobf(*), indprf, nunkp, layout

!     compl      i    indication whether the arithmetic is real (false)
!                     or complex (true)
!     indprf     i    indication if kprob part f has been filled (>0) or not (0)
!     kprobf     i    KPROB part f: contains the number of unknowns per point
!                     accumulated
!     layout     i    Layout to be used, see prinrv
!     npoint     i    Number of nodal points
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered. If the array is complex number=2*i-1
!                     corresponds to the real part of array i and number=2*i
!                     to the imaginary part of array i
!     nunkp      i    number of elements in a row
!     text       i    text to be printed in the heading
!     vector     i    array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iknpt, inusol, itel, iwork(10), n5, nh, nh1
      double precision work(10)

!     i          Counting variable
!     iknpt      Nodal point number
!     inusol     Position of unknown in array USOL
!     itel       last position used in iuser
!     iwork      integer work array
!     n5
!     nh         number of nodes along a subcurve
!     nh1
!     work       work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
  2   format(' vector ',a)
  3   format(5(2x,i5,' : ',f14.6))
  4   format(5(2x,i5,' : ',ss,1pe14.6e2))
  5   format(1x,5(ss,1pe14.6e2))
  6   format(1x,10(ss,1pe11.3e2))
  7   format(1x,10f11.3)
  8   format(4(1x,i6,':',f12.6))
  9   format(4(1x,i5,' : ',ss,1pe14.6e2))

      if (layout==4) then
         n5=1
      else if (layout==7 .or. layout==8) then
         n5=4
      else if(layout==1 .or. layout>=5) then
         n5=5
      else
         n5=10
      end if
      nh=number
      if(compl) nh=(nh+1)/2
      nh1=nh-number/2
      itel=1
      write(irefwr,2) text
      do iknpt=1,npoint
         if(indprf>0) goto 110

!        --- indprf = 0,  number of unknowns constant

          inusol=(iknpt-1)*nunkp
          goto 120

!         --- indprf>0

110       inusol=kprobf(iknpt)
          nunkp=kprobf(iknpt+1)-inusol
          if(nh>nunkp) goto 150
120     inusol=inusol+nh
        if(compl) inusol=2*inusol-nh1
        work(itel)=vector(inusol)
        iwork(itel)=iknpt
        itel=itel+1
        if(itel<=n5) goto 150

!       --- itel = n5+1

          if(layout==6) then
             write(irefwr,3) (iwork(i),work(i),i=1,n5)
          else if(layout==5) then
             write(irefwr,4) (iwork(i),work(i),i=1,n5)
          else if(layout==7) then
             write(irefwr,8) (iwork(i),work(i),i=1,n5)
          else if(layout==8) then
             write(irefwr,9) (iwork(i),work(i),i=1,n5)
          else if(layout==1) then
             write(irefwr,5) (work(i),i=1,n5)
          else if(layout==2) then
             write(irefwr,6) (work(i),i=1,n5)
          else if(layout==3) then
             write(irefwr,7) (work(i),i=1,n5)
          else if (layout==4) then
             write(irefwr,4) iwork(1),work(1)
          end if
          itel=1
150   end do
      if(itel==1) return
      itel=itel-1
      if(layout==6) then
         write(irefwr,3) (iwork(i),work(i),i=1,itel)
      else if(layout==5) then
         write(irefwr,4) (iwork(i),work(i),i=1,itel)
      else if(layout==7) then
         write(irefwr,8) (iwork(i),work(i),i=1,itel)
      else if(layout==8) then
         write(irefwr,9) (iwork(i),work(i),i=1,itel)
      else if(layout==1) then
         write(irefwr,5) (work(i),i=1,itel)
      else if(layout==2) then
         write(irefwr,6) (work(i),i=1,itel)
      else if(layout==3) then
         write(irefwr,7) (work(i),i=1,itel)
      end if
      end
