      subroutine er0007 ( ncrit, idegfd, usol, user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-1989
!
!   copyright (c) 1989-1999  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy information of the idegfd-th degree of freedom
!    from array usol into array user of length npoint.
!    Array user corresponds to the nodal points and contains utmost one
!    unknown per point.
!    Points that do not contain the degree of freedom idegfd get the
!    value zero.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncrit, idegfd
      double precision usol(*), user(npoint)

!     idegfd    i   Actual degree of freedom per point
!     ncrit     i   Number of degrees of freedom in a solution vector
!     user      o   Array of length npoint containing the idegfd-th
!                   degree of freedom of the solution vector, in the
!                   natural sequence.
!                   Points that do not correspond to this degree of
!                   freedom get the value 0 .
!     usol      i   Solution array of type 110 or 115 from which the
!                   degree of freedom idegfd must be extracted.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inunkp
      logical presen

!     i          Counting variable
!     inunkp      Maximum number of unknowns in a point
!     presen     Check if the degree of freedom is present in at least one point
!                (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array user

      do i = 1, npoint
         user(i) = 0d0
      end do
      if ( indprpi.ne.0 ) then

!     --- indprpi . 0

         do i = 1, npoint
            if ( kprobpi(i,idegfd).ne.0 ) &
               user(i) = usol ( kprobpi(i,idegfd) )
         end do
         if ( idegfd.gt.nphysi ) then
            call eropen ( 'er0007' )
            call errint ( idegfd, 1 )
            call errint ( inunkp, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0007' )
         end if
      else if ( indprfi.gt.0 ) then
         presen = .false.
         do i = 1, npoint
            inunkp = kprobfi(i+1) - kprobfi(i)
            if ( idegfd.le.inunkp ) then
               user(i) = usol(kprobfi(i)+idegfd)
               presen = .true.
            end if
         end do
         if ( .not. presen ) then
            call eropen ( 'er0007' )
            call errint ( idegfd, 1 )
            call errint ( inunkp, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0007' )
         end if

      else

!     --- copy

         do i = 1, npoint
            user(i) = usol(nphysi*(i-1)+idegfd)
         end do
      end if
      end
