      subroutine er00741 ( nnodes, inodes, m )
! ======================================================================
!
!        programmer    Guus Segal
!        version 2.0   date 06-02-1999 Extra parameters
!        version 1.0   date 15-11-1998
!
!
!   copyright (c) 1998-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of degrees of freedom per point in a set of nodes, &
!    stored in array inodes
!    If the number of degrees of freedom is not constant in this set a
!    negative value is returned containing the maximum number of degrees of
!    freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!    number_of_unknowns
!    variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, inodes(*), m

!     inodes     i    Integer array containing the node numbers in the set
!     nnodes     i    Number of nodes in the set
!     m          o    Number of degrees of freedom per point in the set
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, iphys, j
      logical varunkns

!     i          Counting variable
!     inode      Node sequence number
!     iphys      Number of unknowns in a point
!     j          Counting variable
!     varunkns   If true the number of unknowns per point is variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( indprfi = 0 ) then
!        m := nphysi
!     else
!        m := number of unknowns in first point
!        varunkns := false
!        For all nodes in set do
!           iphys := number of unknowns in point
!           if ( iphys # m ) then
!              varunkns := true
!              m := max(m,iphys)
!        if ( varunkns ) then
!           m := -m
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( indprfi.eq.0 .and. indprpi.eq.0 ) then

!     --- indprfi = 0

         m = nphysi

      else if ( indprpi.gt.0 ) then

!     --- indprpi > 0, use kprobpi

         inode = inodes(1)
         m = 0
         do j = 1, nphysi
            if ( kprobpi(inode,j).gt.0 ) m = j
         end do
         varunkns = .false.
         do j = 1, m
            if ( kprobpi(inode,j).eq.0 ) varunkns = .true.
         end do

!        --- For all nodes in set do

         do i = 2, nnodes

!        --- iphys := number of unknowns in point

            inode = inodes(i)
            iphys = 0
            do j = 1, nphysi
               if ( kprobpi(inode,j).gt.0 ) iphys = j
            end do
            if ( iphys.ne.m .and. iphys.gt.0 ) then

!           --- iphys # m

               varunkns = .true.
               m = max(m,iphys)

            end if
            if ( .not. varunkns ) then
               do j = 1, m
                  if ( kprobpi(inode,j).eq.0 ) varunkns = .true.
               end do
            end if

         end do
         if ( varunkns ) m = -m

      else

!     --- m := number of unknowns in first point

         inode = inodes(1)
         m = kprobfi(inode+1)-kprobfi(inode)
         varunkns = .false.

!        --- For all nodes in set do

         do i = 2, nnodes

!        --- iphys := number of unknowns in point

            inode = inodes(i)
            iphys = kprobfi(inode+1)-kprobfi(inode)
            if ( iphys.ne.m .and. iphys.gt.0 ) then

!           --- iphys # m

               varunkns = .true.
               m = max(m,iphys)

            end if

         end do
         if ( varunkns ) m = -m

      end if

      end
