      subroutine er0077 ( ichint, irule, ichfun, ndim, n, usolbn, &
                          coorbn, value, mcomp, mvalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 05-10-2011 Extra parameters
!        version  4.0  date 16-06-2005 Extra parameetr lniwrk
!        version  3.0  date 14-10-2002 Extra parameters kmesh, kmeshq
!
!   copyright (c) 1991-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual computation of boundary integrals over curves
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichint, irule, ichfun, ndim, n, mcomp, mvalue
      double precision coorbn(ndim,n)
      complex * 16 usolbn(mcomp,n), value(mvalue)

!     coorbn     i    array containing the co-ordinates of the curves
!     ichfun     i    indication of the value of f. Possibilities:
!                     0   f = 1
!                     >0  f = func ( ichfun, x, y, z )
!                         at this moment only the combination f#1 and ichint=1
!                         is permitted
!     ichint     i    indicates the kind of integral to be computed.
!                     possibilities:
!                      1  /  f u  ds
!                      2  /  f  u.n  ds
!                      3  /  u.t  ds
!                      4  /  n. sigma .n  ds
!                      5  /  n. sigma .t  ds
!                      6  /  n. sigma     ds
!                      7  /  u  n  ds
!                      8  sum f(i) u(i)
!                      9  /  f ds
!     irule      i    type of numerical integration.  possibilities:
!                      1  trapezoid rule,  cartesian co-ordinates
!                      2  trapezoid rule,  axisymmetric co-ordinates
!                      3  simpson's rule   cartesian co-ordinates
!                      4  simpson's rule   axisymmetric co-ordinates
!     mcomp      i    number of components
!     mvalue     i    Number of components used in integral
!     n          i    number of points along the boundary
!     ndim       i    Dimension of the space.
!     usolbn     i    array containing the solution along the curves
!     value     i/o   Value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER00771        actual computation of boundary integrals over curves
!                    trapezoid rule (complex)
!     ER00773        actual computation of boundary integrals over curves
!                    simpson's rule (complex)
!     ER00778        actual computation of boundary integrals over curves
!                    Special case: ichint = 8, summation over nodes (complex)
!     ER0079FUNC     Compute function f and store usolbn * f in usolbn (complex)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0077' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0077'
         write(irefwr,1) 'ichint, irule, ichfun, ndim, n, mcomp', &
                          ichint, irule, ichfun, ndim, n, mcomp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d15.7) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichint==8 ) then

!     --- ichint = 8, summation over nodes

         call er00778 ( ichfun, ndim, n, usolbn, &
                        coorbn, value, mcomp )

      else

!     --- ichint # 8, other cases

         if ( ichint<=2 .or. ichint==9 ) then

!        --- ichint = 1, 2 or 9; fill array f and store u f in usolbn

            call er0079func ( usolbn, coorbn, ndim, n, ichint, mcomp, &
                              ichfun )

         end if  ! ( ichint<=2 .or. ichint==9 )

         if ( irule==2 .or. irule==4 ) then

!        --- Axisymmetric coordinates: multiply usolbn by r

            do i = 1, mcomp
               usolbn(i,:) = usolbn(i,:)*coorbn(1,:)
            end do  ! i = 1, mcomp

         end if  ! ( irule==2 .or. irule==4 ) then

         select case ( irule )
         case(1, 2)

!        --- irule = 1, 2 trapezoid rule

            call er00771 ( ichint, irule, ndim, n, usolbn, &
                           coorbn, value, mcomp, mvalue )

         case(3, 4)

!        --- irule = 3, 4 simpson's rule

            call er00773 ( ichint, irule, ndim, n, usolbn, &
                           coorbn, value, mcomp, mvalue )

         end select  ! case ( irule )

      end if  ! ( ichint==8 )

1000  call erclos ( 'er0077' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'value', value(1)
         write(irefwr,*) 'End er0077'

      end if

      end
