      subroutine ernodsmaxmin ( usol, nnodes, nodes, inppri, rinpri)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 20-09-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Restrict the number of nodes in array nodes to the set
!     satisfying usol>=minvalue and/or usol<=maxvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     maximum
!     minimum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*), inppri(2)
      double precision usol(*), rinpri(2)

!     inppri         i    Array of size 2 referring to minimum and maximum
!                         respectively
!                         inppri(1) = 1: minimum is stored in rinpri(1)
!                         inppri(2) = 2: maximum is stored in rinpri(2)
!     nnodes        i/o   Number of nodes in array nodes
!     nodes         i/o   Contains the node numbers to be printed
!     rinpri         i    See inppri
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numnodes, nodcopy(nnodes)
      double precision minvalue, maxvalue, ucopy(nnodes)

!     i              Counting variable
!     maxvalue       Maximum value for printing
!     minvalue       Minimum value for printing
!     nodcopy        Copy of arrays nodes
!     numnodes       Number of nodal points in array nodes
!     ucopy          Copy of array usol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ernodsmaxmin' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ernodsmaxmin'
         call prinin ( inppri, 2, 'inppri' )
         call prinrl ( rinpri, 2, 'rinpri' )
         call prinin ( nodes, nnodes, 'nodes' )
         call prinrl ( usol, nnodes, 'usol' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inppri(1)>0 ) then

!     --- Minimum value is given

         minvalue = rinpri(inppri(1))
         numnodes = 0
         if (debug) write(irefwr,2) 'minvalue', minvalue

         do i = 1, nnodes

            if ( usol(i)>=minvalue ) then
               numnodes = numnodes+1
               nodcopy(numnodes) = nodes(i)
               ucopy(numnodes) = usol(i)

               if ( debug ) then
                  write(irefwr,1) 'node', nodes(i)
                  write(irefwr,2) 'value', usol(i)
               end if  ! ( debug )

            end if  ! ( usol(nodes(i))>=minvalue )

         end do  ! i = 1, nnodes

         nnodes = numnodes
         nodes(1:numnodes) = nodcopy(1:numnodes)
         usol(1:numnodes) = ucopy(1:numnodes)

      end if  ! ( inppri(1)==1 )

      if ( inppri(2)>0 ) then

!     --- Maximum value is given

         maxvalue = rinpri(inppri(2))
         numnodes = 0

         do i = 1, nnodes
            if ( usol(i)<=maxvalue ) then
               numnodes = numnodes+1
               nodcopy(numnodes) = nodes(i)
               ucopy(numnodes) = usol(i)
            end if  ! ( usol(nodes(i))>=minvalue )
         end do  ! i = 1, nnodes

         nnodes = numnodes
         usol(1:numnodes) = ucopy(1:numnodes)
         nodes(1:numnodes) = nodcopy(1:numnodes)

      end if  ! ( inppri(1)==1 )

1000  call erclos ( 'ernodsmaxmin' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         call prinrl ( usol, nnodes, 'usol' )
         write(irefwr,*) 'End ernodsmaxmin'

      end if  ! ( debug )

      end
