      subroutine errmcurnods ( inodes, numnodes, number, kprobf, ipoint, &
                               nparts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration
!        version  1.0  date 05-10-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     delete points in inoces where degrees of freedom number
!     is not available
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, inodes(numnodes), number, kprobf(*), nparts, &
              ipoint(nparts+1)

!     inodes        i/o   Contains node numbers
!     ipoint        i/o   Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nparts         i    number of parts the boundary consists of
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     numnodes      i/o   Number of nodal points in array inodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, iend, i, ipart, node, inunkp

!     i              Counting variable
!     iend           Last position used at a part
!     inunkp         Number of degrees of freedom in point
!     ipart          Sequence number of part
!     istart         First position used at a part
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'errmcurnods' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errmcurnods'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all parts

      numnodes = 0
      istart = 1
      do ipart = 1, nparts

         iend = ipoint(ipart+1)-1
         ipoint(ipart) = numnodes+1

!        --- Loop over all nodes in part

         do i = istart, iend

            node = inodes(i)  ! node number
            inunkp = kprobf(node+1)-kprobf(node) ! number of unknowns

            if ( number<=inunkp ) then

!           --- number<=inunp hence number of unknowns sufficient
!               store node

               numnodes = numnodes+1
               inodes(numnodes) = node

            end if  ! ( number<=inunkp )

         end do  ! i = istart, iend
         istart = iend+1

      end do  ! ipart = 1, nparts
      ipoint(nparts+1) = numnodes+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errmcurnods'

      end if  ! ( debug )
      call erclos ( 'errmcurnods' )

      end
