      subroutine initdr ( icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.4  date 10-10-2012 Change in comment
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Initialize parameters for subroutine TO0100 and store these parameters
!    in common CSUBWK. Perform some tests
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulebuild
      use sepmodulemain
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*)

!     icurvs   i  Integer array containing the curve numbers for the
!                 computation of the boundary integral. A negative curve
!                 number indicates that the curve must be computed in
!                 the reversed direction.
!                 The curves including their sign must form one contiguous curve
!                 The number of curves is stored in array iinder
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ichois, iclear, inumcv, &
              irulbn, iskip, itypbn, ivec

!     i              Counting variable
!     ichois         parameter ichois see iinder(2)
!     iclear         parameter clearvector see iinder(9)
!     inumcv         Number of curves used for the boundary integral
!     irulbn         Integration rule for boundary integral
!     iskip          parameter allelements see iinder(12)
!     itypbn         Type of boundary integral
!     ivec           parameter ivec see iinder(7)
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     129   Incorrect curve number found
!    1496   Type of boundary integral not yet available
!    1497   Type of integration rule not yet available
!    1498   inumcv has incorrect value
!    1730   The parameter ALLELEMENTS has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'initdr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from initdr'
         call prinin ( iinder, 20, 'iinder' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iskip  = iinder(12)
      if ( iskip<0 ) then
         call errint ( iskip, 1 )
         call errsub ( 1730, 1, 0, 0 )
      end if
      itypbn = iinder(13)
      irulbn = iinder(14)
      inumcv = iinder(15)
      if ( itypbn>0 ) then

!     --- The boundary integral must be computed

         if ( itypbn/=2 ) then
            call errint ( itypbn, 1 )
            call errsub ( 1496, 1, 0, 0 )
         end if
         if ( irulbn/=1 ) then
            call errint ( irulbn, 1 )
            call errsub ( 1497, 1, 0, 0 )
         end if
         if ( inumcv<=0 .or. inumcv>1000 ) then
            call errint ( inumcv, 1 )
            call errsub ( 1498, 1, 0, 0 )
            inumcv = 0
         end if
         do i = 1, inumcv
            if ( abs(icurvs(i))>ncurvs .or. icurvs(i)==0 ) then
               call errint ( icurvs(i), 1 )
               call errint ( ncurvs, 2 )
               call errint ( 0, 3 )
               call errsub ( 129, 3, 0, 0 )
             end if
         end do

      end if
      jcart = ks(isolut2d(iseqi1))%jcart

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End initdr'

      end if  ! ( debug )
      call erclos ( 'initdr' )

      end
