      subroutine intbndinp ( inpcop, nparm, iread, iintbn, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 23-09-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array inpcop with input for subroutine intbndbf
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     integral
!     read
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), icurvs(*), inpcop(*), iread, nparm

!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral.
!     iintbn         i    Contains information for the subroutine to compute the
!                         boundary integrals
!                         See the Programmers Guide for a description
!     inpcop         o    Copy of integer input array extended with defaults
!     iread          i    Defines if parameters should be read or
!                         are given in array IINTBN through the parameter list.
!                         See subroutine intbndbf
!     nparm          i    Number of parameters in array iintbn
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnr,  lniind, maxcal, ipos

!     ipos           Position of input arrays in sepmoduleinput
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     lniind         Length of array IINTBN
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     READBI         Read the input for the subroutine INTBND
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intbndinp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intbndinp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array inpcop

      inpcop(1) = 3
      inpcop(2:nparm) = 1
      inpcop(3) = 0
      inpcop(5) = ndim-1    ! ndim-1

      if ( iread==-1 ) then

!     --- iread = -1, fill inpcop from iintbn

         length = min(iintbn(1),nparm)
         inpcop(2:length) = iintbn(2:length)
         length = inpcop(7)
         if ( length>0 ) inpcop(nparm+1:nparm+length) = icurvs(1:length)

      else if ( iread==0 ) then

!     --- iread = 0, fill inpcop by reading from the standard input file

         iseqnr = 1
         call readbi ( inpcop(2), 1, iseqnr, inpcop(nparm+1) )

!        --- Adapt variables

         call inichinpsol ( inpcop(2), 6 )

      else

!     --- iread>0, fill inpcop by copying from IBUFFR

         iseqnr = iread
         call sepgetinpfinf ( 14, iseqnr, ipos, maxcal )

         inpcop(1) = nparm+100
         lniind = size( in(ipos)%iinput )
         inpcop(2:1+lniind) = in(ipos)%iinput

!        --- Adapt variables

         call inichinpsol ( inpcop(2), lniind )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intbndinp'

      end if  ! ( debug )
      call erclos ( 'intbndinp' )

      end
