      subroutine intcopcompl01 ( usol, iwork, icount, iunk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-12-2002 Extension of shapes
!        version  1.0  date 12-10-2001 Complex variant of intcop01
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   interpolate "intermediate" points
!   update array iwork and icount
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), icount, iunk
      double complex usol(*)

!     icount    i/o   number of points that have been filled in iwork
!     iunk       i    Sequence number of unknown
!     iwork     i/o   integer work array of length: number of nodes in new
!                     mesh
!                     If iwork(i) = 1, node has been treated, &
!                     if iwork(i) = 0 it is free
!     usol       i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, nodp, ielem, ikelmc, inpelm, itype, k, k1

!     ielem      Element number
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     inpelm     number of points per element
!     itype      Type number of element
!     k          Counting variable
!     k1         First point of curve in KMESH part m
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for ielgrp := 1 (1) nelgrp do
!        if ( itype#0 ) then
!           if ( iunk not in all nodes of ielgrp ) then
!              for ielem := 1 (1) nelem_ielgrp do
!                 interpolate points and update iwork and icount
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intcopcompl01' )

      indpra = kprobloc(14)
      ikelmc = 0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         itype = kprobc(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         if ( ishape==3 .or. ishape==5 .or. &
              ishape==11 .or. ishape==13 .or. ishape==21 .or. &
              itype==0 ) then

!        --- Type number 0, or element is linear

            ikelmc = ikelmc + inpelm*inelem

         else

!        --- Type number not 0, or element linear
!            Check if iunk present in all nodes

            if ( ishape==9 .or. ishape==10 ) then

!           --- Linear elements with extra points
!               If unknown present in vertices but not in centroids,
!               raise icount

               if ( kprobatmp(1)>=iunk .and. kprobatmp(inpelm)<iunk ) &
                  icount = icount + inelem
               ikelmc = ikelmc + inpelm*inelem

            else if ( ishape==4 .or. ishape==7 ) then

!           --- Quadratic element, check if unknown present in vertex
!               and not in mid point

               if ( kprobatmp(1)>=iunk .and. kprobatmp(2)<iunk ) then

!              --- Interpolate mid side points

                  do ielem = 1, inelem
                     do k = 2, 6, 2
                        nodp = kmeshc(ikelmc+k)
                        k1 = mod(k+1,6)
                        if ( iwork(nodp)==0 ) then
                           iwork(nodp) = 1
                           icount = icount + 1
                           usol(nodp) = 0.5d0 * &
                                         ( usol(kmeshc(ikelmc+k-1)) + &
                                           usol(kmeshc(ikelmc+k1)) )
                        end if
                     end do
                     ikelmc = ikelmc + inpelm
                  end do
               else
                  ikelmc = ikelmc + inpelm*inelem
               end if
               if ( ishape==7 ) then

!              --- Extended quadratic element
!                  If unknown present in vertices but not in centroids,
!                  raise icount

                  if ( kprobatmp(1)>=iunk .and. kprobatmp(inpelm)<iunk ) &
                     icount = icount + inelem

               end if

            else if ( ishape==6 ) then

!           --- Quadratic element, check if unknown present in vertex
!               and not in mid side points

               if ( kprobatmp(1)>=iunk .and. kprobatmp(2)<iunk ) then

!              --- Interpolate mid side points

                  do ielem = 1, inelem
                     do k = 2, 8, 2
                        nodp = kmeshc(ikelmc+k)
                        k1 = mod(k+1,8)
                        if ( iwork(nodp)==0 ) then
                           iwork(nodp) = 1
                           icount = icount + 1
                           usol(nodp) = 0.5d0 * &
                                         ( usol(kmeshc(ikelmc+k-1)) + &
                                           usol(kmeshc(ikelmc+k1)) )
                        end if
                      end do
                      ikelmc = ikelmc + inpelm
                  end do
               else
                  ikelmc = ikelmc + inpelm*inelem
               end if
               if ( kprobatmp(1)>=iunk .and. kprobatmp(9)<iunk ) then

!              --- Interpolate centroid

                  ikelmc = ikelmc - inpelm*inelem
                  do ielem = 1, inelem
                     nodp = kmeshc(ikelmc+9)
                     if ( iwork(nodp)==0 ) then
                        iwork(nodp) = 1
                        icount = icount + 1
                        usol(nodp) = 0.25d0 * ( &
                                        usol(kmeshc(ikelmc+2)) + &
                                        usol(kmeshc(ikelmc+4)) + &
                                        usol(kmeshc(ikelmc+6)) + &
                                        usol(kmeshc(ikelmc+8)) )
                     end if
                     ikelmc = ikelmc + inpelm
                  end do
               end if
            else

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

         end if
      end do

1000  call erclos ( 'intcopcompl01' )
      end
