      subroutine intcopcomplun ( usol1, usol2, nunkp, &
                                 minnode, maxnode, npointnw, nunks )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-06-2004 Minor change
!        version  1.0  date 12-10-2001 Complex variant of intcopun
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Interpolate usol1 into usol2 using the information stored in the arrays
!   imap and rmap
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointnw, nunkp, minnode, maxnode, &
              nunks
      double complex usol1(*), usol2(*)

!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     npointnw   i    Number of nodes in new mesh
!     nunkp      i    Maximum number of unknowns in a point
!     nunks      i    Number of unknowns per point in usol2
!     usol1      i    Array to be interpolated
!     usol2      i    Result of interpolation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iunkp, nodp, k
      double complex sum

!     iunkp          Sequence number of unknown
!     k              Counting variable
!     nodp           Nodal point sequence number
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ2        Print an integer two-dimensional array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For iunkp := 1 (1) nunks do
!       For nodp := minnode (1) maxnode do
!          sum := 0d0
!          For k := 1 (1) imap(nodp,1) do
!             sum := sum + rmap(nodp,k)*usol1((imap(nodp,k+1)-1)*nunkp+iunkp)
!          usol2((nodp-1)*nunks+iunkp) := sum
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intcopcomplun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intcopcomplun'
         call prinin2 ( imap, npointnw, 6, 'imap' )
         call prinrl2 ( rmap, npointnw, 4, 'rmap' )

      end if

      if ( nunkp.eq.1 ) then

!     --- nunkp = 1

         do nodp = minnode, maxnode

            sum = (0d0,0d0)
            do k = 1, imap(nodp,1)
               sum = sum + rmap(nodp,k)*usol1(imap(nodp,k+1))
            end do
            usol2(nodp) = sum

         end do  ! nodp = minnode, maxnode

      else

!     --- nunkp > 1

         do iunkp = 1, nunks
            do nodp = minnode, maxnode

               sum = (0d0,0d0)
               do k = 1, imap(nodp,1)
                  sum = sum + rmap(nodp,k) * &
                        usol1((imap(nodp,k+1)-1)*nunkp+iunkp)
               end do
               usol2((nodp-1)*nunks+iunkp) = sum

            end do

         end do

      end if

      call erclos ( 'intcopcomplun' )
      if ( debug ) write(irefwr,*) 'End intcopcomplun'

      end
