      subroutine intersect3dch2 ( ichoice_orig, trans, values, plane, &
                                  kdir )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill transformation matrix in array trans
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_orig, kdir
      double precision  trans(3,3), values(*), plane(*)

!     ichoice_orig   i    indicates how the origin must be computed
!                         See subroutine intp3dbf
!     kdir           i    Coordinate direction of largest component
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans         i/o   In this array the Transformation matrix from x,y,z
!                         co-ordinates to ksi-eta co-ordinates is stored as
!                         well as the origin in trans(.,3)
!                         The transformation of x,y,z to ksi,eta is defined by
!                         ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                         eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
!     values         i    Array containing information concerning the plane, &
!                         tangential vectors and origin
!                         See subroutine intp3dbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intersect3dch2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intersect3dch2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array orig with origin

      if ( ichoice_orig==0 ) then

!     --- Compute origin

         trans(1,3) = 0d0
         trans(2,3) = 0d0
         trans(3,3) = 0d0
         trans(kdir,3) = plane(4)/plane(kdir)

      else if ( ichoice_orig==1 ) then

!     --- Compute origin

         trans(1,3) = values(5)
         trans(2,3) = values(6)
         trans(3,3) = values(7)

      else

!     --- ichoice_orig has wrong value

         call errint ( ichoice_orig, 1 )
         call errint ( 0, 2 )
         call errint ( 7, 3 )
         call errchr ( 'ichoice_orig', 1 )
         call errsub ( 2147, 3, 0, 1 )

      end if

1000  call erclos ( 'intersect3dch2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intersect3dch2'

      end if  ! ( debug )

      end
