      subroutine intmap3d1 ( number, interpol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if interpolation of unknowns is necessary or that
!     copying suffices
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number
      logical interpol

!     interpol       o    If true interpolation may be necessary
!     number         i    indicates which is the first degree of freedom
!                         that must be interpolated in each point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap3d1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d1'
         write(irefwr,1) 'npoint, number', npoint, number
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize interpol

      interpol = .false.

!     --- Loop over nodes

      do i = 1, npoint

         if ( kprobpi(i,number)==0 ) then

!        --- Unknown not in node, interpolation may be necessary

            interpol = .true.
            go to 1000

         end if  ! ( kprobpi(i,number)==0 )

      end do  ! i = 1, npoint

1000  call erclos ( 'intmap3d1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'interpol', interpol
         write(irefwr,*) 'End intmap3d1'

      end if  ! ( debug )

      end
