      subroutine intmap3d11 ( kmeshc, inelem, nunkploc, &
                              number, usol, usolcop, ielgrp, inpelm, &
                              nvertex )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Elements of shape 11, 13, 17, 18 or 21
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     3d
!     tetrahedron
!     linear
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), inelem, nunkploc, number, &
              ielgrp, nvertex
      double precision usol(*), usolcop(nunkploc,*)

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    Number of nodes in elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         kmeshc is only related to this element group
!     number         i    Thefirst unknown to be copied has sequence number
!                         number
!     nunkploc       i    Number of unknowns in a point that must be copied
!     nvertex        i    Number of vertices
!     usol           i    Solution vector
!     usolcop       i/o   Solution vector to which the solution must be
!                         copied. usolcop has nunkploc unknowns per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielem, node

!     i              Counting variable
!     ielem          Element number
!     j              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     583   The required degree of freedom is not available in the vertices
!           of the element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap3d11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d11'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if unknown is present in all vertices

      do i = 1, nvertex

         node = kmeshc(i,1)

         do j = 1, nunkploc

            if ( kprobpi(node,j+number-1)==0 ) then

!           --- Unknown not present in vertex

               call errint ( j, 1 )
               call errint ( nunkploc+number-1, 2 )
               call errint ( ielgrp, 3 )
               call errwar ( 583, 3, 0, 0 )
               go to 1000

            end if  ! ( kprobpi(node,j+number-1)==0 ) then

         end do  ! j = 1, nunkploc

      end do  ! i = 1, nvertex

!     --- Copy solution in vertices

      do ielem = 1, inelem

         do i = 1, nvertex

            node = kmeshc(i,ielem)

            do j = 1, nunkploc
               usolcop(j,node) = usol(kprobpi(node,number-1+j))
            end do  ! j = 1, nunkploc

         end do  ! i = 1, nvertex

      end do  ! ielem = 1, inelem

1000  call erclos ( 'intmap3d11' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap3d11'

      end if  ! ( debug )

      end
