      subroutine pr2013 ( coor, npoint, ndim, xmin, xmax, ymin, ymax, &
                          zmin, zmax, d )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.3    date 29-01-1999 Include commons
!        version 1.2    date 19-04-1996 Layout
!        version 1.1    date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0    date 14-12-1987
!
!   copyright (c) 1987-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute xmin, xmax, ymin, ymax, zmin, zmax from array coor
!    The maximal distance between xmax and xmin, ymax and ymin resp.
!    zmax and zmin is stored in d
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      implicit none 
      double precision coor(*), xmin, xmax, ymin, ymax, zmin, zmax, d
      integer npoint, ndim

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     d          o    max ( xmax-xmin, ymax-ymin, zmax-zmin )
!     ndim       i    Dimension of the space.
!     npoint     i    Number of nodal points
!     xmax       i    Maximum x-value
!     xmin       i    Minimum x-value
!     ymax       i    Maximum y-value
!     ymin       i    Minimum y-value
!     zmax       i    Maximum z-value
!     zmin       i    Minimum z-value
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x, y, z

!     i          Counting variable
!     x          x co-ordinate of point
!     y          y co-ordinate of point
!     z          z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      xmin = rinfin
      xmax = -rinfin
      ymin = rinfin
      ymax = -rinfin
      zmin = rinfin
      zmax = -rinfin
      if(ndim.eq.1) then
         do 100 i = 1,npoint
            x = coor(i)
            xmin = min( xmin , x )
            xmax = max( xmax , x )
100      continue
         d = xmax-xmin
      else if(ndim.eq.2) then
         do 110 i = 1,npoint
            x = coor(2*i-1)
            y = coor(2*i  )
            xmin = min( xmin , x )
            xmax = max( xmax , x )
            ymin = min( ymin , y )
            ymax = max( ymax , y )
110      continue
         d = max ( xmax-xmin , ymax-ymin )
      else if(ndim.eq.3) then
         do 120 i = 1,npoint
            x = coor(3*i-2)
            y = coor(3*i-1)
            z = coor(3*i  )
            xmin = min( xmin , x )
            xmax = max( xmax , x )
            ymin = min( ymin , y )
            ymax = max( ymax , y )
            zmin = min( zmin , z )
            zmax = max( zmax , z )
120      continue
         d = max ( xmax-xmin , ymax-ymin , zmax-zmin )
      end if
      end
