      subroutine pr20143d ( nnodes, inodes, number, nunkp, usol, &
                            coor, heading, inppri, tang )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-07-2001 Extra parameter iformat
!        version  1.0  date 17-12-1999
!
!   copyright (c) 1999-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    This subroutine performs the printing of real arrays in 3D, tangential
!    or normal vector; complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), inodes(*), number, nunkp, nnodes
      double precision coor(3,*), tang(3,*)
      complex *16 usol(nunkp,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     inppri         i    Input array with respect to PRINVC (see PRINVC)
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nunkp          i    Maximum number of unknowns in a point
!     tang           i    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ichois, iquant, nodp, iformat
      complex *16 u1, u2, u3, ut
      logical heading

!     i              Counting variable
!     ichois         Choice parameter
!     iformat        Format used for printing
!                    1: e13.5
!                    2: e16.8
!     iquant         indicates which quantity must be considered
!                    Possible values:
!                    0:  The contents of the vector are considered as
!                        stored in the vector corresponding to IVECTR i.e.
!                        without transformation.
!                    1:  The normal component of a vector along the curve
!                        or surface is considered .
!                    2:  The tangential component(s) of a vector along the
!                        curve or surface is considered .
!     j              Counting variable
!     nodp           Nodal point sequence number
!     u1             First component of velocity vector
!     u2             Second component of velocity vector
!     u3             Third component of velocity vector
!     ut             Tangential component of velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ichois = inppri(2)
      iquant = inppri(4)
      heading = inppri(8)==0
      iformat = inppri(10)
      if ( ichois==0 ) then

!     --- ichois = 0, do not print co-ordinates

         if ( iquant==1 ) then

!        --- normal vector

            if ( heading ) write(irefwr,100)
100         format('node      normal components of vector'/)
            do i = 1, nnodes
               nodp = i!nodes(i)
               ut = usol(number  ,nodp)*tang(1,i) + &
                    usol(number+1,nodp)*tang(2,i) + &
                    usol(number+2,nodp)*tang(3,i)
               u1 = usol(number,nodp)-ut*tang(1,i)
               u2 = usol(number+1,nodp)-ut*tang(2,i)
               u3 = usol(number+2,nodp)-ut*tang(3,i)
               if ( iformat==1 ) then
                  write(irefwr,110) inodes(i), u1, u2, u3
               else
                  write(irefwr,111) inodes(i), u1, u2, u3
               end if
110            format((1x,i5,':',ss, &
                      3('(',1pe13.5e2,',',1pe13.5e2,1x,')')))
111            format((1x,i5,':', &
                      3('(',1pe16.8e2,',',1pe16.8e2,1x,')')))
            end do

         else

!        --- tangential vector and components

            if ( heading ) write(irefwr,120)
120         format(' node     tangential component  tangential vector'/)
            do i = 1, nnodes
               nodp = i!nodes(i)
               ut = usol(number  ,nodp)*tang(1,i) + &
                    usol(number+1,nodp)*tang(2,i) + &
                    usol(number+2,nodp)*tang(3,i)
               u1 = ut*tang(1,nodp)
               u2 = ut*tang(2,nodp)
               u3 = ut*tang(3,nodp)
               if ( iformat==1 ) then
                  write(irefwr,130) inodes(i), ut, u1, u2, u3
               else
                  write(irefwr,131) inodes(i), ut, u1, u2, u3
               end if
130            format((1x,i5,':',ss, &
                      4('(',1pe13.5e2,',',1pe13.5e2,1x,')')))
131            format((1x,i5,':',ss, &
                      4('(',1pe16.8e2,',',1pe16.8e2,1x,')')))
            end do

         end if

      else

         if ( iquant==1 ) then

!        --- normal vector

            if ( heading ) write(irefwr,200)
200         format('node      x        normal components of vector'/)
            do i = 1, nnodes
               nodp = i!nodes(i)
               ut = usol(number  ,nodp)*tang(1,i) + &
                    usol(number+1,nodp)*tang(2,i) + &
                    usol(number+2,nodp)*tang(3,i)
               u1 = usol(number,  nodp)-ut*tang(1,i)
               u2 = usol(number+1,nodp)-ut*tang(2,i)
               u3 = usol(number+2,nodp)-ut*tang(3,i)
               if ( iformat==1 ) then
                  write(irefwr,210) inodes(i),(coor(j,inodes(i)),j=1,3), &
                                    u1, u2, u3
               else
                  write(irefwr,211) inodes(i),(coor(j,inodes(i)),j=1,3), &
                                    u1, u2, u3
               end if
210            format((1x,i5,':',ss, &
                      6('(',1pe13.5e2,',',1pe13.5e2,1x,')')))
211            format((1x,i5,':',ss, &
                      6('(',1pe16.8e2,',',1pe16.8e2,1x,')')))
            end do

         else

!        --- tangential vector and components

            if ( heading ) write(irefwr,220)
220         format(' node       x     ', &
                   '  tangential component  tangential vector'/)
            do i = 1, nnodes
               nodp = i!nodes(i)
               ut = usol(number  ,nodp)*tang(1,i) + &
                    usol(number+1,nodp)*tang(2,i) + &
                    usol(number+2,nodp)*tang(3,i)
               u1 = ut*tang(1,i)
               u2 = ut*tang(2,i)
               u3 = ut*tang(3,i)
               if ( iformat==1 ) then
                  write(irefwr,230) inodes(i),(coor(j,inodes(i)),j=1,3), &
                                    ut, u1, u2, u3
               else
                  write(irefwr,231) inodes(i),(coor(j,inodes(i)),j=1,3), &
                                    ut, u1, u2, u3
               end if
230            format((1x,i5,':',ss, &
                      7('(',1pe13.5e2,',',1pe13.5e2,1x,')')))
231            format((1x,i5,':',ss, &
                      7('(',1pe16.8e2,',',1pe16.8e2,1x,')')))
            end do

         end if

      end if

      end
