      subroutine pr2026res ( lenkmtri, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of array kmeshtri, which is used to subdivide an
!     existing 2D mesh into three-point triangles
!     Special case where we restrict ourselves to some surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkmtri, icurvs(*)

!     icurvs         i    Integer array containing the user points, curve or
!                         surface numbers that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve is used. The curve number is
!                              stored in icurvs(2)
!                         -1   The solution is printed along the curves
!                              icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                              icrv2 in icurvs(3).
!                         n>0  The curve numbers numbers are stored in
!                              icurvs(2) to icurvs(n+1). That sequence is used.
!     lenkmtri       o    Length of array kmeshtri
!                         If 0, no subdivision is required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, isurnr

!     isurnr         Surface sequence number
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR2026RES1     Raise length of array kmeshtri, for one surface only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2764   Integer parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!    lenkmtri := 0
!    For all surfaces in icurvs do
!       Raise lenkmtri
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2026res' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2026res'

      end if
      if ( ierror/=0 ) go to 1000
      lenkmtri = 0

!     --- For all surfaces in icurvs do

      if ( icurvs(1)==0 ) then

!     --- icurvs(1) = 0, only one surface

         isurnr = icurvs(2)
         call pr2026res1 ( lenkmtri, kmesht(1,isurnr), &
                           kmesht(2,isurnr) )

      else if ( icurvs(1)==-1 ) then

!     --- icurvs(1) = -1, set of contiguous surfaces

         do isurnr = icurvs(2), icurvs(3)
            call pr2026res1 ( lenkmtri, kmesht(1,isurnr), &
                              kmesht(2,isurnr) )
         end do  ! isurnr = icurvs(2), icurvs(3)

      else if ( icurvs(1)>0 ) then

!     --- icurvs(1)>0, set of surfaces

         do j = 1, icurvs(1)
            isurnr = icurvs(1+j)
            call pr2026res1 ( lenkmtri, kmesht(1,isurnr), &
                              kmesht(2,isurnr) )
         end do  ! j = 1, icurvs(1)

      else

!     --- other value of icurvs(1), not implemented

         call errint ( icurvs(1), 1 )
         call errchr ( 'icurvs(1)', 1 )
         call errsub ( 2764, 1, 0, 1 )

      end if  ! ( icurvs(1)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pr2026res'

      end if
      call erclos ( 'pr2026res' )

      end
