      subroutine pr2032 ( kmeshtri, coorol, coornw, nelem, &
                          ijcrol, npntij, pntclij, nx, ny, nz, &
                          npointnw, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 02-03-2012 Use prgeomtetrahed
!        version  1.4  date 18-06-2004 Skip points with small contribution
!        version  1.3  date 20-08-2003 Correction cell address
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute mapping for all new points that are present in this part of
!   the mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kmeshtri(4,nelem), nx, ny, nz, ijcrol(3,*), &
              npointnw, npntij(*), pntclij(*)
      double precision coorol(3,*), coornw(3,*), eps

!     coornw     i    Array containing the co-ordinates of mesh 2
!     coorol     i    Array containing the co-ordinates of mesh 1
!     eps        i    margin parameter
!     ijcrol     i    Contains the ij cell number of the co-ordinates of mesh2
!                     ijcrol(1,k) contains the i number of coorol_k
!                     ijcrol(2,k) contains the j number of coorol_k
!     kmeshtri   i    Kmesh part c for triangular mesh
!     nelem      i    Number of elements
!     npntij     i    Contains number of coordinates in ij-cell accumulated
!                     npntij(0) = 0
!                     npntij((k-1)*ny*nx+(j-1)*ny+i)- &
!                     npntij((k-1)*ny*nx+(j-1)*ny+i)
!                     is number of points in cell ijk
!     npointnw   i    Number of nodes in new mesh
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in y direction
!     pntclij    i    Contains the node numbers per cell
!                     The sequence is defined by npntij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xmin, xmax, ymin, ymax, zmin, zmax, &
                       xh(4), yh(4), zh(4), dphidx(4,3), x, y, z
      integer i, j, k, ielem, inode, imin, imax, jmin, jmax, ih, ij, &
              nnodes, m, kmin, kmax, ih1
      logical check, geom

!     check          Indicates if there are new nodes in the triangle
!     dphidx         Contains the derivatives of the basis functions
!     geom           If true the geometrical quantities of the element must be
!                    compute
!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ij             help parameter
!     imax           Maximum i-value of cells
!     imin           Minimum i-value of cells
!     inode          Node sequence number
!     j              Counting variable
!     jmax           Maximum j-value of cells
!     jmin           Minimum j-value of cells
!     k              Counting variable
!     kmax           Maximum j-value of cells
!     kmin           Minimum j-value of cells
!     m              General loop variable
!     nnodes         Number of nodes in isolnw
!     x              x co-ordinate of point
!     xh             x-coordinates of vertices
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     yh             y-coordinates of vertices
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zh             z-coordinates of vertices
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRGEOMTETRAHED Compute geometrical quantities of tetrahedron
!     PRINTERPOLNOD3 Check if node is in element, if so interpolate (3D)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all triangles in sub mesh do
!        Compute minimum and maximum cell sequence numbers
!        If ( there are new nodes in this triangle ) then
!           Compute xmin, xmax, ymin, ymax of triangle
!           geom := false
!           For all nodes inode in the cells corresponding to the triangle do
!             if ( imap(inode,1)=0 ) then
!                If ( node in rectangle (xmin,xmax) x (ymin,ymax) ) then
!                   if ( not geom ) then
!                      Compute geometrical quantities
!                      geom := true
!                   compute baricentric co-ordinates of inode
!                   If ( all baricentric co-ordinates in [0,1] ) then
!                      Compute interpolation
!                      Fill imap and rmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2032' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2032'
         write(irefwr,*) 'nelem ', nelem
         write(irefwr,*) 'nx, ny, nz ', nx, ny, nz

      end if

!     --- For all triangles in sub mesh do

      do ielem = 1, nelem

!     --- Compute minimum and maximum cell sequence numbers

         imin = nx+1
         imax = 1
         jmin = ny+1
         jmax = 1
         kmin = nz+1
         kmax = 1
         do i = 1, 4
            inode = kmeshtri(i,ielem)
            if ( debug ) &
               write(irefwr,*) 'i, inode, ielem ', i, inode, ielem
            imin = min(imin,ijcrol(1,inode))
            imax = max(imax,ijcrol(1,inode))
            jmin = min(jmin,ijcrol(2,inode))
            jmax = max(jmax,ijcrol(2,inode))
            kmin = min(kmin,ijcrol(3,inode))
            kmax = max(kmax,ijcrol(3,inode))
         end do
         if ( debug ) &
            write(irefwr,*) 'imin, imax, jmin, jmax, kmin, kmax ', &
                             imin, imax, jmin, jmax, kmin, kmax

!        --- Check if there are new nodes in this triangle

         check = .false.
         do k = kmin, kmax
            ih1 = (k-1)*ny*nx
            do j = jmin, jmax
               ih = (j-1)*ny+ih1
               do i = imin, imax
                  ij = ih+i
                  if ( debug ) write(irefwr,*) 'i,j,k,ij, num nodes ', &
                     i,j,k,ij,npntij(ij+1)-npntij(ij)
                  if ( npntij(ij+1)-npntij(ij)>0 ) then

!                 --- There is at least one node in the range

                     check = .true.
                     go to 100

                  end if
               end do
            end do
         end do
100      if ( check ) then

!        --- Compute xmin, xmax, ymin, ymax, zmin, zmax of tetrahedron

            xmin = coorol(1,kmeshtri(1,ielem))
            ymin = coorol(2,kmeshtri(1,ielem))
            zmin = coorol(3,kmeshtri(1,ielem))
            xmax = xmin
            ymax = ymin
            zmax = zmin
            do i = 2, 4
               xmin = min(coorol(1,kmeshtri(i,ielem)), xmin)
               xmax = max(coorol(1,kmeshtri(i,ielem)), xmax)
               ymin = min(coorol(2,kmeshtri(i,ielem)), ymin)
               ymax = max(coorol(2,kmeshtri(i,ielem)), ymax)
               zmin = min(coorol(3,kmeshtri(i,ielem)), zmin)
               zmax = max(coorol(3,kmeshtri(i,ielem)), zmax)
            end do
            geom = .false.
            xmin = xmin-eps
            xmax = xmax+eps
            ymin = ymin-eps
            ymax = ymax+eps
            zmin = zmin-eps
            zmax = zmax+eps
            if ( debug ) &
               write(irefwr,*) 'xmin, xmax, ymin, ymax, zmin, zmax ', &
                                xmin, xmax, ymin, ymax, zmin, zmax

!           --- For all nodes inode in the cells corresponding to the
!               tetrahedron do

            do k = kmin, kmax
               ih1 = (k-1)*ny*nx
               do j = jmin, jmax
                  ih = (j-1)*ny+ih1
                  do i = imin, imax
                     ij = ih+i
                     nnodes = npntij(ij+1)-npntij(ij)
                     do m = 1, nnodes
                        inode = pntclij(npntij(ij)+m)
                        if ( imap(inode,1)==0 ) then

!                       --- Node has not yet been treated

                            x = coornw(1,inode)
                            y = coornw(2,inode)
                            z = coornw(3,inode)
                            if ( x>=xmin .and. x<=xmax .and. &
                                 y>=ymin .and. y<=ymax .and. &
                                 z>=zmin .and. z<=zmax ) then

!                           --- node in brick
!                               (xmin,xmax)x(ymin,ymax)x(zmin,zmax)

                              if ( .not. geom ) then

!                             --- Compute geometrical quantities

                                 call prgeomtetrahed ( coorol, &
                                                       xh, yh, zh, &
                                                      kmeshtri(1,ielem), &
                                                       dphidx )
                                 if ( ierror/=0 ) go to 1000

                                 geom = .true.

                              end if

!                             --- compute baricentric co-ordinates of inode

                              call printerpolnod3 ( inode, coornw, dphidx, &
                                                    npointnw, xh, yh, &
                                                    zh, ielem, &
                                                    kmeshtri(1,ielem) )

                           end if
                        end if
                     end do
                  end do
               end do
            end do
         end if
      end do

1000  call erclos ( 'pr2032' )
      if ( debug ) write(irefwr,*) 'End pr2032'

      end
