      subroutine pr2034 ( ijkcoor, minnode, maxnode, npntij, nx, ny,nz )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   14-02-1998
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute number of nodes for each ijk combination and store in array
!    npntij. This array is accumulated
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ijkcoor(3,*), minnode, maxnode, npntij(0:*), nx, ny,nz

!     ijkcoor    i    Contains the ijk cell number of the co-ordinates
!                     ijkcoor(1,l) contains the i number of coor_l
!                     ijkcoor(2,l) contains the j number of coor_l
!                     ijkcoor(3,l) contains the k number of coor_l
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     npntij     o    Contains number of coordinates in ij-cell accumulated
!                     npntij(0) = 0
!                     npntij((k-1)*ny*nx+(j-1)*ny+i)- &
!                     npntij((k-1)*ny*nx+(j-1)*ny+i)
!                     is number of points in cell ijk
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, nodp

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array npntij
!    For all nodes in range do
!       (i,j) := ijkcoor(.,nodp)
!       npntij((k-1)*ny*nx+(j-1)*ny+i) := npntij((k-1)*ny*nx+(j-1)*ny+i)+1
!    Accumulate npntij
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Clear array npntij

      do i = 0, nx*ny*nz
         npntij(i) = 0
      end do

!     --- For all nodes in range do

      do nodp = minnode, maxnode

!     --- (i,j,k) := ijkcoor(.,nodp)

         i = ijkcoor(1,nodp)
         j = ijkcoor(2,nodp)
         k = ijkcoor(3,nodp)

!        --- npntij((k-1)*ny*nx+(j-1)*ny+i) := npntij((k-1)*ny*nx+(j-1)*ny+i)+1

         npntij((k-1)*ny*nx+(j-1)*ny+i) = &
            npntij((k-1)*ny*nx+(j-1)*ny+i)+1

      end do

!     --- Accumulate npntij

      do i = 1, nx*ny*nz
         npntij(i) = npntij(i-1)+npntij(i)
      end do

      end
