      subroutine pr2036 ( lenkmtri )
! ======================================================================
!
!        programmer     Guus Segal
!        version  1.1  date 13-01-2003 Extension for prisms
!        version 1.0    date   14-02-1998
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute length of array kmeshtri, which is used to subdivide an
!    existing 3D mesh into four-point tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkmtri

!     lenkmtri   o    Length of array kmeshtri
!                     If 0, no subdivision is required
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp
      logical subdiv

!     ielgrp     element group number
!     subdiv     If true the elements must be subdivided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1586   Subdivision of elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!    subdiv := false
!    lenkmtri := 0
!    For all element groups do
!       inelem := number of elements in element group
!       if ( ishape#3 ) then
!          raise lenkmtri
!          subdiv := true
!       else
!          raise lenkmtri
!    if ( not subdiv ) then
!       lenkmtri := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      subdiv = .false.

!     --- For all element groups do

      lenkmtri = 0
      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         if ( ishape==11 ) then

!        --- inpelm=11 Copy kmeshc to kmeshtri

            lenkmtri = lenkmtri+4*inelem

         else if ( ishape==18 ) then

!        --- ishape=18 Skip centroid

            lenkmtri = lenkmtri+4*inelem
            subdiv = .true.

         else if ( ishape==13 .or. ishape==17 ) then

!        --- inpelm=8 Subdivide elements into 6 tetrahedrons

            lenkmtri = lenkmtri+24*inelem
            subdiv = .true.

         else if ( ishape==12 .or. ishape==15 .or. ishape==16 ) then

!        --- inpelm=10, 14, 15 Subdivide elements into 8 tetrahedrons

            lenkmtri = lenkmtri+48*inelem
            subdiv = .true.

         else if ( ishape==14 ) then

!        --- inpelm=27 Subdivide elements into 48 tetrahedrons

            lenkmtri = lenkmtri+192*inelem
            subdiv = .true.

         else if ( ishape==21 ) then

!        --- ishape=21: linear prism
!            Subdivide elements into 3 tetrahedrons

            lenkmtri = lenkmtri+12*inelem
            subdiv = .true.

         else if ( ishape<=10 ) then

!        --- Surface elements are skipped

            subdiv = .true.

         else

!        --- Other types have not yet been implemented

            call eropen ( 'pr2036' )
            call errint ( ishape, 1 )
            call errsub ( 1586, 1, 0, 0 )
            call erclos ( 'pr2036' )

         end if

      end do

      if ( nsplit==1 .and. .not. subdiv ) lenkmtri = 0

      end
