      subroutine pravsmesh ( iref, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 13-04-2007
!
!   copyright (c) 2007-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write mesh information to file iref in AVS format
!
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     mesh
!     write
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref, nunkp

!     iref           i    File reference number
!     nunkp          i    Number of unknowns to be written in avs file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRAVS3         Write mesh information to file iref in AVS format
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set constants
!     Activate arrays
!     Compute starting addresses
!     write mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pravsmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pravsmesh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(iref,200) npoint, nelem, nunkp
200   format ( 3i8, ' 0 0' )

!     --- write mesh

      call pravs3 ( iref, coor, ndim, npoint, nelem, npelm, kmeshc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pravsmesh'

      end if  ! ( debug )
      call erclos ( 'pravsmesh' )

      end subroutine pravsmesh
