      subroutine prentropy ( isolpres, isoldens, isolentropy, ipost, rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 31-05-2000
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the entropy in all points of the mesh, from the
!    given pressure and density
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isolpres, isoldens, isolentropy, ipost(*)
      double precision rpost(*)

!     ipost          i    Integer array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     isoldens       i    Array of the structure of isol referring to density
!     isolentropy    i    Array of the structure of isol referring to the
!                         entropy
!     isolpres       i    Array of the structure of isol referring to the
!                         pressure
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     PRENTROPY1     Actual computation of entropy
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     Create or activate stagnation pressure array
!     Compute starting addresses
!     Compute stagnation pressure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Create or activate stagnation pressure array

      pressure => ks(isolpres)%sol
      density => ks(isoldens)%sol

      call sepcreasolbf1 ( isolentropy, ks(isoldens)%typevec, &
                           ks(isoldens)%ivec, 'entropy', ks(isoldens)%compl )

!     --- Compute entropy in all vertices

      call prentropy1 ( npoint, pressure, density, usoltemp, ipost, rpost )

      end
