      subroutine prfillkprob2d ( kprob2d, kmesh2d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob2d for a 3d mesh intersected by a plane
!     Simple situation only
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     intersection
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob2d, kmesh2d

!     kmesh2d        i    Standard SEPRAN array containing information of the
!                         2d mesh
!     kprob2d        o    Standard SEPRAN array containing information of the
!                         2d problem description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nnodes, nelem2d

!     i              Counting variable
!     nelem2d        Number of elements in 2d mesh
!     nnodes         Number of nodes in 2d mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillkprob2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillkprob2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      iseqkm = kmesh2d
      npelm = km(iseqkm)%kmesh(4)   ! number of points per element
      nnodes = km(iseqkm)%kmesh(8)  ! number of elements
      nelem2d = km(iseqkm)%kmesh(9) ! number of nodes

!     --- Fill kprob

      call sepcrprobinfo ( kprob2d, 1 )
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc

!     --- Clear array

      kprobloc(4) = 1  ! nunkp
      kprobloc(5) = nnodes  ! nusol
      kprobloc(6) = 3  ! ndtvec
      kprobloc(7) = 1  ! ntype
      kprobloc(8) = 1  ! nelgrp
      kprobloc(9) = 0  ! nkbnd
      kprobloc(10) = 0  ! nbound
      kprobloc(30) = nelem2d  ! nelem
      kprobloc(31) = 2  ! ndim
      kprobloc(33) = 1  ! nphys
      kprobloc(39) = npelm  ! number of points per element
      kprobloc(40) = 1  ! number of problems

!     --- Fill kprob c

      allocate ( kprobpart%kprobc(1) )
      kprobpart%kprobc(1) = 1   !itype
      kprobloc(16) = 1

!     --- Fill kprob e

      ndtvec = 3
      kelgrp = npelm
      allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe(kelgrp,ndtvec), &
                 kp(iseqkp)%kprobsub(iprob)%kprobe2(ndtvec), &
                 kp(iseqkp)%kprobsub(iprob)%kprobe3(ndtvec) )
      kprobe2 => kprobpart%kprobe2
      kprobe3 => kprobpart%kprobe3
      kprobe => kprobpart%kprobe
      kprobe(:,1) = 3  ! ivec = 1, 3 unknowns per point
      kprobe(:,2) = 4  ! ivec = 2, 4 unknowns per point
      kprobe(:,3) = 6  ! ivec = 3, 6 unknowns per point
      kprobe2(1) = -3  ! ivec = 1
      kprobe2(2) = -4  ! ivec = 2
      kprobe2(3) = -6  ! ivec = 3
      kprobloc(18) = kelgrp*ndtvec

!     --- Fill iinputkp

      allocate ( kp(iseqkp)%iinputkp(19+3*npelm) )
      iinputkp => kp(iseqkp)%iinputkp
      iinputkp = 0
      iinputkp = 1   ! ntype
      iinputkp(15) = 1  ! ivers
      iinputkp(16) = 1  ! itype
      iinputkp(17) = 1  ! inunkp
      iinputkp(18) = 3  ! ndtvec
      iinputkp(19) = 0  ! isear
      do i = 1, npelm
         iinputkp(19+i) = 3          ! ivec = 1: nunkp = 3
         iinputkp(19+i+npelm) = 4    ! ivec = 2: nunkp = 4
         iinputkp(19+i+2*npelm) = 6  ! ivec = 3: nunkp = 6
      end do  ! i = 1, npelm
      kprobloc(41) = 19+3*npelm

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillkprob2d'

      end if  ! ( debug )
      call erclos ( 'prfillkprob2d' )

      end
