      subroutine prgeomtetrahed ( coor, xh, yh, zh, kmeshtri, dphidx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute geometrical quantities of tetrahedron
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtri(4)
      double precision coor(3,*), dphidx(4,3), xh(4), yh(4), zh(4)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!                         Contains coordinates of mesh in which interpolation
!                         must take place
!     dphidx         o    Contains the derivatives of the basis functions
!     kmeshtri       i    Contains 3 nodes of the triangle
!     xh             o    x-coordinates of vertices
!     yh             o    y-coordinates of vertices
!     zh             o    z-coordinates of vertices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical singul

!     singul         If true the Jacobian is smaller than eps
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PR2039         Compute gradient of the linear basis functions for a
!                    tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1066   Jacobian singular
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgeomtetrahed' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgeomtetrahed'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xh = coor(1,kmeshtri)
      yh = coor(2,kmeshtri)
      zh = coor(3,kmeshtri)
      call pr2039 ( xh, yh, zh, dphidx, epsmac, singul )
      if ( singul ) then

!     --- Singular mapping

         call errsub ( 1066, 0, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgeomtetrahed'

      end if  ! ( debug )
      call erclos ( 'prgeomtetrahed' )

      end
