      subroutine prin3dobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 29-05-2014 remove call to time subroutine
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 30-11-2001 Copied from plot3dobst
!
!   copyright (c) 2001-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write elements and coordinates of a three-dimensional obstacle
!     to a file in openview format
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     openview
!     mesh
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iresul(5), iskip, idummy

!     idummy         Dummy parameter
!     iresul         Output array of length 5
!                    Contents
!                    1: -
!                    2: npelmw
!                    3: nelemlc
!                    4: if > 0, array kmeshcsub has been allocated
!                    5: if > 0, array kmeshctri has been allocated
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINSFOBST     Write the coordinates and topology of the obstacle to a
!                    file in openview format
!     SEPQU0OBST     Check if a 3d mesh with obstacles must be subdivided
!                    into sub elements
!     TOPOBST        Mark all elemets that are inside and outside the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     581   The mesh is not three-dimensional
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check input
!   Copy iplots into iplots and fill defaults (PL0011)
!   Activate permanent arrays
!   Create surface vertex elements only
!   Perform actual procedure (PLOTSF)
!   Plot the obstacle separately
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prin3dobst' )
      if ( ndim/=3 ) then
         call errint ( ndim, 1 )
         call errsub ( 581, 1, 0, 0 )
      end if

!     --- Write the obstacle separately
!         First fill elemskip with a 1 for all elements inside the obstacle

      allocate ( elemskip(nelem) )

!     --- Fill array elemskip with a 0 for all elements outside the
!         obstacle and a 1 inside the obstacle

      call topobst

!     --- Subdivide the mesh in subelements

      iskip = 0
      call sepqu0obst ( iskip, idummy, iresul, 1 )
      if ( ierror/=0 ) go to 1000

!     --- Compute actual plot

      inelem = iresul(3)
      if ( inelem==0 ) go to 1000

      call prinsfobst ( coor, npoint, inelem, kmeshcsub )
      deallocate ( elemskip )

1000  if ( ierror/=0 ) then

!     --- ierror # 0  Reset ierror and nerror
!                     Return to main program

         ierror = 0
         nerror = 0
         go to 1010

      end if

      if ( iwarn/=0 ) then

!     --- iwarn # 0  Reset iwarn and nwarn
!                    Return to main program

         iwarn = 0
         nwarn = 0

      end if

1010  if ( iresul(4)==1 ) deallocate ( kmeshcsub )
      if ( iresul(5)/=0 ) deallocate ( kmeshctri )
      call erclos ( 'prin3dobst' )

      end subroutine prin3dobst
