      subroutine princompl2 ( array, n, m, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Auxilary help subroutine only meant for debugging purposes
!    The task of prinrl1 is to print a complex array in the form
!    form   seq. number:  n numbers etc.
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, m
      double complex array(n,m)
      character (len=*) text

!     array          i    double precision array to be printed
!     m              i    Number of entries of the array to be printed per line
!     n              i    Number of lines of the array to be printed
!     text           i    Text to be printed in the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, kmax, icount, ind(2)
      double precision d1(2), d2(2)
      character (len=1) chsign(2)

!     chsign         Contains the signs of the imaginary parts of numbers
!     d1             real parts of numbers
!     d2             imaginary parts of numbers
!     i              Counting variable
!     icount         Counter
!     ind            Contains sequence numbers
!     j              Counting variable
!     k              Counting variable
!     kmax           Maximum value of k
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPLNUM   Split a complex number in a real part and the absolute
!                    value of the imaginary part
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'princompl2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from princompl2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write ( irefwr, 100 ) text
      do i = 1, n

         icount = 0
         do j = 1, m, 2

            kmax = min(2,m-icount)
            do k = 1, kmax
               ind(k) = icount+k
               call princomplnum ( array(i,ind(k)), d1(k), d2(k), &
                                   chsign(k) )
            end do  ! k = 1, kmax

            if ( j==1 ) then
               write ( irefwr, 110 ) i,(ind(k), d1(k), chsign(k), &
                                        d2(k),k=1,kmax)
            else
               write ( irefwr, 120 ) (ind(k), d1(k), chsign(k), &
                                      d2(k),k=1,kmax)
            end if  ! ( j==1 )
            icount = icount + 2

         end do  ! j = 1, m, 2

      end do  ! i = 1, n

100   format(1x,a)
110   format(i4,':',ss, (2(1x,i4,':',ss,1pe10.3e2," ",a,1pe10.3e2,"i")))
120   format(5x,ss, (2(1x,i4,':',ss,1pe10.3e2," ",a,1pe10.3e2,"i")))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End princompl2'

      end if  ! ( debug )
      call erclos ( 'princompl2' )

      end
