      subroutine princoor1 ( iseqnr, inppri, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 10-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print one of the coordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr, inppri(*), icurvs(*)

!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     inppri         i    Input array with respect to PRINVC (see PRINVC)
!     iseqnr         i    Component of coor to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nnodes, nparts, iregion, idim
      integer, allocatable :: nodes(:), ipoint(:)

!     idim           dimension of the space
!     ipoint
!     iregion        indicates if the complete region must be printed
!                    (0) or that the printing is restricted to a sub region
!                    defined by the parameters xmin, xmax, ymin, ymax, zmin
!                    zmax in the common block CPLOT.
!                    Possible values:
!                    0:  The complete region is printed
!                    1:  The region is restricted to xmin<=x<=xmax, &
!                        ymin<=y<=ymax, zmin<=z<=zmax
!     nnodes         Number of nodes
!     nodes          Contains the node numbers that must be filled
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINCOOR2      Print the coordinates array for a given set of nodes
!     PRSW02         Restrict nodes to those in a specified region
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3090   idim out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'princoor1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from princoor1'
         write(irefwr,1) 'iseqnr', iseqnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create temporary arrays

      allocate ( nodes(npoint), ipoint(ncurvs+1), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint+ncurvs, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array nodes with nodes in which we have to print

      idim = inppri(3)
      call sepfillnodes ( idim, 0, icurvs, &
                          nodes, nnodes, ipoint, nparts )

      iregion= inppri(7)

      if ( iregion/=0 ) then

!     --- iregion # 0, restrict region

         if ( ierror/=0 ) go to 1000
         call prsw02 ( nodes, ipoint, nparts, &
                       iregion, coor, ndim, idim )
         if ( ierror/=0 ) go to 1000
         if ( ndim==2 ) then
            nnodes = ipoint(nparts+1)-1
         else
            nnodes = nparts
         end if
         if ( debug ) write(irefwr,1) 'nnodes', nnodes

      end if

      if ( iseqnr<0 .or. iseqnr>ndim ) then

!     --- iseqnr has incorrect value

         call errint ( iseqnr, 1 )
         call errint ( ndim, 2 )
         call errsub ( 3090, 2, 0, 0 )
         go to 1000

      end if  ! ( iseqnr< .or. iseqnr>ndim )

!     --- standard case: print

      call princoor2 ( coor, ndim, iseqnr, nodes, nnodes )

      deallocate ( nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End princoor1'

      end if  ! ( debug )
      call erclos ( 'princoor1' )

      end subroutine princoor1
