      subroutine prini2 ( kmeshlc, ivectr, ichois, typevec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.11 date 21-04-2015 New call to prinintmat
!        version  2.10 date 29-05-2014 remove call to time subroutine
!        version  2.9  date 15-11-2006 Extension with 62-63
!
!   copyright (c) 1999-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of standard sepran integer arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemain
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ivectr, kmeshlc, ichois, typevec

!     ichois  i    choice parameter, values:
!                  -1: Print a selection of the complete integer array.
!                      This possibility is only allowed for IVECTR equal to
!                      KMESH, KPROB and INTMAT
!                      If the complete array KMESH must be printed, then
!                      both arrays KMPRB and IVECTR are assumed to be KMESH
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be KMESH and IVECTR is
!                      assumed to be KPROB
!                   0: Print the complete integer array.
!                      This possibility is only allowed for IVECTR equal to
!                      KMESH, KPROB and INTMAT
!                      If the complete array KMESH must be printed, then
!                      both arrays KMPRB and IVECTR are assumed to be KMESH
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be KMESH and IVECTR is
!                      assumed to be KPROB
!                   1: print array kmesh parts c and g
!                   2: print array kmesh part d
!                   3: print array kmesh part e
!                   4: print array kmesh part j
!                   5: print array kmesh part h
!                   6: print array kmesh part l,m and n
!                   7: print array kmesh part o
!                   8: print array kmesh part k
!                   9: print array kmesh parts q, r and s
!                  11: print kprob parts b and i
!                  12: print kprob part f
!                  13: print kprob part e
!                  14: print kprob part h
!                  15: print kprob part i
!                  16: print kprob part k
!                  17: print kprob part l
!                  18: print kprob part m
!                  19: print kprob part n
!                  20: print intmat part a
!                  21: print intmat part b
!                  22: print kprob part o
!                  23: print kprob part p
!                  24: print kprob part r
!                  25: print kprob part s
!                  26: print kprob part v and w
!                  27: print kprob part x
!                  28: print kprob part y
!                  29: print kprob part aa
!                  30: print kmesh iinput and rinput
!                  31: print kmesh part t
!                  32: print kmesh part u
!                  33: print kmesh part v
!                  34: print kmesh part w
!                  35: print kmesh part x
!                  36: print kmesh part z
!                  37: print kmesh part y
!                  38: print kmesh part i
!                  39: print kmesh part aa/nmeshc
!                  40: print kmesh part aa/nmeshd
!                  41: print kmesh part aa/nmeshl
!                  42: print kmesh part ag (volume_surfs, i.e. surfaces
!                                           corresponding to volumes)
!                  43: print kmesh part aa/nmeshh
!                  44: print kmesh part aa/nmesha
!                  45: print kmesh part aa/nmeshb
!                  46: print kmesh part aa/nmeshf
!                  47: print kmesh part aa/nmeshg
!                  48: print kmesh part aa/nmeshk
!                  49: print kmesh parts blockinfo, transportinfo
!                  50: print kmesh part ioutercurvs/surfs
!                  51: print kprob parts blockunkinfo, sendinfo and innercontr
!                  52: print kprob part ab
!                  59: print kmesh part ai (surfnodes)
!                  60: print kmesh part aj (normals)
!                  61: print kmesh part al
!                  62: print kmesh part ae (nodes corresponding to groups)
!                  63: print kmesh part a, b and f
!     ivectr  i    integer array to be printed
!     kmeshlc        i    Output of the mesh generator
!     typevec        i    type of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical prmesh, prprob, printm

!     printm         Indication if array INTMAT must be printed (true) or
!                    not (false)
!     prmesh         Indication if array KMESH must be printed (true) or not
!                    (false)
!     prprob         Indication if array KPROB must be printed (true) or not
!                    (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININTMAT     Print array INTMAT
!     SEPPRINKMESH   Print array KMESH
!     SEPPRINKPROB   Print array KPROB
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS has wrong value
!     698   Array is of incorrect type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prini2' )

!     --- Make some initialisations

      prmesh = .false.
      prprob = .false.
      printm = .false.

!     --- Make some tests

      if ( ichois<-1 .or. ichois==15 .or. &
           ichois>52.and.ichois<=60 .or. ichois>63 ) then

!     --- error 1: ichois has wrong value

         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
         ierror = 0
         go to 1000

      end if

      if ( ichois<=0 ) then

!     --- ichois = 0, -1 detect the type of array to be printed

         select case ( typevec )
         case ( 100 )

!        --- 100,  array is of type KMESH

            prmesh = .true.

         case ( 101 )

!        --- 101,  array is of type KPROB

            prprob = .true.

         case ( 102 )

!        --- 102,  array is of type intmat

            printm = .true.

         case default

!        --- error Array is of the wrong type

            call errint ( typevec, 1 )
            call errsub ( 698, 1, 0, 0 )
            ierror = 0
            go to 1000

         end select  ! case ( typevec )

      else

!     --- ichois>0, only a part of an array must be printed

         if ( ichois<11 .or. ichois>=30 .and. ichois<50 .or. &
              ichois>=61 ) then

!        --- ichois<11,  array is of type KMESH

            prmesh = .true.

         else if ( ichois<20 .or. ichois>21 ) then

!        --- ichois<20 or>21,  array is of type KPROB

            prprob = .true.

         else

!        --- ichois = 21 or 22,  array is of type INTMAT

            printm = .true.

         end if

      end if

!     --- Actual printing

      if ( prmesh ) then

!     --- prmesh = true, print KMESH

         kmesh = ivectr
         call sepprinkmesh ( ichois )

      else if ( prprob ) then

!     --- prprob = true, print KPROB

         kprob = ivectr
         call sepprinkprob ( ichois )

      else if ( printm ) then

!     --- printm = true, print INTMAT

         intmat = ivectr
         call prinintmat ( ichois )

      end if

1000  call erclos ( 'prini2' )
      end subroutine prini2
