      subroutine prinkmeshpar ( blockinfo, transportinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-04-2012 Jump in case of two blocks
!        version  1.1  date 26-04-2003 New definition part 2
!        version  1.0  date 27-03-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the parallel arrays blockinfo and transportinfo
!     in case parallel computing is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     print
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer blockinfo(*), transportinfo(*)

!     blockinfo      i    Array with global block information for block
!                         number iblock
!                         See Programmers Guide Section 24.2 for a description
!     transportinfo  i    Array containing information of the transport of
!                         unknowns between neighbouring blocks
!                         See Programmers Guide Section 24.2 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numneigh, i, iblock, istartinfo, nneigh, m_i, &
              j, ineigh, n_ij, itransinfo

!     i              Counting variable
!     iblock         Block sequence number
!     ineigh         Neighbour of block
!     istartinfo     Position in array blockinfo
!     itransinfo     Position in array transportinfo
!     j              Counting variable
!     m_i            The number of neighbour blocks of block i that are also
!                    neighbours of the present block
!     n_ij           nodes in the interface of iblock and ineigh that are also
!                    neighbour nodes of blocknr
!     nneigh         Number of neighbours of block
!     numneigh       number of neighbours of a block
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkmeshpar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkmeshpar'

      end if
      if ( .not. parallel ) go to 1000
      write(irefwr,*) 'Parallel information of block number ', inodenr
      numneigh = blockinfo(1)

!     --- For all neighbour blocks do

      do i = 1, numneigh

         iblock =  blockinfo(2*i)
         write(irefwr,*) 'Neighbour block ', iblock
         write(irefwr,*) 'Nodes on common sides of blocks', &
                         inodenr, ' and ', iblock
         istartinfo = blockinfo(2*i+1)
         nneigh = transportinfo(istartinfo)
         call prinin ( transportinfo(istartinfo+1), nneigh, 'nodes' )

      end do  ! i = 1, numneigh

      if ( iacnodes<=2 ) go to 1000

      write(irefwr,*) 'Information stored in second part'

!     --- For all neighbour blocks do

      do i = 1, numneigh

         iblock =  blockinfo(2*i)
         write(irefwr,*) 'Neighbour block ', iblock
         istartinfo = blockinfo(2*numneigh+i+1)
         m_i = blockinfo(istartinfo)

!        --- Loop over common neighbours

         do j = 1, m_i

            ineigh = blockinfo(istartinfo+2*j-1)
            itransinfo = blockinfo(istartinfo+2*j)
            n_ij = transportinfo(itransinfo)
            write(irefwr,*) 'Nodes on common sides of ', &
                            'blocks ', inodenr, ' and ', iblock, &
                            ' that are neighbours of block ', ineigh
            call prinin ( transportinfo(itransinfo+1), n_ij, 'nodes' )

         end do  ! j = 1, m_i

      end do  ! i = 1, numneigh

1000  call erclos ( 'prinkmeshpar' )
      if ( debug ) write(irefwr,*) 'End prinkmeshpar'

      end
