      subroutine prinkprobn ( npoint, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 21-04-2010 Remove checkbuf1
!        version  1.0  date 03-10-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of kprob part n (local transformations)
!     for one specific problem
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     problem_definition
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ndim

!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih
      integer, allocatable :: iwork(:,:)

!     ih             Help variable to store a constant temporarily
!     iwork          integer work array of size 4*nusol
!                    In this array per unknown the following information
!                    is stored
!                    (1,.) node corresponding to unknown in standard seq
!                    (2,.) local degree of freedom in node corresponding
!                          to unknown in standard sequence
!                    (3,.) node corresponding to unknown if numbering
!                          with respect to matrix is used
!                    (4,.) local degree of freedom in node corresponding
!                          to unknown if numbering
!                          with respect to matrix is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PR0010         print kprob part n
!     PRFILLPOSUNK   Fill array iwork with positions of an unknown with respect
!                    to the nodes and also renumbered with respect to the matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkprobn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkprobn'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npltra = kprobloc(13)
      maxtrn = kprobloc(45)

      if ( npltra>0 ) then

         indprf = kprobloc(19)
         indprh = kprobloc(50)
         indprp = kprobloc(35)
         ih = kprobloc(27)
         nusol = kprobloc(5)

!        --- activate arrays

         allocate ( iwork(4,nusol), stat = error )
         if ( error/=0 ) call eralloc ( error, 4*nusol, 'iwork' )
         if ( ierror/=0 ) go to 1000

         if ( indprf>0 ) kprobf => kprobpart%kprobf
         if ( indprh>0 ) kprobh => kprobpart%kprobh
         if ( indprp>0 ) kprobp => kprobpart%kprobp

         call prfillposunk ( npoint, iwork )

         call pr0010 ( npltra, ndim, maxtrn, kprobn, coortr, iwork )

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinkprobn'

      end if  ! ( debug )
      call erclos ( 'prinkprobn' )

      end
