      subroutine prinkprobpar1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-05-2015 Extension kprobpar
!        version  1.1  date 12-08-2014 Extension of arrays
!        version  1.0  date 13-08-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print array kprob par
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     parallel
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numlow, numupp

!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkprobpar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkprobpar1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 .or. kprobloc(65)==0 ) go to 1000

      if ( debug ) call prinin ( kprobpar, 20, 'kprobpar' )

      write(irefwr,1) 'nusolglob, nrusolglob', kprobpar(1:2)
      write(irefwr,1) 'nrusol, ninterface', nrusol, ninterface

      if ( associated(kprobpart%kprobhglob) ) then

!     --- Print kprobhglob

         kprobhglob => kprobpart%kprobhglob
         nrusol = kprobloc(5)-kprobloc(10)

         call prinin ( kprobhglob, nrusol, 'kprobhglob' )

      end if  ! ( associated(kprobpart%kprobhglob) )

      if ( associated(kprobpart%onnz) ) then

!     --- Print onnz

         onnz => kprobpart%onnz
         nrusol = kprobloc(66)

         call prinin ( onnz, nrusol, 'onnz' )

      end if  ! ( associated(kprobpart%onnz) )

      if ( associated(kprobpart%nrusollocs) ) then

!     --- Print nrusollocs

         nrusollocs => kprobpart%nrusollocs
         call prinin ( nrusollocs, iacnodes, 'nrusollocs' )

      end if  ! ( associated(kprobpart%nrusollocs)

      if ( associated(kprobpart%ninterfdeg) ) then

!     --- Print ninterfdeg

         ninterfdeg => kprobpart%ninterfdeg
         call prinin ( ninterfdeg, iacnodes+1, 'ninterfdeg' )

      end if  ! ( associated(kprobpart%ninterfdeg) )

      if ( associated(kprobpart%blockinfor) ) then

!     --- Print blockinfor

         blockinfor => kprobpart%blockinfor

         numlow = blockinfor(1)
         numupp = blockinfor(2)

         write(irefwr,100) numlow, numupp
100      format ( 'number of neighbor blocks with lower number = ', i3/ &
                  'number of neighbor blocks with larger number = ',i3 )

         call prinin ( blockinfor(3), numlow+numupp, 'neighbor blocks' )

      end if  ! ( associated(kprobpart%blockinfor) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinkprobpar1'

      end if  ! ( debug )
      call erclos ( 'prinkprobpar1' )

      end subroutine prinkprobpar1
