      subroutine prinmtsimple ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-04-2009 New storage for block matrices
!        version  2.0  date 22-12-2008 Complete revision
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print matrix corresponding to simple or al storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     matrix
!     simple
!     augmented_lagrange
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr    i   Standard SEPRAN array containing information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      type (globmat), pointer :: A

!     A              Structure of type globmat containing the necessary
!                    information of the matrix
!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINDMAT       Print the D matrix in the Simple method
!     PRINRL         Print 1d real vector
!     SEPCOPYMATPM   Copy real PM matrix in type matrix (simple)
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMATDBG  print compact matrix, skip zeros
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinmtsimple' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinmtsimple'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      A => realmt(matr)
      IM => intmt(A%intmatref)

!     --- 13<=jmethod<=16, simple algorithm or schur type algorithm
!         First print the matrix Q

      write(irefwr,*) 'matrix Q of block storage'

      if ( debug ) then

!     --- Debug format

         call sepprinmatdbg ( Q, 0d0 )

      else

!     --- Standard format

         call sepprinmat6 ( Q )

      end if

!     --- Next print the D and the G matrix


      write(irefwr,*) 'Matrix D'
      call prindmat ( S_DIV, irowsG, icolsG, nrusolpress )

      if ( .not. IM%Gsymmetric ) then

!     --- G-matrix is different from D-matrix

         write(irefwr,*) 'Transpose of matrix G'
         call prindmat ( S_G, irowsG, icolsG, nrusolpress )

      end if  ! ( jmethod==15 .or. jmethod==16 ... )

      write(irefwr,*) 'matrix C of block storage'

      call sepprinmat6 ( C )

!     --- print pressure mass matrix

      write(irefwr,*) 'Scaled pressure mass matrix'

      call sepcopymatpm ( 4 )
      call sepprinmat6 ( R )

!     --- print diagonal pressure mass matrix

      call prinrl ( S_PMD, nrusolpress, 'Pressure Mass matrix' )

!     --- print diagonal velocity mass matrix

      call prinrl ( S_UD, nrusolvel, 'Velocity Mass matrix' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinmtsimple'

      end if  ! ( debug )
      call erclos ( 'prinmtsimple' )

      end
