      subroutine prinnowbndcrd ( nowbnd, nloop, imxlps, layer, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-06-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the actual boundary during the creation of the mesh in a specific
!     iteration
!     The boundary is defined by array nowbnd, which is only defined for mshpav
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     boundary
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imxlps, nowbnd(imxlps,*), nloop, layer
      double precision coor(4,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     imxlps         i    maximum loops in nowbnd in paver, also first dimension
!                         of nowbnd
!     layer          i    Present layer sequence number
!     nloop          i    number of closed boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nbndpt, node, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     nbndpt         Number of nodal points in boundary, length of array
!                    kbndpt
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open plot
!     Plot each boundary
!     Close plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnowbndcrd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnowbndcrd'
         write(irefwr,*) 'nloop ', nloop

      end if
      write(irefwr,*) 'Nowbnd in layer ', layer

!     --- Print each boundary

      do i = 1, nloop
         nbndpt = nowbnd(i,1)
         write(irefwr,*) 'Loop ', i,'  nbndpt = ', nbndpt

!        --- Go to first node with pen up

         node = abs(nowbnd(i,2))
         write(irefwr,*) 1, '  node ', node, (coor(k,node), k=1,4)

!        --- Go to all other nodes with pen down

         do j = 2, nbndpt
            node = abs(nowbnd(i,j+1))
            write(irefwr,*) j, '  node ', node, (coor(k,node), k=1,4)
         end do

      end do  !  i = 1, nloop

      call erclos ( 'prinnowbndcrd' )
      if ( debug ) write(irefwr,*) 'End prinnowbndcrd'

      end
