      subroutine prinptrace01 ( partout, solint, iphys, ndim, &
                                nsteps, iparticle )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 29-12-2000
!
!   copyright (c) 2000-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Perform the actual printing for subroutine printrace
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     track
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iphys(*), ndim, nsteps, iparticle
      double precision partout(ndim+1,*), solint(nsteps,*)

!     iparticle      i    Sequence number of particle
!     iphys          i    Number of physical variables
!     ndim           i    Dimension of the space.
!     nsteps         i    Contains the number of positions stored in partout
!     partout        i    Array containing the actual information concerning the
!                         particle trajectories
!                         partout(1,i) contains t_i
!                         partout(2-3,i) contains x_i and y_i
!     solint         i    Array of size nsteps x number of unknowns containing
!                         all the unknowns per point
!                         The corresponding coordinates are stored in partout
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character *320 loctexts(27)
      integer nloctexts(27), ishift, nvectors, nphys, nphystot, &
              i, j, ivecnm, lenname

!     i              Counting variable
!     ishift         Shift of specific variable with respect to diagonal
!     ivecnm         vector sequencer number with repsect to isol
!     j              Counting variable
!     lenname        Length of name
!     loctexts       Contains the local texts to be printed
!     nloctexts      Length of the local texts stored in loctexts
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nphystot       Total number of physical unknowns of all vectors
!     nvectors       Number of vectors to be considered in this subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2299   nphys>20
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create heading
!     Print heading
!     Create second line
!     Print second line
!     Print values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Create heading

      nvectors = iinputpst(85)
      if ( ndim==2 ) then
         loctexts(1) = '          x               y'
         nloctexts(1) = 34
      else
         loctexts(1) = '          x               y               z'
         nloctexts(1) = 50
      end if

      nphystot = 0
      do i = 1, nvectors
         nphys = iphys(i)
         nphystot = nphystot+nphys
         if ( nphys>20 ) nphys = 20
         length = nphys*16
         ivecnm = igroup(i)
         lenname = index ( names(ivecnm), ' ' ) - 1
         if ( lenname<=0 ) lenname = 30
         if ( lenname>=length ) then
            loctexts(i+1) = names(ivecnm)(1:length)
         else
            loctexts(i+1) = ' '
            ishift = (length-lenname)/2
            loctexts(i+1)(1+ishift:lenname+ishift) = &
               names(ivecnm)(1:lenname)
         end if
         nloctexts(i+1) = length

      end do

!     --- Print heading

      write(irefwr,100) iparticle, &
         (loctexts(i)(1:nloctexts(i)), i = 1, nvectors+1)
100   format ( /' Trace of particle ',i3/'    t   ', 27a )

!     --- Create second line

      loctexts(1) = ' '

      do i = 1, nvectors
         nphys = iphys(i)
         if ( nphys>10 ) then

!        --- nphys>20

            call eropen ( 'prinptrace01' )
            call errint ( nphys, 1 )
            call errint ( 10, 2 )
            call errwar ( 2299, 2, 0, 0 )
            call erclos ( 'prinptrace01' )
            nphys = 20

         end if
         if ( nphys>1 ) then

!        --- More than one component

            loctexts(i+1) = ' '
            do j = 1, nphys
               write(loctexts(i+1)(1+(j-1)*16:j*16),110) j
110            format( '  component ', i2 )
            end do
            ivecnm = igroup(i)
            lenname = index ( names(ivecnm), ' ' ) - 1
            if ( names(ivecnm)(1:lenname)=='velocity' ) then

!           --- Special case of a velocity vector

               loctexts(i+1)(1:16) = '       u'
               loctexts(i+1)(17:32) = '       v'
               if ( ndim==3 ) loctexts(i+1)(33:48) = '       w'

            end if

         else

!        --- Only one component

            loctexts(i+1) = ' '

         end if

      end do

!     --- Print second line

      write(irefwr,150) (loctexts(i)(1:nloctexts(i)), i = 1, nvectors+1)
150   format ( 9x, 27a )

!     --- Finally, print values

      do i = 1, nsteps
         write(irefwr,200) (partout(j,i),j=1,ndim+1), &
                           (solint(i,j),j=1,nphystot)
200      format ( f8.3, ss, 100(1pe16.6e2) )
      end do
      end
