      subroutine prinrv ( ivectr, kmeshlc, kproblc, layout, number, text )
! ======================================================================
!
!        programmer    Guus Segal / Jaap van der Zanden
!        version  1.3  date 29-05-2014 remove call to time subroutine
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   print real/complex  sepran array except the large matrix
!         ks(ivectr)%typevec = 110,115 allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecactl
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character text *(*)
      integer ivectr, layout, number
      integer, target :: kmeshlc, kproblc

!     ivectr  i    standard sepran array denoting usol or vector of special
!                  structure defined in nodal points
!                  This defines the array to be printed
!     kmeshlc i    Standard SEPRAN array containing information of the mesh
!     kproblc i    Standard SEPRAN array, containing information of the problem
!     layout  i    parameter that indicates what kind of layout is used
!                  possibilities:
!                  1   The array is printed element after element,
!                      5 in each line. Format: 5e14.6
!                  2   The array is printed element after element,
!                      10 in each line. Format: 10e11.3
!                  3   The array is printed element after element,
!                      10 in each line. Format: 10f11.3
!                  4   Each nodal point (or element when IVECTR is a vector
!                      of special structure defined per element) starts on
!                      a new line. Format: 5e14.6
!                  5   When only one unknown is required in each nodal point
!                      (or element, see 4) (NUMBER>0), this possibility
!                      gives 5 elements in each line provided with the
!                      corresponding nodal point number. Format: 5e11.3
!                  6   See 5, however, Format: 5f11.3
!                  7,8 See 5,6, however, only 4 elements in each row. This
!                      possibility is made for 80 columns output
!     number  i    choice parameter,  possibilities:
!                  -1   coordinates are printed
!                  0    array corresponding to ivectr is printed entirely
!                  >0   only the numberth unknown in each nodal point is printed
!     text    i    text to be printed in the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, itypvc, leng, npointnel
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     k              Counting variable
!     leng           Help variable to compute the length of some arrays
!     npointnel      Number of nodal points or elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0001         print array from positions 1 to length with steps istep
!     ER0002         print array   nunkpi positions in each row; npoint rows
!     ER0004         print array   nunkpi positions in each row; npoint rows
!     ER0005         print unknown number from array vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     101   layout has incorrect value
!     102   NUMBER has wrong value
!     103   Array ISOL has not been filled
!     413   layout = 5, whereas number = 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinrv' )
      kmesh = kmeshlc
      call sepfillkmesh(kmesh)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)
      if ( layout<=0 .or. layout>8 ) then
         call errint ( layout, 1 )
         call errsub ( 101, 1, 0, 0 )
      end if
      if ( number<-1 .or. number>1000 ) then
         call errint ( number, 1 )
         call errint ( 1000, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      if ( number==-1 ) then

!     --- number = -1,  print coordinates

         if ( layout<4 ) call er0001 ( coor, ndim*npoint, &
              layout, text, 1 )
         if ( layout>3 ) call er0002 ( coor, ndim, npoint, &
              text)
      else

!     --- number>=  0

         iprob = ks(ivectr)%iprob
         itypvc = ks(ivectr)%typevec
         call sepactsolbf2 ( ivectr )
         nusoli = ks(ivectr)%nusol
         compl = ks(ivectr)%compl==1
         if ( compl ) then

!        --- Complex vector

            leng = 2
            if ( number>2*nunkpi )  then
               call errint ( number, 1 )
               call errint ( 2*nunkpi, 2 )
               call errsub ( 102, 2, 0, 0 )
            end if
         else

!        --- Real vector

            leng = 1
            if ( number>nunkpi )  then
               call errint ( number, 1 )
               call errint ( nunkpi, 2 )
               call errsub ( 102, 2, 0, 0 )
            end if
         end if
         npointnel = npoint
         if ( itypvc==116 ) npointnel = nelem
         if ( itypvc/=110 .and. itypvc<115 .or. itypvc>116 ) then
            call errint ( itypvc, 1 )
            call errsub ( 103, 1, 0, 0 )
            go to 1000
         end if
         usoltemp => ks(ivectr)%sol
         if ( layout==5 .and. number==0 ) call errwar ( 413, 0, 0, 0 )
         if ( (layout<=3 .or. layout==5) .and. number==0 ) then

!        --- layout<=  3  and number = 0

            call er0001 ( usoltemp, nusoli*leng, layout, text, 1 )

         else

!        --- layout>3  or  number>0

            if ( number==0 ) then

!           --- number = 0

               if ( indprfi==0 ) then
                  nunkpi = leng*nunkpi
                  call er0002 ( usoltemp, nunkpi, npointnel, text )
               else
                  call er0004 ( usoltemp, kprobfi, &
                                indprfi, npointnel, nunkpi, text, compl )
               end if
            else

!           --- number>0,  only layout = 5 or 6  is used

               if ( layout<4 .and. indprfi==0 )  then
                  nunkpi = nunkpi*leng
                  call er0001 ( usoltemp(number), nusoli, layout, &
                                text, nunkpi )
               else

!              --- layout = 5 or 6

                  call er0005 ( usoltemp, npointnel, number, &
                                kprobfi, indprfi, &
                                nunkpi, text, layout, compl )
               end if
            end if
         end if
      end if
1000  ierror = 0
      call erclos ( 'prinrv' )
      end subroutine prinrv
