      subroutine printbn ( iinner, icurprin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 31-01-2003 Extra parameter icurprin
!        version  1.0  date 01-09-2000
!
!   copyright (c) 2000-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    print the boundary of a mesh as given in array kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinner, icurprin

!     icurprin       i    Curve number of curve to be printed when iinner = 4
!     iinner         i    Information of how the curves must be printed
!                         0,1      only the outer boundaries are printed
!                           2      only the inner boundaries are printed
!                           3      all boundaries are printed
!                           4      only one curve is printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical inboun
      integer j, i, icurnr, ilast, kinner, node, nstar3, nstar4, &
              istart, ifcurnr, ilcurnr

!     i              Counting variable
!     icurnr         Curve sequence number
!     ifcurnr        First curve number
!     ilast          Last curve (surface) number read
!     ilcurnr        Last curve number
!     inboun         Logical indicating if a boundary must be marked as inner
!                    boundary (true) or not (false)
!     istart         First position used at a part
!     j              Counting variable
!     kinner         Counter for number of inner boundaries
!     node           Node number
!     nstar3         Starting address of KMESHM part 3 in KMESHM containing the
!                    node numbers of the curves
!     nstar4         Starting address of KMESHM part 4 in KMESHM containing the
!                    inner boundaries
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      write ( irefwr,* ) 'Co-ordinates of the curves'

!     --- Compute some constants

      nstar3 = kmeshm(2)-1
      nstar4 = kmeshm(3)
      ninner = kmeshm(1)
      if ( ninner==0 .and. iinner==2 ) return
      kinner = 0
      if ( iinner==4 ) then

!     --- Only one curve is printed

         ifcurnr = icurprin
         ilcurnr = icurprin
         istart  = kmeshm(4+icurprin)

      else

!     --- All curves are printed

         ifcurnr = 1
         ilcurnr = ncurvs
         istart  = 1

      end if

!     --- Loop over curves

      do icurnr = ifcurnr, ilcurnr
         ilast = kmeshm(5+icurnr)-1
         inboun = .false.
         if ( iinner<3 ) then

!        --- inner = 1 or 2,  check if curve is inner or outer boundary

            if ( kinner<ninner .and. kmeshm(nstar4+kinner)==icurnr) &
               then

!           --- inner boundary found

               kinner = kinner+1
               inboun = .true.

            end if

         end if
         if ( iinner>=3 .or. inboun.and.iinner==2 .or. &
              (.not. inboun).and.iinner<=1 ) then

!        --- Check if boundary has been filled or not
!            If so print else skip boundary

            do j = istart, ilast
               if ( kmeshm(nstar3+j)<=0 ) goto 200
            end do
            write ( irefwr,* ) 'Curve number ', icurnr
            write ( irefwr,* ) 'Node       co-ordinates'

!           --- print boundary

            do j = istart, ilast
               node = kmeshm(nstar3+j)
               write ( irefwr, 100 ) node, (coor(i,node),i=1,ndim)
100            format( i6, ':', 2x, ss, 3(1pe13.5e2) )
            end do

         end if

200      istart = ilast+1
      end do
      end
