      subroutine printscalstore ( integerfound, lengtext, textloc, &
                                  lentxt, rvalue, ivalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-12-2013 Extension with integer scalars
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill name of variable in textloc
!     Investigate if variable is integer or real
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lentxt, lengtext, ivalue
      logical integerfound
      double precision rvalue
      character (len=*) textloc

!     integerfound  i/o   If true an integer has been found
!     ivalue        i/o   Integer value
!     lengtext      i/o   Help parameter to indicate the length of the actual text
!     lentxt         i    length of text parameter, with trailing blanks deleted
!     rvalue         i    real
!     textloc       i/o   String to store a local text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printscalstore' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printscalstore'
         write(irefwr,*) 'integerfound ', integerfound

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( integerfound ) then

         if ( lengtext>113 ) then
             write(irefwr,300) textloc(1:lengtext)
             lengtext = lentxt
             textloc(1:lentxt) = ' '
         end if
         write(textloc(lengtext+1:lengtext+6),220) ivalue
         lengtext = lengtext+6
220      format(i6)

      else

         if ( lengtext>105 ) then
             write(irefwr,300) textloc(1:lengtext)
             lengtext = lentxt
             textloc(1:lentxt) = ' '
         end if
         write(textloc(lengtext+1:lengtext+14),230) rvalue
230      format(1x,ss,1pe13.5)
         lengtext = lengtext+14

      end if
300   format(1x,a)

1000  call erclos ( 'printscalstore' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printscalstore'

      end if  ! ( debug )

      end
