      subroutine printsurface ( isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 17-01-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array kmeshn for one surface as well as the
!     corresponding coordinates
!     3d only
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr

!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, inpelm, j, k, node
      double precision xmin, xmax, ymin, ymax, zmin, zmax

!     i              Counting variable
!     inpelm         number of points per element
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     node           Node number
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printsurface' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printsurface'

      end if
      if ( ierror/=0 ) go to 1000

      istart = 2*nsurfs
      do i = 1, isurnr-1
         istart = istart+kmeshn(i)*kmeshn(i+nsurfs)
      end do
      write(irefwr,*) 'Nodes and coordinates of surface no ', isurnr
      inpelm = kmeshn(isurnr)
      nelem = kmeshn(isurnr+nsurfs)
      do i = 1, nelem
         write(irefwr,200) i, (kmeshn(istart+j), &
             (coor(k,kmeshn(istart+j)),k=1,3), j = 1, inpelm )
200      format ( 'element ', i6/(i8,' : '     , 3d15.8) )
         istart = istart+inpelm
      end do

!     --- Compute minimum and maximum coordinates of surface nodes

      istart = 2*nsurfs
      do i = 1, isurnr-1
         istart = istart+kmeshn(i)*kmeshn(i+nsurfs)
      end do
      xmin = rinfin
      xmax = -rinfin
      ymin = rinfin
      ymax = -rinfin
      zmin = rinfin
      zmax = -rinfin
      do i = 1, inpelm*nelem
         node = kmeshn(istart+i)
         xmin = min(xmin,coor(1,node))
         xmax = max(xmax,coor(1,node))
         ymin = min(ymin,coor(2,node))
         ymax = max(ymax,coor(2,node))
         zmin = min(zmin,coor(3,node))
         zmax = max(zmax,coor(3,node))
      end do  ! i = 1, inpelm*inelem

!     --- Print minimum and maximum coordinates of surface nodes

      write(irefwr,210) xmin, xmax, ymin, ymax, zmin, zmax
210   format( 'Minimum and maximum x-coordinates ', 2d12.3/ &
              'Minimum and maximum y-coordinates ', 2d12.3/ &
              'Minimum and maximum z-coordinates ', 2d12.3 )

1000  if ( debug ) write(irefwr,*) 'End printsurface'
      call erclos ( 'printsurface' )

      end
