      subroutine printzerolev3d ( coor, ilev, nmeshg, iobstfaces, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the faces (3d) of the zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ilev, nmeshg(*), iobstfaces(*), iref
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ilev           i    Sequence number of level being considered
!     iobstfaces     i    contains face sequence numbers of boundary
!                         of levelset
!     iref           i    File reference number
!     nmeshg         i    nmesh part g, contains node numbers corresponding to
!                         faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nnode_face, jstart, numfaces, iface, kstart, &
              node1, node2, node3

!     i              Counting variable
!     iface          face sequence number
!     jstart         starting position of face
!     kstart         Start value in iobstfaces
!     nnode_face     number of nodes corresponding to face
!     node1          Node of face
!     node2          Node of face
!     node3          Node of face
!     numfaces       number of faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printzerolev3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printzerolev3d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(iref,100)
100   format( '//   nodes        x_1          y_1           z_1', &
              '         x_2          y_2           z_2', &
              '         x_3          y_3           z_3' )

      kstart = iobstfaces(ilev)
      numfaces = iobstfaces(kstart)
      if ( debug ) &
         call prinin ( iobstfaces(kstart), numfaces, 'iobstfaces' )

      do i = 1, numfaces

         iface = iobstfaces(kstart+i)       ! face number
         jstart = nmeshg(iface)
         nnode_face = nmeshg(jstart)
         node1 = nmeshg(jstart+1)
         node2 = nmeshg(jstart+2)
         node3 = nmeshg(jstart+3)

         write(iref,110) iface, node1, node2, node3, &
                         coor(1:3,node1), coor(1:3,node2), &
                         coor(1:3,node3)
110      format ( 4i10, 9d13.5 )

         if ( debug ) then
            write(irefwr,1) 'iface', i
            call prinin ( nmeshg(jstart+1), nnode_face, 'nodes' )
         end if  ! ( debug )

      end do  ! i = 1, numfaces

      if ( debug ) write(irefwr,1) 'nnode_face', nnode_face

1000  call erclos ( 'printzerolev3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printzerolev3d'

      end if  ! ( debug )

      end
