      subroutine proutpsub2 ( isubract, ivectr, nproblc, numveclc, iavs, &
                              nameavs, namedx, one_file, iseqlc, iseqnr, &
                              nametec, nameparaview, separate_elgrp, &
                              appendmode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.1  date 24-12-2014 New definition clinsl
!        version  3.0  date 19-04-2014 Extra parameter appendmode
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input and use this to set other parameters
!     with respect to subroutine proutp
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     read
!     write
!     tecplot
!     avs
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleuser
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      use sepmoduletime
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubract, ivectr, nproblc, &
              numveclc, iavs, nameavs, iseqnr, nametec, &
              namedx, one_file, iseqlc, nameparaview, appendmode
      logical separate_elgrp

!     appendmode     i    Defines if append mode must be used (1) or ot (0)
!     iavs          i/o   Indicates which kind of output must be created.
!                         Possible values:
!                         0:  Create output for SEPPOST only in the form of file
!                             sepcomp.out
!                         1:  Create output for AVS only in the form of a file
!                             sepavs_xxx.yyy
!                             With xxx a time sequence number of the shape 001, &
!                             002 etc and yyy a vector sequence number of the
!                             shape 001, 002 etc
!                         2:  Create output for SEPPOST in the form of file
!                             sepcomp.out
!                             and for AVS in the form of a file sepavs_xxx.yyy
!     iseqlc         i    Sequence number for array to be written
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!                         In case of TUDFINVOL it is the vector sequence number
!                         to be used counted from 0
!     isubract      i/o   Parameter isubr_act, see proutpsub1
!     ivectr        i/o   sequence number of first vector to be used
!                         in the array isolvec(.,j) for output purposes.
!                         IVECTR-1 refers to the vector number in SEPPOST
!     nameavs        o    Sequence number of name of avs file in array
!                         file_names
!     namedx         o    Sequence number of name of open dx file in array
!                         file_names
!     nameparaview   o    Sequence number of name of paraview file in array
!                         file_names
!     nametec        o    Sequence number of name of tecplot file in array
!                         file_names
!     nproblc         i/o   Number of various problems
!     numveclc      i/o   Number of derivative vectors that have been written
!                         to file sepcomp.out
!     one_file       o    Indicates if all time steps must be stored in one file
!                         (1) or not (0)
!     separate_elgrp i    If true, each element group is written to a different
!                         output file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nlist
      parameter ( nlist=26 )
      integer niinp, nch, list(nlist), ifirstwrit, maxcal, &
              ipos
      save list, ifirstwrit

!     ifirstwrit     If 0 output of information allowed, &
!                    if 1, this ouput is suppressed
!     ipos           Position of input arrays in sepmoduleinput
!     list           Contains the sequence numbers of the arrays to be stored
!     maxcal         Maximum number of coefficient inputs read
!     nch            Number of characters in a string
!     niinp          Length of integer input array
!     nlist          Length of array list
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READ04         Read card input for subroutine OUTPUT
!                    management index mmindex and copy the contents
!                    of iarray to ibuffr
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data list /nlist*0/, ifirstwrit/0/
! ======================================================================
!
      call eropen ( 'proutpsub2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutpsub2'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifoutp==0 .and. isubract/=3 .and. isubract/=5 ) then

!     --- ifoutp = 0 and isub3#3,  first call of PROUTP
!                                  call by OUTPUT, OUTPOS or OUTTIM
!         Read input and store in array iinput
!         First reserve space in buffer array  (Semi permanent)

         iseqkp = kprob
         nproblc  = kp(iseqkp)%nprob
         niinp  = 1000

         allocate ( iinputout(15*niinp), stat = error )
         if ( error/=0 ) call eralloc ( error, 15*niinp, 'iinputout' )
         if ( ierror/=0 ) go to 1000

!        --- Array iinput is put in IBUFFR

         call read04 ( iinputout, niinp, nproblc, 1, list, nlist, &
                       iseqlc )
         numveclc = iinputout(1)
         iavs     = iinputout(3)
         nameavs  = iinputout(5)
         namedx   = iinputout(6)
         nametec  = iinputout(7)
         one_file = iinputout(8)
         nameparaview = iinputout(9)
         separate_elgrp = iinputout(10)==1
         appendmode = iinputout(11)
         iseqlc = 0

!        --- If isubract>1, store iinput as permanent array for later usage

         if ( numveclc==0 .and. isubract==2 ) then

!        --- numveclc = 0, but isubract = 2: deallocate iinputout

            if ( associated(iinputout) ) deallocate ( iinputout )

         end if

      else if ( isubract==3 ) then

!     --- isubract = 3, call by OUTSOL, i.e. the information has already been
!                    stored in IBUFFR and the first part written to sepcomp.inf

         call sepgetinpfinf ( 9, iseqnr, ipos, maxcal )
         if ( maxcal>0 ) then
            length = size(in(ipos)%iinput)
            allocate ( iinputout(length) )
            iinputout = in(ipos)%iinput
            numveclc = iinputout(1)
            nproblc  = iinputout(2)
            iavs     = iinputout(3)
            iseqlc   = iinputout(4)
            nameavs  = iinputout(5)
            namedx   = iinputout(6)
            nametec  = iinputout(7)
            one_file = iinputout(8)
            nameparaview = iinputout(9)
            separate_elgrp = iinputout(10)==1
            appendmode = iinputout(11)
         else
            iavs     = 1
            iseqlc   = 0
            numveclc = 0
            nameavs  = 0
            namedx   = 0
            nametec  = 0
            nproblc  = 0
            one_file = 0
            nameparaview = 0
            separate_elgrp = .false.
            appendmode = 0
         end if
         if ( nproblc==0 ) then
            iseqkp = kprob
            nproblc  = kp(iseqkp)%nprob
            nproblc = max(nproblc,numvec-ivectr+1)
         end if  ! ( nproblc==0 )

         if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 .and. &
              mainsubr>=2 .and. mainsubr<=3 .and. ifirstwrit==0 ) then

!        --- Write text in case of sepcomp or sepfree and first call

            write ( irefwr, 50 )
 50         format ( ' Write the following vectors', &
                     ' to file sepcomp.out' )
            do iprob = ivectr, ivectr+nproblc-1
               nch = index(names(iprob),' ')-1
               if ( nch<=0 ) nch = len(names(iprob))
               write ( irefwr, 60 ) names(iprob)(1:nch)
 60            format ( 10x, a )
            end do

         end if
         if ( itimloop==1 ) then
            if ( t>=tend-tstep*0.1 ) then
               ifirstwrit = 0
            else
               ifirstwrit = 1
            end if  ! ( t>=tend-dt*0.1 )
         end if  ! ( itimloop==1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nameparaview', nameparaview
         write(irefwr,*) 'End proutpsub2'

      end if
      call erclos ( 'proutpsub2' )

      end
