      subroutine prstagpres ( isolpres, isolmach, isolstagpres, rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 31-05-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the stagnation pressure in all points of the mesh, from the
!    given machnumber and pressure
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isolpres, isolmach, isolstagpres
      double precision rpost(*)

!     isolmach       i    Array of the structure of isol referring to mach
!     isolpres       i    Array of the structure of isol referring to the
!                         pressure
!     isolstagpres   i    Array of the structure of isol referring to the
!                         stagnation pressure
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRSTAGPRES1    Actual computation of stagnation pressure
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     Create or activate stagnation pressure array
!     Compute starting addresses
!     Compute stagnation pressure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Create or activate stagnation pressure array

      pressure => ks(isolpres)%sol
      machno => ks(isolmach)%sol
      call sepcreasolbf1 ( isolstagpres, ks(isolmach)%typevec, &
                           ks(isolmach)%ivec, 'stag_pressure', 0 )
      stagpres => usoltemp

!     --- Compute stagnation pressure in all vertices

      call prstagpres1 ( npoint, pressure, machno, stagpres, rpost )

      end
