      subroutine prsw05 ( ichoice_print, isuppress, nnodes, ndim, &
                          inodes, coor, iformat, heading )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the coordinates for subroutine PRINVC
!     real case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_print, isuppress, nnodes, ndim, inodes(*), iformat
      double precision coor(ndim,*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     ichoice_print  i    0:   The nodal point number is printed followed by the
!                              values in the nodal point
!                         1:   The nodal point number is printed followed by the
!                              coordinates and then the values in the
!                              nodal point
!                         2:   The nodal point number is printed followed by the
!                              coordinates
!                         3:   See 2, however, preceded by the number of nodal
!                              points to be printed
!     iformat        i    Type of format to be used in read statements
!     inodes         i    Infor entry with respect to the array containing the
!                         nodal point numbers at output
!     isuppress      i    If 0 the node numbers are printed, &
!                         if 1 they are suppressed
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw05'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice_print==3 ) then

!     --- ichoice_print = 3, print number of coordinates

         if ( heading ) write(irefwr,*) 'number of nodes'
         write(irefwr,*) nnodes

      end if

      if ( isuppress==0 ) then

!     --- isuppress = 0, print node numbers

         if ( ndim==1 ) then

            if ( heading ) write(irefwr,100)
100         format(' node       coordinates')
            if ( iformat==1 .or. iformat==3 ) then
               write(irefwr,110) (inodes(i), coor(1,inodes(i)), &
                                  i=1,nnodes)
            else
               write(irefwr,111) (inodes(i), coor(1,inodes(i)), &
                                  i=1,nnodes)
            end if
110         format((1x,i5,ss,1pe13.5e2))
111         format((1x,i5,ss,1pe16.8e2))
         else if ( ndim==2 ) then
            if ( heading ) write(irefwr,120)
120         format(' node       x            y')
            if ( iformat==1 .or. iformat==3 ) then
               write(irefwr,130) (inodes(i), &
                                 (coor(j,inodes(i)),j=1,2), &
                                  i=1,nnodes)
            else
               write(irefwr,131) (inodes(i), &
                                 (coor(j,inodes(i)),j=1,2), &
                                  i=1,nnodes)
            end if
130         format((1x,i5,ss,1pe13.5e2,1pe13.5e2))
131         format((1x,i5,ss,1pe13.5e2,1pe16.8e2))
         else
            if ( heading ) write(irefwr,140)
140         format(' node       x            y           z      ')
            if ( iformat==1 .or. iformat==3 ) then
               write(irefwr,150) (inodes(i), &
                                 (coor(j,inodes(i)),j=1,3), &
                                  i=1,nnodes)
            else
               write(irefwr,151) (inodes(i), &
                                 (coor(j,inodes(i)),j=1,3), &
                                  i=1,nnodes)
            end if
150         format((1x,i5,ss,3(1pe13.5e2)))
151         format((1x,i5,ss,3(1pe16.8e2)))
         end if

      else

!     --- isuppress = 1, do not print node numbers

         if ( ndim==1 ) then
            if ( heading ) write(irefwr,160)
160         format('  coordinates')
            if ( iformat==1 .or. iformat==3 ) then
               write(irefwr,170) (coor(1,inodes(i)), i=1,nnodes)
            else
               write(irefwr,171) (coor(1,inodes(i)), i=1,nnodes)
            end if
170         format((1x,ss,1pe13.5e2))
171         format((1x,ss,1pe16.8e2))
         else if ( ndim==2 ) then
            if ( heading ) write(irefwr,180)
180         format('   x             y')
            if ( iformat==1 .or. iformat==3 ) then
               write(irefwr,190) ((coor(j,inodes(i)),j=1,2), &
                                  i=1,nnodes)
            else
               write(irefwr,191) ((coor(j,inodes(i)),j=1,2), &
                                  i=1,nnodes)
            end if
190         format((1x,ss,1pe13.5e2,1pe13.5e2))
191         format((1x,ss,1pe13.5e2,1pe16.8e2))
         else
            if ( heading ) write(irefwr,200)
200         format('   x             y           z      ')
            if ( iformat==1 .or. iformat==3 ) then
               write(irefwr,210) ((coor(j,inodes(i)),j=1,3), &
                                  i=1,nnodes)
            else
               write(irefwr,211) ((coor(j,inodes(i)),j=1,3), &
                                  i=1,nnodes)
            end if
210         format((1x,ss,3(1pe13.5e2)))
211         format((1x,ss,3(1pe16.8e2)))
         end if

      end if

1000  call erclos ( 'prsw05' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw05'

      end if

      end
