      subroutine prsw06c8 ( iformat, inodes, nnodes, usol, heading, &
                            coor, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     complex case
!     Print coordinates
!     Do not print node numbers
!     One unknown to be printed
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nnodes, iformat, ndim
      double precision coor(ndim,*)
      complex * 16 usol(*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     Print of vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06c8' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06c8'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==1 ) then
         if ( heading ) write(irefwr,330)
330      format('      x           value')
         if ( iformat==1 .or. iformat==3 ) then
            write(irefwr,340) (coor(1,inodes(i)), &
                               usol(i), i=1,nnodes)
         else
            write(irefwr,341) (coor(1,inodes(i)), &
                               usol(i), i=1,nnodes)
         end if
340      format((1x,ss,1pe13.5e2,2x,'(',1pe13.5e2,',',1pe13.5e2,1x,')'))
341      format((1x,ss,1pe16.8e2,2x,'(',1pe16.8e2,',',1pe16.8e2,1x,')'))

      else if ( ndim==2 ) then

         if ( heading ) write(irefwr,350)
350      format('      x            y         value')
         if ( iformat==1 .or. iformat==3 ) then
            write(irefwr,360) ((coor(j,inodes(i)),j=1,2), &
                               usol(i), i=1,nnodes)
         else
            write(irefwr,361) ((coor(j,inodes(i)),j=1,2), &
                               usol(i), i=1,nnodes)
         end if
360      format((1x,ss,2(1pe13.5e2),2x, &
                '(',1pe13.5e2,',',1pe13.5e2,1x,')'))
361      format((1x,ss,2(1pe16.8e2),2x, &
                '(',1pe16.8e2,',',1pe16.8e2,1x,')'))

      else

         if ( heading ) write(irefwr,370)
370      format('      x            y           z      ')
         if ( iformat==1 .or. iformat==3 ) then
            write(irefwr,380) ((coor(j,inodes(i)),j=1,3), &
                               usol(i), i=1,nnodes)
         else
            write(irefwr,381) ((coor(j,inodes(i)),j=1,3), &
                               usol(i), i=1,nnodes)
         end if
380      format((1x,ss,3(1pe13.5e2),2x, &
                '(',1pe13.5e2,',',1pe13.5e2,1x,')'))
381      format((1x,ss,3(1pe16.8e2),2x, &
                '(',1pe16.8e2,',',1pe16.8e2,1x,')'))

      end if

1000  call erclos ( 'prsw06c8' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06c8'

      end if  ! ( debug )

      end
