      subroutine prsw06csup ( ichoice_print, number, nunkp, isuppress, &
                              inodes, nnodes, usol, indprf, kprobf, &
                              coor, iformat, heading, ndim, icoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_print, number, nunkp, isuppress, inodes(*), &
              nnodes, indprf, kprobf(*), iformat, ndim, icoor
      double precision coor(ndim,*)
      complex *16 usol(*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     ichoice_print  i    0:   The nodal point number is printed followed by the
!                                 values in the nodal point
!                         1:   The nodal point number is printed followed by the
!                                 coordinates and then the values in the
!                                 nodal point
!                         2:   The nodal point number is printed followed by the
!                                 coordinates
!                         3:   See 2, however, preceded by the number of nodal
!                                 points to be printed
!     icoor          i    Type of coordinate system
!                         1: Cartesian
!                         2: Cylindrical
!                         3: Polar
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     isuppress      i    If 0 the node numbers are printed, &
!                         if 1 they are suppressed
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, istart, icount, ind(nnodes)
      double precision x, y, r, phi
      logical zero

!     i              Counting variable
!     icount         Number of nodes to be printed
!     ind            Integer array of length icount containing the
!                    indices of the nodes to be printed
!     inunkp         Number of degrees of freedom in first point
!     istart         First position used at a part
!     j              Counting variable
!     phi            Angle
!     r              Radius
!     x              x coordinate
!     y              y coordinate
!     zero           If true all unknowns in the node are zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06csup' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06csup'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice_print==0 ) then

!     --- ichoice_print = 0, do not print coordinates

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown
!            First store the indices of non-zero entries

            icount = 0     ! number of non-zero entries
            do i = 1, nnodes

               if ( abs(usol(i))>0d0 ) then

!              --- usol(i) # 0, store sequence number in ind

                  icount = icount+1
                  ind(icount) = i

               end if  ! ( abs(usol(i)>0d0 )

            end do  ! i = 1, nnodes

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( iformat==1 ) then
                  write(irefwr,220) &
                    (inodes(ind(i)), usol(ind(i)), i=1,icount)
               else if ( iformat==2 ) then
                  write(irefwr,221) &
                    (inodes(ind(i)), usol(ind(i)), i=1,icount)
               else if ( iformat==3 ) then
                  write(irefwr,222) &
                    (inodes(ind(i)), usol(ind(i)), i=1,icount)
               else
                  write(irefwr,223) &
                    (inodes(ind(i)), usol(ind(i)), i=1,icount)
               end if
220            format((1x,2(i5,':',ss, &
                      '(',1pe13.5e2,',',1pe13.5e2,1x,')')))
221            format((1x,2(i5,':',ss, &
                      '(',1pe16.8e2,',',1pe16.8e2,1x,')')))
222            format((1x, (i5,':',ss,'(',1pe13.5e2, &
                      ',',1pe13.5e2,1x,')')))
223            format((1x, (i5,':',ss, &
                      '(',1pe16.8e2,',',1pe16.8e2,1x,')')))

            else

!           --- isuppress = 1, do not print node numbers

               if ( iformat==1 ) then
                  write(irefwr,230) (usol(ind(i)), i=1,icount)
               else if ( iformat==2 ) then
                  write(irefwr,231) (usol(ind(i)), i=1,icount)
               else if ( iformat==3 ) then
                  write(irefwr,232) (usol(ind(i)), i=1,icount)
               else
                  write(irefwr,233) (usol(ind(i)), i=1,icount)
               end if
230            format((1x,2(ss, &
                      '(',1pe13.5e2,',',1pe13.5e2,1x,')')))
231            format((1x,2(ss, &
                      '(',1pe16.8e2,',',1pe16.8e2,1x,')')))
232            format((1x, (ss,'(',1pe13.5e2,',',1pe13.5e2,1x,')')))
233            format((1x, ('(',1pe16.8e2,',',1pe16.8e2,1x,')')))

            end if

         else if ( indprf<=0 ) then

!        --- more unknowns

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( heading ) write(irefwr,240)
240            format(' node       values'/)
               do i = 1, nnodes

!              --- Check if solution is zero in node i

                  zero = .true.
                  do j = 1, nunkp
                     if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
                  end do  ! j = 1, nunkp
                  if ( .not. zero ) then

!                 --- Solution is not equal to zero: print

                     if ( iformat==1 ) then
                        write(irefwr,250) inodes(i), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     else if ( iformat==2 ) then
                        write(irefwr,251) inodes(i), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     else if ( iformat==3 ) then
                        write(irefwr,252) inodes(i), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else
                        write(irefwr,253) inodes(i), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     end if
250                  format(1x,i5,':',ss, &
                            2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
251                  format(1x,i5,':', &
                            2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
252                  format(1x,i5,':',ss, &
                            99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
253                  format(1x,i5,':',ss, &
                            99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
                     if ( iformat==1 ) then
                        if ( nunkp>2 ) write(irefwr,260) &
                           (usol(j+(i-1)*nunkp), j=3,nunkp)
                     else if ( iformat==2 ) then
                        if ( nunkp>2 ) write(irefwr,261) &
                           (usol(j+(i-1)*nunkp), j=3,nunkp)
                     end if
260                  format((7x,ss, &
                            2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
261                  format((7x,ss, &
                            2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))

                  end if  ! ( .not. zero )

               end do

            else

!           --- isuppress = 1, do not print node numbers

               if ( heading ) write(irefwr,270)
270            format('   values'/)
               do i = 1, nnodes

!              --- Check if solution is zero in node i

                  zero = .true.
                  do j = 1, nunkp
                     if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
                  end do  ! j = 1, nunkp
                  if ( .not. zero ) then

!                 --- Solution is not equal to zero: print

                     if ( iformat==1 ) then
                        write(irefwr,280) &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else if ( iformat==2 ) then
                        write(irefwr,281) &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else if ( iformat==3 ) then
                        write(irefwr,282) &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else
                        write(irefwr,283) &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     end if
280                  format(1x,ss, &
                            2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
281                  format(1x,ss, &
                            2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
282                  format(1x,ss, &
                            99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
283                  format(1x,ss, &
                            99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))

                  end if  ! ( .not. zero )

               end do

            end if

         else

!        --- number of unknowns not constant

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( heading ) write(irefwr,240)
               do i = 1, nnodes
                  istart = kprobf(i)
                  inunkp = kprobf(i+1)-istart

!                 --- Check if solution is zero in node i

                  zero = .true.
                  do j = 1, inunkp
                     if ( abs(usol(j+istart))>0d0 ) zero = .false.
                  end do  ! j = 1, nunkp
                  if ( .not. zero ) then

!                 --- Solution is not equal to zero: print

                     if ( iformat==1 ) then
                        write(irefwr,250) inodes(i), &
                           (usol(j+istart), j=1,min(5,inunkp))
                     else if ( iformat==2 ) then
                        write(irefwr,251) inodes(i), &
                           (usol(j+istart), j=1,min(5,inunkp))
                     else if ( iformat==3 ) then
                        write(irefwr,252) inodes(i), &
                           (usol(j+istart), j=1,inunkp)
                     else
                        write(irefwr,253) inodes(i), &
                           (usol(j+istart), j=1,inunkp)
                     end if
                     if ( iformat==1 ) then
                        if ( inunkp>5 ) write(irefwr,260) &
                           (usol(j+istart), j=6,inunkp)
                     else if ( iformat==2 ) then
                        if ( inunkp>5 ) write(irefwr,261) &
                           (usol(j+istart), j=6,inunkp)
                     end if

                  end if  ! ( .not. zero )

               end do

            else

!           --- isuppress = 1, do not print node numbers

               if ( heading ) write(irefwr,270)
               do i = 1, nnodes
                  istart = kprobf(i)
                  inunkp = kprobf(i+1)-istart

!                 --- Check if solution is zero in node i

                  zero = .true.
                  do j = 1, inunkp
                     if ( abs(usol(j+istart))>0d0 ) zero = .false.
                  end do  ! j = 1, nunkp
                  if ( .not. zero ) then

!                 --- Solution is not equal to zero: print

                     if ( iformat==1 ) then
                        write(irefwr,280) &
                           (usol(j+istart), j=1,inunkp)
                     else if ( iformat==2 ) then
                        write(irefwr,281) &
                           (usol(j+istart), j=1,inunkp)
                     else if ( iformat==3 ) then
                        write(irefwr,282) &
                           (usol(j+istart), j=1,inunkp)
                     else
                        write(irefwr,283) &
                           (usol(j+istart), j=1,inunkp)
                     end if

                  end if  ! ( .not. zero )

               end do

            end if

         end if

      else if ( icoor<=2 .or. ndim/=2 ) then

!     --- ichoice_print = 1, print coordinates

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown
!            First store the indices of non-zero entries

            icount = 0     ! number of non-zero entries
            do i = 1, nnodes

               if ( abs(usol(i))>0d0 ) then

!              --- usol(i) # 0, store sequence number in ind

                  icount = icount+1
                  ind(icount) = i

               end if  ! ( abs(usol(i)>0d0 )

            end do  ! i = 1, nnodes

            if ( ndim==1 ) then
               if ( heading ) write(irefwr,330)
330            format(' node       x           value')
               if ( iformat==1 .or. iformat==3 ) then
                  write(irefwr,340) (inodes(ind(i)), &
                                     coor(1,inodes(ind(i))), &
                                     usol(ind(i)), i=1,icount)
               else
                  write(irefwr,341) (inodes(ind(i)), &
                                     coor(1,inodes(ind(i))), &
                                     usol(ind(i)), i=1,icount)
               end if
340            format((1x,i5,ss,1pe13.5e2, &
                      '(',1pe13.5e2,',',1pe13.5e2,1x,')'))
341            format((1x,i5,ss,1pe16.8e2, &
                      '(',1pe16.8e2,',',1pe16.8e2,1x,')'))
            else if ( ndim==2 ) then
               if ( heading ) write(irefwr,350)
350            format(' node       x            y         value')
               if ( iformat==1 .or. iformat==3 ) then
                  write(irefwr,360) (inodes(ind(i)), &
                                    (coor(j,inodes(ind(i))),j=1,2), &
                                     usol(ind(i)), i=1,icount)
               else
                  write(irefwr,361) (inodes(ind(i)), &
                                    (coor(j,inodes(ind(i))),j=1,2), &
                                     usol(ind(i)), i=1,icount)
               end if
360            format((1x,i5,ss,2(1pe13.5e2), &
                      '(',1pe13.5e2,',',1pe13.5e2,1x,')'))
361            format((1x,i5,ss,2(1pe16.8e2), &
                      '(',1pe16.8e2,',',1pe16.8e2,1x,')'))
            else

               if ( heading ) write(irefwr,370)
370            format(' node       x            y           z      ')
               if ( iformat==1 .or. iformat==3 ) then
                  write(irefwr,380) (inodes(ind(i)), &
                                    (coor(j,inodes(ind(i))),j=1,3), &
                                     usol(ind(i)), i=1,icount)
               else
                  write(irefwr,381) (inodes(ind(i)), &
                                    (coor(j,inodes(ind(i))),j=1,3), &
                                     usol(ind(i)), i=1,icount)
               end if
380            format((1x,i5,3(1pe13.5e2), &
                      '(',1pe13.5e2,',',1pe13.5e2,1x,')'))
381            format((1x,i5,ss,3(1pe16.8e2), &
                      '(',1pe16.8e2,',',1pe16.8e2,1x,')'))
            end if

         else if ( indprf<=0 ) then

!        --- more unknowns

            if ( heading ) then
               if ( ndim==1 ) then
                  write(irefwr,390)
390               format(' node       x        ', &
                         'values'/)
               else if ( ndim==2 ) then
                  write(irefwr,400)
400               format(' node       x            y         ', &
                         'values'/)
               else
                  write(irefwr,410)
410               format(' node       x            y            ', &
                         'z         values'/)
               end if
            end if
            do i = 1, nnodes

!           --- Check if solution is zero in node i

               zero = .true.
               do j = 1, nunkp
                  if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
               end do  ! j = 1, nunkp
               if ( .not. zero ) then

!              --- Solution is not equal to zero: print

                  if ( ndim==1 ) then
                     if ( iformat==1 ) then
                        write(irefwr,420) inodes(i), coor(1,inodes(i)), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     else if ( iformat==2 ) then
                        write(irefwr,421) inodes(i), coor(1,inodes(i)), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     else if ( iformat==3 ) then
                        write(irefwr,422) inodes(i), coor(1,inodes(i)), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else
                        write(irefwr,423) inodes(i), coor(1,inodes(i)), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     end if
                     if ( iformat==1 ) then
                        if ( nunkp>2 ) write(irefwr,430) &
                           (usol(j+(i-1)*nunkp), j=3,nunkp)
                     else if ( iformat==2 ) then
                        if ( nunkp>2 ) write(irefwr,431) &
                           (usol(j+(i-1)*nunkp), j=3,nunkp)
                     end if
                  else if ( ndim==2 ) then
                     if ( iformat==1 ) then
                        write(irefwr,440) inodes(i), &
                           (coor(j,inodes(i)),j=1,2), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     else if ( iformat==2 ) then
                        write(irefwr,441) inodes(i), &
                           (coor(j,inodes(i)),j=1,2), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     else if ( iformat==3 ) then
                        write(irefwr,442) inodes(i), &
                           (coor(j,inodes(i)),j=1,2), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else
                        write(irefwr,443) inodes(i), &
                           (coor(j,inodes(i)),j=1,2), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     end if
                     if ( iformat==1 ) then
                        if ( nunkp>2 ) write(irefwr,450) &
                           (usol(j+(i-1)*nunkp), j=2,nunkp)
                     else if ( iformat==2 ) then
                        if ( nunkp>2 ) write(irefwr,451) &
                           (usol(j+(i-1)*nunkp), j=2,nunkp)
                     end if
                  else
                     if ( iformat==1 ) then
                        write(irefwr,440) inodes(i), &
                           (coor(j,inodes(i)),j=1,3), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                        if ( nunkp>2 ) write(irefwr,460) &
                           (usol(j+(i-1)*nunkp), j=3,nunkp)
                     else if ( iformat==2 ) then
                        write(irefwr,441) inodes(i), &
                           (coor(j,inodes(i)),j=1,3), &
                           (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                        if ( nunkp>2 ) write(irefwr,461) &
                           (usol(j+(i-1)*nunkp), j=3,nunkp)
                     else if ( iformat==3 ) then
                        write(irefwr,442) inodes(i), &
                           (coor(j,inodes(i)),j=1,3), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     else
                        write(irefwr,443) inodes(i), &
                           (coor(j,inodes(i)),j=1,3), &
                           (usol(j+(i-1)*nunkp), j=1,nunkp)
                     end if
                  end if

               end if  ! ( .not. zero )

            end do

         else

!        --- number of unknowns not constant

            if ( heading ) then
               if ( ndim==1 ) then
                  write(irefwr,390)
               else if ( ndim==2 ) then
                  write(irefwr,400)
               else
                  write(irefwr,410)
               end if
            end if
            do i = 1, nnodes
               istart = kprobf(i)
               inunkp = kprobf(i+1)-istart

!              --- Check if solution is zero in node i

               zero = .true.
               do j = 1, inunkp
                  if ( abs(usol(j+istart))>0d0 ) zero = .false.
               end do  ! j = 1, nunkp
               if ( .not. zero ) then

!              --- Solution is not equal to zero: print

                   if ( ndim==1 ) then
                      if ( iformat==1 ) then
                         write(irefwr,420) inodes(i), &
                            coor(1,inodes(i)), &
                            (usol(j+istart), j=1,min(2,inunkp))
                         if ( inunkp>2 ) write(irefwr,430) &
                            coor(1,inodes(i)), &
                            (usol(j+istart), j=3,inunkp)
                      else if ( iformat==2 ) then
                         write(irefwr,421) inodes(i), &
                            coor(1,inodes(i)), &
                            (usol(j+istart), j=1,min(2,inunkp))
                         if ( inunkp>2 ) write(irefwr,431) &
                            coor(1,inodes(i)), &
                            (usol(j+istart), j=3,inunkp)
                      else if ( iformat==3 ) then
                         write(irefwr,422) inodes(i), &
                            coor(1,inodes(i)), &
                            (usol(j+istart), j=1,inunkp)
                      else
                         write(irefwr,423) inodes(i), &
                            coor(1,inodes(i)), &
                            (usol(j+istart), j=1,inunkp)
                      end if
                   else if ( ndim==2 ) then
                      if ( iformat==1 ) then
                         write(irefwr,440) inodes(i), &
                            (coor(j,inodes(i)),j=1,2), &
                            (usol(j+istart), j=1,min(2,inunkp))
                         if ( inunkp>3 ) write(irefwr,450) &
                            (usol(j+istart), j=4,inunkp)
                      else if ( iformat==2 ) then
                         write(irefwr,441) inodes(i), &
                            (coor(j,inodes(i)),j=1,2), &
                            (usol(j+istart), j=1,min(2,inunkp))
                         if ( inunkp>2 ) write(irefwr,451) &
                            (usol(j+istart), j=3,inunkp)
                      else if ( iformat==3 ) then
                         write(irefwr,442) inodes(i), &
                            (coor(j,inodes(i)),j=1,2), &
                            (usol(j+istart), j=1,inunkp)
                      else
                         write(irefwr,443) inodes(i), &
                            (coor(j,inodes(i)),j=1,2), &
                            (usol(j+istart), j=1,inunkp)
                      end if
                   else
                      if ( iformat==1 ) then
                         write(irefwr,440) inodes(i), &
                            (coor(j,inodes(i)),j=1,3), &
                            (usol(j+istart), j=1,min(2,inunkp))
                         if ( inunkp>2 ) write(irefwr,460) &
                            (usol(j+istart), j=3,inunkp)
                      else if ( iformat==2 ) then
                         write(irefwr,441) inodes(i), &
                            (coor(j,inodes(i)),j=1,3), &
                            (usol(j+istart), j=1,min(2,inunkp))
                         if ( inunkp>2 ) write(irefwr,461) &
                            (usol(j+istart), j=3,inunkp)
                      else if ( iformat==3 ) then
                         write(irefwr,442) inodes(i), &
                            (coor(j,inodes(i)),j=1,3), &
                            (usol(j+istart), j=1,inunkp)
                      else
                         write(irefwr,443) inodes(i), &
                            (coor(j,inodes(i)),j=1,3), &
                            (usol(j+istart), j=1,inunkp)
                      end if
                   end if

               end if  ! ( .not. zero )

            end do

         end if

      else

!     --- ichoice_print = 1, print coordinates and transform to polar ones

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown

            if ( heading ) write(irefwr,350)
            do i = 1, nnodes
               x = coor(1,inodes(i))
               y = coor(2,inodes(i))
               r = sqrt(x**2+y**2)
               if ( x>0d0 ) then
                  phi = atan(y/x)
               else
                  phi = 0d0
               end if
               if ( abs(usol(i))>0d0 ) &
                  write(irefwr,360) inodes(i), r, phi, usol(i)
            end do

         else if ( indprf<=0 ) then

!        --- more unknowns

            if ( heading ) write(irefwr,400)
            do i = 1, nnodes

!           --- Check if solution is zero in node i

               zero = .true.
               do j = 1, nunkp
                  if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
               end do  ! j = 1, nunkp
               if ( .not. zero ) then

!              --- Solution is not equal to zero: print

                  x = coor(1,inodes(i))
                  y = coor(2,inodes(i))
                  r = sqrt(x**2+y**2)
                  if ( x>0d0 ) then
                     phi = atan(y/x)
                  else
                     phi = 0d0
                  end if
                  if ( iformat==1 ) then
                     write(irefwr,440) inodes(i), r, phi, &
                        (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     if ( nunkp>2 ) write(irefwr,450) &
                        (usol(j+(i-1)*nunkp), j=3,nunkp)
                  else if ( iformat==2 ) then
                     write(irefwr,441) inodes(i), r, phi, &
                        (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                     if ( nunkp>2 ) write(irefwr,451) &
                        (usol(j+(i-1)*nunkp), j=3,nunkp)
                  else if ( iformat==3 ) then
                     write(irefwr,442) inodes(i), r, phi, &
                        (usol(j+(i-1)*nunkp), j=1,nunkp)
                  else
                     write(irefwr,443) inodes(i), r, phi, &
                        (usol(j+(i-1)*nunkp), j=1,nunkp)
                  end if

               end if  ! ( .not. zero )

            end do

         else

!        --- number of unknowns not constant

            if ( heading ) write(irefwr,400)
            do i = 1, nnodes
               istart = kprobf(i)
               inunkp = kprobf(i+1)-istart

!              --- Check if solution is zero in node i

               zero = .true.
               do j = 1, inunkp
                  if ( abs(usol(j+istart))>0d0 ) zero = .false.
               end do  ! j = 1, nunkp
               if ( .not. zero ) then

!              --- Solution is not equal to zero: print

                  x = coor(1,inodes(i))
                  y = coor(2,inodes(i))
                  r = sqrt(x**2+y**2)
                  if ( x>0d0 ) then
                     phi = atan(y/x)
                  else
                     phi = 0d0
                  end if
                  if ( iformat==1 ) then
                     write(irefwr,440) inodes(i), r, phi, &
                        (usol(j+istart), j=1,min(2,inunkp))
                     if ( inunkp>2 ) write(irefwr,450) &
                        (usol(j+istart), j=3,inunkp)
                  else if ( iformat==2 ) then
                     write(irefwr,441) inodes(i), r, phi, &
                        (usol(j+istart), j=1,min(2,inunkp))
                     if ( inunkp>2 ) write(irefwr,451) &
                        (usol(j+istart), j=3,inunkp)
                  else if ( iformat==3 ) then
                     write(irefwr,442) inodes(i), r, phi, &
                        (usol(j+istart), j=1,inunkp)
                  else
                     write(irefwr,443) inodes(i), r, phi, &
                        (usol(j+istart), j=1,inunkp)
                  end if

               end if  ! ( .not. zero )

            end do

         end if

      end if
420   format(1x,i5,':',ss,1pe13.5e2, &
             2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
421   format(1x,i5,':',ss,1pe16.8e2, &
             2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
422   format(1x,i5,':',ss,1pe13.5e2, &
             98('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
423   format(1x,i5,':',ss,1pe16.8e2, &
             98('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
430   format((20x,ss, &
             2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
431   format((20x,ss, &
             2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))
440   format((1x,i5,':',ss,2(1pe13.5e2), &
             2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
441   format((1x,i5,':',ss,2(1pe16.8e2), &
             2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))
442   format(1x,i5,':',ss, &
             99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
443   format(1x,i5,':',ss, &
             99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
450   format((33x,ss, &
             2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
451   format((33x,ss, &
             2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))
460   format((46x,ss,('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
461   format((46x,ss,('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06csup'

      end if
      call erclos ( 'prsw06csup' )

      end
