      subroutine prsw06sup02 ( nunkp, isuppress, inodes, nnodes, usol, &
                               iformat, heading )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     real case
!     Special case ichoice_print=0, indprf<=0, number=0 and nunkp>1
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkp, isuppress, inodes(*), &
              nnodes, iformat
      double precision usol(*)
      logical heading

!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     isuppress      i    If 0 the node numbers are printed, &
!                         if 1 they are suppressed
!     nnodes         i    Number of nodes in inodes
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      logical zero

!     i              Counting variable
!     j              Counting variable
!     zero           If true all unknowns in the node are zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06sup02' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06sup02'

      end if
      if ( ierror/=0 ) go to 1000

      if ( isuppress==0 ) then

!     --- isuppress = 0, print node numbers

         if ( heading ) write(irefwr,240)
         do i = 1, nnodes

!        --- Check if solution is zero in node i

            zero = .true.
            do j = 1, nunkp
               if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
            end do  ! j = 1, nunkp
            if ( .not. zero ) then

!           --- Solution is not equal to zero: print

               if ( iformat==1 ) then
                  write(irefwr,250) inodes(i), &
                     (usol(j+(i-1)*nunkp), j=1,min(5,nunkp))
               else if ( iformat==2 ) then
                  write(irefwr,251) inodes(i), &
                     (usol(j+(i-1)*nunkp), j=1,min(5,nunkp))
               else if ( iformat==3 ) then
                  write(irefwr,252) inodes(i), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else
                  write(irefwr,253) inodes(i), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               end if
               if ( iformat==1 ) then
                  if ( nunkp>5 ) write(irefwr,260) &
                     (usol(j+(i-1)*nunkp), j=6,nunkp)
               else if ( iformat==2 ) then
                  if ( nunkp>5 ) write(irefwr,261) &
                     (usol(j+(i-1)*nunkp), j=6,nunkp)
               end if

            end if  ! ( .not. zero )

         end do

      else

!     --- isuppress = 1, do not print node numbers

         if ( heading ) write(irefwr,270)
         do i = 1, nnodes

!        --- Check if solution is zero in node i

            zero = .true.
            do j = 1, nunkp
               if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
            end do  ! j = 1, nunkp
            if ( .not. zero ) then

!           --- Solution is not equal to zero: print

               if ( iformat==1 ) then
                  write(irefwr,280) &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else if ( iformat==2 ) then
                  write(irefwr,281) &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else if ( iformat==3 ) then
                  write(irefwr,282) &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else
                  write(irefwr,283) &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               end if

            end if  ! ( .not. zero )

         end do

      end if

240   format(' node       values'/)
250   format(1x,i9,':',ss,5(1pe13.5e2))
251   format(1x,i9,':',ss,5(1pe16.8e2))
252   format(1x,i9,':',ss,99(1pe13.5e2))
253   format(1x,i9,':',ss,99(1pe16.8e2))
260   format((7x,ss,5(1pe13.5e2)))
261   format((7x,ss,5(1pe16.8e2)))
270   format('   values'/)
280   format(1x,ss,5(1pe13.5e2))
281   format(1x,ss,5(1pe16.8e2))
282   format(1x,ss,99(1pe13.5e2))
283   format(1x,ss,99(1pe16.8e2))

1000  call erclos ( 'prsw06sup02' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06sup02'

      end if

      end
