      subroutine prtoavshead ( isol, iref, numveclc, &
                               numveclccomps, maxvec1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write heading for paraview output
!
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5,*), iref, numveclc, numveclccomps(*), maxvec1

!     iref           i    File reference number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     maxvec1        i    Maximum number of vectors in isol to be written
!     numveclc         i    Number of vectors to be written
!     numveclccomps    i    Array to store the number of degrees of freedom per
!                         point for each vector to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lenname

!     i              Counting variable
!     lenname        Length of name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2867   Value of inunkp not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtoavshead' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtoavshead'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- write number of vectors and corresponding dimensions

      write(iref,100 ) numveclc, (numveclccomps(i), i=1,numveclc)
100   format ( 100i3 )
110   format ( a, a )

!     --- For all vectors 1 to maxvec1 do

      do i = 1, maxvec1

         if ( isol(2,i)==110 .or. isol(2,i)==115 ) then

!        --- Vector has been filled and is of type 110 or 115
!            Determine type of vector, inunkp and number of vectors

            call sepactsolbf2 ( isol(1,i) )

            if ( nphysi==1 .or. nphysi==3 .or. nphysi==6 .or. &
                 nphysi==7 ) then

!           --- Scalar, vector or tensor
!               Find name and length of name

               lenname = index(names(i),' ')-1
               if ( lenname<0 ) lenname = len(names(i))
               write(iref,110) names(i)(1:lenname), ', unk'
               if ( debug ) write(irefwr,*) names(i)(1:lenname)

            else if ( nphysi==4 ) then

!           --- Four degrees of freedom per point
!               This must be a vector consisting of a velocity part
!               and a pressure part (type 903)

               write(iref,110) 'velocity', ', m/sec'
               write(iref,110) 'pressure', ', pascal'

            else

!           --- Situation not yet implemented

               call errint ( nphysi, 1 )
               call errsub ( 2867, 1, 0, 0 )
               go to 1000

            end if  ! ( nphysi==1 )

         end if  ! ( isol(1,i)>0 )

      end do  ! i = 1, maxvec1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtoavshead'

      end if  ! ( debug )
      call erclos ( 'prtoavshead' )

      end
