      subroutine prtotenth ( isolvel, isolenth, &
                             isoltotenth, ipost, rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 31-05-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the total enthalpy in all points of the mesh, from the given
!    velocity and enthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isolvel, isolenth, ipost(*), isoltotenth
      double precision rpost(*)

!     ipost          i    Integer array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     isolenth       i    Array of the structure of isol referring to the
!                         enthalpy
!     isoltotenth   i/o   Array of the structure of isol referring to total
!                         enthalpy
!     isolvel        i    Array of the structure of isol referring to
!                         the velocity
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRTOTENTH1     Actual computation of total enthalpy
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     Create or activate total enthalpy array
!     Compute starting addresses
!     Compute total enthalpy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Create or activate total enthalpy array

      velocity => ks(isolvel)%sol
      enthalpy => ks(isolenth)%sol
      call sepcreasolbf1 ( isoltotenth, ks(isolenth)%typevec, &
                           ks(isolenth)%ivec, 'tot_enthalpy', 0 )

!     --- Compute total enthalpy in all vertices

      call prtotenth1 ( npoint, velocity, enthalpy, usoltemp, ipost, rpost )

      end
