      subroutine sep2paraviewel ( isol, itimelc, iseq, maxvec1, nameparaview )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 09-04-2014 New call to prtoavsdim
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 30-11-2012 Long integers
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution arrays and mesh for each element group to a set of
!     files in avs format such that it can be read by paraview
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemach
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), &
              itimelc, iseq, maxvec1, nameparaview

!     iseq           i    Sequence number for array to be written
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     itimelc        i    Integer time sequence number
!     maxvec1        i    Maximum number of vectors per time-step
!     nameparaview   i    Sequence number of name of paraview file in array
!                         file_names
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, nchar, &
              numveclc, nunkp, numveccomps(100), ielgrp
      character (len=100) filename, filenm

!     filename       Name of file
!     filenm         Name of file
!     ielgrp         element group number
!     iref           File reference number
!     nchar          Number of characters in a word
!     numveccomps    Array to store the number of degrees of freedom per
!                    point for each vector to be written
!     numveclc         Number of vectors to be written
!     nunkp          Maximum number of unknowns to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref, getshapeelgrp

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     GETSHAPEELGRP  Get the shape of an element in group ielgrp
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRININ         print 1d integer array
!     PRTOAVSDIM     Define number of components and number of vectors for
!                    paraview output
!     SEP2PARELGRP   Write solution arrays and mesh to a file in avs format
!                    such that it can be read by paraview
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sep2paraviewel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sep2paraviewel'
         write(irefwr,1) 'iref74', iref74
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=3 ) then

!     --- ndim# 3, not implemented

         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errwar ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim/=3 )

!     --- Open file sepparaview_xxx.inp

      iref = inisetnewref()
      if ( debug ) &
         write(irefwr,1) 'ndim, iref, nameparaview', &
                          ndim, iref, nameparaview
      if ( nameparaview==0 ) then

!     --- nameparaview = 0, make file name default

         filename = 'sepparaview_'
         nchar = 12

      else

!     --- nameparaview>0, use file name from cfilenames

         filename = file_names(nameparaview)
         nchar = index ( filename, ' ' )
         filename(nchar:nchar) = '_'

      end if

!     --- The number of unknowns depends on what is to be written
!         Adapt nunkp and numveclc
!         Fill numveccomps

      call prtoavsdim ( isol, nunkp, numveclc, numveccomps, &
                        maxvec1, ndim )

      if ( debug ) then
         write(irefwr,1) 'nunkp, numveclc, maxvec1', nunkp, numveclc, maxvec1
         call prinin ( numveccomps, numveclc, 'numveccomps' )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do ielgrp = 1, nelgrp

!     --- Create file name

         if ( itimelc+iseq==1 ) then
            write ( filenm, 100 ) filename(1:nchar), ielgrp
100         format ( a,i3.3,'.','inp' )
         else
            write ( filenm, 110 ) filename(1:nchar), ielgrp, itimelc+iseq
110         format ( a,i3.3,'_',i3.3,'.inp' )
         end if  ! ( itimelc+iseq==1 )

!        --- Transform mesh into linear mesh and create mesh consisting of
!            1 element group

         ishape = kmeshf(ielgrp)
         if ( ishape>10 ) then

!        --- Volume element found

            if ( ishape/=11 ) then

!           --- Shape number not yet available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if  ! ( ishape/=11 )

!           --- Open file

            open ( unit=iref, file=filenm )
            if ( debug ) write(irefwr,*) 'file = ', filenm

!           --- Create mesh for element group
!               map solutions to this mesh and write to file iref

            call sep2parelgrp ( isol, maxvec1, iref, nunkp, &
                                numveccomps, numveclc, ielgrp )

!           --- Close file

            close ( unit = iref )

         end if  ! ( ishape>10 )

      end do  ! ielgrp = 1, nelgrp

!     --- Remove iref from lisf

      call inirmref ( iref )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sep2paraviewel'

      end if  ! ( debug )
      call erclos ( 'sep2paraviewel' )

      end
