      subroutine sepactreac
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate and create arrays for module sepmodulebuild
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesmf
      use sepmoduleoldvecs
      use sepmodulebuild
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, ih, nch
      integer, pointer :: islold(:)

!                    they are not
!     ih             Help variable to store a constant temporarily
!     islold         Reference to all SEPRAN solution vectors
!     k              Counting variable
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTIVATE    Reactivate existing array in IBUFFR
!     SEPCREATEBUF   Create permanent space in array IBUFFR/BUFFER
!     SEPFILLOLDSOLS Fill array istold with information of old solution vectors.
!                    Reserve space for the necessary arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactreac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepactreac'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      islold => SMF%islold

      ih = nelem
      if ( isubdv>0 ) then
         ih = ih*2**ndim          ! standard subdivision of elements
      else
         ih = ih*6                ! sometimes subdivision of elements
      end if  ! ( isubdv>0 )

      if ( debug ) &
         write(irefwr,1) 'nelem, ndim, numnat, ih', nelem, ndim, numnat, ih

      if ( associated(flipflop) ) then

!     --- flip flop array has been created before, check length

         if ( size(flipflop)<ih ) then
            deallocate ( flipflop )
            allocate ( flipflop(ih) )
         end if  ! ( size(flipflop)<ih )

      else

!     --- flip flop array has not been created before

         allocate ( flipflop(ih) )

      end if  ! ( associated(flipflop)

!     --- Activate solution arrays
!         This part may be changed in the future
!         Depends on subroutines elem

!     --- Compute pointers for array islold

      numtypesols = 1  ! number of different types of old vectors (at least 1)
      oldsols(1)%nunkp = nunkp

      if ( debug ) then
         write(irefwr,1) 'numtypesols, numold', numtypesols, numold
         write(irefwr,*) 'oldsol ', oldsol
      end if  ! ( debug )

      call sepfilloldsols ( islold, 1 )

      icompl = ks(islold(iseqi1))%compl+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepactreac'

      end if  ! ( debug )
      call erclos ( 'sepactreac' )

      end subroutine sepactreac
