      subroutine sepderintstart
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some tests for subroutine sepcompderint
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekprob
      use sepmodulevecs
      use sepmodulemain
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPDERINTOUTVC Create output vector for sepcompderint and
!                    Activate input vector
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPFILLOLDSOL  Fill old solutions as well as array flipflop in
!                    sepmodulebuild
!     SEPFILLOLDSOLS Fill array istold with information of old solution vectors.
!                    Reserve space for the necessary arrays
!     SEPINITBUILD   Allocate some arrays for subroutine to0050
!                    Initialize some parameters
!     SEPINITDER     Initialize parameters for subroutine TO0100
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderintstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderintstart'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      bufchanged = .true.

!     --- Perform some tests and store parameters in sepmodulebuild

      call sepinitder
      if ( ierror/=0 ) go to 1000

      if ( nusol-nbound-npboun==0 ) go to 1000  ! there are no unknowns

!     --- if usol exits, activate array and renumber if necessary

      nusolcac = nusol
      oldsol = .false.

!     --- Allocate some arrays for subroutine sepbuild
!         Initialize some parameters
!         These parameters and arrays are all used by element subroutines

      call sepinitbuild
      if ( ierror/=0 ) go to 1000

!     --- Fill some parameters in sepmodulematstr

      call sepfillmatstr2

!     --- Compute pointers for array islold

      numtypesols = 1  ! number of different types of old vectors (at least 1)
      oldsols(1)%nunkp = nunkp

      if ( debug ) then
         write(irefwr,1) 'numtypesols, numold', numtypesols, numold
         write(irefwr,*) 'oldsol ', oldsol
      end if  ! ( debug )

      call sepfilloldsols ( SMF%islold, 1 )

      icompl = ks(SMF%islold(iseqi1))%compl

!     --- Create output vector and Initialize if necessary

      call sepderintoutvc

!     --- Fill pointers in sepmodulebuild for subroutine build
!         Create relation between arrays and buffer

      call sepfilloldsol
      if ( ierror/=0 ) go to 1000

      vectin => ks(isolut2d(iseqi1))%sol
      ivectout => ks(SMF%ioutvc)%isol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderintstart'

      end if  ! ( debug )
      call erclos ( 'sepderintstart' )

      end subroutine sepderintstart
