      subroutine sepdervindex ( ikelmc, ipkproba, ioutvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays index2, index3 and index4 for subroutine deriv
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikelmc, ipkproba, ioutvc

!     ikelmc         i    Last position in KMESH part c that has been used
!     ioutvc        i/o   Standard SEPRAN array containing information
!                         about the ouput vector
!     ipkproba       i    start address in kprob part a for element group
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPDERINDEX2   fill index2 with positions of degrees of freedom in element
!                    referring to input vector
!     SEPDERINDEX5   fill index5 with positions of degrees of freedom in element
!                    referring to output vector
!     SEPFILLINDEX34 Fill array index3 with addresses of unknowns for one
!                    element.
!                    Fill array index4 starting position for node in index3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdervindex' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdervindex'
         write(irefwr,1) 'nindex, iseqi1, iseqi2', nindex, iseqi1, iseqi2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array index2

      call sepderindex2 ( ikelmc, ipkproba )
      call sepderindex5 ( ikelmc, ipkproba, ioutvc )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays index3 and index4 if necessary

      if ( debug ) write(irefwr,1) 'numol0, numold', numol0, numold
      if ( numol0>0 .or. numold>1 ) then

!     --- numol0>0, there are several vectors

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         call sepfillindex34 ( E%islold, ikelmc, nindex+inpelm )

      else

!     --- numol0 = 0, there is only one vector

         index3(1,1:icount) = index2(1:icount)
         indoldsols(1)%lenindex3 = icount
         indoldsols(1)%index4 => index4(1,:)
         index4(1,inpelm+1) = 110

      end if

1000  if ( debug ) then

!     --- Debug information


         call prinin1 ( index3, icount, max(numold,numol0), 'index3' )
         call prinin ( index4, inpelm+1, 'index4' )
         write(irefwr,*) 'End sepdervindex'

      end if  ! ( debug )
      call erclos ( 'sepdervindex' )
      end subroutine sepdervindex
