      subroutine sepelderv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector and weights for derivatives
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: iuser(:), islold(:)
      double precision vecloc(numold*maxunk*3*inpelm)
      double precision, pointer :: user(:)

!     islold         User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user           Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     vecloc         Work array in which all old solution vectors for the
!                    quadrature points are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCSUBS        Call specific derivative element subroutine, depending on
!                    itype
!     ELDSUBS        Call specific derivative element subroutine, depending on
!                    itype
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelderv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelderv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iuser => E%iuser
      user => E%user
      islold => E%islold
      E%vec => elemvc1
      E%wvec => elemwg
      elemvc1 = 0d0
      vecold => oldsols(iseqi1)%oldvecs

      if ( itypvo/=116 .and. itypvo/=126 .and. &
           itypvo/=129 ) then

!     --- itypvo = 0, 110, 115
!         Standard case

         call eldsubs

      else

!     --- itypvo = 116, 126, 129 vector defined per element

         call elcsubs

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelderv'

      end if  ! ( debug )
      call erclos ( 'sepelderv' )

      end subroutine sepelderv
