      subroutine seper0075 ( inppri, ivectr, icurvs, text, rinpri )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the values of a SEPRAN array along user points, curves or surfaces
!     If necessary the tangential or normal component is printed
!     Body of the subroutines PRINCR and PRINBN
!     Old subroutine er0075
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), ivectr, icurvs(*)
      double precision rinpri(*)
      character (len=*) text

!     icurvs          i   Integer array containing the curve or surface numbers
!                         that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve or surface is used. The curve of
!                              surface number is stored in icurvs(2)
!                         -1   The solution is printed along the curves or
!                              surfaces icrv1 to icrv2.
!                              icrv1 is stored in icurvs(2) and
!                              icrv2 in icurvs(3).
!                         -2   See -1, however, the sequence of the printing is
!                              reversed. In case of curves this means that the
!                              curves are printed from the end point of icrv2 to
!                              the initial point of icrv2, then the end point of
!                              icrv2-1 to the begin point of icrv2-1 and so on.
!                              In case of surfaces ichoice=-2 is treated as
!                              ichoice=-1
!                         n>0  The curve numbers or surface numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!                         Remark: a negative curve number means that the curve
!                              is printed from end point to initial point, hence
!                              in reversed order. Negative surface numbers are
!                              not yet available.
!     inppri         i    Integer array in which the user indicates which
!                         quantities must be printed by subroutine PRINBN
!                         The entries of INPPRI must be filled as follows
!                         1:  Indicates calling subroutine:
!                             1: PRINVC
!                             2: PRINCR
!                         2:  ichoice, indicates the type of print
!                             Choice parameter consisting of 3 parts:
!                             ichoice_print, isuppress and suppress_zeros
!                             according to
!                             ichoice = ichoice_print+10*isuppress+
!                                       100*suppress_zeros
!                             with:
!                             ichoice_print
!                             0:   The nodal point number is printed followed
!                                  by the values in the nodal point
!                             1:   The nodal point number is printed followed
!                                  by the coordinates and then the values in the
!                                  nodal point
!                             2:   The nodal point number is printed followed
!                                  by the coordinates
!                             3:   See 2, however, preceded by the number of
!                                  nodal points to be printed
!                             isuppress:
!                             if 1 the nodal point number is suppressed if 0 not
!                             suppress_zeros
!                             if 1 points in which the solution is zero are
!                             skipped, if 0 not
!                         3:  IDIM, indicates if user points (idim=1),
!                             curves (idim=2), surfaces (idim=3) or
!                             volumes (idim=4) must be printed.
!                             The user point, curve or surface numbers must be
!                             stored in array ICURVS
!                         4:  IQUANTITY, indicates which quantity must be
!                             printed
!                             Possible values:
!                             0:  The contents of the vector are printed as
!                                 stored in the vector corresponding to IVECTR
!                                 i.e. without transformation.
!                             1:  The normal component of a vector along the
!                                 curve or surface is printed.
!                             2:  The tangential component(s) of a vector along
!                                 the curve or surface is printed.
!                         5:  NUMBER, indicates which degree of freedom in each
!                             point must be printed. This parameter must be seen
!                             in connection with IQUANTITY.
!                             Possible values:
!                             0:  If iquantity=0, all degrees of freedom are
!                                 printed in the nodes defined by icurvs,
!                                 if iquantity>0, the vector from which the
!                                 normal or tangential components must be
!                                 computed is supposed to be a n-dimensional
!                                 vector with first degree of freedom 1,
!                                 second degree of freedom 2 and third degree
!                                 of freedom 3. In this case it is necessary
!                                 that IDIM = NDIM (dimension of space)
!                             i>0: If iquantity=0, the degree of freedom i is
!                                 printed if iquantity>0, the vector from which
!                                 the normal or tangential components must be
!                                 computed is supposed to be a n-dimensional
!                                 vector with first degree of freedom i,
!                                 second degree of freedom i+1 and third
!                                 degree of freedom i+2.
!                                 In this case it is necessary
!                                 that IDIM = NDIM (dimension of space)
!                         6:  ISEQ, indicates the sequence in which the
!                             coordinates along the surface must be ordered
!                             in case idim=3
!                             Possible values:
!                             1:  increasing x, for fixed x increasing y,
!                                 for fixed y increasing z
!                             2:  increasing y, for fixed y increasing z,
!                                 for fixed z increasing x
!                             3:  increasing z, for fixed z increasing x,
!                                 for fixed x increasing y
!                             4:  sequence of nodal point numbers
!                         7:  IREGION, indicates if the complete region must be
!                             printed (0) or that the printing is restricted to
!                             a sub region, defined by the parameters
!                             xmin, xmax, ymin, ymax, zmin,
!                             zmax in the common block CPLOT.
!                             Possible values:
!                             0:  The complete region is printed
!                             1:  The region is restricted to xmin<=x<=xmax,
!                                 ymin<=y<=ymax, zmin<=z<=zmax
!                            -1:  The region is restricted to x=xmin
!                            -2:  The region is restricted to y=ymin
!                            -3:  The region is restricted to z=zmin
!                            -11: The region is restricted to x>=xmin
!                            -12: The region is restricted to y>=ymin
!                            -13: The region is restricted to z>=zmin
!                            -21: The region is restricted to x<=xmin
!                            -22: The region is restricted to y<=ymin
!                            -23: The region is restricted to z<=zmin
!                         8:  IHEADER, indicates if the header must be printed
!                             (0) or not (1)
!                         9:  ICOOR, type of coordinate system
!                             1: Cartesian (x,y)
!                             3: Polar (r, phi)
!                         10: IFORMAT  Format used for printing
!                             1: e13.5
!                             2: e16.8
!                             3: e13.5, one node per line
!                             4: e16.8, one node per line
!                         11: MINVAL (if 0 no minimum value, if>0 value
!                             in rinpri )
!                         12: MAXVAL (if 0 no maximum value, if>0 value
!                             in rinpri )
!     ivectr         i    Standard SEPRAN array referring to the vector to
!                         be printed
!     rinpri         i    Contains the values of minval and maxval
!     text           i    Text to be printed as header for the output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer ichoice, idim, iquant, number, iseqlc, &
              nparts, nnodes, iregion, isuppress, suppress_zeros
      integer, allocatable :: ipoint(:)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     ichoice        Choice parameter, see inppri
!     idim           see inppri
!     ipoint         dummy
!     iquant         See inppri
!     iregion        See inppri
!     iseqlc         See inppri
!     isuppress      If 0 the node numbers are printed, if 1 they are suppressed
!     nnodes         Number of nodes in isolnw
!     nparts         number of parts the boundary consists of
!     number         The number-th unknown in each nodal point of the array
!                    is considered.
!     suppress_zeros if 1 points in which the solution is zero are skipped,
!                    if 0 not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0029         Print solution along boundary (real case)
!     ER0030         Print solution along boundary (complex case)
!     ER00751        Print global unknowns (real case)
!     ER00752        Print global unknowns (complex case)
!     ER0081         Store solution along curves or surface into help array
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERNODSMAXMIN   Restrict the number of nodes in array nodes to the set
!                    satisfying usol >= minvalue and/or usol <= maxvalue
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ichoice is incorrect
!    1517   IQUANTITY out of range
!    1518   The combination IDIM=3, IQUANTITY>0 not yet available
!    1520   iseq out of range
!    1576   Value of integer parameter incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize array inppri
!     Fill array inppri by copying from array inppri
!     Check parameters
!     Compute nodal point numbers in which the vector must be printed
!     Store quantities to be printed in a work array
!     Make print
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seper0075' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seper0075'
         call prinin ( inppri, 10, 'inppri' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Standard checks

      compl = ks(ivectr)%compl==1
      if ( ierror/=0 ) go to 1000

!     --- Check parameters

      ichoice = inppri(2)
      idim   = inppri(3)
      iquant = inppri(4)
      number = inppri(5)
      iseqlc   = inppri(6)
      iregion= inppri(7)
      isuppress = mod(ichoice,10)/10
      ichoice = mod(ichoice,10)
      suppress_zeros = ichoice/100
      if ( ichoice<0 .or. ichoice>3 .or. isuppress>1 .or. &
           suppress_zeros>1 ) then
         call errint ( ichoice+10*isuppress, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( iquant<0 .or. iquant>2 ) then
         call errint ( iquant, 1 )
         call errsub ( 1517, 1, 0, 0 )
      end if
      if ( iquant>0 .and. idim/=ndim ) then
         if ( .not. (idim==2 .and. ndim==3) ) then
            call errint ( iquant, 1 )
            call errint ( idim, 2 )
            call errint ( ndim, 3 )
            call errsub ( 1518, 3, 0, 0 )
         end if
      end if
      if ( iseqlc<1 .or. iseqlc>4 ) then
         call errint ( iseqlc, 1 )
         call errint ( 3, 2 )
         call errsub ( 1520, 2, 0, 0 )
      end if
      if ( iregion>1 .or. iregion<-23 .or. &
           iregion<-4 .and. iregion>-10 .or. &
           iregion<-14 .and. iregion>-20 ) then
         call errint ( iregion, 1 )
         call errint ( 7, 2 )
         call errchr ( 'iregion', 1 )
         call errchr ( 'inppri', 2 )
         call errsub ( 1576, 2, 0, 2 )
      end if
      if ( ierror/=0 ) then
         ierror = 0
         go to 1000
      end if
      if ( debug ) &
         write(irefwr,1) 'ichoice, isuppress', ichoice, isuppress

      allocate ( nodes(npoint), ipoint(ncurvs+1), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Compute nodal point numbers in which the vector must be printed
!         Store quantities to be printed in a work array

      call er0081 ( ivectr, icurvs, idim, iquant, number, ipoint, nparts, &
                    iregion, iseqlc, 1, nnodes )
      if ( nnodes==0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'nnodes, npoint, size(unew), inppri(11:12)', &
                          nnodes, npoint, size(unew), inppri(11:12)
      if ( size(unew)==npoint .and. &
           (inppri(11)>0 .or. inppri(12)>0) ) then

!     --- inppri(11)>0 or inppri(12)>0 Restrict to minimum or maximum values

         call ernodsmaxmin ( unew, nnodes, nodes, inppri(11), &
                             rinpri )
         if ( nnodes==0 ) go to 1000

      end if  ! ( inppri(11)>0 .or. inppri(12)>0 )
      if ( debug ) write(irefwr,1) 'nnodes', nnodes

!     --- Make print

      if ( debug ) then
         write(irefwr,1) 'idim, nnodes, nparts', idim, nnodes, nparts
         call prinin ( nodes, nnodes, 'nodes' )
      end if  ! ( debug )

      if ( nnodes==0 ) go to 1000
      nunkpo = -indprfo
      if ( .not. compl ) then

!     --- real case

         if ( debug ) write(irefwr,*) 'before er0029'
         kprobfo => istore
         call er0029 ( inppri, nodes, nnodes, icurvs, text )

      else

!     --- complex case

         call er0030 ( inppri, nodes, nnodes, icurvs, text )

      end if
      if ( iregion==0 .and. ks(ivectr)%typevec==110 ) then

!     --- The complete region must be printed, check if there are special
!         unknowns

         iprob  = ks(ivectr)%iprob
         iseqkp = kprob
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         call sepfillkprob

         if ( ierror/=0 ) then
            ierror = 0
            go to 1000
         end if
         if ( indprx>0 .or. indpry>0 ) then

!        --- indprx>0

            if ( .not. compl ) then

!           --- real case

               call er00751 ( usol )

            else

!           --- complex case

               call er00752 ( usol )

            end if

         end if

      end if

!     --- Delete superfluous arrays

      if ( iquant>0 ) deallocate(tang)
      if ( associated(normalvec) ) deallocate(normalvec)
      if ( associated(unew) ) deallocate ( unew )
      if ( associated(zunew) ) deallocate ( zunew )
      if ( associated(istore) ) deallocate ( istore )

      deallocate ( nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seper0075'

      end if  ! ( debug )
      call erclos ( 'seper0075' )

      end subroutine seper0075
