      subroutine sepinitder
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize parameters for subroutine sepcompreac
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulebuild
      use sepmodulemain
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, iclear, ivec, ih, itypvc

!     i              Counting variable
!     iclear         parameter clearvector see iinder(9)
!     ih             Help variable to store a constant temporarily
!     ivec           parameter ivec see iinder(7)
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPACTIVATE    Reactivate existing array in IBUFFR
!     SEPCREATEBUF   Create permanent space in array IBUFFR/BUFFER
!     TO0022         test if vectors are of equal type
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!    1127   The parameter ICOMPL has wrong value
!    1491   Sequence number of first vector out of range
!    1492   Sequence number of second vector out of range
!    1728   The parameter NUMVEC has wrong value
!    1729   The parameter CLEARVECTOR has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepinitder' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepinitder'
         call prinin ( iinder, 10, 'iinder' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      ndimlc = ndim
      inpelm = npelm
      icompl = iinder(3)
      if ( icompl<0 .or. icompl>1 ) then
         call errint ( icompl, 1 )
         call errsub ( 1127, 1, 0, 0 )
      end if
      icheld = iinder(4)
      ix     = iinder(5)
      jdegfd = iinder(6)
      ivec   = iinder(7)
      numol0 = iinder(8)
      numold = max ( 1,numol0 )
      if ( numold<0 .or. numold>1000 ) then
         call errint ( numold, 1 )
         call errsub ( 1728, 1, 0, 0 )
      end if
      iclear = iinder(9)
      if ( iclear<0 .or. iclear>1 ) then
         call errint ( iclear, 1 )
         call errsub ( 1729, 1, 0, 0 )
      end if
      if ( iseqi1<=0 .or. iseqi1>numold ) then

!     --- iseqi1 out of range

         call errint ( iseqi1, 1 )
         call errint ( numold, 2 )
         call errsub ( 1491, 2, 0, 0 )
         iseqi1 = 1

      end if
      iseqi2 = iinder(11)
      if ( debug ) write(irefwr,1) 'iseqi1, iseqi2', iseqi1, iseqi2
      if ( numol0==0 .and. iseqi2==2 ) numold = 2
      if ( iseqi2==0 ) iseqi2 = iseqi1
      if ( iseqi2<=0 .or. iseqi2>numold ) then

!     --- iseqi2 out of range

         call errint ( iseqi2, 1 )
         call errint ( numold, 2 )
         call errsub ( 1492, 2, 0, 0 )

      end if
      if ( icompl==1 ) then
         ivec = -ivec
         if ( ivec==0 ) ivec = -1
      end if

      itypvc = ks(isolut2d(iseqi1))%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. &
           itypvc/=116 .and. itypvc/=126 ) then

!     --- Input vector is of the wrong type

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )

      end if
      if ( iseqi2/=iseqi1 ) then

!     --- The two main input vectors are different, check if they have the same
!         structure

         call to0022 ( isolut2d(iseqi1), isolut2d(iseqi2), iseqi1, &
                       iseqi2 )

      end if
      jcart = ks(isolut2d(iseqi1))%jcart+1

      iseqprevsol = iseqi1
      iseqin = iseqi1

      ih = nelem*2**ndim
      if ( associated(flipflop) ) then

!     --- flip flop array has been created before, check length

         if ( size(flipflop)<ih ) then
            deallocate ( flipflop )
            allocate ( flipflop(ih) )
         end if  ! ( size(flipflop)<ih )

      else

!     --- flip flop array has not been created before

         allocate ( flipflop(ih) )

      end if  ! ( associated(flipflop)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icompl', icompl
         write(irefwr,*) 'End sepinitder'

      end if  ! ( debug )
      call erclos ( 'sepinitder' )

      end subroutine sepinitder
