      subroutine sepintdalloc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     De-allocate temporary arrays in for subroutine to0112
!
! **********************************************************************
!
!                       KEYWORDS
!
!     allocate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintdalloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintdalloc'
         write(irefwr,1) 'numold', numold
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      error = 0
      if ( debug ) &
         write(irefwr,*) 'associated(coeffarr)', associated(coeffarr)
      if ( associated(coeffarr) ) deallocate ( coeffarr, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'coeffarr' )

      if ( debug ) &
         write(irefwr,*) 'associated(index2)', associated(index2)
      if ( associated(index2) ) deallocate ( index2, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index2' )

      if ( debug ) &
         write(irefwr,*) 'associated(index3)', associated(index3)
      if ( associated(index3) ) deallocate ( index3, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index3' )

      if ( debug ) write(irefwr,*) 'before index4'
      if ( associated(index4) ) deallocate ( index4, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index4' )

      do k = 1, numtypesols
         if ( debug ) write(irefwr,1) 'k', k
         if ( associated(indoldsols(k)%index3) ) &
            nullify (indoldsols(k)%index3)
         if ( associated(indoldsols(k)%index4) ) &
            nullify (indoldsols(k)%index4)
      end do  ! i = 2, numtypesols

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintdalloc'

      end if  ! ( debug )
      call erclos ( 'sepintdalloc' )

      end subroutine sepintdalloc
