      subroutine sepintindex ( ikelmc, ipkproba )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays index2, index3 and index4 for subroutine integr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikelmc, ipkproba

!     ikelmc         i    Last position in KMESH part c that has been used
!     ipkproba       i    start address in kprob part a for element group
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLINDEX34 Fill array index3 with addresses of unknowns for one
!                    element.
!                    Fill array index4 starting position for node in index3
!     SEPINTINDEX2   fill index2 with positions of degrees of freedom in element
!                    referring to input vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintindex' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintindex'
         write(irefwr,1) 'ikelmc, ipkproba', ikelmc, ipkproba
         write(irefwr,1) 'nindex, iseqin', nindex, iseqin
         write(irefwr,1) 'numol0, numold', numol0, numold
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array index2

      call sepderindex2 ( ikelmc, ipkproba )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays index3 and index4 if necessary

      if ( numol0>0 .or. numold>1 ) then

!     --- numol0>0, there are several vectors

         call sepfillindex34 ( E%islold, ikelmc, nindex+inpelm )

      else

!     --- numol0 = 0, there is only one vector

         index3(1,1:icount) = index2(1:icount)
         indoldsols(1)%lenindex3 = icount
         indoldsols(1)%index4 => index4(1,:)
         index4(1,inpelm+1) = 110

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintindex'

      end if  ! ( debug )
      call erclos ( 'sepintindex' )
      end subroutine sepintindex
