      subroutine sepintmatq
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform symmetric compact storage scheme to row-compact storage
!     Special application q-matrix for simple
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: rows(:), cols(:)
      integer :: sizesav
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     cols           array wirh column numbers
!     rows           Refers to array irowsii
!     sizesav        size of array colsQ
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOMMATDIAGQ Store position of diagonal elements of array cols into
!                    idiag in case of row-compact storage
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPINTMAT1     Transform symmetric compact storage scheme to row-compact
!                    storage
!     SEPINTMAT4     Fill data structure in case of symmetric compact matrix
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmatq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmatq'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)
      sizesav = size(IM%colsQ)

      allocate ( cols(sizesav+nu), rows(nu+1) )

!     --- Copy colsQ in cols and rowsQ in rows

      cols(1:sizesav) = IM%colsQ
      rows = IM%rowsQ

      deallocate ( IM%colsQ )

!     --- Fill arrays rowsQ and colsQ

      if ( IM%Qsymmetric ) then

!     --- Q is symmetric

         allocate ( IM%colsQ(sizesav+nu), IM%diagQ(nu) )

!        --- Fill arrays rowsQ and colsQ

         call sepcommatrev ( IM%rowsQ, IM%colsQ, rows, cols, nu )
         IM%diagQ = IM%rowsQ(1:nu)+1
         cols = IM%colsQ
         rows = IM%rowsQ

         call sepintmat4 ( nu, IM%rowsQ, IM%colsQ, size(IM%colsQ), &
                           rows, cols, sizesav, IM%diagQ )

      else

!     --- Q is unsymmetric

         allocate ( IM%colsQ(2*size(cols)+nu) )

!        --- Fill arrays rowsQ and colsQ

         call sepintmat1 ( nu, IM%rowsQ, IM%colsQ, size(IM%colsQ), &
                           rows, cols, size(cols) )
         call sepcommatdiagq

      end if  ! ( IM%Qsymmetric )

      deallocate ( cols, rows )

1000  if ( debug ) then

!     --- Debug information

         call  sepprincompstr ( nu, IM%rowsQ, IM%colsQ )
         write(irefwr,*) 'End sepintmatq'

      end if  ! ( debug )
      call erclos ( 'sepintmatq' )

      end subroutine sepintmatq
