      function seplenglobvec ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with length of global vector corresponding to isol
!     Only for parallel computations
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     solution_vector
!     vector
!     global
!     length
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulesol
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol
      integer :: seplenglobvec

!     isol           i    Sequence number of vector in array ks
!     seplenglobvec  o    size of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: typevec, ivec

!     ivec           Sequence number of vector of special structure
!     typevec        Type of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2708   vectors of this type not implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplenglobvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplenglobvec'
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      seplenglobvec = 0

      typevec = ks(isol)%typevec

      if ( debug ) &
         write(irefwr,1) 'typevec, iprob, iseqkp', typevec, iprob, iseqkp

      if ( typevec==110 ) then

!     --- solution vector

         seplenglobvec = kprobpar(1)

      else if ( typevec==115 ) then

!     --- vector of special structure

         ivec = ks(isol)%ivec
         kprobe2 => kprobpart%kprobe2
         if ( nunkp<0 ) then

!        --- number of degrees of freedom is constant (-nunkp)

            nunkp = kprobe2(ivec)
            seplenglobvec = -nunkp*npointglob

         else

!        --- number of degrees of freedom is variable
!            Size in kprobfglob

            kprobfglobal => kprobpart%kprobfglobal
            seplenglobvec = kprobfglobal(ivec)%arr(npointglob+1)

         end if  ! ( nunkp<0 )

      else

!     --- other type of vector not yet implemented

         call errint ( typevec, 1 )
         call errsub ( 2708, 1, 0, 0 )

      end if  ! ( typevec==110 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplenglobvec'

      end if  ! ( debug )
      call erclos ( 'seplenglobvec' )

      end function seplenglobvec
