      subroutine sepmaplocvec1 ( ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map global vector referred to by isol into local vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     solution_vector
!     vector
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivec

!     ivec           i    Sequence number of vector of special structure
!                         0 means solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart1, istart2, inode, inunkp
      integer, pointer :: kprobfglob(:)

!     i              Counting variable
!     inode          global node sequence number
!     inunkp         Number of degrees of freedom in point
!     istart1        First position used at a part
!     istart2        First position used at a part
!     kprobfglob     Global array kprobf
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmaplocvec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmaplocvec1'
         write(irefwr,1) 'npoint, size(nodes)', npoint, size(nodes)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprf>0 ) then

!     --- indprf > 0, variable number of unknowns per point

         kprobfglob => kprobfglobal(ivec)%arr

         istart1 = 0
         do i = 1, npoint
            inode = nodes(i)
            istart2 = kprobfglob(inode)
            inunkp = kprobfglob(inode+1)-istart2
            usol(istart1+1:istart1+inunkp) = &
               usolglob(istart2+1:istart2+inunkp)
            istart1 = istart1+inunkp
         end do  ! i = 1, npoint

      else

!     --- indprf = 0, constant number of unknowns per point

         istart1 = 0
         do i = 1, npoint
            inode = nodes(i)
            istart2 = (inode-1)*nunkp
            usol(istart1+1:istart1+nunkp) = &
               usolglob(istart2+1:istart2+nunkp)
            istart1 = istart1+nunkp
         end do  ! i = 1, npoint

      end if  ! ( indprf>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmaplocvec1'

      end if  ! ( debug )
      call erclos ( 'sepmaplocvec1' )

      end subroutine sepmaplocvec1
