      subroutine sepprinkmesh ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of array kmesh or parts of it
!     Old subroutine prinkmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice

!     ichoice  i   choice parameter, values:
!                  -1: Print a selection of the complete integer array.
!                      This possibility is only allowed for IVECTR equal to
!                      KMESH, KPROB and INTMAT
!                      If the complete array KMESH must be printed, then
!                      both arrays KMPRB and IVECTR are assumed to be KMESH
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be KMESH and IVECTR is
!                      assumed to be KPROB
!                   0: Print the complete integer array.
!                      This possibility is only allowed for IVECTR equal to
!                      KMESH, KPROB and INTMAT
!                      If the complete array KMESH must be printed, then
!                      both arrays KMPRB and IVECTR are assumed to be KMESH
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be KMESH and IVECTR is
!                      assumed to be KPROB
!                   1: print array kmesh parts c and g
!                   2: print array kmesh part d
!                   3: print array kmesh part e
!                   4: print array kmesh part j
!                   5: print array kmesh part h
!                   6: print array kmesh part l,m and n
!                   7: print array kmesh part o
!                   8: print array kmesh part k
!                   9: print array kmesh parts q, r and s
!                  30: print kmesh iinput and rinput
!                  31: print kmesh part t
!                  32: print kmesh part u
!                  33: print kmesh part v
!                  34: print kmesh part w
!                  35: print kmesh part x
!                  36: print kmesh part z
!                  37: print kmesh part y
!                  38: print kmesh part i
!                  39: print kmesh part aa/nmeshc
!                  40: print kmesh part aa/nmeshd
!                  41: print kmesh part aa/nmeshl
!                  42: print kmesh part ag (volume_surfs, i.e. surfaces
!                                           corresponding to volumes)
!                  43: print kmesh part aa/nmeshh
!                  44: print kmesh part aa/nmesha
!                  45: print kmesh part aa/nmeshb
!                  46: print kmesh part aa/nmeshf
!                  47: print kmesh part aa/nmeshg
!                  48: print kmesh part aa/nmeshk
!                  49: print kmesh parts blockinfo, transportinfo
!                  50: print kmesh part ioutercurvs/surfs
!                  59: print kmesh part ai (surfnodes)
!                  60: print kmesh part aj (normals)
!                  61: print kmesh part al
!                  62: print kmesh part ae (nodes corresponding to groups)
!                  63: print kmesh part a, b and f
!                  64: print kmesh part aa/nmeshi
!                  66: print kmesh_quant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0002         print array   nunkp positions in each row; npoint rows
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
!     PR0009         Print information of surfaces
!     PR0012         Print accumulated array
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINIOUTERCS   Print array ioutercurvs or ioutersurfs,
!                    depending on the dimension
!     PRINKMESHPAR   Print the contents of the parallel arrays blockinfo and
!                    transportinfo
!     PRINKMESHQUANT Print the contents of array kmeshquant
!     PRINNMESHL     print nmesh part l
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     PRINNORMAL3D1  Print the array normals for all outer surfaces
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPPR0011      Print KMESH part o
!     SEPPR0013      Print KMESH part h
!     SEPPRINKMESHA  Print kmesh parts a, b and f if they exist
!     SEPPRINKMESHAL Print the contents of array kmeshal
!     SEPPRINKMESHC  print kmesh part c
!     SEPPRINKMESHD  print kmesh part d
!     SEPPRINKMESHE  print kmesh part e
!     SEPPRINKMESHS  Print the contents of array kmeshs
!     SEPPRINKMESHT  Print the contents of KMESH part t
!     SEPPRINNMESHA  print nmesh part a
!     SEPPRINNMESHC  print nmesh part c
!     SEPPRINNMESHD  print nmesh part d
!     SEPPRINNMESHI  print nmesh part i
!     SEPPRMSHOB1    Print the contents of ICURVSOBST
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinkmesh'
         write(irefwr,1) 'ichoice', ichoice
         write(irefwr,1) 'kelmd, kelme, kelmj', kelmd, kelme, kelmj
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichoice<=0 .or. ichoice==1 ) call sepprinkmeshc
      if ( (ichoice==0 .or. ichoice==2) .and. kelmd/=0 ) call sepprinkmeshd
      if ( (ichoice==0 .or. ichoice==3) .and. kelme/=0 ) call sepprinkmeshe
      if ( (ichoice<=0 .or. ichoice==4) .and. kelmj/=0 ) then

!     --- ichoice = 0 or 4, print kmesh part j

         write ( irefwr, 110 )
110      format( ' new numbering of nodal points' )
         write ( irefwr,900 ) ( k, kmeshj(k), k = 1, npoint )
      end if  ! ( (ichoice<=0 .or. ichoice==4) .and. kelmj/=0 )

      if ( (ichoice==0 .or. ichoice==5) .and. kelmh/=0 ) then

!     --- ichoice = 0 or 5, print kmesh part h

         call seppr0013

      end if

      if ( (ichoice<=0 .or. ichoice==6) ) then

!     --- ichoice = 0 or 6, print kmesh part l m and n

         if ( nuspnt>0 ) &
            call prinin ( kmeshl, nuspnt, &
                       'nodal point numbers of user points' )
         if ( kelmm/=0 ) call pr0008 ( kmeshm, ncurvs )
         if ( kelmn/=0 ) &
            call pr0009 ( 0, kmeshn, coor, kmeshn, &
                          kmeshn(1), kmeshn(1), nsurfs )

      end if
      if ( (ichoice==0 .or. ichoice==7) .and. kelmo/=0 ) then

!     --- ichoice = 0 or 7, print kmesh part o

         call seppr0011

      end if
      if ( (ichoice==0 .or. ichoice==8) .and. kelmk/=0 ) then

!     --- ichoice = 0 or 8, print kmesh part k

         write ( irefwr,'(/'' levels''/(1x,20i6))') kmeshk

      end if
      if ( ichoice==0 .or. ichoice==9 ) then

!     --- ichoice = 0 or 9, print kmesh parts q, r and s

         if ( kelmq/=0 ) then

!        --- kelmq# 0, array KMESH part q has been filled

            write ( irefwr,'(//'' Subcurve numbers of curves''/)')
            call pr0012 ( ncurvs, kmeshq, kmeshq )

         end if  ! ( kelmq/=0 )

         if ( kelmr/=0 ) then

!        --- kelmr# 0, array KMESH part r has been filled

            write(irefwr,200)
200         format ( //' Subsurface numbers of surfaces'/ )
            call pr0012 ( nsurfs, kmeshr, kmeshr )

         end if

         if ( kelms/=0 ) then

!        --- kelms# 0, array KMESH part s has been filled

            call sepprinkmeshs

         end if

      end if

      if ( (ichoice==0 .or. ichoice==30) .and. kmiinput/=0 ) then

!     --- ichoice = 0 or 30, print iinput and rinput

         length = size ( iinputkm )
         call prinin ( iinputkm, length, 'iinput' )
         length = size ( rinputkm )
         call prinrl ( rinputkm, length, 'rinput' )

      end if

      if ( ichoice==0 .or. ichoice==31 ) then

!     --- ichoice = 0 or 31, print kmesh part t

         if ( kelmt>0 ) call sepprinkmesht

      end if

      if ( (ichoice==0 .or. ichoice==32) .and. kelmu/=0 ) then

!     --- ichoice = 0 or 32, print kmesh part u

         call prinin ( kmeshu, 5, 'kmesh part u' )

      end if

      if ( (ichoice==0 .or. ichoice==33) .and. kelmv/=0 ) then

!     --- ichoice = 0 or 33, print kmesh part v

         write ( irefwr, '(/a/)') &
               ' element groups connected to element groups'
         call pr0012 ( nelgrp, kmeshv, kmeshv(nelgrp+2) )

      end if

      if ( (ichoice==0 .or. ichoice==34) .and. kelmw/=0 ) then

!     --- ichoice = 0 or 34, print kmesh part w

         if ( nintprop>0 ) then

!        --- There are integer properties

            call prinin1 ( intprop, nelgrp, nintprop, &
                           'integer properties per element group' )

         end if
         if ( nrealprop>0 ) then

!        --- There are real properties

            call prinrl1 ( realprop, nelgrp, nrealprop, &
                           'real properties per element group' )

         end if

      end if

      if ( (ichoice==0 .or. ichoice==35) .and. kelmx/=0 ) then

!     --- ichoice = 0 or 35, print kmesh part x and a part of kmeshh

         if ( numobst>0 ) then

!        --- numobst>0, there are obstacles

            write(irefwr,*) &
              ' obstacle type curve   nx     ny   mmgrid '
            call prinin1 ( kmeshx, numobst, 5, ' ' )
            write(irefwr,*) &
              ' obstacle  xmin      xmax        ymin         ymax', &
              '        dx            dy '
            call prinrl1 ( rmeshx, numobst, 6, ' ' )
            if ( kelmh>0 ) then

!           --- kmesh part h exists

               if ( kmeshh(1,1)>0 ) then

!              --- array icurvsobst exists

                  call sepprmshob1
                  call prinin ( iobscurv, ncurvs, 'iobscurv' )
               end if

            end if

         end if

      end if

      if ( (ichoice==0 .or. ichoice==36) .and. kelmz/=0 ) then

!     --- ichoice = 0 or 36, print kmesh part z

         call prinin ( kmeshz, ncurvs, 'kmesh part z' )

      end if

      if ( ichoice<=0 .or. ichoice==38) then

!     --- ichoice = 38, print kmesh part i (co-ordinates)

         call er0002 ( coor, ndim, npoint, 'co-ordinates' )

      end if

      if ( kelmaa>0 .and. ichoice<=0 .or. ichoice==39 ) then
         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(33)==0 ) call sepfillnmeshc
         call sepprinnmeshc
      end if  ! ( kelmaa>0 .and. ichoice<=0 .or. ichoice==39 )

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==40) ) then
         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(33)==0 ) call sepfillnmeshc
         call sepprinnmeshd
      end if  ! ( kelmaa>0 .and. ichoice<=0 .or. ichoice==40 )

      if ( kelmaa>0 .and. ichoice<=0 .or. ichoice==41 ) then

!     --- ichoice = 41, print kmesh part aa/nmeshl
!         Edge numbers corresponding to nodes

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(42)==0 ) call sepfillnmeshl
         call prinnmeshl ( npoint, nmeshl  )

      end if

      if ( kelmag>0 .and. (ichoice<=0 .or. ichoice==42) ) then

!     --- ichoice = 42, print kmesh part ag (VOLUME_SURFS)
!         surface numbers corresponding to the volumes

         write(irefwr,*) 'Surfaces corresponding to volumes'
         call prinnmeshx ( nvolms, volume_surfs )

      end if

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==43) ) then

!     --- ichoice = 43, print kmesh part aa/nmeshh
!         Element numbers corresponding to edges

         if ( nmesh(38)>0 ) then
            write(irefwr,*) 'Elements connected to edges'
            call prinnmeshx ( nedges, nmeshh )
         end if

      end if

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==44) ) then

!     --- ichoice = 44, print kmesh part aa/nmesha
!         Face numbers corresponding to elements

         call sepprinnmesha

      end if

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==45) ) then

!     --- ichoice = 45, print kmesh part aa/nmeshb
!         Edge numbers corresponding to faces

         if ( nmesh(32)>0 ) then
            write(irefwr,*) 'Edge numbers corresponding to faces'
            call prinnmeshx ( nfaces, nmeshb )
         end if

      end if

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==46) ) then

!     --- ichoice = 46, print kmesh part aa/nmeshf
!         Element numbers corresponding to faces

         if ( nmesh(36)>0 ) then
            write(irefwr,*) 'Element numbers corresponding to faces'
            call prinnmeshx ( nfaces, nmeshf )
         end if

      end if

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==47) ) then

!     --- ichoice = 47, print kmesh part aa/nmeshg
!         Node numbers corresponding to faces

         if ( nmesh(37)>0 ) then
            write(irefwr,*) 'Node numbers corresponding to faces'
            call prinnmeshx ( nfaces, nmeshg )
         end if

      end if

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==48) ) then

!     --- ichoice = 48, print kmesh part aa/nmeshk
!         Face numbers corresponding to nodes

         if ( nmesh(41)>0 ) then
            write(irefwr,*) 'Face numbers corresponding to nodes'
            call prinnmeshx ( npoint, nmeshk )
         end if

      end if

      if ( km(iseqkm)%kmesh(53)>0 .and. (ichoice<=0 .or. ichoice==49) ) then

!     --- ichoice = 49, print kmesh parts blockinfo and transportinfo

         call  prinkmeshpar ( blockinfo, transinfo )

      end if

      if ( mmouter>0 .and. (ichoice<=0 .or. ichoice==50) ) then

!     --- ichoice = 50, print kmesh part iouterboun

         call prinioutercs

      end if

      if ( mmsurfnodes>0 .and. (ichoice<=0 .or. ichoice==59) ) then

!     --- ichoice = 59, print kmesh part ai surfnodes

         call prinnmeshx ( nsurfs, surfnodes )

      end if

      if ( mmnormal>0 .and. (ichoice<=0 .or. ichoice==60) ) then

!     --- ichoice = 60, print kmesh part aj normal

         if ( ndim==3 ) call prinnormal3d1 ( nsurfs, surfnodes, normal )

      end if

      if ( kelmal>0 .and. (ichoice<=0 .or. ichoice==61) ) then

!     --- ichoice = 61, print kmesh part al

         call sepprinkmeshal

      end if

      if ( kelmae>0 .and. (ichoice<=0 .or. ichoice==62) ) then

!     --- ichoice = 62, print kmesh part aa/nmeshg
!         Node numbers corresponding to faces

         write(irefwr,*) &
            'Node numbers corresponding to element groups'
         call prinnmeshx ( nelgrp, ielgrp_nodes )

      end if

      if ( ichoice<=0 .or. ichoice==63 ) then

!     --- ichoice = 63, print kmesh parts a, b and f

         call sepprinkmesha

      end if  ! ( ichoice<=0 .or. ichoice==63 )

      if ( kelmaa>0 .and. (ichoice<=0 .or. ichoice==64) ) then

!     --- ichoice = 39, print kmesh part aa/nmeshc
!         Node numbers corresponding to edges

         if ( nmesh(39)>0 ) call sepprinnmeshi

      end if

      if ( kelman>0 .and. (ichoice<=0 .or. ichoice==66) ) then

!     --- ichoice = 66, print kmesh_quant

         call prinkmeshquant ( ndim, kmeshquant )

      end if  ! ( ichoice<=0 .or. ichoice==63 )

900   format( ( 1x, 6(i6,':',i5) ) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinkmesh'

      end if  ! ( debug )
      call erclos ( 'sepprinkmesh' )

      end subroutine sepprinkmesh
