      subroutine sepprinmatdbg ( A, threshold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print compact matrix without zero elements
!     Old subroutine pr0015com
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemat   ! definition of matrices
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: threshold
      type (rmatrix), intent(in) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     threshold      i    threshold value, only elements larger than this value
!                         are printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, ncols, jstart, m, jend, j, ih, icount
      integer :: posU(A%neq), istore(A%neq)
      double precision :: store(A%neq), h
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     h              element of matrix
!     i              Counting variable
!     icount         Counter
!     ih             Help variable to store a constant temporarily
!     istart         start address
!     istore         Work array of length A%neq to store at most one row
!                    of the oclumn indices of the matrix
!     j              Counting variable
!     jend           end address
!     jstart         starting position
!     m              number of items to be printed
!     ncols          number of non-zero A%cols in row
!     posU           Contains next non-used column number per row
!     store          Work array of length A%neq to store at most one row
!                    of the matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinmatdbg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinmatdbg'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%iseqim)

      if ( A%symmetric .and. .not. IM%mumps ) then

!     --- Symmetric matrix

         write(irefwr,100)
100      format(' matrix first part, compact storage'//, &
                ' diagonal of matrix'/)
         do i = 1, A%neq
            if ( abs(A%S(i)).gt.threshold ) then

!           --- A%S(i) > threshold

               icount = icount+1
               istore(icount) = i
               store(icount) = A%S(i)

            end if  ! ( abs(A%S(i)).gt.threshold )

         end do  ! i = 1, A%neq
         write(irefwr,110) (    istore(i),store(i),i=1,icount)
110      format((4(1x,i6,':',ss,1pe12.5e2)))

         write(irefwr,125)
125      format(//'matrix with row and column numbers'/)
         write(irefwr,130)
130      format(//' lower triangular part of the matrix with row and ', &
            'column numbers'/)

          do i = 1, A%neq
             jstart=A%rows(i)+1
             jend=A%rows(i+1)
             icount = 0
             do j = jstart, jend
                h = A%S(j)
                if ( abs(h).gt.threshold ) then

!               --- non-zero matrix element found

                   icount = icount+1
                   store(icount) = h
                   istore(icount) = icolsii(j)

                end if
             end do
             m=min( 4,icount)
             if ( icount.gt.0 ) &
                write(irefwr,200) i,(istore(j),store(j), j = 1, m)
             if ( m.lt.icount ) write(irefwr,210) &
                 (istore(j),store(j),j=m+1,icount)
          end do  ! i = 1, A%neq

      else

!     --- Row compact storage

         write(irefwr,110) (i,A%S(A%diag(i)),i=1,A%neq)
         istart = 0
         do i = 1, A%neq
            ncols = A%diag(i)-istart-1
            if ( ncols>0 ) then
               icount = 0
               jstart = istart+1
               jend = istart+ncols
               do j = jstart, jend
                  h = A%S(j)
                  if ( abs(h).gt.threshold ) then

!                 --- non-zero matrix element found

                     icount = icount+1
                     store(icount) = h
                     istore(icount) = A%cols(j)

                  end if
               end do
               m=min( 4,icount)
               if ( icount.gt.0 ) &
                  write(irefwr,200) i,(istore(j),store(j), j = 1, m)
               if ( m.lt.icount ) write(irefwr,210) &
                   (istore(j),store(j),j=m+1,icount)
200            format(1x,i6,':',4(i5,':',ss,1pe12.5e2))
210            format((8x,4(i5,':',ss,1pe12.5e2)))
            end if  ! ( ncols>0 )

            istart = A%rows(i+1)

         end do  ! i = 1, A%neq

         write(irefwr,250)
250      format(/' upper triangular matrix with column and row numbers'/)

         posU = A%diag+1

         istart = 0
         do i = 1, A%neq
            ncols = A%diag(i)-istart-1
            if ( ncols>0 ) then
               icount = 0
               jstart = istart+1
               jend = istart+ncols
               do j = 1, ncols
                  ih = A%cols(j+istart)
                  h = A%S(posU(ih))
                  if ( abs(h).gt.threshold ) then

!                 --- non-zero matrix element found

                     icount = icount+1
                     store(icount) = h
                     istore(icount) = posU(ih)

                  end if
                  posU(ih) = posU(ih)+1
               end do  ! j = 1, ncols
               m=min( 4,icount)
               if ( icount.gt.0 ) &
                  write(irefwr,200) i,(istore(j),store(j), j = 1, m)
               if ( m.lt.icount ) write(irefwr,210) &
                   (istore(j),store(j),j=m+1,icount)
            end if  ! ( ncols>0 )

            istart = A%rows(i+1)

         end do  ! i = 1, A%neq

      end if  ! ( A%symmetric .and. .not. IM%mumps )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinmatdbg'

      end if  ! ( debug )
      call erclos ( 'sepprinmatdbg' )

      end subroutine sepprinmatdbg
