      subroutine sepreacbody ( listelms )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the reaction force along a number of given curves
!    Old subroutine to0112
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulevecs
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: listelms(nelem)

!     listelms       i    Array of length nelem, containing information about the
!                         elements
!                         if listelms(i) = 0, the element must be skipped,
!                         if listelms(i) = 1, the element must be taken into account
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ikelmc, ipkproba, lenindex3

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     ipkproba       start adress in kprob part a
!     lenindex3      length of index3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPASSEMELGRP  Initialize some parameters for one element group
!     SEPDERASSEM    add element vector to large vector of derived quantities
!     SEPDERVDALLOC  De-allocate temporary arrays in for subroutine to0112
!     SEPFILLSTELM   Initialize some parameters for one element
!                    Fill index arrays
!     SEPMAKEELM     Create element matrix and element vector
!     TO0113         Compute local reaction force by R = Su - f (per element)
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Set pointers
!    For ielgrp member of list_of_element_groups
!       For ielem member of list_of_elements in ielgrp
!          Compute reaction force in ielem
!          add reaction force to output vectors
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreacbody' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreacbody'
         write(irefwr,1) 'iseqi1, numold', iseqi1, numold
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      kproba => kprobpart%kproba
      indpra = kprobloc(14)
      kprobc => kprobpart%kprobc

      iinbld(10) = numol0

!     --- loop over element groups

      ilocelem = 0
      ielem = 0
      ikelmc = 0
      ipkproba = 1

      do ielgrp = 1, nelgrp

!     --- set notvec = 0  (element vector of this group not zero)

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         call sepassemelgrp ( ikelmc )

         if ( debug ) then
            write(irefwr,1) 'ielgrp, inelem, ishape, itype', &
                             ielgrp, inelem, ishape, itype
            write(irefwr,1) 'skipelgrps(ielgrp)', skipelgrps(ielgrp)
         end if  ! ( debug )

         if ( itype<=0 .or. skipelgrps(ielgrp)==0 ) then

!        --- element group ielgrp must be skipped
!            Raise parameters

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem
            go to 500

         end if  ! ( itype<=0 .or. skipelgrps(ielgrp)==0 )

         ifirst = 0

!        --- allocate arrays for element subroutines

         lenindex3 = max(nindex+inpelm, inpelm*nunkp)
         allocate ( index3(max(1,numold),lenindex3), &
                    index1sub(inpelm), stat=error )
         if ( error/=0 ) call eralloc ( error, lenindex3+inpelm, 'index1sub' )
         if ( ierror/=0 ) go to 1000

         deallocate ( index5 )    ! index5 is alloacted in sepassemelgrp
         index5 => index2         ! index5 is made identical to index2
         icouno = icount          ! icouno and icount are the same
         elemvc1 => elemvc

         if ( nindex>0 ) index3 = 0  ! Reset index3

!        --- loop over all elements of this group

         do irelem = 1, inelem

!        --- fill arrays index1, index2, index3, index4, and index5

            ielem  = ielem + 1
            E%ielem = ielem
            E%irelem = irelem

            if ( debug ) &
               write(irefwr,1) 'irelem, ielem, list(ielem)', &
                                irelem, ielem, listelms(ielem)

            if ( listelms(ielem)/=0 ) then

!           --- Element is allowed
!               First copy nodes in array index1

               call sepfillstelm ( ikelmc )
               if ( ierror/=0 ) go to 1000

!              --- Create element matrix and element vector

               call sepmakeelm

!              --- Multiply by old solution

               if ( debug ) write(irefwr,*) 'before to0113'
               call to0113

!              --- add element element vector to large vector

               if ( debug ) write(irefwr,*) 'before sepderassem'
               elemvc2 => elemvc
               vectout2 => vectout
               call sepderassem ( compl )

            end if

            ikelmc = ikelmc+inpelm

         end do

!        --- Deallocate temporary arrays

         if ( debug ) write(irefwr,*) 'before sepdervdalloc'
         call sepdervdalloc
         if ( ierror/=0 ) go to 1000

500      ipkproba = ipkproba+inpelm

      end do  ! ielgrp = 1, nelgrp

      if ( debug ) write(irefwr,*) 'after loop'

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreacbody'

      end if  ! ( debug )
      call erclos ( 'sepreacbody' )

      end subroutine sepreacbody
