      subroutine topmaptetra2 ( kmeshcnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-11-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map 3d array kmeshc onto new array kmesh consisting of
!     linear tetrahedrons only
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*)

!     kmeshcnew      o    New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, istart, ielgrp, inelemnew, inpelmnew, istartnew

!     ielgrp         element group number
!     inelemnew      inelem for new mesh
!     inpelm         number of points per element
!     inpelmnew      number of points per element for new mesh
!     istart         First position used at a part
!     istartnew      starting address in new array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOPMAPTETRA11  Copy elements of shape 11 or 18 into new elements of
!                    shape 11
!     TOPMAPTETRA12  Copy elements of shape 12, 15 or 16 into new elements of
!                    shape 11
!     TOPMAPTETRA14  Copy elements of shape 14 into new elements of shape 11
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topmaptetra2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topmaptetra2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill new array kmeshc

      istart = 1             ! starting address in old kmeshc
      istartnew = 1          ! starting address in new kmeshc
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         if ( ishape<11 ) go to 100

         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem, inpelm, istart', &
                             ielgrp, ishape, inelem, inpelm, istart

         select case ( ishape )

            case ( 19, 20, 22: )

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            case(11, 18)

!           --- Linear tetrahedron

               inelemnew = inelem
               inpelmnew = 4
               call topmaptetra11 ( kmeshc(istart), &
                                    kmeshcnew(istartnew), inpelm, &
                                    inelem, inpelmnew )

            case(12, 15, 16)

!           --- Quadratic tetrahedron

               inelemnew = 8*inelem
               inpelmnew = 4
               call topmaptetra12 ( kmeshc(istart), &
                                    kmeshcnew(istartnew), inpelm, &
                                    inelem )

            case(13, 17)

!           --- Trilinear hexahedron

               inelemnew = inelem
               inpelmnew = 8
               call topmaptetra11 ( kmeshc(istart), &
                                    kmeshcnew(istartnew), inpelm, &
                                    inelem, inpelmnew )

            case(14)

!           --- Quadratic hexahedron

               inelemnew = 8*inelem
               inpelmnew = 8
               call topmaptetra14 ( kmeshc(istart), &
                                    kmeshcnew(istartnew), inpelm, &
                                    inelem )

            case(21)

!           --- Linear prism

               inelemnew = inelem
               inpelmnew = 6
               call topmaptetra11 ( kmeshc(istart), &
                                    kmeshcnew(istartnew), inpelm, &
                                    inelem, inpelmnew )

         end select  ! case ( ishape )
         istartnew = istartnew+inelemnew*inpelmnew
100      istart = istart+inelem*inpelm    ! raise starting address

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topmaptetra2'

      end if  ! ( debug )
      call erclos ( 'topmaptetra2' )

      end
