      subroutine totransmesh ( kmeshtmp, new_mesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-04-2014 Extension with 2d
!        version  1.1  date 03-08-2011 Make orientation of elements positive
!        version  1.0  date 13-04-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform quadratic mesh to linear mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     mesh
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtmp
      logical new_mesh

!     kmeshtmp       o    Contains information of the old mesh
!     new_mesh       o    If true a new mesh is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kmeshnew
      logical subdiv

!     kmeshnew       Help array to store the entries of new kmesh temporary
!     subdiv         If true the elements must be subdivided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPOSJAC      Change sequence of nodes in elements with negative Jacobian
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     TOPMAPTETRA    Map 3d mesh onto a mesh consisting of linear tetrahedrons
!     TOPMAPTRIAN    Map 2d mesh onto a mesh consisting of linear triangles only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set parameter new_mesh
!     Check type of elements
!     Check length mesh
!     Create new mesh
!     Copy kmesh to kmeshtmp
!     Copy kmeshnew into kmesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'totransmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from totransmesh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set parameter new_mesh

      new_mesh = .false.

!     --- Check type of elements

      select case ( ishape )
      case(3,11)

!     --- ishape = 3, 11, no change necessary

         go to 1000

      case(4:7, 10, 12)

!     --- ishape = 4-7, 10, 12, change is necessary

      case default

!     --- value of ishape not yet implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end select  ! case ( ishape )

!     --- The mesh must be changed

      new_mesh = .true.
      kmeshnew = 0
      call sepcrmeshinfo ( kmeshnew )

!     --- Create new mesh

      if ( ndim==2 ) then

!     --- ndim = 2, triangles

         call topmaptrian ( kmeshnew, subdiv )

      else

!     --- ndim = 3, tetrahedrons

         call mshposjac
         call topmaptetra ( kmeshnew, subdiv )

      end if  ! ( ndim==2 )

      if ( debug ) write(irefwr,*) 'subdiv ', subdiv

!     --- Copy kmesh to kmeshtmp
!         Copy kmeshnew into kmesh

      kmeshtmp = kmesh
      kmesh = kmeshnew
      call sepfillkmesh(kmesh)
      if ( ndim==3 ) call mshposjac

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End totransmesh'

      end if  ! ( debug )
      call erclos ( 'totransmesh' )

      end subroutine totransmesh
