      subroutine pl0003 ( ikader, kader, plotfm, yfact, ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.3  date 30-04-2010 Debug statements
!        version  7.2  date 21-05-2001 New call to pl0001
!        version  7.1  date 23-11-2000 Minor correction in comment
!
!   copyright (c) 1983-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        General help subroutine for all SEPRAN plot programs
!        Start plot, compute xmin, xmax, ymin, ymax, aleng and width
!        If necessary compute axmin, aymin, axmax and aymax
!        plot frame work depending on the value of kader
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikader, kader, ichois
      double precision plotfm, yfact

!     ichois     i    Choice parameter consisting of two parts jchois and
!                     icoor according to ichois = jchois + 10*icoor with:
!                     jchois    indication whether xmin, xmax etc. must be
!                               computed, or the picture must be started
!                               possibilities:
!                               0   compute and start the picture
!                               1   compute
!                               2   start the picture
!                     icoor     indicates if array kmeshm and coor are used
!                               to compute the size of the plot (0) or the
!                               arrays icurvs and curves (1)
!     ikader     o    value of the parameter jkader in the common block cplot
!                     at input
!     kader      i    indication of the value of jkader when a frame must
!                     be plotted
!     plotfm     i    Plotting factor. When PLOTFM<0, then the length of
!                     the plot is computed by - (xmax-xmin) * PLOTFM
!                     and the width by - (ymax-ymin) * YFACT * PLOTFM
!                     else the length of the plot is equal to PLOTFM cm
!                     When by some mistake the user has given | PLOTFM | a value
!                     smaller than 1 cm, then the length is chosen equal to
!                     the default length stored in common block consta
!     yfact     i     Multiplication for the y-co-ordinates. With this
!                     parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itest, iscale, jchois, icoor
      save iscale

!     icoor          ichois/10, see ichois
!     iscale         Indication whether scales must be plotted along the sides
!                    (1) or not (0). iscale is computed with the aid of jkader
!     itest          Help parameter to store the value of jmax temporarily
!                    Input for subroutine PLAFEP
!     jchois         mod(ichois,10), see ichois
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PL0000         Compute length and width of the plot
!     PL0001         Compute the parameters xmin, xmax, ymin and ymax
!     PL0002         Compute the parameters xmin, xmax, ymin and ymax
!     PLAF04         Routine to plot arrows with text along the axises
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLSCLS         Subroutine to plot scales and axis at a picture
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0003' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pl0003'
         call ertrace
         write(irefwr,1) 'jtimes, ichois, jkader', &
                          jtimes, ichois, jkader
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialise jkader, ikader and iscale

      if ( mod(jtimes,10)>1 ) go to 1000
      jchois = mod(ichois,10)
      icoor  = ichois/10
      if ( debug ) write(irefwr,1) 'jchois, icoor', jchois, icoor
      if ( jchois/=2 ) then

!     --- jchois = 0 or 1   compute xmin, xmax, ymin, ymax, aleng and width

         iscale = 0
         ikader = jkader
         if( ikader>=0 .or. ikader==-2 )  jkader = kader
         if( ikader==-2 .or. ikader==-3 ) iscale = 1

         if ( debug ) write(irefwr,1) 'jkader, iscale', jkader, iscale

         if ( icoor==0 ) then

!        --- Standard situation, extract information from coor and kmeshm

            call pl0002
            if ( debug ) &
                write(irefwr,2) 'xmin, xmax, ymin, ymax', &
                                 xmin, xmax, ymin, ymax

         else if ( jmax==0 ) then

!        --- Special situation, extract information from curves and icurvs

            call pl0001 ( curvesmsh, 2, icurvsmsh(ncurvs), xmin, xmax, ymin, &
                          ymax )

         end if
         if ( ierror/=0 ) go to 1000
         call pl0000( plotfm, yfact )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,2) 'aleng, width', aleng, width

      if( jchois/=1 ) then

!     --- jchois = 0 or 2  start picture

         itest=jmax
         if ( debug ) write(irefwr,*) 'before plafep/1'
         call plafep( 1, aleng, width, 1d0, plotfm, itest )
         if ( debug ) write(irefwr,*) 'after plafep/1'
         if ( ierror/=0 ) go to 1000

!        --- Compute axmin, axmax, aymin and aymax, depending on kiesx and kiesy

         if(kiesx==-1) then
            axmin=xmin-0.5d0*xfct
            axmax=xmax+0.5d0*xfct
         end if
         if(kiesy==-1) then
            aymin=ymin-0.5d0*yfct
            aymax=ymax+0.5d0*yfct
         end if
         if ( iscale==1 ) then

!        --- Plot scales if iscale = 1, with text textx respectively texty

             call plscls( 0, 1 )

         else

!        --- Plot x and y arrows

            if ( debug ) write(irefwr,*) 'before plaf04'
            call plaf04( 5, 0d0, 0d0 )
            if ( debug ) write(irefwr,*) 'after plaf04'

         end if
         if ( ierror/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pl0003'

      end if  ! ( debug )
      call erclos ( 'pl0003' )

      end
