      subroutine pl0006 ( wid1, hx, dx, hy, dy, h, z2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.6  date 01-09-2006 Extension of cpost1
!        version  8.5  date 06-02-1997 JvdZ: position of `ident'
!        version  8.4  date 09-10-1995 JvdZ: position of `factor'
!        version  8.3  date 04-12-1994 Various versions depending on isite
!        version  8.2  date 10-05-1990 changed layout of cplaf
!
!   copyright (c) 1984-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Plot frame work
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision wid1, hx, dx, hy, dy, h, z2

!     dx      i    length of vector in x-direction
!     dy      i    length of vector in y-direction
!     h       i    Help variable to store a constant temporarily
!     hx      i    ?
!     hy      i    ?
!     wid1    i    ?
!     z2      i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character *80 charac
      double precision xs, ys, amax, f
      integer nchar

!     amax
!     charac
!     f
!     nchar
!     xs
!     ys
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     IDENT1  Get user identification
!     PLAF03  Convert real number to character format
!     PLAFP6  Standard machine dependent plotting
!     PLAFP7  Plot text using absolute co-ordinates
!     PLOTKD  User-defined subroutine for plotting
!     PLOTKT  User-defined subroutine for defining text
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- plot frame work depending on isite

      if ( isite==0 ) then

!     --- isite = 0, general site

         xs=1d0/xfct
         ys=1d0/yfct
         amax=1d2
         if (irotat==3) then

!        --- irotat = 3,  act as irotat=1, however with new info block

            call plafp6(3,ale+3.5d0,0d0)
            call plafp6(2,ale+6.5d0,0d0)
            call plafp6(2,ale+6.5d0,18d0)
            call plafp6(2,ale+3.5d0,18d0)
            call plafp6(2,ale+3.5d0,0d0)
            call plafp6(3,ale+4.5d0,0d0)
            call plafp6(2,ale+4.5d0,18d0)
            call plafp6(3,ale+5.5d0,18d0)
            call plafp6(2,ale+5.5d0,0d0)
            call ident1(1,charac)
            call plafp7(ale+4.7d0,13.0d0,charac,-90d0,0.25d0,40)
            call plafp7(ale+5.7d0,17.5d0,'SEPRAN',-90d0,0.375d0,6)
            call plafp6(3,ale+5.5d0,13.5d0)
            call plafp6(2,ale+6.5d0,13.5d0)
            nchar=9
            charac(1:9)='SCALEX = '
            call plaf03(xs,charac,3,nchar,amax)
            call plafp7(ale+5.7d0,13d0,charac,-90d0,0.25d0,nchar)
            call plafp6(3,ale+5.5d0,6.5d0)
            call plafp6(2,ale+6.5d0,6.5d0)
            nchar=9
            charac(1:9)='SCALEY = '
            call plaf03(ys,charac,3,nchar,amax)
            call plafp7(ale+5.7d0,6d0,charac,-90d0,0.25d0,nchar)
            nchar=0
            if ( jkader==2 ) then

!           --- jkader = 2, plot mesh

               nchar=4
               charac(1:4)='MESH'
               h=0.375d0
            else if ( jkader==3 ) then

!           --- jkader = 3, plot factor

               nchar=9
               charac(1:9)='FACTOR = '
               call plaf03(z2,charac,3,nchar,amax)
               h=0.25d0
            else if ( jkader==4 ) then

!           --- jkader = 4, plot curves

               nchar=6
               charac(1:6)='CURVES'
               h=0.375d0
            else if ( jkader>10 ) then

!           --- jkader>10, plot submesh

               f=jkader-10
               nchar=8
               charac(1:8)='SUBMESH '
               call plaf03(f,charac,-1,nchar,amax)
               h=0.375d0
            end if
            if ( jkader==3 ) then
               call plafp7 ( ale+4.7d0, 4.5d0, charac, -90d0, h, nchar )
            else if ( jkader>1) then
               call plafp7 ( ale+4.7d0, 17d0, charac, -90d0, h, nchar )
            end if

!           --- user dependent names

            call plotkt ( charac, nchar )
            if ( nchar>0 ) &
               call plafp7 ( ale+3.7d0, 17d0, charac, -90d0, 0.375d0, &
                             nchar )
         else if (idirec==1) then

!        --- idirec = 1  normal direction
!            plot first block
!            large horizontal line

            dx = 13d0
            call plafp6 ( 3, 0d0, 0d0 )
            call plafp6 ( 2, hx, 0d0 )
            call plafp6 ( 2, hx, hy )
            call plafp6 ( 2, 0d0, hy )
            call plafp6 ( 2, 0d0, 0d0 )

!           --- vertical line

            call plafp6 ( 3, 0d0, dy )
            call plafp6 ( 2, hx, dy )

!           --- horizontal small line

            call plafp6 ( 3, dx, 0d0 )
            call plafp6 ( 2, dx, hy )

!           --- 2 small lines

            call plafp6 ( 3, dx, h )
            call plafp6 ( 2, hx, h )
            dx = 15.5d0
            call plafp6 ( 3, dx, 0d0 )
            call plafp6 ( 2, dx, dy )
            call plafp6 ( 3, dx, 2*dy )
            call plafp6 ( 2, dx, hy )

!           --- plot user text

            call plotkd ( idirec, wid1 )

!           --- plot xscale,  yscale

            call plafp7 ( 13.5d0, 4.3d0, 'SCALEX = ', 0d0, 0.25d0, 9 )
            nchar = 0
            call plaf03 ( xs, charac, 3, nchar, amax )
            call plafp7 ( 13.2d0, 3.7d0, charac, 0d0, 0.25d0, nchar )
            call plafp7 ( 16d0, 4.3d0, 'SCALEY = ', 0d0, 0.25d0, 9 )
            nchar = 0
            call plaf03 ( ys, charac, 3, nchar, amax )
            call plafp7 ( 15.7d0, 3.7d0, charac, 0d0, 0.25d0, nchar )

!           --- plot sepran  and special texts

            call plafp7 ( 0.4d0, 0.4d0, 'INGENIEURSBUREAU SEPRA', 0d0, &
                          0.4d0, 22 )
            call plafp6 ( 3, 0d0, 4d0 )
            call plafp6 ( 2, 3.35d0, 4d0 )
            call plafp6 ( 2, 3.35d0, 5d0 )
            call plafp7 ( .3d0, 4.3d0, 'SEPRAN', 0d0, 0.3d0, 6 )

!           --- special texts depending on jkader

            if ( jkader==2 ) then

!           --- jkader = 2, plot mesh

               call plafp7 ( 4d0, 2.5d0, 'MESH', 0d0, 0.5d0, 4 )
            else if ( jkader==3 ) then

!           --- jkader = 3, plot factor

               nchar = 9
               charac ( 1:9 ) = 'FACTOR = '
               call plaf03 ( z2, charac, 3, nchar, amax )
               call plafp7 ( 13.5d0, 2.4d0, charac, 0d0, 0.25d0, nchar )
            else if ( jkader==4 ) then

!           --- jkader = 4, plot curves

               call plafp7 ( 4d0, 2.5d0, 'CURVES', 0d0, 0.375d0, 6 )
            else if ( jkader>10 ) then

!           --- jkader>10, plot submesh

               f = jkader-10
               charac ( 1:8 ) = 'SUBMESH '
               nchar = 8
               call plaf03 ( f, charac, -1, nchar, amax )
               call plafp7 ( 4d0, 2.5d0, charac, 0d0, 0.5d0, nchar )
            end if

!           --- change origin

            call plafp6 ( 4, 0d0, hy )
            call plafp6 ( 3, 0d0, 0d0 )

!           --- plot last frame

            call plafp6 ( 2, ale+5d0, 0d0 )
            call plafp6 ( 2, ale+5d0, wid+2.5d0 )
            call plafp6 ( 2, 0d0, wid+2.5d0 )
            call plafp6 ( 2, 0d0, 0d0 )

         else

!        --- idirec  =  -1   rotated direction

            wid1 = wid+2.5d0
            if  ( wid1<hx ) wid1 = hx
            hx = wid1-hx
            dx = wid1-13d0
            h = 2d0*hy/3d0
            call plafp6 ( 3, 0d0, wid1 )
            call plafp6 ( 2, 0d0, hx )
            call plafp6 ( 2, hy, hx )
            call plafp6 ( 2, hy, wid1 )
            call plafp6 ( 2, 0d0, wid1 )
            call plafp6 ( 3, dy, wid1 )
            call plafp6 ( 2, dy, hx )
            call plafp6 ( 3, 0d0, dx )
            call plafp6 ( 2, hy, dx )
            call plafp6 ( 3, h, dx )
            call plafp6 ( 2, h, hx )
            dx = wid1-15.5d0
            call plafp6 ( 3, 0d0, dx )
            call plafp6 ( 2, dy, dx )
            call plafp6 ( 3, 2*dy, dx )
            call plafp6 ( 2, hy, dx )
            call plotkd ( idirec, wid1 )
            call plafp7 ( 4.3d0, wid1-13.5d0, 'SCALEX = ', -90d0, &
                          0.25d0, 9 )
            nchar = 0
            call plaf03 ( xs, charac, 3, nchar, amax )
            call plafp7 ( 3.7d0, wid1-13.2d0, charac, -90d0, 0.25d0, &
                          nchar )
            call plafp7 ( 4.3d0, wid1-16d0, 'SCALEY = ', -90d0, 0.25d0, &
                          9 )
            nchar = 0
            call plaf03 ( ys, charac, 3, nchar, amax )
            call plafp7 ( 3.7d0, wid1-15.7d0, charac, -90d0, 0.25d0, &
                          nchar )
            call plafp7 ( 0.2d0, wid1-.2d0, 'INGENIEURSBUREAU SEPRA', &
                          -90d0, 0.4d0, 22 )
            call plafp6 ( 3, 4d0, wid1 )
            call plafp6 ( 2, 4d0, wid1-3.35d0 )
            call plafp6 ( 2, 5d0, wid1-3.35d0 )
            call plafp7 ( 4.3d0, wid1-.3d0, 'SEPRAN', -90d0, 0.3d0, 6 )

!           --- special texts depending on jkader

            if ( jkader==2 ) then

!           --- jkader = 2, plot mesh

               call plafp7 ( 2.5d0, wid1-4d0, 'MESH', -90d0, 0.5d0, 4 )
            else if ( jkader==3 ) then

!           --- jkader = 3, plot factor

               nchar = 9
               charac ( 1:9 ) = 'FACTOR = '
               call plaf03 ( z2, charac, 3, nchar, amax )
               call plafp7 ( 2.4d0, wid1-13.5d0, charac, -90d0, 0.25d0, &
                             nchar )
            else if ( jkader==4 ) then

!           --- jkader = 4, plot curves

               call plafp7 ( 2.5d0, wid1-4d0, 'CURVES', -90d0, 0.375d0, &
                             6 )
            else if ( jkader>10 ) then

!           --- jkader>10, plot submesh

               f = jkader-10
               charac ( 1:8 ) = 'SUBMESH '
               nchar = 8
               call plaf03 ( f, charac, -1, nchar, amax )
               call plafp7 ( 2.5d0, wid1-4d0, charac, -90d0, 0.5d0, &
                             nchar )
           end if
           call plafp6 ( 4, hy, 0d0 )
           call plafp6 ( 3, 0d0, 0d0 )
           call plafp6 ( 2, ale+5d0, 0d0 )
           call plafp6 ( 2, ale+5d0, wid+2.5d0 )
           call plafp6 ( 2, 0d0, wid+2.5d0 )
           call plafp6 ( 2, 0d0, 0d0 )
         end if
      else

!     --- isite = 2 or 3, either TUE or TNO

         if ( idirec==1 )then
            if ( isite==2 ) then
               call plafp7 ( 3d0, 0.2d0, &
                             'EINDHOVEN UNIVERSITY OF TECHNOLOGY', &
                             0d0, 0.3d0, 34 )
               call plafp7 ( .5d0, 0.2d0, 'VIp', 0d0, 0.7d0, 3 )
            else
               call plafp7 ( 3d0, 0.2d0, &
                             'Kunstoffen en Rubber Instituut TNO', &
                             0d0, 0.3d0, 34 )
               call plafp7 ( .5d0, 0.2d0, 'VIp', 0d0, 0.7d0, 3 )
            end if
         else

!        --- idirec = -1

            wid1 = wid+2.5d0
            if ( wid1<hx ) wid1 = hx
            if ( isite==2 ) then
               call plafp7 ( 0.2d0, wid1-3d0, &
                             'EINDHOVEN UNIVERSITY OF TECHNOLOGY', &
                             -90d0, 0.3d0, 34 )
               call plafp7 ( 0.2d0, wid1-.5d0, 'VIp', -90d0, 0.7d0, 3 )
            else
               call plafp7 ( 0.2d0, wid1-3d0, &
                             'Kunstoffen en Rubber Instituut TNO', &
                             -90d0, 0.3d0, 34 )
               call plafp7 ( 0.2d0, wid1-.5d0, 'VIp', -90d0, 0.7d0, 3 )
            end if
         end if
      end if
      end
