      subroutine pl0009 ( isymbl, x, y, n, itypescales )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 18-10-1999 Extra parameter itypescales
!        version  2.1  date 05-09-1988
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   plot line with co-ordinates x and y
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isymbl, n, itypescales
      double precision x(*), y(*)

!     isymbl         i    indication how the line must be plotted.
!                         Possibilities:
!                          0   the line consists of straight segments
!                         <11  the line consists of straight segments
!                              and symbols
!                         <21  the line consists of symbols only
!     itypescales    i    type of scales used along each of the axis.
!                         Is subdivided into 2 according
!                         to itypescales = itype1+10*itype2+100*itype3
!                         where itype1 refers to the x-axis, itype to the
!                         y-axis
!                         The following values for itypex are available
!                          0  Linear scale
!                          1  Logarithmic scale (base 10)
!     n              i    number of points in the line
!     x              i    array of length n containing the x co-ordinates
!                         of the line
!     y              i    array of length n containing the y co-ordinates
!                         of the line
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypescalx, itypescaly
      double precision x1, y1

!     i              Counting variable
!     itypescalx     Type of scale in x-direction, see itypescales
!     itypescaly     Type of scale in y-direction, see itypescales
!     x1             x-coordinate of current point to be plotted
!     y1             y-coordinate of current point to be plotted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0009' )
      itypescalx = mod(itypescales,10)
      itypescaly = itypescales/10
      x1=x(1)
      y1=y(1)
      if ( itypescalx.eq.1 ) x1 = log10(x1)
      if ( itypescaly.eq.1 ) y1 = log10(y1)
      call plafep (4,x1,y1,0d0,0d0,0)
      if (isymbl.eq.0) then
         do i=2,n
            x1=x(i)
            y1=y(i)
            if ( itypescalx.eq.1 ) x1 = log10(x1)
            if ( itypescaly.eq.1 ) y1 = log10(y1)
            call plafep (5,x1,y1,0d0,0d0,0)
         end do
      else
         do i=1,n
            x1=x(i)
            y1=y(i)
            if ( itypescalx.eq.1 ) x1 = log10(x1)
            if ( itypescaly.eq.1 ) y1 = log10(y1)
            call plafep(17,x1,y1,0d0,0d0,isymbl)
         end do
      end if

      call erclos ( 'pl0009' )
      end
