      subroutine pl0011 ( iplots, rplots, ilplot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 01-09-2006 Extension of cpost1
!        version  1.4  date 20-07-2001 Extension of iplots
!        version  1.3  date 09-11-2000 Extension of iplots
!
!   copyright (c) 1991-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy contents of iplots into ilplot and extend with defaults
!    Do some checking
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulepost
      use sepmodulecons
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ilplot(*), iplots(*)
      double precision rplots(*)

!     ilplot  o   Contains the positions 2 to 7 of iplots, if necessary extended
!                 with defaults
!     iplots  i   Integer array in which the following quantities must be
!                 stored:
!                  1:  Position of last item that has been filled by the user
!                  2:  choice parameter for type of projection
!                      Possible values:
!                       0 : parallel projection plot
!                       1 : perspective projection plot
!                      10 : parallel projection plot (View point automatic)
!                      11 : perspective projection plot (View point automatic)
!                      Default value: 1
!                  3:  colour number of farthest triangle
!                      Default value: 1
!                  4:  colour number of nearest triangle
!                      Default value: maxcolour
!                  5:  Indication if boundaries must be plotted (1) or not (0)
!                      Default value: 1
!                  6:  idirrotat:  Direction of rotation:
!                      Possible values:
!                      1:  rotate in x-y plane
!                      2:  rotate in y-z plane
!                      3:  rotate in z-x plane
!                  7:  iskip:  Indication if element groups must be skipped (1)
!                              or not (0)
!                      Default value: 0
!                      Not yet implemented
!                  8:  numrotations:  Number of rotations
!                      Default value: 1
!                  9:  indication if volumes are specified from position 15
!                      (>0) or not (0)
!                 10:  Plot elements per layer (1) or not (0)
!                 11:  idefstart, defines how the start level must
!                      be computed
!                      1: Start with set of nodes with largest
!                         distance to view point
!                      2: Start with set of surfaces indicated in
!                         positions 12 and 13
!                 12:  ifirst, first surface number
!                 13:  ilast, last surface number
!                 15...14+nelgrp:  ihelp  Integer input array of length NELGRP
!                                       that is only used if ISKIP = 1
!                                       It indicates if element group ielgrp
!                                       must be skipped ihelp(ielgrp) = 0
!                                       or not ihelp(ielgrp) = 1
!     rplots  i   Double precision input array that must be filled as follows:
!                 1:    plotfm
!                 2:    yfact
!                 3-5:  eye-point
!                 6-8:  projection point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!    i        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     112   length of plot incorrect
!     857   YFACT is less than zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0011' )

!     --- Fill ilplot

      ilplot(1) = 1
      ilplot(2) = iinputpst(53)
      ilplot(3) = 1
      ilplot(4) = iinputpst(27)
      ilplot(5) = 1
      ilplot(6) = 1
      ilplot(7) = 0
      ilplot(8) = 1
      ilplot(9) = 0
      ilplot(10) = 0
      ilplot(11) = 1
      ilplot(12) = 1
      ilplot(13) = 1
      do i = 2, min(iplots(1),13)
         ilplot(i) = iplots(i)
      end do

!     --- Check rplots

      if ( rplots(1)<5d0 .or. rplots(1)>alengm ) then

!     --- Error 112: plotfm has incorrect value:

         call erreal ( rplots(1), 1 )
         call erreal ( 5d0, 2 )
         call erreal ( alengm, 3 )
         call errsub ( 112, 0, 3, 0 )
      end if
      if ( rplots(2)<0d0 ) then

!     --- Error 857: yfact has incorrect value:

         call erreal ( rplots(2), 1 )
         call errsub ( 857, 0, 1, 0 )
      end if
      call erclos ( 'pl0011' )
      end
