      subroutine pl0015 ( axiscr, plotfm, isitua )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 21-04-1995 Coordinate basis added, &
!                                      extra parameter isitua
!        version  2.0  date 01-12-1991 Extension with computing of normalised
!                                      axis
!        version  1.0  date 08-04-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       subroutine pl0015 is used to plot the axis in a 3D picture
!       The coordinates of the axis have already been computed
!       The axis is normalised such that it gets the same length in all cases
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     plot_axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isitua
      double precision axiscr(2,4), plotfm

!    axiscr   i   Co-ordinates of axis in projected plane
!                 axiscr(.,1)  Co-ordinates of centre of axis
!                 axiscr(.,2)  Co-ordinates of end point of x-axis
!                 axiscr(.,3)  Co-ordinates of end point of y-axis
!                 axiscr(.,4)  Co-ordinates of end point of z-axis
!    isitua   i   Indicator for coordinate basis
!                   1 :  x - y - z
!                   2 :  z - x - y
!                   3 :  y - z - x
!    plotfm   i   Length of the plot in centimeters
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision vnorm, dunitx, dunity

!    dunitx   Unit length in x-direction
!    dunity   Unit length in y-direction
!    i        General loop variable
!    vnorm    Help parameter to normalize the length of the axis to one
!             centimeter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Resets old name of previous subroutine of higher level
!     EROPEN   Produces concatenated name of local subroutine
!     PLAFEP   Standard SEPRAN plotting routine
!     PLTEXT   Plot text
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'pl0015' )
      dunitx = (xmax-xmin)/(plotfm-2d0)
      dunity = (ymax-ymin)/(width-2d0)
      xmin = xmin - 2d0 * dunitx
      ymin = ymin - 2d0 * dunity
      do 100 i = 2 , 4
          axiscr (1,i) = axiscr (1,i) - axiscr (1,1)
          axiscr (2,i) = axiscr (2,i) - axiscr (2,1)
100   continue
      axiscr (1,1) = xmin + dunitx
      axiscr (2,1) = ymin + dunity

!     --- Normalize the sum of computed unit vectors to length 1 cm, as long as
!         the vector is longer than "eps"

      do 110 i = 2, 4
          vnorm = sqrt( (axiscr(1,i)/dunitx)**2 + &
                        (axiscr(2,i)/dunity)**2 )
          if ( vnorm.gt.1d-4 ) then
               axiscr(1,i) = axiscr(1,i)/vnorm+axiscr(1,1)
               axiscr(2,i) = axiscr(2,i)/vnorm+axiscr(2,1)
          else
               axiscr(1,i) = axiscr(1,1)
               axiscr(2,i) = axiscr(2,1)
          end if
110   continue
      call plafep ( 4, axiscr(1,2), axiscr(2,2), 0d0, 0d0, 0 )
      call plafep ( 5, axiscr(1,1), axiscr(2,1), 0d0, 0d0, 0 )
      call plafep ( 5, axiscr(1,3), axiscr(2,3), 0d0, 0d0, 0 )
      call plafep ( 4, axiscr(1,4), axiscr(2,4), 0d0, 0d0, 0 )
      call plafep ( 5, axiscr(1,1), axiscr(2,1), 0d0, 0d0, 0 )
      if ( isitua.eq.2 ) then

!     --- z - x - y

         call pltext ( axiscr(1,2), axiscr(2,2), 'z', 0d0, .3d0 )
         call pltext ( axiscr(1,3), axiscr(2,3), 'x', 0d0, .3d0 )
         call pltext ( axiscr(1,4), axiscr(2,4), 'y', 0d0, .3d0 )
      else if ( isitua.eq.3 ) then

!     --- y - z - x

         call pltext ( axiscr(1,2), axiscr(2,2), 'y', 0d0, .3d0 )
         call pltext ( axiscr(1,3), axiscr(2,3), 'z', 0d0, .3d0 )
         call pltext ( axiscr(1,4), axiscr(2,4), 'x', 0d0, .3d0 )
      else

!     --- x - y - z

         call pltext ( axiscr(1,2), axiscr(2,2), 'x', 0d0, .3d0 )
         call pltext ( axiscr(1,3), axiscr(2,3), 'y', 0d0, .3d0 )
         call pltext ( axiscr(1,4), axiscr(2,4), 'z', 0d0, .3d0 )
      end if
      call erclos( 'pl0015' )
      end
