      subroutine pl0016 ( plotfm, width, yfact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-11-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pl0016 checks the parameter PLOTFM and computes the width
!       of the plot depending on plotfm and yfact
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision plotfm, yfact, width

!     plotfm  i   Length of the plot in centimeters
!     width   o   Width of the plot in centimeters
!     yfact   i   Scalings factor. The width of the plot is equal to plotfm
!                 multiplied by yfact
!                 If yfact=0, the width of the plot is equal to plotfm
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     110   Width of plot less than 5 cm
!     113   Length of plot less than 5 cm
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0016' )
      if ( plotfm.lt.5d0 ) then
         call erreal ( plotfm, 1 )
         call erreal ( 5d0, 2 )
         call errsub ( 113, 0, 2, 0 )
         plotfm = 15d0
      end if
      width = yfact*plotfm
      if ( width.ge.0d0 .and. width.le.0d0 ) width = plotfm
      if ( width.lt.5d0 ) then
         call erreal ( width, 1 )
         call erreal ( 5d0, 2 )
         call errsub ( 110, 0, 2, 0 )
         width = 15d0
      end if
      call erclos ( 'pl0016' )
      end
