      subroutine plaf04 ( ichois, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.6  date 06-02-1997 Improve y-arrow; JvdZ
!        version  4.5  date 01-01-1993 New call of plaf01
!        version  4.4  date 12-06-1990 New position of texts
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot text or arrows with text, depending on the value of ichois
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois
      double precision x, y

!     ichois  i    Choice parameter
!                  Possible values
!                  1, 2      Plot text with height 0.15*ichois cm
!                  5      Plot arrows with text
!                  6      Plot arrow with text along 2nd y-axis (no x-text)
!     x       i    is only used if ichois = 1 or 2.
!                  Gives x-coordinate of starting point of the text
!     y       i    is only used if ichois = 1 or 2.
!                  Gives y-coordinate of starting point of the text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iaantx, iaanty, iside
      double precision a1, a2, xp, yp, zhelpx, zhelpy, xp1, yp1

!     a1      ?
!     a2      ?
!     iaantx  ?
!     iaanty  ?
!     iside   Indicates at which side the cross section is positioned
!             Possible values:
!             0: There is no cross section point computed in this call
!             1: The cross section point is at side y=ymint
!             2: The cross section point is at side x=xmaxt
!             3: The cross section point is at side y=ymaxt
!             4: The cross section point is at side x=xmint
!     xp      x-coordinate in plotting units
!     xp1     ?
!     yp      y-coordinate in plotting units
!     yp1     ?
!     zhelpx  ?
!     zhelpy  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAF01  Translate user coordinates into plotter coordinates
!     PLAFP6  Standard machine dependent plotting
!     PLAFP7  Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iaantx=len(textx)
      iaanty=len(texty)
      if (ichois.eq.5.or.ichois.eq.6) then

!     --- ichois=5, 6  plot arrows with text

         zhelpx=iaantx*0.25d0
         zhelpy=iaanty*0.25d0
         if (idirec.ge.0) then
            if (ichois.eq.6) go to 100

!           --- idirec=1; x horizontal, y vertical

            if (iaantx.gt.0) then

!           --- x direction

               call plafp7(3.5d0, 0.75d0, textx, 0d0, 0.25d0, iaantx)
               call plafp6(3, zhelpx+3.75d0, 0.875d0)
               call plafp6(2, zhelpx+4.5d0, 0.875d0)
               call plafp6(2, zhelpx+4.25d0, 1.0d0)
               call plafp6(2, zhelpx+4.25d0, 0.75d0)
               call plafp6(2, zhelpx+4.5d0, 0.875d0)
            end if
            if (iaanty.gt.0) then

!           --- y direction

               call plafp7(0.875d0, 3.5d0, texty, 90d0, 0.25d0, iaanty)
               call plafp6(3, 0.75d0, zhelpy+3.75d0)
               call plafp6(2, 0.75d0, zhelpy+4.5d0)
               call plafp6(2, 0.625d0, zhelpy+4.25d0)
               call plafp6(2, 0.875d0, zhelpy+4.25d0)
               call plafp6(2, 0.75d0, zhelpy+4.5d0)
            end if
            go to 200
100         if (iaanty.gt.0) then

!           --- 2nd y direction

               call plafp7(ale+4.00d0, 3.5d0, texty, 90d0,0.25d0,iaanty)
               call plafp6(3, ale+4.00d0, zhelpy+3.75d0)
               call plafp6(2, ale+4.00d0, zhelpy+4.5d0)
               call plafp6(2, ale+3.875d0, zhelpy+4.25d0)
               call plafp6(2, ale+4.125d0, zhelpy+4.25d0)
               call plafp6(2, ale+4.00d0, zhelpy+4.5d0)
            end if
  200       continue
         else

!        --- idirec=-1;  x vertical, y horizontal

            if (iaanty.gt.0) then

!           --- y direction

               call plafp7(3.5d0, wid+2.5d0, texty, 0d0, 0.25d0, iaanty)
               call plafp6(3, zhelpy+3.75d0, wid+2.625d0)
               call plafp6(2, zhelpy+4.5d0, wid+2.625d0)
               call plafp6(2, zhelpy+4.25d0, wid+2.75d0)
               call plafp6(2, zhelpy+4.25d0, wid+2.5d0)
               call plafp6(2, zhelpy+4.5d0, wid+2.625d0)
            end if
            if (iaantx.gt.0) then

!           --- x direction

               call plafp7(0.25d0, wid-2d0, textx, -90d0, 0.25d0,iaantx)
               call plafp6(3, .375d0, wid-2.25d0-zhelpx)
               call plafp6(2, .375d0, wid-3d0-zhelpx)
               call plafp6(2, 0.25d0, wid-2.75d0-zhelpx)
               call plafp6(2, 0.5d0, wid-2.75d0-zhelpx)
               call plafp6(2, .375d0, wid-3d0-zhelpx)
            end if
         end if
      else

!     --- ichois=1, 2   plot text

         call plaf01 ( x, y, xp, yp, xp1, yp1, iside )
         if ( jtest.gt.0 .and. ioutsd.ge.2 ) return
         a2=-0.075d0*ichois
         a1=ichois*0.0375d0
         if (idirec.eq.1) then
            call plafp7(xp+a1, yp+a2, textx, 0d0, 0.15d0*ichois, iaantx)
         else
            call plafp7(xp+a2, yp-a1, textx, -90d0,0.15d0*ichois,iaanty)
         end if
      end if
      end
