      subroutine plbc06 ( ihelp, icont, ibrelm, ibrpnt, itria, kmeshc, &
                          ielem, i1, i2, funk, w, npoint )
! ======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.0    date  20-01-1996 New layout, extra parameter
!        version 1.1    date  22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0    date  29-07-1986
!
!
!   copyright (c) 1986-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!    subroutine to determine
!    - third point of triangle with corners i1 and i2
!    - to fill i1 and i2 in ihelp
!    - next element for contourline
! ***********************************************************************
!
!                      KEYWORDS
!
!   2d
!   element
!   plot_contour
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer ihelp(2,*), ibrelm(*), ibrpnt(*), itria(*), kmeshc(*), &
              icont, ielem, i1, i2, npoint
      double precision funk(*), w

!     funk       i    array containing the function values in the nodes
!     i1        i/o   nodes of line which contour passes entering
!                     element ielem ,at output i1 -i2 have been
!                     adjusted as well as ielem for next triangle
!     i2        i/o   See i1
!     ibrelm     i    help array with elements each node belongs to
!     ibrpnt     i    pointer array for ibrelm
!     icont      o    number of elements of ihelp
!     ielem      i    element side i1-i2 belongs to element ielem
!     ihelp     i/o   array containing the lines (nodes of lines)
!                     which the contour passes
!                     length: 4*npoint+100
!     itria     i/o   help array to indicate which elements have
!                     been considered already
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     npoint     i    Number of nodal points
!     w          i    contour value to be considered
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer icr1, icr2, icr3, i3, ielem1, ielem2
      double precision f1, f3

!     f1
!     f3
!     i3
!     icr1
!     icr2
!     icr3
!     ielem1
!     ielem2
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PLBC05    determine the, atmost 2, elements where both nodes i1 and i2
!               belong to
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1699   Array too small
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- determine third point

100   icr1=kmeshc(3*ielem-2)
      icr2=kmeshc(3*ielem-1)
      icr3=kmeshc(3*ielem  )

      if ( icr1.ne.i1 .and. icr1.ne.i2 ) i3=icr1
      if ( icr2.ne.i1 .and. icr2.ne.i2 ) i3=icr2
      if ( icr3.ne.i1 .and. icr3.ne.i2 ) i3=icr3

      f1=funk(i1)-w
      f1=f1/abs(f1)
      f3=funk(i3)-w
      f3=f3/abs(f3)

      if ( f1*f3.lt.0d0 ) then
           i2=i3
      else
           i1=i3
      end if

!     --- fill ihelp

      icont=icont+1
      if ( icont.gt.2*npoint+50 ) then
         call eropen ( 'plbc06' )
         call errint ( 2*icont, 1 )
         call errint ( 4*npoint+100, 2 )
         call errchr ( 'ihelp', 1 )
         call errsub ( 1699, 2, 0, 1 )
         call erclos ( 'plbc06' )
         go to 1000
      end if
      ihelp(1,icont)=i1
      ihelp(2,icont)=i2

!     --- determine new element number

      call plbc05 ( ibrelm, ibrpnt, i1, i2, ielem1, ielem2 )

      if ( ielem2.eq.0 ) go to 1000

      if ( ielem.eq.ielem1 ) then
         ielem=ielem2
      else
         ielem=ielem1
      end if

      itria(ielem)=0

      goto 100

1000  continue

      end
