      subroutine plbc13 ( jsmoot, inodes, coorp, coorn, npoint, ilev )
! ======================================================================
!
!
!        programmer     Niek Praagman
!        version 1.3    date  20-01-1996 Extra error message
!        version 1.2    date  22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.1    date  17-02-1989 Deletion of superfl. parms
!        version 1.0    date  12-09-1986
!
!
!   copyright (c) 1986-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   subroutine to smooth contourlines using the spline - technique
!   only lines with at least 3 points are taken into account
! ***********************************************************************
!
!                      KEYWORDS
!
!   2d
!   element
!   plot_contour
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer inodes(*), jsmoot, npoint, ilev
      double precision coorp(2,*), coorn(2,*)

!     coorn      i    help array for temporary storage of results
!     coorp     i/o   coordinates of points on line , at output all
!                     internal points have been smoothed
!     ilev       i    number of level being considered
!     inodes     i    array containing information with respect to
!                     the contourlines ( number of nodes and whether
!                     line is closed or open )
!     jsmoot     i    smoothing indicator (see Users Manual)
!     npoint     i    Number of nodal points
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, icoor, jkn, ital, icirc, icoorp
      double precision x, y

!     i          Counting variable
!     icirc      indicator whether curve is open (icirc=0) or
!                closed (icirc=1)
!     icoor
!     icoorp
!     ital
!     jkn
!     x          x co-ordinate of point
!     y          y-coordinate
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PLAFEP    General SEPRAN low level plotting subroutine
!     PLBC14    driving subroutine for spline smoothing
!     PLBC22    consider the length of the spline pieces
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1699   Array too small
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      icoor=0
      jkn=1
      ital=inodes(jkn)
100   if ( ital.ne.0 ) then

         if ( ital.lt.0 ) then
            ital=-ital
            icirc=1
         else
            icirc=0
         end if

         if ( ital.le.2 ) then

!        --- plot line without smoothing

            icoor=icoor+1
            x=coorp(1,icoor)
            y=coorp(2,icoor)
            call plafep(4,x,y,0d0,0d0,0)

            do 110 i=2,ital
                icoor=icoor+1
                x=coorp(1,icoor)
                y=coorp(2,icoor)
                call plafep(5,x,y,0d0,0d0,0)
110         continue

         else

            icoorp=icoor+1
            icoor=icoor+ital
            call plbc22 ( coorp(1,icoorp), ital, icirc )

            call plbc14 ( jsmoot, coorp(1,icoorp), &
                          ital, coorn, icirc, npoint, ilev )

         end if
         if ( icoor.gt.2*npoint+50 ) then
            call eropen ( 'plbc13' )
            call errint ( 2*icoor, 1 )
            call errint ( 4*npoint+100, 2 )
            call errchr ( 'coorp', 1 )
            call errsub ( 1699, 2, 0, 1 )
            call erclos ( 'plbc13' )
            go to 1000
         end if

         jkn=jkn+1
         ital=inodes(jkn)
         goto 100

      end if

1000  continue

      end
