      subroutine plbc14 ( jsmoot, work, n, work2, icirc, npoint, ilev )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.3  date 20-01-1996 New layout
!        version  2.2  date 16-09-1991 stop replaced by call instop
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 03-08-1987
!
!   copyright (c) 1986-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   driving subroutine for spline smoothing
!   see also R.A. van Buul, T.U.Delft, 1984, for more
!   information. Some subroutines are based on this work
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision work(*), work2(*)
      integer jsmoot, n, icirc, npoint, ilev

!     icirc      i    indicator whether curve is open (icirc=0) or
!                     closed (icirc=1)
!     ilev       i    Sequence number of level being considered
!     jsmoot     i    Parameter indicating the amount of smoothing
!                     Minimum value 0, maximum value 5
!     n          i    length of curve (number of points)
!     npoint     i    Number of nodal points
!     work       i    array containing x and y coordinates of curve to
!                     be smoothed
!     work2      i    help array for determination of spline coefficients
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision hstep, rho

!     hstep
!     rho
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PLBC15    smoothes either the x-coordinates or the y-coordinates
!     PLBC16    plots the smoothed contours
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1699   Array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- discretization step for numerical derivatives

      hstep = 0.01d0*(xmax-xmin+ymax-ymin)

!     --- data fit parameter rho

      if ( jsmoot.eq.2 ) rho=1d-1
      if ( jsmoot.eq.3 ) rho=1d-2
      if ( jsmoot.eq.4 ) rho=5d-2
      if ( jsmoot.eq.5 ) rho=5d-3

!     --- check length of array WORK2

      length=5+11*n
      if ( length.gt.8*npoint+100 ) then
         call eropen ( 'plbc14' )
         call errint ( length, 1 )
         call errint ( 8*npoint+100, 2 )
         call errchr ( 'work2', 1 )
         call errsub ( 1699, 2, 0, 1 )
         call erclos ( 'plbc14' )
         go to 1000
      end if

!     --- smooth x-coordinates

      call plbc15 ( work, n, icirc, work2, hstep, rho, 1 )

!     --- smooth y-coordinates

      call plbc15 ( work, n, icirc, work2, hstep, rho, 2 )

!     --- plot curves

      call plbc16 ( work2, n, ilev )
1000  continue
      end
