      subroutine plbc36 ( ncurvs, kmeshm, procor, icolor, nprcor )
! =======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.1    date   21-11-1994 Norms
!        version 2.0    date   30-12-1992 Extra parameter nprcor
!        version 1.0    date   04-04-1990
!
!
!
!   copyright (c) 1990-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Plot curves in projection plane
!
! ***********************************************************************
!
!     KEYWORDS
!
!     curve
!     plot
! ***********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer ncurvs, kmeshm(*), icolor, nprcor
      double precision procor(nprcor,*)

!     icolor  i    colour of lines to be used
!     kmeshm  i    KMESH part m, contains nodal point numbers of the curves
!     ncurvs  i    Number of curves in mesh
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, ipen, ist, ien, ikn
      double precision xp, yp

!     i       loop variable
!     ien     end address of nodes curve
!     ikn     node number
!     ipen    helpvariable for plotting
!     ist     starting address of nodes curve
!     j       Counting variable
!     xp      x-coordinate of projected point
!     yp      y-coordinate of projected point
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from calling string
!     EROPEN  Concatenate name of subroutine to calling string
!     PLAFEP  General SEPRAN low level plotting subroutine
!
! ***********************************************************************
!
!     I/O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all curves and plot for each curve from node to node
!
! =======================================================================
!
      call eropen( 'plbc36' )

!     --- plot all constituting curves of the mesh

      do 200 i = 1 , ncurvs

         ist  = kmeshm(4+i) + kmeshm(2) - 1
         ien  = kmeshm(5+i) + kmeshm(2) - 2

         ipen = 4

         do 100 j = ist , ien

             ikn = kmeshm( j )

!            --- Compute coordinates in projection plane

             xp  = procor( 1,ikn )
             yp  = procor( 2,ikn )

!            --- Move (if first point of curve) or Plot to point xp,yp

             call plafep( ipen, xp, yp, 0d0, 0d0, icolor )

             ipen = 5

100      continue

200   continue

      call erclos( 'plbc36' )

      end
