      subroutine plbc38 ( kmeshc, coor, func, ns, v, procor, &
                          contln, ncntln, icolor, jcolor, nprcor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.1  date 17-11-1994 Disturb vali values such that they
!                                      are never zero)
!        version  2.0  date 30-12-1992 Extra parameter nprcor
!        version  1.2  date 19-12-1992 Plot sequence numbers of contours
!        version  1.1  date 27-06-1992 Initialise tst and ten
!        version  1.0  date 06-03-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot projections of contourlines on a mesh in 3-D as given by
!     KELEM and COOR
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     plot_contour
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), ns, ncntln, icolor, jcolor, nprcor
      double precision coor(*), procor(nprcor,*), contln(*), &
                       func(*), v(*)

!     contln  i    Array ith the contour levels
!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     func    i    function array
!     icolor  i    color parameter for visible parts
!     jcolor  i    color parameter for non-visible parts
!     kmeshc  i    KMESH part c, containing the nodal point numbers of the
!                  elements
!     ncntln  i    length of contln
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     ns      i    number of triangles
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     v       i    co-ordinates of view point (3D)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iv, iview, ivis, idr, kp, lp, mp, &
              i1, i2, i3, ip, iq, icnt
      double precision eps, ext, tnv, tst, ten, f1, f2, f3, value,  yb, &
                       x1, y1, x2, y2, x3, y3, val1, val2, val3, s, xb, &
                       xa, ya, xao, yao, zao, xbo, ybo, zbo, xvip, yvip, &
                       xha, xhb, yha, yhb, view(100)

!     eps     accuracy parameter
!     ext     small value to move points
!     f1      function value
!     f2      function value
!     f3      function value
!     i       loop variable
!     i1      node number
!     i2      node number
!     i3      node number
!     icnt    loop variable
!     idr     loop variable
!     ip      indicator for first cutting point
!     iq      indicator for second cutting point
!     iv      loop variable
!     iview   number of visible subparts
!     ivis    visibility indicator
!     kp      first node of triangle
!     lp      second node of triangle
!     mp      third node of triangle
!     s       mean value of derivative in element
!     ten     end of interval
!     tnv     helppoint in interval
!     tst     start of interval
!     val1    local function value
!     val2    local function value
!     val3    local function value
!     value   actual contour value
!     view    help array to keep up with visible and non-visible parts
!     x1      x-coordinate of first projected point
!     x2      x-coordinate of second projected point
!     x3      x-coordinate of third projected point
!     xa      local x-coordinate of point A
!     xao     original x-coordinate
!     xb      local x-coordinate of point B
!     xbo     original x-coordinate
!     xha     help x-coordinate point A
!     xhb     help x-coordinate point B
!     xvip    x-coordinate contournumber
!     y1      y-coordinate of first projected point
!     y2      y-coordinate of second projected point
!     y3      y-coordinate of third projected point
!     ya      local y-coordinate of point A
!     yao     orignal y-coordinate
!     yb      local y-coordinate of point B
!     ybo     orignal y-coordinate
!     yha     help y-coordinate point A
!     yhb     help y-coordinate point B
!     yvip    y-coordinate contournumber
!     zao     original z-coordinate
!     zbo     original z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAFEP  General SEPRAN plotting subroutine
!     PLBC26  Subroutine to determine whether a part of a line is covered
!             by a triangle in the projection plane
!     PLBC30  Plot part of a line in 3D
!     PLT352  Cancel parts of segment that are not visible
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Do for all contourlines
!        Compute contourvalue
!        Do for all triangles
!           Determine whether contourline is in triangle
!           If line is in triangle then
!              Do for all triangles : check whether line is
!              visible, i.e. whether all other triangles are not
!              in front of this line. If part of line is visible
!              plot line with colour icolor and plot non-visible
!              part with color jcolor.
!           Endif
!        Enddo
!        Plot contour level number at appropriate place
!     Enddo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      tst = 0d0
      ten = 0d0
      eps = 123 * epsmac
      do 600 icnt = 1 , ncntln
           value =  contln ( icnt )
           xvip  = -1d6
           do 500 i = 1 , ns
               i1 = kmeshc(3*i-2)
               i2 = kmeshc(3*i-1)
               i3 = kmeshc(3*i  )
               f1 = func(i1)
               f2 = func(i2)
               f3 = func(i3)
               val1 = f1 - value - eps
               val2 = f2 - value - eps
               val3 = f3 - value - eps
               f1 = abs(val1)
               f2 = abs(val2)
               f3 = abs(val3)
               x1 = procor(1,i1)
               y1 = procor(2,i1)
               x2 = procor(1,i2)
               y2 = procor(2,i2)
               x3 = procor(1,i3)
               y3 = procor(2,i3)
               ip = 0
               iq = 0
               if ( val1 * val2.lt.0 ) then
                    s   = f1 / ( f1 + f2 )
                    xa  = x1 + s * ( x2 - x1 )
                    ya  = y1 + s * ( y2 - y1 )
                    xao = ( 1d0 - s ) * coor(3*i1-2) + s * coor(3*i2-2)
                    yao = ( 1d0 - s ) * coor(3*i1-1) + s * coor(3*i2-1)
                    zao = ( 1d0 - s ) * coor(3*i1  ) + s * coor(3*i2  )
                    ip  = 1
               end if
               if ( val1 * val3.lt.0 ) then
                    s   = f1 / ( f1 + f3 )
                    xb = x1 + s * ( x3 - x1 )
                    yb = y1 + s * ( y3 - y1 )
                    xbo = ( 1d0 - s ) * coor(3*i1-2) + s * coor(3*i3-2)
                    ybo = ( 1d0 - s ) * coor(3*i1-1) + s * coor(3*i3-1)
                    zbo = ( 1d0 - s ) * coor(3*i1  ) + s * coor(3*i3  )
                    iq = 1
               end if
               if ( val2 * val3.lt.0 ) then
                  s = f2 / ( f2 + f3 )
                  if ( ip.eq.1 ) then
                    xb  = x2 + s * ( x3 - x2 )
                    yb  = y2 + s * ( y3 - y2 )
                    xbo = ( 1d0 - s ) * coor(3*i2-2) + s * coor(3*i3-2)
                    ybo = ( 1d0 - s ) * coor(3*i2-1) + s * coor(3*i3-1)
                    zbo = ( 1d0 - s ) * coor(3*i2  ) + s * coor(3*i3  )
                  end if
                  if ( iq.eq.1 ) then
                    xa  = x2 + s * ( x3 - x2 )
                    ya  = y2 + s * ( y3 - y2 )
                    xao = ( 1d0 - s ) * coor(3*i2-2) + s * coor(3*i3-2)
                    yao = ( 1d0 - s ) * coor(3*i2-1) + s * coor(3*i3-1)
                    zao = ( 1d0 - s ) * coor(3*i2  ) + s * coor(3*i3  )
                  end if
               end if
               if ( ip.eq.0 .and. iq.eq.0 ) goto 500

!              --- Move original points marginally

               ext = 1d-4 * ( xbo - xao )
               xao = xao + ext
               xbo = xbo - ext
               ext = 1d-4 * ( ybo - yao )
               yao = yao + ext
               ybo = ybo - ext
               ext = 1d-4 * ( zbo - zao )
               zao = zao + ext
               zbo = zbo - ext

!              --- Move projected points marginally

               ext = 1d-4 * ( xb - xa )
               xa  = xa  + ext
               xb  = xb  - ext
               ext = 1d-4 * ( yb - ya )
               ya  = ya  + ext
               yb  = yb  - ext

!              --- Take hidden lines into account

               ivis = 0
               call plt352( ivis, tst, ten, view, iview )
               do 480 idr = 1 , ns
                   if ( idr.eq.i ) goto 480
                   kp = kmeshc(3*idr-2)
                   lp = kmeshc(3*idr-1)
                   mp = kmeshc(3*idr  )

!                  --- Determine intersection points

                   tst = 0d0
                   ten = 0d0
                   call plbc26( v, xa, ya, xao, yao, zao, &
                                xb, yb, xbo, ybo, zbo, &
                                kp, lp, mp, coor, procor, &
                                ivis, tst, ten, 0, nprcor )
                   if ( ivis.ne.2 ) &
                   call plt352( ivis, tst, ten, view, iview )
                   if ( ivis.eq.2 ) then

!                  --- Line is not visible, if jcolor is gt 0
!                      plot line with color jcolor

                      if ( jcolor.gt.0 ) then
                         call plafep( 4, xa, ya, 0d0, 0d0, 0 )
                         call plafep( 5, xb, yb, 0d0, 0d0, jcolor )
                      end if
                      goto 500
                   end if
480            continue
               tnv = 0d0
               do 490 iv = 1 , iview
                   tst = view(2*iv-1)
                   ten = view(2*iv  )
                   if ( jcolor.gt.0 ) then

!                  --- Plot non visible part

                      if ( (tst - tnv).gt.5d-3 ) then
                         xha = 1.001 * xa
                         xhb = 1.001 * xb
                         yha = 1.001 * ya
                         yhb = 1.001 * yb
                         call plbc30( tnv, tst, xha, yha, xhb, yhb, &
                                      jcolor )
                      end if
                   end if
                   tnv = ten

!                  --- Plot visible part

                   if ( (ten - tst).gt.5.d-3 ) then
                        xha = 1.001 * xa
                        yha = 1.001 * ya
                        xhb = 1.001 * xb
                        yhb = 1.001 * yb
                        call plbc30( tst, ten, xha, yha, xhb, yhb, &
                                     icolor )
                   end if
                   if ( ten - tst.lt.0.99999d0 ) then
                        call plafep( 10, xa, ya, 0d0, 0d0, icnt )
                        xvip = -1d6
                   else
                        xvip = xa
                        yvip = ya
                   end if
490            continue
               if ( jcolor.gt.0 ) then
                  if ( (1d0 - tnv).gt.5d-3 ) &
                     call plbc30( tnv, 1d0, xa, ya, xb, yb, jcolor )
               end if
500        continue

!          --- Plot contour level number icnt at xvip, yvip

           if ( abs( xvip+1d6 ).gt.1d0 ) &
                call plafep( 10, xvip, yvip, 0d0, 0d0, icnt )
600   continue
      end
