      subroutine plelempar ( coor, kmeshc, ilow, nelem, inpelm, &
                             nboun, ndim, elements )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-04-2010 New call to plaf05
!        version  1.0  date 14-02-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    plot elements for one sequence number
!    when jmark<3 the elements are provided with element numbers
!    If this subroutine is changed also change plelem
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, inpelm, kmeshc(inpelm,nelem), ilow, nboun, ndim, &
              elements(*)
      double precision coor(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     elements       i    Array of length nelem indicating if element ielem
!                         must be plotted (>0) or not (0)
!     ilow           i    elements are numbered from element number ilow
!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nboun          i    number of points in element to be plotted
!                         nboun<0 defines special situations:
!                         -1:  The element is a quadrilateral consisting of
!                              boundary points only. On each side the number of
!                              boundary points is equal to inpelm/4+1
!                              The numbering of the nodes is in the direction of
!                              the axis, i.e. first the under boundary, then the
!                              under side points etc. Finally the upper boundary
!                              from left to right
!                         -2:  The element is a quadrilateral consisting of
!                              boundary points and internal points, where the
!                              number of points on a side is equal to
!                              sqrt(inpelm). Internal points are
!                              created by taking the product of directions.
!                              The numbering is the same as in the case -1
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision c1
      integer ielem, j, n, nodp
      double precision :: x1(2,nboun)

!     c1             Help variable
!     ielem          Element number
!     j              Counting variable
!     n              number of nodes to be plotted in an element
!     nodp           Nodal point sequence number
!     x1             work array of length 2*nboun
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plelempar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plelempar'

      end if

      if ( ndim==2 ) then

!     --- ndim = 2, two-dimensional elements

         do ielem = 1, nelem

            if ( elements(ielem+ilow-1)>0 ) then

!           --- store coordinates of boundary points in arrays x1

               if ( nboun==-1 ) then

!              --- nboun = -1, special situation
!                  First the under boundary

                  n = inpelm/4+1
                  do j = 1, n
                     nodp = kmeshc(j,ielem)
                     x1(1,j) = coor(1,nodp)
                     x1(2,j) = coor(2,nodp)
                  end do

!                 --- Next the sides

                  do j = 1, n-2
                     nodp = kmeshc(n+2*j-1,ielem)
                     x1(1,inpelm+1-j) = coor(1,nodp)
                     x1(2,inpelm+1-j) = coor(2,nodp)
                     nodp = kmeshc(n+2*j,ielem)
                     x1(1,n+j) = coor(1,nodp)
                     x1(2,n+j) = coor(2,nodp)
                  end do

!                 --- Upper boundary

                  do j = 1, n
                     nodp = kmeshc(inpelm+1-j,ielem)
                     x1(1,j+2*n-2) = coor(1,nodp)
                     x1(2,j+2*n-2) = coor(2,nodp)
                  end do
                  n = inpelm

               else if ( nboun==-2 ) then

!              --- nboun = -2, special situation
!                  First the under boundary

                  c1 = inpelm
                  c1 = sqrt(c1)
                  n = nint(c1)
                  do j = 1, n
                     nodp = kmeshc(j,ielem)
                     x1(1,j) = coor(1,nodp)
                     x1(2,j) = coor(2,nodp)
                  end do

!                 --- Next the sides

                  do j = 1, n-2
                     nodp = kmeshc(j*n+1,ielem)
                     x1(1,4*n-3-j) = coor(1,nodp)
                     x1(2,4*n-3-j) = coor(2,nodp)
                     nodp = kmeshc(n*(j+1),ielem)
                     x1(1,n+j) = coor(1,nodp)
                     x1(2,n+j) = coor(2,nodp)
                  end do

!                 --- Upper boundary

                  do j = 1, n
                     nodp = kmeshc(inpelm+1-j,ielem)
                     x1(1,j+2*n-2) = coor(1,nodp)
                     x1(2,j+2*n-2) = coor(2,nodp)
                  end do
                  n = 4 * (n-1)

               else

!              --- Standard situation, iboun>0

                  do j = 1, nboun
                     nodp = kmeshc(j,ielem)
                     x1(1,j) = coor(1,nodp)
                     x1(2,j) = coor(2,nodp)
                  end do
                  n = nboun

               end if

!              --- plot element

               call plaf05 ( 1, dcolor, n, x1, 2 )

            end if  ! ( elements(ielem)==1 )

         end do

      else

!     --- ndim = 1, one-dimensional elements

         do ielem = 1, nelem

            if ( elements(ielem+ilow-1)>0 ) then

!           --- store coordinates of boundary points in arrays x1

               do j = 1, inpelm
                  x1(1,j) = coor(1,kmeshc(j,ielem))
               end do

!              --- plot element

               if ( inpelm>1 ) then
                  do j = 2, inpelm
                     call plafep(5, x1(1,j), 0d0, 0d0, 0d0, 0)
                  end do
               end if

            end if  ! ( elements(ielem)==1 )

         end do

      end if

      call erclos ( 'plelempar' )
      if ( debug ) write(irefwr,*) 'End plelempar'

      end
