      subroutine plfl01 ( coor, nbndpt, Eloc, vecold, in, x1, y1 )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.0  date 12-07-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the electric field in point (x1,y1), and (if necessary)
!     the direction it points.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*), eloc(2), vecold(*), x1, y1
      integer in, nbndpt(*)

!     coor     i   KMESH parti, see P.G. 13.1.6.
!     eloc     o   electric field in point (x1,y1)
!     in      i/o  indicator checking on inward(2) or outward(3) flux
!     nbndpt   i   array containing the boundary points
!     vecold   i   electric field
!     x1       o   x-coordinate of starting point
!     y1       o   y-coordinate of starting point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx, dy, x2, y2
      integer ibnd, ikn1, ikn2

!     dx      x1 - x2
!     dy      y1 - y2
!     ibnd    boundary element number
!     ikn1    first node of boundary element IBND
!     ikn2    second node of boundary element IBND
!     x2      x-coordinate of IKN2
!     y2      y-coordinate of IKN2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!         If the outer product between the vectors E and dx is positive then
!         the electric field points outward else inward.
!         PSEUDO CODE
!         Calculate extreme coordinates
!         Get first boundary element
!         Get nodes of this element
!         Get coordinates of these nodes
!         Get electric field in the first node (Eloc(1), Eloc(2))
!         Calculate boundary element vector (dx, dy)
!         If dx * Eloc(2) > dy * Eloc(1) then
!            Inward
!         Else
!            Outward
!         Endif
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ibnd = 1
      ikn1 = nbndpt(2*ibnd-1)
      ikn2 = nbndpt(2*ibnd  )
      x1 = coor(2*ikn1-1)
      y1 = coor(2*ikn1  )
      x2 = coor(2*ikn2-1)
      y2 = coor(2*ikn2  )
      dx = x2 - x1
      dy = y2 - y1
      Eloc(1) = vecold(2*ikn1-1)
      Eloc(2) = vecold(2*ikn1)
      if ( in.eq.1 ) then
         if ( dx * Eloc(2).gt.dy * Eloc(1) ) then
            in = 2
         else
            in = 3
         end if
      end if
      end
