      subroutine plmn01
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-01-1993 Reset centroid in case of option r
!        version  1.0  date 08-01-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Menu define zooming parameters
!   Read zooming parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     interactive
!     menu
!     zooming
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ready
      integer kstop, inum
      double precision resul(4), dx, dy

!     dx      length of picture in x-direction multiplied by 10-6
!     dy      length of picture in y-direction multiplied by 10-6
!     inum    Number of numbers read in a record
!     kstop   help variable to save the value of istop
!     ready   Indicates if the loop has been finished (true) or not (false)
!     resul   work array in which the numbers read are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item from the standard input file
!     ERREAL  Put real in error message
!     ERTEXT  Produce menu
!     INICLS  Clear screen
!     READSR  Read a series of numbers from the input (texts are neglected)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ready := false
!   While (not false) do
!      print sub menu
!      read answer
!      if ( answer = 'none' ) then
!         ready = true
!      else if ( answer = c ) then
!         define centre of zooming
!         check if centre is within region
!      else if ( answer = f ) then
!         define zooming factor
!      else if ( answer = h ) then
!         print help menu
!      else if ( answer = p ) then
!         define plotting rectangle
!      else if ( answer = r ) then
!         reset plotting rectangle and centroid
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ready = .false.

!     --- While loop ready

100   if ( .not. ready ) then

!     --- print sub menu

         call inicls
         call erreal ( xminzm, 1 )
         call erreal ( xmaxzm, 2 )
         call erreal ( yminzm, 3 )
         call erreal ( ymaxzm, 4 )
         call erreal ( xminor, 5 )
         call erreal ( xmaxor, 6 )
         call erreal ( yminor, 7 )
         call erreal ( ymaxor, 8 )
         call erreal ( zfact, 9 )
         call erreal ( xcentr, 10 )
         call erreal ( ycentr, 11 )
         call ertext ( 33, 0, 11, 0 )

!        --- Read answer

         ityprd = 1
         ispecl = 1
         call asknxt
         if ( ispecl.eq.2 ) then

!        --- Blank record found hence RETURN

            ready = .true.
            go to 100
         else if ( ityprd.ne.4 ) then

!        --- No text found, repeat menu

            go to 100
         end if
         if ( string(1:1).eq.'C' ) then

!        --- Letters C found, define centre of zooming

            write(*,*) ' Give centre of zooming (2 co-ordinates)'
            kstop = istop
            istop = 2
            call readsr ( resul, 2, inum )
            istop = kstop
            if ( resul(1).ge.xminzm .and. resul(1).le.xmaxzm .and. &
                 resul(2).ge.yminzm .and. resul(2).le.ymaxzm ) then

!           --- Point is within region

               xcentr = resul(1)
               ycentr = resul(2)
            else

!           --- Point outside region

               call erreal ( resul(1), 1 )
               call erreal ( resul(2), 2 )
               call erreal ( xminzm, 3 )
               call erreal ( xmaxzm, 4 )
               call erreal ( yminzm, 5 )
               call erreal ( ymaxzm, 6 )
               call ertext ( 5, 0, 6, 0 )
               read (*,*)
            end if
         else if ( string(1:1).eq.'F' ) then

!        --- Letters F found, define zooming factor

            write(*,*) ' Give new zooming factor'
            kstop = istop
            istop = 2
            call readsr ( resul, 1, inum )
            istop = kstop
            if ( resul(1).gt.0d0 ) then
               zfact = resul(1)
            else
               call erreal ( resul(1), 1 )
               call ertext ( 6, 0, 1, 0 )
               read (*,*)
            end if
         else if ( string(1:1).eq.'H' ) then

!        --- Help menu

            call inicls
            call ertext ( 34, 0, 0, 0 )
            read (*,*)
            call inicls
            call ertext ( 35, 0, 0, 0 )
            read (*,*)
         else if ( string(1:1).eq.'P' ) then

!        --- Letters P found, define plotting rectangle

            write(*,*) ' Give new rectangle (4 values)'
            kstop = istop
            istop = 2
            call readsr ( resul, 4, inum )
            istop = kstop
            resul(1) = max ( xminor, resul(1) )
            resul(2) = min ( xmaxor, resul(2) )
            resul(3) = max ( yminor, resul(3) )
            resul(4) = min ( ymaxor, resul(4) )
            dx = (xmaxor-xminor)*sqreps
            dy = (ymaxor-yminor)*sqreps
            if ( resul(2)-resul(1).lt.dx .or. resul(4)-resul(3).lt.dy ) &
               then

!           --- Region not correct

               call erreal ( resul(1), 1 )
               call erreal ( resul(2), 2 )
               call erreal ( resul(3), 3 )
               call erreal ( resul(4), 4 )
               call ertext ( 7, 0, 4, 0 )
               read (*,*)
            else
               xminzm = resul(1)
               xmaxzm = resul(2)
               yminzm = resul(3)
               ymaxzm = resul(4)
               xcentr = 0.5d0 * ( xmaxzm-xminzm )
               ycentr = 0.5d0 * ( ymaxzm-yminzm )
            end if
         else if ( string(1:1).eq.'R' ) then

!        --- Letters R found, reset plotting rectangle

            xminzm = xminor
            xmaxzm = xmaxor
            yminzm = yminor
            ymaxzm = ymaxor
            xcentr = 0.5d0 * ( xmaxzm-xminzm )
            ycentr = 0.5d0 * ( ymaxzm-yminzm )
         end if
         go to 100
      end if
      end
