      subroutine plotbouninter ( kstap, kstapl, coor, icount, text1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2003 Extra parameter text1
!        version  1.0  date 24-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot actual boundary during the process of creation of a mesh by msho2d
!     Subroutine meant for checking purposes
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     plot
!     check
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstap, kstapl(2,*), icount
      double precision coor(2,*)
      character *(*) text1

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icount         i    Counter to be used as text in the plotting
!     kstap          i    Number of pairs in kstapl
!     kstapl         i    Array containing the actual boundary build by
!                         pairs of nodes
!     text1          i    text to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x, y
      integer  i, i1, i2, lentext
      character *100 text

!     i              Counting variable
!     i1             First point of line
!     i2             Last point of line
!     lentext        Number of characters in text
!     text           String variable containing some text
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open plot
!     For i := 1, kstap do
!        Plot line
!     Close plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotbouninter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotbouninter'

      end if

!     --- Open plot

      call pl0000 ( 15d0, 1d0 )
      call plafep( 1, aleng, width, 1d0, 15d0, 1 )

!     --- For i := 1, kstap do

      do i = 1, kstap

!     --- Plot line

         i1 = kstapl(1,i)
         i2 = kstapl(2,i)
         x =  coor(1,i1)
         y =  coor(2,i1)
         call plafep( 6, x, y, 0d0, 0d0, 58 )
         call plafep( 4, x, y, 0d0, 0d0, 0 )
         x =  coor(1,i2)
         y =  coor(2,i2)
         call plafep( 5, x, y, 0d0, 0d0, 0 )
         call plafep( 6, x, y, 0d0, 0d0, 63 )

      end do  ! i = 1, kstap

!     --- Close plot

      lentext = len(text1)+1
      text = text1
      text(lentext+1:lentext+18) = '        loop '
      write(text(lentext+19:lentext+24),100) icount
100   format(i6)
      call plafp7 ( 1d0, 1d0, text, 0d0, 0.25d0, lentext+24 )
      call plafep( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      call erclos ( 'plotbouninter' )
      if ( debug ) write(irefwr,*) 'End plotbouninter'

      end
