      subroutine plotc0 ( number, ncntln, plotfm, yfact, ichois, isubr, &
                          iplots, rplots, lnplot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.3  date 07-02-2000 Correction iinner
!        version  4.2  date 28-04-1995 Extension of IPLOTS
!        version  4.1  date 15-01-1993 Extension of RPLOTS
!        version  4.0  date 26-12-1992 Complete new functionality;
!                                      the old one is now in PLCN00)
!        version  3.1  date 05-12-1992 Extension with max colour
!        version  3.0  date 28-09-1992 Complete new definition
!        version  2.2  date 23-03-1992 New call of ini023
!        version  2.1  date 21-03-1991 Error message for ncntln=-1001
!        version  2.0  date 20-10-1990 New parameter list, removal of name, &
!                                      Error message for ncntln=1)
!        version  1.2  date 19-06-1990 New call to pl0004
!        version  1.1  date 02-06-1990 Relative criterion for constant field
!        version  1.0  date 11-04-1988
!
!   copyright (c) 1988-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill arrays IPLOTS and RPLOTS for subroutine PLCN00 in the case of a
!   call by one of the old subroutines PLOTC1/2/3
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     fill
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, ncntln, ichois, isubr, lnplot, iplots(lnplot)
      double precision rplots(*), plotfm, yfact

!     ichois  i  Choice parameter consisting of 4 parts:
!                ichois = jsmoot + 10 x idoubl + 100 x iinner + 1000 x icompt
!                with jsmoot, idoubl, iinner and icompt as defined in common
!                jsmoot  indication which kind of smoothing is used
!                        possibilities:
!                        0:  no smoothing
!                        1:  shuman smoothing
!                        2:  |
!                        3:  |  all spline smoothing : the higher the value the
!                        4:  |  more smoothing and the less interpolation of
!                        5:  |  the values.
!                        If PLOTC is called by subroutine PLOTC2 then jsmoot is
!                        the parameter JINDCOL indicating if array INDCOL is
!                         filled by the user (1) or not (0)
!                idoubl  indication whether the outer boundary is plotted
!                        as a single line (0) or as double line (1)
!                iinner  indication whether inner boundaries must be plotted
!                        (1) or not (0)
!                icompt  Parameter indicating how contour levels should be
!                        computed. Possibilities:
!                        0:  Standard situation as described previously
!                        1:  The minimum level is given in contln(6), the
!                            maximum level must be computed
!                        2:  The maximum level is given in contln(7), the
!                            minimum level must be computed
!                        3:  The minimum level is given in contln(6), the
!                            maximum level in contln(7)
!     iplots  o  Integer control array in which the user defines which choices
!                he wants for this subroutine
!                IPLOTS must be filled  by the user in the following way:
!                Position:
!                1:  Not used
!                2:  (CHOICE) defines the type of contour lines or levels
!                    required. Possible values for CHOICE are:
!                    1:  Contour lines will be drawn. These lines are
!                        provided with a numbered label.
!                        In the case of a 3D grid, contour lines are plotted
!                        on the outer surface only
!                    2:  Contour levels will be coloured, i.e. the interior
!                        space between contour levels as well as the region
!                        smaller than the smallest contour level and the
!                        region larger than the largest contour level is
!                        filled with a specific colour for that level.
!                    3:  See 1, however, not only are the contour levels
!                        plotted, but also the co-ordinates of these contour
!                        lines are outputed in the arrays COORCN and NCOORC
!                        This possibility is only available in 2D
!                3:  (NUMBER) the number'th physical unknown in each node is
!                    considered When this unknown does not appear in a nodal
!                    point, the point is skipped
!                    When array ISOL corresponds to a complex array, &
!                    NUMBER=2*i-1 corresponds to the real part of unknown i
!                    and NUMBER=2*i to the imaginary part of unknown i
!                4:  (NCNTLN) Parameter indicating the number of contour
!                    levels to be used. Possible values of ncntln are:
!                    > 0:  There are ncntln contour levels that must be stored
!                          in array contln from position 6
!                    = 0:  The program computes the contour levels by
!                          dividing the range of the function between maximum
!                          and minimum value into 10 parts in the case of
!                          CHOICE = 1 or 3 and in 20 parts if CHOICE=2
!                    < 0 > -1000: The program computes the contour levels by
!                          dividing the range of the function between maximum
!                          and minimum value into |NCNTLN| - 1 parts. The
!                          values are chosen such that rounded values of the
!                          contour levels appear
!                    < -1000: The program computes the contour levels by
!                          dividing the range of the function between maximum
!                          and minimum value into |NCNTLN + 1000| - 1 parts.
!                5:  (JINDCOL) Parameter indicating the type of colouring to
!                    be used for the contour lines or levels:
!                    Possible values:
!                    0:  The standard colours are used, i.e. standard colour
!                        contour lines in the case of CHOICE=1 or CHOICE=3
!                        and colour numbers 1 to number_of_contour_levels in
!                        the case of CHOICE=2.
!                    1:  The user defines the colour numbers explicitly
!                        through array INDCOL. This possibility is
!                        moment only available for CHOICE=2
!                    2:  The user defines the minimum colour number explicitly
!                        through array INDCOL. This possibility is
!                        only available for CHOICE=2
!                    3:  The user defines the minimum and maximum colour
!                        number explicitly through array INDCOL. This
!                        possibility is only available for CHOICE=2
!                    >0:  The visible parts of the contour lines get the
!                        colour sequence number JINDCOL CHOICE=1 or 3
!                6:  (PROJECT) choice parameter for type of projection
!                    (3D only)
!                    0:  parallel projection plot (View point automatic)
!                    1:  perspective projection plot (View point automatic)
!                    10: parallel projection plot (view point given by the
!                        user in RPLOTS(4-6) )
!                    11: perspective projection plot (view point given by the
!                        user in RPLOTS(4-6) )
!                7:  (JSMOOTH) choice parameter for type of smoothing to be
!                    used for the contour lines. At this moment JSMOOTH is
!                    only implemented for CHOICE=1 or 3 in 2-dimensional
!                    grids.
!                    Possible values:
!                    0:  no smoothing
!                    1:  shuman smoothing
!                    2:  |
!                    3:  |  all spline smoothing : the higher the value the
!                    4:  |  more smoothing and the less interpolation of
!                    5:  |  the values.
!                8:  (IDOUBLE) indication whether the outer boundary is
!                    plotted as a single line (0) or as double line (1)
!                    (2D only)
!                9:  (INNER) indication whether inner boundaries must be
!                    plotted (1) or not (0)
!                    (2D only)
!               10:  (PLOT_CONTOUR) indication if the boundaries of the
!                    contourlevels must be plotted (>0) or not (0). The
!                    value of PLOT_CONTOUR defines the colour.
!                    This possibility is only available in the case CHOICE=2.
!               11:  (TYPE_LEGENDA) defines the type of legenda to be plotted
!                    Possible values:
!                    1:  No legenda
!                    2:  Standard legenda; the text is plotted at the right
!                        side of the picture.
!                    3:  Horizontal legenda; the text plotted below the
!                        picture. Only minimum and maximum values are given.
!                        Only available for CHOICE=2
!               12:  (TYPE_SCALES) defines whether the scaling factors are
!                    plotted (1) or not (2)
!               13:  (ISKIP) indicates if element groups must be excluded (>0)
!                    or not (0)
!                    If ISKIP>0, the value of ISKIP defines the first position
!                    in INPUT where information about the element groups
!                    to be excluded is stored
!               14:  IDEFFM   Indicates if default sizes of the plot are used
!                    (IDEFFM=0) or that the user gives these values
!                    in RPLOTS(1) and RPLOTS(2) (IDEFFM=1)
!                    If IDEFFM=2 the user must also fill RPLOTS(3) with a
!                    premultiplication factor for the solution
!               15:  (IMESH) indicates if the mesh must be plotted in the
!                    picture (>0) or not (0)
!                    If IMESH>0, the value of IMESH defines the colour
!                    sequence number with which the mesh must be plotted.
!                    In 3D only the visible parts of the mesh are plotted
!                    with colour IMESH
!               16:  (INVIS_MESH) indicates if the invisible part of the
!                    mesh must be plotted in the picture (>0) or not (0)
!                    If INVIS_MESH>0, the value of INVIS_MESH defines the
!                    colour sequence number with which the invisible parts
!                    of the mesh must be plotted. (CHOICE=1 and 3D only)
!               17:  (INVIS_CONT) indicates if the invisible part of the
!                    contour lines must be plotted in the picture (>0) or not
!                    (0). If INVIS_CONT>0, the value of INVIS_CONT defines the
!                    colour sequence number with which the invisible parts
!                    of the contours must be plotted. (CHOICE=1 and 3D only)
!               18:  (BACK_COLOUR) indicates if back ground lines must
!                    plotted in the picture (>0) or not (0)
!                    If BACK_COLOUR>0, the value of BACK_COLOUR defines the
!                    colour sequence number with which the back ground lines
!                    must be plotted. (CHOICE=1 and 3D only)
!               19:  (CONS_COLOUR) indicates if constituting lines must be
!                    plotted in the picture (>0) or not (0)
!                    If CONS_COLOUR>0 and < 100, the value of CONS_COLOUR
!                    defines the colour sequence number with which the
!                    constituting lines must be plotted.
!                    If CONS_COLOUR>100 only the visible part of the
!                    constituting lines is plotted, the value of
!                    CONS_COLOUR-100 defines the colour sequence number
!                    with which the constituting lines must be plotted.
!                    This possibility is only available for CHOICE=1 and 3D
!               20:  (ITEXT) indicates if text must be plotted in the
!                    picture.
!                    Possible values:
!                    0:  No text at all
!                    1:  Extra text at bottom
!                    2:  Extra text at bottom and top
!               21:  (ICOMPT) indicates how the contour levels must be
!                    computed from NCNTLN in the case NCNTLN <= 0
!                    0:  The minimum and maximum level are computed
!                    1:  The minimum level is given in contln(6), the
!                        maximum level must be computed
!                    2:  The maximum level is given in contln(7), the
!                        minimum level must be computed
!                    3:  The minimum level is given in contln(6), the
!                        maximum level in contln(7)
!               22:  (IORIENT) indicates the orientation of the base vectors
!                    Possible values:
!                    1 : x - y - z system (also default)
!                    2 : z - x - y system
!                    3 : y - z - x system
!     isubr   i    Indicates by which subroutine PLOTC0 has been called
!                  Possible values:
!                  1:  PLOTC1
!                  2:  PLOTC2
!                  3:  PLOTC3
!     lnplot  i    Length of array IPLOTS
!     ncntln  i    Parameter to compute the number of contour levels
!     number  i    The number-th unknown in each nodal point of the array
!                  is considered. If the array is complex number=2*i-1
!                  corresponds to the real part of array i and number=2*i
!                  to the imaginary part of array i
!     plotfm  i    Plotting factor. When PLOTFM < 0, then the length of
!                  the plot is computed by - (xmax-xmin) * PLOTFM
!                  and the width by - (ymax-ymin) * YFACT * PLOTFM
!                  else the length of the plot is equal to PLOTFM cm
!                  When by some mistake the user has given | PLOTFM | a value
!                  smaller than 1 cm, then the length is chosen equal to
!     rplots  i    Double precision input array that must be filled as follows:
!                  1:    plotfm
!                  2:    yfact
!                  3:    scale factor
!                        This parameter is only used if IDEFFM=2
!                  4-6:  eye-point  (Only if PROJECT>9)
!     yfact   i    Multiplication for the y-co-ordinates. With this
!                  parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icompt, jsmoot, iinner, idoubl

!     i       Counting variable
!     icompt  Parameter indicating how contour levels should be computed.
!             Possibilities:
!             0:  Standard situation as described previously
!             1:  The minimum level is given in contln(6), the maximum
!                 level must be computed
!             2:  The maximum level is given in contln(7), the minimum
!                 level must be computed
!             3:  The minimum level is given in contln(6), the maximum
!                 level in contln(7)
!     idoubl  indication whether the outer boundary is plotted
!             as a single line (0) or as double line (1)
!     iinner  indication whether inner boundaries must be plotted
!             (1) or not (0)
!     jsmoot  indication which kind of smoothing is used
!             possibilities:
!             0:  no smoothing
!             1:  shuman smoothing
!             2:  |
!             3:  |  all spline smoothing : the higher the value the more
!             4:  |  smoothing and the less interpolation of the values .
!             5:  |
!             If PLOTC0 is called by subroutine PLOTC2 then jsmoot is
!             the parameter JINDCOL indicating if array INDCOL is filled
!             by the user (1) or not (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array IPLOTS
!     Fill array RPLOTS
!     Compute parameters from ichois
!     Fill rest of array IPLOTS
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do 100 i = 1, lnplot
         iplots(i) = 0
100   continue
      rplots(1) = plotfm
      rplots(2) = yfact
      rplots(3) = 1d0
      icompt = ichois / 1000
      jsmoot = ichois - 1000 * icompt
      iinner = jsmoot / 100
      jsmoot = jsmoot - 100 * iinner
      idoubl = jsmoot / 10
      jsmoot = jsmoot - 10 * idoubl
      if ( iinner.ne.0 ) iinner = 2
      iplots(2)  = isubr
      iplots(3)  = max(1,number)
      iplots(4)  = ncntln
      iplots(8)  = idoubl
      iplots(9)  = iinner
      iplots(21) = icompt
      iplots(22) = 1
      if ( isubr.eq.2 ) then
         iplots(5) = jsmoot
      else
         iplots(5) = dcolor
         iplots(7) = jsmoot
      end if
      if ( jtimes.ge.10 ) then
         iplots(11) = 2
         iplots(12) = 1
      else
         iplots(11) = 1
         iplots(12) = 2
      end if
      end
