      subroutine plotcurv ( icurvs, curves, ncurvs, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-11-2005 Fortran 90
!        version  1.1  date 21-05-2001 New call to pl0001
!        version  1.0  date 25-10-1997
!
!   copyright (c) 1997-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    this subroutine plots the curves  from the arrays icurvs and curves
!    when ndim = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     curve
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, icurvs(ncurvs)
      double precision curves(2,*)
      character *(*) text

!     curves     i    array containing the coordinates of the curves
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     ncurvs     i    Number of curves in mesh
!     text       i    String variable containing some text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, j, ikader, ncount, iuppl, n, iscolr
      double precision x, y, s, sprec, smean, xpr, ypr, plotfm, yfact

!     charac    String to store a text
!     icurnr    Curve sequence number
!     ikader    Help variable to store jkader temporarily
!     iscolr    Help variable to store the value of dcolor temporarily
!     iuppl     Indication if the pen must move to a point with pen up
!               (4) or down (#4)
!     j         Counting variable
!     n         Number of points in the curves that are plotted until now, &
!               plus present curve
!     ncount    Number of points in the curves that are plotted until now
!     plotfm    Plotting factor. When PLOTFM<0, then the length of
!               the plot is computed by - (xmax-xmin) * PLOTFM
!               and the width by - (ymax-ymin) * YFACT * PLOTFM
!               else the length of the plot is equal to PLOTFM cm
!               When by some mistake the user has given | PLOTFM | a value
!               smaller than 1 cm, then the length is chosen equal to
!               the default length stored in common block consta
!     s         Computed actual length of a curve
!     smean     Half the length of the complete curve
!     sprec     Computed length of the curve until present element
!     x         x-Co-ordinates of a point
!     xpr       x-Co-ordinates of preceding point on curve
!     y         y-Co-ordinates of a point
!     yfact     Multiplication for the y-co-ordinates. With this
!               parameter the user can influence the scale in y-direction
!     ypr       y-Co-ordinates of preceding point on curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     PL0000    Compute length and width of the plot
!     PL0001    Compute the parameters xmin, xmax, ymin and ymax
!     PLAFEP    General SEPRAN low level plotting subroutine
!     PLAFP7    Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotcurv' )
      iscolr = dcolor

      ikader = jkader
      plotfm = 15d0
      yfact = 1d0
      if(jkader>=0) jkader = 4
      ncount = icurvs(ncurvs)
      if ( mod(jtimes,10)<=1 ) then
         if ( jmax==0 ) &
            call pl0001 ( curves, 2, ncount, xmin, xmax, ymin, ymax )

         call pl0000 ( plotfm, yfact )

!        --- start picture

         call plafep ( 1, aleng, width, 0d0, 0d0, jmax )

      end if

!     --- plot curves

      ncount = 1
      do icurnr = 1, ncurvs
         n = icurvs(icurnr)
         s = 0d0
         iuppl = 4

!        --- plot curve

         do j = ncount, n
            if ( iscolr>0 ) dcolor = 1+iscolr
            call plafep ( iuppl, curves(1,j), curves(2,j), 0d0, 0d0, 0 )
            iuppl = 5
         end do

!        --- iplcur = 2, Plot curve numbers
!            compute length of curve

         dcolor = iscolr
         do j = ncount, n
            x = curves(1,j)
            y = curves(2,j)
            if ( j==ncount ) then

!           --- j = ncount, set s equal to zero

               s = 0d0

            else

!           --- Not the first point, raise s

               s = s + sqrt ( (xpr-x)**2 +(ypr-y)**2 )

            end if

            xpr = x
            ypr = y

         end do

         smean = s * 0.5d0

!        --- Find element in the center of the boundary

         do j = ncount, n
            x = curves(1,j)
            y = curves(2,j)
            if ( j==ncount ) then

!           --- j = ncount, set s equal to zero

               s     = 0d0
               sprec = 0d0

            else

!           --- Not the first point, raise s

                s = s + sqrt ( (xpr-x)**2 +(ypr-y)**2 )

            end if

            if ( s>smean ) then

!           --- s>smean, center element found

               s = (smean-sprec) / (s-sprec)
               x = xpr + s * (x-xpr)
               y = ypr + s * (y-ypr)
               call plafep ( 9, x, y, 0.2d0, 0.2d0, icurnr )
               go to 150

            end if

            sprec = s
            xpr = x
            ypr = y

         end do

150      ncount = n+1

      end do

      call plafp7 ( 10d0, 2d0, text, 0d0,0.5d0, len ( text ) )

!     --- Plot time at the bottom of the plot

      call plottime

!     --- close plot file

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      jkader = ikader
      call erclos ( 'plotcurv' )
      dcolor = iscolr

      end
