      subroutine plotelem ( kmeshc, inpelm, coor, dcolor, contract )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot one element (2d) with color dcolor
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     element
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm), dcolor
      double precision coor(2,*)
      logical contract

!     contract       i    If true the elements must be contracted before
!                         plotting
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dcolor         i    Colour sequence number
!     inpelm         i    number of boundary points per element
!     kmeshc         i    kmesh part c, contains the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, nodp
      double precision x1(2,inpelm), c

!     c              help parameter to compute the contraction
!     j              Counting variable
!     nodp           Nodal point sequence number
!     x1             help array to store the coordinates of the nodes of an
!                    element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotelem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotelem'
         write(irefwr,1) 'inpelm, dcolor', inpelm, dcolor
         call prinin ( kmeshc, inpelm, 'kmeshc' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store coordinates of nodes in x1

      do j = 1, inpelm
         nodp = kmeshc(j)
         x1(1,j) = coor(1,nodp)
         x1(2,j) = coor(2,nodp)
      end do

      if ( contract ) then

!     --- Contract the elements

         do j = 1, 2
            c = sum ( x1(j,:) ) / inpelm
            x1(j,:) = (c+4d0*x1(j,:))*0.2d0
         end do  ! j = 1, 2

      end if  ! ( contract )

      call plaf05 ( 2, dcolor, inpelm, x1, 2 )
      call plafep(4, x1(1,1), x1(2,1), 0d0, 0d0, 0)

1000  call erclos ( 'plotelem' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotelem'

      end if  ! ( debug )

      end
