      subroutine plotfunc1minmx ( nfunc, funcx, funcy, ny )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-10-2012 Jump in case of error
!        version  1.0  date 19-05-2010
!
!   copyright (c) 2010-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute minimum and maximum along x and y-axis for function plots
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     function
!     minimum
!     maximum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfunc, ny
      double precision funcx(nfunc), funcy(nfunc,ny)

!     funcx          i    Set of values in x-direction
!     funcy          i    ny sets of values in y-direction
!     nfunc          i    Number of values per function
!     ny             i    number of functions to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xminval, yminval, xmaxval, ymaxval, dx, dy

!     dx             10% of difference in x-direction
!     dy             10% of difference in y-direction
!     xmaxval        Maximum value in x-direction
!     xminval        Minimum value in x-direction
!     ymaxval        Maximum value in y-direction
!     yminval        Minimum value in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PLOTSCALES     Plot scales depending on minimum and maximum values
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute minimum and maximum of x-values
!     Compute minimum and maximum of y-values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfunc1minmx' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfunc1minmx'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ny<=0 ) then

!     --- Error ny out of range

         call errint ( ny, 1 )
         call errint ( 10000, 1 )
         call errchr ( 'ny', 1 )
         call errsub ( 1378, 2, 0, 1 )
         go to 1000

      end if  ! ( ny<=0 )

      if ( kiesx==-1 ) then

!     --- kiesx = -1:  Set axmin and axmax

         xminval = minval(funcx)
         xmaxval = maxval(funcx)
         axmin = xminval
         axmax = xmaxval

      end if  !  ( kiesx==-1 )

      if ( kiesy==-1 ) then

!     --- kiesx = -1:  Set axmin and axmax

         yminval = minval(funcy)
         ymaxval = maxval(funcy)
         aymin = yminval
         aymax = ymaxval

      end if  !  ( kiesx==-1 )

      if ( debug ) &
         write(irefwr,2) 'axmin, axmax, aymin, aymax/1 ', &
                          axmin, axmax, aymin, aymax

!     --- Round values to nice values for plotting

      call plotscales

      dx = (axmax-axmin)*.1d0
      dy = (aymax-aymin)*.1d0

      xmax = axmax+dx*5d0
      ymax = aymax+dy
      xmin = axmin-dx
      ymin = aymin-dy

1000  call erclos ( 'plotfunc1minmx' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'axmin, axmax, aymin, aymax', &
                          axmin, axmax, aymin, aymax
         write(irefwr,*) 'End plotfunc1minmx'

      end if  ! ( debug )

      end
