      subroutine plotfunc1scal ( valmin, valmax, minout, maxout, step, &
                                 ndec, exp1, shift )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute scales to be used for plot function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     scale
!     function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndec, exp1
      double precision valmin, valmax, minout, maxout, step, shift

!     exp1           o    Exponent to be used for these values
!     maxout         o    Scaled and rounded valmax
!     minout         o    Scaled and rounded valmin
!     ndec           o    Number of digits to be used in step
!     shift          o    Shift of numbers if step is much lower than numbers
!     step           o    Step along axis
!     valmax         i    Maximum value of function
!     valmin         i    Minimum value of function
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer exp3
      double precision diff, eps

!     diff           Difference between valmax and valmin
!     eps            Local accuracy
!     exp3            Exponent corresponding to valmax-valmin
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTCOMPSCAL1  Compute exponent and step for scales
!     PLOTCOMPSCAL2  Scale numbers for scales, since they are too large
!     PLOTCOMPSCAL3  Round minout and maxout to round values, related to step
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfunc1scal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfunc1scal'
         write(irefwr,2) 'valmin, valmax', valmin, valmax
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndec = 0                 ! number of decimals
      shift = 0d0              ! shift
      diff = valmax-valmin     ! difference between minimum and maximum
      eps = 1d-6*abs(valmax)
      if ( debug ) write(irefwr,2) 'diff', diff

      if ( diff<=eps ) then

!     --- Function is constant, use range from -1 to 1

         if ( abs(valmax)<1d0 ) then

!        --- constant between -1 and 1

            minout = -1d0    ! minimum value
            maxout = 1d0     ! maximum value
            step = 0.5d0     ! step
            ndec = 1         ! number of digits behind decimal point
            exp1 = 0         ! exponent for mulitplication

         else

!        --- other constant
!            Compute step by setting valmin to 0 temporarily

            call plotcompscal1 ( exp1, 0d0, valmax, minout, &
                                 maxout, step, ndec, exp3 )
            minout = maxout     ! reset minout to maxout

!           --- Make nice values

            call plotcompscal3 ( minout, maxout, step )
            if ( debug ) &
               write(irefwr,2) 'minout, maxout, step', &
                                minout, maxout, step

         end if  ! ( abs(valmax)<1d0 )
         go to 1000

      end if  ! ( diff<=0d0 )

!     --- Compute exponent and step

      call plotcompscal1 ( exp1, valmin, valmax, minout, &
                           maxout, step, ndec, exp3 )

!     --- Round minout and maxout to nice values

      call plotcompscal3 ( minout, maxout, step )

      if ( abs(minout)>1d5 .or. abs(maxout)>1d5 ) then

!     --- Numbers too large, use scaling

         call plotcompscal2 ( exp1, exp3, minout, maxout, step, ndec, &
                              shift )

      end if  ! ( abs(minout)>1d5 .or. abs(maxout)>1d5 )

1000  call erclos ( 'plotfunc1scal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndec, exp1', ndec, exp1
         write(irefwr,2) 'minout, maxout, step, shift', &
                          minout, maxout, step, shift
         write(irefwr,*) 'End plotfunc1scal'

      end if  ! ( debug )

      end
