      subroutine plotintnumber ( x, y, inumbr, sizetxt, ipos, &
                                 xshift, yshift, nchar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot an integer number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     number
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipos, inumbr, nchar
      double precision sizetxt, x, y, xshift, yshift

!     inumbr         i    Number to be plotted
!     ipos           i    Defines position of number with respect to
!                         given coordinates
!                         1:  centered
!                         2:  Left allignment of text
!                         3:  Right allignment of text
!     sizetxt          i    Size of numbers (multiplied by 0.15)
!     x              i    x co-ordinate of point (standard coordinates)
!     xshift         i    Shift in x direction (plotting coordinates)
!     y              i    y co-ordinate of point (standard coordinates)
!     yshift         i    Shift in y direction (plotting coordinates)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iside
      double precision xp, yp, xpboun, ypboun
      character (len=20) text

!     iside          Dummy
!     nchar          Number of characters in a word
!     text           String variable containing some text
!     xp             x-coordinate in plotting units
!     xpboun         dummy
!     yp             y-coordinate in plotting units
!     ypboun         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF01         Translate user coordinates into plotter coordinates
!     PLOTTEXT1      Plot a text
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotintnumber' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotintnumber'
         write(irefwr,1) 'inumbr, ipos', inumbr, ipos
         write(irefwr,2) 'sizetxt', sizetxt
         write(irefwr,2) 'x, y, xshift, yshift', x, y, xshift, yshift
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute position in plotting units xp, yp

      call plaf01 ( x, y, xp, yp, xpboun, ypboun, iside )

      if ( jtest>0 .and. ioutsd>=2 ) go to 1000  ! plot outside window

!     --- Compute length of number and store number in a string

      write(text,100) inumbr
100   format ( i16 )
      text = adjustl(text)            ! remove leading blanks
      nchar = index ( text, ' ' )-1   ! number of non-blank characters

!     --- Plot number

      call plottext1 ( text, nchar, sizetxt, ipos, xp, yp, xshift, &
                       yshift, 0d0 )

1000  call erclos ( 'plotintnumber' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotintnumber'

      end if  ! ( debug )

      end
