      subroutine plotmarkelnod ( imarknodes, imarkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot information from arrays imarkelem and imarknodes
!     defining the status of elements and nodes in case of an obstacle
!     2d only
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     obstacle
!     2d
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imarknodes(*), imarkelem(*)

!     imarkelem      i    See subroutine mshshift2dcor
!     imarknodes     i    See subroutine mshshift2dcor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikader

!     ikader         Help variable to store jkader temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTMARKELS    Plot information from array imarkelem
!     PLOTMARKNODS   Plot information from array imarknodes
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPPLOTCROSSOB Plot the contents of the arrays icrossobst and rcrossobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotmarkelnod' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotmarkelnod'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      call sepflpointobst
      call sepflielemobst
      if ( kmeshad(3)==0 ) call sepfillkmad3

!     --- open plot

      jkader = -1
      textx = 'x'
      texty = 'y'
      call pl0003 ( ikader, 1,  15d0, 1d0, 0 )

!     --- plot elements

      call plotmarkels ( imarkelem )

!     --- plot nodes

      call plotmarknods ( imarknodes )

!     --- plot intersection points

      call sepplotcrossob

!     --- close plot

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotmarkelnod'

      end if  ! ( debug )
      call erclos ( 'plotmarkelnod' )

      end
