      subroutine plotmarknods ( imarknodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot information from array imarknodes
!     defining the status of nodes in case of an obstacle
!     2d only
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     obstacle
!     2d
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imarknodes(*)

!     imarknodes     i    See subroutine plotmarkelnods
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodp, iscolr
      double precision x, y

!     inodp          Nodal point number
!     iscolr         Help variable to store the value of dcolor temporarily
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotmarknods' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotmarknods'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iscolr = dcolor

      do inodp = 1, npoint

         if ( imarknodes(inodp)/=0 ) then

            x = coor(1,inodp)
            y = coor(2,inodp)
            if ( imarknodes(inodp)==-1 ) then
               dcolor = 20
            else
               dcolor = 3
            end if  ! ( imarknodes(inodp)==-1 )

            if ( debug ) write(irefwr,1) 'inodp, dcolor', inodp, dcolor
            call plafep ( 6, x, y, 0d0, 0d0, dcolor )

         end if  ! ( imarknodes(inodp)/=0 )

      end do  ! inodp = 1, npoint

      dcolor = iscolr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotmarknods'

      end if  ! ( debug )
      call erclos ( 'plotmarknods' )

      end
