      subroutine plotobstacle ( debuglc, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 25-11-2012 Long integers
!        version  2.1  date 11-10-2011 Extension with 1d
!        version  2.0  date 20-07-2011 Extra parameters debug, ichoice
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print and plot information about the obstacle depending on some
!     debug parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     plot
!     print
!     debug
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice
      logical debuglc

!     debuglc        i    If true debug statements are carried out otherwise
!                         they are not
!     ichoice        i    Integer choice parameter
!                         See subroutine mshlevelkm2ad
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOT2DOBST     Make two dimensional plot of obstacles
!     SEPPLOT3DOBST  Make three dimensional plot of obstacles
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize some parameters
!     Activate arrays
!     Compute starting addresses
!     Perform printing and plotting
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotobstacle' )
      debug = debuglc

!     --- Compute starting addresses

      if ( debug .or. printobst ) then

!     --- debug true, print ipointobst and ielemobst

         call sepprinipobst ( ipointobst )
         if ( ichoice==0 .or. ichoice>=2 ) call sepprinielobst ( ielemobst )

      end if

      if ( debug .or. plotobst ) then

!     --- plot information of obstacle

         select case ( ndim )

         case (2)

!        --- ndim = 2, open plot

            call plot2dobst ( ichoice )

         case (3)

!        --- ndim = 3

            call sepplot3dobst

         end select  ! case ( ndim )

      end if

      call erclos ( 'plotobstacle' )

      end subroutine plotobstacle
