      subroutine plotsurf3d ( isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 18-05-2005 Fortran 90
!        version  1.0  date 09-04-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot all elements in a three-dimensional surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     surface
!     element
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr

!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, istart

!     i              Counting variable
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH3DQ         plot one surface in 3-D (no hidden lines)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotsurf3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotsurf3d'

      end if
      if ( ierror/=0 ) go to 1000

      istart = 2*nsurfs
      do i = 1, isurnr-1
         istart = istart+kmeshn(i)*kmeshn(nsurfs+i)
      end do  ! i = 1, isurnr-1
      inpelm = kmeshn(isurnr)
      inelem = kmeshn(nsurfs+isurnr)
      jkader = -1
      irotat = 1
      call msh3dq ( isurnr, coor, kmeshn(istart+1), &
                    npoint, 15d0, 1d0, inpelm, inelem )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotsurf3d'

      end if
      call erclos ( 'plotsurf3d' )

      end subroutine plotsurf3d
