      subroutine plottime
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 12-09-2013 Skip if not final plot
!        version  1.1  date 09-06-2010 New format
!        version  1.0  date 14-02-2006
!
!   copyright (c) 2006-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a plot with a time label
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     timing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxchr, nchar, ktimes
      parameter (maxchr=20)
      character (len=maxchr) charac
      double precision xp, yp, sizetxt

!     charac         String variable to store a text
!     ktimes         mod(itimes,10)
!     maxchr         Maximum number of characters allowed in a string
!                    Length of string
!     nchar          Number of characters in a word
!     sizetxt        Size of text in cm's
!     xp             x-coordinate in cm's
!     yp             y-coordinate in cm's
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLCONVERTFLNUM Convert real number to text with isig digits behind
!                    decimal point
!                    Floating format
!     PLCONVERTNUM   Convert real number to text with isig digits behind
!                    decimal point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plottime' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plottime'
         write(irefwr,1) 'itimdep', itimdep
         write(irefwr,2) 't', t
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Skip in case not the final plot

      ktimes = mod(jtimes,10)
      if ( ktimes/=0 .and. ktimes/=3 ) go to 1000

      if ( itimdep/=0 ) then

!     --- Time dependent solution, plot time

         xp = 17d0          ! position of text
         yp = 0.3d0
         sizetxt = 0.25d0   ! size of text
         charac(1:4) = 't = '
         if ( abs(t)>0d0 .and. t<1d-3 .or. t>1d3 ) then

!        --- Plot in floating precision

            call plconvertflnum ( t, nchar, 3, charac(5:maxchr) )

         else

!        --- Plot in fixed precision
!            Convert number to text

            call plconvertnum ( t, 3, charac(5:maxchr), nchar, -1 )

         end if
         nchar = nchar+4

!        --- Plot

         call plafp7 ( xp, yp, charac, 0d0, sizetxt, nchar )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plottime'

      end if
      call erclos ( 'plottime' )

      end
