      subroutine plotzerolev2d ( coor, ilev, nmeshc, iobstfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-09-2013 Extra debug
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the edges (2d) of the zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ilev, nmeshc(*), iobstfaces(*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ilev           i    Sequence number of level being considered
!     iobstfaces     i    contains edge sequence numbers of boundary
!                         of levelset
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nnode_edge, jstart, numedges, iedge, kstart, &
              node1, node2

!     i              Counting variable
!     iedge          edge sequence number
!     jstart         starting position of edge
!     kstart         Start value in iobstfaces
!     nnode_edge     number of nodes corresponding to edge
!     node1          Node of edge
!     node2          Node of edge
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotzerolev2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotzerolev2d'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kstart = iobstfaces(ilev)
      numedges = iobstfaces(kstart)

      do i = 1, numedges

         iedge = iobstfaces(kstart+i)       ! edge number
         jstart = nmeshc(iedge)
         nnode_edge = nmeshc(jstart)
         node1 = nmeshc(jstart+1)
         node2 = nmeshc(jstart+nnode_edge)

         if ( debug ) then
            write(irefwr,1) 'iedge', i
            call prinin ( nmeshc(jstart+1), nnode_edge, 'nodes' )
            write(irefwr,2) 'coordinates', coor(:,node1), coor(:,node2)
         end if  ! ( debug )

         call plafep ( 4, coor(1,node1), coor(2,node1), 0d0, 0d0, 1 )
         call plafep ( 5, coor(1,node2), coor(2,node2), 0d0, 0d0, 1 )

      end do  ! i = 1, numedges

      if ( debug ) write(irefwr,1) 'nnode_edge', nnode_edge

1000  call erclos ( 'plotzerolev2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotzerolev2d'

      end if  ! ( debug )

      end
