      subroutine plpos3 ( ichois, sx, sy )
! ======================================================================
!
!        programmer    Jac Janssen
!        version  1.3  date 22-08-2013 New definition file ref numbers
!        version  1.2  date 16-05-2003 New scaling
!        version  1.1  date 28-06-1994
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine to plot symbol with center (x,y)
!    move with pen up or down to that point, depending on ichois
!    machine dependent subroutine
!    PostScript
! **********************************************************************
!
!                       KEYWORDS
!
!     machine_dependent
!     plot
!     postscript
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois
      real sx, sy

!     ichois         i    the pen is moved to x,y with pen down when
!                         1<=ichois<=10, and with pen up when 11<=ichois<=20
!                         the symbol to be printed is indicated by ichois
!                         (ichois<=10) or ichois-10 (ichois>10)
!     sx             i    Single precision value of x
!     sy             i    Single precision value of y
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsymb
      real size, xoffset, yoffset
      character * 1 textpn
      parameter ( size = 72/3.0, xoffset = 15.0, yoffset = 20.0 )

!     jsymb          Symbol number of symbol to be plotted
!     size           Parameter to scale from program units to cm's on the paper
!                    This parameter is defined by a parameter statement
!                    One postscript unit = 1/72 inch
!     textpn         Indicates whether the pen must move to the center of the
!                    symbol (m) or draw a line to this point (l)
!     xoffset        offset in x direction
!     yoffset        offset in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1757   Symbol number not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ichois.ge.1 .and. ichois.le.10 ) then

!     --- with pen down to centre of centered symbol

         textpn = 'l'
         jsymb = ichois
      else if ( ichois.ge.11 .and. ichois.le.20 ) then

!     --- with pen up to centre of centered symbol

         textpn = 'm'
         jsymb = ichois-10
      else
         call eropen ( 'plpos3' )
         call errint ( ichois, 1 )
         call errwar ( 1757, 1, 0, 0 )
         call erclos ( 'plpos3' )
         go to 1000
      end if

!     --- move to the centre of the centered symbol
!         with pen up (textpn = 'm') or pen down (textpn = 'l'):

      write ( irefps, 100 ) sx*size+xoffset, sy*size+yoffset, textpn
100   format ( 2(1x,f8.3),1x, a )

!     --- Plot correct symbol, depending on ichois

      go to ( 210, 220, 230, 240, 250, 260, 270, 280, 290, 300 ), jsymb

!     --- centered symbol 1, "plus"; ichois = 1 of 11:

210      write ( irefps, '('' plus'')' )
         go to 900

!        --- centered symbol 2, "asterisk"; ichois = 2 of 12:

220      write ( irefps, '('' asterisk'')' )
         go to 900

!        --- centered symbol 3, "hexagon"; ichois = 3 of 13:

230      write ( irefps, '('' hexagon'')' )
         go to 900

!        --- centered symbol 4, "cross"; ichois = 4 of 14:

240      write ( irefps, '('' cross'')' )
         go to 900

!        --- centered symbol 5, "delta"; ichois = 5 of 15:

250      write ( irefps, '('' delta'')' )
         go to 900

!        --- centered symbol 6, "square"; ichois = 6 of 16:

260      write ( irefps, '('' square'')' )
         go to 900

!        --- centered symbol 7, "diamond"; ichois = 7 of 17:

270      write ( irefps, '('' diamond'')' )
         go to 900

!        --- centered symbol 8, "4square"; ichois = 8 of 18

280      write ( irefps, '( 2(1x,f8.3), '' 4square'' )' ) &
                 sx*size+xoffset, sy*size+yoffset
         go to 900

!        --- centered symbol 9, "nabla"; ichois = 9 of 19:

290      write ( irefps, '('' nabla'')' )
         go to 900

!        --- centered symbol 10 "star"; ichois = 10 of 20:

300      write ( irefps, '( 2(1x,f8.3), '' star'' )' ) &
                sx*size+xoffset,  sy*size+yoffset
         go to 900

!        --- move back to the centre of the centered symbol with pen up ('m'):

900   write ( irefps, '( 2(1x,f8.3), '' centre'' )' ) &
              sx*size+xoffset, sy*size+yoffset
1000  end
