      subroutine plt304(i1,i2,i3,x1,x2,x3,y1,y2,y3,dx,dy,amag,bmag, &
                        nx,ny,xmin,ymin)
! ======================================================================
!
!        programmer    niek praagman
!        version  2.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.1  date 26-04-1989 New error criterion
!        version  2.0  date 10-11-1985
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to determine function values on an equidistant grid
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, nx, ny
      double precision amag(*), bmag(*), dx, dy, x1, x2, x3, xmin, y1, &
                       y2, y3, ymin

!     amag       i    array containing function values in the nodal points
!     bmag       o    array containing the values of the function
!                     to be plotted on a equidistant grid
!     dx         i    steplength in x-direction
!     dy         i    steplength in y-direction
!     i1         i    first node number of triangle to be considered
!     i2         i    second node number of triangle to be considered
!     i3         i    third node number of triangle to be considered
!     nx         i    number of equidistant elements in x-direction
!     ny         i    number of equidistant elements in y-direction
!     x1         i    x-coordinate of point i1
!     x2         i    x-coordinate of point i2
!     x3         i    x-coordinate of point i3
!     xmin       i    Minimum x-value of plotting area
!     y1         i    y-coordinate of point i1
!     y2         i    y-coordinate of point i2
!     y3         i    y-coordinate of point i3
!     ymin       i    Minimum y-value of plotting area
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih1, ih2, j, j1, j2
      double precision det, eps, eta, ski, xcoor, xmaxd, xmind, ycoor, &
                       ymaxd, ymind

!     det        determinant of triangle
!     eps        Local accuracy
!     eta
!     i          Counting variable
!     ih1        Help variable to store a constant temporarily
!     ih2        Help variable to store a constant temporarily
!     j          Counting variable
!     j1
!     j2
!     ski
!     xcoor
!     xmaxd
!     xmind
!     ycoor
!     ymaxd
!     ymind
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     607   area of triangle is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- determine eps

      eps=1d2*epsmac

!     --- is triangle o.k.?

      det=(x2-x1)*(y3-y1)-(y2-y1)*(x3-x1)
      if ( abs(det).le.0d0 ) then
          call eropen ( 'plt304' )
          call errint ( i1, 1 )
          call errint ( i2, 2 )
          call errint ( i3, 3 )
          call errsub ( 607, 3, 0, 0 )
          call erclos ( 'plt304' )
          go to 1000
      end if

!     --- determine the smallest quadrangle in which the triangle is
!            contained

      if (x1.lt.x2) then
          xmind=x1
          xmaxd=x2
      else
          xmind=x2
          xmaxd=x1
      end if
      if (x3.lt.xmind) xmind=x3
      if (x3.gt.xmaxd) xmaxd=x3

      if (y1.lt.y2) then
          ymind=y1
          ymaxd=y2
      else
          ymind=y2
          ymaxd=y1
      end if
      if (y3.lt.ymind) ymind=y3
      if (y3.gt.ymaxd) ymaxd=y3

!     --- consider for each point on the rectangular grid , whether
!            it is inside or outside the triangle
!            determine boundaries

      j1=nint((ymind-ymin-eps)/dy)+1
      j2=nint((ymaxd-ymin+eps)/dy)+1
      ih1=nint((xmind-xmin-eps)/dx)+1
      ih2=nint((xmaxd-xmin+eps)/dx)+1

      if ( j1.lt. 1 ) j1= 1
      if ( j1.gt.ny ) j1=ny
      if ( j2.lt. 1 ) j2= 1
      if ( j2.gt.ny ) j2=ny

      if ( ih1.lt. 1 ) ih1= 1
      if ( ih1.gt.nx ) ih1=nx
      if ( ih2.lt. 1 ) ih2= 1
      if ( ih2.gt.nx ) ih2=nx

      do j=j1,j2
          ycoor=ymin+(j-1)*dy
          do i=ih1,ih2
             xcoor=xmin+(i-1)*dx

             ski= ((xcoor-x1)*(y3-y1)-(ycoor-y1)*(x3-x1))/det
             if ( ski.lt.-eps ) goto 100
             eta=-((xcoor-x1)*(y2-y1)-(ycoor-y1)*(x2-x1))/det
             if ( eta.lt.-eps ) goto 100

             if ( (1d0-ski-eta).lt.-eps ) goto 100

!            --- point inside

             bmag(i+(j-1)*nx)=amag(i1)*(1d0-ski-eta)+ &
                              amag(i2)*ski+amag(i3)*eta

100      end do

      end do

1000  end
