      subroutine plt350 ( coor, procor, idrie, ndrie, heltri, &
                          line, nline, v, view, ihelp, nprcor, &
                          isitua )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.0  date 21-04-1995  Coordinate basis added, &
!                                      extra parameter)
!        version  3.1  date 26-01-1993  Small adjustment comments etc
!        version  3.0  date 27-12-1992  Removal of axis, remove opening
!                                      and closing of plot)
!        version  2.5  date 01-07-1992  Initialise plt and ten
!        version  2.4  date 27-09-1991  Plot of axis added
!        version  2.3  date 05-03-1991  Scaling with PL0000
!        version  2.2  date 24-01-1991  Correction for scaling
!        version  2.1  date 27-10-1990  Correction for factor=0
!        version  2.0  date 17-11-1989  New scan method
!        version  1.0  date 27-06-1987
!        version  1.1  date 17-02-1989  Deletion of superfl. parms
!
!   copyright (c) 1987-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to plot projection with hidden lines of 3-D objects
!    It has been assumed that the projection has been computed before
!    and that the opening of the plot has also been performed
!    This subroutine does not close the plot
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     hidden_line
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idrie(*), line(*), ihelp(*), ndrie, nline, nprcor, &
              isitua
      double precision coor(*), procor(nprcor,*), view(*), heltri(*), &
                       v(3)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     heltri  i    helparray with x-coordinates of triangles in projection
!                  plane
!     idrie   i    Array containing the triangles corresponding to the mesh
!                  sorted with respect to their first projection coordinate
!     ihelp   i    Array with sorted line numbers for x-coordinates in
!                  projection plane
!     isitua  i    Indicator coordinate basis
!                    1  :  x - y - z
!                    2  :  z - x - y
!                    3  :  y - z - x
!     line    i    array containing all the lines to be considered and
!                  probably to be drawn
!     ndrie   i    number of triangles
!     nline   i    number of lines
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     v       i    view point
!     view    i    helparray of 20 positions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision tst, ten, xa, ya, xb, yb, xma, xst, yst, xen, &
                       yen, xp, xl, xm
      integer istart, i, j, idr, iv, ivis, ikl, kp, lp, mp, ip, jp, &
                      iview

!     i       Counting variable
!     idr     loop variable
!     ikl     loop variable
!     ip      first node number of line to be considered
!     istart  loop variable
!     iv      loop variable
!     iview   number of actual line parts in array view
!     ivis    visibility indicator
!     j       Counting variable
!     jp      second node number of line to be considered
!     kp      Node in triangle
!     lp      Node in triangle
!     mp      Node in triangle
!     ten     end part of line in varibale t
!     tst     begin part of line in varibale t
!     xa      x-coordinate of starting point
!     xb      x-coordinate of end point
!     xen     x-coordinate
!     xl      x-coordinate of second triangle node
!     xm      x-coordinate of third triangle node
!     xma     local extremum
!     xp      x-coordinate of first triangle node
!     xst     x-coordinate
!     ya      y-coordinate of starting point
!     yb      y-coordinate of end point
!     yen     y-coordinate
!     yst     y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLAFEP  General SEPRAN low level plotting subroutine
!     PLT352  Cancel part of a segment of line that is not visible
!     PLT353  Determine whether part of a line is covered by a triangle
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all lines and check for each line whether one or more
!     triangles are in front of the line
!     Use the fact that lines and triangles are sorted with respect to
!     their first coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plt350' )
      tst = 0d0
      ten = 0d0
      jmax = 1

!     --- Plot with hidden lines

      istart = 1
      do 400 j = 1 , nline

!     --- Determine start and end point of line as sorted in ihelp

          i  = ihelp( j )
          ip = line(2*i-1)
          jp = line(2*i  )
          ivis = 0

!         --- Place whole line (in t-parameter) in array view

          call plt352(ivis, tst, ten, view, iview)

!         --- Determine projected points

          xa = procor(1,ip)
          ya = procor(2,ip)
          xb = procor(1,jp)
          yb = procor(2,jp)
          ikl = 0

!         --- Run through all triangles that are important

          do 380 idr = istart , ndrie
              kp = idrie(3*idr-2)
              lp = idrie(3*idr-1)
              mp = idrie(3*idr  )

!             --- Check whether triangles can be cancelled

              xma = heltri(idr)
              if ( xma.gt.xb ) goto 385

!             --- Check starting triangles

              if ( ikl.eq.0 ) then

!             --- Determine maximum of x-coor of triangle

                   xp = procor(1,kp)
                   xl = procor(1,lp)
                   xm = procor(1,mp)
                   xma = max(xp, xl, xm)
                   if ( xma.lt.xa ) then
                        istart = idr + 1
                   else
                        ikl = 1
                   end if
              end if
              tst = 0d0
              ten = 1d0

!             --- Check whether part of line is made invisible

              call plt353 ( v, ip, jp, kp, lp, mp, coor, procor, ivis, &
                            tst, ten, nprcor, isitua )

!             --- Check result, if nothing visible then ready

              if ( ivis.gt.2 ) then
                   call plt352(ivis, tst, ten, view, iview)
              else if ( ivis.eq.2 ) then
                   goto 400
              end if
380       continue
385       continue

!         --- Plot the iview parts of the line that are visible

          do 390 iv=1, iview
              tst = view(2*iv-1)
              ten = view(2*iv  )
              if ( (ten -tst).gt.5.d-3 ) then
                   xst = xa + tst * ( xb-xa )
                   yst = ya + tst * ( yb-ya )
                   call plafep(4, xst, yst, 0d0, 0d0, 0)
                   xen = xa + ten * ( xb - xa )
                   yen = ya + ten * ( yb - ya )
                   call plafep(5, xen, yen, 0d0, 0d0, 0)
              end if
390       continue
400   continue
      jmax = 0
      call erclos ( 'plt350' )
      end
