      subroutine pltjnx ( cmplst, pltlst, acnseg, ncomp, nextra, &
                          kextra, inc )
! ======================================================================
!
!
!        programmer     Jos van Kan
!        version 1.1    date   21-11-1995 Better error handling
!        version 1.0    date   20-01-1995
!
!
!
!   copyright (c) 1995-1995 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!       pltjnx(lr) (PeeL NeXt LayeR) is an auxiliary routine for plotting 3D
!       funtions on non convex regions. Basically, starting from a component
!       list cmplst with possible candidates for plottable segments it checks
!       which segments are actually plottable. (A segment is plottable if all
!       its ancestors have been plotted. See pltjbp for a definition of
!       ancestor). These segments are put in the pltlst (together with the
!       appropriate value of ilines) and are replaced in the cmplst by their
!       upper component(s). If such an upper component neighbours a segment
!       without lower component, this also is loaded into the cmplst.
!       Whether a segment has been loaded into the cmplst or the pltlst is
!       denoted in nextra : 10 <= nextra (1, iseg): iseg loaded into the cmplst.
!       10 <= nextra (2, iseg) : iseg loaded into the pltlst.
!       A very special case is a segment with forking endpoints (nextra mod 10 = &
!       2, 3 or 6). These have to be handled with care. Actually they have to be
!       loaded into the pltlst twice: once for their lower endpoints and once
!       for their higher endpoints. Which of the two cases applies can be seen
!       from nextra (2, iseg). Every time a segment iseg is loaded into the
!       pltlst nextra (2, iseg) is increased by 10.
! ********************************************************************
!
!                      KEYWORDS
!
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer cmplst(0:*), pltlst(0:*), acnseg(*), ncomp(2,*), &
              inc, nextra(2,*), kextra(2,2, *)

!     acnseg  i    array with accumulated segment numbers
!     cmplst i/o   list of Segment numbers and appropriate ilines (that
!                  is the number of the intersection line that this segment
!                  falls on) of possible candidates for plotting.
!     inc     i    1 = forward loading, -1 = backward loading (reverse = true)
!     kextra  i    array containing information about forked endings of the
!                  segments.
!     ncomp   i    arry containing lower (1, segnum) and upper (2, segnum)
!                  components of segment segnum.
!     nextra  i    array containing information about segment endings and
!                  whether segments have already been loaded into the
!                  complist(1, segnum) > 10 or have already been plotted
!                  (2, segnum) > 10.
!     pltlst  o    list of segment numbers and appropriate ilines that
!                  can be plotted.
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer plsptr, iseg

!     iseg    index in cmplst of segment about to be replaced. Cmplst
!             (iseg) contains the segment number, cmplst (iseg + 1)
!             the line number on which this segment lies.
!     plsptr  ?
! ********************************************************************
!
!                      SUBROUTINES CALLED
!
      logical pltjbl

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
!     PLTJBL  ?
!     PLTJRE  ?
! ********************************************************************
!
!                      I/O
!
! ********************************************************************
!
!                      ERROR MESSAGES
!
!    1262   empty plotlist.
! ********************************************************************
!
!                      PSEUDO CODE
!
!       Empty the pltlst
!       for all segments in the cmplst:
!       {if this segment is plottable then
!       Put it in the pltlst
!       Replace it in the cmplst}
!       for all segments in the pltlst
!       increase nextra (2, iseg) by 10
! ======================================================================
!
      call eropen ('pltjnx')
      pltlst (0) = 0
      plsptr = 1
!       Plotlist emptied
      iseg = 1
100   if (iseg.gt.cmplst (0)) then
         goto 110
      end if
!            CHECK PLOTTABILITY ISEG
      if (pltjbl (cmplst(iseg), cmplst (iseg + 1), ncomp, nextra, &
         kextra, acnseg, inc)) then

!       this segment is plottable and can be removed from the cmplst

         pltlst (plsptr) = cmplst (iseg)
         pltlst (plsptr + 1) = cmplst (iseg + 1)
         plsptr = plsptr + 2
         call pltjre (cmplst, iseg, acnseg, ncomp, nextra, inc)
      else if (((nextra (1, cmplst (iseg)).gt.11) .or. (nextra (2, &
       cmplst (iseg)).gt.1)) .and. (nextra (2, cmplst (iseg)) .lt. &
       10)) then

!       plottable, but can not be replaced yet. forked ending, lower
!       part plotted. Line number made negative to indicate this exceptional
!       case

         pltlst (plsptr) = cmplst (iseg)
         pltlst (plsptr + 1) = - cmplst (iseg + 1)
         plsptr = plsptr + 2
         iseg = iseg + 2
      else

!       not plottable. get next segment

         iseg = iseg + 2
      end if
      goto 100
110   continue
      pltlst (0) = plsptr - 1
      do 120 plsptr = 1, pltlst(0), 2
120   nextra (2, pltlst (plsptr)) = nextra (2, pltlst (plsptr)) + &
         10
      if (pltlst (0).eq.0) then

!     --- error: empty plotlist. This would result in an infinite loop

         call errsub (1262, 0, 0, 0)

!     --- Signify fatal error and exit

      end if
      call erclos ('pltjnx')
      end
