      subroutine pltjvs ( coor, fval, kmeshc, ielem )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.0  date 02-02-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   PLTJVS checks for a particular element, whether the linear triangle
!   of its function values is visible with respect to the direction of
!   the vision lines of the current parallell projection. It does so
!   by calculating the sign of the vector tripel product V.{(X_1 - X_0)
!   x (X_2 - X_0)}, in which V is the direction of the vision lines, &
!   X_0, X_1, and X_2 the vertices (in R^3) of the linear triangle
!   If the triangle is visible according to this criterion it is
!   plotted.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), ielem
      double precision coor (2,*), fval (*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     fval    i    function values in nodes
!     ielem   i    Element number
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i0, i1, i2, kofmesh
      double precision d01, d02, d03, d11, d12, d13, det, &
             x0, x1, x2, y0, y1, y2, z0, z1, z2, &
             r0, r1, r2, t0, t1, t2

!     d01     Difference vector between points 0-1
!     d02     Difference vector between points 0-1
!     d03     Difference vector between points 0-1
!     d11     Difference vector between points 2-1
!     d12     Difference vector between points 2-1
!     d13     Difference vector between points 2-1
!     det     array of length modecl x modecl containing the
!             transformation determinant
!     i0      Cornerpoint indices in coor
!     i1      Cornerpoint indices in coor
!     i2      Cornerpoint indices in coor
!     kofmesh Offset in kmeshc for this element
!     r0      Corner point rotated bottom triangle
!     r1      Corner point rotated bottom triangle
!     r2      Corner point rotated bottom triangle
!     t0      Corner point rotated bottom triangle
!     t1      Corner point rotated bottom triangle
!     t2      Corner point rotated bottom triangle
!     x0      Corner point of triangle
!             After projection xi, zi contain projected coordinates
!     x1      Corner point of triangle
!     x2      Corner point of triangle
!     y0      Corner point of triangle
!     y1      Corner point of triangle
!     y2      Corner point of triangle
!     z0      Corner point of triangle
!     z1      Corner point of triangle
!     z2      Corner point of triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLAFEP  General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltjvs' )

!     --- first we calculate the indices in coor and fval

      kofmesh = 3*(ielem - 1)
      i0 = kmeshc (kofmesh + 1)
      i1 = kmeshc (kofmesh + 2)
      i2 = kmeshc (kofmesh + 3)
      x0 = coor (1, i0) - xc
      x1 = coor (1, i1) - xc
      x2 = coor (1, i2) - xc
      y0 = coor (2, i0) - yc
      y1 = coor (2, i1) - yc
      y2 = coor (2, i2) - yc
      r0 = x0 * cth + y0 * sth
      r1 = x1 * cth + y1 * sth
      r2 = x2 * cth + y2 * sth
      t0 = -x0 * sth + y0 * cth
      t1 = -x1 * sth + y1 * cth
      t2 = -x2 * sth + y2 * cth
      d01 = r0 - r1
      d02 = t0 - t1
      d03 = fval (i0) - fval (i1)
      d11 = r2 - r1
      d12 = t2 - t1
      d13 = fval (i2) - fval (i1)
      det = scfi * (d02*d13 - d03*d12) + (d01*d13 - d03*d11) &
            + ssfi * (d01*d12 - d02*d11)
      if (det.le.0) then
         x0 = r0 + scfi * t0
         z0 = ssfi * t0 + zscale * fval (i0)
         x1 = r1 + scfi * t1
         z1 = ssfi * t1 + zscale * fval (i1)
         x2 = r2 + scfi * t2
         z2 = ssfi * t2 + zscale * fval (i2)
         call plafep (4, x0, z0, 0d0, 0d0, 0)
         call plafep (5, x1, z1, 0d0, 0d0, 0)
         call plafep (5, x2, z2, 0d0, 0d0, 0)
         call plafep (5, x0, z0, 0d0, 0d0, 0)
      end if
      call erclos ( 'pltjvs' )
      end
