      subroutine sepplotfunc2 ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot function along curve or line
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     plot_function
!     driving_subroutine
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotfn
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, input(2), colours(numvecs)
      character (len=80) :: textleg(numvecs)

!     colours        Contains the sequence numbers of the colors to be plotted
!                    they are not
!     i              Counting variable
!     input          Work array to store input for subroutine plotfunc2
!     textleg        Contains the texts to be plotted in the legenda
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTFUNC2      Plot one or more functions in a function plot
!     PRFILLTEXTLEG  Fill array textleg with the text to be used in the legenda
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotfunc2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplotfunc2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill text legenda

      call prfilltextleg ( textleg, inpsep, numvecs, inpsep(istartvecs) )

!     --- plot

      do i = 1, numvecs
         colours(i) = 1+5*i
      end do  ! i = 1, numvecs
      input(1) = 0  ! ncolors
      input(2) = 0  ! nlegenda
      if ( numvecs>1 ) then
         input(1) = numvecs
         input(2) = numvecs
      end if  ! ( numvecs>1 )

      call plotfunc2 ( nnodes, funcx, funcy, numvecs, &
                       textaxis, input, colours, textleg )

!     --- deallocate funcx

      deallocate ( funcx, funcy, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'funcx' )

1000  call erclos ( 'sepplotfunc2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplotfunc2'

      end if  ! ( debug )

      end subroutine sepplotfunc2
