      subroutine sepplotm3 ( iplots, iskpar, plotfm, yfact, rplots )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.10 date 29-05-2014 Extension of timebefore
!        version  3.9  date 07-02-2014 Jump in case of errors
!        version  3.8  date 11-09-2013 Jump in case of errors
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to plot a 3-D mesh with hidden lines removed
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     plot_mesh
!     hidden_line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplots(*), iskpar(*)
      double precision plotfm, yfact, rplots(3)

!     iplots         i    Integer input array for plot subroutine
!                          1:  Position of last item that has been filled by
!                              the user
!                          6:  Plot outer boundaries double (1) or not (0)
!                              In case of coloured 3D plot: plot boundaries of
!                              elements
!                          7:  Skip element groups (>0) or not (0)
!                          8:  Plot renumbered node numbers (1) or original
!                              ones (0)
!                              Default: 0
!                          9:  Make a filled coloured mesh plot (1) or a line
!                              plot (0)
!                              Default: 0
!                         10:  Plot axis (1) or not (0)
!                              Default: 0
!                         11:  Type of projection. Possible values:
!                               0 : perspective projection plot
!                               1 : parallel projection plot
!                              10 : perspective projection plot
!                                   (View point automatic)
!                              11 : parallel projection plot
!                                   (View point automatic)
!                              Default: 10
!                         12:  Colour number of farthest elements
!                              Default: 1
!                         13:  Colour number of nearest elements
!                              Default: 16
!                         14:  Basic coordinate system for plots
!                                1 :  x - y - z
!                                2 :  z - x - y
!                                3 :  y - z - x
!                         15:  numrotations:  Number of rotations
!                              Default value: 1
!                         16:  Use volumes (>0) or not (0)
!                              Default: 0
!                         17:  Plot elements per layer (1) or not (0)
!                         18:  idefstart, defines how the start level must
!                              be computed
!                              1: Start with set of nodes with largest
!                                 distance to view point
!                              2: Start with set of surfaces indicated in
!                                 positions 12 and 13
!                         19:  ifirst, first surface number
!                         20:  ilast, last surface number
!     iskpar         i    array of length NELGRP that is only used if ISKIP>0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     plotfm         i    Plotting factor. When PLOTFM<0, then the length of
!                         the plot is computed by - (xmax-xmin) * PLOTFM
!                         and the width by - (ymax-ymin) * YFACT * PLOTFM
!                         else the length of the plot is equal to PLOTFM cm
!                         When by some mistake the user has given | PLOTFM | a
!                         value smaller than 1 cm, then the length is chosen
!                         equal to the default length stored in common block
!                         consta
!     rplots         i    Real input array for plot subroutine
!     yfact          i    Multiplication for the y-co-ordinates. With this
!                         parameter the user can influence the scale in
!                         y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lentria, npunt, &
              iresul(5), ichoic, iskip, nelgrpsav, nelemsav, &
              nomber(2), neleml, ichrec, special, jtimessav, &
              irotation, jv, i, numrotations, ilevel, &
              iinput(4), nlevels
      integer, allocatable :: elements(:), elmlevels(:)
      integer, allocatable :: itria(:)
      logical compl, uselevels
      double precision viewpoint(3), viewp(3)
      double precision, pointer :: coortmp(:,:), procor(:,:)
      double precision :: timebefore(2)

!     compl          Dummy parameter
!     coortmp        Copy of coor that may be changed
!     elements       Array of length levels(nlevels) (<=nelem)
!                    containing the reordered element sequence numbers
!                    So elements(1) contains the first element number to
!                    be used, elements(2) the second one and so on
!     elmlevels      Array of length nlevels (<nelem) containing the
!                    levels in array elements
!                    The first level consists of the elements 1 to
!                    elmlevels(1) of array elements
!                    The second level contains the elements  elmlevels(1)+1
!                    to elmlevels(2) and so on
!     i              Counting variable
!     ichoic         Choice parameter. With this parameter it is defined which
!                    actions will be performed. Possible values:
!                    1:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex only the real or
!                        imaginary part is used
!                    2:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex the copied solution
!                        vector is also complex
!                    3:  Create a submesh consisting of the surface elements
!                        only of each element only the vertices are stored,
!                        hence the final grid consists of a grid of 3-point
!                        triangles or 4-point quadrilaterals only
!                        This possibility only uses KMESH
!                        It has only been implemented for 3D
!                    4:  Combination of 1 and 3, hence possibly 2 meshes may be
!                        created
!                    5:  Combination of 2 and 3, hence possibly 2 meshes may be
!                        created
!     ichrec         Reduced value of ichois, i.e. without the part iorient
!     iinput         integer input array for subroutine TOPLEVELS
!     ilevel         dummy
!     iresul         Output array of length 5,1 if ichois=2 and otherwise of
!                    length 5,3
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                    (ichois=1,3)
!                    Pos .,3 corresponds to the solution array (ichois=1,3)
!                    Pos. 1,1  INDVER  sequence number with respect to INFOR
!                                      for the "vertex" mesh array
!                    Pos. 2,1  NPELM   number of points per element for this
!                                      mesh
!                    Pos. 3,1  NELEM   number of points per element for this
!                                      mesh
!                    Pos. 4,1  NEWVER  indicates if the mesh is new (1) or is
!                                      in fact the standard mesh (0)
!                    Pos. 5,1  INDLIN  sequence number with respect to INFOR
!                                      for the "line"-elements array
!     irotation      Loop variable for rotation
!     iskip          Dummy parameter
!     itria          Contains the triangles
!     jtimessav      Help variable to save the value of jtimes
!     jv             Indicates if the view point has been filled (1)
!                    or not (0)
!     lentria        Length of array itria
!     neleml         Number of line elements
!     nelemsav       Help parameter to save the old value of nelem
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     nlevels        Number of entries in array elmlevels
!     nomber         Help array to store relative number of unknown considered
!     npunt          total number of (possible) points
!     numrotations   Number of rotations
!     procor         coordinate array for projected points
!                    procor(1,i) contains the x-coordinate of the
!                    projected point
!                    procor(2,i) contains the y-coordinate of the
!                    projected point
!                    procor(3,i) contains the distance of point to the
!                    view-point i is the point number
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a coloured
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a coloured
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
!     uselevels      If true plotting per level must be performed
!     viewp          Array containing the first view point
!     viewpoint      Array containing actual view point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PLFILLVIEWP    Fill array viewpoint depending on the rotation
!     PLTHID         Plot 3D mesh with hidden lines
!     PRININ         print 1d integer array
!     TOPELEMSKIP    Fill array elemskip with information about elements to be
!                    skipped and elements to be plotted
!     TOPLEVELS      Reorder a restricted set of elements in an ordered
!                    level set
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Subdivide mesh into surface mesh consisting of triangles and
!     quadrilaterals only
!     Compute number of points for hidden lines routine
!     Check whether line elements are present
!     Make space reservation for the work arrays of PLTHID
!     Copy array coor in case of quadrilaterals to help array
!     Set pointers in ibuffr
!     Copy mesh
!     Make plot with hidden lines
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotm3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplotm3'
         write(irefwr,1) 'nelem, npoint', nelem, npoint
         call prinin ( iplots, 20, 'iplots' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      nelgrpsav = nelgrp
      nelemsav = nelem

      if ( itime>0 ) call ersettime ( timebefore )

!     --- Subdivide mesh into surface mesh consisting of triangles and
!         quadrilaterals only

      ichoic = 3
      ichrec = iplots(11)
      iorien = iplots(14)
      numrotations = iplots(15)
      if ( iplots(16)>0 ) then
         iskip = -1
      else if ( iplots(7)>0 ) then
         iskip  = 1
      else
         iskip  = 0
      end if
      if ( debug ) write(irefwr,1) 'ichrec', ichrec
      compl = .false.
      nomber(1) = 1
      nomber(2) = 1
      special = 0
      uselevels = iplots(17)>0
      nlevels = 1

      if ( uselevels ) then

!     --- Plot per level
!         Call subroutine toplevels to compute the levels

         allocate ( elemskip(nelem), elmlevels(nelem), &
                    elements(nelem), stat = error )
         if ( error/=0 ) call eralloc ( error, 3*nelem, 'elemskip' )
         if ( ierror/=0 ) go to 1000

!        --- Actual call

         iinput(1) = iplots(7)
         iinput(2) = iplots(18)
         iinput(3) = iplots(19)
         iinput(4) = iplots(20)
         call toplevels ( rplots(3), iskip, elements, elmlevels, &
                          nlevels, iinput )
         if ( ierror/=0 ) go to 1000
         length = elmlevels(1)+nlevels

!        --- Create array elemskip

         call topelemskip ( nlevels, elmlevels, elements )
         if ( ierror/=0 ) go to 1000

         deallocate ( elmlevels, elements, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'elmlevels' )

      end if
      jtimessav = jtimes

!     --- Loop over levels

      do ilevel = 1, nlevels

         if ( debug ) write(irefwr,*) 'before topqu0'
         call topqu0 ( ichoic, nomber, iskip, iskpar, iresul, compl, &
                       0, special, ilevel )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,*) 'after topqu0'
         if ( ierror/=0 ) go to 1000
         inpelm = iresul(2)
         nelemlc  = iresul(3)
         if ( inpelm==3 ) then
            npunt = npoint
         else
            npunt = npoint + nelemlc/2
         end if

!        --- Line elements ?

         if ( iresul(5)/=0 ) then

!        --- line elements

            neleml = size ( kmeshcline ) / 2

         else

!        --- no line elements

            neleml = 0

         end if

!        --- Array itria

         if ( inpelm==3 ) then
            lentria = nelemlc * 3
         else
            lentria = nelemlc * 6
         end if

         allocate ( itria(lentria), procor(3,npunt), coortmp(3,npunt), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, lentria+6*npunt, 'itria' )
         if ( ierror/=0 ) go to 1000

!        --- Copy array coor ( quadrilaterals only )

         procor(:,1:npoint) = coor

!        --- Copy mesh

         if ( iresul(1)>0 ) then
            length = size(kmeshcsub)
            itria(1:length) = kmeshcsub
         else
            length = inpelm*nelemlc
            itria(1:length) = kmeshc(1:length)
         end if  ! ( iresul(1)>0 )

!        --- Make plot

         jv = 1 - ichrec / 10
         if ( debug ) write(irefwr,1) 'jv', jv

!        --- Loop over the rotations

         if ( numrotations>1 .or. nlevels>1 ) then

!        --- More than one rotation, separate pictures required

            if ( jtimes>10 ) jtimes = jtimes-10
            if ( jtimes==2 .or. jtimes==3 ) then
               jtimes = 3
            else
               jtimes = 0
            end if

         end if

         do irotation = 1, numrotations

            if ( numrotations==1 .and. ilevel==nlevels ) then
               jtimes = jtimessav
            else if ( irotation==numrotations .and. &
                      jtimessav>=10 .and. ilevel==nlevels ) then
               jtimes = 11
            end if
            if ( irotation==1 .and. jv==1 ) viewpoint(1:3) = rplots(1:3)
            coortmp(1:3,1:npoint) = coor

            if ( debug ) write(irefwr,*) 'before plfillviewp'
            call plfillviewp ( irotation, jv, viewp, viewpoint, iorien, &
                               numrotations )

            if ( irotation>1 ) then

!           --- Next rotation, adapt viewp

               if ( ichrec>=10 ) ichrec = ichrec-10

!              --- Copy array coor ( quadrilaterals only )

               if ( inpelm==4 ) procor(:,1:npoint) = coor

!              --- Copy mesh

               length = inpelm*nelemlc
               itria(1:length) = kmeshcsub(1:length)

            end if

            if ( debug ) write(irefwr,*) 'before plthid'
            call plthid ( coortmp, npoint, nelemlc, inpelm, itria, plotfm, &
                          yfact, viewpoint, procor, ichrec, &
                          neleml, kmeshcline, iorien )
            if ( debug ) write(irefwr,*) 'after plthid'
            if ( ierror/=0 ) go to 1000
            if ( irotation==1 .and. jv==0 ) viewp(1:3) = viewpoint(1:3)

         end do

      end do

!     --- Delete help arrays

      deallocate ( itria, procor, coortmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'itria' )
      if ( ierror/=0 ) go to 1000

      if ( iresul(4)==1 ) deallocate ( kmeshcsub )
      if ( iresul(5)/=0 ) deallocate ( kmeshcline )
      if ( uselevels ) deallocate ( elemskip )
      if ( jtimes/=2 ) jtimes = jtimessav

      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplotm3'

      end if
      call erclmn ( 'sepplotm3', timebefore )

      end subroutine sepplotm3
