      subroutine to0029 ( itypvc, ichw, ich1, ich2, &
                          vectr1, vectr2, coor, coorc, iwork, npoint, nelem, &
                          nunkp1, nunkp2, ikprf1, ikprf2, &
                          kpref1, kpref2, sch )
! ======================================================================
!
!        programmer    jaap van der zanden
!        version  1.5  date 27-04-1993 Improve minor error
!        version  1.4  date 27-11-1992 Removal of cconst
!        version  1.3  date 23-06-1992 avoid undefined iwork(* &
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   construct vector from vectr1,vectr2 for plotting for vector to be
!   plotted
!   (ich1 from vectr1 together with degree of freedom ich2 of vectr2
!    form the vector)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypvc,ichw,ich1,ich2,iwork(*),npoint,nelem,nunkp1,nunkp2, &
              ikprf1,ikprf2, &
              kpref1(*), kpref2(*)
      double precision vectr1(*), vectr2(*), coor(*), coorc(*), sch

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     coorc   i    array with coordinates of centroid of elements
!     ich1    i    degree of freedom ich1 from vectr1 together with
!     ich2    i    degree of freedom ich2 from vectr2 form
!                  the vector to be plotted
!     ichw    i    ichw=0 iwork should not be used
!     ikprf1  i    kprf1=0 number of degrees of freedom is constant (nunkp1)
!                       .ne.0  degrees of freedom from kpref1
!     ikprf2  see ikprf1
!     itypvc  i    Indication of the type of solution vector:
!                  110: solution vector
!                  115, 116: vectors of special structure
!     iwork   i    Work space to store Information about points
!     kpref1  i    array with degrees of freedom of vectr1 (ikprf1.ne.0)
!     kpref2  i     array with degrees of freedom of vectr2 (ikprf2.ne.0)
!     nelem   i    Number of elements
!     npoint  i    Number of nodal points
!     nunkp1  i    ?
!     nunkp2  i    ?
!     sch     i    scale factor to be applied to constructed vector
!     vectr1  i   Array with first component of  vector to be plotted
!     vectr2  i   Array with second component of  vector to be plotted
!                 standard sepran vectors (type 110,115) defined
!                 in any nodal point, or (type=116) per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idegfd, ih, j1, j2, nusol, nusol1, nusol2
      double precision v1, v2, x, y

!     i       Counting variable
!     idegfd  Degree of freedom that must be used
!     ih      Help variable to store a constant temporarily
!     j1      ?
!     j2      ?
!     nusol   Pointer to unknowns in point i
!     nusol1  ?
!     nusol2  ?
!     v1      first component of vector to be plotted in (x,y)
!     v2      second component of vector to be plotted in (x,y)
!     x       x_1 co-ordinate of point
!     y       x_2 co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
!     PLAFEP  General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
!    vector to be plotted by calling plafep
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     216     degree of freedom does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- tests

      call eropen( 'to0029' )
      if ((ikprf1.eq.0) .and. (ich1.gt.nunkp1)) then
         call errsub ( 216, 0, 0, 0)
         goto 2001
      end if
      if ((ikprf2.eq.0) .and. (ich2.gt.nunkp2)) then
         call errsub ( 216, 0, 0, 0)
         goto 2001
      end if
      j1=1
      j2=1
      if (itypvc.eq.110 .or. itypvc.eq.115) then
         do 1000 i=1,npoint

!        --- find degree of freedom in vectr1 in point i

            if (ichw.ne.0) then
               if (iwork(i).eq.-1) goto 1000
            end if
            if (ikprf1.eq.0) then
               idegfd=(i-1)*nunkp1+ich1
            else
               nusol=kpref1(i)
               nusol1=kpref1(i+1)
               idegfd=nusol+ich1
               if (idegfd.le.nusol1) then
                  j1=1
               else
                  j1=0
               end if
            end if
            if (j1.eq.1) then
               v1=vectr1(idegfd)*sch
            end if
            if (ikprf2.eq.0) then
               idegfd=(i-1)*nunkp2+ich2
            else
               nusol=kpref2(i)
               nusol2=kpref2(i+1)
               idegfd=nusol+ich2
               if (idegfd.le.nusol2) then
                  j2=1
               else
                  j2=0
               end if
            end if
            if ((j1.eq.1).and.(j2.eq.1)) then
               v2=vectr2(idegfd)*sch
               ih=i+i
               x=coor(ih-1)
               y=coor(ih)

!              --- plot vector

               call plafep(8,x,y,x+v1,y+v2,0)
            end if
1000     continue
      else if (itypvc.eq.116) then
         do 2000 i=1,nelem

!        --- find degree of freedom in vectr1 in element i

            idegfd=(i-1)*nunkp1+ich1
            v1=vectr1(idegfd)*sch
            idegfd=(i-1)*nunkp2+ich2
            v2=vectr2(idegfd)*sch
            ih=i+i
            x=coorc(ih-1)
            y=coorc(ih)
            if ( x.ge.xmin .and. x.le.xmax .and. &
                 y.ge.ymin .and. y.le.ymax ) then

!           --- plot vector

               call plafep(8,x,y,x+v1,y+v2,0)
            end if
2000     continue
      end if
2001  call erclos( 'to0029' )
      end
