      subroutine creatv116const ( vector, inpcre, rincre, jpstin, kdeg, &
                                  kfun, nphys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill vector of type 116 with a constant
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpcre(*), jpstin, kdeg, kfun, nphys
      double precision vector(nphys,*), rincre(*)

!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
!     vector        i/o   Vector to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifirst, last
      double precision consta

!     consta         constant to be filled
!     ifirst         First element number
!     last           Last element number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!    1678   For vectors of special structure defined per element, only the
!           filling of a constant over a complete region has been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv116const' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv116const'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set first and last element number

      ifirst = inpcre(jpstin)
      last   = inpcre(jpstin+1)

      if ( kfun>0 ) then

!     --- Function has not yet been implemented

         call errsub ( 1678, 0, 0, 0 )
         go to 1000

      end if

      if ( kdeg>nphys ) then

!     --- kdeg too large

         call errint ( kdeg, 1 )
         call errint ( nphys, 2 )
         call errsub ( 218, 2, 0, 0 )
         go to 1000

      end if

!     --- kfun<0, function is constant

      consta = rincre(-kfun)

      if ( kdeg==0 ) then

!     --- All degrees of freedom

         vector(1:nphys,ifirst:last) = consta

      else

!     --- One degree of freedom

         vector(kdeg,ifirst:last) = consta

      end if  ! ( kdeg==0 )

1000  call erclos ( 'creatv116const' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End creatv116const'

      end if  ! ( debug )

      end
