      subroutine creatv116func ( vecold, ielem1, ielem2, maxphys, mvec, &
                                 kfun, vector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-12-2011 New parameter list
!        version  1.0  date 11-08-2009
!
!   copyright (c) 2009-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill special array of type 116 with special function
!     Not all posibilities have been programmed yet
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem1, ielem2,maxphys, mvec, kfun
      double precision vector(*), vecold(*)

!     ielem1         i    First element number in group
!     ielem2         i    Last element number in group
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     maxphys        i    Maximum number of unknowns per point
!     mvec           i    Number of old vectors to be used in FUNCVECT
!     vecold         i    Input vector
!     vector        i/o   Output vector of type 116, i.e. defined per
!                         element
!                         Values will be filled in this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numelem
      double precision, allocatable :: coorlc1(:,:), usolloc(:)

!     coorlc1        Array with the coordinates corresponding to the
!                    elements ielem1 to ielem2
!     numelem        Number of elements in this group
!     usolloc        Array containing the solution vector defined per element
!                    and restricted to this group only
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CREATFILLCENTR Fill coordinates of centre of elements in array coorlc1
!     CREATMEANVAL   Compute mean value of scalar in element and store
!                    in usolloc
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCVECT       User subroutine to fill a vector as function of old vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3022   mvec>1, or nphys>1 or indprf>0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv116func' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv116func'
         write(irefwr,1) 'ielem1, ielem2', ielem1, ielem2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1100

!     --- Compute some parameters

      numelem = ielem2-ielem1+1

!     --- allocate space for work arrays

      allocate ( coorlc1(ndim,numelem), usolloc(nelem*mvec*maxphys), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, ndim*numelem+nelem*mvec*maxphys, &
                       'coorlc1' )

!     --- Put coordinates of centroids of elements in array coorlc1

      call creatfillcentr ( coorlc1, ielem1, ielem2 )

!     --- Compute values of old vectors in centroid and store in usolloc
!         At this moment it is implemented for mvec=1, maxphys = 1 and indprf=0
!         only

      if ( mvec>1 .or. maxphys>1 ) then

!     --- mvec>1 or maxphys>1 or indprf#0
!         not yet implemented

         call errint ( mvec, 1 )
         call errint ( maxphys, 2 )
         call errsub ( 3022, 2, 0, 0 )
         go to 1000

      end if  ! ( mvec>1 .or. maxphys>1 )

!     --- Compute mean value

      call creatmeanval ( vecold, usolloc, ielem1, ielem2 )

!     --- Fill output array

      call funcvect ( kfun-3000, ndim, coorlc1, &
                      numelem, usolloc, mvec, &
                      vector(ielem1), maxphys )

1000  deallocate ( coorlc1, usolloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorlc1' )

1100  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End creatv116func'

      end if  ! ( debug )
      call erclos ( 'creatv116func' )

      end
