      subroutine er0011 ( usol, ichoice, isrf1, isrf2, idegfd, jstap, &
                          value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.3  date 05-12-2012 Fortran 90
!        version  4.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  4.1  date 10-04-1990 Error correction error 273
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for surfaces    one component only
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer ichoice, isrf1, isrf2, idegfd, jstap
      double precision usol(*), value

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichoice        i    Choice parameter, see bvalue
!     idegfd         i    Degree of freedom sequence number, see bvalue
!     isrf1          i    First surface number
!     isrf2          i    Last surface number
!     jstap          i    see bvalue
!     usol          i/o   Solution vector
!     value          i    see bvalue
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ifirst, iknpt, inpelm, nodenr, istap, &
              istart, j, jnelem, jnpelm, knpelm, iwork(npoint), inusol

!     i              Counting variable
!     ielem          Element number
!     ifirst
!     iknpt          Nodal point number
!     inpelm         number of points per element
!     inusol         Position in usol
!     istap
!     istart         First position used at a part
!     iwork          integer work array of length npoint
!     j              Counting variable
!     jnelem         number of elements in an element group for one super
!                    element
!     jnpelm         Number of nodes in an element
!     knpelm
!     nodenr         Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINIBVL      actual filling of essential boundary condition for one point
!                    and one degree of freedom only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- input / output

      if ( isrf1<=0 .or. isrf1>nsurfs ) then
         call errint ( isrf1, 1 )
         call errint ( 1, 2 )
         call errint ( nsurfs, 3 )
         call errsub ( 185, 3, 0, 0 )
         go to 1000
      end if  ! ( isrf1<=0 .or. isrf1>nsurfs )
      if ( isrf2<=0 .or. isrf2>nsurfs ) then
         call errint ( isrf2, 1 )
         call errint ( 1, 2 )
         call errint ( nsurfs, 3 )
         call errsub ( 186, 3, 0, 0 )
         go to 1000
      end if  ! ( isrf2<=0 .or. isrf2>nsurfs )

!     --- clear array iwork

      iwork=0

!     --- compute inpelm

      inpelm=kmeshn(isrf1)
      if(isrf2>isrf1) then

!     --- isrf2>isrf1  check inpelm

        do j=isrf1,isrf2
           if ( inpelm/=kmeshn(j) ) then
              call errint ( isrf1, 1 )
              call errint ( isrf2, 2 )
              call errint ( isrf1, 3 )
              call errint ( inpelm, 4 )
              call errint ( j, 5 )
              call errint ( kmeshn(j), 6 )
              call errsub ( 272, 6, 0, 0 )
           end if  ! ( inpelm/=kmeshn(j) )
        end do
      end if

!     --- compute the number of elements  inelem

      inelem=0
      do j=isrf1,isrf2
         inelem=inelem+kmeshn(nsurfs+j)
      end do

!     --- compute starting position in kmeshn

      istart=2*nsurfs
      do j=1,isrf1-1
         jnpelm=kmeshn(j)
         jnelem=kmeshn(nsurfs+j)
         istart=istart+jnpelm*jnelem
      end do

!     --- check jstap

      ifirst=1
      istap=1
      knpelm=inpelm
      if(inpelm==10) then

!     --- inpelm = 10  triangle with 10 points

         if(jstap==1) then
            istap=3
            knpelm=7
         else if (jstap==2 .or. jstap==5) then
            goto 300
         else if (jstap==3) then
            ifirst=10
         else
            istap=3
         end if
      else
      if(jstap==1 .or. jstap==2 .or. jstap==4 .or.jstap==5) &
         istap=2
      if(jstap==1) knpelm=min(7,inpelm)
      if(jstap==2 .or. jstap==5) ifirst=2
      if(jstap==3) ifirst=inpelm
      if(jstap==5) knpelm=10
      if(jstap>=3 .and. inpelm<7) call ertrap( &
          'bvalue  (er0011)',273,1,0,0,0)
      end if
      knpelm=min(knpelm,inpelm+1)
      do ielem=1,inelem
         do i=ifirst,knpelm,istap
            iknpt=min(i,inpelm)
            nodenr=kmeshn(istart+iknpt)
            if(iwork(nodenr)==0) then

!           --- iwork(nodenr) = 0:  new nodal point

               iwork(nodenr)=1
               inusol = nodenr
               if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
               if ( inusol>0 ) &
                  call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                   value, compl )
            end if
         end do
         istart=istart+inpelm
      end do
      return

!     --- triangle with 10 points  (mid points)
!         jstap=2 or jstap=5

300   do ielem=1,inelem
         do i=2,9,3
            do j=0,1
               iknpt=i+j
               nodenr=kmeshn(istart+iknpt)
               if(iwork(nodenr)==0) then

!              --- iwork(nodenr) = 0:  new nodal point

                  iwork(nodenr)=1
                  inusol = nodenr
                  if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
                  if ( inusol>0 ) &
                     call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                      value, compl )
               end if
            end do
         end do
         if(jstap==5) then
            iknpt=10
            nodenr=kmeshn(istart+iknpt)
            if(iwork(nodenr)==0) then

!           --- iwork(nodenr) = 0:  new nodal point

               iwork(nodenr)=1
               inusol = nodenr
               if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
               if ( inusol>0 ) &
                  call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                   value, compl )
            end if
         end if
         istart=istart+inpelm
      end do
1000  end
