      subroutine er0017 ( irows, icols, amat, x, y, nusol, nbound, &
                          isign )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 21-04-2015 New parameter list
!        version  4.0  date 08-11-1996 Extra parameter ibcmat (BvR)
!        version  3.0  date 06-01-1995 Extra parameter isign
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     incorporate effect of boundary conditions
!     Matrix real, vectors real
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irows(*), icols(*), nusol, nbound, isign
      double precision amat(*), x(*), y(*)

!     amat           i    part 2 of large matrix corresponding to boundary
!                         conditions see pg -13.6.2
!     icols          i    Contains column numbers of non-zero elements
!                         of matrix in sequence defined by irows
!     irows          i    Contains pointers to rows in matrix and icols
!                         irows(1) = 0
!                         number of non-zeros per row is irows(i+1)-irows(i)
!     isign          i    Multiplication sign, if 1 the effect is added to the
!                         existing vector, if -1 it is subtracted
!     nbound         i    Number of essential boundary conditions
!     nusol          i    Total number of unknowns
!     x              i    input vector to be multiplied
!     y             i/o   The vector y should exist at input
!                         Depending on isign the effect of the multiplication is
!                         added to or subtracted from the original vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jk

!     i              Counting variable
!     j              Counting variable
!     jk             Help variable to store an index
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0017         print matrix corresponding to boundary conditions
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0017' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0017'
         write(irefwr,1) 'nusol, nbound, isign', nusol, nbound, isign
         call prinrl ( x(nusol-nbound+1), nbound, 'x-in')
         call prinrl ( y, nusol, 'y-in')
         write(irefwr,*) 'amat'
         call pr0017 ( amat, irows, icols, nbound, nusol )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nbound
         do j = irows(i)+1, irows(i+1)
           jk = icols(j)
           y(jk) = y(jk)+isign*amat(j)*x(nusol-nbound+i)
         end do  ! j = irows(i)+1, irows(i+1)
      end do  ! i = 1, nbound

1000  call erclos ( 'er0017' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( y, nusol, 'y-out')
         write(irefwr,*) 'End er0017'

      end if  ! ( debug )

      end subroutine er0017
