      subroutine er0036 ( usol, ichoice, ipnt1, ipnt2, idegfd, value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-01-2019
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 17-07-1987
!
!   copyright (c) 1987-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for nodal points     one component only
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer ichoice, ipnt1, ipnt2, idegfd
      double precision usol(*), value

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichoice        i    Choice parameter, see bvalue
!     idegfd         i    Degree of freedom sequence number, see bvalue
!     ipnt1          i    First node number
!     ipnt2          i    Last node number
!     usol          i/o   Solution vector
!     value          i    see bvalue
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodenr, inusol

!     inusol     Position in usol
!     nodenr     Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINIBVL      actual filling of essential boundary condition for one point
!                    and one degree of freedom only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0036' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0036'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( ipnt1<=0 .or. ipnt1>npoint ) then
         call errint ( ipnt1, 1 )
         call errint ( npoint, 2 )
         call errsub ( 444, 2, 0, 0 )
         go to 1000
      end if  ! ( ipnt1<=0 .or. ipnt1>npoint )
      if ( ipnt2<ipnt1 .or. ipnt2>npoint ) then
         call errint ( ipnt2, 1 )
         call errint ( ipnt1, 2 )
         call errint ( npoint, 3 )
         call errsub ( 445, 3, 0, 0 )
         go to 1000
      end if  ! ( ipnt1<=0 .or. ipnt1>npoint )
      do nodenr = ipnt1, ipnt2
         inusol = nodenr
         if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
         if ( inusol>0 ) &
            call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                             value, compl )
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0036'

      end if  ! ( debug )
      call erclos ( 'er0036' )

      end subroutine er0036
