      subroutine er0049 ( inpold, istart, numvec, inpnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-08-2011 Fortran 90
!        version  1.0  date 21-11-1989
!
!   copyright (c) 1989-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder the input array INPCRE of subroutine CREATE
!     For this subroutine INPCRE at input has the name INPOLD, &
!     whereas the output vector gets name INPNEW
!     This subroutine is necessary because the user may give the input in
!     arbitrary order
!     ER0049 should only be called if the user produced input in CREATE
!     in a non-sequential way, i.e. not from 1 to NUMVEC
!     REMARK:
!     If the description of INPCRE is changed this effects the subroutines:
!     READCR, ER0023 and ER0049
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numvec, inpold(*), istart(numvec), inpnew(*)

!     inpnew     i    New array INPCRE
!                     INPNEW is the result of reordering
!     inpold     i    Old array INPCRE to be reordered
!                     INPCRE is the result of subroutine ER0049
!     istart     i    Array of length NUMVEC containing the starting values of
!                     the various vectors in INPOLD
!     numvec     i    Number of vectors of which information is stored in INPCRE
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, kdeg, ipos, iold, m, kfill, icase, kextra, n

!     icase      Loop variable over the cases per vector
!     iold       Next free position to be used in array INPOLD
!     ipos       Position in array ISEPI3
!     ivec       Sequence number of vector of special structure
!     kdeg       Indication of the degrees of freedom to be used
!     kextra     Number of extra relative positions to be used for nodes
!     kfill      Indication how the nodal points must be filled into array NODES
!                Possible values:
!                1:  The nodes are filled in n user points
!                2:  The nodes are filled in the curves C1 to C2
!                3:  The nodes are filled in the surfaces S1 to S2
!                5:  The nodes are filled in the elements i to j
!                6:  The nodes are filled in the element group ielgrp
!                7:  The nodes are filled in the volumes V1 to V2
!                8:  The nodes are filled in the user points P1 to P2
!     m          Number of cases for a vector
!     n          Number of extra relative points to be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inpnew(1) = -1
      inpnew(2) = numvec
      ipos = 3

!     --- Loop over vectors

      do ivec = 1, numvec
         iold = istart(ivec)

!        --- Copy first 5 positions

         inpnew(ipos:ipos+4) = inpold(iold:iold+4)
         ipos = ipos + 5
         iold = iold + 5

!        --- Find number of cases

         m = inpnew(ipos-1)
         do icase = 1, m
            kdeg = inpold(iold)
            inpnew(ipos) = kdeg
            if ( kdeg<0 ) then

!           --- kdeg<0, copy -kdeg extra positions

               inpnew(ipos+1:ipos-kdeg) = inpold(iold+1:iold-kdeg)
               ipos = ipos + 2 - kdeg
               iold = iold + 2 - kdeg

            else

!           --- kdeg>=0, raise ipos and iold

               ipos = ipos + 2
               iold = iold + 2

            end if
            inpnew(ipos-1) = inpold(iold-1)
            kfill = inpold(iold)
            inpnew(ipos) = kfill
            iold = iold + 1
            ipos = ipos + 1

!           --- Compute kextra and raise ipos if necessary

            if ( kfill==0 ) then

!           --- kfill = 0, 0 extra positions

               kextra = 0

            else if ( kfill==1 ) then

!           --- kfill = 1, n extra positions

               kextra = 1

            else if ( kfill==2 .or. kfill==4 .or. kfill==8 ) then

!           --- kfill = 2, 4 or 8   3 extra positions

               kextra = 0
               inpnew(ipos:ipos+3) = inpold(iold:iold+3)
               ipos = ipos + 3
               iold = iold + 3

            else if ( kfill==3 .or. kfill==5 .or. kfill==7 ) then

!           --- kfill = 3, 5 or 7   n+2 extra positions

               kextra = 3

            else if ( kfill==6 ) then

!           --- kfill = 6   n+1 extra positions

               kextra = 2

            end if
            if ( kextra>0 ) then

!           --- kextra>=0, copy positions and raise ipos and iold

               n = inpold(iold+kextra-1)
               inpnew(ipos:ipos+n+kextra) = inpold(iold:iold+n+kextra)
               ipos = ipos + n + kextra
               iold = iold + n + kextra

            end if

         end do

      end do

      end
