      subroutine ercreatglob ( vector, kdeg, kfun, iglgrp, rincre )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-07-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of vector corresponding to globale unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     global_unknown
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vector(*), rincre(*)
      integer kdeg, kfun, iglgrp
      double precision consta

!     iglgrp         i    global element group number
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN < 0
!     vector        i/o   Solution array that must be filled partly
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, nusolclassic, inunkpgr, istkprobx

!     i              Counting variable
!     inunkpgr       Number of global unknowns in a group
!     istart         First position used at a part
!     istkprobx      Start address in kprobx
!     nusolclassic   Number of free classical unknowns, i.e ficitious unknowns
!                    and global unknowns excluded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!    1999   Global element group number out of range
!    2191   kfun >= 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initializations
!     Perform some checks
!     Fill vector depending on kdeg
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatglob' )

!     --- Initializations

      istkprobx = 1
      nglgrp = kprobx(1)
      nglunk = kprobx(2)
      nusolclassic = nusol-nglunk
      istart = nusolclassic
      do i = 1, iglgrp-1
         inunkpgr = kprobx(istkprobx+2)
         istart = istart+inunkpgr
         istkprobx = istkprobx+6
      end do
      inunkpgr = kprobx(istkprobx+2)

!     --- Perform some checks

      if ( kfun.ge.0 ) then

!     --- kfun >= 0, has not yet been implemented

         call errint ( kfun, 1 )
         call errsub ( 2191, 1, 0, 0 )
         go to 1000

      end if
      if ( iglgrp.le.0 .or. iglgrp.gt.nglgrp ) then

!     --- iglgrp out of range

         call errint ( iglgrp, 1 )
         call errint ( nglgrp, 2 )
         call errsub ( 1999, 2, 0, 0 )
         go to 1000

      end if
      consta = rincre(-kfun)

!     --- Fill vector depending on kdeg

      if ( kdeg.eq.0 ) then

!     --- kdeg = 0, all degrees of freedom

         do i = 1, inunkpgr
            vector(istart+i) = consta
         end do

      else if ( kdeg.gt.0 .and. kdeg.le.inunkpgr ) then

!     --- kdeg > 0, one degree of freedom

         vector(istart+kdeg) = consta

      else

!     --- error message, kdeg has wrong value

         call errint ( kdeg, 1 )
         call errint ( inunkpgr, 2 )
         call errsub ( 218, 2, 0, 0 )
         go to 1000

      end if

1000  call erclos ( 'ercreatglob' )
      end
