      subroutine ercreatstand ( kfill, nnodes, inpcre, jpstin, &
                                kfun, rincre, &
                                ipstin, mvec, maxphys, bndcon, &
                                icompl, kdeg, kvec, ivecold, &
                                isolut, istartvc, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 19-06-2015 Adapt nnodes
!        version  1.1  date 17-01-2015 Extension with prreducenodes
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one vector for one option only
!     Standard case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfill, nnodes, &
              inpcre(*), jpstin, kfun, ipstin, mvec, maxphys, &
              icompl, kdeg, kvec, ivecold, isolut(*), &
              istartvc, islold(*)
      double precision rincre(*)
      logical bndcon

!     bndcon         i    If true boundary conditions must be filled, otherwise
!                         a vector must be created
!     icompl         i    Indication if the vector to be created is real (0) or
!                         complex
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin         i    Starting address in array INPCRE
!     islold         i    User input array in which the user puts information
!                         of all preceding solutionss
!                         Integer array of length 5 x numold
!     isolut         i    Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     istartvc       i    Starting position of sequence numbers of vectors in inpcre
!     ivecold        i    ivec for old vector
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     kvec           i    Loop variable over the vectors to be created
!     maxphys        i    Maximum number of unknowns per point
!     mvec           i    Number of old vectors to be used in FUNCVECT
!     nnodes         o    Number of nodes
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nparts, inum
      integer, allocatable :: ipoint(:)

!     inum           Input parameter for subroutine prfillnodes
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCREATSTAND1  Fill one vector for one option only
!                    Standard case, the nodes have been filled in nodes
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRREDUCENODES  Reduce set of nodes to those nodes where the unknown
!                    is present
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatstand' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatstand'
         write(irefwr,1) 'ipstin, jpstin', ipstin, jpstin
         call prinin ( inpcre(ipstin), 10, 'inpcre' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length = npoint

      if ( parallel ) length = max(length,size(kmeshm) )

      allocate ( nodes(length), ipoint(ncurvs+1), stat = error )
      if ( error/=0 ) call eralloc ( error, length+ncurvs+1, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Store nodal point numbers into array NODES

      inum = 0
      if ( kfill==2 .or. kfill==3 ) inum = -1
      if ( debug ) write(irefwr,1) 'inum, kfill', inum, kfill

      call sepfillnodes ( kfill, inum, inpcre(jpstin), &
                          nodes, nnodes, ipoint, nparts )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( nodes, nnodes, 'nodes' )

      if ( indprfo>0 ) then

!     --- indprfo>0, reduce nodes to those nodes where the unknown is present

         call prreducenodes ( kdeg, nodes, nnodes )

      end if  ! ( indprfo>0 )

!     --- Fill array vector(kvec) with vector

      if ( nnodes>0 ) then

!     --- nnodes>0, fill vector

         call ercreatstand1 ( kfill, nnodes, inpcre, jpstin, kfun, &
                              rincre, ipstin, icompl, kdeg, kvec, &
                              isolut, maxphys, mvec, &
                              istartvc, islold, bndcon, ivecold )

      end if  ! ( nnodes>0 )

      deallocate ( nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatstand'

      end if  ! ( debug )
      call erclos ( 'ercreatstand' )

      end subroutine ercreatstand
