      subroutine ercreatsub3 ( isubr, kvec, ifill )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-12-2014 New definition clinsl
!        version  1.1  date 05-13-2013 Extension with change_vector
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print information of create vector if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, kvec, ifill

!     ifill          i    Indicates how the vector must be filled.
!                         Possible values:
!                         -1  The corresponding vector is not filled, &
!                             only created
!                          0  The corresponding vector is created and made
!                             equal to zero
!                         m>0 The filling of the vector is defined by m commands
!     isubr          i    Indicates by what subroutine creatv01 has been called.
!                         Possible values:
!                         1:  By the user or CREATE
!                         2:  By PRESDF or PRESTM
!                         3:  CREAVC
!                         4:  PRESTM
!                         5:  ERCRET
!                         6:  ERPRES
!                         7:  ERPRES called from structure
!                         8:  ERCRET called from structure
!                         9:  ERCRET called from structure (change vector)
!     kvec           i    Sequence number over the vectors
!                         Loop parameter in call ing subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifirstwrit, nch

!     ifirstwrit     If 0 output of information allowed, &
!                    if 1, this ouput is suppressed
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ifirstwrit/0/
! ======================================================================
!
      call eropen ( 'ercreatsub3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 .and. &
           isubr>4 .and. ifirstwrit==0 ) then

!     --- ioutp>=0 and isubr = 5 or 6, print information

         nch = index(names(kvec),' ')-1
         if ( nch<=0 ) nch = len(names(kvec))
         if ( isubr==5 .or. isubr==8 ) then

!        --- call by ercret

            if ( ifill==-1 ) then

!           --- ifill = -1, array is created but not filled

               write ( irefwr, 100 ) names(kvec)(1:nch)
100            format ( ' Create vector ', a, ' without filling' )

            else

!           --- ifill>-1, array is created and filled

               write ( irefwr, 110 ) names(kvec)(1:nch)
110            format ( ' Create and fill vector ', a )

            end if

         else if ( isubr==9 ) then

!        --- call by ercret (change vector)

            write ( irefwr, 115 ) names(kvec)(1:nch)
115         format ( ' Change vector ', a )

         else

!        --- call by erpres

            if ( ifill==-1 ) then

!           --- ifill = -1, array is created but not filled

               write ( irefwr, 100 ) names(kvec)(1:nch)

            else

!           --- ifill>-1, array is created and filled

               write ( irefwr, 120 ) names(kvec)(1:nch)
120            format ( ' Fill boundary conditions in vector ', a )

            end if

         end if

      end if

      if ( itimloop==1 ) then
         if ( t>=tend-tstep*0.1 ) then
            ifirstwrit = 0
         else
            ifirstwrit = 1
         end if  ! ( t>=tend-dt*0.1 )
      end if  ! ( itimloop==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatsub3'

      end if  ! ( debug )
      call erclos ( 'ercreatsub3' )

      end
