      subroutine ercreatsub5 ( ipstin, jpstin, ifirst, inpcre, &
                               n, istartvc, kdeg, kfun, kfill, ivecold, &
                               mvec, islold, maxphys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-04-2013 Correction vector loop
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters that are necessary to fill the vector
!     Compute jpstin (position in inpcre)
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipstin, jpstin, ifirst, inpcre(*), n, istartvc, &
              kdeg, kfun, kfill, ivecold, mvec, islold(*), &
              maxphys

!     ifirst         i    Value of first position of array INPCRE
!     inpcre         i    The integer input read is stored in array INPCRE from
!                         position 1. See subroutine creatv01
!     ipstin         i    Starting address in array INPCRE
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!     istartvc       o    Starting position of sequence numbers of vectors
!                         in inpcre
!     ivecold        o    ivec for old vector
!     jpstin         o    Help variable to compute new value of ipstin partly
!     kdeg           o    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfill          o    Indication how the nodal points must be filled into
!                         array NODES
!                         Possible values:
!                          0:  The nodes are filled in all points
!                          1:  The nodes are filled in n user points
!                          2:  The nodes are filled in the curves C1 to C2
!                          3:  The nodes are filled in the surfaces S1 to S2
!                          5:  The nodes are filled in the elements i to j
!                          6:  The nodes are filled in the element group ielgrp
!                          7:  The nodes are filled in the volumes V1 to V2
!                          8:  The nodes are filled in the user points P1 to P2
!                          9:  The nodes are filled in the active points of
!                              obstacle i
!                         10:  The nodes are filled in the active points of
!                              contact i
!                         11:  The nodes are filled in the non-active points
!                              of contact i
!                         12:  The nodes are filled using a file with node
!                              numbers
!                         13:  The nodes are filled using a file with elements
!                         14:  The nodes are filled using a file with capacities
!                         15:  The global unknowns are filled for global group
!                              iglgrp
!                         16:  All nodes in the obstacle are filled
!                         17:  Only the inner nodes in the obstacle are filled
!                         18:  Only the boundary nodes in the obstacle are
!                              filled
!                         19:  Only the boundary nodes on the obstacle are
!                              filled
!                         20:  Only the boundary nodes on the boundary with
!                              zero level set are filled
!                         22:  Series of curves
!     kfun           o    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     maxphys        o    Maximum number of unknowns per point
!     mvec           o    Number of old vectors to be used in FUNCVECT
!     n              o    number of commands
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ivectmp, numarray, indprfsav, indprpsav, nphyssav

!     i              Counting variable
!     indprfsav      Help parameter to save the value of indprf
!     indprpsav      Help parameter to save the value of indprp
!     ivectmp        Reduced vector number
!     nphyssav       Help parameter to save the value of nphys
!     numarray      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1053   mvec out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub5'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- i=ipstin

      jpstin = ipstin + 1

!     --- i+1

      kdeg   = 0
      kfun   = 0
      kfill  = 0
      if ( ifirst>=jpstin ) kdeg   = inpcre(jpstin)
      if ( kdeg<0 ) jpstin = jpstin - kdeg
      if ( ifirst>=jpstin+1 ) kfun   = inpcre(jpstin+1)

      if ( debug ) write(irefwr,1) 'kdeg, kfun', kdeg, kfun
      if ( kfun>=9001 .and. kfun<=9004 ) then

!     --- kfun = 9001, 9002, quadratic function along curves

         jpstin = jpstin+1

      else if ( kfun>3000 .and. kfun<4000 ) then

!     --- kfun>3000, function of old vectors

         mvec     = inpcre(jpstin+2)
         istartvc = jpstin+3
         jpstin = jpstin+mvec+1
         if ( mvec<=0 .or. mvec>10 ) then

!        --- mvec out of range

            call errint ( mvec, 1 )
            call errint ( 1, 2 )
            call errint ( 10, 3 )
            call errsub ( 1053, 3, 0, 0 )
            mvec = 1

         end if
         if ( debug ) then
            write(irefwr,1) 'mvec, istartvc', mvec, istartvc
            call prinin ( inpcre(istartvc), mvec, 'vector numbers' )
            do i = 1, mvec
               call inireducevec ( inpcre(istartvc+i-1), ivectmp, &
                                   numarray )
               call prinin ( islold(ivectmp), 5, 'vector' )
            end do  ! i = 1, mvec
         end if  ! ( debug ) then

!        --- Compute maxphys and activate arrays
!            This is done at this position to make it possible
!            to compute starting addresses in local subroutines

         maxphys = 0
         indprfsav = indprfi
         indprpsav = indprpi
         nphyssav = nphysi
         do i = 1, mvec
            call inireducevec ( inpcre(istartvc+i-1), ivectmp, &
                                numarray )
            call sepactsolbf2 ( islold(ivectmp) )
            maxphys = max(maxphys,nphysi)
         end do  ! i = 1, mvec
         indprfi = indprfsav
         indprpi = indprpsav
         nphysi = nphyssav

      else if ( kfun>4000 .and. kfun<5000 ) then

!     --- kfun>4000, use old vector

         ivecold = kfun-4000
         if ( debug ) &
            write(irefwr,1) 'ivecold, jpstin', ivecold, jpstin

      end if  ! ( kfun>=9001 .and. kfun<=9004 )
      if ( ifirst>=jpstin+2 ) kfill  = inpcre(jpstin+2)
      jpstin = jpstin + 3
      if ( debug ) &
         write(irefwr,1) 'jpstin, kdeg, kfun, kfill', &
                          jpstin, kdeg, kfun, kfill

!     --- i+4

      if ( kfill==1 ) then

         n = 1
         if ( ifirst>=jpstin ) n = inpcre(jpstin)
         jpstin = jpstin + 1

!        --- i+5

      end if  ! ( kfill==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatsub5'

      end if  ! ( debug )
      call erclos ( 'ercreatsub5' )

      end
