      subroutine ercreatsub8 ( itypvc, kvec, isolut, icompl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 29-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Treatment of periodical boundary conditions while filling a vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypvc, kvec, isolut(5,*), icompl

!     icompl         i    Indication if the vector to be created is real (0) or
!                         complex
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the solution vectors
!     itypvc         i    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     kvec           i    Loop variable over the vectors to be created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSPRCRE       Fill part of USOL corresponding to "right" periodical
!                    boundary conditions (complex case)
!     PRSPRRRE       Fill part of USOL corresponding to "right" periodical
!                    boundary conditions ( real case )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub8' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub8'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      if ( itypvc==110 .and. npboun>0 ) then

!     --- itrans # 2 and npboun>0
!         Periodical boundary conditions
!         Compute effect of periodical boundary conditions

         if ( icompl==0 ) then

!        --- real case

            call prsprrre ( usoltemp, kprobhinv, indprh )

         else

!        --- complex case

            call prsprcre ( usoltemp, kprobhinv, indprh )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatsub8'

      end if  ! ( debug )
      call erclos ( 'ercreatsub8' )

      end subroutine ercreatsub8
