      subroutine pr1003 ( coorold, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-11-1996 Extension with kprobfo
!        version  1.0  date 25-05-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute mesh velocity and store in array usol
!   It is supposed that only the coordinates of the mesh have been changed
!   not the topology
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
     use sepmodulekprob
      use sepmodulekmesh
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coorold(ndim,npoint), usol(*)

!     coorold    i    array containing the co-ordinates at the previous time
!                     step
!     usol       o    Solution vector containing the mesh velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ih, ih1
      double precision dtinv

!     dtinv      1/dt
!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ih1        Help variable to store a constant temporarily
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     u = ( coor_new - coor_old ) /dt
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      dtinv = 1d0/tstep
      if ( indprpo.ne.0 .and. indprfo.eq.0 ) then

!     --- indprpo # 0, renumbering required

         do i = 1, ndim
            do j = 1, npoint
               usol(kprobpo(j,i)) = dtinv*(coor(i,j)-coorold(i,j))
            end do
         end do
      else if ( indprpo.ne.0 ) then

!     --- indprpo # 0, renumbering required, but indprfo=0

         do i = 1, ndim
            do j = 1, npoint
               ih = kprobpo(j,i)
               if ( ih.gt.0 ) usol(ih) = dtinv*(coor(i,j)-coorold(i,j))
            end do
         end do
      else if ( indprfo.ne.0 ) then

!     --- indprfo # 0, number of unknowns per point not constant

         ih = 0
         do j = 1, npoint
            ih1 = kprobfo(j+1)
            if ( ih1.ge.ih+ndim ) then
               do i = 1, ndim
                  usol(ih+i) = dtinv*(coor(i,j)-coorold(i,j))
               end do
            end if
            ih = ih1
         end do
      else

!     --- indprpo = 0, standard sequence

         do i = 1, ndim
            do j = 1, npoint
               usol((j-1)*ndim+i) = dtinv*(coor(i,j)-coorold(i,j))
            end do
         end do
      end if
      end
