      subroutine prparmap ( vector, vectorenum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 31-03-2015 Replace er0061 by inicopindarr1
!        version  1.0  date 08-02-2004
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map vector from global numbering to local numbering without boundary
!     conditions
!     In case of parallel computing global values are filled by sending and
!     receiving from neighbour processors
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     global
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vector(*), vectorenum(*)

!     vector         i    SEPRAN array referring to the vector to be renumbered
!     vectorenum     o    SEPRAN array referring to the renumbered vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPINDARR1  Copy old indexed arrayold into new array arraynew
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prparmap' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prparmap'

      end if
      if ( ierror/=0 ) go to 1000
      if ( indprh>0 ) then

!     --- There are essential boundary conditions, renumber

         call inicopindarr1 ( nrusol, vectorenum, vector, kprobhinv )

      else

!     --- There are no essential boundary conditions, copy

         call dcopy ( nrusol, vector, 1, vectorenum, 1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prparmap'

      end if
      call erclos ( 'prparmap' )

      end
