      subroutine prpcgradbld ( iun, un, resn, itype, first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 10-02-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute residual for the spectral element equations in an element by
!     element way
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
!     residual
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iun(*), itype
      double precision un(*), resn(*)
      logical first

!     first          i    If true it concerns the first step
!     itype          i    Type number of equation
!                         Type number 600 is treated in a special way, i.e.
!                         by subroutine DERIV
!     iun           i/o   Help vector to store the values of un
!                         If first iun must contain the initial solution
!                         otherwise it must contain essential boundary
!                         conditions 0. In that case the contents are lost
!     resn           o    Residual
!     un            i/o   if not first, the iteration vector vn
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinbldlc(10), iinder(4), idumm, idumv, ipres

!     idumm          dummy
!     idumv          dummy
!     iinbldlc       User input array for subroutine BUILD
!     iinder         User input array for subroutine DERIV
!     ipres          Work array to store the result vector in standard sepran
!                    format
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRMAPBACK      Map vector from global numbering to local numbering without
!                    boundary conditions
!     PRPARMAP       Map vector from global numbering to local numbering without
!                    boundary conditions
!                    In case of parallel computing global values are filled by
!                    sending and receiving from neighbor processors
!     SEPBUILD       Build matrix and right-hand side
!     SEPDERIV       Compute derivatives
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( not first ) then
!        Map u^n back into iun
!     if ( itype=600 ) then
!        Use deriv to compute resiual store in ipres
!     else
!        Use build to compute resiual store in ipres
!     Map ipres into resn
!     Remove array ipres
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ipres/0/, idumv/0/
! ======================================================================
!
      call eropen ( 'prpcgradbld' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpcgradbld'

      end if
      if ( ierror/=0 ) go to 1000

      if ( .not. first ) then

!     --- Map u^n back into iun

         usoltemp => ks(iun(1))%sol
         call prmapback ( usoltemp, un )

      end if  ! ( first )

      if ( itype==600 ) then

!     --- Type number 600, use deriv

         iinder(1) = 4
         iinder(2) = 0
         iinder(3) = 0
         iinder(4) = 25

         call sepderiv ( iinder, ipres, iun )

      else

!     --- Other type number, use buildbf

         iinbldlc(1) = 5
         iinbldlc(2) = 2
         iinbldlc(3) = 1
         iinbldlc(4) = 0
         iinbldlc(5) = 1
         call sepfillsmf ( iinbldlc, ipres, idumv, iun )
         call sepbuild

      end if
      if ( ierror/=0 ) go to 1000

!     --- Map ipres into resn

      usoltemp => ks(ipres)%sol
      call prparmap ( usoltemp, resn )

!     --- Remove array ipres

      call seprmsolinfo ( ipres )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpcgradbld'

      end if
      call erclos ( 'prpcgradbld' )

      end subroutine prpcgradbld
