      subroutine prreducenodes1 ( nodes, nnodes, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reduce set of nodes to those nodes where the unknown is present
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     reduction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: idegfd
      integer, intent(inout) :: nodes(*), nnodes

!     idegfd         i    Degree of freedom sequence number
!     nnodes        i/o   Number of nodes in array nodes
!     nodes         i/o   Contains the node numbers
!                         At input set of nodes to be reduced
!                         At output reduced set
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icount

!     i              Counting variable
!     icount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreducenodes1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreducenodes1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0

!     --- Loop over nodes

      do i = 1, nnodes

         if ( kprobpo(nodes(i),idegfd)>0 ) then

!        --- degree of freedom is present in node: copy

            icount = icount+1
            nodes(icount) = nodes(i)

         end if  ! ( kprobpo(nodes(i),idegfd)>0 )

      end do  ! i = 1, nnodes

      nnodes = icount

1000  call erclos ( 'prreducenodes1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prreducenodes1'

      end if  ! ( debug )

      end subroutine prreducenodes1
