      subroutine prsolmat3 ( s, f, x, singular )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of 3 linear equations S x = f
!     Kramer's rule is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     solve
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(3,3), f(3), x(3)
      logical singular

!     f              i    right-hand side vector
!     s              i    matrix S to be solved
!     singular       o    if true the matrix is singular
!     x              o    solution
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision det

!     det            determinant
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsolmat3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsolmat3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      det = ercdet ( s(1,1), s(1,2), s(1,3) )
      if ( abs(det)<1d-50 ) then

!     --- system is singular

         singular = .true.

      else

!     --- system is regular

         singular = .false.

         x(1) = ercdet ( f, s(1,2), s(1,3) ) / det
         x(2) = ercdet ( s(1,1), f, s(1,3) ) / det
         x(3) = ercdet ( s(1,1), s(1,2), f ) / det

      end if  ! ( abs(det)<1d-50 )

1000  call erclos ( 'prsolmat3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsolmat3'

      end if  ! ( debug )

      end
