      subroutine sepassemelgrp ( ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters for one element group
!     Help routine for matrix assembly
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: ikelmc

!     ikelmc        i/o   position in kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILDALLOC  Allocate temporary arrays in for subroutine build
!     SEPFILLICOUNT  fill arrays index1sub and index2 for the element
!                    subroutines
!     TO0009         Compute some element group dependent parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepassemelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepassemelgrp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for one element group

      call to0009
      if ( ielgrp==nelgrp+1 ) ikelmc = 0
      if ( debug ) then
         write(irefwr,*) 'after to0009'
         write(irefwr,1) 'ikelmc, itype, inpelm, ielgrp, inelem', &
                          ikelmc, itype, inpelm, ielgrp, inelem
      end if  ! ( debug )

      E%ielgrp = ielgrp

      if ( debug ) write(irefwr,1) 'nsubel, jnpelm', nsubel, jnpelm
      if ( iinbld(2)/=6 ) then

!     --- Standard case, not boundary element matrix time vector

         matrix = addmat
         vector = addvec

      end if  ! ( iinbld(2)/=6 ) then
      if ( imas==0 ) then
         notmas = 1
      else
         notmas = 0
      end if
      irelem = 0
      if ( debug ) write(irefwr,1) 'notmas', notmas

!     --- If necessary, skip elements, i.e. if itype=-1
!                    or partad=true and skipelgrps(ielgrp)=0

      if ( debug ) then
         write(irefwr,1) 'itype, ielgrp, nelgrp', &
                          itype, ielgrp, nelgrp
         write(irefwr,1) 'skipelgrps(ielgrp)', skipelgrps(ielgrp)
         write(irefwr,*) 'partad', partad
         write(irefwr,*) 'skipbounels', skipbounels
         write(irefwr,*) 'skipinnels', skipinnels
      end if  ! ( debug )
      skipelements = itype==-1 .or. itype==0 .or. &
                     skipbounels .and. ielgrp>nelgrp .or. &
                     skipinnels .and. ielgrp<=nelgrp .or. inelem==0
      if ( partad ) skipelements = skipelements .or. skipelgrps(ielgrp)==0
      if ( skipelements ) goto 1000
      if ( debug ) write(irefwr,*) 'after skipelements'

!     --- Check if a subpart of kprobz must be used

      if ( indprz>0 ) then
         if ( kprobz(ielgrp)/=0 ) rprobz => probzarr(ielgrp)%arr
      end if  ! ( indprz>0 )

!     --- allocate arrays for element subroutines
!         In this case icount is correct

      if ( ielgrp<=nelgrp .or. indpri>0 ) &
         index1 => nodes(ikelmc+1:ikelmc+inpelm)

      call sepfillicount  ! Compute icount
      call sepbuildalloc  ! allocate index2..5 and coefarr/icoefarr

      allocate ( indoldsols(1)%index3(icount), stat=error )
      if ( error/=0 ) call eralloc ( error, icount, 'indoldsols' )
      if ( ierror/=0 ) go to 1000

      indoldsols(1)%index3 => index2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepassemelgrp'

      end if  ! ( debug )
      call erclos ( 'sepassemelgrp' )

      end subroutine sepassemelgrp
