      subroutine sepassemend
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays isol, irhsd, massmt and matr
!     Clear last part of rhsd
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nbound2

!     i              Counting variable
!     nbound2        nbound or in complex case 2*nbound
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepassemend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepassemend'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( clrlpv .and. addvec ) then

!     --- clear last part of right-hand side

         if ( nbound>0) then

!        --- nbound>0

            nbound2 = nbound
            if ( compl) nbound2 = 2*nbound
            do i = 1, numrhs
               rhsd => rhsds(i)%rh
               rhsd(1+nsol-nbound2:nsol) = 0d0
            end do  ! i = 1, numrhs
         end if
      end if
      if ( clrvec .or. addvec ) then
         if ( idefcr==1 ) numrhs = 2
         do i = 1, numrhs
            ks(SMF%irhsd(i))%typevec = 110
            ks(SMF%irhsd(i))%jcart = jcart
            if ( compl ) ks(SMF%irhsd(i))%compl = 1
         end do
         if ( idefcr==1 ) numrhs = 1
      end if
      if ( effbc ) then
         ks(SMF%iusol)%jcart= jcart
         if ( compl ) ks(SMF%iusol)%compl = 1
      end if

!     --- Renumber massmt

      if ( imas==1 .and. npboun>0 ) &
         M_ii(nrusol+1:nrusol+npboun) = &
            M_ii(kprobr(kprobhinv(nrusol+1:nrusol+npboun)))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepassemend'

      end if  ! ( debug )
      call erclos ( 'sepassemend' )

      end subroutine sepassemend
