      subroutine sepassemstelm ( jkelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters for one element
!     Fill index arrays
!     Help routine for matrix assembly
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmodulecpack
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jkelmc

!     jkelmc         i    position in kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, liarry

!     i              Counting variable
!     liarry         length of the integer array icoeffarray
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYELEM    Copy information of elements in type element
!     SEPFILLINDEX1  Fill array index1 with node numbers of sub-elements
!     SEPFILLINDEX34 Fill array index3 with addresses of unknowns for one
!                    element.
!                    Fill array index4 starting position for node in index3
!     SEPFLINDEX4    Fill array index4
!     TO0000         Fill arrays index1 and index2
!     TO00001        Fill array index2 from array index3
!     TOST00         Fill array INDEX5 in the case of periodical boundary
!                    conditions
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepassemstelm' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepassemstelm'
         write(irefwr,1) 'ishape, inpelm', ishape, inpelm
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      skip = .false.  ! do not skip element

!     --- fill array index1

      if ( nsubel==1 ) then

!     --- No sub-elements

         index1 => nodes(jkelmc+1:jkelmc+inpelm)

      else

!     --- Sub-elements

         call sepfillindex1 ( jkelmc )

      end if  ! ( nsubel==1 )

!     --- fill arrays indexren and index2

      if ( ielgrp<=nelgrp ) ilocelem = ilocelem + 1
      ilocelem = max(1,ilocelem)

      call to0000 ( ichnun )

      if ( parallel .and. ielgrp>nelgrp ) then

!     --- parallel computation, a boundary element is used
!         Check if element contains nodal points with sequence
!         number 0.
!         In that case the element must be skipped

         do i = 1, inpelm
            if ( index1(i)==0 ) skip = .true.
         end do

      end if  ! ( parallel .and. ielgrp>nelgrp )

!     --- Fill array indexren (i.e. apply renumbering)

      call to00001

      if ( npboun>0 ) then

!     --- periodical boundary conditions; fill array INDEX5

         call tost00

      else

         index5 = indexren

      end if  ! ( npboun>0 )

!     --- Fill arrays index3 and index4 if necessary

      if ( debug ) write(irefwr,1) 'nindex', nindex
      if ( nindex>0  ) then

!     --- nindex>0, there are several vectors

         call sepfillindex34 ( E%islold, jkelmc, nindex+inpelm )
         
      else

!     --- nindex = 0, there is only one vector

         index3(1,1:icount) = index2(1:icount)
         indoldsols(1)%lenindex3 = icount
         indoldsols(1)%index4 => index4(1,:)
         call sepflindex4

      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy information into type elem

      call sepcopyelem

!     --- compute element matrix and element vector

      elemvc2d = 0d0
      if ( itype/=996 ) then

!     --- Type number 996 is special, it has no element matrix
!         nor a mass matrix

         elemmt2d = 0d0
         elemms = 0d0

      end if  ! ( itype/=996 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepassemstelm'

      end if  ! ( debug )
      call erclos ( 'sepassemstelm' )
      end subroutine sepassemstelm
