      subroutine sepbuildact
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     the large matrix and vector are computed
!     the effect of boundary conditions is taken into account
!     Old subroutine TO0050
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      use sepmodulecpack
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc, jkelmc, jelem, i, lenindex3
      integer :: ifirstgroup
      double precision :: valmax, valmin
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     ifirstgroup    Lowest element group number in loop
!     ikelmc         position in kmesh part c
!     jelem          do loop variable with respect to the elements
!     jkelmc         position in kmesh part c
!     lenindex3      length of index3
!     valmax         Maximum value of diagonal element
!     valmin         Minimum value of  diagonal element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPASSEMDEALL  Deallocate temporary arrays for subroutine sepbuildact
!     SEPASSEMELEM   Create element matrices and vectors for one element
!                    Assemble in large matrix
!     SEPASSEMELGRP  Initialize some parameters for one element group
!     SEPASSEMEND    Fill arrays isol, irhsd, massmt and matr
!                    Clear last part of rhsd
!     SEPASSEMSTART  Start of assembly process
!                    Make all initializations, fill modules and declare arrays
!     SEPASSEMSTELM  Initialize some parameters for one element
!                    Fill index arrays
!     SEPBUILDDALLOC De-allocate temporary arrays in for subroutine build
!     SEPMAKEELM     Create element matrix and element vector
!     SEPPETSCFINMAT Finish creating petsc matrix
!     TO0016         fill array iwork with a 0 when degree of freedom i must not
!                    be transformed, and an i when it must be transformed
!     TO0050EFFBC    Compute effect of periodical boundary conditions
!                    to right-hand side
!     TO0055         compute norm of real matrix, and make sigma equal to
!                    norm/sqreps
!     TOCOPYMATFI    Send and receive matrices S_fi to/from other blocks
!                    Add matrices S_if and S_if to S_iout and S_ii
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildact' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildact'
  1      format ( a, 1x, (10i8) )

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Start of assembly process
!         Make some initializations, fill modules and declare arrays

      call sepassemstart
      if ( ierror/=0 ) go to 1000

!     --- Allocate work arrays

      allocate ( iworktr(nusol), iwork(nusol), stat=error )
      if ( error/=0 ) call eralloc ( error, 2*nusol, 'iworktr' )
      if ( ierror/=0 ) go to 1000

!     --- Fill work array iworktr for local transformations with information if
!         npltra>0

      if ( npltra>0 ) call to0016

!     --- loop over element groups

      ilocelem = 0
      ielem = 0
      ikelmc = 0
      ifirstgroup = 1

!     --- loop over element groups

      if ( debug ) write(irefwr,*) 'before ielgrp'

      if ( iinbld(2)==6 ) ifirstgroup = nelgrp+1

      if ( debug ) write(irefwr,1) 'ifirstgroup, numns', ifirstgroup, numns

      do ielgrp = ifirstgroup, numns

!     --- Initialize some parameters for one element group

         if ( indpra>0 .and. ielgrp<=size(kproba) ) &
            kprobatmp => kproba(ielgrp)%arr
         call sepassemelgrp ( ikelmc )
         if ( skipelements ) goto 480

!        --- allocate arrays for element subroutines

         lenindex3 = max(nindex+inpelm, inpelm*nunkp)
         allocate ( index3(max(numold,1),lenindex3), &
                    index1sub(inpelm), stat=error )
         if ( error/=0 ) call eralloc ( error, lenindex3+inpelm, 'index1sub' )
         if ( ierror/=0 ) go to 1000

         if ( nindex>0 ) index3 = 0  ! Reset index3

!        --- Compute the parameter sigma for boundary conditions of the type
!            u = constant

         if ( matrix .and. sigma<=0d0 .and. itype==-2 ) call to0055

!        --- loop over elements

         if ( debug ) write(irefwr,1) 'inelem', inelem

         do jelem = 1, inelem

!        --- Compute ielem, and start position jkelmc in kmesh part c

            ielem  = ielem + 1
            jkelmc = ikelmc + (jelem-1) * jnpelm
            irelem = jelem

            E%ielem = ielem
            E%irelem = irelem

            if ( debug ) &
               write(irefwr,1) 'iskipelem, ielgrp, nelgrp, nsubel', &
                                iskipelem, ielgrp, nelgrp, nsubel
            if ( iskipelem>0 .and. ielgrp<=nelgrp ) then

!           --- iskipelem>0, some elements must be skipped

               if ( kprobaa(ielem)>0 ) go to 450

            end if  ! ( iskipelem>0 .and. ielgrp<=nelgrp )

            do isubel = 1, nsubel

!           --- Fill some parameters for one element
!               Fill index arrays

               call sepassemstelm ( jkelmc )
               if ( ierror/=0 ) go to 1000
               if ( debug ) write(irefwr,*) 'skip/1 ', skip
               if ( skip ) go to 450

!              --- Create element matrix and element vector

               call sepmakeelm

!              --- Assemble in large matrix and vector

               call sepassemelem
               if ( ierror/=0 ) go to 1000

               if ( debug ) write(irefwr,*) 'skip/2 ', skip

               if ( skip ) go to 480

               if ( fillkmeshw .and. ielgrp<=nelgrp ) then

                  do i = 1, icount
                     realprop(1,ielgrp) = max(realprop(1,ielgrp), &
                                              abs(E%mat(i,i)) )
                  end do  ! i = 1, icount

              end if  ! ( fillkmeshw )

            end do

450      end do

480      ikelmc = ikelmc+jnpelm*inelem
         ielem = ielem + inelem - irelem

         call sepbuilddalloc  ! deallocate temporary arrays

      end do   ! ielgrp = ifirstgroup, numns

!     --- Compute effect of periodical boundary conditions

      if ( addvec .and. npboun>0 .and. matexi ) call to0050effbc
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays isol, irhsd, massmt and matr

      call sepassemend
      if ( ierror/=0 ) go to 1000

!     --- ninterface>0, hence S_iout exists, Fill this part of the matrix

      if ( size(IM%colsout)>1 .and. addmat .and. .not. IM%nosepmat ) &
         call tocopymatfi
      if ( ierror/=0 ) go to 1000

!     --- Deallocate arrays

      call sepassemdeall
      if ( ierror/=0 ) go to 1000

!     --- If jmethod = 56, petsc solver, finish making the matrix

      if ( IM%nosepmat ) call seppetscfinmat

      if ( fillkmeshw ) then

!     --- fillkmeshw true fill intprop

         intprop = 0
         nrealprop = 1
         kmeshw(2) = 1
         valmax = maxval(realprop(1,1:nelgrp))
         valmin = max(minval(realprop(1,1:nelgrp)),1d-10*valmax)
         do i = 1, nelgrp
            if ( realprop(1,i)>1d2*valmin ) intprop(1,i) = 1
         end do  ! i = 1, nelgrp

      end if  ! ( fillkmeshw )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildact'

      end if
      call erclos ( 'sepbuildact' )

      end subroutine sepbuildact
