      subroutine sepbuildfict ( iincommat, lniincommat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 02-02-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the fluid elements corresponding to each lambda point in array
!     kprobfict
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lniincommat
      integer, intent(inout) :: iincommat(lniincommat)

!     iincommat      i    Integer input array containing information concerning
!                         the structure of the large matrix
!                         See subroutine read00 for a description
!     lniincommat    i    Declared length of array iincommat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: changed

!     changed        If true the position of lagrangian multipliers in ficitious
!                    domain elements has been changed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROBFICT       Store the fluid elements corresponding to each lambda point
!                    in array kprobfict
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildfict' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildfict'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!      Store the fluid elements corresponding to each lambda point in array
!      kprobfict
!      This is done because the position of the surface on which the
!      fictitious domain elements are lying may have been changed during
!      the computation

      changed = .false.
      call probfict ( changed )
      if ( ierror/=0 ) go to 1000

!     --- recompute intmat if updated fictitious points are associated
!         with new fluid elements

      if ( changed ) then

!     --- fictitious points have been moved

         iincommat = 0
         iincommat(2) = intmt(intmat)%jmethod
         iincommat(5) = iprob
         kprob => kprob
         kmesh => kmeshloc
         call sepcommat ( iincommat )
         call sepfillmatstr

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildfict'

      end if  ! ( debug )
      call erclos ( 'sepbuildfict' )

      end subroutine sepbuildfict
