      subroutine sepccgcheckeps ( x, repeat1, xnorm, res, iter, &
                                  finished, rhotest, parvers, A, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The accuracy of the process seems to be reached
!     Check if eps must be recomputed
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      integer, intent(in) :: iter
      double precision, intent(in) :: rhotest
      double precision, intent(inout) :: xnorm
      double complex, intent(in) :: x(A%neq), res(A%neq), b(A%neq)
      logical, intent(inout) :: repeat1, finished
      logical, intent(in) :: parvers

!     A              i    Matrix A
!     b              i    Right-hand side
!     finished      i/o   if true the iteration has finished
!     iter           i    Iteration number
!     parvers        i    If true it concerns the parallel version
!     repeat1       i/o   Logical variable indicating if the accuracy check
!                         must be carried out twice (true) or once (false), &
!                         because the cnorm of the solution could not be computed
!     res            i    Residual
!     rhotest        i    cnorm of residual used to terminate CG
!     x              i    Solution vector
!     xnorm          i    Norm of the estimated solution
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: reprint
      logical :: again

!     again          If true the computation may be repated
!                    they are not
!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCCGEPS1     Recompute accuracy eps during cg iteration
!     SEPPARCCGEPS1  Recompute accuracy eps during cg iteration (parallel)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgcheckeps' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgcheckeps'
         write(irefwr,*) 'finished ', finished
         write(irefwr,1) 'iter', iter
         write(irefwr,2) 'eps', A%eps
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

140   finished = rhotest<=A%eps

      if ( debug ) write(irefwr,*) 'finished ', finished

      if ( finished ) then

!     --- The accuracy of the process seems to be reached
!         Check if eps must be recomputed

         if ( repeat1 ) then

!        --- repeat1 is true, recompute eps

            if ( debug ) write(irefwr,*) 'before prcgeps1'

            if ( parvers ) then
               call sepparccgeps1 ( x, again, repeat1, xnorm, A )
            else
               call sepccgeps1 ( x, again, repeat1, xnorm, A, b )
            end if  ! ( parvers )

            if ( debug ) then
               write(irefwr,*) 'again', again
               write(irefwr,2) 'eps', A%eps
            end if  ! ( debug )
            if ( again ) go to 140

         end if

         if ( A%iprint==1 ) then

!        --- iprint = 1  Print information

            reprint = cnorm(res,A%neq)
            write ( irefwr, 150 ) iter-1, reprint
150         format(' The number of executed iterations in the', &
                   ' CG-process is', i5/ &
                   ' The 2-norm of the residual is:', ss,1pe12.2e2 )

         end if

         A%numiter = iter-1
         A%itererr = 0

         finished = .true.

      end if  ! ( finished )

1000  call erclos ( 'sepccgcheckeps' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgcheckeps'

      end if  ! ( debug )

      end subroutine sepccgcheckeps
