      subroutine sepcgprint1 ( res, iter, rhotest, bsnorm, &
                               lambda, xnorm, A, bnorm, r0norm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform extended print for subroutine sepcg during iteration
!     iprint>=2
!     Old subroutine prcgprint1
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      integer, intent(in) :: iter
      double precision, intent(in) :: rhotest, bnorm, bsnorm, r0norm, &
                                      lambda, xnorm
      double precision, intent(in) :: res(A%neq)

!     A              i    Matrix A
!     bnorm          i    Norm of the right-hand side (b,b)
!     bsnorm         i    Preconditioned norm of the right-hand side
!                         (M^(-1)b,M^(-1)b)
!     iter           i    Iteration sequence number
!     lambda         i    Approximation of smallest eigenvalue
!     r0norm         i    norm of start residual
!     res            i    real matrix containing A  - A  B ( B A  B )  B A
!     rhotest        i    norm of residual used to terminate CG
!     xnorm          i    Norm of the estimated solution
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: reprint

!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgprint1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgprint1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      reprint = norm(res, A)

      if ( A%iaccuracy==0 ) then
         write ( irefwr, 200 ) iter, reprint/sqrt(bnorm), &
              sqrt(rhotest/bsnorm), lambda, &
              sqrt(rhotest/xnorm)/lambda

      else if ( A%irelerr==3 ) then

         write ( irefwr, 201 ) iter, reprint/sqrt(bnorm)

      else if ( A%irelerr==1 ) then

         write ( irefwr, 201 ) iter, reprint/sqrt(r0norm)

      else

         write ( irefwr, 201 ) iter, reprint

      end if

200   format ( i7, ss, 1pe10.2e2, 1pe13.2e2, 1pe10.2e2, &
               1pe10.2e2 )
201   format ( i7, ss, 1pe10.2e2, 1pe10.2e2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgprint1'

      end if  ! ( debug )
      call erclos ( 'sepcgprint1' )

      end
