      subroutine sepcnrcmatvec ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A x, where A is non-symmetric row compact matrix
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double complex :: sum1

!     i              General loop variable
!     j              General loop variable
!     sum1           help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcnrcmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcnrcmatvec'
         call prinrl ( x, A%neq, 'x' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      do i =  1, A%neq
         sum1 = 0d0
         do j = A%rows(i)+1, A%rows(i+1)
            sum1 = sum1+A%S(j)*x(A%cols(j))
         end do  ! A%rows(i)+1, A%rows(i+1)
         y(i) = sum1
      end do  ! i =  1,

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End sepcnrcmatvec'

      end if  ! ( debug )
      call erclos ( 'sepcnrcmatvec' )

      end subroutine sepcnrcmatvec
