      subroutine sepcopmatmulrp ( A, B )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy information from A with type mat into B with type pmatrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     real
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulematr
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat) :: A
      type ( pmatrix ) :: B

!     A              i    matrix of type mat to be used in multiplication
!     B              o    internal matrix A_ii stored as type rmatrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos, ipstart, lastp
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     ipos           Position in array
!     ipstart        Starting address in array
!     lastp          last position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopmatmulrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopmatmulrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      B%nrow = A%nrow
      B%neq = nbound
      B%n = nusol

      B%rows => IM%rowsip
      B%cols => IM%colsip
      B%rowsp => IM%rowspp
      B%colsp => IM%colspp

      B%Bip => A%S_ip
      B%Bpi => A%S_pi
      B%Bpp => A%S_pp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopmatmulrp'

      end if  ! ( debug )
      call erclos ( 'sepcopmatmulrp' )

      end subroutine sepcopmatmulrp
