      subroutine sepcopproj ( amat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-02-2016 New parameters list
!        version  1.0  date 04-07-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy information of projection vectors in type matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: amat

!     amat           i    large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopproj' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopproj'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'nproj, nnproj', nproj, nnproj

      amat%nproj = nproj
      amat%deflation = inpsol(36,1)==0

      amat%projrows => projrows
      amat%projcols => projcols
      amat%projrowsa => projrowsa
      amat%projcolsa => projcolsa

      amat%projU => projU
      amat%projAu => projAu
      amat%projE => projE

      allocate ( amat%workstorage(amat%neq), stat = error )
      if ( error/=0 ) call eralloc ( error, amat%neq, 'workstorage' )

1000  call erclos ( 'sepcopproj' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopproj'

      end if  ! ( debug )

      end subroutine sepcopproj
