      subroutine sepcopymatpm ( iseqinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy real R matrix in type matrix (schur)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     real
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqinp

!     iseqinp        i    Sequence number of input array inpsol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYINPSOL  Put information of inpsol and rinsol in matrix structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopymatpm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopymatpm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      R%type_prec = 0
      R%deflation = .false.

!     --- Store type of matrix

      R%neq = np
      R%nglob = np
      R%nproj = 0
      R%prlump = 0
      R%matmultsp = 0

      R%S => S_PM
      R%type_storage = 5
      R%symmetric = .true.

      R%rows => IM%rowsPM
      R%cols => IM%colsPM
      R%Mrows => IM%rowsPM
      R%Mcols => IM%colsPM
      R%diag => IM%diagPM
      R%Mdiag => IM%diagPM
      R%iprint = inpsol(6,iseqinp)
      R%factor = rinsol(3,iseqinp)
      R%adapt = rinsol(2,iseqinp)
      R%iseqmt = iseqmt
      R%iseqim = iseqim

      call sepcopyinpsol ( R, iseqinp )
      R%simple = .false.

      if ( iseqinp==4 ) then

!     --- Schur method

         R%scaling = 0    ! matrix is not scaled

      else

!     --- Block triangular preconditioner

         R%scaling = 2    ! matrix is scaled

      end if  ! ( R%typesolv==16 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopymatpm'

      end if  ! ( debug )
      call erclos ( 'sepcopymatpm' )

      end subroutine sepcopymatpm
