      subroutine sepfillbuild
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill pointers in sepmodulebuild for subroutine build
!     Create relation between arrays and buffer
! **********************************************************************
!
!                       KEYWORDS
!
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulevecs
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenim2, k, indprfloc, lastp
      type (globmat), pointer :: S1, S2, MASS
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     MASS           pointer to realmt(massmt)
!     S1             pointer to realmt(stiffmat(1))
!     S2             pointer to realmt(stiffmat(2))
!     indprfloc      Value of indprf for specific vector in oldsols
!     k              General loop variable
!     lastp          Last position of array
!     lenim2         length of the array imat part 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLMAT     Fill information of matrix with sequence number iseqmt
!                    structure of type mat
!     SEPFILLMATSIMP Fill pointers in sepmodulematr for subroutine build
!                    Create relation between arrays and buffer
!     SEPFILLOLDSOL  Fill old solutions as well as array flipflop in
!                    sepmodulebuild
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillbuild' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillbuild'
         write(irefwr,1) 'nbound, iinbld(2), imas', nbound, iinbld(2), imas
         write(irefwr,*) 'effbc ', effbc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      oldsol1 = .false.

!     --- Array USOL  (Only when the effect of boundary conditions must be
!                      taken into account)

      if ( effbc .and. nbound>0 .or. iinbld(2)==6 ) then

!     --- Effect of boundary conditions, or special case of
!         multiplication of element matrices and solution

         if ( SMF%iusol>0 ) then

            if ( debug ) write(irefwr,1) 'SMF%iusol', SMF%iusol
            usol => ks(SMF%iusol)%sol
            oldsols(1)%lenoldvec = size(usol)

         end if  ! ( SMF%iusol>0 )

      end if  ! ( effbc .and. nbound>0 .or. iinbld(2)==6 )

!     --- Matrices

      symm = IM%symmetric

      if ( debug ) write(irefwr,*) 'clrmat, addmat', clrmat, addmat

      if ( clrmat .or. addmat ) then

         if ( debug ) write(irefwr,1) 'matr', stiffmat(1)
         if ( stiffmat(1)>0 ) then
            iseqmt = stiffmat(1)
            call sepfillmat
            S1 => realmt(iseqmt)
            Smat => realmt(iseqmt)
            S_ii => S1%S_ii
            if ( ninterface>0 ) then

!           --- There are interface unknowns

               S_if => S1%S_if
               S_fi => S1%S_fi

            end if  ! ( ninterface>0 )

            if ( size(S1%S_iout)>1 ) S_iout => S1%S_iout

            if ( IM%simple ) call sepfillmatsimp(S1)! simple

         end if  ! ( stiffmat(3,1)>0 )

         if ( idefcr==1 ) then

!        --- Defect correction

            if ( debug ) write(irefwr,1) 'matr/2', stiffmat(2)
            iseqmt = stiffmat(2)
            call sepfillmat
            S2 => realmt(iseqmt)
            S2_ii => S2%S_ii

         end if  ! ( idefcr==1 )

         if ( nbound>0 ) then

            iseqmt = stiffmat(1)
            call sepfillmat
            S1 => realmt(iseqmt)
            S_ip => S1%S_ip
            S_pi => S1%S_pi
            S_pp => S1%S_pp
            if ( npboun>0 ) S_ic => S1%S_ic

            if ( idefcr==1 ) then
               iseqmt = stiffmat(2)
               call sepfillmat
               S2 => realmt(iseqmt)
               S2_ip => S2%S_ip
               S2_pi => S2%S_pi
               S2_pp => S2%S_pp

               if ( npboun>0 ) S2_ic => S2%S_ic

            end if  ! ( idefcr==1 )
         end if  ! ( nbound>0 )


         S1_ii => S_ii
         S1_ip => S_ip
         S1_pi => S_pi
         S1_pp => S_pp
         S1_ic => S_ic

      else if ( effbc ) then

!     --- The effect of essential boundary conditions is required for an
!         old matrix

         iseqmt = stiffmat(1)
         S1 => realmt(iseqmt)
         S_ip => S1%S_ip
         S_pi => S1%S_pi
         S_pp => S1%S_pp


      end if  ! ( clrmat .or. addmat )

!     --- Mass matrices

      if ( imas==1 ) then

!     --- Diagonal mass matrix

         if ( debug ) write(irefwr,1) 'massmt', massmat
         iseqmt = massmat
         MASS => realmt(iseqmt)
         M_ii  => MASS%S_ii
         MASS%S_ii => M_ii
         MASS%intmul = .true.
         MASS%nrow = IM%nrusol
         if ( parallel .and. typecoupl==1 ) then
            MASS%sendinfo => sendinfopar
            MASS%S_i%sendinfo => MASS%sendinfo
         end if  ! ( typecoupl==0 )

      else if ( imas>=2 ) then

!     --- Non-diagonal mass matrix

         if ( debug ) write(irefwr,1) 'massmt/1', massmat
         iseqmt = massmat
         call sepfillmat
         MASS => realmt(iseqmt)
         M_ii  => MASS%S_ii
         if ( ninterface>0 ) then

!        --- There are interface unknowns

            M_if => MASS%S_if
            M_fi => MASS%S_fi

         end if  ! ( ninterface>0 )

         if ( size(MASS%S_iout)>1 ) M_iout => MASS%S_iout

         if ( nbound>0 ) then
            M_ip  => MASS%S_ip
            M_pi  => MASS%S_pi
            M_pp  => MASS%S_pp
         end if  ! ( nbound>0 )

      end if  ! ( imas>=1 )

!     --- Old solutions and flipflop array

      if ( debug ) write(irefwr,1) 'numold', numold
      call sepfilloldsol

!     --- RIGHT-HAND SIDE

      if ( clrvec .or. addvec .or. effbc .and. nbound>0 ) then

         allocate ( rhsds(numrhs) )
         do k = 1, numrhs
            rhsds(k)%rh => ks(SMF%irhsd(k))%sol
         end do  ! k = 1, numrhs
         rhsd => rhsds(1)%rh

      end if  ! ( clrvec.or. addvec .or. effbc.and.nbound>0 )

!     --- Clear matrices if necessary and read parts of matrix if necessary

      if ( clrmat ) then

!     --- matrix must be cleared

         if ( stiffmat(1)>0 ) S1_ii = 0d0

         if ( idefcr==1 ) S2_ii = 0d0

      end if  ! ( clrmat )

!     --- Clear mass matrix if necessary

      if ( imas>=1 ) M_ii = 0d0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillbuild'

      end if  ! ( debug )
      call erclos ( 'sepfillbuild' )

      end subroutine sepfillbuild
