      subroutine sepfillindex1 ( ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced, &
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy, &
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array index1 with node numbers of sub-elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletypes
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ikelmc

!     ikelmc         i    last position used in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, ind1(3,4), ind3(3,4), ind2(4,4), index21(4,6)
      integer :: j

!     ind1           Array containing the relative sub elements of a quadratic
!                    triangle in the following sense
!                    ind1(j,i) contains the sequence number in nodes of the
!                    j-th node of the i-th sub element
!     ind2           Array containing the relative sub elements of a quadratic
!                    quadrilateral in the following sense
!     ind3           Array containing the relative sub elements of a linear
!                    quadrilateral in the following sense
!     index21        Array containing the relative sub elements of a linear
!                    prism in the following sense
!     istart         Starting address in array
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1586   Subdivision of elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind1 / 1, 2, 6, &
                  2, 3, 4, &
                  4, 5, 6, &
                  2, 4, 6 /
      data ind2 / 1, 2, 9, 8, &
                  2, 3, 4, 9, &
                  4, 5, 6, 9, &
                  8, 9, 6, 7 /
      data index21 / 1, 2, 3, 5,   4, 5, 6, 1,    1, 3, 5, 6, &
                     1, 2, 3, 4,   4, 5, 6, 2,    2, 3, 6, 4 /
      data ind3/ 2, 3, 1,   4, 1, 3,   1, 2, 4,   3, 4, 2 /
! ======================================================================
!
      call eropen ( 'sepfillindex1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillindex1'
         write(irefwr,1) 'ishapeorg, ikelmc, isubel', &
                          ishapeorg, ikelmc, isubel
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store node numbers into index1sub

      select case ( ishapeorg )

         case ( :1, 8:11, 17:20, 22: )

!        --- error subdivision has not been implemented

            call errint ( ishapeorg, 1 )
            call errsub ( 1586, 1, 0, 0 )
            go to 1000

         case ( 2 )

!        --- quadratic line element

            do j = 1, 2
               index1sub(j) = nodes(ikelmc+isubel-1+j)
            end do

         case ( 4, 7 )

!        --- quadratic triangle

            do j = 1, 3
               index1sub(j) = nodes(ikelmc+ind1(j,isubel))
            end do

         case ( 5 )

!        --- bi-linear quadrilateral

            if ( itype==903 ) then

!           --- Special case of type 903

               do j = 1, 3
                  index1sub(j) = nodes(ikelmc+ind3(j,isubel))
               end do

            else

!           --- error subdivision has not been implemented

               call errint ( ishapeorg, 1 )
               call errsub ( 1586, 1, 0, 0 )
               go to 1000

            end if  ! ( itype==903 )

         case ( 6 )

!        --- bi-quadratic quadrilateral

            do j = 1, 4
               index1sub(j) = nodes(ikelmc+ind2(j,isubel))
            end do

         case ( 12, 15, 16 )

!        --- tri-quadratic tetrahedron

            istart = 4+4*isubel
            do j = 1, 4
               index1sub(j) = nodes(ikelmc+itetra(istart+j))
            end do

         case ( 21 )

!        --- trilinear-prism

            do j = 1, 4
               index1sub(j) = nodes(ikelmc+index21(j,isubel))
            end do

      end select  ! case ( ishapeorg )

!     --- Refer index1 to index1sub

      index1 => index1sub(1:inpelm)

1000  call erclos ( 'sepfillindex1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( index1, inpelm, 'subelement' )
         write(irefwr,*) 'End sepfillindex1'

      end if  ! ( debug )

      end subroutine sepfillindex1
