      subroutine sepfillindex24 ( inpvec, index1, &
                                  index2, index4, ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays index2 and index4 for a given vector
!     Corresponds to one element group
!     Old subroutine to0038
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpvec, index1(*), ikelmc
      integer, intent(out) :: index2(*), index4(*)

!     ikelmc     i    Last position in KMESH part c that has been used
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     index2     o    array with position of degrees of freedom in vector
!                     denoted by inpvec for nodal points of current element
!                     ielem
!     index4     o    cumulative number of degrees of freedom in current
!                     element for vector denoted by inpvec, index4(1)=0
!                     index4(k+1)-index4(k)=number of degrees of freedom
!                     in k'th nodal point of current element ielem
!     inpvec     i    standard sepran array denoting usol or vector of special
!                     structure defined in nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jcount, i, inunkp, iknpt, inusol, j

!     i              Counting variable
!     iknpt          Nodal point number
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jcount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillindex24' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillindex24'
         call prinin ( inpvec, 5, 'inpvec' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount = 0
      index4(1) = 0

      select case (ks(inpvec)%typevec )

      case (110, 115 )

!     --- itypvc = 110 or 115

         if ( indprflc==0 ) then

!        --- number of degrees of freedom is constant for itypvc = 110,115

            if ( nunkplc<=1 ) then

!           --- simplest case : 1 degree of freedom per nodal point

               index2(1:inpelm) = index1(1:inpelm)
               do i = 1, inpelm
                  index4(i+1) = i
               end do
               jcount = inpelm

            else

!           --- number of degrees of freedom is constant but>1

               inunkp = nunkplc
               do i = 1,inpelm
                  iknpt = index1(i)
                  inusol = (iknpt-1)*inunkp
                  do j = 1,inunkp
                     index2(jcount+j) = inusol+j
                  end do
                  jcount = jcount+inunkp
                  index4(i+1) = jcount
               end do

            end if

         else

!        --- number of degrees of freedom not constant

            do i = 1,inpelm
               iknpt = index1(i)
               inusol = kprobflc(iknpt)
               inunkp = kprobatmp(i)
               do j = 1,inunkp
                  index2(jcount+j) = inusol+j
               end do
               jcount = jcount+inunkp
               index4(i+1) = jcount
            end do

         end if

      case ( 116, 136 )

!     --- vector of special structure is defined per element , itypvc = 116
!         number of degrees of freedom per element is also in inpvec(4)

         jcount = ks(inpvec)%ivec
         inusol = (ielem-1)*jcount
         do j = 1, jcount
            index2(j) = inusol+j
         end do
         index4(2) = jcount

      case ( 126 )

!     --- vector of special structure is defined per element per node, &
!         itypvc = 126
!         number of degrees of freedom per element is also in inpvec(4)

         jcount = ks(inpvec)%ivec
         inusol = ikelmc*jcount
         do j = 1, jcount*inpelm
            index2(j) = inusol+j
         end do
         index4(2) = jcount

      case ( 129 )

!     --- vector of special structure is defined per element per integration
!         point, itypvc = 129
!         number of degrees of freedom per element is also in inpvec(4)

         jcount = ks(inpvec)%ivec
         inusol = ikelmc*jcount
         do j = 1, jcount*m
            index2(j) = inusol+j
         end do
         index4(2) = jcount

      end select  ! case (inpvec(2) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillindex24'

      end if  ! ( debug )
      call erclos ( 'sepfillindex24' )

      end subroutine sepfillindex24
