      subroutine sepfillsolmod1 ( compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create arrays usol and rhsd
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulevecs
      use sepmodulematr
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icompl
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLSOLMODS Fill several modules for subroutine seplinsol
!                    Special case: simple
!     SEPRENUMSOL    Renumber solution vector such that prescribed boundary
!                    conditions are stored at the end.
!                    Local transformations are carried out if necessary
!                    Real case
!     SEPSOLZUSOL    Make the relation between usol, rhsd and zusol, zrhsd
!                    Complex arithmetic
!     SEPZRENUMSOL   Renumber solution vector such that prescribed boundary
!                    conditions are stored at the end.
!                    Local transformations are carried out if necessary
!                    Complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolmod1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolmod1'
         call ertrace
         write(irefwr,1) 'nusol', nusol
         write(irefwr,*) 'compl ', compl
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      icompl = 1
      if ( compl ) icompl = 2

      if ( indprh==0 ) then

!     --- No renumbering

         usol => usolbuf(1:nusol*icompl)
         rhsd => rhsdbuf(1:nusol*icompl)
         if ( numdef_corr>0 ) rhsd2 => rhsdbuf2

         if ( icompl>1 ) then

!        --- Complex case, make relation between usol, rhsd and zusol, zrhsd

            call sepsolzusol ( usol, rhsd )

         end if  ! ( icompl>1 )

      else

!     --- Renumbering, hence new temporary arrays
!         Create and store renumbered arrays

         allocate ( usol(nusol*icompl), rhsd(nusol*icompl), stat = error )
         if ( error/=0 ) call eralloc ( error, 2*nusol*icompl, 'usol' )
         if ( ierror/=0 ) go to 1000

         usol = usolbuf(1:nusol*icompl)
         rhsd = rhsdbuf(1:nusol*icompl)

         if ( numdef_corr>0 ) rhsd2 = rhsdbuf2(1:nusol*icompl)

         if ( icompl==1 ) then

!        --- Real case

            call seprenumsol ( usol, 0 )
            call seprenumsol ( rhsd, 0 )
            if ( numdef_corr>0 ) call seprenumsol ( rhsd2, 0 )
            if ( iseqres_exact>0 ) call seprenumsol ( resusolex, 1 )
            if ( iseq_exact>0 ) call seprenumsol ( usolex, 1 )
            if ( iseq_start_res>0 ) call seprenumsol ( startres, 1 )

         else

!        --- Complex case

            call sepzrenumsol ( usol, 0, 1 )
            call sepzrenumsol ( rhsd, 0, 2 )
            if ( iseqres_exact>0 ) &
               call sepzrenumsol ( resusolex, 1, 3 )
            if ( iseq_exact>0 ) call sepzrenumsol ( usolex, 1, 4 )
            if ( iseq_start_res>0 ) &
               call sepzrenumsol ( startres, 1, 5 )

         end if  ! ( icompl==1 )

      end if  ! ( indprh==0 )

      if ( IM%simple ) call sepfillsolmods  ! simple

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillsolmod1'

      end if  ! ( debug )
      call erclos ( 'sepfillsolmod1' )

      end subroutine sepfillsolmod1
