      subroutine sepfillsolsimp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters from inpsol related to simple into sepmodulesolve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     simple
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecons
      use sepmodulesolve
      use sepmodulematstr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1405   This combination of the storage scheme and the preconditioner
!           is not implemented.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolsimp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolsimp'
         write(irefwr,1) 'iseqintmat', iseqintmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      schur = typesolv==16

      iseqqpre = inpsol(40,2)     ! sequence number of preconditioner wrt intmat
      if ( iseqqpre<=0 ) iseqqpre = iseqintmat
      iseqdgpre = inpsol(40,3)   ! sequence number for dg matrix
      if ( iseqdgpre<=0 ) iseqdgpre = iseqintmat
      iseqpmpre = inpsol(40,4)   ! sequence number for dg matrix
      if ( iseqpmpre<=0 ) iseqpmpre = iseqintmat

      iprecq = mod(inpsol(4,2),100)       ! Type of preconditioner

      if ( iprecq==6 ) then

!     --- iprecq = 6, default preconditioner

         if ( IM%Qsymmetric ) then

!        --- Q matrix is symmetric

            iprecq = 2

         else

!        --- Q matrix is non-symmetric

            iprecq = 3

         end if

      end if
      if ( iprecq>7 ) then

!     --- Preconditioning is not yet available

         call errint ( IM%jmethod, 1 )
         call errint ( iprecq, 2 )
         call errwar ( 1405, 2, 0, 0 )
         iprecq = 2

      end if

      inpsol(4,2) = iprecq+100*(inpsol(4,2)/100)

      if ( .not. schur ) then

!     --- Standard method

         iprecr = mod(inpsol(4,3),100)
         if ( iprecr==6 ) then

!        --- iprecr = 6, default preconditioner

            if ( IM%Gsymmetric ) then

!           --- R matrix is symmetric

               iprecr = 2

            else

!           --- R matrix is non-symmetric

               iprecr = 3

            end if

         end if
         if ( iprecr>7 ) then

!        --- Preconditioning is not yet available

            call errint ( IM%jmethod, 1 )
            call errint ( iprecr, 2 )
            call errwar ( 1405, 2, 0, 0 )
            iprecr = 2

         end if
         inpsol(4,3) = iprecr+100*(inpsol(4,3)/100)

      else

!     --- Schur complement

         iprecr = mod(inpsol(4,4),100)
         if ( iprecr==6 ) iprecr = 2  ! default preconditioner

         if ( iprecr>7 ) then

!        --- Preconditioning is not yet available

            call errint ( 5, 1 )
            call errint ( iprecr, 2 )
            call errwar ( 1405, 2, 0, 0 )
            iprecr = 2

         end if
         inpsol(4,4) = iprecr+100*(inpsol(4,4)/100)

      end if  ! ( .not. schur )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqqpre, iseqdgpre', iseqqpre, iseqdgpre
         write(irefwr,*) 'End sepfillsolsimp'

      end if  ! ( debug )
      call erclos ( 'sepfillsolsimp' )

      end subroutine sepfillsolsimp
