      subroutine sepfillsolve
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters from inpsol and rinsol into sepmodulesolve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulecons
      use sepmodulesolve
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec, iposst
      type (rmatrix), pointer :: A
      type (intmatrix), pointer :: IM

!     A              Structure of type matrix containing the necessary
!                    information of the matrix
!     IM             Refers to intmt(intmat)
!     iposst         User input parameter consisting of two parts iposst and
!                    itrans according to:
!                    iposst = ipos + 10 * itrans
!     iprec          Type of preconditioning
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLOVERRL  Fill parameters from inpsol and rinsol into sepmodulesolve
!                    Special case: overrelaxation
!     SEPFILLSOLPROJ Fill parameters from inpsol into sepmoduleproj
!     SEPFILLSOLSIMP Fill parameters from inpsol related to simple into
!                    sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolve'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      iposst = inpsol(2,1)
      itrans = iposst / 10
      ipos   = iposst - 10 * itrans
      typesolv = inpsol(3,1)
      ipreco   = inpsol(4,1)
      if ( typesolv==0 .or. typesolv==19 ) ipreco = 0
      itermethod = iposst
      keep     = inpsol(10,1)
      irescomp = inpsol(12,1)
      imetproj = 0
      if ( typesolv==1 ) imetproj = inpsol(18,1)
      numdef_corr = inpsol(21,1)
      iseq_exact = inpsol(23,1)
      iseqres_exact = inpsol(24,1)
      iseq_start_res = inpsol(28,1)
      iseq_end_res = inpsol(29,1)
      ilimit   = inpsol(32,1)
      iscaling = inpsol(34,1)
      num_levels = inpsol(41,1)
      typeparsolv = inpsol(42,1)
      nmax = max(inpsol(5,1),inpsol(43,1))

      iseqprerd = inpsol(40,1)

      posdev = inpsol(2,1)>0
      matsym = inpsol(7,1)>0
      newversion = inpsol(44,1)==0
      skipzeros = inpsol(39,1)>0
      iseqpre = iseqintmat
      if ( iseqprerd>0 ) iseqpre = iseqprerd
      iseqqpre = iseqintmat
      iseqdgpre = iseqintmat

      epsacc = rinsol(1,1)
      lapshift = rinsol(8,1)
      lapshiftim = rinsol(9,1)

      if ( typesolv==19 ) then

!     --- mumps linear solver

         mumps_ren = inpsol(8,1)

      end if  ! ( typesolv==19 )

      if ( typesolv==20 ) then

!     --- Petsc linear solver

         abs_accuray = rinsol(11,1)
         rel_accuray = rinsol(12,1)
         div_accuray = rinsol(13,1)
         dampingfact = rinsol(14,1)
         emin = rinsol(15,1)
         emax = rinsol(16,1)
         iprecpetsc = inpsol(4,1)

      end if  ! ( typesolv==20 )

      if ( abs(lapshiftim)>0 ) then
         clapshift = cmplx ( lapshift, lapshiftim, kind=8 )
      else
         clapshift = lapshift
      end if  ! ( abs(lapshiftim)>0 )

!     --- Parameters for overelaxation

      if ( typesolv==5 ) call sepfilloverrl

!     --- Parameters for simple

      iseqpmpre = iseqintmat
      if ( IM%simple ) call sepfillsolsimp
      if ( imetproj>0 ) call sepfillsolproj

      A => realmt(iseqmt)%S_i

!     --- Store type of matrix

      A%type_prec = 0
      A%typesolv = typesolv
      A%type_storage = jmethod
      A%symmetric = IM%symmetric
      A%nproj = 0
      A%matmultsp = 0
      A%keep = keep
      A%iprint = inpsol(6,1)     ! print level
      A%factor = rinsol(3,1)
      A%adapt = rinsol(2,1)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillsolve'

      end if  ! ( debug )
      call erclos ( 'sepfillsolve' )

      end subroutine sepfillsolve
