      subroutine sepfillsolved
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters in sepmodulesolve that are used by the direct solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     direct_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iposst

!     iposst         User input parameter consisting of two parts iposst and
!                    itrans according to:
!                    iposst = ipos + 10 * itrans
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     776   ITRANS has wrong value (<0 or >2)
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolved' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolved'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposst = 0
      printlev = 0
      skipzeros = .false.
      if ( inpsol(1,1)>=2 ) iposst = inpsol(2,1)
      if ( inpsol(1,1)>=6 ) printlev = inpsol(6,1)
      if ( inpsol(1,1)>=39 ) skipzeros = inpsol(39,1)>0
      itrans = iposst / 10
      ipos   = iposst - 10 * itrans
      iseqres_exact = 0
      iseq_exact= 0
      iseq_start_res = 0
      iseqsol = 1

!     --- Check the input parameters

      if ( itrans<0 .or. itrans>2 ) then
         call errint ( itrans, 1 )
         call errsub ( 776, 1, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposst, itrans', iposst, itrans
         write(irefwr,*) 'End sepfillsolved'

      end if  ! ( debug )
      call erclos ( 'sepfillsolved' )

      end subroutine sepfillsolved
