      subroutine sepgmatvec ( p, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication G p = y (simple)
!     G is gradient matrix and p a pressure vector
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
!     real
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: p(np)
      double precision :: y(nu)

!     p              i    vector to be multiplied
!     y              o    result Gp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision xi

!     i              Counting variable
!     j              Counting variable
!     k              Column number
!     xi             x(i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgmatvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array y

      y = 0d0

      do i = 1, np

!     --- for all non-zero elements d_ij do
!            y_j = y_j + g_ij x_i

         xi = p(i)

         do j = G%rows(i)+1, G%rows(i+1)
            k = G%cols(j)
            y(k) = y(k) + G%GRAD(j)*xi
         end do  ! j = G%rows(i)+1, G%rows(i+1)

      end do  ! i = 1, np

      if ( parallel .and. typecoupl==0 ) then
         length = size(Q%sendinfo)
         call sepparglob ( Q, y, Q%neq )
      end if  ! ( parallel .and. typecoupl==0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgmatvec'

      end if  ! ( debug )
      call erclos ( 'sepgmatvec' )

      end subroutine sepgmatvec
