      subroutine seplinsolbck ( compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber solution vectors back for subroutine linsol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     reordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulesolve
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPRENUMSOLBCK Renumber solution vector back to original form
!                    Real case
!     SEPZRENUMSOLBC Renumber solution vector back to original form
!                    Complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplinsolbck' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplinsolbck'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( compl ) then

!     --- Complex case

  !       call sepzrenumsolbc ( usol, 1 )
         if ( iseqres_exact>0 ) call sepzrenumsolbc ( resusolex, 0 )
         if ( iseq_exact>0 ) call sepzrenumsolbc ( usolex, 0 )
         if ( iseq_start_res>0 ) call sepzrenumsolbc ( startres, 0 )

      else

!     --- Real case

  !       call seprenumsolbck ( usol, 1, usol )
         if ( iseqres_exact>0 ) call seprenumsolbck ( resusolex, 0, resusolex )
         if ( iseq_exact>0 ) call seprenumsolbck ( usolex, 0, usolex )
         if ( iseq_start_res>0 ) call seprenumsolbck ( startres, 0, startres )

      end if  ! ( icompl==1 )

1000  call erclos ( 'seplinsolbck' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplinsolbck'

      end if  ! ( debug )

      end subroutine seplinsolbck
