      subroutine seplinsolkprob ( irhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill relevant parameters into sepmodulekprob for subroutine linsol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: irhsd


!     irhsd          i    refers to rhs vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplinsolkprob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplinsolkprob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some relevant parameters and store in sepmodulekprob

      iprob  = ks(irhsd)%iprob
      iseqkp = ks(irhsd)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobhinv => kprobpart%kprobhinv
      indprh = kprobloc(50)
      nrusol = kprobloc(66)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplinsolkprob'

      end if  ! ( debug )
      call erclos ( 'seplinsolkprob' )

      end subroutine seplinsolkprob
