      subroutine seploctrn ( itrans, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-02-2016 New body
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform a the boundary conditions of a solution vector into
!     the transformed form in case of local transform or vice-versa
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: itrans
      integer, intent(inout) :: isol

!     isol      i/o   Standard SEPRAN array containing information of the
!                     solution array
!                     At input the array must refer to the untransformed vector
!                     at output this vector is transformed (or back)
!     itrans     i    Indicates if transformation from standard vector (i.e.
!                     usual degrees of freedom, for example Cartesian) to
!                     transformed vector (i.e. with transformed degrees of
!                     freedom in points with local transform, for example normal
!                     and tangential components) is defined (1) or that the
!                     backwards transformation to standard vector must be
!                     computed (2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: compl

!     compl      indication whether the arithmetic is real (false)
!                or complex (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPLOCTRNCM    Compute (back-)transformation in case of local
!                    transformations ( complex case )
!     SEPLOCTRNRL    Compute (back-)transformation in case of local
!                    transformations ( real case )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      90   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
!    1445   itrans has a wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check parameters
!     Compute iprob, npltra
!     if ( npltra>0 ) then
!        transform
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seploctrn' )

!     --- Check parameters

      if ( itrans<1 .or. itrans>2 ) then
         call errint ( itrans, 1 )
         call errsub ( 1445, 1, 0, 0 )
      end if
      if ( ks(isol)%typevec/=110 ) then
         call errint ( ks(isol)%typevec, 1 )
         call errsub ( 90, 1, 0, 0 )
      end if

!     --- Compute iprob and fill sepmodulekprob

      iprob = ks(isol)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobf => kprobpart%kprobf
      kprobh => kprobpart%kprobh
      kprobhinv => kprobpart%kprobhinv
      indprf = kprobloc(19)
      indprh = kprobloc(50)
      call sepfillkprob

!     --- Activate arrays

      usoltemp => ks(isol)%sol

      compl = ks(isol)%compl==1

      if ( compl ) then

!     --- Vector is complex  call prsbcc

         call seploctrncm ( itrans, usoltemp )

      else

!     --- Vector is real  call seploctrnrl

         call seploctrnrl ( itrans, usoltemp )

      end if

      call erclos ( 'seploctrn' )
      end subroutine seploctrn
