      subroutine sepmakeelm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create element matrix and element vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0001         Special element for boundary conditions of the type
!                    u=constant
!     ELMSUBS        Call specific element subroutine, depending on itype
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1479   Matrix storage is symmetric, whereas matrix is non-symmetrical
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmakeelm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmakeelm'
         write(irefwr,*) 'matrix, oldsol1', matrix, oldsol1
         write(irefwr,1) 'itype, isubcall', itype, isubcall
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( matrix .and. itype<0 ) then

!     --- itype<0, boundary element of the type u is constant

         call el0001

      else

!     --- itype>0, standard elements

         E%first = ifirst
         if ( irelem==inelem .and. isubel==nsubel ) E%last = 1 ! last element
         jmethod = IM%jmethod  ! reset for shifted laplace

!        --- temporary

         E%iflipflop = flipflop(ilocelem)
         call elmsubs
         if ( itype>=700 .and. itype<=800 ) flipflop(ilocelem) = E%iflipflop

      end if
      if ( ierror/=0 ) go to 1000

      if ( symsubdiv ) then

!     --- In case of linear prismatic elements that are subdivided
!         into 2 groups of 3 tetrahedrons we
!         divide each element matrix and element vector by 6
!         This is a very ad-hoc approach

         elemvc = 0.5d0*elemvc
         elemmt = 0.5d0*elemmt

      end if  ! ( symsubdiv )

!     --- Set matrix and vector if necessary

      if ( E%notvec==1 .or. notvec==1 ) vector = .false.
      if ( notmat==1 ) matrix = .false.
      if ( matrix .and. ifirst==0 ) then

!     --- Check if matrix is symmetrical

         if ( .not. symm .and. &
              ( IM%symmetric .or. IM%Gsymmetric ) ) then
            call errint ( IM%jmethod, 1 )
            call errsub ( 1479, 1, 0, 0 )
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmakeelm'

      end if  ! ( debug )
      call erclos ( 'sepmakeelm' )

      end subroutine sepmakeelm
