      subroutine sepmatmult ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Apply matrix vector multiplication y = A * x
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(in) :: A
      double precision, intent(in) :: x(nusol)
      double precision :: y(nusol)


!     A              i    matrix of type mat to be used in multiplication
!     x              i    vector to be multiplied
!     y              o    result vector of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMAVER1      Matrix vector multiplication y_i = S_ii*x_i (real)
!     SEPMULTBCOND1  Compute y = y + S_ip x_p in real case
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatmult' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatmult'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      y = 0d0
      if ( typecoupl==0 ) typecoupl = 2

      call sepmaver1 ( A, x, y )

      if ( nbound>0 .and. .not. A%diagonal ) &
         call sepmultbcond1 ( A, x, y )

      if ( parallel .and. typecoupl==1 ) then

!     --- Add contributions of all cpu's to common nodes

         call sepparglob ( A%S_i, y, nusol )

      end if  ! ( parallel )

      if ( typecoupl==2 ) typecoupl = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatmult'

      end if  ! ( debug )
      call erclos ( 'sepmatmult' )

      end subroutine sepmatmult
