      subroutine sepmatvecsim ( x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication for a complete simple matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
!     real
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: x(nu+np)
      double precision, intent(out) :: y(nu+np)

!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The matrix vector multiplication is written as
!
!         | Q  G |
!     A = |      |
!         | D  C |
!
!                 | Qu + Gp |
!     hence A x = |         |
!                 | Du + Cp |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatvecsim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatvecsim'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      y(1:nu) = Q*x(1:nu)
      if ( debug ) call prinrl ( y, nu, 'Qu' )
      worksim(1:nu) = G*x(nu+1:nu+np)
      if ( debug ) call prinrl ( worksim, nu, 'Gp' )

      y(1:nu) = y(1:nu)+worksim(1:nu)
      if ( debug ) call prinrl ( y, nu, 'f_u' )

      y(nu+1:nu+np) = D*x(1:nu)
      if ( debug ) call prinrl ( y(nu+1), np, 'Du' )

      worksim(1:np) = C*x(nu+1:nu+np)
      if ( debug ) call prinrl ( worksim, np, 'C*p' )

      y(nu+1:nu+np) = y(nu+1:nu+np)+worksim(1:np)
      if ( debug ) call prinrl ( y(nu+1), np, 'f_p' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatvecsim'

      end if  ! ( debug )
      call erclos ( 'sepmatvecsim' )

      end subroutine sepmatvecsim
