      subroutine sepmultbcond ( x, y, jsign )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     incorporate effect of boundary conditions
!     Matrix real, vectors real
!     Old subroutine er0017
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: jsign
      double precision, intent(in) :: x(*)
      double precision :: y(*)

!     jsign          i    Multiplication sign, if 1 the effect is added to the
!                         existing vector, if -1 it is subtracted
!     x              i    input vector to be multiplied
!     y             i/o   The vector y should exist at input
!                         Depending on isign the effect of the multiplication is
!                         added to or subtracted from the original vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jk

!                    they are not
!     i              Counting variable
!     j              Counting variable
!     jk             Help variable to store an index
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0017         print matrix corresponding to boundary conditions
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmultbcond' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmultbcond'
         write(irefwr,1) 'nusol, nbound, jsign', nusol, nbound, jsign
         call prinrl ( x(nusol-nbound+1), nbound, 'x-in')
         call prinrl ( y, nusol, 'y-in')
         write(irefwr,*) 'amat'
         call pr0017 ( S_ip, irowsip, icolsip, nbound, nusol )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nbound
         do j = irowsip(i)+1, irowsip(i+1)
           jk = icolsip(j)
           y(jk) = y(jk)+jsign*S_ip(j)*x(nusol-nbound+i)
         end do  ! j = irows(i)+1, irows(i+1)
      end do  ! i = 1, nbound

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, nusol, 'y-out')
         write(irefwr,*) 'End sepmultbcond'

      end if  ! ( debug )
      call erclos ( 'sepmultbcond' )
      end subroutine sepmultbcond
