      subroutine sepmultbcond1 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = y + A x_p in real case
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr1
      use sepmodulevecs
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(in) :: A
      double precision, intent(in) :: x(A%S_p%n)
      double precision :: y(A%S_p%n)

!     A              i    matrix of type mat to be used in multiplication
!     x              i    vector to be multiplied
!     y             i/o   result vector of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jk

!     i              Counting variable
!     j              Counting variable
!     jk             Help variable to store an index
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmultbcond1' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmultbcond1'
         write(irefwr,1) 'nbound', A%S_p%neq
         call prinrl ( x(A%S_p%n-A%S_p%neq+1), A%S_p%neq, 'x-in')
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinin ( A%S_p%rows, A%S_p%neq+1, 'A%S_p%rows' )
         call prinin ( A%S_p%cols, size(A%S_p%cols), 'A%S_p%cols' )
      end if  ! ( debug )

      do i = 1, A%S_p%neq
         do j = A%S_p%rows(i)+1, A%S_p%rows(i+1)
           jk = A%S_p%cols(j)
           y(jk) = y(jk)+A%S_p%Bip(j)*x(A%S_p%n-A%S_p%neq+i)
         end do  ! j = irows(i)+1, irows(i+1)
      end do  ! i = 1, A%S_p%neq

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%S_p%n, 'y/2' )
         write(irefwr,*) 'End sepmultbcond1'

      end if  ! ( debug )
      call erclos ( 'sepmultbcond1' )

      end subroutine sepmultbcond1
