      subroutine sepoverrl ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the actual overrelaxation process
!     Old subroutine prov00
! **********************************************************************
!
!                       KEYWORDS
!
!     constraint
!     linear_solver
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: minlevel(:), maxlevel(:)
      logical :: upper, lower

!     lower          Indicates if there is a constraint of the type u(x)>=a(x)
!     maxlevel       Work array to store the constraint b(x)
!     minlevel       Work array to store the constraint a(x)
!     upper          Indicates if there is a constraint of the type u(x)<=b(x)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPOVERRLLOW   Fill array minlevel for overrelaxation
!     SEPOVERRLUPP   Fill array maxlevel for overrelaxation
!     SEPSOLOVEROUT  perform the actual overrelaxation process
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepoverrl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepoverrl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the parameters lower and upper

      lower = A%iconstrain==1 .or. A%iconstrain==3
      upper = A%iconstrain==2 .or. A%iconstrain==3

      allocate ( minlevel(nusol), maxlevel(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nusol, 'minlevel' )
      if ( ierror/=0 ) go to 1000

      if ( lower ) call sepoverrllow ( A, minlevel )  ! Fill minlevel

      if ( upper ) call sepoverrlupp ( A, maxlevel )  ! Fill maxlevel

      call sepsoloverout ( A, minlevel, maxlevel, usol, rhsd )

      deallocate ( minlevel, maxlevel, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'minlevel' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepoverrl'

      end if  ! ( debug )
      call erclos ( 'sepoverrl' )

      end subroutine sepoverrl
