      subroutine sepprecd10 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix for the CGS-process
!     ipreco = 1, diagonal scaling
!     jmethod = 9, special storage for multigrid
!     Old subroutine prcg02_1_9
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulesolve
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINMTROWCOMP  Print matrix in case of amg row compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecd10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecd10'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ipreco = 1   Diagonal scaling
!         jmethod = 9, special row storage
!         The position of the diagonal is stored in A%diag

      do i = 1, A%neq
         j = A%diag(i)
         if (abs(A%S(j))<=0d0) then
            call errint ( i, 1 )
            call erreal ( abs(A%S(j)), 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if
         A%MD(i) = sqrt(A%S(j))
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinmtrowcomp ( A%MD )
         write(irefwr,*) 'End sepprecd10'

      end if  ! ( debug )
      call erclos ( 'sepprecd10' )

      end subroutine sepprecd10
