      subroutine sepprofbackgg ( A, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-01-2021
!
!   copyright (c) 2021-2021  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofgg
!     Backward solve, hence G^t x = y
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(inout) :: usol(A%neq)

!     A              i    Matrix A
!     usol          i/o   At input: right-hand-side vector
!                         At output: solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pi, i, ii
      double precision :: som

!     i              Counting variable
!     ii             Position of diagonal element ii
!     pi             Column number of at most left non-zero element in row i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      do i = A%rowmax, A%rowmin, -1
         ii=A%Mrows(i)
         usol(i)=usol(i)/A%MD(ii)
         pi=A%Mcols(i)
         if(pi<i) &
            call daxpy(i-pi,-usol(i),A%MD(ii-i+pi),1,usol(pi),1)
      end do
      end subroutine sepprofbackgg
