      subroutine sepprojc ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = U E^-1 U^T x (coarse grid projection)
!     Old subroutine sepcproj
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision, intent(out) :: y(A%neq)

!     A             i/o   Matrix A
!     x              i    Vector to be projected
!     y              o    result of projection
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: work(A%nproj)

!     work           Work array to store intermediate results
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPPROJEINV    Compute y = E^-1 x, where E is the matrix (AP)^T P as part
!                    of projection. P is the matrix of projection vectors
!     SEPPROJU       Compute y = U x as part of the projection algorithm
!                    U is the matrix of projection vectors
!     SEPPROJUT      Compute y = U^T x as part of the projection algorithm
!                    U is the matrix of projection vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Definitions:
!
!     U = [ u1 u2 .. uNPROJ ],
!
!     where u1,..,uNPROJ are the projection vectors.
!
!     E = (AU)^T U.
!
!     The coarse grid correction is defined by
!
!     P x = U E^-1 U^T x
!     The projection is defined as follows:
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprojc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprojc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- work = U^T x

      call sepprojUT ( A, x, work )

      if ( debug ) call prinrl ( work, A%nproj, 'work/1' )

!     --- work = E^-1 work

      call sepprojEinv ( A, work, work )

      if ( debug ) call prinrl ( work, A%nproj, 'work/2' )

!     --- y = U work

      call sepprojU ( A, work, y )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y')
         write(irefwr,*) 'End sepprojc'

      end if  ! ( debug )
      call erclos ( 'sepprojc' )

      end subroutine sepprojc
