      subroutine sepsimpledel ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete all temporary created arrays in the simple process
!
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     memory_management
!     delete
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimpledel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimpledel'

      end if

!     --- Delete temporary created arrays in reversed sequence

      if ( sizedgmatpre>0 ) deallocate ( S_Rpre, stat = error )
      if ( sizeqmatpre>0 ) deallocate ( S_Qpre, stat = error )

      if ( associated(resup) ) deallocate ( resup, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'resup' )

      if ( associated(worksim) ) deallocate ( worksim, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'worksim' )

      if ( associated(worky1) ) deallocate ( worky1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'worky1' )

      if ( associated(worky2) ) deallocate ( worky2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'worky2' )

      if ( associated(S_R) ) deallocate ( S_R, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'S_R' )
      if ( parallel .or. .not. (A%typesolv==13 .or. A%typesolv==14) ) then
         if ( associated(S_Qprime) ) deallocate ( S_Qprime, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'S_Qprime' )
      end if  ! ( parallel )
      nullify ( uvels )
      nullify ( pvels )
      nullify ( ru )
      nullify ( rp )
      nullify ( fu )
      nullify ( fp )
      nullify ( S_Qprime )

      if ( debug ) write(irefwr,*) 'End sepsimpledel'
      call erclos ( 'sepsimpledel' )

      end
