      subroutine sepsimplepar
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create two arrays sendinfo_u and sendinfo_p from sendinfo
!     in case of parallel computing
!     Old subroutine prsimplepar
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     simple
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leninnercontrp, leninnercontru, leninner
      logical :: filled

!     filled         If true the parts sendinfop etc have already been filled
!     leninner       Length of array innercontr
!     leninnercontrp Length of array innercontrp
!     leninnercontru Length of array innercontru
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSIMPLEPAR1   Copy contents of global sendinfo into sendinfo for pressure
!                    and velocity
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create temporary arrays with the same length as sendinfo
!     Copy information from sendinfo to these arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimplepar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplepar'
         write(irefwr,1) 'nu', nu
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob
      kprobloc => kprobpart%kprobloc

!     --- Find shifts and length of subarrays

      filled = associated(kprobpart%innercontru)
      leninner = size(kprobpart%innercontr)
      if ( debug ) then
         write(irefwr,*) 'filled', filled
         write(irefwr,1) 'leninner', leninner
      end if  ! ( debug )

      if ( filled ) go to 1000  ! array has already been filled

!     --- Create temporary arrays

      allocate ( innercontru(leninner), innercontrp(leninner), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*leninner, 'sendinfou' )
      if ( ierror/=0 ) go to 1000

!     --- Copy information from sendinfo to these arrays

      call prsimplepar1 ( nu, np, leninnercontrp, leninnercontru )

      if ( debug ) &
         write(irefwr,1) 'leninnercontrp, leninnercontru', &
                          leninnercontrp, leninnercontru
      allocate ( kprobpart%innercontru(leninnercontru) )
      kprobpart%innercontru = innercontru(1:leninnercontru)
      allocate ( kprobpart%innercontrp(leninnercontrp) )
      kprobpart%innercontrp = innercontrp(1:leninnercontrp)
      deallocate ( innercontru, innercontrp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsimplepar'

      end if  ! ( debug )
      call erclos ( 'sepsimplepar' )

      end
