      subroutine sepsollltforw ( n, amat, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2021
!
!   copyright (c) 2021-2021  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform forward solve for a full symmetric positive
!     definite real matrix that has been decomposed by sepdecllt
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     positive_definite
!     real
!     symmetric
!     full_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: n
      double precision, intent(in) :: amat((n*n+n)/2)
      double precision, intent(inout) :: usol(n)

!     amat           i    Decomposed matrix L
!     n              i    number of unknowns
!     usol          i/o   At input: right-hand side vector f
!                         At output: L^-1 f
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ii

!     i              General loop variable
!     ii             Position of diagonal element i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!debug      call eropen ( 'sepsollltforw' )
!debug      debug = .false.! .and. ioutp>=0
!debug      if ( debug ) then
!debug
!debug!     --- Debug information
!debug
!debug         write(irefwr,*) 'Debug information from sepsollltforw'
!debug         call prinrl ( amat, (n*n+n)/2, 'amat' )
!debug         call prinrl ( usol, n, 'rhs' )
!debug
!debug      end if  ! ( debug )
!debug      if ( ierror/=0 ) go to 1000

      ii = 1
      do i = 1, n
         usol(i) = ( usol(i)-ddot(i-1, amat(ii-i+1), 1, usol(1), 1 ) ) / amat(ii)
         ii = ii+i+1
      end do  ! i = 1, n
!debug
!debug1000  if ( debug ) then
!debug
!debug!     --- Debug information
!debug
!debug         call prinrl ( usol, n, 'x' )
!debug         write(irefwr,*) 'End sepsollltforw'
!debug
!debug      end if  ! ( debug )
!debug      call erclos ( 'sepsollltforw' )

      end subroutine sepsollltforw
