      subroutine sepzprofile ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of complex linear equations by Gaussian elimination
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     complex
!     direct_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulecmat
      use sepmodulematr
      use sepmodulesolve
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: decomp

!     decomp         Logical parameter indicating if the decomposition of
!                    the matrix must be computed (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPZPROFILEDEC Compute lU decomposition for system of complex
!                    linear equations
!     SEPZPROFILESOL Solve system of complex linear equations by Gaussian
!                    elimination
!                    The matrix is supposed to be decomposed before
!     SEPZRENUMSOLBC Renumber solution vector back to original form
!                    Complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepzprofile' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzprofile'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%decomp ) call sepzprofiledec ( A )  ! Compute decomposition
      A%decomp = .true.

!     --- Next solve system of equations

      call sepzprofilesol ( A )

!     --- Copy solution array back into buffer

      call sepzrenumsolbc ( usol, 1 )

!     --- Deallocate arrays

      if ( indprh>0 ) then
         deallocate ( usol, rhsd, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'usol' )
      else
         nullify(usol)
         nullify(rhsd)
      end if  ! ( indprh>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepzprofile'

      end if  ! ( debug )
      call erclos ( 'sepzprofile' )

      end subroutine sepzprofile
