      subroutine solvelcheck
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-02-2016 New parameters list
!        version  1.2  date 28-07-2014 Extension with petsc
!        version  1.1  date 10-04-2014 Extension with mumps
!
!   copyright (c) 2008-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some checks for the linear solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulecpack
      use sepmodulematr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: iinsol(:)
      logical :: iterative
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iinsol         Refers to inpsol(:,1)
!     iterative      If true an iterative method is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2273   Parallel solution is only possible for iterative methods
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'solvelcheck' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from solvelcheck'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      iinsol => inpsol(:,1)

      iterative = .not. IM%direct .or. IM%mumps
      if ( parallel .and. .not. iterative ) &
         call errsub ( 2273, 0, 0, 0 )

      if ( IM%direct .and. .not. IM%mumps .or. realmt(iseqmt)%diagonal ) then

         if ( iinsol(3)/=0 ) then

!        --- iterative method required, whereas IM%direct true

            iinsol(3) = 0

         end if

      else if ( iinsol(3)==0 .and. IM%mumps ) then

!     --- MUMPS storage

         iinsol(3) = 19

      else if ( IM%petsc ) then

!     --- Petsc storage

         iinsol(2) = iinsol(3)  ! Copy old value of method into iinsol(2)
         iinsol(3) = 20

      else if ( iinsol(3)==0 ) then

!     --- direct method required, whereas IM%direct true
!         Make method iterative

         iinsol(3) = 1  ! cg
         iinsol(9) = 1  ! start is old solution

      end if  ! ( IM%direct .and. .not. IM%mumps .or. realmt(iseqmt)%diagonal )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinsol(3)', iinsol(3)
         write(irefwr,*) 'End solvelcheck'

      end if  ! ( debug )
      call erclos ( 'solvelcheck' )

      end
