      subroutine to0005
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.1  date 11-12-2017 New storage of intmat
!        version  7.0  date 24-07-2015 Use sepmodulebuild
!        version  6.7  date 21-04-2015 New definition intmat
!
!   copyright (c) 1982-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the length of some arrays corresponding to the matrix
!     and the right-hand side
!     Help subroutine for subroutine TO0050  (BUILD)
! **********************************************************************
!
!                       KEYWORDS
!
!     array_length
!     initialization
!     mass_matrix
!     matrix
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      integer, pointer :: matr(:), massmtr, irhsd(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     irhsd          Standard SEPRAN array, containing information of the
!                    large vector
!     k              Counting variable
!     massmtr         SEPRAN array, containing information of the mass
!                    matrix
!     matr           Standard SEPRAN array, containing information of the
!                    large matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0005' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0005'
         write(irefwr,*) 'clrmat, addmat', clrmat, addmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      matr => stiffmat
      massmtr => massmat
      irhsd => SMF%irhsd

      IM => intmt(intmat)

      if ( debug ) write(irefwr,1) 'iprob', iprob

      nsol = nusol

      if ( compl ) nsol = nsol*2

      nusolcac = nusol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0005'

      end if  ! ( debug )
      call erclos ( 'to0005' )

      end subroutine to0005
