      subroutine to0009
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 24-07-2015 Use sepmodulebuild
!        version  7.4  date 05-12-2012 Long integers
!        version  7.3  date 23-03-2011 Extra debug
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for one element group in subroutine build
!
! **********************************************************************
!
!                       KEYWORDS
!
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iboun1, i

!     i              Counting variable
!     iboun1         boundary element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1586   Subdivision of elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0009' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0009'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get information

      jnpelm = npelm
      itype = kprobc(ielgrp)
      notvec = 0
      notmat = 0
      ifirst = 0
      nsubel = 1
      inpelm = jnpelm
      symsubdiv = .false.
      E%npress = 1

      if ( indpra>0 ) kprobatmp => kprobpart%kproba(ielgrp)%arr

      if ( debug ) then

         write(irefwr,1) 'ielgrp, nelgrp, nsubel, inpelm, isubdv', &
                          ielgrp, nelgrp, nsubel, inpelm, isubdv
         write(irefwr,1) 'itype', itype

      end if  ! ( debug )

      if ( ielgrp<=nelgrp ) then

!     --- ielgrp<=nelgrp  standard element from kmesh

         ishape = kmeshf(ielgrp)
         ishapeorg = ishape
         inelem = Topgrp(ielgrp)%inelem
         jnpelm = Topgrp(ielgrp)%inpelm
         inpelm = jnpelm
         if ( debug ) &
            write(irefwr,1) 'ishape, inpelm, isubdv', &
                             ishape, inpelm, isubdv
         if ( isubdv==1 ) then

!        --- isubdv=1, quadratic elements must be subdivided

            select case ( ishape )

               case (8:10, 14, 17:)

!              --- Error subdivision not implemented

                  call errint ( ishape, 1 )
                  call errsub ( 1586, 1, 0, 0 )
                  go to 1000

               case (-10:1, 3, 5, 11, 13)

!              --- shape = 1, linear elements

                  nsubel = 1

               case (2)

!              --- shape = 2, quadratic line element

                  ishape = 1
                  inpelm = 2
                  nsubel = 2

               case (4,7)

!              --- shape = 4, 7, quadratic triangle

                  ishape = 3
                  inpelm = 3
                  nsubel = 4

               case (6)

!              --- shape = 6, bi-quadratic quadrilateral

                  ishape = 5
                  inpelm = 4
                  nsubel = 4

               case (12, 15, 16)

!              --- shape = 12, 15, 16, tri-quadratic tetrahedron

                  ishape = 11
                  inpelm = 4
                  nsubel = 8

            end select  ! case ( ishape )

            if ( debug ) &
               write(irefwr,1) 'ishape, inpelm, nsubel', &
                                ishape, inpelm, nsubel

         else if ( itype==903 .and. ishape==5 ) then

!        --- Bilinear quadrilateral: at this moment it will be subdivided into
!            2*2 triangles in order to get a symmetric solution

            ishape = 3
            inpelm = 3
            nsubel = 4
            symsubdiv = .true.

         else if ( itype==903 .and. ishape==21 ) then

!        --- Linear symsubdiv: at this moment it will be subdivided into
!            6*3 tetrahedrons

            ishape = 11
            inpelm = 4
            nsubel = 6
            symsubdiv = .true.

         end if
         nodes => kmeshc

!        --- Compute npress

         E%ishape = ishape
         E%ndim = ndim
         call sepelnpress

      else

!     --- ielgrp>nelgrp   standard boundary element

         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobb => kprobpart%kprobb
         ishape = 0
         iboun1 = ielgrp-nelgrp
         jnpelm = kprobj(iboun1,1)
         inpelm = jnpelm
         ishape = kprobj(iboun1,2)
         if ( ishape==0 ) then
            if ( ndim>=2 .and. inpelm==3 ) ishape = 3
         end if
         ishapeorg = ishape
         nodes => kprobi
         inelem = kprobb(iboun1)
         if ( isubdv==1 ) then

!        --- isubdv=1, quadratic elements must be subdivided

            if ( ishape==1 ) then
               ishape = 1
               inpelm = 2
               nsubel = 1
            else if ( ishape==2 ) then
               ishape = 1
               inpelm = 2
               nsubel = 2
            else if ( ishape==3 ) then
               ishape = 3
               inpelm = 3
               nsubel = 1
            else if ( ishape==4 .or. ishape==7 ) then
               ishape = 3
               inpelm = 3
               nsubel = 4
            else if ( ishape>=8 .and. ishape<=9 .or. &
                      ishape==12 .or. &
                      ishape>=14 .and. ishape<=16 ) then

               call errint ( ishape, 1 )
               call errsub ( 1586, 1, 0, 0 )
               goto 1000

            end if

         end if

      end if
      if ( debug ) &
         write(irefwr,1) 'ishape, nsubel, inpelm, nunkp, itype', &
                          ishape, nsubel, inpelm, nunkp, itype

!     --- compute icount

      if ( itype>0 ) then

!     --- type number>0   standard

         if ( debug ) write(irefwr,1) 'indpra, nunkp', indpra, nunkp
         if ( indpra==0 .or. ielgrp>nelgrp ) then

!        --- indpra = 0

            icount = nunkp*inpelm
            if ( itype==922 ) icount = icount-ndim

         else

!        --- indpra>0

            icount = 0
            do i = 1, inpelm
               icount = icount+kprobatmp(i)
            end do

            if ( debug ) then
               call prinin ( kprobatmp, inpelm , 'kproba' )
               write(irefwr,1) 'icount', icount
            end if  ! ( debug )

         end if

      else

!     --- itype<0   special element

         if ( itype==-2 ) then

!        --- itype = -2    boundary conditions of the type u is constant

            icount = 2

         end if
         notvec = 1

      end if
      E%notvec = notvec
      E%notmat = notmat
      E%icount = icount
      E%last = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ishape, nsubel, inpelm', &
                          ishape, nsubel, inpelm
         write(irefwr,1) 'icount', icount
         write(irefwr,*) 'End to0009'

      end if  ! ( debug )
      call erclos ( 'to0009' )

      end
