      subroutine to0036 ( celemmt, celemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 27-10-2010 Remove ihelp
!        version  2.1  date 20-10-1998 Correction for rhs in 3d
!        version  2.0  date 05-10-1998 Extra parameter ndim
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     transformation of element matrix and element vector if necessary
!     Complex version of to0053
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, intent(inout) :: celemmt(icount,icount), celemvc(icount)

!     celemmt    i/o   Element matrix
!     celemvc    i/o   Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: isum, i, ih, i1, i2, i3, j, ihelp(2,icount)
      double precision :: anx, any, &
                          r11, r12, r13, r21, r22, r23, r31, r32, r33, &
                          v11, v12, v13, v21, v22, v23, v31, v32, v33
      double complex :: s1, s2, s3, f1, f2, f3

!     anx        x-component of normal vector
!     any        y-component of normal vector
!     f1         1-th component of right-hand side vector to be transformed
!     f2         2-th component of right-hand side vector to be transformed
!     f3         3-th component of right-hand side vector to be transformed
!     i          Counting variable
!     i1         1-th component of point in index5
!     i2         2-th component of point in index5
!     i3         3-th component of point in index5
!     ih         Position of point in coortr
!     ihelp      integer work array of size 2*icount
!                Position (1,i) contains the position of the i^th
!                unknown to be transformed with respect to the
!                element vector
!                Position (2,i) contains the corresponding position in
!                coortr
!     isum       Integer variable to compute a sum
!     j          Counting variable
!     r11        R(1,1) component of matrix R
!     r12        R(1,2) component of matrix R
!     r13        R(1,3) component of matrix R
!     r21        R(2,1) component of matrix R
!     r22        R(2,2) component of matrix R
!     r23        R(2,3) component of matrix R
!     r31        R(3,1) component of matrix R
!     r32        R(3,2) component of matrix R
!     r33        R(3,3) component of matrix R
!     s1         1-th component of element matrix to be transformed
!     s2         2-th component of element matrix to be transformed
!     s3         3-th component of element matrix to be transformed
!     v11        V(1,1) component of matrix V
!     v12        V(1,2) component of matrix V
!     v13        V(1,3) component of matrix V
!     v21        V(2,1) component of matrix V
!     v22        V(2,2) component of matrix V
!     v23        V(2,3) component of matrix V
!     v31        V(3,1) component of matrix V
!     v32        V(3,2) component of matrix V
!     v33        V(3,3) component of matrix V
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                             T
!       The tranformation matrix is given by V  S  R, &
!                                    T
!       the tranformation vector by V  f
!       with R and V depending on maxtrn:
!       In R^2:
!                | nx   -ny |              | r11   r12 |
!       2:   R = |          |   4,8:   R = |           |
!                | ny    nx |              | r21   r22 |
!                  T                  | v11   v12 |
!       2,4:  R = R       8:      V = |           |
!                                     | v21   v22 |
!       this means:
!      2:
!        T      |  nx s11 + ny s21       nx s12 + ny s22 |
!       R  S  = |                                        |
!               | -ny s11 + nx s21      -ny s12 + nx s22 |
!               |  s11 nx + s12 ny      -s11 ny + s12 nx |
!       S R   = |                                        |
!               |  s21 nx + s22 ny      -s21 ny + s22 nx |
!        T      |  nx f1 + ny f2 |
!       R f   = |                |
!               | -ny f1 + nx f2 |
!      4:
!        T      |  r11 s11 + r21 s21       r11 s12 + r21 s22 |
!       R  S  = |                                            |
!               |  r12 s11 + r22 s21       r12 s12 + r22 s22 |
!               |  s11 r11 + s12 r21       s11 r12 + s12 r22 |
!       S R   = |                                            |
!               |  s21 r11 + s22 r21       s21 r12 + s22 r22 |
!        T      |  r11 f1 + r21 f2 |
!       R f   = |                  |
!               |  r12 f1 + r22 f2 |
!   and similar expressions for maxtrn = 8
!       In R^3:
!       See 4:, but now extended to three dimensions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ndim==2 ) then

!     --- ndim = 2

         isum = 0
         do i = 1, icount
            ih = iworktr(index5(i))
            if ( ih/=0 ) then
               isum = isum+1
               ihelp(1,isum) = i
               ihelp(2,isum) = (ih+1)/2
            end if
         end do
         if ( isum==0 ) go to 1000

         if ( maxtrn==2 ) then

!        --- Standard transformation

            do i = 2, isum, 2
               i1 = ihelp(1,i-1)
               i2 = ihelp(1,i)
               ih = ihelp(2,i-1)
               anx = coortr(1,ih)
               any = coortr(2,ih)
               if ( matrix ) then

!              --- matrix = true:  compute R^T S R
!                  First the R^T S part

                  do j = 1, icount
                     s1 = celemmt(j,i1)
                     s2 = celemmt(j,i2)
                     celemmt(j,i1) = anx*s1+any*s2
                     celemmt(j,i2) = -any*s1+anx*s2
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = celemmt(i1,j)
                     s2 = celemmt(i2,j)
                     celemmt(i1,j) = anx*s1+any*s2
                     celemmt(i2,j) =-any*s1+anx*s2
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute R^T f

                  f1 = celemvc(i1)
                  f2 = celemvc(i2)
                  celemvc(i1) = anx*f1+any*f2
                  celemvc(i2) =-any*f1+anx*f2

               end if

            end do

         else if ( maxtrn==4 ) then

!        --- Symmetric transformation

            do i = 2, isum, 2
               i1 = ihelp(1,i-1)
               i2 = ihelp(1,i)
               ih = ihelp(2,i-1)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r21 = coortr(3,ih)
               r22 = coortr(4,ih)
               if ( matrix ) then

!              --- matrix = true:  compute R^T S R
!                  First the R^T S part

                  do j = 1, icount
                     s1 = celemmt(j,i1)
                     s2 = celemmt(j,i2)
                     celemmt(j,i1) = r11*s1+r21*s2
                     celemmt(j,i2) = r12*s1+r22*s2
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = celemmt(i1,j)
                     s2 = celemmt(i2,j)
                     celemmt(i1,j) = r11*s1+r21*s2
                     celemmt(i2,j) = r12*s1+r22*s2
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute R^T f

                  f1 = celemvc(i1)
                  f2 = celemvc(i2)
                  celemvc(i1) = r11*f1+r21*f2
                  celemvc(i2) = r12*f1+r22*f2

               end if

            end do

         else

!        --- Non-symmetric transformation

            do i = 2, isum, 2
               i1 = ihelp(1,i-1)
               i2 = ihelp(1,i)
               ih = ihelp(2,i-1)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r21 = coortr(3,ih)
               r22 = coortr(4,ih)
               v11 = coortr(5,ih)
               v12 = coortr(6,ih)
               v21 = coortr(7,ih)
               v22 = coortr(8,ih)
               if ( matrix ) then

!              --- matrix = true:  compute V^T S R
!                  First the V^T S part

                  do j = 1, icount
                     s1 = celemmt(j,i1)
                     s2 = celemmt(j,i2)
                     celemmt(j,i1) = v11*s1+v21*s2
                     celemmt(j,i2) = v12*s1+v22*s2
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = celemmt(i1,j)
                     s2 = celemmt(i2,j)
                     celemmt(i1,j) = r11*s1+r21*s2
                     celemmt(i2,j) = r12*s1+r22*s2
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute V^T f

                  f1 = celemvc(i1)
                  f2 = celemvc(i2)
                  celemvc(i1) = v11*f1+v21*f2
                  celemvc(i2) = v12*f1+v22*f2

               end if

            end do

         end if

      else

!     --- ndim = 3

         isum = 0
         do i = 1, icount
            ih = iworktr(index5(i))
            if ( ih/=0 ) then
               isum = isum+1
               ihelp(1,isum) = i
               ihelp(2,isum) = (ih+2)/3
            end if
         end do
         if ( isum==0 ) go to 1000

         if ( maxtrn==9 ) then

!        --- Symmetric transformation

            do i = 3, isum, 3
               i1 = ihelp(1,i-2)
               i2 = ihelp(1,i-1)
               i3 = ihelp(1,i)
               ih = ihelp(2,i)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r13 = coortr(3,ih)
               r21 = coortr(4,ih)
               r22 = coortr(5,ih)
               r23 = coortr(6,ih)
               r31 = coortr(7,ih)
               r32 = coortr(8,ih)
               r33 = coortr(9,ih)
               if ( matrix ) then

!              --- matrix = true:  compute R^T S R
!                  First the R^T S part

                  do j = 1, icount
                     s1 = celemmt(j,i1)
                     s2 = celemmt(j,i2)
                     s3 = celemmt(j,i3)
                     celemmt(j,i1) = r11*s1+r21*s2+r31*s3
                     celemmt(j,i2) = r12*s1+r22*s2+r32*s3
                     celemmt(j,i3) = r13*s1+r23*s2+r33*s3
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = celemmt(i1,j)
                     s2 = celemmt(i2,j)
                     s3 = celemmt(i3,j)
                     celemmt(i1,j) = r11*s1+r21*s2+r31*s3
                     celemmt(i2,j) = r12*s1+r22*s2+r32*s3
                     celemmt(i3,j) = r13*s1+r23*s2+r33*s3
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute R^T f

                  f1 = celemvc(i1)
                  f2 = celemvc(i2)
                  f3 = celemvc(i3)
                  celemvc(i1) = r11*f1+r21*f2+r31*f3
                  celemvc(i2) = r12*f1+r22*f2+r32*f3
                  celemvc(i3) = r13*f1+r23*f2+r33*f3

               end if

            end do

         else

!        --- Non-symmetric transformation

            do i = 3, isum, 3
               i1 = ihelp(1,i-2)
               i2 = ihelp(1,i-1)
               i3 = ihelp(1,i)
               ih = ihelp(2,i)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r13 = coortr(3,ih)
               r21 = coortr(4,ih)
               r22 = coortr(5,ih)
               r23 = coortr(6,ih)
               r31 = coortr(7,ih)
               r32 = coortr(8,ih)
               r33 = coortr(9,ih)
               v11 = coortr(10,ih)
               v12 = coortr(11,ih)
               v13 = coortr(12,ih)
               v21 = coortr(13,ih)
               v22 = coortr(14,ih)
               v23 = coortr(15,ih)
               v31 = coortr(16,ih)
               v32 = coortr(17,ih)
               v33 = coortr(18,ih)
               if ( matrix ) then

!              --- matrix = true:  compute V^T S R
!                  First the V^T S part

                  do j = 1, icount
                     s1 = celemmt(j,i1)
                     s2 = celemmt(j,i2)
                     s3 = celemmt(j,i3)
                     celemmt(j,i1) = v11*s1+v21*s2+v31*s3
                     celemmt(j,i2) = v12*s1+v22*s2+v32*s3
                     celemmt(j,i3) = v13*s1+v23*s2+v33*s3
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = celemmt(i1,j)
                     s2 = celemmt(i2,j)
                     s3 = celemmt(i3,j)
                     celemmt(i1,j) = r11*s1+r21*s2+r31*s3
                     celemmt(i2,j) = r12*s1+r22*s2+r32*s3
                     celemmt(i3,j) = r13*s1+r23*s2+r33*s3
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute V^T f

                  f1 = celemvc(i1)
                  f2 = celemvc(i2)
                  f3 = celemvc(i3)
                  celemvc(i1) = v11*f1+v21*f2+v31*f3
                  celemvc(i2) = v12*f1+v22*f2+v32*f3
                  celemvc(i3) = v13*f1+v23*f2+v33*f3

               end if

            end do

         end if

      end if

1000  end
