      subroutine to0043 ( rinvec, nvec, invec1, coor, &
                          outvec, npoint, iwork, gather, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 21-11-2012 Long integers
!        version  3.0  date 15-04-1999 Extra parameters ibuffr
!        version  2.0  date 13-09-1990 Extra parameters indprh and kprobh
!
!   copyright (c) 1990-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array OUTVEC with information using subroutine FUNVEC
!     as help subroutine
!     TO0043 is called by MANVEC if ICHOIS = 17
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nvec, invec1(nvec), npoint, iwork(*), ndim
      double precision rinvec(*), coor(ndim,npoint), outvec(*)
      logical gather

!    coor      i   Array containing the co-ordinates of the nodes
!    gather    i   indication whether gathering has been carried out (true)
!                  or not (false)
!    invec1    i   Integer input array of size 5 x nvec
!                  INVEC1 refers to NVEC SEPRAN solution arrays
!                  of type 110 or 115
!    iwork     i   Array IWORK is filled by subroutine ER0063 as follows:
!                  IWORK(i)>0, gives start position of unknown in point i
!                  IWORK(i) = 0, unknown is not available in point i
!                  IWORK(i)<0, point is second periodical point
!                  -IWORK(i) refers to first point
!                  Array IWORK is only filled if gather is true
!    ndim      i   Dimension of space
!    npoint    i   Number of nodal points or number of elements in the case of
!                  an array of special structure defined per element
!    nvec      i   Number of vectors that is stored in INVEC1
!    outvec    o   Output vector that is filled by this subroutine
!    rinvec    i   Input array that is filled by the user before the call of
!                  MANVEC. It is passed undisturbed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision :: reavec(nvec)

!   i         General loop variable
!   j         General loop variable
!   reavec    Work array of length NVEC that is used to store the
!             NVEC values of the NVEC input arrays for one node
!             This array is passed to subroutine FUNVEC
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     FUNVEC         Fill outvec in one point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0043' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0043'
         write(irefwr,*) 'gather ', gather
         write(irefwr,1) 'npoint, nvec ', npoint, nvec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( gather ) then

!     --- gather is true, use array iwork

         do i = 1, npoint

            if ( iwork(i)>0 ) then

!           --- iwork(i)>0, degree of freedom in node

               do j = 1, nvec
                  usoltemp => ks(invec1(j))%sol
                  reavec(j) = usoltemp(iwork(i))
               end do
               call funvec ( rinvec, reavec, nvec, coor(1,i), &
                             outvec(iwork(i)) )

            end if

         end do

      else

!     --- gather is false, all nodes are used

         do i = 1, npoint

            do j = 1, nvec
               usoltemp => ks(invec1(j))%sol
               reavec(j) = usoltemp(i)
            end do
            call funvec ( rinvec, reavec, nvec, coor(1,i), &
                          outvec(i))

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( outvec, npoint, 'outvec' )
         write(irefwr,*) 'End to0043'

      end if  ! ( debug )
      call erclos ( 'to0043' )

      end subroutine to0043
