      subroutine to0044 ( iinvec, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 19-10-1991 Replacement of error message
!        version  1.0  date 03-03-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put information from array ICONST into IINVEC as function of ICHOIS
!     Translate output of TO0035 into a form suitable for subroutine MANVEC
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinvec(*), iconst(*)

!   iconst    i   Input integer array. ICONST is used as help array by
!                 subroutine TO0035. It is filled by TO0035 and must be used
!                 to fill IINVEC
!   iinvec   i/o  In this integer input array it is indicated what type of
!                 manipulations must be preformed by subroutine MANVEC.
!                 For some cases IINVEC acts both as input and
!                 as output array.
!                 The entries of IINVEC must be filled by the user as follows:
!                 1  Number of entries of IINVEC that are filled by the user.
!                    IINVEC(1) must be at least equal to 2.
!                    For positions that are not filled by the user (except the
!                    positions 1 and 2) defaults are used.
!                 2  ICHOIS   General choice parameter indicating what type of
!                             algebraic manipulations have to be carried out.
!                             ICHOIS must always have been given a value by
!                             the user.
!                    Description of the possible values of ICHOIS
!                       1     Average: c = average (u1)
!                       2     Extract value of one degree of freedom in one
!                             point from u1.
!                       3     Minimum and maximum value of u1 or | u1 |.
!                       4     Inner product:  c = ( u1 , u2 ) .
!                       5     || u1 ||  or || u1 - u2 ||
!                      26     Substraction of constant:  u out = u1 - c
!                      27     Linear combination:  u out = c1 u1 +  c2 u2
!                      28     Put value of one degree of freedom in one point
!                             into  u out
!                      29     Modulus of complex vector.
!                      30     Phase of complex vector.
!                      31     User defined function of two SEPRAN vectors:
!                             u out =  f ( u1 , u2 ) .
!                      32     User defined function of  n  SEPRAN vectors and
!                             the co-ordinates:
!                             u out =  f ( u1 , u2 , . . . , u n, x ) .
!                      33     Length of vector composed by components:
!                             u out (i) = sqrt ( u1 (i) ** 2 + u2 (i) ** 2 )
!                             or
!                             u out (i) = sqrt(u1(i)**2 + u2(i)**2 + u3(i)**2)
!                 3  IDEGFD   Depending on the value of IDEGFD, the algebraic
!                             manipulations will be performed with respect to
!                             all degrees of freedom of the vectors (IDEGFD=0), &
!                             with all internal degrees of freedom i.e. not
!                             with the essential boundary conditions
!                             (IDEGFD=-1), or with the IDEGFD th  degree of
!                             freedom only (IDEGFD > 0).
!                             Default value: IDEGFD = 0
!                 4  IPOINT   Nodal point number.
!                             Default value: IPOINT = 1
!                 5  ISPEC    The use of ISPEC depends on the VALUE of ICHOIS.
!                             Default value: Depending on ICHOIS.
!                 6  ICHNRM   The use of ICHNRM depends on the VALUE of ICHOIS.
!                             Default value: ICHNRM = 3
!                 7  ISKIP    Depending on the value of ISKIP element groups
!                             must be skipped in the computation (ISKIP > 9)
!                             or not (ISKIP=0).
!                             If ISKIP>9, the value of ISKIP denotes the
!                             starting position in array IINVEC where
!                             information concerning the elements to be skipped
!                             is stored.
!                             In that case it is necessary that the user has
!                             filled the positions IINVEC(ISKIP) , . . . , &
!                             ISKIP(ISKIP+ nelgrp -1), where  nelgrp
!                             denotes the number of element groups. Only
!                             element groups defined in the mesh are taken into
!                             account, so boundary element groups are not
!                             considered as special class. ISKIP in the range
!                             1 < ISKIP < 10 is not allowed.
!                             If IINVEC(ISKIP+ ielgrp -1) = 0, element group
!                             ielgrp  must not be skipped, if
!                             IINVEC(ISKIP+ ielgrp -1) = 1, the element group
!                             must be skipped.
!                             Default value: ISKIP = 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichois

!   ichois    Choice parameter stored in the first position of IINVEC
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1741   ICHOIS and IINVEC(1) have incorrect values
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ichois = iinvec(2)
      if ( ichois.eq.1 .or. ichois.eq.4 ) then
         if ( iinvec(1).lt.5 ) then
            call eropen ( 'to0044' )
            call errint ( ichois, 1 )
            call errint ( iinvec(1), 2 )
            call errint ( 5, 3 )
            call errsub ( 1741, 3, 0, 0 )
            call erclos ( 'to0044' )
            call instop
         end if
         iinvec(5) = iconst(1)
      else if ( ichois.eq.5 .and. iinvec(6).eq.10 ) then
         if ( iinvec(1).lt.4 ) then
            call eropen ( 'to0044' )
            call errint ( ichois, 1 )
            call errint ( iinvec(1), 2 )
            call errint ( 4, 3 )
            call errsub ( 1741, 3, 0, 0 )
            call erclos ( 'to0044' )
            call instop
         end if
         iinvec(4) = iconst(2)
      end if
      end
