      subroutine to0045 ( usol, outvec, idegfd, ndimlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 26-12-1994 New commons cmessc and cmessg
!        version  1.1  date 23-06-1992 New common cmessg
!        version  1.0  date 28-04-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine computes:
!                        2         2          2
!     outvec(i) = sqrt( u (i)  +  u (i)   +  u (i) )
!                        1         2          3
!     Complex case
!     Restrictions:  The input vector must contain the required degrees of
!                    freedom in each nodal point
!                    The output vector must contain exactly one degree of
!                    freedom in each nodal point
!                    All nodes, includiong boundary nodes are used
!     The output vector must have been cleared before
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndimlc, idegfd(ndimlc)
      double precision outvec(*)
      double complex usol(*)

!    idegfd   i   Array in which the various degrees of freedom that must be
!                 used are stored. Length ndimlc
!    ndimlc   i   number of degrees of freedom that have been stored in array
!                 idegfd. The summation must be performed over these unknowns
!    outvec  i/o  Output vector. Must be cleared before this subroutine
!                 Must contain exactly one unknown per point
!    usol     i   Input vector. Must contain exactly nphysi degrees of freedom
!                 per point.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!    i         General loop variable
!    j         General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!  1179   Not constant number of degrees of freedom in input vector
!  1183   Degree of freedom out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0045' )

!     --- Check input

      if ( indprfi.gt.0 ) call errsub ( 1179, 0, 0, 0 )
      do j = 1, ndimlc
         if ( idegfd(j).gt.nphysi ) then
            call errint ( j, 1 )
            call errint ( idegfd(j), 2 )
            call errint ( nphysi, 3 )
            call errsub ( 1183, 3, 0, 0 )
         end if
      end do

!     --- First add the squares to the output vector

      if ( indprpi.eq.0 ) then

!     --- indprpi = 0, vector of special structure

         do j = 1, ndimlc
            do i = 1, npoint
               outvec(i) = outvec(i) + abs ( usol((i-1)*nphysi+ &
                           idegfd(j)) )**2
            end do
         end do
      else

!     --- indprpi > 0, input vector has the structure of the solution vector

         do j = 1, ndimlc
            do i = 1, npoint
               outvec(i) = outvec(i) + &
                           abs ( usol(kprobpi(i,idegfd(j))) ) **2
            end do
         end do
      end if

!     --- Next compute the square root of the output vector

      do i = 1, npoint
         outvec(i) = sqrt(outvec(i))
      end do
      if ( ierror.ne.0 ) go to 1000
1000  call erclos ( 'to0045' )
      end
