      subroutine to0048 ( ichois, iinbld, ielhlp, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 11-12-2017 New storage of intmat
!        version  1.4  date 04-06-2014 Allow -102 in intmat(2)
!        version  1.3  date 26-12-1994 New commons cmessc and cmessg
!
!   copyright (c) 1989-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate the parameter ICHOIS in the subroutines SYSTEM/ SYSTM0/1/2
!     into the array IINBLD as described in subroutine BUILD
!     This subroutine is used in order to be able to use the old SEPRAN
!     subroutines SYSTEM/ SYSTM0/1/2 by calling the newest version of TO0050
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iinbld(*), ielhlp(*), ihelp

!    ichois    i   Choice parameter consisting 5 parts jchois, effbouncond,
!                  allelements, addmatvec and clearlastrhsd according to:
!                  ichois = jchois + 10 * effbouncond + 20 * allelements +
!                           40 * addmatvec + 100 * clearlastrhsd
!                  (When ICHOIS<0 array ISLOLD is used also)
!                  These parameters have the following meaning:
!                  JCHOIS    general choice parameter, possibilities:
!                         1  Both the large matrix and large vector are built.
!                         2  Only the large vector is built.
!                         3  The large matrix is built and the large vector
!                            is made equal to zero.
!                            If EFFBOUNCOND = 0 the large vector may be
!                            unequal to zero, due to
!                            non-homogeneous essential boundary conditions.
!                         5  The large matrix is not built;
!                            the large vector is made equal to zero.
!                            If EFFBOUNCOND = 0 the large vector may be unequal
!                            to zero, due to
!                            non-homogeneous essential boundary conditions.
!                        14  The large matrix is built;
!                            no right-hand-side vector is made.
!                  EFFBOUNCOND   Indication whether the effect of essential
!                             boundary  conditions must be taken into
!                             account (0) or not (1).
!                  ALLELEMENTS  When ALLELEMENTS = 1 not all standard element
!                            sequence numbers are used, when ALLELEMENTS = 0
!                            no standard element sequence numbers are skipped.
!                            When ALLELEMENTS = 1 array IELHLP is used.
!                            See IELHLP.
!                  ADDMATVEC    When ADDMATVEC = 0 the large matrix and vector
!                            are cleared before the assembling process.
!                            This is the standard situation.
!                            When ADDMATVEC = 1 the right-hand-side vector
!                            and the large matrix are not cleared,
!                            but added to the already existing right-hand-side
!                            vector and large matrix.
!                            Hence        RHSD := RHSD + DELTA  RHSD
!                                         MATRIX := MATRIX + @DELTA@MATRIX
!                  CLEARLASTRHSD  When CLEARLASTRHSD = 0 the last part of the
!                            right-hand-side vector corresponding to the
!                            boundary conditions is cleared (standard situation)
!                            when CLEARLASTRHSD = 1 the standard assembling
!                            process is also applied to this last part of the
!                            right-hand-side vector. For most applications
!                            CLEARLASTRHSD = 0 suffices.
!     ielhlp  i   This array of length nelgrp+5 is only used if allelements = 1
!                 ielhlp(ielgrp+5) = 0 means elements of element group ielgrp
!                 are skipped, ielhlp(ielgrp+5) = 1, means that they are
!                 assembled
!    ihelp    o   Starting position in array IELHLP (either 1 or 6) depending on
!                 the fact if it is used (6) or not (1)
!    iinbld    o   In this array it is indicated how the matrix and right-hand
!                  side, must be filled. IINBLD is the same array as given as
!                  input for subroutine BUILD.
!                  IINBLD will be filled as follows
!            1  Number of entries of IINBLD that are filled by the user.
!               Not filled by this subroutine
!            2  JCHOIS    general choice parameter, possibilities:
!                      1  Both the large matrix and large vector are built.
!                      2  Only the large vector is built.
!                      3  The large matrix is built and the large vector
!                         is made equal to zero.
!                         If EFFBOUNCOND = 0 the large vector may be
!                         unequal to zero, due to
!                         non-homogeneous essential boundary conditions.
!                      5  The large matrix is not built;
!                         the large vector is made equal to zero.
!                         If EFFBOUNCOND = 0 the large vector may be unequal
!                         to zero, due to
!                         non-homogeneous essential boundary conditions.
!                     14  The large matrix is built;
!                         no right-hand-side vector is made.
!            3  EFFBOUNCOND   Indication whether the effect of essential
!                         boundary  conditions must be taken into
!                         account (0) or not (1).
!            4  IMAS      Indication if the mass matrix must be computed (1)
!                         or not (0). This position is made equal to 0
!            5  IPREC     Indication if the old solution is stored in ISLOLD
!                         (1) or not (0)
!                         IPREC is not used if IINBLD(10)>0
!            6  ALLELEMENTS  Indication if all standard element
!                         sequence numbers are used (0) or not (1)
!            7  ADDMATVEC    When ADDMATVEC = 0 the large matrix and vector
!                         are cleared before the assembling process.
!                         When ADDMATVEC = 1 the right-hand-side vector
!                         and the large matrix are not cleared, &
!                         but added to the already existing right-hand-side
!                         vector and large matrix.
!                         Hence        RHSD := RHSD + DELTA  RHSD
!                                      MATRIX := MATRIX +  DELTA  MATRIX
!            8   CLEARLASTRHSD  When CLEARLASTRHSD = 0 the last part of the
!                         right-hand-side vector corresponding to the
!                         boundary conditions is cleared (standard situation)
!                         when CLEARLASTRHSD = 1 the standard assembling
!                         process is also applied to this last part of the
!                         right-hand-side vector. For most applications
!                         CLEARLASTRHSD = 0 suffices.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jchois, k
      double precision dummy(1)

!    dummy      Dummy parameter
!    jchois     See ichois at input
!    k          General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINLG         Print and checks length of user arrays
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      26   Array KPROB has not been filled
!     699   Problem number incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0048' )

      if ( ichois<0 ) iinbld(5) = 1
      jchois = abs(ichois)
      if ( jchois>=100 ) then

!     --- jchois>= 100

         jchois = jchois - 100
         iinbld(8) = 1
      end if
      if ( jchois>=40 ) then

!     --- jchois>40

         jchois = jchois-40
         iinbld(7) = 1
      end if
      if ( jchois>=20 ) then

!     --- jchois>20

         jchois = jchois-20
         iinbld(6) = 1
      end if
      if ( jchois>=10 ) then

!     --- jchois>10

         if ( jchois/=14 ) jchois = jchois - 10
         iinbld(3) = 1
      end if
      iinbld(2) = jchois

      call sepfillmatstr

!     --- Check array kprob

      iprob = intmt(intmat)%iprob

!     --- Compute starting address of kprob in the rest of the subroutine
!          to0050

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      ihelp = 1
      if ( iinbld(6)==1 ) ihelp = 6
      if ( jchois==0 .or. jchois>5 .and. jchois/=14 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      dummy(1) = 0d0
      if ( iinbld(6)==1 ) &
           call prinlg ( 'ielhlp', kprobloc(7)+kprobloc(11), &
                         dummy, ielhlp )
      call erclos ( 'to0048' )
      end
