      subroutine to0066 ( islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version 11.0  date 24-07-2015 Use sepmodulebuild
!        version 10.1  date 05-12-2012 Long integers
!        version 10.0  date 15-11-2012 Extra parameters
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of temporary arrays for subroutine TO0050 and
!     reserve space.
!     Store information in array IPOINT.
!     Help subroutine for subroutine TO0050 (BUILD)
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: islold(*)

!     islold     i    This array is used if ichois<0, or numold>0
!                     When ichois<0, it is supposed to consist of one vector
!                     only, which must be of the same structure as the solution
!                     vector
!                     When numold>0, islold contains numold vectors which may
!                     be solution vectors of several problems or vectors of
!                     special structure (type 115) resulting of several problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ikelmc, numsav, mindex, iprobsav

!     i              Loop variable
!     ikelmc         Position in KMESH part c
!     mindex         Help parameter for the length of array index3
!     numsav         Help variable to store the old value of numold
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0009         Compute some element group dependent parameters
!     TO0063         Compute NINDEX
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0066' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0066'

      end if
      if ( ierror/=0 ) go to 1000

      if ( oldsol ) numold = 0

!     --- Compute length of various "element arrays"
!         Required constants: icount
!                             nindex
!         Compute icount, inpelm etc.

      mindex = 0
      numsav = numold
      numold = max(numold,1)

      do ielgrp = 1, numns

!     --- initialize some parameters for one element group

         call to0009
         if ( inelem>0 ) call to0063 ( islold ) ! compute nindex
         mindex = max ( mindex, nindex )

      end do

      if ( numsav>0 ) then
         nindex = mindex
      else
         nindex = 0
      end if

      if ( numsav==0 .or. oldsol ) then

!     --- oldsol      which means that there is an old vector
!                     connected to index2
!                     No vector is connected to index3
!                     Set starting address of index3 equal to index2
!                     Reset numold equal to 1

         numold = 1

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0066'

      end if
      call erclos ( 'to0066' )

      end
