      subroutine to00821 ( vecout, vecinp, nphyso, nphys, jdegfd, &
                           npoint, iskip, iskippoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     map a SEPRAN vector of one structure
!     onto a SEPRAN vector of a different structure
!     vecout has nphyso degrees of freedom per point
!     vecinp has nphys degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nphyso, nphys, jdegfd, npoint, iskip, iskippoints(*)
      double precision vecout(nphyso,npoint), vecinp(nphys,npoint)

!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskippoints    i    If iskippoints(i)=0, node i must be skipped, else
!                         iskippoints(i)>0, node i must not be skipped
!     jdegfd         i    Mapping must be stored from the jdegfd-th
!                         unknown per point in the array vecout
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     nphyso         i    Number of physical unknowns, or maximal degrees of
!                         freedom per point for output vector
!     npoint         i    Number of nodal points
!     vecinp         i    Input vector to be mapped
!     vecout         o    Output vector, result of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, i, j

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to00821' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to00821'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = min(nphys,nphyso-jdegfd+1)
      if ( iskip==0 ) then

!     --- iskip = 0, no skipping

         do i = 1, npoint
            do j = jdegfd, inunkp
               vecout(j,i) = vecinp(j,i)
            end do
         end do

      else

!     --- iskip>0, skipping

         do i = 1, npoint

            if ( iskippoints(i)>0 ) then

!           --- Do not skip node

               do j = jdegfd, inunkp
                  vecout(j,i) = vecinp(j,i)
               end do

            end if  ! ( iskippoints(i)>0 ) then

         end do  ! i = 1, npoint

      end if  ! ( iskip==0 )

1000  call erclos ( 'to00821' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to00821'

      end if  ! ( debug )

      end
