      subroutine to00823 ( vecout, vecinp, kprbfo, jdegfd, &
                           npoint, iskip, iskippoints, kprbfi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     map a SEPRAN vector of one structure
!     onto a SEPRAN vector of a different structure
!     vecout has variable degrees of freedom per point
!     vecinp has variable degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jdegfd, npoint, iskip, iskippoints(*), kprbfo(npoint+1), &
              kprbfi(npoint+1)
      double precision vecout(*), vecinp(*)

!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskippoints    i    If iskippoints(i)=0, node i must be skipped, else
!                         iskippoints(i)>0, node i must not be skipped
!     jdegfd         i    Mapping must be stored from the jdegfd-th
!                         unknown per point in the array vecout
!     kprbfi         i    Array KPROB part f for input vector
!     kprbfo         i    Array KPROB part f for output vector
!     npoint         i    Number of nodal points
!     vecinp         i    Input vector to be mapped
!     vecout         o    Output vector, result of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, i, j, istart, ostart

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     istart         First position used in vecinp
!     j              Counting variable
!     ostart         First position used in vecout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to00823' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to00823'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iskip==0 ) then

!     --- iskip = 0, no skipping

         do i = 1, npoint

            istart = kprbfi(i)
            inunkp = kprbfi(i+1)-istart
            ostart = kprbfo(i)
            inunkp = min(kprbfo(i+1)-ostart,inunkp)
            do j = jdegfd, inunkp
               vecout(ostart+j) = vecinp(istart+j)
            end do

         end do  ! i = 1, npoint

      else

!     --- iskip>0, skipping

         do i = 1, npoint

            if ( iskippoints(i)>0 ) then

!           --- Do not skip node

               istart = kprbfi(i)
               inunkp = kprbfi(i+1)-istart
               ostart = kprbfo(i)
               inunkp = min(kprbfo(i+1)-ostart,inunkp)
               do j = jdegfd, inunkp
                  vecout(ostart+j) = vecinp(istart+j)
               end do

            end if  ! ( iskippoints(i)>0 ) then

         end do  ! i = 1, npoint

      end if  ! ( iskip==0 )

1000  call erclos ( 'to00823' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to00823'

      end if  ! ( debug )

      end
