      subroutine to0213 ( cmatip, cmatpi, cmatpp, celemmt )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  5.0  date 24-07-2015 Use sepmodulebuild
!        version  4.0  date 21-04-2015 New parameter list
!        version  3.1  date 11-04-2015 Extension of jwork
!
!   copyright (c) 1996-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    add element matrix to large matrix part 2, 3, 4
!    i.e. construct not only Sup but also Spu and Spp
!    Hence ibcmat must be 1
!    (corresponding to boundary conditions)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, intent(inout) :: cmatip(*), cmatpi(*), cmatpp(*)
      double complex, intent(in) :: celemmt(icount,icount)

!     celemmt        i    Element matrix stored as two dimensional array
!                         Note that due to the FORTRAN structure and SEPRAN
!                         storage in fact the transpose of elemmt is stored
!     cmatip        i/o   part of the large matrix connected with boundary
!                         conditions (S_ip)
!     cmatpi        i/o   part of the large matrix connected with boundary
!                         conditions (S_pi)
!     cmatpp        i/o   part of the large matrix connected with boundary
!                         conditions (S_pp)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inusol, jtel, j, jnusol, jwork(nusol), &
              iamat, ktel, k, knusol

!     i              General loop variable
!     iamat          entry of the large matrix you want to add the element
!                    matrix entry of Sup(j,i) to
!     inusol         Global sequence number of prescribed variable with respect
!                    to USOL
!     j              General loop variable
!     jnusol         Global sequence number of prescribed variable with respect
!                    to USOL
!     jtel           Local sequence number of prescribed variable with respect
!                    to indexren
!     jwork          integer work array of length nrusol
!                    In jwork(k) the addres of cmat(inusol,knusol) in
!                    array amat is stored
!     k              General loop variable
!     knusol         global seguence number of local free degree of freedom j
!     ktel           Global sequence number of prescribed degree of freedom, &
!                    where the first prescribed d.o.f. has sequence number 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) icount-icnt do
!        inusol := global seguence number of local precribed degree of freedom i
!        itel := local sequence number of i in element
!        Fill array jwork with information of the free unknowns coupled to
!        inusol
!        For k := 1 (1) icnt do
!           knusol := global seguence number of local free degree of freedom k
!           iamat := position of element (inusol, knusol) in matrix
!           cmat(iamat) := cmat(imat)+celemmt(index(k),itel)
!           cmat(imat+leng) := cmat(imat+leng) + celemmt(itel,index(k))
!        Fill array jwork with information of the prescribed dof's coupled to
!        inusol
!        For j := 1 (1) icnt1 do
!           jnusol := global sequence number of local prescr. dof j
!           iamat  := position of element (inusol,jnusol)
!           cmat(iamat) := cmat(iamat)+ celemmt(jtel,itel)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0213' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0213'
         write(irefwr,1) 'nrusol', nrusol
         call prinin ( indexren, icount, 'indexren' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- for every prescribed d.o.f

      do i = 1, icount

!     --- inusol := global seguence number of local
!         precribed degree of freedom i

         inusol = indexren(i)

         if ( inusol<=nrusol ) go to 200  ! only prescribed unknowns

!        --- fill jwork with addresses in amat of all free degrees of
!            freedom coupled to inusol

         ktel = inusol-nrusol
         do k = irowsip(ktel)+1, irowsip(ktel+1)
            jwork(icolsip(k)) = k
         end do

!        --- for every free d.o.f coupled to the d.o.f. i
!            construct Sup and Spu

         do k = 1, icount

            knusol = indexren(k)
            if ( knusol>nrusol ) go to 100   ! only free degrees of freedom
            iamat = jwork(knusol)

!           --- add the entry to the large matrix

            cmatip(iamat) = cmatip(iamat)+celemmt(i,k)
            cmatpi(iamat) = cmatpi(iamat)+celemmt(k,i)

100      end do  ! k = 1, icount

!        --- fill jwork with addresses in amat of all prescribed
!            degrees of freedom coupled to inusol

         jtel = inusol-nrusol
         do j = irowsp(jtel)+1, irowsp(jtel+1)
            jwork(icolsp(j)) = j
         end do

!        --- for every prescribed d.o.f coupled to the d.o.f. i
!            construct Spp

         do j = 1, icount

!        --- jnusol := global seguence number of local
!            precribed degree of freedom
!            jtel := local sequence number of j in element

            jnusol = indexren(j)
            if ( jnusol<=nrusol ) go to 150   ! only prescribed deg of freedom
            iamat = jwork(jnusol)

!           --- add the entry to the large matrix

            cmatpp(iamat) = cmatpp(iamat)+celemmt(j,i)

150      end do  ! j = 1, icount

200   end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0213'

      end if  ! ( debug )
      call erclos ( 'to0213' )

      end
