      subroutine tobuild ( iinbldlc, ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.3  date 09-11-2007 ini060-->ini060text
!        version  1.2  date 03-10-2005 New call to toglob
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform those parts of the matrix assembly process that are not carried
!     out by to0050
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinbldlc(*), ielhlp(*)

!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, &
!                         ielhlp(ielgrp+5) = 1, means that they are assembled
!     iinbldlc         i    User input array for subroutine BUILD, see buildbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILDESSBC  Compute effect of essential boundary conditions to
!                    the right-hand side
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
!     TOFICT         Compute part of matrix corresponding
!                    to fictitious unknowns
!     TOGLOB         Compute part of matrix and right-hand side corresponding
!                    to global unknowns
!     TORENUM        Renumber right hand side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tobuild' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tobuild'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( stiffmat(1)>0 ) Smat => realmt(stiffmat(1))

!     --- Compute indprh

      imas = iinbldlc(4)
      addmat = iinbldlc(2)==4 .or. iinbldlc(2)==2 .or. iinbldlc(2)==5
      addmat = .not. addmat

!     --- Compute effbc

      effbc = nbound>0

      if ( debug ) write(irefwr,*) 'effbc, addmat', effbc, addmat

      if ( iinbldlc(3)>=1 .or. iinbldlc(2)==14 .or. iinbldlc(2)==4 .or. &
           iinbldlc(2)==6 ) effbc = .false.

      if ( (imas>0 .or. iinbldlc(2)/=4) .and. indprx>0 ) then

!     --- kprob(11)>0, hence there are global unknowns
!         Store part corresponding to global unknowns into matrix and vector

         call toglob ( iinbldlc, ielhlp )

      end if
      if ( debug ) write(irefwr,*) 'after toglob'

      if ( addmat .and. indpry>0 ) then

!     --- kprob(51)>0, hence there are fictitious unknowns
!         Store part corresponding to fictitious unknowns into matrix

         call tofict ( iinbldlc, ielhlp )

      end if

      if ( debug ) write(irefwr,*) 'after tofict', effbc

!     --- Compute effect of boundary conditions

      if ( effbc ) call sepbuildessbc

      if ( debug ) write(irefwr,*) 'after effbc'

!     --- renumber right-hand side

      if ( addvec .and. indprh>0 ) call torenum

      if ( debug ) write(irefwr,*) 'after torenum'

      call seprmmatstr2  ! nullify arrays in modulematstr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tobuild'

      end if  ! ( debug )
      call erclos ( 'tobuild' )

      end subroutine tobuild
