      subroutine tofilldmat ( elemmat, jwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 24-07-2015 Extra debug
!        version  3.2  date 21-09-2011 Extra debug
!        version  3.1  date 27-10-2010 array bounds jwork
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the matrices D and G for the SIMPLE algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     simple
!     divergence_matrix
!     gradient_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: jwork(nrusolvel)
      double precision, intent(in) :: elemmat(icount,icount)

!     elemmat         i    element matrix after mapping
!                         rows and columns corresponding to prescribed degrees
!                         of freedom are eliminated
!                         elemmat is stored row-wise (mathematical sequence)
!                         So in fortran we have to use the transpose fo elemmat
!     jwork         i/o   integer work array of size nu
!                         In jwork the sequence numbers of the columns of the
!                         non-zero elements in one row of the matrix are stored
!                         At input jwork must be 0
!                         At output jwork is reset to 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, inusol, jnusol, iamat, nhelp
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     iamat          Position in array AMAT
!     inusol         Position of unknown in array USOL (row number)
!     j              Counting variable
!     jnusol         Position of unknown in array USOL (column number)
!     nhelp          nrusol-nrusolvel
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINDMAT       Print the D matrix in the Simple method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2899   The matrix element is not present in the reduced storage,
!           but the element in the element matrix is not equal to 0
!    2900   The matrix element is not present in the reduced storage,
!           but the element in the element matrix is not equal to 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( jmethod = 13 or 14 ) then
!        For i := 1 (1) icount do
!           inusol := indexren(i)
!           if ( inusol>nrusolvel ) then
!              Fill jwork with sequence numbers
!              For j := 1 (1) icount do
!                 jnusol := indexren(j)
!                 if ( jnusol<=nrusolvel ) then
!                    Add element to S_DIV
!     else ( jmethod = 15 or 16 ) then
!        For i := 1 (1) icount do
!           inusol := indexren(i)
!           if ( inusol>nrusolvel ) then
!              Fill jwork with sequence numbers
!              For j := 1 (1) icount do
!                 jnusol := indexren(j)
!                 if ( jnusol<=nrusolvel ) then
!                    Add element to S_DIV
!                    Add element to S_G
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofilldmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofilldmat'
         write(irefwr,1) 'nrusol, nrusolvel, icount', &
                          nrusol, nrusolvel, icount
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      nhelp = nrusol-nrusolvel
      if ( IM%Gsymmetric ) then

!     --- D = G^T, i.e. fill only the matrix D
!         For i := 1 (1) icount do

         do i = 1, icount

!        --- inusol := indexren(i)

            inusol = indexren(i)-nrusolvel
            if ( inusol>0 .and. inusol<=nhelp ) then

!           --- inusol<=nrusolvel, i.e. row corresponds to D matrix
!               Fill jwork with sequence numbers

               do j = irowsG(inusol)+1, irowsG(inusol+1)
                  jwork(icolsG(j)) = j
               end do  ! j = irowsG(inusol)+1, irowsG(inusol+1)

!              --- For j := 1 (1) icount do

               do j = 1, icount

!              --- jnusol := indexren(j)

                   jnusol = indexren(j)
                   if ( jnusol<=nrusolvel ) then

!                  --- jnusol<=nrusolvel, i.e. column corresponds to D matrix
!                      Add element to S_DIV

                      iamat = jwork(jnusol)
                      if ( iamat>0 ) then

!                     --- iamat>0, standard case

                         S_DIV(iamat) = S_DIV(iamat) + elemmat(j,i)

                      else if ( abs(elemmat(j,i))>0d0 ) then

!                     --- iamat=0, but element in element matrix not equal
!                                 to 0 This is an error

                         call errint ( i, 1 )
                         call errint ( j, 2 )
                         call erreal ( elemmat(j,i), 1 )
                         call errsub ( 2899, 2, 1, 0 )

                      end if  ! ( iamat>0 )

                   end if  ! ( jnusol==inusol )

               end do  ! j = 1, icount

!              --- Clear array jwork

               do j = irowsG(inusol)+1, irowsG(inusol+1)
                  jwork(icolsG(j)) = 0
               end do  ! j = irowsG(inusol)+1, irowsG(inusol+1)

            end if  ! ( inusol<=nrusolvel )

         end do   ! i = 1, icount

      else

!     --- D # G^T, i.e.  fill only the matrices D and G

         do i = 1, icount

!        --- inusol := indexren(i)

            inusol = indexren(i)-nrusolvel
            if ( debug ) write(irefwr,1) 'i, inusol', i, inusol
            if ( inusol>0 .and. inusol<=nhelp ) then

!           --- inusol<=nrusolvel, i.e. row corresponds to D matrix
!               Fill jwork with sequence numbers

               if ( debug ) write(irefwr,1) 'i, inusol', i, inusol
               do j = irowsG(inusol)+1, irowsG(inusol+1)
                  jwork(icolsG(j)) = j
               end do  ! j = irowsG(inusol)+1, irowsG(inusol+1)

!              --- For j := 1 (1) icount do

               do j = 1, icount

!              --- jnusol := indexren(j)

                   jnusol = indexren(j)
                   if ( jnusol<=nrusolvel ) then

!                  --- jnusol<=nrusolvel, i.e. column corresponds to D matrix
!                      Add element to S_DIV

                      iamat = jwork(jnusol)
                      if ( iamat>0 ) then

!                     --- iamat>0, standard case

                         S_DIV(iamat) = S_DIV(iamat) + elemmat(j,i)
                         S_G(iamat) = S_G(iamat) + elemmat(i,j)

                      else if ( abs(elemmat(j,i))>0d0 .or. &
                                abs(elemmat(i,j))>0d0 ) then

!                     --- iamat=0, but element in element matrix not equal
!                                 to 0 This is an error

                         call errint ( i, 1 )
                         call errint ( j, 2 )
                         call erreal ( elemmat(j,i), 1 )
                         call erreal ( elemmat(i,j), 1 )
                         call errsub ( 2900, 2, 2, 0 )

                      end if  ! ( iamat>0 )

                   end if  ! ( jnusol==inusol )

               end do  ! j = 1, icount

!              --- Clear array jwork

               do j = irowsG(inusol)+1, irowsG(inusol+1)
                  jwork(icolsG(j)) = 0
               end do  ! j = irowsG(inusol)+1, irowsG(inusol+1)

            end if  ! ( inusol<=nrusolvel )

         end do   ! i = 1, icount

      end if  ! ( IM%Gsymmetric )

1000  if ( debug ) then
         write(irefwr,*) 'Matrix D'
         call prindmat ( S_DIV, irowsG, icolsG, nhelp )

         if ( .not. IM%Gsymmetric ) then
            write(irefwr,*) 'Transpose of matrix G'
            call prindmat ( S_G, irowsG, icolsG, nhelp )
         end if  ! ( .not. IM%Gsymmetric ... )
         write(irefwr,*) 'End tofilldmat'
      end if  ! ( debug )
      call erclos ( 'tofilldmat' )

      end
