      subroutine tomanvec49 ( ivec1, ivec2, ivecout, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 21-08-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Perform vector manipulations for subroutine manvec in case of
!    ichois = 49
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, ivecout, iconst(*)

!     iconst         i    Contains some constants required to compute the vector
!                         1:  ivec
!                         2:  iprob
!                         3:  idegfd1
!                         4:  idegfd2
!     ivec1          i    input vector to be manipulated
!                         ivec1 must be a SEPRAN array of length 5, which may be
!                         of type 110, 115, 116 or 119
!     ivec2          i    Second input vector to be manipulated
!     ivecout       i/o   Output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, ivec, indprf2,  idegfd1, idegfd2, nphys2
      integer, pointer :: kprobf2(:)
      logical compli

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
!     i              Counting variable
!     idegfd1        First degree of freedom
!     idegfd2        Second degree of freedom
!     indprf2        indication if kprob2 part f has been filled (>0)
!                    or not (0)
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     kprobf2        kprobf for second vector
!     nphys2         Number of physical unknowns, or maximal degrees of
!                    freedom per point (second vector)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TOMANVEC49_1   uout(i) is vector with 1 component in the
!                    points where vec1(i,.) is defined
!                    vec1 and vec2 must be vectors with ndim components
!                    uout(i) = vec1_idegfd1(i) vec2_idegfd2(i)-
!                              vec2_idegfd1(i) vec1_idegfd2(i)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
!     699   Problem number incorrect
!    2275   nphys in input vector<ndim
!    2277   nphys in output vector # 1
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check type of ivec1
!     Reserve space
!     Reserve space for the output vector and activate the
!       corresponding arrays KPROBF, KPROBH and KPROBP
!     Perform actual computations
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec49' )

!     --- Check type of ivec1

      if ( ks(ivec1)%compl==0 ) then
         compli = .false.
      else
         compli = .true.
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if
      if ( ks(ivec2)%compl==0 ) then
         compli = .false.
      else
         compli = .true.
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if

!     --- Reserve space

      call sepactsolbf2 ( ivec2 )
      indprf2 = indprfi
      nphys2 = nphysi
      kprobf2 => kprobfi

      itypvc = ks(ivec1)%typevec
      call sepactsolbf2 ( ivec1 )
      if ( nphysi<ndim ) then

!     --- Number of degrees of freedom too small

         call errint ( nphysi, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2275, 2, 0, 0 )

      end if
      if ( nphys2<ndim ) then

!     --- Number of degrees of freedom too small

         call errint ( nphys2, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2275, 2, 0, 0 )

      end if
      idegfd1 = iconst(1)
      idegfd2 = iconst(2)
      ivec  = iconst(3)
      iprob = iconst(4)
      if ( ivec==0 ) then
         itypvc = 110
      else
         itypvc = 115
      end if
      if ( iprob<1 .or. iprob>nprob )  then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0)
      end if

!     --- First reserve space for the output vector and activate the
!         corresponding arrays KPROBF, KPROBH and KPROBP

      call sepcreasolbf1 ( ivecout, itypvc, ivec, 'vector', &
                              ks(ivec1)%compl )
      vectout => usoltemp

      if ( nphyso/=1 ) then

!     --- Number of degrees of freedom not 1

         call errint ( nphyso, 1 )
         call errsub ( 2277, 1, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Carry out the actual mapping

      vectin => ks(ivec1)%sol
      vectin2 => ks(ivec2)%sol
      if ( compli ) then

!     --- Complex case

      else

!     --- Real case, ichois = 49

         call tomanvec49_1 ( vectin, vectin2, vectout, indprfi, &
                             kprobfi, indprfo, kprobfo, &
                             npoint, nphysi, idegfd1, idegfd2, indprf2, &
                             kprobf2, nphys2 )

      end if

1000  call erclos ( 'tomanvec49' )
      end
