      subroutine tost00
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.1  date 14-12-1995 (Rename kprobr into kprobh)
!        version  1.0  date 26-09-1989
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   index5 is filled with the positions of the unknowns with respect to the
!   solution vector.
!   If there are no periodical boundary conditions, index2 and index5 are
!   identical.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, inunkp, inodp, jcount, inusol


!     inodp          nodal point number
!     inunkp         number of degrees of freedom in a point
!     inusol         position of degrees of freedom in solution array before
!                    renumbering ( sometimes minus one )
!     j              loop variable
!     jcount         counting variable to compute a position in index5
!     k              loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       if (indprf=0) then
!          if (nunkp=1) then
!             fill index5 from index1 and kprobh
!          else (nunkp>1)
!             for j := 1(1)inpelm do
!                fill index5 from index1 and kprobh for each unknown
!       else (indprf>0)
!          for j := 1(1)inpelm
!             compute inusol,inunkp
!             fill index5 from kprobh for each unknown
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (indprf==0) then

!     --- indprf = 0, constant number of unknowns per point

         if (nunkp==1) then

!        --- nunkp = 1, one unknown in each point

            index5(1:inpelm) = kprobh(index1(1:inpelm))

         else

!        --- nunkp>1, more degrees of freedom per point

            do j=1,inpelm
               inusol = (index1(j)-1)*nunkp
               do k=1,nunkp
                  index5((j-1)*nunkp+k)=kprobh(inusol+k)
               end do
            end do
         end if

      else

!     --- indprf>0, number of degrees of freedom per point variable

         jcount = 0
         do j=1,inpelm
            inodp = index1(j)
            inusol = kprobf(inodp)
            inunkp = kprobf(inodp+1)-inusol
            do k=1,inunkp
               index5(jcount+k) = kprobh(inusol+k)
            end do
            jcount = jcount + inunkp
         end do

      end if

      end
