      subroutine prbearmasscon4
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 04-09-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob ac with points that have prescribed pressure
!     due to cavity
! **********************************************************************
!
!                       KEYWORDS
!
!     lubrication
!     mass_conservation
!     incompressible
!     bearing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncav
      integer, allocatable :: iwork(:)

!     ipnodes        Starting address of array INODES
!     iwork          Kprob ad with information of nodes in which we
!                    have a cavity boundary condition
!                    Pos 0 contains the number of nodes
!                    The next positions the node numbers
!     ncav           Number of ndoes with pressure equal to cavity pressure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRBEARMASSCON6 Fill kprob ac with points that have prescribed pressure
!                    due to cavity
!     SEPCOPIBUF     Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents
!                    of iarray to ibuffr
!     SEPIACTIVATE   Reactivate existing array in IBUFFR
!     SEPRMIBUF      delete temporary array from ibuffr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbearmasscon4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbearmasscon4'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Reserve temporary space in ibuffr to store kprobad

      allocate ( iwork(npoint+1), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array

      call prbearmasscon6 ( iwork, npoint, ncav, nodes )
      if ( debug ) write(irefwr,1) 'ncav', ncav

!     --- make kprobad permanent

      if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobad) ) &
         deallocate(kp(iseqkp)%kprobsub(iprob)%kprobad)
      if ( ncav>0 ) then

!     --- There are points that must be prescribed

         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobad(ncav+1) )
         kprobad => kp(iseqkp)%kprobsub(iprob)%kprobad
         kprobad = iwork(1:1+ncav)
         kprobloc(60) = 1+ncav

      else if ( kprobloc(60)>0 ) then

!     --- There are no points that must be prescribed
!         Remove kprobad

         kprobloc(60) = 0

      end if  ! ( ncav>0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prbearmasscon4'

      end if  ! ( debug )
      call erclos ( 'prbearmasscon4' )

      end
