      function prcheckphase1 ( phasenew, phaseold, nelem )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 08-02-2006
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether the phase of an element has passed from fully solid
!     to fully liquid or vice versa. Actual body of prcheckphase
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, phaseold(nelem), phasenew(nelem)
      logical prcheckphase1

!     nelem          Number of elements
!     phasenew       Array containing the phase of each element for the
!                    new situation
!                    0: element is solid
!                    1: element is liquid
!                    2: element is mushy
!     phaseold       Array containing the phase of each element for the
!                    old situation
!                    0: element is solid
!                    1: element is liquid
!                    2: element is mushy
!     prcheckphase1  True when an element skipped the phase transition
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical rewind
      integer i

!     i              Counting variable
!     rewind         If true, time should be rewinded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcheckphase1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcheckphase1'
         call prinin(phaseold, nelem, 'phaseold')
         call prinin(phasenew, nelem, 'phasenew')
      end if
      if ( ierror/=0 ) go to 1000

      i = 1
      rewind = .false.
      do while ( i<=nelem .and. .not. rewind )

         if (phaseold(i)==1 .and. phasenew(i)==0 .or. &
             phaseold(i)==0 .and. phasenew(i)==1 ) rewind = .true.

         i = i + 1

      end do ! ( i<=nelem .and. .not. rewind )

      if ( rewind ) &
         write(irefwr,*) 'element', i-1, 'went from phase', &
                      phaseold(i-1), 'to phase', phasenew(i-1)

      prcheckphase1 = rewind

1000  call erclos ( 'prcheckphase1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'rewind', rewind
         write(irefwr,*) 'End prcheckphase1'

      end if

      end
