      subroutine prcompenthalp1 ( islold, rinenthalpy, numveclc, iinenthalpy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 15-09-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the enthalpy, given the temperature
!     Overrelaxation approach, i.e. both vectors are defined per point
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     temperature
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer islold(*), numveclc, iinenthalpy(*)
      double precision rinenthalpy(*)

!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     islold        i/o   User input array in which the user puts information
!                         of all preceding solutions
!     numveclc        i/o   Number of vectors stored in isol
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nusol, iseqtemp, iseqenth

!     i              Counting variable
!     iseqenth       Sequence number of enthalpy with respect to array isol
!     iseqtemp       Sequence number of temperature in solution vector
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRKIRCHOFF     Transform temperature to normalized Kirchhoff temperature
!                    and compute enthalpy or transform  normalized Kirchhoff
!                    temperature back
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters from iinenthalpy
!     Activate the temperature vector
!     Activate or create the enthalpy vector
!     Compute starting addresses
!     Transform
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompenthalp1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompenthalp1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters from iinenthalpy

      iseqenth = iinenthalpy(2)
      iseqtemp = iinenthalpy(3)
      numveclc = max(numveclc,iseqenth)

!     --- Activate or create the enthalpy vector

      call sepcreasolbf1 ( islold(iseqenth), ks(islold(iseqtemp))%typevec, &
                           ks(islold(iseqtemp))%ivec, 'enthalpy', &
                           ks(islold(iseqtemp))%compl )
      enthalpy => usoltemp
      nusol = size(usoltemp)

!     --- Compute starting addresses

      temperature => ks(islold(iseqtemp))%sol

!     --- Transform

      call prkirchoff ( enthalpy, temperature, &
                        nusol, 2, rinenthalpy )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompenthalp1'

      end if
      call erclos ( 'prcompenthalp1' )

      end subroutine prcompenthalp1
