      subroutine prcompenthalp4 ( islold, ioutput, iinenthalpy, dt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the enthalpy, given the temperature
!     Newton approach, i.e. enthalpy is defined per integration point
!     This routine is also used to update the enthalpy during the iterations
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     temperature
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer islold(*), iinenthalpy(*), ioutput
      double precision dt

!     dt             i    Time step
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     ioutput        o    Result vector of computation, i.e. M H
!     islold        i/o   User input array in which the user puts information
!                         of all preceding solutions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iread(2), inpder(9), idummy(1), iprob, iseqcoff
      double precision output(1)

!     idummy         Dummy parameter
!     inpder         Input array for DERIVS
!     iprob          Actual problem number
!     iread          Defines how the input must be read
!                    Possible values:
!                    -1:  No input is required
!                     0:  All SEPRAN input has been read by
!                         subroutine SEPSTN until
!                         END_OF_SEPRAN_INPUT or
!                         end of file has been found
!                         This input is used
!                     1:  The input is read as described for DERIV
!     iseqcoff       Sequence number of coefficients input
!     output         Dummy parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPDERIVS      Compute derivatives
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters from iinenthalpy
!     Fill specific arrays for derivs
!     Actual filling of enthalpy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompenthalp4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompenthalp4'
         call prinin ( iinenthalpy, 7, 'iinenthalpy' )
!         call sepprintv ( islold(2), 'H')

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Get parameters from iinenthalpy

      iseqcoff = iinenthalpy(7)

!     --- Fill specific arrays for derivs

      iread(1) = -1   ! input stored in inpder
      iread(2) = 0    ! sequence number of coefficients

      inpder(1) = 9  ! number of positions filled
      inpder(2) = 0   ! output vector of type  110 (solution vector)
      inpder(3) = 0   ! real vector
      inpder(4) = 44  ! icheld
      inpder(5) = 1   ! ix
      inpder(6) = 1   ! jdegfd
      inpder(7) = 0   ! ivec
      inpder(8) = 2   ! Number of vectors in islold
      inpder(9) = 0   ! clear vector
      iprob = ks(islold(1))%iprob

      if ( debug ) then

!     --- Debug information

         call prinin ( inpder, 9, 'inpder' )
         call prinin ( iread, 2, 'iread' )
         write(irefwr,*) 'iseqcoff, iprob ', iseqcoff, iprob

      end if  ! ( debug )

!     --- Actual filling of M*H

      call sepderivs ( inpder, ioutput, &
                       islold, iread, output, iprob, idummy )

!     --- Multiply by 1/dt

      usoltemp => ks(ioutput)%sol
      usoltemp = usoltemp/dt

1000  if ( debug ) then

!     --- Debug information

         call sepprintv ( ioutput, 'M*H' )
         write(irefwr,*) 'End prcompenthalp4'

      end if
      call erclos ( 'prcompenthalp4' )

      end
