      subroutine prcomppsiprb ( surfaces, curves, numcurvs, numsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kprob with respect to computation of pseudo stream function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     stream_function
!     free_surface
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleinput
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcurvs, numsurfs, &
              surfaces(numsurfs), curves(numcurvs)

!     curves         i    array containing the curve numbers
!     numcurvs       i    Number of curves in array curves
!     numsurfs       i    number of surfaces in array surfaces
!     surfaces       i    array containing the surface numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: leniinput=200
      integer ichprb, iinput(leniinput), i, istart, icurnr, isurnr, &
              iseqprob

!     i              Counting variable
!     ichprb         Choice parameter for subroutine PROBDF
!     icurnr         Curve sequence number
!     iinput         integer input array for subroutine probdf
!     iseqprob       Sequence number of problem input
!     istart         Starting address information in iinput
!     isurnr         Surface sequence number
!     leniinput      Length of array iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomppsiprb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomppsiprb'
         call prinin ( surfaces, numsurfs, 'surfaces' )
         call prinin ( curves, numcurvs, 'curves' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array iinput

      iinput = 0

      iinput(1) = leniinput  ! declared length
      iinput(3) = leniinput  ! declared length
      iinput(5) = 1          ! nprob
      iinput(6) = 1          ! ntype
      iinput(7) = 1          ! numnatbnd
      iinput(10) = numsurfs  ! numnatsrf
      iinput(12) = numcurvs  ! numesscrv
      iinput(20) = 10        ! ivers
      istart = 66            ! starting address information
      iinput(istart) = 0     ! type number 0 for volumes
      iinput(istart+1) = 811 ! type number 811 for surface
      istart = istart+2

!     --- Boundary elements

      do i = 1, numsurfs
         isurnr = surfaces(i)       ! surface number
         iinput(istart) = 1         ! boundary group number
         iinput(istart+1) = 1       ! number of surfaces
         iinput(istart+2) = isurnr  ! first surface number
         istart = istart+3          ! raise istart
      end do  ! i = 1, numsurfs

!     --- essential boundary conditions

      do i = 1, numcurvs
         icurnr = curves(i)                  ! curve number
         iinput(istart) = 1                  ! bit pattern
         iinput(istart+1) = 0                ! jstep
         iinput(istart+2) = 1                ! number of curves
         iinput(istart+3) = icurnr           ! curve number
         istart = istart+4                   ! raise istart
      end do  ! i = 1, numcurvs
      call inichkarlen ( istart-1, leniinput, 'iinput' )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( iinput, istart-1, 'iinput' )

!     --- Fill array kprob

      nprob = 1

      allocate ( iinputtmp(istart-5) )
      iinputtmp = iinput(6:istart)
      allocate(rinputtmp(1))

      ichprb = 1    ! User defines input in iinput and rinput
      iseqprob = 0  ! Sequence number of input
      call probdfext ( ichprb, iseqprob )
      deallocate(iinputtmp)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcomppsiprb'

      end if  ! ( debug )
      call erclos ( 'prcomppsiprb' )

      end
