      subroutine prdelmats ( massmt, stiffm, mattot, irhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete matrices and right-hand-side vector for time dependent problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     delete
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer massmt, stiffm, mattot, irhsd(5)

!     irhsd         i/o   Standard SEPRAN array, containing information of the
!                         large vector (must be deleted)
!     massmt        i/o   SEPRAN array, containing information of the mass
!                         matrix (must be deleted)
!     mattot        i/o   SEPRAN array, containing information of the matrix
!                         M/dt + S (must be deleted)
!     stiffm        i/o   SEPRAN array, with information of the stiffness matrix
!                          (must be deleted)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPRMMATRIX    Remove array with sequence number mmindex from buf
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prdelmats' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prdelmats'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( massmt> 0 ) call seprmmatrix ( massmt )
      if ( stiffm> 0 ) call seprmmatrix ( stiffm )
      if ( mattot> 0 ) call seprmmatrix ( mattot )
      call seprmsolinfo ( irhsd(1) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prdelmats'

      end if  ! ( debug )
      call erclos ( 'prdelmats' )

      end subroutine prdelmats
