      subroutine prdividereac1 ( npoint, reac, massmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-01-2008 Allow negative values in massmt
!        version  1.0  date 25-08-2005
!
!   copyright (c) 2005-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the reaction force vector by the diagonal mass matrix
!     defined by the surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     subdivision
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint
      double precision massmt(*), reac(*)

!     massmt         i    mass matrix defined for the interface only
!     npoint         i    Number of nodal points
!     reac          i/o   Vector of reaction forces
!                         At output this vector is subdivided by the mass matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prdividereac1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prdividereac1'
         call prinrl ( reac, npoint, 'reac' )
         call prinrl ( massmt, npoint, 'massmat' )

      end if
      if ( ierror/=0 ) go to 1000
      do i = 1, npoint
         if ( abs(massmt(i))>0d0 ) reac(i) = reac(i)/massmt(i)
      end do  ! i = 1, npoint

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( reac, npoint, 'reac/massmat' )
         write(irefwr,*) 'End prdividereac1'

      end if
      call erclos ( 'prdividereac1' )

      end
