      subroutine prdividereac3 ( npoint, reac, massmt, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide all components of the reaction force vector by
!     the diagonal mass matrix defined by the surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     subdivision
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ndegfd
      double precision massmt(*), reac(ndegfd,*)

!     massmt         i    mass matrix defined for the interface only
!     ndegfd         i    Number of degrees of freedom per point in the
!                         reaction force.
!                         This number is supposed to be constant.
!                         Only the first degree of freedom is subdivided by the
!                         mass matrix
!     npoint         i    Number of nodal points
!     reac          i/o   Vector of reaction forces
!                         At output this vector is subdivided by the mass matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prdividereac3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prdividereac3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, npoint
         if ( massmt(i)>0d0 ) &
            reac(1:ndegfd,i) = reac(1:ndegfd,i)/massmt(i)
      end do  ! i = 1, npoint

1000  call erclos ( 'prdividereac3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prdividereac3'

      end if  ! ( debug )

      end
