      subroutine prdivreac ( isol, iseqreac, iseqresul )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the reaction force vector by the diagonal mass matrix
!     defined by the surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     subdivision
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iseqreac, iseqresul

!     iseqreac       i    Sequence number of reaction vector in array ISOL
!     iseqresul      i    Sequence number of result vector in array ISOL
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRDIVIDEREAC1  Subdivide the reaction force vector by the diagonal mass
!                    matrix defined by the surface
!                    1 degree of freedom per point
!     PRDIVIDEREAC3  Subdivide all components of the reaction force vector by
!                    the diagonal mass matrix defined by the surface
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3048   Number of degrees of freedom not constant
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prdivreac' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prdivreac'
         write(irefwr,1) 'iseqreac', iseqreac
         call prinin ( isol(iseqreac), 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      reac => ks(isol(iseqresul))%sol
      if ( iseqresul/=iseqreac ) reac => ks(isol(iseqreac))%sol

      if ( indprf/=0 ) then

!     --- Error: number of degrees of freedom per point not constant

         call errsub ( 3048, 0, 0, 0 )
         go to 1000

      end if  ! ( indprf==0 )

      if ( nunkplc==1 ) then

!     --- ndegfd = 1, only one component in the vector

         call prdividereac1 ( npoint, reac, matrixm )

      else

!     --- nunkplc>1, more components in the vector
!         all are subdivided

         call prdividereac3 ( npoint, reac, matrixm, nunkplc )

      end if  ! ( ndegfd==1 )

!     --- remove mass matrix

      deallocate ( matrixm )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prdivreac'

      end if  ! ( debug )
      call erclos ( 'prdivreac' )

      end subroutine prdivreac
