      subroutine prenthrewind ( npoint_new, iseqtemp, iseqnrenth )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.3  date 05-13-2013 Extension of inpsep
!        version  1.2  date 11-12-2012 Long integers
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set the time one time step back. Reset the temperature to the
!     appropriate temperature for the old time level.
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint_new, iseqtemp, iseqnrenth

!     iseqnrenth     i    sequence number of enthalpy input
!     iseqtemp       i    Sequence number of temperature in solution vector
!     npoint_new     i    Number of points of the new mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpsep(0:3), istart, numveclc
      double precision rinsep(1)

!     inpsep         The integer input read is stored in array INPSEP
!     istart         dummy
!     numveclc         Number of vectors stored in isol
!     rinsep         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCRET         Create vector
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prenthrewind' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prenthrewind'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Switch back to previous time level

      t = t - tstep
      timefinished = .false.

!     --- Rewind one time step: read initial temperature from input

      istart = istartmn
      numveclc = numvecmn
      istartmn = 0
      numvecmn = 0
      inpsep(0) = 4
      inpsep(1) = iseqnrenth
      inpsep(2) = nprob
      inpsep(3) = iseqtemp

      call ercret ( inpsep, rinsep )
      istartmn = istart
      numvecmn = numveclc

1000  call erclos ( 'prenthrewind' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prenthrewind'

      end if

      end
