      subroutine prfillinterf ( iobst, ninterf, coorinter, ninterfnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of interface nodes into array coorinter
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, ninterf, ninterfnew
      double precision coorinter(ndim,ninterfnew)

!     coorinter      o    Contains coordinates of the interface nodes
!                         in the local numbering of the interface
!     iobst          i    Obstacle sequence number
!     ninterf        i    Number of nodes in interface
!     ninterfnew     i    Length of array coorinter
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numbndobst

!     istart         Starting address
!     numbndobst     Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPREAL2DIN Copy the nnodes coordinates indicated by array nodes
!                    from array coor into array coorcopy of size nnodes x ndim
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     PRFILLINTERF1  Fill coordinates of interface nodes into array coorinter
!     PRINRL1        Print 2d real vector
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillinterf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillinterf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc

!     --- actual filling of coordinates of interface

      call prfillinterf1 ( coor, iobst, ndim, icrossobst, rcrossobst, &
                           nmeshc, ninterf, coorinter )

!     --- Get number of nodes at bounday of interface

      call inizerolev ( ipointobst, istart, numbndobst, iobst )

      if ( debug ) &
         call prinrl1 ( coorinter, ninterf, ndim, 'coorinter/1' )

      if ( numbndobst>0 ) then

!     --- There are nodes at the interface
!         Add the coordinates of the nodes to coorinter

         call inicopreal2din ( ipointobst(istart+1), numbndobst, &
                               coor, ndim, coorinter(1,ninterf+1) )

      end if  ! ( numbndobst>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coorinter, ninterf+numbndobst, ndim, &
                        'coorinter' )
         write(irefwr,*) 'End prfillinterf'

      end if  ! ( debug )
      call erclos ( 'prfillinterf' )

      end
