      subroutine prgetninterf ( ilev, numintersect, numelms, numbndobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get number of nodes in interface for obstacle ilev
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ilev, numintersect, numelms, numbndobst

!     ilev           i    Obstacle sequence number
!     numbndobst     o    Number of nodal points on the boundary of the obstacle
!     numelms        o    Number of elements in interface
!     numintersect   o    Number of intersections of interface with edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istarticross

!     istarticross   Starting address in array icrossobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgetninterf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgetninterf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- actual call

      istarticross = icrossobst(ilev)
      numintersect = icrossobst(istarticross)
      istarticross = istarticross+2*numintersect
      numelms = icrossobst(istarticross+1)

!     --- Get number of nodes at bounday of interface

      call inizerolev ( ipointobst, istarticross, numbndobst, ilev )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgetninterf'

      end if  ! ( debug )
      call erclos ( 'prgetninterf' )

      end
