      subroutine prkirchtrans ( rinenthalpy, islold, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 01-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform temperature in normalized temperature and compute
!     the corresponding enthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer islold(*), ichoice
      double precision rinenthalpy(*)

!     ichoice        i    defines the type of transformation
!                         0: Transform physical temperature into normalized
!                            Kirchoff temperature and compute enthalpy
!                         1: Transform normalized Kirchoff temperature back to
!                            physical temperature
!                         2: Compute enthalpy from physical temperature
!                         3: Transform physical temperature into normalized
!                            Kirchoff temperature
!     islold        i/o   User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nusol, iseqenth, iseqtemp

!     i              Counting variable
!     iseqenth       Sequence number of enthalpy with respect to array isol
!     iseqtemp       Sequence number of temperature with respect to array isol
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRKIRCHOFF     Transform temperature to normalized Kirchhoff temperature
!                    and compute enthalpy or transform  normalized Kirchhoff
!                    temperature back
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set parameters
!     Activate the temperature vector
!     Activate or create the enthalpy vector
!     Compute starting addresses
!     Transform
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prkirchtrans' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prkirchtrans'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set parameters, a fixed sequence is used for enthalpy and temperature
!         This is not the real solution vector, but a temporary vector used in
!         the time integration subroutine

      iseqenth = 1
      iseqtemp = 2

!     --- Activate or create the enthalpy vector

      call sepcreasolbf1 ( islold(iseqenth), ks(islold(iseqtemp))%typevec, &
                           ks(islold(iseqtemp))%ivec, 'enthalpy', 0 )
      enthalpy => usoltemp
      temperature => ks(islold(iseqtemp))%sol

!     --- Transform

      nusol = size(enthalpy)
      call prkirchoff ( enthalpy, temperature, &
                        nusol, ichoice, rinenthalpy )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prkirchtrans'

      end if
      call erclos ( 'prkirchtrans' )

      end subroutine prkirchtrans
