      subroutine prmakedistmk1 ( nnodes, nodes, gradphi, iwork, eps, &
                                 ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all nodes in band around interface with | ||grad phi||-1 |<eps
!     with -10 or 10
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     mark
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), iwork(*), ndim
      double precision eps, gradphi(ndim,*)

!     eps            i    Local accuracy with respect to grad phi
!     gradphi        i    Contains the gradient of phi
!     iwork         i/o   integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers inside the region where the
!                         accuracy is tested
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision normphi

!     i              Counting variable
!     j              Counting variable
!     normphi        norm of grad pi in 1one point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistmk1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistmk1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nnodes

         j = nodes(i)
         normphi = dot_product(gradphi(1:ndim,j),gradphi(1:ndim,j))
         normphi = sqrt(normphi)

!        --- if the distance is already correct mark iwork as outside band
!            if iwork(i)>0: 10 else -10

         if ( abs(normphi-1d0)<eps ) iwork(i) = sign(10,iwork(i))

      end do  ! i = 1, nnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistmk1'

      end if  ! ( debug )
      call erclos ( 'prmakedistmk1' )

      end
