      subroutine prsetcells ( set, nset, ndim, ishift, nx, indi, &
                              lenset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-09-2012 Extra parameter lenset
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     make a new set of cells, given the index of the center cell
!     and the shift
! **********************************************************************
!
!                       KEYWORDS
!
!     cell
!     neighbour
!     interface
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, lenset, set(ndim,lenset), nset, ishift, nx(ndim), &
              indi(ndim)

!     indi           i    contains ijk-index of cell in which point is lying
!     ishift         i    Shift with respect to center cell
!     lenset         i    Declared length of array set
!     ndim           i    Dimension of the space.
!     nset           o    Number of elements in set
!     nx             i    number of points in ndim directions of the grid
!     set            o    contains the ijk innidces of the set of cells
!                         we consider
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRSETCELLS2D   make a new set of cells, given the index of the center cell
!                    and the shift (2d)
!     PRSETCELLS3D   make a new set of cells, given the index of the center cell
!                    and the shift (3d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     3112           value of ndim not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetcells' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetcells'
         write(irefwr,1) 'lenset', lenset
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nset = 0

      select case ( ndim )

         case(2)

!        --- ndim = 2, find all cells with index i+/-ishift,j+/-ishift

            call prsetcells2d ( set, nset, ndim, ishift, nx, indi, &
                                lenset )

         case(3)

!        --- ndim = 3, find all cells with index i+/-ishift,j+/-ishift, k..

            call prsetcells3d ( set, nset, ndim, ishift, nx, indi, &
                                lenset )


         case default

!        --- value of ndim not yet implemented

            call errchr ( 'ndim', 1 )
            call errint ( ndim, 1 )
            call errsub ( 3112, 1, 0, 1 )

      end select  ! case ( ndim )

      ishift = ishift+1

1000  call erclos ( 'prsetcells' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsetcells'

      end if  ! ( debug )

      end
