      function prsetmindist ( set, nset, ndim, nx, grid, &
                              numfound, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-09-2012 Extra debug
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with the number of nodes of the interface that are
!     inside the set of cells
! **********************************************************************
!
!                       KEYWORDS
!
!     cell
!     interface
!     level_set
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nset, ndim, set(ndim,nset), nx(ndim), grid(*), nodes(*), numfound
      logical prsetmindist

!     grid           i    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the folwoing way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     ndim           i    Dimension of the space.
!     nodes          o    Contains the node numbersof the interface nodes
!                         which are in cells of set
!     nset           i    Number of nodes in set
!     numfound       i    Number of points in nodes
!     nx             i    number of points in x direction
!     prsetmindist   o    If true the cells contain at least one interface
!                         node
!     set            i    Contains the cells that must be investigated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iposgrid, istart, numnodes

!     i              Counting variable
!     iposgrid       Position of node in grid
!     istart         Starting address in grid
!     numnodes       Number of nodal points in cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniposgrid

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INIPOSGRID     Computes the starting position of cell defined by indi in
!                    grid
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetmindist' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetmindist'
         call ertrace
         write(irefwr,1) 'nx, ndim, nset', nx, ndim, nset
         call prinin1 ( set, nset, ndim, 'set' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Detect the number of nodes of the interface in the set

      numfound = 0
      do i = 1, nset

         iposgrid = iniposgrid ( set(1,i), nx, ndim )

         if ( debug ) &
           write(irefwr,1) 'i, iposgrid, set', &
                            i, iposgrid, set(:,i)

         istart = grid(iposgrid)
         numnodes = grid(iposgrid+1)-istart  ! number of nodes in interface
         if ( numnodes>0 ) then

!        --- There are numnodes nodes of the interface in this grid cell
!            Store these nodes in nodes

            nodes(numfound+1:numfound+numnodes) = &
               grid(istart:istart+numnodes-1)
            numfound = numfound+numnodes

         end if   !( numnodes>0 )

      end do  ! i = 1, nset

      prsetmindist = numfound>0

1000  call erclos ( 'prsetmindist' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsetmindist'

      end if  ! ( debug )

      end
