      subroutine pzidrshadow ( sparam, nrusol, pshadowc, parinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-10-2014 New parameter list
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 24-03-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Compute shadow vectors p for IDR process
!      Store in pshadow
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     parallel
!     idr
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer parinfo(*)
      integer sparam, nrusol
      complex * 16 pshadowc(nrusol,sparam)

!     nrusol         i    Number of free unknowns
!     parinfo       i/o   Contains information about special parallel arrays
!                         See subroutine probparinfo1
!     pshadowc       o    Array of size n x s
!                         Contains all shadow vectors of size n each
!                         So pshadow(i,j) contains i^th component of p_j
!     sparam         i    Defines the parameters s in IDR(s)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision pshadow(nrusol,sparam)

!     i              Counting variable
!     j              Counting variable
!     pshadow        Array of size n x s
!                    Contains all shadow vectors of size n each
!                    So pshadow(i,j) contains i^th component of p_j
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRANDOM        generates a random vector in parallel
!     PRINRL1        Print 2d real vector
!     PRSINGVALDEC   Compute singular value decomposition of array a
!                    Returns with orthogonal basis of space spanned by A
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pzidrshadow' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pzidrshadow'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- The vectors in P are chosen randomly

      do i = 1, sparam
         call prandom ( nrusol, pshadow(1,i) )
      end do  ! i = 1, sparam

      if ( debug ) call prinrl1 ( pshadow, sparam, nrusol, 'Pshadow' )

!     --- P will be defined as an orthonormal basis in the
!         space spanned by the column vectors
!         This is done by applying svdcmp

      call prsingvaldec ( pshadow, nrusol, sparam )
      pshadowc = 0d0
      do i = 1, sparam
         do j = 1, nrusol
            pshadowc(j,i) = pshadow(j,i)
         end do  ! j = 1, nrusol
      end do  ! i = 1, sparam

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( pshadow, sparam, nrusol, 'Pshadow' )
         write(irefwr,*) 'End pzidrshadow'

      end if  ! ( debug )
      call erclos ( 'pzidrshadow' )

      end
