      subroutine sepconvsubstp1 ( finished, first, isubr, iinnavstokes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve one step of the time-dependent Navier-Stokes equations
!     Use an explicit method for the convection with smaller time steps
!     (Substepping)
!     The method applied is a form of operator splitting
!     Old subroutine prconvsubstep1
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     navier_stokes_equation
!     time_integration
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulemain
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr, iinnavstokes(*)
      logical, intent(in) :: first
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     first          i    If true it concerns the first time step
!     iinnavstokes   i    Integer input array for Navier-Stokes
!                         See subroutine readnavstokes for a description
!     isubr          i    Indicates the type of calling subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps
      double precision :: timebefore(2)

!     eps            Local accuracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     SEPTIMCONVSUB  Solve one step of the time-dependent Navier-Stokes
!                    equations
!                    Use an explicit method for the convection with smaller
!                    time steps (Substepping)
!     SEPTIMOUTP     Perform output for subroutine septimprb during a time loop
!     SEPTIMSET      Set some time parameters in the time loop
!     SEPTIMSTART    Start part of subroutine septimprb
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepconvsubstp1' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepconvsubstp1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Start part of subroutine sepconvsubstp1
!         Fill input arrays and constants and write initial condition

      complrun = .false.
      call septimstart ( first, eps, isubr )
      if ( ierror/=0 ) go to 1000

!     --- While loop time-method depending on ibuffr(ipintm) and rintim

      if ( itimloop>=1 ) then

!     --- First set tstep and tout

         call septimset ( first, eps )
         if ( ierror/=0 ) go to 1000

!        --- Call time integration subroutine and raise t

         call septimconvsub ( first, iinnavstokes )
         if ( ierror/=0 ) go to 1000

!        --- Perform output during a time loop

         call septimoutp ( eps )
         if ( ierror/=0 ) go to 1000

      end if

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         finished = .true.

      end if  ! ( t>=tend-0.00001*tstep )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepconvsubstp1'

      end if
      call erclmn ( 'sepconvsubstp1', timebefore )

      end
