      subroutine sepdiffmain2 ( iinputlc, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.3  date 11-12-2012 Long integers
!        version  1.2  date 17-03-2009 New call to readsp
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the first part of subroutine sepdiffmain1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      use sepmodulemach
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*),  manag(lenmanag)

!     iinputlc            Work array of length iinputlc(1) to store the input
!                         from file 73 temporarily.
!                         It contains the user numbers of the solution
!                         vectors to be read
!     manag          o    Manager array that sets the parameters in subroutine
!                         SEPSTL
!                         See subroutine readsp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iref, ihelp(10), ih, nextrecord
      logical check

!     check          Indicates if the file exist (true) or not
!     i              General counting variable
!     ih             Help varaible to store a constant temporarily
!     ihelp          Work array to store input from the input file
!     iref           Help variable to store a reference number
!     nextrecord     Defines last record read by readsp
!                    Possible values:
!                    0: none
!                    1: PROBLEM
!                    2: POSTPROCESSING
!                    3: MESH
!                    4: READ MESH
!                    5: EXTRA_MESH_INPUT
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     INITMX         Initialize parameters xmin, xmax, etc and view point
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     READPROBIINP   Read information concerning the input for probdfbf from
!                    file sepcomp.out and call subroutine probdfbf
!     READSP         Reads information for subroutine SEPSTL
!     SEPFEMESH      creates a linear element mesh based on the nodes
!                    of a spectral element mesh
!     SEPSTART       Standard SEPRAN starting subroutine
! **********************************************************************
!
!                       I/O
!
!    The files meshoutput, sepcomp.inf and sepcomp.out and if it exists
!    sepcomp.freq are read
!    For a description of meshoutput: see the Programmers Guide Section 3.3
!    For a description of the file sepcomp.inf, see subroutine PROUTP
!    For a description of the file sepcomp.out, see subroutine PRINSL
!    For a description of the file sepcomp.freq, see subroutine FVWRITE
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1669   file does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill common blocks
!     Open files
!     Read array manag
!     Start SEPRAN
!     Open files with mesh and solution
!     Read mesh
!     Read problem definition from sepcomp.inf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdiffmain2'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Start the program

      call initcb

      cputiming = .true.
      iprogram = 3

      call initmd

!     --- Open read and write files

      call inifil(5)
      call inifil(6)

!     --- Open standard elements file and error message file

      call inifil(1)
      call inifil(4)

!     --- Read array manag (defaults)

      manag(5) = 10001
      call readsp ( manag, lenmanag, nextrecord )

!     --- Standard SEPRAN start subroutine

      if ( mod(manag(1),100)==0 ) manag(1) = manag(1) + 100
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )
      mainsubr = 4
      call eropen ( 'sepdiffmain1' )
      call eropen ( 'sepdiffmain2' )
      if ( debug ) write(irefwr,*) 'after startbf'

!     --- Open files meshoutput and nameout1/2 and read mesh

      call inifil ( -10 )
      inquire ( file=nameout2, exist=check )
      if ( .not. check ) then

!     --- file does not exist

         call errchr ( nameout2, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if  ! ( .not. check )
      if ( ierror/=0 ) go to 1000

      kmesh => kmesh2d(1)
      call msh071 ( manag(5), iref10, ihelp )
      iref = abs(iref10)
      close ( unit = iref )
      if ( debug ) write(irefwr,*) 'after msh071'
      if ( ierror/=0 ) go to 1000

!     --- Compute xmin, xmax etc.

      call initmx ( ndim, npoint, coor )
      if ( debug ) write(irefwr,*) 'after initmx'
      if ( ierror/=0 ) go to 1000

!     --- Check if spectral mesh has been found
!         If so call femesh

      if ( km(iseqkm)%kmesh(38)/=0 ) then

!     --- Spectral mesh: fill fem mesh in kmesh2d(.,2)

         iref = 22
         call sepfemesh ( iref, 1 )
         close ( unit = iref )

!        --- interchange kmesh2d(1) and kmesh2d(2)

         ih = kmesh2d(1)
         kmesh2d(1) = kmesh2d(2)
         kmesh2d(2) = ih

      end if

      if ( debug ) write(irefwr,*) 'after femeshbf'
      if ( ierror/=0 ) go to 1000

!     --- Read problem definition

      call inifil ( -74 )
      kmesh => kmesh2d(1)
      call readprobiinp
      if ( debug ) write(irefwr,*) 'after readprobiinp'

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepdiffmain2'

      end if
      call erclos ( 'sepdiffmain2' )

      end subroutine sepdiffmain2
