      subroutine sepmatrev ( irowsnew, matnew, irowsii, icolsii, matold, &
                             nrusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy matrix from rowwise to columnwise storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol, irowsii(nrusol+1), icolsii(*)
      integer, intent(in) :: irowsnew(nrusol+1)
      double precision, intent(in) :: matold(*)
      double precision, intent(out) :: matnew(*)

!     icolsii        i    Contains the input column structure
!     irowsii        i    Contains the input row structure
!     irowsnew       i    Contains the row structure of the output matrix
!     matnew         o    New matrix
!     matold         i    Original matrix
!     nrusol         i    Number of free unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, m, ih, ih1
      integer, pointer :: point(:)

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     j              Counting variable
!     m              Counting variable
!     point          pointer array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatrev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatrev'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( point(nrusol) )

      point = irowsnew(1:nrusol)  ! pointer to irowsnew in icolsnew
      ih = 1
      do i = 1, nrusol
         do j = irowsii(i)+1, irowsii(i+1)
            m = icolsii(j)
            point(m) = point(m)+1
            matnew(point(m)) = matold(ih)     ! upper part of matrix
            ih = ih+1
         end do
      end do

      deallocate ( point )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatrev'

      end if  ! ( debug )
      call erclos ( 'sepmatrev' )

      end subroutine sepmatrev
