      subroutine sepnewmarkupdh ( irhsdold, irhsdlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute F* = alpha_s F^n - M alpha_m a^n - S alpha_f u^n
!     Also compute h^n = u^n + dt v^n + dt^2/2 ( 1 - 2 beta_g) a^n
!     Old subroutine pr1114updh
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     newmark
!     generalized_alpha
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irhsdold(*), irhsdlc

!     irhsdlc        i    Standard SEPRAN array, containing information of the
!                         large vector
!     irhsdold      i/o   Sepran array to store the vector
!                         alpha_f F^n - alpha_m M a^n  - alpha_f S u^n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp, ihelp1

!     ihelp          integer work array to store M a^n
!     ihelp1         integer work array to store S u^n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnewmarkupdh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnewmarkupdh'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ihelp = 0
      ihelp1 = 0

      if ( irhsdold(1)>0 .and. t>t0*(1d0+sqreps) ) then

!     --- Generalized alpha method
!         Compute M a^n including boundary conditions
!         Store in ihelp

         call sepmaver ( massmat, isolut2dred(Teq(3)%iseqfr), &
                         ihelp, 5 )

!        --- Compute S u^n including boundary conditions
!            Store in ihelp1

         call sepmaver ( stiffmat, isolut2dred(iseqfrtim), &
                         ihelp1, 5 )

!        --- Compute alpha_f F^n - alpha_m M a^n - alpha_f S u^n
!            store in irhsdold(.,1)

         if (debug ) then
            write(irefwr,2) 'alpha_f, alpha_m', alpha_f, alpha_m
            call sepprintv ( ihelp, 'M a^n' )
         end if  ! (debug )
         rhsdcp => ks(irhsdlc)%sol
         help => ks(ihelp)%sol
         help1 => ks(ihelp1)%sol
         u3 => ks(irhsdold(1))%sol
         u3 = alpha_f*rhsdcp - alpha_m*help - alpha_f*help1
         if (debug ) call sepprintv ( irhsdold, 'irhsdold' )

      end if  ! ( irhsdold(1)>0 )

!     --- Compute vector h = u^n + dt v^n + dt^2/2 ( 1 - 2 beta_g) a^n
!         Store in irhsdold(.,2) (including boundary conditions)
!         In case of Newton only h = dt v^n + dt^2/2 ( 1 - 2 beta_g) a^n

      if ( lineartim==3 .or. lineartim==4 ) then

!     --- lineartim=3, 4, Updated Lagrange method(Newton)


         u1 => ks(isolut2dred(Teq(2)%iseqfr))%sol
         u2 => ks(isolut2dred(Teq(3)%iseqfr))%sol
         call sepcreavec ( isolut2dred(Teq(2)%iseqfr), irhsdold(2) )
         u3 => ks(irhsdold(2))%sol
         u3 = tstep*u1+0.5d0*tstep**2*(1d0-2d0*beta_g)*u2

      else

!     --- lineartim<3, standard method

         u1 => ks(isolut2dred(iseqfrtim))%sol
         u2 => ks(isolut2dred(Teq(2)%iseqfr))%sol
         u3 => ks(isolut2dred(Teq(3)%iseqfr))%sol
         call sepcreavec ( isolut2dred(iseqfrtim), irhsdold(2) )
         unew => ks(irhsdold(2))%sol
         unew = u1 + tstep*u2 + 0.5d0*tstep**2*(1d0-2d0*beta_g)*u3

      end if  ! ( lineartim==3, 4 )
      if ( debug ) call sepprintv ( irhsdold(2), 'h' )

      if ( ihelp>0 ) call seprmsolinfo ( ihelp )
      if ( ihelp1>0 ) call seprmsolinfo ( ihelp1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnewmarkupdh'

      end if  ! ( debug )
      call erclos ( 'sepnewmarkupdh' )

      end
