      subroutine sepprojmain
! ======================================================================
!
!        programmer    Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates projection vectors for the deflated iccg algorithm. The
!     space spanned by the projection vectors approximates the space
!     spanned by the small eigenvectors.
!     Old subroutine prprojmain
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduleproj
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, idegfd
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Loop variable
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEP2DPROJE     Make a reference of the 2d array projE2d to the
!                    pointer A%projE
!     SEPMATLABPROJ  Write projection vectors in matlab format
!     SEPPJMATE      Computes the vectors A u and the Cholesky decomposition
!                    of the matrix E
!     SEPPJSPAR1     Creates projection vectors (physical approach)
!     SEPPJSPAR2     Creates projection vectors (on base of 0 and 1)
!     SEPPJSPAR3     Creates projection vectors (real properties)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1662   The value of method is not equal to 5
!    1663   Diagonal scaling has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprojmain' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprojmain'
         write(irefwr,1) 'nrusol, proj_type, kelmw', nrusol, proj_type, kelmw
         call prinin ( inpsol, 20, 'inpsol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      IM => intmt(realmt(iseqmt)%intmatref)

!     --- ASSUMED IS THAT WE HAVE A SYMMETRICAL SPARSE MATRIX

      if ( .not. IM%symmetric ) then
         call errint ( IM%jmethod, 1 )
         call errsub ( 1662, 1, 0, 0 )
      end if

!     --- Eisenstat preconditioning has not been implemented yet

      if ( ipreco==2 .or. ipreco==7 ) call errsub ( 1663, 0, 0, 0 )
      if ( ierror/=0 ) go to 1000

      if ( print_lv>=1 .or. ioutp>=1 ) write(irefwr,100)
100   format ( ' Making projection vectors...' )

      if ( proj_type<=2 ) then

!     --- proj_type = 1, 2: classical method
!         proj_type = 1: number of deflation vectors is equal to
!                        number of number of sand layers that are
!                        not subject to Dirichlet conditions
!         proj_type = 2: projection vectors for each connected set
!         else if proj_type = 3: projection vectors for each
!                                element group

         call seppjspar1

      else if ( kelmw>0 ) then

         if ( nrealprop>0 ) then

!        --- New method, using real properties in each element group

            call seppjspar3

         else

!        --- Same method, using only 0 and 1 in each element group
!            Integer properties defined

            call seppjspar2

         end if  ! ( kelmw>0 )

      end if  ! ( proj_type<=2 )

      if ( print_lv>=1 ) write(irefwr,180) dble(nnproj)/dble(nrusol)
180   format ( '       -  total length projection vectors =', f4.1, &
                  ' * n (degrees of freedom)' )
      if ( print_lv>=1 ) write(irefwr,190) idint( 100*dble(nnproj) / &
        dble(nproj*nrusol))
190   format ( '       -  average sparsity =', I4, '%' )
      if ( print_lv>=1 ) write(irefwr,200)
200   format ( '  [7] computing Au and E' )

!     --- write deflation vectors to file deflation.dat

      if ( write_matlab_mat ) call sepmatlabproj

      allocate ( projAu(nnproj), projE(nproj*nproj), stat = error )
      if ( error/=0 ) call eralloc ( error, nnproj+nproj*nproj, 'projAu' )
      if ( ierror/=0 ) go to 1000

      call sep2dprojE ( projE )

!     --- Computing Au and E^-1

      call seppjmate

      if ( print_lv>=1 ) write(irefwr,210)
210   format ( '  [8] finishing up' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprojmain'

      end if  ! ( debug )
      call erclos ( 'sepprojmain' )

      end
