      subroutine septimbouncnd ( first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform solution vector to local coordinates
!     Fill essential boundary conditions in solution vector
!     Old subroutine prtimbouncond
! **********************************************************************
!
!                       KEYWORDS
!
!     local_transform
!     time_dependent
!     time_integration
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: first

!     first          i    If true it concerns the time first step
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPTIMBOUNCOND Fill boundary conditions in array isolut2d and check if the
!                    array corresponds to the correct problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimbouncnd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimbouncnd'

      end if
      if ( ierror/=0 ) go to 1000

!     --- In case of local transformations we have to transform
!         un, since boundary conditions are always given in
!         transformed form

      call sepfillkmesh ( kmesh )
      call seploctrn ( 1, isolut2d(Teq(1)%iseqfr) )

!     --- Put new boundary conditions into u* &

      if ( debug ) write(irefwr,*) &
         'itimbc, iseqfr, first ', itimbc, Teq(1)%iseqfr, first
      if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!     --- Either first call, or boundary conditions are time-dependent
!         The boundary conditions are always given in the transformed form

         if ( debug ) &
            write(irefwr,*) 'ireadbctim, iseqfr ', &
                             ireadbctim, Teq(1)%iseqfr
         isol2d => isolut2d(Teq(1)%iseqfr:size(isolut2d))
         isolut2dred => isolut2d
         call septimbouncond ( first )

      end if  ! ( itimbc<2 .and. (first .or. itimbc==0) ) then
      if ( debug ) &
         call sepprintv ( isolut2d(Teq(1)%iseqfr), 'u bc' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( isolut2d(Teq(1)%iseqfr), 5, 'isolut2d(Teq(1)%iseqfr)' )
         write(irefwr,*) 'End septimbouncnd'

      end if
      call erclos ( 'septimbouncnd' )

      end
