      subroutine septimehis2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create real arrays for time history and fill partly
!     Old subroutine prtimehis2
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduletimhis
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecons
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lenpoints=100
      integer :: i, nch1, icoor, ipoints(lenpoints)
      character (len=40) :: namear

!     i              Counting variable
!     icoor          If>0 a timehistory of coordinates must be made
!     ipoints        Contains the nodal points where the time histor. &
!                    must be stored
!     lenpoints      Declared length of array ipoints
!     namear         Name of array
!     nch1           Number of characters in name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRTIMEHIS3     Fill one time step in time history array for one specific
!                    solution array
!     PRTIMEHIS6     Fill one time step in time history array for coor array
!     PRTIMEHIS8     Fill one time step in time history array for zero level
!                    set
!     PRTIMHISNSTEP  Estimate number of time steps
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimehis2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimehis2'
         write(irefwr,2) 'tstep, toutstep', tstep, toutstep
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoor = maxval(vecttim(maxvc1+1:maxvc1+3)%npoint_hist)
      if ( ivecttimlast==0 .and. icoor==0 ) go to 1000  ! no time hist required

!     --- Estimate number of time steps

      call prtimhisnstep ( nstephis )
      istephis = 1           ! first time step

!     --- Loop over all vectors

      do i = 1, ivecttimlast

         npoint_hist = vecttim(i)%npoint_hist
         if ( npoint_hist>0 ) then

!        --- Time history of vector must be stored

            nch1  = index(names(i),' ')-1
            namear = names(i)(1:nch1)//'_timehis'

!           --- Activate arrays

            if ( ierror/=0 ) go to 1000

            ivecttim => vecttim(i)%point_nrs
            zerolev = vecttim(i)%zerolev
            call prtimehis8 ( ipoints, lenpoints )
            if ( ierror/=0 ) go to 1000

            call sepactsolbf2 ( isolut2d(i) )

!           --- Create array

            allocate ( vecttim(i)%arr(nphysi*npoint_hist+1,nstephis) )
            rvectim => vecttim(i)%arr
            rvectim(1,:) = 0d0

!           --- Compute starting positions

            vectin => ks(isolut2d(i))%sol

!           --- Fill first time step

            call prtimehis3 ( ipoints )

         end if  ! ( vecttim(i)%npoint_hist>0 )

      end do  ! i = 1, ivecttimlast

      do i = maxvc1+1, maxvc1+ndim

         npoint_hist = vecttim(i)%npoint_hist
         if ( npoint_hist>0 ) then

!        --- Time history of coor vector must be stored

            namear = 'coor_timehis'


!           --- Activate arrays

            ivecttim => vecttim(i)%point_nrs
            zerolev = vecttim(i)%zerolev
            call prtimehis8 ( ipoints, lenpoints )
            if ( ierror/=0 ) go to 1000

!           --- Create array

            allocate ( vecttim(i)%arr(npoint_hist+1,nstephis) )
            rvectim => vecttim(i)%arr

!           --- Fill first time step

            call prtimehis6 ( i-maxvc1, ipoints )

         end if  ! ( vecttim(i)%npoint_hist>0 )

      end do  ! i = 1, ndim

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'npoint_hist, nstephis', npoint_hist, nstephis
         write(irefwr,*) 'End septimehis2'

      end if  ! ( debug )
      call erclos ( 'septimehis2' )

      end subroutine septimehis2
