      subroutine septimextra ( ieqn, first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform extra actions in time step
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ieqn
      logical, intent(in) :: first

!     first          i    If true it concerns the first time step
!     ieqn           i    Equation sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iaction, istart

!     iaction        Defines action to be performed
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRDERV         Compute derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3098   Value of parameter not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimextra' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimextra'
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iaction = Teq(ieqn)%extratim(1)

      if ( debug ) write(irefwr,1) 'iaction', iaction

      if ( iaction==8 ) then

!     --- A derivative must be computed

         if ( .not. first ) itimloop = itimloop+1
         istart = istartmn
         istartmn = 0
         isolut2d => isolut2d(Teq(ieqn)%iseqfr:maxvec)
         call prderv ( Teq(ieqn)%extratim(2) )
         if ( .not. first ) itimloop = itimloop-1
         istartmn = istart

      else

!     --- Unknown action

         call errchr ( 'iaction', 1 )
         call errint ( iaction, 1 )
         call errsub ( 3098, 1, 0, 1 )

      end if
      isolut2d => isolut3d(:,meshnr)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimextra'

      end if  ! ( debug )
      call erclos ( 'septimextra' )

      end subroutine septimextra
