      subroutine septimiinbld ( first, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 02-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill some parameters in iinbldtim and initialize some
!     other parameters for time integration
!     Old subroutine: pr1105iinb
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ieqn
      logical :: first

!     first          i    If true it concerns the first step
!     ieqn           i    Equation sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimiinbld' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimiinbld'
         write(irefwr,*) 'first', first
         write(irefwr,1) 'maxcalltim', maxcalltim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqim = intmatar(ieqn)
      iprobtim = intmt(iseqim)%iprob

      if ( .not. first ) then

!     --- standard situation change iinbld(2) if necessary
!         Next time step

         if ( Teq(ieqn)%iconsstiff==1 ) then
            if ( Teq(ieqn)%iconsrhs==0 ) then
               iinbldtim(2) = 2
            else
               iinbldtim(2) = 4
            end if
         else if ( Teq(ieqn)%iconsrhs==1 ) then
            iinbldtim(2) = 14
         end if
         if ( Teq(ieqn)%iconsmass==1 ) iinbldtim(4) = 0

      end if  ! ( .not. first .and. ieqn==1 .and. maxcalltim<=1 )

      if ( ncopeqtim>0 ) then

!     --- information per equation

         iinbldtim(15) = Teq(ieqn)%isubdiv

      end if

      iinbldtim(9) = iprobtim
      iinbldtim(10) = numoldtim

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimiinbld'

      end if  ! ( debug )
      call erclos ( 'septimiinbld' )

      end subroutine septimiinbld
