      subroutine septimoutp ( eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform output for subroutine septimprb during a time loop
!     Old subroutine prtimprb03
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: eps

!     eps            i    Local accuracy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROUTP         Performs the actual write actions for OUTPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimoutp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimoutp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icall_outtim==1 .and. toutnext<=t+eps .and. complrun ) then

!     --- toutnext < t, perform output and raise toutnext

         write(irefwr,50) t
 50      format( ' time = ',f12.5 )
         call proutp ( 3, t, 1, iseqouttim, 1, .false., &
                       Teq(1)%iseqfr )
210      toutnext = toutnext+toutstep
         if ( toutstep>0d0 .and. toutnext<t+2d0*eps ) go to 210
         if ( toutnext>tendnext+0.5d0*toutstep ) toutnext = tend + 100d0

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimoutp'

      end if  ! ( debug )
      call erclos ( 'septimoutp' )

      end subroutine septimoutp
