      subroutine septimseqintm ( iseqintmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-12-2017 New storage of intmat
!        version  1.1  date 17-12-2015 New call to incommat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array intmat if necessary and initialize iseqintmat
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: iseqintmat

!     iseqintmat     o    Sequence number with respect to array intmat
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lniincommat=25
      integer :: iincommat(lniincommat), iprob
      logical :: fillintmat

!     fillintmat     If true intmat has been filled
!     iincommat      Integer input array containing information concerning
!                    the structure of the large matrix
!                    See subroutine read00 for a description
!     iprob          Actual problem number
!     lniincommat    Declared length of array iincommat
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimseqintm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimseqintm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intmat => intmatar(iprobtim)
      fillintmat = intmat<=0 .or. intmat>lastintmat
      if ( .not. fillintmat ) then
         fillintmat = intmatarr(intmat)==0
      end if  ! ( .not. fillintmat ) then

      if ( fillintmat ) then

!     --- intmat has not yet been filled

         iincommat = 0
         iincommat(2) = 2  ! jmethod
         iincommat(5) = iprobtim
         call sepcommat ( iincommat )
         if ( ierror/=0 ) go to 1000
         iseqintmat = iprobtim
         if ( debug ) call prinin1 ( intmat1d, iprobtim, 5, 'intmat1d' )

      else

         iprob = intmt(intmat)%iprob
         if ( iprob==iprobtim ) then
            iseqintmat = iprobtim
         else
            iseqintmat = 1
         end if

      end if  ! ( fillintmat ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimseqintm'

      end if  ! ( debug )
      call erclos ( 'septimseqintm' )

      end subroutine septimseqintm
