      subroutine septimsolve ( matrtot, isol, ihelp, ieqn, irhsdlc, first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 05-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations
!     one equation and one time level in case of a time-dependent problem
!     If necessary the reaction force is computed
!     Old subroutine:  prtimsolve
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulematstr
      use sepmodulemain
      use sepmodulematr
      use sepmoduleintmat
      use sepmoduletime
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ihelp, ieqn, irhsdlc
      integer, intent(inout) :: matrtot, isol(*)
      logical, intent(in) :: first

!     first          i    If true it concerns the first time step
!     ieqn           i    Equation sequence number
!     ihelp          i    Help array to store information of a intermediate
!                         vector
!                         IN this case it contains the adapted right-hand side
!                         for the system of equations
!     irhsdlc        i    Standard SEPRAN array, containing information of the
!                         standard right-hand side vector
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     matrtot        i/o   SEPRAN array, containing information of the matrix
!                         M/dt + S
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPADAPINPSOL  Adapt array inpsoltim. Fill array inpsol from inpsoltim
!     SEPLINSOL      Body of linear solver
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPSOLDIR      Actual body of subroutine SOLVE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimsolve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimsolve'
         write(irefwr,*) 'first', first
         call prinin ( matrtot, 5, 'matrtot' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqmt = matrtot

!     --- Adapt array inpsoltim
!         Fill array inpsol from inpsoltim

      call sepadapinpsol ( ieqn, first)

!     --- Solve system of equations

      if ( isol(ieqn)>0 .and. irhsdlc>0 ) &
         ks(isol(ieqn))%compl = ks(irhsdlc)%compl

      if ( .not. realmt(iseqmt)%diagonal ) then

!     --- Non-diagonal matrix

         ireadinpsol = -1
         iseqsol = ieqn
         iseqreac = 0
         iseqintmat = 1
         keepmat = 0
         iseqsolut = Teq(1)%iseqfr
         iseqmt = matrtot
         call seplinsol ( matrtot, isol, ihelp )

      else

!     --- Diagonal matrix

         call sepsoldir ( matrtot, isol(ieqn), ihelp )

      end if  ! ( .not. realmt(iseqmt)%diagonal )

      if ( irhsdlc>0 ) ks(isol(ieqn))%compl = ks(irhsdlc)%compl
      if ( debug ) call sepprintv ( isol(ieqn), 'u^n+1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimsolve'

      end if  ! ( debug )
      call erclos ( 'septimsolve' )

      end subroutine septimsolve
