      subroutine toaddrowcomp ( relemmt, icnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble one element matrix in the large matrix
!     real matrix with compact row storage
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: icnt
      double precision, intent(in) :: relemmt(icount,icount)

!     icnt           o    Number of non-prescribed unknowns in element
!     relemmt        i    Element matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, imatrix, inusol, knusol, nusolmax, nusolmin
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     imatrix        Position in matrix
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     knusol         Position of unknown in array USOL
!     nusolmax       Maximum column number in a row
!                    In symmetrical case the row number
!     nusolmin       Minimum column number in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2899   The matrix element is not present in the reduced storage, &
!           but the element in the element matrix is not equal to 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddrowcomp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddrowcomp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)
      icolsii => IM%colsii
      irowsii => IM%rowsii
      icnt = 0
      nusolmin = 1

      do i = 1, icount

         inusol=indexren(i)
         if ( inusol<=nrusol ) then

!        --- Only free degrees of freedom are added

            do j = irowsii(inusol)+1, irowsii(inusol+1)
               iwork(icolsii(j)) = j
            end do  ! j = irowsii(inusol)+1, irowsii(inusol+1)
            nusolmax = icolsii(irowsii(inusol+1))

            if ( IM%symmetric ) then
               if ( IM%mumps ) then
                  nusolmax = inusol
               else
                  nusolmin = inusol
              end if  ! ( IM%mumps )
            end if  ! ( IM%symmetric )
            icnt = icnt+1

            do j = 1, icount

               knusol = indexren(j)
               if ( knusol>=nusolmin .and. knusol<=nusolmax ) then

!              --- Real unknown

                  imatrix = iwork(knusol)
                  if ( imatrix>0 ) then

!                 --- imatrix>0, standard case

                     Smat%S_ii(imatrix)=Smat%S_ii(imatrix)+relemmt(j,i)

                  else if ( abs(relemmt(j,i))>0d0 ) then

!                 --- imatrix=0, but element in element matrix not equal to 0
!                     This is an error

                     call errint ( i, 1 )
                     call errint ( j, 2 )
                     call erreal ( relemmt(j,i), 1 )
                     call errsub ( 2899, 2, 1, 0 )

                  end if  ! ( imatrix>0 )

               end if  ! ( knusol<nrusol )

            end do  ! j = 1, icount

         end if  ! ( inusol<=nrusol )

      end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddrowcomp'

      end if  ! ( debug )
      call erclos ( 'toaddrowcomp' )

      end
