      subroutine toassemblemmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 24-07-2015 Use sepmodulebuild
!        version  3.3  date 11-04-2015 Remove ibcmat
!        version  3.2  date 05-09-2014 Extension with petsc
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble one element matrix in the mass matrix
!     real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     diagonal_matrix
!     mass_matrix
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenim2, icnt, i, nsum
      logical :: found
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     found          If true already a possibility has been found
!     i              Counting variable
!     icnt           Number of non-prescribed unknowns in element
!     lenim2         length of the array imat part 2
!     nsum           nrusol+ninterface+1, i.e. position of first prescribed
!                    unknown
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0008         Add element vector to right-hand side
!     TO0207         add element matrix to large matrix part 2, 3, 4
!     TOADDMATFI     Assemble one element matrix in the matrix part S_fi
!     TOADDMATIF     Assemble one element matrix in the matrix part S_if
!     TOADDPETSC     Add part of element matrix corresponding to real unknowns
!                    to petsc matrix
!     TOADDROWCOMP   Assemble one element matrix in the large matrix
!                    real unsymmetric matrix with compact row storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toassemblemmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toassemblemmat'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if (debug ) &
         write(irefwr,1) 'notmas, imas, itype, ielgrp, nelgrp', &
                          notmas, imas, itype, ielgrp, nelgrp

      if ( notmas==0 .and. imas==1 .and. itype/=996 .and. &
           ielgrp<=nelgrp ) then

!     --- Diagonal mass matrix; skip type 996

         call to0008 ( elemms, M_ii )

      else if ( notmas==0 .and. imas==2 .and. itype/=996 .and. &
                ielgrp<=nelgrp ) then

!     --- Non-diagonal mass matrix; skip type 996

         Smat => realmt(massmat)
         if ( IM%nosepmat ) then

!        --- building of matrix by petc

            call toaddpetsc ( elemms, icnt )

         else

!        --- real compact symmetric row storage

            call toaddrowcomp ( elemms, icnt )

         end if  ! ( IM%nosepmat )

!        --- add element matrix to second part of mass matrix

         if ( icnt<icount ) then

            nrusol = nusol-nbound
            call to0207 ( elemms )
            nrusol = IM%nrusol

         end if

         if ( icnt<icount .and. ninterface>0 .and. .not. IM%nosepmat ) then

!        --- Not all unknowns are internal and there are interface unknowns
!            First we check if there are interface unknowns in the element

            nsum = nrusol+ninterface+1
            found = .false.
            do i = 1, icount
               if ( indexren(i)>nrusol .and. indexren(i)<nsum ) then
                  found = .true.
                  exit
               end if  ! ( indexren(i)>nrusol .and. indexren(i)<nsum )
            end do  ! i = 1, icount

            if ( found ) then

!           --- ipicolsfi>0: Fill matrix M_fi and M_if

               call toaddmatfi ( elemms )
               call toaddmatif ( elemms )

            end if  ! ( ninterface>0 )

         end if  ! ( icnt<icount .and. ninterface>0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toassemblemmat'

      end if  ! ( debug )
      call erclos ( 'toassemblemmat' )

      end
