      subroutine toassemblevec
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.1  date 15-11-2012 Long integers
!        version  1.0  date 05-12-2008
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble one element vector in the large vector
!     Real vector
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     vector
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!                    they are not
!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     TO0008         Add element vector to right-hand side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toassemblevec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toassemblevec'
         write(irefwr,*) 'vector ', vector
         call prinrl ( elemvc, icount, 'elemvc' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( vector ) then

!     --- Vector true, assemble right-hand side

         do i = 1, numrhs
            rhsd => rhsds(i)%rh
            call to0008 ( elemvc2d(1,i), rhsd )
         end do  ! i = 1, numrhs

      end if  ! ( vector )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toassemblevec'

      end if  ! ( debug )
      call erclos ( 'toassemblevec' )

      end
