      subroutine tofillcmat ( elemmp, jwork, icnt, nwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Extra parameter nwork
!        version  1.2  date 21-09-2011 Extra debug
!        version  1.1  date 27-10-2010 Correction reset jwork
!
!   copyright (c) 2009-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the matrix Cmat for the SIMPLE algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmodulematstr
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nwork
      integer, intent(inout) :: jwork(nwork)
      integer, intent(out) :: icnt
      double precision, intent(in) :: elemmp(icount,icount)

!     elemmp         i    element matrix after mapping
!                         rows and columns corresponding to prescribed degrees
!                         of freedom are eliminated
!                         elemmp is stored row-wise (mathematical sequence)
!                         So in fortran we have to use the transpose fo elemmp
!     icnt           o    Number of non-prescribed unknowns in element
!     jwork         i/o   integer work array of size nu
!                         In jwork the sequence numbers of the columns of the
!                         non-zero elements in one row of the matrix are stored
!                         At input jwork must be 0
!                         At output jwork is reset to 0
!     nwork          i    Declared length of jwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, inusol, jnusol, iamat, jmethodpr, nusolmax, nusolmin

!                    they are not
!     i              Counting variable
!     iamat          Position in array AMAT
!     inusol         Position of unknown in array USOL (row number)
!     j              Counting variable
!     jmethodpr      jmethod with respect to pr0015
!     jnusol         Position of unknown in array USOL (column number)
!     nusolmax       Maximum column number in a row
!                    In symmetrical case the row number
!     nusolmin       Minimum column number in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0004         Print INTMAT corresponding to internal matrix
!     PRININ         print 1d integer array
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2899   The matrix element is not present in the reduced storage,
!           but the element in the element matrix is not equal to 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( jmethod = 13 or 15 ) then
!        For i := 1 (1) icount do
!           inusol := indexren(i)
!           if ( inusol<=nrusolvel ) then
!              Fill jwork with sequence numbers
!              Raise S_C
!              For j := 1 (1) icount do
!                 if ( j # i ) then
!                    jnusol := indexren(j)
!                    if ( jnusol = inusol ) then
!                       Add element to S_C
!                    else if ( jnusol<=nrusolvel ) then
!                       Add element to S_CL
!     else ( jmethod = 14 or 16 ) then
!        For i := 1 (1) icount do
!           inusol := indexren(i)
!           if ( inusol<=nrusolvel ) then
!              Fill jwork with sequence numbers
!              Raise S_C
!              For j := 1 (1) icount do
!                 if ( j # i ) then
!                    jnusol := indexren(j)
!                    if ( jnusol = inusol ) then
!                       Add element to S_C
!                    else if ( jnusol<=nrusolvel ) then
!                       Add element to S_CL
!                       Add element to S_CU
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofillcmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofillcmat'
         call pr0004 ( jmethod, nrusolpress, irowsPM, icolsPM )
         call prinin ( indexren, icount, 'indexren' )
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'nrusolvel, nrusolpress, jmethod, icount', &
                          nrusolvel, nrusolpress, jmethod, icount

      end if
      if ( ierror/=0 ) go to 1000
      icnt = 0
      nusolmin = 1

!     --- For i := 1 (1) icount do

      do i = 1, icount

!     --- inusol := indexren(i)

         inusol = indexren(i)
         if ( debug ) write(irefwr,1) 'i, inusol', i, inusol
         if ( inusol>nrusolvel .and. inusol<=nrusolvel+nrusolpress ) then

!        --- inusol>nrusolvel, i.e. row corresponds to c matrix
!            Fill jwork with sequence numbers
!            Raise S_C

            inusol = inusol-nrusolvel
            if ( debug ) write(irefwr,1) 'i, inusol', i, inusol
            do j = irowsPM(inusol)+1, irowsPM(inusol+1)
               jwork(icolsPM(j)) = j
            end do  ! j = irowsPM(inusol)+1, irowsPM(inusol+1)
            nusolmax = icolsPM(irowsPM(inusol+1))
            icnt = icnt+1

            nusolmin = inusol

!           --- For j := 1 (1) icount do

            do j = 1, icount

         !      if ( j/=i ) then

!              --- j # i, i.e. we are in an off-diagonal of the element
!                  matrix
!                  jnusol := indexren(j)

                  jnusol = indexren(j)-nrusolvel
                  if ( debug ) &
                     write(irefwr,1) 'j, jnusol', j, jnusol
        !          if ( jnusol==inusol ) then

!                 --- jnusol = inusol, this means element not on the
!                     diagonal of the element matrix but still on
!                     the diagonal of the matrix
!                     This is for example possible in case of degenerate
!                     element
!                     Add element to S_C

                  if ( jnusol>=nusolmin .and. jnusol<=nusolmax ) then

!                 --- jnusol>nrusolvel, i.e. column corresponds to c matrix
!                     Add element to S_CL

                     iamat = jwork(jnusol)
                     if ( iamat>0 ) then

!                    --- iamat>0, standard case

                        S_C(iamat) = S_C(iamat) + elemmp(j,i)

                     else if ( abs(elemmp(j,i))>0d0 ) then

!                    --- iamat=0, but element in element matrix not equal
!                                to 0 This is an error

                        call errint ( i, 1 )
                        call errint ( j, 2 )
                        call erreal ( elemmp(j,i), 1 )
                        call errsub ( 2899, 2, 1, 0 )

                     end if  ! ( iamat>0 )

                  end if  ! ( jnusol==inusol )

!               end if  ! ( j/=i ) then

            end do  ! j = 1, icount

!           --- Clear jwork

            do j = irowsPM(inusol)+1, irowsPM(inusol+1)
               jwork(icolsPM(j)) = j
            end do  ! j = irowsPM(inusol)+1, irowsPM(inusol+1)

         end if  ! ( inusol<=nrusolvel )

      end do   ! i = 1, icount

1000  if ( debug ) then
         call sepprinmat6 ( C )
         write(irefwr,*) 'End tofillcmat'
      end if  ! ( debug )
      call erclos ( 'tofillcmat' )

      end
