      subroutine tofillvelmmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 24-07-2015 Extra debug
!        version  2.2  date 24-07-2015 Extra debug
!        version  2.1  date 21-04-2009 New storage for block matrices
!
!   copyright (c) 2007-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add diagonal of element velocity mass matrix to
!     global velocity mass matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     diagonal_matrix
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulematstr
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, jpos

!                    they are not
!     i              Counting variable
!     jpos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofillvelmmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofillvelmmat'
         call ertrace
         call prinrl ( E%dvelmmat, icount, 'E%dvelmmat' )
         call prinin ( indexren, icount, 'indexren' )
         write(irefwr,1) 'nrusolvel', nrusolvel
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, icount
         jpos = indexren(i)
         if ( jpos<=nrusolvel ) then
            S_UD(jpos) = S_UD(jpos)+E%dvelmmat(i)
            if ( debug ) then
               write(irefwr,1) 'i, jpos', i, jpos
               write(irefwr,2) 'S_UD(jpos), E%dvelmmat(i)', &
                                S_UD(jpos), E%dvelmmat(i)
            end if  ! ( debug )
         end if  ! ( indexren(i)<=nrusolvel )
      end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( S_UD, nrusolvel, 'Velocity Mass matrix' )
         write(irefwr,*) 'End tofillvelmmat'

      end if  ! ( debug )
      call erclos ( 'tofillvelmmat' )

      end
