      subroutine chrols ( istart, level, mset, &
                          lastpoint, kpoint, ndist, idist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-04-2006 Fortran 90
!        version  1.0  date 03-07-1995 Renamed subroutine chroot
!
!   copyright (c) 1995-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Generate root level structure corresponding to root istart
!    Only the last level is stored
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, target :: level(*)
      integer :: idist(*), istart, mset, kpoint, ndist, lastpoint

!     idist   o  When ndist>0 in idist the distance of each node to
!                istart times ndist is stored
!     istart  i  Root of rooted level structure
!     kpoint  o  Number of nodal points that have been found in mesh
!                If kpoint<lastpoint the mesh contains more than one
!                connected parts
!     lastpoint  i  Number of points in mesh
!     level      o  Last level of rooted level structure
!     mset       o  Number of elements in level nlevel
!     ndist      i  indication if array idist must be filled (ndist>0) or
!                   not (ndist=0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nstore

!     i       Counting variable
!     nstore  Number of elements in nextlevel
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set first level equal to start point
!     Initialize parameters
!     While ( number of elements in the last set>0 ) do
!        Find next level of neighbours
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      allocate ( nextlevel(npoint) )

!     --- Set first level equal to start point;  Initialize parameters

      do i = 1, lastpoint
         if ( marknodes(i)>-10 ) marknodes(i) = 0
      end do
      if ( istart>0 ) then

!     --- istart>0, one start point

         level(1) = istart
         if ( ndist>0 ) idist(istart) = 0
         mset = 1
         marknodes(istart) = -1
      else

!     --- istart<0, the start is stored in level(1-mset)

         if ( ndist>0 ) then
            do i = 1, mset
               idist(level(i)) = 0
            end do
         end if
         do i = 1, mset
            marknodes(level(i)) = -1
         end do
      end if
      nlevel = 1
      kpoint = mset

!     --- Find next levels

150   if ( mset>0 ) then

!     --- The level next to level is found by sepnextlevel and stored in nextlevel
!          Number of elements in nextlevel is equal to nstore

         lastlevel => level(1:mset)
         call sepnextlevel ( -1, nstore )
         if ( nstore>0 ) then

!        --- nstore>0, hence there are nstore neighbours
!                        Refill array level and reset parameters

            if ( ndist>0 ) then

!           --- ndist>0   fill part of idist

               do i = 1, nstore
                  idist(nextlevel(i)) = nlevel*ndist
               end do
            end if
            nlevel = nlevel+1
            do i = 1, nstore
               level(i) = nextlevel(i)
            end do
            mset = nstore
            kpoint = kpoint+nstore
            go to 150
         end if
      end if

      deallocate ( nextlevel )
      end
