      subroutine chtmc1 ( nposet, ksetnp, minpos, nbr )
! ======================================================================
!
!        programmer     Guus Segal
!        version 3.0    date   29-11-1996 ifirst removed
!        version 2.0    date   22-11-1996 Extra parameter ifirst
!        version 1.2    date   22-12-1994 New layout
!
!
!   copyright (c) 1983-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute nodal point with minimum number of neighbours in the set ksetnp
!    Return with index of this point and number of neighbours
! **********************************************************************
!
!                       KEYWORDS
!
!    renumbering
!    topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ksetnp(*), nposet, minpos, nbr

!     ksetnp  i    array containing the set of nodal points
!     minpos  o    at output minpos gives the position of the nodal point
!                  with the fewest neigbours in ksetnp
!     nbr     o    number of neighbours of nodal point ksetnp(minpos)
!     nposet  i    length of set ksetnp
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, min, inodp

!     i       Counting variable
!     inodp   Nodal point number
!     min     Help variable to compute a minimum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    min := infinity
!    For i := 1 (1) number_of_points in ksetnp
!       nbr := number of neighbours of node ksetnp(i)
!       if ( nbr < min ) then
!          minpos := i
!          min := nbr
!    nbr := min
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmc1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmc1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- find nodal point with minimal number of neighbours in ksetnp

      min = 100000
      do 100 i = 1, nposet
         inodp = ksetnp(i)
         nbr = kmshe1(inodp+1)-kmshe1(inodp)
         if ( nbr<min ) then
            minpos=i
            min=nbr
         end if
100   continue
      nbr = min

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End chtmc1'

      end if  ! ( debug )
      call erclos ( 'chtmc1' )

      end subroutine chtmc1
