      subroutine incm01clear ( numnei, icheck, istore, kfree, nfdeg, &
                               diffunk, node, nodp1, iwork, iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 12-10-2007
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear array icheck
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnei, icheck(*), kfree(2,*), nfdeg, node, &
              iwork(3,*), iseqlc,nodp1, istore(nusol,2)
      logical diffunk

!     diffunk        i    If true there are coupled nodes with boundary
!                         conditions that are not periodical
!     icheck        i/o   work array of length nusol to indicate if a degree of
!                         is present in array istore (1) or not (0)
!     iseqlc           i    Sequence number
!     istore         i    work array of length nusolx2, &
!                         see subroutine incm01basis
!     iwork          i    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incm01basis
!     kfree          i    Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     nfdeg          i    Number of free degrees of freedom in a point
!     node           i    Node number
!     nodp1          i    Node sequence number
!     numnei         i    Number of neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inusol, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01clear' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01clear'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numnei
         icheck( istore(i,1) ) = 0
      end do
      do i = 1, nfdeg
         icheck( kfree(1,i) ) = 0
      end do

      if ( diffunk ) then

!     --- diffunk = true, special situation
!         neighburing nodes are also filled

         nodp1 = node
         inusol = sepgetinusol ( nodp1, inunkp )
         do i = 1, inunkp
            icheck(kprobr(inusol+i)) = 0
         end do
         do j = iwork(2,iseqlc), iwork(2,iseqlc+1) - 1
            nodp1 = iwork(3,j)
            inusol = sepgetinusol ( nodp1, inunkp )
            do i = 1, inunkp
               icheck(kprobr(inusol+i)) = 0
            end do
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incm01clear'

      end if  ! ( debug )
      call erclos ( 'incm01clear' )

      end
