      subroutine incm01neigh ( kfree, ihelp, nodp1, icheck, numnei, &
                               istore, nrep, diffunk, inext, iwork, &
                               extraneigh, knunkp, numneimid, inunkpmin, &
                               ihelpunk, ilowhelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 12-10-2007
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     consider the neighbors of node
!     put real degrees of freedom in istore
!     Mark those one used in icheck
!     Repeat for periodical node
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfree(2,*), ilowhelp, nodp1, icheck(*), numnei, &
              nrep, inext, iwork(3,*), &
              knunkp, numneimid, inunkpmin, ihelpunk(*), &
              ihelp(ilowhelp:*), istore(nusol,2)
      logical diffunk, extraneigh

!     diffunk        i    If true there are coupled nodes with boundary
!                          conditions
!                         that are not periodical
!     extraneigh     i    If true extra fill-in for one row of extra neighbors
!                         is required
!     icheck        i/o   work array of length nusol to indicate if a degree of
!                         is present in array istore (1) or not (0)
!     ihelp          i    integer work array of length max(1+nrusol,npoint+1)
!                         In the case of periodical boundary conditions it is
!                         used as work array to compute the chain
!                         Futhermore it is used to indicate if a point belongs
!                         to the set of periodical points corresponding to a
!                         specific periodic point in the following way:
!                         ihelp(i) = 1, point i corresponds to the periodic
!                         point
!                         ihelp(i) = 0, it does not
!     ihelpunk       i    Work array indicating if unknowns in coupled nodes are
!                         periodical (0) or not (1)
!     ilowhelp       i    Lower boundary of array ihelp
!     inext         i/o   Next point
!     inunkpmin      i    Lowest degree of freedom that must be taken into
!                         account
!     istore        i/o   work array of length nusol to store the non-prescribed
!                         neighbor degrees of freedom of a point
!                         At input this array is 0
!                         At output it contains neighbors of periodical nodes
!     iwork          i    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incm01basis
!     kfree          i    Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     knunkp         i    number of degrees of freedom
!     nodp1          i    Node sequence number
!     nrep          i/o   Number of repetitions for a set of periodical nodes
!     numnei        i/o   Number of neighbor unknowns stored in istore
!     numneimid      o    Number of standard neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos, minsol, ibr, ineigh, inusol, i, knusol, jbr, jpos, &
              jneigh, inunkp
      logical useneigh

!     i              Counting variable
!     ibr            Loop variable for neighbors
!     ineigh         set of new neighbors
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     ipos           Position in array
!     jbr            Loop variable for neighbors
!     jneigh         neighbor
!     jpos           Position in array
!     knusol         Position of unknown in array USOL
!     minsol         Minimum value of knusol corresponding to one row
!     useneigh       If true extra neighbors must be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01neigh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01neigh'
         write(irefwr,1) 'nrep, inext, nodp1', nrep, inext, nodp1
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

250   ipos = kmshe1(nodp1)
      minsol = kfree(1,1)

      if ( debug ) write(irefwr,1) 'minsol', minsol
      do ibr = 1, kmshe1(nodp1+1)-ipos
         ineigh = kmshe2(ipos+ibr)
         if ( debug ) &
            write(irefwr,1) 'ibr, ineigh, ihelp(ineigh)', &
                             ibr, ineigh, ihelp(ineigh)
         if ( ihelp(ineigh)==0 ) then

!        --- neighbor is not other periodical point

            inusol = sepgetinusol ( ineigh, inunkp )
            if ( debug ) write(irefwr,1) 'inusol, inunkp', inusol, inunkp
            do i = 1, inunkp
               if ( indprr==0 ) then
                  knusol = inusol+i
               else
                  knusol = kprobr(inusol+i)
               end if
               if ( icheck(knusol)==0 ) then

!              --- icheck(knusol) = 0, i.e. neighbor corresponds to
!                  free unknown and has not yet been put into istore

                  numnei = numnei+1
                  istore(numnei,1) = knusol
                  istore(numnei,2) = i
                  minsol = min(minsol,knusol)
                  icheck(knusol) = 1

               end if

            end do

         else if ( diffunk ) then

!        --- neighbor is other periodical point and not all unknowns
!            are coupled
!            Put those that are not coupled into istore

            inusol = sepgetinusol ( ineigh, inunkp )
            do i = 1, inunkp
               if ( ihelpunk(i)==1 ) then
                  knusol = kprobr(inusol+i)
                  if ( icheck(knusol)==0 ) then

!                 --- icheck(knusol) = 0, i.e. neighbor corresponds to
!                     free unknown and has not yet been put into istore

                     numnei = numnei+1
                     istore(numnei,1) = knusol
                     istore(numnei,2) = i
                     minsol = min(minsol,knusol)
                     icheck(knusol) = 1

                  end if
               end if

            end do
         end if

      end do
      if ( debug ) call prinin1 ( istore, 2, numnei, 'istore' )

      if ( nrep>1 ) then

!     --- nrep>1, decrease nrep and interchange nodp1
!         repeat the preceding process

         nodp1 = iwork(3,inext)
         inext = inext + 1
         nrep  = nrep - 1
         if ( debug ) write(irefwr,1) 'nodp1, inext, nrep', nodp1, inext, nrep

         go to 250

      end if

      numneimid = numnei
      useneigh = extraneigh
      if ( extrafillin>3 ) useneigh = knunkp>ndim+1
      if ( useneigh ) then

!     --- useneigh = true, take also neighbors of neighbors into
!         account

         do ibr = 1, kmshe1(nodp1+1)-ipos
            ineigh = kmshe2(ipos+ibr)
            if ( ihelp(ineigh)==0 ) then

!           --- neighbor is not other periodical point

               jpos = kmshe1(ineigh)
               do jbr = 1, kmshe1(ineigh+1)-jpos
                  jneigh = kmshe2(jpos+jbr)
                  if ( ihelp(jneigh)==0 ) then

!                 --- neighbor is new

                     inusol = sepgetinusol ( jneigh, inunkp )
                     if ( extrafillin==5 .and. inunkp<ndim+2 ) &
                         inunkp = 0
                     do i = inunkpmin, inunkp
                        if ( indprr==0 ) then
                           knusol = inusol+i
                        else
                           knusol = kprobr(inusol+i)
                        end if
                        if ( icheck(knusol)==0 .and. &
                             knusol>=minsol ) then

!                       --- icheck(knusol) = 0, i.e. neighbor corresponds to
!                           free unknown and has not yet been put into istore

                           numnei = numnei+1
                           istore(numnei,1) = knusol
                           istore(numnei,2) = i
                           icheck(knusol) = 1

                        end if

                     end do

                  end if

               end do

            end if

          end do

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( istore, 2, numnei, 'istore' )
         write(irefwr,*) 'End incm01neigh'

      end if  ! ( debug )
      call erclos ( 'incm01neigh' )

      end
