      subroutine incm04
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 06-01-2016 Use sepmodulematstr
!        version  7.0  date 17-12-2015 New parameter list
!        version  6.0  date 21-04-2015 New parameter list
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of array irowsic corresponding to the "right" part of
!     the periodical boundary conditions.
!     INCM04 may only be called if npboun>0
!     INCM04 must be filled after a call ini014, in fact it is the
!     counterpart of ini014 with respect to periodical boundary
!     conditions.
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer knunkp, ihigh, ilow, ikelmc, inodp, inusol, inpelm, &
              itype, ielgrp, nhelp1, nhelp2, j, knusol, nfnbnr, &
              ibr, ineigh, jnusol, iwork(npoint), inunkp

!     ibr            Relative neighbour number of node
!     ielgrp         Element group number
!     ihigh          Highest position in usol with respect to "right" periodical
!                    boundary conditions
!                    ihigh = nusol-nbound
!     ikelmc         Position in array KMESH part c
!     ilow           Lowest position number in usol with respect to "right"
!                    periodical boundary conditions.
!                    ilow = nusol-nbound-npboun+1
!     ineigh         Node number of neighbour of inodp
!     inodp          Nodal point number
!     inpelm         number of points per element
!     inusol         Position of first unknown in array usol corresponding
!                    to point inodp minus one,  before renumbering
!     itype          Type number of element group
!     iwork          Work array
!     j              Counting variable
!     jnusol         See inusol,  however,  with respect to ineigh
!     knunkp         Number of unknowns in node ineigh
!     knusol         Renumbered inusol or jnusol
!     nfnbnr         Help variable to compute neighbours of node inodp in
!                    KMSHE2 minus 1
!     nhelp1         Number of non-prescribed unknowns in "right" periodical
!                    node
!     nhelp2         Number of non-prescribed unknowns in connected to unknowns
!                    in "right" periodical node,  excluded the corresponding
!                    "left" periodical point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Fill array iwork
!
!    For inodp := 1 (1) npoint do
!       if (iwork(inodp)>0) then
!          compute inusol and inunkp
!          nhelp1 :=0 ; nhelp2 := 0
!          for j := 1(1) inunkp
!             compute knusol
!             if (knusol<=ihigh) then
!                nhelp2 := nhelp2+1
!                nhelp1 := 1
!          if (nhelp1>0) then
!             for ibr := 1 (1) number of neighbours of inodp do
!                ineigh := ibrth neighbour of inodp
!                if (iwork(inodp)# ineigh) then
!                   fill nhelp2 into irowsic
!
!    Accumulate irowsic
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set some constants

      knunkp = nunkp
      ikelmc = 0
      ihigh  = nusol-nbound
      ilow   = ihigh-npboun+1

!     --- fill array iwork

      iwork = 0

!     --- Clear array irowsic

      irowsic(1:npboun+1) = 0

!     --- loop over element groups

      do ielgrp = 1, nelgrp
         itype = kprobc(ielgrp)
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         if (itype==-1) then
            do inodp = 1, inelem

!           --- loop over elements

               iwork(kmeshc(ikelmc+2))=kmeshc(ikelmc+1)
               ikelmc = ikelmc+2

            end do  ! inodp = 1, inelem
         else
            ikelmc = ikelmc+inelem*inpelm
         end if

      end do  !  ielgrp=1, nelgrp

      if ( numpairs>0 ) then

!     --- numpairs>0, hence periodical boundary conditions

         inelem = numpairs
         do inodp = 1, inelem

!        --- loop over pairs

            iwork(kprobper(2,inodp)) = kprobper(1,inodp)

         end do  ! inodp = 1, inelem
         inelem = nelem

      end if  ! ( numpairs>0 )

!     --- loop over nodes

      do inodp = 1, npoint

         if (iwork(inodp)>0) then

!        --- iwork(inodp>0,  so node "right" periodical point

!              compute inunkp and inusol

            inusol = sepgetinusol ( inodp, inunkp )

            nhelp1 = 0
            nhelp2 = 0

!           --- loop over unknowns

            do j = 1, inunkp
               knusol = kprobh(inusol+j)
               if (knusol<=ihigh) then
                  nhelp2 = nhelp2+1
                  nhelp1 = 1
               end if
            end do

            if (nhelp1>0) then

!           --- nhelp1>0,  so there are free degrees of freedom in
!               periodical node

               nfnbnr = kmshe1(inodp)

!              --- loop over neighbours

               do ibr = 1, kmshe1(inodp+1)-nfnbnr

                  ineigh = kmshe2(nfnbnr+ibr)
                  if (iwork(inodp)/=ineigh) then

!                 --- ineigh # iwork(inodp),  i.e. ineigh is not "left"
!                     periodical point

                     jnusol = sepgetinusol ( ineigh, knunkp )

!                    --- loop over unknowns

                     do j = 1, knunkp
                        knusol = kprobh(jnusol+j)
                        if (knusol<=ihigh) nhelp2=nhelp2+1
                     end do

                  end if

               end do

!              --- Fill array irowsic with nhelp2

               do j = 1, inunkp
                  knusol = kprobh(inusol+j)
                  if (knusol>=ilow .and. knusol<=ihigh) &
                      irowsic(knusol-ilow+2) = nhelp2
               end do
            end if
         end if
      end do

!     --- accumulate irowsic

      do j = 2, npboun+1
         irowsic(j) = irowsic(j-1)+irowsic(j)
      end do

      end
