      subroutine incomgen ( iread, intorig )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 11-12-2017 New storage of intmat1d
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.4  date 17-12-2015 New call to incommat
!
!   copyright (c) 1996-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    General call to commat in free surface problems
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleinput
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iread, intorig(*)

!     intorig        i    Original value of intmat1d from which jmethod
!                         must be copied
!     iread          i    Defines how the input must be read
!                         Possible values:
!                          0:  The input is read from the standard input file
!                         >0:  The input as read from the standard input file
!                              by subroutine SEPSTM is used. The value of iread
!                              defines the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniincommat
      parameter ( lniincommat=25 )
      integer iincom, idummy(1), &
              iincop(lniincommat), i, maxcal, ipos

!     i              General loop parameter
!     idummy         Dummy parameter
!     iincom         Reference number with respect to array INFOR for array
!                    JMETOD
!     iincop         Copy of input array extended with defaults
!     ipos           Position of input arrays in sepmoduleinput
!     lniincommat    Length of iincommat
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Standard tests

      call eropen ( 'incomgen' )
      iseqkp = kprob
      nprob = kp(iseqkp)%nprob
      if ( iread<0 ) then

!     --- Input is not read from the standard input file

         do iprob = 1, nprob
            if ( intorig(iprob)>0 ) then
               iincop(2) = intmt(intorig(iprob))%jmethod
            else
               iincop(2) = 2
            end if  ! ( intorig(iprob)>0 )
            iincop(3:lniincommat) = 0
            iincop(1) = lniincommat
            iincop(4) = 0
            iincop(5) = iprob

!           --- Adapt variables

            call inichinpsol ( iincop, lniincommat )
            intmat => intmat1d(iprob)
            call sepcommat ( iincop )
            if ( ierror/=0 ) go to 1000

         end do  ! iprob = 1, nprob

      else

!     --- Input for matstruc has already been read before
!         Get this information and copy it to the back of IBUFFR
!         in order to prevent overwriting by incommat

         call sepgetinpfinf ( 3, 1, ipos, maxcal )

         if ( ierror/=0 ) go to 1000
         do iprob = 1, nprob

            do i = 1, lniincommat
               iincop(i) = in(ipos)%iinput((iprob-1)*lniincommat+i)
            end do

!           --- Adapt variables

            call inichinpsol ( iincop, lniincommat )
            intmat => intmat1d(iprob)
            call sepcommat ( iincop )

         end do

      end if

1000  call erclos ( 'incomgen' )

      end
