      subroutine incommatcols ( lenim2, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmodulematstr
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.0  date 21-04-2015 New parameter list
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill structure of matrix in case of a compact matrix
!     Part 1b, columns numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     iterative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenim2, iwork(*)

!     iwork               work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incommatcomp
!     lenim2         i    length of the array imat part 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCM02         Compute second part of contact matrix
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatcols' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatcols'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      if ( debug ) &
         write(irefwr,1) 'npoint, nelem, nusol, nrusol', &
                          npoint, nelem, nusol, nrusol

      if ( .not. associated(icolsii) ) then

!     --- icolsii has not been allocated yet

         allocate ( icolsii(lenim2), stat = error )
         if ( error/=0 ) call eralloc ( error, lenim2, 'isolsii' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( .not. associated(icolsii) )

!     --- Fill intmat part 1b: column numbers of non-zero elements per row

      call incm02 ( iwork )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( icolsii, 10, 'icolsii' )
         write(irefwr,*) 'End incommatcols'

      end if  ! ( debug )
      call erclos ( 'incommatcols' )

      end subroutine incommatcols
