      subroutine incommatinter2
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 08-05-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute integer information of matrix corresponding to
!     interface unknowns in case of parallel computing
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nodp, numintf, numfree, j, knusol, inunkp, &
                 leninterf, interf(1000), ibr, nbr, nfnbr, iknbr, &
                 indpres, iworki(1000), isum, istart, inusol

!     i              Counting variable
!     ibr            Loop variable for neighbors
!     iknbr          Nodal point number of neighbor
!     indpres        All unknowns with sequence number larger than indpres
!                    are prescribed
!     interf         Contains unknown numbers of interface
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     istart         First position used at a part
!     isum           Integer variable to compute a sum
!     iworki         Integer work array to store  free unknowns
!     j              Counting variable
!     knusol         Position of unknown in array USOL
!     leninterf      Number of interface unknowns in nodal point
!     nbr            number of neighbors of nodal point
!     nfnbr          starting address in array kmshe2
!     nodp           Nodal point sequence number
!     numfree        Number of free unknowns coupled to interface unknowns
!     numintf        Number of interface unknowns in nodal point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0005         Print INTMAT corresponding to boundary conditions
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     irowsfi(1:ninterface) := irowsfi(0:ninterface-1)
!     irowsif(1:nrusol) := irowsif(0:nrusol-1)
!     For all nodes nodp do
!        Store all interface unknowns in array interf
!        If ( there are interface unknowns in nodp ) then
!           numintf := number of interface unknowns in node
!           store free unknowns in node in array iworki
!           For all neighbor nodes ibr of nodp do
!              store interface unknowns in ibr in array iworkf
!              store free unknowns in ibr in array iworki
!           Fill information in icolsfi and raise irowsfi
!           Fill information in icolsif and raise irowsif
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatinter2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatinter2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      indpres = nrusol+ninterface  ! first index of prescribed unknowns (-1)

      if ( debug ) &
         write(irefwr,1) 'npoint, nrusol, ninterface, indpres', &
                          npoint, nrusol, ninterface, indpres

!     --- Reset irowsf.

      irowsfi(1:ninterface) = irowsfi(0:ninterface-1)
      irowsif(1:nrusol) = irowsif(0:nrusol-1)

!     --- For all nodes nodp do

      do nodp = 1, npoint

!     --- Store all interface unknowns in array interf

         inusol = sepgetinusol ( nodp, inunkp )
         numintf = 0

         if ( debug ) &
            write(irefwr,1) 'nodp, inunkp, inusol', nodp, inunkp, inusol

         do j = 1, inunkp

            knusol = kprobh(inusol+j)
            if ( knusol>nrusol .and. knusol<=indpres ) then

!           --- interface unknown found

               numintf = numintf+1
               interf(numintf) = knusol

            end if  ! ( knusol>nrusol .and. knusol<=indpres )

         end do  ! j = 1, inunkp

         if ( numintf>0 ) then

!        --- there are interface unknowns in nodp

            leninterf = numintf
            numfree = 0

            if ( debug ) call prinin ( interf, numintf, 'interf' )

            do j = 1, inunkp

               knusol = kprobh(inusol+j)
               if ( knusol<=indpres ) then

!              --- free unknown found

                  numfree = numfree+1
                  iworki(numfree) = knusol

               end if  ! ( knusol<=indpres )

            end do  ! j = 1, inunkp

!           --- For all neighbor nodes ibr of nodp do

            nfnbr = kmshe1(nodp)
            nbr = kmshe1(nodp+1)-nfnbr

            do ibr = 1,nbr

               iknbr = kmshe2(nfnbr+ibr)

!              --- get the number of d.o.f.'s in this neighboring node

               inusol = sepgetinusol ( iknbr, inunkp )

               if ( debug ) &
                  write(irefwr,1) 'ibr, iknbr, inunkp, inusol', &
                                   ibr, iknbr, inunkp, inusol

               do j = 1, inunkp

                  knusol = kprobh(inusol+j)

                  if ( debug ) write(irefwr,1) 'knusol', knusol
                  if ( knusol<=indpres ) then

!                 --- free or interface unknown found

                     numfree = numfree+1
                     iworki(numfree) = knusol

                     if ( knusol<=nrusol) then

!                    --- free unknown found, store in icolsif
!                        and raise irowsif

                        istart = irowsif(knusol)
                        icolsif(istart+1:istart+numintf) = &
                           interf(1:numintf)
                        irowsif(knusol) = istart+numintf

                     end if  ! ( knusol<=nrusol)

                  end if  ! ( knusol<=indpres )

               end do  ! j = 1, inunkp

            end do  ! ibr = 1,nbr

!           --- For all interface unknowns i in array interf do

            do i = 1, leninterf

!           --- fill arrays icols. and raise irows.

               inusol = interf(i)-nrusol
               isum = irowsfi(inusol)
               icolsfi(isum+1:isum+numfree) = iworki(1:numfree)
               irowsfi(inusol) = irowsfi(inusol)+numfree

            end do  ! i = 1, leninterf

         end if  ! ( numintf>0 )

      end do  ! nodp = 1, npoint

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'structure Sfi'
         call pr0005 ( ninterface, irowsfi(0), icolsfi, 0 )
         write(irefwr,*) 'structure Sif'
         call pr0005 ( nrusol, irowsif(0), icolsif, 0 )
         write(irefwr,*) 'End incommatinter2'

      end if  ! ( debug )
      call erclos ( 'incommatinter2' )

      end subroutine incommatinter2
