      subroutine incommatpar10 ( lensend, icopy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if there are interface unknowns from higher blocks
!     that have not been stored in sendunknowns
!     If so adapt sendunknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: lensend
      integer, intent(out) :: icopy(0:*)

!     icopy          o    Temporary copy of sendunknowns
!     lensend       i/o   Length of array sendinfo
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, iblock, nunks, istartstore, istartsend, nextra, &
                 i, inusol, nstore

!     i              Counting variable
!     iblock         Block sequence number
!     inusol         Position of unknown in array USOL
!     istart         First position used at a part of icopy
!     istartsend     First position used at a part of sendunknowns
!     istartstore    First position used at a part of istore
!     nextra         Extra unknowns to be sent
!     nstore         Number of unknowns on interface
!     nunks          Number of unknowns to be sent
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar10'
         call prinin ( sendunknowns(1), lensend-1, 'sendunknowns' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork

      iwork = 0

!     --- Copy sendunknowns in icopy

      icopy(0:lensend) = sendunknowns(0:lensend)

!     --- For all blocks in istore and sendunknowns do

      istart = icopy(0)
      istartsend = sendunknowns(0)

      do iblock = 1, iacnodes

         nunks = icopy(iblock)-istart  ! number of unknowns in icopy
         istartstore = istore(iblock)  ! start address in istore

!        --- Copy information of icopy into sendunknowns

         sendunknowns(istartsend+1:istartsend+nunks) = &
            icopy(istart+1:istart+nunks)
         istartsend = istartsend+nunks

         if ( debug ) &
            write(irefwr,1) 'iblock, nunks, istartstore, istartsend', &
                             iblock, nunks, istartstore, istartsend

         if ( istartstore>0 .or. nunks>0 ) then

!        --- This block contains information, either in istore or icopy
!            or both
!            Mark all unknowns stored in icopy by 1 in iwork

            iwork(icopy(istart+1:istart+nunks)) = 1

!           --- Check if there are unknowns in istore not marked yet

            if ( istartstore>0 ) then
               nstore = istore(istartstore)
            else
               nstore = 0
            end if  ! ( istartstore>0 )
            nextra = 0     ! number of extra unknowns

            do i = 1, nstore
               inusol = istore(istartstore+i)  ! global number
               inusol = kprobhglobinv(inusol)  ! local number
               if ( iwork(inusol)==0 ) nextra = nextra+1
            end do  ! i = 1, nstore

            if ( debug ) &
               write(irefwr,1) 'nstore, nextra', nstore, nextra

            if ( nextra>0 ) then

!           --- nextra>0, extend with nextra positions

               do i = 1, nstore

                  inusol = istore(istartstore+i)  ! global number
                  inusol = kprobhglobinv(inusol)  ! local number
                  if ( iwork(inusol)==0 ) then

!                 --- New unknown

                     istartsend = istartsend+1
                     sendunknowns(istartsend) = inusol

                  end if  ! ( iwork(inusol)==0 )

               end do  ! i = 1, nstore

            end if  ! ( nextra==0 )

         end if  ! ( istartstore>0 .or. nunks>0 )

         sendunknowns(iblock) = istartsend

!        --- Unmark iwork (reset)

         iwork(icopy(istart+1:istart+nunks)) = 0

         istart = icopy(iblock)

      end do  ! iblock = 1, iacnodes

      lensend = istartsend+1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( sendunknowns(1), lensend-1, 'sendunknowns' )
         write(irefwr,*) 'End incommatpar10'

      end if  ! ( debug )
      call erclos ( 'incommatpar10' )

      end subroutine incommatpar10
