      subroutine incommatpetsc
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmodulematstr
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.0  date 21-04-2015 New parameter list
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array o_nnz with number of nonzeros in off-diagonal blocks
!     for petsc
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arry onnz (1:nrusol)
!     For all neighboring blocks do
!        For all common unknowns on interface do
!           Store number of neighbor unknowns in block in isend
!           send iwork
!     For all neighboring blocks do
!        For all common unknowns on interface do
!           Receive array isend
!           add isend to onnz
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpetsc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpetsc'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create array

      if ( associated(kp(iseqkp)%kprobsub(iprob)%onnz) ) &
           deallocate(kp(iseqkp)%kprobsub(iprob)%onnz)
      if ( associated(kp(iseqkp)%kprobsub(iprob)%nnz) ) &
           deallocate(kp(iseqkp)%kprobsub(iprob)%nnz)
      allocate(kp(iseqkp)%kprobsub(iprob)%onnz(nrusol), &
               kp(iseqkp)%kprobsub(iprob)%nnz(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nrusol, 'nnz' )
      if ( ierror/=0 ) go to 1000
      onnz => kp(iseqkp)%kprobsub(iprob)%onnz
      nnz => kp(iseqkp)%kprobsub(iprob)%nnz

!     --- Fill nnz and  onnz

      do i = 1, nrusol
         nnz(i) = irowsii(i+1)-irowsii(i)
      end do  ! i = 1, nrusol

      if ( size(irowsout)>1 ) then
         do i = 1, nrusol
            onnz(i) = irowsout(i)-irowsout(i-1)
         end do  ! i = 1, nrusol
      end if  ! ( size(irowsout)>1 )

1000  call erclos ( 'incommatpetsc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incommatpetsc'

      end if  ! ( debug )

      end subroutine incommatpetsc
