      subroutine init03 ( istore, istart, ibuf, ielshape, type922found )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.0  date 06-01-2016 Use sepmodulecomio
!        version  9.1  date 13-10-2013 Extension of iinputkp
!
!   copyright (c) 1984-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     the input in array iinputkp is processed
!     and stored in array istore.
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduletexts
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), istart, ibuf,  ielshape(2,*)
      logical type922found

!     ibuf           o    Last position used in istore
!     ielshape       i    Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     istart        i/o   Next position to be used in array IINPUT
!     istore         o    Array containing information about standard elements
!                         array istore is filled as follows:
!                         For each element group   (nelgrp standard elements):
!                         pos. 1-ntype ntype type numbers
!                         next    inunkp, indtvc, iphys
!                         next    if inunkp<=0: inpelm pos. with degrees of
!                                 freedom
!                         next    indtvc*inpelm pos. with degrees of freedom for
!                                 arrays of special structure
!                         next    iphys*inpelm pos. with information of physical
!                                 degrees of freedom
!     type922found   o    If true elements of type 922 hve been found, which
!                         means that Lagragian multipliers are defined by means
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilast, jtest, i, itype, ielgrp, indtvc, inpelm, &
              inunkp, iphys, junkpt, maxren, iinp7, istfl1(2*nrec1)
      double precision dummy(1)

!     dummy          Dummy parameter
!     i              Counting variable
!     ielgrp         element group number
!     iinp7          first pos. iinputkp definition connected elements
!     ilast          last position in array iinputkp
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     iphys          Number of physical variables
!     istfl1         Array in which information from FILE 1 is stored
!     itype          type number of element
!                    itype consists of two parts type and iextra according to
!                    itype = type+10000*iextra with
!                    type    the type number
!                    iextra = 1: extra information is stored in iinputkp (only for
!                                type>99)
!     jtest          = 0, all type numbers<100
!                   >99, at least one type number with type number jtest
!                    all with respect to one standard element
!     junkpt         If 0, the number of degrees of freedom per point is
!                    not constant
!     maxren         Number of groups of renumbered unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIT04         Get information of standard element with type number<100
!     INIT05         Get information of standard element with type number>=100
!     INIT09         Fill information from iinputkp concerning the element groups
!                    corresponding to global unknowns into array kprob part x
!     INITSPFICT     Fill information from iinputkp concerning the element groups
!                    corresponding to fictitious unknowns into array kprob
!                    part y
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      10   Number of alternative types (ntype) is incorrect
!     328   type number of element group equals -1, hence periodical boundary
!           conditions, however, number of nodal points in elements is not 2.
!     550   array length too short
!    1534   Type number does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make some initializations
!     For ielgrp := 1 (1) nelgrp+numnat do
!        check type numbers
!        fill type numbers in istore
!        if ( type number = 0 ) then
!           fill information of dummy element in istore
!        else if ( 0<type number<100 ) then
!           fill information of user element in istore
!        else
!           fill information of standard element in istore
!        compute some general parameters
!     Fill some information in kprob
!     Fill information concerning boundary conditions of the type u equals
!        unknown constant
!     Fill information concerning global groups
!     Fill information concerning fictitious groups
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init03'
         call prinin ( iinputkp, 60, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if

!     --- Make some initializations

      ntype  = iinputkp(1)
      nbcons = iinputkp(13)
      numnat = iinputkp(2)
      if ( debug ) &
         write(irefwr,1) 'nelgrp, ndim, ntype, nbcons, numnat', &
                          nelgrp, ndim, ntype, nbcons, numnat
      nglgrp = iinputkp(29)
      nfictgrp = iinputkp(36)

      kelgrp = 0
      if ( ntype==0 ) ntype = 1
      if ( ntype<0 .or. ntype>20 ) then
         call errint ( ntype, 1 )
         call errint ( 20, 2 )
         call errsub ( 10, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      ibuf = 0
      junkpt = 1
      istart = intarprb(1)
      nunkp = 0
      type922found = .false.

!     --- Loop over element group and boundary element groups

      do ielgrp = 1, nelgrp+numnat

         ilast = istart+ntype-1
         inpelm = ielshape(1,ielgrp)
         ishape = ielshape(2,ielgrp)
         kelgrp = kelgrp+inpelm
         if ( debug ) &
            write(irefwr,*) 'ielgrp, istart, ilast, inpelm, ishape ', &
                             ielgrp, istart, ilast, inpelm, ishape

!        --- check type numbers

         jtest = 0
         do i = istart, ilast
            itype = iinputkp(i)
            if ( itype==0 ) itype = iinputkp(istart)
            if ( itype>99 .or. itype<0 ) jtest = itype
            if ( itype==922 ) type922found = .true.

!           --- fill type numbers in istore

            ibuf = ibuf+1
            istore(ibuf) = mod(itype,10000)
            if ( itype<-3.or. itype>99999 ) then
               call errint ( itype, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 1534, 2, 0, 0 )
            end if

            if ( itype==-1 .and. inpelm/=2 ) then
               call errint ( inpelm, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 328, 2, 0, 0 )
            end if
         end do

         istart = ilast+1

         if ( itype<=-1 ) then

!        --- itype = -1, periodical boundary conditions

            if ( iinputkp(15)>=8 ) then

!           --- verion number>=8, more input possible

               istart = 2*iinputkp(istart)+istart+1

            else if ( iinputkp(15)>=1 ) then

!           --- version number>=1, extra input idegfd

               istart = istart+1

            end if
            goto 300

         end if

         if ( debug ) write(irefwr,*) 'itype ', itype
         if ( itype==0 ) then

!        --- itype = 0, dummy elements, set information in istore equal to 0

            inunkp = 0
            iphys  = 0
            indtvc = 0
            do i = 1, 3+inpelm
               istore(ibuf+i) = 0
            end do
            ibuf = ibuf + 3 + inpelm

         else if ( jtest==0 ) then

!        --- User element (type number between 1 and 99)

            if ( ielgrp>nelgrp ) istart = istart+1
            call init04 ( istore, istart, ibuf, inunkp, &
                          indtvc, iphys, inpelm, ielgrp )
         else

!        --- Standard element (type number>99)

            call init05 ( jtest, istore, istfl1, ibuf, istart, &
                          inunkp, indtvc, iphys, inpelm, ielgrp, &
                          ishape, ndim )
            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'ielgrp ', ielgrp
               write(irefwr,*) 'ibuf, istart, inunkp, nunkp, inpelm ', &
                                ibuf, istart, inunkp, nunkp, inpelm
               call prinin ( istore, ibuf, 'istore' )

            end if

         end if

!        --- compute some general parameters

         if ( ielgrp==1 ) then

!        --- ielgrp = 1

            if ( inunkp/=0 ) then
               nunkp = inunkp
            else
               junkpt = 0
            end if
            ndtvec = indtvc
            nphys = iphys

         else

!        --- ielgrp>1

            if ( inunkp/=nunkp .and. inunkp/=-1 ) junkpt = 0
            if ( inunkp>nunkp ) nunkp = inunkp
            if ( indtvc>ndtvec ) ndtvec = indtvc
            nphys = max(nphys, iphys)

         end if

         if ( ibuf>5000*(nelgrp+numnat) ) then
            call errint ( ibuf, 1 )
            call errint ( 5000*(nelgrp+numnat), 2 )
            call errchr ( 'istore', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if

300   end do

!     --- Fill some information in kprob

      if ( type922found ) ndtvec = ndtvec+1
      if ( debug ) &
         write(irefwr,1) 'lenkprobfp, numnat, nbcons, nelgrp', &
                          lenkprobfp, numnat, nbcons, nelgrp
      if ( nglgrp>0 ) then
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobx(nglgrp*6+3))
         kprobx => kp(iseqkp)%kprobsub(iprob)%kprobx
         kprobloc(11) = 3+nglgrp*6
      end if
      if ( debug ) &
         write(irefwr,1) 'junkpt, kelgrp, nbcons, ndtvec, nglgrp', &
                          junkpt, kelgrp, nbcons, ndtvec, nglgrp
      if ( nphys>0 ) nphys = max(nphys,nunkp)
      intarprb(14) = nphys
      kprobloc(3) = 80
      kprobloc(4) = nunkp
      kprobloc(6) = ndtvec
      kprobloc(7) = ntype
      kprobloc(8) = nelgrp
      kprobloc(12) = numnat+nbcons
      kprobloc(31) = ndim
      kprobloc(32) = nbcons
      kprobloc(33) = max(nphys,nunkp)
      kprobloc(39) = kelgrp

      indpra = 0

      if ( junkpt==0 ) then

!     --- junkpt = 0

         kprobloc(14) = nelgrp
         indpra = nelgrp+numnat
         allocate ( kp(iseqkp)%kprobsub(iprob)%kproba(nelgrp+numnat+nbcons) )
         kproba => kprobpart%kproba
         do i = 1, nelgrp+numnat+nbcons
            nullify ( kprobpart%kproba(i)%arr )
         end do  ! i = 1, nelgrp+numnat+nbcons

      end if
      if ( debug ) &
         write(irefwr,*) 'nelgrp, junkpt, kelgrp, kprobloc(14) ', &
                          nelgrp, junkpt, kelgrp, kprobloc(14)

      if ( numnat+nbcons>0 ) then

!     --- numnat>0

         kprobloc(15) = numnat+nbcons
         if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobb) ) &
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobb(numnat+nbcons) )
         indprb = kprobloc(15)
         kprobb => kprobpart%kprobb
         kprobb = 0

      else

         kprobloc(15) = 0
         indprb = 0

      end if

      length = numnat+nbcons+nelgrp+nglgrp
      if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobc) ) &
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobc(length) )
      kprobc => kprobpart%kprobc
      kprobc = 0

      length = numnat+nbcons
      if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobj) ) &
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobj(length,2) )
      kprobj => kprobpart%kprobj
      kprobj = 0
      kprobloc(23) = length
      indprj = length

      if ( ntype>1 ) then

!     --- ntype>1

         length = ntype*(nelgrp+numnat+nbcons)
         indprd = length
         kprobloc(17) = indprd
         if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobd) ) &
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobd(length) )
         kprobd => kprobpart%kprobd
         kprobd = 0

      else

         kprobloc(17) = 0
         indprd = 0

      end if

      if ( ndtvec>0 ) then

!     --- ndtvec>0

         if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobe) ) &
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe(kelgrp,ndtvec), &
                       kp(iseqkp)%kprobsub(iprob)%kprobe2(ndtvec), &
                       kp(iseqkp)%kprobsub(iprob)%kprobe3(ndtvec) )
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
         kprobe => kprobpart%kprobe
         kprobloc(18) = ndtvec
         indpre = ndtvec
         do i = 1, ndtvec
            nullify(kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr)
         end do  ! i = 1,  ndtvec

      else

         kprobloc(18) = 0
         indpre = 0

      end if

!     --- information concerning boundary conditions of the type u equals
!         unknown constant

      if ( nbcons>0 ) then

         length = nbcons
         kprobloc(28) = length
         if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobo) ) &
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobo(length) )
         kprobo => kprobpart%kprobo
         kprobo = 0

      else

         kprobloc(28) = 0

      end if
      intarprb(11) = junkpt
      indprx = 0

!     --- Fill information concerning global groups

      if ( nglgrp>0 ) then

!     --- nglgrp>0, so there are global groups

         indprx = kprobloc(11)
         nphys = max(nphys,nunkp)
         call init09 ( istfl1, ielshape )

      else

         indprx = 0

      end if

!     --- Fill information concerning fictitious groups

      if ( nfictgrp>0 ) then

!     --- nfictgrp>0, there are fictitious groups

         if ( associated(kp(iseqkp)%kprobsub(iprob)%kproby) ) &
            deallocate(kp(iseqkp)%kprobsub(iprob)%kproby)
         indpry = 13*nfictgrp+11
         allocate ( kp(iseqkp)%kprobsub(iprob)%kproby(indpry) )
         kproby => kp(iseqkp)%kprobsub(iprob)%kproby
         call initspfict ( istfl1 )

      else

         indpry = 0
         kproby => kprobloc

      end if
      kprobloc(51) = indpry

!     --- In case of renumbering (not per level) kprob part ac is filled

      if ( iinputkp(14)>0 ) then

!     --- maxren>0, renumbering (not per level)
!         Compute number of different groups of unknowns, store in maxren

         iinp7 = intarprb(7)
         if ( iinputkp(iinp7)==-1 ) then

!        --- iinputkp(iinp7) = -1, plast renumbering

            maxren = 2

         else

!        --- Standard case, extract maxren from irenum

            maxren = 0
            do i = 0, iinputkp(14)-1
               maxren = max ( maxren, iinputkp(iinp7+i) )
            end do
            if ( iinputkp(14)<nphys ) maxren = maxren+1

         end if
         kprobloc(72) = maxren

      end if

      call prinlg ( 'kprobloc ', lenkprobfp, dummy, kprobloc )

1000  if ( debug ) then
         call prinin ( kprobloc, 60, 'kprobloc')
         write(irefwr,*) 'End init03'
      end if  ! ( debug )
      call erclos ( 'init03' )

      end subroutine init03
