      subroutine init04 ( istore, istart, ibuf, inunkp, indtvc, &
                          iphys, inpelm, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 05-05-1998 Extra parameters
!                                      Complete revision
!        version  4.3  date 05-05-1995 Raise maximum inunkp to 1000
!        version  4.2  date 26-12-1994 New commons cmessc and cmessg
!        version  4.1  date 23-06-1992 New common cmessg
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      information from iinputkp is stored in array istore for a standard
!      element with type number < 100
! **********************************************************************
!
!                       KEYWORDS
!
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, istore(*), ibuf, inunkp, indtvc, &
              iphys, inpelm, ielgrp

!     ibuf      i/o   Last position used in istore
!     ielgrp     i    element group number
!     indtvc     o    Number of vectors of special structure
!     inpelm     o    number of points per element
!     inunkp     o    Number of degrees of freedom in first point
!     iphys      o    Number of physical variables
!     istart    i/o   Next position to be used in array IINPUT
!     istore     o    Array containing information about standard elements
!                     array istore is filled as follows:
!                     For each element group   (nelgrp standard elements):
!                     pos. 1-ntype ntype type numbers
!                     next     inunkp, indtvc, iphys
!                     next     if inunkp <= 0: inpelm pos. with degrees of
!                              freedom
!                     next     indtvc*inpelm pos. with degrees of freedom for
!                              arrays of special structure
!                     next     iphys*inpelm pos. with information of physical
!                              degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilast, i, iphst, ivec

!     i       Counting variable
!     ilast   Last curve (surface) number read
!     iphst   ?
!     ivec    Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       5   Number of degrees of freedom in a point is incorrect
!       7   Number of arrays of special structure is incorrect
!      30   number of unknowns in nodal point is incorrect
!     546   Physical unknown refers to non-existing unknown
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init04' )
      ilast = istart+2
      inunkp = iinputkp(istart)
      indtvc = iinputkp(istart+1)
      iphys  = iinputkp(istart+2)

!     --- copy input in istore

      istart = istart-1
      do i = 1, 3
         istore(ibuf+i) = iinputkp(istart+i)
      end do
      ibuf = ibuf+3
      if ( inunkp.lt.-1 .or. inunkp.ge.16 ) then
         call errint ( inunkp, 1 )
         call errint ( ielgrp, 2 )
         call errsub ( 5, 2, 0, 0 )
      end if
      if ( indtvc.lt.0 .or. indtvc.gt.25 ) then
         call errint ( indtvc, 1 )
         call errint ( 25, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 7, 3, 0, 0 )
      end if
      if ( ierror.ne.0 ) go to 1000

      if ( inunkp.eq.0 ) then

!     --- inunkp  =  0  read degrees of freedom

         istart = ilast+1
         ilast = ilast+inpelm
         do i = istart, ilast
            inunkp = iinputkp(i)
            if ( inunkp.gt.nunkp) nunkp = inunkp
            if ( inunkp.lt.0 .or. inunkp.gt.1000 ) then
               call errint ( inunkp, 1 )
               call errint ( i-istart+1, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 30, 3, 0, 0 )
            end if
         end do

         istart = istart-1
         iphst = istart
         do i = 1, inpelm
            istore(i+ibuf) = iinputkp(i+istart)
         end do
         ibuf = ibuf+inpelm
         istart = istart+1
         inunkp = 0

      end if

      if ( indtvc.gt.0 ) then

!     --- indtvc > 0 arrays of special structure

         do ivec = 1, indtvc
            istart = ilast
            ilast = istart+inpelm
            do i = 1, inpelm
               istore(i+ibuf) = iinputkp(i+istart)
            end do
            ibuf = ibuf+inpelm
         end do

      end if

      if ( iphys.gt.0 ) then

!     --- iphys > 0  physical unknowns different from sequential ones

         do ivec = 1, iphys
            istart = ilast
            ilast = istart+inpelm
            do i = 1, inpelm
               istore(i+ibuf) = iinputkp(i+istart)
            end do

!           --- check iinputkp

            if ( inunkp.gt.0 ) then
               do i = 1, inpelm
                  if ( iinputkp(i+istart).gt.inunkp) then
                     call errint ( iinputkp(i+istart), 1 )
                     call errint ( inunkp, 2 )
                     call errint ( i, 3 )
                     call errint ( ielgrp, 4 )
                     call errsub ( 546, 4, 0, 0 )
                  end if
               end do
            else
               do i = 1, inpelm
                  if ( iinputkp(i+istart).gt.iinputkp(i+iphst) ) then
                     call errint ( iinputkp(i+istart), 1 )
                     call errint ( iinputkp(i+iphst), 2 )
                     call errint ( i, 3 )
                     call errint ( ielgrp, 4 )
                     call errsub ( 546, 4, 0, 0 )
                  end if
               end do
            end if
            ibuf = ibuf+inpelm
         end do

      end if
      istart = ilast+1
1000  call erclos ( 'init04' )
      if ( ierror.ne.0 ) call instop
      end
