      subroutine init07 ( ielshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.2  date 15-03-2012 New type of input
!        version  8.1  date 21-11-2011 Extension with levelset
!        version  8.0  date 27-09-2010 Make work arrays local
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for element groups
!     and boundary groups in array ielshape
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielshape(2,*)

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, iinp, inpelm, &
              natconct, natconpt, natcrpnt, natnocel, natnocpt, &
              natsrpnt, numcrv, natlevset, numobstlc, &
              numpnt, numsrf, natobstcross, natcomcrv, natcomsrf

!     ielgrp         element group number
!     iinp           last position used in array iinputkp
!     inpelm         number of points per element
!     natcomcrv      number of parts where complete_curve elements are defined
!                    for natural boundary conditions
!     natcomsrf      number of parts where complete_surface elements are defined
!                    for natural boundary conditions
!     natconct       Number of contacts where natural boundary elements
!                    are given
!     natconpt       number of parts where contact points are defined as
!                    bound els.
!     natcrpnt       Number of curves where point boundary elements
!                    are given
!     natlevset      number of parts where zero level set elements
!                    are defined
!     natnocel       number of parts where no_contact elements are defined
!     natnocpt       number of parts where no_contact points are defined as
!                    boundary elements
!     natobstcross   number of parts where cross_section_obstacle elements
!                    are defined
!     natsrpnt       Number of surfaces where point boundary elements are
!                    given
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numobstlc      Local number of obstacles
!     numpnt         number of parts where points are given for generation
!                    of boundary elements
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIT07COMCRV   Fill number of nodes per element and element shapes for
!                    boundary groups of type complete curve in array ielshape
!     INIT07COMSRF   Fill number of nodes per element and element shapes for
!                    boundary groups of type complete surface in array ielshape
!     INIT07CONTEL   Fill number of nodes per element and element shapes for
!                    boundary groups of type contact elements in array ielshape
!     INIT07CONTPT   Fill number of nodes per element and element shapes for
!                    boundary groups of type contact points in array ielshape
!     INIT07CROSS    Fill number of nodes per element and element shapes for
!                    boundary groups of type cross obstacle in array ielshape
!     INIT07CRV      Fill number of nodes per element and element shapes for
!                    boundary groups of type curve in array ielshape
!     INIT07LEVSET   Fill number of nodes per element and element shapes for
!                    boundary groups of type zero levelset in array ielshape
!     INIT07OBST     Fill number of nodes per element and element shapes for
!                    boundary groups of type obstacle in array ielshape
!     INIT07PNT      Fill number of nodes per element and element shapes for
!                    boundary groups of type points in array ielshape
!     INIT07PNTEL    Fill number of nodes per element and element shapes for
!                    boundary groups of type point elements in array ielshape
!     INIT07SRF      Fill number of nodes per element and element shapes for
!                    boundary groups of type surface in array ielshape
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get nelgrp
!     For ielgrp := 1 (1) nelgrp do
!        Fill ielshape(.,ielgrp)
!     Get numnat
!     Fill ielshape(.,ielgrp) for all point boundary element groups
!     Fill ielshape(.,ielgrp) for all curve boundary element groups
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init07' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07'
         call prinin ( iinputkp, 20, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get nelgrp and numnat

      numnat = iinputkp(2)
      nbcons = iinputkp(13)
      numpnt = iinputkp(3)
      numcrv = iinputkp(4)
      numsrf = iinputkp(5)
      numobstlc  = iinputkp(17)
      natconct = iinputkp(20)
      natconpt = iinputkp(23)
      natnocel = iinputkp(24)
      natnocpt = iinputkp(25)
      natcrpnt = iinputkp(26)
      natsrpnt = iinputkp(27)
      natobstcross = iinputkp(34)
      natcomcrv = iinputkp(50)
      natcomsrf = iinputkp(51)
      natlevset = iinputkp(55)

      if ( debug ) then
         write(irefwr,1) 'numobstlc, natconct, natconpt, natnocel', &
                          numobstlc, natconct, natconpt, natnocel
         write(irefwr,1) 'natnocpt, natcrpnt, natsrpnt, natobstcross', &
                          natnocpt, natcrpnt, natsrpnt, natobstcross
         write(irefwr,1) 'natlevset, numpnt', natlevset, numpnt
      end if  ! ( debug )

!     --- Clear array ielshape

      ielshape(:,1:nelgrp+numnat+nbcons) = 0

!     --- For ielgrp := 1 (1) nelgrp do
!            Fill ielshape(.,ielgrp)

      do ielgrp = 1, nelgrp
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         ielshape(1,ielgrp) = inpelm
         ielshape(2,ielgrp) = ishape
      end do

!     --- Get numnat

      numnat = iinputkp(2)
      if ( debug ) write(irefwr,1) 'numnat', numnat

!     --- For ibngrp := 1 (1) numnat do
!            Fill ielshape(.,ielgrp) as far as possible

      iinp = intarprb(3)-1
      if ( numpnt>0 ) then

!     --- numpnt>0  point boundary elements

         call init07pnt ( iinp, ielshape, numpnt )
         if ( ierror/=0 ) go to 1000

      end if

      if ( numcrv>0 ) then

!     --- numcrv>0  curve boundary elements

         call init07crv ( iinp, ielshape, numcrv )
         if ( ierror/=0 ) go to 1000

      end if

      if ( numsrf>0 ) then

!     --- numsrf>0  surface boundary elements

         call init07srf ( iinp, ielshape, numsrf )
         if ( ierror/=0 ) go to 1000

      end if

      if ( debug ) &
         write(irefwr,1) 'numobstlc, kelmu', numobstlc, kelmu
      if ( numobstlc>0 ) then

!     --- numobstlc>0  obstacle boundary elements

         call init07obst ( iinp, ielshape )
         if ( ierror/=0 ) go to 1000

      end if

      if ( natconct>0 .or. natnocel>0 ) then

!     --- natconct>0  contact boundary elements, or
!         natnocel>0  no_contact boundary elements

         call init07contel ( iinp, ielshape, natconct, natnocel )
         if ( ierror/=0 ) go to 1000

      end if

      if ( natconpt>0 .or. natnocpt>0 ) then

!     --- natconpt>0  contact point boundary elements or
!         natnocpt>0  no_contact point boundary elements

         call init07contpt ( iinp, ielshape, natconpt, natnocpt )
         if ( ierror/=0 ) go to 1000

      end if

      if ( natcrpnt>0 .or. natsrpnt>0 ) then

!     --- natcrpnt>0  curve point boundary elements or
!         natsrpnt>0  surface point boundary elements

         call init07pntel ( iinp, ielshape, natcrpnt, natsrpnt )
         if ( ierror/=0 ) go to 1000

      end if

      if ( natobstcross>0 ) then

!     --- natobstcross>0 boundary elements due to cross-sections with
!         an obstacle

         call init07cross ( iinp, ielshape, natobstcross )
         if ( ierror/=0 ) go to 1000

      end if

      if ( natcomcrv>0 ) then

!     --- numcomcrv>0 complete curve boundary elements

         call init07comcrv ( iinp, ielshape, natcomcrv )
         if ( ierror/=0 ) go to 1000

      end if  ! ( numcomcrv>0 )

      if ( natcomsrf>0 ) then

!     --- numcomsrf>0 complete surface boundary elements

         call init07comsrf ( iinp, ielshape, natcomsrf )
         if ( ierror/=0 ) go to 1000

      end if  ! ( numcomsrf>0 )

      if ( natlevset>0 ) then

!     --- natlevset>0  zero level set boundary elements

         call init07levset ( iinp, ielshape, natlevset )

      end if  ! ( natlevset>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End init07'

      end if
      call erclos ( 'init07' )

      end
