      subroutine init07cross ( iinp, ielshape, natobstcross )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-03-2012 New input
!        version  1.0  date 05-11-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for
!     boundary groups of type cross obstacle in array ielshape
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, ielshape(2,*), natobstcross

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     iinp          i/o   last position used in array iinputkp
!     natobstcross   i    number of parts where cross_section_obstacle elements
!                         are defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ibngrp, inpelm

!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     271   boundary element group number has wrong value
!    1083   Shape number for boundary group different from before
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init07cross' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07cross'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinputkp(iinp+1), 5+iinputkp(iinp+5), 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- natobstcross>0 boundary elements due to cross-sections with
!         an obstacle

      do j = 1, natobstcross
         ibngrp = iinputkp(iinp+1)
         if ( debug ) &
            write(irefwr,1) 'j, ibngrp, iinp', j, ibngrp, iinp
         if ( ibngrp<=0 .or. ibngrp>numnat ) then
            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errint ( j, 3 )
            call errchr ( 'cross section obstacle', 1 )
            call errsub ( 271, 3, 0, 1 )
            go to 1000
         end if
         iinp = iinp+5+iinputkp(iinp+5)
         inpelm = 2
         ishape = 1
         if ( ielshape(2,ibngrp+nelgrp)/=0 ) then

!        --- check if shapes coincide

            if ( ielshape(2,ibngrp+nelgrp)/=ishape ) then
               call errint ( ibngrp, 1 )
               call errint ( ishape, 2 )
               call errint ( ielshape(2,ibngrp+nelgrp), 3 )
               call errint ( j, 4 )
               call errchr ( 'curve/surface points', 1 )
               call errsub ( 1083, 4, 0, 1 )
               go to 1000
            end if

         end if
         ielshape(1,ibngrp+nelgrp) = inpelm
         ielshape(2,ibngrp+nelgrp) = ishape

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         write(irefwr,*) 'End init07cross'

      end if  ! ( debug )
      call erclos ( 'init07cross' )

      end
