      subroutine printprobinp02 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the part type numbers of array iinput with
!     respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ntype, inpelm, ielgrp, typemax, jinp, i

!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     jinp           last position used in array iinput
!     ntype          Number of different type numbers to be used for one
!                    group
!     typemax        Maximum type number found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp02'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ntype = iinput(1)
      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm

         write(irefwr,1) 'element group', ielgrp
         call prinin ( iinput(iinp), ntype, 'type numbers' )
         typemax = maxval(iinput(iinp:iinp+ntype-1))
         iinp = iinp+ntype

         if ( typemax>0 .and. typemax<100 ) then

!        --- All type numbers between 1 and 99

            jinp = iinp
            call printposint ( iinput(iinp), 'inunkp' )
            call printposint ( iinput(iinp+1), 'indtvc' )
            call printposint ( iinput(iinp+2), 'isear' )
            iinp = iinp+3
            if ( iinput(jinp)==0 ) then
               call prinin ( iinput(iinp), inpelm, 'ndegfd per point' )
               iinp = iinp+inpelm
            end if  ! ( iinput(jinp)==0 )
            do i = 1, iinput(jinp+1)
               write(irefwr,1) 'array of special structure', i
               call prinin ( iinput(iinp), inpelm, 'ndegfd per point' )
               iinp = iinp+inpelm
            end do  ! i = 1, iinput(jinp+1)
            do i = 1, iinput(jinp+2)
               write(irefwr,1) 'isear', i
               call prinin ( iinput(iinp), inpelm, 'nphys per point' )
               iinp = iinp+inpelm
            end do  ! i = 1, iinput(jinp+1)

         else if ( typemax==-1 ) then

!        --- itype = -1, periodical boundary condition

            write(irefwr,1) 'idegfd periodical', iinput(iinp)
            iinp = iinp+1

         else if ( typemax>10000 ) then

!        --- itype>10000, extra information

            jinp = iinp
            iinp = iinp+1
            do i = 1, iinput(jinp)
               write(irefwr,1) 'indtvc_extra', i
               call prinin ( iinput(iinp), 4, 'extra information' )
               iinp = iinp+4
            end do  ! i = 1, iinput(jinp)

         end if  ! ( typemax>0 .and. typemax<100 )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp02'

      end if  ! ( debug )
      call erclos ( 'printprobinp02' )

      end
