      subroutine printprobinp05 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the fictitious elements part of array
!     iinput with respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nfictgrp, ifict, i, numcrv, numsrf, numobst

!     i              Counting variable
!     ifict          Sequence number of fictitious unknown input
!     nfictgrp       number of element groups corresponding to fictitious
!                    unknowns
!     numcrv         number of parts where fictitious unknowns are given
!                    on curves
!     numobst        number of parts where fictitious unknowns are given
!                    on obstacles
!     numsrf         number of parts where fictitious unknowns are given
!                    on surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp05'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nfictgrp = iinput(36)

      if ( nfictgrp==0 ) go to 1000

      write(irefwr,*) 'fictitious elements'

      do ifict = 1, nfictgrp

         call printposint ( iinput(iinp), 'itype' )
         call printposint ( iinput(iinp+1), 'numdegfd' )
         iinp = iinp+2

      end do  ! ifict = 1, nfictgrp

      numcrv = iinput(37)
      do i = 1, numcrv

         call printposint ( iinput(iinp), 'ifict' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call printposint ( iinput(iinp+2), 'icrv1' )
         call printposint ( iinput(iinp+3), 'icrv2' )
         call printposint ( iinput(iinp+4), 'istructgroup' )
         call printposint ( iinput(iinp+5), 'ifluidgroup1' )
         call printposint ( iinput(iinp+6), 'ifluidgroup2' )
         iinp = iinp+7

      end do  ! i = 1, numcrv

      numsrf = iinput(38)
      do i = 1, numsrf

         call printposint ( iinput(iinp), 'ifict' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call printposint ( iinput(iinp+2), 'isrf1' )
         call printposint ( iinput(iinp+3), 'isrf2' )
         call printposint ( iinput(iinp+4), 'istructgroup' )
         call printposint ( iinput(iinp+5), 'ifluidgroup1' )
         call printposint ( iinput(iinp+6), 'ifluidgroup2' )
         iinp = iinp+7

      end do  ! i = 1, numsrf

      numobst = iinput(40)
      do i = 1, numobst

         call printposint ( iinput(iinp), 'iobst' )
         call printposint ( iinput(iinp+1), 'itype' )
         iinp = iinp+4

      end do  ! i = 1, numobst

1000  call erclos ( 'printprobinp05' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp05'

      end if  ! ( debug )

      end
