      subroutine prob00 ( ielshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.8  date 01-02-2012 Change of activate
!        version  9.7  date 14-10-2011 Extra debug
!        version  9.6  date 12-02-2003 Debug statements
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       subroutine prob00 is used to fill array kprob parts f and e
!       fill array kprob parts f and e
!       Warning PROB00 uses memory management subroutines, which implies
!       that without precaution it is not possible to use
!       precomputed starting addresses.
!       The values of starting addresses may be changed by PROB00
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  ielshape(2,*)

!     ielshape       i    Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ivec, ikprbe, inunkp, ielgrp, j, jnunkp, &
              itype, idummy(1), ipos
      logical constunk

!     constunk       If true the number of unknowns per point is constant
!     idummy         Dummy parameter
!     ielgrp         element group number
!     ikprbe         Indication of vector of special sturcture has a
!                    constant number of unknowns per point (<0) or not (>0)
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in first point
!     ipos           Position in array
!     itype          type number of element
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     jnunkp         Number of degrees of freedom in second point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROBFICT00     Find number of structural elements in which the fictitious
!                    unknowns are stored and detect total number of fictitious
!                    unknowns
!     PROBFICT01     Store the structural elements to each fictitious unknown
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPPRINKPROBE  print kprob part e
!     TOPLPH         Fill a part of kprob part e for vectors with non-constant
!                    number of degrees of freedom per point
!     TOPLPJ         Fill kprob part f for vectors with non-constant number
!                    of degrees of freedom per point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob00' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob00'
         call prinin ( kprobloc, 10, 'kprobloc' )
         call prinin ( kprobe, ndtvec, 'kprobe' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      kprobloc(8) = nelgrp
      nusol = nunkp*npoint
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nusol, nunkp, npoint', nusol, nunkp, npoint
         write(irefwr,1) 'indpra', indpra

      end if
      numnat = kprobloc(12)

!     --- Activate KMESH part c

      if ( debug ) write(irefwr,1) 'indpra', indpra

      if ( indpra>0 ) then

!     --- indpra>0  fill kprob part f
!         Reserve space for KPROB part f

         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobf(npoint+1) )
         kprobloc(19) = npoint+1
         kprobf => kprobpart%kprobf
         call toplpj ( ielshape, constunk )


         if ( debug ) then
            write(irefwr,1) 'nusol after toplpj', nusol
            call prinin ( kprobf, npoint+1, 'kprobf' )
         end if  ! ( debug )
         if ( constunk ) then

!        --- Number of unknowns in the vector is constant, reset information
!            delete array

            kprobloc(4) = kprobf(2)-kprobf(1)
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobf )
            kprobloc(19) = 0

         end if

      else if ( kprobloc(19)/=0 ) then

         deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobf )
         kprobloc(19) = 0

      end if
      if ( indprx>0 ) then

!     --- There are global unknowns

         nusol = nusol + kprobx(2)

      end if
      if ( indpry>0 ) then

!     --- There are fictitious unknowns
!         First detect number of elements and number of lambdas

         call probfict00
         nusol = nusol + kproby(2)

!        --- Next fill the first part of kprobfict, i.e. structural elements
!            corresponding to each fictitious unknown

         if ( kelmd==0 ) call sepfillkmeshd
         if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobfict) ) &
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobfict(2,kproby(3)) )
         kprobfict => kp(iseqkp)%kprobsub(iprob)%kprobfict
         kproby(4) = kproby(3)
         call probfict01

      end if

!     --- fill nusol in kprobloc(5)

      kprobloc(5) = nusol

!     --- fill kprob part e for arrays of special structure

      if ( ierror==0 .and. ndtvec>0 ) then

!     --- ndtvec # 0

         ipos = 1
         do ivec = 1, ndtvec

!        --- check whether the number of unknowns in a nodal point is
!            constant or not
!            First for internal elements

            ikprbe = 1
            inunkp = kprobe(ikprbe,ivec)

            if ( debug ) write(irefwr,1) 'inunkp, ikprbe', inunkp, ikprbe
            do ielgrp = 1, nelgrp
               inpelm = Topgrp(ielgrp)%inpelm
               itype = kprobc(ielgrp)
               if ( itype/=-1 ) then
                  do j = 1, inpelm
                     jnunkp = kprobe(ikprbe+j-1, ivec)
                     if ( jnunkp/=inunkp ) goto 160
                  end do
               end if
               ikprbe = ikprbe+inpelm
            end do

!           --- Next for boundary elements

            do ielgrp = nelgrp+1, nelgrp+numnat

               inpelm = ielshape(1,ielgrp)
               do j = 1, inpelm
                  jnunkp = kprobe(ikprbe+j-1,ivec)
                  if ( jnunkp>0 .and. jnunkp/=inunkp ) goto 160
               end do
               ikprbe = ikprbe+inpelm

            end do

!           --- all nodal points have the same number of unknowns

            kprobe2(ivec) = -inunkp
            goto 180

!           --- Reserve space for KPROB part e

160         kprobe2(ivec) = npoint+1

            if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr) ) &
               deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr )
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr(npoint+1) )

            call toplph ( ivec, ielshape, constunk, kprobe3(ivec)%arr )
            if ( ierror/=0 ) go to 1000

            if ( debug ) write(irefwr,*) 'constunk', constunk

            if ( constunk ) then

!           --- Number of unknowns in the vector is constant, reset information
!               delete array

               inunkp = kprobe3(ivec)%arr(2)
               if ( debug ) write(irefwr,1) 'inunkp', inunkp
               deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr )
               kprobe2(ivec) = -inunkp

            end if
180         ipos = ipos+kelgrp

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndtvec', ndtvec
         call sepprinkprobe
         call prinin ( kprobloc, 10, 'kprobloc' )
         write(irefwr,*) 'End prob00'

      end if  ! ( debug )
      call erclos ( 'prob00' )

      end
