      subroutine prob04cvouter ( nkbnd, nodes, found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the outer boundary part of prob04cvold/new
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nkbnd, nodes(*)
      logical found

!     found         i/o   If true a warning has been found in a curve
!     nkbnd         i/o   Number of points with essential boundary conditions
!     nodes         i/o   integer work array of length npoint with node numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodp, j, k, kstep, npointscurve, icurnr, &
              ipart, istart, ncurves, nparts

!     icurnr         Curve sequence number
!     inodp          Nodal point sequence number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kstep          Step in loop
!     ncurves        Number of curves that must be adapted
!     nparts         number of parts the boundary consists of
!     npointscurve   number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2461   No outer curves found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04cvouter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04cvouter'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Special case, all outer boundaries

      if ( mmouter<=0 .or. ndim>2 ) then

!     --- No outer curves found

         call errint ( ndim, 1 )
         call errsub ( 2461, 1, 0, 0 )
         go to 1000

      end if
      nparts = outerboun(1)
      kstep = 1

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         ncurves = outerboun(istart)

         do j = 1, ncurves

            icurnr = abs(outerboun(istart+j))
            call seper0026 ( icurnr, icurnr, kstep, &
                             npointscurve, nodes, .false., kmeshm )
            if ( ierror/=0 ) go to 1000

            do k = 1, npointscurve

               inodp = nodes(k)
               if ( inodp<=0 ) then
                  found = .true.
               else
                  nkbnd = nkbnd+1
                  if ( debug ) &
                     write(irefwr,1) 'nkbnd, inodp/3', &
                                      nkbnd, inodp
               end if

            end do  ! k = 1, npointscurve

         end do  ! k = 1, ncurves

      end do  ! ipart = 1, nparts

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04cvouter'

      end if  ! ( debug )
      call erclos ( 'prob04cvouter' )

      end
