      subroutine prob04sf ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.1  date 13-10-2013 Remove nsurfs
!        version  6.0  date 03-04-2012 Split subroutine
!        version  5.0  date 27-09-2010 Make work arrays local
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isrf1, numsrf, ivers, i

!     i              Counting variable
!     isrf1          First surface number
!     ivers          Version sequence number
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB04SFNEW    Compute the number of nodal points with prescribed boundary
!                    conditions for a set of surfaces
!     PROB04SFOLD    Compute the number of nodal points with prescribed boundary
!                    conditions for surface isrf1 to isrf2
!     PROB04SFOUTER  Compute the number of nodal points with prescribed boundary
!                    conditions for the outer surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04sf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04sf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numsrf = iinput(8)
      ivers  = iinput(15)
      if ( debug ) &
         write(irefwr,1) 'numsrf, ivers', numsrf, ivers

      do i = 1, numsrf

         isrf1 = iinput(iinp+2)
         if ( debug ) write(irefwr,1) 'i, isrf1', i, isrf1

         if ( isrf1/=-1 ) then

!        --- Standard case

            if ( ivers<9 ) then

!           --- Old version

               call prob04sfold ( iinput, iinp, nkbnd )

            else

!           --- New version

               call prob04sfnew ( iinput, iinp, nkbnd )

            end if  ! ( ivers<9 )

         else  ! ( isrf1 = -1 )

!        --- Special case, fill all outer surfaces

            call prob04sfouter ( iinp, nkbnd )

         end if  ! ( isrf1/=-1 )

      end do  ! i = 1, numsrf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nkbnd', nkbnd
         write(irefwr,*) 'End prob04sf'

      end if  ! ( debug )
      call erclos ( 'prob04sf' )

      end
