      subroutine prob04sfels ( inpelm, istart,  iinput, iinp, iwork, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-07-2014 Extra debug
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for a set of nodes corresponding to surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, istart, iinput(*), iinp, iwork(*), nkbnd

!     iinp           i    last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     inpelm         i    number of points per element
!     istart         i    First position used at a part
!     iwork         i/o   integer work array of length npoint
!                         iwork(i) = 0, means that node i has not yet been used
!                         iwork(i)>0,means that node i has been used
!                         iwork(i)<0,means that node i must be used
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, nodalp, inodp

!     ielem          Element number
!     inodp          Nodal point number
!     nodalp         Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04sfels' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04sfels'
         write(irefwr,1) 'inelem, inpelm', inelem, inpelm
         write(irefwr,1) 'kelmn', kelmn
         call prinin ( iinput(iinp+1), inpelm, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over elements

      do ielem = 1, inelem

         do nodalp = 1, inpelm

            if ( iinput(iinp+nodalp)==1 ) then

!           --- iinput(iinp+nodalp) = 1:   prescribed nodal point

               if ( kelmn>0 ) then
                  inodp = kmeshn(istart+nodalp)
               else
                  inodp = kmeshc(istart+nodalp)
               end if

               if ( inodp==0 ) go to 100  ! jump for empty node

               if ( iwork(inodp)==0 ) then

!              --- iwork(inodp) = 0:  new nodal point

                  iwork(inodp) = 1
                  nkbnd = nkbnd+1
                  if ( debug ) &
                     write(irefwr,1) 'nkbnd, inodp', nkbnd, inodp

               end if  ! ( iwork(inodp)==0 )

            end if  ! ( iinput(iinp+nodalp)==1 )

100      end do  ! nodalp = 1, inpelm
         istart = istart+inpelm

      end do  ! ielem = 1, inelem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04sfels'

      end if  ! ( debug )
      call erclos ( 'prob04sfels' )

      end
