      subroutine prob04vlm ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 03-04-2012 Split subroutine
!        version  5.0  date 27-09-2010 Make work arrays local
!        version  4.2  date 09-12-2009 Correction empty nodes
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numvlm, ivers, i

!     i              Counting variable
!     ivers          Version sequence number
!     numvlm         number of parts where surfaces are given for
!                    generation of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB04VLMNEW   Compute the number of nodal points with prescribed boundary
!                    conditions for a set of volumes
!     PROB04VLMOLD   Compute the number of nodal points with prescribed boundary
!                    conditions for volume ivlm1 to ivlm2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04vlm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04vlm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivers  = iinput(15)
      if ( ivers>=5 ) then
         numvlm = iinput(39)
      else
         numvlm = 0
      end if
      if ( debug ) &
         write(irefwr,1) 'numvlm, ivers', numvlm, ivers

      do i = 1, numvlm

         if ( ivers<9 ) then

!        --- Old version

            call prob04vlmold ( iinput, iinp, nkbnd )

         else

!        --- New version

            call prob04vlmnew ( iinput, iinp, nkbnd )

         end if  ! ( ivers<9 )

      end do  ! i = 1, numvlm

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nkbnd', nkbnd
         write(irefwr,*) 'End prob04vlm'

      end if  ! ( debug )
      call erclos ( 'prob04vlm' )

      end
