      subroutine prob05cav ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     conditions in cavity points
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     essential_boundary_condition
!     cavity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibit2, ibit, numnodes, i, inodp, ih, inunkp

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2848   Wrong unknown prescribed in cavity point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05cav' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05cav'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp

      ibit2  = inibt0(inunkp)    ! upper bound bit pattern

      ibit = iinputkp(iinp+1)      ! bit patterb
      iinp = iinp+2              ! raise iinp (iinp is not used yet)

      numnodes = kprobad(1)     ! number of nodes with cav bc

      do i = 1, numnodes

         inodp = kprobad(i+1)

!        --- nodal point found

         if ( indprf>0 ) then

!        --- indprf>0  kprob part f contains the number of unknowns

            inunkp = kprobf(inodp+1)-kprobf(inodp)
            ibit2 = inibt0(inunkp)

         end if

         if ( ibit>ibit2 .or. ibit<0 ) then
            call errint ( inibt1(ibit), 1 )
            call errint ( 1, 2 )
            call errint ( inodp, 3 )
            call errint ( inunkp, 4 )
            call errwar ( 2848, 4, 0, 0 )
         end if

         ih = ibit

         if ( ih==0 ) ih = ibit2

         nkbnd = nkbnd+1
         kprobg(1,nkbnd) = inodp
         kprobg(2,nkbnd) = ih

      end do  ! i = 1, numnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05cav'

      end if  ! ( debug )
      call erclos ( 'prob05cav' )

      end
