      subroutine prob05cvouter ( npointscurve, nodes, inunkp, ibit, kstart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the outer boundary part of prob05cvold/new
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointscurve, nodes(*), inunkp, ibit, kstart

!     ibit           i    Actual bit pattern
!     inunkp         i    Number of degrees of freedom in point
!     kstart         i    Start value of parameter
!     nkbnd         i/o   Number of points with essential boundary conditions
!     nodes         i/o   Contains the node numbers that must be filled
!     npointscurve   i    number of points along a curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipart, k, kstep, inodp, j, ibit2, ih, icurnr, &
              istart, ncurves, nparts

!     ibit2          Maximal bit pattern
!     icurnr         Curve sequence number
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kstep          Step in loop
!     ncurves        Number of curves that must be adapted
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1861   Wrong unknown prescribed at curve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05cvouter' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05cvouter'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ibit2  = inibt0(inunkp)
      nparts = outerboun(1)
      kstep = 1

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         ncurves = outerboun(istart)

         do j = 1, ncurves

            icurnr = abs(outerboun(istart+j))
            call seper0026 ( icurnr, icurnr, kstep, &
                             npointscurve, nodes, .false., kmeshm )

            if ( debug ) &
               write(irefwr,1) 'icurnr, npointscurve', icurnr, npointscurve
            do k = 1, npointscurve

               inodp = nodes(k)
               if ( inodp>0 ) then

!              --- nodal point found

                  if ( indprf>0 ) then

!                 --- indprf>0  kprob part f contains the number of
!                     unknowns

                     inunkp = kprobf(inodp+1)-kprobf(inodp)
                     ibit2 = inibt0(inunkp)

                  end if

                  if ( ibit>ibit2 .or. ibit<0 ) then
                     call errint ( inibt1(ibit), 1 )
                     call errint ( k-kstart+1, 2 )
                     call errint ( icurnr, 3 )
                     call errint ( icurnr, 4 )
                     call errint ( inunkp, 5 )
                     call errwar ( 1861, 5, 0, 0 )
                  end if
                  ih = ibit
                  if ( ih==0 ) ih = ibit2
                  nkbnd = nkbnd+1
                  kprobg(1,nkbnd) = inodp
                  kprobg(2,nkbnd) = ih
                  if ( debug ) &
                     write(irefwr,1) 'nkbnd, inodp/3', &
                                      nkbnd, inodp

               end if

            end do  ! k = 1, npointscurve

         end do  ! k = 1, ncurves

      end do  ! ipart = 1, nparts

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05cvouter'

      end if  ! ( debug )
      call erclos ( 'prob05cvouter' )

      end
