      subroutine prob05levset ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 13-10-2013 Remove iwork
!        version  2.1  date 13-05-2011 Extra debug
!        version  2.0  date 09-09-2005 Extra parameter iwork
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions in case of level set functions
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     obstacle
!     essential_boundary_condition
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlevelset, iinp14, istart, iskip, ilev, ibit2, &
              j, i, numnodes, ibit, ih, inodp, nkbndprev, inunkp
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ih             Help variable to store a constant temporarily
!     iinp14         first position in array iinputkp corresponding to level set
!     ilev           Sequence number of level set function
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     iskip          Defines how the level set nodes must be treated
!                    -1: all nodes outside phi>0 are prescribed
!                     0: no nodes are prescribed
!                     1: all nodes inside phi>0 are prescribed (not the
!                        boundary)
!     istart         First position used at a part
!     iwork          integer work array of length npoint to mark points
!     j              Counting variable
!     nkbndprev      Value of nkbnd at start
!     numlevelset    Number of different level set functions
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2210   Wrong unknown prescribed in obstacle
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all level set functions ilev do
!        iskip := action to be performed for level set function ilev
!        if ( iskip = -1 ) then
!           Set all points outside "obstacle" as prescribed
!        else if ( iskip = 1 ) then
!           Set all points inside "obstacle" as prescribed
!     If zero_level is given then
!        set corresponding nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05levset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05levset'
         nkbndprev = nkbnd

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      numlevelset = iinputkp(48)
      iinp14 = intarprb(18)

      inunkp = nunkp
      ibit2  = inibt0(inunkp)

!     --- For all level set functions ilev do

      do ilev = 1, numlevelset

!     --- iskip := action to be performed for level set function ilev

         istart = ipointobst(ilev)
         iskip = iinputkp(iinp14+ilev-1)
         if ( debug ) write(irefwr,*) 'ilev, iskip ', ilev, iskip
         numnodes = ipointobst(istart)+ &     ! nodes in inner obstacle
                    ipointobst(istart+1)      ! nodes in boun obstacle
         istart = istart+3
         if ( iskip==-1 ) then

!        --- iskip = -1
!            Set all points inside "obstacle" as prescribed

            do j = 1, numnodes

               inodp = ipointobst(istart+j)

!              --- nodal point found

               if ( indprf>0 ) then

!              --- indprf>0  kprob part f contains the number of unknowns

                  inunkp = kprobf(inodp+1)-kprobf(inodp)
                  ibit2 = inibt0(inunkp)

               end if  ! ( indprf>0 )
               nkbnd = nkbnd+1
               kprobg(1,nkbnd) = inodp
               kprobg(2,nkbnd) = ibit2

            end do  ! j = 1, numnodes

         else if ( iskip==1 ) then

!        --- iskip = 1
!            Set all points outside "obstacle" as prescribed
!            First clear iwork

            iwork = 0

!           --- Next set iwork for points inside obstacle

            numnodes = numnodes+ipointobst(istart-1)  ! nodes on boundary obst
            do j = 1, numnodes
               inodp = ipointobst(istart+j)
               iwork(inodp) = 1
            end do  ! j = 1, numnodes

            do inodp = 1, npoint

               if ( iwork(inodp)==0 ) then

!              --- nodal point found

                  if ( indprf>0 ) then

!                 --- indprf>0  kprob part f contains the number of unknowns

                     inunkp = kprobf(inodp+1)-kprobf(inodp)
                     ibit2 = inibt0(inunkp)

                  end if  ! ( indprf>0 )
                  nkbnd = nkbnd+1
                  kprobg(1,nkbnd) = inodp
                  kprobg(2,nkbnd) = ibit2

               end if  ! ( iwork(inodp)==0 )

            end do  ! inodp = 1, npoint

         end if  ! ( iskip==-1 )

      end do  ! ilev = 1, numlevelset

      if ( iinputkp(47)>0 ) then

!     --- zero_level is given
!         set corresponding nodes

         do i = 1, iinputkp(47)

            ibit = iinputkp(iinp+1)
            ilev = iinputkp(iinp+2)
            iinp = iinp+2
            istart = ipointobst(ilev)
            numnodes = ipointobst(istart+2)  ! nodes on boundary obst
            istart = istart+3+ipointobst(istart)+ipointobst(istart+1)

            do j = 1, numnodes

               inodp = ipointobst(istart+j)

!              --- nodal point found

               if ( indprf>0 ) then

!              --- indprf>0  kprob part f contains the number of unknowns

                  inunkp = kprobf(inodp+1)-kprobf(inodp)
                  ibit2 = inibt0(inunkp)

               end if

               if ( ibit>ibit2 .or. ibit<0 ) then
                  call errint ( inibt1(ibit), 1 )
                  call errint ( ilev, 2 )
                  call errint ( inodp, 3 )
                  call errint ( inunkp, 4 )
                  call errwar ( 2210, 4, 0, 0 )
               end if
               ih = ibit
               if ( ih==0 ) ih = ibit2
               nkbnd = nkbnd+1
               kprobg(1,nkbnd) = inodp
               kprobg(2,nkbnd) = ih

            end do  ! j = 1, numnodes

         end do  ! i = 1, iinputkp(47)

      end if  ! ( iinputkp(47)>0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg(1,nkbndprev+1), nkbnd-nkbndprev, 2, &
                        'kprobg' )
         write(irefwr,*) 'End prob05levset'

      end if
      call erclos ( 'prob05levset' )

      end
