      subroutine prob05obst1 ( ipointobst, istart, numobstpnt, &
                               ibit, ibit2, iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kprobg with information of essential boundary conditions
!     w.r.t. nodes stored in ipointobst
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     obstacle
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), istart, numobstpnt, ibit, ibit2, iobst

!     ibit           i    Actual bit pattern
!     ibit2          i    Maximal bit pattern
!     iobst          i    Obstacle sequence number
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     istart         i    First position used at a part
!     numobstpnt     i    Number of points on obstacle to be treated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inodp, inunkp, ih

!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2210   Wrong unknown prescribed in obstacle
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05obst1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05obst1'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      do j = 1, numobstpnt

         inodp = ipointobst(istart+j)

!        --- nodal point found

         if ( indprf.gt.0 ) then

!        --- indprf > 0  kprob part f contains the number of unknowns

            inunkp = kprobf(inodp+1)-kprobf(inodp)
            ibit2 = inibt0(inunkp)

         end if

         if ( ibit.gt.ibit2 .or. ibit.lt.0 ) then
            call errint ( inibt1(ibit), 1 )
            call errint ( iobst, 2 )
            call errint ( inodp, 3 )
            call errint ( inunkp, 4 )
            call errwar ( 2210, 4, 0, 0 )
         end if
         ih = ibit
         if ( ih.eq.0 ) ih = ibit2
         nkbnd = nkbnd+1
         kprobg(1,nkbnd) = inodp
         kprobg(2,nkbnd) = ih

      end do

1000  call erclos ( 'prob05obst1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05obst1'

      end if

      end
