      subroutine prob05vlm ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 03-04-2012 Split subroutine
!        version  4.0  date 27-09-2010 Make work arrays local
!        version  3.3  date 09-12-2009 Correction empty nodes
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for a volume
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numvlm, i, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     numvlm         number of parts where volumes are given for
!                    generation of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PROB05VLMNEW   Fill KPROB part g with information concerning the essential
!                    boundary conditions for a set of volumes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05vlm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05vlm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      numvlm = iinputkp(39)
      if ( debug ) write(irefwr,1) 'numvlm', numvlm

      do i = 1, numvlm

!     --- New version

         call prob05vlmnew ( iinp, inunkp )

      end do  ! i = 1, numvlm

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
         write(irefwr,*) 'End prob05vlm'

      end if
      call erclos ( 'prob05vlm' )

      end
