      subroutine prob05vlmnew ( iinp, inunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for a set of volumes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, inunkp

!     iinp          i/o   last position used in array iinputkp
!     inunkp        i/o   Number of degrees of freedom in point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, iskipbn, i, istart, jinp, &
              ncurvsact, nvolumes, istvlm, ivolumenr, ibit
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ibit           Actual bit pattern
!     inpelm         number of points per element
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     istart         First position used at a part
!     istvlm         Starting address of volumes in iinputkp
!     ivolumenr      volume sequence number
!     iwork          integer work array
!     jinp           last position used in array iinputkp with respect to
!                    iprec
!     ncurvsact      Actual number of curves to be used
!     nvolumes       number of volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PROB05VLMELS   Fill KPROB part g with information concerning the essential
!                    boundary conditions for set of elements in volumes
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2281   the option skip surfaces has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05vlmnew' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05vlmnew'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinputkp(iinp+1), 5+iinputkp(iinp+2)+iinputkp(iinp+1), &
                       'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nvolumes = nsurfs

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint, 'ivolumes' )
      if ( ierror/=0 ) go to 1000

      ibit = iinputkp(iinp+1)
      inpelm = iinputkp(iinp+2)
      nvolumes = iinputkp(iinp+3)
      istvlm = iinp+3
      iinp = iinp+3+nvolumes

      if ( debug ) &
         write(irefwr,1) 'inpelm, nvolumes, istvlm, iinp', &
                          inpelm, nvolumes, istvlm, iinp

!     --- clear array iwork

      iwork = 0

!     --- compute iskipbn

      jinp = iinp+inpelm+2
      iskipbn = iinputkp(jinp-1)
      ncurvsact = iinputkp(jinp)

      if ( iskipbn>=1 ) then

!     --- Boundary must be skipped, &

         call errsub ( 2281, 0, 0, 0 )
         go to 1000

      end if

!     --- Loop over volumes

      do i = 1, nvolumes

!     --- compute the number of elements  inelem

         ivolumenr = iinputkp(istvlm+i)
         if ( debug ) write(irefwr,1) 'i, ivolumenr', i, ivolumenr

!        --- compute inpelm and inelem

         call sepextkmesht ( ivolumenr, ivolumenr, istart, inpelm )
         istart = istart-1

!        --- Fill kprobg for all nodes in volumes

         call prob05vlmels ( inelem, inpelm, ivolumenr, ivolumenr, &
                             kmeshc, istart, iinp, iwork, &
                             inunkp, ibit )

      end do  ! i = 1, nvolumes

      iinp = jinp+ncurvsact

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05vlmnew'

      end if  ! ( debug )
      call erclos ( 'prob05vlmnew' )

      end
