      subroutine prob06 ( numpnt, ikprbi, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.1  date 22-07-2000 Adaptation for multi-block
!        version 10.0  date 06-05-1998 Remove error message, ibngrp
!                                      Extra parameter iposgrp
!        version  9.2  date 14-01-1996 Correction error messages
!
!   copyright (c) 1984-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute length of  kprob part i with respect to point boundary elements
!    This subroutine is the counterpart of subroutine PROB02, &
!    if PROB06 is changed also PROB02 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numpnt, ikprbi, iinp, iposgrp(*)

!     iinp      i/o   last position used in array iinputkp
!     ikprbi     i    last position used in array kprob part i
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     In this subroutine the length with respect to an
!                     element group is added to the existing array
!     numpnt     i    number of parts where points are given for generation
!                     of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, npnt, i, ipoint, jpoint, ibngrp

!     i          Counting variable
!     ibngrp     boundary element group number
!     ipoint     nodal point number
!     j          Counting variable
!     jpoint     jpoint is only be used for IDIM = -1, -2 or -3, otherwise
!                jpoint = 2 ( jdim>30 ), or 1 (rest of the cases)
!     npnt       Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     173   more user points required than available in creation of
!           boundary elements
!     180   user point is not a nodal point
!    1918   User point number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob06' )
      ibngrp = 0
      do j = 1, numpnt
         ibngrp = iinputkp(iinp+1)
         npnt = iinputkp(iinp+2)
         if ( npnt<0 .or. npnt>nuspnt ) then
            call errint ( npnt, 1 )
            call errint ( nuspnt, 2 )
            call errint ( ibngrp, 3 )
            call errsub ( 173, 3, 0, 0)
         end if
         iinp = iinp+2
         do i = 1, npnt

            ipoint = iinputkp(iinp+i)
            if ( ipoint<=0 .or. ipoint>nuspnt ) then
               call errint ( ipoint, 1 )
               call errint ( nuspnt, 2 )
               call errint ( i, 3 )
               call errint ( ibngrp, 4 )
               call errsub ( 1918, 4, 0, 0)
            end if
            jpoint = kmeshl(ipoint)
            if ( jpoint>0 ) then

!           --- point belongs to present block

               kprobb(ibngrp) = kprobb(ibngrp)+1
               ikprbi = ikprbi+1
               iposgrp(ibngrp) = iposgrp(ibngrp)+1

            else

!           --- point is not a user point

               call errint ( ipoint, 1 )
               call errwar ( 180, 1, 0, 0)

            end if

         end do
         iinp = iinp+npnt

      end do

      call erclos ( 'prob06' )

      end
