      subroutine prob09
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.0  date 22-10-2013 New parameter list
!        version  8.2  date 25-02-2005 Fortran 90
!        version  8.1  date 14-10-2002 new storage of composite curves
!
!   copyright (c) 1985-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute npltra and store in kprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nloccv, nlocsf, iinp, nlocobs, iinp5

!     iinp           last position used in array iinputkp
!     iinp5          Starting address of fifth part in iinputkp
!     nloccv         Number of curves with local transforms
!     nlocobs        Number of obstacles with local transforms
!     nlocsf         Number of surfaces with local transforms
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB09CURV     compute npltra and store in kprob with respect to curves
!     PROB09OBST     compute npltra and store in kprob in case of an obstacle
!     PROB09SURF     compute npltra and store in kprob with respect to surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob09' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob09'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npltra = 0
      iinp5 = intarprb(5)
      iinp = iinp5-1
      if ( iinputkp(9)==0 ) go to 1000
      maxtrn = 2
      if ( ndim==3 ) maxtrn = 9

      nloccv = iinputkp(iinp+1)
      nlocsf = iinputkp(iinp+2)
      nlocobs = iinputkp(18)
      iinp = iinp+2

      if ( debug ) then
         write(irefwr,1) 'iinp, ndim, maxtrn', iinp, ndim, maxtrn
         write(irefwr,1) 'nloccv, nlocsf, nlocobs', nloccv, nlocsf, nlocobs
      end if  ! ( debug )

      if ( nloccv>0 ) then

!     --- Local transformations on curves

         call prob09curv ( iinp, nloccv )

      end if  ! ( nloccv>0 )
      if ( ierror/=0 ) go to 1000

      if ( nlocsf>0 ) then

!     --- Local transformations on surfaces

         call prob09surf ( iinp, nlocsf )

      end if  ! ( nlocsf>0 )
      if ( ierror/=0 ) go to 1000

!     --- obstacles

      if ( nlocobs>0 .and. kelmh>0 .and. numobst>0 ) then

!     --- Local transformations on obstacles

         call prob09obst

      end if  ! ( nlocobs>0 .and. kelmh>0 .and. numobst>0 )
      if ( ierror/=0 ) go to 1000

      kprobloc(45) = maxtrn
      kprobloc(13) = npltra

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob09'

      end if  ! ( debug )
      call erclos ( 'prob09' )

      end subroutine prob09
