      subroutine prob10obst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n with degrees of freedom with respect to obstacles
!     and transformation factors (coortr)
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, l, jdgfd1, jdgfd2, jstep, iexclu, kstep, n, &
              kstart, kend, k, nodp, inusol, ideg, jtrans, &
              iprec, inext, j, iinp, ifunc, iinsign, maxobst, nlocobs, &
              iobst, iinp5, inunkp
      double precision xp, yp, xn, yn, anx, any, h

!     anx            x-component of normal vector
!     any            y-component of normal vector
!     h              length of part of curve
!     i              Counting variable
!     ideg           Degree of freedom
!     iexclu         Indicates which points are excluded
!     ifunc          Choice parameter with respect to function calls
!     iinp           last position used in array iinputkp
!     iinp5          Starting address of fifth part in iinputkp
!     iinsign        Sign of curve number
!     inext          Next point in curve
!     inunkp         Number of degrees of freedom in a point
!     inusol         Position of unknown in array USOL
!     iobst          Obstacle sequence number
!     iprec          Preceding point in curve
!     j              Counting variable
!     jdgfd1         Degree of freedom
!     jdgfd2         Degree of freedom
!     jstep          Step in loop
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     k              Counting variable
!     kend           End value of parameter
!     kstart         Start value of parameter
!     kstep          Step in loop
!     l              Counting variable
!     maxobst        Maximum number of obstacles
!     n
!     nlocobs        Number of obstacles with local transforms
!     nodp           Nodal oint sequence number
!     xn             x-coordinate of next point
!     xp             x-coordinate of previous point
!     yn             y-coordinate of next point
!     yp             y-coordinate of previous point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision functr

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     FUNCTR         User function subroutine to compute the local
!                    transformation
!     PROB21         Compute the number of nodes with local transformations at
!                    an obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     336   Length of line between two points is zero
!    2909   Double transformation
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10obst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10obst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinp5 = intarprb(5)
      iinp = iinp5-1

      nlocobs = iinputkp(18)

!     --- allocate work space

      allocate ( iwork(npointglob), iwork1(npointglob), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npointglob, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iinp = iinp+2
      maxobst = numobst
      do i = 1, min(maxobst,nlocobs)
         jdgfd1 = iinputkp(iinp+1)
         jdgfd2 = iinputkp(iinp+2)
         jstep = iinputkp(iinp+4)
         if ( jstep>=0 ) then

!        --- jstep>=  0,  compute iexclu

            iexclu = jstep/100
            jstep = jstep-100*iexclu
         else
            iexclu = 0
         end if
         iobst = abs(iinputkp(iinp+5))
         jtrans = iinputkp(iinp+6)
         iinsign = sign(1,iinputkp(iinp+5))
         iinp = iinp+6
         if ( iobst<=maxobst ) then
            kstep = abs(jstep)+1
            call prob21 ( n, iobst, kstep, kmeshmglob, icurvsobst2d, ncurvs, &
                          iwork1 )
            if ( iinsign==-1 ) then

!           --- Reverse sequence in iwork1

               do k = 1, n
                  iwork(k) = iwork1(n+1-k)
               end do
               do k = 1, n
                  iwork1(k) = iwork(k)
               end do

            end if

            iwork = 0

            if ( jstep>=0 ) then

!           --- jstep>=  0   end points of intervals
!               Correct for initial and end point in the case that exclude>0

               if ( iexclu==0 ) then

!              --- iexclu = 0  No excluding of nodal points

                  kstart = 1
                  kend = n

               else if ( iexclu==1 ) then

!              --- iexclu = 1 Exclude initial point

                  kstart = 2
                  kend = n

               else if ( iexclu==2 ) then

!              --- iexclu = 2 Exclude end point

                  kstart = 1
                  kend = n-1

               else if ( iexclu==3 ) then

!              --- iexclu = 3 Exclude initial and end point

                  kstart = 2
                  kend = n-1

               end if
               do k = kstart, kend, kstep
                  nodp = iwork1(k)
                  if ( iwork(nodp)==0 ) then

                     if ( iwork2(nodp)>0 ) then

!                    --- Node has been tranformed before, give warning and skip

                        call errint ( nodp, 1 )
                        call errwar ( 2909, 1, 0, 0 )
                        go to 200

                     end if  ! ( iwork2(nodp)>0 )
                     if ( indprf>0) then

!                    --- indprf>0  kprob part f contains the number of
!                        unknowns

                        inusol = kprobf(nodp)
                        inunkp = kprobf(nodp+1)-inusol
                        if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 200
                     else
                        inusol = (nodp-1)*nunkp
                     end if

                     iwork(nodp) = 1
                     iwork2(nodp) = iwork2(nodp)+1

                     npltra = npltra+1
                     ideg = inusol+jdgfd1
                     if ( indprh/=0) ideg = kprobh(ideg)
                     kprobn(1,npltra) = ideg
                     ideg = inusol+jdgfd2
                     if ( indprh/=0) ideg = kprobh(ideg)
                     kprobn(2,npltra) = ideg
                     if ( k==1) then
                        iprec = nodp
                     else
                        iprec = iwork1(k-1)
                     end if
                     if ( k==n) then
                        inext = nodp
                     else
                        inext = iwork1(k+1)
                     end if
                     if ( jtrans==1 ) then

!                    --- Compute nx and ny from tangential vector

                        xp = coorglob(1,iprec)
                        yp = coorglob(2,iprec)
                        xn = coorglob(1,inext)
                        yn = coorglob(2,inext)
                        anx = yn-yp
                        any = xn-xp
                        h = sqrt(anx*anx+any*any)
                        if ( h<=0 ) then
                           call errint ( nodp, 1 )
                           call errint ( iprec, 2 )
                           call errint ( inext, 3 )
                           call errint ( iobst, 4 )
                           call errint ( k, 5 )
                           call errsub ( 336, 5, 0, 0 )
                           go to 1000
                        end if

                        if ( maxtrn==2 ) then
                           coortr(1,npltra) = anx/h
                           coortr(2,npltra) = -any/h
                        else
                           coortr(1,npltra) = anx/h
                           coortr(2,npltra) = any/h
                           coortr(3,npltra) = -any/h
                           coortr(4,npltra) = anx/h
                           if ( maxtrn==8 ) then
                              coortr(5,npltra) = coortr(1,npltra)
                              coortr(6,npltra) = coortr(3,npltra)
                              coortr(7,npltra) = coortr(2,npltra)
                              coortr(8,npltra) = coortr(4,npltra)
                           end if
                        end if

                     else

!                    --- compute R and V as given by the user

                        do j = 1, 4*(jtrans-1)
                           ifunc = iinputkp(iinp+j)
                           if ( ifunc<0 ) then
                              coortr(j,npltra) = functr ( -ifunc, &
                                    coorglob(1,nodp), coorglob(2,nodp), 0d0 )
                           else
                              coortr(j,npltra) = rinputkp(ifunc)
                           end if
                        end do

                     end if
                     if ( jtrans==2 .and. maxtrn==8 ) then
                        coortr(5,npltra) = coortr(1,npltra)
                        coortr(6,npltra) = coortr(3,npltra)
                        coortr(7,npltra) = coortr(2,npltra)
                        coortr(8,npltra) = coortr(4,npltra)
                     end if

                  end if

200            end do

            else

!           --- jstep <0      points between intervals

               n = n-1
               do k = 1, n, kstep
                  do j = 1, kstep-1
                     nodp = iwork1(k+j)
                     if ( iwork(nodp)==0 ) then

                        if ( iwork2(nodp)>0 ) then

!                       --- Node has been tranformed before, &
!                           give warning and skip

                           call errint ( nodp, 1 )
                           call errwar ( 2909, 1, 0, 0 )
                           go to 300

                        end if  ! ( iwork2(nodp)>0 )

                        if ( indprf>0) then

!                       --- indprf>0  kprob part f contains the
!                           number of unknowns

                           inusol = kprobf(nodp)
                           inunkp = kprobf(nodp+1)-inusol
                           if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 300
                        else
                           inusol = nunkp*(nodp-1)
                        end if

                        iwork(nodp) = 1
                        iwork2(nodp) = iwork2(nodp)+1

                        npltra = npltra+1
                        ideg = inusol+jdgfd1
                        if ( indprh/=0) ideg = kprobh(ideg)
                        kprobn(1,npltra) = ideg
                        ideg = inusol+jdgfd2
                        if ( indprh/=0 ) ideg = kprobh(ideg)
                        kprobn(2,npltra) = ideg
                        iprec = iwork1(k+j-1)
                        inext = iwork1(k+j+1)
                        if ( jtrans==1 ) then

!                       --- Compute nx and ny from tangential vector

                           xp = coorglob(1,iprec)
                           yp = coorglob(2,iprec)
                           xn = coorglob(1,inext)
                           yn = coorglob(2,inext)
                           anx = yn-yp
                           any = xn-xp
                           h = sqrt(anx*anx+any*any)
                           if ( h<=0 ) then
                              call errint ( nodp, 1 )
                              call errint ( iprec, 2 )
                              call errint ( inext, 3 )
                              call errint ( iobst, 4 )
                              call errint ( k, 5 )
                              call errsub ( 336, 6, 0, 0 )
                              go to 1000
                           end if

                           if ( maxtrn==2 ) then
                              coortr(1,npltra) = anx/h
                              coortr(2,npltra) = -any/h
                           else
                              coortr(1,npltra) = anx/h
                              coortr(2,npltra) = any/h
                              coortr(3,npltra) = -any/h
                              coortr(4,npltra) = anx/h
                              if ( maxtrn==8 ) then
                                 coortr(5,npltra) = coortr(1,npltra)
                                 coortr(6,npltra) = coortr(3,npltra)
                                 coortr(7,npltra) = coortr(2,npltra)
                                 coortr(8,npltra) = coortr(4,npltra)
                              end if
                           end if

                        else

!                       --- compute R and V as given by the user

                           do l = 1, 4*(jtrans-1)
                              ifunc = iinputkp(iinp+l)
                              if ( ifunc<0 ) then
                                 coortr(l,npltra) = functr ( -ifunc, &
                                       coorglob(1,nodp), coorglob(2,nodp), 0d0 )
                              else
                                 coortr(l,npltra) = rinputkp(ifunc)
                              end if
                           end do
                        end if
                        if ( jtrans==2 .and. maxtrn==8 ) then
                           coortr(5,npltra) = coortr(1,npltra)
                           coortr(6,npltra) = coortr(3,npltra)
                           coortr(7,npltra) = coortr(2,npltra)
                           coortr(8,npltra) = coortr(4,npltra)
                        end if
                     end if

300               end do

               end do

            end if
            if ( jtrans==2 ) then
               iinp = iinp+4
            else if ( jtrans==3 ) then
               iinp = iinp+8
            end if

         end if

      end do

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10obst'

      end if  ! ( debug )
      call erclos ( 'prob10obst' )

      end subroutine prob10obst
