      subroutine prob12 ( ikprbi, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 27-09-2010 Make work arrays local
!        version  7.0  date 14-10-2002 Extra parameter kmeshq
!        version  6.2  date 15-10-1999 New call to er0026
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   compute length of array kprob part i with boundary elements
!   corresponding to boundary conditions of the type u equals unknown constant
!   This subroutine is the counterpart of subroutine PROB15, &
!   if PROB12 is changed also PROB15 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikprbi, iinp

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcrv, ibngrp, i, idegfd, &
              jstep, ifirst, ilast, j, n, kstep, icount, kcount, &
              k, isurf, numsrf, inpelm
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     icount         Counter
!     idegfd         Degree of freedom
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     inpelm         number of points per element
!     isurf          Type of surface with respect to mesh generator
!     iwork          work array to store the nodes in the curves
!     j              Counting variable
!     jstep          Step in loop
!     k              Counting variable
!     kcount         Counter
!     kstep          Step in loop
!     n              number of points on the curves icrv1 to icrv2
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numsrf         number of parts where surfaces are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
!     SEPER0084      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     547   jstep = -1 for curve corresponding to boundary conditions of the
!           type u equals unknown constant, hence there are no points left
!     548   number of elements not correct for the creation of boundary
!           conditions of the type u equals unknown constant (jstep not correct)
!     549   degree of freedom for the creation of boundary conditions of the
!           type u equals unknown constant incorrect
!    1537   Boundary condition u equals constant has only been implemented
!           for surfaces if volume elements are present
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob12' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob12'

      end if

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      numnat = iinputkp(2)
      nbcons = iinputkp(13)
      numcrv = iinputkp(iinp)
      numsrf = iinputkp(iinp+1)
      ibngrp = numnat
      iinp = iinp+2
      do i = 1, numcrv
         idegfd = iinputkp(iinp)
         kprobo(i) = idegfd
         jstep = iinputkp(iinp+1)
         ifirst = iinputkp(iinp+2)
         ilast = iinputkp(iinp+3)
         iinp = iinp + 4
         if ( debug ) &
            write(irefwr,*) 'i, idegfd, jstep, ifirst, ilast ', &
                             i, idegfd, jstep, ifirst, ilast

!        --- fill nodal points of curves ifirst to ilast in iwork

         if ( jstep<=0 ) then
            j = 1
         else
            j = jstep
         end if
         if ( kelmm>0 ) then
            call seper0026 ( ifirst, ilast, j, n, iwork, .true., kmeshm )
         else
            call seper0084 ( ifirst, ilast, j, n, iwork )
         end if
         if ( debug ) call prinin ( iwork, n, 'iwork' )

         if ( jstep<0 ) then

!        --- jstep<0  delete non-interesting points from iwork

            kstep = abs(jstep)
            if ( kstep==1) then
               call errint ( i, 1 )
               call errsub ( 547, 1, 0, 0 )
            end if
            inelem = (n-1)/kstep
            if ( inelem*kstep/=n-1) then
               call errint ( inelem*kstep, 1 )
               call errint ( n-1, 2 )
               call errint ( i, 3 )
               call errsub ( 548, 3, 0, 0 )
            end if
            icount = 0
            kcount = 0
            do j = 1, inelem
               kcount = kcount+1
               do k = 2, kstep
                  icount = icount+1
                  kcount = kcount+1
                  iwork(icount) = iwork(kcount)
               end do
            end do
            n = icount
            inelem = n-1

         else if ( jstep>1 ) then

!        --- jstep>1 delete non-interesting points from iwork

            do j = 1, inelem
               iwork(j+1) = iwork(1+j*jstep)
            end do
            n = inelem+1

         end if

!        --- check if degree of freedom exists in nodal points

         if ( indpra==0 ) then
            if ( idegfd>nunkp ) then
               call errint ( idegfd, 1 )
               call errint ( nunkp, 2 )
               call errint ( i, 3 )
               call errsub ( 549, 3, 0, 0 )
            end if
         end if
         kprobb(ibngrp+i) = inelem
         kprobj(ibngrp+i,1) = 2
         kprobj(ibngrp+i,2) = 1
         ikprbi = ikprbi+2*inelem
         if ( debug ) &
            write(irefwr,*) 'ibngrp, inelem, numnat, nbcons, ikprbi ', &
                             ibngrp, inelem, numnat, nbcons, ikprbi

      end do
      ibngrp = numnat+numcrv

      do i = 1, numsrf
         idegfd = iinputkp(iinp)
         ifirst = iinputkp(iinp+1)
         ilast  = iinputkp(iinp+2)
         inpelm = iinputkp(iinp+3)
         kprobo(i) = idegfd
         iinp = iinp + 4 + inpelm

!        --- fill nodal points of surfaces ifirst to ilast in KPROB part i

         if ( kelmn>0 ) then

!        --- kelmn>0, kmesh part n has been filled

            inelem = 0
            do isurf = ifirst, ilast
               inelem = kmeshn(nsurfs+isurf)+inelem
            end do

         else

!        --- kmeshn not filled has not yet been implemented

            call errint ( ifirst, 1 )
            call errint ( ilast, 2 )
            call errsub ( 1537, 2, 0, 0 )

         end if
         kprobb(ibngrp+i) = inelem
         kprobj(ibngrp+i,1) = inpelm
         if ( inpelm==3 ) then
            kprobj(ibngrp+i,2) = 3
         else if ( inpelm==4 ) then
            kprobj(ibngrp+i,2) = 5
         else if ( inpelm==6 ) then
            kprobj(ibngrp+i,2) = 4
         else if ( inpelm==9 ) then
            kprobj(ibngrp+i,2) = 6
         end if
         ikprbi = ikprbi+inpelm*inelem

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob12'

      end if
      call erclos ( 'prob12' )

      end
