      subroutine prob16 ( kprtem, ielshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.2  date 23-04-2003 Introduction icount (RvL)
!        version  5.1  date 28-03-2001 Correction when nphys>0 (CvO)
!        version  5.0  date 04-11-1998 Extra parameters ielshape, kprobi
!
!   copyright (c) 1987-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill physical degrees of freedom in array kprob part p
!   prob16 must be called when nunkp>1, nphys>1 or kproba>0
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprtem(*), ielshape(2,*)

!     ielshape   i    Array of length 2*(nelgrp+numnat) containing:
!                     (1,i) number of nodes in element group i
!                     (2,i) shape number of elements in element group i
!                     if i > nelgrp: i-nelgrp = boundary group number
!                     otherwise it is the element group number
!     kprtem     i    temporary array in which the input concerning physical
!                     degrees of freedom has been stored by subroutine init06
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer np, j, i, istart, iend, &
              ikelmc, itype, iielem, nodp, k, ih, &
              inpelm, inunkp, ielgrp, icount

!     i              Counting variable
!     icount         Counter
!     ielgrp         element group number
!     iend           Last position used at a part
!     ih             Help variable to store a constant temporarily
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in first point
!     istart         First position used at a part
!     itype          Type number of element
!     j              Counting variable
!     k              Counting variable
!     nodp           Nodal point sequence number
!     np             Help variable to store the maximum value of nphys and nunkp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     551   Physical unkown has different position in the same nodal point in
!           different elements.
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob16' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob16'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nunkp  = kprobloc(4)
      numnat = kprobloc(12)
      indprf = kprobloc(19)
      nphys = intarprb(14)
      np = max(nunkp, nphys)

      if ( debug ) &
         write(irefwr,1) 'nunkp, numnat,indprb, indprf, nphys', &
                          nunkp, numnat,indprb, indprf, nphys

!     --- reset kprobp

      kprobp(1:npoint,1:np) = 0

      if ( indprf==0 .and. nphys.eq.0 ) then

!     --- nphys = 0 and kproba = 0, hence the same number of degrees of freedom
!         in each node corresponding to the same physical degrees of freedom

         do j = 1, np
            do i = 1, npoint
               kprobp(i,j) = (i-1)*np+j
            end do
         end do

      else if ( nphys==0 ) then

!     --- nphys = 0 and kproba>0, hence a different number of degrees of freedom
!         in each node but corresponding to the same physical degrees of
!         freedom

         istart = 0
         do i = 1, npoint
            iend = kprobf(i+1)
            inunkp = iend-istart
            do j = 1, inunkp
               kprobp(i,j) = istart+j
            end do
            istart = iend
         end do

      else

!     --- nphys>0  physical degrees of freedom differ from positional ones

         ikelmc = 0
         istart = 0
         inpelm = npelm
         inelem = nelem

         if ( debug ) write(irefwr,1) 'inpelm, inelem', inpelm, inelem

         do ielgrp = 1, nelgrp

            if ( nelgrp>1 ) then

!           --- nelgrp > 1

               inelem = Topgrp(ielgrp)%inelem
               inpelm = Topgrp(ielgrp)%inpelm
               itype = kprobc(ielgrp)

               if ( debug ) &
                  write(irefwr,1) 'inelem, inpelm, itype', &
                                   inelem, inpelm, itype

               if ( itype==-1 ) then
                  ikelmc = ikelmc+inelem*2

!                 --- here it said: goto 190
!                     kprtem does NOT contain info when iytpe = -1, so
!                     the start address doesn't have to be increased

                  goto 210
               end if
            end if

            do iielem = 1, inelem
               do j = 1, inpelm
                  nodp = kmeshc(ikelmc+j)
                  icount=1
                  do k = 1, np
                     inunkp = kprtem(istart+j+(k-1)*inpelm)
                     if ( inunkp>0 ) then
                        if ( indprf==0 ) then
                           inunkp = inunkp+(nodp-1)*nunkp
                        else
                           inunkp = icount+kprobf(nodp)
                        end if
                        ih = kprobp(nodp,k)
                        if ( ih>0 ) then
                           if ( ih/=inunkp ) then
                              call errint ( nodp, 1 )
                              call errint ( inunkp, 2 )
                              call errint ( ielgrp, 3 )
                              call errint ( ih, 4 )
                              call errsub ( 551, 4, 0, 0 )
                           end if
                        else
                           kprobp(nodp,k) = inunkp
                        end if
                        icount=icount+1
                     end if
                  end do
               end do
               ikelmc = ikelmc+inpelm
            end do
            istart = istart+np*inpelm
210      end do

!        --- Next fill for boundary elements

         ikelmc = 0
         do ielgrp = nelgrp+1, nelgrp+numnat
            inpelm = ielshape(1,ielgrp)
            inelem = kprobb(ielgrp-nelgrp)
            itype = kprobc(ielgrp)

            if ( itype==-2 ) then
               ikelmc = ikelmc+2*inelem
               go to 200
            end if
            do iielem = 1, inelem
               do j = 1, inpelm
                  nodp = kprobi(ikelmc+j)
                  icount=1
                  do k = 1, np
                     inunkp = kprtem(istart+j+(k-1)*inpelm)
                     if ( inunkp>0 ) then
                        if ( indprf==0 ) then
                           inunkp = inunkp+(nodp-1)*nunkp
                        else
                           inunkp = icount+kprobf(nodp)
                        end if
                        ih = kprobp(nodp,k)
                        if ( ih>0 ) then
                           if ( ih/=inunkp ) then
                              call errint ( nodp, 1 )
                              call errint ( inunkp, 2 )
                              call errint ( ielgrp, 3 )
                              call errint ( ih, 4 )
                              call errsub ( 551, 4, 0, 0 )
                           end if
                        else
                           kprobp(nodp,k) = inunkp
                        end if
                        icount=icount+1
                     end if
                  end do
               end do
               ikelmc = ikelmc+inpelm
            end do
200         istart = istart+np*inpelm
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobp, npoint, nunkp, 'kprobp' )
         write(irefwr,*) 'End prob16'

      end if  ! ( debug )
      call erclos ( 'prob16' )

      end subroutine prob16
