      subroutine prob19 ( nkbnd, iobst, jstep, kmeshm, icurvsobs, &
                          ncurvs, iobscurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 14-08-1999 New definition icurvsobs
!        version  2.0  date 12-06-1998 Extar parameter iobscurv
!        version  1.0  date 29-11-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the number of nodes with prescribed boundary conditions at
!   obstacle iobst
!   This subroutine is the counterpart of subroutine PROB20, &
!   if PROB20 is changed also PROB19 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jstep, nkbnd, iobst, kmeshm(*), icurvsobs(2,*), ncurvs, &
              iobscurv(*)

!     icurvsobs  i    Array containing information of the active obstacles.
!                     The array corresponds to all curves separately
!                     icurvsobs(1,i) = 1 means that the corresponding point is
!                     active; 0 means not active
!     iobscurv   i    Array of length ncurvs containing information about the
!                     obstacles
!                     iobscurv(i) = 0: curve i not on obstacle
!                     iobscurv(i) = 1: points of curve i on obstacle
!                     iobscurv(i) = 2: begin and end point of curve i on
!                                      obstacle
!                     iobscurv(i) = 3: begin point of curve i on obstacle
!                     iobscurv(i) = 4: end point of curve i on obstacle
!     iobst      i    Obstacle sequence number
!     jstep      i    Indicates which points must be taken into account
!                     and which points must be skipped (See PG)
!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     ncurvs     i    Number of curves in mesh
!     nkbnd     i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i1, i2, icurnr

!     i1         First point of curve in icurvsobs
!     i2         Last point of curve in icurvsobs
!     icurnr     Curve sequence number
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     428   The option jstep = ... has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters
!    For all curves do
!       For all nodes along the curve do
!          if ( node is at obstacle iobst ) then
!             nkbnd := nkbnd + 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do icurnr = 1, ncurvs
         i1 = kmeshm(icurnr+4)
         i2 = kmeshm(icurnr+5)-1
         if ( iobscurv(icurnr).gt.0 ) then

!        --- Possibly active points on curve

            if ( jstep.eq.1 ) then

!           --- all nodes must be used

               do k = i1, i2
                  if ( icurvsobs(1,k).eq.iobst ) nkbnd = nkbnd+1
               end do

            else

!           --- not yet implemented

               call eropen ( 'prob19' )
               call errint ( jstep, 1 )
               call errsub ( 428, 1, 0, 0 )
               call erclos ( 'prob19' )

            end if

         end if

      end do

      end
