      subroutine prob32 ( numcrv, ikprbi, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of array kprob part i with respect to point boundary
!     elements corresponding to curves
!     This subroutine is the counterpart of subroutine PROB34, &
!     if PROB32 is changed also PROB34 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcrv, ikprbi, iinp, iposgrp(*)

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         In this subroutine the length with respect to an
!                         element group is added to the existing array
!     numcrv         i    number of parts where curves are given for generation
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ibngrp, icurnr, lenkmeshm3, j, inum, npointscurve

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     icurnr         Curve sequence number
!     inum           Number of curves
!     j              Counting variable
!     lenkmeshm3     Length of kmesh m part 3
!     npointscurve   number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PROBCURVNUMELM Compute number of elements along a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob32' )

      lenkmeshm3 = kmeshm(3)-kmeshm(2)

      do j = 1, numcrv

         ibngrp = iinputkp(iinp+1)
         inum = iinputkp(iinp+2)

!        --- Loop over the curves

         do i = 1, inum

            icurnr = iinputkp(iinp+2+i)
            if ( icurnr<1 .or. icurnr>ncurvs ) then

!           --- icurnr out of range

               call errint ( icurnr, 1 )
               call errint ( ncurvs, 2 )
               call errsub ( 139, 2, 0, 0 )
               go to 1000

            end if  ! ( icurnr<1 .or. icurnr>ncurvs )

!           --- Detect number of elements along curve

            ishape = 0
            call probcurvnumelm ( icurnr, npointscurve, ibngrp, lenkmeshm3 )

            kprobb(ibngrp) = kprobb(ibngrp)+npointscurve
            ikprbi = ikprbi+npointscurve
            iposgrp(ibngrp) = iposgrp(ibngrp)+npointscurve

         end do
         iinp = iinp+2+inum

      end do  ! j = 1, numcrv

1000  call erclos ( 'prob32' )

      end
