      subroutine prob34 ( numcrv, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kprob part i with point boundary elements corresponding to
!     curve
!     This subroutine is the counterpart of subroutine PROB32, &
!     if PROB32 is changed also PROB34 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcrv, iinp, iposgrp(*)

!     iinp          i/o   last position used in array iinputkp
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         At input the last postions used for an element group
!                         are stored, at output they are updated
!     numcrv         i    number of parts where curves are given for generation
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inum, j, icurnr, npointscurve, &
              lenkmeshm3, ikprbi, ibngrp

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     icurnr         Curve sequence number
!     ikprbi         last position used in array kprob part i
!     inum           Number of curves
!     j              Counting variable
!     lenkmeshm3     Length of kmesh m part 3
!     npointscurve   number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROBCURVELM    Fill elements along a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob34' )
      lenkmeshm3 = kmeshm(3)-kmeshm(2)
      ishape = 0
      do j = 1, numcrv

         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         inum = iinputkp(iinp+2)

!        --- Loop over the curves

         do i = 1, inum

!        --- Compute elements and store in kprobi

            icurnr = iinputkp(iinp+2+i)
            call probcurvelm ( icurnr, npointscurve, kprobi, ikprbi, &
                               lenkmeshm3 )
            iposgrp(ibngrp) = iposgrp(ibngrp)+npointscurve

         end do
         iinp = iinp+2+inum

      end do  ! j = 1, numcrv

      call erclos ( 'prob34' )

      end
