      subroutine probflkprobaa
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-10-2011 Complete new subroutine
!        version  1.7  date 27-09-2010 extra debug information
!        version  1.6  date 29-11-2004 Correction for points in obstacle and
!                                      on boundary
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part aa with information concerning the elements to be
!     skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     element
!     skip
!     plot
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical skip

!     skip           If true elements must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROBLEVSKIPEL  Fill elements of kprob part aa that must be skipped since
!                    they are in the wrong part of the levelset function
!     PROBSKIPELMS   Fill kprob part aa with information concerning the elements
!                    to be skipped
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probflkprobaa' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probflkprobaa'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create kprobaa

      if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobaa) ) &
         deallocate(kp(iseqkp)%kprobsub(iprob)%kprobaa)
      allocate ( kp(iseqkp)%kprobsub(iprob)%kprobaa(nelem) )
      kprobaa => kprobpart%kprobaa
      kprobloc(53) = nelem
      kprobaa = 0

      if ( intarprb(20)>0 ) then

!     --- Skip elements found

         call probskipelms

      end if  ! ( intarprb(20)>0 )

      if ( iinputkp(48)>0 ) then

!     --- Level set functions found

         call problevskipel ( skip )

         if ( .not.skip .and. intarprb(20)<=0 ) then

!        --- No skipping of elements, remove kprobaa

            deallocate(kp(iseqkp)%kprobsub(iprob)%kprobaa)
            kprobloc(53) = 0

         end if  ! ( .not.skip .and. intarprb(20)<=0 )

      end if  ! ( iinputkp(48)>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probflkprobaa'

      end if
      call erclos ( 'probflkprobaa' )

      end subroutine probflkprobaa
