      subroutine probnatobstcr ( natobstcross, ikprbi, iinp, iposgrp, &
                                 kcrosselem, iworkcross, nintersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-05-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part b with respect to cross-sections with an obstacle
!     natural boundary conditions
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleprobinf
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natobstcross, &
              ikprbi, iinp, iposgrp(*), kcrosselem(*), iworkcross(*), &
              nintersect

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!     iworkcross     o    integer work array of length natobstcross that
!                         contains the number of elements per set of boundary
!                         elements corresponding to the intersections
!     kcrosselem     o    array of size nfictunk x 2 with the following contents
!                         kcrosselem(i,1) contains the first node of the element
!                         kcrosselem(i,2) contains the second node of the
!                         element
!     natobstcross   i    number of parts where cross_section_obstacle elements
!                         are defined
!     nintersect     i    Number of intersections found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mmrcrosselem, ibngrp, i
      double precision rcrosselem(nintersect)

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     mmrcrosselem   memory management sequence number of array rcrosselem
!     rcrosselem     Contains the factors defining the intersection point
!                    per element
!                    The intersection point is defined by x1 + fac *(x2-x1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PROB01CROBST   Determine the elements corresponding to cross sections
!                    with an obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnatobstcr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnatobstcr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create and clear kprob z

      length = nelgrp+numnat+nbcons
      if ( associated(kp(iseqkp)%kprobsub(iprob)%probzarr) ) then
         do i = 1, length
            if ( kprobpart%kprobz(i)>0 ) &
               deallocate (kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr )
            nullify(kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr)
         end do  ! i = 1, size(kprobpart%probzarr)
         deallocate ( kp(iseqkp)%kprobsub(iprob)%probzarr )
      end if  ! ( associated(kp(iseqkp)%kprobsub(iprob)%probzarr) )
      if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobz) ) &
         deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobz )
      allocate ( kp(iseqkp)%kprobsub(iprob)%kprobz(length) )
      allocate ( kp(iseqkp)%kprobsub(iprob)%probzarr(length) )
      kprobz => kprobpart%kprobz
      probzarr => kprobpart%probzarr
      do i = 1, length
         nullify(kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr)
      end do  ! i = 1, length
      kprobz = 0
      kprobloc(52) = length
      indprz = length

      if ( kmeshad(4)>0 ) then

!     --- Intersections have been found

         ibngrp = iinputkp(iinp+1)
         call prob01crobst ( kcrosselem, rcrosselem, nintersect, &
                             iinp, ikprbi, iposgrp, &
                             natobstcross, iworkcross )

         if ( nintersect>0 ) then

!        --- Make array rcrosselem permanent

            allocate ( kp(iseqkp)%kprobsub(iprob)%&
                       probzarr(nelgrp+ibngrp)%arr(nintersect) )
            probzarr => kp(iseqkp)%kprobsub(iprob)%probzarr
            probzarr(nelgrp+ibngrp)%arr = rcrosselem(1:nintersect)
            kprobz => kp(iseqkp)%kprobsub(iprob)%kprobz
            kprobz(nelgrp+ibngrp) = nintersect
            if ( debug ) call prinrl ( rcrosselem, nintersect, 'rcrosselem' )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iworkcross, natobstcross, 'iworkcross' )
         call prinin1 ( kcrosselem, 2, nintersect, 'kcrosselem' )
         write(irefwr,*) 'End probnatobstcr'

      end if  ! ( debug )
      call erclos ( 'probnatobstcr' )

      end subroutine probnatobstcr
