      subroutine probparglob10 ( numunknowns )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill global array of arrays kprobfglob for parallel case
!     Only parts referring to vectors of special structure
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecpack
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: numunknowns(npointglob)

!     numunknowns         Contains the number of unknowns for all global points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iblock, npointloc, icontext, leniwork, istart, &
                 ivec, inode
      integer, pointer :: kprobfglob(:)

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     inode          Node sequence number
!     istart         First position used at a part
!     ivec           Sequence number of vector of special structure
!     kprobfglob     Global array kprobf
!     leniwork       Computed length of array iwork
!     npointloc      Number of points in local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PROBPARGLOB3   Fill array numunknowns for one block
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob10'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobe2 => kprobpart%kprobe2
      kprobe3 => kprobpart%kprobe3

!     --- Loop over all vectors of special structure

      do ivec = 1, ndtvec

         leniwork = npointglob+2
         if ( kprobe2(ivec)>0 ) then

!        --- Fill array numunknowns for all local nodes

            numunknowns = 0

            call probparglob3 ( kprobe2(ivec), npoint, nunkp, numunknowns, &
                                mappingpar, kprobe3(ivec)%arr )
            leniwork = leniwork+npointglob+1

            allocate ( iwork(leniwork) )

!           --- For blocks := 2, number_of_blocks do
!               Read mapping array as well as kprobf and kprobh from block

            icontext = 41
            do inode = 2, iacnodes

               length = leniwork
               call inireceivempi ( iwork, length, SPINT, iblock, icontext )
               if ( debug ) write(irefwr,1) 'iblock, length', iblock, length

               npointloc = iwork(1)
               istart = npointloc+2

!              --- Fill array numunknowns for all local nodes

               if ( kprobe2(ivec)>0 ) then

!              --- Fill array numunknowns for all local nodes

                  call probparglob3 ( kprobe2(ivec), npointloc, nunkp, &
                                      numunknowns, iwork(2), iwork(istart) )

               end if  ! ( kprobe2(ivec)>0 )

            end do  ! inode = 2, iacnodes

            deallocate ( iwork )

            kprobfglob => kprobfglobal(ivec)%arr
            allocate ( kprobfglob(npointglob+1), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, npointglob+1, 'kprobfglob' )
            if ( ierror/=0 ) go to 1000

            kprobfglob(1) = 0
            do i = 1, npointglob
               kprobfglob(i+1) = kprobfglob(i)+numunknowns(i)
            end do  ! i = 1, npointglob

            if ( debug ) &
               call prinin ( kprobfglob, npointglob+1, 'kprobfglob' )

         end if  ! ( kprobe2(ivec)>0 )

      end do  ! ivec = 1, ndtvec

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probparglob10'

      end if  ! ( debug )
      call erclos ( 'probparglob10' )

      end subroutine probparglob10
