      subroutine probparglob3 ( indprf, npoint, nunkp, numunknowns, &
                                map, kprobf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-05-2015 Extra debug
!        version  1.0  date 27-03-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array numunknowns for one block
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprf, npoint, nunkp, numunknowns(*), map(*), kprobf(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated (local block)
!     map            i    Contains the globla node number for all local nodes
!     npoint         i    Number of nodal points in local block
!     numunknowns   i/o   Array containing the number of unknowns per global
!                         node
!     nunkp          i    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob3'
         call prinin ( map, npoint, 'map' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprf>0 ) then

!     --- indprf>0, hence kprobf is used, i.e. number of unknowns
!         not constant

         do i = 1, npoint
            numunknowns(map(i)) = kprobf(i+1)-kprobf(i)
         end do  ! i = 1, npoint

      else

!     --- indprf=0, hence number of unknowns constant

         numunknowns(map(1:npoint)) = nunkp

      end if  ! ( indprf>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probparglob3'

      end if  ! ( debug )
      call erclos ( 'probparglob3' )

      end
