      subroutine probparglob6
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     copy data from kprobhsend to kprobhglob on host
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     parallel
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icontext, istart, n

!     i              Counting variable
!     icontext       Tag for sending and receiving, is used as check
!     istart         First position used at a part
!     n              number of free unknowns in parallel node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INISENDMPI     Send information to other node (mpi)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob6'
         call prinin ( kprobhsend, 10, 'kprobhsend' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy information of first node to kprobhglob

      istart = kprobhsend(1)
      n = kprobhsend(istart)

      if ( debug ) write(irefwr,1) 'istart, n', istart, n
      kprobhglob(1:n) = kprobhsend(istart+1:istart+n)

!     --- Copy information to other nodes

      icontext = 51
      do i = 2, iacnodes
         istart = kprobhsend(i)
         n = kprobhsend(istart)

         if ( debug ) write(irefwr,1) 'i, istart, n', i, istart, n

         call inisendmpi ( kprobhsend(istart+1), n, SPINT, i, icontext )

      end do  ! i = 2, iacnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probparglob6'

      end if  ! ( debug )
      call erclos ( 'probparglob6' )

      end subroutine probparglob6
