      subroutine probreorsend
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill new array sendinfopar with numbering corresponding to
!     dirichlet-dirichlet ordering of unknowns (parallel computations)
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     reordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lensend, i, istart, numunks

!     i              Counting variable
!     istart         Start address in sendinfopar
!     lensend        Length of array sendinfopar
!     numunks        Number of unknowns in block
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probreorsend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probreorsend'
         call prinin ( sendinfopar, lensend, 'sendinfopar' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 1  ! start address in sendinfopar
      lensend = size(sendinfopar)

      do while ( istart<lensend )
         numunks = sendinfopar(istart)  ! number of unknowns in block
         sendinfopar(istart+1:istart+numunks) = &
            kprobhdd(kprobhinv(sendinfopar(istart+1:istart+numunks)))
         istart = istart+1+numunks
      end do  ! while ( istart<lensend )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( sendinfopar, lensend, 'sendinfopar' )
         write(irefwr,*) 'End probreorsend'

      end if  ! ( debug )
      call erclos ( 'probreorsend' )

      end subroutine probreorsend
