      subroutine prord0 ( iinput, rinput, lengrl, maxlen, maxrl, &
                          iinp, iinputlc, rinputlc, maxrllc )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.1  date 02-09-2014 error message if numwis and itype>99
!        version 14.0  date 27-09-2010 Remove work arrays
!        version 13.11 date 02-11-2009 Allow series of element groups
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    read cards for definition of type numbers of internal elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengrl, maxlen, maxrl, iinp, &
              iinputlc(*), maxrllc
      double precision rinput(*), rinputlc(*)

!     iinp      i/o   Last position used in IINPUT
!     iinput     o    integer input array
!     iinputlc        integer work array to store the input read locally
!                     This input is copied into iinput afterwards in the correct
!                     sequence
!                     iinputlc is filled as follows:
!                     Per element group:
!                     Pos. 1-20:  type numbers
!                     If type numbers all in the range 1 to 99 followed by extra
!                        information read. This information is stored as read:
!                        type of information:
!                          -1 = end
!                           1 = numdegfd
!                           2 = vec
!                           3 = number
!                         This is followed by number of numbers read, followed
!                         by the numbers itself
!                         In case of vec and number the first of the number read
!                         is the sequence number
!                     If type number = -1 followed by
!                        idegfd
!                        lengrllc = starting address of real in rinputlc
!                     If type number = >10000 followed by
!                        iextra
!                        4*iextra positions with information
!     lengrl     o    Length of real array rinput
!                     (in double precision variables)
!     maxlen     i    Maximum amount of contiguous space in IBUFFR after a
!                     starting address
!     maxrl      i    Maximum number of reals that are allowed in rinput
!     maxrllc    i    Maximum number of reals that are allowed in rinputlc
!     rinput     o    Real input array
!     rinputlc        real work array to store the real input read locally
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=3, nncode3=4
      integer itype3(nncode3), ncode3(nncode3), ncodmx3(nncode3), &
              nscods(ncodsb)
      double precision :: tabmin3(nncode3), tabmax3(nncode3)
      character (len=11) codesb(ncodsb), code3(nncode3)
      integer k, ielgrp, icheck, itypnr, i, iextra, inum, &
              ntype, iinplc, numveclc, nsear, lengrllc
      integer, allocatable :: iwork(:)

!     code3          Table with keywords
!     codesb         Table with subkeywords
!     i              Counting variable
!     icheck         Indication if all type numbers are smaller than 100 (0) or
!                    not (1)
!     ielgrp         element group number
!     iextra         if >0 extra arrays of special structure must be defined
!     iinplc         Last position used in array iinputlc
!     inum           Number of numbers read in a record
!     itype3         Indicates the type of action corresponding to keywords
!                    in code3
!     itypnr         Type number of element
!     iwork          integer work array of length nelgrp+201 to store
!                    starting addresses
!                    The positions 1 .. nelgrp contains the starting addresses
!                    of all element groups in array iinputlc
!                    Pos. nelgrp+1 contains the starting address of numdegfd
!                    for type numbers between 1 and 99
!                    Pos. nelgrp+1+ivec contains the starting address of
!                    vectors of special structure ivec
!                    for type numbers between 1 and 99
!                    Pos. nelgrp+1+Numvec+isear contains the starting address
!                    of physical degrees of freedom isear
!                    for type numbers between 1 and 99
!     k              Counting variable
!     lengrllc       Length of array rinputlc
!     ncode3         Table containing the number of significant letters in code3
!     ncodmx3        See ncodmx, now for code3
!     ncodsb         Number of entries in codesb
!     nncode3        Number of entries in code3
!     nscods         Number of sub keywords
!     nsear          Number of specific degrees of freedom
!     ntype          number of types  (maximal 20)
!     numveclc         Number of vectors of special structure
!     tabmax3        Contains maximum values allowed in table for code3
!     tabmin3        Contains minimum values allowed in table for code3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRORD0COPY     Copy array iinputlc into iinput and rinputlc into rinput
!     PRORD0PER      Read information for periodical boundary conditions
!     PRORD0TYPE     Read information of type numbers
!     PRORD0USER     Read information for user elements
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!    For all element groups:
!    elgrp i = ( type = n1, [n2, n3, ...] )
!       if all types positive and<100 possibly followed by:
!          numdegfd = ...  (default: 1)
!          vec j = ...
!          number = ...
!       if type<0 followed by
!          constant=c1, factor=c2, degfdi, degfdj , ... (may be repeated)
!       if type>10000 followed by mod(itype,10000) records of the shape:
!          n_vertex=., n_midside_point=., n_centroid=., n_face=.
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     550   array length too short
!    1534   Type number does not exist
!    3202   Type number not between 1 and 99 for numerical analysis lab
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'N_VERTEX', 'N_MIDSIDE_', 'N_CENTROID', 'N_FACE' /
      data ncode3  / 5, 5, 5, 5 /
      data ncodmx3 / 8,15,10, 6 /
      data itype3  / 0, 0, 0, 0 /
      data tabmin3 / 0d0, 0d0, 0d0, 0d0 /
      data tabmax3 / 1d3, 1d3, 1d3, 1d3 /
      data codesb / 'ELGRP', 'NUMBER', 'TYPE' /
      data nscods /  3, 6, 4 /

!     code3    Table of local keywords
!              1: N_VERTEX
!              2: N_MIDSIDE_POINT
!              3: N_CENTROID
!              4: N_FACE
!     codesb   Table of local subkeywords
!              1: ELGRP
!              2: NUMBER
!              3: TYPE
! ======================================================================
!
      call eropen ( 'prord0' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord0'
         write(irefwr,1) 'iinp, icode', iinp, icode
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      numveclc = 0
      nsear  = 0

!     --- Copy information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )

      allocate ( iwork(nelgrp+201), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp+201, 'iwork' )
      if ( ierror/=0 ) go to 1000

      lengrllc = 0
      lengrl = 0

!     --- read next nelgrp cards
!         indicate end of record (ityprd=1) only if record has not already
!         been read (hence icode#-11)

      if ( icode/=-11 ) ityprd = 1

      iinplc = 0
      ntype = iinput(1)
      iwork(1:nelgrp) = 0
      if ( debug ) write(irefwr,1) 'icode', icode

200   if ( icode==8 .or. icode==nncodemain+1 .or. icode==-11 ) then

!     --- Read information of types

         if ( icode/=-11 ) icode = 0
         call prord0type ( iinplc, iinputlc, &
                           ntype, iwork, inum, ielgrp )
         if ( debug ) write(irefwr,1) 'icode/1', icode
         if ( icode==8 ) go to 1000  ! TYPES found
         if ( icode>0 ) go to 200    ! repeat

!        --- check if all type numbers are larger than 99 or not

         icheck = 0
         do k = 1, inum
            itypnr = iinputlc(iinplc+k)
            if ( numwis .and. (itypnr<1 .or. itypnr>99) ) then

!           --- Error type number not between 1 and 99 for the
!               numerical analysis lab

               call errint ( itypnr, 1 )
               call errsub ( 3202, 1, 0, 0 )
               go to 1000

            else if ( itypnr>=100 .or. itypnr<=0 .and. itypnr>=-3 ) then
               icheck = 1
            else if ( itypnr<-3 ) then
               call errint ( itypnr, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 1534, 2, 0, 0 )
            end if
         end do
         iextra = itypnr/10000
         iinplc = iinplc+20
         if ( iinplc>maxlen ) then

!        --- Too much information, i.e. maxlen too short

            call errchr ( 'iinputlc', 1 )
            call errint ( iinplc, 1 )
            call errint ( maxlen, 2 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         if ( debug ) write(irefwr,1) 'itypnr', itypnr

         if ( itypnr==-1 ) then

!        --- periodical boundary condition found or boundary condition of
!            the type ur = alpha ul + c

            call prord0per ( iinplc, iinputlc, lengrllc, rinputlc, ielgrp )
            if ( ierror/=0 ) go to 1000

         else if ( icheck==0 ) then

!        --- all elements are user elements

            call prord0user ( ielgrp, iinplc, iinputlc, numveclc, nsear )
            if ( icode>0 ) go to 200

         else if ( iextra>0 ) then

!        --- iextra extra records required

            iinputlc(iinplc+1) = iextra
            iinplc = iinplc+1

            call sepcopycode ( code3, nncode3, ncode3, ncodmx3 )
            call sepcopytype ( nncode3, itype3, tabmin3, tabmax3 )
            nmain = 0
            iretur = icode

            do i = 1, iextra
               table(1:nncode) = 0d0
               ityprd = 1
               if ( debug ) write(irefwr,*) 'before inxt02/4'
               call sepreadsetkeyw
               if ( ierror/=0 ) go to 1000
               iinputlc(iinplc+1:iinplc+nncode3) = &
                  nint(table(1:nncode3))
               iinplc = iinplc + nncode3
            end do
            icode = iretur

         else

!        --- Standard situation, the next record must be read

            icode = 8

         end if
         go to 200

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      if ( lengrllc>maxrllc ) then

!     --- Too much information, i.e. maxlrlc too short

         call errchr ( 'rinputlc', 1 )
         call errint ( lengrllc, 1 )
         call errint ( maxrllc, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy array iinputlc into iinput and rinputlc into rinput

      call prord0copy ( iwork, iinput, iinp, iinputlc, &
                        ntype, rinput, lengrl, rinputlc, nncode3, &
                        lengrllc, numveclc, nsear )

      if ( iinp>maxlen ) then
         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if
      if ( lengrl>maxrl ) then
         call errint ( lengrl, 1 )
         call errint ( maxrl, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if
      iinput(1) = ntype

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp, lengrl', iinp, lengrl
         call prinin ( iinput, iinp, 'iinput' )
         if ( lengrl>0 ) call prinrl ( rinput, lengrl, 'rinput' )
         write(irefwr,*) 'End prord0'

      end if
      call erclos ( 'prord0' )

      end subroutine prord0
