      subroutine prord1copy ( iwork, iinput, iinp, iinputlc, &
                              ntype, nncode3, numvec, nsear, numnat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy array iinputlc into iinput for boundary elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinputlc(*), ntype, nncode3, &
                             numvec, nsear, numnat
      integer, intent(inout) :: iinp, iinput(*), iwork(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput         o    integer input array
!                         See Programmer's guide
!     iinputlc       i    See subroutine prord1
!     iwork         i/o   See subroutine prord1
!     nncode3        i    Number of items in array code3
!     nsear          i    Number of specific degrees of freedom
!     ntype          i    Number of different type numbers to be used for one
!                         group
!     numnat         i    number of boundary element groups
!     numvec         i    Number of vectors of special structure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ibngrp, iinplc, ibngrplast, ibngrpfirst, i, &
                 icheck, k, itypnr, iextra, istart, inpelm, &
                 indtvc, inum, isear, ivec, npelm

!     i              Counting variable
!     ibngrp         boundary element group number
!     ibngrpfirst    first boundary element group number
!     ibngrplast     last boundary errelement group number
!     icheck         Indication if all type numbers are smaller than 100
!                    (0) or  not (1)
!     iextra         Extra space needed at both sides of the array for
!                    control integers
!     iinplc         Last position used in array iinputlc
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     isear          Number of specific degrees of freedom
!     istart         First position used at a part
!     itypnr         Type number of element
!     ivec           Sequence number of vector of special structure
!     k              Counting variable
!     npelm          Number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1085   No information read for element group number
!    1086   No information read for vector of special structure
!    1087   No information read for physical degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord1copy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord1copy'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ibngrp = 1

      do while ( ibngrp<=numnat )

         iinplc = iwork(ibngrp)
         if ( iinplc==0 ) then

!        --- boundary element group has not been found, error

            call errint ( ibngrp, 1 )
            call errsub ( 1085, 1, 0, 0 )
            do i = 1, 20
               iinput(iinp+i) = 0
            end do
            iinp = iinp+20

         else

!        --- boundary element group has been filled

            ibngrpfirst = iinputlc(iinplc)
            ibngrplast = iinputlc(iinplc+1)

            do ibngrp = ibngrpfirst, ibngrplast

               iinplc = iwork(ibngrp)+2
               iinput(iinp+1:iinp+20) = iinputlc(iinplc:iinplc+19)
               iinplc = iinplc+20

!              --- check if all type numbers are larger than 99 or not

               icheck = 0
               do k = 1, ntype
                  itypnr = iinput(iinp+k)
                  if ( itypnr>=100 .or. itypnr==0 ) icheck = 1
               end do
               iextra = itypnr/10000
               iinp = iinp+20

               if ( icheck==0 ) then

!              --- icheck = 0, user elements

                  npelm = iinputlc(iinplc)
                  iinplc = iinplc+1
                  inpelm = max(1,npelm)
                  iinp = iinp+1
                  iinput(iinp) = inpelm

!                 --- Find starting addresses of information

                  istart = iinplc
                  do k = 1, numvec+nsear+1
                     iwork(1000+k) = 0
                  end do
                  indtvc = 0
                  isear  = 0

!                 --- While loop until end of information

300               if ( iinputlc(istart)==1 ) then

!                 --- Type: numdegfd

                     iwork(1001) = istart
                     inum = iinputlc(istart+1)
                     istart = istart+inum+2
                     go to 300

                  else if ( iinputlc(istart)==2 ) then

!                 --- Type: vectors of special structure

                     inum = iinputlc(istart+1)
                     ivec = iinputlc(istart+2)
                     iwork(1001+ivec) = istart
                     istart = istart+inum+2
                     indtvc = max(indtvc,ivec)
                     go to 300

                  else if ( iinputlc(istart)==3 ) then

!                 --- Type: physical degrees of freedom

                     inum = iinputlc(istart+1)
                     ivec = iinputlc(istart+2)
                     isear = max(isear,ivec)
                     iwork(1001+numvec+ivec) = istart
                     istart = istart+inum+2
                     go to 300

                  end if

!                 --- Fill information, first numdegfd

                  iinput(iinp+1) = -1
                  iinput(iinp+2) = indtvc
                  iinput(iinp+3) = isear
                  istart = iwork(1001)
                  if ( istart>0 ) then

!                 --- istart>0 information of degrees of freedom

                     inum = iinputlc(istart+1)
                     if ( inum==1 ) then

!                    --- inum = 1 constant number of degrees of freedom

                        iinput(iinp+1) = iinputlc(istart+2)
                        iinp = iinp+3

                     else

!                    --- inum>1 non-constant number of degrees of freedom

                        iinput(iinp+1) = 0
                        do k = 1, inpelm
                           iinput(iinp+k+3) = iinputlc(istart+1+k)
                        end do
                        iinp = iinp+3+inpelm

                     end if

                  else

!                 --- istart = 0 no information of degrees of freedom

                     iinp = iinp+3

                  end if

!                 --- Next vectors of special structure

                  do i = 1, indtvc
                     istart = iwork(1001+i)
                     if ( istart==0 ) then

!                    --- istart = 0, no information found

                        call errint ( ivec, 1 )
                        call errint ( ibngrp, 2 )
                        call errwar ( 1086, 2, 0, 0 )
                        do k = 1, inpelm
                           iinput(iinp+k) = 0
                        end do
                        iinp = iinp+inpelm

                     else

!                    --- istart>0, information found

                        inum = iinputlc(istart+1)
                        if ( inum==2 ) then
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+3)
                           end do
                        else
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+2+k)
                           end do
                        end if
                        iinp = iinp+inpelm

                     end if

                  end do

!                 --- Finally physical degrees of freedom

                  do i = 1, isear
                     istart = iwork(1001+numvec+i)
                     if ( istart==0 ) then

!                    --- istart = 0, no information found

                        call errint ( ivec, 1 )
                        call errint ( ibngrp, 2 )
                        call errwar ( 1087, 2, 0, 0 )
                        do k = 1, inpelm
                           iinput(iinp+k) = 0
                        end do
                        iinp = iinp+inpelm

                     else

!                    --- istart>0, information found

                        inum = iinputlc(istart+1)
                        if ( inum==2 ) then
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+3)
                           end do
                        else
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+2+k)
                           end do
                        end if
                        iinp = iinp+inpelm

                     end if

                  end do

               else if ( iextra>0 ) then

!              --- iextra>0, extra information

                  iinp = iinp+1
                  iinput(iinp+1) = iextra
                  iinplc = iinplc+1
                  do i = 1, iextra
                     do k = 1, nncode3
                        iinput(iinp+k) = iinputlc(iinplc+k)
                     end do
                     iinplc = iinplc + nncode3
                     iinp = iinp + nncode3
                  end do

               end if

            end do  ! ibngrp = ibngrpfirst, ibngrplast

            ibngrp = ibngrplast+1

         end if  ! ( iinplc==0 )

      end do  ! while ( ibngrp<=numnat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord1copy'

      end if  ! ( debug )
      call erclos ( 'prord1copy' )

      end subroutine prord1copy
