      subroutine prord2cross ( iinputlc, iinplc, itype, natobstcross, &
                               table, istartpnt, tabmax, tabout, &
                               lenobstcr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-03-2012 Extra parameter lenobstcr
!        version  1.0  date 14-04-2012
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for cros_section_obstacle records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     natural_boundary_condition
!     obstacle
!     cross_section
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natobstcross, istartpnt, &
              lenobstcr
      double precision table(*), tabmax(*), tabout(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     istartpnt      i    Start position of keyword p in table
!     itype          o    Type of record found
!     lenobstcr      i    Length of cross_section_obstacles part
!     natobstcross  i/o   number of parts where cross_section_obstacle elements
!                         are defined
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, iexclude

!     iexclude       Type of exclusion method
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2cross' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2cross'
         write(irefwr,1) 'iinplc', iinplc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = 13
      natobstcross = natobstcross + 1
      iexclude = nint(table(14))
      iinputlc(iinplc+1) = itype
      iinputlc(iinplc+2) = nint(table(13))
      iinputlc(iinplc+3) = iexclude
      inum = 0
      if ( table(15)>0d0 ) then

!     --- exclude_curves found

         inum = nint(tabmax(istartpnt+1))
         iposut = nint(table(istartpnt+1))

         iinputlc(iinplc+4) = 1
         iinputlc(iinplc+5) = inum
         iinputlc(iinplc+6:iinplc+5+inum) = &
            nint(tabout(iposut:iposut+inum-1))

      else

!     --- not, exclude_curves found; set equal to zero

         iinputlc(iinplc+4) = 0
         iinputlc(iinplc+5) = 0

      end if

      lenobstcr = lenobstcr+5+inum

1000  call erclos ( 'prord2cross' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinputlc(iinplc), 6+inum, 'iinputlc' )
         write(irefwr,1) 'lenobstcr', lenobstcr
         write(irefwr,*) 'End prord2cross'

      end if  ! ( debug )

      iinplc = iinplc+5+inum

      end
