      subroutine prord2surf ( iinputlc, iinplc, itype, natsrf, table, &
                              istartpnt, tabmax, tabout, lensrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-03-2012 Extra parameter lensrf
!        version  1.0  date 14-04-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for surface records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     natural_boundary_condition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natsrf, istartpnt, &
              lensrf
      double precision table(*), tabmax(*), tabout(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     istartpnt      i    Start position of keyword p in table
!     itype          o    Type of record found
!     lensrf        i/o   Length of surfaces part in iinput for boundary
!                         elements
!     natsrf        i/o   number of parts where surface elements are defined
!                         for natural boundary conditions
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut

!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     407   record with surfaces contains wrong number of surface numbers
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2surf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2surf'
         write(irefwr,1) 'iinplc', iinplc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- surface card found

      itype = 4
      natsrf = natsrf+1

      inum = nint(tabmax(istartpnt+2))
      if ( inum==0 ) then

!     --- No surface numbers found

         call errsub ( 407, 0, 0, 0 )
         go to 1000

      end if  ! ( inum==0 )

      iposut = nint(table(istartpnt+2)) ! starting address in tabout
      iinputlc(iinplc+1) = itype
      iinputlc(iinplc+2) = inum
      iinplc = iinplc+2
      iinputlc(iinplc+1:iinplc+inum) = &
         nint(tabout(iposut:iposut+inum-1))
      if ( debug ) then
         write(irefwr,1) 'iinplc/surfs', iinplc
         call prinin ( iinputlc(iinplc-2), inum+3, 'iinputlc' )
      end if  ! ( debug )
      iinplc = iinplc+inum
      lensrf = lensrf+2+inum

1000  call erclos ( 'prord2surf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinplc', iinplc
         write(irefwr,*) 'End prord2surf'

      end if  ! ( debug )

      end
