      subroutine prord3detect ( iinputlc, iinplc, table, tabout, tabmin, &
                                tabmax, nespnt, lenpnt, &
                                nescrv, nessrf, code, ncodmx, lensrf, &
                                nesvlm, lenvlm, nesnod, neselm, nesgrp, &
                                lengrp, nesobst, nesconct, nesnocon, &
                                nesnodal, lennodal, nesallobst, &
                                nesinnobst, nesbnobst, nesonbnobst, &
                                nesinonbnobst, neszerolev, nescav, &
                                strout, lencrv, nesintersect, &
                                leninters )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-10-2013 Extra parameters
!        version  1.0  date 30-03-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret information read by prord3 and store in iinputlc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmoduleread
      use sepmoduletexts
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  iinputlc(*), iinplc, nespnt, lenpnt, nescrv, nessrf, &
               ncodmx(*), lensrf, nesvlm, lenvlm, &
               nesnod, neselm, nesgrp, lengrp, nesobst, nesconct, &
               nesnocon, nesnodal, lennodal, nesallobst, nesinnobst, &
               nesbnobst, nesonbnobst, nesinonbnobst, neszerolev, &
               nescav, lencrv, nesintersect, leninters
      double precision table(*), tabout(*), tabmin(*), tabmax(*)
      character (len=*) code(*), strout(*)

!     code           i    Table with keywords
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   integer work array where the information read is
!                         stored before processing
!     lencrv         i    Number of entries in array curves
!     lengrp         i    Length of group part
!     leninters      o    Length of intersection part
!     lennodal       i    Length of nodal_point part
!     lenpnt         i    Length of points part
!     lensrf         i    Length of surfaces part
!     lenvlm         i    Length of volumes part
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nesallobst    i/o   Number of obstacles where all nodes are prescribed
!     nesbnobst     i/o   Number of obstacles where all but the inner nodes are
!                         prescribed
!     nescav        i/o   Number of parts where cavity bondary conditions are given
!     nesconct      i/o   Number of contacts at which essential boundary
!                         conditions are given
!     nescrv        i/o   Number of parts where curve essential boundary
!                         conditions are given
!     neselm        i/o   Number of parts where element essential boundary
!                         conditions are given
!     nesgrp        i/o   Number of groups of element groups with
!                         essential boundary conditions
!     nesinnobst    i/o   Number of obstacles where all inner nodes are prescribed
!     nesinonbnobst i/o   Number of obstacles where all he nodes on the
!                         obstacle which are only part of elements completely
!                         inside the obstacle are prescribed
!     nesintersect  i/o   Number of intersections where essential boundary
!                         conditions are given
!     nesnocon      i/o   Number of "no"-contacts with essential boundary
!                         conditions
!     nesnod        i/o   Number of groups of nodes with essential boundary
!                         conditions
!     nesnodal      i/o   Number of groups of nodal points with essential
!                         boundary conditions
!     nesobst       i/o   Number of obstacles with essential boundary
!                         conditions
!     nesonbnobst   i/o   Number of obstacles where all the nodes on the boundary of
!                         the obstacle are prescribed
!     nespnt        i/o   Number of parts where point essential boundary
!                         conditions are given
!     nessrf        i/o   Number of parts where surface essential boundary
!                         conditions are given
!     nesvlm        i/o   Number of parts where volume essential boundary
!                         conditions are given
!     neszerolev    i/o   Number of parts where essential boundary conditions of
!                         the type zero_levelset i are given
!     strout         i    Contains series of strings
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncount, iposut, inum, lcur, iobst, iconct, i, k, jnpelm
      logical ::  nonzerofound

!     i              Counting variable
!     iconct         Sequence number of contact
!     inum           Number of numbers read in a record
!     iobst          Obstacle sequence number
!     iposut         Position in array tabout
!     jnpelm         Number of nodes in an element
!     k              Counting variable
!     lcur           Parameter l in curves record that has been read
!     ncount         Counter
!     nonzerofound   If true a non zero node has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibit

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIBIT         Translate degrees of freedom into bit pattern
!     PRININ         print 1d integer array
!     PRORD3CURVE    Interprete curve information read by prord3 and
!                    store in iinputlc
!     PRORD3SURF     Interprete surface information read by prord3 and
!                    store in iinputlc
!     PRORD3VOLM     Interprete volume information read by prord3 and
!                    store in iinputlc
!     READC301       Check the numrel numbers stored in array relnods
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     434   the number of relative nodes found in record with element # 1
!    2970   Number of surfaces found after keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3detect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3detect'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(1500) )

      iinputlc(iinplc+1) = 0

      ncount = 0

!     --- next card found,  detect

      if ( table(1)>0d0 ) then

!     --- degree of freedom found

         iposut = nint(table(1))
         ncount = nint(tabmax(1))
         ind(1:ncount) = nint(tabout(iposut:iposut+ncount-1))
         iinputlc(iinplc+1) = inibit(ind,ncount)

      end if
      if ( table(2)>0d0 ) then

!     --- point card found

         nespnt = nespnt+1
         iposut = nint(table(2))
         inum = nint(tabmax(2))
         iinputlc(iinplc+2) = 1
         iinputlc(iinplc+3) = inum
         iinputlc(iinplc+4:iinplc+3+inum) = &
            nint(tabout(iposut:iposut+inum-1))
         iinplc = iinplc+inum+3
         lenpnt = lenpnt+inum+2

      else if ( table(3)>0d0 .or. table(24)>0d0 ) then

!     --- curves card found or outer_curves

         call prord3curve ( iinputlc, iinplc, table, tabout, tabmin, &
                            tabmax, nescrv, lencrv )

      else if ( table(5)>0d0 .or. table(25)>0d0 ) then

!     --- surface card found, or outer surfaces detected

         call prord3surf ( iinputlc, iinplc, table, tabout, tabmax, &
                           code, nessrf, lensrf, ncodmx )

      else if ( table(6)>0d0 ) then

!     --- volume card found, detect

         call prord3volm ( iinputlc, iinplc, table, tabout, tabmax, &
                           nesvlm, lenvlm )

      else if ( table(7)>0d0 ) then

!     --- nodes

         nesnod = nesnod+1
         iinputlc(iinplc+2) = 4
         iposut = nint(table(7))
         inum = nint(tabmax(7))
         iinputlc(iinplc+3) = nint(tabout(iposut))
         if ( inum>1 ) then
            iinputlc(iinplc+4) = nint(tabout(iposut+1))
         else
            iinputlc(iinplc+4) = iinputlc(iinplc+3)
         end if
         iinplc = iinplc+4

      else if ( table(8)>0d0 ) then

!     --- element card found, detect

         neselm = neselm+1
         iinputlc(iinplc+2) = 5
         iposut = nint(table(8))
         inum = nint(tabmax(8))
         iinputlc(iinplc+3) = nint(tabout(iposut))
         iinputlc(iinplc+4) = nint(tabout(iposut+1))

         if ( inum/=3 ) then
            call errint ( inum-2, 1 )
            call errsub ( 434, 1, 0, 0 )
         else
            iinputlc(iinplc+5) = nint(tabout(iposut+2))
         end if

         iinplc = iinplc+5

      else if ( table(9)>0d0 ) then

!     --- group card found

         iinputlc(iinplc+2) = 6
         nesgrp = nesgrp+1
         iposut = nint(table(9))
         inum = nint(tabmax(9))
         iinputlc(iinplc+4) = nint(tabout(iposut))
         jnpelm = sepgetinpelm ( iinputlc(iinplc+4) )
         iinputlc(iinplc+3) = jnpelm

         if ( inum<=1 ) then

            iinputlc(iinplc+5:iinplc+4+jnpelm) = 1

         else

!        --- inum>1,  copy contents from work into iinput

            call readc301 ( inum, jnpelm, tabout(iposut+1), work )
            iinputlc(iinplc+5:iinplc+4+jnpelm) = nint(work(1:jnpelm))

         end if
         iinplc = iinplc+4+jnpelm
         lengrp = lengrp+2+jnpelm

      else if ( table(10)>0d0 ) then

!     --- obstacle card found, detect

         iinputlc(iinplc+2) = 7
         nesobst = nesobst+1
         iobst = nint(table(10))
         iinputlc(iinplc+3) = 1
         iinputlc(iinplc+4) = iobst
         iinplc = iinplc+4

      else if ( table(11)>0d0 ) then

!     --- contact card found, detect

         iinputlc(iinplc+2) = 8
         nesconct = nesconct+1
         iconct = nint(table(11))

         iinputlc(iinplc+3) = 1
         iinputlc(iinplc+4) = iconct
         iinplc = iinplc+4

      else if ( table(12)>0d0 ) then

!     --- no_contact card found, detect

         iinputlc(iinplc+2) = 9
         nesnocon = nesnocon+1
         iconct = nint(table(12))

         iinputlc(iinplc+3) = 1
         iinputlc(iinplc+4) = iconct
         iinplc = iinplc+4

      else if ( table(17)>0d0 ) then

!     --- nodal_points card found, detect

         iposut = nint(table(17))
         inum = nint(tabmax(17))

!        --- Check if there are any node numbers larger than zero, &
!            otherwise skip this part

         nonzerofound = .false.
         do i = 1, inum
            if ( nint(tabout(iposut-1+i))>0 ) nonzerofound = .true.
         end do
         if ( nonzerofound ) then

!        --- There is at least one non-zero node

            nesnodal = nesnodal+1
            k = 0  ! number of non-zero nodes
            do i = 1, inum

               lcur = nint(tabout(iposut-1+i))
               if ( lcur>0 ) then

!              --- lcur>0, new node found

                  k = k+1
                  iinputlc(iinplc+3+k) = lcur

               end if  ! ( lcur>0 )

            end do  ! i = 1, inum

            iinputlc(iinplc+3) = k
            iinputlc(iinplc+2) = 10
            iinplc = iinplc+3+k
            lennodal = lennodal+2+k

         end if  ! ( nonzerofound )

      else if ( table(13)>0d0 ) then

!     --- file_nodal_points card found

         iinputlc(iinplc+2) = 10
         nchar = nint(tabmax(13))
         iposut = nint(table(13))
         iinputlc(iinplc+3) = -nextxt
         texts(nextxt) = strout(iposut)(1:nchar)
         nextxt = nextxt+1
         nesnodal = nesnodal+1
         iinplc = iinplc+3

      else if ( table(18)>0d0 ) then

!     --- IN_ALL_OBSTACLE card found

         iinputlc(iinplc+2) = 12
         nesallobst = nesallobst+1
         iobst = nint(table(18))

         iinputlc(iinplc+3) = iobst
         iinplc = iinplc+3

      else if ( table(19)>0d0 ) then

!     --- IN_INNER_OBSTACLE card found

         iinputlc(iinplc+2) = 13
         nesinnobst = nesinnobst+1
         iobst = nint(table(19))

         iinputlc(iinplc+3) = iobst
         iinplc = iinplc+3

      else if ( table(20)>0d0 ) then

!     --- IN_BOUN_OBSTACLE card found

         iinputlc(iinplc+2) = 14
         nesbnobst = nesbnobst+1
         iobst = nint(table(20))

         iinputlc(iinplc+3) = iobst
         iinplc = iinplc+3

      else if ( table(21)>0d0 ) then

!     --- ON_BOUN_OBSTACLE card found

         iinputlc(iinplc+2) = 15
         nesonbnobst = nesonbnobst+1
         iobst = nint(table(21))

         iinputlc(iinplc+3) = iobst
         iinplc = iinplc+3

      else if ( table(22)>0d0 ) then

!     --- INON_BOUN_OBSTACLE card found

         iinputlc(iinplc+2) = 16
         nesinonbnobst = nesinonbnobst+1
         iobst = nint(table(22))

         iinputlc(iinplc+3) = iobst
         iinplc = iinplc+3

      else if ( table(26)>0d0 ) then

!     --- zero_levelset card found, detect

         iinputlc(iinplc+2) = 17
         neszerolev = neszerolev+1
         iinputlc(iinplc+3) = nint(table(26))
         iinplc = iinplc+3

      else if ( table(27)>0d0 ) then

!     --- zero_levelset card found, detect

         iinputlc(iinplc+2) = 18
         nescav = nescav+1
         iinputlc(iinplc+3) = nint(table(27))
         iinplc = iinplc+3

      else if ( table(28)>0d0 ) then

!     --- intersection card found, detect

         iinputlc(iinplc+2) = 19  !intersection
         nesintersect = nesintersect+1
         iposut = nint(table(28)) ! position in tabout
         inum = nint(tabmax(28))  ! number of surfaces

         if ( debug ) &
            write(irefwr,1) 'iinplc, nesintersect, iposut, inum', &
                             iinplc, nesintersect, iposut, inum

         if ( inum<2 ) then

            call errint ( inum, 1 )
            call errint ( 2, 2 )
            call errint ( 97, 3 )
            call errchr ( code(28), 1 )
            call errsub ( 2970, 3, 0, 1 )

         end if  ! ( inum<2 )

         iinputlc(iinplc+3) = 2   ! surface
         iinputlc(iinplc+4) = inum ! number of surfaces
         iinputlc(iinplc+5:iinplc+4+inum) = &
            nint(tabout(iposut:iposut+inum-1))
         leninters = leninters+3+inum
         if ( debug ) then
            write(irefwr,1) 'iinplc, leninters', iinplc, leninters
            call prinin ( iinputlc(iinplc+1), 4, 'iinplc' )
            call prinin ( iinputlc(iinplc+5), inum, 'surfaces' )
         end if  ! ( debug )
         iinplc = iinplc+4+inum

      end if

      deallocate ( work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'leninters', leninters
         write(irefwr,*) 'End prord3detect'

      end if  ! ( debug )
      call erclos ( 'prord3detect' )

      end
