      subroutine prordfictunk ( iinput, maxlen, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Remove work arrays
!        version  1.1  date 29-04-2001 Extension for type 923
!        version  1.0  date 07-02-2000
!
!   copyright (c) 2000-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    read cards for definition of type numbers of fictitious elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp, iinputlc(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput         o    integer input array
!     iinputlc            integer work array to store the input read locally
!                         This input is copied into iinput afterwards in the
!                         correct sequence
!                         iinputlc is filled as follows:
!                         Per fictitious element group:
!                         Pos. 1-20:  type numbers
!                         Pos. 21: numdegfd (if -1 see element type number)
!     maxlen         i    Maximum amount of contiguous space in IBUFFR after a
!                         starting address
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=2
      integer nscods(ncodsb)
      character (len=11) :: codesb(ncodsb)
      integer inum, ifictgrp, icheck, itypnr, i, &
              ntype, nfictgrp, iinplc, numdegfd, iwork(1301)

!     codesb         Table with subkeywords
!     i              Counting variable
!     icheck         Indication if all type numbers are smaller than 100
!                    (0) or  not (1)
!     ifictgrp       fictitious element group number
!     iinplc         Last position used in array iinputlc
!     inum           Number of numbers read in a record
!     itypnr         Type number of element
!     iwork          integer work array of length 1301 to store
!                    starting addresses
!                    The positions 1 .. 1000 contains the starting
!                    addresses of all fictitious element groups in array
!                    iinputlc
!                    Pos. 1001 contains the starting address of numdegfd
!                    for type numbers between 1 and 99
!                    Pos. 1001+ivec contains the starting address of
!                    vectors of special structure ivec
!                    for type numbers between 1 and 99
!                    Pos. 1001+Numvec+isear contains the starting address
!                    of physical degrees of freedom isear
!                    for type numbers between 1 and 99
!     ncodsb         Number of entries in codesb
!     nfictgrp       number of element groups corresponding to fictitious
!                    unknowns
!     nscods         Number of sub keywords
!     ntype          Number of different type numbers to be used for one
!                    group
!     numdegfd       Number of degrees of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!    For all fictitious element groups:
!    fictgrp i = type = t
!       if all types positive and<100 possibly followed by:
!          numdegfd = ...  (default: ndim)
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!      67   ielgrp has wrong value
!     162   Number of type numbers read in record is incorrect
!     163   element group number read is not the next element group number
!     550   array length too short
!    1085   No information read for element group number
!    1092   Type number does not exist
!    1758   No element group number found after elgrp
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codesb / 'FICTGRP', 'TYPE' /
      data nscods /  3, 3 /

!     codesb   Table of local subkeywords
!              1: fictgrp
! ======================================================================
!
!     --- read first card  (problem)

      call eropen ( 'prordfictunk' )

!     --- Copy information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )

      nfictgrp = 0

!     --- read next nfictgrp cards

      ityprd = 1
      iinplc = 0
      ntype = iinput(1)
      iwork(1:1000) = 0
200   if ( icode==14 .or. icode==nmain+1 ) then

!     --- new fictitious element group

         nncode = 1
         code(1) = 'FICTGRP'
         ncode(1) = 3
         ncodmx(1) = 7
         itype(1) = 29
         tabmin(1) = 2d0
         tabmax(1) = 1d6
         table(1) = -2
         iretur = 0
         nmain = -nncodemain
         call sepreadsetkeyw
         icode = iretur
         if ( icode>0 ) then

!        --- Main keyword found

            if ( icode==7 ) then

!           --- Problem found, get next item

               call asknxt

            end if
            go to 200

         end if

         if ( table(1)<0 ) then

!        --- not a fictitious element group number found

            call errsub ( 1758, 0, 0, 0 )
            iposut = 1
            inum = 1

         else
            iposut = nint(table(1))
            inum = nint(tabmax(1))
         end if
         ifictgrp = nint(tabout(iposut))
         if ( ifictgrp<1 .or. ifictgrp>1000 ) then

!        --- ifictgrp out of range

            call errint ( ifictgrp, 1 )
            call errint ( 1000, 2 )
            call errsub ( 67, 2, 0, 0 )

         else if ( iwork(ifictgrp)/=0 ) then

!        --- ifictgrp has been used already

            call errint ( ifictgrp, 1 )
            call errsub ( 163, 1, 0, 0 )

         else

            iwork(ifictgrp) = iinplc+1
            nfictgrp = max(nfictgrp,ifictgrp)

         end if

         ntype = max(ntype,inum-1)
         if ( inum-1/=1 ) then
            call errint ( inum-1, 1 )
            call errint ( 20, 2 )
            call errint ( ifictgrp, 3 )
            call errsub ( 162, 3, 0, 0 )
            call errcrd
         end if
         iinputlc(iinplc+1) = nint(tabout(iposut+1))

!        --- check if all type numbers are larger than 99 or not

         icheck = 0
         itypnr = iinputlc(iinplc+1)
         if ( itypnr>=100 .or. itypnr==0 ) then
            icheck = 1
         else if ( itypnr<0 ) then

!        --- Negative type number not allowed

            call errint ( itypnr, 1 )
            call errint ( ifictgrp, 2 )
            call errsub ( 1092, 2, 0, 0 )

         end if
         iinplc = iinplc+1
         if ( iinplc>maxlen ) then

!        --- Too much information, i.e. maxlen too short

            call errchr ( 'iinputlc', 1 )
            call errint ( iinplc, 1 )
            call errint ( maxlen, 2 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         if ( icheck==0 ) then

!        --- all elements are user elements
!            Read rest of information until a main keyword or fictgrp is found

250         table(1) = -1d20
            nncode = 1
            code(1) = 'NUMDEGFD'
            ncode(1) =  5
            ncodmx(1) =  8
            itype(1) = 28
            tabmin(1) = 1d2
            tabmax(1) = 100d0
            iretur = 0
            nmain = -nncodemain-1
            call sepreadsetkeyw
            icode = iretur

            if ( icode>0 ) then

!           --- icode>0, end of information

               iinplc = iinplc+1
               if( itypnr==923 )then
                  iinputlc(iinplc) = 1
               else
                  iinputlc(iinplc) = ndim
               end if
               go to 200

            else if ( table(1)>0d0 ) then

!           --- numdegfd found

               iposut = nint(table(1))
               inum = nint(tabmax(1))
               iinplc = iinplc+1
               iinputlc(iinplc) = inum

            end if
            go to 250

         else

!        --- Standard situation, the next record must be read

            icode = 14
            iinplc = iinplc+1
            iinputlc(iinplc) = -1

         end if
         go to 200

      end if

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      if ( ierror/=0 ) go to 1000

!     --- Copy array iinputlc into iinput and rinputlc into rinput

      do ifictgrp = 1, nfictgrp
         iinplc = iwork(ifictgrp)
         if ( iinplc==0 ) then

!        --- fictitious element group has not been found, warning

            call errint ( ifictgrp, 1 )
            call errwar ( 1085, 1, 0, 0 )
            iinput(iinp+1) = 0
            iinp = iinp+1

         else

!        --- fictitious element group has been filled

            iinput(iinp+1) = iinputlc(iinplc)
            iinplc = iinplc+1
            iinp = iinp+1
            numdegfd = iinputlc(iinplc)
            iinplc = iinplc+1
            iinp = iinp+1
            iinput(iinp) = numdegfd

         end if

      end do

      if ( iinp>maxlen ) then
         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if
      iinput(36) = nfictgrp
1000  call erclos ( 'prordfictunk' )
      end
