      subroutine prordperbcpnt ( table, code, ncodmx, tabmax, tabout, &
                                 numpoints, maxpoints, iinputpnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret records for definition of periodical boundary conditions
!     Part user points
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numpoints, ncodmx(*), maxpoints, iinputpnt(2,maxpoints)
      double precision table(*), tabmax(*), tabout(*)
      character (len=*) code(*)

!     code           i    Table with keywords, see subroutine prordperbc
!     iinputpnt     i/o   Array of size 2 x maxpoints in which the input read
!                         is stored
!                         See subroutine prordperbc for a description
!     maxpoints      i    Size of array iinputpnt
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     numpoints      i    Number of point pairs stored in iinputpnt
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, ifirst, ilast

!     ifirst         First point number read
!     ilast          Last point number read
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     317   record with pelm contains more than two user points
!    1432   User point connected to itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordperbcpnt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperbcpnt'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check if keywords are not mutually exclusive

      if ( table(2)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 2 ) !curves
      if ( table(3)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 3 ) !surfs
      if ( table(4)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 4 ) !excl
      if ( table(5)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 5 ) !all
      if ( table(9)>0d0 ) call errkeywexcl ( code, ncodmx, 1, 9 ) !C
      inum = nint(tabmax(1))  ! number of user points read (must be 2)
      if ( inum/=2 ) then

!     --- Error, not two user points read

         call errint ( inum, 1 )
         call errsub ( 317, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

      numpoints = numpoints+1
      call inichkarlen ( numpoints, maxpoints, 'iinputpnt' ) ! check array len
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(1))  ! position of user points in tabout
      ifirst = nint(tabout(iposut))  ! first user point
      ilast = nint(tabout(iposut+1)) ! last user point
      if ( ilast==ifirst ) then

!     --- error, both user points are identical

         call errint ( ilast, 1 )
         call errsub ( 1432, 1, 0, 0 )
         call errcrd

      end if

!     --- Store points in iinputpnt

      iinputpnt(1,numpoints) = ifirst   ! first point number
      iinputpnt(2,numpoints) = ilast    ! last point number

1000  call erclos ( 'prordperbcpnt' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prordperbcpnt'

      end if  ! ( debug )

      end
