      subroutine prordprintlev ( icode, iprint, codemn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information concerning print_level and store in iprint
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, iprint
      character *(*) codemn(*)

!     codemn         i    Table with main keywords
!     icode          o    Return code, see prorea
!     iprint         o    Print level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordprintlev' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordprintlev'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- PRINT_LEVEL found, read number

      call asknxt

!     --- Check if PRINT_LEVEL is read, if so read number

      if ( ityprd.eq.4 .and. string(1:5).eq.'PRINT' ) call asknxt

      if ( ityprd.ne.2 ) then

!     --- Not an integer found

         call errchr ( codemn(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd

      else

!     --- integer found

         iprint = max(iprint,nint(realrd))
         if ( iprint.gt.2 ) then

!        --- iprint out of range

             call errint ( iprint, 1 )
             call errint ( 0, 2 )
             call errint ( 2, 3 )
             call errchr ( codemn(icode), 1 )
             call errwar ( 1870, 3, 0, 1 )
             call errcrd
             iprint = min(2,iprint)

         end if

      end if
      ityprd = 1
      icode = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prordprintlev'

      end if
      call erclos ( 'prordprintlev' )

      end
