      subroutine prordrlev ( iinput, jinp, iinp, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-08-2011 Extra debug
!        version  1.0  date 13-08-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorganize contents of level set part of iinput for probdf
!     Perform some checks
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), jinp, iinp, iwork(*)

!     iinp          i/o   Last position used in iwork
!     iinput        i/o   integer input array
!     iwork         i/o   integer work array in which iinput is copied
!                         temporarily
!     jinp           i    last position used in array iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlevelset, numread, i, iseq

!     i              Counting variable
!     iseq           Sequence number
!     numlevelset    Number of different level set functions
!     numread        Number of different level set functions read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2667   Level set sequence number out of range
!    2668   Level set sequence number used more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numlevelset := iinput(48)
!     numread := iinput(jinp+1)
!     if ( numlevelset = 0 ) then
!        for i := 1 (1) numread do
!           numlevelset := max(numlevelset,iinput(jinp+2*i))
!     clear array iwork(iinp+1 ,..., iinp+numlevelset)
!     for i := 1 (1) numread do
!        iseq := iinput(jinp+2*i)
!        if ( iwork(iinp+iseq) = 0 ) then
!           iwork(iinp+iseq) := iinput(jinp+2*i+1)
!        else
!           error
!     iinp := iinp+numlevelset
!     iinput(48) := numlevelset
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordrlev' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordrlev'
         write(irefwr,1) 'iinp, jinp', iinp, jinp
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- numlevelset := iinput(48)
!         numread := iinput(jinp+1)

      numlevelset = iinput(48)
      numread = iinput(jinp+1)
      if ( debug ) &
         write(irefwr,1) 'numlevelset, numread', numlevelset, numread

      if ( numlevelset==0 ) then

!     --- numlevelset = 0, compute from separate information
!         for i := 1 (1) numread do
!            numlevelset := max(numlevelset,iinput(jinp+2*i))

         do i = 1, numread
            numlevelset = max(numlevelset,iinput(jinp+2*i))
         end do  ! i = 1, numread

      end if  ! ( numlevelset==0 )

!     --- clear array iwork(iinp+1 ,..., iinp+numlevelset)

      iwork(iinp+1:iinp+numlevelset) = -10

!     --- for i := 1 (1) numread do

      do i = 1, numread

!     --- iseq := iinput(jinp+2*i)

         iseq = iinput(jinp+2*i)
         if ( debug ) write(irefwr,1) 'i, iseq', i, iseq
         if ( iseq>numlevelset ) then

!        --- Error: iseq out of range

            call errint ( iseq, 1 )
            call errint ( numlevelset, 2 )
            call errsub ( 2667, 2, 0, 0 )

         else

!        --- iseq has correct value

            if ( iwork(iinp+iseq)==-10 ) then

!           --- iwork(iinp+iseq) = -10, not yet filled
!               iwork(iinp+iseq) := iinput(jinp+2*i+1)

               iwork(iinp+iseq) = iinput(jinp+2*i+1)

            else

!           --- error: sequence number used more than once

               call errint ( iseq, 1 )
               call errsub ( 2668, 1, 0, 0 )

            end if  ! ( iwork(iinp+iseq)==-10 )

         end if  ! ( iseq>numlevelset ) then

      end do  ! i = 1, numread

!     --- Clear parts of iwork that have not been filled yet

      do i = 1, numlevelset
         if ( iwork(iinp+i)==-10 ) iwork(iinp+i) = 0
      end do  ! i = 1, numlevelset

!     --- iinp := iinp+numlevelset
!         iinput(48) := numlevelset

      iinp = iinp+numlevelset
      iinput(48) = numlevelset

1000  call erclos ( 'prordrlev' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prordrlev'

      end if

      end
