      subroutine prordskipelm ( iinput, maxlen, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-02-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards for elements to be skipped
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp

!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         the elements to be skipped
!     maxlen         i    Maximum amount of space for array iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=6
      integer itype1(nncode1), ncode1(nncode1), ncodmx1(nncode1)
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      integer inum, ifirst, ilast, i, jinp, numpart
      character (len=19) :: code1(nncode1)

!     code1          Table with keywords
!     i              Counting variable
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     inum           Number of numbers read in a record
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     jinp           last position used in array iinput with respect to
!                    iprec
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     numpart        Number of parts read
!     tabmax1        Contains maximum values allowed in table for code1
!     tabmin1        Contains minimum values allowed in table for code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!       inner_obstacle i
!       on_obstacle i and curves (cj to ck)
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     334   record with curves contains wrong number of curve numbers
!    2220   No curves keyword found
!    2221   Not an obligatory keyword found
!    2222   No data reords found
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Read all input until main keyword is found, store in iinputlc
!    Copy all input in the correct sequence into iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'INNER_OBSTACLE', 'ON_OBSTACLE', 'CURVES', &
                   'C', 'AND', 'TO' /
      data ncode1  /10,  7,  4,  1,  3,    2 /
      data ncodmx1 /14, 11,  6,  1,  3,    2 /
      data itype1  / 0,  0,  2, 14, -1,   -1 /
      data tabmin1 / 1d0, 1d0, 0d0, 2d0, 0d0,   0d0 /
      data tabmax1 / 1d3, 1d3, 0d0, 0d0, 0d0,   0d0 /

!     code1    Table of local keywords
!              1: INNER_OBSTACLE
!              2: ON_OBSTACLE
!              3: CURVES
!              4: C
!              5: AND
!              6: TO
! ======================================================================
!
      call eropen ( 'prordskipelm' )

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      nmain = nncodemain
      iretur = icode

      ityprd = 1

!     --- Read records for the skipping of elements

      numpart = 0
      jinp = iinp+1
      iinp = jinp
300   table(1:nncode) = -1d20
      tabmax(1) = numobst
      tabmax(2) = numobst
      tabmax(4) = ncurvs
      table(4) = 0d0
      call sepreadsetkeyw
      icode = iretur
      if ( icode==0 ) then

!     --- Not a main keyword found, hence inormation

         numpart = numpart+1
         if ( table(1)>0 ) then

!        --- INNER_OBSTACLE found

            iinput(iinp+1) = 1
            iinput(iinp+2) = nint(table(1))
            iinput(iinp+3) = 0
            iinput(iinp+4) = 0
            iinput(iinp+5) = 0
            iinp = iinp+5

         else if ( table(2)>0 ) then

!        --- ON_OBSTACLE found

            if ( table(3)<0d0 ) then

!           --- No curves found

               call errsub ( 2220, 0, 0, 0 )
               call errcrd
               ifirst = 1
               ilast = 1

            else

!           --- Curves found

               inum = nint(tabmax(4))
               if ( inum==0 ) then

                  call errint ( inum, 1 )
                  call errint ( 2, 2 )
                  call errsub ( 334, 2, 0, 0 )
                  call errcrd

               else

                  iposut = nint(table(4))
                  ifirst = nint(tabout(iposut))
                  if ( inum>1 ) then
                     ilast = nint(tabout(iposut+1))
                  else
                     ilast = ifirst
                  end if

               end if
            end if
            iinput(iinp+1) = 2
            iinput(iinp+2) = nint(table(2))
            iinput(iinp+3) = ifirst
            iinput(iinp+4) = ilast
            iinput(iinp+5) = 0
            iinp = iinp+5

         else

!        --- None of both options found

            call errchr ( codemain(16)(1:13), 1 )
            do i = 1, 2
               call errchr ( code1(i)(1:ncodmx1(i)), i+1 )
            end do
            call errsub ( 2221, 0, 0, 3 )
            call errcrd

         end if
         go to 300

      end if

      if ( iinp>=maxlen ) then

!     --- iinp > maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if

      if ( numpart==0 ) then

!     --- iinp > maxlen, too few space in IINPUT

         call errsub ( 2222, 0, 0, 0 )
         call errcrd

      end if
      iinput(jinp) = numpart
      iinput(44) = 1

      call erclos ( 'prordskipelm' )

      end
