      subroutine sepbounlevset2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobstfaces with edges corresponding to the boundary
!     of the region with elements with levelset>0
!     Old subroutine prbounlevset
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     edge
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenbndedges
      integer, allocatable :: bndedges(:)

!     bndedges       contains edge sequence numbers of boundary
!                    of levelset
!     lenbndedges    Computed length of array iobstfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPBOUNLEVSET1 Fill array bndedges with edges corresponding to the
!                    boundary of the region with elements with levelset>0
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINKMESHC  print nmesh part c
!     SEPPRINNMESHD  print nmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbounlevset2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbounlevset2'
         write(irefwr,1) 'nlevelsets, numobst', nlevelsets, numobst
         call sepprinkmeshc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nlevelsets==0 ) nlevelsets = numobst  ! number of obstacles
      if ( debug ) write(irefwr,1) 'nlevelsets', nlevelsets

      if ( nlevelsets==0 ) go to 1000  ! levelset not yet created

!     --- create edges

      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( debug ) write(irefwr,1) 'nedges', nedges

!     --- allocate temporary space to store obstfaces

      lenbndedges = nlevelsets*2+nedges
      allocate ( bndedges(lenbndedges), stat = error )
      if ( error/=0 ) call eralloc ( error, lenbndedges, 'bndedges' )
      if ( ierror/=0 ) go to 1000

!     --- actual call

      if ( debug ) then

!     --- Debug information

         call sepprinielobst ( ielemobst )
         call sepprinnmeshd

      end if  ! ( debug )

      call sepbounlevset1 ( bndedges, lenbndedges )

      if ( associated(km(iseqkm)%obstfaces) ) &
         deallocate(km(iseqkm)%obstfaces)
      allocate ( km(iseqkm)%obstfaces(lenbndedges) )
      obstfaces =>  km(iseqkm)%obstfaces
      obstfaces = bndedges(1:lenbndedges)
      km(iseqkm)%kmeshad(9) = lenbndedges

      if ( debug ) call prinin ( bndedges, lenbndedges, 'bndedges' )

      deallocate ( bndedges, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'bndedges' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbounlevset2'

      end if  ! ( debug )
      call erclos ( 'sepbounlevset2' )

      end
