      subroutine sepedg02 ( mnoped )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the edges part 2 actual edge labels and lmeshc
!     Old subroutine topedg02
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: mnoped

!     mnoped     i    maximum number of nodes per edge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ielem, i, ipd, ipc, &
              irelem, jedge, inpelm, iedge, ikelmc
      integer :: ihelp(mnoped)

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     ihelp          integer help array of length mnoped
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     ipc            Starting address in array nmeshc
!     ipd            pointer in kmeshaad
!     irelem         Relative element sequence number with respect to
!                    element group
!     jedge          edge sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPEDG04       Find elements connected to an edge and store in nmeshd
!     SEPPRINNMESHD  print nmesh part d
!     TOPEDG03       Get local node numbers of an edge
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen  ('sepedg02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepedg02'
         call prinin ( nmeshd, nelem, 'nmeshd' )
         write(irefwr,1) 'nelgrp', nelgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem = 0
      nedges = 0
      ikelmc = 1
      nspec = nmesh(26)

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!           and number of elements in this group (inelem)
!           and number of edges per element in this group (nedpel)
!           and number of nodes per element in his group

         ishape = sepgetshape ( ielgrp )
         inelem = Topgrp(ielgrp)%inelem
         nedpel = sepgetnedpel(ielgrp)
         if ( nedpel==-1 ) nedpel = 0
         inpelm = Topgrp(ielgrp)%inpelm

         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem, inpelm', &
                             ielgrp, ishape, inelem, inpelm

!        --- Loop over the elements in this group

         do irelem = 1, inelem
            ielem = ielem + 1
            ipd = nmeshd(ielem)

            if ( debug ) &
               write(irefwr,1) 'irelem, ielem, ipd, nedpel', &
                                irelem, ielem, ipd, nedpel

!           --- Loop over the edges

            do jedge = 1, nedpel
               if ( nmeshd(ipd+jedge)==0 ) then

!              --- New edge found

                  nedges = nedges + 1
                  nmeshc(nedges) = 0

!                 --- Get the local nodes of this edge and store in ihelp

                  call topedg03 ( jedge, ishape, nndped, ihelp, nspec )

!                 --- Store this edge in ihelp for a while

                  do i = 1, nndped
                     ihelp(i) = kmeshc(ikelmc+ihelp(i)-1)
                  end do

                  if ( debug ) call prinin ( ihelp, nndped, 'ihelp' )

!                 --- Find all elements with this edge and fill nmeshd part 2

                  call sepedg04 ( ihelp, nedges )

               end if
            end do  ! jedge = 1, nedpel

!           --- update pointer in kmesh part c

            ikelmc = ikelmc + inpelm
         end do
      end do

      if (debug ) call sepprinnmeshd

!     --- Redo this but now fill nmeshc

      ikelmc = 1
      ipc = nedges + 1
      ielem = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!           and number of elements in this group (inelem)
!           and number of edges per element in this group (nedpel)
!           and number of nodes per element in his group

         ishape = sepgetshape ( ielgrp )
         inelem = Topgrp(ielgrp)%inelem
         nedpel = sepgetnedpel(ielgrp)
         if ( nedpel==-1 ) nedpel = 0
         inpelm = Topgrp(ielgrp)%inpelm

!        --- Loop over the elements in this group

         do irelem = 1, inelem
            ielem = ielem + 1
            ipd = nmeshd(ielem)

!           --- Loop over the edges

            do jedge = 1, nedpel

!           --- Get the label of this edge from nmeshd

               iedge = abs(nmeshd(ipd+jedge))
               if ( nmeshc(iedge)==0 ) then

!              --- New edge found: get the local nodes of this edge and store
!                  in ihelp

                  call topedg03 ( jedge, ishape, nndped, ihelp, nspec )

!                 --- Store this edge in nmeshc

                  nmeshc(iedge) = ipc
                  nmeshc(ipc) = nndped
                  do i = 1, nndped
                     nmeshc(ipc+i) = kmeshc(ikelmc+ihelp(i)-1)
                  end do
                  ipc = ipc + nndped + 1
               end if
            end do

!           --- update pointer in kmesh part c

            ikelmc = ikelmc + inpelm
         end do
      end do

      length = ipc - 1
1000  if ( debug ) then

!     --- Debug information

         call prinin ( nmeshd, nelem+10, 'nmeshd' )
         write(irefwr,*) 'End sepedg02'

      end if  ! ( debug )
      call erclos ( 'sepedg02' )

      end subroutine sepedg02
