      subroutine sepfa02 ( medpfa )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the faces part 2 actual face labels and lmeshb
!     Old subroutine topfa02
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: medpfa

!     medpfa     i    maximum number of edges per face
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ipa, ipb, irelem, ielem, &
              nedpfarr(6), i, ipd, jface, iface, j, k, &
              locedges(4,6), ihelp(2*medpfa)
      logical extradebug

!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iface          global face number
!     ihelp          integer work array of length 2*medpfa
!     ipa            pointer in kmeshaaa
!     ipb            pointer in kmeshaab
!     ipd            pointer in kmeshaad
!     irelem         Relative element sequence number with respect
!                    to element group
!     j              Counting variable
!     jface          local face number in element
!     k              Counting variable
!     locedges       Array of size 4 x 6 containing the local edge
!                    numbers of each face of a volume element
!     nedpfarr       number of edges per face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     SEPFA04        Fill nmesh part a
!     TOPFA03        Determine local edge numbers for local face number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfa02' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfa02'
         write(irefwr,1) 'nelem, npoint', nelem, npoint
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ielem = 0
      nfaces = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!         and number of elements in this group (inelem)
!         and number of nodes per element in this group

         ishape = kmeshf ( ielgrp )
         inelem = Topgrp(ielgrp)%inelem
         nfapel = sepgetnfapel(ielgrp)
         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem, nfapel' , &
                             ielgrp, ishape, inelem, nfapel

!        --- Get the local edges of this face and store in locedges

         do jface = 1, nfapel
            call topfa03 ( jface, ishape, nedpfarr(jface), &
                           locedges(1,jface) )
         end do
         if ( ierror/=0 ) go to 1000
         if ( extradebug ) then

!        --- Debug information

            call prinin ( nedpfarr, nfapel, 'number of edges per face' )
            do jface = 1, nfapel
               write(irefwr,*) 'local face ', jface
               call prinin ( locedges(1,jface), nedpfarr(jface), &
                             'local edges' )
            end do

         end if

!        --- Loop over the elements in this group

         do irelem = 1, inelem
            ielem = ielem + 1
            ipa = nmesha(ielem)
            if ( debug ) &
               write(irefwr,1) 'irelem, ielem, ipa', &
                                irelem, ielem, ipa

!           --- Loop over the faces

            do jface = 1, nfapel
               if ( nmesha(ipa+jface)==0 ) then

!              --- New face found

                  nfaces = nfaces + 1
                  nmeshb(nfaces) = 0

!                 --- Store this face in ihelp for a while

                  ipd = nmeshd(ielem)

                  do i = 1, nedpfarr(jface)
                     j = abs(locedges(i,jface))
                     if ( locedges(i,jface)>0 ) then
                        ihelp(i) = nmeshd(ipd+j)
                     else
                        ihelp(i) = -nmeshd(ipd+j)
                     end if
                  end do

                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'nfaces, jface ', nfaces, jface
                     call prinin ( ihelp, nedpfarr(jface), 'ihelp' )

                  end if

!                 --- Find all elements with this face and fill nmesha part 2

                  nedpfa = nedpfarr(jface)
                  call sepfa04 ( ihelp, nfaces, ihelp(medpfa+1) )

               end if

            end do

         end do
      end do
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nfaces', nfaces
         if ( extradebug ) call prinnmeshx ( nelem, nmesha )

      end if

!     --- Redo this but now fill nmeshb

      ipb = nfaces + 1
      ielem = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!           and number of elements in this group (inelem)
!           and number of nodes per element in his group

         ishape = kmeshf ( ielgrp )
         inelem = Topgrp(ielgrp)%inelem
         nfapel = sepgetnfapel(ielgrp)

!        --- Get the local edges of this face and store in locedges

         do jface = 1, nfapel
            call topfa03 ( jface, ishape, nedpfarr(jface), &
                           locedges(1,jface) )
         end do

!        --- Loop over the elements in this group

         do irelem = 1, inelem
            ielem = ielem + 1
            ipa = nmesha(ielem)
            ipd = nmeshd(ielem)

!           --- Loop over the faces

            do jface = 1, nfapel

!           --- Get the label of this face from nmesha

               iface = abs(nmesha(ipa+jface))
               if ( nmeshb(iface)==0 ) then

!              --- New face found
!                  Store this face in nmeshb

                  nmeshb(iface) = ipb
                  nmeshb(ipb) = nedpfarr(jface)
                  do i = 1, nedpfarr(jface)
                     j = abs(locedges(i,jface))
                     k = locedges(i,jface)/j
                     nmeshb(ipb+i) = k*nmeshd(ipd+j)
                  end do
                  ipb = ipb + nedpfarr(jface) + 1
               end if
            end do

         end do
      end do

      length = ipb - 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nfaces, length ', nfaces, length
         write(irefwr,*) 'End sepfa02'

      end if
      call erclos ( 'sepfa02' )
      end
