      subroutine sepfillkmeshe2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute nodal points connected to nodal points
!     Old subroutine topnp
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, isum, ikelm, ih, neli, i1, &
              j, ielem, ikelm2, k, nodp, ih1, maxim
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     i1
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ikelm
!     ikelm2
!     inpelm         number of points per element
!     isum           Integer variable to compute a sum
!     iwork          integer work array  see toplm1 and toplm2
!     j              Counting variable
!     k              Counting variable
!     maxim          Computed length of kmshe2
!     neli           number of elements coupled to a node
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1381   Too few space available in array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshe2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshe2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxim = kmshe1(npoint+1)

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- iwork will be cleared

      iwork=0

!     --- fill kmesh parts e1 and e

      isum=0
      ikelm=0
      do i=1,npoint
         kmshe1(i)=isum
         ih=kmshd1(i+1)
         neli=ih-ikelm
         i1=isum
         iwork(i)=2
         do j=1,neli
            ielem=kmshd2(ikelm+j)
            ikelm2=kmeshcpoint(ielem)
            inpelm=kmeshcpoint(ielem+1)-ikelm2
            do k=1,inpelm
               nodp=kmeshc(ikelm2+k)
               if(iwork(nodp)==0) then

!              --- new nodal point

                  iwork(nodp)=1
                  isum=isum+1
                  kmshe2(isum)=nodp
               end if
            end do
         end do

!        --- positions in iwork with iwork(i)>0 will be cleared

         iwork(i)=0
         ih1=i1+1
         iwork(kmshe2(ih1:isum))=0
         ikelm=ih
         if(isum>maxim) then
            call errint ( isum, 1 )
            call errint ( maxim, 2 )
            call errsub ( 1381, 2, 0, 0 )
            go to 1000
         end if
      end do
      kmshe1(npoint+1)=isum
      maxim=isum

      deallocate ( iwork, kmeshcpoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmeshe2'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshe2' )

      end
