      subroutine sepfillkprobhr ( iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of kprobh corresponding to free unknowns in case
!     of renumbering of nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     renumbering
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iwork(nusol)

!     iwork          i    iwork(i)>0, means essential boundary condition for
!                                     unknown i
!                         iwork(i)=0, means unknown i is free
!                         iwork(i)<0, means unknown i corresponds to
!                         periodical bc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, icount, inodp, inusol, inunkp

!     i              Counting variable
!     icount         Counter
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkprobhr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkprobhr'
         call ertrace
         write(irefwr,1) 'indprf', indprf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      icount = 1
      do i = 1, npoint
         inodp = kmeshj(i)
         if ( indprf==0 ) then

!        --- indprf  =  0

            inusol = (inodp-1)*nunkp

         else

!        --- indprf>0

            inusol = kprobf(inodp)
            inunkp = kprobf(inodp+1)-inusol

         end if  ! ( indprf==0 )

         do j = 1, inunkp
            if ( iwork(j+inusol)==0 ) then

!           --- iwork (j+inusol)  =  0

               kprobh(j+inusol) = icount
               icount = icount+1
            end if
         end do  ! j = 1, inunkp

      end do  ! i = 1, npoint

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkprobhr'

      end if  ! ( debug )
      call erclos ( 'sepfillkprobhr' )

      end subroutine sepfillkprobhr
