      subroutine sepfillmatr ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters in module sepmodulematr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenim2, leng1, icompl
      logical :: compl
      type (intmatrix), pointer :: IM
      type (globmat), pointer :: S

!     IM             Refers to intmt(intmat)
!     S              pointer to realmt(iseqmt)
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!                    they are not
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     leng1          Length of local co-ordinates array
!     lenim2         length of the array imat part 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLMATSIMP Fill pointers in sepmodulematr for subroutine build
!                    Create relation between arrays and buffer
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillmatr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillmatr'
         write(irefwr,1) 'matr', matr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqmt = matr

      S => realmt(iseqmt)
      intmat = S%intmatref
      compl = S%complexmat
      call sepfillmatstr
      IM => intmt(intmat)

      icompl = 1
      if ( compl ) icompl = 2
      S_ii  => S%S_ii
      length = size(S_ii)

      if ( ninterface>0 ) then

!     --- There are interface unknowns

         S_if => S%S_if
         S_fi => S%S_fi

      end if  ! ( ninterface>0 )

      S_iout => S%S_iout

      symm = IM%symmetric

      if ( IM%simple ) then

!     --- simple

         call sepfillmatsimp(S)

      end if  ! ( IM%simple )

      Dlapshift => S%Dlapshift

      if ( associated(S%S_ip) ) then

         S_ip => S%S_ip
         S_pi => S%S_pi
         S_pp => S%S_pp

      end if  ! ( matr>0 )
      if ( npboun>0 ) S_ic => S%S_ic

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillmatr'

      end if  ! ( debug )
      call erclos ( 'sepfillmatr' )

      end subroutine sepfillmatr
