      subroutine sepflpointobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize array kmeshad if it does not exist
!     Fill array ipointobst if necessary
!     Old subroutines iniactm0/topfillkmad1
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     topology
!     mesh
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: exist

!     exist          if true kmesh ad exists
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFLPOINTOBS2 Fill array ipointobst (2d)
!     SEPFLPOINTOBS3 Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPINITKMESHAD Initialize kmesh ad
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflpointobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflpointobst'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 .or. ndim==1 ) go to 1000

      if ( km(iseqkm)%kmesh(48)==0 ) then

!     --- kelmaa = 0, Initialize nmesha

         call sepfillkmeshaa

      else if ( km(iseqkm)%nmesh(19)==0 ) then

!     --- nmesh(19) = 0, Initialize nmesha

         call sepfillkmeshaa

      end if  ! ( kelmaa==0 )

      exist = .false.
      if ( debug ) write(irefwr,1) 'kelmad', kelmad
      if ( km(iseqkm)%kmesh(52)>0 ) then

!     --- Array kmesh ad already exists: check if sequence number is
!         correct

         kmeshad => km(iseqkm)%kmeshad
         if ( kmeshad(1)>0 .and. kmeshad(5)==kmeshad(6) ) exist = .true.
         if ( debug ) call prinin ( kmeshad, 6, 'kmeshad' )

      else

!     --- kmesh ad does not exist. Create

         call sepinitkmeshad

      end if  ! ( kelmad>0 )

      if ( debug ) write(irefwr,*) 'exist ', exist

      if ( .not. exist ) then

!     --- Array kmesh ad does not exist, create

         if ( debug ) write(irefwr,1) 'ndim', ndim
         if ( ndim==2 ) then

!        --- ndim = 2

            call sepflpointobs2

         else

!        --- ndim = 3

            call sepflpointobs3

         end if  ! ( ndim==2 )

      end if  ! ( .not. exist )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflpointobst'

      end if  ! ( debug )
      call erclos ( 'sepflpointobst' )

      end subroutine sepflpointobst
