      subroutine sepintmatdg
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform symmetric compact storage scheme to row-compact storage
!     Special application DG-matrix for simple
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: rows(:), cols(:), irowsnew(:), icolsnew(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     cols           array with column npmbers
!     icolsnew       temporary array icolsii
!     irowsnew       temporary array irowsii
!     rows           Refers to array irowsii
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOMMATDIADG Store position of diagonal elements of array cols into
!                    idiag in case of row-compact storage
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPINTMAT1     Transform symmetric compact storage scheme to row-compact
!                    storage
!     SEPINTMAT2     Fill data structure in case of symmetric compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmatdg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmatdg'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      allocate ( cols(size(IM%colsDG)), rows(np+1) )

!     --- Copy colsDG in cols and rowsDG in rows

      cols = IM%colsDG
      rows = IM%rowsDG

      deallocate ( IM%colsDG )

      if ( IM%Gsymmetric ) then

!     --- DG is symmetric

         allocate ( IM%colsDG(size(cols)+np), irowsnew(np+1), &
                    icolsnew(size(cols)+np) )

!        --- Fill arrays rowsDG and colsDG

         call sepintmat2 ( np, irowsnew, icolsnew, size(IM%colsDG), &
                           rows, cols, size(icolsnew), IM%diagDG )
         call sepcommatrev ( IM%rowsDG, IM%colsDG, irowsnew, icolsnew, np )
         IM%diagDG = IM%rowsDG(1:np)+1
         deallocate ( irowsnew, icolsnew )

      else

!     --- DG is unsymmetric

         allocate ( IM%colsDG(2*size(cols)+np) )

!        --- Fill arrays rowsDG and colsDG

         call sepintmat1 ( np, IM%rowsDG, IM%colsDG, size(IM%colsDG), &
                           rows, cols, size(cols) )
         call sepcommatdiadg

      end if  ! ( IM%Gsymmetric )

      deallocate ( cols, rows )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( IM%rowsDG, np+1, 'rowsDG' )
         call prinin ( IM%colsDG, IM%rowsDG(np+1), 'colsDG' )
         write(irefwr,*) 'End sepintmatdg'

      end if  ! ( debug )
      call erclos ( 'sepintmatdg' )

      end subroutine sepintmatdg
