      subroutine seplevflielem ( iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part ad in case of levelset functions
!     Part ielemobst
!     Old subroutine mshlevflielem
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     node
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqlc

!     iseqlc         i    Sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenielemobst

!     lenielemobst   Computed length of array ielemobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPOBSTMARKEL  Mark all elements in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevflielem' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevflielem'
         write(irefwr,1) 'iseqlc', iseqlc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Mark all elements to be in/out or partly in obstacle
!         Allocate temporary space for array ielemobst

      if ( associated(ielemobst) ) nullify(ielemobst)
      allocate ( ielemobst(6+nelem), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 6+nelem, 'ielemobst' )
      if ( ierror/=0 ) go to 1000

!     --- First the elements are created from the nodes only

      call sepobstmarkel ( lenielemobst )

      if ( debug ) then
         write(irefwr,*) 'ipointobst after mshobstmarkel'
         call sepprinipobst ( ipointobst )
         call sepprinielobst ( ielemobst )
      end if

!     --- Make array ielemobst permanent in ibuffr

      length = lenielemobst
      if ( associated(km(iseqkm)%ielemobst) ) &
         deallocate(km(iseqkm)%ielemobst)
      allocate ( km(iseqkm)%ielemobst(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'ielemobst' )
      if ( ierror/=0 ) go to 1000

      km(iseqkm)%ielemobst = ielemobst(1:length)
      deallocate ( ielemobst )

      ielemobst => km(iseqkm)%ielemobst
      kmeshad(2) = length

      kmeshad(7) = iseqlc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplevflielem'

      end if  ! ( debug )
      call erclos ( 'seplevflielem' )

      end
