      function sepmindeg ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find node with minimum degree in the set lastlevel
!     Old subroutine chmindeg
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepmindeg

!     sepmindeg      o    node number with minimum degree
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, minlev, i, j, lev

!     i              Counting variable
!     istart         Present start node
!     j              Counting variable
!     lev            degree of node
!     minlev         present minimum degree
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmindeg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmindeg'
         write(irefwr,1) 'nset', size(lastlevel)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- start with first node

      istart = lastlevel(1)
      minlev = kmshe1(istart+1)-kmshe1(istart)  ! degree of node

      if ( debug ) write(irefwr,1) 'istart, minlev', istart, minlev

!     --- For all other nodes

      do i = 2, size(lastlevel)

         j = lastlevel(i)                 ! new node
         lev = kmshe1(j+1)-kmshe1(j)  ! degree of new node
         if ( lev<minlev ) then

!        --- New node has a smaller degree than old one, make it start value

            istart = j
            minlev = lev

            if ( debug ) write(irefwr,1) 'i, j, lev', i, j, lev

         end if  ! ( lev<minlev )

      end do  ! i = 2, size(lastlevel)

      sepmindeg = istart  ! final result

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, minlev', istart, minlev
         write(irefwr,*) 'End sepmindeg'

      end if  ! ( debug )
      call erclos ( 'sepmindeg' )

      end function sepmindeg
