      subroutine sepnextlevel ( prevlevel, numlevnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    find neigbors corresponding to the set lastlevel
!    nodal points in marknodes with values<prevlevel are neglected
!    new nodal points get the values prevlevel in array marknodes
!    the new neigbors are stored in array nextlevel of length numlevnodes
!    Old subroutine chtmc5
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: prevlevel
      integer, intent(out) :: numlevnodes

!     numlevnodes    o    number of nodal points in nextlevel
!     prevlevel      i    Integer threshold value
!                         If marknodes(i)>prevlevel, node i is free
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, inodp, nodp, istart, numneigh

!     i              Counting variable
!     inodp          Nodal point number
!     istart         First position used at a part
!     k              Counting variable
!     nodp           Nodal point sequence number
!     numneigh       number of neighbors of a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    numlevnodes := 0
!    For i := 1 (1) number_of_points in lastlevel do
!       nodp := lastlevel(i)
!       numneigh := number of neighbors of node nodp
!       For k := 1 (1) numneigh do
!          nodp := neighbor of nodp
!          if ( marknodes(nodp)>prevlevel ) then
!             marknodes(nodp) := prevlevel
!             numlevnodes := numlevnodes + 1
!             nextlevel(numlevnodes) := nodp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnextlevel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnextlevel'
         write(irefwr,1) 'numprevnodes', size(lastlevel)
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numlevnodes=0
      do i = 1, size(lastlevel)
         inodp = lastlevel(i)
         istart = kmshe1(inodp)
         numneigh = kmshe1(inodp+1)-istart
         if ( debug ) write(irefwr,1) 'i, nodp, numneigh', i, nodp, numneigh
         do k = 1, numneigh
            nodp = kmshe2(istart+k)
            if ( marknodes(nodp)>prevlevel ) then
               numlevnodes = numlevnodes+1
               nextlevel(numlevnodes) = nodp
               marknodes(nodp) = prevlevel
            end if
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnextlevel'
      end if  ! ( debug )
      call erclos ( 'sepnextlevel' )
      end
