      subroutine seppointkmshc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill starting addresses of each element in kmeshc into kmeshcpoint
!     Old subroutine toppointkmeshc
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelmc, isum, ielem, inpelm, iielem, ielgrp

!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     isum           Integer variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppointkmshc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppointkmshc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ikelmc=0
      isum=ikelmc
      kmeshcpoint(1)=isum
      ielem=1
      do ielgrp = 1, nelgrp
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         do iielem=1,inelem
            ikelmc=ikelmc+inpelm
            kmeshcpoint(ielem+iielem)=ikelmc
         end do
         ielem=ielem+inelem
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppointkmshc'

      end if  ! ( debug )
      call erclos ( 'seppointkmshc' )

      end
