      subroutine sepprinposdegf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the numbering of nodes corresponding to degrees of freedom
!     for one problem only
!     Old subroutine prinposdegfd
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     problem_definition
!     node
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPRB         Get value of ipkpr as function of iprob
!     PRINPOSUNKNOWN print position of unknowns with respect to nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get general information from kprob
!     Activate arrays
!     Reserve space for work array
!     Compute starting addresses
!     Do the actual printing
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinposdegf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinposdegf'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get general information from kprob

      indprf = kprobloc(19)
      indprh = kprobloc(50)
      indprp = kprobloc(35)

!     --- Compute starting addresses

      nusol = kprobloc(5)
      if ( indprf>0 ) kprobf => kprobpart%kprobf
      if ( indprh>0 ) kprobh => kprobpart%kprobh
      if ( indprp>0 ) kprobp => kprobpart%kprobp

!     --- Do the actual printing

      call prinposunknown ( npoint )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinposdegf'

      end if
      call erclos ( 'sepprinposdegf' )

      end
