      subroutine sepskipelms ( iskip, iskpar, ielems, &
                               ielemlow, ielemtop, nskipelms )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ielems with a 0 for elements that must not be skipped
!     and with a 1 for those who must
!     Old subroutine erskipelms
! **********************************************************************
!
!                       KEYWORDS
!
!     level
!     reordering
!     distance
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iskip, iskpar(*), ielems(*), &
                 ielemlow, ielemtop, nskipelms

!     ielemlow       o    Smallest element number in group
!     ielems         o    Array of length nelem indicating if an element
!                         is still free (0), must be skipped (-1) or is alweady
!                         in use (1)
!     ielemtop       o    Largest element number in group
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskpar         i    array of length NELGRP that is only used if ISKIP > 0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     nskipelms      o    Number of elements that must be skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielgrp, ivolnr

!     ielem          Element number
!     ielgrp         element group number
!     ivolnr         Volume sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array ielems
!    Define range of elements ielemlow to ielemtop from iskip and iskpar
!    Set ielems(i) = -1 for all elements not in ielemlow to ielemtop
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepskipelms' )

!     --- Clear array ielems

      do ielem = 1, nelem
         ielems(ielem) = 0
      end do

!     --- Define range of elements ielemlow to ielemtop from iskip and iskpar
!         Set ielems(i) = -1 for all elements not in ielemlow to ielemtop

      ielemlow = 1
      nskipelms = 0
      if ( iskip>0 ) then

!     --- iskip > 0, skip element groups

         do ielgrp = 1, nelgrp

            inelem = Topgrp(ielgrp)%inelem
            ielemtop = ielemlow+inelem-1
            if ( iskpar(ielgrp)==0 ) then

!           --- Skip this group

               do ielem = ielemlow, ielemtop
                  ielems(ielem) = -1
               end do
               nskipelms = nskipelms+inelem

            end if
            ielemlow = ielemtop+1

         end do

      else if ( iskip<0 ) then

!     --- iskip < 0, skip volumes

         do ivolnr = 1, nvolms
            if ( iskpar(ivolnr)==0 ) then

!           --- Skip this set of volumes

               ielemlow = kmesht(1,ivolnr)
               ielemtop = kmesht(2,ivolnr)
               do ielem = ielemlow, ielemtop
                  ielems(ielem) = -1
               end do
               nskipelms = nskipelms+ielemtop-ielemlow+1

            end if

         end do

      end if

      call erclos ( 'sepskipelms' )
      end
