      subroutine topbv1kprobr ( iwork, factors, jumps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read standard array rinput
!     Fill arrays kprob parts r and s
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     renumbering
!     boundary_condition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iwork(*)
      double precision, intent(in) :: factors(*), jumps(*)

!     factors        i    Array containing the factors for periodical boundary
!                         conditions
!     iwork          i    Work array of length nusol in which the information
!                         about the essential boundary conditions is stored
!                         if iwork(i) = 0, then unknown i is not prescribed
!                         if iwork(i) = 1, then unknown i is prescribed
!     jumps          i    Array of size nusol containing the jumps
!                         for the periodical boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!                    management index mmindex and copy the contents of
!                    iarray to ibuffr
!     TOPBV6         Fill array KPROBH with references of unknowns to array USOL
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv1kprobr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv1kprobr'
         write(irefwr,1) 'npboun', npboun
         write(irefwr,*) 'kprobr', &
         associated(kp(iseqkp)%kprobsub(iprob)%kprobr), &
         size(kp(iseqkp)%kprobsub(iprob)%kprobr)
         write(irefwr,*) 'kprobs', &
         associated(kp(iseqkp)%kprobsub(iprob)%kprobs), &
         size(kp(iseqkp)%kprobsub(iprob)%kprobs)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npboun = kprobloc(34)
      if ( kprobloc(37)>0 ) &
         deallocate(kp(iseqkp)%kprobsub(iprob)%kprobr)
      allocate(kp(iseqkp)%kprobsub(iprob)%kprobr(nusol)  )
      kprobr => kp(iseqkp)%kprobsub(iprob)%kprobr
      kprobloc(37) = nusol

      if ( associated(kprobpart%kprobs) ) &
         deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobs, &
                      kp(iseqkp)%kprobsub(iprob)%rprobs )
      allocate ( kp(iseqkp)%kprobsub(iprob)%kprobs(npboun), &
                 kp(iseqkp)%kprobsub(iprob)%rprobs(npboun,2) )
      kprobs => kprobpart%kprobs
      rprobs => kprobpart%rprobs
      kprobloc(38) = npboun

      call topbv6 ( iwork, factors, jumps )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv1kprobr'

      end if  ! ( debug )
      call erclos ( 'topbv1kprobr' )

      end subroutine topbv1kprobr
