      subroutine topbv5012 ( nusol, istoreleft, istoreright )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Accumulate arrays istoreleft and istoreright
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol, istoreleft(*), istoreright(*)

!     istoreleft    i/o   Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     istoreright   i/o   Contains all periodical left neighbor unknowns
!                         for all right periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbors
!     nusol          i    Total number of unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inusol1, inusol2, inunkp

!     inunkp         Number of degrees of freedom in point
!     inusol1        Position of degree of freedom in solution vector
!     inusol2        Position of degree of freedom in solution vector
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv5012' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv5012'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inusol1 = nusol+1
      inusol2 = nusol+1
      do j = 1, nusol
         inunkp = istoreleft(j)
         istoreleft(j) = inusol1
         inusol1 = inusol1+inunkp
         inunkp = istoreright(j)
         istoreright(j) = inusol2
         inusol2 = inusol2+inunkp
      end do
      istoreleft(nusol+1) = inusol1
      istoreright(nusol+1) = inusol2

1000  call erclos ( 'topbv5012' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv5012'

      end if  ! ( debug )

      end
