      subroutine topbv5loc ( idegfd, rjump, rmult, idegper, rjumploc, &
                             factorloc, nunkp, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-06-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update the arrays idegper, rjumploc and factorloc with information
!     about jump and factors as defined by the bit pattern in idegfd
! **********************************************************************
!
!                       KEYWORDS
!
!     bit_pattern
!     factor
!     jump
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idegfd, idegper(*), nunkp, ielgrp
      double precision rjump, rjumploc(*), factorloc(*), rmult

!     factorloc     i/o   Array of size nunkp containing the factors for
!                         each degree of freedom
!     idegfd         i    Bit pattern indicating degrees of freedom
!     idegper       i/o   Array of size nunkp containing a 1 if the
!                         corresponding unknown is periodical and a 0 if not
!     ielgrp         i    element group number
!     nunkp          i    Maximum number of unknowns in a point
!     rjump          i    Value of jump
!     rjumploc      i/o   Array of size nunkp containing the jumps for each
!                         degree of freedom
!     rmult          i    Multiplication factor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(nunkp), ibit

!     ibit           Actual bit
!     ihelp          integer work array of size nunkp to store the bits
!                    of idegfd
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIBITEXT      Extract the information stored in the bit pattern ibit, &
!                    and put that information in an array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2344   Degree of freedom used more than once for periodical b.c.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     extract the bit pattern stored in idegfd and store in ihelp
!     For all bits ibit do
!        if ( idegper(ibit) = 0 ) then
!           idegper(ibit) := 1; rjumploc(ibit) := rjump
!           factorloc(ibit) := rmult
!        else
!           error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv5loc' )

!     --- extract the bit pattern stored in idegfd and store in ihelp

      call inibitext ( idegfd, ihelp, nunkp )

!     --- For all bits ibit do

      do ibit = 1, nunkp

         if ( ihelp(ibit).eq.1 ) then

!        --- bit found

            if ( idegper(ibit).eq.0 ) then

!           --- idegper(ibit) = 0
!               idegper(ibit) := 1; rjumploc(ibit) := rjump
!               factorloc(ibit) := rmult

               idegper(ibit) = 1
               rjumploc(ibit) = rjump
               factorloc(ibit) = rmult

            else

!           --- error: bit already in use

               call errint ( ibit, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 2344, 2, 0, 0 )

            end if

         end if

      end do

      call erclos ( 'topbv5loc' )
      end
