      subroutine topbv7012 ( nodp1, nodp2, indprf, kprobf, idegper, &
                             nunkp, istoreleft, istoreright )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of right neighbour unknowns in istoreleft and
!     number of left neighbour unknowns in istoreright
!     Also fill arrays rjumps, rmults, unknownleft and unknownright
!     One couple of nodes only
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodp1, nodp2, indprf, kprobf(*), nunkp, idegper(nunkp), &
              istoreleft(*), istoreright(*)

!     idegper        i    Array of size nunkp containing a 1 if the
!                         corresponding unknown is periodical and a 0 if not
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     istoreleft    i/o   Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     istoreright   i/o   Contains all periodical left neighbor unknowns
!                         for all right periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbors
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nodp1          i    Node sequence number (first of couple)
!     nodp2          i    Node sequence number (second of couple)
!     nunkp          i    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inusol1, inunkp, inusol2, j

!     inunkp         Number of degrees of freedom in point
!     inusol1        Position of degree of freedom in solution vector
!     inusol2        Position of degree of freedom in solution vector
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1803   Nodal point is linked with itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv7012' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv7012'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nodp1==nodp2 ) then

!     --- Error 1803: Nodal point is doubly connected

          call errint ( nodp1, 1 )
          call errsub ( 1803, 1, 0, 0 )

      end if

      inunkp = nunkp

      if ( indprf==0 ) then

!     --- indprf = 0, fixed number of unknowns per point

         inusol1 = (nodp1-1)*nunkp
         inusol2 = (nodp2-1)*nunkp

      else

!     --- indprf>0, variable number of unknowns per point

         inusol1 = kprobf(nodp1)
         inunkp = kprobf(nodp1+1)-inusol1
         inusol2 = kprobf(nodp2)
         inunkp = min(kprobf(nodp2+1)-inusol2,inunkp)

      end if

      do j = 1, inunkp

         if ( idegper(j)==1 ) then

!        --- periodical boundary conditions found, fill arrays

            istoreleft(inusol1+j) = istoreleft(inusol1+j)+1
            istoreright(inusol2+j) = istoreright(inusol2+j)+1

         end if  ! ( idegper(j)==1 )

      end do  ! j = 1, inunkp

1000  call erclos ( 'topbv7012' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv7012'

      end if  ! ( debug )

      end
