      subroutine topelemskip ( nlevels, elmlevels, elements )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-07-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill array elemskip with information about elements to be skipped
!      and elements to be plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     3d
!     element
!     skip
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlevels, elmlevels(*), elements(*)

!     elements       i    Array of length levels(nlevels) (<=nelem)
!                         containing the reoredered element sequence numbers
!                         So elements(1) contains the first element number to
!                         be used, elements(2) the second one and so on
!     elmlevels      i    Array of length nlevels (<nelem) containing the
!                         levels in array elements
!                         The first level consists of the elements 1 to
!                         elmlevels(1) of array elements
!                         The second level contains the elements  elmlevels(1)+1
!                         to elmlevels(2) and so on
!     nlevels        i    Number of entries in array elmlevels
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ilevel, ilow, iupp

!     i              Counting variable
!     ilevel         Level sequence number
!     ilow           Lower bound for i
!     iupp           Upper bound for i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Set elemskip to nlevels+1
!      ilow := 1
!      for ilevel := 1 (1) nlevels do
!         iupp := elmlevels(ilevel)
!         for i := ilow (1) iupp do
!             elemskip(elements(i)) := ilevel
!         ilow := iupp+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set elemskip to nlevels+1

      do i = 1, nelem
         elemskip(i) = nlevels+1
      end do

!     --- ilow := 1
!         for ilevel := 1 (1) nlevels do

      ilow = 1
      do ilevel = 1, nlevels

!     --- iupp := elmlevels(ilevel)
!         for i := ilow (1) iupp do
!             elemskip(elements(i)) := ilevel

         iupp = elmlevels(ilevel)
         do i = ilow, iupp
            elemskip(elements(i)) = ilevel
         end do

!        --- ilow := iupp+1

         ilow = iupp+1

      end do
      end
