      subroutine topfillkmshd2 ( inelem, inpelm, kmeshc, kmeshd1, &
                                 kmeshd2, iwork, ielem, markelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-06-2015 New parameter list
!        version  1.1  date 07-02-2014 Change format
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshd2 for one element group
!     Only elements marked in markelem are considered
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     parallel
!     mesh
!     element
!     node
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inelem, inpelm, kmeshc(inpelm,inelem), &
                             markelem(*), kmeshd1(*)
      integer, intent(inout) :: kmeshd2(*), ielem, iwork(*)

!     ielem         i/o   Element number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     iwork         i/o   integer work array
!                         Must be filled with 1 at first call
!                         iwork(i) contains the next sequence number of an
!                         element to be stored in kmeshd
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshd1        i    KMESH part d1
!     kmeshd2       i/o   KMESH part d2, elements coupled to nodes
!     markelem       i    Work array to mark elements that must be used
!                         0: element is skipped
!                         1: element is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: irelem, j, nodp, ih

!     ih             Help variable to store a constant temporarily
!     irelem         Relative element sequence number with respect to
!                    element group
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topfillkmshd2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topfillkmshd2'
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do irelem = 1, inelem

         ielem = ielem+1

         if ( markelem(ielem)==1 ) then

!        --- Element is marked
!            Loop over nodes in element

            do j = 1, inpelm

               nodp = kmeshc(j,irelem)
               ih = iwork(nodp)
               if ( debug ) &
                  write(irefwr,1) 'ielem, nodp, ih, kmeshd(nodp)', &
                                   ielem, nodp, ih, kmeshd1(nodp)
               kmeshd2(ih+kmeshd1(nodp)) = ielem
               iwork(nodp) = ih+1

            end do  ! j = 1, inpelm

         end if  ! ( markelem(ielem+irelem)==1 )

      end do  ! irelem = 1, inelem

1000  call erclos ( 'topfillkmshd2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topfillkmshd2'

      end if  ! ( debug )

      end subroutine topfillkmshd2
