      subroutine toplm2obst ( nelem, npoint, iwork, kmeshdobst, &
                              knelobst, kmeshcobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill second part of kmesh part d for an obstacle
!     kmesh part d contains the elements corresponding to each node
!     The first part has already been filled in toplm1obst
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, iwork(*), kmeshdobst(*), kmeshcobst(3,*), &
              knelobst(*)

!     iwork               integer work array of length npoint
!                         iwork(i) contains the next sequence number of an
!                         element to be stored in kmeshd
!     kmeshcobst     i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     kmeshdobst     i    KMESH part d (only part 1) of the obstacle
!     knelobst       o    Contains kmesh part d2 at output, i.e.
!                         the element numbers of the obstacle
!     nelem          i    Number of elements of the obstacle
!     npoint         i    Number of nodal points of the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, ih, ielem

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize parameters
!     Set iwork to 1
!     Fill kmesh part d2 in knelobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set iwork to 1

      do i = 1, npoint
         iwork(i) = 1
      end do

!     --- Loop over elements

      do ielem = 1, nelem

!     --- Loop over nodes in element

         do j = 1, 3

            nodp = kmeshcobst(j,ielem)
            ih = iwork(nodp)
            knelobst(ih+kmeshdobst(nodp)) = ielem
            iwork(nodp) = ih+1

         end do

      end do

      end
