      subroutine topol02 ( nnodelgr, nodelgrp )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   22-07-1997
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array nodelgrp with element group sequence numbers corresponding
!    to a nodal point
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodelgr(*), nodelgrp(*)

!     iwork      i    integer work array of length nelem in which the
!                     element group sequence numbers corresponding to the
!                     elements is stored, hence
!                     iwork(ielem) = ielgrp
!     nnodelgr   i    Contains the number of element groups corresponding
!                     to the nodes
!     nodelgrp   o    Contains the element group sequence numbers corresponding
!                     to the nodes
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, iend, istart, nodp

!     i          General loop parameter
!     ielgrp     element group number
!     iend       Last position used at a part
!     ihelp      integer work array of length nelgrp
!     istart     First position used at a part
!     nodp       Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     istart := 0
!     for nodp := 1 (1) npoint do
!        iend := kmshd1(nodp+1)
!        Clear array ihelp
!        for i := 1 (1) iend-istart do
!           ielgrp := iwork(kmshd2(istart+i))
!           if ( ihelp(ielgrp)=0 ) then
!              nnodelgr(nodp) := nnodelgr(nodp)+1
!              nodelgrp(nnodelgr(nodp)) := ielgrp
!              ihelp(ielgrp) := 1
!        istart := iend+1
!      Reset nnodelgr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- loop over nodes

      istart = 0
      do nodp = 1, npoint
         iend = kmshd1(nodp+1)

!        --- Clear array ihelp

         ihelp(1:nelgrp) = 0

!        --- loop over elements corresponding to node

         do i = istart+1, iend

            ielgrp = iwork(kmshd2(i))

            if ( ihelp(ielgrp)==0 ) then

!           --- ihelp(ielgrp) = 0

               nnodelgr(nodp) = nnodelgr(nodp)+1
               nodelgrp(nnodelgr(nodp)) = ielgrp
               ihelp(ielgrp) = 1

            end if

         end do

         istart = iend

      end do

!     --- Reset nnodelgr

      do i = npoint, 1, -1
         nnodelgr(i+1) = nnodelgr(i)
      end do
      nnodelgr(1) = 0
      end
