      subroutine topol06 ( nnelgnod, nelgrnod, nnodelgr )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   22-07-1997
!
!
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array kmeshj with new numbering
!    Nodal points are ordered in the sequence of the element groups
!    If an element group has property 0 all nodes correspond to that
!    group, if it has property 1 only the internal points
! **********************************************************************
!
!                       KEYWORDS
!
!    topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnelgnod(*), nelgrnod(*), nnodelgr(*)

!     nelgrnod   i    Contains the nodes corresponding to the element
!                     group sequence numbers
!     nnelgnod   i    Contains the number of nodes corresponding
!                     to the element groups
!     nnodelgr   i    Contains the number of element groups corresponding
!                     to the nodes
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, ielgrp, nodp

!     i          General loop parameter
!     icount     number of unknowns per element
!     ielgrp     element group number
!     iwork      integer work array of length npoint
!                0: means node has not yet been renumbered
!                1: means node has been renumbered
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     icount := 0
!     for ielgrp := 1 (1) nelgrp do
!        if ( kmeshw(ielgrp)=0 ) then
!           for all nodes nodp in element group do
!              if ( iwork(nodp)=0 ) then
!                 iwork(nodp) := 1
!                 icount := icount+1
!                 kmeshj(icount) := nodp
!        else ( kmeshw(ielgrp)=1 ) then
!           for all internal nodes nodp in element group do
!              if ( iwork(nodp)=0 ) then
!                 iwork(nodp) := 1
!                 icount := icount+1
!                 kmeshj(icount) := nodp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      allocate ( iwork(0:npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0
      icount = 0

!     --- Fill array kmeshj with new sequence of node numbers

      do ielgrp = 1, nelgrp
         if ( kmeshw(ielgrp)==0 ) then

!        --- kmeshw(ielgrp)=0
!            loop over nodes in element group

            do i = nnelgnod(ielgrp)+1, nnelgnod(ielgrp+1)
               nodp = nelgrnod(i)
               if ( iwork(nodp)==0 ) then

!              --- nodal point has not yet been renumbered
!                  give number

                  iwork(nodp) = 1
                  icount = icount+1
                  kmeshj(icount) = nodp

               end if

            end do

         else

!        --- kmeshw(ielgrp)=1
!            loop over internal nodes in element group

            do i = nnelgnod(ielgrp)+1, nnelgnod(ielgrp+1)
               nodp = nelgrnod(i)
               if ( nnodelgr(nodp+1)-nnodelgr(nodp)==1 .and. &
                    iwork(nodp)==0 ) then

!              --- nodal point has not yet been renumbered
!                  give number

                  iwork(nodp) = 1
                  icount = icount+1
                  kmeshj(icount) = nodp

               end if

            end do

         end if

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  end
