      subroutine topperadap
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make new arrays kmesh part e and kmesh part j in case of
!     periodical boundary conditions
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     renumbering
!     periodical
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jrenum(10)
      integer, allocatable :: kmeshenew(:)

!     jrenum         Parameter jrenum for TOPOL
!     kmeshenew      Temporary array to store kmesh part e extended with
!                    contributions of periodical nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCMKMESHE     Create a new temporary array kmesh part e in case of
!                    periodical boundary conditions
!     SEPFILLKMESHE  Fill array kmesh part e
!     TOPREN         Renumber nodal points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topperadap' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topperadap'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First create a new array kmesh part e

      call sepfillkmeshe

      length = size(kmeshe)+size(kprobper )

      allocate ( kmeshenew(length), stat=error )
      if ( error/=0 ) call eralloc ( error, length, 'kmeshenew' )
      if ( ierror/=0 ) go to 1000

      numpairs = kprobloc(62)
      call incmkmeshe ( kmshe1, kmshe2, npoint, &
                        kmeshenew, kmeshenew(npoint+2) )

!     --- Remove old kmesh part e
!         Create new array kmesh e by copying from kmeshenew

      kelme = npoint+1+kmeshenew(npoint+1)
      deallocate ( km(iseqkm)%kmeshe )
      allocate ( km(iseqkm)%kmeshe(kelme) )
      km(iseqkm)%kmeshe = kmeshenew(1:kelme)
      km(iseqkm)%kmshe1 => km(iseqkm)%kmeshe(1:npoint+1)
      km(iseqkm)%kmshe2 => km(iseqkm)%kmeshe(npoint+2:kelme)
      kmeshe => km(iseqkm)%kmeshe
      kmshe1 => km(iseqkm)%kmshe1
      kmshe2 => km(iseqkm)%kmshe2

      if ( kelmj>0 ) then

!     --- kmesh part j has been filled,  but a new renumbering is
!         necessary because of the coupling between nodes

         jrenum(1) = 0                ! whole array is filled
         jrenum(2) = 2                ! sloan
         jrenum(3:4) = 0              ! ipref/iprint
         jrenum(5:7) = 0              ! ivertx/ imidp/ icentr
         jrenum(8) = 0                ! ilevel
         jrenum(9) = 4                ! start node given
         jrenum(10) = kmeshj(1)  ! start node
         call topren ( jrenum, kmeshenew )

      end if  ! ( kelmj>0 )

      deallocate ( kmeshenew, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshenew' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topperadap'

      end if  ! ( debug )
      call erclos ( 'topperadap' )

      end
