      subroutine elderv ( icheld, ix, jdegfd, coor, elemvc, elemwg, &
                          iuser, user, vector1, vector2, index1, &
                          index2, icouno )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 06-07-2011 Minor change
!        version  1.2  date 25-02-2010 Extra parameter icouno
!        version  1.1  date 30-08-1999 Copy input parameters for eldervsubr
!
!   copyright (c) 1998-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      User element subroutine to compute derivatives to be used by SEPRAN
!      This is the general version, which produces an error message if it is
!      called except in the case of the Numerical Analysis Lab of Delft
!      University of Technology
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), index1(inpelm), index2(icount), &
              icheld, ix, jdegfd, icouno
      double precision coor(ndimlc,*), vector1(*), vector2(*), &
                       elemvc(icouno), elemwg(icouno), user(*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     elemvc     o    Element vector to be filled by the user
!     elemwg     o    element vector containing the weights for subroutine
!                     deriv, to be filled by the user
!     icheld     i    Choice parameter to distinguish between the various
!                     possibilities of this subroutine
!     icouno     i    Number of unknowns in output vector
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     index2     i    Array of length icount containing the positions of the
!                     degrees of freedom in the element with respect to vector1
!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     ix         i    indication what type of derivative must be computed
!                     See DERIV
!     jdegfd     i    The derivative du jdegfd / d ix is computed
!     user       i    Real user array to pass user information from
!                     main program to subroutine.
!     vector1    i    First input vector
!     vector2    i    Second input vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenhome, lenarray, inpelmcopy, &
              icountcopy, itypecopy
      parameter ( lenarray=100 )
      double precision x(lenarray), u(lenarray), elemvec(0:10*icouno), &
                       y(lenarray)
      character (len=80) filenm
      logical check

!     check       Indicates if the file exist (true) or not
!     elemvec     Local array elemvc that is filled by the user
!                 This array is extended with some positions to check
!                 overwriting by the user
!     filenm      Name of file
!     icountcopy  Copy of icount
!     inpelmcopy  Copy of inpelm
!     itypecopy   Copy of itype
!     lenarray    Length of arrays u and x
!     lenhome     Number of characters in the name of the SEPRAN home directory
!     u           array of length m containing the values of vector1 in the
!                 nodal points of the element
!     x           Contains the x-coordinates of the nodes of the element using
!                 the local node numbering
!     y           Contains the y-coordinates of the nodes of the element using
!                 the local node numbering
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELDERVCHECK    Check if array elderv has been filled correctly in
!                    subroutine ELDERVSUBR
!     ELDERVSUBR     Special user element subroutine to be called by elderv
!                    and meant for the numerical analysis lab at Delft
!                    University of Technology
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     349   Subroutine not available
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( file sphome/bin/update/wi211 exists ) then
!       prepare call to eldervsubr
!       call eldervsubr
!    else
!       give error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elderv' )

      lenhome = index(sphome,' ')-1
      filenm = sphome(1:lenhome)//'/bin/update/wi211'
      inquire ( file=filenm, exist=check )
      if ( check ) then

!     --- wi211 file found, Hence special situation for the "wi211-lab" at
!         Delft University
!         First check array boundaries

         if ( inpelm*ndimlc>lenarray ) then
            call errint ( inpelm*ndimlc, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'x', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( icount>lenarray ) then
            call errint ( icount, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'u', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( ierror/=0 ) go to 1000

!        --- Copy co-ordinates from coor into x

         x(1:inpelm) = coor(1,index1(1:inpelm))
         y(1:inpelm) = coor(2,index1(1:inpelm))

!        --- Copy old solution into u

         u(1:icount) = vector1(index2(1:icount))
         inpelmcopy = inpelm
         icountcopy = icount
         itypecopy = itype

!        --- initialization

         elemwg = 1d0
         elemvec = 0d0

!        --- actual call

         call eldervsubr ( inpelmcopy, x, y, icountcopy, &
                           elemvec(1), u, itypecopy, icheld, icouno )

!        --- Fill elemvc

         elemvc = elemvec(1:icouno)

!        --- Check elemvec

         call eldervcheck ( elemvec, icouno )

      else

!     --- Standard situation: give error message

         call errchr ( 'elderv', 1 )
         call errsub ( 349, 0, 0, 1 )

      end if

1000  call erclos ( 'elderv' )
      if ( ierror/=0 ) call instop

      end
