      subroutine elem2 ( iuser, user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 22-08-2014 New call to elemsubr
!        version  1.2  date 16-03-2011 Use iseqin to copy old solution
!        version  1.1  date 30-08-1999 Copy input parameters for eldervsubr
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      User element subroutine to be used by SEPRAN
!      This is the general version, which produces an error message if it is
!      called except in the case of the Numerical Analysis Lab of Delft
!      University of Technology
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulecactl
      use sepmodulecomio
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision user(*)

!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenhome, lenarray, i, j, inpelmcopy, &
              icountcopy, itypecopy
      parameter ( lenarray=100 )
      double precision x(lenarray), u(lenarray), elemcopy(1:icount**2), &
                       y(lenarray), elmax
      character * 80 filenm
      logical check

!     check       Indicates if the file exist (true) or not
!     elemcopy    Copy of the transpose of the element matrix
!     elmax       Maximum of element matrix in absolute value
!     filenm      Name of file
!     i           Counting variable
!     icountcopy  Copy of icount
!     inpelmcopy  Copy of inpelm
!     itypecopy   Copy of itype
!     j           Counting variable
!     lenarray    Length of arrays u and x
!     lenhome     Number of characters in the name of the SEPRAN home directory
!     u           array of length m containing the values of vector1 in the
!                 nodal points of the element
!     x           Contains the x-coordinates of the nodes of the element using
!                 the local node numbering
!     y           Contains the y-coordinates of the nodes of the element using
!                 the local node numbering
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELEMSUBR  Special user element subroutine to be called by elem2 and
!               meant for the numerical analysis lab at Delft University
!               of Technology
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     349   Subroutine not available
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( file sphome/bin/update/wi211 exists ) then
!       prepare call to elemsubr
!       call elemsubr
!       translate back
!    else
!       give error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elem2' )

      lenhome = index(sphome,' ')-1
      filenm = sphome(1:lenhome)//'/bin/update/wi211'
      inquire ( file=filenm, exist=check )
      if ( check ) then

!     --- wi211 file found, Hence special situation for the "wi211-lab" at
!         Delft University
!         First check array boundaries

         if ( inpelm*ndimlc>lenarray ) then
            call errint ( inpelm*ndimlc, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'x', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( icount>lenarray ) then
            call errint ( icount, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'u', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( ierror/=0 ) go to 1000

!        --- Copy co-ordinates from coor into x

         x(1:inpelm) = coor(1,index1(1:inpelm))
         y(1:inpelm) = coor(2,index1(1:inpelm))

!        --- Copy old solution into u

         u(1:icount) = oldsols(1)%oldvecs(index3(iseqin,1:icount))
         inpelmcopy = inpelm
         icountcopy = icount
         itypecopy = itype
         elemcopy = 0d0
         elemvc(1:icount) = 0d0

         call elemsubr ( inpelmcopy, x, y, icountcopy, &
                         elemcopy, elemvc, elemms, u, itypecopy )

!        --- Store the transpose of the element matrix in elemmt

         do j = 1, icount
            do i = 1, icount
               elemmt(i,j) = elemcopy((i-1)*icount+j)
            end do
         end do

      else

!     --- Standard situation: give error message

         call errchr ( 'elem2', 1 )
         call errsub ( 349, 0, 0, 1 )

      end if

1000  call erclos ( 'elem2' )
      if ( ierror/=0 ) call instop

      end
