      subroutine mshpareluser ( nprocessors, elementsact, nelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nprocessors, nelem
      integer, intent(out) :: elementsact(nelem)

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nelstep, nelblock, nproc, i, j, k, l, ielem
      double precision :: h

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpareluser' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpareluser'
         write(irefwr,1) 'nelem, nprocessors', nelem, nprocessors
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nelblock = nelem/nprocessors
      if ( nelblock*nprocessors/=nelem ) then
         write(irefwr,100) nelem, nprocessors, nelblock
100      format( 'nelem = ', i6, ' nprocessors = ',i6, &
                 'nelblock = ', i6/ &
                 'necessary is nelem = nelblock*nprocessors' )
         call instop
      end if  ! ( nelblock*nprocessors/=nelblock )

      h = nelblock
      nelstep = nint(sqrt(h))

      if ( nelstep*nelstep/=nelblock ) then
         write(irefwr,110) nelblock, nelstep
110      format( 'nelblock = ', i6, 'nelstep = ', i6/ &
                 'necessary is nelstep^2 = nelblock' )
         call instop
      end if  ! ( nelstep*nelstep/=nelblock )

      h = nprocessors
      nproc = nint(sqrt(h))

      if ( nproc*nproc/=nprocessors ) then
         write(irefwr,120) nprocessors, nproc
120      format( 'nprocessors = ', i6, 'nproc = ', i6/ &
                 'necessary is nproc^2 = nprocessors' )
         call instop
      end if  ! ( nproc*nproc/=nprocessors )

      if ( debug ) &
         write(irefwr,1) 'nelblock, nelstep, nproc', &
                          nelblock, nelstep, nproc

      ielem = 0
      do i = 1, nproc
         do j = 1, nelstep
            do k = 1, nproc
               do l = 1, nelstep
                  ielem = ielem+1
                  elementsact(ielem) = k+(i-1)*nproc
               end do  ! l = 1, nelstep
            end do  ! k = 1, nelstep
         end do  ! j = 1, nproc
      end do  ! i = 1, nproc

1000  call erclos ( 'mshpareluser' )
      if ( debug ) then

!     --- Debug information

         call prinin ( elementsact, nelem, 'elementsact' )
         write(irefwr,*) 'End mshpareluser'

      end if  ! ( debug )

      end subroutine mshpareluser
