      subroutine el0607 ( iuser, user, array, method )
! ======================================================================
!
!        programmer    Peter Minev
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.4  date 23-08-2001 New call to elusr3
!        version  2.3  date 03-09-2000 Extension for scalars
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELM607
!     the parameters in the common blocks celint and celiar get a start
!     value
!     array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), method
      double precision user(*), array(*)

!     array     o     real work array in which the values of the coefficients
!                     in the integration points are stored in the sequence:
!                     1: nu
!                     2: f
!                     the coefficients 1-2 require exact n (number of
!                     nodal points) positions
!                     coefficients for the convection will be loaded
!                     through the old solution option
!     iuser     i     user array in which integer information of the
!                     coefficients is stored (See Standard Problems)
!     method    o     choice parameter, see ELM607
!     user      i     user array in which real information of the
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!      70   alpha11 = 0 or alpha22 = 0
!    1605   method has wrong value
!    1697   combination n and mone not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('el0607')
      call elcomm

!     --- set parameters depending on the type numbers

      if ( itype.eq.607 ) then

!     --- itype = 607, 1-D, 2-D and 3-D spectral element for Navier-Stokes
!          (convection-diffusion) equations)

         n = inpelm
         m = inpelm
         mone = int(n**(1d0/ndim)+0.5d0)
         if ( mone**ndim.ne.n ) then
            call errint ( ndim, 1 )
            call errint ( n, 2 )
            call errint ( mone**ndim, 3 )
            call errsub ( 1697, 3, 0, 0 )
         end if
         jdercn = 1
         nvert = 2**ndim
         ndimlc = ndim

      end if

!     --- fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ipos = iuser(ielgrp + 5)
      if (ipos.lt.7) then

         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub (42, 5, 0, 0)
         go to 1000
      end if

      nuser = 0
      call elusr3 (1, array(1), iuser, user, 1)

!     --- compute method

      ipos = ipos + 1
      method = inirepscal ( iuser(ipos) )
      ipos = ipos + 1

!     --- check value of method

      if ((method.lt.0) .or. (method.gt.2)) then

         call errint ( method, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1605, 4, 0, 0)
         go to 1000
      end if

!     --- test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser)
      call prinlg ( 'user', -nuser, user, iuser)

!     --- compute jsecnd
!         first check if nu is unequal to zero

      if ( ind(1).eq.0 ) then

!     --- nu is zero

         jsecnd = 0

         call errwar (70, 0, 0, 0)

      end if

      if ( ind(1).le.0 ) then

!     --- nu is constant

        jsecnd = 1

      else

!     --- nu is variable

         jsecnd = 2

      end if

1000  if (ierror.ne.0) call instop

      call erclos ('el0607')

      end
