      subroutine el0800der ( icheld, iweigh, jderiv, ndimlc, ideriv, &
                             icheli, ix )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-08-2011 Extension with icheld = 7
!        version  1.0  date 07-03-2005
!
!   copyright (c) 2005-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters related to derivatives or integrals
!     Help subroutine for el0800 and related ones
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
!     enthalpy
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, iweigh, jderiv, ndimlc, ideriv, icheli, ix

!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of the derivatives
!                         See subroutine eld800
!     icheli         i    Defines the type of integral to be computed
!     ideriv         o    Indication if derivatives must be computed (1)
!                         or not (0)
!     iweigh         o    Indication if weights and qmat must be filled or not
!     ix             i    indication what type of derivative must be computed
!                         See DERIVA
!     jderiv         o    Indication if the value of the derivatives must be
!                         computed. See common celint
!     ndimlc         o    Local dimension of space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800der' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800der'

      end if
      if ( ierror/=0 ) go to 1000

!     --- if derivatives must be computed, the weights are not necessary

      if ( icheld>0 .and. icheld<26 .or. &
           icheld>30 .and. icheld<45 ) iweigh = 0

!     --- Compute jderiv ( if icheld#9 ) or ideriv ( if icheld=9 ), ndimlc

      if ( icheld==1 .or. icheld==11 .or. icheld==31 ) then

!     --- icheld = 1, 11 or 31

         jderiv = ix + 1
         ndimlc = 1

      else if ( icheld==21 .or. icheld==41 .or. &
                icheli==1 .or. icheli==2 .or. &
                icheli==6 .or. icheli==7 .or. icheli==8 ) then

!     --- icheld = -1, -2, -6, or 9, no derivatives should be computed

         ideriv = 0

      else if ( icheli>2 .and. icheli<=5 ) then

!     --- icheld<-2, derivatives should be computed

         jderiv = -icheld+3

      else if ( icheld==7 ) then

!     --- icheld = 7

         jderiv = 0
         ndimlc = 1

      else if ( icheld>0 ) then

!     --- icheld = 1-20, 22-29

         jderiv = 1

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0800der'

      end if
      call erclos ( 'el0800der' )

      end
