      subroutine el0800heat ( iuser, ishift, iqmat, iqmat1, jheat, &
                              massparm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameters with respect to the mass matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), ishift, iqmat, iqmat1, jheat, massparm

!     iqmat          o    if 1 the qmat matrix must be filled
!     iqmat1         o    Indication whether the right-hand side has to be
!                         transformed
!     ishift        i/o   Shift of specific variable with respect to diagonal
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jheat         i/o   Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     massparm       o    Indicates the largest value of jheat during the
!                         time integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800heat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800heat'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jheat = imas
      massparm = max(jheat,massrule)
      iqmat = inirepscal ( iuser(ipos) )
      iqmat = iqmat/10
      if ( debug ) &
         write(irefwr,1) 'imas, jheat, iqmat, massparm', &
                          imas, jheat, iqmat, massparm

      iqmat1 = 0
      if ( iqmat>1 ) iqmat1 = 1
      if ( iqmat>=1 ) then
         iweigh = iweigh + 10
         iqmat = 1
      end if
      ipos = ipos + 1
      ishift = ishift+1

1000  call erclos ( 'el0800heat' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0800heat'

      end if  ! ( debug )

      end
