      subroutine el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                             iphi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 10-02-2015 Extension for jelp = 8
!        version  1.6  date 27-05-2014 Extension for ishape = 22
!        version  1.5  date 15-02-2013 Extra debug
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the parameter m for an element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer m, jelp, jwcons, irule, igausp, jdiag, iphi

!     igausp         i    Indication if Gauss points must be computed (1)
!                         or not (0)
!     iphi           i    Indication if array phi must be filled (1)
!                         or not (0)
!     irule          i    type of numerical integration.
!     jdiag          i    Indicates if the basis functions satisfy phii(j = 0, &
!                         if i#j (jdiag=1) or not (jdiag=2)
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jwcons         i    Indication if the weight function is constant
!                         (1) or not (2)
!     m              o    number of integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800inm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800inm'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      igausp = 1
      iphi = 1
      jdiag = 2
      jwcons = 2
      select case ( jelp )
         case(1)

            if ( irule==5 ) then
               m = 3
            else
               m = irule - 1
            end if

         case(2)

            if ( irule==2 ) irule = 3
            m = irule - 1

         case(3)

            if ( irule==2 ) then
               m = 1
            else if ( irule==5 ) then
               m = 3
            else
               m = 4
            end if

         case(4)

            if ( irule==2 ) then
               m = 4
            else
               m = 7
            end if

         case(5)

            if ( irule==2 ) then
               m = 1
            else
               m = 4
            end if

         case(6)

            if ( irule==2 ) then
               m = 4
            else
               m = 9
            end if

         case(7)

            if ( irule==2 ) m = 4

         case(8)

            if ( irule==2 ) then
               m = 1
            else if ( irule==3 ) then
               m = 4
            end if  ! ( irule==2 )

         case(9)

            if ( irule==3 ) then
               m = 4
            else if ( irule==4 ) then
               m = 9
            end if

         case(10)

            if ( irule==3 ) m = 7

         case(11)

            if ( irule==2 ) then
               m = 1
            else if ( irule==3 ) then
               m = 4
            else if ( irule==5 ) then
               m = 10
            else if ( irule==6 ) then
               m = 8
            else
               m = 5
            end if

         case(12)

            if ( irule==2 .or. irule==3 ) then
               m = 11
            else if ( irule==4 ) then
               m = 15
            end if  ! ( irule==4 )

         case(13)

            if ( irule==2 ) then
               m = 1
            else
               m = 8
            end if

         case(14)

            if ( irule==2 ) then
               m = 8
            else
               m = 27
            end if

         case(16)

            if ( irule==2 ) then
               m = 11
            else
               m = 15
            end if

         case(17)

            if ( irule==3 ) then
               m = 8
            end if

         case(18)

            if ( irule==3 ) then
               m = 4
            else
               m = 5
            end if

         case(21)

            if ( irule==2 ) then
               m = 1
            else if ( irule==3 ) then
               m = 8
            else
               m = 6
            end if

         case(22)

            if ( irule==2 ) then
               m = 8
            else if ( irule==3 ) then
               m = 14
            else
               m = 18
            end if

         case default

!        --- Error shape number not yet implemented

            call errint ( jelp, 1 )
            call errsub ( 800, 1, 0, 0 )

      end select   ! case ( jelp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jelp, irule, m', jelp, irule, m
         write(irefwr,*) 'End el0800inm'

      end if
      call erclos ( 'el0800inm' )
      end
