      subroutine el0800vec ( jwcons, itypln )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameter jconsf corresponding to right-hand-side vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jwcons, itypln

!     itypln         i    Indicates the type of linearization to be performed
!                         Possible values:
!     jwcons         i    Indication if the weight function is constant
!                         (1) or not (2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icoef

!     icoef          Coefficient sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800vec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800vec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoef = 11

      if ( ind(icoef)==0 ) then

!     --- F = 0, no right-hand side

         if ( itypln<4 ) notvec = 0

      else if ( ind(icoef)<0 .or. ind(icoef)==2002 ) then

!     --- F is constant (per element)

         jconsf = jwcons

      else

!     --- General

         jconsf = 2

      end if

1000  call erclos ( 'el0800vec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0800vec'

      end if  ! ( debug )

      end
