      subroutine el0801rule ( irule, igausp, iphi, jdiag, jwcons, m, &
                              jelp, mn, kstep, n, jcart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set integration parameters with respect to elements of type 801, 907
!     Depending on shape of elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
!     initialization
!     parabolic_equation
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irule, igausp, iphi, jdiag, jwcons, m, jelp, mn, kstep, &
              n, jcart

!     igausp         o    Indication if Gauss points must be computed (1)
!                         or not (0)
!     iphi           o    Indication if array phi must be filled (1)
!                         or not (0)
!     irule          o    type of numerical integration.
!                         See subroutine el0801
!     jcart          o    indication whether the element is a triangle with
!                         Cartesian coordinates (1)
!                         or a circular triangle with cylinder symmetric
!                         coordinates (2)
!     jdiag          o    Indicates if the basis functions satisfy phii(j = 0, &
!                         if i#j (jdiag=1) or not (jdiag=2)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jwcons        i/o   Indication if the weight function is constant
!                         (1) or not (2)
!     kstep          o    Step in loop
!     m              o    number of integration points
!     mn             o    m * n
!     n              o    number of basis functions for each component
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0801rule' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0801rule'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         igausp = 1
         iphi = 1
         jdiag = 2
         jwcons = 2
         if ( jelp<=2 ) then
            m = irule - 1
         else if ( jelp==3 .or. jelp==5 ) then
            if ( irule==2 ) then
               m = 1
            else
               m = 4
            end if
         else if ( jelp==4 ) then
            if ( irule==2 ) then
               m = 4
            else
               m = 7
            end if
         else if ( jelp==6 ) then
            if ( irule==2 ) then
               m = 4
            else
               m = 9
            end if
         end if
         mn = m * n

      else if ( jelp==4 .and. jcart==1 ) then
         kstep = 2
      end if

1000  call erclos ( 'el0801rule' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0801rule'

      end if  ! ( debug )

      end
