      subroutine el0808 ( iuser, user, jelp, kfirst, jheat, isub, &
                          isup, array, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 12-05-2006
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM808 or
!     ELD808 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, kfirst, jheat, isub, isup, iseq

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                  2D            3D
!                    1:   alpha 11         alpha 11       alpha 11
!                    2:                    alpha 12       alpha 12
!                    3:                    alpha 22       alpha 13
!                    4:                                   alpha 22
!                    5:                                   alpha 23
!                    6:                                   alpha 33
!                    7:   u 1              u 1            u 1
!                    8:                    u 2            u 2
!                    9:                                   u 3
!                   10:   beta             beta           beta
!                   11:   f                f              f
!                   12:   rho cp           rho cp         rho cp
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!                    Parameters 1-12 correspond to the first unknown, &
!                    parameters 13-24 to the second one
!     iseq     i     Sequence number of vector in VECOLD/ISLOLD
!                    In case of a call by ELI808 it is the parameter JDEGFD
!     isub     o     Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup     o     Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jelp     o     Indication of the type of basis function subroutine must
!                    be called by ELM808. Possible vaues:
!                    1:  Subroutine ELP610  (Linear line element)
!                    2:  Subroutine ELP611  (Quadratic line element)
!                    3:  Subroutine ELP620  (Linear triangle)
!                    4:  Subroutine ELP622  (Quadratic triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!                    6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                   11:  Subroutine ELP630  (Linear tetrahedron)
!                   12:  Subroutine ELP631  (Quadratic tetrahedron)
!                   13:  Subroutine ELP632  (Trilinear brick)
!                   14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat    o     Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                    0:  no mass matrix
!                    1:  diagonal mass matrix with constant coefficient
!                    2:  diagonal mass matrix with variable coefficient
!                   11:  full mass matrix with constant coefficient
!                   12:  full mass matrix with variable coefficient
!     kfirst   o     This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     user     i     User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, icoef, kchois, mrule, &
              ichtrn, jsecsv, kfirsv, idimsv, icoefs, ishift
      logical linear, funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ichtrn         Defines the direction of the computation of an integral
!     icoef          Coefficient sequence number
!     icoefs         Coefficient sequence number
!     idimsv         Help variable to save idim
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     jsecsv         Help variable to save jsecnd
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM808
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD808 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELD808 or ELI808 calls this subroutine, &
!                        a user function defined as first parameter in
!                        IUSER/USER must be filled in elemvc (icheld=9)
!                    3:  Subroutine ELD808 calls this subroutine, &
!                        alpha times the gradient must be stored in ELEMVC
!                        The first 3 parameters of IUSER/USER are used
!     kfirsv         Help variable to save kfirst
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL1002         Compute JSECND for diffusion terms
!     EL1013         Compute KFIRST and IDIM for convection terms
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The system of equations has the shape
!     D_11 u_1 + D_12 u_2 = f_1
!     D_21 u_1 + D_22 u_2 = f_2, &
!     where D_ij refers to a second order equation for one unknown
!     as in type 800
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!     2:  - &
!                                                        - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  This parameter depends on the type number to be used
!         In case of type number 803 it indicates the type of linearization:
!         Possible values:
!         0,1: Picard
!         2:   Newton
!     Parameters with respect to the first unknown
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    Parameters: 21-35 corresponding to second unknown have exactly
!    the same meaning (D_22)
!    Parameters: 36-50 corresponding to first row second unknown
!    have exactly the same meaning (D_12)
!    Parameters: 51-65 corresponding to second row first unknown
!    have exactly the same meaning (D_21)
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
!     In the case of a call by DERIV:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the cases ICHELD = 6, 16 the positions 6-11 contain
!         information about the matrix A:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!         In the case ICHELD = 21 position 6 contains information about the
!         function f to be filled:
!     6:  Function f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     264   jdegfd has wrong value
!     341   Icheli has wrong value
!     406   Value of icheld not allowed
!     616   Fatal error: METUPW has wrong value
!     771   Rho is non-positive
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1771   Shape number not yet implemented
!    1787   iseq has incorrect value
!    1863   Value of icheli has not yet been implemented for this element shape
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0808' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0808'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill some standard values in the common blocks celp and celint

      if ( icheld<0 ) then
         icheli = mod(-icheld,10)
         ichtrn = -icheld/10
      else
         icheli = 0
         ichtrn = 0
      end if
      if ( icheli/=7 ) then
         call elcomm
      else
         iweigh = 1
         ideriv = 1
         irule  = 1
         igausp = 0
         iphi   = 0
         jcart  = 1
         icoor  = 1
         ncomp  = 1
         jdiag  = 1
         idim   = 1
         jderiv = 0
         jtrans = 0
         jind   = 1
         kstep  = 1
      end if
      jheat  = 0
      jtrans = 0
      metupw = 0
      ishift = 0
      if ( icheld>=0 ) iseq = 1

!     --- Compute kchois

      if ( icheld==0 ) then

!     --- Call by elm808, all coefficients required

         kchois = 0

      else if ( icheli>0 .and. icheli/=7 ) then

!     --- call by eli808, one (new) coefficient required

         kchois = 2

      else if ( icheld==21 ) then

!     --- call by eld808, icheld = 21, one (new) coefficient required

         kchois = 4

      else if ( icheld==6 .or. icheld==16 .or. icheld==26 .or. &
                icheld==28 .or. icheld==29 .or. icheld==36 ) then

!     --- call by eld808, icheld = 6, 16, 26, 28, 29, 36
!         The first six standard coefficients are required

         kchois = 3

      else

!     --- call by eld808, icheld<6, or 10<icheld<16, or icheld = 27
!         or call by eli808, icheli = 7
!         i.e. no coefficients required

         kchois = 1

      end if

!     --- if derivatives must be computed, the weights are not necessary

      if ( icheld>0 .and. icheld<26 .or. &
           icheld>30 .and. icheld<40 ) iweigh = 0

!     --- Compute jderiv ( if icheld#9 ) or ideriv ( if icheld=9 ), ndimlc

      if ( icheld==1 .or. icheld==11 .or. icheld==31 ) then

!     --- icheld = 1, 11 or 31

         jderiv = ix + 1
         ndimlc = 1

      else if ( icheld==21 .or. icheli==1 .or. icheli==2 .or. &
                icheli==6 .or. icheli==7 ) then

!     --- icheld = -1, -2, -6, or 9, no derivatives should be computed

         ideriv = 0

      else if ( icheli>2 .and. icheli<=5 ) then

!     --- icheld<-2, derivatives should be computed

         jderiv = -icheld+3

      else if ( icheld>0 ) then

!     --- icheld = 1-20, 22-29

         jderiv = 1

      end if

      if ( icheld==4 .or. icheld>=7 .and. icheld<=10 .or. &
           icheld==14 .or. icheld>=17 .and. icheld<=20 .or. &
           icheld>=22 .and. icheld<=30 .or. icheld==34 .or. &
           icheld>=37 ) then

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      else if ( icheli>7 .or. ichtrn>ndim ) then

         call errint ( -icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 341, 3, 0, 0 )

      end if
      if ( icheli>=3 .and. icheli<=5 ) then

         ix = icheli-2
         if ( ix>ndim ) then
            call errint ( -icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 341, 3, 0, 0 )
         end if

      end if
      if ( icheli==6 .and. iseq<=0 .or. iseq>ndim ) then
         call errint ( iseq, 1 )
         call errint ( ndim, 2 )
         call errsub ( 264, 2, 0, 0 )
      end if

      if ( icheli==6 ) ncomp = ndim

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = n
      mn = m * n
      jelp = ishape
      if ( jelp==1 .or. jelp==3 .or. jelp==11 ) then

!     --- linear element

         linear = .true.
         mn = m
         jdercn = 0

      else if ( jelp==2 .or. jelp>=4 .and. jelp<=7 .or. &
                jelp>=12 ) then

!     --- Bilinear or quadratic element

         linear = .false.
         jdercn = 1

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

      if ( kchois==0 ) then

!     --- standard situation

         ncoef = 12

      else if ( kchois==1 ) then

!     --- kchois = 1, no coefficients required

         ncoef = 0

      else if ( kchois==2 .or. kchois==4 ) then

!     --- kchois = 2 or 4, one coefficient required

         ncoef = 1

      else if ( kchois==3 ) then

!     --- kchois = 3, six coefficients required

         ncoef = 6

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficient

      if ( ncoef>0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if

         nuser = 0

         if ( kchois==0 ) then

!        --- Standard situation, Compute the parameters jheat, metupw

            jheat = imas
            ipos = ipos + 1
            ishift = ishift+1
            if ( jheat==0 ) notmas = 1

            metupw = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1
            if ( metupw/=0 ) then
               call errint ( metupw, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 616, 4, 0, 0 )
            end if

         else if ( icheld>0 ) then

!        --- icheld>0, Compute iseq

            iseq = inirepscal ( iuser(ipos) )
            ipos = ipos + 4
            ishift = ishift+4
            if ( iseq<0 .or. iseq>numold ) then
               call errint ( iseq, 1 )
               call errint ( numold, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1787, 5, 0, 0 )
            end if
            if ( iseq==0 ) iseq = 1

         end if

         if ( kchois==0 .or. kchois==2 ) then

!        --- Call by BUILD or INTEGR, Compute the parameters jcart and irule

            mrule = inirepscal ( iuser(ipos) )
            if ( mrule==0 ) then
               irule = 1
            else
               irule = mrule
            end if
            ipos = ipos + 1
            ishift = ishift+1
            if ( mrule<0 .or. mrule>40 ) then
               call errint ( mrule, 1 )
               if ( kchois==0 ) then
                  call errint ( 3, 2 )
               else
                  call errint ( 1, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1766, 5, 0, 0 )
            end if

            jcart = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1
            if ( jcart<0 .or. jcart>2 ) then
               call errint ( jcart, 1 )
               if ( kchois==0 ) then
                  call errint ( 4, 2 )
               else
                  call errint ( 2, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1767, 5, 0, 0 )
            end if
            jcart = jcart + 1

         end if

         if ( linear ) then
            jwcons = min(jcart,2)
         else
            jwcons = 2
         end if
         if ( icheld>0 .and. mrule==0 ) irule = 1
         if ( jheat==2 .and. (jelp<=3 .or. jelp==5) .and. &
              mrule==0 ) then
            if ( jelp==2 ) then
               irule = 4
            else
               irule = max ( irule, 3 )
            end if
         else if ( jheat==2 .and. jelp<=6 .and. mrule==0 ) then
            irule = max ( irule, 2 )
         end if
         if ( icheli>2 .and. icheli<=5 ) irule = 1

         if ( irule/=1 ) then

!        --- irule>1, Gauss rule

            igausp = 1
            iphi = 1
            jdiag = 2
            jwcons = 2
            if ( jelp<=2 ) then
               if ( jelp==2 .and. irule==2 ) irule = 3
               m = irule - 1
            else if ( jelp==3 .or. jelp==5 ) then
               if ( irule==2 ) then
                  m = 1
               else
                  m = 4
               end if
            else if ( jelp==4 ) then
               if ( irule==2 ) then
                  m = 4
               else
                  m = 7
               end if
            else if ( jelp==6 ) then
               if ( irule==2 ) then
                  m = 4
               else
                  m = 9
               end if
            else if ( jelp==7 ) then
               if ( irule==2 ) m = 4
            else if ( jelp==11 ) then
               if ( irule==2 ) then
                  m = 1
               else if ( irule==3 ) then
                  m = 4
               else
                  m = 5
               end if
            else if ( jelp==12 ) then
               irule = 1
            else if ( jelp==13 ) then
               if ( irule==2 ) then
                  m = 1
               else
                  m = 8
               end if
            else if ( jelp==14 ) then
               if ( irule==2 ) then
                  m = 8
               else
                  m = 27
               end if
            end if
            mn = m * n

         end if
         isub = 1
         ipos = ipos + 1
         ishift = ishift+1

         do i = isub, ncoef
            call elusr3 ( i, array(1+(i-1)*m), iuser, user , i+ishift)
         end do
         do j = 1, 3
            ipos = ipos + 3
            ishift = ishift+3
            do i = j*ncoef+1, (j+1)*ncoef
               call elusr3 ( i, array(1+(i-1)*m), iuser, user, &
                             i+ishift )
            end do
         end do  ! j = 1, 3

         if ( debug ) then
            call prinin ( ind, 4*ncoef, 'ind' )
            call prinrl1 ( array, 4*ncoef, m, 'coefficients' )
         end if  ! ( debug )

!        --- Test array lengths, number of nodes and number of degrees
!            of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )

      end if

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      if ( kchois==0 .and. matrix .or. kchois==3 ) then

!     --- matrix = true  The element matrix must be computed.
!                        Compute jsecnd, idim and jzero
!         --- First consider the diffusion tensor:

         call el1002 ( 0, 5, jwcons, jelp, 0, array, ndim )
         jsecsv = jsecnd
         call el1002 ( 12, 8, jwcons, jelp, 0, array, ndim )
         jsecnd = max(jsecnd,jsecsv)

         if ( kchois/=3 ) then

!        --- Next consider the Convective terms:
!            compute idim and first component idim

            call el1013 ( 6, 5, kfirst, jelp, 0, ndim )
            do j = 1, 3
               idimsv = idim
               kfirsv = kfirst
               call el1013 ( 12*j+6, 8, kfirst, jelp, 0, ndim )
               idim = max(idim,idimsv)
               kfirst = min(kfirsv,kfirst)
            end do  ! j = 1, 3

!           --- Constant part of the equation:
!               compute jzero

            if ( ind(10)==0 .and. ind(22)==0 .and. &
                 ind(34)==0 .and. ind(46)==0) then

!           --- beta = 0

               jzero = 0

            else if ( (ind(10)<0 .or. ind(10)==2002) .and. &
                      (ind(22)<0 .or. ind(22)==2002) .and. &
                      (ind(34)<0 .or. ind(34)==2002) .and. &
                      (ind(46)<0 .or. ind(46)==2002) ) then

!           --- beta is constant (per element)

               jzero = jwcons

            else

!           --- General

               jzero = 2

            end if

         end if

      end if

      if ( kchois==0 .and. vector .or. kchois==2 ) then

!     --- Vector = true, the element vector must be computed
!                       or kchois = 2, the first parameter must be computed
!         Compute jconsf

         if ( kchois==0 ) then

            icoef = 11
            icoefs = 23

         else

            icoef = 1
            icoefs = 2

         end if
         if ( ind(icoef)==0 .and. ind(icoefs)==0 ) then

!        --- F = 0, no right-hand side

            notvec = 0

         else if ( (ind(icoef)<0 .or. ind(icoef)==2002) .and. &
                   (ind(icoefs)<0 .or. ind(icoefs)==2002) ) then

!        --- F is constant (per element)

            jconsf = jwcons

         else

!        --- General

            jconsf = 2

         end if

      end if

      if ( jheat>0 ) then

!     --- jheat>0, i.e. compute mass matrix

         if ( ind(12)==0 .or. ind(12)<0 .and. array(12*m)<=0 .or. &
              ind(24)==0 .or. ind(24)<0 .and. array(24*m)<=0 ) &
              then

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 771, 3, 0, 0 )

         else if ( ind(12)<0 .or. ind(12)==2002 .or. &
                   ind(24)<0 .or. ind(24)==2002 .or. &
                   ind(36)<0 .or. ind(36)==2002 .or. &
                   ind(48)<0 .or. ind(48)==2002 ) then
            jheat = jwcons + 10*(jheat-1)
         else
            jheat = 2 + 10*(jheat-1)
         end if

      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( matrix .and. kchois==0 .or. kchois>=2 ) then
         isubcf = isub
      else
         isubcf = 11
      end if
      ncoef = 2*ncoef

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do

      if ( icheli>1 .and. icheli/=7 .or. icheld>0 .and. &
           icheld/=21 ) ind(lengthmst) = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( jelp==4 .and. jcart==1 .and. &
           icheld<=0 .and. irule==1 ) kstep = 2

      if ( ichtrn>0 ) then

!     --- ichtrn>0, special integral

         if ( jelp/=1 ) then
            call errint ( -icheld, 1 )
            call errint ( jelp, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1863, 5, 0, 0 )
         else
            jtrans = jtrans + 100*ichtrn
         end if

      end if
      if ( ierror/=0 ) call instop

1000  call erclos ( 'el0808' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0808'

      end if  ! ( debug )

      end
