      subroutine el100132 ( icheld, dudx, elemvc, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-08-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    actual filling of derivatives and weights in elemvc and elemwg
!    with respect to subroutine ELD800
!    icheld = 32, 33, 35: gradient no weights
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ndim
      double precision dudx(m,ndimlc), elemvc(ndimlc,n)

!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    Definitions:
!                    32:   grad u
!                    33:   - grad u
!                    35:   elemvc(1,*)=du/dy   elemvc(2,*)=-du/dx (2D only)
!     ndim     i     Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision h

!     h        Help variable to store a constant temporarily
!     i        General loop variable
!     j        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( jdercn==0 ) then

!     --- Derivatives are constant

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = dudx(1,j)
            end do
         end do

      else if ( ndim==2 .and. n>=6 ) then

!     --- quadratic elements in R2

         do j = 1, ndimlc
            do i = 1, n-1, 2
               elemvc(j,i) = dudx(i,j)
            end do
            do i = 2, n, 2
               elemvc(j,i) = dudx(i,j)
            end do
            elemvc(j,n) = dudx(n,j)
         end do

      else

!     --- General element

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = dudx(i,j)
            end do
         end do

      end if

      if ( icheld==33 ) then

!     --- icheld = 33,  change sign

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = - elemvc(j,i)
            end do
         end do

      else if ( icheld==35 ) then

!     --- icheld = 35, change sign and sequence  (2D only)

         do i = 1, n
            h = elemvc(1,i)
            elemvc(1,i) = elemvc(2,i)
            elemvc(2,i) = - h
         end do

      end if

      end
