      subroutine el1002 ( ishift, jshift, jwcons, jelp, iqmat, array, &
                          ndim )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date 16-07-1998 Additional parameter iqmat (BvR)
!        version 1.1    date 11-01-1997 Extension of types of basis functions
!        version 1.0    date 12-12-1995
!
!
!
!   copyright (c) 1995-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute jsecnd with respect to the diffusion terms
!    Help subroutine for el08xx
! **********************************************************************
!
!                       KEYWORDS
!
!    elliptic_equation
!     initialization
!    parabolic_equation
! **********************************************************************
!
!                       COMMON BLOCKS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishift, jshift, jwcons, jelp, iqmat, ndim
      double precision array(m,*)

!     array      i    Work array for the element subroutines, containing
!                     the values of the coefficients in the integration points
!     iqmat      i    Indication if a transformation to a local axis has to
!                     be performed on the diffusion tensor
!     ishift     i    Shift of coefficient index
!                     The coefficients 1+ishift to 6+ishift are considered
!     jelp       i    Indication of the type of basis function subroutine must
!                     be called by ELxxxx. Possible vaues:
!                      1:  Subroutine ELP610  (Linear line element)
!                      2:  Subroutine ELP611  (Quadratic line element)
!                      3:  Subroutine ELP620  (Linear triangle)
!                      4:  Subroutine ELP624  (Extended quadratic triangle)
!                      5:  Subroutine ELP621  (Bilinear quadrilateral)
!                      6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                     11:  Subroutine ELP630  (Linear tetrahedron)
!                     12:  Subroutine ELP631  (Quadratic tetrahedron)
!                     13:  Subroutine ELP632  (Trilinear brick)
!                     14:  Subroutine ELP633  (Triquadratic brick)
!     jshift     i    Shift with respect to error messages
!                     ishift=0: jshift=5
!                     ishift=12: jshift=8
!     jwcons     i    Indication if the weight function is constant
!                     (1) or not (2)
!     ndim       i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, in1, in2, in3, in4, in5, in6

!     i          Counting variable
!     in1        Sequence number of first coefficient
!     in2        Sequence number of secondt coefficient
!     in3        Sequence number of third coefficient
!     in4        Sequence number of fourth coefficient
!     in5        Sequence number of fifth coefficient
!     in6        Sequence number of sixth coefficient
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRWAR  Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      70   alpha11 = 0 or alpha22 = 0
!    1768   Coefficient is filled that is not used
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      call eropen ( 'el1002' )
      in1 = 1+ishift
      in2 = 2+ishift
      in3 = 3+ishift
      in4 = 4+ishift
      in5 = 5+ishift
      in6 = 6+ishift
      if ( (jelp.le.2 .and. iqmat.eq.0) .or. &
           (ndim.eq.1 .and. iqmat.gt.0) ) then

!     --- ndim = 1, one-dimensional elements

         do i = in2, in6
            if ( ind(i).ne.0 ) then
               call errint ( i+jshift, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               ind(i) = 0
            end if
         end do

         if ( ind(in1).eq.0 ) then

!        --- alpha 11 is zero

            call errwar ( 70, 0, 0, 0)

         end if
         if ( ind(in1).lt.0 ) then
            jsecnd = jwcons
         else if ( ind(in1).eq.0 ) then
            jsecnd = 0
         else
            jsecnd = 2
         end if

      else if ( (jelp.gt.2 .and. jelp.le.10 .and. iqmat.eq.0) .or. &
                (ndim.eq.2 .and. iqmat.gt.0)  ) then

!     --- ndim = 2
!         First check if alpha  , and alpha   are unequal to zero
!                             11           22

         do i = in3, in5, 2
            if ( ind(i).ne.0 ) then
               call errint ( i+jshift, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               ind(i) = 0
            end if
         end do

         if ( ind(in6).ne.0 ) then
            call errint ( 6+jshift, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )
            ind(in6) = 0
         end if

         if ( ind(in1).eq.0 .or. ind(in4).eq.0 ) then

!        --- alpha 11 or alpha 22 are zero

            call errwar ( 70, 0, 0, 0)

         end if

!        --- compute jsecnd

         if ( ind(in2).eq.0 ) then

!        --- alpha 12 = 0

            if ( ind(in1).eq.0 .and. ind(in4).eq.0 ) then

!           --- alpha = 0

               jsecnd = 0

            else if ( ind(in1).le.0 ) then

!           --- alpha 11 is constant

               if ( ind(in4).eq.ind(in1) ) then

!              --- alpha 11 = alpha 22

                  jsecnd = jwcons

               else if ( ind(in4).le.0 ) then

!              --- alpha 22 is constant

                  if ( abs(array(1,in1)-array(1,in4)).gt. &
                       sqreps*abs(array(1,in1)) ) then

!                 --- alpha 11 # alpha 22

                     jsecnd = 4

                  else

!                 --- alpha 11 = alpha 22

                     jsecnd = jwcons

                  end if

               else

!              --- alpha 22 is variable

                  jsecnd = 4

               end if

            else if ( ind(in1).eq.ind(in4) .and. &
                      mst(in1).eq.mst(in4) ) then

!           --- alpha 11 = alpha 22  variable

               jsecnd = 2

            else

!           --- general, except for alpha 12 = 0

               jsecnd = 4

            end if

         else

!        --- alpha 12 # 0

            if ( ind(in1).le.0 ) then

!           --- alpha 11 = constant

               if ( ind(in4).eq.ind(in1) ) then

!              --- alpha 11 = alpha 22

                  jsecnd = 3

               else if ( ind(in4).le.0 ) then

!              --- alpha 22 is constant

                  if ( abs( array(1,in1)-array(1,in4)).gt. &
                       sqreps*abs(array(1,in1)) ) then

!                 --- alpha 11 # alpha 22

                     jsecnd = 4

                  else

!                 --- alpha 11 = alpha 22

                     jsecnd = 3

                  end if

               else

!              --- alpha 22 is variable

                  jsecnd = 4

               end if

            else

!           --- alpha 11 is variable

               if ( ind(in1).eq.ind(in4) .and. &
                    mst(in1).eq.mst(in4) ) then

!              --- alpha 11 = alpha 22  variable

                  jsecnd = 3

               else

!              --- general

                  jsecnd = 4

               end if

            end if

         end if

!        --- Reorganize array and ind and mst for alpha 22

         ind(in3) = ind(in4)
         mst(in3) = mst(in4)
         ivecnums(in3) = ivecnums(in4)
         do i = 1, m
            array(i,in3) = array(i,in4)
         end do
         ind(in4) = 0
         mst(in4) = 0
         ivecnums(in4) = 0

      else

!     --- ndim = 3
!         First check if alpha  , alpha   and alpha   are unequal to zero
!                             11       22          33

         if ( ind(in1).eq.0 .or. ind(in4).eq.0 .or. &
              ind(in6).eq.0 ) then

!        --- alpha 11 = alpha 22 = alpha 33 = 0

            call errwar ( 70, 0, 0, 0 )

         end if
         if ( ind(in2).eq.0 .and. ind(in3).eq.0 .and. &
              ind(in5).eq.0 ) then

!        --- alpha 12 = alpha 13 = alpha23 = 0

            if ( ind(in1).lt.0 ) then

!           --- alpha 11 constant

               if ( ind(in4).eq.ind(in1) .and. &
                    ind(in4).eq.ind(in6) ) then

!              --- alpha 11 = alpha 22 = alpha 33 (constant)

                  jsecnd=jwcons

               else if ( ind(in4).lt.0 .and. &
                         ind(in6).lt.0 ) then

!              --- alpha 22 and alpha 33 constant

                  if ( abs(array(1,in1)-array(1,in4)).gt. &
                       sqreps*abs(array(1,in1)) .or. &
                       abs(array(1,in1)-array(1,in6)).gt. &
                       sqreps*abs(array(1,in1)) ) then

!                 --- NOT: alpha 11 = alpha 22 = alpha 33

                     jsecnd = 4

                  else

!                 --- alpha 11 = alpha 22 = alpha 33

                     jsecnd = jwcons

                  end if

               else

!              --- NOT: alpha 11 = alpha 22 = alpha 33 (constant)

                  jsecnd = 4

               end if

            else if ( ind(in1).eq.ind(in4) .and. &
                      ind(in1).eq.ind(in6) .and. &
                      mst(in1).eq.mst(in4) .and. &
                      mst(in1).eq.mst(in6) ) then

!           --- alpha 11 variable, alpha 11 = alpha 22 = alpha 33

               jsecnd = 2
               if ( ind(in1).eq.0 ) jsecnd = 0

            else

!           --- NOT: alpha 11 = alpha 22 = alpha 33

               jsecnd = 4

            end if

         else

!        --- cross terms alpha 12 etc not all zero

            if ( ind(in1).lt.0 ) then

!           --- alpha 11 constant

               if ( ind(in4).eq.ind(in1) .and. &
                    ind(in6).eq.ind(in1) ) then

!              --- alpha 11 = alpha 22 = alpha 33 (constant)

                  jsecnd = 3

               else if ( ind(in4).lt.0 .and. ind(in6).lt.0 ) then
                  if ( abs(array(1,in1)-array(1,in4)).gt. &
                       sqreps*abs(array(1,in1)) .or. &
                       abs(array(1,in1)-array(1,in6)).gt. &
                       sqreps*abs(array(1,in1)) ) then

!                 --- NOT:  alpha 11 = alpha 22 = alpha 33 (constant)

                     jsecnd=4

                  else

!                 --- alpha 11 = alpha 22 = alpha 33 (constant)

                     jsecnd = 3

                  end if
               else

!              --- General case

                  jsecnd = 4

               end if

            end if

         end if

      end if

      call erclos ( 'el1002' )
      end
