      subroutine el3009 ( vecloc, phi, ndecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.8  date 10-03-2013 Set quadratic for ishape = 2
!        version  2.7  date 12-12-2012 Long integers
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array vecloc with the values of all arrays stored in vecoldlc
!    in the integration points of one element
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndecl
      double precision vecloc(numold,maxunk,*), &
                       phi(ndecl,*)

!     ndecl   i      Number of nodal points
!     phi     i      Array of length n*m in which the values of the basis
!                    functions in the integration points are stored.
!                                         k
!                       phi(i,k) = phi  (x )     (i=1(1)n; k=1(1)m)
!                                     i
!     vecloc  o      In this array the values of the function in the integration
!                    points is stored.
!                    vecloc(i,j,k) corresponds to the kth integration point,
!                    the ith vector and the jth unkwon corresponding to that
!                    vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, ivec, ndegfd, j, istart
      double precision, pointer :: vecoldlc(:)
      double precision save1, work(27)
      logical centroid, centr, vertex, quadrat

!     centr          If true the element contains a centroid
!     centroid       If true the function is available in the centroid
!     istart         Starting position of node in index3
!     ivec           Sequence number of vector in vecoldlc
!     j              General loop variable (over degrees of freedom)
!     k              General loop variable (over integration points)
!     ndegfd         Number of degrees of freedom per point
!     quadrat        If true the elements are quadratic
!     save1          Help variable to store a constant
!     vertex         If true not only the vertices have been filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1774   Only constant numbers of degrees of freedom permitted
!    1792   FUNCC3 may only be called in the combination with NUMOLD>0
! **********************************************************************
!
!                       PSEUDO CODE
!
!   If ( maxunk=0 ): error
!   For ivec := 1 (1) numold do
!       if ( index4n(inpelm+1)<1 or>2 ) then
!          Error: not yet implemented
!       else if ( index4n(inpelm+1) = 1 ) then
!          ndegfd := 1
!       else
!          ndegfd := index4n(1)
!       For j := 1 (1) ndegfd do
!          if ( index4n(inpelm+1) = 1 ) then
!             work(k) := vecoldlc(index3n(k))
!          else ( index4n(inpelm+1) = 2 )
!             work(k) := vecoldlc(index3n((k-1)*ndegfd+j)) (k:=1(1)inpelm)
!          if ( jdiag = 1 ) then
!             vecloc(ivec,j,k) := work(k)     (k:=1(1)inpelm)
!          else ( jdiag # 1 )
!                                  n
!             vecloc(ivec,j,k) := sum work(i) phi (k)      (k:=1(1)m)
!                                 i=1            i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3009' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3009'
         write(irefwr,1) 'maxunk, numold, ishape, n, m,inpelm,jdiag', &
                          maxunk, numold, ishape, n, m,inpelm,jdiag
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Loop over old vectors

      if ( maxunk==0 ) then

!     --- Error 1792:  FUNCC3 may only be used if numold>0 in the call of
!                      BUILD

         call errsub ( 1792, 0, 0, 0 )
         go to 1000

      end if
      centr = ishape==2 .or. ishape>=6 .and. ishape<=10 .or. &
              ishape==14 .or. ishape>=16 .and. ishape<=18
      quadrat = ishape==2 .or. ishape==4 .or. ishape==6 .or. &
                ishape==7 .or. ishape==12 .or. &
                ishape>=14 .and. ishape<=16
      if ( debug ) write(irefwr,*) 'centr, quadrat ', centr, quadrat
      centroid = .true.
      vertex = .true.
      do ivec = 1, numold

         iseqtype = oldsols(ivec)%iseqtype
         if ( iseqtype<=0 ) go to 500

         index3n => indoldsols(iseqtype)%index3
         index4n => indoldsols(iseqtype)%index4
         vecoldlc => oldsols(ivec)%oldvecs

         if ( debug ) then
            write(irefwr,*) 'ivec, index4n(inpelm+1)', &
                             ivec, index4n(inpelm+1)
         end if

         if ( index4n(inpelm+1)==-2 ) then

!        --- Vector has not been filled: skip

            ndegfd = 0

         else if ( index4n(inpelm+1)==116 ) then

!        --- Vector of type 116

            ndegfd = index4n(1)
            do j = 1, ndegfd
               save1 = vecoldlc(index3n(j))
               do k = 1, m
                  vecloc(ivec,j,k) = save1
               end do
            end do
            ndegfd = 0

         else if ( index4n(inpelm+1)==126 ) then

!        --- Vector of type 126

            ndegfd = index4n(1)
            do j = 1, ndegfd
               do k = 1, inpelm
                  work(k) = vecoldlc(index3n((k-1)*ndegfd+j))
               end do

!              --- Copy information from array work into vecloc

               if ( jdiag==1 ) then

!              --- jdiag = 1, integration points are nodal points

                  do k = kstep, inpelm, kstep
                     vecloc(ivec,j,k) = work(k)
                  end do

               else

!              --- jdiag = 2, integration points and nodal points are different

                  if ( n==inpelm+1 ) then
                     if ( n==7 ) then
                        work(7) = (-work(1)-work(3)-work(5) + &
                                   4d0*(work(2)+work(4)+work(6)) )/ 9d0
                     else if ( n==4 ) then
                        work(4) = (work(1)+work(2)+work(3))/3d0
                     else if ( n==5 ) then
                        work(5) = (work(1)+work(2)+work(3)+work(4))* &
                                  0.25d0
                     end if
                  end if

                  do k = 1, m
                     vecloc(ivec,j,k) = ddot ( n, work, 1, phi(1,k), 1 )
                  end do

               end if
            end do
            ndegfd = 0

         else if ( index4n(inpelm+1)==129 ) then

!        --- Vector of type 129

            ndegfd = index4n(1)
            if ( debug ) &
               write(irefwr,1) 'ndegfd, m, index4n(2)', &
                                ndegfd, m, index4n(2)
            if ( m==index4n(2) ) then

!           --- Standard case integration points available

               do j = 1, ndegfd
                  do k = 1, m
                     vecloc(ivec,j,k) = &
                        vecoldlc(index3n((k-1)*ndegfd+j))
                  end do
               end do

            else

!           --- Boundary elements, no integration points available

               do j = 1, maxunk
                  do k = 1, m
                     vecloc(ivec,j,k) = 0d0
                  end do
               end do

            end if
            ndegfd = 0

         else if ( index4n(inpelm+1)==-1 ) then

!        --- Number of unknowns per point not constant
!            At this moment only the centroid may have a different number
!            of unknowns per point

            if ( quadrat ) then

!           --- Quadratic elements, check if one of the following three
!               options is present:
!               value is available in all but the centroid point
!               value is available in the vertices
!               not one of these options

               ndegfd = index4n(1)
               if ( ishape==2 ) then

!              --- Quadratic line element, the number of degrees of freedom
!                  is defined by the vertex only

                  centroid = .false.
                  vertex = .false.

               else if ( index4n(2)-index4n(1)<ndegfd ) then

!              --- Quadratic element, the number of degrees of freedom
!                  in the vertices is larger than in the mid points

                  if ( centr ) centroid = .false.
                  vertex = .false.

               else if ( centr ) then

!              --- Quadratic element with centroid, &
!                  the number of degrees of freedom in the mid side points
!                  is at least equal to the number of points in the vertices
!                   check the centroid

                  if ( ishape==14 ) then

!                 --- ishape = 14

                     if ( index4n(14)-index4n(13)<ndegfd ) &
                        centroid = .false.
                  else

!                 --- ishape # 14

                     if ( index4n(inpelm)- &
                          index4n(inpelm-1)< &
                          ndegfd ) centroid = .false.
                  end if

               end if

            else if ( centr ) then

!           --- centroid is last point

               ndegfd = index4n(1)
               if ( ndegfd<=0 ) then
                  call errint ( ivec, 1 )
                  call errsub ( 1774, 1, 0, 0 )
                  go to 1000
               end if
               centroid = .false.
               if ( index4n(inpelm)-index4n(inpelm-1)>= &
                    ndegfd ) centroid = .true.

            else

!           --- Error 1774:  Only constant number of unknowns per point are
!                            permitted

               call errint ( ivec, 1 )
               call errsub ( 1774, 1, 0, 0 )
               go to 1000

            end if

         else if ( index4n(inpelm+1)<1 .or. &
                   index4n(inpelm+1)>2 ) then

!        --- Error 1774:  Only constant number of unknowns per point are
!                         permitted

            call errint ( ivec, 1 )
            call errsub ( 1774, 1, 0, 0 )
            go to 1000

         else if ( index4n(inpelm+1)==1 ) then

!        --- index4n(inpelm+1) = 1, one unknown in each point

            ndegfd = 1

         else

!        --- index4n(inpelm+1) = 2, constant number of unknowns in each
!            point

            ndegfd = index4n(1)

         end if

!        --- Loop over degrees of freedom

         if ( debug ) write(irefwr,*) 'ndegfd, vertex, centroid ', &
                                       ndegfd, vertex, centroid
         do j = 1, ndegfd

            if ( .not. vertex ) then

!           --- unknown only available in the vertices

               if ( ishape==2 ) then

!              --- quadratic line element

                  istart = 0
                  do k = 1, inpelm, 2
                     work(k) = vecoldlc(index3n(istart+j))
                     istart = index4n(2)
                  end do
                  work(2) = 0.5d0 * ( work(1)+work(3) )

               else if ( ishape<=7 ) then

!              --- quadratic surface element

                  istart = 0
                  do k = 1, inpelm-1, 2
                     work(k) = vecoldlc(index3n(istart+j))
                     istart = index4n(k+1)
                  end do
                  do k = 2, inpelm-2, 2
                     work(k) = 0.5d0 * ( work(k-1)+work(k+1) )
                  end do
                  if ( ishape==6 ) then
                     work(8) = 0.5d0 * ( work(1)+work(7) )
                     work(9) = 0.25d0*(work(2)+work(4)+work(6)+work(8))
                  else if ( ishape==4 ) then
                     work(6) = 0.5d0 * ( work(1)+work(5) )
                  else
                     work(6) = 0.5d0 * ( work(1)+work(5) )
                     work(7) = (work(1)+work(3)+work(5))/3d0
                  end if

               else if ( ishape==12 ) then

!              --- quadratic tetrahedron

                  istart = 0
                  do k = 1, 5, 2
                     work(k) = vecoldlc(index3n(istart+j))
                     istart = index4n(k+1)
                  end do
                  istart = index4n(9)
                  work(10) = vecoldlc(index3n(istart+j))
                  work(2) = 0.5d0 * ( work(1)+work(3) )
                  work(4) = 0.5d0 * ( work(3)+work(5) )
                  work(6) = 0.5d0 * ( work(1)+work(5) )
                  work(7) = 0.5d0 * ( work(1)+work(10) )
                  work(8) = 0.5d0 * ( work(3)+work(10) )
                  work(9) = 0.5d0 * ( work(5)+work(10) )

               else if ( ishape==14 ) then

!              --- quadratic hexahedron

                  istart = 0
                  work(1) = vecoldlc(index3n(istart+j))
                  istart = index4n(2)
                  work(3) = vecoldlc(index3n(istart+j))
                  istart = index4n(6)
                  work(7) = vecoldlc(index3n(istart+j))
                  istart = index4n(8)
                  work(9) = vecoldlc(index3n(istart+j))
                  istart = index4n(18)
                  work(19) = vecoldlc(index3n(istart+j))
                  istart = index4n(20)
                  work(21) = vecoldlc(index3n(istart+j))
                  istart = index4n(24)
                  work(25) = vecoldlc(index3n(istart+j))
                  istart = index4n(26)
                  work(27) = vecoldlc(index3n(istart+j))
                  work(2) = 0.5d0 * ( work(1)+work(3) )
                  work(4) = 0.5d0 * ( work(1)+work(7) )
                  work(6) = 0.5d0 * ( work(3)+work(9) )
                  work(8) = 0.5d0 * ( work(7)+work(9) )
                  work(5) = 0.25d0*(work(2)+work(4)+work(6)+work(8))
                  work(20) = 0.5d0 * ( work(19)+work(21) )
                  work(22) = 0.5d0 * ( work(19)+work(25) )
                  work(24) = 0.5d0 * ( work(21)+work(27) )
                  work(26) = 0.5d0 * ( work(25)+work(27) )
                  work(23) =0.25d0*(work(20)+work(22)+work(24)+work(26))
                  work(10) = 0.5d0 * ( work(1)+work(19) )
                  work(12) = 0.5d0 * ( work(3)+work(21) )
                  work(16) = 0.5d0 * ( work(7)+work(25) )
                  work(18) = 0.5d0 * ( work(9)+work(27) )
                  work(11) = 0.5d0 * ( work(10)+work(12) )
                  work(13) = 0.5d0 * ( work(10)+work(16) )
                  work(15) = 0.5d0 * ( work(12)+work(18) )
                  work(17) = 0.5d0 * ( work(16)+work(18) )
                  work(14) =0.25d0*(work(11)+work(13)+work(15)+work(17))

               else

!              --- Error 1774:  Only constant number of unknowns per point
!                               are permitted

                  call errint ( ivec, 1 )
                  call errsub ( 1774, 1, 0, 0 )
                  go to 1000

               end if

            else if ( .not. centroid ) then

!           --- unknown available in all points except centroid

               if ( ishape/=14 ) then

!              --- ishape # 14

                  istart = 0
                  do k = 1, inpelm-1
                     work(k) = vecoldlc(index3n(istart+j))
                     istart = index4n(k)
                  end do
                  if ( inpelm==4 ) then
                     work(4) = (work(1)+work(2)+work(3))/3d0
                  else if ( inpelm==5 ) then
                     work(5) = 0.25d0*(work(1)+work(2)+work(3)+work(4))
                  else if ( inpelm==7 ) then
                     work(7) = (-work(1)-work(3)-work(5) + &
                               4d0*(work(2)+work(4)+work(6)) )/ 9d0
                  else if ( ishape==6 ) then
                     work(9) = 0.25d0*(work(2)+work(4)+work(6)+work(8))
                  else if ( ishape==17 ) then
                     work(9) = 0.125d0*(work(1)+work(2)+work(3)+work(4)+ &
                                        work(5)+work(6)+work(7)+work(8))
                  else

!                 --- Error 1774:  Only constant number of unknowns per point
!                                  are permitted

                     call errint ( ivec, 1 )
                     call errsub ( 1774, 1, 0, 0 )
                     go to 1000

                  end if

               else

!              --- ishape = 14

                  istart = 0
                  do k = 1, inpelm
                     work(k) = vecoldlc(index3n(istart+j))
                     istart = index4n(k)
                  end do
                  work(14) = 0.125d0*(work(1)+work(3)+work(7)+work(9)+ &
                             work(19)+work(21)+work(25)+work(27))
               end if

            else

!           --- Standard case, unknown available in all points

               istart = 0
               do k = 1, inpelm
                  work(k) = vecoldlc(index3n(istart+j))
                  istart = index4n(k)
               end do

            end if
            if ( index4n(inpelm+1)==-1 ) then
               istart = 0
               do k = 1, inpelm-1
                  work(k) = vecoldlc(index3n(istart+j))
                  istart = index4n(k)
               end do
               if ( centroid ) then
                  if ( istart+j>index4n(inpelm) ) then
                     work(inpelm) = 0d0
                  else
                     work(inpelm) = vecoldlc(index3n(istart+j))
                  end if  ! ( istart+j>index4n(inpelm) )
               end if  ! ( centroid )
            else if ( index4n(inpelm+1)==1 ) then

!           --- index4n(inpelm+1) = 1, one unknown in each point

               do k = 1, inpelm
                  work(k) = vecoldlc(index3n(k))
               end do

            else

!           --- index4n(inpelm+1) = 2, constant number of unknowns in each
!               point

               do k = 1, inpelm
                  work(k) = vecoldlc(index3n((k-1)*ndegfd+j))
               end do

            end if

!           --- Copy information from array work into vecloc

            if ( jdiag==1 ) then

!           --- jdiag = 1, integration points are nodal points

               do k = kstep, inpelm, kstep
                  vecloc(ivec,j,k) = work(k)
               end do
               if ( n==inpelm+1 ) then
                  if ( n==7 ) then
                     work(7) = (-work(1)-work(3)-work(5) + &
                                4d0*(work(2)+work(4)+work(6)) )/ 9d0

!                    --- in case of overshoot or undershoot we use
!                        linear interpolation

                     if ( work(7)<minval(work(1:6))*0.999d0 .or. &
                          work(7)>maxval(work(1:6))*1.001d0 ) &
                          work(7) = (work(2)+work(4)+work(6))/3d0
                        vecloc(ivec,j,7) = work(7)
                  else if ( n==4 ) then
                     vecloc(ivec,j,4) = (work(1)+work(2)+work(3))/3d0
                  else if ( n==5 ) then
                     vecloc(ivec,j,5) = &
                        (work(1)+work(2)+work(3)+work(4))*0.25d0
                  end if
               end if

            else

!           --- jdiag = 2, integration points and nodal points are different

               if ( n==inpelm+1 ) then
                  if ( n==7 ) then
                     work(7) = (-work(1)-work(3)-work(5) + &
                                4d0*(work(2)+work(4)+work(6)) )/ 9d0
                  else if ( n==4 ) then
                     work(4) = (work(1)+work(2)+work(3))/3d0
                  else if ( n==5 ) then
                     work(5) = (work(1)+work(2)+work(3)+work(4))*0.25d0
                  end if
               end if

               do k = 1, m
                  vecloc(ivec,j,k) = ddot ( n, work, 1, phi(1,k), 1 )
               end do

            end if

         end do

500   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el3009'

      end if
      call erclos ( 'el3009' )

      end
