      subroutine el3010 ( elemmt, elemmtloc, nelarge, nesmall, if, jf, &
                          istep, transp, rmul )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 27-02-2010 Debug statements
!        version  3.2  date 15-01-2000 Adaptation because of incorrect
!                                      optimization of GNU-compiler
!        version  3.1  date 04-06-1999 Include commons
!        version  3.0  date 16-07-1993 Extra parameter rmul
!
!   copyright (c) 1993-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill small matrix elemmtloc of size nesmall x nesmall into larger matrix
!    elemmt of size nelarge x nelarge.
!    Also the transpose of a matrix may be filled
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelarge, nesmall, if, jf, istep
      double precision elemmtloc(nesmall,nesmall), &
                       elemmt(nelarge,nelarge), rmul
      logical transp

!     elemmt         o    Large element matrix to be filled
!     elemmtloc      i    Local element matrix to be put into elemmt
!     if             i    First row in which elemmtloc must be put
!     istep          i    Step size of elemmtloc in elemmt
!     jf             i    First column in which elemmtloc must be put
!     nelarge        i    Size of large matrix elemmt
!     nesmall        i    Size of small matrix elemmtloc
!     rmul           i    Multiplication factor
!     transp         i    Indicates if the transpose of the matrix must be
!                         filled (true) or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      logical mult

!     i              Counting variable
!     j              Counting variable
!     mult           Indication whether multiplication must take place or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   -
! **********************************************************************
!
!                       PSEUDO CODE
!
!   elemmt(if+i*istep,jf+j*istep) := elemmtloc(i+1,j+1)  ,i,j,=0(1)nesmall-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3010' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3010'
         call ertrace
         write(irefwr,1) 'jadd, nesmall, if, istep, jf, nelarge', &
                          jadd, nesmall, if, istep, jf, nelarge
         write(irefwr,2) 'rmul', rmul
         call prinrl1 ( elemmtloc, nesmall, nesmall, 'elemmtloc' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mult = abs(1d0-rmul)>1d-8
      if ( debug ) write(irefwr,*) 'mult, transp ', mult, transp

      if ( jadd==0 .and. .not. mult ) then

!     --- Standard sitation, no multiplication, no adding

         if ( transp ) then

!        --- transpose of matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmtloc(j+1,i+1)
               end do
            end do

         else

!        --- matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmtloc(i+1,j+1)
               end do
            end do
         end if

      else if ( jadd==1 .and. .not. mult ) then

!     --- no multiplication, but matrix is added to existing one

         if ( transp ) then

!        --- transpose of matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmt(if+i*istep, jf+j*istep) + &
                  elemmtloc(j+1,i+1)
               end do
            end do

         else

!        --- matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmt(if+i*istep, jf+j*istep) + &
                  elemmtloc(i+1,j+1)
               end do
            end do
         end if

      else if ( jadd==0 ) then

!     --- multiplication by constant, no adding

         if ( transp ) then

!        --- transpose of matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmtloc(j+1,i+1)*rmul
               end do
            end do

         else

!        --- matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmtloc(i+1,j+1)*rmul
               end do
            end do
         end if

      else

!     --- multiplication, and matrix is added to existing one

         if ( transp ) then

!        --- transpose of matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmt(if+i*istep, jf+j*istep) + &
                  elemmtloc(j+1,i+1)*rmul
               end do
            end do

         else

!        --- matrix

            do j = 0, nesmall-1
               do i = 0, nesmall-1
                  elemmt(if+i*istep, jf+j*istep) = &
                  elemmt(if+i*istep, jf+j*istep) + &
                  elemmtloc(i+1,j+1)*rmul
               end do
            end do
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el3010'

      end if  ! ( debug )
      call erclos ( 'el3010' )

      end
