      subroutine el6004 ( alpha, s, w1, w2, d1, d2, det, p1, p2, p3, &
                          p4, work, modecl, mdecl, ichois)
! ======================================================================
!
!        programmer     Patrick Anderson / Maykel Verschueren
!        version 1.0    date  11-12-1997
!
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the diffusive part of the equation
!     EL6019 is called by the element subroutine ELM606
!            it must be preceded by the subroutines ELP640 and ELP642
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduletime
      implicit none
      integer modecl, mdecl, ichois
      double precision alpha(mdecl,26), s(mdecl,mdecl), w1(modecl), &
                       w2(modecl),  d1(modecl,modecl), &
                       d2(modecl,modecl), det(modecl,modecl), &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl), &
                       work(mdecl)

!     alpha    i   array of length mdecl x 3 containing the values of the
!                  first three coefficients in the integration points
!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     d2       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the y-direction
!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     ichois   i
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     p1       i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   see p1
!     p3       i   see p1
!     p4       i   see p1
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     w2       i   array of length modecl containing the weights for
!                  numerical integration in the y-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, ij, kl, kj, lj, il, ik
      double precision sum, h1, h2, h3, h4, help

!     h1         help variable to compute a product
!     h2         help variable to compute a product
!     h3         help variable to compute a product
!     h4         help variable to compute a product
!     help       help variable
!     i          Counting variable
!     ij         position in the matrix s
!     ik         position in the matrix s
!     il         help parameter
!     j          Counting variable
!     k          Counting variable
!     kj         position in the matrix s
!     kl         position in the matrix s
!     l          General loop variable
!     lj         position in the matrix s
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- first initialize s as zero

      do i = 1, mdecl
         do j = 1, mdecl
            s(i,j) = 0d0
         end do
      end do

      if (ichois.eq.1) then

!     --- compute contribution to large matrix:   |M| - tstep*v_n*|D|
!         summation over the integration points
!         --- store -tstep*v_n*|D1|
!         --- store first help factor in work

         do i = 1, modecl
            ij = i
            do j = 1, modecl
               help = alpha(ij,2)*p4(i,j) - alpha(ij,3)*p3(i,j)
               work(ij) = -tstep*help*w1(i)*w1(j)
               ij = ij + modecl
            end do
         end do

!        --- add first factor of convection matrix to s

         do i = 1, modecl
            ij = i
            do j = 1, modecl
               do k = 1, modecl
                  kj = k + (j - 1)*modecl
                  s(kj,ij) = s(kj,ij) + work(ij)*d1(i,k)
               end do
               ij = ij + modecl
            end do
         end do

!        --- store second help factor in work

         do i = 1, modecl
            ij = i
            do j = 1, modecl
               help = alpha(ij,3)*p1(i,j) - alpha(ij,2)*p2(i,j)
               work(ij) = -tstep*help*w1(i)*w1(j)
               ij = ij + modecl
            end do
         end do

!        --- add second factor of convection matrix to s

         do i = 1, modecl
            ij = i
            do j = 1, modecl
               do k = 1, modecl
                  ik = i + (k - 1)*modecl
                s(ik,ij) = s(ik,ij) + work(ij)*d1(j,k)
               end do
               ij = ij + modecl
            end do
         end do

!        --- Add |M|

        do i = 1, modecl
          do j = 1, modecl
            ij = i + (j-1)*modecl
            s(ij,ij) = s(ij,ij) + w1(i)*w2(j)*det(i,j)
          end do
        end do

      end if

      if (ichois.eq.2) then

!     --- compute contribution to large matrix:   |M|
!         summation over the integration points
!         store first help factor in work

        do i = 1, modecl
          do j = 1, modecl
            ij = i + (j-1)*modecl
            s(ij,ij) = w1(i)*w2(j)*det(i,j)
          end do
        end do

      end if

!     --- compute contribution to large matrix:   tstep*|D|*[L]*|D|
!         summation over the integration points
!         store first help factor in work

      if (ichois.eq.3) then
         do i = 1, modecl

            ij = i
            do j = 1, modecl

               h1 =  p3(i,j)*p3(i,j)
               h2 =  p4(i,j)*p4(i,j)
               h3 =  -p3(i,j)*p4(i,j)
               h4 =  -p4(i,j)*p3(i,j)
               work(ij) = tstep*(h1*alpha(ij,9)+h2*alpha(ij,5)+ &
                    h3*alpha(ij,6)+h4*alpha(ij,8) ) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
            end do
         end do

!        --- add first factor of matrix to s

         do i=1, modecl

            ij = i
            do j = 1, modecl
               do k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
                  end do

                  s(ij,kj) = s(ij,kj) + sum

               end do

               ij = ij + modecl
            end do
         end do

!        --- store second help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               h1 = -p3(i,j)*p1(i,j)
               h2 = -p4(i,j)*p2(i,j)
               h3 =  p3(i,j)*p2(i,j)
               h4 =  p4(i,j)*p1(i,j)
               work(ij) = tstep*(h1*alpha(ij,9)+h2*alpha(ij,5)+ &
                    h3*alpha(ij,6)+h4*alpha(ij,8) ) &
                    * (w1(i)*w2(j)/det(i,j))

               ij = ij + modecl
            end do
         end do

!        --- add second factor of matrix to s

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               do k = 1, modecl

                  kl = k
                  do l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
                  end do
               end do

               ij = ij + modecl
            end do
         end do

!        --- store third help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               h1 = -p1(i,j)*p3(i,j)
               h2 = -p2(i,j)*p4(i,j)
               h3 = p1(i,j)*p4(i,j)
               h4 = p2(i,j)*p3(i,j)
               work(ij) = tstep*(h1*alpha(ij,9)+h2*alpha(ij,5)+ &
                    h3*alpha(ij,6)+h4*alpha(ij,8) ) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
            end do
         end do

!        --- add third factor of matrix to s

         do i = 1, modecl

            ij = i
            do j = 1, modecl
               do k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
                  end do
               end do

               ij = ij + modecl
            end do
         end do

!        --- store fourth help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               h1 = p1(i,j)*p1(i,j)
               h2 = p2(i,j)*p2(i,j)
               h3 =  - p1(i,j)*p2(i,j)
               h4 =  - p2(i,j)*p1(i,j)
               work(ij) = tstep*(h1*alpha(ij,9)+h2*alpha(ij,5)+ &
                    h3*alpha(ij,6)+h4*alpha(ij,8) ) &
                    * (w1(i)*w2(j)/det(i,j))

               ij = ij + modecl
            end do
         end do

!        --- add fourth factor of matrix to s

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               do k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
                  end do

                  s(ij,ik) = s(ij,ik) + sum
               end do

               ij = ij + modecl
            end do
         end do
      end if

      if (ichois.eq.4) then

!     --- compute contribution to large matrix:  |M|*(1-c_old^2) - |S|
!         summation over the integration points
!         store first help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl
               h1 =  p3(i,j)*p3(i,j)
               h2 =  p4(i,j)*p4(i,j)
               h3 =  -p3(i,j)*p4(i,j)
               h4 =  -p4(i,j)*p3(i,j)
               work(ij) = - (h1*alpha(ij,20)+h2*alpha(ij,16)+ &
                    h3*alpha(ij,17)+h4*alpha(ij,19) ) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
            end do
         end do

!        --- add first factor of matrix to s

         do i=1, modecl

            ij = i
            do j = 1, modecl
               do k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
                  end do

                  s(ij,kj) = s(ij,kj) + sum
               end do

               ij = ij + modecl
            end do
         end do

!        --- store second help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl
               h1 = -p3(i,j)*p1(i,j)
               h2 = -p4(i,j)*p2(i,j)
               h3 =  p3(i,j)*p2(i,j)
               h4 =  p4(i,j)*p1(i,j)
               work(ij) = - (h1*alpha(ij,20)+h2*alpha(ij,16)+ &
                    h3*alpha(ij,17)+h4*alpha(ij,19) ) &
                    * (w1(i)*w2(j)/det(i,j))

               ij = ij + modecl
            end do
         end do

!        --- add second factor of matrix to s

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               do k = 1, modecl

                  kl = k
                  do l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
                  end do
               end do

               ij = ij + modecl
            end do
         end do

!        --- store third help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl
               h1 = -p1(i,j)*p3(i,j)
               h2 = -p2(i,j)*p4(i,j)
               h3 = p1(i,j)*p4(i,j)
               h4 = p2(i,j)*p3(i,j)
               work(ij) = - (h1*alpha(ij,20)+h2*alpha(ij,16)+ &
                    h3*alpha(ij,17)+h4*alpha(ij,19) ) &
                    * (w1(i)*w2(j)/det(i,j))

               ij = ij + modecl
            end do
         end do

!        --- add third factor of matrix to s

         do i = 1, modecl

            ij = i
            do j = 1, modecl
               do k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
                  end do
               end do

               ij = ij + modecl
            end do
         end do

!        --- store fourth help factor in work

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               h1 = p1(i,j)*p1(i,j)
               h2 = p2(i,j)*p2(i,j)
               h3 =  - p1(i,j)*p2(i,j)
               h4 =  - p2(i,j)*p1(i,j)
               work(ij) = - (h1*alpha(ij,20)+h2*alpha(ij,16)+ &
                    h3*alpha(ij,17)+h4*alpha(ij,19) ) &
                    * (w1(i)*w2(j)/det(i,j))

               ij = ij + modecl
            end do
         end do

!        --- add fourth factor of matrix to s

         do i = 1, modecl

            ij = i
            do j = 1, modecl

               do k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
                  end do

                  s(ij,ik) = s(ij,ik) + sum
               end do

               ij = ij + modecl
            end do
         end do

        do i = 1, modecl
          do j = 1, modecl
            ij = i + (j-1)*modecl
            s(ij,ij) = s(ij,ij) &
                       + (alpha(ij,15)-alpha(ij,14)*alpha(ij,1)**2d0) &
                       *w1(i)*w2(j)*det(i,j)
          end do
        end do

      end if

      end
