      subroutine el6008 ( smtr, u, v, ux, uy, vx, vy, &
                          w1, d1, det, p1, p2, p3, p4, &
                          work, workar, user, iuser, &
                          mone1, m1, modelv )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.2  date 06-07-1999  Small correction
!        version  1.1  date 10-01-1997  Addition of PTT-term
!        version  1.0  date 20-12-1996  test version
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   In this routine the matrices are calculated for the
!   two-dimensional upper convected maxwell model
! **********************************************************************
!
!                       KEYWORDS
!
!     visco_elastic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mone1, m1, modelv, iuser(*)
      double precision smtr(m1,m1,9),user(*), &
                       u(m1), v(m1), ux(m1), uy(m1), vx(m1), vy(m1)
      double precision w1(mone1), d1(mone1,mone1), &
                       det(mone1,mone1), &
                       p1(mone1,mone1), p2(mone1,mone1), &
                       p3(mone1,mone1), p4(mone1,mone1), &
                       work(m1), workar(m1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jddprv, jzprv
      double precision lambda, xi

!     i       loop parameter
!     j       loop parameter
!     jddprv  keep jadd
!     jzprv   keep jzero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

!     --- Keep jadd and jzero, just not to disturb diff. eqn

      jddprv = jadd
      jzprv  = jzero
      lambda = user(80)
      xi     = user(81)

!     --- Build S11 matr : lambda(u ddx + v ddy) - &
!                          2 lambda ux  + 1
!         +++ Build Helmholtz part

      do 10, i = 1, m1
         work(i) =  - 2 * lambda * ux(i) + 1.0d0
 10   continue

!     --- +++ Add PTT part, if necesary

      if (modelv.eq.2) then
         do 12, i = 1, m1
            work(i) =  work(i) + 2 * xi * ux(i)
 12      continue
      end if

!     --- +++ Initialize S11matr

      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,1), w1, w1, det, mone1, m1)

!     --- +++ Add Convection term to S11

      jadd = 1
      call el6042(u, v, lambda, smtr(1,1,1), w1, &
                  d1, p1, p2, p3, p4, workar, mone1, m1)

!     --- Build S12 matr :  -2 lambda uy
!         +++ Build Helmholtz part

      do 20, i = 1, m1
         work(i) = - 2 * lambda * uy(i)
 20   continue

!     --- +++ Add PTT part, if necesary

      if (modelv.eq.2) then
         do 22, i = 1, m1
            work(i) =  work(i) + xi * (uy(i) + vx(i))
 22      continue
      end if

!     --- +++ Initialize S12matr

      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,2), w1, w1, det, mone1, m1)

!     --- Build S13 matr : 0

      do 40, i = 1, m1
         do 30, j = 1, m1
            smtr(i,j,3) = 0.0d0
 30      continue
 40   continue

!     --- Build S21 matr :  - lambda vx
!         +++ Build Helmholtz part

      do 50, i = 1, m1
         work(i) = - lambda *vx(i)
 50   continue

!     --- +++ Add PTT part, if necesary

      if (modelv.eq.2) then
         do 52, i = 1, m1
            work(i) =  work(i) + xi * ( uy(i) + vx(i) ) /2.0d0
 52      continue
      end if

!     --- +++ Initialize S21matr

      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,4), w1, w1, det, mone1, m1)

!     --- Build S22 matr : lambda(u ddx + v ddy)
!         +++ Build Helmholtz part

      do 60, i = 1, m1
         work(i) =  1.0d0
 60   continue
      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,5), w1, w1, det, mone1, m1)

!     --- +++ Add Convection term to S22

      jadd = 1
      call el6042(u, v, lambda, smtr(1,1,5), w1, &
                  d1, p1, p2, p3, p4, workar, mone1, m1)

!     --- Build S23 matr :  - lambda uy
!         +++ Build Helmholtz part

      do 70, i = 1, m1
         work(i) = - lambda * uy(i)
 70   continue

!     --- +++ Add PTT part, if necesary

      if (modelv.eq.2) then
         do 72, i = 1, m1
            work(i) =  work(i) + xi * ( uy(i) + vx(i) ) /2.0d0
 72      continue
      end if

!     --- +++ Initialize S23matr

      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,6), w1, w1, det, mone1, m1)

!     --- Build S31 matr : 0

      do 90, i = 1, m1
         do 80, j = 1, m1
            smtr(i,j,7) = 0.0d0
 80      continue
 90   continue

!     --- Build S32 matr :  - lambda vx
!         +++ Build Helmholtz part

      do 100, i = 1, m1
         work(i) = - 2.0d0 * lambda *vx(i)
 100   continue

!      --- +++ Add PTT part, if necesary

      if (modelv.eq.2) then
         do 102, i = 1, m1
            work(i) =  work(i) + xi *( uy(i) + vx(i) )
 102     continue
      end if

!     --- +++ Initialize S32matr

      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,8), w1, w1, det, mone1, m1)

!     --- Build S33 matr : lambda(u ddx + v ddy) - &
!                          2 lambda vy
!         +++ Build Helmholtz part

      do 110, i = 1, m1
         work(i) =  - 2 * lambda *vy(i) + 1.0d0
 110  continue

!     --- +++ Add PTT part, if necesary

      if (modelv.eq.2) then
         do 112, i = 1, m1
            work(i) =  work(i) + 2.0d0 * xi * vy(i)
 112     continue
      end if

!     --- +++ Initialize S33matr

      jadd = 0

!     --- +++ Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6022(work, smtr(1,1,9), w1, w1, det, mone1, m1)

!     --- +++ Add Convection term to S33

      jadd = 1
      call el6042(u, v, lambda, smtr(1,1,9), w1, &
                  d1, p1, p2, p3, p4, workar, mone1, m1)

!     --- Restore jadd and jzero to previous values

      jadd  = jddprv
      jzero = jzprv
      end
