      subroutine el6010 ( alpha, s, w1,  d1, det, p1, p2, p3, &
                          p4, p5, p6, p7, p8, p9, work, mod, &
                          md, ichois)
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.0  date 04-01-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     EL6010 is called by the element subroutine ELM615
!            it must be preceded by the subroutines ELP640 and ELP642
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mod, md, ichois
      double precision alpha(mod,mod,mod,26), s(md,md), w1(mod), &
                       d1(mod,mod), det(mod,mod,mod), &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), work(mod,mod,mod)

!     alpha    i   array of length md x 3 containing the values of the
!                  first three coefficients in the integration points
!     d1       i   array of length mod x mod contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     det      i   array of length mod x mod x mod containing the
!                  transformation determinant
!     ichois   i
!     md       i   number of integration points in the element
!     mod      i   number of integration points in one dimension
!     p1       i   arrays of length mod x mod containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   see p1
!     p3       i   see p1
!     p4       i   see p1
!     p5       i   see p1
!     p6       i   see p1
!     p7       i   see p1
!     p8       i   see p1
!     p9       i   see p1
!     s        o   array of length md x md containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length mod containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work         work array of length md, to store products temporarily
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, m, n, ijk, imn, ljn, lmk, &
              imk, ijn, ljk, mtd
      double precision sum, h1, h2, h3, h4, h5, h6

!     h1
!     h2
!     h3
!     h4
!     h5
!     h6
!     i              Counting variable
!     ijk
!     ijn
!     imk
!     imn
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     ljk
!     ljn
!     lmk
!     m
!     mtd
!     n
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      mtd = mod*mod

!     --- first initialize s as zero

      do i = 1, md
         do j = 1, md
            s(i,j) = 0d0
         end do
      end do

      if (ichois.eq.1) then

!     --- compute contribution to large matrix:   |M| - tstep*v_n*|D|
!         summation over the integration points
!         --- store -tstep*v_n*|D1|
!         --- store first help factor in work
!         --- store first help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = alpha(i,j,k,2) * p1 (i,j,k)
                  h2 = alpha(i,j,k,3) * p2 (i,j,k)
                  h3 = alpha(i,j,k,4) * p3 (i,j,k)
                  work(i,j,k) = -tstep*(h1+h2+h3)* &
                                        w1(i)*w1(j)*w1(k)
               end do
            end do
         end do

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     ljk = l + (j-1)*mod + (k-1)*mtd
                     s(ljk,ijk) = s(ljk,ijk) + &
                                  work(i,j,k)*d1(i,l)
                  end do
               end do
            end do
         end do

!        --- store second help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = alpha(i,j,k,2) * p4 (i,j,k)
                  h2 = alpha(i,j,k,3) * p5 (i,j,k)
                  h3 = alpha(i,j,k,4) * p6 (i,j,k)
                  work(i,j,k) = -tstep * (h1+h2+h3)* &
                                          w1(i)*w1(j)*w1(k)
               end do
            end do
         end do

!        --- add second factor of convection matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     ljk = i + (l-1)*mod + (k-1)*mtd
                     s(ljk,ijk) = s(ljk,ijk) + &
                                  work(i,j,k)*d1(j,l)
                  end do
               end do
            end do
         end do

!        --- store third help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = alpha(i,j,k,2) * p7(i,j,k)
                  h2 = alpha(i,j,k,3) * p8(i,j,k)
                  h3 = alpha(i,j,k,4) * p9(i,j,k)
                  work(i,j,k) = -tstep*(h1+h2+h3)* &
                                w1(i)*w1(j)*w1(k)
               end do
            end do
         end do

!        --- add third factor of convection matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     ljk = i + (j-1)*mod + (l-1)*mtd
                     s(ljk,ijk) = s(ljk,ijk) + &
                                  work(i,j,k)*d1(k,l)
                  end do
               end do
            end do
         end do

!        --- Add |M|

        do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  s(ijk,ijk) = s(ijk,ijk) + &
                               w1(i)*w1(j)*w1(k)*det(i,j,k)
               end do
            end do
         end do

      end if

      if (ichois.eq.2) then

!     --- compute contribution to large matrix:   |M|
!         summation over the integration points
!         store first help factor in work

        do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  s(ijk,ijk) =  w1(i)*w1(j)*w1(k)*det(i,j,k)
               end do
            end do
         end do

       end if

      if (ichois.eq.3) then

!     --- compute contribution to large matrix:   tstep*|D|*[L]*|D|
!         summation over the integration points
!         store first help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p1(i,j,k)*p1(i,j,k)
                  h2 = p2(i,j,k)*p1(i,j,k) +  p1(i,j,k)*p2(i,j,k)
                  h3 = p3(i,j,k)*p1(i,j,k) +  p1(i,j,k)*p3(i,j,k)
                  h4 = p2(i,j,k)*p2(i,j,k)
                  h5 = p3(i,j,k)*p2(i,j,k) +  p2(i,j,k)*p3(i,j,k)
                  h6 = p3(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add first factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     sum = 0d0
                     do m = 1, mod
                        sum = sum + work(m,j,k)*d1(m,i)*d1(m,l)
                     end do
                     ljk = l + (j-1)*mod + (k-1)*mtd
                     s(ijk,ljk) = s(ijk,ljk) + sum
                  end do
               end do
            end do
         end do

!        --- store second help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p1(i,j,k)*p4(i,j,k)
                  h2 = p2(i,j,k)*p4(i,j,k) +  p1(i,j,k)*p5(i,j,k)
                  h3 = p3(i,j,k)*p4(i,j,k) +  p1(i,j,k)*p6(i,j,k)
                  h4 = p2(i,j,k)*p5(i,j,k)
                  h5 = p3(i,j,k)*p5(i,j,k) +  p2(i,j,k)*p6(i,j,k)
                  h6 = p3(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add second factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do m = 1, mod
                        lmk = l + (m-1)*mod + (k-1)*mtd

!                       --- add second factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(i,m,k)*d1(i,l)*d1(m,j)
                     end do
                  end do
               end do
            end do
         end do

!        --- store third help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p4(i,j,k)*p1(i,j,k)
                  h2 = p5(i,j,k)*p1(i,j,k) +  p4(i,j,k)*p2(i,j,k)
                  h3 = p6(i,j,k)*p1(i,j,k) +  p4(i,j,k)*p3(i,j,k)
                  h4 = p5(i,j,k)*p2(i,j,k)
                  h5 = p6(i,j,k)*p2(i,j,k) +  p5(i,j,k)*p3(i,j,k)
                  h6 = p6(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add third factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do m = 1, mod
                        lmk = l + (m-1)*mod + (k-1)*mtd

!                       --- add third factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(l,j,k)*d1(j,m)*d1(l,i)
                     end do
                  end do
               end do
            end do
         end do

!        --- store fourth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p1(i,j,k)*p7(i,j,k)
                  h2 = p2(i,j,k)*p7(i,j,k) +  p1(i,j,k)*p8(i,j,k)
                  h3 = p3(i,j,k)*p7(i,j,k) +  p1(i,j,k)*p9(i,j,k)
                  h4 = p2(i,j,k)*p8(i,j,k)
                  h5 = p3(i,j,k)*p8(i,j,k) +  p2(i,j,k)*p9(i,j,k)
                  h6 = p3(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add fourth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do n = 1, mod
                        ljn = l + (j-1)*mod + (n-1)*mtd

!                       --- add fourth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(i,j,n)*d1(i,l)*d1(n,k)

                     end do
                  end do
               end do
            end do
         end do

!        --- store fifth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p7(i,j,k)*p1(i,j,k)
                  h2 = p8(i,j,k)*p1(i,j,k) +  p7(i,j,k)*p2(i,j,k)
                  h3 = p9(i,j,k)*p1(i,j,k) +  p7(i,j,k)*p3(i,j,k)
                  h4 = p8(i,j,k)*p2(i,j,k)
                  h5 = p9(i,j,k)*p2(i,j,k) +  p8(i,j,k)*p3(i,j,k)
                  h6 = p9(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add fifth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do n = 1, mod
                        ljn = l + (j-1)*mod + (n-1)*mtd

!                       --- add fifth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(l,j,k)*d1(k,n)*d1(l,i)
                     end do
                  end do
               end do
            end do
         end do

!        --- store sixth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p4(i,j,k)*p4(i,j,k)
                  h2 = p5(i,j,k)*p4(i,j,k) +  p4(i,j,k)*p5(i,j,k)
                  h3 = p6(i,j,k)*p4(i,j,k) +  p4(i,j,k)*p6(i,j,k)
                  h4 = p5(i,j,k)*p5(i,j,k)
                  h5 = p6(i,j,k)*p5(i,j,k) +  p5(i,j,k)*p6(i,j,k)
                  h6 = p6(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add sixth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do m = 1, mod
                  sum = 0d0
                  do l = 1, mod
                     sum = sum + work(i,l,k)*d1(l,m)*d1(l,j)
                  end do
                  imk = i + (m-1)*mod + (k-1)*mtd
                  s(ijk,imk) = s(ijk,imk) + sum
                  end do
               end do
            end do
         end do

!        --- store seventh help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p4(i,j,k)*p7(i,j,k)
                  h2 = p5(i,j,k)*p7(i,j,k) +  p4(i,j,k)*p8(i,j,k)
                  h3 = p6(i,j,k)*p7(i,j,k) +  p4(i,j,k)*p9(i,j,k)
                  h4 = p5(i,j,k)*p8(i,j,k)
                  h5 = p6(i,j,k)*p8(i,j,k) +  p5(i,j,k)*p9(i,j,k)
                  h6 = p6(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add seventh factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do m = 1, mod
                     do n = 1, mod
                        imn = i + (m-1)*mod + (n-1)*mtd

!                       --- add seventh factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,j,n)*d1(j,m)*d1(n,k)

                     end do
                  end do
               end do
            end do
         end do

!        --- store eighth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p7(i,j,k)*p4(i,j,k)
                  h2 = p8(i,j,k)*p4(i,j,k) +  p7(i,j,k)*p5(i,j,k)
                  h3 = p9(i,j,k)*p4(i,j,k) +  p7(i,j,k)*p6(i,j,k)
                  h4 = p8(i,j,k)*p5(i,j,k)
                  h5 = p9(i,j,k)*p5(i,j,k) +  p8(i,j,k)*p6(i,j,k)
                  h6 = p9(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add eighth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do m = 1, mod
                     do n = 1, mod
                        imn = i + (m-1)*mod + (n-1)*mtd

!                       --- add eighth factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,m,k)*d1(k,n)*d1(m,j)
                     end do
                  end do
               end do
            end do
         end do

!        --- store ninth help factor in work

         do i = 1, mod

            do j = 1, mod
               do k = 1, mod
                  h1 = p7(i,j,k)*p7(i,j,k)
                  h2 = p8(i,j,k)*p7(i,j,k) +  p7(i,j,k)*p8(i,j,k)
                  h3 = p9(i,j,k)*p7(i,j,k) +  p7(i,j,k)*p9(i,j,k)
                  h4 = p8(i,j,k)*p8(i,j,k)
                  h5 = p9(i,j,k)*p8(i,j,k) +  p8(i,j,k)*p9(i,j,k)
                  h6 = p9(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,5) + h2*alpha(i,j,k,6) + &
                                h3*alpha(i,j,k,7) + h4*alpha(i,j,k,9) + &
                                h5*alpha(i,j,k,10) + h6*alpha(i,j,k,13)

                  work(i,j,k) = tstep*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add ninth factor of diffusion matrix to s

         do i = 1, mod

            do j = 1, mod

               do k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd

                  do n = 1, mod

                  sum = 0d0
                  do l = 1, mod
                     sum = sum + work(i,j,l)*d1(l,n)*d1(l,k)
                  end do

                  ijn = i + (j-1)*mod + (n-1)*mtd
                  s(ijk,ijn) = s(ijk,ijn) + sum
                  end do
               end do
            end do
         end do

      end if

      if (ichois.eq.4) then

!     --- compute contribution to large matrix:  |M|*(1-c_old^2) - |S|
!         summation over the integration points
!         store first help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p1(i,j,k)*p1(i,j,k)
                  h2 = p2(i,j,k)*p1(i,j,k) +  p1(i,j,k)*p2(i,j,k)
                  h3 = p3(i,j,k)*p1(i,j,k) +  p1(i,j,k)*p3(i,j,k)
                  h4 = p2(i,j,k)*p2(i,j,k)
                  h5 = p3(i,j,k)*p2(i,j,k) +  p2(i,j,k)*p3(i,j,k)
                  h6 = p3(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add first factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     sum = 0d0
                     do m = 1, mod
                        sum = sum + work(m,j,k)*d1(m,i)*d1(m,l)
                     end do
                     ljk = l + (j-1)*mod + (k-1)*mtd
                     s(ijk,ljk) = s(ijk,ljk) + sum
                  end do
               end do
            end do
         end do

!        --- store second help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p1(i,j,k)*p4(i,j,k)
                  h2 = p2(i,j,k)*p4(i,j,k) +  p1(i,j,k)*p5(i,j,k)
                  h3 = p3(i,j,k)*p4(i,j,k) +  p1(i,j,k)*p6(i,j,k)
                  h4 = p2(i,j,k)*p5(i,j,k)
                  h5 = p3(i,j,k)*p5(i,j,k) +  p2(i,j,k)*p6(i,j,k)
                  h6 = p3(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add second factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do m = 1, mod
                        lmk = l + (m-1)*mod + (k-1)*mtd

!                       --- add second factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(i,m,k)*d1(i,l)*d1(m,j)
                     end do
                  end do
               end do
            end do
         end do

!        --- store third help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p4(i,j,k)*p1(i,j,k)
                  h2 = p5(i,j,k)*p1(i,j,k) +  p4(i,j,k)*p2(i,j,k)
                  h3 = p6(i,j,k)*p1(i,j,k) +  p4(i,j,k)*p3(i,j,k)
                  h4 = p5(i,j,k)*p2(i,j,k)
                  h5 = p6(i,j,k)*p2(i,j,k) +  p5(i,j,k)*p3(i,j,k)
                  h6 = p6(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add third factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do m = 1, mod
                        lmk = l + (m-1)*mod + (k-1)*mtd

!                       --- add third factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(l,j,k)*d1(j,m)*d1(l,i)
                     end do
                  end do
               end do
            end do
         end do

!        --- store fourth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p1(i,j,k)*p7(i,j,k)
                  h2 = p2(i,j,k)*p7(i,j,k) +  p1(i,j,k)*p8(i,j,k)
                  h3 = p3(i,j,k)*p7(i,j,k) +  p1(i,j,k)*p9(i,j,k)
                  h4 = p2(i,j,k)*p8(i,j,k)
                  h5 = p3(i,j,k)*p8(i,j,k) +  p2(i,j,k)*p9(i,j,k)
                  h6 = p3(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add fourth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do n = 1, mod
                        ljn = l + (j-1)*mod + (n-1)*mtd

!                       --- add fourth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(i,j,n)*d1(i,l)*d1(n,k)

                     end do
                  end do
               end do
            end do
         end do

!        --- store fifth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p7(i,j,k)*p1(i,j,k)
                  h2 = p8(i,j,k)*p1(i,j,k) +  p7(i,j,k)*p2(i,j,k)
                  h3 = p9(i,j,k)*p1(i,j,k) +  p7(i,j,k)*p3(i,j,k)
                  h4 = p8(i,j,k)*p2(i,j,k)
                  h5 = p9(i,j,k)*p2(i,j,k) +  p8(i,j,k)*p3(i,j,k)
                  h6 = p9(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add fifth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do l = 1, mod
                     do n = 1, mod
                        ljn = l + (j-1)*mod + (n-1)*mtd

!                       --- add fifth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(l,j,k)*d1(k,n)*d1(l,i)
                     end do
                  end do
               end do
            end do
         end do

!        --- store sixth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p4(i,j,k)*p4(i,j,k)
                  h2 = p5(i,j,k)*p4(i,j,k) +  p4(i,j,k)*p5(i,j,k)
                  h3 = p6(i,j,k)*p4(i,j,k) +  p4(i,j,k)*p6(i,j,k)
                  h4 = p5(i,j,k)*p5(i,j,k)
                  h5 = p6(i,j,k)*p5(i,j,k) +  p5(i,j,k)*p6(i,j,k)
                  h6 = p6(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add sixth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do m = 1, mod
                  sum = 0d0
                  do l = 1, mod
                     sum = sum + work(i,l,k)*d1(l,m)*d1(l,j)
                  end do
                  imk = i + (m-1)*mod + (k-1)*mtd
                  s(ijk,imk) = s(ijk,imk) + sum
                  end do
               end do
            end do
         end do

!        --- store seventh help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p4(i,j,k)*p7(i,j,k)
                  h2 = p5(i,j,k)*p7(i,j,k) +  p4(i,j,k)*p8(i,j,k)
                  h3 = p6(i,j,k)*p7(i,j,k) +  p4(i,j,k)*p9(i,j,k)
                  h4 = p5(i,j,k)*p8(i,j,k)
                  h5 = p6(i,j,k)*p8(i,j,k) +  p5(i,j,k)*p9(i,j,k)
                  h6 = p6(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add seventh factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do m = 1, mod
                     do n = 1, mod
                        imn = i + (m-1)*mod + (n-1)*mtd

!                       --- add seventh factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,j,n)*d1(j,m)*d1(n,k)

                     end do
                  end do
               end do
            end do
         end do

!        --- store eighth help factor in work

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  h1 = p7(i,j,k)*p4(i,j,k)
                  h2 = p8(i,j,k)*p4(i,j,k) +  p7(i,j,k)*p5(i,j,k)
                  h3 = p9(i,j,k)*p4(i,j,k) +  p7(i,j,k)*p6(i,j,k)
                  h4 = p8(i,j,k)*p5(i,j,k)
                  h5 = p9(i,j,k)*p5(i,j,k) +  p8(i,j,k)*p6(i,j,k)
                  h6 = p9(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add eighth factor of diffusion matrix to s

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do m = 1, mod
                     do n = 1, mod
                        imn = i + (m-1)*mod + (n-1)*mtd

!                       --- add eighth factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,m,k)*d1(k,n)*d1(m,j)
                     end do
                  end do
               end do
            end do
         end do

!        --- store ninth help factor in work

         do i = 1, mod

            do j = 1, mod
               do k = 1, mod
                  h1 = p7(i,j,k)*p7(i,j,k)
                  h2 = p8(i,j,k)*p7(i,j,k) +  p7(i,j,k)*p8(i,j,k)
                  h3 = p9(i,j,k)*p7(i,j,k) +  p7(i,j,k)*p9(i,j,k)
                  h4 = p8(i,j,k)*p8(i,j,k)
                  h5 = p9(i,j,k)*p8(i,j,k) +  p8(i,j,k)*p9(i,j,k)
                  h6 = p9(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,16)+h2*alpha(i,j,k,17) + &
                                h3*alpha(i,j,k,18)+h4*alpha(i,j,k,20) + &
                                h5*alpha(i,j,k,21)+h6*alpha(i,j,k,24)

                  work(i,j,k) = -1.0d0*work(i,j,k)* &
                                      w1(i)*w1(j)*w1(k)/det(i,j,k)
               end do
            end do
         end do

!        --- add ninth factor of diffusion matrix to s

         do i = 1, mod

            do j = 1, mod

               do k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd

                  do n = 1, mod

                  sum = 0d0
                  do l = 1, mod
                     sum = sum + work(i,j,l)*d1(l,n)*d1(l,k)
                  end do

                  ijn = i + (j-1)*mod + (n-1)*mtd
                  s(ijk,ijn) = s(ijk,ijn) + sum
                  end do
               end do
            end do
         end do

         do i = 1, mod
            do j = 1, mod
               do k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  s(ijk,ijk) = s(ijk,ijk) +  (alpha(i,j,k,15) - &
                             alpha(i,j,k,14)*alpha(i,j,k,1)**2d0)* &
                             w1(i)*w1(j)*w1(k)*det(i,j,k)
               end do
            end do
         end do

      end if

      end
