      subroutine el6012 ( beta, f, old, w, det, mdecl )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  02-01-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the Helmholtz part of the equation in the case
!     of finite element preconditioning
!
!     EL6012 is called by the element subroutine ELM600
!
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none
      integer mdecl
      double precision beta(mdecl), f(mdecl), old(mdecl), w(mdecl), &
                       det

!     beta     i     array of length mdecl containing the values of the
!                    third coefficient in the integration points
!     det      i     jacobian of the transform
!     f       i/o    array of length mdecl containing the spectral
!                    element residue vector, at input f may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!     mdecl    i     number of integration/interpolation points
!     old      i     array of length mdecl containing the solution of
!                    the previous iteration step in the nodal points
!     w        i     array of length mdecl containing the weights for
!                    numerical integration
!***********************************************************************
!
!                 COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i

!     i     counting variable
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- check if Helmholtz part must be added to an existing vector
!         or not, if not set f equal to zero

      if (jadd.eq.0) then

!     --- jadd = 0, set f equal to zero

         do 100 i = 1, mdecl
            f(i) = 0d0
100      continue

      end if

!     --- add Helmholtz part to f

      do 200 i = 1, mdecl
         f(i) = f(i) + beta(i)*w(i)*old(i)*det
200   continue

      end
