      subroutine el6013 ( alpha, s, w1, d1, det, work, mdecl, ichois)
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.0  date 04-02-2000
!
!   copyright (c) 2000-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ichois
      double precision alpha(mdecl,26), s(mdecl,mdecl), w1(mdecl), &
                       d1(mdecl,mdecl), det, work(mdecl)

!     alpha    i   array of length mdecl x 3 containing the values of the
!                  first three coefficients in the integration points
!     d1       i   array of length mdecl x mdecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     det      i   array of length mdecl x mdecl containing the
!                  transformation determinant
!     ichois   i
!     mdecl    i   number of integration points in the element
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length mdecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- first initialize s as zero

      do i = 1, mdecl
         do j = 1, mdecl
            s(i,j) = 0d0
         end do
      end do

      if (ichois.eq.1) then

!     --- compute contribution to large matrix:   |M| - tstep*v_n*|D|
!         summation over the integration points
!         --- store -tstep*v_n*|D1|
!         --- store first help factor in work

!          add convection matrix to s

!          first compute u(i) * w(i)

         do i = 1, mdecl
            work(i) = -tstep*alpha(i,2)*w1(i)
         end do

         do  i = 1, mdecl
            do  j = 1, mdecl
               s(j,i) = s(j,i) + work(i)*d1(i,j)
            end do
         end do

!        --- Add |M|

         do  i = 1, mdecl
            s(i,i) = s(i,i) + w1(i)*det
         end do

      end if

      if (ichois.eq.2) then

!     --- compute contribution to large matrix:   |M|
!         summation over the integration points
!         store first help factor in work

         do  i = 1, mdecl
            s(i,i) = s(i,i) + w1(i)*det
         end do

      end if

!     --- compute contribution to large matrix:   tstep*|D|*[L]*|D|
!         summation over the integration points
!         store first help factor in work

      if (ichois.eq.3) then

         do k = 1, mdecl
            work(k) = tstep*alpha(k,5)*w1(k)
         end do

         do i = 1, mdecl
            do j = 1, i
               sum = 0d0
               do k = 1, mdecl
                  sum = sum + work(k)*d1(k,j)*d1(k,i)
               end do
               s(i,j) = sum/det
            end do
         end do

         do  i = 1, mdecl
            do  j = 1, i-1
               s(j,i) = s(i,j)
            end do
         end do

      end if

      if (ichois.eq.4) then

!     --- compute contribution to large matrix:  |M|*(1-c_old^2) - |S|
!         summation over the integration points
!         store first help factor in work

         do k = 1, mdecl
            work(k) = -alpha(k,16)*w1(k)
         end do

         do i = 1, mdecl
            do j = 1, i
               sum = 0d0
               do k = 1, mdecl
                  sum = sum + work(k)*d1(k,j)*d1(k,i)
               end do
               s(i,j) = sum/det
            end do
         end do

         do  i = 1, mdecl
            do  j = 1, i-1
               s(j,i) = s(i,j)
            end do
         end do

         do  i = 1, mdecl
            s(i,i) = s(i,i) + (alpha(i,15)- &
                     alpha(i,14)*alpha(i,1)**2d0)*w1(i)*det
         end do

      end if

      end
