      subroutine el6021(u, s, ncompd, kfirst, w1, w2, d1, d2, p1, p2, &
                        p3, p4, work, modecl, mdecl)

!***********************************************************************
!
!     Programmer Luc Timmermans
!
!     version 3.0  date 21-03-1992   complete rewrite
!     version 2.0  date 07-11-1991   change in parameterlist
!     version 1.0  date 23-09-1991   test version
!
!***********************************************************************
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the convection part of the equation
!
!     EL6021 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!     INPUT/OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none

      integer ncompd, kfirst, modecl, mdecl
      double precision u(mdecl,ncompd), s(mdecl,mdecl), w1(modecl), &
                       w2(modecl), d1(modecl,modecl), d2(modecl,modecl), &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl), work(mdecl)
!
!     u        i   array of length mdecl x 2 containing the values of the
!                  second coefficient in the integration points
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     w2       i   array of length modecl containing the weights for
!                  numerical integration in the y-direction corresponding
!                  to the reference element
!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     d2       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the y-direction
!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     p1-p4    i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     work         work array of length mdecl, to store products temporarily
!     ncompd   i   number of components of the velocity vector
!     kfirst   i   first component of the velocity vector that must be
!                  taken into account
!     modecl   i   number of integration points in the one dimension
!     mdecl    i   number of integration points in the element
!
!***********************************************************************
!
!     PARAMETERS IN COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, k, ij, ik, kj
      double precision help
!
!     i, j, k      counting variables
!     ij, kj, ik   positions in the matrix s
!     help         help variable to compute a product
!
!***********************************************************************
!
!     * first check if convection matrix must be added to an existing
!       matrix or not, if not set s equal to zero
!
      if (jadd.eq.0) then
!        * jadd = 0, set s equal to zero

         do 200 i = 1, mdecl
            do 100 j = 1, mdecl

               s(i,j) = 0d0
100         continue
200      continue

      end if
!
!     * add convection matrix to s
!
      if (idim.eq.1) then
!        * idim = 1, only one component of the velocity vector
!
!        * store first help factor in work
!
         if (kfirst.eq.1) then
!           * kfirst = 1, u1 # 0, u2 = 0

            do 300 i = 1, modecl

               ij = i
               do 250 j = 1, modecl

                  work(ij) = w1(i)*w2(j)*u(ij,1)*p4(i,j)
                  ij = ij + modecl
250            continue
300         continue
         else
!           * kfirst = 2, u1 = 0, u2 # 0

            do 400 i = 1, modecl

               ij = i
               do 350 j = 1, modecl

                  work(ij) = - w1(i)*w2(j)*u(ij,1)*p3(i,j)
                  ij = ij + modecl
350            continue
400         continue
         end if
!
!        * add first factor of convection matrix to s
!
         do 700 i = 1, modecl

            ij = i
            do 600 j = 1, modecl

               do 500 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  s(kj,ij) = s(kj,ij) + work(ij)*d1(i,k)
500            continue

            ij = ij + modecl
600         continue
700      continue

!
!        * store second help factor in work
!
         if (kfirst.eq.1) then
!           * kfirst = 1, u1 # 0, u2 = 0

            do 800 i = 1, modecl

               ij = i
               do 750 j = 1, modecl

                  work(ij) = - w1(i)*w2(j)*u(ij,1)*p2(i,j)
                  ij = ij + modecl
750            continue
800         continue
         else
!           * kfirst = 2, u1 = 0, u2 # 0

            do 900 i = 1, modecl

               ij = i
               do 850 j = 1, modecl

                  work(ij) = w1(i)*w2(j)*u(ij,1)*p1(i,j)
                  ij = ij + modecl
850            continue
900         continue
         end if
!
!        * add second factor of convection matrix to s
!
         do 1200 i = 1, modecl

            ij = i
            do 1100 j = 1, modecl

               do 1000 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  s(ik,ij) = s(ik,ij) + work(ij)*d2(j,k)
1000           continue

               ij = ij + modecl
1100        continue
1200     continue

      else
!        * idim = 2, two components of the velocity vector
!
!        * store first help factor in work
!
         do 1400 i = 1, modecl

            ij = i
            do 1300 j = 1, modecl

               help = u(ij,1)*p4(i,j) - u(ij,2)*p3(i,j)
               work(ij) = help*w1(i)*w2(j)
               ij = ij + modecl
1300        continue
1400     continue

!
!        * add first factor of convection matrix to s
!
         do 1700 i = 1, modecl

            ij = i
            do 1600 j = 1, modecl

               do 1500 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  s(kj,ij) = s(kj,ij) + work(ij)*d1(i,k)
1500           continue

               ij = ij + modecl
1600        continue
1700     continue

!
!        * store second help factor in work
!
         do 1900 i = 1, modecl

            ij = i
            do 1800 j = 1, modecl

               help = u(ij,2)*p1(i,j) - u(ij,1)*p2(i,j)
               work(ij) = help*w1(i)*w2(j)
               ij = ij + modecl
1800        continue
1900     continue

!
!        * add second factor of convection matrix to s
!
         do 2200 i = 1, modecl

            ij = i
            do 2100 j = 1, modecl

               do 2000 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  s(ik,ij) = s(ik,ij) + work(ij)*d2(j,k)
2000           continue

               ij = ij + modecl
2100        continue
2200     continue

      end if

      end
