      subroutine el6024 ( alpha, s, w1, w2, d1, d2, det, p1, p2, p3, &
                          p4, work, modecl, mdecl )
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 4.1    date  25-01-1996  Layout
!        version 4.0    date  15-09-1995  Error removed and addition of
!                                         jsecnd=3,4    Patrick Anderson
!        version 3.0    date  21-03-1992  complete rewrite
!        version 2.0    date  07-11-1991  change in parameterlist
!        version 1.0    date  20-09-1991  test version
!
!
!
!   copyright (c) 1991-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the diffusive part of the equation
!
!     EL6024 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision alpha(mdecl,3), s(mdecl,mdecl), w1(modecl), &
                       w2(modecl), d1(modecl,modecl), &
                       d2(modecl,modecl), det(modecl,modecl), &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl), &
                       work(mdecl)

!     alpha    i   array of length mdecl x 3 containing the values of the
!                  first three coefficients in the integration points
!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     d2       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the y-direction
!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     p1       i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   see p1
!     p3       i   see p1
!     p4       i   see p1
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     w2       i   array of length modecl containing the weights for
!                  numerical integration in the y-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
!
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, k, l, ij, kl, kj, lj, il, ik
      double precision sum, h1, h2, h3, a11con

!     a11con     Constant value of alpha
!     h1         help variable to compute a product
!     h2         help variable to compute a product
!     h3         help variable to compute a product
!     i          Counting variable
!     ij         position in the matrix s
!     ik         position in the matrix s
!     il         help parameter
!     j          Counting variable
!     k          Counting variable
!     kj         position in the matrix s
!     kl         position in the matrix s
!     l          General loop variable
!     lj         position in the matrix s
!     sum        Help parameter to compute a sum
!***********************************************************************
!
!                SUBROUTINES CALLED
!
!     INSTOP    Stop the program
!***********************************************************************
!
!                ERROR MESSAGES
!
!     None
!***********************************************************************
!
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- first initialize s as zero

      do 200 i = 1, mdecl
         do 100 j = 1, mdecl
            s(i,j) = 0d0
100      continue
200   continue

      if (jsecnd.eq.1) then

!      --- jsecnd = 1, alpha11 = alpha22 =constant, alpha12 = 0

         a11con = alpha(1,1)

!        --- summation over the integration points
!            store first help factor in work

         do 400 i = 1, modecl
            h1 = a11con*w1(i)
            ij = i
            do 300 j = 1, modecl
               h2 = p3(i,j)*p3(i,j) + p4(i,j)*p4(i,j)
               work(ij) = h1*h2*w2(j)/det(i,j)
               ij = ij + modecl
300         continue
400      continue

!        --- add first factor of diffusion matrix to s

         do 800 i = 1, modecl
            ij = i
            do 700 j = 1, modecl
               do 600 k = 1, modecl
                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 500 l = 1, modecl
                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
500               continue
                  s(ij,kj) = s(ij,kj) + sum
600            continue
               ij = ij + modecl
700         continue
800      continue

!        --- store second help factor in work

         do 1000 i = 1, modecl
            h1 = a11con*w1(i)
            ij = i
            do 900 j = 1, modecl
               h2 = - p1(i,j)*p3(i,j) - p2(i,j)*p4(i,j)
               work(ij) = h1*h2*w2(j)/det(i,j)
               ij = ij + modecl
900         continue
1000     continue

!        --- add second factor of diffusion matrix to s

         do 1400 i = 1, modecl
            ij = i
            do 1300 j = 1, modecl
               do 1200 k = 1, modecl
                  kl = k
                  do 1100 l = 1, modecl
                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
1100              continue
1200           continue
               ij = ij + modecl
1300        continue
1400     continue

!        --- add third factor of diffusion matrix to s

         do 1800 i = 1, modecl

            ij = i
            do 1700 j = 1, modecl

               do 1600 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 1500 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl + modecl
1500              continue
1600           continue

               ij = ij + modecl
1700        continue
1800     continue

!        --- store third help factor in work

         do 2000 i = 1, modecl

            h1 = a11con*w1(i)
            ij = i
            do 1900 j = 1, modecl

               h2 = p1(i,j)*p1(i,j) + p2(i,j)*p2(i,j)
               work(ij) = h1*h2*w2(j)/det(i,j)
               ij = ij + modecl
1900        continue
2000     continue

!        --- add fourth factor of diffusion matrix to s

         do 2400 i = 1, modecl

            ij = i
            do 2300 j = 1, modecl

               do 2200 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 2100 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
2100              continue

                  s(ij,ik) = s(ij,ik) + sum
2200           continue

               ij = ij + modecl
2300        continue
2400     continue

      else if (jsecnd.eq.2) then

!     --- jsecnd = 2, alpha11 = alpha22 = variable, alpha12 = 0
!         summation over the integration points
!         store first help factor in work

         do 2600 i = 1, modecl

            ij = i
            do 2500 j = 1, modecl
               h1 = p3(i,j)*p3(i,j) + p4(i,j)*p4(i,j)
               work(ij) = h1*alpha(ij,1)*w1(i)*w2(j)/det(i,j)
               ij = ij + modecl
2500        continue
2600     continue

!        --- add first factor of diffusion matrix to s

         do 3000 i = 1, modecl

            ij = i
            do 2900 j = 1, modecl
               do 2800 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 2700 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
2700              continue

                  s(ij,kj) = s(ij,kj) + sum
2800           continue

               ij = ij + modecl
2900        continue
3000     continue

!        --- store second help factor in work

         do 3200 i = 1, modecl

            ij = i
            do 3100 j = 1, modecl

               h1 = - p1(i,j)*p3(i,j) - p2(i,j)*p4(i,j)
               work(ij) = h1*alpha(ij,1)*w1(i)*w2(j)/det(i,j)
               ij = ij + modecl
3100        continue
3200     continue

!        --- add second factor of diffusion matrix to s

         do 3600 i = 1, modecl

            ij = i
            do 3500 j = 1, modecl

               do 3400 k = 1, modecl

                  kl = k
                  do 3300 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                  kl = kl + modecl
3300              continue

3400           continue

               ij = ij + modecl
3500        continue
3600     continue

!        --- add third factor of diffusion matrix to s

         do 4000 i = 1, modecl

            ij = i
            do 3900 j = 1, modecl
               do 3800 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 3700 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
3700              continue
3800           continue

               ij = ij + modecl
3900        continue
4000     continue

!        --- store third help factor in work

         do 4200 i = 1, modecl

            ij = i
            do 4100 j = 1, modecl

               h1 = p1(i,j)*p1(i,j) + p2(i,j)*p2(i,j)
               work(ij) = h1*alpha(ij,1)*w1(i)*w2(j)/det(i,j)
               ij = ij + modecl
4100        continue
4200     continue

!        --- add fourth factor of diffusion matrix to s

         do 4600 i = 1, modecl

            ij = i
            do 4500 j = 1, modecl

               do 4400 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 4300 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
4300              continue

                  s(ij,ik) = s(ij,ik) + sum
4400           continue

               ij = ij + modecl
4500        continue
4600     continue

      else if (jsecnd.eq.3) then

!     --- jsecnd = 3, alpha11 = alpha22 = variable
!         alpha12 = alpha 21 # 0
!         summation over the integration points
!         store first help factor in work

         do 5100 i = 1, modecl

            ij = i
            do 5000 j = 1, modecl

               h1 =   p3(i,j)*p3(i,j) + p4(i,j)*p4(i,j)
               h2 = - p3(i,j)*p4(i,j) - p4(i,j)*p3(i,j)
               work(ij)=(h1*alpha(ij,1)+h2*alpha(ij,2))* &
                        (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
5000        continue
5100     continue

!        --- add first factor of diffusion matrix to s

         do 5500 i = 1, modecl

            ij = i
            do 5400 j = 1, modecl
               do 5300 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 5200 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
5200              continue

                  s(ij,kj) = s(ij,kj) + sum
5300           continue

               ij = ij + modecl
5400       continue
5500     continue

!        --- store second help factor in work

         do 5700 i = 1, modecl

            ij = i
            do 5600 j = 1, modecl

               h1 = - p3(i,j)*p1(i,j) - p4(i,j)*p2(i,j)
               h2 =   p3(i,j)*p2(i,j) + p4(i,j)*p1(i,j)
               work(ij)=(h1*alpha(ij,1)+h2*alpha(ij,2))* &
                         (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
5600        continue
5700     continue

!        --- add second factor of diffusion matrix to s

         do 6100 i = 1, modecl

            ij = i
            do 6000 j = 1, modecl

               do 5900 k = 1, modecl

                  kl = k
                  do 5800 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                  kl = kl + modecl
5800              continue

5900           continue

               ij = ij + modecl
6000        continue
6100     continue

!        --- store third help factor in work

         do 6300 i = 1, modecl

            ij = i
            do 6200 j = 1, modecl

               h1 = - p1(i,j)*p3(i,j) - p2(i,j)*p4(i,j)
               h2 =   p2(i,j)*p3(i,j) + p1(i,j)*p4(i,j)
               work(ij)=(h1*alpha(ij,1)+h2*alpha(ij,2))* &
                          (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
6200        continue
6300     continue

!        --- add third factor of diffusion matrix to s

         do 6700 i = 1, modecl

            ij = i
            do 6600 j = 1, modecl
               do 6500 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 6400 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
6400              continue
6500           continue

               ij = ij + modecl
6600        continue
6700     continue

!
!        --- store fourth help factor in work
!
         do 6900 i = 1, modecl

            ij = i
            do 6800 j = 1, modecl

               h1 =   p1(i,j)*p1(i,j) + p2(i,j)*p2(i,j)
               h2 = - p1(i,j)*p2(i,j) - p2(i,j)*p1(i,j)
               work(ij)=(h1*alpha(ij,1)+h2*alpha(ij,2))* &
                          (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
6800        continue
6900     continue

!
!        --- add fourth factor of diffusion matrix to s
!
         do 7300 i = 1, modecl

            ij = i
            do 7200 j = 1, modecl

               do 7100 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 7000 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
7000              continue
                  s(ij,ik) = s(ij,ik) + sum
7100           continue

               ij = ij + modecl
7200        continue
7300     continue

      else if (jsecnd.eq.4) then
!        --- jsecnd = 4, alpha11 = variable , alpha22 = variable
!
!        --- summation over the integration points
!          store first help factor in work
!
         do 7600 i = 1, modecl

            ij = i
            do 7500 j = 1, modecl

               h1 =   p3(i,j)*p3(i,j)
               h2 =   p4(i,j)*p4(i,j)
               h3 = - p3(i,j)*p4(i,j) - p4(i,j)*p3(i,j)
               work(ij) = (h1*alpha(ij,3)+h2*alpha(ij,1)+h3*alpha(ij,2)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
7500        continue
7600     continue

!
!        --- add first factor of diffusion matrix to s
!
         do 8000 i=1, modecl

            ij = i
            do 7900 j = 1, modecl
               do 7800 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 7700 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
7700              continue

                  s(ij,kj) = s(ij,kj) + sum
7800           continue

               ij = ij + modecl
7900        continue
8000     continue

!
!        --- store second help factor in work
!
         do 8200 i = 1, modecl

            ij = i
            do 8100 j = 1, modecl

               h1 = - p3(i,j)*p1(i,j)
               h2 = - p4(i,j)*p2(i,j)
               h3 =   p3(i,j)*p2(i,j) + p4(i,j)*p1(i,j)
               work(ij) = (h1*alpha(ij,3)+h2*alpha(ij,1)+h3*alpha(ij,2)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
8100        continue
8200     continue

!
!        --- add second factor of diffusion matrix to s
!
         do 8600 i = 1, modecl

            ij = i
            do 8500 j = 1, modecl

               do 8400 k = 1, modecl

                  kl = k
                  do 8300 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                  kl = kl + modecl
8300              continue
8400           continue

               ij = ij + modecl
8500        continue
8600     continue

!
!        --- store third help factor in work
!
         do 8800 i = 1, modecl

            ij = i
            do 8700 j = 1, modecl

               h1 = - p1(i,j)*p3(i,j)
               h2 = - p2(i,j)*p4(i,j)
               h3 =   p1(i,j)*p4(i,j) + p2(i,j)*p3(i,j)
               work(ij) = (h1*alpha(ij,3)+h2*alpha(ij,1)+h3*alpha(ij,2)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
8700        continue
8800     continue

!
!        --- add third factor of diffusion matrix to s
!
         do 9200 i = 1, modecl

            ij = i
            do 9100 j = 1, modecl
               do 9000 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 8900 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
8900              continue
9000           continue

               ij = ij + modecl
9100        continue
9200     continue

!        --- store fourth help factor in work

         do 9400 i = 1, modecl

            ij = i
            do 9300 j = 1, modecl

               h1 =   p1(i,j)*p1(i,j)
               h2 =   p2(i,j)*p2(i,j)
               h3 = - p1(i,j)*p2(i,j) - p2(i,j)*p1(i,j)
               work(ij) = (h1*alpha(ij,3)+h2*alpha(ij,1)+h3*alpha(ij,2)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
9300        continue
9400     continue

!        --- add fourth factor of diffusion matrix to s

         do 9800 i = 1, modecl

            ij = i
            do 9700 j = 1, modecl

               do 9600 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 9500 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
9500              continue

                  s(ij,ik) = s(ij,ik) + sum
9600           continue

               ij = ij + modecl
9700        continue
9800     continue

      end if

      if (ierror.ne.0) call instop

      end
