      subroutine el6034 ( alpha, f, old, w1, d1, det, p1, p2, &
                          p3, p4, work, m1)
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 3.1    date  25-01-1996 layout
!        version 3.0    date  12-10-1995 completely rewriten with factorization
!                                        - P. Minev
!        version 2.0    date  26-09-1994 equal number og=f nodes in each
!                                        direction - P. Minev
!        version 1.0    date  14-04-1992
!
!   copyright (c) 1992-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!***********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the diffusive part of the equation in the
!     case of finite element preconditioning
!
!     EL6034 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!                      KEYWORDS
!
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none

      integer m1
      double precision alpha(m1,m1,3), f(m1,m1), &
                       old(m1,m1), &
                       w1(m1), d1(m1,m1), &
                       det(m1,m1), &
                       p1(m1,m1), p2(m1,m1), &
                       p3(m1,m1), p4(m1,m1), &
                       work(m1,m1)

!     alpha    i   array of length mxmx3 containing the values of the
!                  first three coefficients in the integration points
!     d1       i   array of length m x m contaning the values of the
!                  derivatives of the basisfunctions in the 1 direction
!     det      i   array of length m x m containing the
!                  transformation determinant
!     f        o   array of length mxm containing the spectral
!                  element residue vector
!     m1       i   number of integration points in one direction
!     old      i   array of length mxm containing the solution of
!                  the previous iteration step in the nodal points
!     p1       i   array of length m x m containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   array of length m x m containing the partial
!                  derivatives of the coordinates due to transformation
!     p3       i   array of length m x m containing the partial
!                  derivatives of the coordinates due to transformation
!     p4       i   array of length m x m containing the partial
!                  derivatives of the coordinates due to transformation
!     w1       i   array of length m containing the weights for
!                  numerical integration in the 1 direction corresponding
!                  to the reference element
!     work         work array of length mxm, to store products temporarily
!***********************************************************************
!
!                      COMMON BLOCKS
!
!***********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, h1, h2, a11con

!     a11con                   constant value of alpha
!     h1                       help variable to compute a product
!     h2                       help variable to compute a product
!     i                        counting variable
!     j                        counting variable
!     k                        counting variable
!     sum                      help variable to compute a sum
!***********************************************************************
!
!               SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB    print error messages
!     INSTOP    stop the program
!***********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     1602   these values of alpha not done
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      if (jsecnd.eq.1) then

!     --- jsecnd = 1, alpha11 = alpha22 =constant, alpha12 = 0

         a11con = alpha(1,1,1)

!        --- summation over the integration points
!            store first help factor in work

         do 400 i = 1, m1

            do 300 j = 1, m1

               sum = 0d0
               do 301 k = 1, m1
               sum = sum + d1(i,k)*old(k,j)
301            continue

            work(i,j) = a11con*sum

300         continue
400      continue

!        ---add first factor of diffusion part to f

         do 800 i = 1, m1

            do 700 j = 1, m1

               sum = 0d0
               do 600 k = 1, m1

               h1 = (p3(k,j)*p3(k,j) + p4(k,j)*p4(k,j))* &
                    w1(k)*w1(j)/det(k,j)
               h2 = (-p2(i,k)*p4(i,k) - p1(i,k)*p3(i,k))* &
                    w1(i)*w1(k)/det(i,k)

               sum = sum + h1*d1(k,i)*work(k,j) + &
                           h2*d1(k,j)*work(i,k)
600            continue
               f(i,j) = sum
700         continue
800      continue

!        --- store second help factor in work

         do 1000 i = 1, m1

            do 900 j = 1, m1

               sum = 0d0
               do 901 k = 1, m1
               sum = sum + d1(i,k)*old(j,k)
901            continue
               work(i,j) = a11con*sum

900         continue
1000     continue

!        --- add second factor of diffusion matrix to s

         do 1400 i = 1, m1

            do 1300 j = 1, m1

               sum = 0d0
               do 1200 k = 1, m1

               h1 = (p2(i,k)*p2(i,k) + p1(i,k)*p1(i,k))* &
                    w1(i)*w1(k)/det(i,k)
               h2 = (-p2(k,j)*p4(k,j) - p1(k,j)*p3(k,j))* &
                    w1(k)*w1(j)/det(k,j)

               sum = sum + h1*d1(k,j)*work(k,i) + &
                           h2*d1(k,i)*work(j,k)

1200           continue

               f(i,j) = f(i,j) + sum
1300        continue
1400     continue


!     else if (jsecnd.eq.2) then

!     --- jsecnd = 2, alpha11 = alpha22 = variable, alpha12 = 0
!         summation over the integration points
!         store first help factor in work

      else

!     --- jsecnd = 3 or jsecnd = 4, not done

         call eropen('el6034')
         call errsub ( 1602, 0, 0, 0)
         call erclos('el6034')

      end if

      if (ierror.ne.0) call instop

      end
