      subroutine el6036(old, old2, alpha, w1, d1, p1, p2, p3, p4, det, &
                        work, work2 ,m1, f, ichois)
! ======================================================================
!
!        programmer   M. Speetjes, P. Anderson
!        version 1.2   date  23-01-1997 Minor correction
!        version 1.1   date  28-08-1996 Correction and Layout
!        version 1.0   date  14-05-1996
!
!   copyright (c) 1996-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the diffusive part of the equation in the
!     case of finite element preconditioning
!     el6036 is called by the element subroutine ELD607
!            it must be preceded by the subroutine ELP642
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!    diffusion
!    residual
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer m1, ichois
      double precision alpha(m1,m1,3), f(m1,m1), &
                       old(m1,m1),old2(m1,m1), &
                       w1(m1), d1(m1,m1), &
                       det(m1,m1), &
                       p1(m1,m1), p2(m1,m1), &
                       p3(m1,m1), p4(m1,m1), &
                       work(m1,m1), work2(m1,m1)

!     alpha    i   array of length mxmx3 containing the values of the
!                  first three coefficients of the diffusion coefficient
!                  in the integration points
!     d1       i   array of length m x m contaning the values of the
!                  derivatives of the basisfunctions in the 1 direction
!     det      i   array of length m x m containing the
!                  transformation determinant
!     f        o   array of length mxm containing the spectral
!                  element residue vector
!     ichois   i   indicated which component of the diffusion residual
!                  to be calculated in case of non-constant viscosity
!     m1       i   number of integration points in one direction
!     old      i   array of length mxm containing the first
!                  component of the old solution
!     old2     i   array of length mxm containing the second
!                  component of the old solution
!     p1       i   array of length m x m containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     w1       i   array of length m containing the weights for
!                  numerical integration in the 1 direction corresponding
!                  to the reference element
!     work         work array of length mxm, to store products temporarily
!     work2        work array of length mxm, to store products temporarily
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, sum2, h1, h2, a11con

!     a11con         constant value of alpha
!     h1             help variable to compute a product
!     h2             help variable to compute a product
!     i              counting variable
!     j              counting variable
!     k              counting variable
!     sum            help variable to compute a sum
!     sum2           help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB    print error messages
!     INSTOP    stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1602   these values of alpha not done
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (jsecnd.eq.1) then

!     --- jsecnd = 1, alpha11 = alpha22 =constant, alpha12 = 0
!         compute alpha*nabla(old) = M^(-1) * D(old), with f = d(old)
!         Swap between 'old' and 'old2', since in this case the computational
!         procedure of the residual is equivalent for the first and second
!         component

         if(ichois.eq.2) then
            do 111 i =1,m1
               do 112 j =1,m1
                  old(i,j) = old2(i,j)
112            continue
111         continue
         end if

         a11con = alpha(1,1,1)

!        --- summation over the integration points
!            store first help factor in work

         do 400 i = 1, m1

            do 300 j = 1, m1

               sum = 0d0
               do 301 k = 1, m1
                  sum = sum + d1(i,k)*old(k,j)
301            continue

            work(i,j) = a11con*sum

300         continue
400      continue

!        --- add first factor of diffusion part to f

         do 800 i = 1, m1

            do 700 j = 1, m1

               sum = 0d0
               do 600 k = 1, m1

                  h1 = -(p3(k,j)*p3(k,j) + p4(k,j)*p4(k,j))* &
                       w1(k)*w1(j)/det(k,j)
                  h2 = (p2(i,k)*p4(i,k) + p1(i,k)*p3(i,k))* &
                       w1(i)*w1(k)/det(i,k)

                  sum = sum + h1*d1(k,i)*work(k,j) + &
                           h2*d1(k,j)*work(i,k)
600            continue
               f(i,j) = sum
700         continue
800      continue

!        --- store second help factor in work

         do 1000 i = 1, m1

            do 900 j = 1, m1

               sum = 0d0
               do 901 k = 1, m1
                  sum = sum + d1(i,k)*old(j,k)
901            continue
               work(i,j) = a11con*sum

900         continue
1000     continue

!        --- add second factor of diffusion matrix to s

         do 1400 i = 1, m1

            do 1300 j = 1, m1

               sum = 0d0
               do 1200 k = 1, m1

                  h1 = -(p2(i,k)*p2(i,k) + p1(i,k)*p1(i,k))* &
                       w1(i)*w1(k)/det(i,k)
                  h2 = (p2(k,j)*p4(k,j) + p1(k,j)*p3(k,j))* &
                       w1(k)*w1(j)/det(k,j)

                  sum = sum + h1*d1(k,j)*work(k,i) + &
                              h2*d1(k,i)*work(j,k)

1200           continue

               f(i,j) = f(i,j) + sum

1300        continue
1400     continue

      end if

      if (jsecnd.eq.2) then

!     --- jsecnd = 2, alpha11 = alpha22 = variable, alpha12 = 0
!         compute grad.(alpha (grad(u))^T + alpha grad(u))
!                 = M^(-1)  * D(u), where
!         u = (old,old2)^T , &
!         f1 = D(u)*e_x , f2 = D(u)*e_y  the first and second component
!                                    of the residual.
!         compute f1

          if(ichois.eq.1) then

!         --- summation over the integration points
!             store first help factor in work

             do 405 i = 1, m1

                do 305 j = 1, m1

                   sum = 0d0
                   sum2 = 0d0
                   do 306 k = 1, m1
                      sum = sum + d1(i,k)*old(k,j)
                      sum2 = sum2 + d1(i,k)*old2(k,j)
306                continue

                   work(i,j) = sum
                   work2(i,j) = sum2

305             continue
405          continue

!            --- add first factor of diffusion part to f

             do 805 i = 1, m1

                do 705 j = 1, m1

                   sum = 0d0
                   do 605 k = 1, m1

                      h1 = w1(k)*w1(j)*alpha(k,j,1)/det(k,j)
                      h2 = w1(i)*w1(k)*alpha(i,k,1)/det(i,k)

                      sum = sum + h1*d1(k,i)*(p3(k,j)*p4(k,j)*work2(k,j) &
                          - (p3(k,j)*p3(k,j) + &
                             2.0d0*p4(k,j)*p4(k,j))*work(k,j)) &
                          +  h2*d1(k,j)*((2.0d0*p2(i,k)*p4(i,k) + &
                             p1(i,k)*p3(i,k))* &
                              work(i,k) - p1(i,k)*p4(i,k)*work2(i,k))
605                continue
                   f(i,j) = sum
705             continue
805          continue

!            --- store second help factor in work

             do 1005 i = 1, m1

                do 905 j = 1, m1

                   sum = 0d0
                   sum2 = 0d0

                   do 906 k = 1, m1
                      sum = sum + d1(j,k)*old(i,k)
                      sum2 = sum2 + d1(j,k)*old2(i,k)
906                continue

                   work(i,j) = sum
                   work2(i,j) = sum2

905             continue
1005         continue

!            --- add second factor of diffusion matrix to s

             do 1405 i = 1, m1

                do 1305 j = 1, m1

                   sum = 0d0
                   do 1205 k = 1, m1

                   h1 = w1(i)*w1(k)*alpha(i,k,1)/det(i,k)
                   h2 = w1(k)*w1(j)*alpha(k,j,1)/det(k,j)

                   sum = sum + h1*d1(k,j)*(p1(i,k)*p2(i,k)*work2(i,k) &
                      - (p1(i,k)*p1(i,k) + &
                         2.0d0*p2(i,k)*p2(i,k))*work(i,k)) &
                      +  h2*d1(k,i)*((2.0d0*p2(k,j)*p4(k,j) &
                      + p1(k,j)*p3(k,j))* &
                        work(k,j) - p2(k,j)*p3(k,j)*work2(k,j))

1205               continue

                   f(i,j) = f(i,j) + sum

1305            continue
1405         continue

          end if

!         --- compute f2

          if(ichois.eq.2) then

!         --- summation over the integration points
!             store first help factor in work

             do 407 i = 1, m1

                do 307 j = 1, m1

                   sum = 0d0
                   sum2 = 0d0

                   do 308 k = 1, m1

                      sum = sum + d1(i,k)*old(k,j)
                      sum2 = sum2 + d1(i,k)*old2(k,j)
308                continue

                work(i,j) = sum
                work2(i,j) = sum2

307             continue
407          continue

!            --- add first factor of diffusion part to f

             do 807 i = 1, m1

                do 707 j = 1, m1

                   sum = 0d0
                   do 608 k = 1, m1

                   h1 = w1(k)*w1(j)*alpha(k,j,1)/det(k,j)
                   h2 = w1(i)*w1(k)*alpha(i,k,1)/det(i,k)

                   sum = sum + h1*d1(k,i)*(p3(k,j)*p4(k,j)*work(k,j) &
                         - (p4(k,j)*p4(k,j) + &
                           2.0d0*p3(k,j)*p3(k,j))*work2(k,j)) &
                        +  h2*d1(k,j)*((2.0d0*p1(i,k)*p3(i,k) + &
                           p2(i,k)*p4(i,k))* &
                           work2(i,k) - p2(i,k)*p3(i,k)*work(i,k))
608                continue

                   f(i,j) = sum

707             continue
807          continue

!            --- store second help factor in work

             do 1007 i = 1, m1

                do 907 j = 1, m1

                   sum = 0d0
                   sum2 = 0d0
                   do 908 k = 1, m1
                      sum = sum + d1(j,k)*old(i,k)
                      sum2 = sum2 + d1(j,k)*old2(i,k)
908                continue
                   work(i,j) = sum
                   work2(i,j) = sum2

907             continue
1007         continue

!            --- add second factor of diffusion matrix to s

             do 1407 i = 1, m1

                do 1307 j = 1, m1

                   sum = 0d0
                   do 1207 k = 1, m1

                       h1 = w1(i)*w1(k)*alpha(i,k,1)/det(i,k)
                       h2 = w1(k)*w1(j)*alpha(k,j,1)/det(k,j)

                       sum = sum + h1*d1(k,j)*(p1(i,k)*p2(i,k)*work(i,k) &
                         - (p2(i,k)*p2(i,k) + &
                           2.0d0*p1(i,k)*p1(i,k))*work2(i,k)) &
                         +  h2*d1(k,i)*((2.0d0*p1(k,j)*p3(k,j) + &
                            p2(k,j)*p4(k,j))* &
                            work2(k,j) - p1(k,j)*p4(k,j)*work(k,j))

1207               continue

                   f(i,j) = f(i,j) + sum

1307            continue
1407         continue

          end if

      end if

      if(jsecnd.ge.3) then

!     --- jsecnd = 3 or jsecnd = 4, not done

         call eropen('el6036')
         call errsub ( 1602, 0, 0, 0)
         call erclos('el6036')
      end if

      if (ierror.ne.0) call instop

      end
