      subroutine el6043 ( u, v, lambda, f, old, w1, d1, &
                          p1, p2, p3, p4, &
                          work, modecl, mdecl )
! ======================================================================
!
!        programmer Patrick Anderson
!
!        version 1.0    date  20-03-1997
!
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convection residual in the case of spectral elements
!     used in visco elastic routine
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision u(mdecl), v(mdecl),w1(modecl), &
                       d1(modecl,modecl), lambda, &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl),work(mdecl), &
                       f(mdecl),old(mdecl)

!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     f       i/o  array of lenght mdecl containing the righthand-side vector
!     lambda   i   ?
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in the one dimension
!     old      i   array of lenght mdecl containing the old solution
!     p1       i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     u        i   array of length mdecl x 2 containing the values of the
!                  convection coefficient of LHSD in the integration points
!     v        i   ?
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, ij, ik, kj
      double precision help, sum

!     help       help variable to compute a product
!     i          Counting variable
!     ij         position in the matrix s
!     ik         position in the matrix s
!     j          Counting variable
!     k          Counting variable
!     kj         position in the matrix s
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- first check if convection matrix must be added to an existing
!         matrix or not, if not set s equal to zero

      if (jadd.eq.0) then

!     --- jadd = 0, set f equal to zero

         do 200 i = 1, mdecl
               f(i) = 0d0
200      continue

      end if

!     --- store first help factor in work

      do 1400 i = 1, modecl

         ij = i
         do 1300 j = 1, modecl

            help = u(ij)*p4(i,j) - v(ij)*p3(i,j)
            work(ij) = help*lambda*w1(i)*w1(j)
            ij = ij + modecl
1300     continue
1400  continue

!     --- add first factor to the residual

      do 1700 i = 1, modecl

         ij = i
         do 1600 j = 1, modecl

            sum = 0d0
            do 1500 k = 1, modecl

               kj = k + (j - 1)*modecl
             sum = sum + d1(i,k)*old(kj)
1500        continue

         f(ij) = f(ij) + work(ij)*sum
            ij = ij + modecl
1600     continue
1700  continue

!     --- store second help factor in work

      do 1900 i = 1, modecl

         ij = i
         do 1800 j = 1, modecl

            help = v(ij)*p1(i,j) - u(ij)*p2(i,j)
            work(ij) = help*lambda*w1(i)*w1(j)
            ij = ij + modecl
1800     continue
1900  continue

!     --- add second factor to the residual

      do 2200 i = 1, modecl

         ij = i
         do 2100 j = 1, modecl

            sum = 0d0
            do 2000 k = 1, modecl

               ik = i + (k - 1)*modecl
             sum = sum + d1(j,k)*old(ik)
2000        continue

         f(ij) = f(ij) + work(ij)*sum
            ij = ij + modecl
2100     continue
2200  continue

      end
