      subroutine el6061(u1, u2, d1, p1, p2, p3, p4, det, &
                        m1, f, ipelwg, delta, csmago)
! ======================================================================
!
!        programmer     Michel Speetjens
!        version 1.1    date   05-08-1996 Minor changes. P. Anderson
!        version 1.0    date   01-12-1995
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!         Computes eddy viscosity for LES according to the
!         Smagorinsky model.
!         It must be preceded by the subroutine ELP640
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!    large_eddy
!    Smagorinsky
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer m1
      double precision f(m1,m1), ipelwg(m1,m1), &
                       u1(m1,m1),u2(m1,m1), d1(m1,m1), &
                       det(m1,m1), &
                       p1(m1,m1), p2(m1,m1), &
                       p3(m1,m1), p4(m1,m1)
      double precision delta, csmago

!     csmago   i   Smagorinsky constant
!     d1       i   array of length m x m contaning the values of the
!                  derivatives of the basisfunctions in the 1 direction
!     delta    i   LES filter size
!     det      i   array of length m x m containing the
!                  transformation determinant
!     f        o   array of length mxm containing the eddy viscosity
!     ipelwg   i
!     m1       i   number of integration points in one direction
!     p1       i   array of length m x m containing the partial
!                  derivatives of the coordinates due to
!                  2-D transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     u1       i   array of length mxm containing the first velocity
!                  component
!     u2       i   array of length mxm containing the second velocity
!                  component
! **********************************************************************
!
!                       COMMON BLOCKS
!
      integer ierror, nerror
      common /cconst/ ierror, nerror
      save /cconst/

!                 /cconst/
!    Error codes and number of errors to print
!    ierror  i/o  Error number of last error occurred
!                 0        No error present
!                 >0        Last run-time error
!    nerror   -   Number of error messages printed
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum1, sum2, sum3, sum4, h1, h2, h3 ,c

!     c          help variable
!     h1         Help variable
!     h2         Help variable
!     h3         Help variable
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     sum1       Parameter to compute a sum
!     sum2       Parameter to compute a sum
!     sum3       Parameter to compute a sum
!     sum4       Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('el6061')

      c = (delta*csmago)**2

      do 100 i = 1 , m1
         do 110 j = 1 , m1

            ipelwg(i,j) = 1d0

            sum1 = 0.0d0
            sum2 = 0.0d0
            sum3 = 0.0d0
            sum4 = 0.0d0

            do 120 k = 1, m1

               sum1 = sum1 + d1(i,k)*u1(k,j)
               sum2 = sum2 + d1(j,k)*u1(i,k)
               sum3 = sum3 + d1(i,k)*u2(k,j)
               sum4 = sum4 + d1(j,k)*u2(i,k)

120         continue

            h1 = p4(i,j)*sum1 - p2(i,j)*sum2
            h2 = p1(i,j)*sum2 - p3(i,j)*sum1 &
              + p4(i,j)*sum3 - p2(i,j)*sum4
            h3 = p1(i,j)*sum4 - p3(i,j)*sum3

            f(i,j) = 2.0d0*(h1**2 + (h2**2)/2.0d0 + h3**2)
            f(i,j) = c*sqrt(f(i,j))/det(i,j)

110      continue
100   continue

      call erclos('el6061')

      if (ierror.ne.0) call instop

      end
