      subroutine el6087 ( mass, w1, det, mone, m )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   29-09-1994
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the spectral element diagonal mass matrix; 3-D case
!
!     EL6087 is called by the element subroutine ELM607
!            it must be preceded by the subroutines ELP640 and ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer mone, m
      double precision mass(m), w1(mone), det(mone,mone,mone)

!     det      i   array of length mone x mone x mone containing the
!                  determinant of the jacobian
!     m        i   number of integration points in the element
!     mass     o   array of length m containing the spectral element
!                  diagonal mass-matrix
!     mone     i   number of integration points in one dimension
!     w1       i   array of length mone containing the weights for
!                  numerical integration in 1 direction correspon- &
!                  ding to the reference element
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, j, ijk, k, mtwo
      double precision swap

!     i          Counting variable
!     ijk        help parameter
!     j          Counting variable
!     k          Counting variable
!     mtwo       mtwo = mone*mone
!     swap       swap parameter
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      mtwo = mone*mone
      do 200 i = 1, mone
         do 100 j = 1, mone
            do 300 k = 1, mone
               ijk = i + (j-1)*mone + (k-1)*mtwo
               mass(ijk) = w1(i)*w1(j)*w1(k)*det(i,j,k)
300         continue
100      continue
200   continue

      if (mone.eq.2) then

         swap = mass(3)
         mass(3) = mass(4)
         mass(4) = swap
         swap = mass(7)
         mass(7) = mass(8)
         mass(8) = swap
      end if

      end
