      subroutine eld608 ( icheld, coor, elemvc, iuser, user, &
                          index1, swork )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   18-11-1994
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     fill spectral element vector for inhomogeneous Neumann
!     condition; 2D surface in 3D space.
!
!     ELD608 is a help subroutine for subroutine DERIV
!     it is called through the intermediate subroutine ELD000
!
!     so:
!
!     DERIV
!       -  loop over spectral elements
!          -  ELD000
!             -  ELD608
!
!     itype = 608  :   two dimensional spectral element in 3D space
!                      (m nodal points)
! **********************************************************************
!
!                      KEYWORDS
!
!     derivative
!    elliptic_equation
!    element_vector
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS

      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer iuser(*), index1(*), icheld
      double precision coor(*), elemvc(*), user(*), swork(*)

!     coor     i     array of length npoint containing the coordinates of
!                    the nodal points with respect to the global numbering
!                    x  = coor (1,i)
!                     i
!     elemvc   o     element vector to be computed
!     icheld   i     choise parameter
!     index1   i     array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     iuser    i     integer user array to pass user information from
!                    main program to subroutine, see STANDARD PROBLEMS
!     swork          spectral element work array
!                    total length is: 7*m + 2*mone
!     user     i     real user array to pass user information from
!                    main program to subroutine, see STANDARD PROBLEMS
!***********************************************************************
!
!                        COMMON BLOCKS


!***********************************************************************
!
!                  LOCAL PARAMETERS

      integer iparry, ipxref, ipx, ipdet, ipw, ipd1, i
      save iparry, ipxref, ipx, ipdet, ipw, ipd1

!     i         Counting variable
!     iparry    indicates array in which the coefficients are stored
!               length m
!     ipd1      indicates the values of the derivatives of the basic
!               functions in the Gauss-Lobatto points; length m
!     ipdet     indicates the determinant of the transformation to the
!               reference 2d element, length m
!     ipw       indicates the corresponding weights in the element
!               length mone
!     ipx       indicates the corresponding points in the element, &
!               length m x ndim
!     ipxref    indicates the Legendre Gauss-Lobatto points corresponding
!               to the reference element [-1,1], length mone
!***********************************************************************
!
!                    I/O
!
!     the user provides the coefficients through the arrays IUSER and
!     USER in the usual way, the sequence is:
!
!     1:  alpha, non-zero
!
!***********************************************************************
!
!                  SUBROUTINES CALLED
!
      double precision funccf

!     EL0608    initialize parameters for subroutine ELM608
!     EL6020    compute contributions to the spectral element vector
!               corresponding to nonhomogeneous Neumann conditions
!               on 2-D surface in a 3-D space
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP649    store global integration points
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRSUB    Error messages
!     FUNCCF    User function subroutine to compute the value of a coefficient
!               in a point
! **********************************************************************
!
!                    ERROR MESSAGES
!
!    1781   The possibility ifunc > 10000 not yet available
! ********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
!
      call eropen('eld608')

      if (ifirst.eq.0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations

         iparry = 1
         ipxref = iparry + m
         ipx = ipxref + mone
         ipdet = ipx + m*ndimlc
         ipw = ipdet + m
         ipd1 = ipw + mone

!        --- compute points and weights for numerical integration
!            corresponding to the reference element
!            ipdet is used as a dummy array

         call elp640(1, swork(ipxref), swork(ipw), swork(ipd1), &
                     swork(ipdet), mone)

      end if

      if ( ierror.ne.0 ) go to 1000

      call el0608(iuser, user, swork(iparry) )

      if( (icheld.eq.10).and.(jconsf.ne.0) ) then

!     --- compute the source term
!         first compute integration points, weights and determinant

         call elp649 ( coor, index1, swork(ipx), swork(ipd1), &
                       swork(ipdet), mone, m)

!        --- fill variable coefficients if necessary

         if (ind(1).gt.0) then

!        --- coefficient is variable

            if ( ind(1).gt.10000 ) &
               call errsub ( 1781, 0, 0, 0 )

            do 200 i = 1, m
               swork(iparry+i-1) = funccf ( ind(1), swork(ipx+i-1), &
                                   swork(ipx+i-1+m), swork(ipx+i-1+2*m))
200         continue

         end if

         call el6020(swork(iparry), elemvc, swork(ipw), swork(ipdet), &
                           mone)

      else

!     --- set the source vector to 0

         do 100 i = 1, m
            elemvc(i) = 0d0
100      continue

      end if

1000  call erclos('eld608')

      end
