      subroutine elf000 ( itel, jtel, ifunc, func, ipos, iuser, user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 15-08-2007 Allow coef %scalar
!        version  2.2  date 03-12-1994 New norms
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1983-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill a part of  arrays iuser and user corresponding to
!      one standard element and one coefficient
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), itel, jtel, ifunc, ipos
      double precision user(*), func

!     func    i    value of the coefficient when it is a constant
!     ifunc   i    integer information of the coefficient
!                  when ifunc = 0  the coefficient is constant with value func
!                  when ifunc>0 the coefficient is a function
!     ipos    i    position of parameter in array iwork in subroutine fil...
!     itel   i/o   last position used in iuser
!     iuser  i/o   Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     jtel   i/o   last position used in user
!     user   i/o   Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     115   ifunc<0
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elf000' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elf000'
         write(irefwr,1) 'itel, ifunc, jtel', itel, ifunc, jtel
         write(irefwr,2) 'func', func
  1      format ( a, 1x, (10i11) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itel=itel+1
      if ( ifunc<-10000 ) then

!     --- ifunc<-10000, func = scalar

         iuser(itel) = inirepscal(-ifunc)
         if ( debug) then
            write(irefwr,1) 'itel, iuser(itel)', &
                             itel, iuser(itel)
         end if  ! ( debug )

      else if ( ifunc<0 ) then

!     --- error 115   ifunc<0

         call errint ( ifunc, 1 )
         call errint ( ipos, 2 )
         call errsub ( 115, 2, 0, 0 )

      else if (ifunc==0) then

!     --- ifunc = 0   func is constant

        if ( abs(func)<=0d0 ) then

!       --- func = 0

          iuser(itel)=0

        else

!       --- func # 0

          jtel = jtel+1
          iuser(itel) = -jtel
          user(jtel) = func

          if ( debug) then
            write(irefwr,1) 'jtel, itel, iuser(itel)', &
                             jtel, itel, iuser(itel)
            write(irefwr,2) 'user(jtel)', user(jtel)
          end if  ! ( debug )

        end if

      else

!     --- ifunc>0

        iuser(itel) = ifunc

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elf000'

      end if  ! ( debug )
      call erclos ( 'elf000' )

      end
