      subroutine elfillcoeffin2 ( coeffintpnts, coeffnodes, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients computed in nodes in integration points
!     interpolation in subelements must be used
!     jdiag = 2, nodes and integration points differ
!     quadratic triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     element
!     fill
!     vector
!     triangle
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndegfd
      double precision coeffnodes(n,ndegfd), coeffintpnts(m,ndegfd)

!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!     coeffnodes     i    Real array in which the values of the
!                         coefficients in the nodal points are stored
!     ndegfd         i    Number of degrees of freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision fact1, fact2

!     fact1          multiplication factor
!     fact2          multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillcoeffin2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillcoeffin2'
         call prinrl1 ( coeffnodes, ndegfd, n, 'coeffnodes' )
         call prinrl1 ( coeffintpnts, ndegfd, m, 'coeffintpnts' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- first fill first integration point (centroid)

      if ( n==7 ) then

!     --- 7 nodes, hence centroid has been filled

         coeffintpnts(1,1:ndegfd) = coeffnodes(7,1:ndegfd)

      else

!     --- 6 nodes, hence centroid has not been filled

         coeffintpnts(1,1:ndegfd) = &
            (coeffnodes(2,1:ndegfd)+coeffnodes(4,1:ndegfd)+ &
             coeffnodes(6,1:ndegfd))/3d0

      end if  ! ( n==7 )

      if ( m==4 ) then

!     --- four point Gauss rule
!         fill other 3 points

         coeffintpnts(2,1:ndegfd) = &
            (coeffnodes(2,1:ndegfd)+coeffnodes(6,1:ndegfd))*0.4d0+ &
             coeffnodes(1,1:ndegfd)*0.2d0
         coeffintpnts(3,1:ndegfd) = &
            (coeffnodes(4,1:ndegfd)+coeffnodes(2,1:ndegfd))*0.4d0+ &
             coeffnodes(3,1:ndegfd)*0.2d0
         coeffintpnts(4,1:ndegfd) = &
            (coeffnodes(6,1:ndegfd)+coeffnodes(4,1:ndegfd))*0.4d0+ &
             coeffnodes(5,1:ndegfd)*0.2d0

      else if ( m==7 ) then

!     --- four point Gauss rule
!         fill other 6 points

         fact1 = 1d0-2d0*alam3(1,3)
         fact2 = 1d0-2d0*alam3(1,2)
         coeffintpnts(2,1:ndegfd) = &
            (coeffnodes(2,1:ndegfd)+coeffnodes(6,1:ndegfd))*fact1+ &
             coeffnodes(4,1:ndegfd)*fact2
         coeffintpnts(3,1:ndegfd) = &
            (coeffnodes(4,1:ndegfd)+coeffnodes(2,1:ndegfd))*fact1+ &
             coeffnodes(6,1:ndegfd)*fact2
         coeffintpnts(4,1:ndegfd) = &
            (coeffnodes(6,1:ndegfd)+coeffnodes(4,1:ndegfd))*fact1+ &
             coeffnodes(2,1:ndegfd)*fact2

         fact1 = alam3(1,6)*2d0
         fact2 = (alam3(1,5)-0.5d0)*2d0
         coeffintpnts(5,1:ndegfd) = &
            (coeffnodes(2,1:ndegfd)+coeffnodes(6,1:ndegfd))*fact1+ &
             coeffnodes(1,1:ndegfd)*fact2
         coeffintpnts(6,1:ndegfd) = &
            (coeffnodes(4,1:ndegfd)+coeffnodes(2,1:ndegfd))*fact1+ &
             coeffnodes(3,1:ndegfd)*fact2
         coeffintpnts(7,1:ndegfd) = &
            (coeffnodes(6,1:ndegfd)+coeffnodes(4,1:ndegfd))*fact1+ &
             coeffnodes(5,1:ndegfd)*fact2

      else
         call errint ( 4, 1 )
         call errsub ( 1497, 1, 0, 0 )
         go to 1000

      end if  ! ( m==4 )

1000  call erclos ( 'elfillcoeffin2' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coeffintpnts, ndegfd, m, 'coeffintpnts/2' )
         write(irefwr,*) 'End elfillcoeffin2'

      end if  ! ( debug )

      end
