      subroutine elfillsol01 ( ichoice, usol, iseqvel, iseqpres, ndim, &
                               n, npres )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill a solution vector for one element depending on the value of ichoice
!     This subroutine is meant for (Navier-) Stokes equations, i.e.
!     there must be velocities and pressures
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     fill_solution
!     element
!     element
!     pressure
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, iseqvel(*), iseqpres(*), ndim, n, npres
      double precision usol(*)

!     ichoice        i    Integer choice parameter
!                         Possible values
!                         1:  u = 1, v = 1, w = 1, p = 0
!     iseqpres       i    Sequence number of pressure with respect to array isol
!     iseqvel        i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
!     n              i    number of basis functions for each component
!     ndim           i    Dimension of the space.
!     npres          i    Number of pressure parameters per element
!     usol           o    Solution vector to be filled for the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillsol01' )
      if ( ichoice.eq.1 ) then

!     --- ichoice = 1, u = 1, p = 0

         do i = 1, n*ndim
            usol(iseqvel(i)) = 1d0
         end do
         do i = 1, npres
            usol(iseqpres(i)) = 0d0
         end do

      else

!     --- Value of ichoice not implemented

         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )

      end if
      call erclos ( 'elfillsol01' )
      end
