      subroutine elflr0 ( ichois, ivalue, ivalh, user, result, x, uold, &
                          index1, phi, index )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 04-07-2003 Layout
!        version  1.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.1  date 02-06-1988
!
!   copyright (c) 1987-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill function values in array result for the integration points
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), index(*), ichois, ivalue, ivalh
      double precision result(*), user(*), uold(*), x(*), phi(*)

!      ichois    i   choice parameter. Possibilities:
!                    1:  Standard situation
!                    2:  Both the arrays uold and index are used
!      index     i   special index array of length m that is only used
!                    when uold is used and ichois>1
!                    index(i) contains the starting point of the nodal
!                    point in uold
!      index1    i   nodal point numbers of the element
!      ivalh     i   result of subroutine eluser when ivalue>2000
!                    the function values will be evaluated in the integration
!     node
!      ivalue    i   result of subroutine eluser (ivalue>0)
!      phi       i   array containing the values of the basis functions
!                    in the gauss points
!      result    o   array containing the function in the integration points
!                    integration points of the element
!      uold      i   array containing the old solution in the nodal points
!      user      i   input array,  see subroutine system
!      x         i   array containing the co-ordinates in the
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xc, yc, zc, work(100), f, h
      integer i, j, ihi, ihst

!     f              array of length m containing the spectral element
!                    vector
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ihi            ?
!     ihst           ?
!     j              Counting variable
!     work           Work array
!     xc             x-coordinate of the midth-point
!     yc             y-coordinate of the midth-point
!     zc             z-coordinate of the midth-point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision funcc1, funccf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FUNCC1         User function subroutine to compute the value of a
!                    coefficient in a point depending on a preceding solution
!     FUNCCF         User function subroutine to compute the value of a
!                    coefficient in a point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elflr0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elflr0'

      end if

      if(ivalue.le.1000) then

!     --- 0 < ivalue < 1000

          do 100 i=1,m
             xc=x(i)
             yc=x(i+m)
             zc=x(i+m*2)
100          result(i)=funccf(ivalue,xc,yc,zc)
      else if(ivalue.le.2000) then

!     --- 1000 < ivalue < 2000

        if(ichois.eq.1) then
          do 120 i=1,m
             xc=x(i)
             yc=x(i+m)
             zc=x(i+2*m)
             do 110 j=1,ncomp
110             work(j)=uold(i+(j-1)*m)
120          result(i)=funcc1(ivalue-1000,xc,yc,zc,work)
        else if(ichois.eq.2) then
          ihst=1
          do 140 i=1,m
             xc=x(i)
             yc=x(i+m)
             zc=x(i+2*m)
             do 130 j=ihst,index(i)
130             work(j-ihst+1)=uold(j)
             ihst=index(i)+1
140          result(i)=funcc1(ivalue-1000,xc,yc,zc,work)
        end if
      else if(ivalue.eq.2001) then

!     --- ivalue = 2001

          if(ivalh.gt.0) then

!         --- ivalh > 0

             do 250 i=1,n
250             result(i)=user(ivalh+index1(i))
          else

!         --- ivalh < 0

             do 260 i=1,m
260             result(i)=0d0
             ihi=0
             do 280 i=1,inpelm
                f=user(-ivalh+index1(i))
                do 270 j=1,m
270                result(j)=result(j)+f*phi(ihi+j)
280             ihi=ihi+n
          end if
      else if(ivalue.eq.2002) then

!     --- ivalue = 2002

          h=user(ivalh+irelem)
          do 310 i=1,m
310          result(i)=h
      end if

      call erclos ( 'elflr0' )
      if ( debug ) write(irefwr,*) 'End elflr0'

      end
