      subroutine eli015 ( ichint, ichfun, mvec, usollc, coor, index1, &
                          value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.9  date 13-08-2013 New call to elp623
!        version  1.8  date 15-02-2004 New call to elm910basefn
!        version  1.7  date 04-07-2003 Remove common celwrk
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine eli015 is used to compute the boundary integral over
!     a biqudratic surface quadrilateral
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichint, mvec, index1(9), ichfun
      double precision coor(3,*)
      double complex usollc(mvec,3), value(*)

!     coor    i   Array containing the co-ordinates of the mesh
!     ichfun  i   indication of the value of f.
!                 Possibible values:
!                 0         f = 1
!                 >0        f = func ( ichfun, x, y, z ) (real case)
!                           or
!                           f = cfunc ( ichfun, x, y, z ) (complex
!                               case)
!                           at this moment only the combination f#1
!                           and ichint=1 is permitted
!     ichint  i   indicates the kind of integral to be computed.
!                 possibilities:
!                 1          /  f u  ds
!                 2          /  f  u.n  ds
!                 9          /  f ds
!     index1  i   Array containing the nodal point numbers of the element
!     mvec    i   Number of components of the solution vector
!     usollc  i   Local solution vector
!     value   o   Contains the value of the computed integralc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipphi, ipqmat, ippsi, ipdpsidx, ipdpdxi, ipend
      double complex som, f(9)
      double precision x(9,3), w(9), dphidx(243), xgauss(27)
      save ipphi, ipqmat, ippsi, ipdpsidx, ipdpdxi

!     dphidx         Array of length n x m x ndim  containing the
!                    derivative
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     f              array of length m containing the rhs vector
!     i              Counting variable
!     ipdpdxi        Starting address of array dphidxi
!     ipdpsidx       Starting address of array dpsidx
!     ipend          Last position in array
!     ipphi          Starting address of array phi in array
!     ippsi          Starting address in array work of psi
!     ipqmat         Starting address of array qmat
!     som            Help parameter to compute a som
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex cfunc

!     CFUNC          Computes the value of the vector in a point as function of
!                    the co-ordinates (complex case)
!     ELP623         Compute basis functions and weights for biquadratic
!                    quadrilateral
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli015' )

!     --- Compute the weight functions

      m = 9
      iweigh = 1
      ideriv = 0
      irule = 1
      igausp = 0
      icoor = 1
      ipphi = 1
      ipqmat = ipphi+81
      ippsi = ipqmat+81
      ipdpsidx = ippsi+36
      ipdpdxi = ipdpsidx+72
      ipend = ipdpdxi+81*2
      if ( ipend>433 ) then

!     --- Array work too small

         call errint ( ipend, 1 )
         call errint ( 432, 2 )
         call errchr ( 'work', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      call elp623 ( coor, x, w, dphidx, xgauss, work(ipphi), index1, &
                    work(ipqmat), work(ippsi), work(ipdpsidx), &
                    work(ipdpdxi) )

      if ( ichfun==0 ) then
         do i = 1, m
            f(i) = (1d0,0d0)
         end do
      else
         do i = 1, m
            f(i) = cfunc ( ichfun, x(i,1), x(i,2), x(i,3) )
         end do
      end if
      if ( ichint==1 ) then
         som = (0d0,0d0)
         do i = 1, m
            som = som + w(i)*f(i)*usollc(1,i)
         end do
         value(1) = som
      else if ( ichint==9 ) then
         som = (0d0,0d0)
         do i = 1, m
            som = som + w(i)*f(i)
         end do
         value(1) = som
      end if

1000  call erclos ( 'eli015' )

      end
