      function elic01 ( icheli, jdegfd, f, w, u, dudx, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 27-12-2005 Fortran 90
!        version  1.2  date 04-07-2003 Remove common celwrk
!        version  1.1  date 04-06-1996 Extension with icheli = 6
!        version  1.0  date 05-08-1993
!
!   copyright (c) 1993-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute a volume integral over one element depending on icheli
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheli, jdegfd
      double precision  dudx(m,*), u(m,*), elic01, f(m), &
                        w(m), work(m)

!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    dudx is only filled if jderiv>0
!                    At this moment dudx is used as one-dimensional array
!                    because, ndim is 1
!     elic01   o     Computed integral
!     f        i     Array containing the fucntion f to be integrated in the
!                    integration points
!     icheli   i     choice parameter. See subroutine integr. Possibilities:
!                    1:   / f(x) dx
!                    2:   / f(x) u      (x) dx
!                                 jdegfd
!                    3-5: / f(x) du      (x)/dx  dx   i=icheld-2
!                                  jdegfd      i
!                    6:   / f(x) |u      (x)| dx
!                                  jdegfd
!     jdegfd   i     Degree of freedom to be used from u
!     u        i     Solution to be integrated in integration points
!     w        i     Contains the weights for the numerical integration
!     work           Work array of length m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!    i      General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!    ddot    Computes inner product of two vectors (BLAS subroutine)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Store f, uf or f du/dx into work

      if ( icheli==1 ) then

!     --- icheli = 1, only f

         do i = 1, m
            work(i) = f(i)
         end do
      else if ( icheli==2 ) then

!     --- icheli = 2, f u

         do i = 1, m
            work(i) = f(i) * u(i,jdegfd)
         end do
      else if ( icheli==6 ) then

!     --- icheli = 6, f |u|

         do i = 1, m
            work(i) = f(i) * abs(u(i,jdegfd))
         end do
      else

!     --- icheli = 3-5, f du/dx

         if ( jdercn==0 ) then

!        --- Derivative constant

            do i = 1, m
               work(i) = f(i) * dudx(1,jdegfd)
            end do
         else

!        --- Derivative variable

            do i = 1, m
               work(i) = f(i) * dudx(i,jdegfd)
            end do
         end if
      end if
      elic01 = ddot ( m, work, 1, w, 1 )
      end
