      subroutine elind0 ( islold, numld1, isub, isup, iuser )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.13 date 15-11-2012 Extra debug
!        version  2.12 date 07-06-2006 Extension for shape 15/16
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine fills array ich in common block celiar for coefficients
!     that depend on a prior solution
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numld1, islold(numld1), isub, isup, iuser(*)

!     islold  i      User input array in which the user puts information
!                    of all prior solutions
!                    Integer array of length 5 x numld1
!     isub    i      Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup    i      Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser   i      Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     numld1  i      Number of old vectors that are stored in VECOLD
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idegfd, inunkp, itypvc, ivec, jnunkp, k, klast, ndegfd
      logical variab, allpnt

!     allpnt         Indication if all points contain the degree of freedom
!                    (true) or not (false)
!     i              Counting variable
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     inunkp         Number of degrees of freedom in first point
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector in array VECOLD.
!                    Hence array VECOLD(IVEC) is considered
!     jnunkp         Number of degrees of freedom in second point
!     k              Counting variable
!     klast          Last relative node index for which a variable is present
!     ndegfd         Number of degrees of freedom in VECTOR(IVEC)
!     variab         Indication if the number of degrees of freedom per point is
!                    constant (false) or not (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivec

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIVEC         Return with vector number corresponding to ivecin
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      785  Coefficient refers to prior solution vector
!           that has not been provided  to the subroutine
!      787  Coefficient refers to prior solution vector that
!           does not contain the required degree of freedom in any point
!      788  Coefficient refers to prior solution vector but
!           the degree of freedom is not present in the first point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elind0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elind0'
         write(irefwr,1) 'isub, isup', isub, isup
         call prinin ( ind(isub), isup-isub+1, 'ind' )
         call prinin ( mst(isub), isup-isub+1, 'mst' )
         call prinin ( ivecnums(isub), isup-isub+1, 'ivecnums' )
  1      format ( a, 1x, (10i8) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Loop over all relevant coefficients

      do i = isub, isup

         if ( debug ) write(irefwr,1) 'i, ind(i)', i, ind(i)
         if ( ind(i)>=2003 .and. ind(i)<=2004 ) then

!        --- ind(i)=2003/4, coefficient depends on prior solution

            ipos = ivecnums(i)

            if ( debug ) write(irefwr,1) 'ipos', ipos

            ivec = inivec(iuser(ipos))

            idegfd = max(1,mst(i))
            iseqtype = oldsols(ivec)%iseqtype
            index4n => indoldsols(iseqtype)%index4

            if ( debug ) write(irefwr,1) 'ivec, idegfd', ivec, idegfd
            if ( ivec>numld1 ) then

!           --- ivec>numld1: error 785

               call errint ( i, 1 )
               call errint ( ivec, 2 )
               call errint ( numld1, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 785, 4, 0, 0 )
               go to 1000

            end if

            itypvc = ks(islold(ivec))%typevec
            if ( debug ) write(irefwr,1) 'itypvc', itypvc
            if ( itypvc==116 ) then

!           --- itypvc = 116, vector of special structure defined per element

               ndegfd = ks(islold(ivec))%n
               if ( ndegfd==inpelm .and. mst(i)==0 ) then
                  ich(i) = 1
               else if ( ndegfd==1 .or. mst(i)/=0 ) then
                  ich(i) = 0
               else
                  ich(i) = 100+ndegfd
               end if

            else if ( itypvc==126 ) then

!           --- itypvc = 126, vector of special structure defined per element
!               per node

               ndegfd = ks(islold(ivec))%n
               ich(i) = 1000+ndegfd

            else if ( itypvc==129 ) then

!           --- itypvc = 129, vector of special structure defined per element
!               per integration point

               ndegfd = ks(islold(ivec))%n
               ich(i) = 2000+ndegfd

            else

!           --- itypvc = 110 or 115
!               Check number of degrees of freedom per point

               variab = .false.
               inunkp = index4n(1)
               if ( debug ) write(irefwr,1) 'inunkp', inunkp
               do k = 2, inpelm
                  if ( index4n(k) - index4n(k-1)/=inunkp ) &
                           then

!                 --- variable degrees of freedom per point

                     variab = .true.
                     go to 110

                  end if
               end do

110            if ( .not. variab ) then

!              --- constant number of degrees of freedom

                  if ( idegfd>inunkp ) then
                     call errint ( i, 1 )
                     call errint ( idegfd, 2 )
                     call errint ( inunkp, 3 )
                     call errint ( ielgrp, 4 )
                     call errsub ( 787, 4, 0, 0 )
                  end if
                  if ( inunkp==1 ) then
                     ich(i) = 1
                  else
                     ich(i) = 2
                  end if

               else

!              --- Variable number of degrees of freedom per point

                  if ( idegfd>inunkp ) then
                     call errint ( i, 1 )
                     call errint ( idegfd, 2 )
                     call errint ( inunkp, 3 )
                     call errint ( ielgrp, 4 )
                     call errsub ( 788, 4, 0, 0 )
                  end if
                  allpnt = .true.
                  klast = inpelm
                  do k = 2, inpelm
                     if ( index4n(k) - index4n(k-1) < &
                             idegfd ) then

!                    --- Degree of freedom idegfd not in this point

                        allpnt = .false.
                        klast = k-1
                        go to 130

                     end if
                  end do

130               if ( allpnt ) then

!                 --- allpnt = true,  degree of freedom in all nodes

                     ich(i) = 5

                  else

!                 --- allpnt = false,  degree of freedom not in all nodes

                     jnunkp = index4n(2) - index4n(1)
                     if ( jnunkp==0 ) then

!                    --- Degree of freedom in vertices, but not in
!                        the mid points

                        if ( inunkp==1 ) then
                           ich(i) = 3
                        else
                           ich(i) = 4
                        end if

                     else if ( klast==inpelm-1 .and. &
                               (ishape==7 .or. ishape==9) ) then

!                    --- Special situation, variable not in centroid

                        ich(i) = 7

                     else if ( klast==10 .and. &
                               (ishape==15 .or. ishape==16) ) then

!                    --- Special situation, variable not in centroids

                        ich(i) = 8

                     else

!                    --- Variable situation

                        ich(i) = 6

                     end if

                  end if

               end if

            end if

         end if

         if ( (ishape==7 .or. ishape==9 .or. ishape==10) .and. &
              (ich(i)==2 .or. ich(i)==5) ) ich(i) = 7

      end do
1000  if ( debug ) then

!           --- Debug information

         write(irefwr,*) 'End elind0'

      end if
      call erclos ( 'elind0' )

      end
