      subroutine elm800dif ( jelp, alpha, elemmt, w, dphidx, psi, phi, &
                             coor2d, index1, iuser, &
                             user, vecloc, second, phixi, x, xgauss )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.3  date 06-08-2014 Extra debug
!        version  2.2  date 13-08-2013 New call to elup02
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill diffusion part of element stifness matrix for elements corresponding
!    to a second order scalar diffusion equation
!    Help subroutine for elm800
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     diffusion
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp, index1(*), iuser(*)
      double precision elemmt(icount**2,*), w(*), dphidx(*), psi(*), &
                       phi(*), alpha(*), user(*), vecloc(*), &
                       coor2d(*), phixi(*), x(n,3), xgauss(m,3)
      logical second

!     alpha          i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemmt         o    Element matrix
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible values:
!                          1:  Subroutine ELP610  (Linear line element)
!                          2:  Subroutine ELP611  (Quadratic line element)
!                          3:  Subroutine ELP620  (Linear triangle)
!                          4:  Subroutine ELP624  (Extended quadratic triangle)
!                          5:  Subroutine ELP621  (Bilinear quadrilateral)
!                          6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                         11:  Subroutine ELP630  (Linear tetrahedron)
!                         12:  Subroutine ELP631  (Quadratic tetrahedron)
!                         13:  Subroutine ELP632  (Trilinear brick)
!                         14:  Subroutine ELP633  (Triquadratic brick)
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i    array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phixi(i,k)
!                             i
!     psi            i    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     second        i/o   Indicates if second derivative term is important in
!                         case of upwinding
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    array of length n x ndim containing the coordinates of
!                         the nodes
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work(6*m), dphidxloc(n*n*ndim), eps(n*6), &
                       a1(27,27), a2(27,27), a3(27,27), &
                       qmat(3*3*9)
      integer ifirstdif
      save ifirstdif

!     a1             work array to be used by elup04
!     a2             work array to be used by elup04
!     a3             work array to be used by elup04
!     dphidxloc      work array to store dphidx corresponding to local element
!                    subroutine
!     eps            Local accuracy
!     ifirstdif      Local parameter ifirst with same meaning as ifirst
!                    but now for local subroutines
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     work           Work array of length 3 x m to be used by lower subroutines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     EL3004         Compute integrals of the shape
!                    /_e alpha grad phi_i . grad phi_j dx
!     EL3005         Compute contribution to element stiffness matrix
!                    corresponding to the second derivative part of a
!                    three-dimensional equation
!     EL3008         Compute contribution to element stiffness matrix
!                    corresponding to the second derivative part of a
!                    one-dimensional equation
!     ELUP00         Add contribution of second derivative part to matrix in the
!                    case of upwinding (2D, quadratic triangles)
!     ELUP01         Add contribution of second derivative part to matrix in the
!                    of upwinding (1D, quadratic element)
!     ELUP02         Compute help arrays for second derivative part in case of
!                    case upwinding. Bilinear and quadratic two-dimensional
!                    element
!     ELUP03         Add contribution of second derivative part to matrix in the
!                    case of upwinding (2D, quadrilaterals)
!     ELUP04         Add contribution of second derivative part to matrix in the
!                    case of upwinding (3D, quadratic elements)
!     ELUP05         Compute help arrays for second derivative part in case of
!                    upwinding. Trilinear and quadratic three-dimensional
!                    element
!     ELUP06         Add contribution of second derivative part to matrix in the
!                    case of upwinding (3D, hexahedra)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800dif' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800dif'
         write(irefwr,1) 'ielem, ifirst, ifirstdif, jelp', &
                          ielem, ifirst, ifirstdif, jelp
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 ) ifirstdif = 0
      if ( jelp<=2 ) then

!     --- One-dimensional elements

         call el3008 ( alpha, elemmt, w, dphidx, n, n )
         if ( idefcr==1 ) call dcopy ( icount**2, &
              elemmt(1,1), 1, elemmt(1,2), 1 )

         jadd = 1
         if ( second .and. jelp==2 .and. jcart==1 ) call &
              elup01 ( elemmt, alpha, dphidx, phi, &
                       psi, w )

      else if ( jelp<=10 ) then

!     --- Two-dimensional elements

         call el3004 ( alpha, elemmt, w, dphidx, m, n, n )
         if ( idefcr==1 ) call dcopy ( icount**2, &
              elemmt(1,1), 1, elemmt(1,2), 1 )
         jadd = 1
         if ( debug ) write(irefwr,*) 'n, m, irule, second ', &
            n, m, irule, second
         if ( n>3 .and. second .and. jelp/=10 ) then

!        --- Second order term may not be neglected in upwind part

            if ( irule==1 ) then

!           --- irule = 1, newton cotes rule

               call elup00 ( elemmt, alpha, dphidx, phi, psi, w, &
                             a1, a2 )

            else

!           --- irule>1, gauss rule

               call elup02 ( alpha, dphidxloc, eps, user, index1, &
                             work, coor2d, jelp, vecloc, &
                             qmat, iuser, a1, a2, phi, phixi )
               call elup03 ( elemmt, dphidx, phi, psi, w, a1, a2 )

            end if

         end if

      else

!     --- Three-dimensional elements

         call el3005 ( alpha, elemmt, w, dphidx, m, n, n, &
                       work )
         if ( idefcr==1 ) call dcopy ( icount**2, &
              elemmt(1,1), 1, elemmt(1,2), 1 )
         jadd = 1
         if ( debug ) then
            write(irefwr,1) 'n, m, irule, idefcr', &
                             n, m, irule, idefcr
            write(irefwr,*) 'second', second
         end if  ! ( debug )

         if ( jelp/=21 .and. n>4 .and. second ) then

!        --- Second order term may not be neglected in upwind part

            if ( irule==1 ) then

!           --- irule = 1, newton cotes rule

               call elup06 ( elemmt, alpha, dphidx, phi, &
                             psi, w, a1, a2, a3 )
            else

!           --- irule>1, gauss rule

               call elup04 ( alpha, dphidxloc, eps, user, index1, &
                             work, coor2d, jelp, vecloc, iuser, &
                             a1, a2, a3, phixi, phi, x, xgauss, w, &
                             ifirstdif )
               call elup05 ( elemmt, dphidx, phi, &
                             psi, w, a1, a2, a3 )

            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'ifirstdif ', ielem, ifirstdif
         write(irefwr,*) 'End elm800dif'

      end if
      call erclos ( 'elm800dif' )

      end
