      subroutine elm816rhsd ( elemvc, vec1, vec2, dphidx, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 13-11-2014
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for elements of type 816
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: elemvc(icount)
      double precision, intent(in) :: vec1(m), vec2(m), &
                                      dphidx(n,m,ndim), weight(m)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemvc         o    Element vector
!     vec1           i    first input vector in quadrature points
!     vec2           i    second input vector in quadrature points
!     weight         i    Array containing the weights
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k
      double precision :: dvec2dx(m,ndim), som

!     dvec2dx        Contains the gradient of vec2
!     i              Counting variable
!     k              Counting variable
!     som            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELUDERIV       Compute the derivative of a given function in one element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The right-hand side vector to be computed is
!
!     f_i = /_e  vec1 grad vec2 . grad phi_i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm816rhsd' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm816rhsd'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the gradient of vec2 (dvec2dx)

      call eluderiv ( vec2, dvec2dx, dphidx, 1, ndim, kstep )

!     --- Compute rhs

      do i = 1, n

         som = 0d0
         do k = 1, m
            som = som + weight(k)*vec1(k)* &
                  dot_product(dphidx(i,k,1:ndim),dvec2dx(k,1:ndim))
         end do  ! k = 1, m

         elemvc(i) = -som

      end do  ! i = 1, n
1000  call erclos ( 'elm816rhsd' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm816rhsd'

      end if  ! ( debug )

      end subroutine elm816rhsd
