      subroutine elp648(x,xpref,phi, n)
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   02-04-1993
!
!   copyright (c) 1993-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     compute quantities for one-dimensional spectral reference
!     element [-1,1]
!
!     computed are:  values of the basis functions
!                    in the points
!***********************************************************************
!
!                      KEYWORDS
!
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer n
      double precision phi(n), x, xpref(n)

!     n          i    number of basis functions for each component
!     phi        i    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!     x          i    x co-ordinate of point
!     xpref      i
!***********************************************************************
!
!                   COMMON BLOCKS
!
!***********************************************************************
!
!                   LOCAL PARAMETERS

      integer i
      double precision rmv, xi, poly1, poly3, poly4, &
                       rdum0,rdum1,rdum2,rdum3,rdum4

!     i          Counting variable
!     poly1
!     poly3
!     poly4
!     rdum0      dummy variable
!     rdum1      dummy variable
!     rdum2      dummy variable
!     rdum3      dummy variable
!     rdum4      dummy variable
!     rmv
!     xi         Gauss-Lobatto point
!***********************************************************************
!
!                 SUBROUTINES CALLED
!
!     ELLEGP  ?
!     ELLGLP    compute the Legendre Gauss-Lobatto points
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  print error messages
!***********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1622   Poly4 = 0
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- compute the Gauss-Lobatto points

      call ellglp(xpref, n)

!     --- convert integer n to double n

      rmv = n

!     --- determine L_{n+1}(x) and L_{n-1}(x)

      call ellegp(n, poly3, rdum1, rdum3, rdum2, poly1, rdum4,x)
      do 100 i = 1, n
         xi = xpref(i)
         call ellegp(n-1, poly4, rdum0, rdum1, rdum2, &
                      rdum3, rdum4, xi)
         if ( poly4.ge.0d0 .and. poly4.le.0d0 ) then

!        --- poly4 = 0

            call eropen ( 'elp648' )
            call errsub ( 1622, 0, 0, 0 )
            call erclos ( 'elp648' )

         end if
         if ( abs(x-xi).lt.1.0d-9) then
             phi(i)=1.0d0
         else
             phi(i) = (poly3 - poly1)/ &
                      ((2.0d0*rmv - 1.0d0)*poly4*(x - xi))
         end if
100   continue
      end
