      subroutine elplambda1d ( lambda, wh, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the one-dimensional linear basis functions in case of
!     a non-Newton Cotes rule
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision lambda(m), wh(m)
      character (len=*) text

!     lambda         o    Array containing the linear basis functions in the
!                         integration points
!     text           i    String variable containing some text
!     wh             o    Contains the weights in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision d

!     d              help parameter to compute a constant
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elplambda1d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elplambda1d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( irule )

         case(2)

!        --- irule = 2  one-point Gauss rule

            lambda(1) = 0.5d0
            wh(1) = 1d0

         case(3)

!        --- irule = 3  two-point Gauss rule

            lambda(1:2) = alam4(1:2)
            wh(1:2) = 0.5d0

         case(4)

!        --- irule = 4  three-point Gauss rule

            lambda(1:3) = alam5(1:3)
            wh(1:3) = wgs5(1:3)

         case(5)

!        --- irule = 5  Simpson's rule

            do k = 1, 3
               lambda(k) = (k-1)*0.5d0
            end do

            d = 1d0/6d0
            wh(1) = d
            wh(2) = 4d0*d
            wh(3) = d

         case default

!        --- Error value of irule not implemented

         call errorelvar ( irule, 1770, text, 1 )
         irule = 1

      end select  ! case ( irule )

1000  call erclos ( 'elplambda1d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elplambda1d'

      end if  ! ( debug )

      end
