      subroutine elpup1 ( alpha, u, phi, psi, dphidx, x, second )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.4  date 04-07-2003 Remove common celwrk
!        version  3.3  date 22-05-1995 Rename ctime in ctimen
!        version  3.2  date 28-06-1994 Correction upwind with u
!        version  3.1  date 29-09-1993 Extension with jsecnd = 0
!        version  3.0  date 09-09-1993 Removal of ndecl and mdecl
!        version  2.0  date 01-03-1993 extra parameter second
!        version  1.0  date 27-10-1992
!
!   copyright (c) 1992-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute modified basis functions for upwind elements
!    One-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(m), u(m), phi(n,m), psi(n,m), &
                       dphidx(n,m), x(n)
      logical second

!     alpha    i     Array of length m  containing the factors alpha
!                    in the integration points
!     dphidx   i     Array of length n x m containing the values of the
!                    derivatives of the basis functions in the integration
!     node
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points
!     psi      o     Array of length n x m containing the values of the modified
!                    basis functions in the integration points
!     second   o     Indicates if the second order term is significant or may
!                    be neglected in the upwinding part
!     u        i     Array of length m containing the velocity in the
!                    integration points
!     x        i     Contains the co-ordinates of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision beta, h, xksi(6), tau(6)
      integer i, k, metupl

!     beta           factor beta (see method)
!     h              element width (see method)
!     i              Counting variable
!     k              Counting variable
!     metupl         mod(metupw,100): defines type of upwinding
!     tau            Array of length 6 to store the values of tau in the
!                    integration points
!     xksi           Array of length m to store the values of ksi in the
!                    integration points (see method)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!    - &
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p  = tau u * d phi  / dx
!        i                i
!                   h xi                     uh
!       with tau = ------ beta =  ----- &
!                     2                     2 alpha
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi = &
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       h = element diameter (linear) or half diameter (quadratic)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      h = x(2) - x(1)
      metupl = mod ( metupw, 100 )

!     --- Compute beta and ksi in the integration points

      if ( metupl.le.4 ) then

!     --- metupl = 1, .. , 4:  old methods

         do i = 1, m
            if ( jsecnd.gt.0 .and. alpha(i).gt.1d-10 ) then

!           --- alpha(i) > 1d-10,  hence second order term plays a role

               beta = u(i)*h/(2d0*alpha(i))
               if ( abs(beta).lt.1d-10 ) then
                  xksi(i) = 0d0
               else if ( metupl.eq.1 ) then

!              --- metupl = 1   classical upwind scheme

                  xksi(i) = 1d0
               else if ( metupl.eq.2 ) then

!              --- metupl = 2   il'in scheme

                  xksi(i) = 1d0/tanh(beta)-1d0/beta
               else if ( metupl.eq.3 ) then

!              --- metupl = 3    doubly assymptotic approximation

                  if ( abs(beta).gt.3d0 ) then
                     xksi(i)=sign(1d0,beta)
                  else
                     xksi(i) = beta / 3d0
                  end if
               else if ( metupl.eq.4 ) then

!              --- metupl = 4    critical approxksi(i)mation

                  if ( beta.lt.-1d0 ) then
                     xksi(i) = -1d0 - 1d0 / beta
                  else if ( beta.gt.1d0 ) then
                     xksi(i) = 1d0 - 1d0 / beta
                  else
                     xksi(i) = 0d0
                  end if
               end if
            else
               xksi(i) = 1d0
            end if
         end do

!        --- compute new basis functions, it has been assumed that the old
!            basis functions are already stored in array phi

         if ( jdercn.eq.0 ) then

!        --- jdercn = 0,  constant derivatives

            do k = 1, m
               do i = 1, n
                  psi(i,k) = phi(i,k) + u(k)*h*xksi(k)*0.5d0 * &
                             dphidx(i,1)
               end do
            end do
         else

!        --- jdercn = 1,  variable derivatives

            do k = 1, m
               do i = 1, n
                  psi(i,k) = phi(i,k) + u(k)*h*xksi(k)*0.5d0 * &
                             dphidx(i,k)
               end do
            end do
         end if
      else

!     --- metupl = 5, 6:  new methods

         if ( metupl.eq.5 ) then

!        --- metupl = 5

            do i = 1, m
               if ( jsecnd.gt.0 .and. alpha(i).gt.1d-10 ) then

!              --- alpha(i) > 1d-10,  hence second order term plays a role

                  beta = abs(u(i))*h/(2d0*alpha(i))
                  if ( abs(u(i)).le.1d-5*alpha(i) ) then
                     tau(i) = 0d0
                  else
                     beta = beta*beta
                     tau(i) = h/(2d0*abs(u(i)))*sqrt(beta/(9d0+beta))
                  end if
               else if ( u(i).le.1d-10*alpha(i) ) then
                  tau(i) = 0d0
               else
                  tau(i) = h/(2d0*abs(u(i)))
               end if
            end do
         else

!        --- metupl = 6

            if ( jsecnd.eq.0 ) then

!           --- second order term not available

               do i = 1, m
                  beta = (2d0*abs(u(i))/h)**2
                  if ( tstep.gt.0d0 ) beta = beta + 4d0/(tstep*tstep)
                  tau(i) = 1d0/sqrt(beta)
               end do
            else

!           --- second order term available

               do i = 1, m
                  beta = (2d0*abs(u(i))/h)**2 + (4d0*alpha(i)/(h*h))**2
                  if ( tstep.gt.0d0 ) beta = beta + 4d0/(tstep*tstep)
                  tau(i) = 1d0/sqrt(beta)
               end do
            end if
         end if
         if ( jdercn.eq.0 ) then

!        --- jdercn = 0,  constant derivatives

            do k = 1, m
               do i = 1, n
                  psi(i,k) = phi(i,k) + u(k) * tau(k) * dphidx(i,1)
               end do
            end do
         else

!        --- jdercn = 1,  variable derivatives

            do k = 1, m
               do i = 1, n
                  psi(i,k) = phi(i,k) + u(k) * tau(k) * dphidx(i,k)
               end do
            end do
         end if
      end if
      do k = 1, m
         if ( abs(alpha(k))*1d2.gt.abs(u(k)) ) second = .true.
      end do
      end
