      subroutine els000 ( ichels, jnodp, strmj, elemvc, coor, index1, &
                          index2, index3, ishape )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  5.2  date 12-12-2012 Long integers
!        version  5.1  date 12-10-2005 Extension with 605
!        version  5.0  date 23-12-1996 Extra parameter ishape
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute element vector depending on the values
!    of the parameters
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index3(*)
      integer ichels, jnodp, index1(*), index2(*), ishape
      double precision strmj, coor(*), elemvc(*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     elemvc  o    Element vector
!     ichels  i    ?
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     index2  i    Array of length icount containing the positions of the
!                  degrees of freedom in the element with respect to usol
!     index3  i    cumulative number of degrees of freedom
!                  in the nodal points of element ielem
!     ishape  i    Shape number of elements
!     jnodp   i    ?
!     strmj   i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELS400  ?
!     ELSTRM  ?
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     221   Subroutine elm... or els... not available for this type number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'els000' )
      select case ( itype )
         case ( :-1, 102, 103, 106, 107, 112:399, 401, 403, 405, &
                407:413, 418, 419, 423:428, 432, 434:439, 444:606, &
                608:614, 616:699, 702, 703, 706:799, 801:899, 904, &
                906:998, 1000: )

!        --- Type number has not yet been implemented

            call errint ( itype, 1 )
            call errsub ( 221, 1, 0, 0 )
         case ( 1:99 )

!        --- user element routine

            call elstrm ( ichels, jnodp, strmj, elemvc, coor, index1, &
                          velocity, index2, index3 )
         case ( 100, 101, 104, 105, 108:111, 400, 402, 404, 406, &
                414:417, 420:422, 429:431, 433, 440:443, 607, 615, &
                700, 701, 704, 705, 800, 900:903, 905, 999 )

!        --- Navier-Stokes element subroutine

            call els400 ( ichels, jnodp, strmj, elemvc, coor, index1, &
                          index2, index3, ishape )
      end select  ! case ( itype )
      call erclos ( 'els000' )
      end
