      subroutine els400 ( ichels, jnodp, psi, elemvc, coor, indlocex1, &
                          indlocex2, indlocex3, ishape )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  7.3  date 12-12-2012 Long integers
!        version  7.2  date 31-03-2008 Correction shape 10
!        version  7.1  date 12-10-2005 Include commons
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     ichels=1 : stream function
!     ichels=2 : potential function
!                linear and quadratic conforming elements
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleconsts
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indlocex3(*)
      integer ichels, jnodp, indlocex1(*), indlocex2(*), ishape
      double precision psi, elemvc(*), coor(*)

!     coor    i    array of length ndim x npointcac containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     elemvc  i    Element vector
!     ichels  i    Choice parameter (=1: stream function; =2: potential)
!     indlocex1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     indlocex2  i    Array of length icount containing the positions of the
!                  degrees of freedom in the element with respect to usol
!     indlocex3  i    Two-dimensional integer array of length NUMOLD x NindlocEX
!                  containing the positions of the "old" solutions in array
!                  VECOLD with respect to the present element
!                  For example VECOLD(indlocEX3(i,j)) contains the j th
!                  unknown with respect to the i th old solution vector.
!                  The number i refers to the i th vector corresponding to
!                  IVCOLD in the call of SYSTM2 or DERIVA
!                  (In this case NUMOLD=1)
!     ishape  i    Shape number of elements
!     jnodp   i    Nodal point number
!     psi     i    Value of stram function (or potential) in jnodp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,i0,i1,i2,ih,ih1,ih2,ii,inodp,ipp,ivert, &
              jd,k,npnt,jnpelm
      integer iv
      integer indloc(16)
      double precision h,p1k,p1k3,p1k6,p2k,p2k3,p2k6,su1,su2, &
                       sump1,sump2,sums1,sums2,ug11,ug12,ug21,ug22, &
                       xg11,xg12,xx1,xx2,yy1,yy2
      double precision x1(9), x2(9), uvel1(9), uvel2(9), wg(3), phig1(9), &
             phig2(9), xksi1(3), xksi2(3), dxdks1(3), dxdks2(3), &
             dydks1(3), dydks2(3)
      save ipp,indloc,wg,phig1,xksi1,phig2,xksi2,x1,x2

!     dxdks1  ?
!     dxdks2  ?
!     dydks1  ?
!     dydks2  ?
!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     i0      ?
!     i1      ?
!     i2      ?
!     ih      Help variable to store a constant temporarily
!     ih1     Help variable to store a constant temporarily
!     ih2     Help variable to store a constant temporarily
!     ii      ?
!     indloc     Integer array containing sequence number of vertices
!     inodp   Nodal point number
!     ipp     ?
!     iv      ?
!     ivert   ?
!     jd      ?
!     jnpelm  Number of nodes in element, withour centroid
!     k       Counting variable
!     npnt    ?
!     nvert   Number of vertices
!     p1k     ?
!     p1k3    ?
!     p1k6    ?
!     p2k     ?
!     p2k3    ?
!     p2k6    ?
!     phig1   ?
!     phig2   ?
!     su1     ?
!     su2     ?
!     sump1   ?
!     sump2   ?
!     sums1   ?
!     sums2   ?
!     uvel1      ?
!     uvel2      ?
!     ug11    ?
!     ug12    ?
!     ug21    ?
!     ug22    ?
!     wg      ?
!     x1      x_1-coordinates of nodal points
!     x2      x_2-coordinates of nodal points
!     xg11    ?
!     xg12    ?
!     xksi1   ?
!     xksi2   ?
!     xx1     ?
!     xx2     ?
!     yy1     ?
!     yy2     ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     219   Degree of freedom is not available in point
!     220   Specified nodal point number is not a vertex of the element
!     221   Subroutine elm... or els... not available for this type number
!     223   Parameter ichels has illegal value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data wg /0.5555555555555556d0, 0.8888888888888889d0, &
               0.5555555555555556d0/
      data phig1 /0.687298334620742d0 ,0.0d0, -0.087298334620742d0, &
                  0.4d0               ,1.0d0,   0.4d0, &
                 -0.087298334620742d0 ,0.0d0, 0.687298334620742d0/
      data xksi1/-0.774596669241483d0 ,0.0d0, 0.774596669241483d0/
      data phig2 /0.837298334620742d0 ,0.375d0, 0.062701665379258d0, &
                  0.212701665379258d0 ,0.75d0 , 0.987298334620742d0, &
                 -0.05d0              ,-0.125d0,-0.05d0/
      data xksi2/-0.887298334620742d0 ,-0.5d0 ,-0.112701665379258d0/
! ======================================================================
!
      call eropen( 'els400' )
      if ( ichels<1 .or. ichels>2 ) then
         call errsub ( 223, 0, 0, 0 )
         goto 1000
      end if
      ipp=inpelm
      if ( inpelm==5 .or. inpelm==7 .or. inpelm==9 .or. &
           ishape==10 ) ipp=ipp-1
      jnpelm = ipp

      do i=1,ipp
         indloc(i)=i
         indloc(i+ipp)=i
         x1(i) = 0d0
         x2(i) = 0d0
         if (ichels==1 .and. indlocex3(i+1)-indlocex3(i)<2) then
            call errint(2,1)
            call errint(indlocex1(i),2)
            call errint(indlocex3(i+1)-indlocex3(i),3)
            call errsub ( 219, 3, 0, 0 )
         end if
      end do

!     --- get coordinates and velocities
!         for velocities the first and id'th degrees of freedom
!         in each nodal point

      jd=1
      if ( itype==406 .or. itype==417 .or. itype==429 .or. &
           itype==433 .or. itype==443 ) jd=2
      do i=1,inpelm
         ih=2*indlocex1(i)
         x1(i)=coor(ih-1)
         x2(i)=coor(ih)
      end do
      if ( itype==104 .or. itype==105 .or. itype==110 .or. &
           itype==111 ) then
         nvert=jnpelm/2
         do i=1,nvert
            iv=indlocex3(2*i-1)+1
            uvel1(2*i-1)=velocity(indlocex2(iv))
            uvel2(2*i-1)=velocity(indlocex2(iv+jd))
         end do

!        --- simple interpolation

         uvel1(ipp+1)=uvel1(1)
         uvel2(ipp+1)=uvel2(1)
         do i=2,ipp,2
            uvel1(i)=0.5d0*(uvel1(i-1)+uvel1(i+1))
            uvel2(i)=0.5d0*(uvel2(i-1)+uvel2(i+1))
         end do

!        --- kees kassels

         if ( itype==110 .or. itype==111 ) then
            su1=0d0
            su2=0d0
            do i=1,ipp
              su1=su1+uvel1(i)
              su2=su2+uvel2(i)
            end do
            uvel1(ipp+1)=su1 /ipp
            uvel2(ipp+1)=su2 /ipp
         end if
      else
         do i=1,ipp
            iv=indlocex3(i)+1
            uvel1(i)=velocity(indlocex2(iv))
            uvel2(i)=velocity(indlocex2(iv+jd))
         end do
      end if
      ih1=3
      ih2=12
      h=1/6.0d0
      ivert=0
      if ( inpelm<6 ) then
         do i=1,inpelm
            inodp=indlocex1(i)
            if(inodp==jnodp) ivert=i
         end do
      else
         nvert=jnpelm/2
         do i=1,nvert
            inodp=indlocex1(i+i-1)
            if(inodp==jnodp) ivert=i
         end do
      end if
      if (ivert==0) then

!     --- jnodp is not a vertex

         call errint ( jnodp, 1 )
         call errint ( ielem, 2 )
         call errsub ( 220, 2, 0, 0 )
         goto 1000
      end if

!     --- findloc what type of coordinates are to be used

      if ( itype==100 .or. itype==104 .or. itype==108 .or. &
           itype==110 .or. itype==999 .or. itype==615) jcart=1
      if ( itype==400 .or. itype==414 .or. &
           itype==420 .or. itype==430 .or. itype==440 .or. &
           itype==700 .or. itype==701 .or. itype==800 ) &
           jcart=1
      if ( itype==402 .or. itype==415 .or. &
           itype==422 .or. itype==441) &
           jcart=3
      if ((itype==101).or.(itype==105).or.(itype==109).or. &
          (itype==111)) jcart=2
      if ( itype==404 .or. itype==416 .or. &
           itype==421 .or. itype==431 .or. itype==442 .or. &
           itype==704 .or. itype==705 ) &
           jcart=2
      if ( itype==406 .or. itype==417 .or. itype==429 .or. &
           itype==433 .or. itype==443) jcart=2
      goto (100,200,400),jcart
         call errint(itype,1)
         call errsub ( 221, 1, 0, 0 )
         goto 1000

!        --- compute psi/phi in vertices  ( cartesian coordinates )

100   if (ichels==1) then

!     --- stream function

         if (ishape==10 .or. inpelm<=5 ) then

!        --- linear element

            inodp=ivert
            elemvc(inodp)=psi
            npnt=inodp+jnpelm-1
            do i=inodp,npnt
               i0=indloc(i)
               i1=indloc(i+1)
               xx2=-x1(i0)+x1(i1)
               yy2=-x2(i0)+x2(i1)
               elemvc(i1)=elemvc(i0)+ &
                   (yy2*(uvel1(i0)+uvel1(i1)) &
                   -xx2*(uvel2(i0)+uvel2(i1)))/2
            end do

         else

!        --- compute psi along straight and/or curved boundaries

            inodp=ivert*2-1
            elemvc(inodp)=psi
            npnt=inodp+jnpelm-2
            do i=inodp,npnt,2
               i0=indloc(i)
               i1=indloc(i+1)
               i2=indloc(i+2)
               xx1=x1(i0)-2*x1(i1)+x1(i2)
               yy1=x2(i0)-2*x2(i1)+x2(i2)
               xx2=-0.5d0*(x1(i0)-x1(i2))
               yy2=-0.5d0*(x2(i0)-x2(i2))
               if ((abs(xx1)<1.0d-6*abs(xx2)) .and. &
                   (abs(yy1)<1.0d-6*abs(yy2))) then

!              --- boundary is straight

                  elemvc(i2)=elemvc(i0)+ &
                     (yy2*(uvel1(i0)+4*uvel1(i1)+uvel1(i2)) &
                     -xx2*(uvel2(i0)+4*uvel2(i1)+uvel2(i2)))/ih1
                  elemvc(i1)=elemvc(i0)+ &
                    (yy2*(5*uvel1(i0)+8*uvel1(i1)-uvel1(i2)) &
                    -xx2*(5*uvel2(i0)+8*uvel2(i1)-uvel2(i2)))/ih2
               else

!              --- boundary is curved

                  elemvc(i2)=elemvc(i0)+ &
                     (yy2*(uvel1(i0)+4*uvel1(i1)+uvel1(i2)) &
                     -yy1*(uvel1(i0)-uvel1(i2)) &
                     -xx2*(uvel2(i0)+4*uvel2(i1)+uvel2(i2)) &
                     +xx1*(uvel2(i0)-uvel2(i2)))/ih1
                  elemvc(i1)=elemvc(i0)+ &
                    (yy2*(5*uvel1(i0)+8*uvel1(i1)-uvel1(i2)) &
                    -yy1*(3.5d0*uvel1(i0)+3*uvel1(i1)-0.5d0*uvel1(i2)) &
                    -xx2*(5*uvel2(i0)+8*uvel2(i1)-uvel2(i2)) &
                    +xx1*(3.5d0*uvel2(i0)+3*uvel2(i1)-0.5d0*uvel2(i2)))/ih2
               end if
            end do
         end if
      else

!     --- potential function

         if ( inpelm<=4 .or. ishape==10 ) then

!        --- linear element

            inodp=ivert
            elemvc(inodp)=psi
            npnt=inodp+jnpelm-1
            do i=inodp,npnt
               i0=indloc(i)
               i1=indloc(i+1)
               xx2=-x1(i0)+x1(i1)
               yy2=-x2(i0)+x2(i1)
               elemvc(i1)=elemvc(i0)+ &
                   (xx2*(uvel1(i0)+uvel1(i1)) &
                   +yy2*(uvel2(i0)+uvel2(i1)))/2
            end do
         else

!        --- compute phi along straight and/or curved boundaries

            inodp=ivert*2-1
            elemvc(inodp)=psi
            npnt=inodp+jnpelm-2
            do i=inodp,npnt,2
               i0=indloc(i)
               i1=indloc(i+1)
               i2=indloc(i+2)
               xx1=x1(i0)-2*x1(i1)+x1(i2)
               yy1=x2(i0)-2*x2(i1)+x2(i2)
               xx2=-0.5d0*(x1(i0)-x1(i2))
               yy2=-0.5d0*(x2(i0)-x2(i2))
               if ((abs(xx1)<1.0d-6*abs(xx2)) .and. &
                   (abs(yy1)<1.0d-6*abs(yy2))) then

!              --- boundary is straight

                  elemvc(i2)=elemvc(i0)+ &
                     (xx2*(uvel1(i0)+4*uvel1(i1)+uvel1(i2)) &
                     +yy2*(uvel2(i0)+4*uvel2(i1)+uvel2(i2)))/3
                  elemvc(i1)=elemvc(i0)+ &
                    (xx2*(5*uvel1(i0)+8*uvel1(i1)-uvel1(i2)) &
                    +yy2*(5*uvel2(i0)+8*uvel2(i1)-uvel2(i2)))/12
               else

!              --- boundary is curved

                  elemvc(i2)=elemvc(i0)+ &
                     (xx2*(uvel1(i0)+4*uvel1(i1)+uvel1(i2)) &
                     -xx1*(uvel1(i0)-uvel1(i2)) &
                     +yy2*(uvel2(i0)+4*uvel2(i1)+uvel2(i2)) &
                     -yy1*(uvel2(i0)-uvel2(i2)))/3
                  elemvc(i1)=elemvc(i0)+ &
                    (xx2*(5*uvel1(i0)+8*uvel1(i1)-uvel1(i2)) &
                    -xx1*(3.5d0*uvel1(i0)+3*uvel1(i1)-0.5d0*uvel1(i2)) &
                    +yy2*(5*uvel2(i0)+8*uvel2(i1)-uvel2(i2)) &
                    -yy1*(3.5d0*uvel2(i0)+3*uvel2(i1)-0.5d0*uvel2(i2)))/12
               end if
            end do
         end if
      end if
      goto 1000

!     --- compute psi/phi in vertices   ( axis-symmetric coordinates )

200   if ( inpelm<=5 ) then

!     --- linear element (simpson integration)

         inodp=ivert
         elemvc(inodp)=psi
         npnt=inodp+jnpelm-1
         if ( ichels==1 ) then

!        --- stream function

            do i=inodp,npnt
               i0=indloc(i)
               i1=indloc(i+1)
               xx2=-x1(i0)+x1(i1)
               yy2=-x2(i0)+x2(i1)
               elemvc(i1)=elemvc(i0)+ &
                   (yy2*(x1(i0)*(2*uvel1(i0)+uvel1(i1)) &
                        +x1(i1)*(2*uvel1(i1)+uvel1(i0))) &
                   -xx2*(x1(i0)*(2*uvel2(i0)+uvel2(i1)) &
                        +x1(i1)*(2*uvel2(i1)+uvel2(i0))))*h*2*pi
            end do
         else

!        --- potential function

            do i=inodp,npnt
               i0=indloc(i)
               i1=indloc(i+1)
               xx2=-x1(i0)+x1(i1)
               yy2=-x2(i0)+x2(i1)
               elemvc(i1)=elemvc(i0)+ &
                   (xx2*(x1(i0)*(2*uvel1(i0)+uvel1(i1)) &
                        +x1(i1)*(2*uvel1(i1)+uvel1(i0))) &
                   +yy2*(x1(i0)*(2*uvel2(i0)+uvel2(i1)) &
                        +x1(i1)*(2*uvel2(i1)+uvel2(i0))))/6
            end do
         end if
      else

!     --- compute psi/phi along straight and/or curved boundaries

         inodp=ivert*2-1
         elemvc(inodp)=psi
         npnt=inodp+jnpelm-2
         do ii=inodp,npnt,2
            i0=indloc(ii)
            i1=indloc(ii+1)
            i2=indloc(ii+2)
            xx1=x1(i0)-2*x1(i1)+x1(i2)
            yy1=x2(i0)-2*x2(i1)+x2(i2)
            xx2=-0.5d0*(x1(i0)-x1(i2))
            yy2=-0.5d0*(x2(i0)-x2(i2))
            do i=1,3
               dxdks1(i)=xksi1(i)*xx1+xx2
               dxdks2(i)=xksi2(i)*xx1+xx2
               dydks1(i)=xksi1(i)*yy1+yy2
               dydks2(i)=xksi2(i)*yy1+yy2
            end do
            sums1=0.0d0
            sums2=0.0d0
            sump1=0.0d0
            sump2=0.0d0
            do k=1,3

!           --- compute uvel1,uvel2,x1 in integration points on both intervals
!               avoid indlocicator elemvcs

               p1k =phig1(k)
               p1k3=phig1(k+3)
               p1k6=phig1(k+6)
               p2k =phig2(k)
               p2k3=phig2(k+3)
               p2k6=phig2(k+6)
               ug11=uvel1(i0)*p1k+uvel1(i1)*p1k3+uvel1(i2)*p1k6
               ug12=uvel1(i0)*p2k+uvel1(i1)*p2k3+uvel1(i2)*p2k6
               ug21=uvel2(i0)*p1k+uvel2(i1)*p1k3+uvel2(i2)*p1k6
               ug22=uvel2(i0)*p2k+uvel2(i1)*p2k3+uvel2(i2)*p2k6
               xg11=x1(i0)*p1k+x1(i1)*p1k3+x1(i2)*p1k6
               xg12=x1(i0)*p2k+x1(i1)*p2k3+x1(i2)*p2k6
               sums1=sums1+(ug11*dydks1(k)-ug21*dxdks1(k))*xg11*wg(k)
               sums2=sums2+(ug12*dydks2(k)-ug22*dxdks2(k))*xg12*wg(k)
               sump1=sump1+(ug11*dxdks1(k)+ug21*dydks1(k))*xg11*wg(k)
               sump2=sump2+(ug12*dxdks2(k)+ug22*dydks2(k))*xg12*wg(k)
            end do
            if (ichels==1) then
               elemvc(i2)=elemvc(i0)+sums1*2*pi
               elemvc(i1)=elemvc(i0)+sums2*pi
            end if
            if (ichels==2) then
               elemvc(i2)=elemvc(i0)+sump1
               elemvc(i1)=elemvc(i0)+sump2*0.5d0
            end if
         end do
      end if
      goto 1000

!     --- compute psi/phi in vertices   ( polar coordinates )

400   if ( inpelm<=5 ) then

!     --- linear element (simpson integration)

         inodp=ivert
         elemvc(inodp)=psi
         npnt=inodp+jnpelm-1
         if (ichels==1) then

!        --- stream function

            do i=inodp,npnt
               i0=indloc(i)
               i1=indloc(i+1)
               xx2=-x1(i0)+x1(i1)
               yy2=-x2(i0)+x2(i1)
               elemvc(i1)=elemvc(i0)+ &
                   (yy2*(x1(i0)*(2*uvel1(i0)+uvel1(i1)) &
                        +x1(i1)*(2*uvel1(i1)+uvel1(i0))) &
                   -xx2*(2*uvel2(i0)+uvel2(i1) &
                        +2*uvel2(i1)+uvel2(i0)))*h
            end do
         else

!        --- potential function

            do i=inodp,npnt
               i0=indloc(i)
               i1=indloc(i+1)
               xx2=-x1(i0)+x1(i1)
               yy2=-x2(i0)+x2(i1)
               elemvc(i1)=elemvc(i0)+ &
                   (xx2*(2*uvel1(i0)+uvel1(i1) &
                        +2*uvel1(i1)+uvel1(i0)) &
                   +yy2*(x1(i0)*(2*uvel2(i0)+uvel2(i1)) &
                        +x1(i1)*(2*uvel2(i1)+uvel2(i0))))/6
            end do
         end if
      else

!     --- compute psi/phi along straight and/or curved boundaries

         inodp=ivert*2-1
         elemvc(inodp)=psi
         npnt=inodp+jnpelm-2
         do ii=inodp,npnt,2
            i0=indloc(ii)
            i1=indloc(ii+1)
            i2=indloc(ii+2)
            xx1=x1(i0)-2*x1(i1)+x1(i2)
            yy1=x2(i0)-2*x2(i1)+x2(i2)
            xx2=-0.5d0*(x1(i0)-x1(i2))
            yy2=-0.5d0*(x2(i0)-x2(i2))
            do i=1,3
               dxdks1(i)=xksi1(i)*xx1+xx2
               dxdks2(i)=xksi2(i)*xx1+xx2
               dydks1(i)=xksi1(i)*yy1+yy2
               dydks2(i)=xksi2(i)*yy1+yy2
            end do
            sums1=0.0d0
            sums2=0.0d0
            sump1=0.0d0
            sump2=0.0d0
            do k=1,3

!           --- compute uvel1,uvel2,x1 in integration points on both intervals
!               avoid indlocicator elemvcs

               p1k =phig1(k)
               p1k3=phig1(k+3)
               p1k6=phig1(k+6)
               p2k =phig2(k)
               p2k3=phig2(k+3)
               p2k6=phig2(k+6)
               ug11=uvel1(i0)*p1k+uvel1(i1)*p1k3+uvel1(i2)*p1k6
               ug12=uvel1(i0)*p2k+uvel1(i1)*p2k3+uvel1(i2)*p2k6
               ug21=uvel2(i0)*p1k+uvel2(i1)*p1k3+uvel2(i2)*p1k6
               ug22=uvel2(i0)*p2k+uvel2(i1)*p2k3+uvel2(i2)*p2k6
               xg11=x1(i0)*p1k+x1(i1)*p1k3+x1(i2)*p1k6
               xg12=x1(i0)*p2k+x1(i1)*p2k3+x1(i2)*p2k6
               sums1=sums1+(ug11*xg11*dydks1(k)-ug21*dxdks1(k))*wg(k)
               sums2=sums2+(ug12*xg12*dydks2(k)-ug22*dxdks2(k))*wg(k)
               sump1=sump1+(ug11*dxdks1(k)+ug21*xg11*dydks1(k))*wg(k)
               sump2=sump2+(ug12*dxdks2(k)+ug22*xg12*dydks2(k))*wg(k)
            end do
            if (ichels==1) then
               elemvc(i2)=elemvc(i0)+sums1
               elemvc(i1)=elemvc(i0)+sums2*0.5d0
            end if
            if (ichels==2) then
               elemvc(i2)=elemvc(i0)+sump1
               elemvc(i1)=elemvc(i0)+sump2*0.5d0
            end if
         end do
      end if

1000  call erclos( 'els400' )
      end
