      subroutine elusr3 ( number, value, iuser, user, icoeff )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 16-04-2013 Correction scalars
!        version  2.4  date 20-08-2007 Allow coef %scalar
!        version  2.3  date 22-11-2004 Fortran 90
!
!   copyright (c) 1989-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      this subroutine is used to fill constants from array user and
!      iuser as help quantities for element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecons
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!     include 'SPcommon/csubwkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), value(m)
      integer iuser(*), number, icoeff

!     icoeff   i     Sequence number of coefficient for error messages
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     number   i     Sequence number of coefficient to be filled with respect
!                    to the arrays in common block celiar
!     user     i     User array in which real information of the
!                    coefficients is stored (See Standard Problems)
!     value    o     Array of length m in which the value of the coefficient
!                    is stored if the coefficient is a constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nuserh, maxpos, jelgrp, ivectr
      double precision help

!     help           help variable to store a constant
!     ivectr         reduced vector sequence number
!     jelgrp         Sequence number of element group
!     maxpos         Last position in iuser that may be used
!     nuserh         Help variable to compute nuser
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivec
      double precision getscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETSCAL        Return with value of scalar
!     INIVEC         Return with vector number corresponding to ivecin
!     PRININLARGE    Print 1d integer vector (large numbers)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     795   Number of old vector out of range
!     796   ICHOIS for FUNCCF incorrect
!     797   Constant for multiplication incorrect of reference to not allowed
!    2263   element group number exceeds iuser(2)
!    2264   ipos in iuser too large for element group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elusr3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elusr3'
         write(irefwr,1) 'ielgrp', ielgrp
         call prininlarge ( iuser, iuser(3), 'iuser' )
  1      format ( a, 1x, (10i11) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array value, in order to prevent later overflows

      value = 0d0

      if ( ielgrp>iuser(2) ) then

!     --- ielgrp too large

         call errint ( ielgrp, 1 )
         call errint ( iuser(2), 2 )
         call errint ( itype, 3 )
         call errsub ( 2263, 3, 0, 0 )
         go to 1000

      end if

      if ( ielgrp==iuser(2) ) then

!     --- ielgrp is largest element group

         maxpos = iuser(1)

      else

!     --- ielgrp is not the largest element group

         maxpos = iuser(ielgrp+6)-1
         if ( maxpos<=0 ) then

!        --- maxpos<0, because next element not stored
!            first check if there are larger elements that have coefficients

            jelgrp = ielgrp+1
100         if ( jelgrp<=nelgrp ) then

!           --- jelgrp<=nelgrp

               maxpos = iuser(jelgrp+6)-1
               if ( maxpos<=0 ) then

!              --- still no value found

                  jelgrp = jelgrp+1
                  go to 100

               end if

            else

!           --- ielgrp is largest element group

               maxpos = iuser(1)

            end if

         end if

      end if
      if ( ipos>maxpos ) then

!     --- ipos too large

         call errint ( ielgrp, 1 )
         call errint ( itype, 2 )
         call errint ( icoeff, 3 )
         call errint ( ipos, 4 )
         call errint ( maxpos, 5 )
         call errsub ( 2264, 5, 0, 0 )
         go to 1000

      end if
      ind(number) = iuser(ipos)
      mst(number) = 0
      nuserh = 0
      if ( debug ) &
         write(irefwr,1) 'number, ipos, ind(number)', &
                          number, ipos, ind(number)
      ipos = ipos+1

      if ( ind(number)==0 ) then

!     --- ind(number)  =  0, clearing has already been performed

      else if ( ind(number)<0 ) then

!     --- ind(number)<0

         if ( ind(number)>=-5 ) then
            call errint ( ipos-1, 1 )
            call errint ( ind(number), 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if
         nuserh = -ind(number)
         value = user(nuserh)

      else if ( ind(number)>100000000 ) then

!     --- special case, scalar found

         help = getscal ( ind(number)-100000000 )
         ind(number) = -ind(number)
         value = help

         if ( debug ) write(irefwr,2) 'help', help

      else

!     --- ind(number)>0

         if ( ind(number)==2003 ) then

!        --- ind(number) = 2003, coefficient is equal to an old vector

            ivecnums(number) = ipos
            mst(number) = iuser(ipos+1)
            ivectr = inivec ( iuser(ipos) )
            if ( ivectr<=0 .or. ivectr>=1000 ) then
               call errint ( ivectr, 1 )
               call errint ( ipos, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 795, 5, 0, 0 )
            end if
            ipos = ipos + 2

         else if ( ind(number)==2004 ) then

!        --- ind(number) = 2004, coefficient is equal to an old vector
!                                multiplied by a constant or function

            mst(number) = iuser(ipos+1)
            ivecnums(number) = ipos
            ivectr = inivec ( iuser(ipos) )
            if ( ivectr<=0 .or. ivectr>=1000 ) then
               call errint ( ivectr, 1 )
               call errint ( ipos, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 795, 5, 0, 0 )
            end if
            if ( iuser(ipos+2)>=1000 ) then
               call errint ( iuser(ipos+2), 1 )
               call errint ( ipos+2, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 796, 5, 0, 0 )
            end if
            if ( iuser(ipos+2)<=0 ) then

!           --- iuser(ipos+2)<0,  multiply by constant

               if ( iuser(ipos+2)>=-5) then
                  call errint ( iuser(ipos+2), 1 )
                  call errint ( ipos+2, 2 )
                  call errint ( ielem, 3 )
                  call errint ( itype, 4 )
                  call errint ( ielgrp, 5 )
                  call errsub ( 797, 5, 0, 0 )
               end if
            end if
            ipos = ipos + 3

         else if ( ind(number)==2001 .or. ind(number)==2002 ) then

!        --- ind(number) = 2001 or 2002

            mst(number) = iuser(ipos)-1
            if ( ind(number)==2001 ) nuserh = mst(number)+npointcac
            if ( ind(number)==2002 ) nuserh = mst(number)+nelemcac
            ipos = ipos+1

         end if

      end if
      nuser = max( nuser, nuserh )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'number, ind, ivecnums', &
                          number, ind(number), ivecnums(number)
         write(irefwr,*) 'End elusr3'

      end if  ! ( debug )
      call erclos ( 'elusr3' )

      end
