      subroutine fr0020 ( nv, tv1, tv2 )
! ======================================================================
!
!        programmer    Kees Vuik
!        version  1.1  date 21-07-2005 Make a unique choice in special cases
!        version  1.0  date 20-07-2001
!
!   copyright (c) -200-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute two tangential vectors, given the normal vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     friction
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision nv(*), tv1(*), tv2(*)

!     nv     i    Normal
!     tv1    o    Tangent 1 to nv
!     tv2    o    Tangent 2 to nv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibase
      double precision d, base, e(3)

!     base           Largest component of normal
!     d              length of vector
!     e              unit vector
!     i              Counting variable
!     ibase          Index of largest component of normal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!       none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!        none
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'fr0020' )

!     --- n is most orthogonal to e1, e2, or e3?
!         the orhogonality is given by how small n.e(1..3) is

      if ( abs(nv(1)-1d0)<1d-15 ) then

!     --- n = (1,0,0)

         tv1(1) = 0d0
         tv1(2) = 1d0
         tv1(3) = 0d0
         tv2(1) = 0d0
         tv2(2) = 0d0
         tv2(3) = 1d0

      else if ( abs(nv(2)-1d0)<1d-15 ) then

!     --- n = (0,1,0)

         tv1(1) = 0d0
         tv1(2) = 0d0
         tv1(3) = 1d0
         tv2(1) = 1d0
         tv2(2) = 0d0
         tv2(3) = 0d0

      else if ( abs(nv(3)-1d0)<1d-15 ) then

!     --- n = (0,0,1)

         tv1(1) = 1d0
         tv1(2) = 0d0
         tv1(3) = 0d0
         tv2(1) = 0d0
         tv2(2) = 1d0
         tv2(3) = 0d0

      else if ( abs(nv(1)+1d0)<1d-15 ) then

!     --- n = (-1,0,0)

         tv1(1) = 0d0
         tv1(2) = -1d0
         tv1(3) = 0d0
         tv2(1) = 0d0
         tv2(2) = 0d0
         tv2(3) = -1d0

      else if ( abs(nv(2)+1d0)<1d-15 ) then

!     --- n = (0,-1,0)

         tv1(1) = 0d0
         tv1(2) = 0d0
         tv1(3) = -1d0
         tv2(1) = -1d0
         tv2(2) = 0d0
         tv2(3) = 0d0

      else if ( abs(nv(3)+1d0)<1d-15 ) then

!     --- n = (0,0,-1)

         tv1(1) = -1d0
         tv1(2) = 0d0
         tv1(3) = 0d0
         tv2(1) = 0d0
         tv2(2) = -1d0
         tv2(3) = 0d0

      else

         ibase = 1
         base = abs(nv(1))
         do i=2,3
            if (abs(nv(i))<base) then
               ibase = i
               base = abs(nv(i))
            end if
         end do
         do i=1,3
            e(i) = 0d0
         end do
         e(ibase) = 1d0

!        --- make tv1 orthogonal to nv and e(ibase)

         tv1(1) = nv(2)*e(3) - nv(3)*e(2)
         tv1(2) = nv(3)*e(1) - nv(1)*e(3)
         tv1(3) = nv(1)*e(2) - nv(2)*e(1)
         d = sqrt( tv1(1)**2 + tv1(2)**2 + tv1(3)**2 )

!        --- scale tv1

         do i=1,3
            tv1(i) = tv1(i) / d
         end do

!        --- make tv2 orhtogonal to nv and tv1

         tv2(1) = nv(2)*tv1(3) - nv(3)*tv1(2)
         tv2(2) = nv(3)*tv1(1) - nv(1)*tv1(3)
         tv2(3) = nv(1)*tv1(2) - nv(2)*tv1(1)

!        --- scale tv2 to length 1d0

         d = sqrt( tv2(1)**2 + tv2(2)**2 + tv2(3)**2 )
         do i=1,3
            tv2(i) = tv2(i) / d
         end do

      end if

      call erclos( 'fr0020' )
      end
