      subroutine sepel0152
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 152
!     natural boundary condition for complex general elliptic equation
!     (one unknown per point)
!     Old subroutine el0152
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPEL0152CF    Fill complex coefficients for elements of type 152
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0152' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0152'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

!     --- Get information of coefficients

      E%typenatbnd =  sepelgetintcf ( codenatbnd,  nncodenatbnd,&
                                      ncodenatbnd, 1, 'typenatbnd' )
      if ( E%typenatbnd==0 ) E%typenatbnd = 2
      if ( debug ) write(irefwr,1) 'typenatbnd', E%typenatbnd
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1
      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule

      call sepgetcoorsys
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Fill all real coefficients

      E%ncoef = 13
      call sepel0152cf

      if ( E%typenatbnd==2 ) then

!     --- Boundary conditions of type 2

         E%zeroterm = E%ind(1)/=0
         if ( .not. E%zeroterm ) E%notmat = 1
         if ( E%ind(2)==0 ) E%notvec = 1

      else if ( E%typenatbnd==3 .or. E%typenatbnd==5 ) then

!     --- Boundary conditions of type 3, 5. Use convection

         E%typeconv = 0
         E%indvel = 11

      else if ( E%typenatbnd==4 .or. E%typenatbnd==6 ) then

!     --- Boundary conditions of type 4, 6. Use convection

         E%notmat = 1

      end if  ! ( E%typenatbnd==3 )

      if ( E%typenatbnd==3 )  B%fillderivn = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,1) 'B%m, B%n, B%cart', B%m, B%n, B%cart
         write(irefwr,1) 'B%rule', B%rule
         write(irefwr,1) 'E%notmat, E%notvec', E%notmat, E%notvec
         write(irefwr,*) 'End sepel0152'

      end if  ! ( debug )
      call erclos ( 'sepel0152' )

      end subroutine sepel0152
