      subroutine sepel0251t
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Test coefficients for type 251
!     boundary conditions for linear elastic problem
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: igprob, i
      double precision :: h

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     igprob         Type of stress-strain relations
!                    igprob(here) = igprob(manual) + 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     556   Thickness h has wrong value
!    1768   Coefficient is filled that is not used
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0251t' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0251t'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      igprob = E%stress_strain+1

      do i = E%ndim+1, 3
         if ( E%ind(i)/=0 ) then
            call errint ( i+5, 1 )
            call errint ( E%ielem, 2 )
            call errint ( E%itype, 3 )
            call errint ( E%ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )
            E%ind(i) = 0
         end if
      end do

      if ( E%ndim==2 .and. igprob/=3 .and. igprob/=6 .and. &
           igprob/=8 ) then

!     --- igprob = 1, 2, 4 or 5  function h  (thickness of plate)

         if ( E%ind(4)==0 ) then

!        --- ind(4)=0, hence h = 1

            E%ind(4) = -1
            E%coeffs(:,4) = 1d0

         else if ( E%ind(4)<0 ) then

!        --- ind(4)<0, hence h is constant

            h = E%coeffs(1,4)
            if ( h<=0d0 ) then
               call errint ( E%ielem, 1 )
               call errint ( E%itype, 2 )
               call errint ( E%ielgrp, 3 )
               call erreal ( h, 1 )
               call errsub ( 556, 3, 1, 0 )
            end if

         end if

      else

         if ( E%ind(4)/=0 ) then
            call errint ( 9, 1 )
            call errint ( E%ielem, 2 )
            call errint ( E%itype, 3 )
            call errint ( E%ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )
            E%ind(4) = 0
         end if

      end if

      if ( E%ndim==1 ) then
         do i = 6, 10
            if ( E%ind(i)/=0 ) then
               call errint ( i+5, 1 )
               call errint ( E%ielem, 2 )
               call errint ( E%itype, 3 )
               call errint ( E%ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               E%ind(i) = 0
            end if
         end do

      else if ( E%ndim==2 ) then
         if ( E%ind(7)/=0 ) then
            call errint ( 12, 1 )
            call errint ( E%ielem, 2 )
            call errint ( E%itype, 3 )
            call errint ( E%ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )
            E%ind(7) = 0
         end if
         do i = 9, 10
            if ( E%ind(i)/=0 ) then
               call errint ( i+5, 1 )
               call errint ( E%ielem, 2 )
               call errint ( E%itype, 3 )
               call errint ( E%ielgrp, 4 )
               call errwar ( 1768, 4, 0, 0 )
               E%ind(i) = 0
            end if
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0251t'

      end if  ! ( debug )
      call erclos ( 'sepel0251t' )

      end subroutine sepel0251t
