      subroutine sepel0700
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 700
!     Laplace equation
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETIQMAT  Get the parameter iqmat as stored in iuser
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPELTYPERHOCP Fill information about density rho and rho cp
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0700' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0700'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

!     --- Get information of coefficients

      call sepelgetiqmat  ! iqmat
      E%ipos = E%ipos+1         ! skip icoef2
      E%ishift = E%ishift+1

      if ( debug ) write(irefwr,1) 'ipos, ishift/2', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      call sepgetcoorsys

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Fill all real coefficients

      E%ncoef = 3
      E%indrho = 1
      E%indcp = 2
      E%indrhocp = 3

      call sepelalloccf ( 0, 0 ) ! allocate coefficient arrays

!     --- Define rho cp

      call sepeltyperhocp

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

      E%notvec = 1  ! element vector is zero

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0700'

      end if  ! ( debug )
      call erclos ( 'sepel0700' )

      end subroutine sepel0700
