      subroutine sepel0810
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM810 or
!     ELD810 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
!     Old subroutine el0810
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulestrings
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     See subroutine sepeld810int
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0810' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0810'
         call prinin ( E%iuser, 30, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      E%ipos = E%ipos+2   ! skip 2 positions
      if ( debug ) write(irefwr,1) 'E%ipos/1', E%ipos

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      B%rule = max(1,B%rule)
      if ( debug ) write(irefwr,1) 'E%ipos/2', E%ipos

      call sepgetcoorsys

      E%ipos = E%ipos+1   ! skip 1 position

!     --- Get information of basis functions

      call sepelbasefnch

      E%ncoef = 15
      E%compold = 1

      call sepelalloccf ( 0, 0 ) ! allocate coefficient arrays

      do i = 1, E%ncoef
         E%coeffsnode(1:B%n,i) = E%coeffs(1,i)
      end do  ! i = 1, E%ncoef
      E%coeffcp = E%coeffs

      iseqenth = max(1,E%iuser(E%ipos+1)) ! sequence number of enthalpy
      indvector(iseqenth) = 1  ! vector is used

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqenth', iseqenth
         write(irefwr,*) 'End sepel0810'

      end if
      call erclos ( 'sepel0810' )

      end
