      subroutine sepelallocb
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-06-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate arrays in B of type basisfn
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nd, nd2, ndim, nnodes

!     nd             dimension of basis function
!     nd2            number of second derivs
!     ndim           Dimension of space
!     nnodes         number of nodes that are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelallocb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelallocb'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m = B%m
      n = B%n
      nd = B%nd
      nd2 = B%nd2
      nvert = B%nvert
      inpelm = E%npelm
      irule = B%rule
      ndim = E%ndim
      nnodes = max(B%n,E%npelm)

      allocate ( B%Jac(m), stat = error )
      if ( error/=0 ) call eralloc ( error, m, 'Jac' )
      allocate ( B%phi(n,m), stat = error )
      if ( error/=0 ) call eralloc ( error, m*n, 'phi' )
      allocate ( B%w(m), stat = error )
      if ( error/=0 ) call eralloc ( error, m, 'w' )
      allocate ( B%w2(n), stat = error )
      if ( error/=0 ) call eralloc ( error, m, 'w' )
      allocate ( B%xyz(n,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, n*ndim, 'xyz' )
      allocate ( B%lam(nvert,m), stat = error )
      if ( error/=0 ) call eralloc ( error, m*(nd+1), 'lam' )
      allocate ( B%lamn(nvert,n), stat = error )
      if ( error/=0 ) call eralloc ( error, n*(nd+1), 'lamn' )
      allocate ( B%lamder(nvert,m,nd), stat = error )
      if ( error/=0 ) call eralloc ( error, m*nd+1, 'lamder' )
      allocate ( B%lamderc(nvert,m,nd), stat = error )
      if ( error/=0 ) call eralloc ( error, m*nd+1, 'lamderc' )
      allocate ( B%phic(n,m), stat = error )
      if ( error/=0 ) call eralloc ( error, m*n, 'phic' )
      allocate ( B%wc(m), stat = error )
      if ( error/=0 ) call eralloc ( error, m, 'wc' )
      allocate ( B%phider(n,m,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, m*n*nd, 'phider' )
      allocate ( B%phiderc(n,m,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, m*n*ndim, 'phiderc' )
      allocate ( B%phidern(n,n,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, n*n*ndim, 'phidern' )
      allocate ( B%phidercn(n,n,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, n*n*ndim, 'phidercn' )
      allocate ( B%dxidx(n,nd,nd), stat = error )
      if ( error/=0 ) call eralloc ( error, m*nd*nd, 'dxidx' )
      allocate ( B%phider2(n,m,nd2), stat = error )
      if ( error/=0 ) call eralloc ( error, m*n*nd2, 'phider2' )
      allocate ( B%qmat(3,ndim,m), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*ndim*m, 'qmat' )
      allocate ( B%dxdxi(n,ndim,3), stat = error )
      if ( error/=0 ) call eralloc ( error, n*ndim*3, 'qmat' )
      allocate ( B%xyzgc(m,nd), stat = error )
      if ( error/=0 ) call eralloc ( error, m*nd, 'xyzgc' )
      allocate ( B%xyzgcn(n,nd), stat = error )
      if ( error/=0 ) call eralloc ( error, n*nd, 'xyzgcn' )
      allocate ( B%xyzg(m,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, m*ndim, 'xyzg' )
      allocate ( B%xyzg(m,ndim), stat = error )
      if ( error/=0 ) call eralloc ( error, m*ndim, 'xyzg' )

      if ( E%ktang>0 ) then
         allocate ( B%tang(m,ndim), stat = error )
         if ( error/=0 ) call eralloc ( error, m*ndim, 'tang' )
      end if  ! ( E%ktang>0 )

      if ( B%rule==1 .and. n==m ) then

!     --- Gauss points are also nodes

          B%dxdxig => B%dxdxi

      else

!     --- Gauss points differ from nodes

         allocate ( B%dxdxig(m,ndim,3), stat = error )
         if ( error/=0 ) call eralloc ( error, m*ndim*3, 'dxdxig' )

      end if  ! ( B%rule==1 )

      if ( B%type_basisfunc==30 ) then

         B%philoc => B%lam

      else

         B%philoc => B%phi

      end if  ! ( B%type_basisfunc==30 )
      if ( ierror/=0 ) go to 1000

      B%qmat = 0d0
      B%w = 0d0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelallocb'

      end if  ! ( debug )
      call erclos ( 'sepelallocb' )

      end subroutine sepelallocb
