      subroutine sepelcconvect
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for convection equation
!     It is assumed that convection coefficients are stored in coefficients
!     7 to 9
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jdiag

!     jdiag          if 1 the basis functions are 0 or 1 in the quadrature pnts
!                    if  0 they are general
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
!     SEPELCCONVECT0 Fill element matrix for convection equation (complex)
!     SEPELCCONVECTCO Fill element matrix for compressible convection equation
!                    Complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     973   Required option has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelcconvect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelcconvect'
         write(irefwr,1) 'typeconv, E%indvel', E%typeconv, E%indvel
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%symm = .false.

      select case ( E%typeconv )

      case(0, 4:6)

!     --- typeconv = 0, 4-6: incompressible convection

         call sepelcconvect0 ( E%ccoeffs(1,E%indvel), E%ind(E%indvel) )

      case(1)

!     --- typeconv = 1: compressible convection

         call sepelcconvectco

      case default

         call errsub ( 973, 0, 0, 0 )

      end select  ! case ( E%typeconv )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'element matrix/conv' )
         write(irefwr,*) 'End sepelcconvect'

      end if  ! ( debug )
      call erclos ( 'sepelcconvect' )

      end subroutine sepelcconvect
