      subroutine sepelconvect0t ( veloc, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for convection equation
!     Transpose matrix is used
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ind(*)
      double precision, target :: veloc(B%m,*)

!     ind            i    indicates if the "new" velocity components are
!                         constant of variable
!     veloc          i    Contains the new velocity components for this
!                         term. They are -1 or 1 depending on the sign of the
!                         derivatives
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELCONVECT1T Fill element matrix for convection equation
!                    The derivatives of the basis functions are constant
!                    Transpose matrix is used
!     SEPELCONVECT2T Fill element matrix for convection equation
!                    The derivatives of the basis functions are variable
!                    Transpose matrix is used
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvect0t' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvect0t'
         write(irefwr,1) 'methodupw', E%methodupw
         write(irefwr,*) 'B%derivconst ', B%derivconst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%methodupw==0 ) then

!     --- Convection without upwind

         if ( B%derivconst ) then

!        --- Derivatives of basis functions are constant

            call sepelconvect1t ( veloc, ind, B%jdiag, B%phi )

         else

!        --- Derivatives of basis functions are variable

            call sepelconvect2t ( veloc, ind, B%jdiag, B%phi )

         end if  ! ( B%derivconst )

      else

!     --- upwind method

         if ( B%derivconst ) then

!        --- Derivatives of basis functions are constant

            call sepelconvect1t ( veloc, ind, 2, B%psi )

         else

!        --- Derivatives of basis functions are variable

            call sepelconvect2t ( veloc, ind, 2, B%psi )

         end if  ! ( B%derivconst )

      end if  ! ( B%methodupw==0 )

1000  call erclos ( 'sepelconvect0t' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'element_matrix' )
         write(irefwr,*) 'End sepelconvect0t'

      end if  ! ( debug )

      end subroutine sepelconvect0t
