      subroutine sepelconvectab
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for convection equation
!     The convection term is defined by the sum of the absolute values
!     of the derivatives
!     No upwind
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jdiag, ind(3)
      double precision, target :: veloc(B%m,3)

!     ind            indicates if the "new" velocity components are
!                    constant of variable
!     jdiag          if 1 the basis functions are 0 or 1 in the quadrature pnts
!                    if  0 they are general
!     veloc          Contains the new velocity components for this
!                    term. They are -1 or 1 depending on the sign of the
!                    derivatives
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELBASEUPW   Compute the modified basis functions psi due to upwind
!     SEPELCONVECT0  Fill element matrix for convection equation
!     SEPELCONVECT6  Fill arrays veloc and ind in case of convection based on
!                    sum of absolute values of derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvectab' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvectab'
         write(irefwr,1) 'methodupw', E%methodupw
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays veloc and ind

      call sepelconvect6 ( veloc, ind )

      if ( E%methodupw>0 ) then

!     --- upwind compute psi

         E%veloc => veloc
         call sepelbaseupw

      end if  ! ( E%methodupw>0 )

      if ( debug ) then
         call prinrl1 ( veloc, B%nd, B%m, 'uvel' )
         call prinrl1 ( B%psi, B%m, B%n, 'psi' )
      end if  ! ( debug )

!     --- Update matrix with convection part defined by veloc

      call sepelconvect0 ( veloc, ind )

1000  call erclos ( 'sepelconvectab' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'element_matrix' )
         write(irefwr,*) 'End sepelconvectab'

      end if  ! ( debug )

      end subroutine sepelconvectab
