      subroutine sepelconvectco
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for compressible convection equation
!     It is assumed that velocity coefficients are stored in coefficients
!     7 to 9
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELCONVECT3  Fill element matrix for compressible convection equation
!                    Constant derivatives of basis functions
!     SEPELCONVECT4  Fill element matrix for compressible convection equation
!                    Variable derivatives of basis functions
!     SEPELCONVECT5  Compute correction to element stiffness matrix
!                    corresponding to the first derivative part of a
!                    one-, two- or three-dimensional equation
!                    in case of upwind and compressible convection
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The incompressible form of the convection equation is
!
!     div ( rho cp u c)
!
!     In standard Galerkin this term becomes:
!
!     - ( rho cp u c ) grad phi_i
!
!     in case of upwind we have to add:
!       div ( rho cp u c) (psi-phi)_i
!     with phi the standard basis function and psi the updated (upwind) one
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvectco' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvectco'
         write(irefwr,*) 'B%derivconst ', B%derivconst
         write(irefwr,1) 'E%methodupw ', E%methodupw
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First part without upwind

      if ( B%derivconst ) then

!     --- Derivatives of basis functions are constant

         call sepelconvect3

      else

!     --- Derivatives of basis functions are variable

         call sepelconvect4

      end if  ! ( B%derivconst )

      if ( E%methodupw>0 ) call sepelconvect5 ! Upwind part

1000  call erclos ( 'sepelconvectco' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvectco'

      end if  ! ( debug )

      end subroutine sepelconvectco
