      subroutine sepeld02 ( elemwg, elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     with respect to subroutine sepeld705
!     icheld = 2, 3, 5: gradient
!     Old subroutine el10012
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: elemwg(ndimlc,B%n), elemvc(ndimlc,B%n)

!     elemvc         o    Element vector
!     elemwg         o    element vector containing the weights for subroutine
!                         deriva
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: h
      double precision, pointer :: dudx(:,:), dphidx(:,:,:)

!                    they are not
!     dphidx         Reference to array B%phider
!                    (derivatives of basis functions in nodes)
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld02'
         write(irefwr,*) 'B%derivconst ', B%derivconst
         write(irefwr,1) 'n, ndimlc, icheld', B%n, ndimlc, icheld
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldvec(:,:,1)
      dphidx => B%phider(:,:,:)
      n = B%n

      if ( debug ) then
         call prinrl1 ( dudx, ndimlc, n, 'dudx' )
         call prinrl1 ( dphidx(1,1,1), ndimlc, n, 'dphidx' )
      end if  ! ( debug )

      if ( B%derivconst ) then

!     --- Derivatives are constant

         do j = 1, ndimlc
            elemwg(j,1:n) = abs( dphidx(1:n,1,j) )
            elemvc(j,1:n) = dudx(1,j) * elemwg(j,1:n)
         end do

      else if ( ndim==2 .and. n>=6 ) then

!     --- quadratic elements in R2

         do j = 1, ndimlc
            do i = 1, n-1, 2
               elemwg(j,i) = abs( dphidx(i,i,j) )
               elemvc(j,i) = dudx(i,j) * elemwg(j,i)
            end do
            do i = 2, n, 2
               elemvc(j,i) = dudx(i,j)
            end do
            elemvc(j,n) = dudx(n,j)
         end do

      else

!     --- General element

         do j = 1, ndimlc
            do i = 1, n
               elemvc(j,i) = dudx(i,j)
            end do
         end do

      end if

      if ( icheld==3 ) then

!     --- icheld = 3,  change sign

         elemvc = -elemvc

      else if ( icheld==5 ) then

!     --- icheld  = 5, change sign and sequence  (2D only)

         do i = 1, n
            h = elemvc(1,i)
            elemvc(1,i) = elemvc(2,i)
            elemvc(2,i) = - h
            h = elemwg(1,i)
            elemwg(1,i) = elemwg(2,i)
            elemwg(2,i) = h
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeld02'

      end if  ! ( debug )
      call erclos ( 'sepeld02' )

      end subroutine sepeld02
