      subroutine sepeld081
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in E%vec
!     with respect to subroutine sepeld705
!     icheld = 8: smoother for to0090 (1d)
!     Old subroutine eld118
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h
      double precision, pointer :: u(:), a11(:), x(:)

!     a11            First component of diffusion tensor
!                    they are not
!     h              length of element
!     u              Reference to uprev
!                    Old solution in nodes
!     x              coordinates of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld081' )
      debug = .false.

      u => uprev(:,1)
      a11 => E%coeffs(:,1)
      x => B%xyz(:,1)

      n = B%n

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld081'
         call prinrl ( x, n, 'x' )
         call prinrl ( u, n, 'u' )
         call prinrl ( a11, n, 'a11' )


      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%derivconst ) then

!     --- Derivatives are constant

         E%wvec = B%w
         E%vec = sign(0.5d0,x(2)-x(1))*a11*(u(2)-u(1))

      else

!     --- Derivatives are variable

         E%wvec = B%w
         h = sign ( 1d0, x(3)-x(1) ) / 6d0
         E%vec(1) = h*a11(1) * (-3*u(1) + 4*u(2) -   u(3) )
         E%vec(2) = h*a11(2) * (-4*u(1)          + 4*u(3) )
         E%vec(3) = h*a11(3) * (   u(1) - 4*u(2) + 3*u(3) )

      end if  ! ( B%derivconst )

1000  call erclos ( 'sepeld081' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, n, 'elemvc' )
         call prinrl ( E%wvec, n, 'elemwg' )
         write(irefwr,*) 'End sepeld081'

      end if  ! ( debug )

      end subroutine sepeld081
