      subroutine sepeld705
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector ans weight vector for general
!     elliptic equations with one unknown
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icounw

!     icounw         length of elemwg(*)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELD0300     Fill element independent quantities for derivatives
!                    of element type 325
!     SEPELD0705     Fill element independent quantities for derivatives
!                    of element type 705
!     SEPELD705DER   Actual computing of derivatives for elliptic equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELM705CF    Fill coefficients for general elliptic equation
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELOLDVECSK  Fill old vectors for element (only one vector)
!     SEPELUDERIV    Compute the derivative of a given function in one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld705' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld705'
         write(irefwr,1) 'icheld, numoldvecs', icheld, numoldvecs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) then

!     --- First element

         if ( itype==300 .or. itype==325 ) then
            call sepeld0300
         else
            call sepeld0705
         end if  ! ( itype==300 .or. itype==325 )

      end if  ! ( E%first==0 )

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( debug ) &
         write(irefwr,1) 'E%compold, iseqi1, iseqi2', &
                          E%compold, iseqi1, iseqi2
      if ( E%compold>0 ) then

!     --- All old vectors are required

         call sepeloldvecs

      else

!     --- Only one vector required

         call sepeloldvecsk ( iseqprevsol )
         if ( iseqi2/=iseqprevsol ) call sepeloldvecsk ( iseqi2 )
         call sepeluderiv

      end if  ! ( E%compold>0 )

      if ( icheld==9 ) then
         uprev2 => oldsols(2)%locvecsnode(1:B%n,1:nunkp)
      else if ( icheld==10 ) then
         uprev2 => oldsols(numold)%locvecsnode(1:B%n,1:nunkp)
      end if  ! ( icheld==9 )

!     --- Fill all coefficients

      if ( E%ncoef>0 ) call sepelm705cf

      if ( icheld==9 .or. icheld>10 .and. icheld<=20 .or. &
           icheld==29 .or. icheld>=41 ) then
         icounw = 0
      else
         icounw = icouno
      end if  ! ( icheld>10 .and. icheld<=20 ... )

!     --- Initialize weight vector, if any

      if ( icounw>0 ) E%wvec = 1d0

!     --- Compute derivatives

      call sepeld705der

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icheld', icheld
         call prinrl ( E%vec, icouno, 'elemvc')
         if ( icounw>0 ) call prinrl ( E%wvec, icounw, 'elemwg')
         write(irefwr,*) 'End sepeld705'

      end if  ! ( debug )
      call erclos ( 'sepeld705' )

      end subroutine sepeld705
