      subroutine sepeldc04
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill elemvc for type 150 (icheld=4)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex, pointer :: dudx(:), dudy(:), dudz(:)
      double precision :: coeff(B%m)
      double precision :: h

!     coeff          Contains the first coefficient (diffusion)
!     dudx           du/dx
!     dudy           du/dy
!     dudz           du/dz
!     h              Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldc04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldc04'
         write(irefwr,1) 'ndim, m', ndim, m
         write(irefwr,*) 'B%derivconst ', B%derivconst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldcvec(:,1,1)
      dudy => gradoldcvec(:,2,1)
      coeff = real(E%ccoeffs(:,1),kind=8)

      if ( debug ) then
         call princompl ( dudx, m, 'dudx' )
         call princompl ( dudy, m, 'dudy' )
         call prinrl ( coeff, m, 'coeff' )
      end if  ! ( debug )

      if ( ndim==2 ) then

!     --- ndim = 2

         if ( B%derivconst ) then

!        --- Derivatives are constant

            h = abs(dudx(1))**2+abs(dudy(1))**2
            E%vec = coeff * h

         else

!        --- Derivatives are variable


            E%vec = coeff * (abs(dudx)**2+abs(dudy)**2)

         end if  ! ( B%derivconst )

      else

!     --- ndim = 3

         dudz => gradoldcvec(:,3,1)
         if ( B%derivconst ) then

!        --- Derivatives are constant

            h = abs(dudx(1))**2+abs(dudy(1))**2+abs(dudz(1))**2
            E%vec = coeff * h

         else

!        --- Derivatives are variable


            E%vec = coeff * (abs(dudx)**2+abs(dudy)**2+abs(dudz)**2)

         end if  ! ( B%derivconst )

      end if  ! ( ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, m, 'elemvc' )
         write(irefwr,*) 'End sepeldc04'

      end if  ! ( debug )
      call erclos ( 'sepeldc04' )

      end subroutine sepeldc04
