      subroutine sepeldc07
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in E%cvec
!     with respect to subroutine sepeld705
!     icheld = 7: div u
!     Old subroutine el10017
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double complex :: divergence
      double complex, pointer :: u(:,:)
      double precision, pointer :: dphidx(:,:,:)

!                    they are not
!     divergence     Help paramter to compute the divergence
!     dphidx         Reference to array B%phider
!                    (derivatives of basis functions in nodes)
!     i              General loop variable
!     j              General loop variable
!     u              Reference to uprev
!                    Old solution in nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL1     Print 2d complex vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The divergence is defined as div u = sum_i du_i/dx_i
!     In case of constant derivatives this implies
!        sum_i ( sum_j u_j dphi_j/dx_i )
!     else
!        sum_i ( sum_j u_k dphi_j dx_i(j) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldc07' )
      debug = .false.

      u => ucprev
      dphidx => B%phider(:,:,:)
      n = B%n

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldc07'
         call princompl1 ( u, ndim, n, 'u' )
         call princompl1 ( dphidx(1,1,1),m,n,'dphidx' )
         call princompl1 ( dphidx(1,1,2),m,n,'dphidy' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%derivconst ) then

!     --- Derivatives are constant

         divergence = 0d0
         do i = 1, ndim
            divergence = divergence+sum(u(1:n,i)*dphidx(1:n,1,i))
         end do  ! i = 1, ndim
         E%cvec(1:n) = divergence

      else

!     --- Derivatives are variable

         E%cvec = 0d0
         do i = 1, ndim
            do j = 1, n
               E%cvec(j) = E%cvec(j) + sum(u(1:n,i)*dphidx(1:n,j,i))
            end do  ! k = 1, n
         end do  ! i = 1, ndim

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldc07'

      end if  ! ( debug )
      call erclos ( 'sepeldc07' )

      end subroutine sepeldc07
