      subroutine sepeldiffup00
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add contribution of second derivative part to matrix in the case of
!     upwind
!     Two-dimensional quadratic or bilinear elements; Newton Cotes rule
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     upwind
!     diffusion
!     convection_diffusion
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, l, m, n, kstep
      double precision :: som, som1, a1(B%n), a2(B%n)
      double precision, pointer :: phix(:,:), phiy(:,:)
      double precision, pointer :: alpha11(:), alpha12(:), alpha22(:)

!     a1             Help array
!     a2             Help array
!     alpha11        Diffusion coefficient D_11
!     alpha12        Diffusion coefficient D_12
!     alpha22        Diffusion coefficient D_22
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     kstep          Step in loop over quadrature pointer
!     l              General loop variable
!     m              Number of quadrature points
!     n              Number of basis functions
!     phix           Contains x-derivatives of basis functions in quadrature
!                    points
!     phiy           Contains y-derivatives of basis functions in quadrature
!                    points
!     som            Help parameter to compute a sum
!     som1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffup00' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffup00'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      alpha11 => E%difftensor(:,1)
      alpha12 => E%difftensor(:,2)
      alpha22 => E%difftensor(:,4)
      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      n = B%n
      m = B%m
      kstep = B%kstep

!     --- First store some terms to compute the second derivative

      do i = 1, n
         do j = 1, n
            do l = 1, n
               a1(l) = alpha11(l)*phix(j,l)+alpha12(l)*phiy(j,l)
               a2(l) = alpha12(l)*phix(j,l)+alpha22(l)*phiy(j,l)
            end do

!           --- second derivative part

            som = 0d0
            do k = kstep, m, kstep
               som1 = 0d0
               do l = 1, n
                  som1 = som1 + a1(l)*phix(l,k)+a2(l)*phiy(l,k)
               end do
               som = som - (B%psi(i,k)-B%phi(i,k)) * som1 * B%w(k)
            end do
            E%mat(j,i) = E%mat(j,i) + som
         end do
      end do

1000  call erclos ( 'sepeldiffup00' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiffup00'

      end if  ! ( debug )

      end subroutine sepeldiffup00
