      subroutine sepeldiffupw
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute part of upwind discretization related to diffusion
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     upwind
!     diffusion
!     convection_diffusion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELDIFFUP00  Add contribution of second derivative part to matrix
!                    in the case of upwind
!                    Two-dimensional quadratic or bilinear elements
!                    Newton Cotes rule
!     SEPELDIFFUP01  Compute part of upwind discretization related to diffusion
!                    1d quadratic elements, Cartesian coordinates
!     SEPELDIFFUP02  Compute part of upwind discretization related to diffusion
!                    Two-dimensional quadratic or bilinear elements; Gauss rule
!     SEPELDIFFUP04  Compute part of upwind discretization related to diffusion
!                    Three-dimensional quadratic or bilinear elements; Gauss rule
!     SEPELDIFFUP06  Compute part of upwind discretization related to diffusion
!                    Thre-dimensional elements
!                    Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffupw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffupw'
         write(irefwr,1) 'ndim, n, m, rule', E%ndim, B%n, B%m, B%rule
         write(irefwr,1) 'type_basisfunc', B%type_basisfunc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 .or. .not. B%secondupw ) go to 1000

      select case ( E%ndim )

      case(1)

!     --- one-dimensional elements

         if ( B%type_basisfunc==2 .and. B%cart==1 ) then

!        --- Upwind is only applied for quadratic elements and Cartesian coors

            call sepeldiffup01

         end if  ! ( B%type_basisfunc==2 .and. B%cart==1 )

      case(2)

!     --- two-dimensional elements

         if ( B%n>3 .and. B%type_basisfunc/=10 ) then

!        --- Second order term may not be neglected in upwind part

            if ( B%rule==1 ) then

!           --- irule = 1, newton cotes rule

               call sepeldiffup00

            else

!           --- irule>1, gauss rule

               call sepeldiffup02

            end if

         end if

      case(3)

!     --- three-dimensional elements

         if ( B%n>4 .and. B%secondupw ) then

!        --- Second order term may not be neglected in upwind part

            if ( B%rule==1 ) then

!           --- irule = 1, newton cotes rule

               call sepeldiffup06

            else

!           --- irule>1, gauss rule

               call sepeldiffup04

            end if  ! ( B%rule==1 )

         end if  ! ( B%n>4 .and. B%secondupw )

      case default

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( E%ishape, 1 )
         call errint ( E%ielem, 2 )
         call errint ( E%itype, 3 )
         call errint ( E%ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end select  ! case ( E%ndim )

1000  call erclos ( 'sepeldiffupw' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiffupw'

      end if  ! ( debug )

      end subroutine sepeldiffupw
