      double precision function sepeli705int ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element integral for problems with one degree of freedom
!     per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
!     integral
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

!     sepeli705int   o    Computed element integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ichelisav

!     ichelisav      Help parameter to save old value of icheli
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: sepelic00, sepelic01

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELIC00      Compute a volume integral over one element depending
!                    on icheli
!     SEPELIC01      Compute a volume integral over one element depending
!                    on icheli
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeli705int' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icheli', icheli
         write(irefwr,*) 'Debug information from sepeli705int'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ichelisav = icheli  !save

!     --- Compute integral depending on icheli

      if ( mod(icheli,10)/=6 ) then

!     --- icheli = 1-5, 7, 8

         icheli = mod(icheli,10)
         sepeli705int = sepelic00 ( )

      else

!     --- icheli = 6

         icheli = 2
         sepeli705int = sepelic01 ( )

      end if
      icheli = ichelisav  ! reset

1000  if ( debug ) then

!     --- Debug information

         if ( E%ishape>=15 .and. E%ishape<=16 ) then

!        --- Debug statements, check if all centroids are positioned correctly

   !         call elcheckcentr ( E%ishape, index1, coor )

         end if
         write(irefwr,*) 'End sepeli705int'

      end if  ! ( debug )
      call erclos ( 'sepeli705int' )

      end function sepeli705int
