      subroutine sepeliload
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill and check parameter iload for elasticity problems
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1451   Combination iload, ndim and ishape not allowed
!    1912   Parameter jboun has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeliload' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeliload'
         write(irefwr,1) 'ipos', E%ipos
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%iload = inirepscal ( E%iuser(E%ipos) )
      if ( debug ) write(irefwr,1) 'iload', E%iload
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1
      if ( E%iload==0 .or. E%iload==1 ) E%iload = 2
      if ( E%iload<2 .or. E%iload>4 ) then
         call errint ( E%iload, 1 )
         call errint ( E%ielem, 2 )
         call errint ( E%itype, 3 )
         call errint ( E%ielgrp, 4 )
         call errsub ( 1912, 4, 0, 0 )
      end if
      if ( E%iload==3 .and. E%ndim/=2 .or. &
           ( E%iload==4 .and. E%ishape>6 ) ) then

         call errint ( E%iload, 1 )
         call errint ( E%ishape, 2 )
         call errint ( E%ndim, 3 )
         call errint ( E%ielem, 4 )
         call errint ( E%itype, 5 )
         call errint ( E%ielgrp, 6 )
         call errsub ( 1451, 6, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeliload'

      end if  ! ( debug )
      call erclos ( 'sepeliload' )

      end subroutine sepeliload
