      subroutine sepelload ( source )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element vector with a load
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: source(B%m)

!     source         i    right-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELLOAD1     Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelload' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelload'
         write(irefwr,*) 'fillpsi ', B%fillpsi
         write(irefwr,1) 'methodupw, applyupw', E%methodupw, E%applyupw
         write(irefwr,1) 'B%jdiag, jcart', B%jdiag, B%cart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%fillpsi .and. E%methodupw>0 .and. E%applyupw<1 ) then

!     --- Upwind must be applied

         call sepelload1 ( source, 2, B%psi )

      else

!     --- No upwind

         call sepelload1 ( source, B%jdiag, B%phi )

      end if  ! ( E%methodupw>0 .and. E%applyupw<1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelload'

      end if  ! ( debug )
      call erclos ( 'sepelload' )

      end subroutine sepelload
