      subroutine sepellowupp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find lower and upper bound of variable coefficients
!     Fill oldvectors if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELALLOCVECS Allocate locvecs in oldsols for all vectors that are used
!     SEPELVARCOFS   Compute lower and upper bound of space dependent
!                    coefficients
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepellowupp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepellowupp'
         write(irefwr,1) 'E%compold', E%compold
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find lower and upper bound for space dependent coefficients

      call sepelvarcofs ( 1 )

!     --- Check if Gauss points must be filled

      if ( E%isupcfvar>=E%isubcfvar ) B%fillgauss = .true. ! space dependent

!     --- Check if ugauss must be filled

      do i = E%isubcfvar, E%isupcfvar
         if ( E%ind(i)>1000 .and. E%ind(i)<=2000 ) then
            indvector(iseqprevsol) = 1
            exit
         end if  ! ( E%ind(i)>1000 .and. E%ind(i)<=2000 )
         if ( E%ind(i)>=10000 ) then
            indvector(1:numoldvecs) = 1
            exit
         end if  ! ( E%ind(i)>1000 .and. E%ind(i)<=2000 )
      end do  ! i = E%isubcfvar, E%isupcfvar

      if ( debug ) write(irefwr,1) 'E%compold', E%compold

!     --- Some old vectors must be filled in locvecs

      if ( E%compold>0 ) call sepelallocvecs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepellowupp'

      end if  ! ( debug )
      call erclos ( 'sepellowupp' )

      end subroutine sepellowupp
