      subroutine sepelmultrho
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply velocity by rho cp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: rho(:), veloc(:,:)

!     rho            density rho
!     veloc          velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmultrho' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmultrho'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rho => E%coeffs(:,E%indrhocp)
      veloc => E%coeffs(:,E%indvel:E%indvel+2)

      E%coeffs(:,E%ncoef+1:E%ncoef+3) = veloc(:,1:3)

      if ( E%ind(E%indvel)/=0 ) veloc(:,1) = rho*veloc(:,1)
      if ( E%ind(E%indvel+1)/=0 ) veloc(:,2) = rho*veloc(:,2)
      if ( E%ind(E%indvel+2)/=0 ) veloc(:,3) = rho*veloc(:,3)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmultrho'

      end if  ! ( debug )
      call erclos ( 'sepelmultrho' )

      end subroutine sepelmultrho
