      subroutine sepelp07first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Quadratic triangle (6 nodes)
!     Old subroutine elp624first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lambda(3,7), phi(7,7), w(7)
      character (len=27), parameter :: text = 'Extended quadratic triangle'

!     lambda         Contains values of lambda in quadrature points
!     phi            contains basis functions
!     text           String variable containing some text
!     w              Contains weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     SEPELP07DET    compute some element independent quantities
!                    Extended quadratic triangle (7 nodes)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp07first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp07first'
         write(irefwr,1) 'irule, kstep', B%rule, B%kstep
         write(irefwr,1) 'B%cart', B%cart
         write(irefwr,*) 'B%fillderivn', B%fillderivn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      B%phider = 0d0  ! initialize derivates vector

      if ( B%rule==1 ) B%jdiag = 1 ! basis functions: phi_i(x_j) = delta_ij

      if ( B%cart>3 ) call errorelvar ( B%cart-1, 1769, text, 0 )

!     --- Actual subroutine

      if ( B%fillderivn ) &
         call sepelp07det ( lambda, text, 1, B%n, w, phi, B%phidercn )
      call sepelp07det ( B%lam, text, B%rule, B%m, B%wc, B%phi, B%phiderc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp07first'

      end if  ! ( debug )
      call erclos ( 'sepelp07first' )

      end subroutine sepelp07first
