      subroutine sepelp12phixi ( m, phixi, intern, imidp, dlamdxi, lambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute dphi/dxi in quadrature points
!     Quadratic tetrahedrons
!     Old subroutine elp631
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: m, intern(10), imidp(6,2)
      double precision, intent(in) :: lambda(4,m), dlamdxi(4,3)
      double precision, intent(out) :: phixi(10,m,3)

!     dlamdxi        i    Contains the derivatives of the linear basis functions
!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
!     lambda         i    Array containing the values of lambda in the
!                         integration points  lambda(i,k) = lambda_i(xk)
!     m              i    number of quadrature points
!     phixi          o    Array of length 10 x m x 3 containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, i1, i2, j
      double precision :: h

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     i1             help parameter in computation
!     i2             help parameter in computation
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp12phixi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp12phixi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, 4
         do k = 1, m
            h = 4d0*lambda(i,k)-1d0
            phixi(intern(i),k,1:3) = h*dlamdxi(i,1:3)
         end do  ! k = 1, m
      end do  ! i = 1, 4
      do i = 1, 6
         i1 = imidp(i,1)
         i2 = imidp(i,2)
         do k = 1, m
            do j = 1,3
               h = lambda(i1,k)*dlamdxi(i2,j)+ &
                   lambda(i2,k)*dlamdxi(i1,j)
               phixi(intern(i+4),k,j) = 4d0*h
            end do  ! j = 1,3
         end do  ! k = 1, m
      end do  ! i = 1, 6

1000  call erclos ( 'sepelp12phixi' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp12phixi'

      end if  ! ( debug )

      end subroutine sepelp12phixi
