      subroutine sepelp18first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Extended linear tetrahedron (5 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: lambda(:,:)
      double precision :: phicen
      character (len=27), parameter :: text = 'Extended linear tetrahedron'
      integer :: k, i

!     i              Counting variable
!     k              Loop variable
!     lambda         Contains values of lambda in quadrature points
!     phicen         Value of alam1*alam2*alam3 in quadrature point
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELPLAMBDA3D Fill the three-dimensional linear basis functions in case
!                    of a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp18first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp18first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lambda => B%lam

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( B%m/=5 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000
         lambda = 0d0
         B%phi = 0d0
         do k = 1, 4
            lambda(k,k) = 1d0
            B%phi(k,k) = 1d0
         end do  ! k = 1, 4
         B%phi(5,5) = 1d0
         B%wc = 1d0/24d0

         B%jdiag = 1   ! basis functions satisfy phi_i(x_j) = delta_ij

      case (2:6)

!     --- Gauss rules

         call sepelplambda3d ( lambda, text )
         do k = 1, B%m
            phicen = lambda(1,k)*lambda(2,k)*lambda(3,k)*lambda(4,k)
            B%phi(5,k) = 256d0 * phicen
            do i = 1, 4
               B%phi(i,k) = lambda(i,k) - 64d0*phicen
            end do
         end do

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

!     --- Fill basis functions in quadrature points

      B%phic = B%phi

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp18first'

      end if  ! ( debug )
      call erclos ( 'sepelp18first' )

      end subroutine sepelp18first
