      subroutine sepelp22phixi ( m, m1, m2, phixi, dphitridxi, phitri, &
                                 phiz, phiderz )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute dphi/dxi in quadrature points
!     quadratic wedge element
!     Old subroutine elp639first
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: m, m1, m2
      double precision, intent(in) :: dphitridxi(6,m1,2), phitri(6,m1), &
                                      phiz(3,3), phiderz(3,3)
      double precision, intent(out) :: phixi(18,m,3)

!     m              i    number of quadrature points
!     m1             i    Number of integration points in xi,eta direction
!     m2             i    Number of integration points in zeta direction
!     phiderz        i    Contains the derivatives of the basis functions in a
!                         1d quadratic reference element
!     phixi          o    Array of length 10 x m x 3 containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     phitri         i    Contains the values of the quadratic 2d basis function
!                         in a triangle reference element
!     phiz           i    Contains the values of the quadratic 1d basis function
!                         in a line reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indm, i, j

!     i              Counting variable
!     indm           Index in 3d basis function
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp22phixi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp22phixi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, m1

         do j = 1, m2

            indm = i+(j-1)*m1

            phixi(1:6,indm,1) = dphitridxi(1:6,i,1)*phiz(1,j)
            phixi(1:6,indm,2) = dphitridxi(1:6,i,2)*phiz(1,j)
            phixi(1:6,indm,3) = phitri(1:6,i)*phiderz(1,j)

            phixi(7:12,indm,1) = dphitridxi(1:6,i,1)*phiz(2,j)
            phixi(7:12,indm,2) = dphitridxi(1:6,i,2)*phiz(2,j)
            phixi(7:12,indm,3) = phitri(1:6,i)*phiderz(2,j)

            phixi(13:18,indm,1) = dphitridxi(1:6,i,1)*phiz(3,j)
            phixi(13:18,indm,2) = dphitridxi(1:6,i,2)*phiz(3,j)
            phixi(13:18,indm,3) = phitri(1:6,i)*phiderz(3,j)

         end do  ! j = 1, m2

      end do  ! i = 1, m1

1000  call erclos ( 'sepelp22phixi' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp22phixi'

      end if  ! ( debug )

      end subroutine sepelp22phixi
