      subroutine sepelpmaptang
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map three-dimensional array dphidx onto array dphids on surface
!     using mapping array qmat
!     Old subroutine elpmaptang
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     basis_function
!     element
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision dphids(m,2)

!     dphids         Contains the derivatives of the basis functions
!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpmaptang' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpmaptang'
         call prinrl1 ( B%phider(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), m, n, 'dphidy' )
         call prinrl1 ( B%phider(1,1,3), m, n, 'dphidz' )
         call prinrl1 ( B%qmat(1,1,1), m, 9, 'qmat' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- dphids = grad phi . s

      do i = 1, n
         dphids(1:m,1) = B%phider(i,1:m,1) * B%qmat(2,1,1:m) + &
                         B%phider(i,1:m,2) * B%qmat(2,2,1:m) + &
                         B%phider(i,1:m,3) * B%qmat(2,3,1:m)
         dphids(1:m,2) = B%phider(i,1:m,1) * B%qmat(3,1,1:m) + &
                         B%phider(i,1:m,2) * B%qmat(3,2,1:m) + &
                         B%phider(i,1:m,3) * B%qmat(3,3,1:m)
         B%phider(i,1:m,1) = dphids(1:m,1)
         B%phider(i,1:m,2) = dphids(1:m,2)
      end do  ! i = 1, n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpmaptang'

      end if  ! ( debug )
      call erclos ( 'sepelpmaptang' )

      end subroutine sepelpmaptang
