      subroutine sepelvarcofs ( isub )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute lower and upper bound of space dependent coefficients
!     The sequence number of the lowest space dependent coefficient is
!     stored in E%isubcfvar and of the highest one in E%isupcfvar
!     E%compold is filled as well as array ist in common celiarf90
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     initialization
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isub

!     isub           i    Integer help parameter for do-loops to indicate the
!                         smallest coefficient i that is dependent on space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      logical :: funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              General loop variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPELVARCOFS1  Fill array ich in common block celiar for coefficients
!                    that depend on a prior solution
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvarcofs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvarcofs'
         write(irefwr,1) 'isub, ncoefs', isub, E%ncoef
         call prinin ( E%ind, E%ncoef, 'ind' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find lowest space dependent coefficient

      E%isubcfvar = E%ncoef+1  ! first estimate: all coefficients are constant
      do i = isub, E%ncoef

         if ( E%ind(i)>0 ) then

!        --- coefficient i is space dependent

            E%isubcfvar = i
            exit

         end if  ! ( E%ind(i)>0 )

      end do  ! i = 1, isub, E%ncoef

!     --- Find highest space dependent coefficient

      E%isupcfvar = E%ncoef  ! first estimate

      do i = E%ncoef, E%isubcfvar, -1

         if ( E%ind(i)>0 ) then

!        --- coefficient i is space dependent

            E%isupcfvar = i
            exit

         end if  ! ( E%ind(i)>0 )

      end do  ! i = E%ncoef, E%isubcfvar, -1

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set E%compold

      funchk = .false.

      E%ist(E%isubcfvar:E%isupcfvar) = 0

      do i = E%isubcfvar, E%isupcfvar

         if ( debug ) write(irefwr,1) 'i, ind(i)', i, E%ind(i)

         if ( E%ind(i)>0 .and. E%ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1, E%isupcfvar

               if ( E%ind(j)==E%ind(i) ) then
                  E%ist(j) = i
               end if  ! ( E%ind(j)==E%ind(i) )

            end do  !  j = i+1, E%isupcfvar

         end if  ! ( E%ind(i)>0 .and. E%ind(i)<2000 )

         if ( E%ind(i)>1000 .and. E%ind(i)<=2004 .or. E%ind(i)>=10000 ) then
            E%compold = 1
            indvector(iseqprevsol) = 1
         end if  ! ( E%ind(i)>1000 .and. E%ind(i)<=2004 )

         if ( E%ind(i)>10000 ) funchk = .true.

         if ( E%ind(i)>=2003 .and. E%ind(i)<=2004 ) then

!        --- Fill array ich with information

            call sepelvarcofs1 ( i )

         end if  ! ( E%ind(i)>=2003 .and. E%ind(i)<=2004 )

         if ( (E%ishape==7 .or. E%ishape==9 .or. E%ishape==10) .and. &
              (E%ich(i)==2 .or. E%ich(i)==5) ) E%ich(i) = 7

      end do  ! E%isubcfvar, E%isupcfvar

      if ( funchk ) E%compold = E%compold + 2

1000  call erclos ( 'sepelvarcofs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isubcfvar, isupcfvar', E%isubcfvar, E%isupcfvar
         write(irefwr,*) 'End sepelvarcofs'

      end if  ! ( debug )

      end subroutine sepelvarcofs
