      subroutine sepgetstressrl
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read stress-strain relations and store information in E
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulestrings
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: igprob

!     igprob      Type of stress-strain relations:
!                 igprob = irelat + 10*inonlin
!                 Possible values:
!                 inonlin = 0
!                 irelat  1: plane stress, isotropic material
!                            or 3D stress, isotropic material (Cartesian)
!                         2: plane strain, isotropic material (Cartesian)
!                         3: axi-symmetric stress, isotropic material
!                                                            (axi-symmetric)
!                         4: plane stress, orthotropic material (Cartesian)
!                         5: plane strain, orthotropic material (Cartesian)
!                         6: axi-symmetric stress, orthotropic material
!                            (axi-symmetric)
!                         7: User defined elasticity matrix (3x3)
!                            (Cartesian)
!                         8: User defined elasticity matrix (4x4)
!                            (axi-symmetric)
!                 inonlin>0
!                 inonlin 1: Geometrical and physical nonlinear behaviour
!                         2: 1 + reduced integration compressibility term
!                         3: nonlin + variable anisotropy
!                         4: 3 + reduced integration of compressibility term
!                 irelat  0,1: 3D
!                         3: 2D axi-symmetric
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf, inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     400   The value of stress-strain is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetstressrl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetstressrl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%modelelastnl = 0
      E%choicefnmater = 0
      igprob = sepelgetintcf ( codestressmod, nncodestressmod, &
                                        ncodestressmod , 1,&
                                        'stress_strain_relation' )
      E%stress_strain = igprob
      if ( E%stress_strain>=10 ) then

!     --- E%stress_strain >= 10, hence it also contains modelelastnl

          E%modelelastnl = E%stress_strain/10
          E%stress_strain = mod(E%stress_strain,10)

      end if  ! ( stress_strain>10 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgetstressrl'

      end if  ! ( debug )
      call erclos ( 'sepgetstressrl' )

      end subroutine sepgetstressrl
