      subroutine el0900in ( iuser )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 17-01-2001 Use el0900m
!        version  1.1  date 03-09-2000 Extension for scalars
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the number of integration points for this element group
!   using array iuser
! **********************************************************************
!
!                       KEYWORDS
!
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)

!     iuser     i    Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mrule

!     mrule       Type of integration rule as given by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0900M        Initialize number of integration points for subroutines
!                    ELD900/ELI900/ELM900
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is for the Navier-Stokes equations is:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     406   Value of icheld not allowed
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0900in' )
      m = 1

!     --- Call by eld900

      if ( icheld.lt.0 .or. icheld.gt.53 ) then

!     --- icheld out of range

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      end if

      ipos = iuser ( ielgrp + 5 )
      if ( ipos.lt.7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if

      ipos = ipos + 2

      mrule = inirepscal ( iuser(ipos) )
      mrule = mod(mrule,100)
      if ( mrule.lt.0 .or. mrule.gt.3 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      ipos = ipos + 1
      jcart = inirepscal ( iuser(ipos) )
      if ( jcart.lt.0 .or. jcart.gt.2 ) then
         call errint ( jcart, 1 )
         call errint ( 2, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )
      end if
      jcart = jcart + 1

!     --- Set parameters depending on the shape numbers

      if ( mrule.gt.0 ) then
         irule = mrule
      else
         irule = 3
      end if
      if ( itype.eq.900 .and. inpelm.eq.6 ) then
         n = 7
      else if ( itype.eq.901 .and. inpelm.eq.7 ) then
         n = 7
      else if ( itype.gt.900 .and. inpelm.eq.5 ) then
         n = 4
      else if ( itype.gt.900 .and. ishape.eq.17 ) then
         n = 8
      else
         n = inpelm
      end if
      if ( jcart.ge.2 .and. irule.eq.1 .and. inpelm.gt.n ) irule = 3
      call el0900m

      if ( ierror.ne.0 ) call instop

      call erclos ( 'el0900in' )

      end
