      subroutine el1016 ( p, indpres, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.3  date 12-12-2012 Long integers
!        version  2.1  date 05-05-2007 Debug statements
!
!   copyright (c) 1997-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Extract the pressure from the solution array in the case of a
!    Taylor Hood element and store in the vector elemvc1
!    If a transformation is required the value in the extra point(s) is
!    computed by interpolation and the derivatives are not yet computed
!    For that purpose subroutine EL3000 should be used
!     EL1016 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
!     extract_unknown
!     fill
!     fill_solution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indpres(*), number
      double precision  p(*)

!     indpres    i    Contains the sequence numbers of the pressure unknowns
!                     in the solution vector
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered.
!     p               Work array containing the pressure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jcheld
      double precision sum
      double precision, pointer :: vecoldlc(:)

!     i              Counting variable
!     jcheld         if ( icheld<= 10 ) then icheld else icheld-20
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1016' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1016'
         write(irefwr,1) 'n, number, nvert, inpelm, icheld', &
                          n, number, nvert, inpelm, icheld
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqtype = oldsols(number)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(number)%oldvecs
      p(1:nvert) = vecoldlc(index3n(indpres(1:nvert)))
      if ( debug ) call prinrl ( p, nvert, 'p_in' )
      jcheld = mod(icheld,40)
      if ( jcheld==27 .or. jcheld==7 .and. nvert==inpelm ) then

!     --- Only in the vertices

         do i = 1, nvert
            elemvc1(i) = p(i)
         end do

      else if ( jcheld==7 ) then

!     --- In all points

         p(nvert+1) = p(1)
         if ( ishape==4 .or. ishape==6 ) then
            do i = 1, nvert
               elemvc1(2*i-1) = p(i)
            end do
            do i = 1, nvert
               elemvc1(2*i) = 0.5d0*(p(i)+p(i+1))
            end do
            if ( ishape==6 ) elemvc1(9) = 0.25d0*(p(1)+p(2)+p(3)+p(4))
         else if ( ishape==10 ) then
            do i = 1, nvert
               elemvc1(i) = p(i)
            end do
            elemvc1(4) = (p(1)+p(2)+p(3))/3d0
         else if ( ishape==9 .or. ishape==18 ) then
            do i = 1, nvert
               elemvc1(i) = p(i)
            end do
            elemvc1(5) = (p(1)+p(2)+p(3)+p(4))*0.25d0
         else if ( ishape==12 ) then
            elemvc1(1) = p(1)
            elemvc1(2) = 0.5d0*(p(1)+p(2))
            elemvc1(3) = p(2)
            elemvc1(4) = 0.5d0*(p(2)+p(3))
            elemvc1(5) = p(3)
            elemvc1(6) = 0.5d0*(p(1)+p(3))
            elemvc1(7) = 0.5d0*(p(1)+p(4))
            elemvc1(8) = 0.5d0*(p(2)+p(4))
            elemvc1(9) = 0.5d0*(p(3)+p(4))
            elemvc1(10) = p(4)
         else if ( ishape==14 ) then
            do i = 1, 4
               elemvc1(2*i-1) = p(i)
               elemvc1(18+2*i-1) = p(i+4)
            end do
            elemvc1(2) = 0.5d0*(p(1)+p(2))
            elemvc1(4) = 0.5d0*(p(1)+p(3))
            elemvc1(6) = 0.5d0*(p(2)+p(4))
            elemvc1(8) = 0.5d0*(p(3)+p(4))
            elemvc1(5) = 0.25d0*(p(1)+p(2)+p(3)+p(4))
            elemvc1(20) = 0.5d0*(p(5)+p(6))
            elemvc1(22) = 0.5d0*(p(5)+p(7))
            elemvc1(24) = 0.5d0*(p(6)+p(8))
            elemvc1(26) = 0.5d0*(p(8)+p(7))
            elemvc1(23) = 0.25d0*(p(5)+p(6)+p(7)+p(8))
            elemvc1(10) = 0.5d0*(p(1)+p(5))
            elemvc1(12) = 0.5d0*(p(2)+p(6))
            elemvc1(16) = 0.5d0*(p(3)+p(7))
            elemvc1(18) = 0.5d0*(p(4)+p(8))
            elemvc1(11) = 0.5d0*(elemvc1(10)+elemvc1(12))
            elemvc1(13) = 0.5d0*(elemvc1(10)+elemvc1(16))
            elemvc1(15) = 0.5d0*(elemvc1(12)+elemvc1(18))
            elemvc1(17) = 0.5d0*(elemvc1(16)+elemvc1(18))
            elemvc1(14) = 0.5d0*(elemvc1(11)+elemvc1(17))
         else if ( ishape==17 ) then
            sum = 0d0
            do i = 1, nvert
               elemvc1(i) = p(i)
               sum = sum+p(i)
            end do
            elemvc1(9) = 0.125d0*sum
         end if

      else

!     --- only in centroid of cell

         sum = 0d0
         do i = 1, nvert
            sum = sum+p(i)
         end do
         elemvc1(1) = sum/nvert

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el1016'

      end if  ! ( debug )
      call erclos ( 'el1016' )

      end
