      subroutine el2006 ( elemvc, sp, div, w, x, eps, u, ugauss, f, phi, &
                          rho, dphidx, sigma, gradu, mconv, xgauss, &
                          omega, icheld, umesh, umeshgauss, imesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 13-02-2006 Debug information
!        version  3.0  date 19-02-2004 Extra parameters for mesh velocity
!        version  2.4  date 04-07-2003 Remove common celwrk
!
!   copyright (c) 1994-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the pressure for the incompressible Navier-Stokes equations
!     in the nodal points and store in array elemvc
!     It is supposed that the gradient of the pressure and the velocity
!     in the centroid have been eliminated before and hence a correction
!     is necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision w(m), sp(ndimlc*n,ndimlc+1), u(n,ndimlc), &
                       x(n,*), eps, f(m,2), phi(n,m), omega(m), &
                       elemvc(n), rho(m), dphidx(n,m,ndimlc), &
                       gradu(9,m), sigma(4,m), ugauss(m,ndimlc), &
                       xgauss(m,ndimlc), div(ndimlc*n,ndimlc+1), &
                       umesh(n,ndimlc), umeshgauss(m,ndimlc)
      integer mconv, icheld, ncent, imesh

!     div            i    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemvc         i    Element vector
!     eps            i    Parameter epsilon for the penalty method
!     f              i    Function f in right-hand side
!     gradu          i    Contains the gradient of u as defined by
!                         subroutine el4917
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of eld900
!     imesh          i    Sequence number of mesh velocity in input vector
!                         If 0 no mesh velocity is present
!     mconv          i    Indicates if convection must be taken into account
!                         (>0)  or not (0)
!     omega          i    Angular velocity
!     phi            i    array of lenght ndecl * mdecl, values of shape
!                         functions in integration points
!     rho            i    density rho
!     sigma          i    Contains the stress tensor as defined by subroutine
!                         el4917
!     sp             i    Divergence matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the
!                         velocity in the sequence of the solution vector
!     u              i    array of length n x ncomp in which the values of
!                         u in the nodal points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                          i   j
!     ugauss         i    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
!     umesh          i    mesh velocity in nodes
!     umeshgauss     i    mesh velocity in gauss points
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled
!                         when gauss integration is applied.
!                         xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                           i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision diag, sum, sum1, pc, gradp(2), det, g1, g2, fact

!     det            Inverse of determinant of L11
!     diag           Diagonal of pressure mass matrix
!     fact           Multiplication factor
!     g1             First component of gradient
!     g2             Second component of gradient
!     gradp          Contains the gradient of p in the centroid
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     ncent          Two times n
!     pc             Pressure in the centroid
!     sum            Help parameter to compute a sum
!     sum1           Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV        0    |    p
!     In the penalty method the zero matrix is replaced by eps M
!                     T
!     Hence Suu u - SP p = f
!           DIV u        = 0
!     is approximated by
!           Suu u -    SP p  = f
!           DIV u + eps M p  = 0
!                       -1
!    or:  p = - 1/eps  M   DIV u
!    The matrix M is exactly the same as the one used in the building of the
!    element matrix
!    The pressure thus computed is equal to the pressure in the centroid.
!    The next step is to compute the gradient of the pressure which is related
!    to the eliminated velocity according to:
!             -1    T
!    gradp = L   ( L   p + f  - S u ), &
!             22    12      2    2
!    where L   is the diagonal matrix used in the elimination, L   a part
!           22                                                  12
!    of the pressure matrix, f  the part of the right-had side corresponding
!                             2
!    to the centroid and S u the part of the matrix multiplied by u that
!                         2
!    corresponds to the centroid.
!    Afterwards the pressure must be evaluated in the nodal points by a
!    linear polynomial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el2006' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el2006'
         write(irefwr,1) 'itype, jcart', itype, jcart
         call prinrl1 ( f, 2, m, 'f' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( itype==901 ) then

!     --- itype = 901, extract pressure from elemvc(1)

         pc = elemvc(1)

      else

!     --- itype = 900, compute pressure in centroid
!         Compute the diagonal of the mass matrix
!         Store multiplication factor in diag

         sum = 0d0
         do i = 1, m
            sum = sum+w(i)
         end do
         diag = 1d0/(eps*sum*2d0)

!        --- multiply the velocity by the divergence matrix and minus the
!            inverse of the mass matrix divided by eps
!            This gives the pressure in the centroid (pc)

         sum = 0d0
         do j = 1, ndimlc
            do i = 1, n
               sum = sum + div((i-1)*ndimlc+j,1) * u(i,j)
            end do
         end do
         pc = - diag * sum

      end if
      if ( icheld==20 ) then

!     --- icheld = 20

         elemvc(1) = pc
         go to 1000

      end if
      if ( debug ) then
         write(irefwr,1) 'jdiag, mconv, imesh', jdiag, mconv, imesh
         write(irefwr,2) 'pc', pc
      end if  ! ( debug )

!     --- Compute the gradient of the pressure
!         compute f2-S2 u for convective and coriolis terms only
!         Store in work
!         Multiply by phi(n) and store in gradp

      if ( jdiag==1 ) then

!     --- jdiag = 1, newton cotes rule

         sum = f(n,1)
         if ( mconv/=0 ) &
             sum = sum - (u(n,1)*gradu(1,n)+ &
                          u(n,2)*gradu(3,n))
         if ( ind(2)/=0 ) sum = sum + 2d0*omega(n)*u(n,2)
         gradp(1) =  w(n)*rho(n)*sum
         sum = f(n,2)
         if ( mconv/=0 ) &
             sum = sum - (u(n,1)*gradu(2,n)+ &
                          u(n,2)*gradu(4,n))
         if ( imesh/=0 ) &
             sum = sum + (umesh(n,1)*gradu(2,n)+ &
                          umesh(n,2)*gradu(4,n))
         if ( ind(2)/=0 ) sum = sum - 2d0*omega(n)*u(n,1)
         gradp(2) = w(n)*rho(n)*sum

      else

!     --- jdiag = 2, Gauss rule

         fact = 2d0
         do i = 1, 2
            sum = 0d0
            do k = 1, m
               sum1 = f(k,i)
               if ( mconv/=0 ) sum1 = sum1 - &
                  (ugauss(k,1)*gradu(i,k)+ugauss(k,2)*gradu(i+2,k))
               if ( imesh/=0 ) sum1 = sum1 + &
                  (umeshgauss(k,1)*gradu(i,k)+ &
                   umeshgauss(k,2)*gradu(i+2,k))
               if ( ind(2)/=0 ) sum1 = sum1 + &
                  fact*omega(k)*ugauss(k,3-i)
               sum = sum + w(k)*rho(k)*phi(n,k)*sum1
            end do
            gradp(i) = sum
            fact = -fact
         end do

      end if
      if ( debug ) call prinrl ( gradp, 2, 'gradp/1' )

!     --- Next compute S u2 and subtract from gradp

      if ( jcart==1 ) then

!     --- Cartesian co-ordinates

         do i = 1, 2
            sum = 0d0
            do k = 1, m
               sum = sum + w(k)*(sigma(i,k)*dphidx(n,k,i)+ &
                                 sigma(4,k)*dphidx(n,k,3-i))
            end do
            gradp(i) = gradp(i) - sum
         end do
         if ( debug ) call prinrl1 ( sigma, m, 4, 'sigma' )

      else if ( jcart==2 ) then

!     --- Axisymmetric co-ordinates

         sum = 0d0
         do k = 1, m
            sum = sum + w(k)*(sigma(1,k)*dphidx(n,k,1)+ &
                              sigma(4,k)*dphidx(n,k,2)+ &
                              sigma(3,k)*phi(n,k)/xgauss(k,1))
         end do
         gradp(1) = gradp(1) - sum
         sum = 0d0
         do k = 1, m
            sum = sum + w(k)*(sigma(2,k)*dphidx(n,k,2)+ &
                              sigma(4,k)*dphidx(n,k,1))
         end do
         gradp(2) = gradp(2) - sum

      else

!     --- Polar co-ordinates

         sum = 0d0
         do k = 1, m
            sum = sum + w(k)*(sigma(1,k)*dphidx(n,k,1)+ &
                             (sigma(4,k)*dphidx(n,k,2)+ &
                              sigma(2,k)*phi(n,k))/xgauss(k,1))
         end do
         gradp(1) = gradp(1) - sum
         sum = 0d0
         do k = 1, m
            sum = sum + w(k)*(sigma(4,k)*dphidx(n,k,1)+ &
                             (sigma(2,k)*dphidx(n,k,2)- &
                              sigma(4,k)*phi(n,k))/xgauss(k,1))
         end do
         gradp(2) = gradp(2) - sum

      end if
      if ( debug ) call prinrl ( gradp, 2, 'gradp/2' )

!     --- Finally multiply by -L22 inverse

      ncent = 2*n
      det = 1d0/(sp(ncent-1,2)*sp(ncent,3)-sp(ncent-1,3)*sp(ncent,2))
      g1 = gradp(1)
      g2 = gradp(2)
      gradp(1) = -det*(sp(ncent,3)*g1-sp(ncent-1,3)*g2)
      gradp(2) = -det*(sp(ncent-1,2)*g2-sp(ncent,2)*g1)
      if ( debug ) call prinrl ( gradp, 2, 'gradp/3' )

      if ( icheld<=42 ) then

!     --- Compute the pressure in the nodal points

         do i = 1, inpelm
            elemvc(i) = pc + gradp(1)*(x(i,1)-x(n,1)) + &
                             gradp(2)*(x(i,2)-x(n,2))
         end do

      else

!     --- Compute the pressure in the integration points

         do i = 1, m
            elemvc(i) = pc + gradp(1)*(xgauss(i,1)-x(n,1)) + &
                             gradp(2)*(xgauss(i,2)-x(n,2))
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el2006'

      end if
      call erclos ( 'el2006' )

      end
