      subroutine el2700 (F, detF)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 01-11-2002  extension for 2D (RvL)
!        version  1.0  date 14-04-1999
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute deformation gradient from displacement gradient
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision F(m,ndim,ndim), detF(m)

!     detf       o    determinant of f
!     f         i/o   At input:  dudx(k,j,i)
!                     At output:
!                     Deformation gradient f(k,j,i) = F_ij( x^k)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k

!     j          Counting variable
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                         c
!            F = (grad  u)  + I
!     grad u is already stored conjugated in dfgrad
!     see EL3007
!          k
!     F  (x ) is stored as: f(k,j,i)  k: integration point
!      ij                           i,j: components
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Deformation gradient

      do k = 1, m
         do j = 1, ndim
            F(k,j,j) = 1d0 + F(k,j,j)
         end do
      end do

!     --- Determinant of deformation gradient:

      if (ndim.eq.2) then

       do k = 1, m
        detF(k) = F(k,1,1)*F(k,2,2) - F(k,2,1)*F(k,1,2)
       end do

      else if (ndim.eq.3) then

       do k = 1, m
           detF(k) = F(k,1,1)*(F(k,2,2)*F(k,3,3) - F(k,2,3)*F(k,3,2))+ &
                     F(k,1,2)*(F(k,2,3)*F(k,3,1) - F(k,2,1)*F(k,3,3))+ &
                     F(k,1,3)*(F(k,2,1)*F(k,3,2) - F(k,2,2)*F(k,3,1))
       end do

      end if
      end
