      subroutine el4023 (ichint,itcoor,linear,rho,irotat,omega, &
          uold,duoldx,w,xg,phig,dphidx,acc,nloc,mloc,lrowloc,psig, &
          dpsidx,n1)
!
! ======================================================================
!
!        programmer   jaap van der zanden
!
!        version 4.3  date   22-12-1992 expl. decl; save common
!        version 4.2  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 4.1  date   19-01-1987 save
!        version 4    date   27-06-1985 itcoor=3
!        version 3    date   02-05-1985 simplified parameter list
!        version 2    date   24-01-1985
!
!
!   copyright (c) 1984,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
      use sepmodulecactl
      implicit none
      integer ichint, itcoor, linear, irotat, nloc, mloc, lrowloc, n1
      double precision rho, omega, uold(*), duoldx(*), w(*), xg(*), &
             phig(*), dphidx(*), psig(*), dpsidx(*), acc(*)
! **********************************************************************
!
!   DESCRIPTION
!
!    the element matrix due to convective acceleration is supplemented
!        and/or, if required, the coriolis acceleration
!
! **********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
!     for variables see elp220 , elm400
!
!     acc   element matrix to be supplemented with convective acceler.
!
!     n      number of basis functiions
!
!     m     number of integration point
!
! **********************************************************************
!
!
!     acc(i,j) (lrowloc,lrowloc)
!
!     jni   = (j-1)*n+i   irow  = (i-1)*lrowloc
!     nln   = n*lrowloc
!     nln1  = nln+n
!
!     the matrix acc is supplemented with the convective acceleration
!        and/or the coriolis acceleration
!
! **********************************************************************
!
!   COMMON  BLOCKS
!
! **********************************************************************
! **********************************************************************
!
!   ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!     None.
! **********************************************************************
!
!   LOCAL  VARIABLES
!
! **********************************************************************
      integer i, ik, irow, j, jk, jni, k
      double precision rhow, sum, sumc, sum1, sum2, sum3, sum11, sum12, &
                       sum21, sum22, sum31, sum32, uw, vw
      integer n2,n3,m2,m3,nq,nln,nln1,m4,m5,n2ln,n2ln1,n2ln2,n1q
      save    n2,n3,m2,m3,nq,nln,nln1,m4,m5,n2ln,n2ln1,n2ln2,n1q
! ======================================================================
      if (ifirst.eq.0) then
         n2=nloc+nloc
         n3=n2+nloc
         m2=mloc+mloc
         m3=m2+mloc
         nq=nloc*mloc
         nln=nloc*lrowloc
         nln1=nloc+nln
         if (itcoor.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
            n2ln=n2*lrowloc
            n2ln1=n2ln+nloc
            n2ln2=n2ln+n2
            n1q=n1*mloc
         end if
      end if
      if ((ichint.eq.0).and.(itcoor.eq.0).and.(nloc.eq.mloc)) then
!
!*  cartesian coordinates , newton-cotes integration
!
      if ((linear.eq.2).or.(linear.eq.1)) then
!
!*   newton linearization (first part) or picard (1)
!
         do 20 i=1,nloc
            irow=(i-1)*lrowloc
            jni=i
            rhow=rho*w(i)
            uw=uold(i)*rhow
            vw=uold(nloc+i)*rhow
            do 10 j=1,nloc
               sum=uw*dphidx(jni)+vw*dphidx(nq+jni)
               acc(irow+j)=acc(irow+j)+sum
               acc(irow+j+nln1)=acc(irow+j+nln1)+sum
               jni=jni+nloc
10          continue
20       continue
      end if
      if ((linear.eq.2).or.(linear.eq.3)) then
!
!*   newton linearization (second part) or picard (3)
!
         do 50 i=1,nloc
            irow=(i-1)*lrowloc
            rhow=rho*w(i)
!
!*    only for i=j matrix elements have to be computed
!
            acc(irow+i)=acc(irow+i)+rhow*duoldx(i)
            acc(irow+nln1+i)=acc(irow+nln1+i)+rhow*duoldx(n3+i)
            acc(irow+nloc+i)=acc(irow+nloc+i)+rhow*duoldx(nloc+i)
            acc(irow+nln+i)=acc(irow+nln+i)+rhow*duoldx(n2+i)
50       continue
      end if
      else if ((ichint.eq.1).and.(itcoor.eq.2)) then
!
!*  polar coordinates /  gauss integration
!
      if ((linear.eq.2).or.(linear.eq.1)) then
!
!*   newton linearization (first part) or picard (1)
!
         do 120 i=1,nloc
            irow=(i-1)*lrowloc
            jk=1
            do 110 j=1,nloc
               sum1=0.0d0
               sum2=0.0d0
               sum3=0.0d0
               ik=(i-1)*mloc+1
               do 105 k=1,mloc
                  rhow=phig(ik)*w(k)
                  sum1=sum1+rhow*(uold(k)*dphidx(jk) &
                                 +uold(mloc+k)*dphidx(nq+jk)/xg(k))
                  rhow=rhow*phig(jk)
                  sum2=sum2+rhow*uold(mloc+k)/xg(k)
                  sum3=sum3+rhow*uold(k)/xg(k)
                  ik=ik+1
                  jk=jk+1
105            continue
               acc(irow+j)=acc(irow+j)+rho*sum1
               acc(irow+j+nln1)=acc(irow+j+nln1)+rho*(sum1+sum3)
               acc(irow+j+nloc)=acc(irow+j+nloc)-rho*sum2
110         continue
120      continue
      end if
      if ((linear.eq.2).or.(linear.eq.3)) then
!
!*   newton linearization (second part) or picard linearization (3)
!
         do 150 i=1,nloc
            irow=(i-1)*lrowloc
            jk=1
            do 140 j=1,nloc
               sum11=0.0d0
               sum12=0.0d0
               sum21=0.0d0
               sum22=0.0d0
               sum2=0.0d0
               ik=(i-1)*mloc+1
               do 130 k=1,mloc
                  rhow=phig(ik)*phig(jk)*w(k)
                  sum11=sum11+rhow*duoldx(k)
                  sum12=sum12+rhow*duoldx(mloc+k)/xg(k)
                  sum21=sum21+rhow*duoldx(m2+k)
                  sum22=sum22+rhow*duoldx(m3+k)/xg(k)
                  sum2=sum2+rhow*uold(mloc+k)/xg(k)
                  ik=ik+1
                  jk=jk+1
130            continue
               acc(irow+j)=acc(irow+j)+rho*sum11
               acc(irow+j+nln1)=acc(irow+j+nln1)+rho*sum22
               acc(irow+j+nloc)=acc(irow+j+nloc)+rho*(sum12-sum2)
               acc(irow+j+nln)=acc(irow+j+nln)+rho*(sum21+sum2)
140         continue
150      continue
      end if
      else if ((ichint.eq.1).and. &
               ((itcoor.eq.0).or.(itcoor.eq.1))) then
!
!*  cartesian coordinates / axisymmetrical coord. /  gauss integration
!
      if ((linear.eq.2).or.(linear.eq.1)) then
!
!*   newton linearization (first part) or picard linearization (1)
!
         do 320 i=1,nloc
            irow=(i-1)*lrowloc
            jk=1
            do 310 j=1,nloc
               sum1=0.0d0
               ik=(i-1)*mloc+1
               do 305 k=1,mloc
                  rhow=phig(ik)*w(k)
                  sum1=sum1+rhow*(uold(k)*dphidx(jk) &
                                 +uold(mloc+k)*dphidx(nq+jk))
                  ik=ik+1
                  jk=jk+1
305            continue
               acc(irow+j)=acc(irow+j)+rho*sum1
               acc(irow+j+nln1)=acc(irow+j+nln1)+rho*sum1
310         continue
320      continue
      end if
      if ((linear.eq.2).or.(linear.eq.3)) then
!
!*   newton linearization (second part) or picard linearization (3)
!
         do 350 i=1,nloc
            irow=(i-1)*lrowloc
            jk=1
            do 340 j=1,nloc
               sum11=0.0d0
               sum12=0.0d0
               sum21=0.0d0
               sum22=0.0d0
               ik=(i-1)*mloc+1
               do 330 k=1,mloc
                  rhow=phig(ik)*phig(jk)*w(k)
                  sum11=sum11+rhow*duoldx(k)
                  sum12=sum12+rhow*duoldx(mloc+k)
                  sum21=sum21+rhow*duoldx(m2+k)
                  sum22=sum22+rhow*duoldx(m3+k)
                  ik=ik+1
                  jk=jk+1
330            continue
               acc(irow+j)=acc(irow+j)+rho*sum11
               acc(irow+j+nln1)=acc(irow+j+nln1)+rho*sum22
               acc(irow+j+nloc)=acc(irow+j+nloc)+rho*sum12
               acc(irow+j+nln)=acc(irow+j+nln)+rho*sum21
340         continue
350      continue
      end if
      else if ((ichint.eq.1).and.(itcoor.eq.3)) then
!
!*  axisymmetrical coordinates with swirl
!
      if ((linear.eq.2).or.(linear.eq.1)) then
!
!*   newton linearization (first part) or picard linearization (1)
!
         do 420 i=1,nloc
            irow=(i-1)*lrowloc
            jk=0
            do 410 j=1,nloc
               sum1=0.0d0
               ik=(i-1)*mloc
               do 405 k=1,mloc
                  sum1=sum1+(uold(k)*dphidx(jk+k) &
                            +uold(mloc+k)*dphidx(nq+jk+k)) &
                            *phig(ik+k)*w(k)
405            continue
               acc(irow+j)=acc(irow+j)+rho*sum1
               acc(irow+j+nln1)=acc(irow+j+nln1)+rho*sum1
               if (j.le.n1) then
                  sum2=0.0d0
                  do 406 k=1,mloc
                     sum2=sum2+phig(ik+k)*w(k) &
                               *uold(m2+k)*psig(jk+k)/xg(k)
406               continue
                  acc(irow+j+n2)=acc(irow+j+n2)-rho*sum2
                  if (i.le.n1) then
                     sum1=0.0d0
                     do 407 k=1,mloc
                        sum1=sum1+psig(ik+k)*w(k) &
                            *(uold(k)*(dpsidx(jk+k)+psig(jk+k)/xg(k)) &
                             +uold(mloc+k)*dpsidx(n1q+jk+k))
407                  continue
                     acc(irow+j+n2ln2)=acc(irow+j+n2ln2)+rho*sum1
                  end if
               end if
               jk=jk+mloc
410         continue
420      continue
      end if
      if ((linear.eq.2).or.(linear.eq.3)) then
!
!*   newton linearization (second part) or picard linearization (3)
!
         do 450 i=1,nloc
            irow=(i-1)*lrowloc
            jk=0
            do 440 j=1,nloc
               sum11=0.0d0
               sum12=0.0d0
               sum21=0.0d0
               sum22=0.0d0
               ik=(i-1)*mloc
               do 430 k=1,mloc
                  rhow=phig(ik+k)*phig(jk+k)*w(k)
                  sum11=sum11+rhow*duoldx(k)
                  sum12=sum12+rhow*duoldx(mloc+k)
                  sum21=sum21+rhow*duoldx(m2+k)
                  sum22=sum22+rhow*duoldx(m3+k)
430            continue
               acc(irow+j)=acc(irow+j)+rho*sum11
               acc(irow+j+nln1)=acc(irow+j+nln1)+rho*sum22
               acc(irow+j+nloc)=acc(irow+j+nloc)+rho*sum12
               acc(irow+j+nln)=acc(irow+j+nln)+rho*sum21
               if (j.le.n1) then
                  sum2=0.0d0
                  do 431 k=1,mloc
                     sum2=sum2+w(k)*phig(ik+k)*uold(m2+k) &
                              *psig(jk+k)/xg(k)
431               continue
                  acc(irow+j+n2)=acc(irow+j+n2)-rho*sum2
               end if
               if (i.le.n1) then
                  sum31=0.0d0
                  sum32=0.0d0
                  do 432 k=1,mloc
                     rhow=psig(ik+k)*phig(jk+k)*w(k)
                     sum31=sum31+rhow*(duoldx(m4+k)+uold(m2+k)/xg(k))
                     sum32=sum32+rhow*duoldx(m5+k)
432               continue
                  acc(irow+j+n2ln)=acc(irow+j+n2ln)+rho*sum31
                  acc(irow+j+n2ln1)=acc(irow+j+n2ln1)+rho*sum32
               end if
               jk=jk+mloc
440         continue
450      continue
      end if
      end if
      if (irotat.eq.1) then
!
!*    coriolis acceleration for irotat=1
!
      if ((ichint.eq.0).and.(itcoor.eq.0).and.(nloc.eq.mloc)) then
!
!*  cartesian coordinates (newton-cotes integration)
!
         do 600 i=1,nloc
            irow=(i-1)*lrowloc
            sumc=2*rho*omega*w(i)
            acc(irow+nloc+i)=acc(irow+nloc+i)-sumc
            acc(irow+nln+i)=acc(irow+nln+i)+sumc
600      continue
      else if ((ichint.eq.1).and.((itcoor.eq.0).or.(itcoor.eq.2))) then
!
!*   cartesian or polar coordinates
!
         do 630 i=1,nloc
            irow=(i-1)*lrowloc
            jk=1
            do 620 j=1,nloc
               sumc=0.0d0
               ik=(i-1)*mloc+1
               do 610 k=1,mloc
                  sumc=sumc+phig(ik)*phig(jk)*w(k)
                  ik=ik+1
                  jk=jk+1
610            continue
               acc(irow+j+nloc)=acc(irow+j+nloc)-2*rho*omega*sumc
               acc(irow+j+nln)=acc(irow+j+nln)+2*rho*omega*sumc
620         continue
630      continue
      else if ((ichint.eq.1).and.(itcoor.eq.3)) then
!
!*    axisymmetrical coordinates with swirl
!
         do 780 i=1,nloc
            irow=(i-1)*lrowloc
            jk=0
            do 770 j=1,nloc
               sumc=0.0d0
               ik=(i-1)*mloc
               if (j.le.n1) then
                  do 760 k=1,mloc
                     sumc=sumc+phig(ik+k)*psig(jk+k)*w(k)
760               continue
                  acc(irow+j+n2)=acc(irow+j+n2)-2*rho*omega*sumc
               end if
               if (i.le.n1) then
                  sumc=0.0d0
                  do 761 k=1,mloc
                     sumc=sumc+psig(ik+k)*phig(jk+k)*w(k)
761               continue
                  acc(irow+j+n2ln)=acc(irow+j+n2ln)+2*rho*omega*sumc
               end if
               jk=jk+mloc
770         continue
780      continue
      end if
      end if
      end
