      subroutine el8211 (rhsd, w, psi, Jd, Jn, npres, iflag)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  1.0  date 06-03-2000
!
!   copyright (c) 1994-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   compute the divergence part of the element vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, iflag
      double precision w(*), rhsd(*), psi(npres,m), Jd(*), Jn(*)

!     iflag          i    use J(0) or Jd(1)
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     npres          i    Number of pressure parameters per element
!     psi            i    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!                         Linear basis functions for the pressure
!     rhsd           o    Right-hand-side vector to be computed
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision sum, J

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      J = 0d0
      do i = 1, npres
         sum = 0d0
         do k = 1, m
            if (iflag.eq.0) J = Jd(k)*Jn(k)
            if (iflag.eq.1) J = Jd(k)
            sum = sum + w(k)*psi(i,k)*(J-1d0)/J
         end do
         rhsd(i) = sum
      end do

      end
