      subroutine el8230 ( elemmt, s, div, isequ, iseqp, npres, ifsi )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.1  date 04-07-2004 changes indices (i,j) in total
!                                      element stiffness matrix (ND)
!        version  2.0  date 25-11-2002 Add option for fluid-structure
!                                      interaction applications (JdH)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Assemble element stiffness matrix for general nonlinear solid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, isequ(*), iseqp(*), ifsi
      double precision elemmt(icount,icount), s(ndim*n,ndim*n), &
                       div(ndim*n,npres)

!     div     i    Divergence matrix
!                  div(i,k) corresponds to the k-th row and i-th column
!                  of the matrix, The k refers to the basis functions of the
!                  pressure, i to the basis functions of the velocity in the
!                  sequence of the solution vector
!     elemmt  o    Element matrix to be filled
!                  Only the part corresponding to div and grad is filled
!     ifsi    i    Indicator for fully coupled fluid-solid interaction problem
!     iseqp   i    Sequence for pressure dof's
!     isequ   i    Sequence for displacement dof's
!     npres   i    Number of pressure unknowns
!     s       i    Element matrix without the gradient and divergence part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element matrix has the following structure
!         | Suu  Sup |
!     S = |          |
!         | Spu   0  |
!     with Suu the material stiffness matrix and Sup and Spu
!     the divergence matrix
!     Depending of the element type the submatrices are filled
!     corresponding to the correct dof.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill material stiffness part  Suu

      do i = 1, ndim*n
         do j = 1, ndim*n
            if ( ifsi.gt.0 ) then
               elemmt(isequ(i),isequ(j)) = tstep * s(j,i)
            else
               elemmt(isequ(i),isequ(j)) = s(j,i)
            end if
         end do
      end do

!     --- Check element type to see whether we have to deal with
!         the pressure degrees of freedom

      if ( itype.gt.200 ) then

!     --- itype > 200 so there are pressure unknowns
!         Fill Sup

         do i = 1, npres
            do j = 1, ndim*n
               if ( ifsi.gt.0 ) then
                  elemmt(isequ(j),iseqp(i)) = -tstep * div(j,i)
               else
                  elemmt(isequ(j),iseqp(i)) = -div(j,i)
               end if
            end do
         end do

!        --- Fill Spu

         do i = 1, npres
            do j = 1, ndim*n
               if ( ifsi.gt.0 ) then
                  elemmt(iseqp(i),isequ(j)) = -tstep * div(j,i)
               else
                  elemmt(iseqp(i),isequ(j)) = -div(j,i)
               end if
            end do
         end do

!        --- Fill 0 part

         do i = 1, npres
            do j = 1, npres
               elemmt(iseqp(i),iseqp(j)) = 0d0
            end do
         end do

      end if

      end
