      subroutine el8231 ( elemvc, f, rhsdiv, isequ, iseqp, npres, s, &
                          div, ufsi, pfsi, index2, ifsi )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 25-11-2002 Extend for fluid-structure
!                                      interaction anslysis (JdH)
!        version  1.0  date 16-10-2000 Derived from el1006
!
!   copyright (c) 2000-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Assemble element vector for general nonlinear solid
!    Add rhs contribution associated with fluid-structure interaction
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!      include 'SPcommon/csubwkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, isequ(*), iseqp(*), index2(*), ifsi
      double precision elemvc(icount), f(ndimlc*n), rhsdiv(npres), &
                       s(ndimlc*n,ndimlc*n), div(ndimlc*n,npres), &
                       ufsi(ndimlc*n), pfsi(npres)

!     div     i    Divergence matrix
!                  div(i,k) corresponds to the k-th row and i-th column
!                  of the matrix, The k refers to the basis functions of the
!                  pressure, i to the basis functions of the velocity in the
!                  sequence of the solution vector
!     elemvc  o    Element vector
!     f       i    Element vector without the effect of the pressure
!     ifsi    i    Indicator for fully coupled fluid-solid interaction analysis
!     index2  i    Array of length icount containing the positions of the
!                  degrees of freedom in the element with respect to usol
!     iseqp   i    Sequence for pressure dof's
!     isequ   i    Sequence for displacement dof's
!     npres   i    Number of pressure unknowns
!     pfsi    i    Element vector with pressure degrees of freedom
!     rhsdiv  i    Divergence right hand side
!     s       i    Element matrix without the gradient and divergence part
!     ufsi    i    Element vector with displacements degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element vector has the following structure
!
!         | fu |
!     f = |    |
!         | fp |
!
!     with fu the material rhs and fp the divergence rhs.
!
!     Depending of the element type the submatrices are filled
!     corresponding to the correct dof.
!
!     Fluid-structure interaction contribution
!
!     Matrix-vector multiplication:
!     | Suu  Sup | | u |
!     |          |.|   |
!     | Spu   0  | | p |
!     with Suu the material stiffness matrix, Sup and Spu
!     the divergence matrix and u and p the degrees of freedom.
!     Depending of the element type the subvectors are filled
!     corresponding to the correct dof.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill in material stiffness part fu

      do i = 1, ndimlc*n
         elemvc(isequ(i)) = f(i)
      end do

!     --- Check element type to see whether we have to deal with
!         the pressure degree of freedom

      if ( itype.gt.200 ) then

!     --- itype > 200 so there are pressure unknowns
!         Fill fp

         do i = 1, npres
            elemvc(iseqp(i)) = rhsdiv(i)
         end do

      end if

      if ( ifsi.gt.0 ) then

!     --- Add contribution of fluid-structure interaction to rhs
!         Fill vector part associated with stiffness Suu
!         Exclude the effect of essential boundary conditions

         do i = 1, ndimlc*n
            if ( index2(isequ(i)).le.nrusol ) then
               do j = 1, ndimlc*n
                  if ( index2(isequ(j)).gt.nrusol ) ufsi(j) = 0d0
                  elemvc(isequ(i)) = elemvc(isequ(i)) + s(i,j)*ufsi(j)
               end do
            end if
         end do

!        --- Check element type to see whether we have to deal with
!            the pressure degrees of freedom

         if (itype.gt.200) then

!        --- itype > 200 so there are pressure unknowns
!            Fill part associated with Sup
!            Exclude the effect of essential boundary conditions

            do i = 1, ndimlc*n
               if ( index2(isequ(i)).le.nrusol ) then
                  do j = 1, npres
                     if ( index2(iseqp(j)).gt.nrusol ) pfsi(j) = 0d0
                     elemvc(isequ(i)) = elemvc(isequ(i))- &
                                        div(i,j)*pfsi(j)
                  end do
               end if
            end do

!           --- Fill part associated with Spu
!               Exclude the effect of essential boundary conditions

            do i = 1, npres
               if ( index2(iseqp(i)).le.nrusol ) then
                  do j = 1, ndimlc*n
                     if ( index2(isequ(j)).gt.nrusol ) ufsi(j) = 0d0
                     elemvc(iseqp(i)) = elemvc(iseqp(i))- &
                                        div(j,i)*ufsi(j)
                  end do
               end if
            end do

         end if

      end if

      end
