      subroutine elfillpresmmt ( psi, pp, kappa, npres, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.0  date 17-09-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill pressure mass matrix weighted by coefficient kappa
!     Hence pp = / kappa psi_i psi_j
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres
      double precision pp(npres,npres), kappa(m), &
                       psi(npres,m), w(m)

!     kappa          i    Contains multiplication coefficients
!     npres          i    Number of pressure parameters per element
!     pp             o    Pressure mass matrix
!     psi            i    Array of length n x m containing the values of the
!                         pressure basis functions in the integration points
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, nsave, jdiagsav
      double precision f(npres), g(m), work(m)

!     f              array of length m to store a vector
!     g              array of length m to store a vector
!     j              Counting variable
!     jdiagsav       Help variable to store the value of jdiag
!     nsave          Help variable to store the value of n
!     work           Work array of length m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillpresmmt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillpresmmt'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nsave = n
      n = npres
      jdiagsav = jdiag
      jdiag = 2

!     --- Fill the pressure matrix according to
!         PP(i,j) = / kappa * psi(i) * psi(j) dx
!                   x

      do j = 1, npres

!     --- Re-initialise array WORK2

         f = 0d0

         g = kappa*psi(j,1:m)

         call el3003( g, f, w, psi, m, npres )

         pp(1:npres,j) = f

      end do

      n = nsave
      jdiag = jdiagsav

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( pp, npres, npres, 'pp' )
         write(irefwr,*) 'End elfillpresmmt'

      end if  ! ( debug )
      call erclos ( 'elfillpresmmt' )

      end
