      function eli400(icheli,jdegfd,coor,iuser,user,vectorloc,index1, &
                      index2)
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.6  date 23-08-2001 Correct call to prinlg
!        version  5.5  date 15-09-1995 Allow icheli=2
!        version  5.4  date 23-11-1992 JvdZ: improve err. rep.
!
!   copyright (c) 1985-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute element integral
!     quadratic triangle
!     itype=400 :    cartesian coordinates
!     itype=404 :    axisymmetric coordinates
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision array, work
      common /celwrk/ array(500),work(1000)
      save   /celwrk/

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheli, jdegfd, index2(*), iuser(*), index1(*)
      double precision eli400, vectorloc(*), user(*), coor(*)

!     coor    i    array of length ndim x npointcac containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     eli400  o    Element integral
!     icheli  i    Defines the type of integral to be computed
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     index2  i    Array of length icount containing the positions of the
!                  degrees of freedom in the element with respect to usol
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     jdegfd  i    The derivative du jdegfd / d ix is computed
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     vectorloc  i    In this array all preceding solutions are stored, i.e.
!                  all solutions that have been computed before and
!                  have been carried to system or deriva by the parameter
!                  islold in the parameter list of these main subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idum(1), iposloc, k, nuserloc
      double precision dum(1), sign, sum
      integer nloc,mloc,iphiloc,ipx,ipy,ipw,ipphix,ipphiy,ipf,indf,mstf
      save    nloc,mloc,iphiloc,ipx,ipy,ipw,ipphix,ipphiy,ipf,indf,mstf

!     dum     General double precision dummy variable
!     i       Counting variable
!     idum    General dummy variable
!     indf    Indicates if f is a function or a constant
!     ipf     Starting address of function values f
!     iphiloc    Starting address in array work of phi
!     iposloc
!     ipphix  Starting address in array work of dphidx
!     ipphiy  Starting address in array work of dphidy
!     ipw     Starting address of weights
!     ipx     Starting address of x-coordinates
!     ipy     Starting address of y-coordinates
!     k       Counting variable
!     m       number of integration points
!     mstf    ?
!     n       number of basis functions for each component
!     nuserloc   ?
!     sign    ?
!     sum     Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFLR5  Get user-defined parameters (with funccf)
!     ELP212  ?
!     ELUSER  Get parameters from arrays user,iuser
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      42   Array IUSER(.) has value which is too small
!     341   Icheli has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('eli400')
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eli400'
         write(irefwr,1) 'icheli', icheli
         call prinin ( index1, 6, 'index1' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst.eq.0) then

!     --- ifirst = 0,  initialise some parameters and arrays

         nloc=6
         mloc=6
         iphiloc=0
         if ( itype.eq.404 .or. itype.eq.421 ) iphiloc=2
         ipx=1
         ipy=ipx+mloc
         ipw=ipy+mloc
         ipphix=ipw+mloc
         ipphiy=ipphix+mloc*nloc
         ipf=ipphiy+mloc*nloc
         iposloc=iuser(ielgrp+5)
         if ( iposloc.lt.7) then
            call errint(ielgrp+5,1)
            call errint(iposloc,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 42, 5, 0, 0)
         end if
         nuserloc=0
         call eluser(indf,mstf,array(ipf),iposloc,iuser,user,nelemcac, &
                     npointcac,nuserloc,mloc)
         call prinlg('iuser',iposloc-1,user,iuser)
         call prinlg('user',-nuserloc,user,iuser)
         if ( inpelm.ne.nloc ) then
            call errint(nloc,1)
            call errint(inpelm,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 3, 5, 0, 0)
         end if
         if ( icheli.lt.0 .or. icheli.gt.2 .and. icheli.ne.12 ) then
            call errint ( icheli, 1 )
            call errint(itype,2)
            call errint(ielgrp,3)
            call errsub ( 341, 3, 0, 0)
         end if
         if ( ierror.ne.0 ) call instop
      end if
      call elp212(iphiloc,coor,sign,'volint/eli400',array(ipx),array(ipy), &
         array(ipw),array(ipphix),array(ipphiy),index1,work)
      if ( indf.gt.0 ) call elflr5(1,indf,mstf,user,array(ipf), &
          array(ipx),array(ipy),array,dum,nloc,mloc,k,index1,dum,idum,dum)
      sum=0d0
      if ( icheli.eq.1 ) then
         do 400 i=1,nloc
            sum=sum+array(ipw+i-1)*array(ipf+i-1)
400      continue
      else
         do 410 i=1,nloc
            work(i) = vectorloc(index2(2*i-2+jdegfd))
410      continue
         if ( icheli.eq.12 ) then
            do 420 i=1,nloc
               work(i) = abs(work(i))
420         continue
         end if

         do 450 i=1,nloc
            sum=sum+array(ipw+i-1)*array(ipf+i-1)*work(i)
450      continue
      end if
      eli400=sum

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'sum', sum
         write(irefwr,*) 'End eli400'

      end if  ! ( debug )
      call erclos ( 'eli400' )
      end function eli400
