      subroutine elm200
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.8  date 29-05-2014 remove call to time subroutine
!        version  3.7  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for general nonlinear solids
!     ELM200 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine elmsubs
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMSUBS
!             - ELM200
!    The subroutine is able to handle the following element shapes and cases:
!    2D:  6 noded triangle
!         7 noded extended triangle
!         9 noded quad
!    3D:  27 node brick
!         15 node tetrahedron
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     non_linear
!     mechanics
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      use sepmoduleelem
      use sepmodulework
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnp, maxnu, maxcoef, maxusr, iseqnr
      parameter (maxnu = 27, maxnp = 8, maxcoef=45, maxusr=1000)
      integer isub, isup, jelp
      double precision dum(1)
      save isub, isup, jelp
      integer i, j, imatlaw, iusrvc, iusrflg, ncoef, &
              nd, ndisp, npres, mdiv, nsig, idum
      integer isequ(3*maxnu), iseqp(maxnp)
      integer, pointer :: index3n1(:)
      double precision, pointer :: vecoldlc(:), vecoldlc1(:)
      save imatlaw, iusrvc, iusrflg, ncoef, nd, ndisp, &
           npres, isequ, iseqp, nsig
      double precision coor0(ndim,inpelm), coorn(ndim,inpelm), &
                       coor1(ndim,inpelm), ufsi(3*maxnu), &
                       pfsi(maxnp), &
                       pp(maxnp,maxnp), rhsdiv(maxnp), fdiv(maxnu), &
                       kappa(maxnu), qmat(3*maxnu), psipres(4*27)
      integer ipx, ipxg, ipw, ipphix, ipphi0, ipphin, ipphi, ipdphi0, &
              ipdphin, ipdphi, ippsix, ipdpsi, ipend, &
              ippsi, ipcoef, ipu, ipug, ipFn, ipJn, ipFd, ipJd, &
              ippn, ipp, ipusrvc, ipB, ipelS, ipelR, ipelL, ipelU, &
              ipwork, ifsi, ipxgn, ipxg0, ichfib, icent
      save ipx, ipxg, ipw, ipphix, ipphi0, ipphin, ipphi, ipdphi0, &
           ipdphin, ipdphi, ippsix, ipdpsi, &
           ippsi, ipcoef, ipu, ipug, ipFn, ipJn, ipFd, ipJd, &
           ippn, ipp, ipusrvc, ipB, ipelS, ipelR, ipelL, ipelU, &
           ipwork, ifsi, ipxgn, ipxg0, ichfib, icent

!     coor0          Coordinates in undeformed configuration
!     coor1          Coordinates in current configuration
!     coorn          Coordinates in reference configuration
!     dum            dummy array
!     fdiv           Dummy
!     i              Counting variable
!     icent          Index of centroid
!     ichfib         Indicates which fiber subroutine has to be used
!                    See el0200
!     idum           dummy variable
!     ifsi           Sequence number of old solution vector for
!                    fully coupled fluid-structure interaction
!     imatlaw        Type of material constitutive equation to be used
!     ipb            Pointer to B-matrix
!     ipcoef         Pointer to coefficients
!     ipdphi         Pointer to dphi/dx on current configuration
!     ipdphi0        Pointer to dphi/dx on undeformed configuration
!     ipdphin        Pointer to dphi/dx on reference configuration
!     ipdpsi         Pointer to dpsi/dx on current configuration
!     ipell          Pointer to element divergence matrix
!     ipelr          Pointer to element rhs
!     ipels          Pointer to element material stiffness matrix
!     ipelu          Pointer to element divergence rhs
!     ipend          Last position in array
!     ipfd           Pointer to F_delta
!     ipfn           Pointer to F_n
!     ipjd           Pointer to J_delta
!     ipjn           Pointer to J_n
!     ipp            Pointer to hydrostatic pressure
!     ipphi          Pointer to basisfunctions on current configuration
!     ipphi0         Pointer to basisfunctions on undeformed configuration
!     ipphin         Pointer to basisfunctions on reference configuration
!     ipphix         Pointer to phiksi
!     ippn           Pointer to hydrostatic pressure on reference configuration
!     ippsi          Pointer to linear basis functions
!     ippsix         Pointer to psiksi
!     ipu            Pointer to nodal displacements
!     ipug           Pointer to displacements in integration points
!     ipusrvc        Pointer to user vector
!     ipw            Pointer to weighing functions
!     ipwork         Pointer to work array
!     ipx            Pointer to element coordinates
!     ipxg           Pointer to integration point coordinates
!     ipxg0          ?
!     ipxgn          ?
!     iseqnr         Sequence number for first old solution vector
!     iseqp          Sequence for pressure dof's
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iusrflg        User flag
!     iusrvc         if>0, a user defined vector with iusrvec dof's present
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELelm200x. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                     7:  Subroutine ELP627  (Extended quadratic triangle)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!                    16:  Subroutine ELP637  (Extended Quadratic Tetrahedron)
!     kappa          Dummy
!     maxcoef        Maximum number of coefficients
!     maxnp          Maximum number of pressure dof's
!     maxnu          Maximum number of displacement dof's
!     maxusr         Maximum number of uservector dof's
!     mdiv           Defines if divergence right-hand side is used (1)
!                    or not (0)
!     ncoef          Number of coefficients in the equation
!     nd             Dimension of tangent stiffness matrix
!     ndisp          Number of displacement dof's
!     npres          Number of pressure dof's
!     nsig           ?
!     pfsi           Element vector with pressure degrees of freedom
!     pp             dummy
!     psipres        Contains the pressure basis functions
!                    psipres(i,k) refers to i-th basis function in
!                    integration point k
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rhsdiv         Divergence right-hand side (if mdiv=1)
!     ufsi           Element vector with displacement degrees of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0200         performs the initializations for subroutine ELM200/ELD200
!     EL1000         Compute the divergence matrix for Navier-Stokes
!     EL1004         Compute gradient and divergence matrix (Taylor-Hood)
!     EL2600         Get old displacement vector and deformation gradient for
!                    general nonlinear solid
!     EL2720         Fill matrix B for general nonlinear solid element
!     EL8200         compute the stress part of the element matrix for general
!                    nonlinear solid elements
!     EL8210         compute the stress part of the element vector for general
!                    nonlinear solid element
!     EL8211         compute the divergence part of the element vector
!     EL8230         Assemble element stiffness matrix for general nonlinear
!                    solid
!     EL8231         Assemble element vector for general nonlinear solid
!     ELFILLPSIPRES  Fill the pressure basis functions in the integration points
!                    Crouzeix Raviart elements
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM900MASSMAT  Fill element mass matrix for the incompressible
!                    Navier-Stokes equations
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!     See subroutine el0200 for a description
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The following system is build:
!
!     | S     -L |  | du |   | R |
!     |          |  |    | = |   |
!     |   T      |  |    |   |   |
!     | -L     0 |  | dp |   | U |
!
!     If case of element type 200 no divergence part is build.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('elm200')
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm200'
         write(irefwr,1) 'ifirst', ifirst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if (ifirst==0) then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0200 ( E%iuser, E%user, jelp, imatlaw, iusrvc, &
                       iusrflg, nd, isub, isup, coeffarr, npres, ndisp, &
                       isequ, iseqp, ncoef, ifsi, nsig, maxusr, &
                       ichfib )
         if ( inpelm==27 ) then
            icent = 14
         else
            icent = n
         end if

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser)

!        --- Fill pointers in array coeffarr

         ipcoef  = 1
         ipx     = ipcoef  + maxnu*maxcoef
         ipxg0   = ipx     + maxnu*ndim
         ipxgn   = ipxg0   + maxnu*ndim
         ipxg    = ipxgn   + maxnu*ndim
         ipw     = ipxg    + maxnu*ndim
         ipphix  = ipw     + maxnu
         ipphi0  = ipphix  + maxnu*maxnu*ndim
         ipphin  = ipphi0  + maxnu*maxnu
         ipphi   = ipphin  + maxnu*maxnu
         ipdphi0 = ipphi   + maxnu*maxnu
         ipdphin = ipdphi0 + maxnu*maxnu*ndim
         ipdphi  = ipdphin + maxnu*maxnu*ndim
         ippsi   = ipdphi  + maxnu*maxnu*ndim
         ipu     = ippsi   + maxnp*maxnu
         ipug    = ipu     + maxnu*ndim
         ipFn    = ipug    + maxnu*ndim
         ipJn    = ipFn    + maxnu*ndim*ndim
         ipFd    = ipJn    + maxnu
         ipJd    = ipFd    + maxnu*ndim*ndim
         ippn    = ipJd    + maxnu
         ipp     = ippn    + maxnu
         ipusrvc = ipp     + maxnu
         ipB     = ipusrvc + maxnu*maxusr
         ipelS   = ipB     + (ndim*ndim)*ndim*maxnu*maxnu
         ipelL   = ipelS   + (maxnu*ndim)**2
         ipelR   = ipelL   + (maxnu*ndim)*maxnp
         ipelU   = ipelR   + maxnu*ndim
         ipdpsi  = ipelU   + maxnp
         ippsix  = ipdpsi  + maxnp*maxnu*ndim
         ipwork  = ippsix  + maxnp*maxnu*ndim
         ipend   = ipwork  + max((n+1)*m,n**2+n+m)

!        --- Check length of array coeffarr (declared in elmgrinf)

         call inichkarlen ( ipend, 69696, 'coeffarr' )

      end if
      mdiv=0

!     --- Set iseqnr according to the type of problem

      if ( ifsi>0 ) then
         iseqnr = ifsi
      else
         iseqnr = 1
      end if

!     --- Get coordinates of deformed mesh on increment n (coorn) and
!         compute current and initial coordinates (coor1 and coor0)

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n1 => indoldsols(iseqtype)%index3
      vecoldlc1 => oldsols(iseqnr+1)%oldvecs
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         do j = 1 , ndim
            coorn(j,i) = coor(j,index1(i))
            coor1(j,i) = coorn(j,i) + &
                         vecoldlc(index3n(isequ((i-1)*ndim+j)))
            coor0(j,i) = coorn(j,i) - &
                         vecoldlc1(index3n1(isequ((i-1)*ndim+j)))
         end do
      end do

!     --- Compute basis functions on reference and deformed mesh

      call elm800basefn ( coor0, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphi0), &
                          coeffarr(ipxg0), coeffarr(ipphi0), index1, qmat, dum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix) )
      call elm800basefn ( coorn, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphin), &
                          coeffarr(ipxgn), coeffarr(ipphin), index1, qmat, dum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix) )
      call elm800basefn ( coor1, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphi), &
                          coeffarr(ipxg), coeffarr(ipphi), index1, qmat, dum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix) )
      if (ierror/=0) goto 1000

!     --- Get all old vectors

      call el2600 ( coeffarr(ipphi0), coeffarr(ipphin), coeffarr(ipphi), &
                    coeffarr(ipdphi0), coeffarr(ipdphin), &
                    coeffarr(ippsi), coeffarr(ipx), coeffarr(ipxg), &
                    coeffarr(ipu), coeffarr(ipug), &
                    coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                    coeffarr(ipp), coeffarr(ippn), coeffarr(ipusrvc), iseqnr, &
                    iusrvc, isequ, iseqp, npres, coeffarr(ipxg0), &
                    coeffarr(ipxgn), maxusr )

!     --- Displacement and pressure solution for fluid-structure interaction

      if ( ifsi>0 ) then

         iseqtype = oldsols(ifsi)%iseqtype
         index3n => indoldsols(iseqtype)%index3
         vecoldlc => oldsols(ifsi)%oldvecs
         do i = 1,n*ndim
            ufsi(i) = vecoldlc(index3n(isequ(i)))
         end do

         if (itype>200) then
            do i = 1, npres
               pfsi(i) = vecoldlc(index3n(iseqp(i)))
            end do
         end if

      end if

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if (ind(i)>0 .or. ist(i)>0) then

!        --- Coefficient is variable

            call elflr71 (i, E%user, coeffarr, coeffarr(ipx), &
                         coeffarr(ipxg), m, n, coeffarr(ipphi), &
                         1, index1, &
                         E%iuser, coeffarr(ipug), work )
         end if
      end do

!     --- compute B-matrix

      call el2720 (coeffarr(ipB), coeffarr(ipdphi), nd, coeffarr(ipphi), coeffarr(ipxg))

!     --- Fill parts of matrix if necessary

      if (matrix) then

!     --- Compute stiffness matrix

         call el8200 ( coeffarr(ipelS), coeffarr(ipxg), coor0, coeffarr(ipw), &
                       coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                       coeffarr(ipp), coeffarr(ipB), coeffarr(ipusrvc), nd, imatlaw, &
                       iusrvc, iusrflg, coeffarr(ipcoef), coeffarr(ipxg0), &
                       ncoef, index1, nsig, ichfib, maxusr )

!        --- Compute divergence matrix

         if (itype==201) then
            call elfillpsipres ( psipres, npres, coeffarr(ipxg), coeffarr(ipx), &
                                 icent )
            call el1000 ( coeffarr(ipelL), coeffarr(ipelL), coeffarr(ipdphi), coeffarr, &
                          coeffarr(ipphi), coeffarr(ipw), coeffarr(ipx), coeffarr(ipxg), &
                          0, dum, dum, dum, dum, 0, ishape, psipres, &
                          icent, coeffarr )
         end if
         if (itype==202) then
            call el1004 (coeffarr(ipelL), coeffarr(ipelL), coeffarr(ippsi), &
                         coeffarr(ipdphi), coeffarr(ipphi), coeffarr(ipw), &
                         coeffarr(ipxg), 0, dum, pp, kappa, rhsdiv, &
                         fdiv, mdiv, coeffarr )
         end if
      end if

!     --- Fill parts of vector if necessary

      if (vector .and. notvec==0) then

!     --- First set element vector equal to zero

         do i = 1, icount
            elemvc(i) = 0d0
         end do
         do i = 1, 3*n
            coeffarr(ipelR+i-1) = 0d0
         end do
         do i = 1, npres
            coeffarr(ipelU+i-1) = 0d0
         end do

!        --- Compute stiffness vector

         call el8210 ( coeffarr(ipelR), coeffarr(ipxg), coor0, coeffarr(ipw), &
                       coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                       coeffarr(ipp), coeffarr(ipB), coeffarr(ipusrvc), nd, imatlaw, &
                       iusrvc, iusrflg, coeffarr(ipcoef), coeffarr(ipxg0), &
                       ncoef, index1, nsig, ichfib, maxusr )

!        --- Compute divergence vector

         if (itype>200) then
            call el8211 (coeffarr(ipelU), coeffarr(ipw), coeffarr(ippsi), &
                         coeffarr(ipJd), coeffarr(ipJn), npres, 0)
         end if

      end if

!     --- Fill parts of mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0,  compute mass matrix

         call elm900massmat ( elemms, coeffarr, coeffarr(ipw), &
                              coeffarr(ipphi), dum, &
                              dum, dum, dum, dum(1), &
                              dum, 2, 0, dum, dum, &
                              dum, dum, index1, 0, dum, &
                              dum, idum, isequ )

      end if  ! ( notmas==0 )

!     --- Assemble total element stiffness matrix

      if (matrix) then
         call el8230 (elemmt, coeffarr(ipelS), coeffarr(ipelL), &
                      isequ, iseqp, npres, ifsi)
      end if

!     --- Assemble total element vector

      if (vector) then
         call el8231 (elemvc, coeffarr(ipelR), coeffarr(ipelU), &
                      isequ, iseqp, npres, coeffarr(ipelS), coeffarr(ipelL), &
                      ufsi, pfsi, index5, ifsi)
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         write(irefwr,*) 'imas ', imas
         call prinrl(elemvc,icount,'elemvc')
         call prinrl1(elemmt,icount,icount,'elemmt')
         if ( notmas==0 ) then
            if ( imas==1 ) then
               call prinrl(elemms,icount,'elemms')
            else
               call prinrl1(elemms,icount,icount,'elemms')
            end if  ! ( imas==1 )
         end if  ! ( notmas==0 )
         write(irefwr,*) 'End elm200'

      end if  ! ( debug )
      call erclos ('elm200')

      end
