      subroutine elm401
! ======================================================================
!
!        programmer    Jaap van der Zanden/Guus Segal
!        version  7.1  date 09-03-2013 Correction index2
!        version  7.0  date 29-05-1996 improve precision of data;
!                                      improve test for straight boundary
!                                      Remove superfluous parameters
!        version  6.2  date 23-12-1992 improve err. rep.
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     natural boundary conditions : itype=401,403,405,407
!                                         423,424,425
!                                         437
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ij, ik, indsn, indst, iposus, itcoor,  &
              j, jk, k, mstasn, mstast, mult
      double precision xg, xx1, xx2, yy1, yy2, sng, stg, sum1, sum2, sum3
      double precision vtlold(12)
      double precision dxdksi(3), dydksi(3), phig(9), x(3), xksi(3), y(3),  &
                sn(3), st(3), wg1(3), ws(3), wg(3)
      save itcoor, indsn, indst, mstasn, mstast, mult, phig, xksi, sn, st, &
           wg, ws

!     dxdksi
!     dydksi
!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ij         help parameter
!     ik
!     indsn
!     indst
!     iposus
!     itcoor
!     j          Counting variable
!     jk
!     k          Counting variable
!     mstasn
!     mstast
!     mult
!     phig
!     sn
!     sng
!     st
!     stg
!     sum1       Parameter to compute a sum
!     sum2
!     sum3
!     vtlold
!     wg
!     wg1
!     ws
!     x          x co-ordinate of point
!     xg         x co-ordinates of the Gauss points
!     xksi
!     xx1
!     xx2
!     y          y-coordinate read from the SEPRAN neutral plot file
!     yy1
!     yy2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFLAR    Get user-defined parameters (with funccf)
!     ELUSER    Get parameters from arrays user,E%iuser
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PRININ    print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array E%iuser(.) has value which is too small
!     221   Subroutine elm... or els... not available for this type number
! **********************************************************************
!
!                       PSEUDO CODE
!
!     fill element vector for natural boundary conditions
!                (quadratic line element)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ws /0.3333333333333333d0, 1.3333333333333333d0, &
               0.3333333333333333d0/
      data wg /0.5555555555555556d0, 0.8888888888888889d0, &
               0.5555555555555556d0/
      data phig /0.687298334620742d0 ,0.0d0, -0.087298334620742d0, &
                 0.4d0               ,1.0d0,  0.4d0, &
                -0.087298334620742d0 ,0.0d0,  0.687298334620742d0/
      data xksi/-0.774596669241483d0 ,0.0d0,  0.774596669241483d0/
! ======================================================================
!
      call eropen('elm401')
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm401'
         call prinin ( index2, icount, 'index2' )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if (ifirst==0) then
      notmat=1
      notvec=0
      iposus=E%iuser(ielgrp+5)
      if (iposus<7) then
         call errint(ielgrp+5,1)
         call errint(iposus  ,2)
         call errint(ielem,3)
         call errint(itype,4)
         call errint(ielgrp,5)
         call errsub ( 42, 5, 0, 0)
      end if
      nuser=0
      itcoor=0
      if ( itype==437 ) then

!     --- itype = 437,  element with temperature and swirl

         itcoor = 4
         mult = 4
      else if (itype>422) then

!     --- itype = 423, 424 or 425   Temperature dependent

         if(itype==423) itcoor=1
         if(itype==424) itcoor=2
         if(itype==425) itcoor=3
         mult=3
      else
         mult=2
         if (itype==401) itcoor=1
         if (itype==403) itcoor=3
         if (itype==405) itcoor=2
         if (itype==407) then
            itcoor=4
            mult = 3
         end if
      end if
      do i=1,icount
         elemvc(i)=0d0
      end do

!     --- read external tractions; these may be functions

      call eluser(indsn,mstasn,sn,iposus,E%iuser,E%user,nelemcac, &
                  npointcac,nuser,inpelm)
      call eluser(indst,mstast,st,iposus,E%iuser,E%user,nelemcac, &
                  npointcac,nuser,inpelm)
      endif
      x(1:3)=coor(1,index1(1:3))
      y(1:3)=coor(2,index1(1:3))
      do 30 i=1,icount
         vtlold(i)=vecold(index2(i))
30    continue

!     --- if not constant compute the external tractions
!         as function of (x,y)

      if (indsn>0) &
             call elflar(indsn,mstasn,E%user,sn,x,y,y,vtlold, &
                         inpelm,2,index1,1,1)
      if (indst>0) &
             call elflar(indst,mstast,E%user,st,x,y,y,vtlold, &
                         inpelm,2,index1,1,1)
      xx1=x(1)-2*x(2)+x(3)
      xx2=0.5d0*(x(1)-x(3))
      yy1=y(1)-2*y(2)+y(3)
      yy2=0.5d0*(y(1)-y(3))
      goto (100,200,300,400),itcoor

!     --- error : illegal type or itcoor

         call errint(itype,1)
         call errsub ( 221, 1, 0, 0)
         go to 1000

!        --- cartesian coordinates
!            dxdksi for newton-cotes integration (simpson)

100   if ((abs(xx1)<1.0d-6*abs(xx2)) .and. &
          (abs(yy1)<1.0d-6*abs(yy2))) then

!     --- boundary is straight

         do 120 i=1,3
            elemvc(mult*i-mult+1)=(-sn(i)*yy2-st(i)*xx2)*ws(i)
            elemvc(mult*i-mult+2)=(sn(i)*xx2-st(i)*yy2)*ws(i)
120      continue
      else

!     --- boundary is curved

         do 130 i=1,3
            dxdksi(i)=(i-2)*xx1-xx2
            dydksi(i)=(i-2)*yy1-yy2
130      continue
         do 140 i=1,3
            elemvc(mult*i-mult+1)=(sn(i)*dydksi(i)+st(i)*dxdksi(i))* &
               ws(i)
            elemvc(mult*i-mult+2)=(-sn(i)*dxdksi(i)+st(i)*dydksi(i))* &
               ws(i)
140      continue
      end if
      go to 1000

!     --- axis-symmetrical coordinates
!         dxdksi, dydksi for gauss integration (3 points)
!                        later  weights are incorporated in them
!         no different procedure for straight and curved boundaries

200   do 210 i=1,3
         dxdksi(i)=xksi(i)*xx1-xx2
         dydksi(i)=xksi(i)*yy1-yy2
210   continue

!     --- integration can be performed
!         Correct weights with factor 2 pi

      do 215 k = 1,3
215      wg1(k) = 2d0 * pi * wg(k)
      ik=1
      do 240 i=1,3
         sum1=0.0d0
         sum2=0.0d0
         do 230 k=1,3
            sng=0.0d0
            stg=0.0d0
            xg=0.0d0
            jk=k
            do 220 j=1,3

!           --- compute sn,st,x in integration point xksi(k)

               sng=sng+sn(j)*phig(jk)
               stg=stg+st(j)*phig(jk)
               xg=xg+x(j)*phig(jk)
               jk=jk+3
220         continue
            sum1=sum1+( sng*dydksi(k)+stg*dxdksi(k))*phig(ik)*xg*wg1(k)
            sum2=sum2+(-sng*dxdksi(k)+stg*dydksi(k))*phig(ik)*xg*wg1(k)
            ik=ik+1
230      continue
         elemvc(mult*i-mult+1)=sum1
         elemvc(mult*i-mult+2)=sum2
240   continue
      go to 1000

!     --- polar  coordinates
!         dxdksi, dydksi for gauss integration (3 points)
!                        later  weights are incorporated in them
!         no different procedure for straight and curved boundaries

300   do 310 i=1,3
         dxdksi(i)=xksi(i)*xx1-xx2
         dydksi(i)=xksi(i)*yy1-yy2
310   continue

!     --- integration can be performed

      ik=1
      do 340 i=1,3
         sum1=0.0d0
         sum2=0.0d0
         do 330 k=1,3
            sng=0.0d0
            stg=0.0d0
            xg=0.0d0
            jk=k
            do 320 j=1,3

!           --- compute sn,st,x in integration point xksi(k)

               sng=sng+sn(j)*phig(jk)
               stg=stg+st(j)*phig(jk)
               xg=xg+x(j)*phig(jk)
               jk=jk+3
320         continue
            sum1=sum1+( sng*dydksi(k)*xg+stg*dxdksi(k))*phig(ik)*wg(k)
            sum2=sum2+(-sng*dxdksi(k)+stg*dydksi(k)*xg)*phig(ik)*wg(k)
            ik=ik+1
330      continue
         elemvc(mult*i-mult+1)=sum1
         elemvc(mult*i-mult+2)=sum2
340   continue
      go to 1000

!     --- axis-symmetrical coordinates with swirl
!         dxdksi, dydksi for gauss integration (3 points)
!                        later  weights are incorporated in them
!         no different procedure for straight and curved boundaries

400   do 410 i=1,3
         dxdksi(i)=xksi(i)*xx1-xx2
         dydksi(i)=xksi(i)*yy1-yy2
410   continue

!     --- Correct weights for factor 2 pi

      do k =1,3
         wg1(k) = 2d0 * pi * wg(k)
      end do

!     --- integration can be performed

      ik=1
      ij=0
      do 440 i=1,3
         sum1=0.0d0
         sum3=0.0d0
         do 430 k=1,3
            sng=0.0d0
            stg=0.0d0
            xg=0.0d0
            jk=k
            do 420 j=1,3

!           --- compute sn,st,x in integration point xksi(k)

               sng=sng+sn(j)*phig(jk)
               stg=stg+st(j)*phig(jk)
               xg=xg+x(j)*phig(jk)
               jk=jk+3
420         continue
            sum1=sum1+( sng*dydksi(k)+stg*dxdksi(k))*phig(ik)*xg*wg1(k)
            sum3=sum3+(-sng*dxdksi(k)+stg*dydksi(k))*phig(ik)*xg*wg1(k)
            ik=ik+1
430      continue
         elemvc(ij+1)=sum1
         elemvc(ij+3)=sum3
         ij=ij+mult
440   continue
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm401'
      end if  ! ( debug )
      call erclos('elm401')
      end
