      subroutine elm408(array)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 23-12-1992 JvdZ: improve err. rep.
!        version  1.4  date 22-11-1992 replace ertrap; remove name
!        version  1.3  date 27-06-1992 Minor change in call to elflr5
!        version  1.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.1  date 03-05-1988
!
!   copyright (c) 1987-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill element matrix and element vector for boundary conditions
!     corresponding to free boundary
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision work
      common /celwrk/ work(1500)
      save   /celwrk/

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision array(*)

!    array     work array of length 50
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kchois, ipx, ipw, ipphis, ipphi, ipgaus, iptang, ipgamm, &
              indgam, mstgam, igaus, k, i
      double precision dum(1), sum1, sum2, h
      save kchois,ipx,ipw,ipphis,ipphi,ipgaus,iptang,ipgamm, &
           indgam,mstgam
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!   ELFLR5  get user-defined parameters (with funccf)
!   ELP121  shape functions for quadratic line element
!   ELUSR0  fill constants from array user and E%iuser
!   ERCLOS  reset old name of calling tree
!   EROPEN  add name of routine to calling tree
!   ERRINT  add an integer value to parameters for error message
!   ERRSUB  report error
!   INSTOP  stop the SEPRAN program
!   PRINLG  check and print array length
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   3: Element has incorrect number of nodal points for thid type
!   42: array E%iuser(.) has value which is too small
!   74: incorrect number of degrees of freedom for this type
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- itype=408 :    cartesian coordinates   quadratic triangle
!         I / O
!         none

      call eropen('elm408')
      if(ifirst.eq.0) then

!     --- ifirst = 0,  initialise some parameters and arrays

         kchois=105
         n=3
         m=3
         ipx=1
         ipw=1+2*n
         ipphis=ipw+m
         ipphi=ipphis+n*m
         ipgaus=ipphi+n*m
         iptang=ipgaus+m*2
         ipgamm=iptang+2*m
         notmat=1
         ipos=E%iuser(ielgrp+5)
         if(ipos.lt.7) then
            call errint(ielgrp+5,1)
            call errint(ipos,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 42, 5, 0, 0)
         end if
         nuser=0
         call elusr0(indgam,mstgam,array(ipgamm),E%iuser,E%user)
         call prinlg('E%iuser',ipos-1,E%user,E%iuser)
         call prinlg('user',-nuser,E%user,E%iuser)
         if (inpelm.ne.n) then
            call errint(n,1)
            call errint(inpelm,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 3, 5, 0, 0)
         end if
         if(icount.ne.2*n) then
            call errint(icount,1)
            call errint(2*n,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 74, 5, 0, 0)
         end if
         if(ierror.ne.0) call instop
      end if

!     --- computation of basis functions

      call elp121(kchois,coor,array(ipx),array(ipw), &
              array(ipphis),index1,array(ipphi),array(ipgaus), &
              array(iptang))
      if(vector) then
         if(indgam.gt.0) call elflr5(igaus,indgam,mstgam,E%user, &
                array(ipgamm),array(ipgaus),array(ipgaus+m),array,dum, &
                n,m,ncomp,index1,array(ipphi),index1,work)
         do 150 i=1,n
            sum1=0d0
            sum2=0d0
            do k=0,m-1
               h=array(ipw+k)*array(ipgamm+k)*array(ipphis+(i-1)*n+k)
               sum1=sum1+h*array(iptang+k)
               sum2=sum2+h*array(iptang+k+n)
            end do
            E%vec(2*i-1)=-sum1
150         E%vec(2*i)=-sum2
      end if
      call erclos('elm408')
      end
