      subroutine elm900check ( x, elemmt, npres, nveloc, elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute an analytical solution and multiply the stiffness matrix by this
!     solution. Compare with right-hand side (type 900/901/902)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, nveloc
      double precision x(nveloc/2,*), elemmt(icount,icount), &
                       elemvc(icount)

!     elemmt         i    Element matrix
!     elemvc         i    Element vector
!     npres          i    Number of pressure parameters per element
!     nveloc         i    Number of velocity parameters
!     x              i    co-ordinates of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision u(nveloc/2), v(nveloc/2), p(npres), y(icount), &
                       sol(icount)

!     i              Counting variable
!     p              Array containing the pressure
!     sol            Solution vector reorganized to sepran sequence
!     u              contains u-velocities
!     v              contains v-velocities
!     y              matrix times vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMATMULT      Matrix vector multiplication for a full element matrix
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900check' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900check'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Fill solution into u,,v , p
!         In this case u = x(1+t), v = -2z(1+t), p = 0

      print *,'t ', t
      do i = 1, nveloc/2
         u(i) = x(i,1)*(1d0+t)
         v(i) = -2d0*x(i,2)*(1d0+t)
      end do
      call prinrl(u,nveloc/2,'u')
      call prinrl(v,nveloc/2,'v')
      do i = 1, npres
         p(i) = 0d0
      end do

!     --- Put everything in vector sol

      do i = 1, nveloc/2
         sol(2*i-1) = u(i)
         sol(2*i) = v(i)
      end do
      do i = 1, npres
         sol(nveloc+i) = p(i)
      end do

!     --- Multiply by element matrix (transpose)

      call elmatmult ( elemmt, sol, y, icount )

!     --- Subtract from right-hand side

      do i = 1, icount
         y(i) = y(i)-elemvc(i)
      end do
      call prinrl ( y, icount, 'rhsd-amat u' )

1000  call erclos ( 'elm900check' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm900check'

      end if

      end
