      subroutine elm910rhsld3 ( coeffarr, ndim, work, phi, elemvc, &
                                induv, iworktr, index2, &
                                coortr, maxtrn, index1, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-09-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the element vector for type 910/911 in case of iload = 3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, induv(ndim,*), iworktr(*), &
              index2(*), maxtrn, index1(*)
      double precision coeffarr(m,*), work(*), phi(n,m), elemvc(*), &
                       coortr(maxtrn,*), w(*)

!     coeffarr       i    Array with coefficents per integration point
!     coortr         i    kprob part n (local transformations)
!     elemvc         o    Element vector
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     index2         i    Array of length icount containing the positions of the
!                         degrees of freedom in the element with respect to USOL
!     induv          i    Index array of size ncomp x inpelm that contains
!                         the positions of the degrees of freedom 1 to ncomp
!                         in the element
!                         Hence induv(k,i) contains the position of the k^th
!                         degree of freedom in the i^th node of the element
!                         with respect to the element vector
!     iworktr        i    array of length nusol output of to0016; contents:
!                         iworktr(i) = 0  no tranformation for degree of
!                                         freedom i
!                         iworktr(i) = j  when i must be transformed
!                                       j is the sequence number in kprobn
!     maxtrn         i    Maximum number of real variables per point with
!                         respect to the local transformations
!     ndim           i    Dimension of the space.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     w              i    array of length m containing the weights for
!                         integration
!     work                Work array of length 2n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jpos, ih
      double precision r12, r13

!     i              Counting variable
!     ih             Help variable to store the column in coortr the present
!                    node is pointing to
!     j              Counting variable
!     jpos           Position in array
!     r12            Contains r12, r22 or r32 of transformation matrix
!                    i.e. component of t1
!     r13            Contains r12, r22 or r33 of transformation matrix
!                    i.e. component of t2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2239   No local transformation found in point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm910rhsld3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm910rhsld3'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      if ( ndim.eq.2 ) then

!     --- ndim = 2
!         load vector is a scalar corresponding to second component
!         This is the same as for iload = 0

         i = 2
         call el3003 ( coeffarr(1,i), work, w, phi, m, n )
         do j = 1, n
            elemvc(induv(i,j)) = work(j)
         end do

      else

!     --- load given in Cartesian directions, must be transformed to
!         tangential directions  (3D only)
!         Clear elemvc

         do i = 1, icount
            elemvc(i) = 0d0
         end do

!        --- Loop over the ndim components

         do i = 1, ndim

            if ( ind(i).ne.0 ) then

!           --- Component is unequal to zero

               call el3003 ( coeffarr(1,i), work, w, phi, m, n )

!              --- Transform each point in tangential directions

               do j = 1, n
                  jpos = induv(1,j)
                  ih = iworktr(index2(jpos+2))/3
                  if ( ih.le.0 ) then

!                 --- ih <= 0, no local transformation

                     call errint ( index1(j), 1 )
                     call errint ( ielem, 2 )
                     call errint ( ielgrp, 3 )
                     call errint ( itype, 4 )
                     call errsub ( 2239, 4, 0, 0 )
                     go to 1000

                  end if
                  r12 = coortr(3*i-1,ih)
                  r13 = coortr(3*i,ih)
                  elemvc(jpos+1)=elemvc(jpos+1)+work(j)*r12
                  elemvc(jpos+2)=elemvc(jpos+2)+work(j)*r13
               end do

            end if

         end do

      end if  ! ( ndim.eq.2 ) then

1000  call erclos ( 'elm910rhsld3' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm910rhsld3'

      end if

      end
