      subroutine elmass27 ( elemms, elemmsloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element mass matrix for the special case of 27 node brick
!     with 3 unknowns per point except for the centroid, where there are
!     7 unknowns due to the presence of the pressure
!     The local element matrix refers to 1 degree of freedom only
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemms(85,85), elemmsloc(27,27)

!     elemms         o    Element mass matrix
!     elemmsloc      i    Element mass matrix for one component only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store first 14 rows and columns
!     Store first 14 rows and columns 15 to 27
!     Store rows 15 to 27 and first 14 columns
!     Store rows and columns 15 to 27
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Store first 14 rows and columns

      do i = 1, 14
         do j = 1, 14
            do k = 1, 3
               elemms((i-1)*3+k,(j-1)*3+k) = elemmsloc(i,j)
            end do
         end do
      end do

!     --- Store first 14 rows and columns 15 to 27

      do i = 1, 14
         do j = 15, 27
            do k = 1, 3
               elemms((i-1)*3+k,(j-1)*3+k+4) = elemmsloc(i,j)
            end do
         end do
      end do

!     --- Store rows 15 to 27 and first 14 columns

      do i = 15, 27
         do j = 1, 14
            do k = 1, 3
               elemms((i-1)*3+k+4,(j-1)*3+k) = elemmsloc(i,j)
            end do
         end do
      end do

!     --- Store rows and columns 15 to 27

      do i = 15, 27
         do j = 15, 27
            do k = 1, 3
               elemms((i-1)*3+k+4,(j-1)*3+k+4) = elemmsloc(i,j)
            end do
         end do
      end do
      end
