      subroutine elp220 ( ichois, ichint, coor, index1, isg, icrv, delta, &
                          x, xg, e1, dldx1, w, &
                          phi, dphidx, truc, trpsut, qphixx )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  5.2  date 18-12-1998 Include common
!        version  5.1  date 29-05-1996 Improve accuracy of data
!
!   copyright (c) 1984-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector two-dimensional Navier-Stokes equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     flow
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ichint, index1(*), isg, icrv
      double precision delta, coor(*), x(*), xg(*), e1(*), dldx1(*), &
                       w(*), phi(*), dphidx(*), truc(*), trpsut(*), &
                       qphixx(*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     delta      o    jacobian of element
!     dldx1      o    derivatives of linear baricentric functions
!     dphidx     o    Array of length n x m x ndim  containing the derivatives
!                     of the basis functions in the sequence:
!                     d phi / dx (xg ) = dphidx (i,k,1);
!                          i        k
!                     d phi / dy (xg ) = dphidx (i,k,2);
!                          i        k
!                     d phi / dz (xg ) = dphidx (i,k,3);
!                          i        k
!                     If the element is a linear simplex, the
!                     derivatives are constant and only k = 1 is filled.
!     e1         o    derivatives of linear basis functions
!     ichint     i    indicates the kind of integral to be computed.
!                     possibilities:
!                      1  /  f u  ds
!                      2  /  f  u.n  ds
!                      3  /  f  u.t  ds
!                      4  /  f  n. sigma .n  ds
!                      5  /  f  n. sigma .t  ds
!                      6  /  f  n. sigma     ds
!                      7  /  f  u  n  ds
!                      8  sum f(i) u(i)
!     ichois     i    Choice parameter
!                       0:  cartesian coordinates      |  icoor=0
!                       1:  axi-symmetric coordinates  |       =1
!                       2:  polar coordinates          |       =2
!                       3:  cartesian coord/gauss integ|       =3
!                       for curved elements gauss integration is used !!
!                           unless itruv=0 and itrps=0
!                           isg,icrv,x,w,qphixx are stored
!                           wnc/wng , dpdks0/dpdks1 , ... are computed once
!                     +10:  + dphidx            |  ideriv=1
!                     +20:  + truc              |  itruv=1 (and also ideriv=1)
!                     +40:  + trpsut            |  itrps=1
!     icrv       i    icrv=0,2 (triangle is not curved or is curved)
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     isg        o    sIGN of the jacobian (anti-clockwise : >0 )
!     phi        o    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!     qphixx     o    integral of dphi(i)/dksi * dphi(j)/dksi on reference
!                     triangle
!     trpsut     o    transformation matrix for divergence-free
!                     element (from u,v at mid-side nodes to
!                     psi at vertices and u.t at mid-side nodes)
!                     not yet used
!     truc       o    transformation matrices for elimination of u7,v7
!                     with help of integral((x-x7)*div(u)dx),...
!     w          o    array of length m containing the weights for integration
!     x          o    co-ordinates of points
!     xg         o    co-ordinates of the Gauss points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dpdks0(49), dpdet0(49), wnc(7), wng(7), &
                       detjac(7), dpdks1(49), dpdet1(49), phigt(42), &
                       dptdks(42), dydeta(7), &
                       dptdet(42), dxdksi(7), dydksi(7), dxdeta(7), &
                       baricc(21), phinc(49), phig(49)
      integer inum, itcoor, jchint, jtcoor, itruv, itrps
      save inum, itcoor, jchint, jtcoor, itruv, itrps, &
           wnc, wng, phig, phigt, phinc, &
           dpdks0, dpdet0, dpdks1, dpdet1, dptdks, dptdet, baricc
      integer i
      double precision curved, sign

!     baricc     baricentric coordinates of integration points
!                 (itcoor>0)
!     curved     indication if the element is curved (true) or not (false)
!     detjac     determinant of jacobi in nodal points
!     dpdet0     derivatives of basis functions in reference triangle
!     dpdet1     derivatives of basis functions in integration points
!     dpdks0     derivatives of basis functions in reference triangle
!     dpdks1     derivatives of basis functions in integration points
!     dptdet     derivatives of transformation in integration points
!     dptdks     derivatives of transformation in integration points
!     dxdeta     derivative of x with respect to eta
!     dxdksi     derivative of x with respect to ksi
!     dydeta     derivative of y with respect to eta
!     dydksi     derivative of y with respect to ksi
!     i          Counting variable
!     ideriv     computation of dphidx    (=1) or not (=0)
!     inum       Number of numbers read in a record
!     itcoor     type of coordinates (see ichois)
!     itrps      computation of trpsut    (=1) or not (=0)
!     itruv      computation of truc,truc(13) (=1) or not (=0)
!     jchint
!     jtcoor
!     phig
!     phigt      transformation functions in integration points
!     phinc
!     sign
!     wnc
!     wng
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP271         on reference element Int(dphi(i)/dksi * dphi(j)/dksi)dx, ..
!     ELP272         fill coordinates; test if element (triangle,quadril) is
!                    curved
!     ELP273         derivative of baric. coord. (itcoor=0, straight triangle)
!     ELP274         derivatives of basis functions (straight triangle)
!     ELP275         compute transformation matrices for elimination of uc,vc
!     ELP277         Jacobian matrix for curved triangles; xg in integration
!                    node
!     ELP278         check jacobian; compute area for curved triangles;
!                    compute weights
!     ELP279         dphi/dx on curved triangle
!     ELP281         compute basis functions in integration points, &
!                    compute derivatives of transformation basis functions
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2193   Transformation to divergence-free elements has not yet been
!           implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!         1     2     3      4     5      6          7
      data dpdks0/ &
       -3.d0,-1.d0, 1.d0, .25d0, 1.d0,-.25d0,-0.3333333333333333d0, &
        4.d0, 0.d0,-4.d0,  1.d0, 0.d0, -1.d0, 0.d0, &
       -1.d0, 1.d0, 3.d0, .25d0,-1.d0,-.25d0, 0.3333333333333333d0, &
        0.d0, 0.d0, 0.d0,  5.d0, 4.d0, -1.d0, 1.3333333333333333d0, &
        0.d0, 0.d0, 0.d0,-.75d0, 0.d0, .75d0, 0.d0, &
        0.d0, 0.d0, 0.d0,  1.d0,-4.d0, -5.d0,-1.3333333333333333d0, &
        0.d0, 0.d0, 0.d0,-6.75d0,0.d0,6.75d0, 0.d0/

!         1     2     3      4     5      6          7
      data dpdet0/ &
       -3.d0,-.25d0, 1.d0, .25d0, 1.d0,-1.d0,-0.3333333333333333d0, &
        0.d0, -5.d0,-4.d0,  1.d0, 0.d0, 0.d0,-1.3333333333333333d0, &
        0.d0, .75d0, 0.d0,-.75d0, 0.d0, 0.d0, 0.d0, &
        0.d0, -1.d0, 4.d0,  5.d0, 0.d0, 0.d0, 1.3333333333333333d0, &
       -1.d0,-.25d0,-1.d0, .25d0, 3.d0, 1.d0, 0.3333333333333333d0, &
        4.d0, -1.d0, 0.d0,  1.d0,-4.d0, 0.d0, 0.d0, &
        0.d0,6.75d0, 0.d0,-6.75d0,0.d0, 0.d0, 0.d0/

!    initialize wnc : weights for newton-cotes integration on
!                     reference triangle with area 0.5
      data wnc/.025d0,.06666666666666666d0,.025d0, &
               .06666666666666666d0, &
               .025d0,.06666666666666666d0,.225d0/

!    initialize wng : weights for gauss integration on
!                     reference triangle with area 0.5
      data wng/.0629695902724135d0, .0661970763942532d0, &
               .0629695902724135d0, .0661970763942532d0, &
               .0629695902724135d0, .0661970763942532d0, &
               .11250d0/

!    initialize phinc : basis functions in nodal points
      data phinc/1.0d0,7*0.0d0,1.0d0,7*0.0d0,1.0d0,7*0.0d0, &
                 1.0d0,7*0.0d0,1.0d0,7*0.0d0,1.0d0,7*0.0d0, &
                 1.0d0/
! ======================================================================
!
      call eropen( 'elp220' )

!     --- ifirst=0  compute parameters integrals qphixx,qphixx(50),qphixx(99), &
!                   and (itcoor>0) baricc,phig,dptdks,...

      if (ifirst.eq.0) then

!     --- ifirst=0  compute parameters itcoor,ideriv,itruv,itrps

         inum=ichois/10
         itcoor=ichois-10*inum
         jtcoor=itcoor
         if (itcoor.eq.0) then
            jchint=0
         else
            jchint=1
         end if
         ideriv=0
         itruv=0
         itrps=0
         if (inum.ne.0) then
            goto (11,12,12,14,15,16,17),inum
17          itrps=1
16          itruv=1
11          ideriv=1
            goto 20
12          ideriv=1
            itruv=1
            goto 20
15          ideriv=1
14          itrps=1
20       end if
         if ((itcoor.ne.0).and.(itruv.eq.1)) ideriv=1
         call elp271(dpdks0,dpdet0,wnc,qphixx,qphixx(50),qphixx(99),7)
         call elp281(baricc,phig,phigt, &
                     dptdks,dptdet,dpdks1,dpdet1)
      end if

!     --- fill coordinates in x,x(8)  determine whether element is curved

      call elp272(coor,index1,7,x,x(8),curved)
      icrv=nint(curved)
      if (icrv.lt.1) then

!     --- boundaries of element are straight : special treatment
!             compute e1,dldx1,w,isg,delta,xg
!                     dphidx    if ideriv=1

         if (itcoor.eq.0) then
            call elp273(itcoor,x,x(8),wnc,e1,e1(4),dldx1,dldx1(4), &
                        sign,delta,w,xg,xg(8),baricc,7)
         else
            call elp273(itcoor,x,x(8),wng,e1,e1(4),dldx1,dldx1(4), &
                        sign,delta,w,xg,xg(8),baricc,7)
         end if
         if (ideriv.eq.1) then
            call elp274(itcoor,dldx1,dphidx,baricc)
            call elp274(itcoor,dldx1(4),dphidx(50),baricc)
         end if
      else

!     --- boundaries of element are curved  (reset itcoor)
!             compute w,isg,delta,xg, &
!                     dphidx    if ideriv=1

         if (itcoor.eq.0) itcoor=3
         call elp277(itcoor,x,x(8),dxdksi,dydksi,dxdeta,dydeta, &
                     xg,xg(8),phigt,dptdks,dptdet,7)
         call elp278(itcoor,wng,dxdksi,dydksi,dxdeta,dydeta, &
                     xg,detjac,sign,delta,w,7)
         if ( ierror/=0 ) go to 1000
         if (ideriv.eq.1) then
            call elp279(dxdksi,dydksi,dxdeta,dydeta,detjac, &
                        dpdks1,dpdet1,dphidx,dphidx(50),7,7)
         end if
      end if

!     --- compute truc        if itruv=1
!                 trpsut      if itrps=1

      if (itruv.eq.1) call elp275(itcoor,x,x(8),e1,e1(4),sign,curved, &
          w,xg,xg(8),phig,dphidx,dphidx(50),truc,truc(13),7,7)
      if (itrps.eq.1) then

!     --- no transformation to divergence free basis functions avialable

         call errsub ( 2193, 0, 0, 0 )
         call instop

!        --- if (itrps.eq.1) call elp276(itcoor,e1,e1(4),trpsut)

      end if

!     --- set ichint and fill phi

      isg=nint(sign)
      if ((itcoor.eq.0).and.((icrv.eq.0).or.((itruv.eq.0).and. &
          (itrps.eq.0)))) then
         ichint=0
         if ((ifirst.eq.0).or.(ichint.ne.jchint)) then
            jchint=0
            do i=1,49
               phi(i)=phinc(i)
            end do
         end if
         do i=1,14
            xg(i)=x(i)
         end do
      else
         ichint=1
         if ((ifirst.eq.0).or.(ichint.ne.jchint)) then
            jchint=1
            do i=1,49
               phi(i)=phig(i)
            end do
         end if
      end if

!     --- reset itcoor to original value

      itcoor=jtcoor
1000  call erclos( 'elp220' )
      end
