      subroutine elp278 ( itcoor, wnc, dxdksi, dydksi, dxdeta, dydeta, &
                          x1g, detjac, sign, delta, w, nloc )
! ======================================================================
!
!        programmer     Jaap van der Zanden
!        version 3.5    date   26-12-1994 New commons cmessc and cmessg
!        version 3.4    date   23-06-1992 New common cmessg
!        version 3.3    date   22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1984-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!      check jacobian, compute area for curved triangles
!      compute weights
! ********************************************************************
!
!                      KEYWORDS
!
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
      integer itcoor,nloc
      double precision sign, delta, wnc(*), dxdksi(*), dydksi(*), &
             dxdeta(*), dydeta(*), x1g(*), detjac(*), w(*)

!     delta   i    jacobian of element
!     detjac  i    ?
!     dxdeta  i    ?
!     dxdksi  i    ?
!     dydeta  i    ?
!     dydksi  i    ?
!     itcoor  i    ?
!     n       i    number of basis functions for each component
!     sign    i    ?
!     w       i    array of length m containing the weights for integration
!     wnc     i    ?
!     x1g     i    ?
! ********************************************************************
!
!                      COMMON BLOCKS
!

! ********************************************************************
!                      LOCAL PARAMETERS
!
      integer i
      double precision djac, h

!     djac    ?
!     h       Help variable to store a constant temporarily
!     i       Counting variable
! ********************************************************************
!
!                      SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! ********************************************************************
!
!                      I/O
!
!    none
! ********************************************************************
!
!                      ERROR MESSAGES
!
!     229   Jacobian for isoparametric element approximately zero.
!     230   Jacobian of element is zero or has changing sign
! ********************************************************************
!
!                      PSEUDO CODE
!
!   trivial
! ======================================================================
      call eropen( 'elp278' )
      do 10 i=1,nloc
         detjac(i)=dxdksi(i)*dydeta(i)-dydksi(i)*dxdeta(i)
10    continue
!
!     check jacobian
!
      sign=1
      djac=detjac(1)
      if (djac.lt.0) sign=-1
      if (abs(djac).le.1.0d-16) then
!
!     error 229 :  delta=0
!
         call errint ( itype, 1 )
         call errint ( ielem, 2 )
         call erreal ( djac, 1 )
         call errsub ( 229, 2, 1, 0)
         go to 1000
      end if
      if (sign.gt.0) then
!
!     sign=1 :  jacobian of first point is greater than 0
!
        do 30 i=2,nloc
           djac=detjac(i)
           if (djac.lt.0) then
!
!          error 230 ,  detjac(i).lt.0 (different sign)
!
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 230, 3, 0, 0)
            end if
30       continue
      else
!
!     sign=-1  jacobian<0
!
          do 50 i=2,nloc
             djac=detjac(i)
             if (djac.gt.0) then
!
!         error 230  :  detjac(i).gt.0 (different sign)
!
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 230, 3, 0, 0)
               go to 1000
            end if
50       continue
      end if
!
      delta=0.0d0
      do 70 i=1,nloc
         h=abs(detjac(i))*wnc(i)
         delta=delta+h
         if ((itcoor.eq.0).or.(itcoor.eq.3)) then
            w(i)=h
         else if (itcoor.eq.1) then
            w(i)=h*x1g(i)*2d0*pi
         else
            w(i)=h*x1g(i)
         end if
70    continue
!
!     set delta equal to 2*area
!
      delta=2*delta
1000  call erclos( 'elp278' )
      end
