      subroutine elsyminv ( s, sinv )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  1.1  date 13-08-2012 New call to elmatinv
!        version  1.0  date 13-04-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the inverse of a symmetric stored 3 x 3 matrix s
!   and store in sinv
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_inversion
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(6), sinv(6)

!     s        i     3x3 matrix to be inverted
!     sinv     o     inversion of s
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work1(3,3), work2(3,3)
      logical singular

!     singular       if false the matrix inverse subroutine stops in case the
!                    matrix is singular
!     work1          Work array
!     work2          Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMATINV    Compute the inverse of a 3x3 matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      work1(1,1) = s(1)
      work1(2,2) = s(2)
      work1(3,3) = s(3)
      work1(1,2) = s(4)
      work1(2,1) = s(4)
      work1(2,3) = s(5)
      work1(3,2) = s(5)
      work1(3,1) = s(6)
      work1(1,3) = s(6)
      singular= .false.
      call elmatinv (work1, work2, singular)
      sinv(1) = work2(1,1)
      sinv(2) = work2(2,2)
      sinv(3) = work2(3,3)
      sinv(4) = work2(1,2)
      sinv(5) = work2(2,3)
      sinv(6) = work2(3,1)
      end
