      subroutine sepel0900
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 900
!     Navier-Stokes equation, penalty approach
!     Old subroutine el0900
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: elgetintcoef, inirepscal

!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Corrects integer value in case of scalars
!     SEPADAPRULE900 Adapt integration rule for elements of type 900-902
!     SEPEL0900CF    Fill real coefficients for elements of type 900
!     SEPEL0900DIV   Set npress and icent and allocate phipress, gradmat
!                    and divmat for elements of type 900
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETTIME   Extract time parameter from iuser
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELMODCONVEC Fill modelconv, type_compress and typeconvect for
!                    Navier-Stokes equation
!     SEPELMODELV    Fill modelvisc for Navier-Stokes equation
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0900' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0900'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

!     --- time integration information

      call sepelgettime

!     --- type of viscosity model and model right-hand side

      call sepelmodelv

      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      call sepgetcoorsys
      npsi = 1
      if ( E%ishape==4 .or. E%ishape==6 .or. E%ishape==7 .or. &
           E%ishape==14 .or. E%ishape==16 ) npsi = E%ndim+1

!     --- Read modelconv, type_compress and typeconvect

      call sepelmodconvec

      if ( debug ) write(irefwr,1) 'ipos, ishift/3', E%ipos, E%ishift

!     --- Adapt integration rule for specific cases

      call sepadaprule900

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Fill all real coefficients

      call sepel0900cf

!     --- Set npress and icent and allocate phipress, gradmat and divmat
!         Also gradv, etaeff, sub matrices and so on

      call sepel0900div

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,1) 'B%m, B%n, B%cart', B%m, B%n, B%cart
         write(irefwr,1) 'B%rule, B%jdiag', B%rule, B%jdiag
         write(irefwr,*) 'End sepel0900'

      end if  ! ( debug )
      call erclos ( 'sepel0900' )

      end subroutine sepel0900
