      subroutine sepel900viscst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Old subroutine sepelnsstress
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPEL900VISC2D Fill stress tensor for Navier-Stokes into matrix S
!                    Cartesian coordinates (2d)
!     SEPEL900VISC3D Fill stress tensor for Navier-Stokes into matrix S
!                    Cartesian coordinates (3d)
!     SEPEL900VISCAX Fill stress tensor for Navier-Stokes into matrix S
!                    Axisymmetric coordinates
!     SEPEL900VISCPL Fill stress tensor for Navier-Stokes into matrix S
!                    Polar coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2984   combination of compressible flow and reduced stress term
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900viscst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900viscst'
         write(irefwr,1) 'typecontinuity, modelvisc', &
                          E%typecontinuity, E%modelvisc
         write(irefwr,1) 'jcart, ndim', B%cart, E%ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%typecontinuity==1 .or. E%typecontinuity==4 ) then

!     --- compressible flow

         if ( E%modelvisc==10 ) then

!        --- The combination compressible flow and E%modelvisc=10 not possible

            call errint ( E%typecontinuity, 1 )
            call errint ( E%modelvisc, 2 )
            call errsub ( 2984, 2, 0, 0 )
            go to 1000

         end if  ! ( E%modelvisc==10 )

      end if  ! ( E%typecontinuity==1 .or. E%typecontinuity==4 )

      select case ( B%cart )
      case (1)

!     --- Cartesian coordinates

         if ( E%ndim==2 ) then

!        --- ndim = 2, two-dimensional

            call sepel900visc2d

         else

!        --- ndim = 3, three-dimensional

            call sepel900visc3d

         end if  ! ( E%ndim==2 )

      case (2,4)

!     --- Axisymmetric coordinates

         call sepel900viscax

      case (3)

!     --- Polar coordinates

         call sepel900viscpl

      end select  ! case ( B%cart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900viscst'

      end if  ! ( debug )
      call erclos ( 'sepel900viscst' )

      end subroutine sepel900viscst
