      subroutine sepelnselong ( elemvc, a1, u, nelemvc, ndcomp, mdecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the elongation for the Navier-Stokes equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
     integer, intent(in) :: nelemvc, ndcomp, mdecl
     double precision, intent(in) :: u(mdecl,*), a1(9,m)
     double precision, intent(inout) :: elemvc(ndcomp,nelemvc)

!     a1             i    A1 tensor defined by gradv + gradv^T
!     elemvc         o    Element vector
!     mdecl          i    number of quadrature points in element,
!                         same as  m  in common /celint/
!     ndcomp         i    Indicates the number of quantities that must be filled
!                         in elemvc and elemwg per nodal point
!     nelemvc        i    Length parameter for elemvc
!     u              i    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      double precision :: normu

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnselong' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnselong'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 .and. jcart/=4 ) then

!     --- ndimlc = 2, except cylindrical co-ordinates with swirl

         do k = 1, nelemvc
            normu = u(k,1)**2+u(k,2)**2
            if ( normu<=epsmac )  then
               elemvc(1,k) = 0d0
            else
               elemvc(1,k) = (0.5d0*(u(k,1)**2*a1(1,k) + &
                                     u(k,2)**2*a1(2,k)) + &
                              u(k,1)*u(k,2)*a1(4,k) ) / normu
            end if
         end do

      else

!     --- ndimlc = 3 or cylindrical co-ordinates with swirl

         if ( itype==410 ) then
         
!        --- Type 410

            do k = 1, nelemvc
               normu = u(k,1)**2+u(k,2)**2+u(k,3)**2
               if ( normu<=epsmac ) then
                  elemvc(1,k) = 0d0
               else
                  elemvc(1,k) = (0.5d0*(u(k,1)**2*a1(1,k) + &
                                        u(k,2)**2*a1(4,k) + &
                                        u(k,3)**2*a1(6,k)) + &
                                 u(k,1)*u(k,2)*a1(2,k) + &
                                 u(k,1)*u(k,3)*a1(3,k) + &
                                 u(k,2)*u(k,3)*a1(5,k))/normu
               end if
            end do
            
         else
         
!        --- Standard case
         
            do k = 1, nelemvc
               normu = u(k,1)**2+u(k,2)**2+u(k,3)**2
               if ( normu<=epsmac ) then
                  elemvc(1,k) = 0d0
               else
                  elemvc(1,k) = (0.5d0*(u(k,1)**2*a1(1,k) + &
                                        u(k,2)**2*a1(2,k) + &
                                        u(k,3)**2*a1(3,k)) + &
                                 u(k,1)*u(k,2)*a1(4,k) + &
                                 u(k,1)*u(k,3)*a1(6,k) + &
                                 u(k,2)*u(k,3)*a1(5,k))/normu
               end if
            end do
            
         end if  ! ( itype==410 )

      end if

1000  call erclos ( 'sepelnselong' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnselong'

      end if  ! ( debug )

      end subroutine sepelnselong
