      subroutine sepelnssecinv ( a1, secinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute second inverse in case of Navier-Stokes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: secinv(m)
      double precision, intent(in) :: a1(9,m)

!     a1             i    A1 tensor defined by gradv + gradv^T
!     secinv         o    Array containing the second invariant in the
!                         integration nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The second inverse II is defined as II = 1/2 A_1:A_1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnssecinv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnssecinv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 .and. (jcart==1 .or. jcart==3) ) then

!     --- ndimlc = 2, Cartesian or Polar co-ordinates

         secinv = 0.5d0*(a1(1,:)**2+a1(2,:)**2)+a1(4,:)**2

      else if ( ndimlc==2 .and. jcart==2 ) then

!     --- ndimlc = 2, Cylinder co-ordinates

         secinv = 0.5d0*(a1(1,:)**2+a1(2,:)**2+a1(3,:)**2)+a1(4,:)**2

      else

!     --- ndimlc = 3

         if ( itype==410 ) then

!        --- itype = 410, the sequence of the stress tensor is defined
!            different from usual

            secinv = 0.5d0*(a1(1,:)**2+a1(4,:)**2+a1(6,:)**2)+&
                            a1(2,:)**2+a1(3,:)**2+a1(5,:)**2

         else

            secinv = 0.5d0*(a1(1,:)**2+a1(2,:)**2+a1(3,:)**2)+&
                            a1(4,:)**2+a1(5,:)**2+a1(6,:)**2

         end if  ! ( itype==410 ) then

      end if  ! ( ndimlc==2 .and. (jcart==1 .or. jcart==3) )

1000  if ( debug ) then

!     --- Debug information

         call prinrl (secinv , m, 'secinv' )
         write(irefwr,*) 'End sepelnssecinv'

      end if  ! ( debug )
      call erclos ( 'sepelnssecinv' )

      end subroutine sepelnssecinv
