      subroutine el8002 ( elemmt, s, div, w, x, xgauss, elast, anu )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-04-2003 make npres integer
!        version  1.0  date 03-09-1994 Extracted from el4920
!
!   copyright (c) 1992-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the penalty matrix for rubber elements
!     and add to the existing element matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     rubber_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision w(m), div(ndimlc*n,*), xgauss(m,*), &
                       x(n,*), elast(m), s(n*ndimlc,n*ndimlc), &
                       elemmt(n*ndimlc,n*ndimlc), anu(m)

!     anu     i    Array containing the Poisson parameter nu
!     div     i    Divergence matrix
!                  div(i,k) corresponds to the k-th row and i-th column
!                  of the matrix, The k refers to the basis functions of the
!                  pressure, i to the basis functions of the velocity in the
!                  sequence of the solution vector
!     elast   i    Array containing the Elasticity modulus E
!     elemmt  o    Element matrix to be filled
!     s       i    Previous version of element matrix
!     w       i    array of length mdecl containing the weights for integration
!     x       i    array containing the coordinates of the nodal points
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
!     xgauss  i    array of length m x ndim containing the co-ordinates of
!                  the gauss points.  array xgauss is only filled when gauss
!                  integration is applied.
!                  xg  = xgauss (i,1);  yg  = xgauss (i,2);  zg  = xgauss (i,3)
!                    i                    i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, icent, npres
      double precision diag(4), sum
      save icent

!     diag    Diagonal of the pressure mass matrix
!     i       Counting variable
!     icent   Index of centroid
!     j       Counting variable
!     k       Counting variable
!     npres   Number of pressure parameters per element
!     sum     Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The "Rubber" matrix has the shape:
!            |                T  |
!            |   S        -DIV   |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV         M   |    p
!     or:
!                      T
!           Suu u - DIV p  = f
!           DIV u +  M p  = 0
!                 -1
!    or:  p = -  M   DIV u
!                  T  -1
!       ( Suu + DIV  M   DIV ) u = f
!    The matrix M is defined by
!    M(i,j) = / psi  psi  (E/3(1-2nu)) dx
!             x    i    j
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el8002' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el8002'
         write(irefwr,1) 'jtrans, n, m, ndimlc', &
                          jtrans, n, m, ndimlc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jtrans.eq.1 ) then

!     --- 2D case, at this moment restricted to 7 point extended quadratic
!                  triangle
!                  The centroid will be eliminated, hence only the constant
!                  part of the pressure is taken into account
!         --- Store multiplication factor in sum1

         sum = 0d0
         do k = 1, m
            sum = sum + w(k) * elast(k)/(3d0*(1d0-2d0*anu(k)))
         end do
         diag(1) = 1d0/sum

!        --- Add contribution of penalty matrix

         do i = 1, (n-1)*ndimlc
            do j = 1, (n-1)*ndimlc
               s(j,i) = s(j,i) + div(i,1) * diag(1) * div(j,1)
            end do
         end do

      else

!     --- 2D and 3D case without elimination
!         Compute the diagonal of the mass matrix

         if ( ndimlc.eq.2 ) then
            icent = n
         else
            icent = 14
         end if
         if ( n.eq.4 ) then
            npres = 1
         else
            npres = ndimlc+1
         end if
         do i = 1, npres
            diag(i) = 0d0
         end do
         do k = 1, m
            diag(1) = diag(1) + w(k)/elast(k)*(3d0*(1d0-2d0*anu(k)))
         end do

         do i = 1, npres-1
            do k = 1, m
               diag(i+1) = diag(i+1) + w(k)/ &
                  elast(k)*(3d0*(1d0-2d0*anu(k)))* &
                  (xgauss(k,i)-x(icent,i))**2
            end do
         end do

!        --- store the inverse of diag in diag

         do i = 1, npres
            diag(i) = 1d0 / diag(i)
         end do

!        --- compute the penalty matrix and add to the element matrix

         do i = 1, n*ndimlc
            do j = 1, n*ndimlc
               sum = 0d0
               do k = 1, npres
                  sum = sum + div(i,k) * diag(k) * div(j,k)
               end do
               elemmt(j,i) = s(j,i) + sum
            end do
         end do
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el8002'

      end if  ! ( debug )
      call erclos ( 'el8002' )

      end subroutine el8002
