      subroutine el8251 (v, p, dpsi, work, ncoef, npres)
! ======================================================================
!
!        programmer    Bram Sengers
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 27-02-2002  Changed minus sign
!        version  1.0  date 22-06-2001
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Computes relative fluid velocity for biphase material
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, npres
      double precision  p(*), dpsi(8,m,ndim), work(*), &
                        v(ndim,m)

!     dpsi           i    Global derivatives of linear basis functions
!     ncoef          i    Number of coefficients in the equation
!     npres          i    Number of pressure parameters per element
!     p              i    Pressure in vertices
!     v              o    Fluid velocity in integration points
!     work           i    Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, maxcoef
      parameter (maxcoef=50)
      double precision coef(maxcoef), dpdx(ndim,m), Kperm(ndim,ndim)

!     coef           Coefficients
!     dpdx           Pressure gradient in integration points
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kperm          Permeability tensor
!     maxcoef        Max number of coefficients
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over the integration points

      do k = 1, m

!     --- Compute pressure gradient in 3 directions

         do i = 1, ndim
            dpdx(i,k) = 0d0
            do j = 1, npres
               dpdx(i,k) = dpdx(i,k) + p(j)*dpsi(j,k,i)
            end do
         end do

!        --- Get material parameters in current point

         do i = 1, ncoef
            coef(i) = work(k+m*(i-1))
         end do

!        --- Get permeability tensor

         Kperm(1,1) = coef(20-5)
         Kperm(2,2) = coef(21-5)
         Kperm(3,3) = coef(22-5)

         Kperm(1,2) = coef(23-5)
         Kperm(2,3) = coef(24-5)
         Kperm(3,1) = coef(25-5)

         Kperm(2,1) = coef(23-5)
         Kperm(3,2) = coef(24-5)
         Kperm(1,3) = coef(25-5)

!        --- Compute relative velocity

         do i = 1,ndim
            v(i,k) = 0d0
            do j = 1, ndim
               v(i,k) = v(i,k) - Kperm(i,j)*dpdx(j,k)
            end do
            v(i,k) = v(i,k)/coef(28-5)
         end do

      end do

      end
