      subroutine elmat2mat4 ( se4, se, direc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-10-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform 4th order matrix workr to 2nd order matrix d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision se4(3,3,3,3), se(6,6)
      integer direc

!     direc          i    ?
!     se             i    tangential material stiffness
!     se4            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmat2mat4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmat2mat4'

      end if
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'elmat2mat4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmat2mat4'

      end if
      if (direc==0) then
      se4(1,1,1,1)=se(1,1)
      se4(1,1,1,2)=se(1,4)
      se4(1,1,1,3)=se(1,6)
      se4(1,1,2,1)=se(1,4)
      se4(1,1,2,2)=se(1,2)
      se4(1,1,2,3)=se(1,5)
      se4(1,1,3,1)=se(1,6)
      se4(1,1,3,2)=se(1,5)
      se4(1,1,3,3)=se(1,3)
      se4(1,2,1,1)=se(4,1)
      se4(1,2,1,2)=se(4,4)
      se4(1,2,1,3)=se(4,6)
      se4(1,2,2,1)=se(4,4)
      se4(1,2,2,2)=se(4,2)
      se4(1,2,2,3)=se(4,5)
      se4(1,2,3,1)=se(4,6)
      se4(1,2,3,2)=se(4,5)
      se4(1,2,3,3)=se(4,3)
      se4(1,3,1,1)=se(6,1)
      se4(1,3,1,2)=se(6,4)
      se4(1,3,1,3)=se(6,6)
      se4(1,3,2,1)=se(6,4)
      se4(1,3,2,2)=se(6,2)
      se4(1,3,2,3)=se(6,5)
      se4(1,3,3,1)=se(6,6)
      se4(1,3,3,2)=se(6,5)
      se4(1,3,3,3)=se(6,3)
      se4(2,1,1,1)=se(4,1)
      se4(2,1,1,2)=se(4,4)
      se4(2,1,1,3)=se(4,6)
      se4(2,1,2,1)=se(4,4)
      se4(2,1,2,2)=se(4,2)
      se4(2,1,2,3)=se(4,5)
      se4(2,1,3,1)=se(4,6)
      se4(2,1,3,2)=se(4,5)
      se4(2,1,3,3)=se(4,3)
      se4(2,2,1,1)=se(2,1)
      se4(2,2,1,2)=se(2,4)
      se4(2,2,1,3)=se(2,6)
      se4(2,2,2,1)=se(2,4)
      se4(2,2,2,2)=se(2,2)
      se4(2,2,2,3)=se(2,5)
      se4(2,2,3,1)=se(2,6)
      se4(2,2,3,2)=se(2,5)
      se4(2,2,3,3)=se(2,3)
      se4(2,3,1,1)=se(5,1)
      se4(2,3,1,2)=se(5,4)
      se4(2,3,1,3)=se(5,6)
      se4(2,3,2,1)=se(5,4)
      se4(2,3,2,2)=se(5,2)
      se4(2,3,2,3)=se(5,5)
      se4(2,3,3,1)=se(5,6)
      se4(2,3,3,2)=se(5,5)
      se4(2,3,3,3)=se(5,3)
      se4(3,1,1,1)=se(6,1)
      se4(3,1,1,2)=se(6,4)
      se4(3,1,1,3)=se(6,6)
      se4(3,1,2,1)=se(6,4)
      se4(3,1,2,2)=se(6,2)
      se4(3,1,2,3)=se(6,5)
      se4(3,1,3,1)=se(6,6)
      se4(3,1,3,2)=se(6,5)
      se4(3,1,3,3)=se(6,3)
      se4(3,2,1,1)=se(5,1)
      se4(3,2,1,2)=se(5,4)
      se4(3,2,1,3)=se(5,6)
      se4(3,2,2,1)=se(5,4)
      se4(3,2,2,2)=se(5,2)
      se4(3,2,2,3)=se(5,5)
      se4(3,2,3,1)=se(5,6)
      se4(3,2,3,2)=se(5,5)
      se4(3,2,3,3)=se(5,3)
      se4(3,3,1,1)=se(3,1)
      se4(3,3,1,2)=se(3,4)
      se4(3,3,1,3)=se(3,6)
      se4(3,3,2,1)=se(3,4)
      se4(3,3,2,2)=se(3,2)
      se4(3,3,2,3)=se(3,5)
      se4(3,3,3,1)=se(3,6)
      se4(3,3,3,2)=se(3,5)
      se4(3,3,3,3)=se(3,3)
      else
        se(1,1) = se4(1,1,1,1)
      se(1,2) = se4(1,1,2,2)
      se(1,3) = se4(1,1,3,3)
      se(1,4) = se4(1,1,1,2)
      se(1,5) = se4(1,1,2,3)
      se(1,6) = se4(1,1,3,1)
        se(2,1) = se4(2,2,1,1)
      se(2,2) = se4(2,2,2,2)
      se(2,3) = se4(2,2,3,3)
      se(2,4) = se4(2,2,1,2)
      se(2,5) = se4(2,2,2,3)
      se(2,6) = se4(2,2,3,1)
      se(3,1) = se4(3,3,1,1)
      se(3,2) = se4(3,3,2,2)
      se(3,3) = se4(3,3,3,3)
      se(3,4) = se4(3,3,1,2)
      se(3,5) = se4(3,3,2,3)
      se(3,6) = se4(3,3,3,1)
      se(4,1) = se4(1,2,1,1)
      se(4,2) = se4(1,2,2,2)
      se(4,3) = se4(1,2,3,3)
      se(4,4) = se4(1,2,1,2)
      se(4,5) = se4(1,2,2,3)
      se(4,6) = se4(1,2,3,1)
      se(5,1) = se4(2,3,1,1)
      se(5,2) = se4(2,3,2,2)
      se(5,3) = se4(2,3,3,3)
      se(5,4) = se4(2,3,1,2)
      se(5,5) = se4(2,3,2,3)
      se(5,6) = se4(2,3,3,1)
      se(6,1) = se4(3,1,1,1)
      se(6,2) = se4(3,1,2,2)
      se(6,3) = se4(3,1,3,3)
      se(6,4) = se4(3,1,1,2)
      se(6,5) = se4(3,1,2,3)
      se(6,6) = se4(3,1,3,1)
      end if

      end
