      subroutine elrotstf ( d, q )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  1.1  date 07-07-2005 Roy Kerckhoffs: rewritten in
!                                      terms of 4th order matrix
!        version  1.0  date 28-03-1996
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Rotate 6x6 stiffness matrix, first transform 6x6 matrix to
!       3x3x3x3 matrix, rotate and transform back to 6x6 matrix.
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     rotation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision d(6,6), q(3,3)

!     d         i/o   stiffnessmatrix to rotate
!     q          i    rotation matrix (local to global)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work(3,3,3,3), workr(3,3,3,3), temp
      integer i, j, k, l, m, n, o, p

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     m              Counting variable
!     n              Counting variable
!     o              Counting variable
!     p              Counting variable
!     temp           temporal help variable
!     work           4th order matrix of 2nd order matrix d
!     workr          rotated 4th order matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMAT2MAT4     Transform 4th order matrix workr to 2nd order matrix d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       setup rotation matrix r for stiffness rotation, using q
!       then calculate rotation with:
!         dr = r'.d.r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i=1,3
       do j=1,3
        do k=1,3
         do l=1,3
          workr(i,j,k,l)=0d0
          work(i,j,k,l)=0d0
         end do
        end do
       end do
      end do

!     --- Transform 2nd order matrix d to 4th order matrix work

      call elmat2mat4(work, d, 0)

!     --- The actual rotation

      do i=1,3
       do j=1,3
        do k=1,3
         do l=1,3
          temp=0d0
          do m=1,3
           do n=1,3
            do o=1,3
             do p=1,3
              temp = temp + q(i,m)*q(j,n)*q(k,o)*q(l,p)*work(m,n,o,p)
             end do
            end do
           end do
          end do
          workr(i,j,k,l)=temp
         end do
        end do
       end do
      end do

!     --- Transform 4th order matrix workr to 2nd order matrix d

      call elmat2mat4(workr, d, 1)
      end
