      subroutine elsgcrav ( elemvc, blwork, pksest, w, detf, detfr, &
                          ndim, icount, nnode, nip, nsc )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.0    date   14-11-1997 derived from ELNLAV
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     assemble vector for nonlinear solid analysis in case of Souza
!     method
! **********************************************************************
!
!                       KEYWORDS
!
!    mechanics
!    non_linear
!    souza
!    vector
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer          icount, ndim, nip, nnode, nsc
      double precision elemvc(icount), blwork(nsc,ndim,nnode,nip), &
                       pksest(nsc,nip), w(nip), detf(nip), detfr

!     blwork     i    blmatrix: blwork(i,j,k,l)
!                     i: corresponding stress/strain vector postion
!                     j: direction
!                     k: node
!                     l: integration point
!     detf       i    determinant of f
!     detfr      i    determinant of at centre
!     elemvc     o    Element vector
!     icount     i    number of unknowns per element
!     ndim       i    Dimension of the space.
!     nip        i    number of integration points
!     nnode      i    number of nodes
!     nsc        i    number of stress components
!     pksest     i    stress estimate
!     w          i    weightfactors for numerical integration
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer eln1, di1, t1, ix1, ipn
      double precision tmp1, ct

!     ct
!     di1     dimension counter
!     eln1    element node counter
!     ipn     integration point counter
!     ix1
!     t1      counter
!     tmp1    temporary variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      do eln1=1,nnode

!     --- loop over dimensions

         do di1=1,ndim

!        --- loop over integration points

            tmp1 = 0d0

            do ipn = 1, nip

               ct = (detfr/detf(ipn))**(1d0/3d0)

!              --- sufficient loops to perform right multiplications

               do t1 =1, nsc

!              --- _
!                  --- F = B  * S * (J/J)^(1/3)
!                  L

                  tmp1 = tmp1 + pksest(t1,ipn) &
                       *blwork(t1,di1,eln1,ipn) &
                       *w(ipn)*ct

               end do
            end do

            ix1 = (eln1-1)*ndim + di1
            elemvc( ix1 ) = elemvc(ix1) - tmp1

         end do
      end do

      end
