      subroutine sepel0256
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 256
!     Old subroutine el0256
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     initialization
!     parabolic_equation
!     shell
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     SEPEL0250CF    Fill real coefficients for elements of type 250
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETRULE     Get the integration rule as stored in iuser
!     SEPGETSTRESSRL Read stress-strain relations and store information in E
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1766   Type of integration rule incorrect
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0256' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0256'
         call prinin ( E%iuser, 30, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%iqmat = 2
      B%fillderivn = .true.

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get information of coefficients

      E%ipos = E%ipos+1          ! skip first position
      E%ishift = 1

!     --- Get information of coefficients

      call sepgetstressrl  ! stress-strain relation
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'ipos, ishift/2', E%ipos, E%ishift

!     --- make default integration rule 2-pt Gauss

      if ( B%rule==0 ) B%rule = 3
      B%cart = 1

      E%iload = inirepscal ( E%iuser(E%ipos) )
      if ( E%iload<0 .or. E%iload>1 ) then
         call errint ( E%iload, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      E%ipos = E%ipos+2          ! skip one position
      E%ishift = E%ishift+2
      if ( debug ) write(irefwr,1) 'ipos, ishift/3', E%ipos, E%ishift

!     --- Get information of basis functions

      call sepelbasefnch
      if ( ierror/=0 ) go to 1000
      n = B%n
      m = B%m

      if ( E%ishape/=5 ) then

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( E%ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if  ! ( E%ishape/=5 )

!     --- Allocate D-matrix

      allocate ( E%Dmat(5,5,m), E%bsh(5,35,n,m,2), E%bs(2,3,n,m) )
      E%Dmat = 0d0
      E%bsh = 0d0
      E%bs = 0d0

!     --- Allocate special arrays for shell

      allocate ( B%qfib(3,3,4), B%xhat(4,3), B%fibjac(4,2), E%subvec(B%n) )

!     --- Fill all real coefficients

      E%ncoef = 38
      call sepel0250cf

      E%fillbmat = matrix .or. E%initial_strain .or. E%initial_stress

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0256'

      end if  ! ( debug )
      call erclos ( 'sepel0256' )

      end subroutine sepel0256
