      subroutine sepelmkbl ( nip )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  2.1  date 14-09-2005 Roy Kerckhoffs, changed
!                                      blwork(6,3,8,*) into (6,3,B%n,nip)
!        version  2.0  date 21-01-1998 Marcel Rutten, added 2D axisymmetry:
!                                      call and  declaration of variables
!                                      adapted
!        version  1.2  date 21-01-1997 changed inner loop: -O3 optimize
!                                      did not work on all machines
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Setup the linear strain-displacement matrix for nonlinear analysis.
!   The matrix includes current deformation effects.
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
!     strain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          nip

!     nip        i    number of integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision, pointer :: bl(:,:,:,:), dfgrad(:,:,:), &
                                   dphidx(:,:,:), phi(:,:), &
                                   xgauss(:), ugauss(:)

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      bl => E%blloc
      dfgrad => E%dfgrad
      dphidx => E%dphidx
      phi => B%phi
      xgauss => E%xgaussloc(:,1)
      ugauss => E%ugaussloc(:,1)
      bl = 0d0
      do l = 1,nip
         do k = 1,B%n
            do j=1,E%ndim

!           --- The first half of Bl is the same for all types of coordinate
!               systems:

               do i=1,E%ndim
                  bl(i,j,k,l)= dfgrad(l,i,j) * dphidx(k,l,i)

               end do

!              --- The second half of Bl is coordinate system dependent:

               if ( E%ndim==3 ) then

!              --- 3D Cartesian

                  bl(4,j,k,l)= dfgrad(l,1,j)* dphidx(k,l,2) &
                              +dfgrad(l,2,j)* dphidx(k,l,1)
                  bl(5,j,k,l)= dfgrad(l,2,j)* dphidx(k,l,3) &
                              +dfgrad(l,3,j)* dphidx(k,l,2)
                  bl(6,j,k,l)= dfgrad(l,3,j)* dphidx(k,l,1) &
                              +dfgrad(l,1,j)* dphidx(k,l,3)

               else if ( E%ndim==2 ) then

!              --- 2D, all coordinate systems:

                     bl(4,j,k,l)= dfgrad(l,1,j)* dphidx(k,l,2) &
                                 +dfgrad(l,2,j)* dphidx(k,l,1)

               end if
            end do

!           --- Fill circumferential part of Bl in case of axisymmetry:

            if ( B%cart==2 ) then

               if (nip==1 .and. B%rule/=2) then

!              --- Looks like we have reduced integration in a rectangular
!                  --- element
!                  element on our hands: phi_r = 0.25d0
!                  ( see elp621.f for details )

                 bl(3,1,k,l) = (0.25d0/xgauss(l)) * &
                               (1d0 + ugauss(l)/xgauss(l))
               else

                 bl(3,1,k,l) = (phi(k,l)/xgauss(l)) * &
                               (1d0 + ugauss(l)/xgauss(l))
               end if
            end if

         end do
      end do
      end
