      subroutine sepelmplate
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for plate problems
!     Two-dimensional elements
!     Old subroutine elm255
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     parabolic_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision, pointer :: s(:,:)
      double precision, target :: f(m)

!     f              contains the diagonal part due to contribution of df1/du
!                    term
!     i              Counting variable
!     s              Element matrix to be filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELLOAD1     Fill the element vector with a load
!     SEPELMPLATE1   Compute bending part element matrix for linear elasticity
!                    element (Plate element)
!     SEPELMPLATE2   Compute shear part element matrix for linear elasticity
!                    element (Plate element)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmplate' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmplate'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill element matrix
!         First the bending part; the weight is corrected by h^3/12

      call sepelmplate1

!     --- Next the shear part; the weight is corrected by h

      call sepelmplate2

!     --- Next compute contribution due to the distributed load

      if ( E%ind(39)/=0 ) then

!     --- df1/du # 0 store contribution temporary in array work

         s => E%mat(1:3*n:3,1:3*n:3)
         E%subvec => f
         call sepelload1 ( E%coeffs(:,39), B%jdiag, B%phi )

         do i = 1, n
            s(i,i) = s(i,i) - f(i)
         end do  ! i = 1, n

      end if  ! ( E%ind(39)/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmplate'

      end if  ! ( debug )
      call erclos ( 'sepelmplate' )

      end subroutine sepelmplate
