      subroutine sepelp105
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of extra basis functions for shell element
!     Bi-linear quadrilateral (4 nodes)
!     Old routines: el8127 and el8124
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m1
      double precision :: lambda(2), weight(2)
      double precision, pointer :: x(:,:), qfib(:,:,:)

!     lambda         Array containing the linear basis functions in a point
!     m1             number of quadrature points (1D)
!     qfib           Contains the fiber coordinate system
!     weight         weights in 1D
!     x              co-ordinates of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELPFIBJAC   Compute the Jacobian for the integration of the
!                    shell element
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
!     SEPELPQFIB     Compute the fiber coordinate system
!     SEPELPXHAT     Compute the normal vector in the nodes
!     SEPFILLBMATSH  Fill the B-matrix for the shell element
!                    This matrix contains all curvature effects
!     SEPFILLBSMAT   Fill matrix B^s for plate problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp105' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp105'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) then

!        --- Compute zeta and weights in the zeta direction
!            This is the fiber direction

         m1 = 2
         call sepelplambda1d ( lambda, 'Bilinear quadrilateral', 3, &
                               m1, weight )
         B%zeta(1:2) = -1d0+lambda(1:2)*2d0
         B%wzeta(1:2) = 1d0

      end if  ! ( E%first==0 )
      if ( ierror/=0 ) go to 1000

!     --- Define X^_a, i.e. the normal vector in the nodes
!         At this moment it is defined locally, but this should be defined
!         globally in the future

      call sepelpxhat

!     --- Define the fiber coordinate system and store in matrix Qfib

      call  sepelpqfib

!     --- Fill Jacobian

      call  sepelpfibjac

!     --- Transform the coordinates of the vertices to the fiber system in the
!         shell

      x => B%xyz
      qfib => B%qfib
      x(:,1) = qfib(1,1,:)*x(:,1)+qfib(1,2,:)*x(:,2)+qfib(1,3,:)*x(:,3)
      x(:,2) = qfib(2,1,:)*x(:,1)+qfib(2,2,:)*x(:,2)+qfib(2,3,:)*x(:,3)

      if ( debug ) call prinrl1 ( x, 2, 4, 'x' )

!     --- Fill matrix B phi i

      call sepfillbsmat
      call sepfillbmatsh

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp105'

      end if  ! ( debug )
      call erclos ( 'sepelp105' )

      end subroutine sepelp105
