      subroutine sepelpxhat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal vector in the nodes
!     Old subroutine el8123
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k
      double precision :: e1(3), e2(3), e3(3), norm1, norm2
      double precision, pointer :: x(:,:)

!     e1               vector e_1 (see pseudo code)
!     e2               vector e_2 (see pseudo code)
!     e3               help vector
!     j                Counting variable
!     k                Counting variable
!     norm1            Inverse of norm of vector
!     norm2            Inverse of norm of vector
!     x                co-ordinates of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute cross product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The normal vector is defined as follows
!
!      e_1 = x - x-
!      e_2 = x+ - x
!      n = e_1 x e_2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpxhat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpxhat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      x => B%xyz

!     --- Define X^_a, i.e. the normal vector in the nodes
!         At this moment it is defined locally, but this should be defined
!         globally in the future

      do k = 1, 4
         do j = 1, 3
            e1(j) = x(k,j) - x(icyc4m(k),j)
            e2(j) = x(icyc4p(k),j) - x(k,j)
         end do
         norm1 = 1d0/sqrt(e1(1)**2+e1(2)**2+e1(3)**2)
         norm2 = 1d0/sqrt(e2(1)**2+e2(2)**2+e2(3)**2)
         do j = 1, 3
            e1(j) = e1(j)*norm1
            e2(j) = e2(j)*norm2
         end do
         call eloutr ( e1, e2, e3 )
         norm1 = 1d0/sqrt(e3(1)**2+e3(2)**2+e3(3)**2)
         do j = 1, 3
            B%xhat(k,j) = e3(j)*norm1
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%xhat, 3, 4, 'xhat' )
         write(irefwr,*) 'End sepelpxhat'

      end if  ! ( debug )
      call erclos ( 'sepelpxhat' )

      end subroutine sepelpxhat
