      subroutine sepelri01
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.2    date   21-01-1998 Marcel Rutten,extension to axisymmetry
!        version 1.1    date   12-11-1997 added detf0 computation
!        version 1.0    date   28-01-1997
!
!
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   This routine computes the deformation gradient in the center of
!   the element. For linear 3D elements (tetraeders m=4 or bricks m=8 )
!   this is done by averaging.
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     solid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer t1, t2, t3
      double precision rnip
      double precision, pointer :: fr(:,:), detf0, w(:), ugaussr(:), &
                                   ugauss(:,:), xgauss(:,:), dphdx0(:,:), &
                                   xgaussr(:)

!     rnip       double precision of n
!     t1         counter
!     t2         counter
!     t3         counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      rnip = dble( B%m )

!     --- reset fr

      fr => E%gradred(1,:,:)
      fr = 0d0
      detf0 => E%detfr(1)
      dphdx0 => E%detf0(:,1,:)
      dphdx0 = 0
      w => B%w
      ugaussr => E%ugaussr(1:2,1)
      ugauss => E%ugauss
      xgauss => B%xyzg
      xgaussr => E%xgaussr(1:2,1)

!     --- reset E%wr

      E%wr = 0d0
      ugaussr = 0d0
      xgaussr = 0d0

!     --- do the averaging

      do t1 = 1,B%m

!     --- 1: weights:

         E%wr = E%wr + w(t1)

!        --- 2: deformation gradient:

        do  t2 = 1,E%ndim
            do t3 = 1,E%ndim
               fr(t2,t3) = fr(t2,t3) + gradoldvec(t1,t2,t3)/rnip
            end do
        end do

!       --- 3: derivatives of basis functions:

         do t2 = 1,E%ndim
            do t3 = 1,B%n
               dphdx0(t3,t2) = dphdx0(t3,t2) + B%phider(t3,t1,t2)/rnip
            end do
         end do

!        --- 4:  Gauss points:

          xgaussr(1) = xgaussr(1) + (1d0/rnip) * xgauss(t1,1)
          xgaussr(2) = xgaussr(2) + (1d0/rnip) * xgauss(t1,2)

!         --- 5:  Solution in Gauss points:

          ugaussr(1) = ugaussr(1) + (1d0/rnip) * ugauss(t1,1)
          ugaussr(2) = ugaussr(2) + (1d0/rnip) * ugauss(t1,2)

      end do

!     --- determinant of deformation in RI point:

      if ( E%ndim.eq.3 ) then

        detf0 = fr(1,1)*( fr(2,2)*fr(3,3) - fr(2,3)*fr(3,2) ) + &
                fr(1,2)*( fr(2,1)*fr(3,3) - fr(2,3)*fr(3,1) ) + &
                fr(1,3)*( fr(2,1)*fr(3,2) - fr(2,2)*fr(3,1) )

      else if ( E%ndim.eq.2 ) then

        detf0 = fr(1,1)*fr(2,2) - fr(1,2)*fr(2,1)

        if ( B%cart.eq.2 ) then
          detf0 = detf0 * ( 1d0 + ugaussr(1)/xgaussr(1))
        end if

      end if
      end
