      subroutine sepelvshell
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector for shell element
!     Old subroutine elm256
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELLOAD1     Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvshell' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvshell'
         write(irefwr,1) 'iload', E%iload
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%subvec = 0d0

      if ( E%iload==0 .and. E%ind(23)/=0) then

!     --- load in normal direction

         call sepelload1 ( E%coeffs(:,23), B%jdiag, B%phi )

         do i = 1, n
            do j = 1, 3
               E%vec(5*(i-1)+j) = E%subvec(i)*B%xhat(i,j)
            end do
         end do

      else if ( E%iload==1 ) then

!     --- Load in Cartesian direction

         do j = 1, 3

            if ( E%ind(22+j)/=0 ) then

!           --- fj # 0  store contribution temporary in array work

               call sepelload1 ( E%coeffs(:,22+j), B%jdiag, B%phi )
               do i = 1, n
                  E%vec(5*(i-1)+j) = E%subvec(i)
               end do

            end if

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelvshell'

      end if  ! ( debug )
      call erclos ( 'sepelvshell' )

      end subroutine sepelvshell
