      subroutine creatv01 ( ibuffr, buffer, kmesh, kprob, isolut,
     +                      inpcre, rincre, islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.12 date 07-07-2011 Extra debug
!        version  1.11 date 27-09-2010 Make work arrays local
!        version  1.10 date 13-05-2010 Suppress printing during time steps
!
!   copyright (c) 2003-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a SEPRAN real or complex array (vector)
!     A part of the array is filled by creatv01
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons2'
      include 'SPcommon/cbuffr'
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cinout'
      include 'SPcommon/cconst'
      include 'SPcommon/celp'
      include 'SPcommon/csepcm'
      include 'SPcommon/csepint'
      include 'SPcommon/cpost5'
      include 'SPcommon/ctimen'
      include 'SPcommon/clinsl'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), isolut(5,*), inpcre(*), ibuffr(*),
     +        islold(5,*)
      double precision rincre(*), buffer(*)

!     buffer i/o   Real buffer array in which all large data is stored
!     ibuffr i/o   Integer buffer array in which all large data is stored
!     inpcre  i    Array containing the integer input
!                  In this array it is indicated how the arrays must be created
!                  and filled
!                  If INPCRE has been given by the user must be filled as
!                  follows:
!                  Pos 1: number of entries filled by the user
!                  Pos 2: number of different vectors in isolut
!                  Per vector:
!                  Pos 1:  ICHVC
!                      2:  IPROB
!                      3:  IVEC
!                      4:  ICOMPL
!                      5:  IFILL
!                  If IFILL>0, followed by extra information as described in
!                  the programmer's guide
!                  If INPCRE is filled by subroutine CREATE or PRESDF Pos. 1
!                  has a different meaning:
!                  Pos. 1 = -isubr-100*(ivecsq-1) with
!                  isubr sequence number of caling subroutine:
!                      1:  CREATE
!                      2:  PRESDF
!                      3:  CREAVC
!                      4:  PRESTM
!                      5:  ERCRET
!                      6:  ERPRES
!                  ivecsq sequence number of first vector
!                  At output inpcre(1) gets the sequence number of the last
!                  vector filled
!                  From position 2 as if filled by the user
!     islold  i    User input array in which the user puts information
!                  of all preceding solutions
!                  Integer array of length 5 x numold
!     isolut i/o   Array(s) to be filled
!     kmesh   i    Standard SEPRAN array containing information of the mesh
!     kprob   i    Standard SEPRAN array containing information of the problem
!                  definition. See the Programmer's Guide 13.2 for a complete
!                  description
!     rincre  i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifirst, npoint, nv, n, ichvc, iprob, ivec, icompl, ivecsq,
     +        ifill, ipstin, kvec, nprob, itypvc, jcompl, jvec, jprob,
     +        ipcoor, ipkmsc, ipkmsl, ipkmsm, ipkmsn, ipvect, k,
     +        ndim, kmesht, indprf, indprh, indprp, nphys, ipnode,
     +        ipwork, ipkprf, ipkprh, ipkprp, jpstin, kdeg, kfun, kfill,
     +        ipkprb, ndtvec, isubr, nusol, nbound, iinfor, npboun, ipc,
     +        ipcint, ipicrvob, ipkmsh, nbufol, istartvc, ipkprv,
     +        maxphys, istore(4,10), mvec, ipuoldcp, ipucopy, i, nunkp,
     +        ipcoorcp, ipiwork, ipos, nelem, length, jdegfd, nch,
     +        npelm, ipipointobst, kelmad, ipkmsq, ipouter,
     +        ipkmss, ipkprfold, nphysold, nunkpold,
     +        ipvectold, ivecold, indprfold, indprpold, ifirstwrit
      logical new, bndcon, obstaclefound, debug
      character (len=80) filename

!     bndcon         If true boundary conditions must be filled, otherwise
!                    a vector must be created
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     filename       Name of file
!     i              Counting variable
!     ichvc          Indication of the type of vector to be created.
!                    Possible values
!                    0: Vector either exists or is a solution vector
!                    1: Vector is a solution vector
!                    2: Vector is an array of special structure
!                    3: Vector is an array of special structure defined per
!                       element
!                    4: Vector is an array of special structure defined per node
!                       per element
!     icompl         Indication if the vector to be created is real (0) or
!                    complex
!     ifill          Indicates how the vector must be filled.
!                    Possible values:
!                    -1  The corresponding vector is not filled,
!                        only created
!                     0  The corresponding vector is created and made
!                        equal to zero
!                    m>0 The filling of the vector is defined by m commands
!     ifirst         Value of first position of array INPCRE
!     ifirstwrit     If 0 output of information allowed,
!                    if 1, this ouput is suppressed
!     iinfor         Maximum number of vectors stored in carray
!     indprf         Indication if KPROB part f has been filled (>0) or not (=0)
!     indprfold      Indication if KPROB part f has been filled (>0) or not (=0)
!                    This one refers to old vector
!     indprh         Indication if KPROB part h has been filled (>0) or not (=0)
!     indprp         Indication if KPROB part p has been filled (>0) or not (=0)
!     indprpold      Indication if KPROB part p has been filled (>0) or not (=0)
!                    This one refers to old vector
!     ipc            Starting address of array containing array C for "right"
!                    periodical boundary conditions
!     ipcint         Starting address of integer part of array C
!     ipcoor         Starting address of co-ordinates
!     ipcoorcp       starting address of copy of co-ordinates array
!     ipicrvob       Starting address of array ICURVOBS
!     ipipointobst   Starting address of array ipointobst
!     ipiwork        Starting address of array iwork
!     ipkmsc         Starting address of array KMESH part c
!     ipkmsh         Starting address of array KMESH part h
!     ipkmsl         Starting address of array KMESH part l
!     ipkmsm         Starting address of array KMESH part m
!     ipkmsn         Starting address of array KMESH part n
!     ipkmsq         Starting address of array KMESH part q in array
!                    IBUFFR
!     ipkmss         Starting address of array KMESH part s in array
!                    IBUFFR
!     ipkprb         Starting address of array KPROB for actual problem number-1
!     ipkprf         Starting address of kprob part f
!     ipkprfold      Starting address of kprob part f vor old vector
!     ipkprh         Starting address of kprob part h
!     ipkprp         Starting address of kprob part p
!     ipkprv         Starting address of kprob part v
!     ipnode         Starting address of array NODES
!     ipos           Position in an array
!     ipouter        Starting address of array outerboun
!     iprob          Problem number
!     ipstin         Starting address in array INPCRE
!     ipucopy        Starting address of array ucopy
!     ipuoldcp       Starting address of array uoldcp
!     ipvect         Starting address of array VECTOR(kvec)
!     ipvectold      Starting address of array VECTOR (old one)
!     ipwork         Starting address of array WORK
!     istartvc       Starting position of sequence numbers of vectors in inpcre
!     istore         Array containing information over the old solution vectors
!                    Contents
!                    1,i:  indprf
!                    2,i:  indprh
!                    3,i:  indprp
!                    4,i:  nphys
!     isubr          Indicates by what subroutine creatv01 has been called.
!                    Possible values:
!                    1:  By the user or CREATE
!                    2:  By PRESDF or PRESTM
!     itypvc         Type number of solution vector (110=solution vector,
!                    115=vector of special structure)
!     ivec           Gives the sequence number of the array of special
!                    structure if
!                    ICHVC = 2, or the number of degrees of freedom per element
!                    if
!                    ICHVC = 3 or the number of degrees of freedom per node if
!                    ICHVC = 4
!     ivecold        ivec for old vector
!     ivecsq         Sequence number of vector in isolut
!     jcompl         Value of icompl as stored in isolut at input
!     jdegfd         Number of degrees of freedom per point
!     jprob          Value of iprob as stored in isolut at input
!     jpstin         Help variable to compute new value of ipstin partly
!     jvec           Value of ivec as stored in isolut at input
!     k              General loop variable
!     kdeg           Indication which degrees of freedom must be filled
!                    Possibilities:
!                    0:    All degrees of freedom
!                    >0:   Only the KDEGth degree of freedom
!                    <0:   The -KDEG degrees of freedom stored in array IARDEG
!                          must be filled
!     kelmad         memory management reference number of array KMESH part ad
!     kfill          Indication how the nodal points must be filled into
!                    array NODES
!                    Possible values:
!                     0:  The nodes are filled in all points
!                     1:  The nodes are filled in n user points
!                     2:  The nodes are filled in the curves C1 to C2
!                     3:  The nodes are filled in the surfaces S1 to S2
!                     5:  The nodes are filled in the elements i to j
!                     6:  The nodes are filled in the element group ielgrp
!                     7:  The nodes are filled in the volumes V1 to V2
!                     8:  The nodes are filled in the user points P1 to P2
!                     9:  The nodes are filled in the active points of
!                         obstacle i
!                    10:  The nodes are filled in the active points of
!                         contact i
!                    11:  The nodes are filled in the non-active points
!                         of contact i
!                    12:  The nodes are filled using a file with node
!                         numbers
!                    13:  The nodes are filled using a file with elements
!                    14:  The nodes are filled using a file with capacities
!                    15:  The global unknowns are filled for global group
!                         iglgrp
!                    16:  All nodes in the obstacle are filled
!                    17:  Only the inner nodes in the obstacle are filled
!                    18:  Only the boundary nodes in the obstacle are filled
!                    19:  Only the boundary nodes on the obstacle are filled
!                    20:  Only the boundary nodes on the boundary with
!                         zero level set are filled
!     kfun           Indication of how the degrees of freedom must be filled.
!                    Possibilities:
!                     0:  Not filled
!                    <0:  Constant value stored in RINCRE(-KFUN)
!                    >0:  Function subroutine
!     kmesht         Starting address of KMESH part t in KMESH
!     kvec           Loop variable over the vectors
!     length         Estimated length of an array
!     maxphys        Maximum number of unknowns per point
!     mvec           Number of old vectors to be used in FUNCVECT
!     n              Number of commands
!     nbound         Number of essential boundary conditions
!     nbufol         help variable to store the old value of nbuffr
!     nch            Number of characters in a string
!     ndim           Dimension of the space
!     ndtvec         Number of vectors of special structure (type 115)
!     nelem          Number of elements
!     new            Indication if the vector to be created is new (true) or old
!                    (false)
!     npboun         Number of periodical boundary conditions
!     npelm          Number of points per element
!     nphys          Number of physical unknowns, or maximal degrees of freedom
!                    per point
!     nphysold       Number of physical unknowns, or maximal degrees of freedom
!                    per point for old vector
!     npoint         Number of nodal points in the mesh
!     nprob          Number of problems stored in array KPROB
!     nunkp          Number of unknowns per point
!     nunkpold       Number of unknowns per point for old vector
!     nusol          Number of unknowns
!     nv             Number of vectors to be created
!     obstaclefound  If true values must be given in points in an obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigettext, inimvc, inidgttext, iniprb, inidbl

!     CREATV116      Fill arrays of type 116 or 126
!     CREATV801      Fill arrays of type 801
!     CREATVSPEC     Fill array usol with values in the case that 1000<fkun<2000
!     ER0023         Fill nodal points where the vector must be filled in array
!                    NODES
!     ER0048         Fill values in the vector in the points stored in array
!                    NODES  real case
!     ER0069         Fill values in the vector in the points stored in array
!                    NODES  complex case
!     ER0090         Fill nodal values from a file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPCOR       Copy coordinates for a set of nodes
!     ERCREATGLOB    Fill part global unknowns of vector
!     ERCREATQUAD    Fill boundary conditions that are defined by a quadratic
!                    polynomial
!     ERFILCRV       Fill array vector in the nodal points stored in array NODES
!     ERFILLCOPVEC   Copy the contents of array uold into array unew for
!                    all nodes stored in array nodes
!     ERFILLVC       Copy the contents of array vector into array uoldcp for all
!                    points in array nodes and all degrees of freedom
!     ERGETIFILL     Check if values must be given in obstacles
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     FUNCVECT       User subroutine to fill a vector as function of old vectors
!     INI006         clear real space
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI061         compute pointers to degrees of freedom of vectors
!     INI062         reserve space for temporary arrays
!     INI068         compute pointers to degrees of freedom of vectors
!     INI069         Reserve space at the end of IBUFFR for a semi-permanent
!                    array
!     INI086         compute pointers to degrees of freedom of vectors
!     INIACTMK       General memory management subroutine to activate or create
!                    a permanent array
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INIMVC         Get value of iinfor from common carray
!     INIPRB         Get value of ipkprb as function of iprob
!     INIRST         Reset BUFFER to old value after a call to ini079
!     MSHFILLKMESHA3 Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     MSHFILLKMESHAD Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^2 only)
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRSPRCRE       Fill part of USOL corresponding to "right" periodical
!                    boundary conditions (complex case)
!     PRSPRRRE       Fill part of USOL corresponding to "right" periodical
!                    boundary conditions ( real case )
!     TOBVALZC       Clear part of usol corresponding to boundary conditions
!                    (comp)
!     TOBVALZR       Clear part of usol corresponding to boundary conditions
!                    (real)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      75   Array KMESH has not been filled
!     206   Input vector with illegal sequence number ivec
!     699   Problem number incorrect
!    1053   mvec out of range
!    1113   First position of INPCRE has wrong value
!    1114   Parameter ICHVC indicating the type of vector to be created out
!           of range
!    1126   Number of vectors to be created out of range ( <1 or>10000 )
!    1127   The parameter ICOMPL has wrong value
!    1675   Wrong value of reference to texts
!    1676   kdeg=0 and inunkp>1
!    1677   Complex vectors have not yet been implemented
!    2208   There are no obstacles, whereas essential boundary conditions
!           for the obstacle are required
!    2400   Addition of sequence numbers may cause unwanted effects
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check input
!     Compute starting addresses of INPCRE, RINCRE
!     Compute nv
!     For kvec := 1 (1) nv
!         Compute ICHVC, IPROB, IVEC, ICOMPL, IFILL
!         Create vector or compute starting addresses
!         If ( IFILL=0 ) then
!            Set vector equal to 0
!         else if ( IFILL>0 ) then
!            For k := 1 (1) IFILL  do
!               Select type of creation
!               Get necessary vectors in memory
!               Fill vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ifirstwrit/0/
! ======================================================================
!
      call eropen ( 'creatv01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv01'
         call prinin ( inpcre, 25, 'inpcre' )
         call prinrl ( rincre, 1, 'rincre' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check input

      iinfor = inimvc ( )
      ifirst = inpcre(1)
      ipstin = 2
      if ( ifirst<-10000 .or. ifirst>=10000 ) then
         call errint ( ifirst, 1 )
         call errsub ( 1113, 1, 0, 0 )
      end if
      if ( ifirst<0 ) then

!     --- ifirst<0

         isubr = - ifirst
         ivecsq = isubr/100
         isubr = mod(isubr,100)
         ifirst = 1000000

      else

!     --- ifirst>0

         isubr = 1
         ivecsq = 0

      end if
      bndcon = isubr==2 .or. isubr==4 .or. isubr==6
      if ( debug )
     +   write(irefwr,1) 'isubr, ivecsq, ifirst', isubr, ivecsq, ifirst
      if ( kmesh(2)/=100 ) then
         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0 , 0 )
      end if
      if ( kprob(2)/=101 ) then
         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0 , 0 )
      end if

!     --- Compute nv

      npelm  = kmesh(4)
      ndim   = kmesh(6)
      npoint = kmesh(8)
      nelem  = kmesh(9)
      nprob  = kprob(40)
      if ( ifirst>=ipstin ) then
         nv = inpcre(ipstin)
      else
         nv = 1
      end if
      if ( nv<=0 .or. nv>10000 ) then
         call errint ( nv, 1 )
         call errsub ( 1126, 1, 0, 0 )
      end if
      ipouter = 1
      maxphys = 0
      mvec = 0

      if ( debug )
     +   write(irefwr,1) 'npelm, ndim, npoint, nelem, nprob, nv',
     +                    npelm, ndim, npoint, nelem, nprob, nv

!     --- Get relevant vectors into memory

      call ini060text ( ibuffr, -kmesh(17), 'kmeshc' )
      call ini060text ( ibuffr, -kmesh(23), 'coor' )
      if ( kmesh(26)/=0 )
     +    call ini060text ( ibuffr, -kmesh(26), 'kmeshl' )
      if ( kmesh(27)/=0 )
     +   call ini060text ( ibuffr, -kmesh(27), 'kmeshm' )
      if ( kmesh(28)/=0 )
     +   call ini060text ( ibuffr, -kmesh(28), 'kmeshn' )
      if ( kmesh(31)/=0 )
     +   call ini060text ( ibuffr, -kmesh(31), 'kmeshq' )
      if ( kmesh(33)/=0 )
     +   call ini060text ( ibuffr, -kmesh(33), 'kmeshs' )

!     --- create or activate kmesh part ah with outer boundary

      call iniactmk ( ibuffr, 1, 58, kmesh, kmesh, ibuffr )

      kmesht = max ( kmesh(36), 1 )
      if ( kmesh(22)>0 .and. kmesh(42)>0 ) then

!     --- Possibly obstacles

         call ini060text ( ibuffr, -kmesh(22), 'kmeshh' )
         ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
         if ( ibuffr(ipkmsh)>0 ) then
            call ini060text ( ibuffr, ibuffr(ipkmsh), 'icurvsobst' )
         end if

      else

         ipkmsh = 1

      end if

!     --- Loop over nv vectors

      do kvec = 1+ivecsq, nv+ivecsq

!     --- Compute ICHVC, IPROB, IVEC, ICOMPL and IFILL

         ichvc  = 0
         iprob  = 1
         ivec   = 1
         icompl = 0
         ifill  = 0
         if ( ifirst>=ipstin+1 ) ichvc  = inpcre(ipstin+1)
         if ( ifirst>=ipstin+2 ) iprob  = inpcre(ipstin+2)
         if ( ifirst>=ipstin+3 ) ivec   = inpcre(ipstin+3)
         if ( ifirst>=ipstin+4 ) icompl = inpcre(ipstin+4)
         if ( ifirst>=ipstin+5 ) ifill  = inpcre(ipstin+5)
         if ( debug )
     +      write(irefwr,1) 'ipstin, ichvc, iprob, ivec, icompl, ifill',
     +                       ipstin, ichvc, iprob, ivec, icompl, ifill
         ipstin = ipstin + 5
         if ( ichvc<0 .or. ichvc>5 ) then
            call errint ( ichvc, 1 )
            call errsub ( 1114, 1, 0, 0 )
         end if
         if ( ivecsq>0 .and. nv>1 .and. ifill==-1 ) then

!        --- Possibly wrong vector to be filled

            nch = index(names(kvec),' ')-1
            if ( nch<=0 ) nch = len(names(kvec))
            call errchr ( names(kvec)(1:nch), 1 )
            call errint ( kvec, 1 )
            nch = index(names(ivecsq+1),' ')-1
            if ( nch<=0 ) nch = len(names(ivecsq+1))
            call errchr ( names(ivecsq+1)(1:nch), 2 )
            call errint ( ivecsq+1, 2 )
            call errint ( nv, 3 )
            call errwar ( 2400, 3, 0, 2 )

         end if

         itypvc = isolut(2,kvec)
         ipkprb = iniprb ( iprob, kprob )

!        --- Determine type of vector to be created

         if ( ichvc==0 ) then

!        --- ichvc = 0, Either old vector or solution vector

            if ( itypvc/=110 .and. itypvc/=115 .and.
     +           itypvc/=116 .and. itypvc/=126 ) then

!           --- New vector

               itypvc = 110
               new    = .true.

            else if ( isolut(1,kvec)<0 .or.
     +                isolut(1,kvec)>iinfor ) then

!           --- New vector

               itypvc = 110
               new    = .true.

            else

!           --- Old vector

               new = .false.

            end if

         else if ( ichvc==1 ) then

!        --- ichvc = 1, solution vector

            if ( itypvc/=110 .or. isolut(1,kvec)<0 .or.
     +           isolut(1,kvec)>iinfor )  then

!           --- New vector

               itypvc = 110
               new    = .true.

            else

!           --- Possibly old vector

               jprob  = isolut(4,kvec)/1000 + 1
               jcompl = mod(isolut(3,kvec),10)
               if ( iprob/=jprob .or. icompl/=jcompl ) then
                  new = .true.
               else if ( kprob(ipkprb+5)/=isolut(5,kvec) ) then
                  new = .true.
               else
                  new = .false.
               end if

            end if

         else if ( ichvc==2 ) then

!        --- ichvc = 2, vector of special structure

            if ( itypvc/=115 .or. isolut(1,kvec)<0 .or.
     +           isolut(1,kvec)>iinfor )  then

!           --- New vector

               itypvc = 115
               new    = .true.

            else

!           --- Possibly old vector

               jprob  = isolut(4,kvec)/1000 + 1
               jcompl = mod(isolut(3,kvec),10)
               jvec   = isolut(4,kvec) - 1000 * (jprob-1)
               if ( iprob/=jprob .or. icompl/=jcompl .or.
     +              jvec/=ivec ) then
                  new = .true.
               else
                  new = .false.
               end if

            end if

         else if ( ichvc==3 ) then

!        --- ichvc = 3, vector of special structure per element

            if ( itypvc/=116 .or. isolut(1,kvec)<0 .or.
     +           isolut(1,kvec)>iinfor )  then

!           --- New vector

               itypvc = 116
               new = .true.

            else

!           --- Possibly old vector

               jprob  = isolut(4,kvec)/1000 + 1
               jcompl = mod(isolut(3,kvec),10)
               jvec   = isolut(4,kvec) - 1000 * (jprob-1)
               if ( iprob/=jprob .or. icompl/=jcompl .or.
     +              jvec/=ivec ) then
                  new = .true.
               else
                  new = .false.
               end if

            end if

         else if ( ichvc==4 ) then

!        --- ichvc = 4, vector of special structure per element per node

            if ( itypvc/=126 .or. isolut(1,kvec)<0 .or.
     +           isolut(1,kvec)>iinfor )  then

!           --- New vector

               itypvc = 126
               new = .true.

            else

!           --- Possibly old vector

               jprob  = isolut(4,kvec)/1000 + 1
               jcompl = mod(isolut(3,kvec),10)
               jvec   = isolut(4,kvec) - 1000 * (jprob-1)
               if ( iprob/=jprob .or. icompl/=jcompl .or.
     +              jvec/=ivec ) then
                  new = .true.
               else
                  new = .false.
               end if

            end if

         else if ( ichvc==5 ) then

!        --- ichvc = 5, vector with capacities

            if ( itypvc/=801 .or. isolut(1,kvec)<0 .or.
     +           isolut(1,kvec)>iinfor )  then

!           --- New vector

               itypvc = 801
               nphys = ivec/1000+1
               ivec = mod(ivec,1000)
               new = .true.

            else

!           --- Possibly old vector

               jprob  = isolut(4,kvec)/1000 + 1
               jcompl = mod(isolut(3,kvec),10)
               jvec   = isolut(4,kvec) - 1000 * (jprob-1)
               nphys = ivec/1000+1
               ivec   = mod(ivec,1000)
               jdegfd = isolut(5,kvec)/jvec
               if ( iprob/=jprob .or. icompl/=jcompl .or.
     +              jvec/=ivec .or. jdegfd/=nphys ) then
                  new = .true.
               else
                  new = .false.
               end if

            end if

         end if
         if ( .not. new ) then

!        --- old vector, compute iprob, ivec and icompl

            iprob  = isolut(4,kvec)/1000 + 1
            icompl = mod(isolut(3,kvec),10)
            ivec   = isolut(4,kvec) - 1000 * (iprob-1)
            jcart  = isolut(3,kvec)/10+1

         end if

         if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 .and.
     +        isubr>4 .and. ifirstwrit==0 ) then

!        --- ioutp>=0 and isubr = 5 or 6, print information

            nch = index(names(kvec),' ')-1
            if ( nch<=0 ) nch = len(names(kvec))
            if ( isubr==5 ) then

!           --- call by ercret

               if ( ifill==-1 ) then

!              --- ifill = -1, array is created but not filled

                  write ( irefwr, 100 ) names(kvec)(1:nch)
100               format ( ' Create vector ', a, ' without filling' )

               else

!              --- ifill>-1, array is created and filled

                  write ( irefwr, 110 ) names(kvec)(1:nch)
110               format ( ' Create and fill vector ', a )

               end if

            else

!           --- call by erpres

               if ( ifill==-1 ) then

!              --- ifill = -1, array is created but not filled

                  write ( irefwr, 100 ) names(kvec)(1:nch)

               else

!              --- ifill>-1, array is created and filled

                  write ( irefwr, 120 ) names(kvec)(1:nch)
120               format ( ' Fill boundary conditions in vector ', a )

               end if

            end if

         end if

         if ( itimloop==1 ) then
            if ( t>=tend-tstep*0.1 ) then
               ifirstwrit = 0
            else
               ifirstwrit = 1
            end if  ! ( t>=tend-dt*0.1 )
         end if  ! ( itimloop==1 )

!        --- Check some parameters

         if ( iprob<0 .or. iprob>nprob ) then
            call errint ( iprob, 1 )
            call errint ( nprob, 2 )
            call errsub ( 699, 2, 0, 0 )
         end if
         if ( icompl<0 .or. icompl>1 ) then
            call errint ( icompl, 1 )
            call errsub ( 1127, 1, 0, 0 )
         end if
         if ( itypvc==115 ) then
            ndtvec = kprob ( ipkprb+6)
            if ( ivec<=0 .or. ivec>ndtvec ) then
               call errint ( ivec, 1 )
               call errint ( ndtvec, 1 )
               call errsub ( 206, 2, 0, 0 )
            end if
         end if
         isolut(3,kvec) = icompl+10*(jcart-1)

!        --- Check if information of obstacles must be generated

         call ergetifill ( inpcre, ifill, ipstin, ifirst,
     +                     obstaclefound )
         if ( obstaclefound ) then

!        --- Information of obstacles must be generated

            if ( kmesh(42)==0 ) then

!           --- Error no obstacles present

               call errsub ( 2208, 0, 0, 0 )
               go to 1000

            end if

            kelmad = kmesh(52)
            if ( kelmad==0 ) then
               if ( ndim==2 ) then
                  call mshfillkmeshad ( ibuffr, buffer, kmesh, 2 )
               else
                  call mshfillkmesha3 ( ibuffr, buffer, kmesh, 2 )
               end if
            else
               call ini060text ( ibuffr, kmesh(kelmad), 'ipointobst' )
            end if

         else

!        --- No information of obstacles must be generated

            ipipointobst = 1

         end if

!        --- Create space

         indprh = 0
         if ( debug )
     +      write(irefwr,1) 'kvec, itypvc, ivec, iprob',
     +                       kvec, itypvc, ivec, iprob
         call ini086 ( ibuffr, isolut(1,kvec), -itypvc, ivec, iprob,
     +                 kmesh, kprob, indprf, indprp, nphys, 'vector' )

         if ( debug ) write(irefwr,1) 'nphys', nphys

!        --- Fill space for work arrays IWORK and NODES, both of length NPOINT
!            In case of a call to er0090 we need an integer array IWORK and
!            a double precision array WORK
!            IWORK has length NPOINT and WORK intlen*NPOINT*nphys integers
!            In case of creatvspec we need two integer arrays index1 and index2
!            with length npelm*(nphys+1)

         if ( itypvc==801 ) then
            length = (intlen*nphys+2)*(ivec*(ivec-1))/2
         else
            length = 2*npoint + intlen*max(npoint,nphys*nelem)
            length = max(length,npelm*(nphys+1))
         end if
         call ini069 ( ibuffr, ipnode, length, nbufol )
         if ( ierror/=0 ) go to 1000
         ipwork = ipnode + npoint
         if ( isolut(2,kvec)==110 ) indprh = kprob(ipkprb+50)
         if ( indprh>0 )
     +      call ini060text ( ibuffr, -indprh, 'kprobh' )
         if ( kprob(ipkprb+46)/=0 )
     +        call ini060text ( ibuffr, -kprob(ipkprb+46), 'kprobv' )
         if ( ierror/=0 ) go to 1000

!        --- Compute starting addresses

         ipcoor = inidgttext ( kmesh(23), 'coor' )
         ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
         ipkmsl = 1
         ipkmsm = 1
         ipkmsn = 1
         ipkmsq = 1
         ipkmss = 1
         ipkprf = 1
         ipkprh = 1
         ipkprp = 1
         ipkprv = 1
         if ( kmesh(26)>0 )
     +      ipkmsl = inigettext ( kmesh(26), 'kmeshc' )
         if ( kmesh(27)>0 )
     +      ipkmsm = inigettext ( kmesh(27), 'kmeshm' )
         if ( kmesh(28)>0 )
     +      ipkmsn = inigettext ( kmesh(28), 'kmeshn' )
         if ( kmesh(31)>0 )
     +      ipkmsq = inigettext ( kmesh(31), 'kmeshq' )
         if ( kmesh(33)>0 )
     +      ipkmss = inigettext ( kmesh(33), 'kmeshs' )
         if ( kmesh(58)>0 )
     +      ipouter = inigettext ( kmesh(58), 'outerboun' )
         if ( indprf>0 ) ipkprf = inigettext ( indprf, 'kprobf' )
         if ( indprh>0 ) ipkprh = inigettext ( indprh, 'kprobh' )
         if ( indprp>0 ) ipkprp = inigettext ( indprp, 'kprobp' )
         if ( kprob(ipkprb+46)>0 )
     +      ipkprv = inigettext ( kprob(ipkprb+46),'kprobv' )
         ipvect = inidgttext ( isolut(1,kvec), 'vector' )
         if ( kmesh(22)>0 .and. kmesh(42)>0 ) then

!        --- Possibly obstacles

            ipkmsh = inigettext ( kmesh(22), 'kmeshh' )
            if ( ibuffr(ipkmsh)>0 ) then
               ipicrvob = inigettext ( kmesh(22), 'icurvsobst' )
            end if

         else

            ipicrvob = 1

         end if
         if ( obstaclefound )
     +      ipipointobst = inigettext ( kmesh(kelmad), 'ipointobst' )
         if ( ierror/=0 ) go to 1000

         if ( new .or. ifill==0 .and. .not. bndcon ) then

!        --- ifill = 0, or new array, clear array

            call ini006 ( buffer(ipvect), isolut(5,kvec)*(1+icompl) )

         else if ( bndcon ) then

!        --- isubr = 2, PRESDF/TM; set last part of array equal to zero

            nusol  = kprob(ipkprb+5)
            nbound = kprob(ipkprb+10)
            if ( nbound>0 ) then

!           --- There are essential boundary conditions

               if ( icompl==0 ) then
                  call tobvalzr ( buffer(ipvect), nusol, nbound,
     +                            ibuffr(ipkprh+nusol) )
               else
                  call tobvalzc ( buffer(ipvect), nusol, nbound,
     +                            ibuffr(ipkprh+nusol) )
               end if

            end if

         end if
         if ( ierror/=0 ) go to 1000

!        --- ifill>0, Loop over IFILL cases

         do k = 1, ifill

!        --- i=ipstin

            jpstin = ipstin + 1

!           --- i+1

            kdeg   = 0
            kfun   = 0
            kfill  = 0
            if ( ifirst>=jpstin ) kdeg   = inpcre(jpstin)
            if ( kdeg<0 ) jpstin = jpstin - kdeg
            if ( ifirst>=jpstin+1 ) kfun   = inpcre(jpstin+1)
            if ( kfun>=9001 .and. kfun<=9004 ) then

!           --- kfun = 9001, 9002, quadratic function along curves

               jpstin = jpstin+1

            else if ( kfun>3000 .and. kfun<4000 ) then

!           --- kfun>3000, function of old vectors

               mvec     = inpcre(jpstin+2)
               istartvc = jpstin+3
               jpstin = jpstin+mvec+1
               if ( mvec<=0 .or. mvec>10 ) then

!              --- mvec out of range

                  call errint ( mvec, 1 )
                  call errint ( 1, 2 )
                  call errint ( 10, 3 )
                  call errsub ( 1053, 3, 0, 0 )
                  mvec = 1

               end if

!              --- Compute maxphys and activate arrays
!                  This is done at this position to make it possible
!                  to compute starting addresses in local subroutines

               maxphys = 0
               do i = 1, mvec
                  call ini068 ( ibuffr,
     +                          islold(1,inpcre(istartvc+i-1)),
     +                          kprob, istore(1,i), istore(2,i),
     +                          istore(3,i), nunkp, istore(4,i) )
                  maxphys = max(maxphys,istore(4,i))
               end do

            else if ( kfun>4000 .and. kfun<5000 ) then

!           --- kfun>4000, use old vector

               ivecold = kfun-4000
               if ( debug )
     +            write(irefwr,1) 'ivecold, jpstin', ivecold, jpstin

            end if  ! ( kfun>=9001 .and. kfun<=9004 )
            if ( ifirst>=jpstin+2 ) kfill  = inpcre(jpstin+2)
            jpstin = jpstin + 3
            if ( debug )
     +         write(irefwr,1) 'jpstin, kdeg, kfun, kfill',
     +                          jpstin, kdeg, kfun, kfill

!           --- i+4

            if ( kfill==1 ) then
               n = 1
               if ( ifirst>=jpstin ) n = inpcre(jpstin)
               jpstin = jpstin + 1

!              --- i+5

            end if

            if ( itypvc==116 .or. itypvc==126 ) then

!           --- itypvc = 116, 126
!               Compute pointers to work arrays

               ipiwork = ipnode+intlen*nelem*nphys
               ipwork = inidbl(ipnode)
               call creatv116 ( buffer, buffer(ipvect), kmesh,
     +                          buffer(ipwork), ibuffr(ipiwork),
     +                          isolut, ibuffr(ipkmsc),
     +                          buffer(ipcoor), nphys, itypvc, kfill,
     +                          kfun, kdeg, icompl, inpcre, rincre,
     +                          jpstin, kvec, istore, istartvc,
     +                          maxphys, mvec )
               if ( ierror/=0 ) go to 1000

            else if ( itypvc==801 ) then

!           --- itypvc = 801, type is capacity vector
!               Compute pointers to work arrays

               ipiwork = ipnode+intlen*nphys*(ivec*(ivec-1))/2
               ipwork = inidbl(ipnode)
               call creatv801 ( buffer(ipvect), buffer(ipwork),
     +                          ibuffr(ipiwork), nphys, kfill, kfun,
     +                          kdeg, icompl, inpcre, rincre, jpstin,
     +                          ivec )

            else if ( kfill==12 ) then

!           --- kfill = 12, use file with node numbers

               if ( kdeg==0 ) then

!              --- kdeg = 0, all degrees of freedom

                  if ( kprob(4+ipkprb)>1 ) then

!                 --- more than one unknown per point (not yet implemented)

                     call errwar ( 1676, 0, 0, 0 )

                  end if
                  kdeg = 1

               end if
               if ( icompl/=0 )
     +            call errsub ( 1677, 0, 0, 0 )

               ipos = inpcre(jpstin)
               if ( ipos<=0 .or. ipos>=nextxt ) then

!              --- ipos out of range

                  call errint ( jpstin, 1 )
                  call errint ( ipos, 2 )
                  call errint ( nextxt, 3 )
                  call errsub ( 1675, 3, 0, 0 )
                  go to 1000

               end if
               filename = texts(ipos)

!              --- Compute pointers to work arrays

               ipiwork = ipnode+intlen*npoint
               ipwork = inidbl(ipnode)
               call er0090 ( filename, buffer(ipvect), kdeg,
     +                       kprob(ipkprb+1), ibuffr(ipkprf),
     +                       kmesh, buffer(ipwork), ibuffr(ipiwork) )

            else if ( kfill==15 ) then

!           --- kfill = 15, use unknowns corresponding to global element group

               call ercreatglob ( buffer(ipvect), kprob(ipkprb+1), kdeg,
     +                            kfun, inpcre(jpstin), rincre )

            else if ( kfill==0 .and. kfun>1000 .and.
     +                kfun<2000 ) then

!           --- kfill = 0 and 1000< kfun<2000
!               special function (only in case of all nodes)

               call creatvspec ( kfun, kmesh, kprob, indprf, indprp,
     +                           ibuffr(ipkprf), ibuffr(ipkprp),
     +                           buffer(ipvect), buffer(ipcoor),
     +                           ibuffr(ipnode), ibuffr(ipnode+npelm),
     +                           npoint, ibuffr(ipkmsc), icompl )

            else

!           --- Store nodal point numbers into array NODES

               call er0023 ( kfill, kmesh, ibuffr(ipkmsc),
     +                       ibuffr(ipkmsl), ibuffr(ipkmsm),
     +                       ibuffr(ipkmsn), kmesh(kmesht), n,
     +                       inpcre(jpstin), ibuffr(ipnode),
     +                       ibuffr(ipwork), ibuffr(ipicrvob),
     +                       ibuffr(ipkmsh), kprob(ipkprb+1),
     +                       ibuffr(ipkprv), ibuffr(ipipointobst),
     +                       ibuffr(ipkmsq), ibuffr(ipouter) )
               if ( ierror/=0 ) go to 1000

!              --- Fill array vector(kvec) with vector

               if ( n>0 .and. kfun>=9001 .and. kfun<=9004 ) then

!              --- kfun = 9001-9004, quadratic functions along curves or
!                  surfaces

                  call ercreatquad ( buffer(ipvect), ibuffr(ipnode),
     +                               n, npoint, kdeg, inpcre(ipstin+3),
     +                               indprf, indprp, ibuffr(ipkprf),
     +                               ibuffr(ipkprp), rincre,
     +                               buffer(ipcoor), ndim, nphys, kfun,
     +                               kfill, icompl, ibuffr(ipkmsm),
     +                               ibuffr(ipkmss), inpcre(jpstin),
     +                               kmesh, ibuffr(ipkmsq) )

               else if ( n>0 .and. kfun>3000 .and. kfun<4000 ) then

!              --- kfun>3000, function of old vectors
!                  first activate the old vectors and compute maxphys

!                  --- Reserve space at the end of ibuffr to store both
!                     the array containing the old vectors (uoldcp) as well
!                     as the new solution (ucopy)

                  call ini062 ( ibuffr, intlen*n*
     +                          (mvec*maxphys+nphys+ndim), ipcoorcp )
                  ipcoorcp = inidbl(ipcoorcp)
                  ipuoldcp = ipcoorcp+n*ndim
                  ipucopy  = ipuoldcp+n*mvec*maxphys

!                 --- Clear both arrays

                  call ini006 ( buffer(ipuoldcp),
     +                          n*(mvec*maxphys+nphys) )

!                 --- recompute all starting addresses

                  ipcoor = inidgttext ( kmesh(23), 'coor' )

!                 --- Copy coordinates to coorcp

                  call ercopcor ( buffer(ipcoor), buffer(ipcoorcp),
     +                            ndim, n, ibuffr(ipnode) )

!                 --- Put old solution vectors in uoldcp

                  do i = 1, mvec
                     ipvect = inidgttext (
     +                  islold(1,inpcre(istartvc+i-1)), 'vector' )
                     if ( istore(1,i)>0 )
     +                  ipkprf = inigettext ( istore(1,i), 'kprobf' )
                     if ( istore(2,i)>0 )
     +                  ipkprh = inigettext ( istore(2,i), 'kprobh' )
                     if ( istore(3,i)>0 )
     +                  ipkprp = inigettext ( istore(3,i), 'kprobp' )
                     call erfillvc ( buffer(ipvect), ibuffr(ipnode), n,
     +                               buffer(ipuoldcp+(i-1)*n*maxphys),
     +                               istore(1,i), ibuffr(ipkprf),
     +                               ibuffr(ipkprp), npoint )
                  end do

!                 --- Put the new vector in array ucopy by calling the user
!                     subroutine FUNCVECT

                  call funcvect ( kfun-3000, ndim, buffer(ipcoorcp),
     +                            n, buffer(ipuoldcp), mvec,
     +                            buffer(ipucopy), maxphys )

!                 --- Finally copy contents to array vector

                  if ( indprf>0 )
     +               ipkprf = inigettext ( indprf, 'kprobf' )
                  if ( indprp>0 )
     +               ipkprp = inigettext ( indprp, 'kprobp' )
                  ipvect = inidgttext ( isolut(1,kvec), 'vector' )
                  call erfilcrv ( buffer(ipvect), ibuffr(ipnode), n,
     +                            npoint, kdeg, inpcre(ipstin+2),
     +                            indprf, indprp, ibuffr(ipkprf),
     +                            ibuffr(ipkprp), nphys,
     +                            buffer(ipucopy) )

               else if ( n>0 .and. kfun>4000 ) then

!              --- kfun>4000, vector
!                  Activate array and corresponding arrays

                  call ini061 ( ibuffr, islold(1,ivecold), kprob,
     +                          indprfold, indprpold, nunkpold,
     +                          nphysold )
                  if ( indprfold>0 ) then
                     ipkprfold = inigettext ( indprfold, 'kprobfold' )
                  else
                     ipkprfold = 1
                  end if  ! ( indprfold>0 )
                  ipvectold = inidgttext ( islold(1,ivecold), 'vector' )
                  if ( indprf>0 )
     +               ipkprf = inigettext ( indprf, 'kprobf' )
                  if ( indprp>0 )
     +               ipkprp = inigettext ( indprp, 'kprobp' )
                  ipvect = inidgttext ( isolut(1,kvec), 'vector' )
                  call erfillcopvec ( ibuffr(ipnode), kdeg,
     +                                inpcre(ipstin+2),
     +                                buffer(ipvectold), buffer(ipvect),
     +                                indprfold, indprf,
     +                                ibuffr(ipkprfold),
     +                                ibuffr(ipkprf), nphysold, nphys,
     +                                n )

               else if ( n>0 .and. kfun/=0 ) then

                  if ( icompl==0 ) then

!                 --- Real case

                     call er0048 ( buffer(ipvect), ibuffr(ipnode),
     +                             n, npoint, kdeg, inpcre(ipstin+2),
     +                             kfun, indprf, indprp, ibuffr(ipkprf),
     +                             ibuffr(ipkprp), rincre,
     +                             buffer(ipcoor), ndim, nphys, bndcon )
                  else

!                 --- Complex case

                     call er0069 ( buffer(ipvect), ibuffr(ipnode),
     +                             n, npoint, kdeg, inpcre(ipstin+2),
     +                             kfun, indprf, indprp, ibuffr(ipkprf),
     +                             ibuffr(ipkprp), rincre,
     +                             buffer(ipcoor), ndim, nphys, bndcon )
                  end if

               end if

            end if
            if ( debug )
     +         write(irefwr,1) 'ipstin, jpstin', ipstin, jpstin

!           --- raise ipstin

            ipstin = jpstin - 1

!           --- i+4

            if ( kfill==1 ) then
               ipstin = ipstin + 1 + n

!              --- i+6

               ipstin = ipstin - 1

!              --- i+5, 5 items per function when n=1, are other cases OK ?

            else if ( kfill==2 .or.kfill==4 .or.kfill==8 ) then
               ipstin = ipstin + 3
            else if ( kfill==3 .or.kfill==5 .or.kfill==7 ) then
               ipstin = ipstin + 3 + inpcre(ipstin+3)
            else if ( kfill==6 ) then
               ipstin = ipstin + 2 + inpcre(ipstin+2)
            else if ( kfill>=9 ) then
               ipstin = ipstin + 1
            end if

         end do
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,1) 'ipstin, kfill', ipstin, kfill

         nusol  = kprob(ipkprb+5)
         nbound = kprob(ipkprb+10)
         npboun = kprob(ipkprb+34)

         if ( itypvc==110 .and. npboun>0 ) then

!        --- itrans # 2 and npboun>0
!            Periodical boundary conditions
!            Compute effect of periodical boundary conditions

            call ini060text ( ibuffr, kprob(ipkprb+38), 'kprobs' )
            ipc = inidgttext ( kprob(ipkprb+38), 'kprobs' )
            ipvect = inidgttext ( isolut(1,kvec), 'vector' )
            ipcint = inigettext ( kprob(ipkprb+38), 'kprobs' ) +
     +               intlen*2*npboun
            indprh = kprob(ipkprb+50)
            if ( icompl==0 ) then

!           --- real case

               call prsprrre ( buffer(ipvect), buffer(ipc),
     +                         ibuffr(ipcint), npboun, nusol,
     +                         nbound, ibuffr(ipkprh+nusol), indprh )

            else

!           --- complex case

               call prsprcre ( buffer(ipvect), buffer(ipc),
     +                         ibuffr(ipcint), npboun, nusol,
     +                         nbound, ibuffr(ipkprh+nusol), indprh )

            end if

         end if
         call inirst ( nbufol )

      end do

1000  if ( inpcre(1)<-4 ) inpcre(1) = ivecsq+nv
      call erclos ( 'creatv01' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( isolut, 2, 5, 'isolut' )
         write(irefwr,*) 'End creatv01'

      end if

      end
