      subroutine readscboun1 ( ibuffr, icode, kmesh, inpsep,
     +                         istart, rinsep, istrin, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-07-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input following prescribe_boundary_conditions vector
!     No sequence number found
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cinout'
      include 'SPcommon/cseprdnew'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), icode, kmesh(*), inpsep(*), istart, istrin,
     +        nprob
      double precision rinsep(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     icode          i    Defines type of subroutine
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Positions 1 to 4 are filled by readscbouncond
!                         Information of rest of positions:
!                         5:  number of integers stored
!                         6:  number of reals stored
!                         7:  starting address in array rinsep
!                         followed by numint positions of actual information
!                         as read by readc0
!                         followed by aray iinputinf of length nreal
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nprob          i    Number of various problems
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leninpcre, lenrincre, nmain, nextra
      parameter ( leninpcre=10, lenrincre=10, nmain = 0, nextra=6 )
      integer inpcre(leninpcre), icompl, inforinpcre(lenrincre), numint,
     +        nreal, ipkmsn, nmcode(nmain+nextra), iprob
      double precision rincre(lenrincre)
      character (len=9) codemn(nmain+nextra)
      logical debug

!     codemn         Table with main keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icompl         Indicates if vector is real (0) or complex (1)
!     inforinpcre    Array of length of rincre that contains a 0 if the
!                    number stored in rincre is a real and a reference
!                    to scalars if it is a scalar
!     inpcre         The integer input read is stored in array INPCRE from
!                    position 1.
!     ipkmsn         Starting address of array KMESH part n
!     iprob          Actual problem number
!     leninpcre      Declared length of array inpcre
!     lenrincre      Declared length of array rincre
!     nextra         Extra number of entries in codemn
!     nmain          Number of entries in codemn
!     nmcode         Table containing the number of significant characters
!                    in the main keyword table codemn, including those of
!                    the extra texts
!     nreal          Number of reals stored
!     numint         Number of integers stored
!     rincre         Double precision input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     READC0         Read standard data records
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscboun1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscboun1'
         write(irefwr,1) 'istrin', istrin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kmesh(28)>0 ) then
         call ini060text ( ibuffr, -kmesh(28 ), 'kmeshn' )
         ipkmsn = inigettext ( kmesh(28), 'kmeshn')
      else
         ipkmsn = 1
      end if

!     --- Fill codemn and nmcode

      codemn(1:nextra) = ' '
      nmcode(1:nextra) = 0

!     --- Read rest of record

      nreal = 0
      icompl = 0
      lastitem = lastitem-1
      iprob = nprob
      call readc0 ( inpcre, rincre, numint, nreal, icode, icompl,
     +              kmesh, codemn, nmain, nmcode, ibuffr(ipkmsn),
     +              nextra, inforinpcre, iprob )
      iprob = max(1,iprob)

      call inichkarlen ( numint, leninpcre, 'inpcre' )
      call inichkarlen ( nreal, lenrincre, 'rincre' )
      if ( ierror/=0 ) go to 1000

!     --- Store information in inpsep and rinsep

      istart = istart-4
      inpsep(istart+1) = -10      ! seq number
      inpsep(istart+2) = iprob    ! problem numberincrer
      istart = istart+3
      inpsep(istart+1) = numint
      inpsep(istart+2) = nreal
      inpsep(istart+3) = istrin
      istart = istart+3
      inpsep(istart+1:istart+numint) = inpcre(1:numint)
      istart = istart+numint
      if ( nreal>0 ) then
         inpsep(istart+1:istart+nreal) = inforinpcre(1:nreal)
         rinsep(istrin:istrin+nreal-1) = rincre(1:nreal)
         istart = istart+nreal
         istrin = istrin+nreal
      end if  ! ( nreal>0 )
      istart = istart+1

1000  call erclos ( 'readscboun1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscboun1'

      end if  ! ( debug )

      end
