c    ***************************************************************
c    *
c    *  oil radial bearing
c    *  boundary  1,2      no flow  (half bearing)
c    *  boundary  5,6      essential boundary conditions
c    *  boundary  3,4 <--> boundary 7,8  periodical boundary conditions
c    *
c    *  Start: Reynolds equation without cavitation
c    *  Second part:  Solution including cavitation constraints
c    *
c    *  The cavitation is taken into account by an iterative
c    *  solution method with constraints
c    *
c    ***************************************************************
c
      program bearing
c
c    *  standard declarations
c
      use sepmoduleoldrouts
      use sepmodulecomio
      implicit double precision (a-h,o-z)
      integer :: kmesh,kprob,intmat,matr,isol,
     .           islold,irhsd,iuser(100), iwork(20)
      double precision :: work(20),user(100)

      kmesh=0
      kprob=0
      iuser = 0
      intmat = 0
      matr = 0
      isol = 0
      islold = 0
      irhsd = 0
      user=0d0
      iuser(1)=100
      user(1)=100
c
c    ***************************************************************
c    *
c    *  start of program (standard for each sepran program)
c    *
c    ***************************************************************
c
      print *,'PROGRAM BEARING'
      istart=0
      irotat=1
      ioutp=0
      itime=0
      call start(istart,irotat,ioutp,itime)
c
c   *  mesh generation
c
      ichois=0
      call mesh(ichois,iinput,rinput,kmesh)
c
c   *   problem definition
c
      ichois=0
      call probdf(ichois,kprob,kmesh,iinput)
c
c   *  define type of method
c      jmetod = 1 symmetric matrix
c
      jmetod=1
      call commat(jmetod,kmesh,kprob,intmat)
c
c    ***************************************************************
c    *
c    *  fill boundary conditions  (all essential bc's are zero)
c    *
c    ***************************************************************
c
      idim=0
      call bvalue(ichois,idim,kmesh,kprob,isol,value,icrv1,icrv2,idegfd,
     .            jstap)
c
c    ***************************************************************
c    *
c    *  Build the system of equations and define the parameters
c    *
c    *  Values of iwork and work:
c    *
c    *   6:  film thickness h
c    *   7:  dynamic viscosity mu
c    *   8:  buffer term ht
c    *   9:  constant k
c    *  10:  reference pressure p
c    *  11,12:  velocity components
c    *
c    ***************************************************************
c
      iwork=0
      work=0d0
      iwork(6)=1
      work(7)=4d-2
      work(11)=204d-2
      ielgrp=1
      nparm=20
      call fil100(ielgrp,iuser,user,kprob,nparm,iwork,work)
c
      call systm0(1,matr,intmat,kmesh,kprob,irhsd,
     .            isol,iuser,user,islold,ielhlp)
c
c   *  solve the system of equations
c      cavity is not taken into account
c      matrix is positive definite
c
      call solve(1,matr,isol,irhsd,intmat,kprob)
c
c   *  print solution
c
      call prinrv(isol,kmesh,kprob,5,1,'pressure')
c
c   *  compute and print load
c
      call load(isol,kmesh,kprob,fx,fy,ftot,phi)
      write ( irefwr, 100 ) fx,fy,ftot,phi
100   format(' load without effect of cavity'/
     .       ' horizontal component of load',ss,1pe12.3e2/
     .       ' vertical component of load  ',ss,1pe12.3e2/
     .       ' modulus of load             ',ss,1pe12.3e2/
     .       ' attitude angle              ',ss,1pe12.3e2)
c
c    ***************************************************************
c    *
c    * second part: overrelaxation with constraints in order to take
c    * cavitaion into account
c    * As initial value the solution from the first part (without cavitation
c    * boundary condition is used.
c    *
c    ***************************************************************
c    *
c
c   *  redefine type of method for subroutine overcs
c      and rebuild system of equations
c
      call commat(9,kmesh,kprob,intmat)
c
      call systm0(1,matr,intmat,kmesh,kprob,irhsd,
     .            isol,iuser,user,islold,ielhlp)
c
c   * solve system of equations with the aid of subroutine overcs
c     constrained optimalisation (p>=0)
c
      iwork(1)=1
      iwork(2)=1
      iwork(3)=1
      work(1)=-.3d5
      eps=1d-3
      nmax=400
c
      call overcs(1,matr,irhsd,isol,intmat,kmesh,kprob,eps,
     .            nmax,iwork,work)
c
c   *  print solution
c
      call prinrv(isol,kmesh,kprob,5,1,'pressure')
c
c   *  compute and print load
c
      call load(isol,kmesh,kprob,fx,fy,ftot,phi)
      write ( irefwr, 110 ) fx,fy,ftot,phi
110   format(' load including the effect of cavity'/
     .       ' horizontal component of load',ss,1pe12.3e2/
     .       ' vertical component of load  ',ss,1pe12.3e2/
     .       ' modulus of load             ',ss,1pe12.3e2/
     .       ' attitude angle              ',ss,1pe12.3e2)
c
c   *  stop sepran
c
      print *,'END PROGRAM BEARING'
      call finish(0)
      end
c
c   *  function funccf for the computation of h and the
c      arguments in the integral (sin(2x/D) resp cos (2x/D))
c
      function funccf(ichois,x,y,z)
      implicit double precision (a-h,o-z)
c
      pi=4d0*atan(1d0)
      diam=50d-3
      deltar=8.2d-5
      eps=.4d0
      phi=2d0*(x/diam)
c
      goto( 10 , 20 , 30 ),ichois
c
c*****h(x)   minimal film thickness at x=0
c
10    funccf=deltar*(1d0-eps*cos(phi))
      return
c
c*****x-component load
c
20    funccf=-cos(phi)
      return
c
c*****y-component load
c
30    funccf=-sin(phi)
      end
c
c   * subroutine load coputes the load and the attitude angle
c
c     input:
c               isol   solution vector
c               kmesh  contains information of the mesh
c               kprob  contains information of the problem definition
c
c     output:
c               fx     horizontal component of load
c               fy     vertical component of load
c               ftot   modulus of load
c               phi    attitude angle
c
      subroutine load(isol,kmesh,kprob,fx,fy,ftot,phi)
      implicit double precision (a-h,o-z)
      integer :: isol,kmesh,kprob, iwork(1),iuser(100)
      double precision :: work(1), user(100)

c   *  fill parameters for subroutine volint to compute the integrals
c
      iuser = 0
      user = 0d0
      iuser(1)=100
      user(1)=100
      iwork(1)=2
      ielgrp=1
      iparm=1
      call fil100(ielgrp,iuser,user,kprob,iparm,iwork,work)
c
      fx=volint(0,2,1,kmesh,kprob,isol,iuser,user,ielhlp)
c
      iwork(1)=3
      call fil100(ielgrp,iuser,user,kprob,iparm,iwork,work)
c
      fy=volint(0,2,1,kmesh,kprob,isol,iuser,user,ielhlp)
c
      ftot=sqrt(fy*fy+fx*fx)
      phi=atan(fy/fx)
c
      end
