      program eigvalex
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 1.0    date   07-01-1996
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     example for the computation of eigenvalues
!
! ********************************************************************
!
!                      KEYWORDS
!
!     eigenvalue
!     eigenvector
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecomio
      implicit none
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer kmesh(200), kprob(200), intmat(5), iinbld(10), iinout(1),
     +        istiff(5), mass(5), icheig(10), ivec(5,3), iinstr(2),
     +        iinfil(1), idummy, i, numeig, iuser(100), irhsd(5)
      double precision user(100), eval(10), time

!    eval     Variable length array in which the eigevalues are stored
!    i        General loop variable
!    icheig   Input array for subroutine EIGVAL
!             ICHEIG is filled as follows:
!             Pos. 1:  1, i.e. eigenvectors must be computed
!                  2:  1, i.e. information about the iteration must be printed
!                  3-8: 0, these psitions are not yet used
!    idummy   Dummy parameter
!    iinbld   Input array for subroutine BUILD
!             IINBLD is filled as follows:
!             Pos. 1:  4, i.e. positions 1-4 are filled
!                  2: 14, matrix is build, but not the right-hand side
!                  3:  1, the effect of boundary conditions is not taken into
!                         account
!                  4:  1, the mass matrix is diagonal
!    iinfil   Input array for subroutine FILLCF
!             IINFIL is filled as follows:
!             Pos. 1:  0, i.e. only position 1 is filled
!    iinout   Input array for subroutine OUTSOL
!             IINFIL is filled as follows:
!             Pos. 1:  0, i.e. only position 1 is filled
!    iinstr   Input array for subroutine SEPSTN
!             IINSTR is filled as follows:
!             Pos. 1:  2, i.e. positions 1-2 are filled
!             Pos. 2:  1, i.e. all input until end_of_sepran_input is read
!    intmat   Standard SEPRAN array containing information of the storage
!             scheme of the matrix
!    irhsd    Array containing information about the right-hand side
!    istiff   Contains information about the stiffness matrix
!    iuser    Variable length user array to be filled by FILCOF which
!             contains information about the coefficients
!    ivec     Contains information about the three eigenvectors
!             ivec(j,i), j=1,5 relates to eigenvector i
!    kmesh    Standard SEPRAN array containing information of the mesh
!    kprob    Standard SEPRAN array containing information of the problem
!    mass     Contains information about the mass matrix
!    numeig   Number of eigenvalues and eigenvectors to be computed
!    time     Dummy parameter
!    user     Variable length double precision user array to be filled by FILCOF
!             which contains information about the coefficients
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     BUILD   Build matrices
!     EIGVAL  Comute eigenvalues and eigenvectors
!     FILLCF  Fill parameters for equations
!     FINISH  Stop SEPRAN
!     OUTSOL  Write eigenvectors to backing storage
!     SEPSTN  Start SEPRAN and read mesh
!
! ********************************************************************
!
!                    I/O
!
!    This program reads the standard SEPRAN files.
!    It assumes that the mesh has been generated by program SEPMESH
!    with input file eigval.msh and output file meshoutput
!    It reads input from the standard input file (eigval.prb)
!    It produces one file: sepcomp.out for program SEPPOST
!    Program SEPPOST requires the input file eigval.pst
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   -
! ********************************************************************
!
!                    PSEUDO CODE
!
!   Fill length of variable length arrays
!   Start SEPRAN and read mesh
!   Fill coefficients
!   Build matrices
!   Compute eigenvalues and eigenvectors
!   Print eigenvalues
!   Write eigenvectors for postprocessing
! ======================================================================
!
!     --- Fill length of variable length arrays

      kmesh(1) = 200
      kprob(1) = 200
      user(1)  = 100
      iuser(1) = 100
      eval(1)  = 10
      kmesh(3) = 0
      kprob(3) = 0
      user(3)  = 0
      iuser(3) = 0
      eval(3)  = 0

!     --- Start SEPRAN and read mesh

      iinstr(1) = 2
      iinstr(2) = 1
      call sepstn ( kmesh, kprob, intmat, iinstr )

!     --- fill coefficients for Laplace equation

      iinfil(1) = 0
      call fillcf ( iuser, user, kprob, kmesh, iinfil )

!     --- build mass matrix and stiffness matrix
!         not the right-hand side!

      iinbld(1) = 4
      iinbld(2) = 14
      iinbld(3) = 1
      iinbld(4) = 1
      call build ( iinbld, istiff, intmat, kmesh, kprob, irhsd, mass,
     +             ivec, idummy, iuser, user )

!     --- compute eigenvalues and eigenvectors

      icheig(1) = 1
      icheig(2) = 1
      icheig(3:8) = 0

      numeig = 3
      call eigval ( eval, numeig, ivec, istiff, intmat, mass, intmat,
     +              kprob, icheig )

!     --- output

      do i = 1, numeig
         write ( irefwr, 120 ) i, eval(i+5)
120      format(' eigenvalue',i4,':',e12.5)
      end do

!     --- write eigenvectors to "sepcomp.out"

      time = 0d0
      iinout(1) = 0
      call outsol ( kmesh, kprob, ivec, iinout, time, iuser, user )

!     --- Stop SEPRAN

      call finish(0)
      end
