! ****************************************************************************
!
!      File:  exam3-1-1b.f
!
!      Contents:  Main program for the test example described
!                 in the SEPRAN manual examples 3-1-1
!                 Artificial analytical example
!                 This program uses the version in which the user
!                 writes the main program himself
!                 Mesh generation must have been performed before by sepmesh
!                 Postprocessing must be performed by seppost
!
!      Usage:     Compile and link this program with the SEPRAN libraries:
!                 seplink exam3-1-1b
!                 Run this program with input: exam3-1-1b.prb
!
!                 exam3-1-1b < exam3-1-1b.prb > exam3-1-1b.out
!
!      version 1.0    date   05-11-93
!
! ****************************************************************************
!
      program exam311b
      use sepmoduleoldrouts
      implicit none
      integer lniusr, lnuser
      parameter ( lniusr=100, lnuser=100 )
      integer kmesh, kprob, intmat, matr,
     +        isol, irhsd, iuser(lniusr), iinbld(1)
      integer iprob, massmt, islold, ipos, istop
      double precision user(lnuser)

!     --- Clear all arrays in order to prevent unwanted side effects

      data kmesh /0/, kprob /0/, intmat/0/, matr /0/,
     +     isol /0/, irhsd /0/, iuser /lniusr*0/,
     +     user /lnuser*0d0/

!     --- Set first positions of variable length arrays

      iuser(1)  = lniusr
      user(1)   = lnuser

!     --- Start SEPRAN, read the mesh

      call sepstr ( kmesh, kprob, intmat )

!     --- Fill essential boundary conditions

      call presdf ( kmesh, kprob, isol )

!     --- Fill coefficients for the differential equation

      iprob = 1
      call filcof ( iuser, user, kprob, kmesh, iprob )

!     --- Build matrix and right-hand side (default situation)

      iinbld(1) = 0
      call build ( iinbld, matr, intmat, kmesh, kprob, irhsd,
     +             massmt, isol, islold, iuser, user )

!     --- Solve system of equations by a direct (profile) method
!         The matrix is positive definite

      ipos = 1
      call solve ( ipos, matr, isol, irhsd, intmat, kprob )

!     --- Write the solution to file sepcomp.out, for postprocessing
!         purposes

      call output ( kmesh, kprob, isol )

!     --- Stop SEPRAN

      istop = 0
      call finish ( istop )

      end


!     --- function funccf for the definition of the coefficients


      function funccf ( ichois, x, y, z )
      implicit none
      integer ichois
      double precision funccf, x, y, z

      if ( ichois==1 ) then

!     ---  du /dn  =  y

         funccf = y

      else

!     ---  u + du/dn  =  2x

         funccf = 2*x

      end if

      end
