! ********************************************************************
! *
! *   example for the solution of the coupled Boussinesq equations
! *   Square cavity problem  (Free convection)
! *   Flow by temperature difference (Standard benchmark problem)
! *
! *   This program must be preceded by a call of sepmesh with input bouss.msh
! *   Input for this program bousscop.prb
! *   programmer Guus Segal
! *   date 19-10-91
! *
! ********************************************************************
! *
      program bousscop
      use sepmoduleoldrouts
      implicit none
      integer kmesh, kprob, iuser(100), intmat,
     .        isol(2), itertp, maxitr, iprint, ichois, iwork,
     .        ielhlp, istop, iinder(4)
      double precision user(100), eps, work
      data iinder /4*0/

!
!   Array isol is filled as follows:
!
!     isol(.,1):  information concerning the solution array (velocity
!                                                            and temperature)
!     isol(.,2):  information concerning the pressure

      iuser(1) = 100
      user(1) = 100
      kmesh = 0
      kprob = 0
      isol = 0
      intmat = 0
      iuser(3) = 0
      user(3) = 0

!    *  Read mesh and input for problem

      call sepstr ( kmesh, kprob, intmat )

!    *  Definition of boundary conditions

      call presdf ( kmesh, kprob, isol )

!   *    solve the system of non-linear equations

      itertp = 1
      maxitr = 10
      eps    = 1d-4
      iprint = -2
      ichois = -1
      call nonlin ( itertp, maxitr, eps, iprint, iwork, work, intmat,
     .              ichois, iuser, user, kmesh, kprob, isol, ielhlp )

!    *   compute the pressure and store in isol(.,2)
!        iuser,user as required by nonlin (is still present)

      iinder(1) = 4
      iinder(4) = 1
      call deriv ( iinder, isol(2), kmesh, kprob, isol, iuser, user )

!   *  Write the computed solution and pressure to the file sepcomp.out
!      using subroutine output

      call output ( kmesh, kprob, isol )
      call finish(istop)
      end
!
! = = = = =
!
      subroutine filnln ( niter, iuser, user, kmesh, kprob, isol, relax,
     .                    differ )

!    *  help subroutine for NONLIN

      implicit none
      integer iuser(*), kmesh, kprob, isol, niter, iprob
      double precision user(*), relax, differ
      iprob = 1
      if ( niter.eq.0 ) then
         call filcof ( iuser, user, kprob, kmesh, iprob )
      else if ( niter.le.2 ) then
         call chncof ( iuser, user, kprob, kmesh, iprob )
      end if
      end
