! ********************************************************************
! *
! *   example for the solution of the uncoupled Boussinesq equations
! *   Square cavity problem  (Free convection)
! *   Flow by temperature difference (Standard benchmark problem)
! *
! *   This program must be preceded by a call of sepmesh with input bouss.msh
! *   Input for this program bousscop.prb
! *   programmer Guus Segal
! *   date 19-10-91
! *
! ********************************************************************
! *
      program boussdec
      use sepmoduleoldrouts
      use sepmodulecomio
      implicit none
      integer kmesh, kprob, iuser(100,2), intmat(2),
     .        isol(3), maxitr, i, iprob,
     .        istop, iold(2), irhsd(2),
     .        matr(2), iinbld(10), ipos, iinvec(6), massmt, iter,
     .        iinder(8)
      double precision user(100,2), eps, rinvec(1), diffuv, difft
      data iinbld /10*0/, iinvec/6*0/, iinder/8*0/

!
!   Array isol is filled as follows:
!
!     isol(.,1):  information concerning the velocity
!     isol(.,2):  information concerning the temperature
!     isol(.,3):  information concerning the pressure

      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0
      irhsd = 0
      do 100 i = 1, 2
         iuser(1,i) = 100
         user(1,i)  = 100
         iuser(3,i) = 0
         user(3,i)  = 0
100   continue

!    *  Read mesh and input for problem

      call sepstr ( kmesh, kprob, intmat )

!    *  Both the velocity vector and the temperature are initialised by
!       subroutine CREATE
!       In this way also the essential boundary conditions are filled

      call create ( 0, kmesh, kprob, isol )

!   *    solve the coupled system of non-linear equations in a decoupled way

      maxitr = 20
      eps    = 1d-4

      do 200 iter = 1, maxitr

!      *  At most maxitr iteration are carried out
!         First copy the old solution from ISOL into IOLD

         call copyvc ( isol(1), iold(1) )
         call copyvc ( isol(2), iold(2) )

!      *  In the first step the velocity is computed:

         if ( iter.eq.1 ) then

!          *  First iteration, fill arrays IUSER and USER for the
!             Navier-Stokes equations: IUSER(.,1) and USER(.,1)

            iprob = 1
            call filcof ( iuser(1,1), user(1,1), kprob, kmesh, iprob )

         end if

!       *  Build matrix and right-hand side

         iinbld(1)  = 10
         iinbld(2)  = 1
         iinbld(10) =  2
         call build ( iinbld, matr(1), intmat(1), kmesh, kprob,
     .                irhsd(1), massmt, isol(1), isol(1),
     .                iuser(1,1), user(1,1) )

!      *  Solve system of linear equations

         ipos = 0
         call solve ( ipos, matr(1), isol(1), irhsd(1),
     .                intmat(1), kprob )

!      *  In the second step the temperature is computed:

         if ( iter.eq.1 ) then

!          *  First iteration, fill arrays IUSER and USER for the
!             Convection-diffusion equations: IUSER(.,2) and USER(.,2)

            iprob = 2
            call filcof ( iuser(1,2), user(1,2), kprob, kmesh, iprob )

         end if

!       *  Build matrix and right-hand side
!          The just computed velocity is used

         call build ( iinbld, matr(2), intmat(2), kmesh, kprob,
     .                irhsd(2), massmt, isol(2), isol(1),
     .                iuser(1,2), user(1,2) )

!      *  Solve system of linear equations

         call solve ( ipos, matr(2), isol(2), irhsd(2),
     .                intmat(2), kprob )

!      *  Compare old and new solution:

         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(5) = 1
         iinvec(6) = 3
         call manvec ( iinvec, rinvec, isol(1), iold(1), iold,
     .                 kmesh, kprob )
         diffuv = rinvec(1)
         call manvec ( iinvec, rinvec, isol(2), iold(2), iold,
     .                 kmesh, kprob )
         difft  = rinvec(1)

         write(irefwr,110) iter, diffuv, difft
110      format(' Iteration',i5,' :'/
     .          ' Maximal difference in velocity is', d12.3/
     .          ' Maximal difference in temperature is', d12.3/ )

!      *  Stop the iteration if the differences are small enough

         if ( difft.lt.eps .and. diffuv.lt.eps ) go to 250

200   continue

!    *   compute the pressure and store in isol(.,3)
!        iuser,user as required by build (is still present)

250   iinder(1) = 8
      iinder(4) = 1
      iinder(8) = 2
      call deriv ( iinder, isol(3), kmesh, kprob, isol, iuser, user )

!   *  Write the computed solution and pressure to the file sepcomp.out
!      using subroutine output

      call output ( kmesh, kprob, isol )
      call finish(istop)
      end

      function func ( ichois, x, y, z )
      integer ichois
      double precision func, x, y, z
      if ( ichois.eq.1 ) then

!       *  ichois = 1, the temperature is set equal to 1-x

         func = 1 - x

      end if

      end
