c ****************************************************************************

      program opt_shape_sup
      implicit none
      integer (kind=8) pbuffr
      parameter ( pbuffr=5000000 )
      integer ibuffr
      common ibuffr(pbuffr)
      call freebsub ( ibuffr, ibuffr, pbuffr)
      end


c ----------------------------------------------------------------------
c  subroutine funcvect for computing the pressure from the
c  gradient of the potential (ICHOICE=1) and for
c  computing p^2/2 (ICHOICE=2)
c ----------------------------------------------------------------------
      subroutine funcvect(ichoice,ndim,coor,numnodes,uold,
     +                    nuold,result,nphys)
      implicit none
      integer ichoice,ndim,numnodes,nuold,nphys
      double precision coor(ndim,numnodes),uold(numnodes,nphys,nuold),
     +                 result(numnodes,*)

      integer i1
      double precision getconst,Frm2

      IF (ichoice.EQ.1) THEN
      Frm2=1d0/((getconst('Froude'))**2)
      do i1=1,numnodes
         result(i1,1)=0.5-0.5*(uold(i1,1,1)**2+uold(i1,2,1)**2)
     +                -Frm2*coor(2,i1)
      end do
      END IF

      IF (ichoice.EQ.2) THEN
      do i1=1,numnodes
         result(i1,1)=0.5*uold(i1,1,1)**2
      end do
      END IF

      end

c ----------------------------------------------------------------------
c  window function for wave damping
c ----------------------------------------------------------------------
      function win0(xc)
      implicit none
      double precision xc,pi,win0,xl,xr,bl,br

      pi=3.14159265
      xl=-4.4
      bl=2.2
      xr=15.4
      br=2.2

      win0=0d0
      IF (xl-bl.LE.xc.AND.xc.LE.xl) THEN
         win0=(1d0+cos(pi*(xc-xl)/bl))/2d0
      ELSE IF (xl.LT.xc.AND.xc.LT.xr) THEN
         win0=1d0
      ELSE IF (xr.LE.xc.AND.xc.LE.xr+br) THEN
         win0=(1d0+cos(pi*(xc-xr)/br))/2d0
      END IF

      end

c ----------------------------------------------------------------------
c  subroutine funcsolcr for the adjustment of the free boundary
c ----------------------------------------------------------------------
      subroutine funcsolcr ( coorol, coornw, uold, anorm, numnodes,
     +                       nphys, nuold )
      implicit none

      include 'SPcommon/clinsl'

      integer numnodes, nuold, nphys
      double precision coorol(2,numnodes), uold(numnodes,nphys,nuold),
     +                 coornw(2,numnodes), anorm(2,numnodes)
      integer i,i1,i2,it_ctr,getint,nm1,nm2,nm3,nm4
      double precision getvar,getconst,win0,win1,k,res,dres,rhs
      double precision E,Fr,Frm2,curv,cosa,h1,h2,h3,h3s,d2xds2,d2yds2,
     +                 trm(1:2000,1:4),nv(1:2),tv(1:2),pp,pm,lp,lm
      double precision ks,mu1,mu2,mu3,mu4,mu0,gam,nalc2,alcingr
      double precision grad(1:2000),sgr(1:2000),sgr1(1:2000),
     +                 alh(1:2000),all(1:2000),alc(1:2000)

c     print *,'numnodes,nphys,nuold ',numnodes,nphys,nuold
c     call prinrl(uold,numnodes,'uold')
c     call prinrl1(anorm,numnodes,2,'anorm')

c ----------------------------------------------------------------------
c  get cost functional value, froude number, iteration count
c ----------------------------------------------------------------------
      E    = getvar ('costfunc')
      Fr   = (getconst('Froude'))
      Frm2 = 1d0/(Fr**2)
      it_ctr=nitercom


c ----------------------------------------------------------------------
c  check orientation fs: should be left to right
c ----------------------------------------------------------------------
      IF (dsign(1d0,coorol(1,2)-coorol(1,1)).LE.0d0) THEN
         print *,'error in funcsolcr: orientation fs right to left'
      END IF


c ----------------------------------------------------------------------
c  compute gradient
c ----------------------------------------------------------------------
      grad(1)=0d0
      grad(numnodes)=0d0
      do i = 2, numnodes-1

c  --tangent lengths
         h1=SQRT((coorol(1,i)-coorol(1,i-1))**2
     +          +(coorol(2,i)-coorol(2,i-1))**2)
         h2=SQRT((coorol(1,i+1)-coorol(1,i))**2
     +          +(coorol(2,i+1)-coorol(2,i))**2)
         h3=0.5*SQRT((coorol(1,i+1)-coorol(1,i-1))**2
     +              +(coorol(2,i+1)-coorol(2,i-1))**2)

c  --p*Fr^{-2}*n.e_d
         cosa=(coorol(1,i+1)-coorol(1,i-1))/(2d0*h3)
         pm=0.5-0.5*(((uold(i,1,1)-uold(i-1,1,1))/h1)**2)
     +      -0.5*Frm2*(coorol(2,i)+coorol(2,i-1))
         pp=0.5-0.5*(((uold(i+1,1,1)-uold(i,1,1))/h2)**2)
     +      -0.5*Frm2*(coorol(2,i)+coorol(2,i+1))
         trm(i,1)=Frm2*cosa*0.5*(pm+pp)

c  --0.5*curv*p^2 (curv=1/R)
         d2xds2=(coorol(1,i+1)-coorol(1,i))/h2-
     +          (coorol(1,i)-coorol(1,i-1))/h1
         d2yds2=(coorol(2,i+1)-coorol(2,i))/h2-
     +          (coorol(2,i)-coorol(2,i-1))/h1
         curv=sqrt(d2xds2**2+d2yds2**2)/h3
         IF (d2yds2.LT.0d0) THEN
            curv=-curv
         END IF
         trm(i,2)=curv*((0.5*(pp+pm))**2)/2d0

c  --d_t(lambda d_t{phi})
         lp=0.5*uold(i+1,1,2)+0.5*uold(i,1,2)
         lm=0.5*uold(i-1,1,2)+0.5*uold(i,1,2)
         trm(i,3)=(1d0/h3)*
     +          (lp*(uold(i+1,1,1)-uold(i,1,1))/h2-
     +           lm*(uold(i,1,1)-uold(i-1,1,1))/h1)

      end do

      do i = 2, numnodes-1
         grad(i)=-trm(i,1)-trm(i,2)-trm(i,3)
      end do

c ----------------------------------------------------------------------
c  stabilized grad
c  sgr1 + mu1 sgr1_xxxx=grad
c  mu1:=mu1 h^4
c ----------------------------------------------------------------------
c  --set constants
       mu1=(5d0)

c  --set boundary values
      sgr1(1)=0d0
      sgr1(2)=0d0
      sgr1(numnodes)=0d0
      sgr1(numnodes-1)=0d0

c  --initial estimate
      do i = 3, numnodes-2
         sgr1(i)=grad(i)
      end do

c  --solve smoother
      res=1d2
      i1=0
      do while (i1.LE.1d5)
         res=0d0
         do i = 3, numnodes-2
            h3=0.5*SQRT((coorol(1,i+1)-coorol(1,i-1))**2
     +                 +(coorol(2,i+1)-coorol(2,i-1))**2)
            rhs=grad(i)
            rhs=rhs-mu1*(sgr1(i-2)+sgr1(i+2)-4d0*(sgr1(i-1)+sgr1(i+1)))
            dres=rhs-sgr1(i)*(1d0+6d0*mu1)
            res=res+dres**2
            sgr1(i)=rhs/(1d0+6d0*mu1)
         end do
         res=res/numnodes
         i1=i1+1
      end do

c ----------------------------------------------------------------------
c  window grad
c  sgr - mu2 (1-win0) sgr_xx = sgr1
c ----------------------------------------------------------------------

c  --set constants
      mu2=1d2

c  --set boundary values
      sgr(1)=0d0
      sgr(numnodes)=0d0

c  --initial estimate
      do i = 2, numnodes-1
         sgr(i)=sgr1(i)
      end do

c  --solve window
      res=1d1
      i1=0
      do while (res.GT.1d-4)
         res=0d0
         do i = 2, numnodes-1
            mu1=(1d0-win0(coorol(1,i)))*mu2
            h3=0.5*SQRT((coorol(1,i+1)-coorol(1,i-1))**2
     +                 +(coorol(2,i+1)-coorol(2,i-1))**2)
            rhs=sgr1(i)
            rhs=rhs+(mu1/(h3**2))*(sgr(i-1)+sgr(i+1))
            dres=rhs-sgr(i)*(1d0+(2d0*mu1)/(h3**2))
            res=res+dres**2
            sgr(i)=rhs/(1d0+(2d0*mu1)/(h3**2))
         end do
         i1=i1+1
      end do


c ----------------------------------------------------------------------
c  PRECONDITIONER 1
c  high wave number preconditioning
c ----------------------------------------------------------------------

c  --set constants
      mu1=1d0/((Frm2-1d0)**2)

c  --set boundary values
      alh(1)=0d0
      alh(numnodes)=0d0

c  --initial estimate
      do i = 2, numnodes-1
         alh(i)=mu1*sgr(i)
      end do

c  --solve preconditioner
      res=1d1
      i1=0
      do while (res.GE.1d-16.AND.i1.LE.5d3)
         res=0d0
         do i = 2, numnodes-1
            h3=0.5*SQRT((coorol(1,i+1)-coorol(1,i-1))**2
     +                 +(coorol(2,i+1)-coorol(2,i-1))**2)
            rhs=mu1*sgr(i)
            rhs=rhs+(mu1/(h3**2))*(alh(i-1)+alh(i+1))
            dres=rhs-alh(i)*(1d0+(2d0*mu1)/(h3**2))
            res=res+dres**2
            alh(i)=rhs/(1d0+(2d0*mu1)/(h3**2))
         end do
         i1=i1+1
      end do


c ----------------------------------------------------------------------
c  construct the correction alc
c ----------------------------------------------------------------------

      alc(1)=0d0
      alc(numnodes)=0d0
      do i = 2, numnodes-1
         alc(i)=alh(i)
      end do

c ----------------------------------------------------------------------
c  inner-prods, norms, and step-size
c ----------------------------------------------------------------------
c  --norm correction
      nalc2=0d0
      do i=1,numnodes-1
         IF(-2d0.LE.coorol(1,i).AND.coorol(1,i).LE.8d0) THEN
            h1=SQRT((coorol(1,i)-coorol(1,i-1))**2
     +             +(coorol(2,i)-coorol(2,i-1))**2)
            nalc2=nalc2+
     +            h1*(alc(i)**2+alc(i)*alc(i+1)+alc(i+1)**2)/3d0
         END IF
      end do

c  --inner product alfa1 and grad
      alcingr=0d0
      do i=1,numnodes-1
         h1=SQRT((coorol(1,i)-coorol(1,i-1))**2
     +          +(coorol(2,i)-coorol(2,i-1))**2)
         alcingr=alcingr+
     +          h1*((alc(i)*grad(i))/3d0+
     +              (alc(i)*grad(i+1)+alc(i+1)*grad(i))/6d0+
     +              (alc(i+1)*grad(i+1))/3d0)
      end do
      alcingr=abs(alcingr)

      gam=1d0

c ----------------------------------------------------------------------
c  boundary adjustment
c ----------------------------------------------------------------------
      do i = 1, numnodes
         h1=SQRT((coorol(1,i)-coorol(1,i-1))**2
     +          +(coorol(2,i)-coorol(2,i-1))**2)
         h2=SQRT((coorol(1,i+1)-coorol(1,i))**2
     +          +(coorol(2,i+1)-coorol(2,i))**2)
         do i1=1,2
            tv(i1)=(coorol(i1,i-1)*(-h2/(h1*(h1+h2)))+
     +              coorol(i1,i)  *((h1-h2)/(-h1*h2))+
     +              coorol(i1,i+1)*(h1/(h2*(h1+h2))))
         end do
         nv(1)=-tv(2)/sqrt(tv(1)**2+tv(2)**2)
         nv(2)=tv(1)/sqrt(tv(1)**2+tv(2)**2)

         coornw(1,i) = coorol(1,i)-gam*alc(i)*nv(1)
         coornw(2,i) = coorol(2,i)-gam*alc(i)*nv(2)
      end do


c ----------------------------------------------------------------------
c  write to file
c ----------------------------------------------------------------------
      open (43,file="res.txt",access='append',form='formatted')
      WRITE(43,'(I4,5E13.5)' ) it_ctr,E,alcingr,nalc2
      close(43)

      nm1=it_ctr/1000
      nm2=(it_ctr-(nm1*1000))/100
      nm3=(it_ctr-nm1*1000-nm2*100)/10
      nm4=(it_ctr-nm1*1000-nm2*100-nm3*10)

      IF (10*(it_ctr/10).EQ.it_ctr) THEN
         open(41,file="fs_"//char(48+nm1)//char(48+nm2)//char(48+nm3)//
     +                       char(48+nm4),
     +           access='sequential',form='formatted')

         do i=1,numnodes
            write(41,'(7 E15.6)') coornw(1,i),coornw(2,i),uold(i,1,3),
     +                            sgr(i),alh(i),all(i),alc(i)
         end do

         close(41)
      END IF

      open(42,file="fs_cur",
     +        access='sequential',form='formatted')

      do i=1,numnodes
         write(42,'(7 E15.6)') coornw(1,i),coornw(2,i),uold(i,1,3),
     +                         sgr(i),alh(i),all(i),alc(i)
      end do
      close(42)


c ----------------------------------------------------------------------
c  write to screen
c ----------------------------------------------------------------------
      WRITE(*,1001) it_ctr,E,alcingr,nalc2
1001  FORMAT(/,2x,I4,2x,'E=',E10.2,2x,'(alc,gr)=',E10.2,2x,
     +        '|alc|^2=',E10.2)

      end


c ----------------------------------------------------------------------
c  subroutine funccr for the adjustment of the bump
c ----------------------------------------------------------------------

      subroutine funccr ( coorol, coornw, uold, inodp, ilocal,
     +                    anorm, n )
      implicit none
      include 'SPcommon/clinsl'
      integer n, inodp, ilocal
      integer i,it_ctr
      double precision coorol(2,*), coornw(2,*), uold(*), anorm(2,*)
      double precision getvar,getconst,X,Y,L,T

      it_ctr=nitercom

      L=4.4
      T=it_ctr/5d0
      do i=1,n
         coornw(1,i)=coorol(1,i)
         IF (T.LE.1d0.AND.0d0.LE.coorol(1,i).AND.coorol(1,i).LE.L) THEN
            X=coorol(1,i)
            Y=-1.0+6.75*((0.44*T)/(L**3))*X*(X-L)*(X-L)
            coornw(2,i)=Y
         ELSE
            coornw(2,i)=coorol(2,i)
         END IF
      end do

      end

      subroutine elem2 ( coor, elemmt, elemvc, elemms, iuser, user,
     +                   uold, matrix, vector, index1, index2, index3,
     +                   index4, notmas, numold )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 30-08-1999 Copy input parameters for eldervsubr
c        version 1.0    date 15-10-1998
c
c
c   copyright (c) 1998-1999  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c      User element subroutine to be used by SEPRAN
c      This is the general version, which produces an error message if it is
c      called except in the case of the Numerical Analysis Lab of Delft
c      University of Technology
c **********************************************************************
c
c                       KEYWORDS
c
c    matrix
c    vector
c    element
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
      implicit none
      include 'SPcommon/cactl'
      include 'SPcommon/cmacht'
      include 'SPcommon/celint'
      include 'SPcommon/cconst'

c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer iuser(*), numold, index1(inpelm), index2(icount),
     +        index3(numold,*), index4(numold,inpelm), notmas
      double precision coor(ndimlc,*), elemmt(icount,icount),
     +                 elemvc(icount), elemms(icount), user(*), uold(*)
      logical matrix, vector

c     coor       i    array of length ndim x npoint containing the co-ordinates
c                     of the nodal points with respect to the global numbering
c     elemms     o    Element mass matrix to be filled by the user
c     elemmt     o    Element matrix to be filled by the user
c     elemvc     o    Element vector to be filled by the user
c     index1     i    Array of length inpelm containing the point numbers
c                     of the nodal points in the element
c     index2     i    Array of length icount containing the positions of the
c                     degrees of freedom in the element with respect to usol
c     index3     i    Two-dimensional integer array of length NUMOLD x NINDEX
c                     containing the positions of the "old" solutions in array
c                     UOLD with respect to the present element
c                     For example UOLD(INDEX3(i,j)) contains the j th
c                     unknown with respect to the i th old solution vector.
c                     The number i refers to the i th vector corresponding to
c                     IVCOLD in the call of SYSTM2 or DERIVA
c     index4     i    Two-dimensional integer array of length NUMOLD x INPELM
c                     containing the number of unknowns per point accumulated
c                     in array UOLD with respect to the present element.
c                     For example INDEX4(i,1) contains the number of unknowns
c                     in the first point with respect to the i th vector stored
c                     in UOLD.
c                     The number of unknowns in the j th point with respect to
c                     i th vector in UOLD is equal to
c                     INDEX4(i,j) - INDEX4(i,j-1)  (j>1)
c     iuser      i    Integer user array to pass user information from
c                     main program to subroutine. See STANDARD PROBLEMS
c     matrix     i    if true the element matrix must be filled
c     notmas    i/o   Indicates whether the element mass matrix is zero (1) or
c                     not (0)
c     numold     i    Number of vectors that are stored in UOLD
c     uold       i    Array containing the previous iteration (old solution)
c     user       i    Real user array to pass user information from
c                     main program to subroutine. See STANDARD PROBLEMS
c     vector     i    if true the element vector must be filled
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer lenhome, lenarray, lenmatrix, i, j, ndimcopy, inpelmcopy,
     +        icountcopy, itypecopy
      parameter ( lenarray=100, lenmatrix=1000 )
      double precision x(2),y(2),h1
      double precision phi(2),p(2)
      double precision getconst,Frm2,pc
c      double precision x(lenarray), u(lenarray), elemcopy(lenmatrix)
c      character * 80 filenm
c      logical check
c     check       Indicates if the file exist (true) or not
c     elemcopy    Copy of the transpose of the element matrix
c     filenm      Name of file
c     i           Counting variable
c     icountcopy  Copy of icount
c     inpelmcopy  Copy of inpelm
c     itypecopy   Copy of itype
c     j           Counting variable
c     lenarray    Length of arrays u and x
c     lenhome     Number of characters in the name of the SEPRAN home directory
c     lenmatrix   Length of array elemcopy
c     ndimcopy    Copy of ndimlc
c     u           array of length m containing the values of vector1 in the
c                 nodal points of the element
c     x           Contains the coordinates of the nodes of the element using the
c                 local node numbering
c                 The sequence is: first all x-coordinates, then all
c                 y-coordinates
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ELEMSUBR  Special user element subroutine to be called by elem2 and
c               meant for the numerical analysis lab at Delft University
c               of Technology
c     ERCLOS    Resets old name of previous subroutine of higher level
c     EROPEN    Produces concatenated name of local subroutine
c     ERRCHR    Put character in error message
c     ERRINT    Put integer in error message
c     ERRSUB    Error messages
c     INSTOP    Stop the program
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     349   Subroutine not available
c     550   array length too short
c **********************************************************************
c
c                       PSEUDO CODE
c
c    If ( file sphome/bin/update/wi211 exists ) then
c       prepare call to elemsubr
c       call elemsubr
c       translate back
c    else
c       give error message
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c        --- Copy co-ordinates from coor into x and y

         do i = 1,2
            x(i) = coor(1,index1(i))
            y(i) = coor(2,index1(i))
         end do

c        --- element length and tangent length

         h1=sqrt((x(2)-x(1))**2+(y(2)-y(1))**2)

c        --- Copy old solution into phi and p

         do i = 1,2
            phi(i) = uold(index3(1,i))
              p(i) = uold(index3(4,i))
c            print *,'phi(i),p(i)', i,phi(i),p(i)
         end do

c         ndimcopy = ndimlc
c         inpelmcopy = inpelm
c         icountcopy = icount
c         itypecopy = itype
c
c
c     +                   elemvc, elemms, u, itypecopy )
c
c        --- Store the transpose of the element matrix in elemmt

         do i = 1,2
            do j = 1,2
               elemmt(i,j) = 0d0
            end do
         end do

         Frm2=1d0/((getconst('Froude'))**2)
         pc=0.5-0.5*(((phi(2)-phi(1))/h1)**2)
     +         -Frm2*(0.5*y(1)+0.5*y(2))
         elemvc(2)=-pc*(phi(2)-phi(1))/h1
         elemvc(1)=-elemvc(2)


c     --- Standard situation: give error message

      end
