      subroutine freebsub
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.0  date 30-01-2016 Remove parameters
!        version  2.8  date 22-11-2012 Long integers
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Solve free boundary problems with SEPRAN
!   Not only the problem is solved but also the mesh is created
!   Output is written to the files meshoutput, sepcomp.inf and sepcomp.out
!   for post-processing purposes
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iinstr(5)
      integer :: i, ipos, istop, maxcall

!     i              Counting variable
!     iinstr         Input array defined by the user for subroutine sepstmnew
!                    With this array the user may indicate which actions
!                    he wants to be performed by this subroutine
!                    IINSTR must be filled as follows:
!                    Pos. 1: Last entry number that has been filled by the
!                            user. If 0 or 1 only defaults are used
!                         2: IREAD Indicates if all SEPRAN input until end
!                                  of file or END_OF_SEPRAN_INPUT is read (1)
!                                  or that only the input as described for
!                                  SEPSTR is read (0)
!                            Default value: 0
!                         3: -
!     ipos           Position of input arrays in sepmoduleinput
!     istop          Indication if file 2 must be closed and saved (#0)
!                    or not (0)
!     maxcall        Maximum value of icall in sepmoduleinp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINISH         Stop SEPRAN
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPSTMNEW      Open SEPRAN and create mesh
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
!     TOFREESB       Executes all work of FREEBSUB except for the reading of
!                    mesh and problem
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear all arrays
!   Open SEPRAN, read mesh (SEPSTL)
!   Read rest of input and store in IBUFFR
!   Fill essential boundary conditions if required
!   The remainder of the tasks is carried out by TO0094
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Initialize all arrays

      call startsepran

!     --- Open SEPRAN and read mesh

      iinstr(1) = 5
      iinstr(2) = 1
      iinstr(3) = 0
      iinstr(4) = 0
      iinstr(5) = 3
      call sepstmnew ( iinstr, maxmesh )
      if ( ierror/=0 ) go to 1000
      mainsubr = 3
      call eropen ( 'freebsub' )

      removesol = .true.
      removeintmat = .true.

!     --- Rest of the tasks are carried out by tofreesb
!         First compute starting address of arrays inpsep and rinsep

      call sepgetinpfinf ( 10, 1, ipos, maxcall )

!     --- Actual subroutine

      call tofreesb ( in(ipos)%iinput, re(ipos)%rinput )

      print *, 'sepfree has finished'
1000  call erclos ( 'freebsub' )

!     --- Close all files, including file 2 if necessary

      istop = 0
      if ( inew/=0 ) istop = 1
      call finish ( istop )

      end subroutine freebsub
