      subroutine inmt00 ( n, intmt1, intmt2, amat1, amat2, thresh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-01-1992
!
!   copyright (c) 1992-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print all differences between matrix 1 and 2 exceeding the threshold
!   value thresh
!   Restriction: At this moment only metod1 = metod2 = 5 is implemented
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, intmt1(*), intmt2(*)
      double precision amat1(*), amat2(*), thresh

!     amat1   i   Lower triangle of matrix first matrix
!     amat2   i   Lower triangle of matrix second matrix
!     intmt1  i   INTMAT part 1 for matrix 1 and 2
!     intmt2  i   INTMAT part 2 for matrix 1 and 2
!     n       i   Number of rows in the matrix
!     thresh  i   Threshold value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, imat, icount, jcount, icol
      double precision dif

!    dif     Difference between elements of the two matrices
!    i       general loop variable
!    icol    Column number
!    icount  Number of differences found in the diagonal
!    imat    Position in the matrix
!    j       general loop variable
!    jcount  Number of differences found in the under triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Compare diagonal
!   For each row
!      Compare elements in lower triangle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compare  matrix

      imat = 0
      jcount = 0
      do i = 2, n
         do j = 1, intmt1(i+1) - intmt1(i)
            icol = intmt2(intmt1(i)+j)
            imat = imat + 1
            dif = abs ( amat1(imat) - amat2(imat) )
            if ( dif>thresh ) then

!           --- difference exceeds threshold value

               if ( jcount==0 ) then

!              --- jcount = 0, print heading

                  write(irefwr,160)
160               format( ' The following differences have been found', &
                          ' in the lower triangle:'/ &
                          '    row    column    difference  ', &
                          '    matrix 1         matrix 2  '/ )
               end if
               write(irefwr,170) i, icol, dif, amat1(imat), amat2(imat)
170            format ( i7, ':',  i7,':',ss, 1pe14.2e2, 2(1pe19.8e2) )
               jcount = jcount + 1
            end if
         end do
      end do
      if ( jcount>0 ) write(irefwr,210) jcount
210   format( 'Number of differences found is ', i4)
      end
