      subroutine marchn ( iprmar, rprmar, kmeshlc, kproblc, intmatlc, &
                          numveclc, isol, isolol, iuserloc, userloc, &
                          iwrkit, wrkit, ielhlp )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.26 date 29-05-2014 Extension of timebefore
!        version  2.25 date 05-12-2012 Long integers
!        version  2.24 date 24-06-2002 Call to seprmprobinfo
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   time-stepping (or false time-stepping) for sets of nprob equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleuser
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rprmar(*), wrkit(*)
      integer iprmar(*), numveclc, &
              isol(*), isolol(*), iwrkit(*), ielhlp(*)
      integer, target :: intmatlc(*), kmeshlc, kproblc, iuserloc(*)
      double precision, target :: userloc(*)

!     ielhlp  i    This array of length nelgrp+5 is only used if allelements = 1
!                  ielhlp(ielgrp+5) = 0 means elements of element group ielgrp
!                  are skipped, ielhlp(ielgrp+5) = 1, means that they are
!                  assembled
!     intmatlc i   Standard SEPRAN array of length 5 containing information
!                  concerning the structure of the matrix. See the Programmer's
!                  Guide 13.5 for a complete description.
!     iprmar  i    integer array with integer parameters
!                  length: iprmar(1)+1
!                  numipr=iprmar(1) number of integer parameters
!                  itytim=iprmar(2) type of time integration (default=0)
!                        =-1-nstep,..,0 false time-step method
!                        =1 linear equations
!                        =2 nonlinear equations
!                  ichtim=iprmar(3) time step selection (default=0)
!                        =0 standard (time step prescribed)
!                        =1 time step may be decreased/increased
!                  ntotal=iprmar(4) total number of time steps (default=10)
!                  iter =iprmar(5) iterations for nonlinear equations
!                         (default=1)
!                  ioutq =iprmar(6) output selection (default=0)
!                        =0 no intermediate output
!                        =1 every ioutq'th time step isol to backing storage
!                  iprint=iprmar(7) output of messages (default=1)
!                  jincrm=iprmar(8) residual form of equations
!                        =0 equations solve the unknowns (default)
!                        =1 equations solve the increment of the unknowns
!                           (nonl. eq)
!     isol   i/o   Standard SEPRAN array containing information of the solution
!                  array
!     isolol  o    Corresponds to the solution of the preceding iteration
!     iuserloc i   Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     iwrkit  i    not used
!     kmeshlc i    Standard SEPRAN array containing information of the mesh
!     kproblc i    Standard SEPRAN array, containing information of the problem
!     numveclc  i    number of vectors in isol,isolol (>= nprob)
!     rprmar  i    real array with real parameters
!                  length: rprmar(1)+1
!                  numrpr=rprmar(1) number of real parameters
!                  deltat=rprmar(2) time step
!                  t0    =rprmar(3) initial time (default=0) / actual time
!                  theta =rprmar(4) for theta method (default=0.5)
!                  eps   =rprmar(5) criterion for truncation for
!                                   false time-step method
!                  epsusr=rprmar(6) criterion for self-selecting time step
!                                   for ichtim=1 (default=0.1)
!     userloc i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     wrkit   i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i    , idivg,  idum, iposlv, leng, &
              iprint, itytim, jincrm, jtime,  nstep,  &
              iter, ntotal, numipr, numrpr
      integer ioutq,  iprec, numold,  &
              jprob,  jtype,  ndegfd, ipusri, ipusrr
      logical decrdt, incrdt,  ready,  iexit
      double precision cd      ,  cr     ,  deltat,  &
                       difnew(9),  difn1(9),  difn2(9),  difold(9),  &
                       dif0(9) ,  dum    ,  difhlp(9),  energy(9),  &
                       eps     ,  epsusr ,    omegac ,  &
                       resnew(9),  resold(9),  res0(9),  snorm(9),  &
                       thetam,  a,  b,  p,  q
      integer   isolit(9), iresid(9), ihelp(9), inorm(9), icompl
      integer   iinbldlc(15)
      integer, target :: massmtr, matrix(9)
      logical compl, relatv,  residu,  separ, new
      save omegac, cd, cr
      type (intmatrix), pointer :: IM
      double precision :: timebefore(2)

!     IM             Refers to intmt(intmat)
!     a
!     b
!     cd
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     cr
!     decrdt
!     deltat
!     dif0
!     difhlp
!     difn1
!     difn2
!     difnew
!     difold
!     dum            General double precision dummy variable
!     energy
!     eps            Local accuracy
!     epsusr         required accuracy for the solution
!     i              Counting variable
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     idivg
!     idum           General dummy variable
!     iexit
!     ihelp
!     iinbldlc       User input array for subroutine BUILD
!                    iinbld is filled as follows:
!                    1  Number of entries of IINBLD that are filled by the user.
!                       If IINBLD(1) = 0, only defaults are used
!                    2  JCHOIS    general choice parameter, possibilities:
!                              1  Both the large matrix and large vector are
!                                 built.
!                              2  Only the large vector is built.
!                              3  The large matrix is built and the large vector
!                                 is made equal to zero.
!                                 If EFFBOUNCOND = 0 the large vector may be
!                                 unequal to zero, due to
!                                 non-homogeneous essential boundary conditions.
!                              5  The large matrix is not built;
!                                 the large vector is made equal to zero.
!                                 If EFFBOUNCOND = 0 the large vector may be
!                                 unequal to zero, due to
!                                 non-homogeneous essential boundary conditions.
!                             14  The large matrix is built;
!                                 no right-hand-side vector is made.
!                    3  EFFBOUNCOND   Indication whether the effect of essential
!                                 boundary  conditions must be taken into
!                                 account (0) or not (1).
!                    4  IMAS      Indication if the mass matrix must be computed
!                                 (>1) or not (0)
!                                 If IMAS=1, the mass matrix is stored as a
!                                 diagonal matrix, if IMAS=2 the mass matrix is
!                                 stored as a large matrix. The structure of
!                                 this matrix must be stored in INTMAT(.,2)
!                    5  IPREC     Indication if the old solution is stored in
!                                 ISLOLD (1) or not (0)
!                                 IPREC is not used if IINBLD(10)>0
!                    6  ALLELEMENTS  Indication if all standard element
!                                 sequence numbers are used (0) or not (1)
!                                 In that case the positions 16 to 15+NELGRP+ &
!                                 NUMNATBND   (NELGRP is number of element
!                                 groups and NUMNATBND the number of boundary
!                                 groups) are considered as an array IELHLP to
!                                 element be filled by the user in the
!                                 following way:
!                                 In IELHLP(IELGRP)
!                                 (1<=IELGRP<=NELGRP+NUMNATBND)
!                                 the user must store, whether elements with
!                                 standard element sequence number IELGRP (or
!                                 IELGRP = NELGRP + IBNGRP for boundary
!                                 elements) are added to the large matrix and
!                                 large vector or not.
!                                 When IELHLP(IELGRP) = 0 the elements with
!                                 standard element sequence number IELGRP are
!                                 skipped, &
!                                 when IELHLP(IELGRP) = 1 these elements are
!                                 added.
!                    7  ADDMATVEC    When ADDMATVEC = 0 the large matrix and
!                                 vector are cleared before the assembling
!                                 process.
!                                 When ADDMATVEC = 1 the right-hand-side vector
!                                 and the large matrix are not cleared, &
!                                 but added to the already existing right-hand
!                                 side vector and large matrix.
!                                 Hence        RHSD := RHSD + DELTA  RHSD
!                                              MATRIX := MATRIX +  DELTA  MATRIX
!                    8   CLEARLASTRHSD  When CLEARLASTRHSD = 0 the last part of
!                                 the right-hand-side vector corresponding to
!                                 the boundary conditions is cleared
!                                 (standard situation)
!                                 when CLEARLASTRHSD = 1 the standard assembling
!                                 process is also applied to this last part of
!                                 the right-hand-side vector. For most
!                                 applications CLEARLASTRHSD = 0 suffices.
!                    9   IPROB    Problem definition number
!                   10   NUMOLD   Number of arrays that have been filled in
!                                 array ISLOLD
!                   11   ISPECTRAL Indication if a spectral element method is
!                                 used needing extra information (1) or not (0)
!                   12   NRIGHTHS Number of right-hand sides that must be
!                                 computed by subroutine BUILD in this call.
!                                 This parameter is only used if JCHOIS = 1 or 2
!                                 For all these vectors the other parameters in
!                                 IINBLD are used in exactly the same way.
!                                 The information about the right-hand sides is
!                                 stored in the following way:
!                                 f  refers to rhsd(.,i)  . = 1 (1) 5
!                                  i
!     incrdt
!     inorm
!     ioutq
!     iposlv
!     iprec          Surface number of surface to be copied etc.
!     iprint
!     ipusri
!     ipusrr
!     iresid
!     isolit         Corresponds to the iteration vector. In the iteration
!                    vector the solution of one iteration for one equation is
!                    stored
!     iter           Iteration sequence number
!     itytim         Consists of two parts: numprb and linear according to
!                    itytim = linear + 10 * (numprb-1)
!                    The meaning of these parameters is:
!                    linear indicates if the matrices are linear (1) or not (2)
!                    numprb defines the number of coupled problems to be solved
!                    in one call of this subroutine.
!                    numprb has only been implemented for ichtim = 5
!     jincrm
!     jprob
!     jtime
!     jtype          Help parameter to store reduced type number
!     leng           size of array
!     massmtr
!     matrix         input matrix
!     ndegfd         Number of degrees of freedom per point
!     new
!     nstep
!     ntotal
!     numipr
!     numold         Number of vectors that are stored in VECOLD
!     numrpr
!     omegac          Relaxation parameter
!     p
!     q
!     ready          Indicates if the loop has been finished (true) or not
!                    (false)
!     relatv
!     res0
!     residu
!     resnew
!     resold
!     separ
!     snorm
!     thetam
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision sepnorm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     FILMAR         user routine that may apply boundary conditions and/or
!                    fill iuser/user  and/or  adapt omega  and/or write
!                    to backing storage, etc.
!     INSTOP         Stop the program
!     PRINTTIME      Print the CPU time at end of subroutine
!     SEPALGEBR      algebraical operations on SEPRAN vectors
!     SEPBUILD       Build matrix and right-hand side
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPMAVER       perform matrix vector multiplication
!     SEPNORM        compute norm of vector or norm of difference of two vectors
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLDIR      Actual body of subroutine SOLVE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     241   Relaxation parameter has wrong value
!    1007    Parameter of integer input array has illegal value
!    1008    Parameter of real    input array has illegal value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data omegac,cd,cr/1.0d0, 1.02d0, 1.02d0/
! ======================================================================
!
!     --- initialize omegac=relaxation factor
!              cd    percentage divergence of difference allowed
!              cr    percentage divergence of residual   allowed

      call eropen( 'marchn' )
      if ( itime>0 ) call ersettime ( timebefore )

      stiffmat => matrix
      massmat => massmtr
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)
      IM => intmt(intmatlc(1))

!     --- initialize dif0(iprob), difold(iprob), ..

      idivg =0
      incrdt=.false.
      decrdt=.false.
      iexit=.false.
      inpsol(1,1) = 2
      if (max(nprob,numveclc)>9) then
         write(irefwr,90) nprob,numveclc
 90      format('  ERROR in subroutine MARCHN'/ &
                ' dimensions of local arrays too small for nprob,', &
                ' numveclc=',2i8 / &
                ' maximum value must not exceed 9')

         call instop
      end if
      do iprob=1,nprob
         dif0(iprob)   = 2d16
         difold(iprob) = 1d16
         res0(iprob)   = 2d16
         resold(iprob) = 1d16
         resnew(iprob) = 1d16
      end do

!     --- set parameter defaults
!         itytim=iprmar(2), ichtim=iprmar(3), ntotal=iprmar(4), &
!         iter =iprmar(5), ioutq =iprmar(6), iprint=iprmar(7), &
!         jincrm=iprmar(8)
!         deltat=rprmar(2), t0    =rprmar(3), thetam=rprmar(4), &
!         eps   =rprmar(5), epsusr=rprmar(6)
!         analyze input parameters iprmar/rprmar (and initialize defaults)

      numipr = iprmar(1)
      numrpr = nint(rprmar(1))
      itytim = 0
      ichtim = 0
      ntotal = 10
      iter  = 1
      ioutq  = 0
      iprint = 1
      jincrm = 0
      if (numipr<0) then
         call errint(1,1)
         call errchr('IPRMAR',1)
         call errint(numipr,2)
         call errsub ( 1007, 2, 0, 1)
      else if (numipr>=7) then
         itytim = iprmar(2)
         ichtim = iprmar(3)
         ntotal = iprmar(4)
         iter  = iprmar(5)
         ioutq  = iprmar(6)
         iprint = iprmar(7)
         jincrm = iprmar(8)
      else if (numipr>=6) then
         itytim = iprmar(2)
         ichtim = iprmar(3)
         ntotal = iprmar(4)
         iter  = iprmar(5)
         ioutq  = iprmar(6)
         iprint = iprmar(7)
      else if (numipr==5) then
         itytim = iprmar(2)
         ichtim = iprmar(3)
         ntotal = iprmar(4)
         iter  = iprmar(5)
         ioutq  = iprmar(6)
      else if (numipr==4) then
         itytim = iprmar(2)
         ichtim = iprmar(3)
         ntotal = iprmar(4)
         iter  = iprmar(5)
      else if (numipr==3) then
         itytim = iprmar(2)
         ichtim = iprmar(3)
         ntotal = iprmar(4)
      else if (numipr==2) then
         itytim = iprmar(2)
         ichtim = iprmar(3)
      else if (numipr==1) then
         itytim = iprmar(2)
      end if
      nstep = 0
      if (itytim<-1-nstep .or. itytim>1) then
         call errint(2,1)
         call errchr('IPRMAR',1)
         call errint(itytim,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      if (ichtim<0 .or. ichtim>1) then
         call errint(3,1)
         call errchr('IPRMAR',1)
         call errint(ichtim,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      if (ntotal<0) then
         call errint(4,1)
         call errchr('IPRMAR',1)
         call errint(ntotal,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      if (iter <0 .and. itytim==2) then
         call errint(5,1)
         call errchr('IPRMAR',1)
         call errint(iter,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      if ((ioutq<0).or.(ioutq>ntotal)) then
         call errint(6,1)
         call errchr('IPRMAR',1)
         call errint(ioutq,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      if ((iprint<-3).or.(iprint>3)) then
         call errint(7,1)
         call errchr('IPRMAR',1)
         call errint(iprint,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      if ((jincrm<0).or.(jincrm>1)) then
         call errint(8,1)
         call errchr('IPRMAR',1)
         call errint(jincrm,2)
         call errsub ( 1007, 2, 0, 1)
      end if
      t0       = 0.0d0
      thetam   = 0.5d0
      eps      = 0.001d0
      epsusr   = 0.1d0
      if (numrpr<1) then
         call errint(1,1)
         call errchr('RPRMAR',1)
         call erreal(dble(numrpr),1)
         call errsub ( 1008, 1, 1, 1)
      else if (numrpr>=5) then
         deltat   = rprmar(2)
         t0       = rprmar(3)
         thetam   = rprmar(4)
         eps      = rprmar(5)
         epsusr   = rprmar(6)
      else if (numrpr>=4) then
         deltat   = rprmar(2)
         t0       = rprmar(3)
         thetam   = rprmar(4)
         eps      = rprmar(5)
      else if (numrpr==3) then
         deltat   = rprmar(2)
         t0       = rprmar(3)
         thetam   = rprmar(4)
      else if (numrpr==2) then
         deltat   = rprmar(2)
         t0       = rprmar(3)
      else if (numrpr==1) then
         deltat   = rprmar(2)
      end if
      if (deltat  <0) then
         call errint(2,1)
         call errchr('RPRMAR',1)
         call erreal(deltat,1)
         call errsub ( 1008, 1, 1, 1)
      end if
      if (thetam  <0.5d0 .or. thetam>1.0d0) then
         call errint(4,1)
         call errchr('RPRMAR',1)
         call erreal(thetam,1)
         call errsub ( 1008, 1, 1, 1)
      end if
      if (eps     <0) then
         call errint(5,1)
         call errchr('RPRMAR',1)
         call erreal(eps,1)
         call errsub ( 1008, 1, 1, 1)
      end if
      if (epsusr  <0) then
         call errint(6,1)
         call errchr('RPRMAR',1)
         call erreal(epsusr,1)
         call errsub ( 1008, 1, 1, 1)
      end if

!     --- end of tests

      relatv=.true.
      separ =.true.
      residu=.true.

!     --- initialize extra arrays

      isolit=0
      irhsd=0
      iresid=0
      ihelp=0
      inorm=0
      matrix=0
      jtime=itime
      itime=0
      nstep=0

      iseqkp = kprob

!     --- if isol(iprob) did not exist create it

      do iprob=1,nprob
         intmat => intmatlc(iprob)
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kprobc => kprobpart%kprobc
         iseqim = intmat
         IM => intmt(intmat)
         if (IM%symmetric) then
            iposlv=1
         else
            iposlv=0
         end if
         iter=0
         compl=IM%complexm
         new = isol(iprob)<=0 .or. isol(iprob)>lastsol
         if ( .not. new ) new = solarr(isol(iprob))==0
         if ( new ) then
            if (.not.compl) then
               icompl=0
            else
               icompl=1
            end if
            nusol=kprobloc(5)
            call sepcreasol110 ( isol(iprob), 'isol', icompl )
            usoltemp = 0d0

         end if

         new = isolol(iprob)<=0 .or. isolol(iprob)>lastsol
         if ( .not.new ) new = solarr(isolol(iprob))==0
         if ( new ) then
            call sepcopyvc (isol(iprob), isolol(iprob) )
         end if
      end do

!     --- copy initial value to ihelp ; initialize isolit

      do iprob=1,max(nprob,numveclc)
         if (iprob<=nprob) &
            call sepcopyvc (isol(iprob),ihelp(iprob))
         call sepcopyvc (isolol(iprob),isolit(iprob))
      end do

!     --- step 1 : (re)fill iuser and user with coefficients and parameters
!             (and essential boundary conditions in isol(iprob))

100   do iprob=1,nprob
      kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
      if (iprob==1) then
         ipusri=1
         ipusrr=1
         iuser => iuserloc(ipusri:iuserloc(1))
         user => userloc(ipusrr:nint(userloc(1)))
      else
         leng=iuserloc(ipusri)-iuserloc(ipusri+2)-5
         ipusri=ipusri+iuserloc(ipusri+2)+5
         iuserloc(ipusri)=leng
         iuser => iuserloc(ipusri:iuserloc(1))
         leng=nint(userloc(ipusrr)- userloc(ipusrr+2))-5
         ipusrr=ipusrr+ nint(userloc(ipusrr+2))+5
         userloc(ipusrr)=leng
         user => userloc(ipusrr:nint(userloc(1)))
      end if
      if (IM%symmetric) then
         iposlv=1
      else
         iposlv=0
      end if
      iter=0
      compl=IM%complexm

!     --- (re)fill ihelp with boundary
!         conditions; at first step isolit may be refilled with initial value

      if (jincrm==0) then
         call filmar(nstep,iprob,numveclc,iuser,user, &
                  kmesh,kprob,isol,isolit,omegac,difold(iprob))
      else if (jincrm==1) then
         call filmar(nstep,iprob,numveclc,iuser,user, &
                  kmesh,kprob,ihelp,isolit,omegac,difold(iprob))

!        --- for time-stepping boundary conditions
!            isol(iprob)=(ihelp(iprob)) - isolit(iprob)

         a=1d0
         b=-1d0
         call sepalgebr (3,0,ihelp(iprob), &
                         isolit(iprob), isol(iprob), &
                         a, b, dum,dum,idum)
      end if
      if (max(nprob,numveclc)>9) then
         write(irefwr,110) nprob,numveclc
110      format('  ERROR in subroutine MARCHN'/ &
                ' dimensions of local arrays too small for nprob,', &
                ' numveclc=', 2i8 / &
                ' maximum value must not exceed 9')
         call instop
      end if

!     --- copy (new) iterand into solold, i.e. allow that isolit has been set
!         to the initial condition in filmar at nstep=0

      if (nstep==0) &
         call sepcopyvc (isolit(iprob),isolol(iprob))

!     --- set-up theta and deltat for this time step

      if (iprob==1) then
         if (nstep>=0 .and. itytim==-nstep-1) then
            deltat=wrkit(nstep+1)
            theta =1.0d0
         else if (nstep>0 .and. itytim==-nstep) then
            deltat=rprmar(2)
            theta = thetam
         else if (nstep==0 .and. itytim==0) then
            deltat=rprmar(2)
            theta = 1.0d0
         else if (nstep>0 .and. itytim>=0) then
            theta = thetam
         end if
         if (incrdt .and. ichtim/=0 .and. nstep>2) then
            deltat=2*deltat
            incrdt=.false.
            write(irefwr,120) deltat
120         format(' warning from subroutine marchn'/ &
                   ' (false) time step increased, deltat=',e10.3)
         else if (decrdt .and. ichtim/=0 .and. nstep>2) then
            deltat=0.5d0*deltat
            decrdt=.false.
            write(irefwr,130) deltat
130         format(' warning from subroutine marchn'/ &
                   ' time step decreased, deltat=',e10.3)
         else
            incrdt=.false.
            decrdt=.false.
         end if
         t0=t0+deltat
      end if

!     --- step 3 : build system and solve

      jtype=kprobc(1)
      if ((jtype==440 .or. jtype==414 .or. jtype==14) .or. &
          (jtype==442 .or. jtype==416 .or. jtype==16) .or. &
           (jtype==446 .and. iprob<2) ) then

!     --- for test purposes

         iprec=0
         numold=max(1,nprob,numveclc)
      else if (nprob>=1) then
         iprec=0
         numold=max(nprob,numveclc)
      else if (numveclc>=1) then
         iprec=0
         numold=numveclc
      else
         iprec=1
         numold=0
      end if
      iinbldlc(1)=10
      iinbldlc(2)=1
      iinbldlc(3)=0
      iinbldlc(4)=0
      iinbldlc(5)=iprec
      iinbldlc(6)=0
      iinbldlc(7)=0
      iinbldlc(8)=1
      iinbldlc(9)=iprob
      iinbldlc(10)=numold
      iinbldlc(11)=0
      stiffmat => matrix(iprob:9)
      call sepfillsmf ( iinbldlc, irhsd(iprob), &
                        isol(iprob), isolit )
      call sepbuild
      if ((nstep<=2).and.(iprint/=0).and.(jtime==1)) &
         call printtime('build ', timebefore)

!     --- step 3a : determine residual for previous (false) time step

      if (itytim<=1 .and. residu .and. jincrm==0 .and. &
          nstep>0) then
         call sepmaver ( matrix(iprob), isolit(iprob), &
                         ihelp(iprob), 2 )

!        --- subtract irhsd(iprob) from ihelp(iprob)

         if (.not.compl) then
            a=1d0
            b=-1d0
            call sepalgebr (3,0,ihelp(iprob), &
                            irhsd(iprob), iresid(iprob), &
                            a, b,dum,dum,idum)
         else
            a=1d0
            b=1d0
            p=-1d0
            q=-1d0
            call sepalgebr (3,0,ihelp(iprob), &
                            irhsd(iprob), iresid(iprob), &
                            a, b, p, q,idum)
         end if

!        --- determine the norm of the residual

         call sepalgebr (7,0,isolit(iprob), &
                         isolit(iprob), isolit(iprob), &
                         snorm(iprob),dum,dum,dum,idum)
         snorm(iprob)=sqrt(snorm(iprob))

!        --- for the residual exclude the boundary conditions

         call sepalgebr (10,0,iresid(iprob), &
                         iresid(iprob), iresid(iprob), &
                         resnew(iprob),dum,dum,dum,idum)
         resnew(iprob)=sqrt(resnew(iprob))
         if (snorm(iprob)>epsmac) &
            resnew(iprob)=resnew(iprob)/snorm(iprob)
         if ((nstep<=2).and.(iprint/=0).and.(jtime==1)) &
            call printtime('maver', timebefore)
      end if

!     --- step 3b : solve for intermediate time step

      inpsol(2,1) = iposlv
      call sepsoldir ( matrix(iprob), &
                       isol(iprob), irhsd(iprob) )
      if ((nstep<=2).and.(iprint/=0).and.(jtime==1)) &
         call printtime('solve ', timebefore)

!     --- step 3c : determine residual for incremental solution

      if (jincrm==1 .and. residu) then

!     --- determine the norm of isolit(iprob)

         call sepalgebr (7,0,isolit(iprob), &
                         isolit(iprob), isolit(iprob), &
                         difn1(iprob),difn2(iprob),dum,dum,idum)
         snorm(iprob)=sqrt(difn1(iprob))

!        --- compute residual energy(iprob), &
!            inner prod: ( isolit(iprob), right-hand side)
!            average per point (excluding boundary conditions)

         call sepalgebr (10,0,isolit(iprob), &
                         irhsd(iprob), irhsd(iprob), &
                         difn1(iprob),dum,dum,dum,idum)
         if (snorm(iprob)>epsmac) &
         energy(iprob)=difn1(iprob)/snorm(iprob)

!        --- determine inner product of irhsd(iprob)
!            (including boundary conditions)

         call sepalgebr (7,0,irhsd(iprob), &
                         irhsd(iprob), irhsd(iprob), &
                         difn1(iprob),dum,dum,dum,idum)
         difn2(iprob)=sqrt(difn1(iprob))
         if (difn2(iprob)>eps) &
             energy(iprob)=energy(iprob)/difn2(iprob)

!        --- compute residual difference, i.e. inner prod: ( rhsd, rhsd )
!            (excluding boundary conditions)

         call sepalgebr (10,0,irhsd(iprob), &
                         irhsd(iprob), irhsd(iprob), &
                         difn1(iprob),dum,dum,dum,idum)
         resnew(iprob)=sqrt(difn1(iprob))
         if (difn2(iprob)>eps .and. &
            abs(resnew(iprob)-difn2(iprob))>sqreps) &
            resnew(iprob)=resnew(iprob)/difn2(iprob)
      end if

!     --- step 3d : extrapolate for next time step with theta method
!                   and prepare new isolit(iprob) which can be used
!                   in problem iprob+1

      omegac=1.0d0/theta
      if ((abs(omegac)<epsmac).or.(abs(omegac-1.0d0)<epsmac)) then
         if (jincrm==0) then
           call sepcopyvc (isol(iprob),isolit(iprob))
         else if (jincrm==1) then
            a=1d0
            b=1d0
            call sepalgebr (3,0,isol(iprob), &
                            isolol(iprob), isolit(iprob), &
                            a, b, dum,dum,idum)
         end if
      else
         if ((omegac<0.0d0).or.(omegac>2.0d0)) then
            call errsub ( 241, 0, 0, 0 )
            call instop
         end if
         if (.not.compl) then
            if (jincrm==0) then
               b=1d0-omegac
               call sepalgebr (3,0,isol(iprob), &
                               isolol(iprob), isolit(iprob), &
                               omegac,b,dum,dum,idum)
            else if (jincrm==1) then
               b=1d0
               call sepalgebr (3,0,isol(iprob), &
                               isolol(iprob), isolit(iprob), &
                               omegac,b,dum,dum,idum)
            end if
         else
            if (jincrm==0) then
               p=1d0-omegac
               q=1d0-omegac
               call sepalgebr (3,0,isol(iprob), &
                               isolol(iprob), isolit(iprob), &
                               omegac,omegac,p, q, idum)
            else if (jincrm==1) then
               p=1d0
               q=1d0
               call sepalgebr (3,0,isol(iprob), &
                               isolol(iprob), isolit(iprob), &
                               omegac,omegac,p,q,idum)
            end if
         end if
         if (jincrm==0) then
           call sepcopyvc (isolit(iprob),isol(iprob))
         end if
      end if
      end do

      nstep=nstep+1
      ready =.false.

!     --- step 4 : determine difference with old solution

      do iprob=1,nprob
      kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
      if (.not.separ) then
         if (jincrm==0) then
            difnew(iprob) =  sepnorm ( 3, 1, 0, isolol(iprob), &
                                      isol(iprob), idum )
         else if (jincrm==1) then

!        --- determine relative difference with old solution (increment)

            call sepalgebr (6,0,isol(iprob), &
                            isol(iprob), isol(iprob), &
                            difn1(iprob),difn2(iprob),dum,dum,idum)
            difnew(iprob)=max(abs(difn1(iprob)),difn2(iprob))
            difnew(iprob)=difnew(iprob)*abs(omegac)
            call sepalgebr (6,0,isolit(iprob), &
                            isolit(iprob), isolit(iprob), &
                            difn1(iprob),difn2(iprob),dum,dum,idum)
            difn2(iprob)=max(abs(difn1(iprob)),difn2(iprob))
            difnew(iprob)=difnew(iprob)/difn2(iprob)
         end if
         if (relatv .and. jincrm==0) then

!        --- determine minimum and maximum

            if (.not.compl) then
               call sepalgebr (6,0,isol(iprob), &
                               isol(iprob), isol(iprob), &
                               difn1(iprob),difn2(iprob),dum,dum,idum)
            else
               a=1d0
               call sepalgebr (8,0,isolit(iprob), &
                               inorm(iprob), inorm(iprob), &
                               a,dum,dum,dum,idum)
               call sepalgebr (6,0,inorm(iprob), &
                               inorm(iprob), inorm(iprob), &
                               difn1(iprob),difn2(iprob),dum,dum,idum)
            end if
            difnew(iprob)=difnew(iprob)/ &
                          max(abs(difn1(iprob)),difn2(iprob))
         end if
      else if (separ) then

!     --- determine difference vector and difference per degree of freedom

         if (.not.compl .and. jincrm==0) then
            a=1d0
            b=-1d0
            call sepalgebr (3,0,isol(iprob), &
                            isolol(iprob), ihelp(iprob), &
                            a, b, dum,dum,idum)
         else if (compl .and. jincrm==0) then
            a=1d0
            b=1d0
            p=-1d0
            q=-1d0
            call sepalgebr (3,0,isol(iprob), &
                            isolol(iprob), ihelp(iprob), &
                            a, b, p, q, idum)
         else if (jincrm==1) then
            call sepcopyvc (isol(iprob),ihelp(iprob))
         end if
         difnew(iprob)=0.0d0
         ndegfd=max(kprobloc(4),kprobloc(33))
         do i=1,ndegfd
            if (.not.compl) then
               call sepalgebr (6,i,ihelp(iprob), &
                               ihelp(iprob), ihelp(iprob), &
                               difn1(iprob),difn2(iprob),dum,dum,idum)
            else
               a=1d0
               call sepalgebr (8,i,ihelp(iprob), &
                               inorm(iprob), inorm(iprob), &
                               a, dum,dum,dum,idum)
               call sepalgebr (6,i,inorm(iprob), &
                               inorm(iprob), inorm(iprob), &
                               difn1(iprob),difn2(iprob),dum,dum,idum)
            end if
            difhlp(iprob)=max(abs(difn1(iprob)),difn2(iprob))
            if (relatv) then

!           --- determine minimum and maximum

               if (.not.compl) then
                  call sepalgebr (6,i,isolit(iprob), &
                                  isolit(iprob), isolit(iprob), &
                                  difn1(iprob),difn2(iprob),dum,dum, idum)
               else
                  a=1d0
                  call sepalgebr (8,i,isolit(iprob), &
                                  inorm(iprob), inorm(iprob), &
                                  a,dum,dum,dum,idum)
                  call sepalgebr (6,i,inorm(iprob), &
                                  inorm(iprob), inorm(iprob), &
                                  difn1(iprob),difn2(iprob),dum, &
                                  dum, idum)
               end if
               dum=max(abs(difn1(iprob)),difn2(iprob))
               if (dum>sqreps) difhlp(iprob)=difhlp(iprob)/dum
            end if
            if (difnew(iprob)<difhlp(iprob)) &
               difnew(iprob)=difhlp(iprob)
         end do
      end if
      if (abs(iprint)>=2) then
         if (nprob==1) then
            write(irefwr,140) nstep,difnew(iprob)
140         format(' marchn : nstep=',i5, &
                   '   new difference =',e12.3)
         else
            write(irefwr,150) nstep,difnew(iprob),iprob
150         format(' marchn : nstep=',i5, &
                   '   new difference =',e12.3,'    iprob=',i2)
         end if
         if (nstep>1) then
            if ( difnew(iprob)<epsmac .and. &
                 difold(iprob)<epsmac ) then
               difn2(iprob)=1.0d0
            else if (difold(iprob)>epsmac) then
               difn2(iprob)=difnew(iprob)/difold(iprob)
            else
               difn2(iprob)=difnew(iprob)
            end if
            if (abs(iprint)>=3 ) &
               write(irefwr,160) difn2(iprob)
160            format(21x, &
                      '   difnew/difold  =',e12.3)
            if (residu) then
               if (jincrm==0) then
                  if (nstep>1) then
                     write(irefwr,170) resnew(iprob)
170                  format(21x, &
                            '  (old) residual  =',e12.3)
                  end if
               else if (jincrm==1) then
                     write(irefwr,180) resnew(iprob)
180                  format(21x, &
                            '        residual  =',e12.3)
               end if
               if ( resnew(iprob)<epsmac .and. &
                   resold(iprob)<epsmac ) then
                  difn2(iprob)=1.0d0
               else if (resold(iprob)>epsmac) then
                  difn2(iprob)=resnew(iprob)/resold(iprob)
               else
                  difn2(iprob)=resnew(iprob)
               end if
               if (abs(iprint)>=3 .and. &
                   ((nstep>2 .and. jincrm==0) .or. &
                    (nstep>1 .and. jincrm==1))) &
                  write(irefwr,190) difn2(iprob)
190               format(21x, &
                         '   resnew/resold  =',e12.3)
            end if
         end if
      end if
      if (nstep==1) then
         difold(iprob)=difnew(iprob)
         resold(iprob)=resnew(iprob)
         goto 298
      end if

!     --- (False) Time-Step Method
!         step 5 : test whether time stepping is too fast
!                  for residu BOTH tests must be satisfied together

      if ((.not.residu .and. difnew(iprob)>cd*difold(iprob) .and. &
                             difnew(iprob)>eps) .or. &
          ( residu .and.(difnew(iprob)>cd*difold(iprob) .and. &
                             difnew(iprob)>eps) .and. &
           resnew(iprob)>cr*resold(iprob) .and. &
           resnew(iprob)>eps )) then
         idivg=idivg+1

!        --- prevent that superfluous false-time step is performed

         if (nstep<5 .and. itytim<=0) then
            if ((.not.residu .and. difnew(iprob)<=eps) .or. &
                ( residu .and. difnew(iprob)<=eps .and. &
                 resnew(iprob)<=eps)) then
               if (iprob==1) then
                  ready=.true.
               end if
            else
               ready=.false.

!              --- increase of difference is allowed for nstep<5  or one step

               dif0(iprob)=difold(iprob)
               difold(iprob)=difnew(iprob)
               res0(iprob)=resold(iprob)
               resold(iprob)=resnew(iprob)
               if (itytim>0) then
                  decrdt=.true.
               else if (itytim<=0) then
                  incrdt=.true.
               end if
            end if
         else if (nstep<ntotal .or. iprob/=nprob) then
            if ((.not.residu .and. difold(iprob)<cd*dif0(iprob)).or. &
                ( residu .and. difold(iprob)<cd*dif0(iprob) .and. &
                 resold(iprob)<cr*res0(iprob))) then

!           --- increase of difference not repeated for nstep>=5

               dif0(iprob)=difold(iprob)
               difold(iprob)=difnew(iprob)
               res0(iprob)=resold(iprob)
               resold(iprob)=resnew(iprob)
               goto 298
            else

!           --- (repetitive) increase of difference of solution (nstep>=5)

               if (itytim<=0) then
                  write(irefwr,210)
210               format(/,72('%')/)
                  if (iprint<0) then
                     write(irefwr,220)
220                  format(' warning from subroutine marchn')
                  else
                     write(irefwr,230)
230                  format(' error in subroutine marchn')
                  end if
                  write(irefwr,240) nstep,dif0(iprob),difold(iprob), &
                                    difnew(iprob)
240               format(/' false time-step method', &
                          ' diverges at step',i5/ &
                          '  dif-2,   dif-1,   difnew :' /3e20.12)
                  if (residu) &
                     write(irefwr,250) res0(iprob), resold(iprob), &
                                       resnew(iprob)
250                  format('  res-2,   res-1,   resnew :'/ 3e20.12)
                  write(irefwr,'(/,72(''%'')/)')
                  if (float(idivg)/(nstep*nprob)>=0.5d0) then
                     ready=.true.
                     if (iprint>=0) iexit=.true.
                  end if
               else if (itytim>0) then
                  write(irefwr,'(/,72(''%'')/)')
                  if (abs(iprint)/=0) then
                     write(irefwr,220)
                  end if
                  write(irefwr,260) nstep,dif0(iprob),difold(iprob), &
                                    difnew(iprob)
260               format(/ ' differences of time-step method', &
                           ' increase at step',i5/ &
                           '  dif-2,   dif-1,   difnew :'/3e20.12)

                  if (residu) &
                     write(irefwr,270) res0(iprob), resold(iprob), &
                                       resnew(iprob)
270                  format('  and/or residuals increase '/ &
                            '  res-2,   res-1,   resnew :'/3e20.12)

                  write(irefwr,'(/,72(''%'')/)')
               end if
               dif0(iprob)=difold(iprob)
               difold(iprob)=difnew(iprob)
               res0(iprob)=resold(iprob)
               resold(iprob)=resnew(iprob)
               if (itytim<=0) then
                  incrdt=.true.
               else if (itytim>0) then
                  decrdt=.true.
               end if
               goto 298
            end if
         else

!        --- too many time steps

            ready=.true.
            if (iprint>=0) iexit=.true.
            if (itytim<=0) then
               write(irefwr,'(/,72(''%'')/)')
               if (iprint<0) then
                  write(irefwr,220)
               else
                  write(irefwr,230)
               end if
               write(irefwr,280)
280            format(' divergence of (false) time-step', &
                      ' method, and ')
               write(irefwr,290) nstep
290            format(' too many time steps ( =',i5,' )')
               if (iprint/=0) &
                  write(irefwr,310) difnew(iprob)
310               format(' difference with result of preceding time', &
                         ' step = ', e12.2)
               if ((iprint/=0).and.(residu)) &
                  write(irefwr,320) resnew(iprob)
320               format(' residual               of preceding time', &
                         ' step = ',e12.2)
               write(irefwr,'(/,72(''%'')/)')
            end if
         end if

!        --- step 6 : test whether truncation criterium has been reached or not

      else if ((.not.residu .and. difnew(iprob)<=eps ).or. &
               ( residu .and. difnew(iprob)<=eps .and. &
                 resnew(iprob)<=eps)) then
         if (abs(iprint)>=1 .and. itytim<=0) then
            if (nprob==1) then
               write(irefwr,'('' marchn : criteria have been met'')')
            else
               write(irefwr,350) iprob
350            format(' marchn : criteria have been met', &
                      ' for problem',i2)
            end if
         end if
         if (iprob==1 .and. itytim<=0) ready=.true.
      else

!     --- other cases (not yet completed)

         ready=.false.
      end if
      if (nstep>=ntotal .and. iprob==nprob) then

!     --- maximum number of time steps not sufficient

         write(irefwr,'(/,72(''%'')/)')
         if (iprint<0 .and. itytim<=0) then
            write(irefwr,'('' warning from subroutine marchn'')')
         else if (iprint>=0 .and. itytim<=0) then
            write(irefwr,'('' error in subroutine marchn'')')
         end if
         if (itytim<=0) then
            write(irefwr,'('' too many time steps ( ='',i5,'' )'')') nstep
         end if
         if (abs(iprint)==1 .and. itytim<=0) then
            do jprob=1,iprob
               if (jprob==1) write(irefwr,'(/,72(''%'')/)')
               write(irefwr,410) difnew(iprob)
410            format(' difference with result of preceding (time)', &
                      ' step = ',e12.2)
               if (residu) &
                  write(irefwr,420) resnew(iprob)
420               format(' residual',15x,'of preceding (time) step =', &
                         e13.2)
                if (jprob==nprob) write(irefwr,'(/,72(''%'')/)')
            end do
         end if
         ready=.true.
         if (iprint>=0) iexit=.true.
      else
         dif0(iprob)=difold(iprob)
         difold(iprob)=difnew(iprob)
         res0(iprob)=resold(iprob)
         resold(iprob)=resnew(iprob)
      end if

!     --- prepare new isolol

298   call sepcopyvc (isolit(iprob),isolol(iprob))
      end do
      if (.not.ready) then
         goto 100
      end if
      if (abs(iprint)>=1) &
            write(irefwr,430) nstep
430         format(' marchn : time stepping completed after', i5, &
                   '  time steps')
      do iprob=1,nprob
         call sepcopyvc (isolit(iprob),isol(iprob))
      end do
      if ((jtype==14 .or. jtype==414 .or. &
           jtype==16 .or. jtype==416 .or. &
           jtype==40 .or. jtype==440 .or. &
           jtype==46 .or. jtype==446).and. &
           nprob==1 .and. isolit(2)/=0 ) then
         call sepcopyvc (isolit(2),isol(2))
      end if

!     --- step 7 : remove matrix(iprob), isolit(iprob), &
!                  irhsd(iprob),inorm(iprob) by resetting
!                  of ibfree and iinfor
!            in order to clear space, the reverse order of creation
!            must be used!!

      do iprob=nprob,1,-1
         if (compl) call seprmsolinfo ( inorm(iprob) )
         if (residu .and. jincrm==0) &
                 call seprmsolinfo ( iresid(iprob) )
         call seprmsolinfo ( irhsd(iprob) )
         call seprmmatrix ( matrix(iprob) )
      end do
      do iprob=nprob,1,-1
         call seprmsolinfo ( isolit(iprob) )
         call seprmsolinfo ( ihelp(iprob) )
      end do
      itime=jtime
      if (itime==1) call printtime('marchn', timebefore)
      if (iexit) call instop
      call erclos( 'marchn' )
      end
