      subroutine printvcsurf ( kproblc, ivectr, &
                               isurnr, text, itrans, number, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 19-08-2005 Extra parameters
!        version  1.0  date 30-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the values of a vector along the surface isurnr
!     This subroutine is just meant to simplify the call to prinvc
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     surface
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr(*), kproblc(*), isurnr, itrans, &
              number, iseq
      character *(*) text

!     iseq           i    indicates the sequence in which the coordinates
!                         along the surface must be ordered
!                         Possible values:
!                         1:  increasing x, for fixed x increasing y,
!                             for fixed y increasing z
!                         2:  increasing y, for fixed y increasing z,
!                             for fixed z increasing x
!                         3:  increasing z, for fixed z increasing x,
!                             for fixed x increasing y
!                         4:  sequence of nodal point numbers
!     isurnr         i    surface sequence number of the surface along which the
!                         solution must be printed
!     itrans         i    indicates which quantity must be printed
!                         Possible values:
!                         0:  The contents of the vector are printed as stored
!                             in the vector corresponding to IVECTR i.e. without
!                             transformation.
!                         1:  The normal component of a vector along the curve
!                             or surface is printed.
!                         2:  The tangential component(s) of a vector along the
!                             curve or surface is printed.
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     kproblc          i    Standard SEPRAN array, containing information of
!                         the problem
!     number         i    indicates which degree of freedom in each point
!                         must be printed. This parameter must be seen in
!                         connection with ITRANS.
!                         Possible values:
!                         0:  If ITRANS=0, all degrees of freedom are printed
!                             in the nodes defined by icurvs,
!                             if ITRANS>0, the vector from which the normal or
!                             tangential components must be computed is supposed
!                             to be a n-dimensional vector with first degree of
!                             freedom 1, second degree of freedom 2 and third
!                             degree of freedom 3.
!                         i>0: If ITRANS=0, the degree of freedom i is printed
!                             if ITRANS>0, the vector from which the normal or
!                             tangential components must be computed is supposed
!                             to be a n-dimensional vector with first degree of
!                             freedom i, second degree of freedom i+1 and third
!                             degree of freedom i+2.
!     text           i    String variable containing some text to be printed
!                         in the heading of the ouput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inppri(6), icurvs(2)
      double precision :: rinpri(2)

!     icurvs         Input array with respect to PRINVCBF (see PRINVC)
!     inppri         Input array with respect to PRINVCBF (see PRINVC)
!     rinpri         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTVECT      Print the values of a SEPRAN solution vector or vector of
!                    special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printvcsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printvcsurf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays inppri and icurvs with information to print the surface

      inppri(1) = 6     ! number of items
      inppri(2) = 100   ! point number is printed followed by the values
                        ! zeros are suppressed
      inppri(3) = 3     ! print in user surfaces
      inppri(4) = itrans  ! type of transformation for printing
      inppri(5) = number
      inppri(6) = iseq
      icurvs(1) = 0
      icurvs(2) = isurnr

!     --- Perform the printing

      call printvect ( inppri, ivectr, icurvs, text, rinpri )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printvcsurf'

      end if
      call erclos ( 'printvcsurf' )

      end
