      subroutine sepbouniv ( ichois, irule, idim, number, kmeshlc, kproblc, &
                             icrv1, icrv2, ivectr, iuser, user, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichois, irule, idim, number, kmeshlc, &
              kproblc, icrv1, icrv2, ivectr, iuser(*)
      double precision, intent(in) :: user(*)
      double precision, intent(out) :: value(*)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iintbn(7), i, numcrv
      integer, allocatable :: icurvs(:)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbouniv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbouniv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      
      kprob2d(1) = kproblc
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      kprob => kprob2d(1)

!     --- Fill array iintbn from the parameter list

      iintbn(1) = 1
      iintbn(3) = ichois/100
      iintbn(2) = ichois-100*iintbn(3)
      iintbn(4) = irule
      iintbn(5) = 1
      iintbn(6) = number
      numcrv = abs(icrv2-icrv1)+1
      iintbn(7) = numcrv

!     --- Check icrv1 and icrv2

      if ( icrv1<1 .or. icrv1>ncurvs ) then
         call errint ( icrv1, 1 )
         call errint ( ncurvs, 2 )
         call errsub ( 185, 2, 0, 0 )
      end if
      if ( icrv2<icrv1 .or. icrv2>ncurvs ) then
         call errint ( icrv2, 1 )
         call errint ( icrv1, 2 )
         call errint ( ncurvs, 3 )
         call errsub ( 186, 3, 0, 0 )
      end if

!     --- Reserve space for array icurvs

      allocate ( icurvs(numcrv), stat = error )
      if ( error/=0 ) call eralloc ( error, numcrv, 'icurvs' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array icurvs

      if ( idim>0 ) then

!     --- Standard direction

         if ( icrv2>icrv1 ) then
            do i = 1, numcrv
               icurvs(i) = icrv1+i-1
            end do
         else
            do i = 1, numcrv
               icurvs(i) = icrv2-i+1
            end do
         end if

      else

!     --- Reversed direction

         if ( icrv2>icrv1 ) then
            do i = 1, numcrv
               icurvs(i) = -(icrv2-i+1)
            end do
         else
            do i = 1, numcrv
               icurvs(i) = -(icrv1+i-1)
            end do
         end if

      end if

      iprob  = ks(ivectr)%iprob
      iseqkp = kprob
      kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
      call sepfillkprob

      call sepactsolbf2 ( ivectr )

!     --- Actual computation

      call er0058 ( iintbn, ivectr, icurvs, value )

      deallocate ( icurvs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'icurvs' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbouniv'

      end if  ! ( debug )
      call erclos ( 'sepbouniv' )

      end subroutine sepbouniv
