      subroutine toflms( nx, ny, xlow, xhigh, ylow, yhigh, kmeshc, &
                         coor, kmeshm )
!
! ********************************************************************
!
!
!        programmer     Guus Segal
!        version 1.0    date   07-05-1991
!
!
!
!   copyright (c) 1991  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!   *   toflms fills some parts of sub arrays of KMESH for a rectangular
!       equidistant region.
!       TOFLMS is help subroutine for subroutine PLOTCO
!
! ********************************************************************
!
!                           INPUT / OUTPUT
      implicit none
      integer nx, ny, kmeshc(*), kmeshm(*)
      double precision xlow, xhigh, ylow, yhigh, coor(2,*)
!
!    coor      o   Contains the co-ordinates of the nodes
!
!    kmeshc    o   Contains KMESH part c
!
!    kmeshm    o   Contains KMESH part m
!
!    nx        i   Number of points in the x-direction
!
!    ny        i   Number of points in the y-direction
!
!    xlow      i   Under bound of x-coordinates
!
!    ylow      i   Under bound of y-coordinates
!
!    xhigh     i   Upper bound of x-coordinates
!
!    yhigh     i   Upper bound of y-coordinates
!
! ********************************************************************
!
!                      PARAMETERS IN COMMON BLOCKS

!
!   - &
! ********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, j, icount
      double precision dx, dy

!
!    dx     Step size of coordinates in x-direction
!
!    dy     Step size of coordinates in y-direction
!
!    i      General loop variable
!
!    j      General loop variable
!
!    icount Counting variable
!
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!
!    - &
!
! ********************************************************************
!
!                 ERROR MESSAGES
!
!   - &
!
! ********************************************************************
!
!    * Fill KMESHC

      do 110 j = 1, ny-1
         do 100 i = 1, nx-1
            kmeshc(((j-1)*(nx-1)+i-1)*4+1) = (j-1)*nx+i
            kmeshc(((j-1)*(nx-1)+i-1)*4+2) = (j-1)*nx+i+1
            kmeshc(((j-1)*(nx-1)+i-1)*4+3) = (j-1)*nx+i+1+nx
            kmeshc(((j-1)*(nx-1)+i-1)*4+4) = (j-1)*nx+i+nx
100      continue
110   continue

!    *  Fill COOR

      dx = (xhigh-xlow)/(nx-1)
      dy = (yhigh-ylow)/(ny-1)

      do 130 j = 1, ny
         do 120 i = 1, nx
            coor(1,(j-1)*nx+i) = xlow+(i-1)*dx
            coor(2,(j-1)*nx+i) = ylow+(j-1)*dy
120      continue
130   continue

!    * Fill KMESHM

      kmeshm(1) = 0
      kmeshm(2) = 7
      kmeshm(3) = 7+2*(nx+ny-2)
      kmeshm(4) = 0
      kmeshm(5) = 1
      kmeshm(6) = 2*(nx+ny-2)

      icount = 6
      do 140 i = 1, nx
         kmeshm(icount+i) = i
140   continue
      icount = icount + nx-1
      do 150 i = 1, ny
         kmeshm(icount+i) = i*nx
150   continue
      icount = icount + ny-1
      do 160 i = 1, nx
         kmeshm(icount+i) = nx*ny+1-i
160   continue
      icount = icount + nx-1
      do 170 i = 1, ny
         kmeshm(icount+i) = (ny-i)*nx+1
170   continue

      end
