      subroutine prmumpsrowind ( nrusol, numnonzeros, rowpointers, &
                                 rowindex )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the row indices of all non-zero matrix elements
!     Subroutine is meant for mumps
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
!     mumps
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol, numnonzeros, rowpointers(nrusol+1)
      integer, intent(out) :: rowindex(numnonzeros)

!     nrusol         i    Number of free unknowns
!     numnonzeros    i    number of non-zeros in matrix
!     rowindex       o    Contains the row indices of all non-zero
!                         matrix elements
!     rowpointers    i    Row pointers to columns in matrix and array columns
!                         First entry is 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, i, ncols

!     i              Counting variable
!     istart         First position used at a part
!     ncols          number of non-zero columns in row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmumpsrowind' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmumpsrowind'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 0
      do i = 1, nrusol
         ncols = rowpointers(i+1)-istart
         rowindex(istart+1:istart+ncols) = i
         istart = istart+ncols
      end do  ! i = 1, nrusol

1000  call erclos ( 'prmumpsrowind' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmumpsrowind'

      end if  ! ( debug )

      end
