      subroutine sepmumpsmat ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations by a direct method using mumps
!     Old subroutine prmumpsmat
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
!     mumps
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecpack
      implicit none
      include 'dmumps_struc.h'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numnonzeros
      type (dmumps_struc) mumps_par(maxvc1)
      logical :: new
      save mumps_par

!                    they are not
!     mumps_par      Standard type of mumps
!                    Is used to distinguish between the various cases
!     new            if true a new structure of the matrix has been made
!     numnonzeros    number of non-zeros in matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPMUMPSBODY   Actual body for the call to the mumps solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmumpsmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmumpsmat'
         write(irefwr,1) 'iseqsolut', iseqsolut
         write(irefwr,*) 'skipmumps', skipmumps
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some parameters

      if ( skipmumps ) then

!     --- skipmumps true, hence not on host

         nusol = 1
         nrusol = 1
         nusol = 1
         numnonzeros = 1

      end if  ! ( skipmumps )

!     --- Renumber or copy right-hand side into rhsd

      if ( skipmumps ) go to 500

      numnonzeros = irowsii(1+nrusol)

500   if ( debug ) write(irefwr,1) 'numnonzeros', numnonzeros

      new = A%newmumps>0  ! mark matrix structure as new
                         ! this means that a new call of commat has been done
      S_ii => A%S
      call sepmumpsbody ( numnonzeros, mumps_par(iseqsolut), new, usol, rhsd )
      A%newmumps = -1   ! mark matrix structure as already used in mumps

1000  if ( debug ) then

!     --- Debug information

         if ( .not. skipmumps ) call prinrl ( usol, nrusol, 'usol' )
         write(irefwr,*) 'End sepmumpsmat'

      end if  ! ( debug )
      call erclos ( 'sepmumpsmat' )

      end
