      subroutine petscaddmats1 ( nrusol1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add sepran Siout matrix to empty petsc matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulematstr
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol1

!     nrusol1        i    Number of free unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nrow, i, inusol, rowindex(1), ncol, istart
      integer, allocatable :: colindex(:)

!     colindex       Contains the column numbers of the non-zero of the
!                    row of the matrix in c-sense, hence starting with 0
!     i              Counting variable
!     inusol         Position of unknown in array USOL
!     istart         First position used at a part
!     ncol           Number of non-zero elements per row in the lower
!                    triangular matrix
!     nrow           Number of rows to be added (Here 1)
!     rowindex       Row number in c-sense, hence starting with 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MATSETVALUES   Inserts or adds a block of values into a matrix (petsc)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'petscaddmats1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from petscaddmats1'
         write(irefwr,1) 'nrusol1, nusolglob', nrusol1, nusolglob
   !      call prinin ( irowsout, nrusol1+1, 'irowsout' )
   !      call prinin ( icolsout, irowsout(nrusol1+1), 'icolsout' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nrow = 1  ! number of rows to be copied in 1 call

      allocate ( colindex(nusolglob), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol1, 'colindex' )
      if ( ierror/=0 ) go to 1000

!     --- Loop over rows

      do i = 1, nrusol1

         inusol = i                ! row index
         istart = irowsout(i)         ! start in array icolsout (-1)
         ncol = irowsout(i+1)-istart  ! number of columns in row

         if ( debug .and. ncol>0 ) &
            write(irefwr,1) 'i, inusol, istart, ncol', &
                             i, inusol, istart, ncol

         inusol = kprobhglob(inusol)
         colindex(1:ncol) = icolsout(istart+1:istart+ncol)-1

         rowindex(1) = inusol-1  ! petsc numbering

         if ( ncol>0 ) then
            if ( debug ) then
               write(irefwr,1) 'rowindex', rowindex
               call prinin ( colindex, ncol, 'colindex' )
               call prinrl ( S_iout(istart+1), ncol, 'S_iout' )
            end if  ! ( debug )

            call MatSetValues ( A(iseqmat), nrow, rowindex, ncol, colindex, &
                                S_iout(istart+1), ADD_VALUES, ierrpetsc )
         end if  ! ( ncol>0 )

      end do  ! i = 1, nrusol1

      deallocate ( colindex, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'colindex' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End petscaddmats1'

      end if  ! ( debug )
      call erclos ( 'petscaddmats1' )

      end subroutine petscaddmats1
