      subroutine seppetscmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform sepran matrix into petsc matrix
!     The sepran matrix is kept, the petsc matrix is new
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulematstr
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (globmat), pointer :: A
      integer :: nrusol1
      type (intmatrix), pointer :: IM

!     A              Structure of type globmat containing the necessary
!                    information of the matrix
!     IM             Refers to intmt(intmat)
!     nrusol1        Local value of nrusol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PETSCADDMATS   Add sepran matrix to empty petsc matrix
!     PETSCADDMATS1  Add sepran Siout matrix to petsc matrix
!     PETSCCREAMATS  reserve space for petsc matrix and start creation
!     SEPPETSCFINMAT Finish creating petsc matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      A => realmt(iseqmt)
      S_ii => A%S_ii
      S_iout => A%S_iout
      nrusol1 = A%S_i%neq

      IM => intmt(realmt(iseqmt)%intmatref)

!     --- Create matrix, i.e. reserve space

      call petsccreamats ( nrusol1, nnz, onnz, nusolglob )

!     --- Copy sepran matrix into petsc matrix

      call petscaddmats ( nrusol1, irowsii, icolsii, &
                          kprobhglob, S_ii )

      if ( size(IM%colsout)>1 ) call petscaddmats1 ( nrusol1 )

!     --- Assemble petsc matrix

      call seppetscfinmat

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppetscmat'

      end if  ! ( debug )
      call erclos ( 'seppetscmat' )

      end subroutine seppetscmat
