      subroutine topq04 ( kmesh, nplsub, kmeshc, kmscnw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-01-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Construct a submesh of vertices only in case of surface elements
!  If the mesh contains triangles all quadrilaterals are subdivided
!  Line elements and connection elements are skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), nplsub, kmeshc(*), kmscnw(*)

!     kmesh   i    Standard SEPRAN array containing information of the mesh
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     kmscnw  o    New KMESh part c consisting of the subdivided elements
!     nplsub  i    Number of points in the submesh consisting of elements
!                  with vertex nodes only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishape, inelem, kelmf, kelmg, ielgrp, nelgrp, kelma, &
              inpelm, i, j, ikelmc, ikmscn

!     i       Counting variable
!     ielgrp  element group number
!     ikelmc  position in kmesh part c
!     ikmscn  position in new kmesh part c (kmscnw)
!     inelem  Number of elements in a specific element group
!     inpelm  number of points per element
!     ishape  Shape number of elements
!     j       Counting variable
!     kelma   Indicates if KMESH part a has been filled (>0) or not (0)
!             If kelma>0, kelma is the starting address of KMESH part a in KMESH
!     kelmf   Indicates if KMESH part f has been filled (>0) or not (0)
!             If kelmf>0, kelmf is the starting address of KMESH part f in KMESH
!     kelmg   Indicates if KMESH part g has been filled (>0) or not (0)
!             If kelmg>0, kelmg is the starting address of KMESH part g in KMESH
!     nelgrp  Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For ielgrp := 1 (1) nelgrp do
!      ishape := kmeshf(ielgrp)
!      inelem := kmeshg(ielgrp)
!      if ( ishape = 3 ) then
!         Copy inelem elements
!      else if ( ishape = 6 or 7 ) then
!         Copy the vertices of inelem elements
!      else if ( ishape = 5 ) then
!         if ( nplsub=3 ) then
!            Subdivide inelem quadrilaterals
!         else ( nplsub = 4 )
!            Copy inelem quadrilaterals
!      else if ( ishape = 6 ) then
!         if ( nplsub=3 ) then
!            Subdivide inelem quadrilaterals (use vertices only)
!         else ( nplsub = 4 )
!            Copy the vertices of inelem quadrilaterals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check if there are triangles

      nelgrp = kmesh(5)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

      ikmscn = 0
      ikelmc = 0
      do 200 ielgrp = 1, nelgrp
         ishape = kmesh(kelmf+ielgrp-1)
         inelem = kmesh(kelmg+ielgrp-1)
         inpelm = kmesh(kelma+ielgrp) - kmesh(kelma+ielgrp-1)
         if ( ishape.eq.3 ) then

!        --- Copy inelem triangles

            do 100 i = 1, inelem * 3
               kmscnw(ikmscn+i) = kmeshc(ikelmc+i)
100         continue
            ikmscn = ikmscn + 3 * inelem

         else if ( ishape.eq.6 .or. ishape.eq.7 ) then

!        --- Copy the vertices of inelem triangles

            do 120 j = 1, 3
               do 110 i = 1, inelem
                  kmscnw(ikmscn+(i-1)*3+j) = &
                  kmeshc(ikelmc+(i-1)*inpelm+2*j-1)
110            continue
120         continue
            ikmscn = ikmscn + 3 * inelem

         else if ( ishape.eq.5 ) then

!        --- Bilinear quadrilaterals

            if ( nplsub.eq.3 ) then

!           --- Subdivide into triangles

               do 130 i = 1, inelem
                  kmscnw(ikmscn+1 ) = kmeshc(ikelmc+1)
                  kmscnw(ikmscn+2 ) = kmeshc(ikelmc+2)
                  kmscnw(ikmscn+3 ) = kmeshc(ikelmc+4)
                  kmscnw(ikmscn+4 ) = kmeshc(ikelmc+2)
                  kmscnw(ikmscn+5 ) = kmeshc(ikelmc+3)
                  kmscnw(ikmscn+6 ) = kmeshc(ikelmc+4)
                  ikmscn = ikmscn + 6
                  ikelmc = ikelmc + 4
130            continue
               ikelmc = ikelmc - inelem * inpelm

            else

!           --- Copy inelem quadrilaterals

               do 140 i = 1, inelem * 4
                  kmscnw(ikmscn+i) = kmeshc(ikelmc+i)
140            continue
               ikmscn = ikmscn + 4 * inelem

            end if

         else if ( ishape.eq.6 ) then

!        --- Biquadratic quadrilaterals

            if ( nplsub.eq.3 ) then

!           --- Subdivide into triangles

               do 150 i = 1, inelem
                  kmscnw(ikmscn+1 ) = kmeshc(ikelmc+1)
                  kmscnw(ikmscn+2 ) = kmeshc(ikelmc+3)
                  kmscnw(ikmscn+3 ) = kmeshc(ikelmc+7)
                  kmscnw(ikmscn+4 ) = kmeshc(ikelmc+3)
                  kmscnw(ikmscn+5 ) = kmeshc(ikelmc+5)
                  kmscnw(ikmscn+6 ) = kmeshc(ikelmc+7)
                  ikmscn = ikmscn + 6
                  ikelmc = ikelmc + 9
150            continue
               ikelmc = ikelmc - inelem * inpelm

            else

!           --- Copy the vertices of inelem quadrilaterals

               do 170 j = 1, 4
                  do 160 i = 1, inelem
                     kmscnw(ikmscn+(i-1)*4+j) = &
                     kmeshc(ikelmc+(i-1)*inpelm+2*j-1)
160               continue
170            continue
               ikmscn = ikmscn + 4 * inelem

            end if

         end if
         ikelmc = ikelmc + inelem * inpelm
200   continue
      end
