      subroutine topq13 ( kmesh, kmeshc, neleml, kline )

! =======================================================================
!
!        programmer   Niek Praagman
!
!
!        version 1.1    date   27-05-1994 Determine meshtp also if one group
!        version 1.0    date   05-05-1994
!
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Fill line elements array in case that surface and/or volume elements
!     need not to be subdivided. Divide quadratic line elements (2 and 3D)
!     into elements of linear shape.
!
! ***********************************************************************
!
!     KEYWORDS
!
!     2d
!     3d
!     line_element
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS

      implicit none
      integer kmesh(*), kmeshc(*), neleml, kline(*)

!     kline   o   Array containing "line"elements
!
!     kmesh   i   Standard SEPRAN array containing information concerning
!                 the mesh
!
!     kmeshc  i   Old array KMESH part c, containing the nodes of the
!                 elements. KMESHC must be subdivided
!
!     neleml  o   Number of line elements in kline
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer kelma, inpelm, inelem, kelmf, kelmg, nelgrp, ielgrp, &
              kelmc, iielem, j, k, meshtp

!     ielgrp   Element group number (loop variable)
!
!     iielem   Relative element number with respect to element group
!
!     inelem   Number of elements in an element group
!
!     inpelm   Number of nodes in an element
!
!     j        Count helpvariable
!
!     k        Count helpvariable
!
!     kelma    Start adress of KMESH part a in array KMESH.
!              KMESH part a contains the number of nodes per element of a
!              specific element group accumulated.
!              If KELMA = 0, then KMESH part a has not been filled
!
!     kelmc    Last position used in array KMESHC
!
!     kelmf    Start adress of KMESH part f in array KMESH
!              Type numbers of the elements
!
!     kelmg    Start adress of KMESH part g in array KMESH.
!              KMESH part g contains the number of elements of an element group
!              KMESH part g is only filled if NELGRP > 1
!
!     meshtp   Type of element according to SEPRAN rules
!
!     nelgrp   Number of element groups
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Run through all element groups
!
!       Determine for each group shape number
!
!       Check whether line elements and if so store in line elements array
!
! =======================================================================

!     Fill some constants

      inpelm = kmesh(4)
      nelgrp = kmesh(5)
      inelem = kmesh(9)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

      kelmc  = 0
      neleml = 0

!     Loop over element groups

      do 200 ielgrp = 1, nelgrp

!        Compute meshtp number of element group IELGRP

         if ( kelma.ne.0 ) inpelm = kmesh ( kelma + ielgrp ) - &
                                      kmesh ( kelma + ielgrp - 1 )

         meshtp = kmesh ( kelmf + ielgrp - 1 )

         if ( nelgrp.gt.1 ) inelem = kmesh ( kelmg + ielgrp - 1 )

         if ( meshtp.eq.1 ) then

!           Linear line element, copy vertices

            j = 2 * neleml

            do 100 iielem = 1, 2*inelem

                kline( j + iielem ) = kmeshc( kelmc + iielem )

100         continue

         else if ( meshtp.eq.2 ) then

!           Quadratic line element, store vertices only

            do 150 iielem = 1 , inelem

                j = 2 * neleml + 2 * ( iielem - 1 )
                k =     kelmc  + 3 * ( iielem - 1 )

                kline( j + 1 ) = kmeshc( k + 1 )
                kline( j + 2 ) = kmeshc( k + 3 )

150         continue

         end if

         neleml = neleml + inelem

         kelmc = kelmc + inpelm * inelem

200   continue

      end
