      subroutine topqu2 ( kmesh, number, lensub, itrian, npoint, indprf, &
                          indprp, kprobf, kprobp, nelemt, iskip, ihelp, &
                          nphys, kmeshc, neleml, special, centroid, &
                          triangul )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.7  date 10-03-2006 Extension with interface elements
!        version  6.6  date 16-02-2005 Correction connection elements
!        version  6.5  date 07-02-2005 Debug statements
!        version  6.4  date 02-08-2004 Extra debug
!        version  6.3  date 05-12-2003 Correction in case of triangles only
!        version  6.2  date 18-12-2002 Check of shapes
!        version  6.1  date 14-11-1997 Make triangul also output
!        version  6.0  date 28-05-1997 Extra parameter triangul
!        version  5.1  date 23-12-1996 Extension with 4-point triangles
!        version  5.0  date 14-01-1996 Extra parameter centroid
!        version  4.0  date 30-12-1995 Extra parameter special
!        version  3.6  date 10-09-1994 Extension with 5-point quadrilaterals
!        version  3.5  date 05-05-1994 Special treatment for line elements
!                                      only in 2D or 3D
!        version  3.4  date 15-06-1993 Linear and quadratic line elements
!
!   copyright (c) 1988-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if a mesh consists of linear triangles (2D) or linear tetra- &
!     hedrons (3D) only, or that a subdivision into triangles or tetra- &
!     hedrons must be made depending on the unknowns considered in number
!     If so, the length of the local array KMESHC, containing the new
!     mesh is computed.
!     TOPQU2 is used as help subroutine for several output subroutines
!     like INTP2D, PLCN00, PLTRAC
!     It must be used in co-operation with subroutine TOPQU3
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     mesh
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, lensub, npoint, indprf, kprobf(*), indprp, &
              kprobp(npoint,*), nelemt, iskip, nphys, kmesh(*), &
              itrian(*), ihelp(*), kmeshc(*), neleml, special, centroid
      logical triangul

!     centroid  o   If 1 a 9 point element has been found of which the
!                   only 8 points have been filled
!     ihelp     i   array of length NELGRP that is only used if ISKIP = 1
!                   It indicates if element group ielgrp must be skipped
!                   ihelp(ielgrp) = 0 or not ihelp(ielgrp) = 1
!     indprf    i   indication if kprob part f has been filled (>0) or not (0)
!     indprp    i   Indication if array kprob part p has been filled (1)
!                   or not (0)
!     iskip     i   Indication if element groups must be skipped a-priori (>0)
!                   or not (0)
!     itrian    o   Output array of length NELGRP in which information of the
!                   various element groups is stored with respect to the
!                   subdivision into linear triangles.
!                   ITRIAN ( ielgrp ) corresponds to the ielgrp th element
!                   group and its value has the following meaning:
!                   -3:   (extra) Subdivision quadratic line element
!                   -2:   Linear line element
!                   -1:   Skip the element group
!                    0:   No subdivision required
!                    1:   Subdivision of a quadrilateral with 4 or 5 points into
!                         2 triangles
!                    2:   Subdivision of a triangle with 6 or 7 points into 4
!                         triangles
!                    3:   Subdivision of a triangle with 6 or 7 points into 1
!                         triangle
!                    4:   Subdivision of a quadrilateral with 9 points into 8
!                         triangles
!                    5:   Subdivision of a quadrilateral with 9 points into 8
!                         triangles. The centroid must be computed by
!                         interpolation
!                    6:   Subdivision of a quadrilateral with 9 points into 2
!                         triangles.
!                    7:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                         into 8 tetrahedrons
!                    8:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                         into 1 tetrahedron
!                    9:   Subdivision of a hexahedron with 8 points into 6
!                         tetrahedrons
!                   10:   Subdivision of a hexahedron with 27 points into 48
!                         tetrahedrons
!                   11:   Subdivision of a hexahedron with 27 points into 6
!                         tetrahedrons
!                   12:   Subdivision of a hexahedron with 27 points into 6
!                         hexahedrons
!                   13:   Subdivision of a hexahedron with 27 points into 1
!                         hexahedron
!     kmesh     i   Standard SEPRAN array containing information of the mesh
!     kmeshc    i   kmesh part c, contains the nodal points corresponding to the
!                   elements
!     kprobf    i   KPROB part f: contains the number of unknowns per point
!                   accumulated
!     kprobp    i   SEPRAN array KPROB part p containing the positions of the
!                   the unkowns per point
!     lensub    o   If zero the original mesh is used, otherwise lensub>0
!                   gives the length of the KMESH part c
!     neleml    o   Number of "line"elements in the new mesh
!     nelemt    o   Number of elements in the new mesh
!     nphys     i   Number of physical unknowns, or maximal degrees of freedom
!                   per point
!     npoint    i   Number of nodal points
!     number    i   The number-th unknown in each nodal point of the array
!                   is considered.
!     special  i/o  Standard situation: special=0 at input
!                   If special=2 and the unknown is present in the vertices, &
!                   special is set to 0, else if the unknown is only present
!                   in the midside points, special remains unchanged
!     triangul i/o  if true hexahedrons must be subdivided into tetrahedrons, &
!                   if false in hexahedrons
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, kelma, inelem, kelmf, kelmg, ielgrp, ishape, &
              nelgrp, i, inodp, ikelmc, ivrtri(4), ivrqua(8), &
              imdtri(6), imdqua(12), in, ielem, jkelmc
      logical subdiv, presen, check, debug, finished

!     check          Logical indicating if a unknown is present in all points of
!                    an element (true) or not (false)
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     imdqua         nodenumbers of midpoints for quadratic hexahedrons
!     imdtri         nodenumbers of midpoints for quadratic triangles and
!                    tetrahedrons
!     in             helppointer
!     inelem         Number of elements in a specific element group
!     inodp          Nodal point number
!     inpelm         number of points per element
!     ishape         Shape number of elements
!     ivrqua         Vertices numbers for quadratic hexahedron
!     ivrtri         Vertices numbers for quadratic tetrahedron
!     jkelmc         position in kmesh part c
!     kelma          Indicates if KMESH part a has been filled (>0) or not (0)
!                    If kelma>0, kelma is the starting address of KMESH part a
!                    in KMESH
!     kelmf          Indicates if KMESH part f has been filled (>0) or not (0)
!                    If kelmf>0, kelmf is the starting address of KMESH part f
!                    in KMESH
!     kelmg          Indicates if KMESH part g has been filled (>0) or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH part g
!                    in KMESH
!     nelgrp         Number of element groups
!     presen         Logical parameter indicating if the required unknowns are
!                    present in the mesh (true) or not (false)
!     subdiv         Logical parameter indicating if the mesh must be subdivided
!                    (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     701   NUMBER has wrong value.
!     800   Shape number not available
!    1707   The unknown NUMBER is not present in one of the nodes of the
!           solution array
! **********************************************************************
!
!                       PSEUDO CODE
!
!       ITER (for all element groups)
!          Check if element must be skipped
!          If not consider shape of element
!          If element is quadratic line then
!              subdivide in two linear lines
!          else if element is linear line
!              do not subdivide
!          else if element is linear triangle then
!              no special treatment
!          else if element is bilinear quadrilateral then
!              subdivide into 2 triangles
!          else if element is quadratic triangle then
!             If unknown is present in all nodes then
!                subdivide into 4 triangles
!             else if unknown is present in all vertices then
!                subdivide into 1 triangle
!             else
!                error
!             end if
!          else if element is quadratic quadrilateral then
!             If unknown is present in all boundary nodes then
!                if unknown is not present in central point interpolate
!                subdivide into 8 triangles
!             else if unknown is present in all vertices then
!                subdivide into 2 triangles
!             else
!                error
!             end if
!          else if element is linear tetrahedron then
!             no special treatment
!          else if element is one of the quadratic tetrahedrons then
!             if unknown is present in all 10 boundary points then
!                subdivide into 8 tetrahedrons
!             else if unknown is only present in the four vertices then
!                subdivide into 1 tetrahedron
!             else
!                error
!             end if
!          else if element is trilinear hexahedron then
!             subdivide into 6 tetrahedrons
!          else if element is triquadratic hexahedron then
!             if unknown is present in all vertices and "mid" nodes then
!                subdivide in 48 tetrahedrons
!             else if unknown is present in all vertices then
!                subdivide into 6 tetrahedrons
!             else
!                error
!             end if
!          else
!             element not recognized, error !
!          end if
!       end iteration
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivrtri / 1, 3, 5, 10 /
      data ivrqua / 1, 3, 7, 9, 19, 21, 25, 27 /
      data imdtri / 2, 4, 6, 7, 8, 9 /
      data imdqua / 2, 4, 6, 8, 10, 12, 16, 18, 20, 22, 24, 26 /
! ======================================================================
!
!     --- Fill some constants

      call eropen ( 'topqu2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu2'
         write(irefwr,100) 'number, nphys, iskip', number, nphys, iskip
         write(irefwr,*) 'triangul ', triangul
         write(irefwr,100) 'indprf, indprp', indprf, indprp
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      inpelm = kmesh(4)
      nelgrp = kmesh(5)
      inelem = kmesh(9)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

!     --- Initialize new constants

      subdiv = .false.
      presen = .false.
      centroid = 0
      lensub = 0
      nelemt = 0
      neleml = 0
      if ( debug ) then
         write(irefwr,100) 'nelgrp', nelgrp
         if ( iskip>0 ) call prinin ( ihelp, nelgrp, 'ihelp' )
      end if

!     --- Check number and idegfd

      if ( number<=0 .or. number>nphys ) then

!     --- number>nphys   Error

         call errint ( number, 1 )
         call errint ( nphys, 2 )
         call errsub ( 701, 2, 0, 0 )
         go to 1000

      end if

      if ( .not. triangul ) then

!     --- Elements must be sudivided into hexahedrons
!         This is only possible if all volume elements are hexadrons
!         If Tetrahedrons are found, triangul is made true

         do ielgrp = 1, nelgrp
            ishape = kmesh(kelmf+ielgrp-1)
            if ( ishape==11 .or. ishape==12 .or.ishape==15 .or. &
                 ishape==16 .or. ishape==18 ) triangul = .true.
            if ( ishape<=1 .and. ishape/=-10001 .or. &
                 ishape>18 .and. ishape<50 ) then

!           --- Shape number not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )

            end if
         end do

      end if  ! ( .not. triangul )

!     --- Loop over element groups

      ikelmc = 0
      do ielgrp = 1, nelgrp

!     --- Compute shape number, number of nodes and number of elements
!         of element group IELGRP

         if ( kelma/=0 ) inpelm = kmesh(kelma+ielgrp) - &
                                    kmesh(kelma+ielgrp-1)
         if ( nelgrp>1 ) inelem = kmesh(kelmg+ielgrp-1)
         ishape = kmesh(kelmf+ielgrp-1)

         check = .true.
         if ( debug ) &
            write(irefwr,100) 'ielgrp, ishape, inelem', &
                               ielgrp, ishape, inelem

!        --- Check elements

         if ( iskip>0 .and. ihelp(ielgrp)==0 .or. &
              ishape== -10001 .or. ishape>50 ) then

!        --- This element group must be skipped

            subdiv = .true.
            itrian(ielgrp) = -1

         else if ( ishape==1 ) then

!        --- Linear line element
!            no extra treatment

            itrian( ielgrp ) = -2
            subdiv = .true.
            neleml = neleml + inelem

         else if ( ishape==2 ) then

!        --- Quadratic line element
!            use vertices only, if not possible skip

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               if ( kprobp(kmeshc(ikelmc+1),number)==0 .or. &
                    kprobp(kmeshc(ikelmc+3),number)==0 ) &
                    check = .false.

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               inodp = kmeshc(ikelmc+1)
               if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               inodp = kmeshc(ikelmc+3)
               if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.

            end if

            if ( check ) then

!           --- Unknown is present in vertices

               itrian(ielgrp) = -3
               neleml = neleml + inelem

            else

!           --- Skip this group

               itrian(ielgrp) = -1

            end if

         else if ( ishape==3 .or. ishape==10 ) then

!        --- Linear triangular element
!            Check whether unknown is present in vertices

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 3
                  if ( kprobp(kmeshc(ikelmc+i),number)==0 ) &
                    check = .false.
               end do

               if ( .not. check .and. nelgrp==1 ) then

!              --- In the first element we did not have the present
!                  unknown, but it may be available in a subset of the elements
!                  because we are considering a surface mesh originally
!                  arising from perhaps a number of element groups

                   jkelmc = ikelmc
                   finished = .false.
                   ielem = 0
                   do while ( .not. finished )

                      check = .true.
                      do i = 1, 3
                         if ( kprobp(kmeshc(jkelmc+i),number)==0 ) &
                           check = .false.
                      end do
                      jkelmc = jkelmc+3
                      ielem = ielem+1
                      finished = check .or. ielem>inelem

                   end do  ! while ( .not. finished )

               end if

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 3
                  inodp = kmeshc(ikelmc+i)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in vertices

               itrian(ielgrp) = 0
               nelemt = nelemt + inelem
               lensub = lensub + inelem * 3
               presen = .true.
               if ( ishape==10 ) subdiv = .true.

            else

!           --- Skip this group

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==4 .or. ishape==7 ) then

!        --- Quadratic triangular element
!            Check if unknown is present in all nodes or in the vertices only

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 3
                  in = ivrtri(i)
                  if ( kprobp(kmeshc(ikelmc+in),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 3
                  in = ivrtri(i)
                  inodp = kmeshc(ikelmc+in)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in vertices, check mid points

               presen = .true.
               if ( indprp>0 ) then

!              --- indprp>0, use array KPROBP

                  do i = 1, 3
                     in = imdtri(i)
                     if ( kprobp(kmeshc(ikelmc+in),number)==0 ) &
                       check = .false.
                  end do

               else if ( indprf>0 ) then

!              --- indprf>0, use array KPROBF

                  do i = 1, 3
                     in = imdtri(i)
                     inodp = kmeshc(ikelmc+in)
                     if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                       check = .false.
                  end do

               end if

               if ( check ) then

!              --- Unknown is present in all 6 points

                  itrian(ielgrp) = 2
                  nelemt = nelemt + inelem * 4
                  lensub = lensub + inelem * 12
                  subdiv = .true.

               else

!              --- Unknown is present in vertices only

                  itrian(ielgrp) = 3
                  nelemt = nelemt + inelem
                  lensub = lensub + inelem * 3
                  subdiv = .true.

               end if

            else

!           --- Unknown is not present in vertices, skip

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==5 .or. ishape==9 ) then

!        --- Bilinear quadrilateral element
!            Check if unknown is present in vertices

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 4
                  if ( kprobp(kmeshc(ikelmc+i),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 4
                  inodp = kmeshc(ikelmc+i)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in all vertices

               itrian(ielgrp) = 1
               nelemt = nelemt + inelem * 2
               lensub = lensub + inelem * 6
               subdiv = .true.
               presen = .true.

            else

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==6 ) then

!        --- Biquadratic quadrilateral element
!            Check if unknown is present in all nodes or in the vertices only

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 8, 2
                  if ( kprobp(kmeshc(ikelmc+i),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 8, 2
                  inodp = kmeshc(ikelmc+i)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in vertices, check mid points

               presen = .true.
               if ( indprp>0 ) then

!              --- indprp>0, use array KPROBP

                  do i = 2, 8, 2
                     if ( kprobp(kmeshc(ikelmc+i),number)==0 ) &
                       check = .false.
                  end do

               else if ( indprf>0 ) then

!              --- indprf>0, use array KPROBF

                  do i = 2, 8, 2
                     inodp = kmeshc(ikelmc+i)
                     if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                       check = .false.
                  end do

               end if

               if ( check ) then

!              --- Unknown is present in all 8 points of the boundary
!                  Check last point (centroid)

                  if ( indprp>0 ) then

!                 --- indprp>0, use array KPROBP

                     if ( kprobp(kmeshc(ikelmc+9),number)==0 ) &
                       check = .false.

                  else if ( indprf>0 ) then

!                 --- indprf>0, use array KPROBF

                     inodp = kmeshc(ikelmc+9)
                     if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                       check = .false.

                  end if
                  if ( .not. check ) then
                     itrian(ielgrp) = 5
                     centroid = 1
                  else
                     itrian(ielgrp) = 4
                  end if

                  nelemt = nelemt + inelem * 8
                  lensub = lensub + inelem * 24
                  subdiv = .true.

               else

!              --- Unknown is present in vertices only

                  itrian(ielgrp) = 6
                  nelemt = nelemt + inelem * 2
                  lensub = lensub + inelem * 6
                  subdiv = .true.

               end if

            else

!           --- Unknown is not present in vertices, skip

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==11 ) then

!        --- Linear tetrahedron
!            Check whether unknown is present in vertices

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 4
                  if ( kprobp(kmeshc(ikelmc+i),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 4
                  inodp = kmeshc(ikelmc+i)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in the vertices

               itrian(ielgrp) = 0
               nelemt = nelemt + inelem
               lensub = lensub + inelem * 4
               presen = .true.

            else

!           --- Unknown not present, skip group

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==12 .or. ishape==15 .or. &
                   ishape==16 ) then

!        --- Quadratic tetrahedron
!            Check if unknown is present in all boundary nodes or
!            in the vertices only

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 4
                  in = ivrtri(i)
                  if ( kprobp(kmeshc(ikelmc+in),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 4
                  in = ivrtri(i)
                  inodp = kmeshc(ikelmc+in)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in vertices, check mid points

               presen = .true.
               if ( indprp>0 ) then

!              --- indprp>0, use array KPROBP

                  do i = 1, 6
                     in = imdtri(i)
                     if ( kprobp(kmeshc(ikelmc+in),number)==0 ) &
                       check = .false.
                  end do

               else if ( indprf>0 ) then

!              --- indprf>0, use array KPROBF

                  do i = 1, 6
                     in = imdtri(i)
                     inodp = kmeshc(ikelmc+in)
                     if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                       check = .false.
                  end do

               end if

               if ( check ) then

!              --- Unknown is present in all boundary points

                  itrian(ielgrp) = 7
                  nelemt = nelemt + inelem * 8
                  lensub = lensub + inelem * 32
                  subdiv = .true.

               else

!              --- Unknown is present in vertices only

                  itrian(ielgrp) = 8
                  nelemt = nelemt + inelem
                  lensub = lensub + inelem * 4
                  subdiv = .true.

               end if

            else

!           --- Unknown is not present in vertices, skip

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==13 ) then

!        --- Linear hexahedron
!            Check if unknown is present in vertices

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 8
                  if ( kprobp(kmeshc(ikelmc+i),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 8
                  inodp = kmeshc(ikelmc+i)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check .and. triangul ) then

!           --- Subdivide into six tetrahedrons

               itrian(ielgrp) = 9
               nelemt = nelemt + inelem * 6
               lensub = lensub + inelem * 24
               subdiv = .true.
               presen = .true.

            else if ( check ) then

!           --- Do not subdivide

               itrian(ielgrp) = 0
               nelemt = nelemt + inelem
               lensub = lensub + inelem * 8
               presen = .true.

            else

!           --- Unknown not present, skip

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else if ( ishape==14 ) then

!        --- Tri-Quadratic hexahedron
!            Check if unknown is present in all nodes or in the vertices only

            if ( indprp>0 ) then

!           --- indprp>0, use array KPROBP

               do i = 1, 8
                  in = ivrqua(i)
                  if ( kprobp(kmeshc(ikelmc+in),number)==0 ) &
                    check = .false.
               end do

            else if ( indprf>0 ) then

!           --- indprf>0, use array KPROBF

               do i = 1, 8
                  in = ivrqua(i)
                  inodp = kmeshc(ikelmc+in)
                  if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                    check = .false.
               end do

            end if

            if ( check ) then

!           --- Unknown is present in vertices, check mid points

               presen = .true.
               if ( indprp>0 ) then

!              --- indprp>0, use array KPROBP

                  do i = 1, 12
                     in = imdqua(i)
                     if ( kprobp(kmeshc(ikelmc+in),number)==0 ) &
                       check = .false.
                  end do

               else if ( indprf>0 ) then

!              --- indprf>0, use array KPROBF

                  do i = 1, 12
                     in = imdqua(i)
                     inodp = kmeshc(ikelmc+i)
                     if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                       check = .false.
                  end do

               end if

               if ( check ) then

!              --- Unknown is present in all vertices and midpoints
!                  Check whether unknown is present in centre point

                  if ( indprp>0 ) then

!                 --- indprp>0, use array KPROBP

                     if ( kprobp(kmeshc(ikelmc+27),number)==0 ) &
                       check = .false.

                  else if ( indprf>0 ) then

!                 --- indprf>0, use array KPROBF

                     inodp = kmeshc(ikelmc+27)
                     if ( kprobf(inodp+1)-kprobf(inodp)<number) &
                       check = .false.

                  end if

                  if ( .not. check ) then

!                 --- Unknown not present in centroid
!                     determine by interpolation
!                     not yet implemented, skip

                     itrian(ielgrp) = -1
                     subdiv = .true.

                  else if ( triangul ) then

!                 --- Subdivide into tetrahedrons

                     itrian(ielgrp) = 10

                     nelemt = nelemt + inelem *  48
                     lensub = lensub + inelem * 192
                     subdiv = .true.

                  else

!                 --- Subdivide into hexahedrons

                     itrian(ielgrp) = 12

                     nelemt = nelemt + inelem *  8
                     lensub = lensub + inelem * 64

                  end if
                  presen = .true.

               else if ( triangul ) then

!              --- Unknown only given in vertices

                  itrian(ielgrp) = 11
                  nelemt = nelemt + inelem *  6
                  lensub = lensub + inelem * 24
                  subdiv = .true.

               else

!              --- Unknown only given in vertices

                  itrian(ielgrp) = 13
                  nelemt = nelemt + inelem
                  lensub = lensub + inelem * 8
                  subdiv = .true.

               end if

            else

!           --- Unknown is not present in vertices, skip

               itrian(ielgrp) = -1
               subdiv = .true.

            end if

         else

!        --- Unknown type of element or line element : skip

            itrian(ielgrp) = -1
            subdiv = .true.

         end if

         ikelmc = ikelmc + inelem * inpelm
         if ( debug ) write(irefwr,100) 'lensub', lensub

      end do

!     --- If only line elements than set presen .TRUE.

      if ( nelemt==0 .and. neleml>0 ) presen = .true.
      if ( special==2 .and. presen ) special = 0

      if ( .not. subdiv ) lensub = 0
      if ( special==0 .and. .not. presen ) then
         call errint ( number, 1 )
         call errsub ( 1707, 1, 0, 0 )
      end if

1000  call erclos ( 'topqu2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topqu2'

      end if

      end
