      subroutine topqu4 ( number, idegfd, usol, solutn, npoint, nunkp, &
                          indprf, kprobf, indprp, kprobp, indprh, &
                          kprobh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 23-03-1992 New layout
!        version  2.0  date 09-06-1989 New call, less parameters
!        version  1.0  date 20-06-1988 Developed at Toshiba PC
!
!   copyright (c) 1988-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the solution array usol (which may also be an array of special
!     structure) into the new organised solution array solutn
!     The new array has length number x npoint.
!     Array solutn contains the unknowns ordered corresponding to the nodes
!     Depending on the value of ichois, points are computed by interpolation
!     or not
!     Both usol and solutn must be real, for the complex equivalent see TOPQU5
! **********************************************************************
!
!                       KEYWORDS
!
!     extract_unknown
!     interpolation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, idegfd, npoint, nunkp, indprf, kprobf(npoint+1), &
              indprp, kprobp(npoint,*), indprh, kprobh(*)
      double precision usol(*), solutn(number,npoint)

!     idegfd  i   The first degree of freedom to be used is number idegfd in
!                 each node
!     indprf  i   Indication if array KPROBF has been filled (>0) or not (0)
!     indprh  i   Indication if array KPROBH has been filled (>0) or not (0)
!     indprp  i   Indication if array KPROBP has been filled (>0) or not (0)
!     kprobf  i   Array KPROB part f: gives number of degrees of freedom in
!                 the nodes accumulated
!     kprobh  i   Array KPROB part h: gives the sequence of the unknowns
!     kprobp  i   Array KPROB part p: gives positions of the unknowns in
!                 each node for each physical unkown. If a position is zero
!                 the degree of freedom is not available in that point
!     npoint  i   Number of nodes in the mesh
!     number  i   Number of degrees of freedom per point that must be copied
!     nunkp   i   Maximal number of degrees of freedom in a node
!     solutn  o   Array of length number x npoint in which the reordered
!                 solution array is stored in the sequence:
!                 u (x ) = solutn ( i,k )  (i=1(1)number; k=1(1)npoint)
!                  i  k
!     usol    i   Real solution array of length nusol in which the solution
!                 to be copied is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inusol, inunkp, jnusol

!    i        Loop variable
!    inunkp   Number of unknowns in a node
!    inusol   Position in array USOL.
!             If zero, no solution is available
!    j        Loop variable
!    jnusol   Renumbered position in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     - &
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array solutn

      do 110 j = 1, number
         do 100 i = 1, npoint
            solutn ( j,i ) = 0d0
100      continue
110   continue

!     --- Copy usol into solutn

      if ( indprp.gt.0 ) then

!     --- indprp > 0, use array kprob part p

         do 210 j = 1,number
            do 200 i = 1, npoint
               inusol = kprobp ( i, idegfd + j - 1 )
               if ( inusol.gt.0 ) solutn ( j,i ) = usol ( inusol )
200         continue
210      continue

      else if ( indprf.gt.0 ) then

!     --- indprf > 0, use array kprob part f  ( not constant number of degrees
!                                               of freedom )

         do 250 i = 1, npoint
            inusol = kprobf ( i )
            inunkp = kprobf ( i + 1 ) - inusol
            do 240 j = 1, min ( number, inunkp - idegfd + 1 )
               jnusol = inusol + j + idegfd - 1
               if ( indprh.gt.0 ) jnusol = kprobh ( jnusol )
               solutn ( j,i ) = usol ( jnusol )
240         continue
250      continue

      else

!     --- indprf = 0 and indprp = 0  Constant number of degrees of freedom

         do 310 j = 1, min ( number, nunkp - idegfd + 1 )
            do 300 i = 1, npoint
               inusol = ( i - 1 ) * nunkp + j + idegfd - 1
               if ( indprh.gt.0 ) inusol = kprobh ( inusol )
               solutn ( j,i ) = usol ( inusol )
300         continue
310      continue

      end if

      end
