      subroutine topqu9obst ( kmeshc, nelem, npelm, kelout, nelout, &
                              npoint,  help, itriacolour, itriaout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-07-2001 Copied from topqu0
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Cancel all double elements in a set of surface mesh-elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     double
!     fill
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, nelout, npoint, npelm, kmeshc(npelm,nelem), &
              kelout(npelm,*), itriacolour(*), itriaout(*)
      double precision help(nelem)

!     help           i    help variable of length nelem, see Pseudo Code
!     itriacolour    i    Array of length nelem containing the following
!                         information
!                         itriacolour(i) = 1: element i is external
!                         itriacolour(i) = 2: element i intersects the obstacle
!                         itriacolour(i) = 3: element i is internal
!     itriaout       o    See itriacolour, but now for new mesh corresponding
!                         to kelout
!     kelout         o    New array KMESHC, where all multiple elements have
!                         been canceled
!     kmeshc         i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     nelem          i    Number of elements
!     nelout         i    Number of elements in KELOUT
!     npelm          i    Number of points per element
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielem, j, icopy(4), icpseq(4), icount
      double precision base2, base3, base4
      logical triang, quadrt

!     base2   base value in npoint system : npoint**2
!     base3   base value in npoint system : npoint**3
!     base4   base value in npoint system : npoint**4
!     icopy   Array in which the ndal point numbers of the vertices of one
!             element are copied
!     icount  Counter
!     icpseq  Sorted array icopy, sorting with respect to decreasing sequence
!     ielem   Element number
!     inpelm  number of points per element
!     j       Counting variable
!     quadrt  Indicates if quadratic elements have been found (true) or not
!     triang  Indicates if triangular elements have been found (true) or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     TOPQ00  Find all multiple elements in a grid, where the node numbers
!             have been translated to real numbers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For ielem := 1 (1) number_of_elements
!        Copy vertices of element in icopy
!        Sort these vertices in decreasing sequence and store in icpseq
!        Identify each element by a unique number and store this
!        number in help(ielem):
!          help(ielem) = sum_j icpseq(j) * npoint ** (j-1)
!     Set those values in help that are identical equal to zero (PLT369)
!     store not multiple elements in kelout
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      base2 = npoint
      base3 = npoint * base2
      base4 = npoint * base3

!     --- Determine quadrt, inpelm and triang

      if ( npelm.gt.4 ) then
         quadrt = .true.
      else
         quadrt = .false.
      end if
      if ( npelm.eq.4 .or. npelm.eq.9 ) then
         triang = .false.
         inpelm = 4
      else
         triang = .true.
         inpelm = 3
      end if

!     --- Fill array help with a unique code for each element

      do ielem = 1, nelem

!     --- Copy vertices in array icopy

         if ( quadrt ) then

!        --- Quadratic mesh

            do j = 1, inpelm
               icopy(j) = kmeshc(2*j-1,ielem)
            end do
         else

!        --- Linear mesh

            do j = 1, inpelm
               icopy(j) = kmeshc(j,ielem)
            end do
         end if

!        --- Sort vertices in increasing sequence and store icpseq

         if ( triang ) then

!        --- Triangles

            icpseq(1) = max ( icopy(1), icopy(2), icopy(3) )
            icpseq(3) = min ( icopy(1), icopy(2), icopy(3) )
            do j = 1, 3
               if ( icopy(j).ne.icpseq(1) ) then
                  if ( icopy(j).ne.icpseq(3) ) then
                     icpseq(2) = icopy(j)
                     go to 130
                  end if
               end if
            end do
130         help(ielem) = icpseq(1) + icpseq(2)*base2 + icpseq(3)*base3
         else

!        --- Quadrilaterals

            icpseq(1) = max ( icopy(1), icopy(2), icopy(3), icopy(4) )
            icpseq(4) = min ( icopy(1), icopy(2), icopy(3), icopy(4) )
            icount = 2
            do j = 1, 4
               if ( icopy(j).ne.icpseq(1) ) then
                  if ( icopy(j).ne.icpseq(4) ) then
                     icpseq(icount) = icopy(j)
                     if ( icount.eq.3 ) go to 150
                     icount = icount + 1
                  end if
               end if
            end do
150         if ( icpseq(2).lt.icpseq(3) ) then
                 j         = icpseq(2)
                 icpseq(2) = icpseq(3)
                 icpseq(3) = j
            end if
            help(ielem) = icpseq(1) + icpseq(2)*base2 + icpseq(3)*base3+ &
                          icpseq(4)*base4
         end if
      end do

!     --- Set positions in help corresponding to double elements, &
!         i.e. two elements with equal value, equal to zero
!         kelout is used as work space

      call topq00 ( help, kelout, nelem, 1 )

!     --- Fill array kelout (run through all elements : if help value
!         is not zero then element remains important)

      nelout = 0
      do ielem = 1, nelem
         if ( help(ielem).gt. 0.1d0 ) then
            nelout = nelout + 1
            do j = 1, npelm
               kelout(j,nelout) = kmeshc(j,ielem)
            end do
            itriaout(nelout) = itriacolour(ielem)
         end if
      end do

      end
