      subroutine writb2 ( ibuffr, inum, numarr, namevc, kmesh, &
                          ihelp )
! ====================================================================== &
!
!        programmer    Niek Praagman
!        version  4.3  date 21-11-2012 Long integers
!        version  4.2  date 01-08-2001 new definition kmesh part h
!        version  4.1  date 04-04-2001 Adaptation for empty surfaces
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  array kmesh is written to backing storage
!  Warning: The information of KMESH part r3 is not correct in the
!           backing storage file, because the information of file 3
!           is not copied. As soon as this information will be used
!           in a subroutine different from MESH, an update is necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inum, numarr, kmesh(*), ihelp(*), ibuffr(*)
      character *(*) namevc

!    ibuffr    i/o   Integer buffer array in which all large data is stored
!    ihelp           Work array of length NREC2 which is used as a buffer
!    inum       i    Choice parameter at the input of subroutine WRITBS
!                    When INUM<=0, KMESH is written on the next free positions
!                    of file 2. When INUM = -1, the subroutine prints a text, as
!                    described in the Programmer's Guide.
!                    When INUM>0, KMESH is written on positions of a preceding
!                    written array, with array number INUM.
!    kmesh      i    Standard SEPRAN array containing information of the mesh
!    namevc     i    Name of the array to be written to file 2.
!                    Character variable, is used for printing only
!    numarr     o    At output NUMARR gets the sequence number of KMESH in
!                    file 2.
!                    NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipstar, ipkmsh
      integer iprio, length, nsurfs, nvolms, lenglc, &
              i, irecnr, iwork(2), ncurvs

!     i           Loop variable
!     ipkmsh      Start address of KMESH part h in array IBUFFR
!     iprio       Help variable to store the old priority temporarily
!                 The new priority is made equal to the old one
!     ipstar      Start address of a part of KMESH in array IBUFFR
!     irecnr      Record number in a direct access file
!     iwork       Two-dimensional work array in which two positions for
!                 each part of the array to be written are stored
!                 The first position is always -1000001 to indicate that
!                 the new version of writbs is used, the second position
!                 contains the length of the array part
!                 iwork is used in order to be compatible with preceding
!                 versions of arrays KMESH that have been written
!     lenglc      Local length of array
!     length      Computed total length of the arrays to be written
!     ncurvs      Number of curves
!     nsurfs      Number of surfaces
!     nvolms      Number of volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget
      integer inigpr, inilen

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     INI060  Reactivate existing array in IBUFFR
!     INI062  reserve space for temporary arrays
!     INIGET  Compute pointer of array in IBUFFR
!     INIGPR  Get priority of array
!     INILEN  Compute length of array in IBUFFR
!     INISPR  Set priority of array
!     WRIT00  writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The complete array is written to file 2
!     The subroutine finds which parts of KMESH have been filled and which
!     have not been filled.
!     The actual writing is performed by subroutine WRIT00, which
!     uses a buffer of length NREC2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'writb2' )

!     --- compute length of complete array
!         First set some parameters

      iwork(1) =-1000003
      length   = kmesh(3)
      ncurvs   = kmesh(11)
      nsurfs   = kmesh(12)
      nvolms   = kmesh(13)

!     --- Add length of permanent parts to length of KMESH
!         Part c:  kmesh(17)
!         Part d:  kmesh(18)
!         Part e:  kmesh(19)
!         Part h:  kmesh(22)
!         Part i:  kmesh(23)
!         Add length of auxiliary parts to length
!         Part j:  kmesh(24)
!         Part k:  kmesh(25)
!         Part m:  kmesh(27)
!         Part n:  kmesh(28)
!         Part o:  kmesh(29)
!         Part q:  kmesh(31)
!         Part r:  kmesh(32)
!         Part s:  kmesh(33)
!         Part iinput:  kmesh(34)
!         Part rinput:  kmesh(35)

      do i = 17, 35
         if ( i<20 .or. i>21 .and. i/=30 .and. kmesh(i)>0 ) &
              length = length + inilen(kmesh(i))+2
      end do

!     --- Add parts that are stored on backing storage, but whose reference
!         can not be found immediately in KMESH
!         Parts of KMESH part h

      if ( kmesh(22)>0 ) then

!     --- KMESH(22)>0, i.e. KMESH part h exists

         call ini060 ( ibuffr, -kmesh(22) )
         ipkmsh = iniget(kmesh(22))
         length = length + 2*ncurvs + inilen(ibuffr(ipkmsh+4)) + &
                  inilen(ibuffr(ipkmsh+5))

         do i = 1,nsurfs+nvolms
            if ( ibuffr(ipkmsh+6*i)>0 ) then
               length = length + inilen(ibuffr(ipkmsh+6*i)) + &
                        inilen(ibuffr(ipkmsh+6*i+1)) + &
                        inilen(ibuffr(ipkmsh+6*i+2)) + 5
               if ( ibuffr(ipkmsh+6*i+3)>0 ) &
                  length = length + inilen(ibuffr(ipkmsh+6*i+3))
               if ( ibuffr(ipkmsh+6*i+4)>0 ) &
                  length = length + inilen(ibuffr(ipkmsh+6*i+4))
            end if
         end do

      end if

!     --- write parts of arrays to backing storage
!         Array KMESH:

      call writ00( inum, numarr, namevc, kmesh, length, 1, kmesh(3), &
                   ihelp )

!     --- KMESH part c,d and e

      do i = 17, 19

         iprio = inigpr(kmesh(i))
         call ini060 ( ibuffr, -kmesh(i) )
         ipstar = iniget(kmesh(i))
         lenglc = inilen(kmesh(i))
         call inispr ( kmesh(i), iprio )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, length, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 2, &
                       lenglc, ihelp)

      end do

      if ( kmesh(22)>0 ) then

!     --- KMESH part h

         ipkmsh = iniget(kmesh(22))
         lenglc = inilen(kmesh(22))
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, length, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, ibuffr(ipkmsh), length, 2, &
                       lenglc, ihelp)

!        --- Read next parts of kmesh part h and write to file 2

         ipkmsh = iniget(kmesh(22))
         irecnr = ibuffr(ipkmsh+2)
         lenglc = 2*ncurvs
         call ini060 ( ibuffr, irecnr )
         ipstar = iniget(irecnr)
         call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 2, &
                       lenglc, ihelp)
         ipkmsh = iniget(kmesh(22))
         irecnr = ibuffr(ipkmsh+4)
         lenglc = inilen(ibuffr(ipkmsh+4))
         call ini060 ( ibuffr, irecnr )
         ipstar = iniget(irecnr)
         call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 2, &
                       lenglc, ihelp)
         ipkmsh = iniget(kmesh(22))
         irecnr = ibuffr(ipkmsh+5)
         lenglc = inilen(ibuffr(ipkmsh+5))
         call ini060 ( ibuffr, irecnr )
         ipstar = iniget(irecnr)
         call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 2, &
                       lenglc, ihelp)

         do i = 1,nsurfs+nvolms
            ipkmsh = iniget(kmesh(22))
            if ( ibuffr(ipkmsh+6*i)>0 ) then

!           --- irecnr>0, hence surface or volume is not empty

               call ini062 ( ibuffr, 5, ipstar )
               ipkmsh = iniget(kmesh(22))
               ibuffr(ipstar) = inilen(ibuffr(ipkmsh+6*i))
               ibuffr(ipstar+1) = inilen(ibuffr(ipkmsh+6*i+1))
               ibuffr(ipstar+2) = inilen(ibuffr(ipkmsh+6*i+2))
               ibuffr(ipstar+3) = 0
               ibuffr(ipstar+4) = 0
               if ( ibuffr(ipkmsh+6*i+3)>0 ) &
                  ibuffr(ipstar+3) = inilen(ibuffr(ipkmsh+6*i+3))
               if ( ibuffr(ipkmsh+6*i+4)>0 ) &
                  ibuffr(ipstar+4) = inilen(ibuffr(ipkmsh+6*i+4))
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, 5, ihelp )
               lenglc = inilen(ibuffr(ipkmsh+6*i))
               call ini060 ( ibuffr, ibuffr(ipkmsh+6*i))
               ipstar = iniget(ibuffr(ipkmsh+6*i))
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
               ipkmsh = iniget(kmesh(22))
               lenglc = inilen(ibuffr(ipkmsh+6*i+1))
               call ini060 ( ibuffr, ibuffr(ipkmsh+6*i+1))
               ipstar = iniget(ibuffr(ipkmsh+6*i+1))
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
               ipkmsh = iniget(kmesh(22))
               lenglc = inilen(ibuffr(ipkmsh+6*i+2))
               call ini060 ( ibuffr, ibuffr(ipkmsh+6*i+2))
               ipstar = iniget(ibuffr(ipkmsh+6*i+2))
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
               ipkmsh = iniget(kmesh(22))
               if (  ibuffr(ipkmsh+6*i+3)>0 ) then
                  lenglc = inilen(ibuffr(ipkmsh+6*i+3))
                  call ini060 ( ibuffr, ibuffr(ipkmsh+6*i+3))
                  ipstar = iniget(ibuffr(ipkmsh+6*i+3))
                  call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                                length, 2, lenglc, ihelp )
               end if
               ipkmsh = iniget(kmesh(22))
               if (  ibuffr(ipkmsh+6*i+4)>0 ) then
                  lenglc = inilen(ibuffr(ipkmsh+6*i+4))
                  call ini060 ( ibuffr, ibuffr(ipkmsh+6*i+4))
                  ipstar = iniget(ibuffr(ipkmsh+6*i+4))
                  call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                                length, 2, lenglc, ihelp )
               end if

            end if

         end do

      end if

!     --- KMESH parts i - s, iinput, rinput

      do i = 23, 35
         if ( i/=30 .and. kmesh(i)/=0 ) then

            iprio = inigpr(kmesh(i))
            call ini060 ( ibuffr, -kmesh(i) )
            ipstar = iniget(kmesh(i))
            lenglc = inilen(kmesh(i))
            call inispr ( kmesh(i), iprio )
            iwork(2) = lenglc
            call writ00 ( inum, numarr, namevc, iwork, length, 2, 2, &
                          ihelp )
            call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, &
                          2, lenglc, ihelp)

         end if
      end do

!     --- Close writing, by writing nothing but using ifirst = 3 in writ00

      call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 3, 0, &
                    ihelp)
      call erclos ( 'writb2' )

      end
