      subroutine writb5 ( ibuffr, inum, numarr, namevc, intmat, &
                          kprob, iwork )
! ====================================================================== &
!
!        programmer    Niek Praagman
!        version  3.1  date 21-11-2012 Long integers
!        version  3.0  date 03-04-1999 Extra parameter ibuffr
!        version  2.1  date 26-02-1999 Include comons
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  writb5 writes array of intmat type to backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character * (*) namevc
      integer ibuffr(*), inum, numarr, intmat(*), iwork(*), kprob(*)

!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     intmat     i    Standard SEPRAN array containing information of the
!                     solution
!     inum       i    Choice parameter at the input of subroutine WRITBS
!                     When INUM<=0, ISOL is written on the next free positions
!                     of file 2. When INUM = -1, the subroutine prints a text, &
!                     as described in the Programmer's Guide.
!                     When INUM>0, ISOL is written on positions of a preceding
!                     written array, with array number INUM.
!     iwork           Work array of length NREC2 which is used as a buffer
!     kprob      i    Standard SEPRAN array containing information of the
!                     problem definition. See the Programmer's Guide 13.2 for a
!                     complete description
!     namevc     i    Name of the array to be written to file 2.
!                     Character variable, is used for printing only
!     numarr     o    At output NUMARR gets the sequence number of ISOL in
!                     file 2.
!                     NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipstar
      integer ihelp(2), length, iprio, lenglc, k, iprob, ipkprb, &
              nprob

!     ihelp     Help array to store two positions from infor into the backing
!               storage in order to have the same files for old and new
!               versions of the backing storage subroutines
!     ipkprb      Starting address of part of KPROB with respect to
!                 a complete problem in array KPROB (minus one)
!     iprio     Help variable to store the old priority temporarily
!               The new priority is made equal to the old one
!     iprob       Problem number
!     ipstar    Starting address of various parts of kprob in array IBUFFR
!     k         General loop variable
!     lenglc    Length of local array in KPROB
!     length    Total length of array to be written to backing storage
!     nprob       Number of problems stored in array KPROB
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INI060    Reactivate existing array in IBUFFR
!     INSTOP    Stop the program
!     WRIT00    writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     699   Problem number incorrect
!    1716   Super elements are not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'writb5' )

      nprob = max ( 1, kprob(40) )
      iprob = intmat(1)/1000 + 1
      if ( iprob<=0 .or. iprob>nprob ) then
         call errint ( 1, iprob )
         call errint ( 2, nprob )
         call errsub ( 699, 2, 0, 0 )
         go to 1000
      end if

!     --- Compute number of problems and store starting address of part of
!         kprob (minus one ) in ipkprb

      ipkprb = 0
      do k = 2, iprob
         ipkprb = ipkprb + kprob(3+ipkprb)
      end do
      if ( kprob(ipkprb+29)/=0 ) then

!     --- nsuper>0, i.e. super elements; Not yet implemented

         call errsub ( 1716, 0, 0, 0 )
         go to 1000

      end if

!     --- Compute length of array

      length = 5
      if ( intmat(3)>0 ) length = length + infor1(1,intmat(3)) + 2
      if ( intmat(4)>0 ) length = length + infor1(1,intmat(4)) + 2

!     --- Write part of array to backing storage

      call writ00 ( inum, numarr, namevc, intmat, length, 1, 5, iwork )

!     --- Get intmat arrays into ibuffr

      do k = 3, 4

         if ( intmat(k)>0 ) then

!        --- intmat(k)>0, hence array exists

            iprio = infor1(2,intmat(k))
            call ini060 ( ibuffr, -intmat(k) )
            infor1(2,intmat(k)) = iprio
            ipstar = infor(intmat(k))
            lenglc = infor1(1,intmat(k))

            ihelp(1) = infor(intmat(k))
            ihelp(2) = infor1(1,intmat(k))
            call writ00 ( inum, numarr, namevc, ihelp, length, 2, 2, &
                          iwork )
            call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                          length, 2, lenglc, iwork )
         end if

      end do

!     --- close writing by writing nothing but using ifirst = 3 in writ00

      call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 3, &
                    0, iwork )
1000  call erclos ( 'writb5' )
      if ( ierror/=0 ) call instop
      end
